package org.isda.cdm;

import java.lang.Boolean;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.AdditionalFixedPaymentsMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class to specify the events that will give rise to the payment additional fixed payments.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="AdditionalFixedPayments", source="FpML_5_10")
public class AdditionalFixedPayments extends RosettaModelObject {
	private final Boolean interestShortfallReimbursement;
	private final Boolean principalShortfallReimbursement;
	private final Boolean writedownReimbursement;
	private static AdditionalFixedPaymentsMeta metaData = new AdditionalFixedPaymentsMeta();
	
	AdditionalFixedPayments(AdditionalFixedPaymentsBuilder builder) {
		this.interestShortfallReimbursement = builder.getInterestShortfallReimbursement();
		this.principalShortfallReimbursement = builder.getPrincipalShortfallReimbursement();
		this.writedownReimbursement = builder.getWritedownReimbursement();
	}
	
	/**
	 * An additional Fixed Payment Event. Corresponds to the payment by or on behalf of the Issuer of an actual interest amount in respect to the reference obligation that is greater than the expected interest amount. ISDA 2003 Term: Interest Shortfall Reimbursement.
	 */
	@RosettaSynonym(value="interestShortfallReimbursement", source="FpML_5_10")
	@RosettaSynonym(value="interestShortfallReimbursement", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="interestShortfallReimbursement", source="DTCC_11_0")
	@RosettaSynonym(value="interestShortfallReimbursement", source="DTCC_9_0")
	@RosettaSynonym(value="interestShortfallReimbursement", source="CME_ClearedConfirm_1_17")
	public final Boolean getInterestShortfallReimbursement() {
		return interestShortfallReimbursement;
	}
	
	/**
	 * An additional Fixed Payment Event. Corresponds to the payment by or on behalf of the Issuer of an actual principal amount in respect to the reference obligation that is greater than the expected principal amount. ISDA 2003 Term: Principal Shortfall Reimbursement.
	 */
	@RosettaSynonym(value="principalShortfallReimbursement", source="FpML_5_10")
	@RosettaSynonym(value="principalShortfallReimbursement", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="principalShortfallReimbursement", source="DTCC_11_0")
	@RosettaSynonym(value="principalShortfallReimbursement", source="DTCC_9_0")
	@RosettaSynonym(value="principalShortfallReimbursement", source="CME_ClearedConfirm_1_17")
	public final Boolean getPrincipalShortfallReimbursement() {
		return principalShortfallReimbursement;
	}
	
	/**
	 * An Additional Fixed Payment. Corresponds to the payment by or on behalf of the issuer of an amount in respect to the reference obligation in reduction of the prior writedowns. ISDA 2003 Term: Writedown Reimbursement.
	 */
	@RosettaSynonym(value="writedownReimbursement", source="FpML_5_10")
	@RosettaSynonym(value="writedownReimbursement", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="writedownReimbursement", source="DTCC_11_0")
	@RosettaSynonym(value="writedownReimbursement", source="DTCC_9_0")
	@RosettaSynonym(value="writedownReimbursement", source="CME_ClearedConfirm_1_17")
	public final Boolean getWritedownReimbursement() {
		return writedownReimbursement;
	}
	
	
	@Override
	public RosettaMetaData<? extends AdditionalFixedPayments> metaData() {
		return metaData;
	} 
	
	public AdditionalFixedPaymentsBuilder toBuilder() {
		AdditionalFixedPaymentsBuilder builder = new AdditionalFixedPaymentsBuilder();
		ofNullable(getInterestShortfallReimbursement()).ifPresent(builder::setInterestShortfallReimbursement);
		ofNullable(getPrincipalShortfallReimbursement()).ifPresent(builder::setPrincipalShortfallReimbursement);
		ofNullable(getWritedownReimbursement()).ifPresent(builder::setWritedownReimbursement);
		return builder;
	}

	public static AdditionalFixedPaymentsBuilder builder() {
		return new AdditionalFixedPaymentsBuilder();
	}

	public static  class AdditionalFixedPaymentsBuilder extends RosettaModelObjectBuilder {
	
		protected Boolean interestShortfallReimbursement;
		protected Boolean principalShortfallReimbursement;
		protected Boolean writedownReimbursement;
	
		public AdditionalFixedPaymentsBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends AdditionalFixedPayments> metaData() {
			return metaData;
		} 
	
		
		public Boolean getInterestShortfallReimbursement() {
			return interestShortfallReimbursement;
		}
		
		
		public Boolean getPrincipalShortfallReimbursement() {
			return principalShortfallReimbursement;
		}
		
		
		public Boolean getWritedownReimbursement() {
			return writedownReimbursement;
		}
		
	
		public AdditionalFixedPaymentsBuilder setInterestShortfallReimbursement(Boolean interestShortfallReimbursement) {
			this.interestShortfallReimbursement = interestShortfallReimbursement;
			return this;
		}
		public AdditionalFixedPaymentsBuilder setPrincipalShortfallReimbursement(Boolean principalShortfallReimbursement) {
			this.principalShortfallReimbursement = principalShortfallReimbursement;
			return this;
		}
		public AdditionalFixedPaymentsBuilder setWritedownReimbursement(Boolean writedownReimbursement) {
			this.writedownReimbursement = writedownReimbursement;
			return this;
		}
		
		public AdditionalFixedPayments build() {
			return new AdditionalFixedPayments(this);
		}
	
		@Override
		public AdditionalFixedPaymentsBuilder prune() {
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getInterestShortfallReimbursement()!=null) return true;
			if (getPrincipalShortfallReimbursement()!=null) return true;
			if (getWritedownReimbursement()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("interestShortfallReimbursement"), Boolean.class, interestShortfallReimbursement, this);
			processor.processBasic(path.newSubPath("principalShortfallReimbursement"), Boolean.class, principalShortfallReimbursement, this);
			processor.processBasic(path.newSubPath("writedownReimbursement"), Boolean.class, writedownReimbursement, this);
			
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			AdditionalFixedPaymentsBuilder _that = (AdditionalFixedPaymentsBuilder) o;
		
			if (interestShortfallReimbursement != null ? !interestShortfallReimbursement.equals(_that.interestShortfallReimbursement) : _that.interestShortfallReimbursement != null) return false;
			if (principalShortfallReimbursement != null ? !principalShortfallReimbursement.equals(_that.principalShortfallReimbursement) : _that.principalShortfallReimbursement != null) return false;
			if (writedownReimbursement != null ? !writedownReimbursement.equals(_that.writedownReimbursement) : _that.writedownReimbursement != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (interestShortfallReimbursement != null ? interestShortfallReimbursement.hashCode() : 0);
			_result = 31 * _result + (principalShortfallReimbursement != null ? principalShortfallReimbursement.hashCode() : 0);
			_result = 31 * _result + (writedownReimbursement != null ? writedownReimbursement.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "AdditionalFixedPaymentsBuilder {" +
				"interestShortfallReimbursement=" + this.interestShortfallReimbursement + ", " +
				"principalShortfallReimbursement=" + this.principalShortfallReimbursement + ", " +
				"writedownReimbursement=" + this.writedownReimbursement +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("interestShortfallReimbursement"), Boolean.class, interestShortfallReimbursement, this);
		processor.processBasic(path.newSubPath("principalShortfallReimbursement"), Boolean.class, principalShortfallReimbursement, this);
		processor.processBasic(path.newSubPath("writedownReimbursement"), Boolean.class, writedownReimbursement, this);
		
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		AdditionalFixedPayments _that = (AdditionalFixedPayments) o;
	
		if (interestShortfallReimbursement != null ? !interestShortfallReimbursement.equals(_that.interestShortfallReimbursement) : _that.interestShortfallReimbursement != null) return false;
		if (principalShortfallReimbursement != null ? !principalShortfallReimbursement.equals(_that.principalShortfallReimbursement) : _that.principalShortfallReimbursement != null) return false;
		if (writedownReimbursement != null ? !writedownReimbursement.equals(_that.writedownReimbursement) : _that.writedownReimbursement != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (interestShortfallReimbursement != null ? interestShortfallReimbursement.hashCode() : 0);
		_result = 31 * _result + (principalShortfallReimbursement != null ? principalShortfallReimbursement.hashCode() : 0);
		_result = 31 * _result + (writedownReimbursement != null ? writedownReimbursement.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "AdditionalFixedPayments {" +
			"interestShortfallReimbursement=" + this.interestShortfallReimbursement + ", " +
			"principalShortfallReimbursement=" + this.principalShortfallReimbursement + ", " +
			"writedownReimbursement=" + this.writedownReimbursement +
		'}';
	}
}
