package org.isda.cdm;

import java.lang.String;
import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.RegimeTerms;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.AdditionalRegimeMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;

/**
 * A class to specify the additional regulatory regime(s) that might be elected by the parties to a legal agreement where such provision exists, such as the ISDA 2016 and 2018 CSA for Initial Margin.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class AdditionalRegime extends RosettaModelObject {
	private final String regime;
	private final List<RegimeTerms> regimeTerms;
	private static AdditionalRegimeMeta metaData = new AdditionalRegimeMeta();
	
	AdditionalRegime(AdditionalRegimeBuilder builder) {
		this.regime = builder.getRegime();
		this.regimeTerms = ofNullable(builder.getRegimeTerms()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
	}
	
	/**
	 * The additional regulatory regime as specified by the parties.
	 */
	@RosettaSynonym(value="regime_name", source="ISDA_Create_1_0")
	public final String getRegime() {
		return regime;
	}
	
	/**
	 * The regulatory regime terms which are referred to as part of certain legal agreements, such as such as the ISDA 2016 and 2018 CSA for Initial Margin.
	 */
	public final List<RegimeTerms> getRegimeTerms() {
		return regimeTerms;
	}
	
	
	@Override
	public RosettaMetaData<? extends AdditionalRegime> metaData() {
		return metaData;
	} 
	
	public AdditionalRegimeBuilder toBuilder() {
		AdditionalRegimeBuilder builder = new AdditionalRegimeBuilder();
		ofNullable(getRegime()).ifPresent(builder::setRegime);
		ofNullable(getRegimeTerms()).ifPresent(regimeTerms -> regimeTerms.forEach(builder::addRegimeTerms));
		return builder;
	}

	public static AdditionalRegimeBuilder builder() {
		return new AdditionalRegimeBuilder();
	}

	public static  class AdditionalRegimeBuilder extends RosettaModelObjectBuilder {
	
		protected String regime;
		protected List<RegimeTerms.RegimeTermsBuilder> regimeTerms;
	
		public AdditionalRegimeBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends AdditionalRegime> metaData() {
			return metaData;
		} 
	
		
		public String getRegime() {
			return regime;
		}
		
		
		public List<RegimeTerms.RegimeTermsBuilder> getRegimeTerms() {
			return regimeTerms;
		}
		
		public RegimeTerms.RegimeTermsBuilder getOrCreateRegimeTerms(int index) {
			if (regimeTerms==null) {
				this.regimeTerms = new ArrayList<>();
			}
			return getIndex(regimeTerms, index, ()->new RegimeTerms.RegimeTermsBuilder());
		}
		
	
		public AdditionalRegimeBuilder setRegime(String regime) {
			this.regime = regime;
			return this;
		}
		public AdditionalRegimeBuilder addRegimeTerms(RegimeTerms regimeTerms) {
			if(this.regimeTerms == null){
				this.regimeTerms = new ArrayList<>();
			}
			this.regimeTerms.add(regimeTerms.toBuilder());
			return this;
		}
		public AdditionalRegimeBuilder addRegimeTerms(RegimeTerms regimeTerms, int _idx) {
			if(this.regimeTerms == null){
				this.regimeTerms = new ArrayList<>();
			}
			getIndex(this.regimeTerms, _idx, () -> regimeTerms.toBuilder());
			this.regimeTerms.set(_idx, regimeTerms.toBuilder());
			return this;
		}
		public AdditionalRegimeBuilder addRegimeTerms(List<RegimeTerms> regimeTermss) {
			if(this.regimeTerms == null){
				this.regimeTerms = new ArrayList<>();
			}
			for (RegimeTerms toAdd : regimeTermss) {
				this.regimeTerms.add(toAdd.toBuilder());
			}
			return this;
		}
		public AdditionalRegimeBuilder addRegimeTermsBuilder(RegimeTerms.RegimeTermsBuilder regimeTerms) {
			if(this.regimeTerms == null){
				this.regimeTerms = new ArrayList<>();
			}
			this.regimeTerms.add(regimeTerms);
			return this;
		}
		
		public AdditionalRegimeBuilder clearRegimeTerms() {
			this.regimeTerms = null;
			return this;
		}
		
		public AdditionalRegime build() {
			return new AdditionalRegime(this);
		}
	
		@Override
		public AdditionalRegimeBuilder prune() {
			if (regimeTerms!=null) regimeTerms = regimeTerms.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getRegime()!=null) return true;
			if (getRegimeTerms()!=null && getRegimeTerms().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("regime"), String.class, regime, this);
			
			processRosetta(path.newSubPath("regimeTerms"), processor, RegimeTerms.class, regimeTerms);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			AdditionalRegimeBuilder _that = (AdditionalRegimeBuilder) o;
		
			if (regime != null ? !regime.equals(_that.regime) : _that.regime != null) return false;
			if (!ListEquals.listEquals(regimeTerms, _that.regimeTerms)) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (regime != null ? regime.hashCode() : 0);
			_result = 31 * _result + (regimeTerms != null ? regimeTerms.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "AdditionalRegimeBuilder {" +
				"regime=" + this.regime + ", " +
				"regimeTerms=" + this.regimeTerms +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("regime"), String.class, regime, this);
		
		processRosetta(path.newSubPath("regimeTerms"), processor, RegimeTerms.class, regimeTerms);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		AdditionalRegime _that = (AdditionalRegime) o;
	
		if (regime != null ? !regime.equals(_that.regime) : _that.regime != null) return false;
		if (!ListEquals.listEquals(regimeTerms, _that.regimeTerms)) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (regime != null ? regime.hashCode() : 0);
		_result = 31 * _result + (regimeTerms != null ? regimeTerms.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "AdditionalRegime {" +
			"regime=" + this.regime + ", " +
			"regimeTerms=" + this.regimeTerms +
		'}';
	}
}
