package org.isda.cdm;

import java.lang.String;
import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.AdditionalRepresentationElection;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.AdditionalRepresentationMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;

/**
 * A class to specify the Additional Representation. ISDA 2016 Credit Support Annex for Initial Margin, paragraph 13, General Principles, (o): Additional Representation(s).
 * 
 * @version 2.5.4
 */
@RosettaClass
public class AdditionalRepresentation extends RosettaModelObject {
	private final String customElection;
	private final List<AdditionalRepresentationElection> partyElection;
	private static AdditionalRepresentationMeta metaData = new AdditionalRepresentationMeta();
	
	AdditionalRepresentation(AdditionalRepresentationBuilder builder) {
		this.customElection = builder.getCustomElection();
		this.partyElection = ofNullable(builder.getPartyElection()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
	}
	
	/**
	 * A supplemental custom election that might be specified by the parties for the purpose of specifying the Additional Representation.
	 */
	@RosettaSynonym(value="specify", source="ISDA_Create_1_0")
	public final String getCustomElection() {
		return customElection;
	}
	
	/**
	 * A qualification as to whether the Additional Representation is applicable.
	 */
	public final List<AdditionalRepresentationElection> getPartyElection() {
		return partyElection;
	}
	
	
	@Override
	public RosettaMetaData<? extends AdditionalRepresentation> metaData() {
		return metaData;
	} 
	
	public AdditionalRepresentationBuilder toBuilder() {
		AdditionalRepresentationBuilder builder = new AdditionalRepresentationBuilder();
		ofNullable(getCustomElection()).ifPresent(builder::setCustomElection);
		ofNullable(getPartyElection()).ifPresent(partyElection -> partyElection.forEach(builder::addPartyElection));
		return builder;
	}

	public static AdditionalRepresentationBuilder builder() {
		return new AdditionalRepresentationBuilder();
	}

	public static  class AdditionalRepresentationBuilder extends RosettaModelObjectBuilder {
	
		protected String customElection;
		protected List<AdditionalRepresentationElection.AdditionalRepresentationElectionBuilder> partyElection;
	
		public AdditionalRepresentationBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends AdditionalRepresentation> metaData() {
			return metaData;
		} 
	
		
		public String getCustomElection() {
			return customElection;
		}
		
		
		public List<AdditionalRepresentationElection.AdditionalRepresentationElectionBuilder> getPartyElection() {
			return partyElection;
		}
		
		public AdditionalRepresentationElection.AdditionalRepresentationElectionBuilder getOrCreatePartyElection(int index) {
			if (partyElection==null) {
				this.partyElection = new ArrayList<>();
			}
			return getIndex(partyElection, index, ()->new AdditionalRepresentationElection.AdditionalRepresentationElectionBuilder());
		}
		
	
		public AdditionalRepresentationBuilder setCustomElection(String customElection) {
			this.customElection = customElection;
			return this;
		}
		public AdditionalRepresentationBuilder addPartyElection(AdditionalRepresentationElection partyElection) {
			if(this.partyElection == null){
				this.partyElection = new ArrayList<>();
			}
			this.partyElection.add(partyElection.toBuilder());
			return this;
		}
		public AdditionalRepresentationBuilder addPartyElection(AdditionalRepresentationElection partyElection, int _idx) {
			if(this.partyElection == null){
				this.partyElection = new ArrayList<>();
			}
			getIndex(this.partyElection, _idx, () -> partyElection.toBuilder());
			this.partyElection.set(_idx, partyElection.toBuilder());
			return this;
		}
		public AdditionalRepresentationBuilder addPartyElection(List<AdditionalRepresentationElection> partyElections) {
			if(this.partyElection == null){
				this.partyElection = new ArrayList<>();
			}
			for (AdditionalRepresentationElection toAdd : partyElections) {
				this.partyElection.add(toAdd.toBuilder());
			}
			return this;
		}
		public AdditionalRepresentationBuilder addPartyElectionBuilder(AdditionalRepresentationElection.AdditionalRepresentationElectionBuilder partyElection) {
			if(this.partyElection == null){
				this.partyElection = new ArrayList<>();
			}
			this.partyElection.add(partyElection);
			return this;
		}
		
		public AdditionalRepresentationBuilder clearPartyElection() {
			this.partyElection = null;
			return this;
		}
		
		public AdditionalRepresentation build() {
			return new AdditionalRepresentation(this);
		}
	
		@Override
		public AdditionalRepresentationBuilder prune() {
			if (partyElection!=null) partyElection = partyElection.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getCustomElection()!=null) return true;
			if (getPartyElection()!=null && getPartyElection().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("customElection"), String.class, customElection, this);
			
			processRosetta(path.newSubPath("partyElection"), processor, AdditionalRepresentationElection.class, partyElection);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			AdditionalRepresentationBuilder _that = (AdditionalRepresentationBuilder) o;
		
			if (customElection != null ? !customElection.equals(_that.customElection) : _that.customElection != null) return false;
			if (!ListEquals.listEquals(partyElection, _that.partyElection)) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (customElection != null ? customElection.hashCode() : 0);
			_result = 31 * _result + (partyElection != null ? partyElection.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "AdditionalRepresentationBuilder {" +
				"customElection=" + this.customElection + ", " +
				"partyElection=" + this.partyElection +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("customElection"), String.class, customElection, this);
		
		processRosetta(path.newSubPath("partyElection"), processor, AdditionalRepresentationElection.class, partyElection);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		AdditionalRepresentation _that = (AdditionalRepresentation) o;
	
		if (customElection != null ? !customElection.equals(_that.customElection) : _that.customElection != null) return false;
		if (!ListEquals.listEquals(partyElection, _that.partyElection)) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (customElection != null ? customElection.hashCode() : 0);
		_result = 31 * _result + (partyElection != null ? partyElection.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "AdditionalRepresentation {" +
			"customElection=" + this.customElection + ", " +
			"partyElection=" + this.partyElection +
		'}';
	}
}
