package org.isda.cdm;

import java.lang.Boolean;
import java.lang.String;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.AdditionalRightsEventMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class to specify the Pledgor/Obligor/Chargor Additional Rights Event election. ISDA 2016 English Law Credit Support Deed for Initial Margin, paragraph 13, General Principles, (k): Chargor Additional Rights Event. | ISDA 2016 Japanese Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (k): Obligor Additional Rights Event. | ISDA 2016 New York Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (k): Pledgor Additional Rights Event.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class AdditionalRightsEvent extends RosettaModelObject {
	private final Boolean isApplicable;
	private final String qualification;
	private static AdditionalRightsEventMeta metaData = new AdditionalRightsEventMeta();
	
	AdditionalRightsEvent(AdditionalRightsEventBuilder builder) {
		this.isApplicable = builder.getIsApplicable();
		this.qualification = builder.getQualification();
	}
	
	/**
	 * The Pledgor Additional Rights Event election is applicable when True, and not applicable when False.
	 */
	@RosettaSynonym(value="applicable", source="ISDA_Create_1_0")
	public final Boolean getIsApplicable() {
		return isApplicable;
	}
	
	/**
	 * The qualification of the Pledgor Additional Rights Event election, when specified.
	 */
	@RosettaSynonym(value="specify", source="ISDA_Create_1_0")
	public final String getQualification() {
		return qualification;
	}
	
	
	@Override
	public RosettaMetaData<? extends AdditionalRightsEvent> metaData() {
		return metaData;
	} 
	
	public AdditionalRightsEventBuilder toBuilder() {
		AdditionalRightsEventBuilder builder = new AdditionalRightsEventBuilder();
		ofNullable(getIsApplicable()).ifPresent(builder::setIsApplicable);
		ofNullable(getQualification()).ifPresent(builder::setQualification);
		return builder;
	}

	public static AdditionalRightsEventBuilder builder() {
		return new AdditionalRightsEventBuilder();
	}

	public static  class AdditionalRightsEventBuilder extends RosettaModelObjectBuilder {
	
		protected Boolean isApplicable;
		protected String qualification;
	
		public AdditionalRightsEventBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends AdditionalRightsEvent> metaData() {
			return metaData;
		} 
	
		
		public Boolean getIsApplicable() {
			return isApplicable;
		}
		
		
		public String getQualification() {
			return qualification;
		}
		
	
		public AdditionalRightsEventBuilder setIsApplicable(Boolean isApplicable) {
			this.isApplicable = isApplicable;
			return this;
		}
		public AdditionalRightsEventBuilder setQualification(String qualification) {
			this.qualification = qualification;
			return this;
		}
		
		public AdditionalRightsEvent build() {
			return new AdditionalRightsEvent(this);
		}
	
		@Override
		public AdditionalRightsEventBuilder prune() {
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getIsApplicable()!=null) return true;
			if (getQualification()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("isApplicable"), Boolean.class, isApplicable, this);
			processor.processBasic(path.newSubPath("qualification"), String.class, qualification, this);
			
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			AdditionalRightsEventBuilder _that = (AdditionalRightsEventBuilder) o;
		
			if (isApplicable != null ? !isApplicable.equals(_that.isApplicable) : _that.isApplicable != null) return false;
			if (qualification != null ? !qualification.equals(_that.qualification) : _that.qualification != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (isApplicable != null ? isApplicable.hashCode() : 0);
			_result = 31 * _result + (qualification != null ? qualification.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "AdditionalRightsEventBuilder {" +
				"isApplicable=" + this.isApplicable + ", " +
				"qualification=" + this.qualification +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("isApplicable"), Boolean.class, isApplicable, this);
		processor.processBasic(path.newSubPath("qualification"), String.class, qualification, this);
		
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		AdditionalRightsEvent _that = (AdditionalRightsEvent) o;
	
		if (isApplicable != null ? !isApplicable.equals(_that.isApplicable) : _that.isApplicable != null) return false;
		if (qualification != null ? !qualification.equals(_that.qualification) : _that.qualification != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (isApplicable != null ? isApplicable.hashCode() : 0);
		_result = 31 * _result + (qualification != null ? qualification.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "AdditionalRightsEvent {" +
			"isApplicable=" + this.isApplicable + ", " +
			"qualification=" + this.qualification +
		'}';
	}
}
