package org.isda.cdm;

import java.lang.Boolean;
import java.lang.String;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.AdditionalTerminationEventMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class to specify an optional termination event, additional to the Termination Events that will be deemed an Access Condition (Initial Margin CSA) or a Specified Condition (Variation Margin CSA)
 * 
 * @version 2.5.4
 */
@RosettaClass
public class AdditionalTerminationEvent extends RosettaModelObject {
	private final Boolean applicable;
	private final String name;
	private static AdditionalTerminationEventMeta metaData = new AdditionalTerminationEventMeta();
	
	AdditionalTerminationEvent(AdditionalTerminationEventBuilder builder) {
		this.applicable = builder.getApplicable();
		this.name = builder.getName();
	}
	
	/**
	 * Whether the additional termination event is applicable for the relevant party
	 */
	@RosettaSynonym(value="", source="ISDA_Create_1_0")
	@RosettaSynonym(value="", source="ISDA_Create_1_0")
	public final Boolean getApplicable() {
		return applicable;
	}
	
	/**
	 * The name of the additional termination event
	 */
	@RosettaSynonym(value="name", source="ISDA_Create_1_0")
	public final String getName() {
		return name;
	}
	
	
	@Override
	public RosettaMetaData<? extends AdditionalTerminationEvent> metaData() {
		return metaData;
	} 
	
	public AdditionalTerminationEventBuilder toBuilder() {
		AdditionalTerminationEventBuilder builder = new AdditionalTerminationEventBuilder();
		ofNullable(getApplicable()).ifPresent(builder::setApplicable);
		ofNullable(getName()).ifPresent(builder::setName);
		return builder;
	}

	public static AdditionalTerminationEventBuilder builder() {
		return new AdditionalTerminationEventBuilder();
	}

	public static  class AdditionalTerminationEventBuilder extends RosettaModelObjectBuilder {
	
		protected Boolean applicable;
		protected String name;
	
		public AdditionalTerminationEventBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends AdditionalTerminationEvent> metaData() {
			return metaData;
		} 
	
		
		public Boolean getApplicable() {
			return applicable;
		}
		
		
		public String getName() {
			return name;
		}
		
	
		public AdditionalTerminationEventBuilder setApplicable(Boolean applicable) {
			this.applicable = applicable;
			return this;
		}
		public AdditionalTerminationEventBuilder setName(String name) {
			this.name = name;
			return this;
		}
		
		public AdditionalTerminationEvent build() {
			return new AdditionalTerminationEvent(this);
		}
	
		@Override
		public AdditionalTerminationEventBuilder prune() {
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getApplicable()!=null) return true;
			if (getName()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("applicable"), Boolean.class, applicable, this);
			processor.processBasic(path.newSubPath("name"), String.class, name, this);
			
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			AdditionalTerminationEventBuilder _that = (AdditionalTerminationEventBuilder) o;
		
			if (applicable != null ? !applicable.equals(_that.applicable) : _that.applicable != null) return false;
			if (name != null ? !name.equals(_that.name) : _that.name != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (applicable != null ? applicable.hashCode() : 0);
			_result = 31 * _result + (name != null ? name.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "AdditionalTerminationEventBuilder {" +
				"applicable=" + this.applicable + ", " +
				"name=" + this.name +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("applicable"), Boolean.class, applicable, this);
		processor.processBasic(path.newSubPath("name"), String.class, name, this);
		
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		AdditionalTerminationEvent _that = (AdditionalTerminationEvent) o;
	
		if (applicable != null ? !applicable.equals(_that.applicable) : _that.applicable != null) return false;
		if (name != null ? !name.equals(_that.name) : _that.name != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (applicable != null ? applicable.hashCode() : 0);
		_result = 31 * _result + (name != null ? name.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "AdditionalTerminationEvent {" +
			"applicable=" + this.applicable + ", " +
			"name=" + this.name +
		'}';
	}
}
