package org.isda.cdm;

import java.lang.String;
import org.isda.cdm.AdditionalTypeEnum;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.AdditionalTypeMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * The specification of the Additional Type of transaction that can require the collection or delivery of initial margin under a given regulatory regime for the purposes of Covered Transactions, as specified in ISDA 2016 Credit Support Annex for Initial Margin, paragraph 13, General Principles, (b)(B).
 * 
 * @version 2.5.4
 */
@RosettaClass
public class AdditionalType extends RosettaModelObject {
	private final String customValue;
	private final AdditionalTypeEnum standardValue;
	private static AdditionalTypeMeta metaData = new AdditionalTypeMeta();
	
	AdditionalType(AdditionalTypeBuilder builder) {
		this.customValue = builder.getCustomValue();
		this.standardValue = builder.getStandardValue();
	}
	
	/**
	 * The qualification of the Additional Type of transaction that can require the collection or delivery of initial margin when specified as a custom value by the parties to the legal agreement.
	 */
	@RosettaSynonym(value="additional_type_specify", source="ISDA_Create_1_0")
	public final String getCustomValue() {
		return customValue;
	}
	
	/**
	 * The qualification of the Additional Type of transaction that can require the collection or delivery of initial margin when specified as a standard value.
	 */
	@RosettaSynonym(value="additional_type", source="ISDA_Create_1_0")
	public final AdditionalTypeEnum getStandardValue() {
		return standardValue;
	}
	
	
	@Override
	public RosettaMetaData<? extends AdditionalType> metaData() {
		return metaData;
	} 
	
	public AdditionalTypeBuilder toBuilder() {
		AdditionalTypeBuilder builder = new AdditionalTypeBuilder();
		ofNullable(getCustomValue()).ifPresent(builder::setCustomValue);
		ofNullable(getStandardValue()).ifPresent(builder::setStandardValue);
		return builder;
	}

	public static AdditionalTypeBuilder builder() {
		return new AdditionalTypeBuilder();
	}

	public static  class AdditionalTypeBuilder extends RosettaModelObjectBuilder {
	
		protected String customValue;
		protected AdditionalTypeEnum standardValue;
	
		public AdditionalTypeBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends AdditionalType> metaData() {
			return metaData;
		} 
	
		
		public String getCustomValue() {
			return customValue;
		}
		
		
		public AdditionalTypeEnum getStandardValue() {
			return standardValue;
		}
		
	
		public AdditionalTypeBuilder setCustomValue(String customValue) {
			this.customValue = customValue;
			return this;
		}
		public AdditionalTypeBuilder setStandardValue(AdditionalTypeEnum standardValue) {
			this.standardValue = standardValue;
			return this;
		}
		
		public AdditionalType build() {
			return new AdditionalType(this);
		}
	
		@Override
		public AdditionalTypeBuilder prune() {
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getCustomValue()!=null) return true;
			if (getStandardValue()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("customValue"), String.class, customValue, this);
			processor.processBasic(path.newSubPath("standardValue"), AdditionalTypeEnum.class, standardValue, this);
			
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			AdditionalTypeBuilder _that = (AdditionalTypeBuilder) o;
		
			if (customValue != null ? !customValue.equals(_that.customValue) : _that.customValue != null) return false;
			if (standardValue != null ? !standardValue.equals(_that.standardValue) : _that.standardValue != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (customValue != null ? customValue.hashCode() : 0);
			_result = 31 * _result + (standardValue != null ? standardValue.getClass().getName().hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "AdditionalTypeBuilder {" +
				"customValue=" + this.customValue + ", " +
				"standardValue=" + this.standardValue +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("customValue"), String.class, customValue, this);
		processor.processBasic(path.newSubPath("standardValue"), AdditionalTypeEnum.class, standardValue, this);
		
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		AdditionalType _that = (AdditionalType) o;
	
		if (customValue != null ? !customValue.equals(_that.customValue) : _that.customValue != null) return false;
		if (standardValue != null ? !standardValue.equals(_that.standardValue) : _that.standardValue != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (customValue != null ? customValue.hashCode() : 0);
		_result = 31 * _result + (standardValue != null ? standardValue.getClass().getName().hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "AdditionalType {" +
			"customValue=" + this.customValue + ", " +
			"standardValue=" + this.standardValue +
		'}';
	}
}
