package org.isda.cdm;

import java.lang.String;
import java.util.List;
import org.isda.cdm.metafields.FieldWithMetaString;
import org.isda.cdm.metafields.FieldWithMetaString.FieldWithMetaStringBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.AddressMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;

/**
 * A class to specify a post or street address.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="Address", source="FpML_5_10")
public class Address extends RosettaModelObject {
	private final String city;
	private final FieldWithMetaString country;
	private final String postalCode;
	private final String state;
	private final List<String> street;
	private static AddressMeta metaData = new AddressMeta();
	
	Address(AddressBuilder builder) {
		this.city = builder.getCity();
		this.country = ofNullable(builder.getCountry()).map(f->f.build()).orElse(null);
		this.postalCode = builder.getPostalCode();
		this.state = builder.getState();
		this.street = builder.getStreet();
	}
	
	/**
	 * The city component of the postal address.
	 */
	@RosettaSynonym(value="city", source="FpML_5_10")
	public final String getCity() {
		return city;
	}
	
	/**
	 * The ISO 3166 standard code for the country within which the postal address is located.
	 */
	@RosettaSynonym(value="country", source="FpML_5_10")
	public final FieldWithMetaString getCountry() {
		return country;
	}
	
	/**
	 * The code, required for computerized mail sorting systems, that is allocated to a physical address by a national postal authority.
	 */
	@RosettaSynonym(value="postalCode", source="FpML_5_10")
	public final String getPostalCode() {
		return postalCode;
	}
	
	/**
	 * A country subdivision used in postal addresses in some countries. For example, US states, Canadian provinces, Swiss cantons, ...
	 */
	@RosettaSynonym(value="state", source="FpML_5_10")
	public final String getState() {
		return state;
	}
	
	/**
	 * The set of street and building number information that identifies a postal address within a city.
	 */
	@RosettaSynonym(value="streetAddress", source="FpML_5_10")
	public final List<String> getStreet() {
		return street;
	}
	
	
	@Override
	public RosettaMetaData<? extends Address> metaData() {
		return metaData;
	} 
	
	public AddressBuilder toBuilder() {
		AddressBuilder builder = new AddressBuilder();
		ofNullable(getCity()).ifPresent(builder::setCity);
		ofNullable(getCountry()).ifPresent(builder::setCountry);
		ofNullable(getPostalCode()).ifPresent(builder::setPostalCode);
		ofNullable(getState()).ifPresent(builder::setState);
		ofNullable(getStreet()).ifPresent(street -> street.forEach(builder::addStreet));
		return builder;
	}

	public static AddressBuilder builder() {
		return new AddressBuilder();
	}

	public static  class AddressBuilder extends RosettaModelObjectBuilder {
	
		protected String city;
		protected FieldWithMetaStringBuilder country;
		protected String postalCode;
		protected String state;
		protected List<String> street;
	
		public AddressBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends Address> metaData() {
			return metaData;
		} 
	
		
		public String getCity() {
			return city;
		}
		
		
		public FieldWithMetaStringBuilder getCountry() {
			return country;
		}
		
		public FieldWithMetaStringBuilder getOrCreateCountry() {
			if (country!=null) {
				return country;
			}
			else return country = new FieldWithMetaStringBuilder();
		}
		
		
		public String getPostalCode() {
			return postalCode;
		}
		
		
		public String getState() {
			return state;
		}
		
		
		public List<String> getStreet() {
			return street;
		}
		
	
		public AddressBuilder setCity(String city) {
			this.city = city;
			return this;
		}
		public AddressBuilder setCountry(FieldWithMetaString country) {
			this.country = country.toBuilder();
			return this;
		}
		public AddressBuilder setCountryRef(String country) {
			return setCountry(FieldWithMetaString.builder().setValue(country).build());
		}
		public AddressBuilder setPostalCode(String postalCode) {
			this.postalCode = postalCode;
			return this;
		}
		public AddressBuilder setState(String state) {
			this.state = state;
			return this;
		}
		public AddressBuilder addStreet(String street) {
			if(this.street == null){
				this.street = new ArrayList<>();
			}
			this.street.add(street);
			return this;
		}
		public AddressBuilder addStreet(String street, int _idx) {
			if(this.street == null){
				this.street = new ArrayList<>();
			}
			getIndex(this.street, _idx, () -> street);
			this.street.set(_idx, street);
			return this;
		}
		public AddressBuilder addStreet(List<String> streets) {
			if(this.street == null){
				this.street = new ArrayList<>();
			}
			for (String toAdd : streets) {
				this.street.add(toAdd);
			}
			return this;
		}
		
		public AddressBuilder clearStreet() {
			this.street = null;
			return this;
		}
		
		public Address build() {
			return new Address(this);
		}
	
		@Override
		public AddressBuilder prune() {
			if (country!=null && !country.prune().hasData()) country = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getCity()!=null) return true;
			if (getCountry()!=null) return true;
			if (getPostalCode()!=null) return true;
			if (getState()!=null) return true;
			if (getStreet()!=null && !getStreet().isEmpty()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("city"), String.class, city, this);
			processor.processBasic(path.newSubPath("postalCode"), String.class, postalCode, this);
			processor.processBasic(path.newSubPath("state"), String.class, state, this);
			processor.processBasic(path.newSubPath("street"), String.class, street, this);
			
			processRosetta(path.newSubPath("country"), processor, FieldWithMetaString.class, country);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			AddressBuilder _that = (AddressBuilder) o;
		
			if (city != null ? !city.equals(_that.city) : _that.city != null) return false;
			if (country != null ? !country.equals(_that.country) : _that.country != null) return false;
			if (postalCode != null ? !postalCode.equals(_that.postalCode) : _that.postalCode != null) return false;
			if (state != null ? !state.equals(_that.state) : _that.state != null) return false;
			if (!ListEquals.listEquals(street, _that.street)) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (city != null ? city.hashCode() : 0);
			_result = 31 * _result + (country != null ? country.hashCode() : 0);
			_result = 31 * _result + (postalCode != null ? postalCode.hashCode() : 0);
			_result = 31 * _result + (state != null ? state.hashCode() : 0);
			_result = 31 * _result + (street != null ? street.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "AddressBuilder {" +
				"city=" + this.city + ", " +
				"country=" + this.country + ", " +
				"postalCode=" + this.postalCode + ", " +
				"state=" + this.state + ", " +
				"street=" + this.street +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("city"), String.class, city, this);
		processor.processBasic(path.newSubPath("postalCode"), String.class, postalCode, this);
		processor.processBasic(path.newSubPath("state"), String.class, state, this);
		street.stream().forEach(a->processor.processBasic(path.newSubPath("street"), String.class, a, this));
		
		processRosetta(path.newSubPath("country"), processor, FieldWithMetaString.class, country);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		Address _that = (Address) o;
	
		if (city != null ? !city.equals(_that.city) : _that.city != null) return false;
		if (country != null ? !country.equals(_that.country) : _that.country != null) return false;
		if (postalCode != null ? !postalCode.equals(_that.postalCode) : _that.postalCode != null) return false;
		if (state != null ? !state.equals(_that.state) : _that.state != null) return false;
		if (!ListEquals.listEquals(street, _that.street)) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (city != null ? city.hashCode() : 0);
		_result = 31 * _result + (country != null ? country.hashCode() : 0);
		_result = 31 * _result + (postalCode != null ? postalCode.hashCode() : 0);
		_result = 31 * _result + (state != null ? state.hashCode() : 0);
		_result = 31 * _result + (street != null ? street.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "Address {" +
			"city=" + this.city + ", " +
			"country=" + this.country + ", " +
			"postalCode=" + this.postalCode + ", " +
			"state=" + this.state + ", " +
			"street=" + this.street +
		'}';
	}
}
