package org.isda.cdm;

import org.isda.cdm.AdjustableDates;
import org.isda.cdm.RelativeDates;
import org.isda.cdm.metafields.MetaFields;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.AdjustableOrRelativeDatesMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class giving the choice between defining a series of dates as an explicit list of dates together with applicable adjustments or as relative to some other series of (anchor) dates.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class AdjustableOrRelativeDates extends RosettaModelObject implements GlobalKey {
	private final AdjustableDates adjustableDates;
	private final MetaFields meta;
	private final RelativeDates relativeDates;
	private static AdjustableOrRelativeDatesMeta metaData = new AdjustableOrRelativeDatesMeta();
	
	AdjustableOrRelativeDates(AdjustableOrRelativeDatesBuilder builder) {
		this.adjustableDates = ofNullable(builder.getAdjustableDates()).map(f->f.build()).orElse(null);
		this.meta = ofNullable(builder.getMeta()).map(f->f.build()).orElse(null);
		this.relativeDates = ofNullable(builder.getRelativeDates()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * A series of dates that shall be subject to adjustment if they would otherwise fall on a day that is not a business day in the specified business centers, together with the convention for adjusting the date.
	 */
	@RosettaSynonym(value="adjustableDates", source="FpML_5_10")
	@RosettaSynonym(value="adjustableDates", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="adjustableDates", source="DTCC_11_0")
	@RosettaSynonym(value="adjustableDates", source="DTCC_9_0")
	@RosettaSynonym(value="adjustableDates", source="CME_ClearedConfirm_1_17")
	public final AdjustableDates getAdjustableDates() {
		return adjustableDates;
	}
	
	public final MetaFields getMeta() {
		return meta;
	}
	
	/**
	 * A series of dates specified as some offset to another series of dates (the anchor dates).
	 */
	@RosettaSynonym(value="relativeDates", source="FpML_5_10")
	@RosettaSynonym(value="relativeDates", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="relativeDates", source="DTCC_11_0")
	@RosettaSynonym(value="relativeDates", source="DTCC_9_0")
	@RosettaSynonym(value="relativeDates", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="relativeDateSequence", source="FpML_5_10")
	public final RelativeDates getRelativeDates() {
		return relativeDates;
	}
	
	
	@Override
	public RosettaMetaData<? extends AdjustableOrRelativeDates> metaData() {
		return metaData;
	} 
	
	public AdjustableOrRelativeDatesBuilder toBuilder() {
		AdjustableOrRelativeDatesBuilder builder = new AdjustableOrRelativeDatesBuilder();
		ofNullable(getAdjustableDates()).ifPresent(builder::setAdjustableDates);
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		ofNullable(getRelativeDates()).ifPresent(builder::setRelativeDates);
		return builder;
	}

	public static AdjustableOrRelativeDatesBuilder builder() {
		return new AdjustableOrRelativeDatesBuilder();
	}

	public static  class AdjustableOrRelativeDatesBuilder extends RosettaModelObjectBuilder implements GlobalKeyBuilder<AdjustableOrRelativeDatesBuilder> {
	
		protected AdjustableDates.AdjustableDatesBuilder adjustableDates;
		protected MetaFields.MetaFieldsBuilder meta;
		protected RelativeDates.RelativeDatesBuilder relativeDates;
	
		public AdjustableOrRelativeDatesBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends AdjustableOrRelativeDates> metaData() {
			return metaData;
		} 
	
		
		public AdjustableDates.AdjustableDatesBuilder getAdjustableDates() {
			return adjustableDates;
		}
		
		public AdjustableDates.AdjustableDatesBuilder getOrCreateAdjustableDates() {
			if (adjustableDates!=null) {
				return adjustableDates;
			}
			else return adjustableDates = new AdjustableDates.AdjustableDatesBuilder();
		}
		
		
		public MetaFields.MetaFieldsBuilder getMeta() {
			return meta;
		}
		
		public MetaFields.MetaFieldsBuilder getOrCreateMeta() {
			if (meta!=null) {
				return meta;
			}
			else return meta = new MetaFields.MetaFieldsBuilder();
		}
		
		
		public RelativeDates.RelativeDatesBuilder getRelativeDates() {
			return relativeDates;
		}
		
		public RelativeDates.RelativeDatesBuilder getOrCreateRelativeDates() {
			if (relativeDates!=null) {
				return relativeDates;
			}
			else return relativeDates = new RelativeDates.RelativeDatesBuilder();
		}
		
	
		public AdjustableOrRelativeDatesBuilder setAdjustableDates(AdjustableDates adjustableDates) {
			this.adjustableDates = adjustableDates.toBuilder();
			return this;
		}
		public AdjustableOrRelativeDatesBuilder setAdjustableDatesBuilder(AdjustableDates.AdjustableDatesBuilder adjustableDates) {
			this.adjustableDates = adjustableDates;
			return this;
		}
		public AdjustableOrRelativeDatesBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		public AdjustableOrRelativeDatesBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		public AdjustableOrRelativeDatesBuilder setRelativeDates(RelativeDates relativeDates) {
			this.relativeDates = relativeDates.toBuilder();
			return this;
		}
		public AdjustableOrRelativeDatesBuilder setRelativeDatesBuilder(RelativeDates.RelativeDatesBuilder relativeDates) {
			this.relativeDates = relativeDates;
			return this;
		}
		
		public AdjustableOrRelativeDates build() {
			return new AdjustableOrRelativeDates(this);
		}
	
		@Override
		public AdjustableOrRelativeDatesBuilder prune() {
			if (adjustableDates!=null && !adjustableDates.prune().hasData()) adjustableDates = null;
			if (meta!=null && !meta.prune().hasData()) meta = null;
			if (relativeDates!=null && !relativeDates.prune().hasData()) relativeDates = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getAdjustableDates()!=null && getAdjustableDates().hasData()) return true;
			if (getMeta()!=null && getMeta().hasData()) return true;
			if (getRelativeDates()!=null && getRelativeDates().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("adjustableDates"), processor, AdjustableDates.class, adjustableDates);
			processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
			processRosetta(path.newSubPath("relativeDates"), processor, RelativeDates.class, relativeDates);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			AdjustableOrRelativeDatesBuilder _that = (AdjustableOrRelativeDatesBuilder) o;
		
			if (adjustableDates != null ? !adjustableDates.equals(_that.adjustableDates) : _that.adjustableDates != null) return false;
			if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
			if (relativeDates != null ? !relativeDates.equals(_that.relativeDates) : _that.relativeDates != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (adjustableDates != null ? adjustableDates.hashCode() : 0);
			_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
			_result = 31 * _result + (relativeDates != null ? relativeDates.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "AdjustableOrRelativeDatesBuilder {" +
				"adjustableDates=" + this.adjustableDates + ", " +
				"meta=" + this.meta + ", " +
				"relativeDates=" + this.relativeDates +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("adjustableDates"), processor, AdjustableDates.class, adjustableDates);
		processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
		processRosetta(path.newSubPath("relativeDates"), processor, RelativeDates.class, relativeDates);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		AdjustableOrRelativeDates _that = (AdjustableOrRelativeDates) o;
	
		if (adjustableDates != null ? !adjustableDates.equals(_that.adjustableDates) : _that.adjustableDates != null) return false;
		if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
		if (relativeDates != null ? !relativeDates.equals(_that.relativeDates) : _that.relativeDates != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (adjustableDates != null ? adjustableDates.hashCode() : 0);
		_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
		_result = 31 * _result + (relativeDates != null ? relativeDates.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "AdjustableOrRelativeDates {" +
			"adjustableDates=" + this.adjustableDates + ", " +
			"meta=" + this.meta + ", " +
			"relativeDates=" + this.relativeDates +
		'}';
	}
}
