package org.isda.cdm;

import java.lang.Boolean;
import java.lang.String;
import java.time.ZonedDateTime;
import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.PositionStatusEnum;
import org.isda.cdm.Product;
import org.isda.cdm.metafields.ReferenceWithMetaContract;
import org.isda.cdm.metafields.ReferenceWithMetaContract.ReferenceWithMetaContractBuilder;
import org.isda.cdm.metafields.ReferenceWithMetaExecution;
import org.isda.cdm.metafields.ReferenceWithMetaExecution.ReferenceWithMetaExecutionBuilder;
import org.isda.cdm.metafields.ReferenceWithMetaParty;
import org.isda.cdm.metafields.ReferenceWithMetaParty.ReferenceWithMetaPartyBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.AggregationParametersMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaQualified;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.time.ZonedDateTime;
		import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;

/**
 * Parameters to be used to filter events that are relevant to a given portfolio in order to calculate the state of this portfolio. The attributes correspond to all the possible aggregation criteria that can be used and these criteria can be combined. All the attributes are optional.
 * 
 * @version 2.5.4
 */
@RosettaClass
@RosettaQualified(attribute="productQualifier",qualifiedClass=EconomicTerms.class)
public class AggregationParameters extends RosettaModelObject {
	private final List<ReferenceWithMetaContract> contractReference;
	private final ZonedDateTime dateTime;
	private final List<ReferenceWithMetaExecution> executionReference;
	private final List<ReferenceWithMetaParty> party;
	private final PositionStatusEnum positionStatus;
	private final List<Product> product;
	private final List<String> productQualifier;
	private final Boolean totalPosition;
	private static AggregationParametersMeta metaData = new AggregationParametersMeta();
	
	AggregationParameters(AggregationParametersBuilder builder) {
		this.contractReference = ofNullable(builder.getContractReference()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.dateTime = builder.getDateTime();
		this.executionReference = ofNullable(builder.getExecutionReference()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.party = ofNullable(builder.getParty()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.positionStatus = builder.getPositionStatus();
		this.product = ofNullable(builder.getProduct()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.productQualifier = builder.getProductQualifier();
		this.totalPosition = builder.getTotalPosition();
	}
	
	/**
	 * To aggregate based only on given Contract(s).
	 */
	public final List<ReferenceWithMetaContract> getContractReference() {
		return contractReference;
	}
	
	/**
	 * To aggregate as of a particular date
	 */
	public final ZonedDateTime getDateTime() {
		return dateTime;
	}
	
	/**
	 * To aggregate based only on given Execution(s).
	 */
	public final List<ReferenceWithMetaExecution> getExecutionReference() {
		return executionReference;
	}
	
	/**
	 * To aggregate based on a selection of party(ies) / legal entity(ies).
	 */
	public final List<ReferenceWithMetaParty> getParty() {
		return party;
	}
	
	/**
	 * To aggregate based on position status (EXECUTED, SETTLED etc)
	 */
	public final PositionStatusEnum getPositionStatus() {
		return positionStatus;
	}
	
	/**
	 * To aggregate based on a selection of products.
	 */
	public final List<Product> getProduct() {
		return product;
	}
	
	/**
	 * To aggregate based on a selection of product type(s).
	 */
	public final List<String> getProductQualifier() {
		return productQualifier;
	}
	
	/**
	 * Specifies whether to calculate total position to given date, or only daily position for the given date.
	 */
	public final Boolean getTotalPosition() {
		return totalPosition;
	}
	
	
	@Override
	public RosettaMetaData<? extends AggregationParameters> metaData() {
		return metaData;
	} 
	
	public AggregationParametersBuilder toBuilder() {
		AggregationParametersBuilder builder = new AggregationParametersBuilder();
		ofNullable(getContractReference()).ifPresent(contractReference -> contractReference.forEach(builder::addContractReference));
		ofNullable(getDateTime()).ifPresent(builder::setDateTime);
		ofNullable(getExecutionReference()).ifPresent(executionReference -> executionReference.forEach(builder::addExecutionReference));
		ofNullable(getParty()).ifPresent(party -> party.forEach(builder::addParty));
		ofNullable(getPositionStatus()).ifPresent(builder::setPositionStatus);
		ofNullable(getProduct()).ifPresent(product -> product.forEach(builder::addProduct));
		ofNullable(getProductQualifier()).ifPresent(productQualifier -> productQualifier.forEach(builder::addProductQualifier));
		ofNullable(getTotalPosition()).ifPresent(builder::setTotalPosition);
		return builder;
	}

	public static AggregationParametersBuilder builder() {
		return new AggregationParametersBuilder();
	}

	public static  class AggregationParametersBuilder extends RosettaModelObjectBuilder {
	
		protected List<ReferenceWithMetaContractBuilder> contractReference;
		protected ZonedDateTime dateTime;
		protected List<ReferenceWithMetaExecutionBuilder> executionReference;
		protected List<ReferenceWithMetaPartyBuilder> party;
		protected PositionStatusEnum positionStatus;
		protected List<Product.ProductBuilder> product;
		protected List<String> productQualifier;
		protected Boolean totalPosition;
	
		public AggregationParametersBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends AggregationParameters> metaData() {
			return metaData;
		} 
	
		
		public List<ReferenceWithMetaContractBuilder> getContractReference() {
			return contractReference;
		}
		
		public ReferenceWithMetaContractBuilder getOrCreateContractReference(int index) {
			if (contractReference==null) {
				this.contractReference = new ArrayList<>();
			}
			return getIndex(contractReference, index, ()->new ReferenceWithMetaContractBuilder());
		}
		
		
		public ZonedDateTime getDateTime() {
			return dateTime;
		}
		
		
		public List<ReferenceWithMetaExecutionBuilder> getExecutionReference() {
			return executionReference;
		}
		
		public ReferenceWithMetaExecutionBuilder getOrCreateExecutionReference(int index) {
			if (executionReference==null) {
				this.executionReference = new ArrayList<>();
			}
			return getIndex(executionReference, index, ()->new ReferenceWithMetaExecutionBuilder());
		}
		
		
		public List<ReferenceWithMetaPartyBuilder> getParty() {
			return party;
		}
		
		public ReferenceWithMetaPartyBuilder getOrCreateParty(int index) {
			if (party==null) {
				this.party = new ArrayList<>();
			}
			return getIndex(party, index, ()->new ReferenceWithMetaPartyBuilder());
		}
		
		
		public PositionStatusEnum getPositionStatus() {
			return positionStatus;
		}
		
		
		public List<Product.ProductBuilder> getProduct() {
			return product;
		}
		
		public Product.ProductBuilder getOrCreateProduct(int index) {
			if (product==null) {
				this.product = new ArrayList<>();
			}
			return getIndex(product, index, ()->new Product.ProductBuilder());
		}
		
		
		public List<String> getProductQualifier() {
			return productQualifier;
		}
		
		
		public Boolean getTotalPosition() {
			return totalPosition;
		}
		
	
		public AggregationParametersBuilder addContractReference(ReferenceWithMetaContract contractReference) {
			if(this.contractReference == null){
				this.contractReference = new ArrayList<>();
			}
			this.contractReference.add(contractReference.toBuilder());
			return this;
		}
		public AggregationParametersBuilder addContractReference(ReferenceWithMetaContract contractReference, int _idx) {
			if(this.contractReference == null){
				this.contractReference = new ArrayList<>();
			}
			getIndex(this.contractReference, _idx, () -> contractReference.toBuilder());
			this.contractReference.set(_idx, contractReference.toBuilder());
			return this;
		}
		public AggregationParametersBuilder addContractReference(List<ReferenceWithMetaContract> contractReferences) {
			if(this.contractReference == null){
				this.contractReference = new ArrayList<>();
			}
			for (ReferenceWithMetaContract toAdd : contractReferences) {
				this.contractReference.add(toAdd.toBuilder());
			}
			return this;
		}
		public AggregationParametersBuilder addContractReferenceBuilder(ReferenceWithMetaContractBuilder contractReference) {
			if(this.contractReference == null){
				this.contractReference = new ArrayList<>();
			}
			this.contractReference.add(contractReference);
			return this;
		}
		
		public AggregationParametersBuilder addContractReferenceRef(Contract.ContractBuilder contractReference) {
			return addContractReference(ReferenceWithMetaContract.builder().setValueBuilder(contractReference).build());
		}
		
		public AggregationParametersBuilder addContractReferenceRef(Contract.ContractBuilder contractReference, int _idx) {
			return addContractReference(ReferenceWithMetaContract.builder().setValueBuilder(contractReference).build(), _idx);
		}
		
		public AggregationParametersBuilder addContractReferenceRef(List<Contract> contractReferences) {
			for (Contract toAdd : contractReferences) {
				addContractReferenceRef(toAdd);
			}
			return this;
		}
		
		public AggregationParametersBuilder addContractReferenceRef(Contract contractReference) {
			return addContractReferenceRef(contractReference.toBuilder());
		}
		
		public AggregationParametersBuilder clearContractReference() {
			this.contractReference = null;
			return this;
		}
		public AggregationParametersBuilder setDateTime(ZonedDateTime dateTime) {
			this.dateTime = dateTime;
			return this;
		}
		public AggregationParametersBuilder addExecutionReference(ReferenceWithMetaExecution executionReference) {
			if(this.executionReference == null){
				this.executionReference = new ArrayList<>();
			}
			this.executionReference.add(executionReference.toBuilder());
			return this;
		}
		public AggregationParametersBuilder addExecutionReference(ReferenceWithMetaExecution executionReference, int _idx) {
			if(this.executionReference == null){
				this.executionReference = new ArrayList<>();
			}
			getIndex(this.executionReference, _idx, () -> executionReference.toBuilder());
			this.executionReference.set(_idx, executionReference.toBuilder());
			return this;
		}
		public AggregationParametersBuilder addExecutionReference(List<ReferenceWithMetaExecution> executionReferences) {
			if(this.executionReference == null){
				this.executionReference = new ArrayList<>();
			}
			for (ReferenceWithMetaExecution toAdd : executionReferences) {
				this.executionReference.add(toAdd.toBuilder());
			}
			return this;
		}
		public AggregationParametersBuilder addExecutionReferenceBuilder(ReferenceWithMetaExecutionBuilder executionReference) {
			if(this.executionReference == null){
				this.executionReference = new ArrayList<>();
			}
			this.executionReference.add(executionReference);
			return this;
		}
		
		public AggregationParametersBuilder addExecutionReferenceRef(Execution.ExecutionBuilder executionReference) {
			return addExecutionReference(ReferenceWithMetaExecution.builder().setValueBuilder(executionReference).build());
		}
		
		public AggregationParametersBuilder addExecutionReferenceRef(Execution.ExecutionBuilder executionReference, int _idx) {
			return addExecutionReference(ReferenceWithMetaExecution.builder().setValueBuilder(executionReference).build(), _idx);
		}
		
		public AggregationParametersBuilder addExecutionReferenceRef(List<Execution> executionReferences) {
			for (Execution toAdd : executionReferences) {
				addExecutionReferenceRef(toAdd);
			}
			return this;
		}
		
		public AggregationParametersBuilder addExecutionReferenceRef(Execution executionReference) {
			return addExecutionReferenceRef(executionReference.toBuilder());
		}
		
		public AggregationParametersBuilder clearExecutionReference() {
			this.executionReference = null;
			return this;
		}
		public AggregationParametersBuilder addParty(ReferenceWithMetaParty party) {
			if(this.party == null){
				this.party = new ArrayList<>();
			}
			this.party.add(party.toBuilder());
			return this;
		}
		public AggregationParametersBuilder addParty(ReferenceWithMetaParty party, int _idx) {
			if(this.party == null){
				this.party = new ArrayList<>();
			}
			getIndex(this.party, _idx, () -> party.toBuilder());
			this.party.set(_idx, party.toBuilder());
			return this;
		}
		public AggregationParametersBuilder addParty(List<ReferenceWithMetaParty> partys) {
			if(this.party == null){
				this.party = new ArrayList<>();
			}
			for (ReferenceWithMetaParty toAdd : partys) {
				this.party.add(toAdd.toBuilder());
			}
			return this;
		}
		public AggregationParametersBuilder addPartyBuilder(ReferenceWithMetaPartyBuilder party) {
			if(this.party == null){
				this.party = new ArrayList<>();
			}
			this.party.add(party);
			return this;
		}
		
		public AggregationParametersBuilder addPartyRef(Party.PartyBuilder party) {
			return addParty(ReferenceWithMetaParty.builder().setValueBuilder(party).build());
		}
		
		public AggregationParametersBuilder addPartyRef(Party.PartyBuilder party, int _idx) {
			return addParty(ReferenceWithMetaParty.builder().setValueBuilder(party).build(), _idx);
		}
		
		public AggregationParametersBuilder addPartyRef(List<Party> partys) {
			for (Party toAdd : partys) {
				addPartyRef(toAdd);
			}
			return this;
		}
		
		public AggregationParametersBuilder addPartyRef(Party party) {
			return addPartyRef(party.toBuilder());
		}
		
		public AggregationParametersBuilder clearParty() {
			this.party = null;
			return this;
		}
		public AggregationParametersBuilder setPositionStatus(PositionStatusEnum positionStatus) {
			this.positionStatus = positionStatus;
			return this;
		}
		public AggregationParametersBuilder addProduct(Product product) {
			if(this.product == null){
				this.product = new ArrayList<>();
			}
			this.product.add(product.toBuilder());
			return this;
		}
		public AggregationParametersBuilder addProduct(Product product, int _idx) {
			if(this.product == null){
				this.product = new ArrayList<>();
			}
			getIndex(this.product, _idx, () -> product.toBuilder());
			this.product.set(_idx, product.toBuilder());
			return this;
		}
		public AggregationParametersBuilder addProduct(List<Product> products) {
			if(this.product == null){
				this.product = new ArrayList<>();
			}
			for (Product toAdd : products) {
				this.product.add(toAdd.toBuilder());
			}
			return this;
		}
		public AggregationParametersBuilder addProductBuilder(Product.ProductBuilder product) {
			if(this.product == null){
				this.product = new ArrayList<>();
			}
			this.product.add(product);
			return this;
		}
		
		public AggregationParametersBuilder clearProduct() {
			this.product = null;
			return this;
		}
		public AggregationParametersBuilder addProductQualifier(String productQualifier) {
			if(this.productQualifier == null){
				this.productQualifier = new ArrayList<>();
			}
			this.productQualifier.add(productQualifier);
			return this;
		}
		public AggregationParametersBuilder addProductQualifier(String productQualifier, int _idx) {
			if(this.productQualifier == null){
				this.productQualifier = new ArrayList<>();
			}
			getIndex(this.productQualifier, _idx, () -> productQualifier);
			this.productQualifier.set(_idx, productQualifier);
			return this;
		}
		public AggregationParametersBuilder addProductQualifier(List<String> productQualifiers) {
			if(this.productQualifier == null){
				this.productQualifier = new ArrayList<>();
			}
			for (String toAdd : productQualifiers) {
				this.productQualifier.add(toAdd);
			}
			return this;
		}
		
		public AggregationParametersBuilder clearProductQualifier() {
			this.productQualifier = null;
			return this;
		}
		public AggregationParametersBuilder setTotalPosition(Boolean totalPosition) {
			this.totalPosition = totalPosition;
			return this;
		}
		
		public AggregationParameters build() {
			return new AggregationParameters(this);
		}
	
		@Override
		public AggregationParametersBuilder prune() {
			if (contractReference!=null) contractReference = contractReference.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (executionReference!=null) executionReference = executionReference.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (party!=null) party = party.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (product!=null) product = product.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getContractReference()!=null && getContractReference().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getDateTime()!=null) return true;
			if (getExecutionReference()!=null && getExecutionReference().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getParty()!=null && getParty().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getPositionStatus()!=null) return true;
			if (getProduct()!=null && getProduct().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getProductQualifier()!=null && !getProductQualifier().isEmpty()) return true;
			if (getTotalPosition()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("dateTime"), ZonedDateTime.class, dateTime, this);
			processor.processBasic(path.newSubPath("positionStatus"), PositionStatusEnum.class, positionStatus, this);
			processor.processBasic(path.newSubPath("productQualifier"), String.class, productQualifier, this);
			processor.processBasic(path.newSubPath("totalPosition"), Boolean.class, totalPosition, this);
			
			processRosetta(path.newSubPath("contractReference"), processor, ReferenceWithMetaContract.class, contractReference);
			processRosetta(path.newSubPath("executionReference"), processor, ReferenceWithMetaExecution.class, executionReference);
			processRosetta(path.newSubPath("party"), processor, ReferenceWithMetaParty.class, party);
			processRosetta(path.newSubPath("product"), processor, Product.class, product);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			AggregationParametersBuilder _that = (AggregationParametersBuilder) o;
		
			if (!ListEquals.listEquals(contractReference, _that.contractReference)) return false;
			if (dateTime != null ? !dateTime.equals(_that.dateTime) : _that.dateTime != null) return false;
			if (!ListEquals.listEquals(executionReference, _that.executionReference)) return false;
			if (!ListEquals.listEquals(party, _that.party)) return false;
			if (positionStatus != null ? !positionStatus.equals(_that.positionStatus) : _that.positionStatus != null) return false;
			if (!ListEquals.listEquals(product, _that.product)) return false;
			if (!ListEquals.listEquals(productQualifier, _that.productQualifier)) return false;
			if (totalPosition != null ? !totalPosition.equals(_that.totalPosition) : _that.totalPosition != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (contractReference != null ? contractReference.hashCode() : 0);
			_result = 31 * _result + (dateTime != null ? dateTime.hashCode() : 0);
			_result = 31 * _result + (executionReference != null ? executionReference.hashCode() : 0);
			_result = 31 * _result + (party != null ? party.hashCode() : 0);
			_result = 31 * _result + (positionStatus != null ? positionStatus.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (product != null ? product.hashCode() : 0);
			_result = 31 * _result + (productQualifier != null ? productQualifier.hashCode() : 0);
			_result = 31 * _result + (totalPosition != null ? totalPosition.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "AggregationParametersBuilder {" +
				"contractReference=" + this.contractReference + ", " +
				"dateTime=" + this.dateTime + ", " +
				"executionReference=" + this.executionReference + ", " +
				"party=" + this.party + ", " +
				"positionStatus=" + this.positionStatus + ", " +
				"product=" + this.product + ", " +
				"productQualifier=" + this.productQualifier + ", " +
				"totalPosition=" + this.totalPosition +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("dateTime"), ZonedDateTime.class, dateTime, this);
		processor.processBasic(path.newSubPath("positionStatus"), PositionStatusEnum.class, positionStatus, this);
		productQualifier.stream().forEach(a->processor.processBasic(path.newSubPath("productQualifier"), String.class, a, this));
		processor.processBasic(path.newSubPath("totalPosition"), Boolean.class, totalPosition, this);
		
		processRosetta(path.newSubPath("contractReference"), processor, ReferenceWithMetaContract.class, contractReference);
		processRosetta(path.newSubPath("executionReference"), processor, ReferenceWithMetaExecution.class, executionReference);
		processRosetta(path.newSubPath("party"), processor, ReferenceWithMetaParty.class, party);
		processRosetta(path.newSubPath("product"), processor, Product.class, product);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		AggregationParameters _that = (AggregationParameters) o;
	
		if (!ListEquals.listEquals(contractReference, _that.contractReference)) return false;
		if (dateTime != null ? !dateTime.equals(_that.dateTime) : _that.dateTime != null) return false;
		if (!ListEquals.listEquals(executionReference, _that.executionReference)) return false;
		if (!ListEquals.listEquals(party, _that.party)) return false;
		if (positionStatus != null ? !positionStatus.equals(_that.positionStatus) : _that.positionStatus != null) return false;
		if (!ListEquals.listEquals(product, _that.product)) return false;
		if (!ListEquals.listEquals(productQualifier, _that.productQualifier)) return false;
		if (totalPosition != null ? !totalPosition.equals(_that.totalPosition) : _that.totalPosition != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (contractReference != null ? contractReference.hashCode() : 0);
		_result = 31 * _result + (dateTime != null ? dateTime.hashCode() : 0);
		_result = 31 * _result + (executionReference != null ? executionReference.hashCode() : 0);
		_result = 31 * _result + (party != null ? party.hashCode() : 0);
		_result = 31 * _result + (positionStatus != null ? positionStatus.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (product != null ? product.hashCode() : 0);
		_result = 31 * _result + (productQualifier != null ? productQualifier.hashCode() : 0);
		_result = 31 * _result + (totalPosition != null ? totalPosition.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "AggregationParameters {" +
			"contractReference=" + this.contractReference + ", " +
			"dateTime=" + this.dateTime + ", " +
			"executionReference=" + this.executionReference + ", " +
			"party=" + this.party + ", " +
			"positionStatus=" + this.positionStatus + ", " +
			"product=" + this.product + ", " +
			"productQualifier=" + this.productQualifier + ", " +
			"totalPosition=" + this.totalPosition +
		'}';
	}
}
