package org.isda.cdm;

import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.Trade;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.AllocationOutcomeMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;
		import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;

/**
 * A class to specify the allocated outcome as the combination of the previous Trade, which is either an execution or a contract and which state is specified as &#39;Allocated&#39;, and a set of Trade(s) of the same execution or contract type as before allocation. The allocation to one single trade is deemed appropriate: e.g. the trade could be effectively allocated from the fund manager doing the execution to a single fund entity.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class AllocationOutcome extends RosettaModelObject {
	private final List<Trade> allocatedTrade;
	private final Trade originalTrade;
	private static AllocationOutcomeMeta metaData = new AllocationOutcomeMeta();
	
	AllocationOutcome(AllocationOutcomeBuilder builder) {
		this.allocatedTrade = ofNullable(builder.getAllocatedTrade()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.originalTrade = ofNullable(builder.getOriginalTrade()).map(f->f.build()).orElse(null);
	}
	
	@RosettaSynonym(value="allocatedTrade", source="Rosetta_Workbench")
	public final List<Trade> getAllocatedTrade() {
		return allocatedTrade;
	}
	
	@RosettaSynonym(value="originalTrade", source="Rosetta_Workbench")
	public final Trade getOriginalTrade() {
		return originalTrade;
	}
	
	
	@Override
	public RosettaMetaData<? extends AllocationOutcome> metaData() {
		return metaData;
	} 
	
	public AllocationOutcomeBuilder toBuilder() {
		AllocationOutcomeBuilder builder = new AllocationOutcomeBuilder();
		ofNullable(getAllocatedTrade()).ifPresent(allocatedTrade -> allocatedTrade.forEach(builder::addAllocatedTrade));
		ofNullable(getOriginalTrade()).ifPresent(builder::setOriginalTrade);
		return builder;
	}

	public static AllocationOutcomeBuilder builder() {
		return new AllocationOutcomeBuilder();
	}

	public static  class AllocationOutcomeBuilder extends RosettaModelObjectBuilder {
	
		protected List<Trade.TradeBuilder> allocatedTrade;
		protected Trade.TradeBuilder originalTrade;
	
		public AllocationOutcomeBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends AllocationOutcome> metaData() {
			return metaData;
		} 
	
		
		public List<Trade.TradeBuilder> getAllocatedTrade() {
			return allocatedTrade;
		}
		
		public Trade.TradeBuilder getOrCreateAllocatedTrade(int index) {
			if (allocatedTrade==null) {
				this.allocatedTrade = new ArrayList<>();
			}
			return getIndex(allocatedTrade, index, ()->new Trade.TradeBuilder());
		}
		
		
		public Trade.TradeBuilder getOriginalTrade() {
			return originalTrade;
		}
		
		public Trade.TradeBuilder getOrCreateOriginalTrade() {
			if (originalTrade!=null) {
				return originalTrade;
			}
			else return originalTrade = new Trade.TradeBuilder();
		}
		
	
		public AllocationOutcomeBuilder addAllocatedTrade(Trade allocatedTrade) {
			if(this.allocatedTrade == null){
				this.allocatedTrade = new ArrayList<>();
			}
			this.allocatedTrade.add(allocatedTrade.toBuilder());
			return this;
		}
		public AllocationOutcomeBuilder addAllocatedTrade(Trade allocatedTrade, int _idx) {
			if(this.allocatedTrade == null){
				this.allocatedTrade = new ArrayList<>();
			}
			getIndex(this.allocatedTrade, _idx, () -> allocatedTrade.toBuilder());
			this.allocatedTrade.set(_idx, allocatedTrade.toBuilder());
			return this;
		}
		public AllocationOutcomeBuilder addAllocatedTrade(List<Trade> allocatedTrades) {
			if(this.allocatedTrade == null){
				this.allocatedTrade = new ArrayList<>();
			}
			for (Trade toAdd : allocatedTrades) {
				this.allocatedTrade.add(toAdd.toBuilder());
			}
			return this;
		}
		public AllocationOutcomeBuilder addAllocatedTradeBuilder(Trade.TradeBuilder allocatedTrade) {
			if(this.allocatedTrade == null){
				this.allocatedTrade = new ArrayList<>();
			}
			this.allocatedTrade.add(allocatedTrade);
			return this;
		}
		
		public AllocationOutcomeBuilder clearAllocatedTrade() {
			this.allocatedTrade = null;
			return this;
		}
		public AllocationOutcomeBuilder setOriginalTrade(Trade originalTrade) {
			this.originalTrade = originalTrade.toBuilder();
			return this;
		}
		public AllocationOutcomeBuilder setOriginalTradeBuilder(Trade.TradeBuilder originalTrade) {
			this.originalTrade = originalTrade;
			return this;
		}
		
		public AllocationOutcome build() {
			return new AllocationOutcome(this);
		}
	
		@Override
		public AllocationOutcomeBuilder prune() {
			if (allocatedTrade!=null) allocatedTrade = allocatedTrade.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (originalTrade!=null && !originalTrade.prune().hasData()) originalTrade = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getAllocatedTrade()!=null && getAllocatedTrade().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getOriginalTrade()!=null && getOriginalTrade().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("allocatedTrade"), processor, Trade.class, allocatedTrade);
			processRosetta(path.newSubPath("originalTrade"), processor, Trade.class, originalTrade);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			AllocationOutcomeBuilder _that = (AllocationOutcomeBuilder) o;
		
			if (!ListEquals.listEquals(allocatedTrade, _that.allocatedTrade)) return false;
			if (originalTrade != null ? !originalTrade.equals(_that.originalTrade) : _that.originalTrade != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (allocatedTrade != null ? allocatedTrade.hashCode() : 0);
			_result = 31 * _result + (originalTrade != null ? originalTrade.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "AllocationOutcomeBuilder {" +
				"allocatedTrade=" + this.allocatedTrade + ", " +
				"originalTrade=" + this.originalTrade +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("allocatedTrade"), processor, Trade.class, allocatedTrade);
		processRosetta(path.newSubPath("originalTrade"), processor, Trade.class, originalTrade);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		AllocationOutcome _that = (AllocationOutcome) o;
	
		if (!ListEquals.listEquals(allocatedTrade, _that.allocatedTrade)) return false;
		if (originalTrade != null ? !originalTrade.equals(_that.originalTrade) : _that.originalTrade != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (allocatedTrade != null ? allocatedTrade.hashCode() : 0);
		_result = 31 * _result + (originalTrade != null ? originalTrade.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "AllocationOutcome {" +
			"allocatedTrade=" + this.allocatedTrade + ", " +
			"originalTrade=" + this.originalTrade +
		'}';
	}
}
