package org.isda.cdm;

import org.isda.cdm.AllocationOutcome;
import org.isda.cdm.Trade;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.AllocationPrimitiveMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * The primitive event to represent a split/allocation of a trade. As part of this primitive event the type of trade, either an execution or a contract, does not get altered. In the case of an execution, the further transformation of each split execution into a contract will be the purpose of the ContractFormation primitive.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class AllocationPrimitive extends RosettaModelObject {
	private final AllocationOutcome after;
	private final Trade before;
	private static AllocationPrimitiveMeta metaData = new AllocationPrimitiveMeta();
	
	AllocationPrimitive(AllocationPrimitiveBuilder builder) {
		this.after = ofNullable(builder.getAfter()).map(f->f.build()).orElse(null);
		this.before = ofNullable(builder.getBefore()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * Outcome of the allocation as a list of Trades, which is either a split execution or a split contract, plus a pointer to the previous execution or contract marked as &#39;Allocated&#39;.
	 */
	@RosettaSynonym(value="after", source="Rosetta_Workbench")
	public final AllocationOutcome getAfter() {
		return after;
	}
	
	/**
	 * The trade before allocation, which is either an execution if allocation happens before the contract is formed, or an already formed contract.
	 */
	@RosettaSynonym(value="before", source="Rosetta_Workbench")
	public final Trade getBefore() {
		return before;
	}
	
	
	@Override
	public RosettaMetaData<? extends AllocationPrimitive> metaData() {
		return metaData;
	} 
	
	public AllocationPrimitiveBuilder toBuilder() {
		AllocationPrimitiveBuilder builder = new AllocationPrimitiveBuilder();
		ofNullable(getAfter()).ifPresent(builder::setAfter);
		ofNullable(getBefore()).ifPresent(builder::setBefore);
		return builder;
	}

	public static AllocationPrimitiveBuilder builder() {
		return new AllocationPrimitiveBuilder();
	}

	public static  class AllocationPrimitiveBuilder extends RosettaModelObjectBuilder {
	
		protected AllocationOutcome.AllocationOutcomeBuilder after;
		protected Trade.TradeBuilder before;
	
		public AllocationPrimitiveBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends AllocationPrimitive> metaData() {
			return metaData;
		} 
	
		
		public AllocationOutcome.AllocationOutcomeBuilder getAfter() {
			return after;
		}
		
		public AllocationOutcome.AllocationOutcomeBuilder getOrCreateAfter() {
			if (after!=null) {
				return after;
			}
			else return after = new AllocationOutcome.AllocationOutcomeBuilder();
		}
		
		
		public Trade.TradeBuilder getBefore() {
			return before;
		}
		
		public Trade.TradeBuilder getOrCreateBefore() {
			if (before!=null) {
				return before;
			}
			else return before = new Trade.TradeBuilder();
		}
		
	
		public AllocationPrimitiveBuilder setAfter(AllocationOutcome after) {
			this.after = after.toBuilder();
			return this;
		}
		public AllocationPrimitiveBuilder setAfterBuilder(AllocationOutcome.AllocationOutcomeBuilder after) {
			this.after = after;
			return this;
		}
		public AllocationPrimitiveBuilder setBefore(Trade before) {
			this.before = before.toBuilder();
			return this;
		}
		public AllocationPrimitiveBuilder setBeforeBuilder(Trade.TradeBuilder before) {
			this.before = before;
			return this;
		}
		
		public AllocationPrimitive build() {
			return new AllocationPrimitive(this);
		}
	
		@Override
		public AllocationPrimitiveBuilder prune() {
			if (after!=null && !after.prune().hasData()) after = null;
			if (before!=null && !before.prune().hasData()) before = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getAfter()!=null && getAfter().hasData()) return true;
			if (getBefore()!=null && getBefore().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("after"), processor, AllocationOutcome.class, after);
			processRosetta(path.newSubPath("before"), processor, Trade.class, before);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			AllocationPrimitiveBuilder _that = (AllocationPrimitiveBuilder) o;
		
			if (after != null ? !after.equals(_that.after) : _that.after != null) return false;
			if (before != null ? !before.equals(_that.before) : _that.before != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (after != null ? after.hashCode() : 0);
			_result = 31 * _result + (before != null ? before.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "AllocationPrimitiveBuilder {" +
				"after=" + this.after + ", " +
				"before=" + this.before +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("after"), processor, AllocationOutcome.class, after);
		processRosetta(path.newSubPath("before"), processor, Trade.class, before);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		AllocationPrimitive _that = (AllocationPrimitive) o;
	
		if (after != null ? !after.equals(_that.after) : _that.after != null) return false;
		if (before != null ? !before.equals(_that.before) : _that.before != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (after != null ? after.hashCode() : 0);
		_result = 31 * _result + (before != null ? before.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "AllocationPrimitive {" +
			"after=" + this.after + ", " +
			"before=" + this.before +
		'}';
	}
}
