package org.isda.cdm;

import com.rosetta.model.lib.records.Date;
import java.lang.String;
import org.isda.cdm.AmendmentEffectiveDateEnum;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.AmendmentEffectiveDateMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import com.rosetta.model.lib.records.Date;
		import static java.util.Optional.ofNullable;

/**
 * A class to specify the effective date of the Amendment to Termination Currency. This date can be specified as either an actual date, a specific date (e.g. the annex date) or as a custom provision. 
 * 
 * @version 2.5.4
 */
@RosettaClass
public class AmendmentEffectiveDate extends RosettaModelObject {
	private final String customProvision;
	private final Date date;
	private final AmendmentEffectiveDateEnum specificDate;
	private static AmendmentEffectiveDateMeta metaData = new AmendmentEffectiveDateMeta();
	
	AmendmentEffectiveDate(AmendmentEffectiveDateBuilder builder) {
		this.customProvision = builder.getCustomProvision();
		this.date = builder.getDate();
		this.specificDate = builder.getSpecificDate();
	}
	
	/**
	 * The effective date of the Amendment to Termination Currency when specified as a non normalized custom provision.
	 */
	public final String getCustomProvision() {
		return customProvision;
	}
	
	/**
	 * The effective date of the Amendment to Termination Currency when specified as an actual date.
	 */
	public final Date getDate() {
		return date;
	}
	
	/**
	 * The effective date of the Amendment to Termination Currency when specified as a specific date (e.g. the annex date).
	 */
	@RosettaSynonym(value="annex_date", source="ISDA_Create_1_0")
	public final AmendmentEffectiveDateEnum getSpecificDate() {
		return specificDate;
	}
	
	
	@Override
	public RosettaMetaData<? extends AmendmentEffectiveDate> metaData() {
		return metaData;
	} 
	
	public AmendmentEffectiveDateBuilder toBuilder() {
		AmendmentEffectiveDateBuilder builder = new AmendmentEffectiveDateBuilder();
		ofNullable(getCustomProvision()).ifPresent(builder::setCustomProvision);
		ofNullable(getDate()).ifPresent(builder::setDate);
		ofNullable(getSpecificDate()).ifPresent(builder::setSpecificDate);
		return builder;
	}

	public static AmendmentEffectiveDateBuilder builder() {
		return new AmendmentEffectiveDateBuilder();
	}

	public static  class AmendmentEffectiveDateBuilder extends RosettaModelObjectBuilder {
	
		protected String customProvision;
		protected Date date;
		protected AmendmentEffectiveDateEnum specificDate;
	
		public AmendmentEffectiveDateBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends AmendmentEffectiveDate> metaData() {
			return metaData;
		} 
	
		
		public String getCustomProvision() {
			return customProvision;
		}
		
		
		public Date getDate() {
			return date;
		}
		
		
		public AmendmentEffectiveDateEnum getSpecificDate() {
			return specificDate;
		}
		
	
		public AmendmentEffectiveDateBuilder setCustomProvision(String customProvision) {
			this.customProvision = customProvision;
			return this;
		}
		public AmendmentEffectiveDateBuilder setDate(Date date) {
			this.date = date;
			return this;
		}
		public AmendmentEffectiveDateBuilder setSpecificDate(AmendmentEffectiveDateEnum specificDate) {
			this.specificDate = specificDate;
			return this;
		}
		
		public AmendmentEffectiveDate build() {
			return new AmendmentEffectiveDate(this);
		}
	
		@Override
		public AmendmentEffectiveDateBuilder prune() {
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getCustomProvision()!=null) return true;
			if (getDate()!=null) return true;
			if (getSpecificDate()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("customProvision"), String.class, customProvision, this);
			processor.processBasic(path.newSubPath("date"), Date.class, date, this);
			processor.processBasic(path.newSubPath("specificDate"), AmendmentEffectiveDateEnum.class, specificDate, this);
			
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			AmendmentEffectiveDateBuilder _that = (AmendmentEffectiveDateBuilder) o;
		
			if (customProvision != null ? !customProvision.equals(_that.customProvision) : _that.customProvision != null) return false;
			if (date != null ? !date.equals(_that.date) : _that.date != null) return false;
			if (specificDate != null ? !specificDate.equals(_that.specificDate) : _that.specificDate != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (customProvision != null ? customProvision.hashCode() : 0);
			_result = 31 * _result + (date != null ? date.hashCode() : 0);
			_result = 31 * _result + (specificDate != null ? specificDate.getClass().getName().hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "AmendmentEffectiveDateBuilder {" +
				"customProvision=" + this.customProvision + ", " +
				"date=" + this.date + ", " +
				"specificDate=" + this.specificDate +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("customProvision"), String.class, customProvision, this);
		processor.processBasic(path.newSubPath("date"), Date.class, date, this);
		processor.processBasic(path.newSubPath("specificDate"), AmendmentEffectiveDateEnum.class, specificDate, this);
		
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		AmendmentEffectiveDate _that = (AmendmentEffectiveDate) o;
	
		if (customProvision != null ? !customProvision.equals(_that.customProvision) : _that.customProvision != null) return false;
		if (date != null ? !date.equals(_that.date) : _that.date != null) return false;
		if (specificDate != null ? !specificDate.equals(_that.specificDate) : _that.specificDate != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (customProvision != null ? customProvision.hashCode() : 0);
		_result = 31 * _result + (date != null ? date.hashCode() : 0);
		_result = 31 * _result + (specificDate != null ? specificDate.getClass().getName().hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "AmendmentEffectiveDate {" +
			"customProvision=" + this.customProvision + ", " +
			"date=" + this.date + ", " +
			"specificDate=" + this.specificDate +
		'}';
	}
}
