package org.isda.cdm;

import org.isda.cdm.AdjustableOrRelativeDate;
import org.isda.cdm.AdjustableOrRelativeDates;
import org.isda.cdm.BusinessCenterTime;
import org.isda.cdm.ExerciseFeeSchedule;
import org.isda.cdm.MultipleExercise;
import org.isda.cdm.metafields.MetaFields;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.AmericanExerciseMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class defining the exercise period for an American style option together with any rules governing the notional amount of the underlying which can be exercised on any given exercise date and any associated exercise fees.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="AmericanExercise", source="FpML_5_10")
@RosettaSynonym(value="AmericanExercise", source="CME_SubmissionIRS_1_0")
@RosettaSynonym(value="AmericanExercise", source="DTCC_11_0")
@RosettaSynonym(value="AmericanExercise", source="DTCC_9_0")
@RosettaSynonym(value="AmericanExercise", source="CME_ClearedConfirm_1_17")
public class AmericanExercise extends RosettaModelObject implements GlobalKey {
	private final AdjustableOrRelativeDate commencementDate;
	private final BusinessCenterTime earliestExerciseTime;
	private final ExerciseFeeSchedule exerciseFeeSchedule;
	private final AdjustableOrRelativeDate expirationDate;
	private final BusinessCenterTime expirationTime;
	private final BusinessCenterTime latestExerciseTime;
	private final MetaFields meta;
	private final MultipleExercise multipleExercise;
	private final AdjustableOrRelativeDates relevantUnderlyingDate;
	private static AmericanExerciseMeta metaData = new AmericanExerciseMeta();
	
	AmericanExercise(AmericanExerciseBuilder builder) {
		this.commencementDate = ofNullable(builder.getCommencementDate()).map(f->f.build()).orElse(null);
		this.earliestExerciseTime = ofNullable(builder.getEarliestExerciseTime()).map(f->f.build()).orElse(null);
		this.exerciseFeeSchedule = ofNullable(builder.getExerciseFeeSchedule()).map(f->f.build()).orElse(null);
		this.expirationDate = ofNullable(builder.getExpirationDate()).map(f->f.build()).orElse(null);
		this.expirationTime = ofNullable(builder.getExpirationTime()).map(f->f.build()).orElse(null);
		this.latestExerciseTime = ofNullable(builder.getLatestExerciseTime()).map(f->f.build()).orElse(null);
		this.meta = ofNullable(builder.getMeta()).map(f->f.build()).orElse(null);
		this.multipleExercise = ofNullable(builder.getMultipleExercise()).map(f->f.build()).orElse(null);
		this.relevantUnderlyingDate = ofNullable(builder.getRelevantUnderlyingDate()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The first day of the exercise period for an American style option.
	 */
	@RosettaSynonym(value="commencementDate", source="FpML_5_10")
	@RosettaSynonym(value="commencementDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="commencementDate", source="DTCC_11_0")
	@RosettaSynonym(value="commencementDate", source="DTCC_9_0")
	@RosettaSynonym(value="commencementDate", source="CME_ClearedConfirm_1_17")
	public final AdjustableOrRelativeDate getCommencementDate() {
		return commencementDate;
	}
	
	/**
	 * The earliest time at which notice of exercise can be given by the buyer to the seller (or seller&#39;s agent) to, and including, the expiration date.
	 */
	@RosettaSynonym(value="earliestExerciseTime", source="FpML_5_10")
	@RosettaSynonym(value="earliestExerciseTime", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="earliestExerciseTime", source="DTCC_11_0")
	@RosettaSynonym(value="earliestExerciseTime", source="DTCC_9_0")
	@RosettaSynonym(value="earliestExerciseTime", source="CME_ClearedConfirm_1_17")
	public final BusinessCenterTime getEarliestExerciseTime() {
		return earliestExerciseTime;
	}
	
	/**
	 * The fees associated with an exercise date. The fees are conditional on the exercise occurring. The fees can be specified as actual currency amounts or as percentages of the notional amount being exercised.
	 */
	@RosettaSynonym(value="exerciseFeeSchedule", source="FpML_5_10")
	@RosettaSynonym(value="exerciseFeeSchedule", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="exerciseFeeSchedule", source="DTCC_11_0")
	@RosettaSynonym(value="exerciseFeeSchedule", source="DTCC_9_0")
	@RosettaSynonym(value="exerciseFeeSchedule", source="CME_ClearedConfirm_1_17")
	public final ExerciseFeeSchedule getExerciseFeeSchedule() {
		return exerciseFeeSchedule;
	}
	
	/**
	 * The last day within an exercise period for an American style option. For a European style option it is the only day within the exercise period.
	 */
	@RosettaSynonym(value="expirationDate", source="FpML_5_10")
	@RosettaSynonym(value="expirationDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="expirationDate", source="DTCC_11_0")
	@RosettaSynonym(value="expirationDate", source="DTCC_9_0")
	@RosettaSynonym(value="expirationDate", source="CME_ClearedConfirm_1_17")
	public final AdjustableOrRelativeDate getExpirationDate() {
		return expirationDate;
	}
	
	/**
	 * The latest time for exercise on expirationDate.
	 */
	@RosettaSynonym(value="expirationTime", source="FpML_5_10")
	@RosettaSynonym(value="expirationTime", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="expirationTime", source="DTCC_11_0")
	@RosettaSynonym(value="expirationTime", source="DTCC_9_0")
	@RosettaSynonym(value="expirationTime", source="CME_ClearedConfirm_1_17")
	public final BusinessCenterTime getExpirationTime() {
		return expirationTime;
	}
	
	/**
	 * For a Bermuda or American style option, the latest time on an exercise business day (excluding the expiration date) within the exercise period that notice can be given by the buyer to the seller or seller&#39;s agent. Notice of exercise given after this time will be deemed to have been given on the next exercise business day.
	 */
	@RosettaSynonym(value="latestExerciseTime", source="FpML_5_10")
	@RosettaSynonym(value="latestExerciseTime", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="latestExerciseTime", source="DTCC_11_0")
	@RosettaSynonym(value="latestExerciseTime", source="DTCC_9_0")
	@RosettaSynonym(value="latestExerciseTime", source="CME_ClearedConfirm_1_17")
	public final BusinessCenterTime getLatestExerciseTime() {
		return latestExerciseTime;
	}
	
	public final MetaFields getMeta() {
		return meta;
	}
	
	/**
	 * As defined in the 2000 ISDA Definitions, Section 12.4. Multiple Exercise, the buyer of the option has the right to exercise all or less than all the unexercised notional amount of the underlying swap on one or more days in the exercise period, but on any such day may not exercise less than the minimum notional amount or more that the maximum notional amount, and if an integral multiple amount is specified, the notional amount exercised must be equal to, or be an integral multiple of, the integral multiple amount.
	 */
	@RosettaSynonym(value="multipleExercise", source="FpML_5_10")
	@RosettaSynonym(value="multipleExercise", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="multipleExercise", source="DTCC_11_0")
	@RosettaSynonym(value="multipleExercise", source="DTCC_9_0")
	@RosettaSynonym(value="multipleExercise", source="CME_ClearedConfirm_1_17")
	public final MultipleExercise getMultipleExercise() {
		return multipleExercise;
	}
	
	/**
	 * The date on the underlying set by the exercise of an option. What this date is depends on the option (e.g. in a swaption it is the swap effective date, in an extendible/cancelable provision it is the swap termination date).
	 */
	@RosettaSynonym(value="relevantUnderlyingDate", source="FpML_5_10")
	@RosettaSynonym(value="relevantUnderlyingDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="relevantUnderlyingDate", source="DTCC_11_0")
	@RosettaSynonym(value="relevantUnderlyingDate", source="DTCC_9_0")
	@RosettaSynonym(value="relevantUnderlyingDate", source="CME_ClearedConfirm_1_17")
	public final AdjustableOrRelativeDates getRelevantUnderlyingDate() {
		return relevantUnderlyingDate;
	}
	
	
	@Override
	public RosettaMetaData<? extends AmericanExercise> metaData() {
		return metaData;
	} 
	
	public AmericanExerciseBuilder toBuilder() {
		AmericanExerciseBuilder builder = new AmericanExerciseBuilder();
		ofNullable(getCommencementDate()).ifPresent(builder::setCommencementDate);
		ofNullable(getEarliestExerciseTime()).ifPresent(builder::setEarliestExerciseTime);
		ofNullable(getExerciseFeeSchedule()).ifPresent(builder::setExerciseFeeSchedule);
		ofNullable(getExpirationDate()).ifPresent(builder::setExpirationDate);
		ofNullable(getExpirationTime()).ifPresent(builder::setExpirationTime);
		ofNullable(getLatestExerciseTime()).ifPresent(builder::setLatestExerciseTime);
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		ofNullable(getMultipleExercise()).ifPresent(builder::setMultipleExercise);
		ofNullable(getRelevantUnderlyingDate()).ifPresent(builder::setRelevantUnderlyingDate);
		return builder;
	}

	public static AmericanExerciseBuilder builder() {
		return new AmericanExerciseBuilder();
	}

	public static  class AmericanExerciseBuilder extends RosettaModelObjectBuilder implements GlobalKeyBuilder<AmericanExerciseBuilder> {
	
		protected AdjustableOrRelativeDate.AdjustableOrRelativeDateBuilder commencementDate;
		protected BusinessCenterTime.BusinessCenterTimeBuilder earliestExerciseTime;
		protected ExerciseFeeSchedule.ExerciseFeeScheduleBuilder exerciseFeeSchedule;
		protected AdjustableOrRelativeDate.AdjustableOrRelativeDateBuilder expirationDate;
		protected BusinessCenterTime.BusinessCenterTimeBuilder expirationTime;
		protected BusinessCenterTime.BusinessCenterTimeBuilder latestExerciseTime;
		protected MetaFields.MetaFieldsBuilder meta;
		protected MultipleExercise.MultipleExerciseBuilder multipleExercise;
		protected AdjustableOrRelativeDates.AdjustableOrRelativeDatesBuilder relevantUnderlyingDate;
	
		public AmericanExerciseBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends AmericanExercise> metaData() {
			return metaData;
		} 
	
		
		public AdjustableOrRelativeDate.AdjustableOrRelativeDateBuilder getCommencementDate() {
			return commencementDate;
		}
		
		public AdjustableOrRelativeDate.AdjustableOrRelativeDateBuilder getOrCreateCommencementDate() {
			if (commencementDate!=null) {
				return commencementDate;
			}
			else return commencementDate = new AdjustableOrRelativeDate.AdjustableOrRelativeDateBuilder();
		}
		
		
		public BusinessCenterTime.BusinessCenterTimeBuilder getEarliestExerciseTime() {
			return earliestExerciseTime;
		}
		
		public BusinessCenterTime.BusinessCenterTimeBuilder getOrCreateEarliestExerciseTime() {
			if (earliestExerciseTime!=null) {
				return earliestExerciseTime;
			}
			else return earliestExerciseTime = new BusinessCenterTime.BusinessCenterTimeBuilder();
		}
		
		
		public ExerciseFeeSchedule.ExerciseFeeScheduleBuilder getExerciseFeeSchedule() {
			return exerciseFeeSchedule;
		}
		
		public ExerciseFeeSchedule.ExerciseFeeScheduleBuilder getOrCreateExerciseFeeSchedule() {
			if (exerciseFeeSchedule!=null) {
				return exerciseFeeSchedule;
			}
			else return exerciseFeeSchedule = new ExerciseFeeSchedule.ExerciseFeeScheduleBuilder();
		}
		
		
		public AdjustableOrRelativeDate.AdjustableOrRelativeDateBuilder getExpirationDate() {
			return expirationDate;
		}
		
		public AdjustableOrRelativeDate.AdjustableOrRelativeDateBuilder getOrCreateExpirationDate() {
			if (expirationDate!=null) {
				return expirationDate;
			}
			else return expirationDate = new AdjustableOrRelativeDate.AdjustableOrRelativeDateBuilder();
		}
		
		
		public BusinessCenterTime.BusinessCenterTimeBuilder getExpirationTime() {
			return expirationTime;
		}
		
		public BusinessCenterTime.BusinessCenterTimeBuilder getOrCreateExpirationTime() {
			if (expirationTime!=null) {
				return expirationTime;
			}
			else return expirationTime = new BusinessCenterTime.BusinessCenterTimeBuilder();
		}
		
		
		public BusinessCenterTime.BusinessCenterTimeBuilder getLatestExerciseTime() {
			return latestExerciseTime;
		}
		
		public BusinessCenterTime.BusinessCenterTimeBuilder getOrCreateLatestExerciseTime() {
			if (latestExerciseTime!=null) {
				return latestExerciseTime;
			}
			else return latestExerciseTime = new BusinessCenterTime.BusinessCenterTimeBuilder();
		}
		
		
		public MetaFields.MetaFieldsBuilder getMeta() {
			return meta;
		}
		
		public MetaFields.MetaFieldsBuilder getOrCreateMeta() {
			if (meta!=null) {
				return meta;
			}
			else return meta = new MetaFields.MetaFieldsBuilder();
		}
		
		
		public MultipleExercise.MultipleExerciseBuilder getMultipleExercise() {
			return multipleExercise;
		}
		
		public MultipleExercise.MultipleExerciseBuilder getOrCreateMultipleExercise() {
			if (multipleExercise!=null) {
				return multipleExercise;
			}
			else return multipleExercise = new MultipleExercise.MultipleExerciseBuilder();
		}
		
		
		public AdjustableOrRelativeDates.AdjustableOrRelativeDatesBuilder getRelevantUnderlyingDate() {
			return relevantUnderlyingDate;
		}
		
		public AdjustableOrRelativeDates.AdjustableOrRelativeDatesBuilder getOrCreateRelevantUnderlyingDate() {
			if (relevantUnderlyingDate!=null) {
				return relevantUnderlyingDate;
			}
			else return relevantUnderlyingDate = new AdjustableOrRelativeDates.AdjustableOrRelativeDatesBuilder();
		}
		
	
		public AmericanExerciseBuilder setCommencementDate(AdjustableOrRelativeDate commencementDate) {
			this.commencementDate = commencementDate.toBuilder();
			return this;
		}
		public AmericanExerciseBuilder setCommencementDateBuilder(AdjustableOrRelativeDate.AdjustableOrRelativeDateBuilder commencementDate) {
			this.commencementDate = commencementDate;
			return this;
		}
		public AmericanExerciseBuilder setEarliestExerciseTime(BusinessCenterTime earliestExerciseTime) {
			this.earliestExerciseTime = earliestExerciseTime.toBuilder();
			return this;
		}
		public AmericanExerciseBuilder setEarliestExerciseTimeBuilder(BusinessCenterTime.BusinessCenterTimeBuilder earliestExerciseTime) {
			this.earliestExerciseTime = earliestExerciseTime;
			return this;
		}
		public AmericanExerciseBuilder setExerciseFeeSchedule(ExerciseFeeSchedule exerciseFeeSchedule) {
			this.exerciseFeeSchedule = exerciseFeeSchedule.toBuilder();
			return this;
		}
		public AmericanExerciseBuilder setExerciseFeeScheduleBuilder(ExerciseFeeSchedule.ExerciseFeeScheduleBuilder exerciseFeeSchedule) {
			this.exerciseFeeSchedule = exerciseFeeSchedule;
			return this;
		}
		public AmericanExerciseBuilder setExpirationDate(AdjustableOrRelativeDate expirationDate) {
			this.expirationDate = expirationDate.toBuilder();
			return this;
		}
		public AmericanExerciseBuilder setExpirationDateBuilder(AdjustableOrRelativeDate.AdjustableOrRelativeDateBuilder expirationDate) {
			this.expirationDate = expirationDate;
			return this;
		}
		public AmericanExerciseBuilder setExpirationTime(BusinessCenterTime expirationTime) {
			this.expirationTime = expirationTime.toBuilder();
			return this;
		}
		public AmericanExerciseBuilder setExpirationTimeBuilder(BusinessCenterTime.BusinessCenterTimeBuilder expirationTime) {
			this.expirationTime = expirationTime;
			return this;
		}
		public AmericanExerciseBuilder setLatestExerciseTime(BusinessCenterTime latestExerciseTime) {
			this.latestExerciseTime = latestExerciseTime.toBuilder();
			return this;
		}
		public AmericanExerciseBuilder setLatestExerciseTimeBuilder(BusinessCenterTime.BusinessCenterTimeBuilder latestExerciseTime) {
			this.latestExerciseTime = latestExerciseTime;
			return this;
		}
		public AmericanExerciseBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		public AmericanExerciseBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		public AmericanExerciseBuilder setMultipleExercise(MultipleExercise multipleExercise) {
			this.multipleExercise = multipleExercise.toBuilder();
			return this;
		}
		public AmericanExerciseBuilder setMultipleExerciseBuilder(MultipleExercise.MultipleExerciseBuilder multipleExercise) {
			this.multipleExercise = multipleExercise;
			return this;
		}
		public AmericanExerciseBuilder setRelevantUnderlyingDate(AdjustableOrRelativeDates relevantUnderlyingDate) {
			this.relevantUnderlyingDate = relevantUnderlyingDate.toBuilder();
			return this;
		}
		public AmericanExerciseBuilder setRelevantUnderlyingDateBuilder(AdjustableOrRelativeDates.AdjustableOrRelativeDatesBuilder relevantUnderlyingDate) {
			this.relevantUnderlyingDate = relevantUnderlyingDate;
			return this;
		}
		
		public AmericanExercise build() {
			return new AmericanExercise(this);
		}
	
		@Override
		public AmericanExerciseBuilder prune() {
			if (commencementDate!=null && !commencementDate.prune().hasData()) commencementDate = null;
			if (earliestExerciseTime!=null && !earliestExerciseTime.prune().hasData()) earliestExerciseTime = null;
			if (exerciseFeeSchedule!=null && !exerciseFeeSchedule.prune().hasData()) exerciseFeeSchedule = null;
			if (expirationDate!=null && !expirationDate.prune().hasData()) expirationDate = null;
			if (expirationTime!=null && !expirationTime.prune().hasData()) expirationTime = null;
			if (latestExerciseTime!=null && !latestExerciseTime.prune().hasData()) latestExerciseTime = null;
			if (meta!=null && !meta.prune().hasData()) meta = null;
			if (multipleExercise!=null && !multipleExercise.prune().hasData()) multipleExercise = null;
			if (relevantUnderlyingDate!=null && !relevantUnderlyingDate.prune().hasData()) relevantUnderlyingDate = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getCommencementDate()!=null && getCommencementDate().hasData()) return true;
			if (getEarliestExerciseTime()!=null && getEarliestExerciseTime().hasData()) return true;
			if (getExerciseFeeSchedule()!=null && getExerciseFeeSchedule().hasData()) return true;
			if (getExpirationDate()!=null && getExpirationDate().hasData()) return true;
			if (getExpirationTime()!=null && getExpirationTime().hasData()) return true;
			if (getLatestExerciseTime()!=null && getLatestExerciseTime().hasData()) return true;
			if (getMeta()!=null && getMeta().hasData()) return true;
			if (getMultipleExercise()!=null && getMultipleExercise().hasData()) return true;
			if (getRelevantUnderlyingDate()!=null && getRelevantUnderlyingDate().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("commencementDate"), processor, AdjustableOrRelativeDate.class, commencementDate);
			processRosetta(path.newSubPath("earliestExerciseTime"), processor, BusinessCenterTime.class, earliestExerciseTime);
			processRosetta(path.newSubPath("exerciseFeeSchedule"), processor, ExerciseFeeSchedule.class, exerciseFeeSchedule);
			processRosetta(path.newSubPath("expirationDate"), processor, AdjustableOrRelativeDate.class, expirationDate);
			processRosetta(path.newSubPath("expirationTime"), processor, BusinessCenterTime.class, expirationTime);
			processRosetta(path.newSubPath("latestExerciseTime"), processor, BusinessCenterTime.class, latestExerciseTime);
			processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
			processRosetta(path.newSubPath("multipleExercise"), processor, MultipleExercise.class, multipleExercise);
			processRosetta(path.newSubPath("relevantUnderlyingDate"), processor, AdjustableOrRelativeDates.class, relevantUnderlyingDate);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			AmericanExerciseBuilder _that = (AmericanExerciseBuilder) o;
		
			if (commencementDate != null ? !commencementDate.equals(_that.commencementDate) : _that.commencementDate != null) return false;
			if (earliestExerciseTime != null ? !earliestExerciseTime.equals(_that.earliestExerciseTime) : _that.earliestExerciseTime != null) return false;
			if (exerciseFeeSchedule != null ? !exerciseFeeSchedule.equals(_that.exerciseFeeSchedule) : _that.exerciseFeeSchedule != null) return false;
			if (expirationDate != null ? !expirationDate.equals(_that.expirationDate) : _that.expirationDate != null) return false;
			if (expirationTime != null ? !expirationTime.equals(_that.expirationTime) : _that.expirationTime != null) return false;
			if (latestExerciseTime != null ? !latestExerciseTime.equals(_that.latestExerciseTime) : _that.latestExerciseTime != null) return false;
			if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
			if (multipleExercise != null ? !multipleExercise.equals(_that.multipleExercise) : _that.multipleExercise != null) return false;
			if (relevantUnderlyingDate != null ? !relevantUnderlyingDate.equals(_that.relevantUnderlyingDate) : _that.relevantUnderlyingDate != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (commencementDate != null ? commencementDate.hashCode() : 0);
			_result = 31 * _result + (earliestExerciseTime != null ? earliestExerciseTime.hashCode() : 0);
			_result = 31 * _result + (exerciseFeeSchedule != null ? exerciseFeeSchedule.hashCode() : 0);
			_result = 31 * _result + (expirationDate != null ? expirationDate.hashCode() : 0);
			_result = 31 * _result + (expirationTime != null ? expirationTime.hashCode() : 0);
			_result = 31 * _result + (latestExerciseTime != null ? latestExerciseTime.hashCode() : 0);
			_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
			_result = 31 * _result + (multipleExercise != null ? multipleExercise.hashCode() : 0);
			_result = 31 * _result + (relevantUnderlyingDate != null ? relevantUnderlyingDate.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "AmericanExerciseBuilder {" +
				"commencementDate=" + this.commencementDate + ", " +
				"earliestExerciseTime=" + this.earliestExerciseTime + ", " +
				"exerciseFeeSchedule=" + this.exerciseFeeSchedule + ", " +
				"expirationDate=" + this.expirationDate + ", " +
				"expirationTime=" + this.expirationTime + ", " +
				"latestExerciseTime=" + this.latestExerciseTime + ", " +
				"meta=" + this.meta + ", " +
				"multipleExercise=" + this.multipleExercise + ", " +
				"relevantUnderlyingDate=" + this.relevantUnderlyingDate +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("commencementDate"), processor, AdjustableOrRelativeDate.class, commencementDate);
		processRosetta(path.newSubPath("earliestExerciseTime"), processor, BusinessCenterTime.class, earliestExerciseTime);
		processRosetta(path.newSubPath("exerciseFeeSchedule"), processor, ExerciseFeeSchedule.class, exerciseFeeSchedule);
		processRosetta(path.newSubPath("expirationDate"), processor, AdjustableOrRelativeDate.class, expirationDate);
		processRosetta(path.newSubPath("expirationTime"), processor, BusinessCenterTime.class, expirationTime);
		processRosetta(path.newSubPath("latestExerciseTime"), processor, BusinessCenterTime.class, latestExerciseTime);
		processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
		processRosetta(path.newSubPath("multipleExercise"), processor, MultipleExercise.class, multipleExercise);
		processRosetta(path.newSubPath("relevantUnderlyingDate"), processor, AdjustableOrRelativeDates.class, relevantUnderlyingDate);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		AmericanExercise _that = (AmericanExercise) o;
	
		if (commencementDate != null ? !commencementDate.equals(_that.commencementDate) : _that.commencementDate != null) return false;
		if (earliestExerciseTime != null ? !earliestExerciseTime.equals(_that.earliestExerciseTime) : _that.earliestExerciseTime != null) return false;
		if (exerciseFeeSchedule != null ? !exerciseFeeSchedule.equals(_that.exerciseFeeSchedule) : _that.exerciseFeeSchedule != null) return false;
		if (expirationDate != null ? !expirationDate.equals(_that.expirationDate) : _that.expirationDate != null) return false;
		if (expirationTime != null ? !expirationTime.equals(_that.expirationTime) : _that.expirationTime != null) return false;
		if (latestExerciseTime != null ? !latestExerciseTime.equals(_that.latestExerciseTime) : _that.latestExerciseTime != null) return false;
		if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
		if (multipleExercise != null ? !multipleExercise.equals(_that.multipleExercise) : _that.multipleExercise != null) return false;
		if (relevantUnderlyingDate != null ? !relevantUnderlyingDate.equals(_that.relevantUnderlyingDate) : _that.relevantUnderlyingDate != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (commencementDate != null ? commencementDate.hashCode() : 0);
		_result = 31 * _result + (earliestExerciseTime != null ? earliestExerciseTime.hashCode() : 0);
		_result = 31 * _result + (exerciseFeeSchedule != null ? exerciseFeeSchedule.hashCode() : 0);
		_result = 31 * _result + (expirationDate != null ? expirationDate.hashCode() : 0);
		_result = 31 * _result + (expirationTime != null ? expirationTime.hashCode() : 0);
		_result = 31 * _result + (latestExerciseTime != null ? latestExerciseTime.hashCode() : 0);
		_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
		_result = 31 * _result + (multipleExercise != null ? multipleExercise.hashCode() : 0);
		_result = 31 * _result + (relevantUnderlyingDate != null ? relevantUnderlyingDate.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "AmericanExercise {" +
			"commencementDate=" + this.commencementDate + ", " +
			"earliestExerciseTime=" + this.earliestExerciseTime + ", " +
			"exerciseFeeSchedule=" + this.exerciseFeeSchedule + ", " +
			"expirationDate=" + this.expirationDate + ", " +
			"expirationTime=" + this.expirationTime + ", " +
			"latestExerciseTime=" + this.latestExerciseTime + ", " +
			"meta=" + this.meta + ", " +
			"multipleExercise=" + this.multipleExercise + ", " +
			"relevantUnderlyingDate=" + this.relevantUnderlyingDate +
		'}';
	}
}
