package org.isda.cdm;

import java.lang.Boolean;
import java.lang.String;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.AppropriatedCollateralValuationMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class to specify the Valuation of Appropriate Collateral that is applicable to the English Law ISDA CSA. ISDA 2016 English Law Credit Support Deed for Initial Margin, paragraph 13, General Principles, (u): Valuation of Appropriated Collateral.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class AppropriatedCollateralValuation extends RosettaModelObject {
	private final String election;
	private final Boolean isSpecified;
	private static AppropriatedCollateralValuationMeta metaData = new AppropriatedCollateralValuationMeta();
	
	AppropriatedCollateralValuation(AppropriatedCollateralValuationBuilder builder) {
		this.election = builder.getElection();
		this.isSpecified = builder.getIsSpecified();
	}
	
	/**
	 * The parties&#39; election that qualify the Valuation of Appropriate Collateral in the case where it is deemed applicable.
	 */
	@RosettaSynonym(value="specify", source="ISDA_Create_1_0")
	public final String getElection() {
		return election;
	}
	
	/**
	 * The qualification of whether the Valuation of Appropriate Collateral provision is applicable (True) or not applicable (False).
	 */
	@RosettaSynonym(value="specified", source="ISDA_Create_1_0")
	public final Boolean getIsSpecified() {
		return isSpecified;
	}
	
	
	@Override
	public RosettaMetaData<? extends AppropriatedCollateralValuation> metaData() {
		return metaData;
	} 
	
	public AppropriatedCollateralValuationBuilder toBuilder() {
		AppropriatedCollateralValuationBuilder builder = new AppropriatedCollateralValuationBuilder();
		ofNullable(getElection()).ifPresent(builder::setElection);
		ofNullable(getIsSpecified()).ifPresent(builder::setIsSpecified);
		return builder;
	}

	public static AppropriatedCollateralValuationBuilder builder() {
		return new AppropriatedCollateralValuationBuilder();
	}

	public static  class AppropriatedCollateralValuationBuilder extends RosettaModelObjectBuilder {
	
		protected String election;
		protected Boolean isSpecified;
	
		public AppropriatedCollateralValuationBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends AppropriatedCollateralValuation> metaData() {
			return metaData;
		} 
	
		
		public String getElection() {
			return election;
		}
		
		
		public Boolean getIsSpecified() {
			return isSpecified;
		}
		
	
		public AppropriatedCollateralValuationBuilder setElection(String election) {
			this.election = election;
			return this;
		}
		public AppropriatedCollateralValuationBuilder setIsSpecified(Boolean isSpecified) {
			this.isSpecified = isSpecified;
			return this;
		}
		
		public AppropriatedCollateralValuation build() {
			return new AppropriatedCollateralValuation(this);
		}
	
		@Override
		public AppropriatedCollateralValuationBuilder prune() {
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getElection()!=null) return true;
			if (getIsSpecified()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("election"), String.class, election, this);
			processor.processBasic(path.newSubPath("isSpecified"), Boolean.class, isSpecified, this);
			
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			AppropriatedCollateralValuationBuilder _that = (AppropriatedCollateralValuationBuilder) o;
		
			if (election != null ? !election.equals(_that.election) : _that.election != null) return false;
			if (isSpecified != null ? !isSpecified.equals(_that.isSpecified) : _that.isSpecified != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (election != null ? election.hashCode() : 0);
			_result = 31 * _result + (isSpecified != null ? isSpecified.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "AppropriatedCollateralValuationBuilder {" +
				"election=" + this.election + ", " +
				"isSpecified=" + this.isSpecified +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("election"), String.class, election, this);
		processor.processBasic(path.newSubPath("isSpecified"), Boolean.class, isSpecified, this);
		
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		AppropriatedCollateralValuation _that = (AppropriatedCollateralValuation) o;
	
		if (election != null ? !election.equals(_that.election) : _that.election != null) return false;
		if (isSpecified != null ? !isSpecified.equals(_that.isSpecified) : _that.isSpecified != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (election != null ? election.hashCode() : 0);
		_result = 31 * _result + (isSpecified != null ? isSpecified.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "AppropriatedCollateralValuation {" +
			"election=" + this.election + ", " +
			"isSpecified=" + this.isSpecified +
		'}';
	}
}
