package org.isda.cdm;

import com.rosetta.model.lib.records.Date;
import java.lang.String;
import java.math.BigDecimal;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.AssetPoolMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import com.rosetta.model.lib.records.Date;
		import java.math.BigDecimal;
		import static java.util.Optional.ofNullable;

/**
 * Characterizes the asset pool behind an asset backed bond.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="AssetPool", source="FpML_5_10")
public class AssetPool extends RosettaModelObject {
	private final BigDecimal currentFactor;
	private final Date effectiveDate;
	private final BigDecimal initialFactor;
	private final String version;
	private static AssetPoolMeta metaData = new AssetPoolMeta();
	
	AssetPool(AssetPoolBuilder builder) {
		this.currentFactor = builder.getCurrentFactor();
		this.effectiveDate = builder.getEffectiveDate();
		this.initialFactor = builder.getInitialFactor();
		this.version = builder.getVersion();
	}
	
	/**
	 * The part of the mortgage that is currently outstanding. It is expressed similarly to the initial factor, as factor multiplier to the mortgage. This term is formally defined as part of the &#39;ISDA Standard Terms Supplement for use with credit derivatives transactions on mortgage-backed security with pas-as-you-go or physical settlement&#39;.
	 */
	@RosettaSynonym(value="currentFactor", source="FpML_5_10")
	@RosettaSynonym(value="currentFactor", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="currentFactor", source="DTCC_11_0")
	@RosettaSynonym(value="currentFactor", source="DTCC_9_0")
	@RosettaSynonym(value="currentFactor", source="CME_ClearedConfirm_1_17")
	public final BigDecimal getCurrentFactor() {
		return currentFactor;
	}
	
	/**
	 * Optionally it is possible to specify a version effective date when a version is supplied.
	 */
	@RosettaSynonym(value="effectiveDate", source="FpML_5_10")
	@RosettaSynonym(value="effectiveDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="effectiveDate", source="DTCC_11_0")
	@RosettaSynonym(value="effectiveDate", source="DTCC_9_0")
	@RosettaSynonym(value="effectiveDate", source="CME_ClearedConfirm_1_17")
	public final Date getEffectiveDate() {
		return effectiveDate;
	}
	
	/**
	 * The part of the mortgage that is outstanding on trade inception, i.e. has not been repaid yet as principal. It is expressed as a multiplier factor to the mortgage: 1 means that the whole mortgage amount is outstanding, 0.8 means that 20% has been repaid.
	 */
	@RosettaSynonym(value="initialFactor", source="FpML_5_10")
	@RosettaSynonym(value="initialFactor", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="initialFactor", source="DTCC_11_0")
	@RosettaSynonym(value="initialFactor", source="DTCC_9_0")
	@RosettaSynonym(value="initialFactor", source="CME_ClearedConfirm_1_17")
	public final BigDecimal getInitialFactor() {
		return initialFactor;
	}
	
	/**
	 * The asset pool version.
	 */
	@RosettaSynonym(value="version", source="FpML_5_10")
	@RosettaSynonym(value="version", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="version", source="DTCC_11_0")
	@RosettaSynonym(value="version", source="DTCC_9_0")
	@RosettaSynonym(value="version", source="CME_ClearedConfirm_1_17")
	public final String getVersion() {
		return version;
	}
	
	
	@Override
	public RosettaMetaData<? extends AssetPool> metaData() {
		return metaData;
	} 
	
	public AssetPoolBuilder toBuilder() {
		AssetPoolBuilder builder = new AssetPoolBuilder();
		ofNullable(getCurrentFactor()).ifPresent(builder::setCurrentFactor);
		ofNullable(getEffectiveDate()).ifPresent(builder::setEffectiveDate);
		ofNullable(getInitialFactor()).ifPresent(builder::setInitialFactor);
		ofNullable(getVersion()).ifPresent(builder::setVersion);
		return builder;
	}

	public static AssetPoolBuilder builder() {
		return new AssetPoolBuilder();
	}

	public static  class AssetPoolBuilder extends RosettaModelObjectBuilder {
	
		protected BigDecimal currentFactor;
		protected Date effectiveDate;
		protected BigDecimal initialFactor;
		protected String version;
	
		public AssetPoolBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends AssetPool> metaData() {
			return metaData;
		} 
	
		
		public BigDecimal getCurrentFactor() {
			return currentFactor;
		}
		
		
		public Date getEffectiveDate() {
			return effectiveDate;
		}
		
		
		public BigDecimal getInitialFactor() {
			return initialFactor;
		}
		
		
		public String getVersion() {
			return version;
		}
		
	
		public AssetPoolBuilder setCurrentFactor(BigDecimal currentFactor) {
			this.currentFactor = currentFactor;
			return this;
		}
		public AssetPoolBuilder setEffectiveDate(Date effectiveDate) {
			this.effectiveDate = effectiveDate;
			return this;
		}
		public AssetPoolBuilder setInitialFactor(BigDecimal initialFactor) {
			this.initialFactor = initialFactor;
			return this;
		}
		public AssetPoolBuilder setVersion(String version) {
			this.version = version;
			return this;
		}
		
		public AssetPool build() {
			return new AssetPool(this);
		}
	
		@Override
		public AssetPoolBuilder prune() {
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getCurrentFactor()!=null) return true;
			if (getEffectiveDate()!=null) return true;
			if (getInitialFactor()!=null) return true;
			if (getVersion()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("currentFactor"), BigDecimal.class, currentFactor, this);
			processor.processBasic(path.newSubPath("effectiveDate"), Date.class, effectiveDate, this);
			processor.processBasic(path.newSubPath("initialFactor"), BigDecimal.class, initialFactor, this);
			processor.processBasic(path.newSubPath("version"), String.class, version, this);
			
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			AssetPoolBuilder _that = (AssetPoolBuilder) o;
		
			if (currentFactor != null ? !currentFactor.equals(_that.currentFactor) : _that.currentFactor != null) return false;
			if (effectiveDate != null ? !effectiveDate.equals(_that.effectiveDate) : _that.effectiveDate != null) return false;
			if (initialFactor != null ? !initialFactor.equals(_that.initialFactor) : _that.initialFactor != null) return false;
			if (version != null ? !version.equals(_that.version) : _that.version != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (currentFactor != null ? currentFactor.hashCode() : 0);
			_result = 31 * _result + (effectiveDate != null ? effectiveDate.hashCode() : 0);
			_result = 31 * _result + (initialFactor != null ? initialFactor.hashCode() : 0);
			_result = 31 * _result + (version != null ? version.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "AssetPoolBuilder {" +
				"currentFactor=" + this.currentFactor + ", " +
				"effectiveDate=" + this.effectiveDate + ", " +
				"initialFactor=" + this.initialFactor + ", " +
				"version=" + this.version +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("currentFactor"), BigDecimal.class, currentFactor, this);
		processor.processBasic(path.newSubPath("effectiveDate"), Date.class, effectiveDate, this);
		processor.processBasic(path.newSubPath("initialFactor"), BigDecimal.class, initialFactor, this);
		processor.processBasic(path.newSubPath("version"), String.class, version, this);
		
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		AssetPool _that = (AssetPool) o;
	
		if (currentFactor != null ? !currentFactor.equals(_that.currentFactor) : _that.currentFactor != null) return false;
		if (effectiveDate != null ? !effectiveDate.equals(_that.effectiveDate) : _that.effectiveDate != null) return false;
		if (initialFactor != null ? !initialFactor.equals(_that.initialFactor) : _that.initialFactor != null) return false;
		if (version != null ? !version.equals(_that.version) : _that.version != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (currentFactor != null ? currentFactor.hashCode() : 0);
		_result = 31 * _result + (effectiveDate != null ? effectiveDate.hashCode() : 0);
		_result = 31 * _result + (initialFactor != null ? initialFactor.hashCode() : 0);
		_result = 31 * _result + (version != null ? version.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "AssetPool {" +
			"currentFactor=" + this.currentFactor + ", " +
			"effectiveDate=" + this.effectiveDate + ", " +
			"initialFactor=" + this.initialFactor + ", " +
			"version=" + this.version +
		'}';
	}
}
