package org.isda.cdm;

import java.math.BigDecimal;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.AutomaticExerciseMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.math.BigDecimal;
		import static java.util.Optional.ofNullable;

/**
 * A type to define automatic exercise of a swaption. With automatic exercise the option is deemed to have exercised if it is in the money by more than the threshold amount on the exercise date.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="AutomaticExercise", source="FpML_5_10")
public class AutomaticExercise extends RosettaModelObject {
	private final BigDecimal thresholdRate;
	private static AutomaticExerciseMeta metaData = new AutomaticExerciseMeta();
	
	AutomaticExercise(AutomaticExerciseBuilder builder) {
		this.thresholdRate = builder.getThresholdRate();
	}
	
	/**
	 * A threshold rate. The threshold of 0.10% would be represented as 0.001
	 */
	@RosettaSynonym(value="thresholdRate", source="FpML_5_10")
	@RosettaSynonym(value="thresholdRate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="thresholdRate", source="DTCC_11_0")
	@RosettaSynonym(value="thresholdRate", source="DTCC_9_0")
	@RosettaSynonym(value="thresholdRate", source="CME_ClearedConfirm_1_17")
	public final BigDecimal getThresholdRate() {
		return thresholdRate;
	}
	
	
	@Override
	public RosettaMetaData<? extends AutomaticExercise> metaData() {
		return metaData;
	} 
	
	public AutomaticExerciseBuilder toBuilder() {
		AutomaticExerciseBuilder builder = new AutomaticExerciseBuilder();
		ofNullable(getThresholdRate()).ifPresent(builder::setThresholdRate);
		return builder;
	}

	public static AutomaticExerciseBuilder builder() {
		return new AutomaticExerciseBuilder();
	}

	public static  class AutomaticExerciseBuilder extends RosettaModelObjectBuilder {
	
		protected BigDecimal thresholdRate;
	
		public AutomaticExerciseBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends AutomaticExercise> metaData() {
			return metaData;
		} 
	
		
		public BigDecimal getThresholdRate() {
			return thresholdRate;
		}
		
	
		public AutomaticExerciseBuilder setThresholdRate(BigDecimal thresholdRate) {
			this.thresholdRate = thresholdRate;
			return this;
		}
		
		public AutomaticExercise build() {
			return new AutomaticExercise(this);
		}
	
		@Override
		public AutomaticExerciseBuilder prune() {
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getThresholdRate()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("thresholdRate"), BigDecimal.class, thresholdRate, this);
			
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			AutomaticExerciseBuilder _that = (AutomaticExerciseBuilder) o;
		
			if (thresholdRate != null ? !thresholdRate.equals(_that.thresholdRate) : _that.thresholdRate != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (thresholdRate != null ? thresholdRate.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "AutomaticExerciseBuilder {" +
				"thresholdRate=" + this.thresholdRate +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("thresholdRate"), BigDecimal.class, thresholdRate, this);
		
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		AutomaticExercise _that = (AutomaticExercise) o;
	
		if (thresholdRate != null ? !thresholdRate.equals(_that.thresholdRate) : _that.thresholdRate != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (thresholdRate != null ? thresholdRate.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "AutomaticExercise {" +
			"thresholdRate=" + this.thresholdRate +
		'}';
	}
}
