package org.isda.cdm;

import org.isda.cdm.TriggerEvent;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.BarrierMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * As per ISDA 2002 Definitions.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="Barrier", source="FpML_5_10")
public class Barrier extends RosettaModelObject {
	private final TriggerEvent barrierCap;
	private final TriggerEvent barrierFloor;
	private static BarrierMeta metaData = new BarrierMeta();
	
	Barrier(BarrierBuilder builder) {
		this.barrierCap = ofNullable(builder.getBarrierCap()).map(f->f.build()).orElse(null);
		this.barrierFloor = ofNullable(builder.getBarrierFloor()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * A trigger level approached from beneath.
	 */
	@RosettaSynonym(value="barrierCap", source="FpML_5_10")
	@RosettaSynonym(value="barrierCap", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="barrierCap", source="DTCC_11_0")
	@RosettaSynonym(value="barrierCap", source="DTCC_9_0")
	@RosettaSynonym(value="barrierCap", source="CME_ClearedConfirm_1_17")
	public final TriggerEvent getBarrierCap() {
		return barrierCap;
	}
	
	/**
	 * A trigger level approached from above.
	 */
	@RosettaSynonym(value="barrierFloor", source="FpML_5_10")
	@RosettaSynonym(value="barrierFloor", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="barrierFloor", source="DTCC_11_0")
	@RosettaSynonym(value="barrierFloor", source="DTCC_9_0")
	@RosettaSynonym(value="barrierFloor", source="CME_ClearedConfirm_1_17")
	public final TriggerEvent getBarrierFloor() {
		return barrierFloor;
	}
	
	
	@Override
	public RosettaMetaData<? extends Barrier> metaData() {
		return metaData;
	} 
	
	public BarrierBuilder toBuilder() {
		BarrierBuilder builder = new BarrierBuilder();
		ofNullable(getBarrierCap()).ifPresent(builder::setBarrierCap);
		ofNullable(getBarrierFloor()).ifPresent(builder::setBarrierFloor);
		return builder;
	}

	public static BarrierBuilder builder() {
		return new BarrierBuilder();
	}

	public static  class BarrierBuilder extends RosettaModelObjectBuilder {
	
		protected TriggerEvent.TriggerEventBuilder barrierCap;
		protected TriggerEvent.TriggerEventBuilder barrierFloor;
	
		public BarrierBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends Barrier> metaData() {
			return metaData;
		} 
	
		
		public TriggerEvent.TriggerEventBuilder getBarrierCap() {
			return barrierCap;
		}
		
		public TriggerEvent.TriggerEventBuilder getOrCreateBarrierCap() {
			if (barrierCap!=null) {
				return barrierCap;
			}
			else return barrierCap = new TriggerEvent.TriggerEventBuilder();
		}
		
		
		public TriggerEvent.TriggerEventBuilder getBarrierFloor() {
			return barrierFloor;
		}
		
		public TriggerEvent.TriggerEventBuilder getOrCreateBarrierFloor() {
			if (barrierFloor!=null) {
				return barrierFloor;
			}
			else return barrierFloor = new TriggerEvent.TriggerEventBuilder();
		}
		
	
		public BarrierBuilder setBarrierCap(TriggerEvent barrierCap) {
			this.barrierCap = barrierCap.toBuilder();
			return this;
		}
		public BarrierBuilder setBarrierCapBuilder(TriggerEvent.TriggerEventBuilder barrierCap) {
			this.barrierCap = barrierCap;
			return this;
		}
		public BarrierBuilder setBarrierFloor(TriggerEvent barrierFloor) {
			this.barrierFloor = barrierFloor.toBuilder();
			return this;
		}
		public BarrierBuilder setBarrierFloorBuilder(TriggerEvent.TriggerEventBuilder barrierFloor) {
			this.barrierFloor = barrierFloor;
			return this;
		}
		
		public Barrier build() {
			return new Barrier(this);
		}
	
		@Override
		public BarrierBuilder prune() {
			if (barrierCap!=null && !barrierCap.prune().hasData()) barrierCap = null;
			if (barrierFloor!=null && !barrierFloor.prune().hasData()) barrierFloor = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getBarrierCap()!=null && getBarrierCap().hasData()) return true;
			if (getBarrierFloor()!=null && getBarrierFloor().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("barrierCap"), processor, TriggerEvent.class, barrierCap);
			processRosetta(path.newSubPath("barrierFloor"), processor, TriggerEvent.class, barrierFloor);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			BarrierBuilder _that = (BarrierBuilder) o;
		
			if (barrierCap != null ? !barrierCap.equals(_that.barrierCap) : _that.barrierCap != null) return false;
			if (barrierFloor != null ? !barrierFloor.equals(_that.barrierFloor) : _that.barrierFloor != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (barrierCap != null ? barrierCap.hashCode() : 0);
			_result = 31 * _result + (barrierFloor != null ? barrierFloor.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "BarrierBuilder {" +
				"barrierCap=" + this.barrierCap + ", " +
				"barrierFloor=" + this.barrierFloor +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("barrierCap"), processor, TriggerEvent.class, barrierCap);
		processRosetta(path.newSubPath("barrierFloor"), processor, TriggerEvent.class, barrierFloor);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		Barrier _that = (Barrier) o;
	
		if (barrierCap != null ? !barrierCap.equals(_that.barrierCap) : _that.barrierCap != null) return false;
		if (barrierFloor != null ? !barrierFloor.equals(_that.barrierFloor) : _that.barrierFloor != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (barrierCap != null ? barrierCap.hashCode() : 0);
		_result = 31 * _result + (barrierFloor != null ? barrierFloor.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "Barrier {" +
			"barrierCap=" + this.barrierCap + ", " +
			"barrierFloor=" + this.barrierFloor +
		'}';
	}
}
