package org.isda.cdm;

import org.isda.cdm.Bond;
import org.isda.cdm.ConvertibleBond;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.BondChoiceModelMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * Either a bond or convertible bond.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class BondChoiceModel extends RosettaModelObject {
	private final Bond bond;
	private final ConvertibleBond convertibleBond;
	private static BondChoiceModelMeta metaData = new BondChoiceModelMeta();
	
	BondChoiceModel(BondChoiceModelBuilder builder) {
		this.bond = ofNullable(builder.getBond()).map(f->f.build()).orElse(null);
		this.convertibleBond = ofNullable(builder.getConvertibleBond()).map(f->f.build()).orElse(null);
	}
	
	public final Bond getBond() {
		return bond;
	}
	
	public final ConvertibleBond getConvertibleBond() {
		return convertibleBond;
	}
	
	
	@Override
	public RosettaMetaData<? extends BondChoiceModel> metaData() {
		return metaData;
	} 
	
	public BondChoiceModelBuilder toBuilder() {
		BondChoiceModelBuilder builder = new BondChoiceModelBuilder();
		ofNullable(getBond()).ifPresent(builder::setBond);
		ofNullable(getConvertibleBond()).ifPresent(builder::setConvertibleBond);
		return builder;
	}

	public static BondChoiceModelBuilder builder() {
		return new BondChoiceModelBuilder();
	}

	public static  class BondChoiceModelBuilder extends RosettaModelObjectBuilder {
	
		protected Bond.BondBuilder bond;
		protected ConvertibleBond.ConvertibleBondBuilder convertibleBond;
	
		public BondChoiceModelBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends BondChoiceModel> metaData() {
			return metaData;
		} 
	
		
		public Bond.BondBuilder getBond() {
			return bond;
		}
		
		public Bond.BondBuilder getOrCreateBond() {
			if (bond!=null) {
				return bond;
			}
			else return bond = new Bond.BondBuilder();
		}
		
		
		public ConvertibleBond.ConvertibleBondBuilder getConvertibleBond() {
			return convertibleBond;
		}
		
		public ConvertibleBond.ConvertibleBondBuilder getOrCreateConvertibleBond() {
			if (convertibleBond!=null) {
				return convertibleBond;
			}
			else return convertibleBond = new ConvertibleBond.ConvertibleBondBuilder();
		}
		
	
		public BondChoiceModelBuilder setBond(Bond bond) {
			this.bond = bond.toBuilder();
			return this;
		}
		public BondChoiceModelBuilder setBondBuilder(Bond.BondBuilder bond) {
			this.bond = bond;
			return this;
		}
		public BondChoiceModelBuilder setConvertibleBond(ConvertibleBond convertibleBond) {
			this.convertibleBond = convertibleBond.toBuilder();
			return this;
		}
		public BondChoiceModelBuilder setConvertibleBondBuilder(ConvertibleBond.ConvertibleBondBuilder convertibleBond) {
			this.convertibleBond = convertibleBond;
			return this;
		}
		
		public BondChoiceModel build() {
			return new BondChoiceModel(this);
		}
	
		@Override
		public BondChoiceModelBuilder prune() {
			if (bond!=null && !bond.prune().hasData()) bond = null;
			if (convertibleBond!=null && !convertibleBond.prune().hasData()) convertibleBond = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getBond()!=null && getBond().hasData()) return true;
			if (getConvertibleBond()!=null && getConvertibleBond().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("bond"), processor, Bond.class, bond);
			processRosetta(path.newSubPath("convertibleBond"), processor, ConvertibleBond.class, convertibleBond);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			BondChoiceModelBuilder _that = (BondChoiceModelBuilder) o;
		
			if (bond != null ? !bond.equals(_that.bond) : _that.bond != null) return false;
			if (convertibleBond != null ? !convertibleBond.equals(_that.convertibleBond) : _that.convertibleBond != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (bond != null ? bond.hashCode() : 0);
			_result = 31 * _result + (convertibleBond != null ? convertibleBond.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "BondChoiceModelBuilder {" +
				"bond=" + this.bond + ", " +
				"convertibleBond=" + this.convertibleBond +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("bond"), processor, Bond.class, bond);
		processRosetta(path.newSubPath("convertibleBond"), processor, ConvertibleBond.class, convertibleBond);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		BondChoiceModel _that = (BondChoiceModel) o;
	
		if (bond != null ? !bond.equals(_that.bond) : _that.bond != null) return false;
		if (convertibleBond != null ? !convertibleBond.equals(_that.convertibleBond) : _that.convertibleBond != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (bond != null ? bond.hashCode() : 0);
		_result = 31 * _result + (convertibleBond != null ? convertibleBond.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "BondChoiceModel {" +
			"bond=" + this.bond + ", " +
			"convertibleBond=" + this.convertibleBond +
		'}';
	}
}
