package org.isda.cdm;

import org.isda.cdm.OptionStrike;
import org.isda.cdm.ReferenceSwapCurve;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.BondOptionStrikeMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class to specify the strike of a bond or convertible bond option.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="BondOptionStrike", source="FpML_5_10")
public class BondOptionStrike extends RosettaModelObject {
	private final OptionStrike price;
	private final ReferenceSwapCurve referenceSwapCurve;
	private static BondOptionStrikeMeta metaData = new BondOptionStrikeMeta();
	
	BondOptionStrike(BondOptionStrikeBuilder builder) {
		this.price = ofNullable(builder.getPrice()).map(f->f.build()).orElse(null);
		this.referenceSwapCurve = ofNullable(builder.getReferenceSwapCurve()).map(f->f.build()).orElse(null);
	}
	
	@RosettaSynonym(value="price", source="FpML_5_10")
	@RosettaSynonym(value="price", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="price", source="DTCC_11_0")
	@RosettaSynonym(value="price", source="DTCC_9_0")
	@RosettaSynonym(value="price", source="CME_ClearedConfirm_1_17")
	public final OptionStrike getPrice() {
		return price;
	}
	
	/**
	 * The strike of an option when expressed by reference to a swap curve. (Typically the case for a convertible bond option.)
	 */
	@RosettaSynonym(value="referenceSwapCurve", source="FpML_5_10")
	@RosettaSynonym(value="referenceSwapCurve", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="referenceSwapCurve", source="DTCC_11_0")
	@RosettaSynonym(value="referenceSwapCurve", source="DTCC_9_0")
	@RosettaSynonym(value="referenceSwapCurve", source="CME_ClearedConfirm_1_17")
	public final ReferenceSwapCurve getReferenceSwapCurve() {
		return referenceSwapCurve;
	}
	
	
	@Override
	public RosettaMetaData<? extends BondOptionStrike> metaData() {
		return metaData;
	} 
	
	public BondOptionStrikeBuilder toBuilder() {
		BondOptionStrikeBuilder builder = new BondOptionStrikeBuilder();
		ofNullable(getPrice()).ifPresent(builder::setPrice);
		ofNullable(getReferenceSwapCurve()).ifPresent(builder::setReferenceSwapCurve);
		return builder;
	}

	public static BondOptionStrikeBuilder builder() {
		return new BondOptionStrikeBuilder();
	}

	public static  class BondOptionStrikeBuilder extends RosettaModelObjectBuilder {
	
		protected OptionStrike.OptionStrikeBuilder price;
		protected ReferenceSwapCurve.ReferenceSwapCurveBuilder referenceSwapCurve;
	
		public BondOptionStrikeBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends BondOptionStrike> metaData() {
			return metaData;
		} 
	
		
		public OptionStrike.OptionStrikeBuilder getPrice() {
			return price;
		}
		
		public OptionStrike.OptionStrikeBuilder getOrCreatePrice() {
			if (price!=null) {
				return price;
			}
			else return price = new OptionStrike.OptionStrikeBuilder();
		}
		
		
		public ReferenceSwapCurve.ReferenceSwapCurveBuilder getReferenceSwapCurve() {
			return referenceSwapCurve;
		}
		
		public ReferenceSwapCurve.ReferenceSwapCurveBuilder getOrCreateReferenceSwapCurve() {
			if (referenceSwapCurve!=null) {
				return referenceSwapCurve;
			}
			else return referenceSwapCurve = new ReferenceSwapCurve.ReferenceSwapCurveBuilder();
		}
		
	
		public BondOptionStrikeBuilder setPrice(OptionStrike price) {
			this.price = price.toBuilder();
			return this;
		}
		public BondOptionStrikeBuilder setPriceBuilder(OptionStrike.OptionStrikeBuilder price) {
			this.price = price;
			return this;
		}
		public BondOptionStrikeBuilder setReferenceSwapCurve(ReferenceSwapCurve referenceSwapCurve) {
			this.referenceSwapCurve = referenceSwapCurve.toBuilder();
			return this;
		}
		public BondOptionStrikeBuilder setReferenceSwapCurveBuilder(ReferenceSwapCurve.ReferenceSwapCurveBuilder referenceSwapCurve) {
			this.referenceSwapCurve = referenceSwapCurve;
			return this;
		}
		
		public BondOptionStrike build() {
			return new BondOptionStrike(this);
		}
	
		@Override
		public BondOptionStrikeBuilder prune() {
			if (price!=null && !price.prune().hasData()) price = null;
			if (referenceSwapCurve!=null && !referenceSwapCurve.prune().hasData()) referenceSwapCurve = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getPrice()!=null && getPrice().hasData()) return true;
			if (getReferenceSwapCurve()!=null && getReferenceSwapCurve().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("price"), processor, OptionStrike.class, price);
			processRosetta(path.newSubPath("referenceSwapCurve"), processor, ReferenceSwapCurve.class, referenceSwapCurve);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			BondOptionStrikeBuilder _that = (BondOptionStrikeBuilder) o;
		
			if (price != null ? !price.equals(_that.price) : _that.price != null) return false;
			if (referenceSwapCurve != null ? !referenceSwapCurve.equals(_that.referenceSwapCurve) : _that.referenceSwapCurve != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (price != null ? price.hashCode() : 0);
			_result = 31 * _result + (referenceSwapCurve != null ? referenceSwapCurve.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "BondOptionStrikeBuilder {" +
				"price=" + this.price + ", " +
				"referenceSwapCurve=" + this.referenceSwapCurve +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("price"), processor, OptionStrike.class, price);
		processRosetta(path.newSubPath("referenceSwapCurve"), processor, ReferenceSwapCurve.class, referenceSwapCurve);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		BondOptionStrike _that = (BondOptionStrike) o;
	
		if (price != null ? !price.equals(_that.price) : _that.price != null) return false;
		if (referenceSwapCurve != null ? !referenceSwapCurve.equals(_that.referenceSwapCurve) : _that.referenceSwapCurve != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (price != null ? price.hashCode() : 0);
		_result = 31 * _result + (referenceSwapCurve != null ? referenceSwapCurve.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "BondOptionStrike {" +
			"price=" + this.price + ", " +
			"referenceSwapCurve=" + this.referenceSwapCurve +
		'}';
	}
}
