package org.isda.cdm;

import java.math.BigDecimal;
import org.isda.cdm.CleanOrDirtyPrice;
import org.isda.cdm.RelativePrice;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.BondPriceAndYieldModelMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;
		import java.math.BigDecimal;

/**
 * Bond price and yield valuation model for the security leg in a securities financing transaction, closely modelled onto the BondPriceAndYield.model in FpML.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class BondPriceAndYieldModel extends RosettaModelObject {
	private final BigDecimal allInPrice;
	private final CleanOrDirtyPrice cleanOrDirtyPrice;
	private final BigDecimal inflationFactor;
	private final RelativePrice relativePrice;
	private final BigDecimal yieldToMaturity;
	private static BondPriceAndYieldModelMeta metaData = new BondPriceAndYieldModelMeta();
	
	BondPriceAndYieldModel(BondPriceAndYieldModelBuilder builder) {
		this.allInPrice = builder.getAllInPrice();
		this.cleanOrDirtyPrice = ofNullable(builder.getCleanOrDirtyPrice()).map(f->f.build()).orElse(null);
		this.inflationFactor = builder.getInflationFactor();
		this.relativePrice = ofNullable(builder.getRelativePrice()).map(f->f.build()).orElse(null);
		this.yieldToMaturity = builder.getYieldToMaturity();
	}
	
	/**
	 * Bond all-in-price which is a price that includes all relevant price adjustments (i.e. accrued interest, haircut or margin ratio, inflation factor,etc.). It expresses a price in terms of percentage of nominal amount.
	 */
	public final BigDecimal getAllInPrice() {
		return allInPrice;
	}
	
	/**
	 * Either the clean or dirty price of the bond.
	 */
	public final CleanOrDirtyPrice getCleanOrDirtyPrice() {
		return cleanOrDirtyPrice;
	}
	
	/**
	 * The inflation factor is specified for inflation-linked products which require some additional elements to calculate prices correctly.
	 */
	public final BigDecimal getInflationFactor() {
		return inflationFactor;
	}
	
	/**
	 * Bond price relative to a Benchmark.
	 */
	public final RelativePrice getRelativePrice() {
		return relativePrice;
	}
	
	/**
	 * Price specified as a yield to maturity.
	 */
	public final BigDecimal getYieldToMaturity() {
		return yieldToMaturity;
	}
	
	
	@Override
	public RosettaMetaData<? extends BondPriceAndYieldModel> metaData() {
		return metaData;
	} 
	
	public BondPriceAndYieldModelBuilder toBuilder() {
		BondPriceAndYieldModelBuilder builder = new BondPriceAndYieldModelBuilder();
		ofNullable(getAllInPrice()).ifPresent(builder::setAllInPrice);
		ofNullable(getCleanOrDirtyPrice()).ifPresent(builder::setCleanOrDirtyPrice);
		ofNullable(getInflationFactor()).ifPresent(builder::setInflationFactor);
		ofNullable(getRelativePrice()).ifPresent(builder::setRelativePrice);
		ofNullable(getYieldToMaturity()).ifPresent(builder::setYieldToMaturity);
		return builder;
	}

	public static BondPriceAndYieldModelBuilder builder() {
		return new BondPriceAndYieldModelBuilder();
	}

	public static  class BondPriceAndYieldModelBuilder extends RosettaModelObjectBuilder {
	
		protected BigDecimal allInPrice;
		protected CleanOrDirtyPrice.CleanOrDirtyPriceBuilder cleanOrDirtyPrice;
		protected BigDecimal inflationFactor;
		protected RelativePrice.RelativePriceBuilder relativePrice;
		protected BigDecimal yieldToMaturity;
	
		public BondPriceAndYieldModelBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends BondPriceAndYieldModel> metaData() {
			return metaData;
		} 
	
		
		public BigDecimal getAllInPrice() {
			return allInPrice;
		}
		
		
		public CleanOrDirtyPrice.CleanOrDirtyPriceBuilder getCleanOrDirtyPrice() {
			return cleanOrDirtyPrice;
		}
		
		public CleanOrDirtyPrice.CleanOrDirtyPriceBuilder getOrCreateCleanOrDirtyPrice() {
			if (cleanOrDirtyPrice!=null) {
				return cleanOrDirtyPrice;
			}
			else return cleanOrDirtyPrice = new CleanOrDirtyPrice.CleanOrDirtyPriceBuilder();
		}
		
		
		public BigDecimal getInflationFactor() {
			return inflationFactor;
		}
		
		
		public RelativePrice.RelativePriceBuilder getRelativePrice() {
			return relativePrice;
		}
		
		public RelativePrice.RelativePriceBuilder getOrCreateRelativePrice() {
			if (relativePrice!=null) {
				return relativePrice;
			}
			else return relativePrice = new RelativePrice.RelativePriceBuilder();
		}
		
		
		public BigDecimal getYieldToMaturity() {
			return yieldToMaturity;
		}
		
	
		public BondPriceAndYieldModelBuilder setAllInPrice(BigDecimal allInPrice) {
			this.allInPrice = allInPrice;
			return this;
		}
		public BondPriceAndYieldModelBuilder setCleanOrDirtyPrice(CleanOrDirtyPrice cleanOrDirtyPrice) {
			this.cleanOrDirtyPrice = cleanOrDirtyPrice.toBuilder();
			return this;
		}
		public BondPriceAndYieldModelBuilder setCleanOrDirtyPriceBuilder(CleanOrDirtyPrice.CleanOrDirtyPriceBuilder cleanOrDirtyPrice) {
			this.cleanOrDirtyPrice = cleanOrDirtyPrice;
			return this;
		}
		public BondPriceAndYieldModelBuilder setInflationFactor(BigDecimal inflationFactor) {
			this.inflationFactor = inflationFactor;
			return this;
		}
		public BondPriceAndYieldModelBuilder setRelativePrice(RelativePrice relativePrice) {
			this.relativePrice = relativePrice.toBuilder();
			return this;
		}
		public BondPriceAndYieldModelBuilder setRelativePriceBuilder(RelativePrice.RelativePriceBuilder relativePrice) {
			this.relativePrice = relativePrice;
			return this;
		}
		public BondPriceAndYieldModelBuilder setYieldToMaturity(BigDecimal yieldToMaturity) {
			this.yieldToMaturity = yieldToMaturity;
			return this;
		}
		
		public BondPriceAndYieldModel build() {
			return new BondPriceAndYieldModel(this);
		}
	
		@Override
		public BondPriceAndYieldModelBuilder prune() {
			if (cleanOrDirtyPrice!=null && !cleanOrDirtyPrice.prune().hasData()) cleanOrDirtyPrice = null;
			if (relativePrice!=null && !relativePrice.prune().hasData()) relativePrice = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getAllInPrice()!=null) return true;
			if (getCleanOrDirtyPrice()!=null && getCleanOrDirtyPrice().hasData()) return true;
			if (getInflationFactor()!=null) return true;
			if (getRelativePrice()!=null && getRelativePrice().hasData()) return true;
			if (getYieldToMaturity()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("allInPrice"), BigDecimal.class, allInPrice, this);
			processor.processBasic(path.newSubPath("inflationFactor"), BigDecimal.class, inflationFactor, this);
			processor.processBasic(path.newSubPath("yieldToMaturity"), BigDecimal.class, yieldToMaturity, this);
			
			processRosetta(path.newSubPath("cleanOrDirtyPrice"), processor, CleanOrDirtyPrice.class, cleanOrDirtyPrice);
			processRosetta(path.newSubPath("relativePrice"), processor, RelativePrice.class, relativePrice);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			BondPriceAndYieldModelBuilder _that = (BondPriceAndYieldModelBuilder) o;
		
			if (allInPrice != null ? !allInPrice.equals(_that.allInPrice) : _that.allInPrice != null) return false;
			if (cleanOrDirtyPrice != null ? !cleanOrDirtyPrice.equals(_that.cleanOrDirtyPrice) : _that.cleanOrDirtyPrice != null) return false;
			if (inflationFactor != null ? !inflationFactor.equals(_that.inflationFactor) : _that.inflationFactor != null) return false;
			if (relativePrice != null ? !relativePrice.equals(_that.relativePrice) : _that.relativePrice != null) return false;
			if (yieldToMaturity != null ? !yieldToMaturity.equals(_that.yieldToMaturity) : _that.yieldToMaturity != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (allInPrice != null ? allInPrice.hashCode() : 0);
			_result = 31 * _result + (cleanOrDirtyPrice != null ? cleanOrDirtyPrice.hashCode() : 0);
			_result = 31 * _result + (inflationFactor != null ? inflationFactor.hashCode() : 0);
			_result = 31 * _result + (relativePrice != null ? relativePrice.hashCode() : 0);
			_result = 31 * _result + (yieldToMaturity != null ? yieldToMaturity.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "BondPriceAndYieldModelBuilder {" +
				"allInPrice=" + this.allInPrice + ", " +
				"cleanOrDirtyPrice=" + this.cleanOrDirtyPrice + ", " +
				"inflationFactor=" + this.inflationFactor + ", " +
				"relativePrice=" + this.relativePrice + ", " +
				"yieldToMaturity=" + this.yieldToMaturity +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("allInPrice"), BigDecimal.class, allInPrice, this);
		processor.processBasic(path.newSubPath("inflationFactor"), BigDecimal.class, inflationFactor, this);
		processor.processBasic(path.newSubPath("yieldToMaturity"), BigDecimal.class, yieldToMaturity, this);
		
		processRosetta(path.newSubPath("cleanOrDirtyPrice"), processor, CleanOrDirtyPrice.class, cleanOrDirtyPrice);
		processRosetta(path.newSubPath("relativePrice"), processor, RelativePrice.class, relativePrice);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		BondPriceAndYieldModel _that = (BondPriceAndYieldModel) o;
	
		if (allInPrice != null ? !allInPrice.equals(_that.allInPrice) : _that.allInPrice != null) return false;
		if (cleanOrDirtyPrice != null ? !cleanOrDirtyPrice.equals(_that.cleanOrDirtyPrice) : _that.cleanOrDirtyPrice != null) return false;
		if (inflationFactor != null ? !inflationFactor.equals(_that.inflationFactor) : _that.inflationFactor != null) return false;
		if (relativePrice != null ? !relativePrice.equals(_that.relativePrice) : _that.relativePrice != null) return false;
		if (yieldToMaturity != null ? !yieldToMaturity.equals(_that.yieldToMaturity) : _that.yieldToMaturity != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (allInPrice != null ? allInPrice.hashCode() : 0);
		_result = 31 * _result + (cleanOrDirtyPrice != null ? cleanOrDirtyPrice.hashCode() : 0);
		_result = 31 * _result + (inflationFactor != null ? inflationFactor.hashCode() : 0);
		_result = 31 * _result + (relativePrice != null ? relativePrice.hashCode() : 0);
		_result = 31 * _result + (yieldToMaturity != null ? yieldToMaturity.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "BondPriceAndYieldModel {" +
			"allInPrice=" + this.allInPrice + ", " +
			"cleanOrDirtyPrice=" + this.cleanOrDirtyPrice + ", " +
			"inflationFactor=" + this.inflationFactor + ", " +
			"relativePrice=" + this.relativePrice + ", " +
			"yieldToMaturity=" + this.yieldToMaturity +
		'}';
	}
}
