package org.isda.cdm;

import java.lang.Boolean;
import org.isda.cdm.ProductIdentifier;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.BondReferenceMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * Reference to a bond underlier to represent an asset swap or Condition Precedent Bond.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="BondReference", source="FpML_5_10")
public class BondReference extends RosettaModelObject {
	private final ProductIdentifier bond;
	private final Boolean conditionPrecedentBond;
	private final Boolean discrepancyClause;
	private static BondReferenceMeta metaData = new BondReferenceMeta();
	
	BondReference(BondReferenceBuilder builder) {
		this.bond = ofNullable(builder.getBond()).map(f->f.build()).orElse(null);
		this.conditionPrecedentBond = builder.getConditionPrecedentBond();
		this.discrepancyClause = builder.getDiscrepancyClause();
	}
	
	/**
	 * Reference to a bond underlier.
	 */
	@RosettaSynonym(value="bond", source="FpML_5_10")
	@RosettaSynonym(value="bond", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="bond", source="DTCC_11_0")
	@RosettaSynonym(value="bond", source="DTCC_9_0")
	@RosettaSynonym(value="bond", source="CME_ClearedConfirm_1_17")
	public final ProductIdentifier getBond() {
		return bond;
	}
	
	/**
	 * To indicate whether the Condition Precedent Bond is applicable. The swap contract is only valid if the bond is issued and if there is any dispute over the terms of fixed stream then the bond terms would be used.
	 */
	@RosettaSynonym(value="conditionPrecedentBond", source="FpML_5_10")
	@RosettaSynonym(value="conditionPrecedentBond", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="conditionPrecedentBond", source="DTCC_11_0")
	@RosettaSynonym(value="conditionPrecedentBond", source="DTCC_9_0")
	@RosettaSynonym(value="conditionPrecedentBond", source="CME_ClearedConfirm_1_17")
	public final Boolean getConditionPrecedentBond() {
		return conditionPrecedentBond;
	}
	
	/**
	 * To indicate whether the Discrepancy Clause is applicable.
	 */
	@RosettaSynonym(value="discrepancyClause", source="FpML_5_10")
	@RosettaSynonym(value="discrepancyClause", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="discrepancyClause", source="DTCC_11_0")
	@RosettaSynonym(value="discrepancyClause", source="DTCC_9_0")
	@RosettaSynonym(value="discrepancyClause", source="CME_ClearedConfirm_1_17")
	public final Boolean getDiscrepancyClause() {
		return discrepancyClause;
	}
	
	
	@Override
	public RosettaMetaData<? extends BondReference> metaData() {
		return metaData;
	} 
	
	public BondReferenceBuilder toBuilder() {
		BondReferenceBuilder builder = new BondReferenceBuilder();
		ofNullable(getBond()).ifPresent(builder::setBond);
		ofNullable(getConditionPrecedentBond()).ifPresent(builder::setConditionPrecedentBond);
		ofNullable(getDiscrepancyClause()).ifPresent(builder::setDiscrepancyClause);
		return builder;
	}

	public static BondReferenceBuilder builder() {
		return new BondReferenceBuilder();
	}

	public static  class BondReferenceBuilder extends RosettaModelObjectBuilder {
	
		protected ProductIdentifier.ProductIdentifierBuilder bond;
		protected Boolean conditionPrecedentBond;
		protected Boolean discrepancyClause;
	
		public BondReferenceBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends BondReference> metaData() {
			return metaData;
		} 
	
		
		public ProductIdentifier.ProductIdentifierBuilder getBond() {
			return bond;
		}
		
		public ProductIdentifier.ProductIdentifierBuilder getOrCreateBond() {
			if (bond!=null) {
				return bond;
			}
			else return bond = new ProductIdentifier.ProductIdentifierBuilder();
		}
		
		
		public Boolean getConditionPrecedentBond() {
			return conditionPrecedentBond;
		}
		
		
		public Boolean getDiscrepancyClause() {
			return discrepancyClause;
		}
		
	
		public BondReferenceBuilder setBond(ProductIdentifier bond) {
			this.bond = bond.toBuilder();
			return this;
		}
		public BondReferenceBuilder setBondBuilder(ProductIdentifier.ProductIdentifierBuilder bond) {
			this.bond = bond;
			return this;
		}
		public BondReferenceBuilder setConditionPrecedentBond(Boolean conditionPrecedentBond) {
			this.conditionPrecedentBond = conditionPrecedentBond;
			return this;
		}
		public BondReferenceBuilder setDiscrepancyClause(Boolean discrepancyClause) {
			this.discrepancyClause = discrepancyClause;
			return this;
		}
		
		public BondReference build() {
			return new BondReference(this);
		}
	
		@Override
		public BondReferenceBuilder prune() {
			if (bond!=null && !bond.prune().hasData()) bond = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getBond()!=null && getBond().hasData()) return true;
			if (getConditionPrecedentBond()!=null) return true;
			if (getDiscrepancyClause()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("conditionPrecedentBond"), Boolean.class, conditionPrecedentBond, this);
			processor.processBasic(path.newSubPath("discrepancyClause"), Boolean.class, discrepancyClause, this);
			
			processRosetta(path.newSubPath("bond"), processor, ProductIdentifier.class, bond);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			BondReferenceBuilder _that = (BondReferenceBuilder) o;
		
			if (bond != null ? !bond.equals(_that.bond) : _that.bond != null) return false;
			if (conditionPrecedentBond != null ? !conditionPrecedentBond.equals(_that.conditionPrecedentBond) : _that.conditionPrecedentBond != null) return false;
			if (discrepancyClause != null ? !discrepancyClause.equals(_that.discrepancyClause) : _that.discrepancyClause != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (bond != null ? bond.hashCode() : 0);
			_result = 31 * _result + (conditionPrecedentBond != null ? conditionPrecedentBond.hashCode() : 0);
			_result = 31 * _result + (discrepancyClause != null ? discrepancyClause.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "BondReferenceBuilder {" +
				"bond=" + this.bond + ", " +
				"conditionPrecedentBond=" + this.conditionPrecedentBond + ", " +
				"discrepancyClause=" + this.discrepancyClause +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("conditionPrecedentBond"), Boolean.class, conditionPrecedentBond, this);
		processor.processBasic(path.newSubPath("discrepancyClause"), Boolean.class, discrepancyClause, this);
		
		processRosetta(path.newSubPath("bond"), processor, ProductIdentifier.class, bond);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		BondReference _that = (BondReference) o;
	
		if (bond != null ? !bond.equals(_that.bond) : _that.bond != null) return false;
		if (conditionPrecedentBond != null ? !conditionPrecedentBond.equals(_that.conditionPrecedentBond) : _that.conditionPrecedentBond != null) return false;
		if (discrepancyClause != null ? !discrepancyClause.equals(_that.discrepancyClause) : _that.discrepancyClause != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (bond != null ? bond.hashCode() : 0);
		_result = 31 * _result + (conditionPrecedentBond != null ? conditionPrecedentBond.hashCode() : 0);
		_result = 31 * _result + (discrepancyClause != null ? discrepancyClause.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "BondReference {" +
			"bond=" + this.bond + ", " +
			"conditionPrecedentBond=" + this.conditionPrecedentBond + ", " +
			"discrepancyClause=" + this.discrepancyClause +
		'}';
	}
}
