package org.isda.cdm;

import org.isda.cdm.BondPriceAndYieldModel;
import org.isda.cdm.Money;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.BondValuationModelMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * Bond valuation model for the security leg in a securities financing transaction, closely modelled onto the BondCollateral.model in FpML.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class BondValuationModel extends RosettaModelObject {
	private final Money accrualsAmount;
	private final BondPriceAndYieldModel bondPriceAndYieldModel;
	private final Money nominalAmount;
	private static BondValuationModelMeta metaData = new BondValuationModelMeta();
	
	BondValuationModel(BondValuationModelBuilder builder) {
		this.accrualsAmount = ofNullable(builder.getAccrualsAmount()).map(f->f.build()).orElse(null);
		this.bondPriceAndYieldModel = ofNullable(builder.getBondPriceAndYieldModel()).map(f->f.build()).orElse(null);
		this.nominalAmount = ofNullable(builder.getNominalAmount()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * Accruals amount for the bond in the security leg
	 */
	public final Money getAccrualsAmount() {
		return accrualsAmount;
	}
	
	/**
	 * Price and yield model for valuing a bond security leg.
	 */
	public final BondPriceAndYieldModel getBondPriceAndYieldModel() {
		return bondPriceAndYieldModel;
	}
	
	/**
	 * The quantity of the underlier expressed as a nominal amount.
	 */
	@RosettaSynonym(value="nominalAmount", source="FpML_5_10")
	public final Money getNominalAmount() {
		return nominalAmount;
	}
	
	
	@Override
	public RosettaMetaData<? extends BondValuationModel> metaData() {
		return metaData;
	} 
	
	public BondValuationModelBuilder toBuilder() {
		BondValuationModelBuilder builder = new BondValuationModelBuilder();
		ofNullable(getAccrualsAmount()).ifPresent(builder::setAccrualsAmount);
		ofNullable(getBondPriceAndYieldModel()).ifPresent(builder::setBondPriceAndYieldModel);
		ofNullable(getNominalAmount()).ifPresent(builder::setNominalAmount);
		return builder;
	}

	public static BondValuationModelBuilder builder() {
		return new BondValuationModelBuilder();
	}

	public static  class BondValuationModelBuilder extends RosettaModelObjectBuilder {
	
		protected Money.MoneyBuilder accrualsAmount;
		protected BondPriceAndYieldModel.BondPriceAndYieldModelBuilder bondPriceAndYieldModel;
		protected Money.MoneyBuilder nominalAmount;
	
		public BondValuationModelBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends BondValuationModel> metaData() {
			return metaData;
		} 
	
		
		public Money.MoneyBuilder getAccrualsAmount() {
			return accrualsAmount;
		}
		
		public Money.MoneyBuilder getOrCreateAccrualsAmount() {
			if (accrualsAmount!=null) {
				return accrualsAmount;
			}
			else return accrualsAmount = new Money.MoneyBuilder();
		}
		
		
		public BondPriceAndYieldModel.BondPriceAndYieldModelBuilder getBondPriceAndYieldModel() {
			return bondPriceAndYieldModel;
		}
		
		public BondPriceAndYieldModel.BondPriceAndYieldModelBuilder getOrCreateBondPriceAndYieldModel() {
			if (bondPriceAndYieldModel!=null) {
				return bondPriceAndYieldModel;
			}
			else return bondPriceAndYieldModel = new BondPriceAndYieldModel.BondPriceAndYieldModelBuilder();
		}
		
		
		public Money.MoneyBuilder getNominalAmount() {
			return nominalAmount;
		}
		
		public Money.MoneyBuilder getOrCreateNominalAmount() {
			if (nominalAmount!=null) {
				return nominalAmount;
			}
			else return nominalAmount = new Money.MoneyBuilder();
		}
		
	
		public BondValuationModelBuilder setAccrualsAmount(Money accrualsAmount) {
			this.accrualsAmount = accrualsAmount.toBuilder();
			return this;
		}
		public BondValuationModelBuilder setAccrualsAmountBuilder(Money.MoneyBuilder accrualsAmount) {
			this.accrualsAmount = accrualsAmount;
			return this;
		}
		public BondValuationModelBuilder setBondPriceAndYieldModel(BondPriceAndYieldModel bondPriceAndYieldModel) {
			this.bondPriceAndYieldModel = bondPriceAndYieldModel.toBuilder();
			return this;
		}
		public BondValuationModelBuilder setBondPriceAndYieldModelBuilder(BondPriceAndYieldModel.BondPriceAndYieldModelBuilder bondPriceAndYieldModel) {
			this.bondPriceAndYieldModel = bondPriceAndYieldModel;
			return this;
		}
		public BondValuationModelBuilder setNominalAmount(Money nominalAmount) {
			this.nominalAmount = nominalAmount.toBuilder();
			return this;
		}
		public BondValuationModelBuilder setNominalAmountBuilder(Money.MoneyBuilder nominalAmount) {
			this.nominalAmount = nominalAmount;
			return this;
		}
		
		public BondValuationModel build() {
			return new BondValuationModel(this);
		}
	
		@Override
		public BondValuationModelBuilder prune() {
			if (accrualsAmount!=null && !accrualsAmount.prune().hasData()) accrualsAmount = null;
			if (bondPriceAndYieldModel!=null && !bondPriceAndYieldModel.prune().hasData()) bondPriceAndYieldModel = null;
			if (nominalAmount!=null && !nominalAmount.prune().hasData()) nominalAmount = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getAccrualsAmount()!=null && getAccrualsAmount().hasData()) return true;
			if (getBondPriceAndYieldModel()!=null && getBondPriceAndYieldModel().hasData()) return true;
			if (getNominalAmount()!=null && getNominalAmount().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("accrualsAmount"), processor, Money.class, accrualsAmount);
			processRosetta(path.newSubPath("bondPriceAndYieldModel"), processor, BondPriceAndYieldModel.class, bondPriceAndYieldModel);
			processRosetta(path.newSubPath("nominalAmount"), processor, Money.class, nominalAmount);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			BondValuationModelBuilder _that = (BondValuationModelBuilder) o;
		
			if (accrualsAmount != null ? !accrualsAmount.equals(_that.accrualsAmount) : _that.accrualsAmount != null) return false;
			if (bondPriceAndYieldModel != null ? !bondPriceAndYieldModel.equals(_that.bondPriceAndYieldModel) : _that.bondPriceAndYieldModel != null) return false;
			if (nominalAmount != null ? !nominalAmount.equals(_that.nominalAmount) : _that.nominalAmount != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (accrualsAmount != null ? accrualsAmount.hashCode() : 0);
			_result = 31 * _result + (bondPriceAndYieldModel != null ? bondPriceAndYieldModel.hashCode() : 0);
			_result = 31 * _result + (nominalAmount != null ? nominalAmount.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "BondValuationModelBuilder {" +
				"accrualsAmount=" + this.accrualsAmount + ", " +
				"bondPriceAndYieldModel=" + this.bondPriceAndYieldModel + ", " +
				"nominalAmount=" + this.nominalAmount +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("accrualsAmount"), processor, Money.class, accrualsAmount);
		processRosetta(path.newSubPath("bondPriceAndYieldModel"), processor, BondPriceAndYieldModel.class, bondPriceAndYieldModel);
		processRosetta(path.newSubPath("nominalAmount"), processor, Money.class, nominalAmount);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		BondValuationModel _that = (BondValuationModel) o;
	
		if (accrualsAmount != null ? !accrualsAmount.equals(_that.accrualsAmount) : _that.accrualsAmount != null) return false;
		if (bondPriceAndYieldModel != null ? !bondPriceAndYieldModel.equals(_that.bondPriceAndYieldModel) : _that.bondPriceAndYieldModel != null) return false;
		if (nominalAmount != null ? !nominalAmount.equals(_that.nominalAmount) : _that.nominalAmount != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (accrualsAmount != null ? accrualsAmount.hashCode() : 0);
		_result = 31 * _result + (bondPriceAndYieldModel != null ? bondPriceAndYieldModel.hashCode() : 0);
		_result = 31 * _result + (nominalAmount != null ? nominalAmount.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "BondValuationModel {" +
			"accrualsAmount=" + this.accrualsAmount + ", " +
			"bondPriceAndYieldModel=" + this.bondPriceAndYieldModel + ", " +
			"nominalAmount=" + this.nominalAmount +
		'}';
	}
}
