package org.isda.cdm;

import org.isda.cdm.BusinessCenters;
import org.isda.cdm.BusinessDayConventionEnum;


import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.BusinessDateRangeMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;
		import com.rosetta.model.lib.records.Date;

/**
 * A class defining a range of contiguous business days by defining an unadjusted first date, an unadjusted last date and a business day convention and business centers for adjusting the first and last dates if they would otherwise fall on a non business day in the specified business centers. The days between the first and last date must also be good business days in the specified centers to be counted in the range.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="BusinessDateRange", source="FpML_5_10")
public class BusinessDateRange extends DateRange {
	private final BusinessCenters businessCenters;
	private final BusinessDayConventionEnum businessDayConvention;
	private static BusinessDateRangeMeta metaData = new BusinessDateRangeMeta();
	
	BusinessDateRange(BusinessDateRangeBuilder builder) {
		super(builder);
		this.businessCenters = ofNullable(builder.getBusinessCenters()).map(f->f.build()).orElse(null);
		this.businessDayConvention = builder.getBusinessDayConvention();
	}
	
	/**
	 * The business center(s), specified either explicitly or by reference to those specified somewhere else in the instance document.
	 */
	public final BusinessCenters getBusinessCenters() {
		return businessCenters;
	}
	
	/**
	 * The convention for adjusting a date if it would otherwise fall on a day that is not a business day, as specified by an ISDA convention (e.g. Following, Precedent).
	 */
	@RosettaSynonym(value="businessDayConvention", source="FpML_5_10")
	@RosettaSynonym(value="businessDayConvention", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="businessDayConvention", source="DTCC_11_0")
	@RosettaSynonym(value="businessDayConvention", source="DTCC_9_0")
	@RosettaSynonym(value="businessDayConvention", source="CME_ClearedConfirm_1_17")
	public final BusinessDayConventionEnum getBusinessDayConvention() {
		return businessDayConvention;
	}
	
	
	@Override
	public RosettaMetaData<? extends BusinessDateRange> metaData() {
		return metaData;
	} 
	
	public BusinessDateRangeBuilder toBuilder() {
		BusinessDateRangeBuilder builder = new BusinessDateRangeBuilder();
		ofNullable(getBusinessCenters()).ifPresent(builder::setBusinessCenters);
		ofNullable(getBusinessDayConvention()).ifPresent(builder::setBusinessDayConvention);
		ofNullable(getUnadjustedFirstDate()).ifPresent(builder::setUnadjustedFirstDate);
		ofNullable(getUnadjustedLastDate()).ifPresent(builder::setUnadjustedLastDate);
		return builder;
	}

	public static BusinessDateRangeBuilder builder() {
		return new BusinessDateRangeBuilder();
	}

	public static  class BusinessDateRangeBuilder extends DateRangeBuilder {
	
		protected BusinessCenters.BusinessCentersBuilder businessCenters;
		protected BusinessDayConventionEnum businessDayConvention;
	
		public BusinessDateRangeBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends BusinessDateRange> metaData() {
			return metaData;
		} 
	
		
		public BusinessCenters.BusinessCentersBuilder getBusinessCenters() {
			return businessCenters;
		}
		
		public BusinessCenters.BusinessCentersBuilder getOrCreateBusinessCenters() {
			if (businessCenters!=null) {
				return businessCenters;
			}
			else return businessCenters = new BusinessCenters.BusinessCentersBuilder();
		}
		
		
		public BusinessDayConventionEnum getBusinessDayConvention() {
			return businessDayConvention;
		}
		
	
		public BusinessDateRangeBuilder setBusinessCenters(BusinessCenters businessCenters) {
			this.businessCenters = businessCenters.toBuilder();
			return this;
		}
		public BusinessDateRangeBuilder setBusinessCentersBuilder(BusinessCenters.BusinessCentersBuilder businessCenters) {
			this.businessCenters = businessCenters;
			return this;
		}
		public BusinessDateRangeBuilder setBusinessDayConvention(BusinessDayConventionEnum businessDayConvention) {
			this.businessDayConvention = businessDayConvention;
			return this;
		}
		@Override public BusinessDateRangeBuilder setUnadjustedFirstDate(Date unadjustedFirstDate) {
			this.unadjustedFirstDate = unadjustedFirstDate;
			return this;
		}
		@Override public BusinessDateRangeBuilder setUnadjustedLastDate(Date unadjustedLastDate) {
			this.unadjustedLastDate = unadjustedLastDate;
			return this;
		}
		
		public BusinessDateRange build() {
			return new BusinessDateRange(this);
		}
	
		@Override
		public BusinessDateRangeBuilder prune() {
			super.prune();
			if (businessCenters!=null && !businessCenters.prune().hasData()) businessCenters = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (super.hasData()) return true;
			if (getBusinessCenters()!=null && getBusinessCenters().hasData()) return true;
			if (getBusinessDayConvention()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
			super.process(path, processor);
		
			processor.processBasic(path.newSubPath("businessDayConvention"), BusinessDayConventionEnum.class, businessDayConvention, this);
			
			processRosetta(path.newSubPath("businessCenters"), processor, BusinessCenters.class, businessCenters);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
			if (!super.equals(o)) return false;
		
			BusinessDateRangeBuilder _that = (BusinessDateRangeBuilder) o;
		
			if (businessCenters != null ? !businessCenters.equals(_that.businessCenters) : _that.businessCenters != null) return false;
			if (businessDayConvention != null ? !businessDayConvention.equals(_that.businessDayConvention) : _that.businessDayConvention != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = super.hashCode();
			_result = 31 * _result + (businessCenters != null ? businessCenters.hashCode() : 0);
			_result = 31 * _result + (businessDayConvention != null ? businessDayConvention.getClass().getName().hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "BusinessDateRangeBuilder {" +
				"businessCenters=" + this.businessCenters + ", " +
				"businessDayConvention=" + this.businessDayConvention +
			'}' + " " + super.toString();
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		super.process(path, processor);
		
		processor.processBasic(path.newSubPath("businessDayConvention"), BusinessDayConventionEnum.class, businessDayConvention, this);
		
		processRosetta(path.newSubPath("businessCenters"), processor, BusinessCenters.class, businessCenters);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
		if (!super.equals(o)) return false;
	
		BusinessDateRange _that = (BusinessDateRange) o;
	
		if (businessCenters != null ? !businessCenters.equals(_that.businessCenters) : _that.businessCenters != null) return false;
		if (businessDayConvention != null ? !businessDayConvention.equals(_that.businessDayConvention) : _that.businessDayConvention != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = super.hashCode();
		_result = 31 * _result + (businessCenters != null ? businessCenters.hashCode() : 0);
		_result = 31 * _result + (businessDayConvention != null ? businessDayConvention.getClass().getName().hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "BusinessDateRange {" +
			"businessCenters=" + this.businessCenters + ", " +
			"businessDayConvention=" + this.businessDayConvention +
		'}' + " " + super.toString();
	}
}
