package org.isda.cdm;

import org.isda.cdm.BusinessCenters;
import org.isda.cdm.BusinessDayConventionEnum;
import org.isda.cdm.metafields.MetaFields;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.BusinessDayAdjustmentsMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class defining the business day convention and financial business centers used for adjusting any relevant date if it would otherwise fall on a day that is not a business day in the specified business center.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="BusinessDayAdjustments", source="FpML_5_10")
@RosettaSynonym(value="BusinessDayAdjustments", source="CME_SubmissionIRS_1_0")
@RosettaSynonym(value="BusinessDayAdjustments", source="DTCC_11_0")
@RosettaSynonym(value="BusinessDayAdjustments", source="DTCC_9_0")
@RosettaSynonym(value="BusinessDayAdjustments", source="CME_ClearedConfirm_1_17")
public class BusinessDayAdjustments extends RosettaModelObject implements GlobalKey {
	private final BusinessCenters businessCenters;
	private final BusinessDayConventionEnum businessDayConvention;
	private final MetaFields meta;
	private static BusinessDayAdjustmentsMeta metaData = new BusinessDayAdjustmentsMeta();
	
	BusinessDayAdjustments(BusinessDayAdjustmentsBuilder builder) {
		this.businessCenters = ofNullable(builder.getBusinessCenters()).map(f->f.build()).orElse(null);
		this.businessDayConvention = builder.getBusinessDayConvention();
		this.meta = ofNullable(builder.getMeta()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The business center(s), specified either explicitly or by reference to those specified somewhere else in the instance document.
	 */
	public final BusinessCenters getBusinessCenters() {
		return businessCenters;
	}
	
	/**
	 * The convention for adjusting a date if it would otherwise fall on a day that is not a business day.
	 */
	@RosettaSynonym(value="businessDayConvention", source="FpML_5_10", maps=2)
	@RosettaSynonym(value="businessDayConvention", source="CME_SubmissionIRS_1_0", maps=2)
	@RosettaSynonym(value="businessDayConvention", source="DTCC_11_0", maps=2)
	@RosettaSynonym(value="businessDayConvention", source="DTCC_9_0", maps=2)
	@RosettaSynonym(value="businessDayConvention", source="CME_ClearedConfirm_1_17", maps=2)
	public final BusinessDayConventionEnum getBusinessDayConvention() {
		return businessDayConvention;
	}
	
	public final MetaFields getMeta() {
		return meta;
	}
	
	
	@Override
	public RosettaMetaData<? extends BusinessDayAdjustments> metaData() {
		return metaData;
	} 
	
	public BusinessDayAdjustmentsBuilder toBuilder() {
		BusinessDayAdjustmentsBuilder builder = new BusinessDayAdjustmentsBuilder();
		ofNullable(getBusinessCenters()).ifPresent(builder::setBusinessCenters);
		ofNullable(getBusinessDayConvention()).ifPresent(builder::setBusinessDayConvention);
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		return builder;
	}

	public static BusinessDayAdjustmentsBuilder builder() {
		return new BusinessDayAdjustmentsBuilder();
	}

	public static  class BusinessDayAdjustmentsBuilder extends RosettaModelObjectBuilder implements GlobalKeyBuilder<BusinessDayAdjustmentsBuilder> {
	
		protected BusinessCenters.BusinessCentersBuilder businessCenters;
		protected BusinessDayConventionEnum businessDayConvention;
		protected MetaFields.MetaFieldsBuilder meta;
	
		public BusinessDayAdjustmentsBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends BusinessDayAdjustments> metaData() {
			return metaData;
		} 
	
		
		public BusinessCenters.BusinessCentersBuilder getBusinessCenters() {
			return businessCenters;
		}
		
		public BusinessCenters.BusinessCentersBuilder getOrCreateBusinessCenters() {
			if (businessCenters!=null) {
				return businessCenters;
			}
			else return businessCenters = new BusinessCenters.BusinessCentersBuilder();
		}
		
		
		public BusinessDayConventionEnum getBusinessDayConvention() {
			return businessDayConvention;
		}
		
		
		public MetaFields.MetaFieldsBuilder getMeta() {
			return meta;
		}
		
		public MetaFields.MetaFieldsBuilder getOrCreateMeta() {
			if (meta!=null) {
				return meta;
			}
			else return meta = new MetaFields.MetaFieldsBuilder();
		}
		
	
		public BusinessDayAdjustmentsBuilder setBusinessCenters(BusinessCenters businessCenters) {
			this.businessCenters = businessCenters.toBuilder();
			return this;
		}
		public BusinessDayAdjustmentsBuilder setBusinessCentersBuilder(BusinessCenters.BusinessCentersBuilder businessCenters) {
			this.businessCenters = businessCenters;
			return this;
		}
		public BusinessDayAdjustmentsBuilder setBusinessDayConvention(BusinessDayConventionEnum businessDayConvention) {
			this.businessDayConvention = businessDayConvention;
			return this;
		}
		public BusinessDayAdjustmentsBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		public BusinessDayAdjustmentsBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		
		public BusinessDayAdjustments build() {
			return new BusinessDayAdjustments(this);
		}
	
		@Override
		public BusinessDayAdjustmentsBuilder prune() {
			if (businessCenters!=null && !businessCenters.prune().hasData()) businessCenters = null;
			if (meta!=null && !meta.prune().hasData()) meta = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getBusinessCenters()!=null && getBusinessCenters().hasData()) return true;
			if (getBusinessDayConvention()!=null) return true;
			if (getMeta()!=null && getMeta().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("businessDayConvention"), BusinessDayConventionEnum.class, businessDayConvention, this);
			
			processRosetta(path.newSubPath("businessCenters"), processor, BusinessCenters.class, businessCenters);
			processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			BusinessDayAdjustmentsBuilder _that = (BusinessDayAdjustmentsBuilder) o;
		
			if (businessCenters != null ? !businessCenters.equals(_that.businessCenters) : _that.businessCenters != null) return false;
			if (businessDayConvention != null ? !businessDayConvention.equals(_that.businessDayConvention) : _that.businessDayConvention != null) return false;
			if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (businessCenters != null ? businessCenters.hashCode() : 0);
			_result = 31 * _result + (businessDayConvention != null ? businessDayConvention.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "BusinessDayAdjustmentsBuilder {" +
				"businessCenters=" + this.businessCenters + ", " +
				"businessDayConvention=" + this.businessDayConvention + ", " +
				"meta=" + this.meta +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("businessDayConvention"), BusinessDayConventionEnum.class, businessDayConvention, this);
		
		processRosetta(path.newSubPath("businessCenters"), processor, BusinessCenters.class, businessCenters);
		processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		BusinessDayAdjustments _that = (BusinessDayAdjustments) o;
	
		if (businessCenters != null ? !businessCenters.equals(_that.businessCenters) : _that.businessCenters != null) return false;
		if (businessDayConvention != null ? !businessDayConvention.equals(_that.businessDayConvention) : _that.businessDayConvention != null) return false;
		if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (businessCenters != null ? businessCenters.hashCode() : 0);
		_result = 31 * _result + (businessDayConvention != null ? businessDayConvention.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "BusinessDayAdjustments {" +
			"businessCenters=" + this.businessCenters + ", " +
			"businessDayConvention=" + this.businessDayConvention + ", " +
			"meta=" + this.meta +
		'}';
	}
}
