package org.isda.cdm;

import com.rosetta.model.lib.annotations.RosettaSynonym;

/**
 * The enumerated values to specify the convention for adjusting any relevant date if it would otherwise fall on a day that is not a valid business day.
 * 
 * @version 2.5.4
 */
public enum BusinessDayConventionEnum {
    

	/**
	 * The non-business date will be adjusted to the first following day that is a business day
	 */
	@RosettaSynonym(value = "FOLLOWING", source = "FpML_5_10")
	@RosettaSynonym(value = "FOLLOWING", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "FOLLOWING", source = "DTCC_11_0")
	@RosettaSynonym(value = "FOLLOWING", source = "DTCC_9_0")
	@RosettaSynonym(value = "FOLLOWING", source = "CME_ClearedConfirm_1_17")
	FOLLOWING,
	    

	/**
	 * Per 2000 ISDA Definitions, Section 4.11. FRN Convention; Eurodollar Convention. FRN is included here as a type of business day convention although it does not strictly fall within ISDA&#39;s definition of a Business Day Convention and does not conform to the simple definition given above.
	 */
	@RosettaSynonym(value = "FRN", source = "FpML_5_10")
	@RosettaSynonym(value = "FRN", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "FRN", source = "DTCC_11_0")
	@RosettaSynonym(value = "FRN", source = "DTCC_9_0")
	@RosettaSynonym(value = "FRN", source = "CME_ClearedConfirm_1_17")
	FRN,
	    

	/**
	 * The non-business date will be adjusted to the first following day that is a business day unless that day falls in the next calendar month, in which case that date will be the first preceding day that is a business day.
	 */
	@RosettaSynonym(value = "MODFOLLOWING", source = "FpML_5_10")
	@RosettaSynonym(value = "MODFOLLOWING", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "MODFOLLOWING", source = "DTCC_11_0")
	@RosettaSynonym(value = "MODFOLLOWING", source = "DTCC_9_0")
	@RosettaSynonym(value = "MODFOLLOWING", source = "CME_ClearedConfirm_1_17")
	MODFOLLOWING,
	    

	/**
	 * The non-business day will be adjusted to the first preceding day that is a business day.
	 */
	@RosettaSynonym(value = "PRECEDING", source = "FpML_5_10")
	@RosettaSynonym(value = "PRECEDING", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "PRECEDING", source = "DTCC_11_0")
	@RosettaSynonym(value = "PRECEDING", source = "DTCC_9_0")
	@RosettaSynonym(value = "PRECEDING", source = "CME_ClearedConfirm_1_17")
	PRECEDING,
	    

	/**
	 * The non-business date will be adjusted to the first preceding day that is a business day unless that day falls in the previous calendar month, in which case that date will be the first following day that us a business day.
	 */
	@RosettaSynonym(value = "MODPRECEDING", source = "FpML_5_10")
	@RosettaSynonym(value = "MODPRECEDING", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "MODPRECEDING", source = "DTCC_11_0")
	@RosettaSynonym(value = "MODPRECEDING", source = "DTCC_9_0")
	@RosettaSynonym(value = "MODPRECEDING", source = "CME_ClearedConfirm_1_17")
	MODPRECEDING,
	    

	/**
	 * The non-business date will be adjusted to the nearest day that is a business day - i.e. if the non-business day falls on any day other than a Sunday or a Monday, it will be the first preceding day that is a business day, and will be the first following business day if it falls on a Sunday or a Monday.
	 */
	@RosettaSynonym(value = "NEAREST", source = "FpML_5_10")
	@RosettaSynonym(value = "NEAREST", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "NEAREST", source = "DTCC_11_0")
	@RosettaSynonym(value = "NEAREST", source = "DTCC_9_0")
	@RosettaSynonym(value = "NEAREST", source = "CME_ClearedConfirm_1_17")
	NEAREST,
	    

	/**
	 * The date will not be adjusted if it falls on a day that is not a business day.
	 */
	@RosettaSynonym(value = "NONE", source = "FpML_5_10")
	@RosettaSynonym(value = "NONE", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "NONE", source = "DTCC_11_0")
	@RosettaSynonym(value = "NONE", source = "DTCC_9_0")
	@RosettaSynonym(value = "NONE", source = "CME_ClearedConfirm_1_17")
	NONE,
	    

	/**
	 * The date adjustments conventions are defined elsewhere, so it is not required to specify them here.
	 */
	@RosettaSynonym(value = "NotApplicable", source = "FpML_5_10")
	@RosettaSynonym(value = "NotApplicable", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "NotApplicable", source = "DTCC_11_0")
	@RosettaSynonym(value = "NotApplicable", source = "DTCC_9_0")
	@RosettaSynonym(value = "NotApplicable", source = "CME_ClearedConfirm_1_17")
	NOT_APPLICABLE    
	;

	private final String displayName;
	
	BusinessDayConventionEnum() {
		this.displayName = null;
	}


	@Override
	public String toString() {
		return displayName != null ?  displayName : name();
	}
	
}
