package org.isda.cdm;

import java.lang.String;
import org.isda.cdm.ContactInformation;
import org.isda.cdm.Identifier;
import org.isda.cdm.metafields.MetaFields;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.BusinessUnitMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class to specify an organizational unit.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="BusinessUnit", source="FpML_5_10")
public class BusinessUnit extends RosettaModelObject implements GlobalKey {
	private final ContactInformation contactInformation;
	private final Identifier identifier;
	private final MetaFields meta;
	private final String name;
	private static BusinessUnitMeta metaData = new BusinessUnitMeta();
	
	BusinessUnit(BusinessUnitBuilder builder) {
		this.contactInformation = ofNullable(builder.getContactInformation()).map(f->f.build()).orElse(null);
		this.identifier = ofNullable(builder.getIdentifier()).map(f->f.build()).orElse(null);
		this.meta = ofNullable(builder.getMeta()).map(f->f.build()).orElse(null);
		this.name = builder.getName();
	}
	
	/**
	 * The contact information for such business unit, when different from the contact information associated with the party.
	 */
	@RosettaSynonym(value="contactInfo", source="FpML_5_10")
	public final ContactInformation getContactInformation() {
		return contactInformation;
	}
	
	/**
	 * An identifier used to uniquely identify the organizational unit
	 */
	public final Identifier getIdentifier() {
		return identifier;
	}
	
	public final MetaFields getMeta() {
		return meta;
	}
	
	/**
	 * A name used to describe the organizational unit
	 */
	@RosettaSynonym(value="name", source="FpML_5_10")
	public final String getName() {
		return name;
	}
	
	
	@Override
	public RosettaMetaData<? extends BusinessUnit> metaData() {
		return metaData;
	} 
	
	public BusinessUnitBuilder toBuilder() {
		BusinessUnitBuilder builder = new BusinessUnitBuilder();
		ofNullable(getContactInformation()).ifPresent(builder::setContactInformation);
		ofNullable(getIdentifier()).ifPresent(builder::setIdentifier);
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		ofNullable(getName()).ifPresent(builder::setName);
		return builder;
	}

	public static BusinessUnitBuilder builder() {
		return new BusinessUnitBuilder();
	}

	public static  class BusinessUnitBuilder extends RosettaModelObjectBuilder implements GlobalKeyBuilder<BusinessUnitBuilder> {
	
		protected ContactInformation.ContactInformationBuilder contactInformation;
		protected Identifier.IdentifierBuilder identifier;
		protected MetaFields.MetaFieldsBuilder meta;
		protected String name;
	
		public BusinessUnitBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends BusinessUnit> metaData() {
			return metaData;
		} 
	
		
		public ContactInformation.ContactInformationBuilder getContactInformation() {
			return contactInformation;
		}
		
		public ContactInformation.ContactInformationBuilder getOrCreateContactInformation() {
			if (contactInformation!=null) {
				return contactInformation;
			}
			else return contactInformation = new ContactInformation.ContactInformationBuilder();
		}
		
		
		public Identifier.IdentifierBuilder getIdentifier() {
			return identifier;
		}
		
		public Identifier.IdentifierBuilder getOrCreateIdentifier() {
			if (identifier!=null) {
				return identifier;
			}
			else return identifier = new Identifier.IdentifierBuilder();
		}
		
		
		public MetaFields.MetaFieldsBuilder getMeta() {
			return meta;
		}
		
		public MetaFields.MetaFieldsBuilder getOrCreateMeta() {
			if (meta!=null) {
				return meta;
			}
			else return meta = new MetaFields.MetaFieldsBuilder();
		}
		
		
		public String getName() {
			return name;
		}
		
	
		public BusinessUnitBuilder setContactInformation(ContactInformation contactInformation) {
			this.contactInformation = contactInformation.toBuilder();
			return this;
		}
		public BusinessUnitBuilder setContactInformationBuilder(ContactInformation.ContactInformationBuilder contactInformation) {
			this.contactInformation = contactInformation;
			return this;
		}
		public BusinessUnitBuilder setIdentifier(Identifier identifier) {
			this.identifier = identifier.toBuilder();
			return this;
		}
		public BusinessUnitBuilder setIdentifierBuilder(Identifier.IdentifierBuilder identifier) {
			this.identifier = identifier;
			return this;
		}
		public BusinessUnitBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		public BusinessUnitBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		public BusinessUnitBuilder setName(String name) {
			this.name = name;
			return this;
		}
		
		public BusinessUnit build() {
			return new BusinessUnit(this);
		}
	
		@Override
		public BusinessUnitBuilder prune() {
			if (contactInformation!=null && !contactInformation.prune().hasData()) contactInformation = null;
			if (identifier!=null && !identifier.prune().hasData()) identifier = null;
			if (meta!=null && !meta.prune().hasData()) meta = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getContactInformation()!=null && getContactInformation().hasData()) return true;
			if (getIdentifier()!=null && getIdentifier().hasData()) return true;
			if (getMeta()!=null && getMeta().hasData()) return true;
			if (getName()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("name"), String.class, name, this);
			
			processRosetta(path.newSubPath("contactInformation"), processor, ContactInformation.class, contactInformation);
			processRosetta(path.newSubPath("identifier"), processor, Identifier.class, identifier);
			processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			BusinessUnitBuilder _that = (BusinessUnitBuilder) o;
		
			if (contactInformation != null ? !contactInformation.equals(_that.contactInformation) : _that.contactInformation != null) return false;
			if (identifier != null ? !identifier.equals(_that.identifier) : _that.identifier != null) return false;
			if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
			if (name != null ? !name.equals(_that.name) : _that.name != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (contactInformation != null ? contactInformation.hashCode() : 0);
			_result = 31 * _result + (identifier != null ? identifier.hashCode() : 0);
			_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
			_result = 31 * _result + (name != null ? name.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "BusinessUnitBuilder {" +
				"contactInformation=" + this.contactInformation + ", " +
				"identifier=" + this.identifier + ", " +
				"meta=" + this.meta + ", " +
				"name=" + this.name +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("name"), String.class, name, this);
		
		processRosetta(path.newSubPath("contactInformation"), processor, ContactInformation.class, contactInformation);
		processRosetta(path.newSubPath("identifier"), processor, Identifier.class, identifier);
		processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		BusinessUnit _that = (BusinessUnit) o;
	
		if (contactInformation != null ? !contactInformation.equals(_that.contactInformation) : _that.contactInformation != null) return false;
		if (identifier != null ? !identifier.equals(_that.identifier) : _that.identifier != null) return false;
		if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
		if (name != null ? !name.equals(_that.name) : _that.name != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (contactInformation != null ? contactInformation.hashCode() : 0);
		_result = 31 * _result + (identifier != null ? identifier.hashCode() : 0);
		_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
		_result = 31 * _result + (name != null ? name.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "BusinessUnit {" +
			"contactInformation=" + this.contactInformation + ", " +
			"identifier=" + this.identifier + ", " +
			"meta=" + this.meta + ", " +
			"name=" + this.name +
		'}';
	}
}
