package org.isda.cdm;

import org.isda.cdm.BusinessCenterEnum;
import org.isda.cdm.CalculationAgent;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.CalculationAgentModelMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * This class corresponds to the FpML CalculationAgent.model.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class CalculationAgentModel extends RosettaModelObject {
	private final CalculationAgent calculationAgent;
	private final BusinessCenterEnum calculationAgentBusinessCenter;
	private static CalculationAgentModelMeta metaData = new CalculationAgentModelMeta();
	
	CalculationAgentModel(CalculationAgentModelBuilder builder) {
		this.calculationAgent = ofNullable(builder.getCalculationAgent()).map(f->f.build()).orElse(null);
		this.calculationAgentBusinessCenter = builder.getCalculationAgentBusinessCenter();
	}
	
	/**
	 * The ISDA calculation agent responsible for performing duties as defined in the applicable product definitions.
	 */
	@RosettaSynonym(value="calculationAgent", source="FpML_5_10", path="trade")
	@RosettaSynonym(value="calculationAgent", source="CME_SubmissionIRS_1_0", path="trade")
	@RosettaSynonym(value="calculationAgent", source="CME_ClearedConfirm_1_17", path="trade")
	@RosettaSynonym(value="calculationAgent", source="Rosetta_Workbench")
	@RosettaSynonym(value="calculationAgent", source="FpML_5_10", path="trade.swaption")
	@RosettaSynonym(value="calculationAgent", source="CME_SubmissionIRS_1_0", path="trade.swaption")
	@RosettaSynonym(value="calculationAgent", source="CME_ClearedConfirm_1_17", path="trade.swaption")
	@RosettaSynonym(value="calculationAgent", source="FpML_5_10", path="swaption")
	@RosettaSynonym(value="calculationAgent", source="CME_SubmissionIRS_1_0", path="swaption")
	@RosettaSynonym(value="calculationAgent", source="CME_ClearedConfirm_1_17", path="swaption")
	public final CalculationAgent getCalculationAgent() {
		return calculationAgent;
	}
	
	/**
	 * The city in which the office through which ISDA Calculation Agent is acting for purposes of the transaction is located. The short-form confirm for a trade that is executed under a Sovereign or Asia-Pacific Master Confirmation Agreement (MCA), does not need to specify the Calculation Agent. However, the confirm does need to specify the Calculation Agent city. This is due to the fact that the MCA sets the value for Calculation Agent but does not set the value for Calculation Agent city.
	 */
	@RosettaSynonym(value="calculationAgentBusinessCenter", source="FpML_5_10", path="trade")
	@RosettaSynonym(value="calculationAgentBusinessCenter", source="CME_SubmissionIRS_1_0", path="trade")
	@RosettaSynonym(value="calculationAgentBusinessCenter", source="CME_ClearedConfirm_1_17", path="trade")
	@RosettaSynonym(value="calculationAgentBusinessCenter", source="Rosetta_Workbench")
	@RosettaSynonym(value="calculationAgentBusinessCenter", source="DTCC_11_0", path="Body.OTC_Matching.Trade.FpML.trade")
	@RosettaSynonym(value="calculationAgentBusinessCenter", source="DTCC_9_0", path="Body.OTC_Matching.Trade.FpML.trade")
	public final BusinessCenterEnum getCalculationAgentBusinessCenter() {
		return calculationAgentBusinessCenter;
	}
	
	
	@Override
	public RosettaMetaData<? extends CalculationAgentModel> metaData() {
		return metaData;
	} 
	
	public CalculationAgentModelBuilder toBuilder() {
		CalculationAgentModelBuilder builder = new CalculationAgentModelBuilder();
		ofNullable(getCalculationAgent()).ifPresent(builder::setCalculationAgent);
		ofNullable(getCalculationAgentBusinessCenter()).ifPresent(builder::setCalculationAgentBusinessCenter);
		return builder;
	}

	public static CalculationAgentModelBuilder builder() {
		return new CalculationAgentModelBuilder();
	}

	public static  class CalculationAgentModelBuilder extends RosettaModelObjectBuilder {
	
		protected CalculationAgent.CalculationAgentBuilder calculationAgent;
		protected BusinessCenterEnum calculationAgentBusinessCenter;
	
		public CalculationAgentModelBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends CalculationAgentModel> metaData() {
			return metaData;
		} 
	
		
		public CalculationAgent.CalculationAgentBuilder getCalculationAgent() {
			return calculationAgent;
		}
		
		public CalculationAgent.CalculationAgentBuilder getOrCreateCalculationAgent() {
			if (calculationAgent!=null) {
				return calculationAgent;
			}
			else return calculationAgent = new CalculationAgent.CalculationAgentBuilder();
		}
		
		
		public BusinessCenterEnum getCalculationAgentBusinessCenter() {
			return calculationAgentBusinessCenter;
		}
		
	
		public CalculationAgentModelBuilder setCalculationAgent(CalculationAgent calculationAgent) {
			this.calculationAgent = calculationAgent.toBuilder();
			return this;
		}
		public CalculationAgentModelBuilder setCalculationAgentBuilder(CalculationAgent.CalculationAgentBuilder calculationAgent) {
			this.calculationAgent = calculationAgent;
			return this;
		}
		public CalculationAgentModelBuilder setCalculationAgentBusinessCenter(BusinessCenterEnum calculationAgentBusinessCenter) {
			this.calculationAgentBusinessCenter = calculationAgentBusinessCenter;
			return this;
		}
		
		public CalculationAgentModel build() {
			return new CalculationAgentModel(this);
		}
	
		@Override
		public CalculationAgentModelBuilder prune() {
			if (calculationAgent!=null && !calculationAgent.prune().hasData()) calculationAgent = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getCalculationAgent()!=null && getCalculationAgent().hasData()) return true;
			if (getCalculationAgentBusinessCenter()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("calculationAgentBusinessCenter"), BusinessCenterEnum.class, calculationAgentBusinessCenter, this);
			
			processRosetta(path.newSubPath("calculationAgent"), processor, CalculationAgent.class, calculationAgent);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			CalculationAgentModelBuilder _that = (CalculationAgentModelBuilder) o;
		
			if (calculationAgent != null ? !calculationAgent.equals(_that.calculationAgent) : _that.calculationAgent != null) return false;
			if (calculationAgentBusinessCenter != null ? !calculationAgentBusinessCenter.equals(_that.calculationAgentBusinessCenter) : _that.calculationAgentBusinessCenter != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (calculationAgent != null ? calculationAgent.hashCode() : 0);
			_result = 31 * _result + (calculationAgentBusinessCenter != null ? calculationAgentBusinessCenter.getClass().getName().hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "CalculationAgentModelBuilder {" +
				"calculationAgent=" + this.calculationAgent + ", " +
				"calculationAgentBusinessCenter=" + this.calculationAgentBusinessCenter +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("calculationAgentBusinessCenter"), BusinessCenterEnum.class, calculationAgentBusinessCenter, this);
		
		processRosetta(path.newSubPath("calculationAgent"), processor, CalculationAgent.class, calculationAgent);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		CalculationAgentModel _that = (CalculationAgentModel) o;
	
		if (calculationAgent != null ? !calculationAgent.equals(_that.calculationAgent) : _that.calculationAgent != null) return false;
		if (calculationAgentBusinessCenter != null ? !calculationAgentBusinessCenter.equals(_that.calculationAgentBusinessCenter) : _that.calculationAgentBusinessCenter != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (calculationAgent != null ? calculationAgent.hashCode() : 0);
		_result = 31 * _result + (calculationAgentBusinessCenter != null ? calculationAgentBusinessCenter.getClass().getName().hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "CalculationAgentModel {" +
			"calculationAgent=" + this.calculationAgent + ", " +
			"calculationAgentBusinessCenter=" + this.calculationAgentBusinessCenter +
		'}';
	}
}
