package org.isda.cdm;

import java.lang.String;
import org.isda.cdm.metafields.FieldWithMetaBusinessCenterEnum;
import org.isda.cdm.metafields.FieldWithMetaBusinessCenterEnum.FieldWithMetaBusinessCenterEnumBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.CalculationDateLocationElectionMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class to specify each of the party elections with respect to the Calculation Date Location. ISDA 2016 English Law Credit Support Deed for Initial Margin, paragraph 13, General Principles, (d)(i): Calculation Date Location. | ISDA 2016 Japanese Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (e)(i): Calculation Date Location. | ISDA 2016 New York Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (d)(i): Calculation Date Location.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class CalculationDateLocationElection extends RosettaModelObject {
	private final FieldWithMetaBusinessCenterEnum businessCenter;
	private final String customLocation;
	private final String party;
	private static CalculationDateLocationElectionMeta metaData = new CalculationDateLocationElectionMeta();
	
	CalculationDateLocationElection(CalculationDateLocationElectionBuilder builder) {
		this.businessCenter = ofNullable(builder.getBusinessCenter()).map(f->f.build()).orElse(null);
		this.customLocation = builder.getCustomLocation();
		this.party = builder.getParty();
	}
	
	/**
	 * The Calculation Date Location when specified as a business center which corresponds to the FpML list of business centers or can be mapped to it.
	 */
	@RosettaSynonym(value="partyA_location", source="ISDA_Create_1_0")
	@RosettaSynonym(value="partyB_location", source="ISDA_Create_1_0")
	public final FieldWithMetaBusinessCenterEnum getBusinessCenter() {
		return businessCenter;
	}
	
	/**
	 * The Calculation Date Location when specified a location which doesn&#39;t correspond to the FpML list of business centers or cannot be mapped to it.
	 */
	@RosettaSynonym(value="partyA_specify", source="ISDA_Create_1_0")
	@RosettaSynonym(value="partyB_specify", source="ISDA_Create_1_0")
	public final String getCustomLocation() {
		return customLocation;
	}
	
	/**
	 * The elective party.
	 */
	@RosettaSynonym(value="", source="ISDA_Create_1_0")
	@RosettaSynonym(value="", source="ISDA_Create_1_0")
	public final String getParty() {
		return party;
	}
	
	
	@Override
	public RosettaMetaData<? extends CalculationDateLocationElection> metaData() {
		return metaData;
	} 
	
	public CalculationDateLocationElectionBuilder toBuilder() {
		CalculationDateLocationElectionBuilder builder = new CalculationDateLocationElectionBuilder();
		ofNullable(getBusinessCenter()).ifPresent(builder::setBusinessCenter);
		ofNullable(getCustomLocation()).ifPresent(builder::setCustomLocation);
		ofNullable(getParty()).ifPresent(builder::setParty);
		return builder;
	}

	public static CalculationDateLocationElectionBuilder builder() {
		return new CalculationDateLocationElectionBuilder();
	}

	public static  class CalculationDateLocationElectionBuilder extends RosettaModelObjectBuilder {
	
		protected FieldWithMetaBusinessCenterEnumBuilder businessCenter;
		protected String customLocation;
		protected String party;
	
		public CalculationDateLocationElectionBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends CalculationDateLocationElection> metaData() {
			return metaData;
		} 
	
		
		public FieldWithMetaBusinessCenterEnumBuilder getBusinessCenter() {
			return businessCenter;
		}
		
		public FieldWithMetaBusinessCenterEnumBuilder getOrCreateBusinessCenter() {
			if (businessCenter!=null) {
				return businessCenter;
			}
			else return businessCenter = new FieldWithMetaBusinessCenterEnumBuilder();
		}
		
		
		public String getCustomLocation() {
			return customLocation;
		}
		
		
		public String getParty() {
			return party;
		}
		
	
		public CalculationDateLocationElectionBuilder setBusinessCenter(FieldWithMetaBusinessCenterEnum businessCenter) {
			this.businessCenter = businessCenter.toBuilder();
			return this;
		}
		public CalculationDateLocationElectionBuilder setBusinessCenterRef(BusinessCenterEnum businessCenter) {
			return setBusinessCenter(FieldWithMetaBusinessCenterEnum.builder().setValue(businessCenter).build());
		}
		public CalculationDateLocationElectionBuilder setCustomLocation(String customLocation) {
			this.customLocation = customLocation;
			return this;
		}
		public CalculationDateLocationElectionBuilder setParty(String party) {
			this.party = party;
			return this;
		}
		
		public CalculationDateLocationElection build() {
			return new CalculationDateLocationElection(this);
		}
	
		@Override
		public CalculationDateLocationElectionBuilder prune() {
			if (businessCenter!=null && !businessCenter.prune().hasData()) businessCenter = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getBusinessCenter()!=null) return true;
			if (getCustomLocation()!=null) return true;
			if (getParty()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("customLocation"), String.class, customLocation, this);
			processor.processBasic(path.newSubPath("party"), String.class, party, this);
			
			processRosetta(path.newSubPath("businessCenter"), processor, FieldWithMetaBusinessCenterEnum.class, businessCenter);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			CalculationDateLocationElectionBuilder _that = (CalculationDateLocationElectionBuilder) o;
		
			if (businessCenter != null ? !businessCenter.equals(_that.businessCenter) : _that.businessCenter != null) return false;
			if (customLocation != null ? !customLocation.equals(_that.customLocation) : _that.customLocation != null) return false;
			if (party != null ? !party.equals(_that.party) : _that.party != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (businessCenter != null ? businessCenter.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (customLocation != null ? customLocation.hashCode() : 0);
			_result = 31 * _result + (party != null ? party.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "CalculationDateLocationElectionBuilder {" +
				"businessCenter=" + this.businessCenter + ", " +
				"customLocation=" + this.customLocation + ", " +
				"party=" + this.party +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("customLocation"), String.class, customLocation, this);
		processor.processBasic(path.newSubPath("party"), String.class, party, this);
		
		processRosetta(path.newSubPath("businessCenter"), processor, FieldWithMetaBusinessCenterEnum.class, businessCenter);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		CalculationDateLocationElection _that = (CalculationDateLocationElection) o;
	
		if (businessCenter != null ? !businessCenter.equals(_that.businessCenter) : _that.businessCenter != null) return false;
		if (customLocation != null ? !customLocation.equals(_that.customLocation) : _that.customLocation != null) return false;
		if (party != null ? !party.equals(_that.party) : _that.party != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (businessCenter != null ? businessCenter.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (customLocation != null ? customLocation.hashCode() : 0);
		_result = 31 * _result + (party != null ? party.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "CalculationDateLocationElection {" +
			"businessCenter=" + this.businessCenter + ", " +
			"customLocation=" + this.customLocation + ", " +
			"party=" + this.party +
		'}';
	}
}
