package org.isda.cdm;

import com.rosetta.model.lib.records.Date;
import java.lang.Integer;
import java.math.BigDecimal;
import org.isda.cdm.FloatingRateDefinition;
import org.isda.cdm.FxLinkedNotionalAmount;
import org.isda.cdm.Money;


import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.CalculationPeriodMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import com.rosetta.model.lib.records.Date;
		import java.math.BigDecimal;
		import static java.util.Optional.ofNullable;

/**
 * A class defining the parameters used in the calculation of a fixed or floating rate calculation period amount. This class forms part of cashflows representation of a swap stream.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="CalculationPeriod", source="FpML_5_10")
public class CalculationPeriod extends CalculationPeriodBase {
	private final Integer calculationPeriodNumberOfDays;
	private final BigDecimal dayCountYearFraction;
	private final BigDecimal fixedRate;
	private final FloatingRateDefinition floatingRateDefinition;
	private final Money forecastAmount;
	private final BigDecimal forecastRate;
	private final FxLinkedNotionalAmount fxLinkedNotionalAmount;
	private final BigDecimal notionalAmount;
	private final Date unadjustedEndDate;
	private final Date unadjustedStartDate;
	private static CalculationPeriodMeta metaData = new CalculationPeriodMeta();
	
	CalculationPeriod(CalculationPeriodBuilder builder) {
		super(builder);
		this.calculationPeriodNumberOfDays = builder.getCalculationPeriodNumberOfDays();
		this.dayCountYearFraction = builder.getDayCountYearFraction();
		this.fixedRate = builder.getFixedRate();
		this.floatingRateDefinition = ofNullable(builder.getFloatingRateDefinition()).map(f->f.build()).orElse(null);
		this.forecastAmount = ofNullable(builder.getForecastAmount()).map(f->f.build()).orElse(null);
		this.forecastRate = builder.getForecastRate();
		this.fxLinkedNotionalAmount = ofNullable(builder.getFxLinkedNotionalAmount()).map(f->f.build()).orElse(null);
		this.notionalAmount = builder.getNotionalAmount();
		this.unadjustedEndDate = builder.getUnadjustedEndDate();
		this.unadjustedStartDate = builder.getUnadjustedStartDate();
	}
	
	/**
	 * The number of days from the adjusted effective / start date to the adjusted termination / end date calculated in accordance with the applicable day count fraction.
	 */
	@RosettaSynonym(value="calculationPeriodNumberOfDays", source="FpML_5_10")
	@RosettaSynonym(value="calculationPeriodNumberOfDays", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="calculationPeriodNumberOfDays", source="DTCC_11_0")
	@RosettaSynonym(value="calculationPeriodNumberOfDays", source="DTCC_9_0")
	@RosettaSynonym(value="calculationPeriodNumberOfDays", source="CME_ClearedConfirm_1_17")
	public final Integer getCalculationPeriodNumberOfDays() {
		return calculationPeriodNumberOfDays;
	}
	
	/**
	 * The year fraction value of the calculation period, result of applying the ISDA rules for day count fraction defined in the ISDA Annex.
	 */
	@RosettaSynonym(value="dayCountYearFraction", source="FpML_5_10")
	@RosettaSynonym(value="dayCountYearFraction", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="dayCountYearFraction", source="DTCC_11_0")
	@RosettaSynonym(value="dayCountYearFraction", source="DTCC_9_0")
	@RosettaSynonym(value="dayCountYearFraction", source="CME_ClearedConfirm_1_17")
	public final BigDecimal getDayCountYearFraction() {
		return dayCountYearFraction;
	}
	
	/**
	 * The calculation period fixed rate. A per annum rate, expressed as a decimal. A fixed rate of 5% would be represented as 0.05.
	 */
	@RosettaSynonym(value="fixedRate", source="FpML_5_10")
	@RosettaSynonym(value="fixedRate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="fixedRate", source="DTCC_11_0")
	@RosettaSynonym(value="fixedRate", source="DTCC_9_0")
	@RosettaSynonym(value="fixedRate", source="CME_ClearedConfirm_1_17")
	public final BigDecimal getFixedRate() {
		return fixedRate;
	}
	
	/**
	 * The floating rate reset information for the calculation period.
	 */
	@RosettaSynonym(value="floatingRateDefinition", source="FpML_5_10")
	@RosettaSynonym(value="floatingRateDefinition", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="floatingRateDefinition", source="DTCC_11_0")
	@RosettaSynonym(value="floatingRateDefinition", source="DTCC_9_0")
	@RosettaSynonym(value="floatingRateDefinition", source="CME_ClearedConfirm_1_17")
	public final FloatingRateDefinition getFloatingRateDefinition() {
		return floatingRateDefinition;
	}
	
	/**
	 * The amount representing the forecast of the accrued value of the calculation period. An intermediate value used to generate the forecastPaymentAmount in the PaymentCalculationPeriod.
	 */
	@RosettaSynonym(value="forecastAmount", source="FpML_5_10")
	@RosettaSynonym(value="forecastAmount", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="forecastAmount", source="DTCC_11_0")
	@RosettaSynonym(value="forecastAmount", source="DTCC_9_0")
	@RosettaSynonym(value="forecastAmount", source="CME_ClearedConfirm_1_17")
	public final Money getForecastAmount() {
		return forecastAmount;
	}
	
	/**
	 * A value representing the forecast rate used to calculate the forecast future value of the accrual period. This is a calculated rate determined based on averaging the rates in the rateObservation elements, and incorporates all of the rate treatment and averaging rules. A value of 1% should be represented as 0.01.
	 */
	@RosettaSynonym(value="forecastRate", source="FpML_5_10")
	@RosettaSynonym(value="forecastRate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="forecastRate", source="DTCC_11_0")
	@RosettaSynonym(value="forecastRate", source="DTCC_9_0")
	@RosettaSynonym(value="forecastRate", source="CME_ClearedConfirm_1_17")
	public final BigDecimal getForecastRate() {
		return forecastRate;
	}
	
	/**
	 * The amount that a cashflow will accrue interest on. This is the calculated amount of the FX linked - i.e. the other currency notional amount multiplied by the appropriate FX spot rate.
	 */
	@RosettaSynonym(value="fxLinkedNotionalAmount", source="FpML_5_10")
	@RosettaSynonym(value="fxLinkedNotionalAmount", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="fxLinkedNotionalAmount", source="DTCC_11_0")
	@RosettaSynonym(value="fxLinkedNotionalAmount", source="DTCC_9_0")
	@RosettaSynonym(value="fxLinkedNotionalAmount", source="CME_ClearedConfirm_1_17")
	public final FxLinkedNotionalAmount getFxLinkedNotionalAmount() {
		return fxLinkedNotionalAmount;
	}
	
	/**
	 * The amount that a cashflow will accrue interest on.
	 */
	@RosettaSynonym(value="notionalAmount", source="FpML_5_10")
	@RosettaSynonym(value="notionalAmount", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="notionalAmount", source="DTCC_11_0")
	@RosettaSynonym(value="notionalAmount", source="DTCC_9_0")
	@RosettaSynonym(value="notionalAmount", source="CME_ClearedConfirm_1_17")
	public final BigDecimal getNotionalAmount() {
		return notionalAmount;
	}
	
	/**
	 * The calculation end date, unadjusted.
	 */
	@RosettaSynonym(value="unadjustedEndDate", source="FpML_5_10")
	@RosettaSynonym(value="unadjustedEndDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="unadjustedEndDate", source="DTCC_11_0")
	@RosettaSynonym(value="unadjustedEndDate", source="DTCC_9_0")
	@RosettaSynonym(value="unadjustedEndDate", source="CME_ClearedConfirm_1_17")
	public final Date getUnadjustedEndDate() {
		return unadjustedEndDate;
	}
	
	/**
	 * The calculation start date, unadjusted.
	 */
	@RosettaSynonym(value="unadjustedStartDate", source="FpML_5_10")
	@RosettaSynonym(value="unadjustedStartDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="unadjustedStartDate", source="DTCC_11_0")
	@RosettaSynonym(value="unadjustedStartDate", source="DTCC_9_0")
	@RosettaSynonym(value="unadjustedStartDate", source="CME_ClearedConfirm_1_17")
	public final Date getUnadjustedStartDate() {
		return unadjustedStartDate;
	}
	
	
	@Override
	public RosettaMetaData<? extends CalculationPeriod> metaData() {
		return metaData;
	} 
	
	public CalculationPeriodBuilder toBuilder() {
		CalculationPeriodBuilder builder = new CalculationPeriodBuilder();
		ofNullable(getCalculationPeriodNumberOfDays()).ifPresent(builder::setCalculationPeriodNumberOfDays);
		ofNullable(getDayCountYearFraction()).ifPresent(builder::setDayCountYearFraction);
		ofNullable(getFixedRate()).ifPresent(builder::setFixedRate);
		ofNullable(getFloatingRateDefinition()).ifPresent(builder::setFloatingRateDefinition);
		ofNullable(getForecastAmount()).ifPresent(builder::setForecastAmount);
		ofNullable(getForecastRate()).ifPresent(builder::setForecastRate);
		ofNullable(getFxLinkedNotionalAmount()).ifPresent(builder::setFxLinkedNotionalAmount);
		ofNullable(getNotionalAmount()).ifPresent(builder::setNotionalAmount);
		ofNullable(getUnadjustedEndDate()).ifPresent(builder::setUnadjustedEndDate);
		ofNullable(getUnadjustedStartDate()).ifPresent(builder::setUnadjustedStartDate);
		ofNullable(getAdjustedEndDate()).ifPresent(builder::setAdjustedEndDate);
		ofNullable(getAdjustedStartDate()).ifPresent(builder::setAdjustedStartDate);
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		return builder;
	}

	public static CalculationPeriodBuilder builder() {
		return new CalculationPeriodBuilder();
	}

	public static  class CalculationPeriodBuilder extends CalculationPeriodBaseBuilder {
	
		protected Integer calculationPeriodNumberOfDays;
		protected BigDecimal dayCountYearFraction;
		protected BigDecimal fixedRate;
		protected FloatingRateDefinition.FloatingRateDefinitionBuilder floatingRateDefinition;
		protected Money.MoneyBuilder forecastAmount;
		protected BigDecimal forecastRate;
		protected FxLinkedNotionalAmount.FxLinkedNotionalAmountBuilder fxLinkedNotionalAmount;
		protected BigDecimal notionalAmount;
		protected Date unadjustedEndDate;
		protected Date unadjustedStartDate;
	
		public CalculationPeriodBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends CalculationPeriod> metaData() {
			return metaData;
		} 
	
		
		public Integer getCalculationPeriodNumberOfDays() {
			return calculationPeriodNumberOfDays;
		}
		
		
		public BigDecimal getDayCountYearFraction() {
			return dayCountYearFraction;
		}
		
		
		public BigDecimal getFixedRate() {
			return fixedRate;
		}
		
		
		public FloatingRateDefinition.FloatingRateDefinitionBuilder getFloatingRateDefinition() {
			return floatingRateDefinition;
		}
		
		public FloatingRateDefinition.FloatingRateDefinitionBuilder getOrCreateFloatingRateDefinition() {
			if (floatingRateDefinition!=null) {
				return floatingRateDefinition;
			}
			else return floatingRateDefinition = new FloatingRateDefinition.FloatingRateDefinitionBuilder();
		}
		
		
		public Money.MoneyBuilder getForecastAmount() {
			return forecastAmount;
		}
		
		public Money.MoneyBuilder getOrCreateForecastAmount() {
			if (forecastAmount!=null) {
				return forecastAmount;
			}
			else return forecastAmount = new Money.MoneyBuilder();
		}
		
		
		public BigDecimal getForecastRate() {
			return forecastRate;
		}
		
		
		public FxLinkedNotionalAmount.FxLinkedNotionalAmountBuilder getFxLinkedNotionalAmount() {
			return fxLinkedNotionalAmount;
		}
		
		public FxLinkedNotionalAmount.FxLinkedNotionalAmountBuilder getOrCreateFxLinkedNotionalAmount() {
			if (fxLinkedNotionalAmount!=null) {
				return fxLinkedNotionalAmount;
			}
			else return fxLinkedNotionalAmount = new FxLinkedNotionalAmount.FxLinkedNotionalAmountBuilder();
		}
		
		
		public BigDecimal getNotionalAmount() {
			return notionalAmount;
		}
		
		
		public Date getUnadjustedEndDate() {
			return unadjustedEndDate;
		}
		
		
		public Date getUnadjustedStartDate() {
			return unadjustedStartDate;
		}
		
	
		public CalculationPeriodBuilder setCalculationPeriodNumberOfDays(Integer calculationPeriodNumberOfDays) {
			this.calculationPeriodNumberOfDays = calculationPeriodNumberOfDays;
			return this;
		}
		public CalculationPeriodBuilder setDayCountYearFraction(BigDecimal dayCountYearFraction) {
			this.dayCountYearFraction = dayCountYearFraction;
			return this;
		}
		public CalculationPeriodBuilder setFixedRate(BigDecimal fixedRate) {
			this.fixedRate = fixedRate;
			return this;
		}
		public CalculationPeriodBuilder setFloatingRateDefinition(FloatingRateDefinition floatingRateDefinition) {
			this.floatingRateDefinition = floatingRateDefinition.toBuilder();
			return this;
		}
		public CalculationPeriodBuilder setFloatingRateDefinitionBuilder(FloatingRateDefinition.FloatingRateDefinitionBuilder floatingRateDefinition) {
			this.floatingRateDefinition = floatingRateDefinition;
			return this;
		}
		public CalculationPeriodBuilder setForecastAmount(Money forecastAmount) {
			this.forecastAmount = forecastAmount.toBuilder();
			return this;
		}
		public CalculationPeriodBuilder setForecastAmountBuilder(Money.MoneyBuilder forecastAmount) {
			this.forecastAmount = forecastAmount;
			return this;
		}
		public CalculationPeriodBuilder setForecastRate(BigDecimal forecastRate) {
			this.forecastRate = forecastRate;
			return this;
		}
		public CalculationPeriodBuilder setFxLinkedNotionalAmount(FxLinkedNotionalAmount fxLinkedNotionalAmount) {
			this.fxLinkedNotionalAmount = fxLinkedNotionalAmount.toBuilder();
			return this;
		}
		public CalculationPeriodBuilder setFxLinkedNotionalAmountBuilder(FxLinkedNotionalAmount.FxLinkedNotionalAmountBuilder fxLinkedNotionalAmount) {
			this.fxLinkedNotionalAmount = fxLinkedNotionalAmount;
			return this;
		}
		public CalculationPeriodBuilder setNotionalAmount(BigDecimal notionalAmount) {
			this.notionalAmount = notionalAmount;
			return this;
		}
		public CalculationPeriodBuilder setUnadjustedEndDate(Date unadjustedEndDate) {
			this.unadjustedEndDate = unadjustedEndDate;
			return this;
		}
		public CalculationPeriodBuilder setUnadjustedStartDate(Date unadjustedStartDate) {
			this.unadjustedStartDate = unadjustedStartDate;
			return this;
		}
		@Override public CalculationPeriodBuilder setAdjustedEndDate(Date adjustedEndDate) {
			this.adjustedEndDate = adjustedEndDate;
			return this;
		}
		@Override public CalculationPeriodBuilder setAdjustedStartDate(Date adjustedStartDate) {
			this.adjustedStartDate = adjustedStartDate;
			return this;
		}
		@Override public CalculationPeriodBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		@Override public CalculationPeriodBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		
		public CalculationPeriod build() {
			return new CalculationPeriod(this);
		}
	
		@Override
		public CalculationPeriodBuilder prune() {
			super.prune();
			if (floatingRateDefinition!=null && !floatingRateDefinition.prune().hasData()) floatingRateDefinition = null;
			if (forecastAmount!=null && !forecastAmount.prune().hasData()) forecastAmount = null;
			if (fxLinkedNotionalAmount!=null && !fxLinkedNotionalAmount.prune().hasData()) fxLinkedNotionalAmount = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (super.hasData()) return true;
			if (getCalculationPeriodNumberOfDays()!=null) return true;
			if (getDayCountYearFraction()!=null) return true;
			if (getFixedRate()!=null) return true;
			if (getFloatingRateDefinition()!=null && getFloatingRateDefinition().hasData()) return true;
			if (getForecastAmount()!=null && getForecastAmount().hasData()) return true;
			if (getForecastRate()!=null) return true;
			if (getFxLinkedNotionalAmount()!=null && getFxLinkedNotionalAmount().hasData()) return true;
			if (getNotionalAmount()!=null) return true;
			if (getUnadjustedEndDate()!=null) return true;
			if (getUnadjustedStartDate()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
			super.process(path, processor);
		
			processor.processBasic(path.newSubPath("calculationPeriodNumberOfDays"), Integer.class, calculationPeriodNumberOfDays, this);
			processor.processBasic(path.newSubPath("dayCountYearFraction"), BigDecimal.class, dayCountYearFraction, this);
			processor.processBasic(path.newSubPath("fixedRate"), BigDecimal.class, fixedRate, this);
			processor.processBasic(path.newSubPath("forecastRate"), BigDecimal.class, forecastRate, this);
			processor.processBasic(path.newSubPath("notionalAmount"), BigDecimal.class, notionalAmount, this);
			processor.processBasic(path.newSubPath("unadjustedEndDate"), Date.class, unadjustedEndDate, this);
			processor.processBasic(path.newSubPath("unadjustedStartDate"), Date.class, unadjustedStartDate, this);
			
			processRosetta(path.newSubPath("floatingRateDefinition"), processor, FloatingRateDefinition.class, floatingRateDefinition);
			processRosetta(path.newSubPath("forecastAmount"), processor, Money.class, forecastAmount);
			processRosetta(path.newSubPath("fxLinkedNotionalAmount"), processor, FxLinkedNotionalAmount.class, fxLinkedNotionalAmount);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
			if (!super.equals(o)) return false;
		
			CalculationPeriodBuilder _that = (CalculationPeriodBuilder) o;
		
			if (calculationPeriodNumberOfDays != null ? !calculationPeriodNumberOfDays.equals(_that.calculationPeriodNumberOfDays) : _that.calculationPeriodNumberOfDays != null) return false;
			if (dayCountYearFraction != null ? !dayCountYearFraction.equals(_that.dayCountYearFraction) : _that.dayCountYearFraction != null) return false;
			if (fixedRate != null ? !fixedRate.equals(_that.fixedRate) : _that.fixedRate != null) return false;
			if (floatingRateDefinition != null ? !floatingRateDefinition.equals(_that.floatingRateDefinition) : _that.floatingRateDefinition != null) return false;
			if (forecastAmount != null ? !forecastAmount.equals(_that.forecastAmount) : _that.forecastAmount != null) return false;
			if (forecastRate != null ? !forecastRate.equals(_that.forecastRate) : _that.forecastRate != null) return false;
			if (fxLinkedNotionalAmount != null ? !fxLinkedNotionalAmount.equals(_that.fxLinkedNotionalAmount) : _that.fxLinkedNotionalAmount != null) return false;
			if (notionalAmount != null ? !notionalAmount.equals(_that.notionalAmount) : _that.notionalAmount != null) return false;
			if (unadjustedEndDate != null ? !unadjustedEndDate.equals(_that.unadjustedEndDate) : _that.unadjustedEndDate != null) return false;
			if (unadjustedStartDate != null ? !unadjustedStartDate.equals(_that.unadjustedStartDate) : _that.unadjustedStartDate != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = super.hashCode();
			_result = 31 * _result + (calculationPeriodNumberOfDays != null ? calculationPeriodNumberOfDays.hashCode() : 0);
			_result = 31 * _result + (dayCountYearFraction != null ? dayCountYearFraction.hashCode() : 0);
			_result = 31 * _result + (fixedRate != null ? fixedRate.hashCode() : 0);
			_result = 31 * _result + (floatingRateDefinition != null ? floatingRateDefinition.hashCode() : 0);
			_result = 31 * _result + (forecastAmount != null ? forecastAmount.hashCode() : 0);
			_result = 31 * _result + (forecastRate != null ? forecastRate.hashCode() : 0);
			_result = 31 * _result + (fxLinkedNotionalAmount != null ? fxLinkedNotionalAmount.hashCode() : 0);
			_result = 31 * _result + (notionalAmount != null ? notionalAmount.hashCode() : 0);
			_result = 31 * _result + (unadjustedEndDate != null ? unadjustedEndDate.hashCode() : 0);
			_result = 31 * _result + (unadjustedStartDate != null ? unadjustedStartDate.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "CalculationPeriodBuilder {" +
				"calculationPeriodNumberOfDays=" + this.calculationPeriodNumberOfDays + ", " +
				"dayCountYearFraction=" + this.dayCountYearFraction + ", " +
				"fixedRate=" + this.fixedRate + ", " +
				"floatingRateDefinition=" + this.floatingRateDefinition + ", " +
				"forecastAmount=" + this.forecastAmount + ", " +
				"forecastRate=" + this.forecastRate + ", " +
				"fxLinkedNotionalAmount=" + this.fxLinkedNotionalAmount + ", " +
				"notionalAmount=" + this.notionalAmount + ", " +
				"unadjustedEndDate=" + this.unadjustedEndDate + ", " +
				"unadjustedStartDate=" + this.unadjustedStartDate +
			'}' + " " + super.toString();
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		super.process(path, processor);
		
		processor.processBasic(path.newSubPath("calculationPeriodNumberOfDays"), Integer.class, calculationPeriodNumberOfDays, this);
		processor.processBasic(path.newSubPath("dayCountYearFraction"), BigDecimal.class, dayCountYearFraction, this);
		processor.processBasic(path.newSubPath("fixedRate"), BigDecimal.class, fixedRate, this);
		processor.processBasic(path.newSubPath("forecastRate"), BigDecimal.class, forecastRate, this);
		processor.processBasic(path.newSubPath("notionalAmount"), BigDecimal.class, notionalAmount, this);
		processor.processBasic(path.newSubPath("unadjustedEndDate"), Date.class, unadjustedEndDate, this);
		processor.processBasic(path.newSubPath("unadjustedStartDate"), Date.class, unadjustedStartDate, this);
		
		processRosetta(path.newSubPath("floatingRateDefinition"), processor, FloatingRateDefinition.class, floatingRateDefinition);
		processRosetta(path.newSubPath("forecastAmount"), processor, Money.class, forecastAmount);
		processRosetta(path.newSubPath("fxLinkedNotionalAmount"), processor, FxLinkedNotionalAmount.class, fxLinkedNotionalAmount);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
		if (!super.equals(o)) return false;
	
		CalculationPeriod _that = (CalculationPeriod) o;
	
		if (calculationPeriodNumberOfDays != null ? !calculationPeriodNumberOfDays.equals(_that.calculationPeriodNumberOfDays) : _that.calculationPeriodNumberOfDays != null) return false;
		if (dayCountYearFraction != null ? !dayCountYearFraction.equals(_that.dayCountYearFraction) : _that.dayCountYearFraction != null) return false;
		if (fixedRate != null ? !fixedRate.equals(_that.fixedRate) : _that.fixedRate != null) return false;
		if (floatingRateDefinition != null ? !floatingRateDefinition.equals(_that.floatingRateDefinition) : _that.floatingRateDefinition != null) return false;
		if (forecastAmount != null ? !forecastAmount.equals(_that.forecastAmount) : _that.forecastAmount != null) return false;
		if (forecastRate != null ? !forecastRate.equals(_that.forecastRate) : _that.forecastRate != null) return false;
		if (fxLinkedNotionalAmount != null ? !fxLinkedNotionalAmount.equals(_that.fxLinkedNotionalAmount) : _that.fxLinkedNotionalAmount != null) return false;
		if (notionalAmount != null ? !notionalAmount.equals(_that.notionalAmount) : _that.notionalAmount != null) return false;
		if (unadjustedEndDate != null ? !unadjustedEndDate.equals(_that.unadjustedEndDate) : _that.unadjustedEndDate != null) return false;
		if (unadjustedStartDate != null ? !unadjustedStartDate.equals(_that.unadjustedStartDate) : _that.unadjustedStartDate != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = super.hashCode();
		_result = 31 * _result + (calculationPeriodNumberOfDays != null ? calculationPeriodNumberOfDays.hashCode() : 0);
		_result = 31 * _result + (dayCountYearFraction != null ? dayCountYearFraction.hashCode() : 0);
		_result = 31 * _result + (fixedRate != null ? fixedRate.hashCode() : 0);
		_result = 31 * _result + (floatingRateDefinition != null ? floatingRateDefinition.hashCode() : 0);
		_result = 31 * _result + (forecastAmount != null ? forecastAmount.hashCode() : 0);
		_result = 31 * _result + (forecastRate != null ? forecastRate.hashCode() : 0);
		_result = 31 * _result + (fxLinkedNotionalAmount != null ? fxLinkedNotionalAmount.hashCode() : 0);
		_result = 31 * _result + (notionalAmount != null ? notionalAmount.hashCode() : 0);
		_result = 31 * _result + (unadjustedEndDate != null ? unadjustedEndDate.hashCode() : 0);
		_result = 31 * _result + (unadjustedStartDate != null ? unadjustedStartDate.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "CalculationPeriod {" +
			"calculationPeriodNumberOfDays=" + this.calculationPeriodNumberOfDays + ", " +
			"dayCountYearFraction=" + this.dayCountYearFraction + ", " +
			"fixedRate=" + this.fixedRate + ", " +
			"floatingRateDefinition=" + this.floatingRateDefinition + ", " +
			"forecastAmount=" + this.forecastAmount + ", " +
			"forecastRate=" + this.forecastRate + ", " +
			"fxLinkedNotionalAmount=" + this.fxLinkedNotionalAmount + ", " +
			"notionalAmount=" + this.notionalAmount + ", " +
			"unadjustedEndDate=" + this.unadjustedEndDate + ", " +
			"unadjustedStartDate=" + this.unadjustedStartDate +
		'}' + " " + super.toString();
	}
}
