package org.isda.cdm;

import com.rosetta.model.lib.records.Date;
import org.isda.cdm.AdjustableDate;
import org.isda.cdm.AdjustableOrRelativeDate;
import org.isda.cdm.BusinessDayAdjustments;
import org.isda.cdm.CalculationPeriodFrequency;
import org.isda.cdm.StubPeriodTypeEnum;
import org.isda.cdm.metafields.MetaFields;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.CalculationPeriodDatesMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;
		import com.rosetta.model.lib.records.Date;

/**
 * A class for defining the parameters used to generate the calculation period dates schedule, including the specification of any initial or final stub calculation periods. A calculation period schedule consists of an optional initial stub calculation period, one or more regular calculation periods and an optional final stub calculation period. In the absence of any initial or final stub calculation periods, the regular part of the calculation period schedule is assumed to be between the effective date and the termination date. No implicit stubs are allowed, i.e. stubs must be explicitly specified using an appropriate combination of firstPeriodStartDate, firstRegularPeriodStartDate and lastRegularPeriodEndDate.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class CalculationPeriodDates extends RosettaModelObject implements GlobalKey {
	private final BusinessDayAdjustments calculationPeriodDatesAdjustments;
	private final CalculationPeriodFrequency calculationPeriodFrequency;
	private final AdjustableOrRelativeDate effectiveDate;
	private final Date firstCompoundingPeriodEndDate;
	private final AdjustableDate firstPeriodStartDate;
	private final Date firstRegularPeriodStartDate;
	private final Date lastRegularPeriodEndDate;
	private final MetaFields meta;
	private final StubPeriodTypeEnum stubPeriodType;
	private final AdjustableOrRelativeDate terminationDate;
	private static CalculationPeriodDatesMeta metaData = new CalculationPeriodDatesMeta();
	
	CalculationPeriodDates(CalculationPeriodDatesBuilder builder) {
		this.calculationPeriodDatesAdjustments = ofNullable(builder.getCalculationPeriodDatesAdjustments()).map(f->f.build()).orElse(null);
		this.calculationPeriodFrequency = ofNullable(builder.getCalculationPeriodFrequency()).map(f->f.build()).orElse(null);
		this.effectiveDate = ofNullable(builder.getEffectiveDate()).map(f->f.build()).orElse(null);
		this.firstCompoundingPeriodEndDate = builder.getFirstCompoundingPeriodEndDate();
		this.firstPeriodStartDate = ofNullable(builder.getFirstPeriodStartDate()).map(f->f.build()).orElse(null);
		this.firstRegularPeriodStartDate = builder.getFirstRegularPeriodStartDate();
		this.lastRegularPeriodEndDate = builder.getLastRegularPeriodEndDate();
		this.meta = ofNullable(builder.getMeta()).map(f->f.build()).orElse(null);
		this.stubPeriodType = builder.getStubPeriodType();
		this.terminationDate = ofNullable(builder.getTerminationDate()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The specification of the business day convention and financial business centers used for adjusting any calculation period date if it would otherwise fall on a day that is not a business day in the specified business center.
	 */
	@RosettaSynonym(value="calculationPeriodDatesAdjustments", source="FpML_5_10", path="calculationPeriodDates")
	@RosettaSynonym(value="calculationPeriodDatesAdjustments", source="CME_SubmissionIRS_1_0", path="calculationPeriodDates")
	@RosettaSynonym(value="calculationPeriodDatesAdjustments", source="DTCC_11_0", path="calculationPeriodDates")
	@RosettaSynonym(value="calculationPeriodDatesAdjustments", source="DTCC_9_0", path="calculationPeriodDates")
	@RosettaSynonym(value="calculationPeriodDatesAdjustments", source="CME_ClearedConfirm_1_17", path="calculationPeriodDates")
	@RosettaSynonym(value="calculationPeriodDatesAdjustments", source="FpML_5_10", path="interestLegCalculationPeriodDates.interestLegPaymentDates.periodicDates")
	public final BusinessDayAdjustments getCalculationPeriodDatesAdjustments() {
		return calculationPeriodDatesAdjustments;
	}
	
	/**
	 * The frequency at which calculation period end dates occur with the regular part of the calculation period schedule and their roll date convention.
	 */
	@RosettaSynonym(value="calculationPeriodFrequency", source="FpML_5_10", path="calculationPeriodDates")
	@RosettaSynonym(value="calculationPeriodFrequency", source="CME_SubmissionIRS_1_0", path="calculationPeriodDates")
	@RosettaSynonym(value="calculationPeriodFrequency", source="DTCC_11_0", path="calculationPeriodDates")
	@RosettaSynonym(value="calculationPeriodFrequency", source="DTCC_9_0", path="calculationPeriodDates")
	@RosettaSynonym(value="calculationPeriodFrequency", source="CME_ClearedConfirm_1_17", path="calculationPeriodDates")
	@RosettaSynonym(value="periodicPayment", source="FpML_5_10")
	@RosettaSynonym(value="periodicPayment", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="periodicPayment", source="DTCC_11_0")
	@RosettaSynonym(value="periodicPayment", source="DTCC_9_0")
	@RosettaSynonym(value="periodicPayment", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="calculationPeriodFrequency", source="FpML_5_10", path="interestLegCalculationPeriodDates.interestLegPaymentDates.periodicDates")
	public final CalculationPeriodFrequency getCalculationPeriodFrequency() {
		return calculationPeriodFrequency;
	}
	
	/**
	 * The first day of the terms of the trade. This day may be subject to adjustment in accordance with a business day convention.
	 */
	@RosettaSynonym(value="effectiveDate", source="FpML_5_10")
	@RosettaSynonym(value="effectiveDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="effectiveDate", source="DTCC_11_0")
	@RosettaSynonym(value="effectiveDate", source="DTCC_9_0")
	@RosettaSynonym(value="effectiveDate", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="effectiveDate", source="FpML_5_10", path="calculationPeriodDates")
	@RosettaSynonym(value="effectiveDate", source="CME_SubmissionIRS_1_0", path="calculationPeriodDates")
	@RosettaSynonym(value="effectiveDate", source="DTCC_11_0", path="calculationPeriodDates")
	@RosettaSynonym(value="effectiveDate", source="DTCC_9_0", path="calculationPeriodDates")
	@RosettaSynonym(value="effectiveDate", source="CME_ClearedConfirm_1_17", path="calculationPeriodDates")
	@RosettaSynonym(value="relativeEffectiveDate", source="FpML_5_10", path="calculationPeriodDates")
	@RosettaSynonym(value="relativeEffectiveDate", source="CME_SubmissionIRS_1_0", path="calculationPeriodDates")
	@RosettaSynonym(value="relativeEffectiveDate", source="DTCC_11_0", path="calculationPeriodDates")
	@RosettaSynonym(value="relativeEffectiveDate", source="DTCC_9_0", path="calculationPeriodDates")
	@RosettaSynonym(value="relativeEffectiveDate", source="CME_ClearedConfirm_1_17", path="calculationPeriodDates")
	@RosettaSynonym(value="", source="FpML_5_10")
	@RosettaSynonym(value="", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="", source="DTCC_11_0")
	@RosettaSynonym(value="", source="DTCC_9_0")
	@RosettaSynonym(value="", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="effectiveDate", source="FpML_5_10", path="interestLegCalculationPeriodDates")
	@RosettaSynonym(value="settlementDate", source="FpML_5_10", path="nearLeg")
	public final AdjustableOrRelativeDate getEffectiveDate() {
		return effectiveDate;
	}
	
	/**
	 * The end date of the initial compounding period when compounding is applicable. It must only be specified when the compoundingMethod element is present and not equal to a value of None. This date may be subject to adjustment in accordance with any adjustments specified in calculationPeriodDatesAdjustments.
	 */
	@RosettaSynonym(value="firstCompoundingPeriodEndDate", source="FpML_5_10", path="calculationPeriodDates")
	@RosettaSynonym(value="firstCompoundingPeriodEndDate", source="CME_SubmissionIRS_1_0", path="calculationPeriodDates")
	@RosettaSynonym(value="firstCompoundingPeriodEndDate", source="DTCC_11_0", path="calculationPeriodDates")
	@RosettaSynonym(value="firstCompoundingPeriodEndDate", source="DTCC_9_0", path="calculationPeriodDates")
	@RosettaSynonym(value="firstCompoundingPeriodEndDate", source="CME_ClearedConfirm_1_17", path="calculationPeriodDates")
	public final Date getFirstCompoundingPeriodEndDate() {
		return firstCompoundingPeriodEndDate;
	}
	
	/**
	 * The start date of the calculation period. FpML specifies that for interest rate swaps this date must only be specified if it is not equal to the effective date. It is always specified in the case of equity swaps and credit default swaps with periodic payments. This date may be subject to adjustment in accordance with a business day convention.
	 */
	@RosettaSynonym(value="firstPeriodStartDate", source="FpML_5_10", path="calculationPeriodDates")
	@RosettaSynonym(value="firstPeriodStartDate", source="CME_SubmissionIRS_1_0", path="calculationPeriodDates")
	@RosettaSynonym(value="firstPeriodStartDate", source="DTCC_11_0", path="calculationPeriodDates")
	@RosettaSynonym(value="firstPeriodStartDate", source="DTCC_9_0", path="calculationPeriodDates")
	@RosettaSynonym(value="firstPeriodStartDate", source="CME_ClearedConfirm_1_17", path="calculationPeriodDates")
	@RosettaSynonym(value="periodicPayment", source="FpML_5_10")
	@RosettaSynonym(value="periodicPayment", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="periodicPayment", source="DTCC_11_0")
	@RosettaSynonym(value="periodicPayment", source="DTCC_9_0")
	@RosettaSynonym(value="periodicPayment", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="calculationStartDate", source="FpML_5_10", path="interestLegCalculationPeriodDates.interestLegPaymentDates.periodicDates")
	public final AdjustableDate getFirstPeriodStartDate() {
		return firstPeriodStartDate;
	}
	
	/**
	 * The start date of the regular part of the calculation period schedule. It must only be specified if there is an initial stub calculation period. This day may be subject to adjustment in accordance with any adjustments specified in calculationPeriodDatesAdjustments.
	 */
	@RosettaSynonym(value="firstRegularPeriodStartDate", source="FpML_5_10", path="calculationPeriodDates")
	@RosettaSynonym(value="firstRegularPeriodStartDate", source="CME_SubmissionIRS_1_0", path="calculationPeriodDates")
	@RosettaSynonym(value="firstRegularPeriodStartDate", source="DTCC_11_0", path="calculationPeriodDates")
	@RosettaSynonym(value="firstRegularPeriodStartDate", source="DTCC_9_0", path="calculationPeriodDates")
	@RosettaSynonym(value="firstRegularPeriodStartDate", source="CME_ClearedConfirm_1_17", path="calculationPeriodDates")
	public final Date getFirstRegularPeriodStartDate() {
		return firstRegularPeriodStartDate;
	}
	
	/**
	 * The end date of the regular part of the calculation period schedule. It must only be specified if there is a final stub calculation period. This day may be subject to adjustment in accordance with any adjustments specified in calculationPeriodDatesAdjustments.
	 */
	@RosettaSynonym(value="lastRegularPeriodEndDate", source="FpML_5_10", path="calculationPeriodDates")
	@RosettaSynonym(value="lastRegularPeriodEndDate", source="CME_SubmissionIRS_1_0", path="calculationPeriodDates")
	@RosettaSynonym(value="lastRegularPeriodEndDate", source="DTCC_11_0", path="calculationPeriodDates")
	@RosettaSynonym(value="lastRegularPeriodEndDate", source="DTCC_9_0", path="calculationPeriodDates")
	@RosettaSynonym(value="lastRegularPeriodEndDate", source="CME_ClearedConfirm_1_17", path="calculationPeriodDates")
	public final Date getLastRegularPeriodEndDate() {
		return lastRegularPeriodEndDate;
	}
	
	public final MetaFields getMeta() {
		return meta;
	}
	
	/**
	 * Method to allocate any irregular period remaining after regular periods have been allocated between the effective and termination date.
	 */
	@RosettaSynonym(value="stubPeriodType", source="FpML_5_10", path="calculationPeriodDates")
	@RosettaSynonym(value="stubPeriodType", source="CME_SubmissionIRS_1_0", path="calculationPeriodDates")
	@RosettaSynonym(value="stubPeriodType", source="DTCC_11_0", path="calculationPeriodDates")
	@RosettaSynonym(value="stubPeriodType", source="DTCC_9_0", path="calculationPeriodDates")
	@RosettaSynonym(value="stubPeriodType", source="CME_ClearedConfirm_1_17", path="calculationPeriodDates")
	public final StubPeriodTypeEnum getStubPeriodType() {
		return stubPeriodType;
	}
	
	/**
	 * The last day of the terms of the trade. This date may be subject to adjustments in accordance with the business day convention. It can also be specified in relation to another scheduled date (e.g. the last payment date).
	 */
	@RosettaSynonym(value="terminationDate", source="FpML_5_10", path="calculationPeriodDates")
	@RosettaSynonym(value="terminationDate", source="CME_SubmissionIRS_1_0", path="calculationPeriodDates")
	@RosettaSynonym(value="terminationDate", source="DTCC_11_0", path="calculationPeriodDates")
	@RosettaSynonym(value="terminationDate", source="DTCC_9_0", path="calculationPeriodDates")
	@RosettaSynonym(value="terminationDate", source="CME_ClearedConfirm_1_17", path="calculationPeriodDates")
	@RosettaSynonym(value="relativeTerminationDate", source="FpML_5_10", path="calculationPeriodDates")
	@RosettaSynonym(value="relativeTerminationDate", source="CME_SubmissionIRS_1_0", path="calculationPeriodDates")
	@RosettaSynonym(value="relativeTerminationDate", source="DTCC_11_0", path="calculationPeriodDates")
	@RosettaSynonym(value="relativeTerminationDate", source="DTCC_9_0", path="calculationPeriodDates")
	@RosettaSynonym(value="relativeTerminationDate", source="CME_ClearedConfirm_1_17", path="calculationPeriodDates")
	@RosettaSynonym(value="", source="FpML_5_10")
	@RosettaSynonym(value="", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="", source="DTCC_11_0")
	@RosettaSynonym(value="", source="DTCC_9_0")
	@RosettaSynonym(value="", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="terminationDate", source="FpML_5_10", path="interestLegCalculationPeriodDates")
	@RosettaSynonym(value="terminationDate", source="FpML_5_10", path="")
	@RosettaSynonym(value="settlementDate", source="FpML_5_10", path="farLeg")
	public final AdjustableOrRelativeDate getTerminationDate() {
		return terminationDate;
	}
	
	
	@Override
	public RosettaMetaData<? extends CalculationPeriodDates> metaData() {
		return metaData;
	} 
	
	public CalculationPeriodDatesBuilder toBuilder() {
		CalculationPeriodDatesBuilder builder = new CalculationPeriodDatesBuilder();
		ofNullable(getCalculationPeriodDatesAdjustments()).ifPresent(builder::setCalculationPeriodDatesAdjustments);
		ofNullable(getCalculationPeriodFrequency()).ifPresent(builder::setCalculationPeriodFrequency);
		ofNullable(getEffectiveDate()).ifPresent(builder::setEffectiveDate);
		ofNullable(getFirstCompoundingPeriodEndDate()).ifPresent(builder::setFirstCompoundingPeriodEndDate);
		ofNullable(getFirstPeriodStartDate()).ifPresent(builder::setFirstPeriodStartDate);
		ofNullable(getFirstRegularPeriodStartDate()).ifPresent(builder::setFirstRegularPeriodStartDate);
		ofNullable(getLastRegularPeriodEndDate()).ifPresent(builder::setLastRegularPeriodEndDate);
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		ofNullable(getStubPeriodType()).ifPresent(builder::setStubPeriodType);
		ofNullable(getTerminationDate()).ifPresent(builder::setTerminationDate);
		return builder;
	}

	public static CalculationPeriodDatesBuilder builder() {
		return new CalculationPeriodDatesBuilder();
	}

	public static  class CalculationPeriodDatesBuilder extends RosettaModelObjectBuilder implements GlobalKeyBuilder<CalculationPeriodDatesBuilder> {
	
		protected BusinessDayAdjustments.BusinessDayAdjustmentsBuilder calculationPeriodDatesAdjustments;
		protected CalculationPeriodFrequency.CalculationPeriodFrequencyBuilder calculationPeriodFrequency;
		protected AdjustableOrRelativeDate.AdjustableOrRelativeDateBuilder effectiveDate;
		protected Date firstCompoundingPeriodEndDate;
		protected AdjustableDate.AdjustableDateBuilder firstPeriodStartDate;
		protected Date firstRegularPeriodStartDate;
		protected Date lastRegularPeriodEndDate;
		protected MetaFields.MetaFieldsBuilder meta;
		protected StubPeriodTypeEnum stubPeriodType;
		protected AdjustableOrRelativeDate.AdjustableOrRelativeDateBuilder terminationDate;
	
		public CalculationPeriodDatesBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends CalculationPeriodDates> metaData() {
			return metaData;
		} 
	
		
		public BusinessDayAdjustments.BusinessDayAdjustmentsBuilder getCalculationPeriodDatesAdjustments() {
			return calculationPeriodDatesAdjustments;
		}
		
		public BusinessDayAdjustments.BusinessDayAdjustmentsBuilder getOrCreateCalculationPeriodDatesAdjustments() {
			if (calculationPeriodDatesAdjustments!=null) {
				return calculationPeriodDatesAdjustments;
			}
			else return calculationPeriodDatesAdjustments = new BusinessDayAdjustments.BusinessDayAdjustmentsBuilder();
		}
		
		
		public CalculationPeriodFrequency.CalculationPeriodFrequencyBuilder getCalculationPeriodFrequency() {
			return calculationPeriodFrequency;
		}
		
		public CalculationPeriodFrequency.CalculationPeriodFrequencyBuilder getOrCreateCalculationPeriodFrequency() {
			if (calculationPeriodFrequency!=null) {
				return calculationPeriodFrequency;
			}
			else return calculationPeriodFrequency = new CalculationPeriodFrequency.CalculationPeriodFrequencyBuilder();
		}
		
		
		public AdjustableOrRelativeDate.AdjustableOrRelativeDateBuilder getEffectiveDate() {
			return effectiveDate;
		}
		
		public AdjustableOrRelativeDate.AdjustableOrRelativeDateBuilder getOrCreateEffectiveDate() {
			if (effectiveDate!=null) {
				return effectiveDate;
			}
			else return effectiveDate = new AdjustableOrRelativeDate.AdjustableOrRelativeDateBuilder();
		}
		
		
		public Date getFirstCompoundingPeriodEndDate() {
			return firstCompoundingPeriodEndDate;
		}
		
		
		public AdjustableDate.AdjustableDateBuilder getFirstPeriodStartDate() {
			return firstPeriodStartDate;
		}
		
		public AdjustableDate.AdjustableDateBuilder getOrCreateFirstPeriodStartDate() {
			if (firstPeriodStartDate!=null) {
				return firstPeriodStartDate;
			}
			else return firstPeriodStartDate = new AdjustableDate.AdjustableDateBuilder();
		}
		
		
		public Date getFirstRegularPeriodStartDate() {
			return firstRegularPeriodStartDate;
		}
		
		
		public Date getLastRegularPeriodEndDate() {
			return lastRegularPeriodEndDate;
		}
		
		
		public MetaFields.MetaFieldsBuilder getMeta() {
			return meta;
		}
		
		public MetaFields.MetaFieldsBuilder getOrCreateMeta() {
			if (meta!=null) {
				return meta;
			}
			else return meta = new MetaFields.MetaFieldsBuilder();
		}
		
		
		public StubPeriodTypeEnum getStubPeriodType() {
			return stubPeriodType;
		}
		
		
		public AdjustableOrRelativeDate.AdjustableOrRelativeDateBuilder getTerminationDate() {
			return terminationDate;
		}
		
		public AdjustableOrRelativeDate.AdjustableOrRelativeDateBuilder getOrCreateTerminationDate() {
			if (terminationDate!=null) {
				return terminationDate;
			}
			else return terminationDate = new AdjustableOrRelativeDate.AdjustableOrRelativeDateBuilder();
		}
		
	
		public CalculationPeriodDatesBuilder setCalculationPeriodDatesAdjustments(BusinessDayAdjustments calculationPeriodDatesAdjustments) {
			this.calculationPeriodDatesAdjustments = calculationPeriodDatesAdjustments.toBuilder();
			return this;
		}
		public CalculationPeriodDatesBuilder setCalculationPeriodDatesAdjustmentsBuilder(BusinessDayAdjustments.BusinessDayAdjustmentsBuilder calculationPeriodDatesAdjustments) {
			this.calculationPeriodDatesAdjustments = calculationPeriodDatesAdjustments;
			return this;
		}
		public CalculationPeriodDatesBuilder setCalculationPeriodFrequency(CalculationPeriodFrequency calculationPeriodFrequency) {
			this.calculationPeriodFrequency = calculationPeriodFrequency.toBuilder();
			return this;
		}
		public CalculationPeriodDatesBuilder setCalculationPeriodFrequencyBuilder(CalculationPeriodFrequency.CalculationPeriodFrequencyBuilder calculationPeriodFrequency) {
			this.calculationPeriodFrequency = calculationPeriodFrequency;
			return this;
		}
		public CalculationPeriodDatesBuilder setEffectiveDate(AdjustableOrRelativeDate effectiveDate) {
			this.effectiveDate = effectiveDate.toBuilder();
			return this;
		}
		public CalculationPeriodDatesBuilder setEffectiveDateBuilder(AdjustableOrRelativeDate.AdjustableOrRelativeDateBuilder effectiveDate) {
			this.effectiveDate = effectiveDate;
			return this;
		}
		public CalculationPeriodDatesBuilder setFirstCompoundingPeriodEndDate(Date firstCompoundingPeriodEndDate) {
			this.firstCompoundingPeriodEndDate = firstCompoundingPeriodEndDate;
			return this;
		}
		public CalculationPeriodDatesBuilder setFirstPeriodStartDate(AdjustableDate firstPeriodStartDate) {
			this.firstPeriodStartDate = firstPeriodStartDate.toBuilder();
			return this;
		}
		public CalculationPeriodDatesBuilder setFirstPeriodStartDateBuilder(AdjustableDate.AdjustableDateBuilder firstPeriodStartDate) {
			this.firstPeriodStartDate = firstPeriodStartDate;
			return this;
		}
		public CalculationPeriodDatesBuilder setFirstRegularPeriodStartDate(Date firstRegularPeriodStartDate) {
			this.firstRegularPeriodStartDate = firstRegularPeriodStartDate;
			return this;
		}
		public CalculationPeriodDatesBuilder setLastRegularPeriodEndDate(Date lastRegularPeriodEndDate) {
			this.lastRegularPeriodEndDate = lastRegularPeriodEndDate;
			return this;
		}
		public CalculationPeriodDatesBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		public CalculationPeriodDatesBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		public CalculationPeriodDatesBuilder setStubPeriodType(StubPeriodTypeEnum stubPeriodType) {
			this.stubPeriodType = stubPeriodType;
			return this;
		}
		public CalculationPeriodDatesBuilder setTerminationDate(AdjustableOrRelativeDate terminationDate) {
			this.terminationDate = terminationDate.toBuilder();
			return this;
		}
		public CalculationPeriodDatesBuilder setTerminationDateBuilder(AdjustableOrRelativeDate.AdjustableOrRelativeDateBuilder terminationDate) {
			this.terminationDate = terminationDate;
			return this;
		}
		
		public CalculationPeriodDates build() {
			return new CalculationPeriodDates(this);
		}
	
		@Override
		public CalculationPeriodDatesBuilder prune() {
			if (calculationPeriodDatesAdjustments!=null && !calculationPeriodDatesAdjustments.prune().hasData()) calculationPeriodDatesAdjustments = null;
			if (calculationPeriodFrequency!=null && !calculationPeriodFrequency.prune().hasData()) calculationPeriodFrequency = null;
			if (effectiveDate!=null && !effectiveDate.prune().hasData()) effectiveDate = null;
			if (firstPeriodStartDate!=null && !firstPeriodStartDate.prune().hasData()) firstPeriodStartDate = null;
			if (meta!=null && !meta.prune().hasData()) meta = null;
			if (terminationDate!=null && !terminationDate.prune().hasData()) terminationDate = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getCalculationPeriodDatesAdjustments()!=null && getCalculationPeriodDatesAdjustments().hasData()) return true;
			if (getCalculationPeriodFrequency()!=null && getCalculationPeriodFrequency().hasData()) return true;
			if (getEffectiveDate()!=null && getEffectiveDate().hasData()) return true;
			if (getFirstCompoundingPeriodEndDate()!=null) return true;
			if (getFirstPeriodStartDate()!=null && getFirstPeriodStartDate().hasData()) return true;
			if (getFirstRegularPeriodStartDate()!=null) return true;
			if (getLastRegularPeriodEndDate()!=null) return true;
			if (getMeta()!=null && getMeta().hasData()) return true;
			if (getStubPeriodType()!=null) return true;
			if (getTerminationDate()!=null && getTerminationDate().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("firstCompoundingPeriodEndDate"), Date.class, firstCompoundingPeriodEndDate, this);
			processor.processBasic(path.newSubPath("firstRegularPeriodStartDate"), Date.class, firstRegularPeriodStartDate, this);
			processor.processBasic(path.newSubPath("lastRegularPeriodEndDate"), Date.class, lastRegularPeriodEndDate, this);
			processor.processBasic(path.newSubPath("stubPeriodType"), StubPeriodTypeEnum.class, stubPeriodType, this);
			
			processRosetta(path.newSubPath("calculationPeriodDatesAdjustments"), processor, BusinessDayAdjustments.class, calculationPeriodDatesAdjustments);
			processRosetta(path.newSubPath("calculationPeriodFrequency"), processor, CalculationPeriodFrequency.class, calculationPeriodFrequency);
			processRosetta(path.newSubPath("effectiveDate"), processor, AdjustableOrRelativeDate.class, effectiveDate);
			processRosetta(path.newSubPath("firstPeriodStartDate"), processor, AdjustableDate.class, firstPeriodStartDate);
			processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
			processRosetta(path.newSubPath("terminationDate"), processor, AdjustableOrRelativeDate.class, terminationDate);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			CalculationPeriodDatesBuilder _that = (CalculationPeriodDatesBuilder) o;
		
			if (calculationPeriodDatesAdjustments != null ? !calculationPeriodDatesAdjustments.equals(_that.calculationPeriodDatesAdjustments) : _that.calculationPeriodDatesAdjustments != null) return false;
			if (calculationPeriodFrequency != null ? !calculationPeriodFrequency.equals(_that.calculationPeriodFrequency) : _that.calculationPeriodFrequency != null) return false;
			if (effectiveDate != null ? !effectiveDate.equals(_that.effectiveDate) : _that.effectiveDate != null) return false;
			if (firstCompoundingPeriodEndDate != null ? !firstCompoundingPeriodEndDate.equals(_that.firstCompoundingPeriodEndDate) : _that.firstCompoundingPeriodEndDate != null) return false;
			if (firstPeriodStartDate != null ? !firstPeriodStartDate.equals(_that.firstPeriodStartDate) : _that.firstPeriodStartDate != null) return false;
			if (firstRegularPeriodStartDate != null ? !firstRegularPeriodStartDate.equals(_that.firstRegularPeriodStartDate) : _that.firstRegularPeriodStartDate != null) return false;
			if (lastRegularPeriodEndDate != null ? !lastRegularPeriodEndDate.equals(_that.lastRegularPeriodEndDate) : _that.lastRegularPeriodEndDate != null) return false;
			if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
			if (stubPeriodType != null ? !stubPeriodType.equals(_that.stubPeriodType) : _that.stubPeriodType != null) return false;
			if (terminationDate != null ? !terminationDate.equals(_that.terminationDate) : _that.terminationDate != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (calculationPeriodDatesAdjustments != null ? calculationPeriodDatesAdjustments.hashCode() : 0);
			_result = 31 * _result + (calculationPeriodFrequency != null ? calculationPeriodFrequency.hashCode() : 0);
			_result = 31 * _result + (effectiveDate != null ? effectiveDate.hashCode() : 0);
			_result = 31 * _result + (firstCompoundingPeriodEndDate != null ? firstCompoundingPeriodEndDate.hashCode() : 0);
			_result = 31 * _result + (firstPeriodStartDate != null ? firstPeriodStartDate.hashCode() : 0);
			_result = 31 * _result + (firstRegularPeriodStartDate != null ? firstRegularPeriodStartDate.hashCode() : 0);
			_result = 31 * _result + (lastRegularPeriodEndDate != null ? lastRegularPeriodEndDate.hashCode() : 0);
			_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
			_result = 31 * _result + (stubPeriodType != null ? stubPeriodType.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (terminationDate != null ? terminationDate.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "CalculationPeriodDatesBuilder {" +
				"calculationPeriodDatesAdjustments=" + this.calculationPeriodDatesAdjustments + ", " +
				"calculationPeriodFrequency=" + this.calculationPeriodFrequency + ", " +
				"effectiveDate=" + this.effectiveDate + ", " +
				"firstCompoundingPeriodEndDate=" + this.firstCompoundingPeriodEndDate + ", " +
				"firstPeriodStartDate=" + this.firstPeriodStartDate + ", " +
				"firstRegularPeriodStartDate=" + this.firstRegularPeriodStartDate + ", " +
				"lastRegularPeriodEndDate=" + this.lastRegularPeriodEndDate + ", " +
				"meta=" + this.meta + ", " +
				"stubPeriodType=" + this.stubPeriodType + ", " +
				"terminationDate=" + this.terminationDate +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("firstCompoundingPeriodEndDate"), Date.class, firstCompoundingPeriodEndDate, this);
		processor.processBasic(path.newSubPath("firstRegularPeriodStartDate"), Date.class, firstRegularPeriodStartDate, this);
		processor.processBasic(path.newSubPath("lastRegularPeriodEndDate"), Date.class, lastRegularPeriodEndDate, this);
		processor.processBasic(path.newSubPath("stubPeriodType"), StubPeriodTypeEnum.class, stubPeriodType, this);
		
		processRosetta(path.newSubPath("calculationPeriodDatesAdjustments"), processor, BusinessDayAdjustments.class, calculationPeriodDatesAdjustments);
		processRosetta(path.newSubPath("calculationPeriodFrequency"), processor, CalculationPeriodFrequency.class, calculationPeriodFrequency);
		processRosetta(path.newSubPath("effectiveDate"), processor, AdjustableOrRelativeDate.class, effectiveDate);
		processRosetta(path.newSubPath("firstPeriodStartDate"), processor, AdjustableDate.class, firstPeriodStartDate);
		processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
		processRosetta(path.newSubPath("terminationDate"), processor, AdjustableOrRelativeDate.class, terminationDate);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		CalculationPeriodDates _that = (CalculationPeriodDates) o;
	
		if (calculationPeriodDatesAdjustments != null ? !calculationPeriodDatesAdjustments.equals(_that.calculationPeriodDatesAdjustments) : _that.calculationPeriodDatesAdjustments != null) return false;
		if (calculationPeriodFrequency != null ? !calculationPeriodFrequency.equals(_that.calculationPeriodFrequency) : _that.calculationPeriodFrequency != null) return false;
		if (effectiveDate != null ? !effectiveDate.equals(_that.effectiveDate) : _that.effectiveDate != null) return false;
		if (firstCompoundingPeriodEndDate != null ? !firstCompoundingPeriodEndDate.equals(_that.firstCompoundingPeriodEndDate) : _that.firstCompoundingPeriodEndDate != null) return false;
		if (firstPeriodStartDate != null ? !firstPeriodStartDate.equals(_that.firstPeriodStartDate) : _that.firstPeriodStartDate != null) return false;
		if (firstRegularPeriodStartDate != null ? !firstRegularPeriodStartDate.equals(_that.firstRegularPeriodStartDate) : _that.firstRegularPeriodStartDate != null) return false;
		if (lastRegularPeriodEndDate != null ? !lastRegularPeriodEndDate.equals(_that.lastRegularPeriodEndDate) : _that.lastRegularPeriodEndDate != null) return false;
		if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
		if (stubPeriodType != null ? !stubPeriodType.equals(_that.stubPeriodType) : _that.stubPeriodType != null) return false;
		if (terminationDate != null ? !terminationDate.equals(_that.terminationDate) : _that.terminationDate != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (calculationPeriodDatesAdjustments != null ? calculationPeriodDatesAdjustments.hashCode() : 0);
		_result = 31 * _result + (calculationPeriodFrequency != null ? calculationPeriodFrequency.hashCode() : 0);
		_result = 31 * _result + (effectiveDate != null ? effectiveDate.hashCode() : 0);
		_result = 31 * _result + (firstCompoundingPeriodEndDate != null ? firstCompoundingPeriodEndDate.hashCode() : 0);
		_result = 31 * _result + (firstPeriodStartDate != null ? firstPeriodStartDate.hashCode() : 0);
		_result = 31 * _result + (firstRegularPeriodStartDate != null ? firstRegularPeriodStartDate.hashCode() : 0);
		_result = 31 * _result + (lastRegularPeriodEndDate != null ? lastRegularPeriodEndDate.hashCode() : 0);
		_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
		_result = 31 * _result + (stubPeriodType != null ? stubPeriodType.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (terminationDate != null ? terminationDate.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "CalculationPeriodDates {" +
			"calculationPeriodDatesAdjustments=" + this.calculationPeriodDatesAdjustments + ", " +
			"calculationPeriodFrequency=" + this.calculationPeriodFrequency + ", " +
			"effectiveDate=" + this.effectiveDate + ", " +
			"firstCompoundingPeriodEndDate=" + this.firstCompoundingPeriodEndDate + ", " +
			"firstPeriodStartDate=" + this.firstPeriodStartDate + ", " +
			"firstRegularPeriodStartDate=" + this.firstRegularPeriodStartDate + ", " +
			"lastRegularPeriodEndDate=" + this.lastRegularPeriodEndDate + ", " +
			"meta=" + this.meta + ", " +
			"stubPeriodType=" + this.stubPeriodType + ", " +
			"terminationDate=" + this.terminationDate +
		'}';
	}
}
