package org.isda.cdm;

import org.isda.cdm.RollConventionEnum;


import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.CalculationPeriodFrequencyMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class to specify the frequency at which calculation period end dates occur within the regular part of the calculation period schedule and their roll date convention.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="CalculationPeriodFrequency", source="FpML_5_10")
public class CalculationPeriodFrequency extends Frequency {
	private final RollConventionEnum rollConvention;
	private static CalculationPeriodFrequencyMeta metaData = new CalculationPeriodFrequencyMeta();
	
	CalculationPeriodFrequency(CalculationPeriodFrequencyBuilder builder) {
		super(builder);
		this.rollConvention = builder.getRollConvention();
	}
	
	/**
	 * The roll convention specifies the period term as part of a periodic schedule, i.e. the calculation period end date within the regular part of the calculation period. The value could be a rule, e.g. IMM Settlement Dates, which is the 3rd Wednesday of the month, or it could be a specific day of the month, such as the first day of the applicable month. It is used in conjunction with a frequency and the regular period start date of a calculation period.
	 */
	@RosettaSynonym(value="rollConvention", source="FpML_5_10")
	@RosettaSynonym(value="rollConvention", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="rollConvention", source="DTCC_11_0")
	@RosettaSynonym(value="rollConvention", source="DTCC_9_0")
	@RosettaSynonym(value="rollConvention", source="CME_ClearedConfirm_1_17")
	public final RollConventionEnum getRollConvention() {
		return rollConvention;
	}
	
	
	@Override
	public RosettaMetaData<? extends CalculationPeriodFrequency> metaData() {
		return metaData;
	} 
	
	public CalculationPeriodFrequencyBuilder toBuilder() {
		CalculationPeriodFrequencyBuilder builder = new CalculationPeriodFrequencyBuilder();
		ofNullable(getRollConvention()).ifPresent(builder::setRollConvention);
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		ofNullable(getPeriod()).ifPresent(builder::setPeriod);
		ofNullable(getPeriodMultiplier()).ifPresent(builder::setPeriodMultiplier);
		return builder;
	}

	public static CalculationPeriodFrequencyBuilder builder() {
		return new CalculationPeriodFrequencyBuilder();
	}

	public static  class CalculationPeriodFrequencyBuilder extends FrequencyBuilder {
	
		protected RollConventionEnum rollConvention;
	
		public CalculationPeriodFrequencyBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends CalculationPeriodFrequency> metaData() {
			return metaData;
		} 
	
		
		public RollConventionEnum getRollConvention() {
			return rollConvention;
		}
		
	
		public CalculationPeriodFrequencyBuilder setRollConvention(RollConventionEnum rollConvention) {
			this.rollConvention = rollConvention;
			return this;
		}
		@Override public CalculationPeriodFrequencyBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		@Override public CalculationPeriodFrequencyBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		@Override public CalculationPeriodFrequencyBuilder setPeriod(PeriodExtendedEnum period) {
			this.period = period;
			return this;
		}
		@Override public CalculationPeriodFrequencyBuilder setPeriodMultiplier(Integer periodMultiplier) {
			this.periodMultiplier = periodMultiplier;
			return this;
		}
		
		public CalculationPeriodFrequency build() {
			return new CalculationPeriodFrequency(this);
		}
	
		@Override
		public CalculationPeriodFrequencyBuilder prune() {
			super.prune();
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (super.hasData()) return true;
			if (getRollConvention()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
			super.process(path, processor);
		
			processor.processBasic(path.newSubPath("rollConvention"), RollConventionEnum.class, rollConvention, this);
			
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
			if (!super.equals(o)) return false;
		
			CalculationPeriodFrequencyBuilder _that = (CalculationPeriodFrequencyBuilder) o;
		
			if (rollConvention != null ? !rollConvention.equals(_that.rollConvention) : _that.rollConvention != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = super.hashCode();
			_result = 31 * _result + (rollConvention != null ? rollConvention.getClass().getName().hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "CalculationPeriodFrequencyBuilder {" +
				"rollConvention=" + this.rollConvention +
			'}' + " " + super.toString();
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		super.process(path, processor);
		
		processor.processBasic(path.newSubPath("rollConvention"), RollConventionEnum.class, rollConvention, this);
		
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
		if (!super.equals(o)) return false;
	
		CalculationPeriodFrequency _that = (CalculationPeriodFrequency) o;
	
		if (rollConvention != null ? !rollConvention.equals(_that.rollConvention) : _that.rollConvention != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = super.hashCode();
		_result = 31 * _result + (rollConvention != null ? rollConvention.getClass().getName().hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "CalculationPeriodFrequency {" +
			"rollConvention=" + this.rollConvention +
		'}' + " " + super.toString();
	}
}
