package org.isda.cdm;

import java.lang.Boolean;
import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.AmericanExercise;
import org.isda.cdm.BermudaExercise;
import org.isda.cdm.CancelableProvisionAdjustedDates;
import org.isda.cdm.EuropeanExercise;
import org.isda.cdm.ExerciseNotice;
import org.isda.cdm.FinalCalculationPeriodDateAdjustment;
import org.isda.cdm.SimplePayment;
import org.isda.cdm.metafields.ReferenceWithMetaAccount;
import org.isda.cdm.metafields.ReferenceWithMetaAccount.ReferenceWithMetaAccountBuilder;
import org.isda.cdm.metafields.ReferenceWithMetaParty;
import org.isda.cdm.metafields.ReferenceWithMetaParty.ReferenceWithMetaPartyBuilder;


import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.CancelableProvisionMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;
		import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;

/**
 * A class defining the right of a party to cancel a swap transaction on the specified exercise dates. The provision is for &#39;walk-away&#39; cancellation (i.e. the fair value of the swap is not paid). A fee payable on exercise can be specified. As a difference from the FpML construct, the canonical model extends the BuyerSeller class.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="CancelableProvision", source="FpML_5_10")
public class CancelableProvision extends BuyerSeller {
	private final AmericanExercise americanExercise;
	private final BermudaExercise bermudaExercise;
	private final CancelableProvisionAdjustedDates cancelableProvisionAdjustedDates;
	private final EuropeanExercise europeanExercise;
	private final ExerciseNotice exerciseNotice;
	private final List<FinalCalculationPeriodDateAdjustment> finalCalculationPeriodDateAdjustment;
	private final Boolean followUpConfirmation;
	private final SimplePayment initialFee;
	private static CancelableProvisionMeta metaData = new CancelableProvisionMeta();
	
	CancelableProvision(CancelableProvisionBuilder builder) {
		super(builder);
		this.americanExercise = ofNullable(builder.getAmericanExercise()).map(f->f.build()).orElse(null);
		this.bermudaExercise = ofNullable(builder.getBermudaExercise()).map(f->f.build()).orElse(null);
		this.cancelableProvisionAdjustedDates = ofNullable(builder.getCancelableProvisionAdjustedDates()).map(f->f.build()).orElse(null);
		this.europeanExercise = ofNullable(builder.getEuropeanExercise()).map(f->f.build()).orElse(null);
		this.exerciseNotice = ofNullable(builder.getExerciseNotice()).map(f->f.build()).orElse(null);
		this.finalCalculationPeriodDateAdjustment = ofNullable(builder.getFinalCalculationPeriodDateAdjustment()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.followUpConfirmation = builder.getFollowUpConfirmation();
		this.initialFee = ofNullable(builder.getInitialFee()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * American exercise. FpML implementations consists in an exercise substitution group.
	 */
	@RosettaSynonym(value="americanExercise", source="FpML_5_10")
	@RosettaSynonym(value="americanExercise", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="americanExercise", source="DTCC_11_0")
	@RosettaSynonym(value="americanExercise", source="DTCC_9_0")
	@RosettaSynonym(value="americanExercise", source="CME_ClearedConfirm_1_17")
	public final AmericanExercise getAmericanExercise() {
		return americanExercise;
	}
	
	/**
	 * Bermuda exercise. FpML implementations consists in an exercise substitution group.
	 */
	@RosettaSynonym(value="bermudaExercise", source="FpML_5_10")
	@RosettaSynonym(value="bermudaExercise", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="bermudaExercise", source="DTCC_11_0")
	@RosettaSynonym(value="bermudaExercise", source="DTCC_9_0")
	@RosettaSynonym(value="bermudaExercise", source="CME_ClearedConfirm_1_17")
	public final BermudaExercise getBermudaExercise() {
		return bermudaExercise;
	}
	
	/**
	 * The adjusted dates associated with a cancelable provision. These dates have been adjusted for any applicable business day convention.
	 */
	@RosettaSynonym(value="cancelableProvisionAdjustedDates", source="FpML_5_10")
	@RosettaSynonym(value="cancelableProvisionAdjustedDates", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="cancelableProvisionAdjustedDates", source="DTCC_11_0")
	@RosettaSynonym(value="cancelableProvisionAdjustedDates", source="DTCC_9_0")
	@RosettaSynonym(value="cancelableProvisionAdjustedDates", source="CME_ClearedConfirm_1_17")
	public final CancelableProvisionAdjustedDates getCancelableProvisionAdjustedDates() {
		return cancelableProvisionAdjustedDates;
	}
	
	/**
	 * European exercise. FpML implementations consists in an exercise substitution group.
	 */
	@RosettaSynonym(value="europeanExercise", source="FpML_5_10")
	@RosettaSynonym(value="europeanExercise", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="europeanExercise", source="DTCC_11_0")
	@RosettaSynonym(value="europeanExercise", source="DTCC_9_0")
	@RosettaSynonym(value="europeanExercise", source="CME_ClearedConfirm_1_17")
	public final EuropeanExercise getEuropeanExercise() {
		return europeanExercise;
	}
	
	/**
	 * Definition of the party to whom notice of exercise should be given.
	 */
	@RosettaSynonym(value="exerciseNotice", source="FpML_5_10")
	@RosettaSynonym(value="exerciseNotice", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="exerciseNotice", source="DTCC_11_0")
	@RosettaSynonym(value="exerciseNotice", source="DTCC_9_0")
	@RosettaSynonym(value="exerciseNotice", source="CME_ClearedConfirm_1_17")
	public final ExerciseNotice getExerciseNotice() {
		return exerciseNotice;
	}
	
	/**
	 * Business date convention adjustment to final payment period per leg (swapStream) upon exercise event. The adjustments can be made in-line with leg level BDC&#39;s or they can be specified separately.
	 */
	@RosettaSynonym(value="finalCalculationPeriodDateAdjustment", source="FpML_5_10")
	@RosettaSynonym(value="finalCalculationPeriodDateAdjustment", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="finalCalculationPeriodDateAdjustment", source="DTCC_11_0")
	@RosettaSynonym(value="finalCalculationPeriodDateAdjustment", source="DTCC_9_0")
	@RosettaSynonym(value="finalCalculationPeriodDateAdjustment", source="CME_ClearedConfirm_1_17")
	public final List<FinalCalculationPeriodDateAdjustment> getFinalCalculationPeriodDateAdjustment() {
		return finalCalculationPeriodDateAdjustment;
	}
	
	/**
	 * A flag to indicate whether follow-up confirmation of exercise (written or electronic) is required following telephonic notice by the buyer to the seller or seller&#39;s agent.
	 */
	@RosettaSynonym(value="followUpConfirmation", source="FpML_5_10")
	@RosettaSynonym(value="followUpConfirmation", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="followUpConfirmation", source="DTCC_11_0")
	@RosettaSynonym(value="followUpConfirmation", source="DTCC_9_0")
	@RosettaSynonym(value="followUpConfirmation", source="CME_ClearedConfirm_1_17")
	public final Boolean getFollowUpConfirmation() {
		return followUpConfirmation;
	}
	
	/**
	 * An initial fee for the cancelable option.
	 */
	@RosettaSynonym(value="initialFee", source="FpML_5_10")
	@RosettaSynonym(value="initialFee", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="initialFee", source="DTCC_11_0")
	@RosettaSynonym(value="initialFee", source="DTCC_9_0")
	@RosettaSynonym(value="initialFee", source="CME_ClearedConfirm_1_17")
	public final SimplePayment getInitialFee() {
		return initialFee;
	}
	
	
	@Override
	public RosettaMetaData<? extends CancelableProvision> metaData() {
		return metaData;
	} 
	
	public CancelableProvisionBuilder toBuilder() {
		CancelableProvisionBuilder builder = new CancelableProvisionBuilder();
		ofNullable(getAmericanExercise()).ifPresent(builder::setAmericanExercise);
		ofNullable(getBermudaExercise()).ifPresent(builder::setBermudaExercise);
		ofNullable(getCancelableProvisionAdjustedDates()).ifPresent(builder::setCancelableProvisionAdjustedDates);
		ofNullable(getEuropeanExercise()).ifPresent(builder::setEuropeanExercise);
		ofNullable(getExerciseNotice()).ifPresent(builder::setExerciseNotice);
		ofNullable(getFinalCalculationPeriodDateAdjustment()).ifPresent(finalCalculationPeriodDateAdjustment -> finalCalculationPeriodDateAdjustment.forEach(builder::addFinalCalculationPeriodDateAdjustment));
		ofNullable(getFollowUpConfirmation()).ifPresent(builder::setFollowUpConfirmation);
		ofNullable(getInitialFee()).ifPresent(builder::setInitialFee);
		ofNullable(getBuyerAccountReference()).ifPresent(builder::setBuyerAccountReference);
		ofNullable(getBuyerPartyReference()).ifPresent(builder::setBuyerPartyReference);
		ofNullable(getSellerAccountReference()).ifPresent(builder::setSellerAccountReference);
		ofNullable(getSellerPartyReference()).ifPresent(builder::setSellerPartyReference);
		return builder;
	}

	public static CancelableProvisionBuilder builder() {
		return new CancelableProvisionBuilder();
	}

	public static  class CancelableProvisionBuilder extends BuyerSellerBuilder {
	
		protected AmericanExercise.AmericanExerciseBuilder americanExercise;
		protected BermudaExercise.BermudaExerciseBuilder bermudaExercise;
		protected CancelableProvisionAdjustedDates.CancelableProvisionAdjustedDatesBuilder cancelableProvisionAdjustedDates;
		protected EuropeanExercise.EuropeanExerciseBuilder europeanExercise;
		protected ExerciseNotice.ExerciseNoticeBuilder exerciseNotice;
		protected List<FinalCalculationPeriodDateAdjustment.FinalCalculationPeriodDateAdjustmentBuilder> finalCalculationPeriodDateAdjustment;
		protected Boolean followUpConfirmation;
		protected SimplePayment.SimplePaymentBuilder initialFee;
	
		public CancelableProvisionBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends CancelableProvision> metaData() {
			return metaData;
		} 
	
		
		public AmericanExercise.AmericanExerciseBuilder getAmericanExercise() {
			return americanExercise;
		}
		
		public AmericanExercise.AmericanExerciseBuilder getOrCreateAmericanExercise() {
			if (americanExercise!=null) {
				return americanExercise;
			}
			else return americanExercise = new AmericanExercise.AmericanExerciseBuilder();
		}
		
		
		public BermudaExercise.BermudaExerciseBuilder getBermudaExercise() {
			return bermudaExercise;
		}
		
		public BermudaExercise.BermudaExerciseBuilder getOrCreateBermudaExercise() {
			if (bermudaExercise!=null) {
				return bermudaExercise;
			}
			else return bermudaExercise = new BermudaExercise.BermudaExerciseBuilder();
		}
		
		
		public CancelableProvisionAdjustedDates.CancelableProvisionAdjustedDatesBuilder getCancelableProvisionAdjustedDates() {
			return cancelableProvisionAdjustedDates;
		}
		
		public CancelableProvisionAdjustedDates.CancelableProvisionAdjustedDatesBuilder getOrCreateCancelableProvisionAdjustedDates() {
			if (cancelableProvisionAdjustedDates!=null) {
				return cancelableProvisionAdjustedDates;
			}
			else return cancelableProvisionAdjustedDates = new CancelableProvisionAdjustedDates.CancelableProvisionAdjustedDatesBuilder();
		}
		
		
		public EuropeanExercise.EuropeanExerciseBuilder getEuropeanExercise() {
			return europeanExercise;
		}
		
		public EuropeanExercise.EuropeanExerciseBuilder getOrCreateEuropeanExercise() {
			if (europeanExercise!=null) {
				return europeanExercise;
			}
			else return europeanExercise = new EuropeanExercise.EuropeanExerciseBuilder();
		}
		
		
		public ExerciseNotice.ExerciseNoticeBuilder getExerciseNotice() {
			return exerciseNotice;
		}
		
		public ExerciseNotice.ExerciseNoticeBuilder getOrCreateExerciseNotice() {
			if (exerciseNotice!=null) {
				return exerciseNotice;
			}
			else return exerciseNotice = new ExerciseNotice.ExerciseNoticeBuilder();
		}
		
		
		public List<FinalCalculationPeriodDateAdjustment.FinalCalculationPeriodDateAdjustmentBuilder> getFinalCalculationPeriodDateAdjustment() {
			return finalCalculationPeriodDateAdjustment;
		}
		
		public FinalCalculationPeriodDateAdjustment.FinalCalculationPeriodDateAdjustmentBuilder getOrCreateFinalCalculationPeriodDateAdjustment(int index) {
			if (finalCalculationPeriodDateAdjustment==null) {
				this.finalCalculationPeriodDateAdjustment = new ArrayList<>();
			}
			return getIndex(finalCalculationPeriodDateAdjustment, index, ()->new FinalCalculationPeriodDateAdjustment.FinalCalculationPeriodDateAdjustmentBuilder());
		}
		
		
		public Boolean getFollowUpConfirmation() {
			return followUpConfirmation;
		}
		
		
		public SimplePayment.SimplePaymentBuilder getInitialFee() {
			return initialFee;
		}
		
		public SimplePayment.SimplePaymentBuilder getOrCreateInitialFee() {
			if (initialFee!=null) {
				return initialFee;
			}
			else return initialFee = new SimplePayment.SimplePaymentBuilder();
		}
		
	
		public CancelableProvisionBuilder setAmericanExercise(AmericanExercise americanExercise) {
			this.americanExercise = americanExercise.toBuilder();
			return this;
		}
		public CancelableProvisionBuilder setAmericanExerciseBuilder(AmericanExercise.AmericanExerciseBuilder americanExercise) {
			this.americanExercise = americanExercise;
			return this;
		}
		public CancelableProvisionBuilder setBermudaExercise(BermudaExercise bermudaExercise) {
			this.bermudaExercise = bermudaExercise.toBuilder();
			return this;
		}
		public CancelableProvisionBuilder setBermudaExerciseBuilder(BermudaExercise.BermudaExerciseBuilder bermudaExercise) {
			this.bermudaExercise = bermudaExercise;
			return this;
		}
		public CancelableProvisionBuilder setCancelableProvisionAdjustedDates(CancelableProvisionAdjustedDates cancelableProvisionAdjustedDates) {
			this.cancelableProvisionAdjustedDates = cancelableProvisionAdjustedDates.toBuilder();
			return this;
		}
		public CancelableProvisionBuilder setCancelableProvisionAdjustedDatesBuilder(CancelableProvisionAdjustedDates.CancelableProvisionAdjustedDatesBuilder cancelableProvisionAdjustedDates) {
			this.cancelableProvisionAdjustedDates = cancelableProvisionAdjustedDates;
			return this;
		}
		public CancelableProvisionBuilder setEuropeanExercise(EuropeanExercise europeanExercise) {
			this.europeanExercise = europeanExercise.toBuilder();
			return this;
		}
		public CancelableProvisionBuilder setEuropeanExerciseBuilder(EuropeanExercise.EuropeanExerciseBuilder europeanExercise) {
			this.europeanExercise = europeanExercise;
			return this;
		}
		public CancelableProvisionBuilder setExerciseNotice(ExerciseNotice exerciseNotice) {
			this.exerciseNotice = exerciseNotice.toBuilder();
			return this;
		}
		public CancelableProvisionBuilder setExerciseNoticeBuilder(ExerciseNotice.ExerciseNoticeBuilder exerciseNotice) {
			this.exerciseNotice = exerciseNotice;
			return this;
		}
		public CancelableProvisionBuilder addFinalCalculationPeriodDateAdjustment(FinalCalculationPeriodDateAdjustment finalCalculationPeriodDateAdjustment) {
			if(this.finalCalculationPeriodDateAdjustment == null){
				this.finalCalculationPeriodDateAdjustment = new ArrayList<>();
			}
			this.finalCalculationPeriodDateAdjustment.add(finalCalculationPeriodDateAdjustment.toBuilder());
			return this;
		}
		public CancelableProvisionBuilder addFinalCalculationPeriodDateAdjustment(FinalCalculationPeriodDateAdjustment finalCalculationPeriodDateAdjustment, int _idx) {
			if(this.finalCalculationPeriodDateAdjustment == null){
				this.finalCalculationPeriodDateAdjustment = new ArrayList<>();
			}
			getIndex(this.finalCalculationPeriodDateAdjustment, _idx, () -> finalCalculationPeriodDateAdjustment.toBuilder());
			this.finalCalculationPeriodDateAdjustment.set(_idx, finalCalculationPeriodDateAdjustment.toBuilder());
			return this;
		}
		public CancelableProvisionBuilder addFinalCalculationPeriodDateAdjustment(List<FinalCalculationPeriodDateAdjustment> finalCalculationPeriodDateAdjustments) {
			if(this.finalCalculationPeriodDateAdjustment == null){
				this.finalCalculationPeriodDateAdjustment = new ArrayList<>();
			}
			for (FinalCalculationPeriodDateAdjustment toAdd : finalCalculationPeriodDateAdjustments) {
				this.finalCalculationPeriodDateAdjustment.add(toAdd.toBuilder());
			}
			return this;
		}
		public CancelableProvisionBuilder addFinalCalculationPeriodDateAdjustmentBuilder(FinalCalculationPeriodDateAdjustment.FinalCalculationPeriodDateAdjustmentBuilder finalCalculationPeriodDateAdjustment) {
			if(this.finalCalculationPeriodDateAdjustment == null){
				this.finalCalculationPeriodDateAdjustment = new ArrayList<>();
			}
			this.finalCalculationPeriodDateAdjustment.add(finalCalculationPeriodDateAdjustment);
			return this;
		}
		
		public CancelableProvisionBuilder clearFinalCalculationPeriodDateAdjustment() {
			this.finalCalculationPeriodDateAdjustment = null;
			return this;
		}
		public CancelableProvisionBuilder setFollowUpConfirmation(Boolean followUpConfirmation) {
			this.followUpConfirmation = followUpConfirmation;
			return this;
		}
		public CancelableProvisionBuilder setInitialFee(SimplePayment initialFee) {
			this.initialFee = initialFee.toBuilder();
			return this;
		}
		public CancelableProvisionBuilder setInitialFeeBuilder(SimplePayment.SimplePaymentBuilder initialFee) {
			this.initialFee = initialFee;
			return this;
		}
		@Override public CancelableProvisionBuilder setBuyerAccountReference(ReferenceWithMetaAccount buyerAccountReference) {
			this.buyerAccountReference = buyerAccountReference.toBuilder();
			return this;
		}
		@Override public CancelableProvisionBuilder setBuyerAccountReferenceBuilder(ReferenceWithMetaAccountBuilder buyerAccountReference) {
			this.buyerAccountReference = buyerAccountReference;
			return this;
		}
		@Override public CancelableProvisionBuilder setBuyerAccountReferenceRef(Account.AccountBuilder buyerAccountReference) {
			return setBuyerAccountReference(ReferenceWithMetaAccount.builder().setValueBuilder(buyerAccountReference).build());
		}
		@Override public CancelableProvisionBuilder setBuyerAccountReferenceRef(Account buyerAccountReference) {
			return setBuyerAccountReferenceRef(Account.builder());
		}
		@Override public CancelableProvisionBuilder setBuyerPartyReference(ReferenceWithMetaParty buyerPartyReference) {
			this.buyerPartyReference = buyerPartyReference.toBuilder();
			return this;
		}
		@Override public CancelableProvisionBuilder setBuyerPartyReferenceBuilder(ReferenceWithMetaPartyBuilder buyerPartyReference) {
			this.buyerPartyReference = buyerPartyReference;
			return this;
		}
		@Override public CancelableProvisionBuilder setBuyerPartyReferenceRef(Party.PartyBuilder buyerPartyReference) {
			return setBuyerPartyReference(ReferenceWithMetaParty.builder().setValueBuilder(buyerPartyReference).build());
		}
		@Override public CancelableProvisionBuilder setBuyerPartyReferenceRef(Party buyerPartyReference) {
			return setBuyerPartyReferenceRef(Party.builder());
		}
		@Override public CancelableProvisionBuilder setSellerAccountReference(ReferenceWithMetaAccount sellerAccountReference) {
			this.sellerAccountReference = sellerAccountReference.toBuilder();
			return this;
		}
		@Override public CancelableProvisionBuilder setSellerAccountReferenceBuilder(ReferenceWithMetaAccountBuilder sellerAccountReference) {
			this.sellerAccountReference = sellerAccountReference;
			return this;
		}
		@Override public CancelableProvisionBuilder setSellerAccountReferenceRef(Account.AccountBuilder sellerAccountReference) {
			return setSellerAccountReference(ReferenceWithMetaAccount.builder().setValueBuilder(sellerAccountReference).build());
		}
		@Override public CancelableProvisionBuilder setSellerAccountReferenceRef(Account sellerAccountReference) {
			return setSellerAccountReferenceRef(Account.builder());
		}
		@Override public CancelableProvisionBuilder setSellerPartyReference(ReferenceWithMetaParty sellerPartyReference) {
			this.sellerPartyReference = sellerPartyReference.toBuilder();
			return this;
		}
		@Override public CancelableProvisionBuilder setSellerPartyReferenceBuilder(ReferenceWithMetaPartyBuilder sellerPartyReference) {
			this.sellerPartyReference = sellerPartyReference;
			return this;
		}
		@Override public CancelableProvisionBuilder setSellerPartyReferenceRef(Party.PartyBuilder sellerPartyReference) {
			return setSellerPartyReference(ReferenceWithMetaParty.builder().setValueBuilder(sellerPartyReference).build());
		}
		@Override public CancelableProvisionBuilder setSellerPartyReferenceRef(Party sellerPartyReference) {
			return setSellerPartyReferenceRef(Party.builder());
		}
		
		public CancelableProvision build() {
			return new CancelableProvision(this);
		}
	
		@Override
		public CancelableProvisionBuilder prune() {
			super.prune();
			if (americanExercise!=null && !americanExercise.prune().hasData()) americanExercise = null;
			if (bermudaExercise!=null && !bermudaExercise.prune().hasData()) bermudaExercise = null;
			if (cancelableProvisionAdjustedDates!=null && !cancelableProvisionAdjustedDates.prune().hasData()) cancelableProvisionAdjustedDates = null;
			if (europeanExercise!=null && !europeanExercise.prune().hasData()) europeanExercise = null;
			if (exerciseNotice!=null && !exerciseNotice.prune().hasData()) exerciseNotice = null;
			if (finalCalculationPeriodDateAdjustment!=null) finalCalculationPeriodDateAdjustment = finalCalculationPeriodDateAdjustment.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (initialFee!=null && !initialFee.prune().hasData()) initialFee = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (super.hasData()) return true;
			if (getAmericanExercise()!=null && getAmericanExercise().hasData()) return true;
			if (getBermudaExercise()!=null && getBermudaExercise().hasData()) return true;
			if (getCancelableProvisionAdjustedDates()!=null && getCancelableProvisionAdjustedDates().hasData()) return true;
			if (getEuropeanExercise()!=null && getEuropeanExercise().hasData()) return true;
			if (getExerciseNotice()!=null && getExerciseNotice().hasData()) return true;
			if (getFinalCalculationPeriodDateAdjustment()!=null && getFinalCalculationPeriodDateAdjustment().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getFollowUpConfirmation()!=null) return true;
			if (getInitialFee()!=null && getInitialFee().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
			super.process(path, processor);
		
			processor.processBasic(path.newSubPath("followUpConfirmation"), Boolean.class, followUpConfirmation, this);
			
			processRosetta(path.newSubPath("americanExercise"), processor, AmericanExercise.class, americanExercise);
			processRosetta(path.newSubPath("bermudaExercise"), processor, BermudaExercise.class, bermudaExercise);
			processRosetta(path.newSubPath("cancelableProvisionAdjustedDates"), processor, CancelableProvisionAdjustedDates.class, cancelableProvisionAdjustedDates);
			processRosetta(path.newSubPath("europeanExercise"), processor, EuropeanExercise.class, europeanExercise);
			processRosetta(path.newSubPath("exerciseNotice"), processor, ExerciseNotice.class, exerciseNotice);
			processRosetta(path.newSubPath("finalCalculationPeriodDateAdjustment"), processor, FinalCalculationPeriodDateAdjustment.class, finalCalculationPeriodDateAdjustment);
			processRosetta(path.newSubPath("initialFee"), processor, SimplePayment.class, initialFee);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
			if (!super.equals(o)) return false;
		
			CancelableProvisionBuilder _that = (CancelableProvisionBuilder) o;
		
			if (americanExercise != null ? !americanExercise.equals(_that.americanExercise) : _that.americanExercise != null) return false;
			if (bermudaExercise != null ? !bermudaExercise.equals(_that.bermudaExercise) : _that.bermudaExercise != null) return false;
			if (cancelableProvisionAdjustedDates != null ? !cancelableProvisionAdjustedDates.equals(_that.cancelableProvisionAdjustedDates) : _that.cancelableProvisionAdjustedDates != null) return false;
			if (europeanExercise != null ? !europeanExercise.equals(_that.europeanExercise) : _that.europeanExercise != null) return false;
			if (exerciseNotice != null ? !exerciseNotice.equals(_that.exerciseNotice) : _that.exerciseNotice != null) return false;
			if (!ListEquals.listEquals(finalCalculationPeriodDateAdjustment, _that.finalCalculationPeriodDateAdjustment)) return false;
			if (followUpConfirmation != null ? !followUpConfirmation.equals(_that.followUpConfirmation) : _that.followUpConfirmation != null) return false;
			if (initialFee != null ? !initialFee.equals(_that.initialFee) : _that.initialFee != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = super.hashCode();
			_result = 31 * _result + (americanExercise != null ? americanExercise.hashCode() : 0);
			_result = 31 * _result + (bermudaExercise != null ? bermudaExercise.hashCode() : 0);
			_result = 31 * _result + (cancelableProvisionAdjustedDates != null ? cancelableProvisionAdjustedDates.hashCode() : 0);
			_result = 31 * _result + (europeanExercise != null ? europeanExercise.hashCode() : 0);
			_result = 31 * _result + (exerciseNotice != null ? exerciseNotice.hashCode() : 0);
			_result = 31 * _result + (finalCalculationPeriodDateAdjustment != null ? finalCalculationPeriodDateAdjustment.hashCode() : 0);
			_result = 31 * _result + (followUpConfirmation != null ? followUpConfirmation.hashCode() : 0);
			_result = 31 * _result + (initialFee != null ? initialFee.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "CancelableProvisionBuilder {" +
				"americanExercise=" + this.americanExercise + ", " +
				"bermudaExercise=" + this.bermudaExercise + ", " +
				"cancelableProvisionAdjustedDates=" + this.cancelableProvisionAdjustedDates + ", " +
				"europeanExercise=" + this.europeanExercise + ", " +
				"exerciseNotice=" + this.exerciseNotice + ", " +
				"finalCalculationPeriodDateAdjustment=" + this.finalCalculationPeriodDateAdjustment + ", " +
				"followUpConfirmation=" + this.followUpConfirmation + ", " +
				"initialFee=" + this.initialFee +
			'}' + " " + super.toString();
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		super.process(path, processor);
		
		processor.processBasic(path.newSubPath("followUpConfirmation"), Boolean.class, followUpConfirmation, this);
		
		processRosetta(path.newSubPath("americanExercise"), processor, AmericanExercise.class, americanExercise);
		processRosetta(path.newSubPath("bermudaExercise"), processor, BermudaExercise.class, bermudaExercise);
		processRosetta(path.newSubPath("cancelableProvisionAdjustedDates"), processor, CancelableProvisionAdjustedDates.class, cancelableProvisionAdjustedDates);
		processRosetta(path.newSubPath("europeanExercise"), processor, EuropeanExercise.class, europeanExercise);
		processRosetta(path.newSubPath("exerciseNotice"), processor, ExerciseNotice.class, exerciseNotice);
		processRosetta(path.newSubPath("finalCalculationPeriodDateAdjustment"), processor, FinalCalculationPeriodDateAdjustment.class, finalCalculationPeriodDateAdjustment);
		processRosetta(path.newSubPath("initialFee"), processor, SimplePayment.class, initialFee);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
		if (!super.equals(o)) return false;
	
		CancelableProvision _that = (CancelableProvision) o;
	
		if (americanExercise != null ? !americanExercise.equals(_that.americanExercise) : _that.americanExercise != null) return false;
		if (bermudaExercise != null ? !bermudaExercise.equals(_that.bermudaExercise) : _that.bermudaExercise != null) return false;
		if (cancelableProvisionAdjustedDates != null ? !cancelableProvisionAdjustedDates.equals(_that.cancelableProvisionAdjustedDates) : _that.cancelableProvisionAdjustedDates != null) return false;
		if (europeanExercise != null ? !europeanExercise.equals(_that.europeanExercise) : _that.europeanExercise != null) return false;
		if (exerciseNotice != null ? !exerciseNotice.equals(_that.exerciseNotice) : _that.exerciseNotice != null) return false;
		if (!ListEquals.listEquals(finalCalculationPeriodDateAdjustment, _that.finalCalculationPeriodDateAdjustment)) return false;
		if (followUpConfirmation != null ? !followUpConfirmation.equals(_that.followUpConfirmation) : _that.followUpConfirmation != null) return false;
		if (initialFee != null ? !initialFee.equals(_that.initialFee) : _that.initialFee != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = super.hashCode();
		_result = 31 * _result + (americanExercise != null ? americanExercise.hashCode() : 0);
		_result = 31 * _result + (bermudaExercise != null ? bermudaExercise.hashCode() : 0);
		_result = 31 * _result + (cancelableProvisionAdjustedDates != null ? cancelableProvisionAdjustedDates.hashCode() : 0);
		_result = 31 * _result + (europeanExercise != null ? europeanExercise.hashCode() : 0);
		_result = 31 * _result + (exerciseNotice != null ? exerciseNotice.hashCode() : 0);
		_result = 31 * _result + (finalCalculationPeriodDateAdjustment != null ? finalCalculationPeriodDateAdjustment.hashCode() : 0);
		_result = 31 * _result + (followUpConfirmation != null ? followUpConfirmation.hashCode() : 0);
		_result = 31 * _result + (initialFee != null ? initialFee.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "CancelableProvision {" +
			"americanExercise=" + this.americanExercise + ", " +
			"bermudaExercise=" + this.bermudaExercise + ", " +
			"cancelableProvisionAdjustedDates=" + this.cancelableProvisionAdjustedDates + ", " +
			"europeanExercise=" + this.europeanExercise + ", " +
			"exerciseNotice=" + this.exerciseNotice + ", " +
			"finalCalculationPeriodDateAdjustment=" + this.finalCalculationPeriodDateAdjustment + ", " +
			"followUpConfirmation=" + this.followUpConfirmation + ", " +
			"initialFee=" + this.initialFee +
		'}' + " " + super.toString();
	}
}
