package org.isda.cdm;

import com.rosetta.model.lib.records.Date;
import org.isda.cdm.metafields.MetaFields;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.CancellationEventMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import com.rosetta.model.lib.records.Date;
		import static java.util.Optional.ofNullable;

/**
 * The adjusted dates for a specific cancellation date, including the adjusted exercise date and adjusted termination date.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class CancellationEvent extends RosettaModelObject implements GlobalKey {
	private final Date adjustedEarlyTerminationDate;
	private final Date adjustedExerciseDate;
	private final MetaFields meta;
	private static CancellationEventMeta metaData = new CancellationEventMeta();
	
	CancellationEvent(CancellationEventBuilder builder) {
		this.adjustedEarlyTerminationDate = builder.getAdjustedEarlyTerminationDate();
		this.adjustedExerciseDate = builder.getAdjustedExerciseDate();
		this.meta = ofNullable(builder.getMeta()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The early termination date that is applicable if an early termination provision is exercised. This date should already be adjusted for any applicable business day convention.
	 */
	@RosettaSynonym(value="adjustedEarlyTerminationDate", source="FpML_5_10")
	@RosettaSynonym(value="adjustedEarlyTerminationDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="adjustedEarlyTerminationDate", source="DTCC_11_0")
	@RosettaSynonym(value="adjustedEarlyTerminationDate", source="DTCC_9_0")
	@RosettaSynonym(value="adjustedEarlyTerminationDate", source="CME_ClearedConfirm_1_17")
	public final Date getAdjustedEarlyTerminationDate() {
		return adjustedEarlyTerminationDate;
	}
	
	/**
	 * The date on which option exercise takes place. This date should already be adjusted for any applicable business day convention.
	 */
	@RosettaSynonym(value="adjustedExerciseDate", source="FpML_5_10")
	@RosettaSynonym(value="adjustedExerciseDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="adjustedExerciseDate", source="DTCC_11_0")
	@RosettaSynonym(value="adjustedExerciseDate", source="DTCC_9_0")
	@RosettaSynonym(value="adjustedExerciseDate", source="CME_ClearedConfirm_1_17")
	public final Date getAdjustedExerciseDate() {
		return adjustedExerciseDate;
	}
	
	public final MetaFields getMeta() {
		return meta;
	}
	
	
	@Override
	public RosettaMetaData<? extends CancellationEvent> metaData() {
		return metaData;
	} 
	
	public CancellationEventBuilder toBuilder() {
		CancellationEventBuilder builder = new CancellationEventBuilder();
		ofNullable(getAdjustedEarlyTerminationDate()).ifPresent(builder::setAdjustedEarlyTerminationDate);
		ofNullable(getAdjustedExerciseDate()).ifPresent(builder::setAdjustedExerciseDate);
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		return builder;
	}

	public static CancellationEventBuilder builder() {
		return new CancellationEventBuilder();
	}

	public static  class CancellationEventBuilder extends RosettaModelObjectBuilder implements GlobalKeyBuilder<CancellationEventBuilder> {
	
		protected Date adjustedEarlyTerminationDate;
		protected Date adjustedExerciseDate;
		protected MetaFields.MetaFieldsBuilder meta;
	
		public CancellationEventBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends CancellationEvent> metaData() {
			return metaData;
		} 
	
		
		public Date getAdjustedEarlyTerminationDate() {
			return adjustedEarlyTerminationDate;
		}
		
		
		public Date getAdjustedExerciseDate() {
			return adjustedExerciseDate;
		}
		
		
		public MetaFields.MetaFieldsBuilder getMeta() {
			return meta;
		}
		
		public MetaFields.MetaFieldsBuilder getOrCreateMeta() {
			if (meta!=null) {
				return meta;
			}
			else return meta = new MetaFields.MetaFieldsBuilder();
		}
		
	
		public CancellationEventBuilder setAdjustedEarlyTerminationDate(Date adjustedEarlyTerminationDate) {
			this.adjustedEarlyTerminationDate = adjustedEarlyTerminationDate;
			return this;
		}
		public CancellationEventBuilder setAdjustedExerciseDate(Date adjustedExerciseDate) {
			this.adjustedExerciseDate = adjustedExerciseDate;
			return this;
		}
		public CancellationEventBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		public CancellationEventBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		
		public CancellationEvent build() {
			return new CancellationEvent(this);
		}
	
		@Override
		public CancellationEventBuilder prune() {
			if (meta!=null && !meta.prune().hasData()) meta = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getAdjustedEarlyTerminationDate()!=null) return true;
			if (getAdjustedExerciseDate()!=null) return true;
			if (getMeta()!=null && getMeta().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("adjustedEarlyTerminationDate"), Date.class, adjustedEarlyTerminationDate, this);
			processor.processBasic(path.newSubPath("adjustedExerciseDate"), Date.class, adjustedExerciseDate, this);
			
			processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			CancellationEventBuilder _that = (CancellationEventBuilder) o;
		
			if (adjustedEarlyTerminationDate != null ? !adjustedEarlyTerminationDate.equals(_that.adjustedEarlyTerminationDate) : _that.adjustedEarlyTerminationDate != null) return false;
			if (adjustedExerciseDate != null ? !adjustedExerciseDate.equals(_that.adjustedExerciseDate) : _that.adjustedExerciseDate != null) return false;
			if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (adjustedEarlyTerminationDate != null ? adjustedEarlyTerminationDate.hashCode() : 0);
			_result = 31 * _result + (adjustedExerciseDate != null ? adjustedExerciseDate.hashCode() : 0);
			_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "CancellationEventBuilder {" +
				"adjustedEarlyTerminationDate=" + this.adjustedEarlyTerminationDate + ", " +
				"adjustedExerciseDate=" + this.adjustedExerciseDate + ", " +
				"meta=" + this.meta +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("adjustedEarlyTerminationDate"), Date.class, adjustedEarlyTerminationDate, this);
		processor.processBasic(path.newSubPath("adjustedExerciseDate"), Date.class, adjustedExerciseDate, this);
		
		processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		CancellationEvent _that = (CancellationEvent) o;
	
		if (adjustedEarlyTerminationDate != null ? !adjustedEarlyTerminationDate.equals(_that.adjustedEarlyTerminationDate) : _that.adjustedEarlyTerminationDate != null) return false;
		if (adjustedExerciseDate != null ? !adjustedExerciseDate.equals(_that.adjustedExerciseDate) : _that.adjustedExerciseDate != null) return false;
		if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (adjustedEarlyTerminationDate != null ? adjustedEarlyTerminationDate.hashCode() : 0);
		_result = 31 * _result + (adjustedExerciseDate != null ? adjustedExerciseDate.hashCode() : 0);
		_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "CancellationEvent {" +
			"adjustedEarlyTerminationDate=" + this.adjustedEarlyTerminationDate + ", " +
			"adjustedExerciseDate=" + this.adjustedExerciseDate + ", " +
			"meta=" + this.meta +
		'}';
	}
}
