package org.isda.cdm;

import org.isda.cdm.CashSettlementReferenceBanks;
import org.isda.cdm.QuotationRateTypeEnum;
import org.isda.cdm.metafields.FieldWithMetaString;
import org.isda.cdm.metafields.FieldWithMetaString.FieldWithMetaStringBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.CashPriceMethodMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A type defining the parameters necessary for each of the ISDA cash price methods for cash settlement.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="businessDateRange", source="FpML_5_10")
public class CashPriceMethod extends RosettaModelObject {
	private final FieldWithMetaString cashSettlementCurrency;
	private final CashSettlementReferenceBanks cashSettlementReferenceBanks;
	private final QuotationRateTypeEnum quotationRateType;
	private static CashPriceMethodMeta metaData = new CashPriceMethodMeta();
	
	CashPriceMethod(CashPriceMethodBuilder builder) {
		this.cashSettlementCurrency = ofNullable(builder.getCashSettlementCurrency()).map(f->f.build()).orElse(null);
		this.cashSettlementReferenceBanks = ofNullable(builder.getCashSettlementReferenceBanks()).map(f->f.build()).orElse(null);
		this.quotationRateType = builder.getQuotationRateType();
	}
	
	/**
	 * The currency in which the cash settlement amount will be calculated and settled. The list of valid currencies is not presently positioned as an enumeration as part of the CDM because that scope is limited to the values specified by ISDA and FpML. As a result, implementers have to make reference to the relevant standard, such as the ISO 4217 standard for currency codes.
	 */
	@RosettaSynonym(value="cashSettlementCurrency", source="FpML_5_10")
	@RosettaSynonym(value="cashSettlementCurrency", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="cashSettlementCurrency", source="DTCC_11_0")
	@RosettaSynonym(value="cashSettlementCurrency", source="DTCC_9_0")
	@RosettaSynonym(value="cashSettlementCurrency", source="CME_ClearedConfirm_1_17")
	public final FieldWithMetaString getCashSettlementCurrency() {
		return cashSettlementCurrency;
	}
	
	/**
	 * A container for a set of reference institutions. These reference institutions may be called upon to provide rate quotations as part of the method to determine the applicable cash settlement amount. If institutions are not specified, it is assumed that reference institutions will be agreed between the parties on the exercise date, or in the case of swap transaction to which mandatory early termination is applicable, the cash settlement valuation date.
	 */
	@RosettaSynonym(value="cashSettlementReferenceBanks", source="FpML_5_10")
	@RosettaSynonym(value="cashSettlementReferenceBanks", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="cashSettlementReferenceBanks", source="DTCC_11_0")
	@RosettaSynonym(value="cashSettlementReferenceBanks", source="DTCC_9_0")
	@RosettaSynonym(value="cashSettlementReferenceBanks", source="CME_ClearedConfirm_1_17")
	public final CashSettlementReferenceBanks getCashSettlementReferenceBanks() {
		return cashSettlementReferenceBanks;
	}
	
	/**
	 * Which rate quote is to be observed, either Bid, Mid, Offer or Exercising Party Pays. The meaning of Exercising Party Pays is defined in the 2000 ISDA Definitions, Section 17.2. Certain Definitions Relating to Cash Settlement, paragraph (j).
	 */
	@RosettaSynonym(value="quotationRateType", source="FpML_5_10")
	@RosettaSynonym(value="quotationRateType", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="quotationRateType", source="DTCC_11_0")
	@RosettaSynonym(value="quotationRateType", source="DTCC_9_0")
	@RosettaSynonym(value="quotationRateType", source="CME_ClearedConfirm_1_17")
	public final QuotationRateTypeEnum getQuotationRateType() {
		return quotationRateType;
	}
	
	
	@Override
	public RosettaMetaData<? extends CashPriceMethod> metaData() {
		return metaData;
	} 
	
	public CashPriceMethodBuilder toBuilder() {
		CashPriceMethodBuilder builder = new CashPriceMethodBuilder();
		ofNullable(getCashSettlementCurrency()).ifPresent(builder::setCashSettlementCurrency);
		ofNullable(getCashSettlementReferenceBanks()).ifPresent(builder::setCashSettlementReferenceBanks);
		ofNullable(getQuotationRateType()).ifPresent(builder::setQuotationRateType);
		return builder;
	}

	public static CashPriceMethodBuilder builder() {
		return new CashPriceMethodBuilder();
	}

	public static  class CashPriceMethodBuilder extends RosettaModelObjectBuilder {
	
		protected FieldWithMetaStringBuilder cashSettlementCurrency;
		protected CashSettlementReferenceBanks.CashSettlementReferenceBanksBuilder cashSettlementReferenceBanks;
		protected QuotationRateTypeEnum quotationRateType;
	
		public CashPriceMethodBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends CashPriceMethod> metaData() {
			return metaData;
		} 
	
		
		public FieldWithMetaStringBuilder getCashSettlementCurrency() {
			return cashSettlementCurrency;
		}
		
		public FieldWithMetaStringBuilder getOrCreateCashSettlementCurrency() {
			if (cashSettlementCurrency!=null) {
				return cashSettlementCurrency;
			}
			else return cashSettlementCurrency = new FieldWithMetaStringBuilder();
		}
		
		
		public CashSettlementReferenceBanks.CashSettlementReferenceBanksBuilder getCashSettlementReferenceBanks() {
			return cashSettlementReferenceBanks;
		}
		
		public CashSettlementReferenceBanks.CashSettlementReferenceBanksBuilder getOrCreateCashSettlementReferenceBanks() {
			if (cashSettlementReferenceBanks!=null) {
				return cashSettlementReferenceBanks;
			}
			else return cashSettlementReferenceBanks = new CashSettlementReferenceBanks.CashSettlementReferenceBanksBuilder();
		}
		
		
		public QuotationRateTypeEnum getQuotationRateType() {
			return quotationRateType;
		}
		
	
		public CashPriceMethodBuilder setCashSettlementCurrency(FieldWithMetaString cashSettlementCurrency) {
			this.cashSettlementCurrency = cashSettlementCurrency.toBuilder();
			return this;
		}
		public CashPriceMethodBuilder setCashSettlementCurrencyRef(String cashSettlementCurrency) {
			return setCashSettlementCurrency(FieldWithMetaString.builder().setValue(cashSettlementCurrency).build());
		}
		public CashPriceMethodBuilder setCashSettlementReferenceBanks(CashSettlementReferenceBanks cashSettlementReferenceBanks) {
			this.cashSettlementReferenceBanks = cashSettlementReferenceBanks.toBuilder();
			return this;
		}
		public CashPriceMethodBuilder setCashSettlementReferenceBanksBuilder(CashSettlementReferenceBanks.CashSettlementReferenceBanksBuilder cashSettlementReferenceBanks) {
			this.cashSettlementReferenceBanks = cashSettlementReferenceBanks;
			return this;
		}
		public CashPriceMethodBuilder setQuotationRateType(QuotationRateTypeEnum quotationRateType) {
			this.quotationRateType = quotationRateType;
			return this;
		}
		
		public CashPriceMethod build() {
			return new CashPriceMethod(this);
		}
	
		@Override
		public CashPriceMethodBuilder prune() {
			if (cashSettlementCurrency!=null && !cashSettlementCurrency.prune().hasData()) cashSettlementCurrency = null;
			if (cashSettlementReferenceBanks!=null && !cashSettlementReferenceBanks.prune().hasData()) cashSettlementReferenceBanks = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getCashSettlementCurrency()!=null) return true;
			if (getCashSettlementReferenceBanks()!=null && getCashSettlementReferenceBanks().hasData()) return true;
			if (getQuotationRateType()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("quotationRateType"), QuotationRateTypeEnum.class, quotationRateType, this);
			
			processRosetta(path.newSubPath("cashSettlementCurrency"), processor, FieldWithMetaString.class, cashSettlementCurrency);
			processRosetta(path.newSubPath("cashSettlementReferenceBanks"), processor, CashSettlementReferenceBanks.class, cashSettlementReferenceBanks);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			CashPriceMethodBuilder _that = (CashPriceMethodBuilder) o;
		
			if (cashSettlementCurrency != null ? !cashSettlementCurrency.equals(_that.cashSettlementCurrency) : _that.cashSettlementCurrency != null) return false;
			if (cashSettlementReferenceBanks != null ? !cashSettlementReferenceBanks.equals(_that.cashSettlementReferenceBanks) : _that.cashSettlementReferenceBanks != null) return false;
			if (quotationRateType != null ? !quotationRateType.equals(_that.quotationRateType) : _that.quotationRateType != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (cashSettlementCurrency != null ? cashSettlementCurrency.hashCode() : 0);
			_result = 31 * _result + (cashSettlementReferenceBanks != null ? cashSettlementReferenceBanks.hashCode() : 0);
			_result = 31 * _result + (quotationRateType != null ? quotationRateType.getClass().getName().hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "CashPriceMethodBuilder {" +
				"cashSettlementCurrency=" + this.cashSettlementCurrency + ", " +
				"cashSettlementReferenceBanks=" + this.cashSettlementReferenceBanks + ", " +
				"quotationRateType=" + this.quotationRateType +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("quotationRateType"), QuotationRateTypeEnum.class, quotationRateType, this);
		
		processRosetta(path.newSubPath("cashSettlementCurrency"), processor, FieldWithMetaString.class, cashSettlementCurrency);
		processRosetta(path.newSubPath("cashSettlementReferenceBanks"), processor, CashSettlementReferenceBanks.class, cashSettlementReferenceBanks);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		CashPriceMethod _that = (CashPriceMethod) o;
	
		if (cashSettlementCurrency != null ? !cashSettlementCurrency.equals(_that.cashSettlementCurrency) : _that.cashSettlementCurrency != null) return false;
		if (cashSettlementReferenceBanks != null ? !cashSettlementReferenceBanks.equals(_that.cashSettlementReferenceBanks) : _that.cashSettlementReferenceBanks != null) return false;
		if (quotationRateType != null ? !quotationRateType.equals(_that.quotationRateType) : _that.quotationRateType != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (cashSettlementCurrency != null ? cashSettlementCurrency.hashCode() : 0);
		_result = 31 * _result + (cashSettlementReferenceBanks != null ? cashSettlementReferenceBanks.hashCode() : 0);
		_result = 31 * _result + (quotationRateType != null ? quotationRateType.getClass().getName().hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "CashPriceMethod {" +
			"cashSettlementCurrency=" + this.cashSettlementCurrency + ", " +
			"cashSettlementReferenceBanks=" + this.cashSettlementReferenceBanks + ", " +
			"quotationRateType=" + this.quotationRateType +
		'}';
	}
}
