package org.isda.cdm;

import org.isda.cdm.AdjustableDates;
import org.isda.cdm.BusinessDateRange;
import org.isda.cdm.RelativeDateOffset;
import org.isda.cdm.metafields.MetaFields;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.CashSettlementPaymentDateMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class defining the cash settlement payment date(s) as either a set of explicit dates, together with applicable adjustments, or as a date relative to some other (anchor) date, or as any date in a range of contiguous business days.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class CashSettlementPaymentDate extends RosettaModelObject implements GlobalKey {
	private final AdjustableDates adjustableDates;
	private final BusinessDateRange businessDateRange;
	private final MetaFields meta;
	private final RelativeDateOffset relativeDate;
	private static CashSettlementPaymentDateMeta metaData = new CashSettlementPaymentDateMeta();
	
	CashSettlementPaymentDate(CashSettlementPaymentDateBuilder builder) {
		this.adjustableDates = ofNullable(builder.getAdjustableDates()).map(f->f.build()).orElse(null);
		this.businessDateRange = ofNullable(builder.getBusinessDateRange()).map(f->f.build()).orElse(null);
		this.meta = ofNullable(builder.getMeta()).map(f->f.build()).orElse(null);
		this.relativeDate = ofNullable(builder.getRelativeDate()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * A series of dates that shall be subject to adjustment if they would otherwise fall on a day that is not a business day in the specified business centers, together with the convention for adjusting the date.
	 */
	@RosettaSynonym(value="adjustableDates", source="FpML_5_10")
	@RosettaSynonym(value="adjustableDates", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="adjustableDates", source="DTCC_11_0")
	@RosettaSynonym(value="adjustableDates", source="DTCC_9_0")
	@RosettaSynonym(value="adjustableDates", source="CME_ClearedConfirm_1_17")
	public final AdjustableDates getAdjustableDates() {
		return adjustableDates;
	}
	
	/**
	 * A range of contiguous business days.
	 */
	@RosettaSynonym(value="businessDateRange", source="FpML_5_10")
	@RosettaSynonym(value="businessDateRange", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="businessDateRange", source="DTCC_11_0")
	@RosettaSynonym(value="businessDateRange", source="DTCC_9_0")
	@RosettaSynonym(value="businessDateRange", source="CME_ClearedConfirm_1_17")
	public final BusinessDateRange getBusinessDateRange() {
		return businessDateRange;
	}
	
	public final MetaFields getMeta() {
		return meta;
	}
	
	/**
	 * A date specified as some offset to another date (the anchor date).
	 */
	@RosettaSynonym(value="relativeDate", source="FpML_5_10")
	@RosettaSynonym(value="relativeDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="relativeDate", source="DTCC_11_0")
	@RosettaSynonym(value="relativeDate", source="DTCC_9_0")
	@RosettaSynonym(value="relativeDate", source="CME_ClearedConfirm_1_17")
	public final RelativeDateOffset getRelativeDate() {
		return relativeDate;
	}
	
	
	@Override
	public RosettaMetaData<? extends CashSettlementPaymentDate> metaData() {
		return metaData;
	} 
	
	public CashSettlementPaymentDateBuilder toBuilder() {
		CashSettlementPaymentDateBuilder builder = new CashSettlementPaymentDateBuilder();
		ofNullable(getAdjustableDates()).ifPresent(builder::setAdjustableDates);
		ofNullable(getBusinessDateRange()).ifPresent(builder::setBusinessDateRange);
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		ofNullable(getRelativeDate()).ifPresent(builder::setRelativeDate);
		return builder;
	}

	public static CashSettlementPaymentDateBuilder builder() {
		return new CashSettlementPaymentDateBuilder();
	}

	public static  class CashSettlementPaymentDateBuilder extends RosettaModelObjectBuilder implements GlobalKeyBuilder<CashSettlementPaymentDateBuilder> {
	
		protected AdjustableDates.AdjustableDatesBuilder adjustableDates;
		protected BusinessDateRange.BusinessDateRangeBuilder businessDateRange;
		protected MetaFields.MetaFieldsBuilder meta;
		protected RelativeDateOffset.RelativeDateOffsetBuilder relativeDate;
	
		public CashSettlementPaymentDateBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends CashSettlementPaymentDate> metaData() {
			return metaData;
		} 
	
		
		public AdjustableDates.AdjustableDatesBuilder getAdjustableDates() {
			return adjustableDates;
		}
		
		public AdjustableDates.AdjustableDatesBuilder getOrCreateAdjustableDates() {
			if (adjustableDates!=null) {
				return adjustableDates;
			}
			else return adjustableDates = new AdjustableDates.AdjustableDatesBuilder();
		}
		
		
		public BusinessDateRange.BusinessDateRangeBuilder getBusinessDateRange() {
			return businessDateRange;
		}
		
		public BusinessDateRange.BusinessDateRangeBuilder getOrCreateBusinessDateRange() {
			if (businessDateRange!=null) {
				return businessDateRange;
			}
			else return businessDateRange = new BusinessDateRange.BusinessDateRangeBuilder();
		}
		
		
		public MetaFields.MetaFieldsBuilder getMeta() {
			return meta;
		}
		
		public MetaFields.MetaFieldsBuilder getOrCreateMeta() {
			if (meta!=null) {
				return meta;
			}
			else return meta = new MetaFields.MetaFieldsBuilder();
		}
		
		
		public RelativeDateOffset.RelativeDateOffsetBuilder getRelativeDate() {
			return relativeDate;
		}
		
		public RelativeDateOffset.RelativeDateOffsetBuilder getOrCreateRelativeDate() {
			if (relativeDate!=null) {
				return relativeDate;
			}
			else return relativeDate = new RelativeDateOffset.RelativeDateOffsetBuilder();
		}
		
	
		public CashSettlementPaymentDateBuilder setAdjustableDates(AdjustableDates adjustableDates) {
			this.adjustableDates = adjustableDates.toBuilder();
			return this;
		}
		public CashSettlementPaymentDateBuilder setAdjustableDatesBuilder(AdjustableDates.AdjustableDatesBuilder adjustableDates) {
			this.adjustableDates = adjustableDates;
			return this;
		}
		public CashSettlementPaymentDateBuilder setBusinessDateRange(BusinessDateRange businessDateRange) {
			this.businessDateRange = businessDateRange.toBuilder();
			return this;
		}
		public CashSettlementPaymentDateBuilder setBusinessDateRangeBuilder(BusinessDateRange.BusinessDateRangeBuilder businessDateRange) {
			this.businessDateRange = businessDateRange;
			return this;
		}
		public CashSettlementPaymentDateBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		public CashSettlementPaymentDateBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		public CashSettlementPaymentDateBuilder setRelativeDate(RelativeDateOffset relativeDate) {
			this.relativeDate = relativeDate.toBuilder();
			return this;
		}
		public CashSettlementPaymentDateBuilder setRelativeDateBuilder(RelativeDateOffset.RelativeDateOffsetBuilder relativeDate) {
			this.relativeDate = relativeDate;
			return this;
		}
		
		public CashSettlementPaymentDate build() {
			return new CashSettlementPaymentDate(this);
		}
	
		@Override
		public CashSettlementPaymentDateBuilder prune() {
			if (adjustableDates!=null && !adjustableDates.prune().hasData()) adjustableDates = null;
			if (businessDateRange!=null && !businessDateRange.prune().hasData()) businessDateRange = null;
			if (meta!=null && !meta.prune().hasData()) meta = null;
			if (relativeDate!=null && !relativeDate.prune().hasData()) relativeDate = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getAdjustableDates()!=null && getAdjustableDates().hasData()) return true;
			if (getBusinessDateRange()!=null && getBusinessDateRange().hasData()) return true;
			if (getMeta()!=null && getMeta().hasData()) return true;
			if (getRelativeDate()!=null && getRelativeDate().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("adjustableDates"), processor, AdjustableDates.class, adjustableDates);
			processRosetta(path.newSubPath("businessDateRange"), processor, BusinessDateRange.class, businessDateRange);
			processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
			processRosetta(path.newSubPath("relativeDate"), processor, RelativeDateOffset.class, relativeDate);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			CashSettlementPaymentDateBuilder _that = (CashSettlementPaymentDateBuilder) o;
		
			if (adjustableDates != null ? !adjustableDates.equals(_that.adjustableDates) : _that.adjustableDates != null) return false;
			if (businessDateRange != null ? !businessDateRange.equals(_that.businessDateRange) : _that.businessDateRange != null) return false;
			if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
			if (relativeDate != null ? !relativeDate.equals(_that.relativeDate) : _that.relativeDate != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (adjustableDates != null ? adjustableDates.hashCode() : 0);
			_result = 31 * _result + (businessDateRange != null ? businessDateRange.hashCode() : 0);
			_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
			_result = 31 * _result + (relativeDate != null ? relativeDate.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "CashSettlementPaymentDateBuilder {" +
				"adjustableDates=" + this.adjustableDates + ", " +
				"businessDateRange=" + this.businessDateRange + ", " +
				"meta=" + this.meta + ", " +
				"relativeDate=" + this.relativeDate +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("adjustableDates"), processor, AdjustableDates.class, adjustableDates);
		processRosetta(path.newSubPath("businessDateRange"), processor, BusinessDateRange.class, businessDateRange);
		processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
		processRosetta(path.newSubPath("relativeDate"), processor, RelativeDateOffset.class, relativeDate);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		CashSettlementPaymentDate _that = (CashSettlementPaymentDate) o;
	
		if (adjustableDates != null ? !adjustableDates.equals(_that.adjustableDates) : _that.adjustableDates != null) return false;
		if (businessDateRange != null ? !businessDateRange.equals(_that.businessDateRange) : _that.businessDateRange != null) return false;
		if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
		if (relativeDate != null ? !relativeDate.equals(_that.relativeDate) : _that.relativeDate != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (adjustableDates != null ? adjustableDates.hashCode() : 0);
		_result = 31 * _result + (businessDateRange != null ? businessDateRange.hashCode() : 0);
		_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
		_result = 31 * _result + (relativeDate != null ? relativeDate.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "CashSettlementPaymentDate {" +
			"adjustableDates=" + this.adjustableDates + ", " +
			"businessDateRange=" + this.businessDateRange + ", " +
			"meta=" + this.meta + ", " +
			"relativeDate=" + this.relativeDate +
		'}';
	}
}
