package org.isda.cdm;

import java.lang.Boolean;
import java.lang.Integer;
import java.lang.String;
import java.math.BigDecimal;
import org.isda.cdm.BusinessCenterTime;
import org.isda.cdm.Money;
import org.isda.cdm.QuotationRateTypeEnum;
import org.isda.cdm.ValuationDate;
import org.isda.cdm.ValuationMethodEnum;
import org.isda.cdm.metafields.FieldWithMetaString;


import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.CashSettlementTermsMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;
		import java.math.BigDecimal;

/**
 * In FpML, PhysicalSettlementTerms and CashSettlementTerms extend SettlementTerms. In the CDM, this extension paradigm has not been used because SettlementTerms class has been used for purposes related to securities transactions, while it is not used as such in the FpML standard (i.e. only as an abstract construct.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="CashSettlementTerms", source="FpML_5_10")
public class CashSettlementTerms extends SettlementBase {
	private final Boolean accruedInterest;
	private final Money cashSettlementAmount;
	private final Integer cashSettlementBusinessDays;
	private final String dealer;
	private final Boolean fixedSettlement;
	private final Money minimumQuotationAmout;
	private final Money quotationAmount;
	private final QuotationRateTypeEnum quotationMethod;
	private final BigDecimal recoveryFactor;
	private final ValuationDate valuationDate;
	private final ValuationMethodEnum valuationMethod;
	private final BusinessCenterTime valuationTime;
	private static CashSettlementTermsMeta metaData = new CashSettlementTermsMeta();
	
	CashSettlementTerms(CashSettlementTermsBuilder builder) {
		super(builder);
		this.accruedInterest = builder.getAccruedInterest();
		this.cashSettlementAmount = ofNullable(builder.getCashSettlementAmount()).map(f->f.build()).orElse(null);
		this.cashSettlementBusinessDays = builder.getCashSettlementBusinessDays();
		this.dealer = builder.getDealer();
		this.fixedSettlement = builder.getFixedSettlement();
		this.minimumQuotationAmout = ofNullable(builder.getMinimumQuotationAmout()).map(f->f.build()).orElse(null);
		this.quotationAmount = ofNullable(builder.getQuotationAmount()).map(f->f.build()).orElse(null);
		this.quotationMethod = builder.getQuotationMethod();
		this.recoveryFactor = builder.getRecoveryFactor();
		this.valuationDate = ofNullable(builder.getValuationDate()).map(f->f.build()).orElse(null);
		this.valuationMethod = builder.getValuationMethod();
		this.valuationTime = ofNullable(builder.getValuationTime()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * Indicates whether accrued interest is included (true) or not (false). For cash settlement this specifies whether quotations should be obtained inclusive or not of accrued interest. For physical settlement this specifies whether the buyer should deliver the obligation with an outstanding principal balance that includes or excludes accrued interest. ISDA 2003 Term: Include/Exclude Accrued Interest.
	 */
	@RosettaSynonym(value="accruedInterest", source="FpML_5_10")
	@RosettaSynonym(value="accruedInterest", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="accruedInterest", source="DTCC_11_0")
	@RosettaSynonym(value="accruedInterest", source="DTCC_9_0")
	@RosettaSynonym(value="accruedInterest", source="CME_ClearedConfirm_1_17")
	public final Boolean getAccruedInterest() {
		return accruedInterest;
	}
	
	/**
	 * The amount paid by the seller to the buyer for cash settlement on the cash settlement date. If not otherwise specified, would typically be calculated as 100 (or the Reference Price) minus the price of the Reference Obligation (all expressed as a percentage) times Floating Rate Payer Calculation Amount. ISDA 2003 Term: Cash Settlement Amount.
	 */
	@RosettaSynonym(value="cashSettlementAmount", source="FpML_5_10")
	@RosettaSynonym(value="cashSettlementAmount", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="cashSettlementAmount", source="DTCC_11_0")
	@RosettaSynonym(value="cashSettlementAmount", source="DTCC_9_0")
	@RosettaSynonym(value="cashSettlementAmount", source="CME_ClearedConfirm_1_17")
	public final Money getCashSettlementAmount() {
		return cashSettlementAmount;
	}
	
	/**
	 * The number of business days used in the determination of the cash settlement payment date. If a cash settlement amount is specified, the cash settlement payment date will be this number of business days following the calculation of the final price. If a cash settlement amount is not specified, the cash settlement payment date will be this number of business days after all conditions to settlement are satisfied. ISDA 2003 Term: Cash Settlement Date.
	 */
	@RosettaSynonym(value="cashSettlementBusinessDays", source="FpML_5_10")
	@RosettaSynonym(value="cashSettlementBusinessDays", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="cashSettlementBusinessDays", source="DTCC_11_0")
	@RosettaSynonym(value="cashSettlementBusinessDays", source="DTCC_9_0")
	@RosettaSynonym(value="cashSettlementBusinessDays", source="CME_ClearedConfirm_1_17")
	public final Integer getCashSettlementBusinessDays() {
		return cashSettlementBusinessDays;
	}
	
	/**
	 * A dealer from whom quotations are obtained by the calculation agent on the reference obligation for purposes of cash settlement. ISDA 2003 Term: Dealer.
	 */
	@RosettaSynonym(value="dealer", source="FpML_5_10")
	@RosettaSynonym(value="dealer", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="dealer", source="DTCC_11_0")
	@RosettaSynonym(value="dealer", source="DTCC_9_0")
	@RosettaSynonym(value="dealer", source="CME_ClearedConfirm_1_17")
	public final String getDealer() {
		return dealer;
	}
	
	/**
	 * Used for Recovery Lock, to indicate whether fixed Settlement is Applicable or Not Applicable. If Buyer fails to deliver an effective Notice of Physical Settlement on or before the Buyer NOPS Cut-off Date, and if Seller fails to deliver an effective Seller NOPS on or before the Seller NOPS Cut-off Date, then either: (a) if Fixed Settlement is specified in the related Confirmation as not applicable, then the Seller NOPS Cut-off Date shall be the Termination Date; or (b) if Fixed Settlement is specified in the related Confirmation as applicable, then: (i) if the Fixed Settlement Amount is a positive number, Seller shall, subject to Section 3.1 (except for the requirement of satisfaction of the Notice of Physical Settlement Condition to Settlement), pay the Fixed Settlement Amount to Buyer on the Fixed Settlement Payment Date; and (ii) if the Fixed Settlement Amount is a negative number, Buyer shall, subject to Section 3.1 (except for the requirement of satisfaction of the Notice of Physical Settlement Condition to Settlement), pay the absolute value of the Fixed Settlement Amount to Seller on the Fixed Settlement Payment Date.
	 */
	@RosettaSynonym(value="fixedSettlement", source="FpML_5_10")
	@RosettaSynonym(value="fixedSettlement", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="fixedSettlement", source="DTCC_11_0")
	@RosettaSynonym(value="fixedSettlement", source="DTCC_9_0")
	@RosettaSynonym(value="fixedSettlement", source="CME_ClearedConfirm_1_17")
	public final Boolean getFixedSettlement() {
		return fixedSettlement;
	}
	
	/**
	 * In the determination of a cash settlement amount, if weighted average quotations are to be obtained, the minimum quotation amount specifies a minimum intended threshold amount of outstanding principal balance of the reference obligation for which the quote should be obtained. If not specified, the ISDA definitions provide for a fallback amount of the lower of either USD 1,000,000 (or its equivalent in the relevant obligation currency) or the quotation amount. ISDA 2003 Term: Minimum Quotation Amount.
	 */
	@RosettaSynonym(value="minimumQuotationAmount", source="FpML_5_10")
	@RosettaSynonym(value="minimumQuotationAmount", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="minimumQuotationAmount", source="DTCC_11_0")
	@RosettaSynonym(value="minimumQuotationAmount", source="DTCC_9_0")
	@RosettaSynonym(value="minimumQuotationAmount", source="CME_ClearedConfirm_1_17")
	public final Money getMinimumQuotationAmout() {
		return minimumQuotationAmout;
	}
	
	/**
	 * In the determination of a cash settlement amount, if weighted average quotations are to be obtained, the quotation amount specifies an upper limit to the outstanding principal balance of the reference obligation for which the quote should be obtained. If not specified, the ISDA definitions provide for a fallback amount equal to the floating rate payer calculation amount. ISDA 2003 Term: Quotation Amount.
	 */
	@RosettaSynonym(value="quotationAmount", source="FpML_5_10")
	@RosettaSynonym(value="quotationAmount", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="quotationAmount", source="DTCC_11_0")
	@RosettaSynonym(value="quotationAmount", source="DTCC_9_0")
	@RosettaSynonym(value="quotationAmount", source="CME_ClearedConfirm_1_17")
	public final Money getQuotationAmount() {
		return quotationAmount;
	}
	
	/**
	 * The type of price quotations to be requested from dealers when determining the market value of the reference obligation for purposes of cash settlement. For example, Bid, Offer or Mid-market. ISDA 2003 Term: Quotation Method.
	 */
	@RosettaSynonym(value="quotationMethod", source="FpML_5_10")
	@RosettaSynonym(value="quotationMethod", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="quotationMethod", source="DTCC_11_0")
	@RosettaSynonym(value="quotationMethod", source="DTCC_9_0")
	@RosettaSynonym(value="quotationMethod", source="CME_ClearedConfirm_1_17")
	public final QuotationRateTypeEnum getQuotationMethod() {
		return quotationMethod;
	}
	
	/**
	 * Used for fixed recovery, specifies the recovery level, determined at contract inception, to be applied on a default. Used to calculate the amount paid by the seller to the buyer for cash settlement on the cash settlement date. Amount calculation is (1 minus the Recovery Factor) multiplied by the Floating Rate Payer Calculation Amount. The currency will be derived from the Floating Rate Payer Calculation Amount.
	 */
	@RosettaSynonym(value="recoveryFactor", source="FpML_5_10")
	@RosettaSynonym(value="recoveryFactor", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="recoveryFactor", source="DTCC_11_0")
	@RosettaSynonym(value="recoveryFactor", source="DTCC_9_0")
	@RosettaSynonym(value="recoveryFactor", source="CME_ClearedConfirm_1_17")
	public final BigDecimal getRecoveryFactor() {
		return recoveryFactor;
	}
	
	/**
	 * The number of business days after conditions to settlement have been satisfied when the calculation agent obtains a price quotation on the Reference Obligation for purposes of cash settlement. There may be one or more valuation dates. This is typically specified if the cash settlement amount is not a fixed amount. ISDA 2003 Term: Valuation Date.
	 */
	@RosettaSynonym(value="valuationDate", source="FpML_5_10")
	@RosettaSynonym(value="valuationDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="valuationDate", source="DTCC_11_0")
	@RosettaSynonym(value="valuationDate", source="DTCC_9_0")
	@RosettaSynonym(value="valuationDate", source="CME_ClearedConfirm_1_17")
	public final ValuationDate getValuationDate() {
		return valuationDate;
	}
	
	/**
	 * The ISDA defined methodology for determining the final price of the reference obligation for purposes of cash settlement. (ISDA 2003 Term: Valuation Method). For example, Market, Highest etc.
	 */
	@RosettaSynonym(value="valuationMethod", source="FpML_5_10")
	@RosettaSynonym(value="valuationMethod", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="valuationMethod", source="DTCC_11_0")
	@RosettaSynonym(value="valuationMethod", source="DTCC_9_0")
	@RosettaSynonym(value="valuationMethod", source="CME_ClearedConfirm_1_17")
	public final ValuationMethodEnum getValuationMethod() {
		return valuationMethod;
	}
	
	/**
	 * The time of day in the specified business center when the calculation agent seeks quotations for an amount of the reference obligation for purposes of cash settlement. ISDA 2003 Term: Valuation Time.
	 */
	@RosettaSynonym(value="valuationTime", source="FpML_5_10")
	@RosettaSynonym(value="valuationTime", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="valuationTime", source="DTCC_11_0")
	@RosettaSynonym(value="valuationTime", source="DTCC_9_0")
	@RosettaSynonym(value="valuationTime", source="CME_ClearedConfirm_1_17")
	public final BusinessCenterTime getValuationTime() {
		return valuationTime;
	}
	
	
	@Override
	public RosettaMetaData<? extends CashSettlementTerms> metaData() {
		return metaData;
	} 
	
	public CashSettlementTermsBuilder toBuilder() {
		CashSettlementTermsBuilder builder = new CashSettlementTermsBuilder();
		ofNullable(getAccruedInterest()).ifPresent(builder::setAccruedInterest);
		ofNullable(getCashSettlementAmount()).ifPresent(builder::setCashSettlementAmount);
		ofNullable(getCashSettlementBusinessDays()).ifPresent(builder::setCashSettlementBusinessDays);
		ofNullable(getDealer()).ifPresent(builder::setDealer);
		ofNullable(getFixedSettlement()).ifPresent(builder::setFixedSettlement);
		ofNullable(getMinimumQuotationAmout()).ifPresent(builder::setMinimumQuotationAmout);
		ofNullable(getQuotationAmount()).ifPresent(builder::setQuotationAmount);
		ofNullable(getQuotationMethod()).ifPresent(builder::setQuotationMethod);
		ofNullable(getRecoveryFactor()).ifPresent(builder::setRecoveryFactor);
		ofNullable(getValuationDate()).ifPresent(builder::setValuationDate);
		ofNullable(getValuationMethod()).ifPresent(builder::setValuationMethod);
		ofNullable(getValuationTime()).ifPresent(builder::setValuationTime);
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		ofNullable(getSettlementCurrency()).ifPresent(builder::setSettlementCurrency);
		return builder;
	}

	public static CashSettlementTermsBuilder builder() {
		return new CashSettlementTermsBuilder();
	}

	public static  class CashSettlementTermsBuilder extends SettlementBaseBuilder {
	
		protected Boolean accruedInterest;
		protected Money.MoneyBuilder cashSettlementAmount;
		protected Integer cashSettlementBusinessDays;
		protected String dealer;
		protected Boolean fixedSettlement;
		protected Money.MoneyBuilder minimumQuotationAmout;
		protected Money.MoneyBuilder quotationAmount;
		protected QuotationRateTypeEnum quotationMethod;
		protected BigDecimal recoveryFactor;
		protected ValuationDate.ValuationDateBuilder valuationDate;
		protected ValuationMethodEnum valuationMethod;
		protected BusinessCenterTime.BusinessCenterTimeBuilder valuationTime;
	
		public CashSettlementTermsBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends CashSettlementTerms> metaData() {
			return metaData;
		} 
	
		
		public Boolean getAccruedInterest() {
			return accruedInterest;
		}
		
		
		public Money.MoneyBuilder getCashSettlementAmount() {
			return cashSettlementAmount;
		}
		
		public Money.MoneyBuilder getOrCreateCashSettlementAmount() {
			if (cashSettlementAmount!=null) {
				return cashSettlementAmount;
			}
			else return cashSettlementAmount = new Money.MoneyBuilder();
		}
		
		
		public Integer getCashSettlementBusinessDays() {
			return cashSettlementBusinessDays;
		}
		
		
		public String getDealer() {
			return dealer;
		}
		
		
		public Boolean getFixedSettlement() {
			return fixedSettlement;
		}
		
		
		public Money.MoneyBuilder getMinimumQuotationAmout() {
			return minimumQuotationAmout;
		}
		
		public Money.MoneyBuilder getOrCreateMinimumQuotationAmout() {
			if (minimumQuotationAmout!=null) {
				return minimumQuotationAmout;
			}
			else return minimumQuotationAmout = new Money.MoneyBuilder();
		}
		
		
		public Money.MoneyBuilder getQuotationAmount() {
			return quotationAmount;
		}
		
		public Money.MoneyBuilder getOrCreateQuotationAmount() {
			if (quotationAmount!=null) {
				return quotationAmount;
			}
			else return quotationAmount = new Money.MoneyBuilder();
		}
		
		
		public QuotationRateTypeEnum getQuotationMethod() {
			return quotationMethod;
		}
		
		
		public BigDecimal getRecoveryFactor() {
			return recoveryFactor;
		}
		
		
		public ValuationDate.ValuationDateBuilder getValuationDate() {
			return valuationDate;
		}
		
		public ValuationDate.ValuationDateBuilder getOrCreateValuationDate() {
			if (valuationDate!=null) {
				return valuationDate;
			}
			else return valuationDate = new ValuationDate.ValuationDateBuilder();
		}
		
		
		public ValuationMethodEnum getValuationMethod() {
			return valuationMethod;
		}
		
		
		public BusinessCenterTime.BusinessCenterTimeBuilder getValuationTime() {
			return valuationTime;
		}
		
		public BusinessCenterTime.BusinessCenterTimeBuilder getOrCreateValuationTime() {
			if (valuationTime!=null) {
				return valuationTime;
			}
			else return valuationTime = new BusinessCenterTime.BusinessCenterTimeBuilder();
		}
		
	
		public CashSettlementTermsBuilder setAccruedInterest(Boolean accruedInterest) {
			this.accruedInterest = accruedInterest;
			return this;
		}
		public CashSettlementTermsBuilder setCashSettlementAmount(Money cashSettlementAmount) {
			this.cashSettlementAmount = cashSettlementAmount.toBuilder();
			return this;
		}
		public CashSettlementTermsBuilder setCashSettlementAmountBuilder(Money.MoneyBuilder cashSettlementAmount) {
			this.cashSettlementAmount = cashSettlementAmount;
			return this;
		}
		public CashSettlementTermsBuilder setCashSettlementBusinessDays(Integer cashSettlementBusinessDays) {
			this.cashSettlementBusinessDays = cashSettlementBusinessDays;
			return this;
		}
		public CashSettlementTermsBuilder setDealer(String dealer) {
			this.dealer = dealer;
			return this;
		}
		public CashSettlementTermsBuilder setFixedSettlement(Boolean fixedSettlement) {
			this.fixedSettlement = fixedSettlement;
			return this;
		}
		public CashSettlementTermsBuilder setMinimumQuotationAmout(Money minimumQuotationAmout) {
			this.minimumQuotationAmout = minimumQuotationAmout.toBuilder();
			return this;
		}
		public CashSettlementTermsBuilder setMinimumQuotationAmoutBuilder(Money.MoneyBuilder minimumQuotationAmout) {
			this.minimumQuotationAmout = minimumQuotationAmout;
			return this;
		}
		public CashSettlementTermsBuilder setQuotationAmount(Money quotationAmount) {
			this.quotationAmount = quotationAmount.toBuilder();
			return this;
		}
		public CashSettlementTermsBuilder setQuotationAmountBuilder(Money.MoneyBuilder quotationAmount) {
			this.quotationAmount = quotationAmount;
			return this;
		}
		public CashSettlementTermsBuilder setQuotationMethod(QuotationRateTypeEnum quotationMethod) {
			this.quotationMethod = quotationMethod;
			return this;
		}
		public CashSettlementTermsBuilder setRecoveryFactor(BigDecimal recoveryFactor) {
			this.recoveryFactor = recoveryFactor;
			return this;
		}
		public CashSettlementTermsBuilder setValuationDate(ValuationDate valuationDate) {
			this.valuationDate = valuationDate.toBuilder();
			return this;
		}
		public CashSettlementTermsBuilder setValuationDateBuilder(ValuationDate.ValuationDateBuilder valuationDate) {
			this.valuationDate = valuationDate;
			return this;
		}
		public CashSettlementTermsBuilder setValuationMethod(ValuationMethodEnum valuationMethod) {
			this.valuationMethod = valuationMethod;
			return this;
		}
		public CashSettlementTermsBuilder setValuationTime(BusinessCenterTime valuationTime) {
			this.valuationTime = valuationTime.toBuilder();
			return this;
		}
		public CashSettlementTermsBuilder setValuationTimeBuilder(BusinessCenterTime.BusinessCenterTimeBuilder valuationTime) {
			this.valuationTime = valuationTime;
			return this;
		}
		@Override public CashSettlementTermsBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		@Override public CashSettlementTermsBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		@Override public CashSettlementTermsBuilder setSettlementCurrency(FieldWithMetaString settlementCurrency) {
			this.settlementCurrency = settlementCurrency.toBuilder();
			return this;
		}
		@Override public CashSettlementTermsBuilder setSettlementCurrencyRef(String settlementCurrency) {
			return setSettlementCurrency(FieldWithMetaString.builder().setValue(settlementCurrency).build());
		}
		
		public CashSettlementTerms build() {
			return new CashSettlementTerms(this);
		}
	
		@Override
		public CashSettlementTermsBuilder prune() {
			super.prune();
			if (cashSettlementAmount!=null && !cashSettlementAmount.prune().hasData()) cashSettlementAmount = null;
			if (minimumQuotationAmout!=null && !minimumQuotationAmout.prune().hasData()) minimumQuotationAmout = null;
			if (quotationAmount!=null && !quotationAmount.prune().hasData()) quotationAmount = null;
			if (valuationDate!=null && !valuationDate.prune().hasData()) valuationDate = null;
			if (valuationTime!=null && !valuationTime.prune().hasData()) valuationTime = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (super.hasData()) return true;
			if (getAccruedInterest()!=null) return true;
			if (getCashSettlementAmount()!=null && getCashSettlementAmount().hasData()) return true;
			if (getCashSettlementBusinessDays()!=null) return true;
			if (getDealer()!=null) return true;
			if (getFixedSettlement()!=null) return true;
			if (getMinimumQuotationAmout()!=null && getMinimumQuotationAmout().hasData()) return true;
			if (getQuotationAmount()!=null && getQuotationAmount().hasData()) return true;
			if (getQuotationMethod()!=null) return true;
			if (getRecoveryFactor()!=null) return true;
			if (getValuationDate()!=null && getValuationDate().hasData()) return true;
			if (getValuationMethod()!=null) return true;
			if (getValuationTime()!=null && getValuationTime().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
			super.process(path, processor);
		
			processor.processBasic(path.newSubPath("accruedInterest"), Boolean.class, accruedInterest, this);
			processor.processBasic(path.newSubPath("cashSettlementBusinessDays"), Integer.class, cashSettlementBusinessDays, this);
			processor.processBasic(path.newSubPath("dealer"), String.class, dealer, this);
			processor.processBasic(path.newSubPath("fixedSettlement"), Boolean.class, fixedSettlement, this);
			processor.processBasic(path.newSubPath("quotationMethod"), QuotationRateTypeEnum.class, quotationMethod, this);
			processor.processBasic(path.newSubPath("recoveryFactor"), BigDecimal.class, recoveryFactor, this);
			processor.processBasic(path.newSubPath("valuationMethod"), ValuationMethodEnum.class, valuationMethod, this);
			
			processRosetta(path.newSubPath("cashSettlementAmount"), processor, Money.class, cashSettlementAmount);
			processRosetta(path.newSubPath("minimumQuotationAmout"), processor, Money.class, minimumQuotationAmout);
			processRosetta(path.newSubPath("quotationAmount"), processor, Money.class, quotationAmount);
			processRosetta(path.newSubPath("valuationDate"), processor, ValuationDate.class, valuationDate);
			processRosetta(path.newSubPath("valuationTime"), processor, BusinessCenterTime.class, valuationTime);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
			if (!super.equals(o)) return false;
		
			CashSettlementTermsBuilder _that = (CashSettlementTermsBuilder) o;
		
			if (accruedInterest != null ? !accruedInterest.equals(_that.accruedInterest) : _that.accruedInterest != null) return false;
			if (cashSettlementAmount != null ? !cashSettlementAmount.equals(_that.cashSettlementAmount) : _that.cashSettlementAmount != null) return false;
			if (cashSettlementBusinessDays != null ? !cashSettlementBusinessDays.equals(_that.cashSettlementBusinessDays) : _that.cashSettlementBusinessDays != null) return false;
			if (dealer != null ? !dealer.equals(_that.dealer) : _that.dealer != null) return false;
			if (fixedSettlement != null ? !fixedSettlement.equals(_that.fixedSettlement) : _that.fixedSettlement != null) return false;
			if (minimumQuotationAmout != null ? !minimumQuotationAmout.equals(_that.minimumQuotationAmout) : _that.minimumQuotationAmout != null) return false;
			if (quotationAmount != null ? !quotationAmount.equals(_that.quotationAmount) : _that.quotationAmount != null) return false;
			if (quotationMethod != null ? !quotationMethod.equals(_that.quotationMethod) : _that.quotationMethod != null) return false;
			if (recoveryFactor != null ? !recoveryFactor.equals(_that.recoveryFactor) : _that.recoveryFactor != null) return false;
			if (valuationDate != null ? !valuationDate.equals(_that.valuationDate) : _that.valuationDate != null) return false;
			if (valuationMethod != null ? !valuationMethod.equals(_that.valuationMethod) : _that.valuationMethod != null) return false;
			if (valuationTime != null ? !valuationTime.equals(_that.valuationTime) : _that.valuationTime != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = super.hashCode();
			_result = 31 * _result + (accruedInterest != null ? accruedInterest.hashCode() : 0);
			_result = 31 * _result + (cashSettlementAmount != null ? cashSettlementAmount.hashCode() : 0);
			_result = 31 * _result + (cashSettlementBusinessDays != null ? cashSettlementBusinessDays.hashCode() : 0);
			_result = 31 * _result + (dealer != null ? dealer.hashCode() : 0);
			_result = 31 * _result + (fixedSettlement != null ? fixedSettlement.hashCode() : 0);
			_result = 31 * _result + (minimumQuotationAmout != null ? minimumQuotationAmout.hashCode() : 0);
			_result = 31 * _result + (quotationAmount != null ? quotationAmount.hashCode() : 0);
			_result = 31 * _result + (quotationMethod != null ? quotationMethod.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (recoveryFactor != null ? recoveryFactor.hashCode() : 0);
			_result = 31 * _result + (valuationDate != null ? valuationDate.hashCode() : 0);
			_result = 31 * _result + (valuationMethod != null ? valuationMethod.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (valuationTime != null ? valuationTime.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "CashSettlementTermsBuilder {" +
				"accruedInterest=" + this.accruedInterest + ", " +
				"cashSettlementAmount=" + this.cashSettlementAmount + ", " +
				"cashSettlementBusinessDays=" + this.cashSettlementBusinessDays + ", " +
				"dealer=" + this.dealer + ", " +
				"fixedSettlement=" + this.fixedSettlement + ", " +
				"minimumQuotationAmout=" + this.minimumQuotationAmout + ", " +
				"quotationAmount=" + this.quotationAmount + ", " +
				"quotationMethod=" + this.quotationMethod + ", " +
				"recoveryFactor=" + this.recoveryFactor + ", " +
				"valuationDate=" + this.valuationDate + ", " +
				"valuationMethod=" + this.valuationMethod + ", " +
				"valuationTime=" + this.valuationTime +
			'}' + " " + super.toString();
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		super.process(path, processor);
		
		processor.processBasic(path.newSubPath("accruedInterest"), Boolean.class, accruedInterest, this);
		processor.processBasic(path.newSubPath("cashSettlementBusinessDays"), Integer.class, cashSettlementBusinessDays, this);
		processor.processBasic(path.newSubPath("dealer"), String.class, dealer, this);
		processor.processBasic(path.newSubPath("fixedSettlement"), Boolean.class, fixedSettlement, this);
		processor.processBasic(path.newSubPath("quotationMethod"), QuotationRateTypeEnum.class, quotationMethod, this);
		processor.processBasic(path.newSubPath("recoveryFactor"), BigDecimal.class, recoveryFactor, this);
		processor.processBasic(path.newSubPath("valuationMethod"), ValuationMethodEnum.class, valuationMethod, this);
		
		processRosetta(path.newSubPath("cashSettlementAmount"), processor, Money.class, cashSettlementAmount);
		processRosetta(path.newSubPath("minimumQuotationAmout"), processor, Money.class, minimumQuotationAmout);
		processRosetta(path.newSubPath("quotationAmount"), processor, Money.class, quotationAmount);
		processRosetta(path.newSubPath("valuationDate"), processor, ValuationDate.class, valuationDate);
		processRosetta(path.newSubPath("valuationTime"), processor, BusinessCenterTime.class, valuationTime);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
		if (!super.equals(o)) return false;
	
		CashSettlementTerms _that = (CashSettlementTerms) o;
	
		if (accruedInterest != null ? !accruedInterest.equals(_that.accruedInterest) : _that.accruedInterest != null) return false;
		if (cashSettlementAmount != null ? !cashSettlementAmount.equals(_that.cashSettlementAmount) : _that.cashSettlementAmount != null) return false;
		if (cashSettlementBusinessDays != null ? !cashSettlementBusinessDays.equals(_that.cashSettlementBusinessDays) : _that.cashSettlementBusinessDays != null) return false;
		if (dealer != null ? !dealer.equals(_that.dealer) : _that.dealer != null) return false;
		if (fixedSettlement != null ? !fixedSettlement.equals(_that.fixedSettlement) : _that.fixedSettlement != null) return false;
		if (minimumQuotationAmout != null ? !minimumQuotationAmout.equals(_that.minimumQuotationAmout) : _that.minimumQuotationAmout != null) return false;
		if (quotationAmount != null ? !quotationAmount.equals(_that.quotationAmount) : _that.quotationAmount != null) return false;
		if (quotationMethod != null ? !quotationMethod.equals(_that.quotationMethod) : _that.quotationMethod != null) return false;
		if (recoveryFactor != null ? !recoveryFactor.equals(_that.recoveryFactor) : _that.recoveryFactor != null) return false;
		if (valuationDate != null ? !valuationDate.equals(_that.valuationDate) : _that.valuationDate != null) return false;
		if (valuationMethod != null ? !valuationMethod.equals(_that.valuationMethod) : _that.valuationMethod != null) return false;
		if (valuationTime != null ? !valuationTime.equals(_that.valuationTime) : _that.valuationTime != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = super.hashCode();
		_result = 31 * _result + (accruedInterest != null ? accruedInterest.hashCode() : 0);
		_result = 31 * _result + (cashSettlementAmount != null ? cashSettlementAmount.hashCode() : 0);
		_result = 31 * _result + (cashSettlementBusinessDays != null ? cashSettlementBusinessDays.hashCode() : 0);
		_result = 31 * _result + (dealer != null ? dealer.hashCode() : 0);
		_result = 31 * _result + (fixedSettlement != null ? fixedSettlement.hashCode() : 0);
		_result = 31 * _result + (minimumQuotationAmout != null ? minimumQuotationAmout.hashCode() : 0);
		_result = 31 * _result + (quotationAmount != null ? quotationAmount.hashCode() : 0);
		_result = 31 * _result + (quotationMethod != null ? quotationMethod.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (recoveryFactor != null ? recoveryFactor.hashCode() : 0);
		_result = 31 * _result + (valuationDate != null ? valuationDate.hashCode() : 0);
		_result = 31 * _result + (valuationMethod != null ? valuationMethod.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (valuationTime != null ? valuationTime.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "CashSettlementTerms {" +
			"accruedInterest=" + this.accruedInterest + ", " +
			"cashSettlementAmount=" + this.cashSettlementAmount + ", " +
			"cashSettlementBusinessDays=" + this.cashSettlementBusinessDays + ", " +
			"dealer=" + this.dealer + ", " +
			"fixedSettlement=" + this.fixedSettlement + ", " +
			"minimumQuotationAmout=" + this.minimumQuotationAmout + ", " +
			"quotationAmount=" + this.quotationAmount + ", " +
			"quotationMethod=" + this.quotationMethod + ", " +
			"recoveryFactor=" + this.recoveryFactor + ", " +
			"valuationDate=" + this.valuationDate + ", " +
			"valuationMethod=" + this.valuationMethod + ", " +
			"valuationTime=" + this.valuationTime +
		'}' + " " + super.toString();
	}
}
