package org.isda.cdm;

import org.isda.cdm.CashflowTypeEnum;
import org.isda.cdm.Money;
import org.isda.cdm.PayerReceiver;
import org.isda.cdm.metafields.FieldWithMetaString;


import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.CashTransferBreakdownMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;
		import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;

/**
 * @version 2.5.4
 */
@RosettaClass
public class CashTransferBreakdown extends TransferBreakdown {
	private final Money amount;
	private final CashflowTypeEnum cashflowType;
	private final PayerReceiver payerReceiver;
	private static CashTransferBreakdownMeta metaData = new CashTransferBreakdownMeta();
	
	CashTransferBreakdown(CashTransferBreakdownBuilder builder) {
		super(builder);
		this.amount = ofNullable(builder.getAmount()).map(f->f.build()).orElse(null);
		this.cashflowType = builder.getCashflowType();
		this.payerReceiver = ofNullable(builder.getPayerReceiver()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The currency amount of the payment.
	 */
	@RosettaSynonym(value="amount", source="Rosetta_Workbench")
	public final Money getAmount() {
		return amount;
	}
	
	/**
	 * The qualification of the type of cashflow, when not inferred from a derived through lineage e.g. brokerage fee, premium, upfront fee etc.
	 */
	@RosettaSynonym(value="cashflowType", source="Rosetta_Workbench")
	public final CashflowTypeEnum getCashflowType() {
		return cashflowType;
	}
	
	/**
	 * The payer and receiver party information.
	 */
	public final PayerReceiver getPayerReceiver() {
		return payerReceiver;
	}
	
	
	@Override
	public RosettaMetaData<? extends CashTransferBreakdown> metaData() {
		return metaData;
	} 
	
	public CashTransferBreakdownBuilder toBuilder() {
		CashTransferBreakdownBuilder builder = new CashTransferBreakdownBuilder();
		ofNullable(getAmount()).ifPresent(builder::setAmount);
		ofNullable(getCashflowType()).ifPresent(builder::setCashflowType);
		ofNullable(getPayerReceiver()).ifPresent(builder::setPayerReceiver);
		ofNullable(getLineage()).ifPresent(lineage -> lineage.forEach(builder::addLineage));
		ofNullable(getIdentifier()).ifPresent(builder::setIdentifier);
		ofNullable(getTransferCalculation()).ifPresent(builder::setTransferCalculation);
		return builder;
	}

	public static CashTransferBreakdownBuilder builder() {
		return new CashTransferBreakdownBuilder();
	}

	public static  class CashTransferBreakdownBuilder extends TransferBreakdownBuilder {
	
		protected Money.MoneyBuilder amount;
		protected CashflowTypeEnum cashflowType;
		protected PayerReceiver.PayerReceiverBuilder payerReceiver;
	
		public CashTransferBreakdownBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends CashTransferBreakdown> metaData() {
			return metaData;
		} 
	
		
		public Money.MoneyBuilder getAmount() {
			return amount;
		}
		
		public Money.MoneyBuilder getOrCreateAmount() {
			if (amount!=null) {
				return amount;
			}
			else return amount = new Money.MoneyBuilder();
		}
		
		
		public CashflowTypeEnum getCashflowType() {
			return cashflowType;
		}
		
		
		public PayerReceiver.PayerReceiverBuilder getPayerReceiver() {
			return payerReceiver;
		}
		
		public PayerReceiver.PayerReceiverBuilder getOrCreatePayerReceiver() {
			if (payerReceiver!=null) {
				return payerReceiver;
			}
			else return payerReceiver = new PayerReceiver.PayerReceiverBuilder();
		}
		
	
		public CashTransferBreakdownBuilder setAmount(Money amount) {
			this.amount = amount.toBuilder();
			return this;
		}
		public CashTransferBreakdownBuilder setAmountBuilder(Money.MoneyBuilder amount) {
			this.amount = amount;
			return this;
		}
		public CashTransferBreakdownBuilder setCashflowType(CashflowTypeEnum cashflowType) {
			this.cashflowType = cashflowType;
			return this;
		}
		public CashTransferBreakdownBuilder setPayerReceiver(PayerReceiver payerReceiver) {
			this.payerReceiver = payerReceiver.toBuilder();
			return this;
		}
		public CashTransferBreakdownBuilder setPayerReceiverBuilder(PayerReceiver.PayerReceiverBuilder payerReceiver) {
			this.payerReceiver = payerReceiver;
			return this;
		}
		@Override public CashTransferBreakdownBuilder addLineage(Lineage lineage) {
			if(this.lineage == null){
				this.lineage = new ArrayList<>();
			}
			this.lineage.add(lineage.toBuilder());
			return this;
		}
		@Override public CashTransferBreakdownBuilder addLineage(Lineage lineage, int _idx) {
			if(this.lineage == null){
				this.lineage = new ArrayList<>();
			}
			getIndex(this.lineage, _idx, () -> lineage.toBuilder());
			this.lineage.set(_idx, lineage.toBuilder());
			return this;
		}
		@Override public CashTransferBreakdownBuilder addLineage(List<Lineage> lineages) {
			if(this.lineage == null){
				this.lineage = new ArrayList<>();
			}
			for (Lineage toAdd : lineages) {
				this.lineage.add(toAdd.toBuilder());
			}
			return this;
		}
		@Override public CashTransferBreakdownBuilder addLineageBuilder(Lineage.LineageBuilder lineage) {
			if(this.lineage == null){
				this.lineage = new ArrayList<>();
			}
			this.lineage.add(lineage);
			return this;
		}
		
		@Override public CashTransferBreakdownBuilder clearLineage() {
			this.lineage = null;
			return this;
		}
		@Override public CashTransferBreakdownBuilder setIdentifier(FieldWithMetaString identifier) {
			this.identifier = identifier.toBuilder();
			return this;
		}
		@Override public CashTransferBreakdownBuilder setIdentifierRef(String identifier) {
			return setIdentifier(FieldWithMetaString.builder().setValue(identifier).build());
		}
		@Override public CashTransferBreakdownBuilder setTransferCalculation(TransferCalculation transferCalculation) {
			this.transferCalculation = transferCalculation.toBuilder();
			return this;
		}
		@Override public CashTransferBreakdownBuilder setTransferCalculationBuilder(TransferCalculation.TransferCalculationBuilder transferCalculation) {
			this.transferCalculation = transferCalculation;
			return this;
		}
		
		public CashTransferBreakdown build() {
			return new CashTransferBreakdown(this);
		}
	
		@Override
		public CashTransferBreakdownBuilder prune() {
			super.prune();
			if (amount!=null && !amount.prune().hasData()) amount = null;
			if (payerReceiver!=null && !payerReceiver.prune().hasData()) payerReceiver = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (super.hasData()) return true;
			if (getAmount()!=null && getAmount().hasData()) return true;
			if (getCashflowType()!=null) return true;
			if (getPayerReceiver()!=null && getPayerReceiver().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
			super.process(path, processor);
		
			processor.processBasic(path.newSubPath("cashflowType"), CashflowTypeEnum.class, cashflowType, this);
			
			processRosetta(path.newSubPath("amount"), processor, Money.class, amount);
			processRosetta(path.newSubPath("payerReceiver"), processor, PayerReceiver.class, payerReceiver);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
			if (!super.equals(o)) return false;
		
			CashTransferBreakdownBuilder _that = (CashTransferBreakdownBuilder) o;
		
			if (amount != null ? !amount.equals(_that.amount) : _that.amount != null) return false;
			if (cashflowType != null ? !cashflowType.equals(_that.cashflowType) : _that.cashflowType != null) return false;
			if (payerReceiver != null ? !payerReceiver.equals(_that.payerReceiver) : _that.payerReceiver != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = super.hashCode();
			_result = 31 * _result + (amount != null ? amount.hashCode() : 0);
			_result = 31 * _result + (cashflowType != null ? cashflowType.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (payerReceiver != null ? payerReceiver.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "CashTransferBreakdownBuilder {" +
				"amount=" + this.amount + ", " +
				"cashflowType=" + this.cashflowType + ", " +
				"payerReceiver=" + this.payerReceiver +
			'}' + " " + super.toString();
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		super.process(path, processor);
		
		processor.processBasic(path.newSubPath("cashflowType"), CashflowTypeEnum.class, cashflowType, this);
		
		processRosetta(path.newSubPath("amount"), processor, Money.class, amount);
		processRosetta(path.newSubPath("payerReceiver"), processor, PayerReceiver.class, payerReceiver);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
		if (!super.equals(o)) return false;
	
		CashTransferBreakdown _that = (CashTransferBreakdown) o;
	
		if (amount != null ? !amount.equals(_that.amount) : _that.amount != null) return false;
		if (cashflowType != null ? !cashflowType.equals(_that.cashflowType) : _that.cashflowType != null) return false;
		if (payerReceiver != null ? !payerReceiver.equals(_that.payerReceiver) : _that.payerReceiver != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = super.hashCode();
		_result = 31 * _result + (amount != null ? amount.hashCode() : 0);
		_result = 31 * _result + (cashflowType != null ? cashflowType.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (payerReceiver != null ? payerReceiver.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "CashTransferBreakdown {" +
			"amount=" + this.amount + ", " +
			"cashflowType=" + this.cashflowType + ", " +
			"payerReceiver=" + this.payerReceiver +
		'}' + " " + super.toString();
	}
}
