package org.isda.cdm;

import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.CashTransferBreakdown;
import org.isda.cdm.CashflowTypeEnum;
import org.isda.cdm.Money;
import org.isda.cdm.PayerReceiver;
import org.isda.cdm.metafields.FieldWithMetaString;


import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.CashTransferComponentMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;
		import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;

/**
 * @version 2.5.4
 */
@RosettaClass
public class CashTransferComponent extends TransferBase {
	private final Money amount;
	private final List<CashTransferBreakdown> breakdown;
	private final CashflowTypeEnum cashflowType;
	private final PayerReceiver payerReceiver;
	private static CashTransferComponentMeta metaData = new CashTransferComponentMeta();
	
	CashTransferComponent(CashTransferComponentBuilder builder) {
		super(builder);
		this.amount = ofNullable(builder.getAmount()).map(f->f.build()).orElse(null);
		this.breakdown = ofNullable(builder.getBreakdown()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.cashflowType = builder.getCashflowType();
		this.payerReceiver = ofNullable(builder.getPayerReceiver()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The currency amount.
	 */
	@RosettaSynonym(value="amount", source="Rosetta_Workbench")
	@RosettaSynonym(value="paymentAmount", source="DTCC_11_0")
	@RosettaSynonym(value="paymentAmount", source="DTCC_9_0")
	public final Money getAmount() {
		return amount;
	}
	
	/**
	 * The cash transfer breakdown, when the transfer corresponds to a net amount across several components which breakdown is deemed relevant (e.g. the net cash transfer related to several contracts, or to the net proceeds across several payout components).
	 */
	@RosettaSynonym(value="breakdown", source="Rosetta_Workbench")
	@RosettaSynonym(value="breakdown", source="DTCC_11_0")
	@RosettaSynonym(value="breakdown", source="DTCC_9_0")
	public final List<CashTransferBreakdown> getBreakdown() {
		return breakdown;
	}
	
	/**
	 * The qualification of the type of cashflow, when not inferred from a derived through lineage e.g. brokerage fee, premium, upfront fee etc.
	 */
	@RosettaSynonym(value="cashflowType", source="Rosetta_Workbench")
	@RosettaSynonym(value="cashflowType", source="DTCC_11_0", path="calculationDetails.grossCashflow")
	@RosettaSynonym(value="cashflowType", source="DTCC_9_0", path="calculationDetails.grossCashflow")
	public final CashflowTypeEnum getCashflowType() {
		return cashflowType;
	}
	
	/**
	 * The payer and receiver party information.
	 */
	public final PayerReceiver getPayerReceiver() {
		return payerReceiver;
	}
	
	
	@Override
	public RosettaMetaData<? extends CashTransferComponent> metaData() {
		return metaData;
	} 
	
	public CashTransferComponentBuilder toBuilder() {
		CashTransferComponentBuilder builder = new CashTransferComponentBuilder();
		ofNullable(getAmount()).ifPresent(builder::setAmount);
		ofNullable(getBreakdown()).ifPresent(breakdown -> breakdown.forEach(builder::addBreakdown));
		ofNullable(getCashflowType()).ifPresent(builder::setCashflowType);
		ofNullable(getPayerReceiver()).ifPresent(builder::setPayerReceiver);
		ofNullable(getIdentifier()).ifPresent(builder::setIdentifier);
		ofNullable(getTransferCalculation()).ifPresent(builder::setTransferCalculation);
		return builder;
	}

	public static CashTransferComponentBuilder builder() {
		return new CashTransferComponentBuilder();
	}

	public static  class CashTransferComponentBuilder extends TransferBaseBuilder {
	
		protected Money.MoneyBuilder amount;
		protected List<CashTransferBreakdown.CashTransferBreakdownBuilder> breakdown;
		protected CashflowTypeEnum cashflowType;
		protected PayerReceiver.PayerReceiverBuilder payerReceiver;
	
		public CashTransferComponentBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends CashTransferComponent> metaData() {
			return metaData;
		} 
	
		
		public Money.MoneyBuilder getAmount() {
			return amount;
		}
		
		public Money.MoneyBuilder getOrCreateAmount() {
			if (amount!=null) {
				return amount;
			}
			else return amount = new Money.MoneyBuilder();
		}
		
		
		public List<CashTransferBreakdown.CashTransferBreakdownBuilder> getBreakdown() {
			return breakdown;
		}
		
		public CashTransferBreakdown.CashTransferBreakdownBuilder getOrCreateBreakdown(int index) {
			if (breakdown==null) {
				this.breakdown = new ArrayList<>();
			}
			return getIndex(breakdown, index, ()->new CashTransferBreakdown.CashTransferBreakdownBuilder());
		}
		
		
		public CashflowTypeEnum getCashflowType() {
			return cashflowType;
		}
		
		
		public PayerReceiver.PayerReceiverBuilder getPayerReceiver() {
			return payerReceiver;
		}
		
		public PayerReceiver.PayerReceiverBuilder getOrCreatePayerReceiver() {
			if (payerReceiver!=null) {
				return payerReceiver;
			}
			else return payerReceiver = new PayerReceiver.PayerReceiverBuilder();
		}
		
	
		public CashTransferComponentBuilder setAmount(Money amount) {
			this.amount = amount.toBuilder();
			return this;
		}
		public CashTransferComponentBuilder setAmountBuilder(Money.MoneyBuilder amount) {
			this.amount = amount;
			return this;
		}
		public CashTransferComponentBuilder addBreakdown(CashTransferBreakdown breakdown) {
			if(this.breakdown == null){
				this.breakdown = new ArrayList<>();
			}
			this.breakdown.add(breakdown.toBuilder());
			return this;
		}
		public CashTransferComponentBuilder addBreakdown(CashTransferBreakdown breakdown, int _idx) {
			if(this.breakdown == null){
				this.breakdown = new ArrayList<>();
			}
			getIndex(this.breakdown, _idx, () -> breakdown.toBuilder());
			this.breakdown.set(_idx, breakdown.toBuilder());
			return this;
		}
		public CashTransferComponentBuilder addBreakdown(List<CashTransferBreakdown> breakdowns) {
			if(this.breakdown == null){
				this.breakdown = new ArrayList<>();
			}
			for (CashTransferBreakdown toAdd : breakdowns) {
				this.breakdown.add(toAdd.toBuilder());
			}
			return this;
		}
		public CashTransferComponentBuilder addBreakdownBuilder(CashTransferBreakdown.CashTransferBreakdownBuilder breakdown) {
			if(this.breakdown == null){
				this.breakdown = new ArrayList<>();
			}
			this.breakdown.add(breakdown);
			return this;
		}
		
		public CashTransferComponentBuilder clearBreakdown() {
			this.breakdown = null;
			return this;
		}
		public CashTransferComponentBuilder setCashflowType(CashflowTypeEnum cashflowType) {
			this.cashflowType = cashflowType;
			return this;
		}
		public CashTransferComponentBuilder setPayerReceiver(PayerReceiver payerReceiver) {
			this.payerReceiver = payerReceiver.toBuilder();
			return this;
		}
		public CashTransferComponentBuilder setPayerReceiverBuilder(PayerReceiver.PayerReceiverBuilder payerReceiver) {
			this.payerReceiver = payerReceiver;
			return this;
		}
		@Override public CashTransferComponentBuilder setIdentifier(FieldWithMetaString identifier) {
			this.identifier = identifier.toBuilder();
			return this;
		}
		@Override public CashTransferComponentBuilder setIdentifierRef(String identifier) {
			return setIdentifier(FieldWithMetaString.builder().setValue(identifier).build());
		}
		@Override public CashTransferComponentBuilder setTransferCalculation(TransferCalculation transferCalculation) {
			this.transferCalculation = transferCalculation.toBuilder();
			return this;
		}
		@Override public CashTransferComponentBuilder setTransferCalculationBuilder(TransferCalculation.TransferCalculationBuilder transferCalculation) {
			this.transferCalculation = transferCalculation;
			return this;
		}
		
		public CashTransferComponent build() {
			return new CashTransferComponent(this);
		}
	
		@Override
		public CashTransferComponentBuilder prune() {
			super.prune();
			if (amount!=null && !amount.prune().hasData()) amount = null;
			if (breakdown!=null) breakdown = breakdown.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (payerReceiver!=null && !payerReceiver.prune().hasData()) payerReceiver = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (super.hasData()) return true;
			if (getAmount()!=null && getAmount().hasData()) return true;
			if (getBreakdown()!=null && getBreakdown().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getCashflowType()!=null) return true;
			if (getPayerReceiver()!=null && getPayerReceiver().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
			super.process(path, processor);
		
			processor.processBasic(path.newSubPath("cashflowType"), CashflowTypeEnum.class, cashflowType, this);
			
			processRosetta(path.newSubPath("amount"), processor, Money.class, amount);
			processRosetta(path.newSubPath("breakdown"), processor, CashTransferBreakdown.class, breakdown);
			processRosetta(path.newSubPath("payerReceiver"), processor, PayerReceiver.class, payerReceiver);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
			if (!super.equals(o)) return false;
		
			CashTransferComponentBuilder _that = (CashTransferComponentBuilder) o;
		
			if (amount != null ? !amount.equals(_that.amount) : _that.amount != null) return false;
			if (!ListEquals.listEquals(breakdown, _that.breakdown)) return false;
			if (cashflowType != null ? !cashflowType.equals(_that.cashflowType) : _that.cashflowType != null) return false;
			if (payerReceiver != null ? !payerReceiver.equals(_that.payerReceiver) : _that.payerReceiver != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = super.hashCode();
			_result = 31 * _result + (amount != null ? amount.hashCode() : 0);
			_result = 31 * _result + (breakdown != null ? breakdown.hashCode() : 0);
			_result = 31 * _result + (cashflowType != null ? cashflowType.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (payerReceiver != null ? payerReceiver.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "CashTransferComponentBuilder {" +
				"amount=" + this.amount + ", " +
				"breakdown=" + this.breakdown + ", " +
				"cashflowType=" + this.cashflowType + ", " +
				"payerReceiver=" + this.payerReceiver +
			'}' + " " + super.toString();
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		super.process(path, processor);
		
		processor.processBasic(path.newSubPath("cashflowType"), CashflowTypeEnum.class, cashflowType, this);
		
		processRosetta(path.newSubPath("amount"), processor, Money.class, amount);
		processRosetta(path.newSubPath("breakdown"), processor, CashTransferBreakdown.class, breakdown);
		processRosetta(path.newSubPath("payerReceiver"), processor, PayerReceiver.class, payerReceiver);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
		if (!super.equals(o)) return false;
	
		CashTransferComponent _that = (CashTransferComponent) o;
	
		if (amount != null ? !amount.equals(_that.amount) : _that.amount != null) return false;
		if (!ListEquals.listEquals(breakdown, _that.breakdown)) return false;
		if (cashflowType != null ? !cashflowType.equals(_that.cashflowType) : _that.cashflowType != null) return false;
		if (payerReceiver != null ? !payerReceiver.equals(_that.payerReceiver) : _that.payerReceiver != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = super.hashCode();
		_result = 31 * _result + (amount != null ? amount.hashCode() : 0);
		_result = 31 * _result + (breakdown != null ? breakdown.hashCode() : 0);
		_result = 31 * _result + (cashflowType != null ? cashflowType.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (payerReceiver != null ? payerReceiver.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "CashTransferComponent {" +
			"amount=" + this.amount + ", " +
			"breakdown=" + this.breakdown + ", " +
			"cashflowType=" + this.cashflowType + ", " +
			"payerReceiver=" + this.payerReceiver +
		'}' + " " + super.toString();
	}
}
