package org.isda.cdm;

import java.lang.Boolean;
import java.lang.String;
import java.math.BigDecimal;
import org.isda.cdm.AdjustableOrAdjustedOrRelativeDate;
import org.isda.cdm.CashflowTypeEnum;
import org.isda.cdm.Money;
import org.isda.cdm.PayerReceiver;
import org.isda.cdm.PaymentDiscounting;
import org.isda.cdm.PremiumExpression;
import org.isda.cdm.metafields.MetaFields;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.CashflowMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;
		import java.math.BigDecimal;

/**
 * A class to specify a cashflow, i.e. the outcome of either of computation (e.g. interest accrual) or an assessment of some sort (e.g. a fee). The cashflow can then be turned into a cash transfer, artefact to be used as the input to a payment system or the outcome of it. The associated rosettaKey denotes the ability to associate a hash value to the Cashflow instantiations for the purpose of model cross-referencing, in support of functionality such as the event effect and the lineage.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class Cashflow extends RosettaModelObject implements GlobalKey {
	private final Money cashflowAmount;
	private final String cashflowCalculation;
	private final AdjustableOrAdjustedOrRelativeDate cashflowDate;
	private final CashflowTypeEnum cashflowType;
	private final BigDecimal discountFactor;
	private final MetaFields meta;
	private final PayerReceiver payerReceiver;
	private final Boolean paymentDelay;
	private final PaymentDiscounting paymentDiscounting;
	private final PremiumExpression premiumExpression;
	private final Money presentValueAmount;
	private static CashflowMeta metaData = new CashflowMeta();
	
	Cashflow(CashflowBuilder builder) {
		this.cashflowAmount = ofNullable(builder.getCashflowAmount()).map(f->f.build()).orElse(null);
		this.cashflowCalculation = builder.getCashflowCalculation();
		this.cashflowDate = ofNullable(builder.getCashflowDate()).map(f->f.build()).orElse(null);
		this.cashflowType = builder.getCashflowType();
		this.discountFactor = builder.getDiscountFactor();
		this.meta = ofNullable(builder.getMeta()).map(f->f.build()).orElse(null);
		this.payerReceiver = ofNullable(builder.getPayerReceiver()).map(f->f.build()).orElse(null);
		this.paymentDelay = builder.getPaymentDelay();
		this.paymentDiscounting = ofNullable(builder.getPaymentDiscounting()).map(f->f.build()).orElse(null);
		this.premiumExpression = ofNullable(builder.getPremiumExpression()).map(f->f.build()).orElse(null);
		this.presentValueAmount = ofNullable(builder.getPresentValueAmount()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The currency amount of the payment.
	 */
	@RosettaSynonym(value="paymentAmount", source="FpML_5_10")
	@RosettaSynonym(value="paymentAmount", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="paymentAmount", source="DTCC_11_0")
	@RosettaSynonym(value="paymentAmount", source="DTCC_9_0")
	@RosettaSynonym(value="paymentAmount", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="fixedAmount", source="FpML_5_10")
	@RosettaSynonym(value="fixedAmount", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="fixedAmount", source="DTCC_11_0")
	@RosettaSynonym(value="fixedAmount", source="DTCC_9_0")
	@RosettaSynonym(value="fixedAmount", source="CME_ClearedConfirm_1_17")
	public final Money getCashflowAmount() {
		return cashflowAmount;
	}
	
	/**
	 * This is a conceptual placeholder for providing the breakdown into the cashflow calculation components, leveraging the fact that the CDM provides calculation components, starting with the FixedAmount and the FloatingAmount. Further evaluation of expected usage needs to take place to confirm and prioritize such implementation.
	 */
	@RosettaSynonym(value="cashflowCalculation", source="Rosetta_Workbench")
	public final String getCashflowCalculation() {
		return cashflowCalculation;
	}
	
	public final AdjustableOrAdjustedOrRelativeDate getCashflowDate() {
		return cashflowDate;
	}
	
	/**
	 * The qualification of the type of cashflow, e.g. brokerage fee, premium, upfront fee etc. Particularly relevant when it cannot be inferred directly through lineage.
	 */
	@RosettaSynonym(value="paymentType", source="FpML_5_10")
	@RosettaSynonym(value="paymentType", source="DTCC_11_0")
	@RosettaSynonym(value="paymentType", source="DTCC_9_0")
	@RosettaSynonym(value="paymentType", source="CME_ClearedConfirm_1_17")
	public final CashflowTypeEnum getCashflowType() {
		return cashflowType;
	}
	
	/**
	 * The value representing the discount factor used to calculate the present value of the cashflow.
	 */
	@RosettaSynonym(value="discountFactor", source="FpML_5_10")
	@RosettaSynonym(value="discountFactor", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="discountFactor", source="DTCC_11_0")
	@RosettaSynonym(value="discountFactor", source="DTCC_9_0")
	@RosettaSynonym(value="discountFactor", source="CME_ClearedConfirm_1_17")
	public final BigDecimal getDiscountFactor() {
		return discountFactor;
	}
	
	public final MetaFields getMeta() {
		return meta;
	}
	
	public final PayerReceiver getPayerReceiver() {
		return payerReceiver;
	}
	
	/**
	 * Applicable to CDS on MBS to specify whether payment delays are applicable to the fixed Amount. RMBS typically have a payment delay of 5 days between the coupon date of the reference obligation and the payment date of the synthetic swap. CMBS do not, on the other hand, with both payment dates being on the 25th of each month.
	 */
	@RosettaSynonym(value="paymentDelay", source="FpML_5_10", path="trade.creditDefaultSwap.feeLeg")
	@RosettaSynonym(value="paymentDelay", source="DTCC_11_0", path="trade.creditDefaultSwap.feeLeg")
	@RosettaSynonym(value="paymentDelay", source="DTCC_9_0", path="trade.creditDefaultSwap.feeLeg")
	@RosettaSynonym(value="paymentDelay", source="CME_ClearedConfirm_1_17", path="trade.creditDefaultSwap.feeLeg")
	public final Boolean getPaymentDelay() {
		return paymentDelay;
	}
	
	/**
	 * FpML specifies the FpML PaymentDiscounting.model group for representing the discounting elements that can be associated with a payment.
	 */
	public final PaymentDiscounting getPaymentDiscounting() {
		return paymentDiscounting;
	}
	
	/**
	 * FpML specifies the Premium.model group for representing the option premium when expressed in a way other than an amount.
	 */
	public final PremiumExpression getPremiumExpression() {
		return premiumExpression;
	}
	
	/**
	 * The amount representing the present value of the forecast payment.
	 */
	@RosettaSynonym(value="presentValueAmount", source="FpML_5_10")
	@RosettaSynonym(value="presentValueAmount", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="presentValueAmount", source="DTCC_11_0")
	@RosettaSynonym(value="presentValueAmount", source="DTCC_9_0")
	@RosettaSynonym(value="presentValueAmount", source="CME_ClearedConfirm_1_17")
	public final Money getPresentValueAmount() {
		return presentValueAmount;
	}
	
	
	@Override
	public RosettaMetaData<? extends Cashflow> metaData() {
		return metaData;
	} 
	
	public CashflowBuilder toBuilder() {
		CashflowBuilder builder = new CashflowBuilder();
		ofNullable(getCashflowAmount()).ifPresent(builder::setCashflowAmount);
		ofNullable(getCashflowCalculation()).ifPresent(builder::setCashflowCalculation);
		ofNullable(getCashflowDate()).ifPresent(builder::setCashflowDate);
		ofNullable(getCashflowType()).ifPresent(builder::setCashflowType);
		ofNullable(getDiscountFactor()).ifPresent(builder::setDiscountFactor);
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		ofNullable(getPayerReceiver()).ifPresent(builder::setPayerReceiver);
		ofNullable(getPaymentDelay()).ifPresent(builder::setPaymentDelay);
		ofNullable(getPaymentDiscounting()).ifPresent(builder::setPaymentDiscounting);
		ofNullable(getPremiumExpression()).ifPresent(builder::setPremiumExpression);
		ofNullable(getPresentValueAmount()).ifPresent(builder::setPresentValueAmount);
		return builder;
	}

	public static CashflowBuilder builder() {
		return new CashflowBuilder();
	}

	public static  class CashflowBuilder extends RosettaModelObjectBuilder implements GlobalKeyBuilder<CashflowBuilder> {
	
		protected Money.MoneyBuilder cashflowAmount;
		protected String cashflowCalculation;
		protected AdjustableOrAdjustedOrRelativeDate.AdjustableOrAdjustedOrRelativeDateBuilder cashflowDate;
		protected CashflowTypeEnum cashflowType;
		protected BigDecimal discountFactor;
		protected MetaFields.MetaFieldsBuilder meta;
		protected PayerReceiver.PayerReceiverBuilder payerReceiver;
		protected Boolean paymentDelay;
		protected PaymentDiscounting.PaymentDiscountingBuilder paymentDiscounting;
		protected PremiumExpression.PremiumExpressionBuilder premiumExpression;
		protected Money.MoneyBuilder presentValueAmount;
	
		public CashflowBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends Cashflow> metaData() {
			return metaData;
		} 
	
		
		public Money.MoneyBuilder getCashflowAmount() {
			return cashflowAmount;
		}
		
		public Money.MoneyBuilder getOrCreateCashflowAmount() {
			if (cashflowAmount!=null) {
				return cashflowAmount;
			}
			else return cashflowAmount = new Money.MoneyBuilder();
		}
		
		
		public String getCashflowCalculation() {
			return cashflowCalculation;
		}
		
		
		public AdjustableOrAdjustedOrRelativeDate.AdjustableOrAdjustedOrRelativeDateBuilder getCashflowDate() {
			return cashflowDate;
		}
		
		public AdjustableOrAdjustedOrRelativeDate.AdjustableOrAdjustedOrRelativeDateBuilder getOrCreateCashflowDate() {
			if (cashflowDate!=null) {
				return cashflowDate;
			}
			else return cashflowDate = new AdjustableOrAdjustedOrRelativeDate.AdjustableOrAdjustedOrRelativeDateBuilder();
		}
		
		
		public CashflowTypeEnum getCashflowType() {
			return cashflowType;
		}
		
		
		public BigDecimal getDiscountFactor() {
			return discountFactor;
		}
		
		
		public MetaFields.MetaFieldsBuilder getMeta() {
			return meta;
		}
		
		public MetaFields.MetaFieldsBuilder getOrCreateMeta() {
			if (meta!=null) {
				return meta;
			}
			else return meta = new MetaFields.MetaFieldsBuilder();
		}
		
		
		public PayerReceiver.PayerReceiverBuilder getPayerReceiver() {
			return payerReceiver;
		}
		
		public PayerReceiver.PayerReceiverBuilder getOrCreatePayerReceiver() {
			if (payerReceiver!=null) {
				return payerReceiver;
			}
			else return payerReceiver = new PayerReceiver.PayerReceiverBuilder();
		}
		
		
		public Boolean getPaymentDelay() {
			return paymentDelay;
		}
		
		
		public PaymentDiscounting.PaymentDiscountingBuilder getPaymentDiscounting() {
			return paymentDiscounting;
		}
		
		public PaymentDiscounting.PaymentDiscountingBuilder getOrCreatePaymentDiscounting() {
			if (paymentDiscounting!=null) {
				return paymentDiscounting;
			}
			else return paymentDiscounting = new PaymentDiscounting.PaymentDiscountingBuilder();
		}
		
		
		public PremiumExpression.PremiumExpressionBuilder getPremiumExpression() {
			return premiumExpression;
		}
		
		public PremiumExpression.PremiumExpressionBuilder getOrCreatePremiumExpression() {
			if (premiumExpression!=null) {
				return premiumExpression;
			}
			else return premiumExpression = new PremiumExpression.PremiumExpressionBuilder();
		}
		
		
		public Money.MoneyBuilder getPresentValueAmount() {
			return presentValueAmount;
		}
		
		public Money.MoneyBuilder getOrCreatePresentValueAmount() {
			if (presentValueAmount!=null) {
				return presentValueAmount;
			}
			else return presentValueAmount = new Money.MoneyBuilder();
		}
		
	
		public CashflowBuilder setCashflowAmount(Money cashflowAmount) {
			this.cashflowAmount = cashflowAmount.toBuilder();
			return this;
		}
		public CashflowBuilder setCashflowAmountBuilder(Money.MoneyBuilder cashflowAmount) {
			this.cashflowAmount = cashflowAmount;
			return this;
		}
		public CashflowBuilder setCashflowCalculation(String cashflowCalculation) {
			this.cashflowCalculation = cashflowCalculation;
			return this;
		}
		public CashflowBuilder setCashflowDate(AdjustableOrAdjustedOrRelativeDate cashflowDate) {
			this.cashflowDate = cashflowDate.toBuilder();
			return this;
		}
		public CashflowBuilder setCashflowDateBuilder(AdjustableOrAdjustedOrRelativeDate.AdjustableOrAdjustedOrRelativeDateBuilder cashflowDate) {
			this.cashflowDate = cashflowDate;
			return this;
		}
		public CashflowBuilder setCashflowType(CashflowTypeEnum cashflowType) {
			this.cashflowType = cashflowType;
			return this;
		}
		public CashflowBuilder setDiscountFactor(BigDecimal discountFactor) {
			this.discountFactor = discountFactor;
			return this;
		}
		public CashflowBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		public CashflowBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		public CashflowBuilder setPayerReceiver(PayerReceiver payerReceiver) {
			this.payerReceiver = payerReceiver.toBuilder();
			return this;
		}
		public CashflowBuilder setPayerReceiverBuilder(PayerReceiver.PayerReceiverBuilder payerReceiver) {
			this.payerReceiver = payerReceiver;
			return this;
		}
		public CashflowBuilder setPaymentDelay(Boolean paymentDelay) {
			this.paymentDelay = paymentDelay;
			return this;
		}
		public CashflowBuilder setPaymentDiscounting(PaymentDiscounting paymentDiscounting) {
			this.paymentDiscounting = paymentDiscounting.toBuilder();
			return this;
		}
		public CashflowBuilder setPaymentDiscountingBuilder(PaymentDiscounting.PaymentDiscountingBuilder paymentDiscounting) {
			this.paymentDiscounting = paymentDiscounting;
			return this;
		}
		public CashflowBuilder setPremiumExpression(PremiumExpression premiumExpression) {
			this.premiumExpression = premiumExpression.toBuilder();
			return this;
		}
		public CashflowBuilder setPremiumExpressionBuilder(PremiumExpression.PremiumExpressionBuilder premiumExpression) {
			this.premiumExpression = premiumExpression;
			return this;
		}
		public CashflowBuilder setPresentValueAmount(Money presentValueAmount) {
			this.presentValueAmount = presentValueAmount.toBuilder();
			return this;
		}
		public CashflowBuilder setPresentValueAmountBuilder(Money.MoneyBuilder presentValueAmount) {
			this.presentValueAmount = presentValueAmount;
			return this;
		}
		
		public Cashflow build() {
			return new Cashflow(this);
		}
	
		@Override
		public CashflowBuilder prune() {
			if (cashflowAmount!=null && !cashflowAmount.prune().hasData()) cashflowAmount = null;
			if (cashflowDate!=null && !cashflowDate.prune().hasData()) cashflowDate = null;
			if (meta!=null && !meta.prune().hasData()) meta = null;
			if (payerReceiver!=null && !payerReceiver.prune().hasData()) payerReceiver = null;
			if (paymentDiscounting!=null && !paymentDiscounting.prune().hasData()) paymentDiscounting = null;
			if (premiumExpression!=null && !premiumExpression.prune().hasData()) premiumExpression = null;
			if (presentValueAmount!=null && !presentValueAmount.prune().hasData()) presentValueAmount = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getCashflowAmount()!=null && getCashflowAmount().hasData()) return true;
			if (getCashflowCalculation()!=null) return true;
			if (getCashflowDate()!=null && getCashflowDate().hasData()) return true;
			if (getCashflowType()!=null) return true;
			if (getDiscountFactor()!=null) return true;
			if (getMeta()!=null && getMeta().hasData()) return true;
			if (getPayerReceiver()!=null && getPayerReceiver().hasData()) return true;
			if (getPaymentDelay()!=null) return true;
			if (getPaymentDiscounting()!=null && getPaymentDiscounting().hasData()) return true;
			if (getPremiumExpression()!=null && getPremiumExpression().hasData()) return true;
			if (getPresentValueAmount()!=null && getPresentValueAmount().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("cashflowCalculation"), String.class, cashflowCalculation, this);
			processor.processBasic(path.newSubPath("cashflowType"), CashflowTypeEnum.class, cashflowType, this);
			processor.processBasic(path.newSubPath("discountFactor"), BigDecimal.class, discountFactor, this);
			processor.processBasic(path.newSubPath("paymentDelay"), Boolean.class, paymentDelay, this);
			
			processRosetta(path.newSubPath("cashflowAmount"), processor, Money.class, cashflowAmount);
			processRosetta(path.newSubPath("cashflowDate"), processor, AdjustableOrAdjustedOrRelativeDate.class, cashflowDate);
			processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
			processRosetta(path.newSubPath("payerReceiver"), processor, PayerReceiver.class, payerReceiver);
			processRosetta(path.newSubPath("paymentDiscounting"), processor, PaymentDiscounting.class, paymentDiscounting);
			processRosetta(path.newSubPath("premiumExpression"), processor, PremiumExpression.class, premiumExpression);
			processRosetta(path.newSubPath("presentValueAmount"), processor, Money.class, presentValueAmount);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			CashflowBuilder _that = (CashflowBuilder) o;
		
			if (cashflowAmount != null ? !cashflowAmount.equals(_that.cashflowAmount) : _that.cashflowAmount != null) return false;
			if (cashflowCalculation != null ? !cashflowCalculation.equals(_that.cashflowCalculation) : _that.cashflowCalculation != null) return false;
			if (cashflowDate != null ? !cashflowDate.equals(_that.cashflowDate) : _that.cashflowDate != null) return false;
			if (cashflowType != null ? !cashflowType.equals(_that.cashflowType) : _that.cashflowType != null) return false;
			if (discountFactor != null ? !discountFactor.equals(_that.discountFactor) : _that.discountFactor != null) return false;
			if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
			if (payerReceiver != null ? !payerReceiver.equals(_that.payerReceiver) : _that.payerReceiver != null) return false;
			if (paymentDelay != null ? !paymentDelay.equals(_that.paymentDelay) : _that.paymentDelay != null) return false;
			if (paymentDiscounting != null ? !paymentDiscounting.equals(_that.paymentDiscounting) : _that.paymentDiscounting != null) return false;
			if (premiumExpression != null ? !premiumExpression.equals(_that.premiumExpression) : _that.premiumExpression != null) return false;
			if (presentValueAmount != null ? !presentValueAmount.equals(_that.presentValueAmount) : _that.presentValueAmount != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (cashflowAmount != null ? cashflowAmount.hashCode() : 0);
			_result = 31 * _result + (cashflowCalculation != null ? cashflowCalculation.hashCode() : 0);
			_result = 31 * _result + (cashflowDate != null ? cashflowDate.hashCode() : 0);
			_result = 31 * _result + (cashflowType != null ? cashflowType.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (discountFactor != null ? discountFactor.hashCode() : 0);
			_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
			_result = 31 * _result + (payerReceiver != null ? payerReceiver.hashCode() : 0);
			_result = 31 * _result + (paymentDelay != null ? paymentDelay.hashCode() : 0);
			_result = 31 * _result + (paymentDiscounting != null ? paymentDiscounting.hashCode() : 0);
			_result = 31 * _result + (premiumExpression != null ? premiumExpression.hashCode() : 0);
			_result = 31 * _result + (presentValueAmount != null ? presentValueAmount.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "CashflowBuilder {" +
				"cashflowAmount=" + this.cashflowAmount + ", " +
				"cashflowCalculation=" + this.cashflowCalculation + ", " +
				"cashflowDate=" + this.cashflowDate + ", " +
				"cashflowType=" + this.cashflowType + ", " +
				"discountFactor=" + this.discountFactor + ", " +
				"meta=" + this.meta + ", " +
				"payerReceiver=" + this.payerReceiver + ", " +
				"paymentDelay=" + this.paymentDelay + ", " +
				"paymentDiscounting=" + this.paymentDiscounting + ", " +
				"premiumExpression=" + this.premiumExpression + ", " +
				"presentValueAmount=" + this.presentValueAmount +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("cashflowCalculation"), String.class, cashflowCalculation, this);
		processor.processBasic(path.newSubPath("cashflowType"), CashflowTypeEnum.class, cashflowType, this);
		processor.processBasic(path.newSubPath("discountFactor"), BigDecimal.class, discountFactor, this);
		processor.processBasic(path.newSubPath("paymentDelay"), Boolean.class, paymentDelay, this);
		
		processRosetta(path.newSubPath("cashflowAmount"), processor, Money.class, cashflowAmount);
		processRosetta(path.newSubPath("cashflowDate"), processor, AdjustableOrAdjustedOrRelativeDate.class, cashflowDate);
		processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
		processRosetta(path.newSubPath("payerReceiver"), processor, PayerReceiver.class, payerReceiver);
		processRosetta(path.newSubPath("paymentDiscounting"), processor, PaymentDiscounting.class, paymentDiscounting);
		processRosetta(path.newSubPath("premiumExpression"), processor, PremiumExpression.class, premiumExpression);
		processRosetta(path.newSubPath("presentValueAmount"), processor, Money.class, presentValueAmount);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		Cashflow _that = (Cashflow) o;
	
		if (cashflowAmount != null ? !cashflowAmount.equals(_that.cashflowAmount) : _that.cashflowAmount != null) return false;
		if (cashflowCalculation != null ? !cashflowCalculation.equals(_that.cashflowCalculation) : _that.cashflowCalculation != null) return false;
		if (cashflowDate != null ? !cashflowDate.equals(_that.cashflowDate) : _that.cashflowDate != null) return false;
		if (cashflowType != null ? !cashflowType.equals(_that.cashflowType) : _that.cashflowType != null) return false;
		if (discountFactor != null ? !discountFactor.equals(_that.discountFactor) : _that.discountFactor != null) return false;
		if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
		if (payerReceiver != null ? !payerReceiver.equals(_that.payerReceiver) : _that.payerReceiver != null) return false;
		if (paymentDelay != null ? !paymentDelay.equals(_that.paymentDelay) : _that.paymentDelay != null) return false;
		if (paymentDiscounting != null ? !paymentDiscounting.equals(_that.paymentDiscounting) : _that.paymentDiscounting != null) return false;
		if (premiumExpression != null ? !premiumExpression.equals(_that.premiumExpression) : _that.premiumExpression != null) return false;
		if (presentValueAmount != null ? !presentValueAmount.equals(_that.presentValueAmount) : _that.presentValueAmount != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (cashflowAmount != null ? cashflowAmount.hashCode() : 0);
		_result = 31 * _result + (cashflowCalculation != null ? cashflowCalculation.hashCode() : 0);
		_result = 31 * _result + (cashflowDate != null ? cashflowDate.hashCode() : 0);
		_result = 31 * _result + (cashflowType != null ? cashflowType.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (discountFactor != null ? discountFactor.hashCode() : 0);
		_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
		_result = 31 * _result + (payerReceiver != null ? payerReceiver.hashCode() : 0);
		_result = 31 * _result + (paymentDelay != null ? paymentDelay.hashCode() : 0);
		_result = 31 * _result + (paymentDiscounting != null ? paymentDiscounting.hashCode() : 0);
		_result = 31 * _result + (premiumExpression != null ? premiumExpression.hashCode() : 0);
		_result = 31 * _result + (presentValueAmount != null ? presentValueAmount.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "Cashflow {" +
			"cashflowAmount=" + this.cashflowAmount + ", " +
			"cashflowCalculation=" + this.cashflowCalculation + ", " +
			"cashflowDate=" + this.cashflowDate + ", " +
			"cashflowType=" + this.cashflowType + ", " +
			"discountFactor=" + this.discountFactor + ", " +
			"meta=" + this.meta + ", " +
			"payerReceiver=" + this.payerReceiver + ", " +
			"paymentDelay=" + this.paymentDelay + ", " +
			"paymentDiscounting=" + this.paymentDiscounting + ", " +
			"premiumExpression=" + this.premiumExpression + ", " +
			"presentValueAmount=" + this.presentValueAmount +
		'}';
	}
}
