package org.isda.cdm;

import java.lang.Boolean;
import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.PaymentCalculationPeriod;
import org.isda.cdm.PrincipalExchange;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.CashflowRepresentationMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;

/**
 * A class defining the cashflow representation of a swap trade.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="Cashflows", source="FpML_5_10")
public class CashflowRepresentation extends RosettaModelObject {
	private final Boolean cashflowsMatchParameters;
	private final List<PaymentCalculationPeriod> paymentCalculationPeriod;
	private final List<PrincipalExchange> principalExchange;
	private static CashflowRepresentationMeta metaData = new CashflowRepresentationMeta();
	
	CashflowRepresentation(CashflowRepresentationBuilder builder) {
		this.cashflowsMatchParameters = builder.getCashflowsMatchParameters();
		this.paymentCalculationPeriod = ofNullable(builder.getPaymentCalculationPeriod()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.principalExchange = ofNullable(builder.getPrincipalExchange()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
	}
	
	/**
	 * A true/false flag to indicate whether the cashflows match the parametric definition of the stream, i.e. whether the cashflows could be regenerated from the parameters without loss of information.
	 */
	@RosettaSynonym(value="cashflowsMatchParameters", source="FpML_5_10")
	@RosettaSynonym(value="cashflowsMatchParameters", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="cashflowsMatchParameters", source="DTCC_11_0")
	@RosettaSynonym(value="cashflowsMatchParameters", source="DTCC_9_0")
	@RosettaSynonym(value="cashflowsMatchParameters", source="CME_ClearedConfirm_1_17")
	public final Boolean getCashflowsMatchParameters() {
		return cashflowsMatchParameters;
	}
	
	/**
	 * The adjusted payment date and associated calculation period parameters required to calculate the actual or projected payment amount. A list of payment calculation period elements may be ordered in the document by ascending adjusted payment date. An FpML document containing an unordered list of payment calculation periods is still regarded as a conformant document.
	 */
	@RosettaSynonym(value="paymentCalculationPeriod", source="FpML_5_10")
	@RosettaSynonym(value="paymentCalculationPeriod", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="paymentCalculationPeriod", source="DTCC_11_0")
	@RosettaSynonym(value="paymentCalculationPeriod", source="DTCC_9_0")
	@RosettaSynonym(value="paymentCalculationPeriod", source="CME_ClearedConfirm_1_17")
	public final List<PaymentCalculationPeriod> getPaymentCalculationPeriod() {
		return paymentCalculationPeriod;
	}
	
	/**
	 * The initial, intermediate and final principal exchange amounts. Typically required on cross currency interest rate swaps where actual exchanges of principal occur. A list of principal exchange elements may be ordered in the document by ascending adjusted principal exchange date. An FpML document containing an unordered principal exchange list is still regarded as a conformant document.
	 */
	@RosettaSynonym(value="principalExchange", source="FpML_5_10")
	@RosettaSynonym(value="principalExchange", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="principalExchange", source="DTCC_11_0")
	@RosettaSynonym(value="principalExchange", source="DTCC_9_0")
	@RosettaSynonym(value="principalExchange", source="CME_ClearedConfirm_1_17")
	public final List<PrincipalExchange> getPrincipalExchange() {
		return principalExchange;
	}
	
	
	@Override
	public RosettaMetaData<? extends CashflowRepresentation> metaData() {
		return metaData;
	} 
	
	public CashflowRepresentationBuilder toBuilder() {
		CashflowRepresentationBuilder builder = new CashflowRepresentationBuilder();
		ofNullable(getCashflowsMatchParameters()).ifPresent(builder::setCashflowsMatchParameters);
		ofNullable(getPaymentCalculationPeriod()).ifPresent(paymentCalculationPeriod -> paymentCalculationPeriod.forEach(builder::addPaymentCalculationPeriod));
		ofNullable(getPrincipalExchange()).ifPresent(principalExchange -> principalExchange.forEach(builder::addPrincipalExchange));
		return builder;
	}

	public static CashflowRepresentationBuilder builder() {
		return new CashflowRepresentationBuilder();
	}

	public static  class CashflowRepresentationBuilder extends RosettaModelObjectBuilder {
	
		protected Boolean cashflowsMatchParameters;
		protected List<PaymentCalculationPeriod.PaymentCalculationPeriodBuilder> paymentCalculationPeriod;
		protected List<PrincipalExchange.PrincipalExchangeBuilder> principalExchange;
	
		public CashflowRepresentationBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends CashflowRepresentation> metaData() {
			return metaData;
		} 
	
		
		public Boolean getCashflowsMatchParameters() {
			return cashflowsMatchParameters;
		}
		
		
		public List<PaymentCalculationPeriod.PaymentCalculationPeriodBuilder> getPaymentCalculationPeriod() {
			return paymentCalculationPeriod;
		}
		
		public PaymentCalculationPeriod.PaymentCalculationPeriodBuilder getOrCreatePaymentCalculationPeriod(int index) {
			if (paymentCalculationPeriod==null) {
				this.paymentCalculationPeriod = new ArrayList<>();
			}
			return getIndex(paymentCalculationPeriod, index, ()->new PaymentCalculationPeriod.PaymentCalculationPeriodBuilder());
		}
		
		
		public List<PrincipalExchange.PrincipalExchangeBuilder> getPrincipalExchange() {
			return principalExchange;
		}
		
		public PrincipalExchange.PrincipalExchangeBuilder getOrCreatePrincipalExchange(int index) {
			if (principalExchange==null) {
				this.principalExchange = new ArrayList<>();
			}
			return getIndex(principalExchange, index, ()->new PrincipalExchange.PrincipalExchangeBuilder());
		}
		
	
		public CashflowRepresentationBuilder setCashflowsMatchParameters(Boolean cashflowsMatchParameters) {
			this.cashflowsMatchParameters = cashflowsMatchParameters;
			return this;
		}
		public CashflowRepresentationBuilder addPaymentCalculationPeriod(PaymentCalculationPeriod paymentCalculationPeriod) {
			if(this.paymentCalculationPeriod == null){
				this.paymentCalculationPeriod = new ArrayList<>();
			}
			this.paymentCalculationPeriod.add(paymentCalculationPeriod.toBuilder());
			return this;
		}
		public CashflowRepresentationBuilder addPaymentCalculationPeriod(PaymentCalculationPeriod paymentCalculationPeriod, int _idx) {
			if(this.paymentCalculationPeriod == null){
				this.paymentCalculationPeriod = new ArrayList<>();
			}
			getIndex(this.paymentCalculationPeriod, _idx, () -> paymentCalculationPeriod.toBuilder());
			this.paymentCalculationPeriod.set(_idx, paymentCalculationPeriod.toBuilder());
			return this;
		}
		public CashflowRepresentationBuilder addPaymentCalculationPeriod(List<PaymentCalculationPeriod> paymentCalculationPeriods) {
			if(this.paymentCalculationPeriod == null){
				this.paymentCalculationPeriod = new ArrayList<>();
			}
			for (PaymentCalculationPeriod toAdd : paymentCalculationPeriods) {
				this.paymentCalculationPeriod.add(toAdd.toBuilder());
			}
			return this;
		}
		public CashflowRepresentationBuilder addPaymentCalculationPeriodBuilder(PaymentCalculationPeriod.PaymentCalculationPeriodBuilder paymentCalculationPeriod) {
			if(this.paymentCalculationPeriod == null){
				this.paymentCalculationPeriod = new ArrayList<>();
			}
			this.paymentCalculationPeriod.add(paymentCalculationPeriod);
			return this;
		}
		
		public CashflowRepresentationBuilder clearPaymentCalculationPeriod() {
			this.paymentCalculationPeriod = null;
			return this;
		}
		public CashflowRepresentationBuilder addPrincipalExchange(PrincipalExchange principalExchange) {
			if(this.principalExchange == null){
				this.principalExchange = new ArrayList<>();
			}
			this.principalExchange.add(principalExchange.toBuilder());
			return this;
		}
		public CashflowRepresentationBuilder addPrincipalExchange(PrincipalExchange principalExchange, int _idx) {
			if(this.principalExchange == null){
				this.principalExchange = new ArrayList<>();
			}
			getIndex(this.principalExchange, _idx, () -> principalExchange.toBuilder());
			this.principalExchange.set(_idx, principalExchange.toBuilder());
			return this;
		}
		public CashflowRepresentationBuilder addPrincipalExchange(List<PrincipalExchange> principalExchanges) {
			if(this.principalExchange == null){
				this.principalExchange = new ArrayList<>();
			}
			for (PrincipalExchange toAdd : principalExchanges) {
				this.principalExchange.add(toAdd.toBuilder());
			}
			return this;
		}
		public CashflowRepresentationBuilder addPrincipalExchangeBuilder(PrincipalExchange.PrincipalExchangeBuilder principalExchange) {
			if(this.principalExchange == null){
				this.principalExchange = new ArrayList<>();
			}
			this.principalExchange.add(principalExchange);
			return this;
		}
		
		public CashflowRepresentationBuilder clearPrincipalExchange() {
			this.principalExchange = null;
			return this;
		}
		
		public CashflowRepresentation build() {
			return new CashflowRepresentation(this);
		}
	
		@Override
		public CashflowRepresentationBuilder prune() {
			if (paymentCalculationPeriod!=null) paymentCalculationPeriod = paymentCalculationPeriod.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (principalExchange!=null) principalExchange = principalExchange.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getCashflowsMatchParameters()!=null) return true;
			if (getPaymentCalculationPeriod()!=null && getPaymentCalculationPeriod().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getPrincipalExchange()!=null && getPrincipalExchange().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("cashflowsMatchParameters"), Boolean.class, cashflowsMatchParameters, this);
			
			processRosetta(path.newSubPath("paymentCalculationPeriod"), processor, PaymentCalculationPeriod.class, paymentCalculationPeriod);
			processRosetta(path.newSubPath("principalExchange"), processor, PrincipalExchange.class, principalExchange);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			CashflowRepresentationBuilder _that = (CashflowRepresentationBuilder) o;
		
			if (cashflowsMatchParameters != null ? !cashflowsMatchParameters.equals(_that.cashflowsMatchParameters) : _that.cashflowsMatchParameters != null) return false;
			if (!ListEquals.listEquals(paymentCalculationPeriod, _that.paymentCalculationPeriod)) return false;
			if (!ListEquals.listEquals(principalExchange, _that.principalExchange)) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (cashflowsMatchParameters != null ? cashflowsMatchParameters.hashCode() : 0);
			_result = 31 * _result + (paymentCalculationPeriod != null ? paymentCalculationPeriod.hashCode() : 0);
			_result = 31 * _result + (principalExchange != null ? principalExchange.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "CashflowRepresentationBuilder {" +
				"cashflowsMatchParameters=" + this.cashflowsMatchParameters + ", " +
				"paymentCalculationPeriod=" + this.paymentCalculationPeriod + ", " +
				"principalExchange=" + this.principalExchange +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("cashflowsMatchParameters"), Boolean.class, cashflowsMatchParameters, this);
		
		processRosetta(path.newSubPath("paymentCalculationPeriod"), processor, PaymentCalculationPeriod.class, paymentCalculationPeriod);
		processRosetta(path.newSubPath("principalExchange"), processor, PrincipalExchange.class, principalExchange);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		CashflowRepresentation _that = (CashflowRepresentation) o;
	
		if (cashflowsMatchParameters != null ? !cashflowsMatchParameters.equals(_that.cashflowsMatchParameters) : _that.cashflowsMatchParameters != null) return false;
		if (!ListEquals.listEquals(paymentCalculationPeriod, _that.paymentCalculationPeriod)) return false;
		if (!ListEquals.listEquals(principalExchange, _that.principalExchange)) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (cashflowsMatchParameters != null ? cashflowsMatchParameters.hashCode() : 0);
		_result = 31 * _result + (paymentCalculationPeriod != null ? paymentCalculationPeriod.hashCode() : 0);
		_result = 31 * _result + (principalExchange != null ? principalExchange.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "CashflowRepresentation {" +
			"cashflowsMatchParameters=" + this.cashflowsMatchParameters + ", " +
			"paymentCalculationPeriod=" + this.paymentCalculationPeriod + ", " +
			"principalExchange=" + this.principalExchange +
		'}';
	}
}
