package org.isda.cdm;

import java.lang.String;
import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.PostingObligationsElection;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.ChargorPostingObligationsMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;

/**
 * A class to specify the chargor(s) collateral posting obligations as specified under the terms of the English Law ISDA 2016 Credit Support Annex for Initial Margin, paragraph 13, General Principles, (ii).
 * 
 * @version 2.5.4
 */
@RosettaClass
public class ChargorPostingObligations extends RosettaModelObject {
	private final String chargor;
	private final List<PostingObligationsElection> partyElection;
	private static ChargorPostingObligationsMeta metaData = new ChargorPostingObligationsMeta();
	
	ChargorPostingObligations(ChargorPostingObligationsBuilder builder) {
		this.chargor = builder.getChargor();
		this.partyElection = ofNullable(builder.getPartyElection()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
	}
	
	@RosettaSynonym(value="chargor", source="ISDA_Create_1_0")
	public final String getChargor() {
		return chargor;
	}
	
	/**
	 * The chargor party(ies) to which the posting obligations apply to, which can be either one of the parties to the legal agreement, or both of those.
	 */
	public final List<PostingObligationsElection> getPartyElection() {
		return partyElection;
	}
	
	
	@Override
	public RosettaMetaData<? extends ChargorPostingObligations> metaData() {
		return metaData;
	} 
	
	public ChargorPostingObligationsBuilder toBuilder() {
		ChargorPostingObligationsBuilder builder = new ChargorPostingObligationsBuilder();
		ofNullable(getChargor()).ifPresent(builder::setChargor);
		ofNullable(getPartyElection()).ifPresent(partyElection -> partyElection.forEach(builder::addPartyElection));
		return builder;
	}

	public static ChargorPostingObligationsBuilder builder() {
		return new ChargorPostingObligationsBuilder();
	}

	public static  class ChargorPostingObligationsBuilder extends RosettaModelObjectBuilder {
	
		protected String chargor;
		protected List<PostingObligationsElection.PostingObligationsElectionBuilder> partyElection;
	
		public ChargorPostingObligationsBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends ChargorPostingObligations> metaData() {
			return metaData;
		} 
	
		
		public String getChargor() {
			return chargor;
		}
		
		
		public List<PostingObligationsElection.PostingObligationsElectionBuilder> getPartyElection() {
			return partyElection;
		}
		
		public PostingObligationsElection.PostingObligationsElectionBuilder getOrCreatePartyElection(int index) {
			if (partyElection==null) {
				this.partyElection = new ArrayList<>();
			}
			return getIndex(partyElection, index, ()->new PostingObligationsElection.PostingObligationsElectionBuilder());
		}
		
	
		public ChargorPostingObligationsBuilder setChargor(String chargor) {
			this.chargor = chargor;
			return this;
		}
		public ChargorPostingObligationsBuilder addPartyElection(PostingObligationsElection partyElection) {
			if(this.partyElection == null){
				this.partyElection = new ArrayList<>();
			}
			this.partyElection.add(partyElection.toBuilder());
			return this;
		}
		public ChargorPostingObligationsBuilder addPartyElection(PostingObligationsElection partyElection, int _idx) {
			if(this.partyElection == null){
				this.partyElection = new ArrayList<>();
			}
			getIndex(this.partyElection, _idx, () -> partyElection.toBuilder());
			this.partyElection.set(_idx, partyElection.toBuilder());
			return this;
		}
		public ChargorPostingObligationsBuilder addPartyElection(List<PostingObligationsElection> partyElections) {
			if(this.partyElection == null){
				this.partyElection = new ArrayList<>();
			}
			for (PostingObligationsElection toAdd : partyElections) {
				this.partyElection.add(toAdd.toBuilder());
			}
			return this;
		}
		public ChargorPostingObligationsBuilder addPartyElectionBuilder(PostingObligationsElection.PostingObligationsElectionBuilder partyElection) {
			if(this.partyElection == null){
				this.partyElection = new ArrayList<>();
			}
			this.partyElection.add(partyElection);
			return this;
		}
		
		public ChargorPostingObligationsBuilder clearPartyElection() {
			this.partyElection = null;
			return this;
		}
		
		public ChargorPostingObligations build() {
			return new ChargorPostingObligations(this);
		}
	
		@Override
		public ChargorPostingObligationsBuilder prune() {
			if (partyElection!=null) partyElection = partyElection.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getChargor()!=null) return true;
			if (getPartyElection()!=null && getPartyElection().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("chargor"), String.class, chargor, this);
			
			processRosetta(path.newSubPath("partyElection"), processor, PostingObligationsElection.class, partyElection);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			ChargorPostingObligationsBuilder _that = (ChargorPostingObligationsBuilder) o;
		
			if (chargor != null ? !chargor.equals(_that.chargor) : _that.chargor != null) return false;
			if (!ListEquals.listEquals(partyElection, _that.partyElection)) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (chargor != null ? chargor.hashCode() : 0);
			_result = 31 * _result + (partyElection != null ? partyElection.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "ChargorPostingObligationsBuilder {" +
				"chargor=" + this.chargor + ", " +
				"partyElection=" + this.partyElection +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("chargor"), String.class, chargor, this);
		
		processRosetta(path.newSubPath("partyElection"), processor, PostingObligationsElection.class, partyElection);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		ChargorPostingObligations _that = (ChargorPostingObligations) o;
	
		if (chargor != null ? !chargor.equals(_that.chargor) : _that.chargor != null) return false;
		if (!ListEquals.listEquals(partyElection, _that.partyElection)) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (chargor != null ? chargor.hashCode() : 0);
		_result = 31 * _result + (partyElection != null ? partyElection.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "ChargorPostingObligations {" +
			"chargor=" + this.chargor + ", " +
			"partyElection=" + this.partyElection +
		'}';
	}
}
