package org.isda.cdm;

import java.math.BigDecimal;
import org.isda.cdm.CleanPrice;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.CleanOrDirtyPriceMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;
		import java.math.BigDecimal;

/**
 * Class specifying the bond price as either clean or dirty in a bond valuation model.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class CleanOrDirtyPrice extends RosettaModelObject {
	private final CleanPrice cleanPrice;
	private final BigDecimal dirtyPrice;
	private static CleanOrDirtyPriceMeta metaData = new CleanOrDirtyPriceMeta();
	
	CleanOrDirtyPrice(CleanOrDirtyPriceBuilder builder) {
		this.cleanPrice = ofNullable(builder.getCleanPrice()).map(f->f.build()).orElse(null);
		this.dirtyPrice = builder.getDirtyPrice();
	}
	
	/**
	 * The clean price and accruals presented separately.
	 */
	public final CleanPrice getCleanPrice() {
		return cleanPrice;
	}
	
	/**
	 * The dirty price presented as a single number.
	 */
	@RosettaSynonym(value="dirtyPrice", source="FpML_5_10")
	public final BigDecimal getDirtyPrice() {
		return dirtyPrice;
	}
	
	
	@Override
	public RosettaMetaData<? extends CleanOrDirtyPrice> metaData() {
		return metaData;
	} 
	
	public CleanOrDirtyPriceBuilder toBuilder() {
		CleanOrDirtyPriceBuilder builder = new CleanOrDirtyPriceBuilder();
		ofNullable(getCleanPrice()).ifPresent(builder::setCleanPrice);
		ofNullable(getDirtyPrice()).ifPresent(builder::setDirtyPrice);
		return builder;
	}

	public static CleanOrDirtyPriceBuilder builder() {
		return new CleanOrDirtyPriceBuilder();
	}

	public static  class CleanOrDirtyPriceBuilder extends RosettaModelObjectBuilder {
	
		protected CleanPrice.CleanPriceBuilder cleanPrice;
		protected BigDecimal dirtyPrice;
	
		public CleanOrDirtyPriceBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends CleanOrDirtyPrice> metaData() {
			return metaData;
		} 
	
		
		public CleanPrice.CleanPriceBuilder getCleanPrice() {
			return cleanPrice;
		}
		
		public CleanPrice.CleanPriceBuilder getOrCreateCleanPrice() {
			if (cleanPrice!=null) {
				return cleanPrice;
			}
			else return cleanPrice = new CleanPrice.CleanPriceBuilder();
		}
		
		
		public BigDecimal getDirtyPrice() {
			return dirtyPrice;
		}
		
	
		public CleanOrDirtyPriceBuilder setCleanPrice(CleanPrice cleanPrice) {
			this.cleanPrice = cleanPrice.toBuilder();
			return this;
		}
		public CleanOrDirtyPriceBuilder setCleanPriceBuilder(CleanPrice.CleanPriceBuilder cleanPrice) {
			this.cleanPrice = cleanPrice;
			return this;
		}
		public CleanOrDirtyPriceBuilder setDirtyPrice(BigDecimal dirtyPrice) {
			this.dirtyPrice = dirtyPrice;
			return this;
		}
		
		public CleanOrDirtyPrice build() {
			return new CleanOrDirtyPrice(this);
		}
	
		@Override
		public CleanOrDirtyPriceBuilder prune() {
			if (cleanPrice!=null && !cleanPrice.prune().hasData()) cleanPrice = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getCleanPrice()!=null && getCleanPrice().hasData()) return true;
			if (getDirtyPrice()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("dirtyPrice"), BigDecimal.class, dirtyPrice, this);
			
			processRosetta(path.newSubPath("cleanPrice"), processor, CleanPrice.class, cleanPrice);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			CleanOrDirtyPriceBuilder _that = (CleanOrDirtyPriceBuilder) o;
		
			if (cleanPrice != null ? !cleanPrice.equals(_that.cleanPrice) : _that.cleanPrice != null) return false;
			if (dirtyPrice != null ? !dirtyPrice.equals(_that.dirtyPrice) : _that.dirtyPrice != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (cleanPrice != null ? cleanPrice.hashCode() : 0);
			_result = 31 * _result + (dirtyPrice != null ? dirtyPrice.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "CleanOrDirtyPriceBuilder {" +
				"cleanPrice=" + this.cleanPrice + ", " +
				"dirtyPrice=" + this.dirtyPrice +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("dirtyPrice"), BigDecimal.class, dirtyPrice, this);
		
		processRosetta(path.newSubPath("cleanPrice"), processor, CleanPrice.class, cleanPrice);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		CleanOrDirtyPrice _that = (CleanOrDirtyPrice) o;
	
		if (cleanPrice != null ? !cleanPrice.equals(_that.cleanPrice) : _that.cleanPrice != null) return false;
		if (dirtyPrice != null ? !dirtyPrice.equals(_that.dirtyPrice) : _that.dirtyPrice != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (cleanPrice != null ? cleanPrice.hashCode() : 0);
		_result = 31 * _result + (dirtyPrice != null ? dirtyPrice.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "CleanOrDirtyPrice {" +
			"cleanPrice=" + this.cleanPrice + ", " +
			"dirtyPrice=" + this.dirtyPrice +
		'}';
	}
}
