package org.isda.cdm;

import java.lang.String;
import java.math.BigDecimal;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.CleanPriceMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.math.BigDecimal;
		import static java.util.Optional.ofNullable;

/**
 * Class to specify the clean price of a bond in a bond valuation model, with accruals presented separately, and modelled onto the cleanPrice model in BonPriceAndYield.model in FpML.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class CleanPrice extends RosettaModelObject {
	private final BigDecimal accruals;
	private final BigDecimal cleanPrice;
	private final String dirtyPrice;
	private static CleanPriceMeta metaData = new CleanPriceMeta();
	
	CleanPrice(CleanPriceBuilder builder) {
		this.accruals = builder.getAccruals();
		this.cleanPrice = builder.getCleanPrice();
		this.dirtyPrice = builder.getDirtyPrice();
	}
	
	/**
	 * The accruals as a number.
	 */
	@RosettaSynonym(value="accruals", source="FpML_5_10")
	public final BigDecimal getAccruals() {
		return accruals;
	}
	
	/**
	 * The clean price as a number.
	 */
	@RosettaSynonym(value="cleanPrice", source="FpML_5_10")
	public final BigDecimal getCleanPrice() {
		return cleanPrice;
	}
	
	/**
	 * Placeholder for a calculation of dirtyPrice based on cleanPrice and accruals.
	 */
	public final String getDirtyPrice() {
		return dirtyPrice;
	}
	
	
	@Override
	public RosettaMetaData<? extends CleanPrice> metaData() {
		return metaData;
	} 
	
	public CleanPriceBuilder toBuilder() {
		CleanPriceBuilder builder = new CleanPriceBuilder();
		ofNullable(getAccruals()).ifPresent(builder::setAccruals);
		ofNullable(getCleanPrice()).ifPresent(builder::setCleanPrice);
		ofNullable(getDirtyPrice()).ifPresent(builder::setDirtyPrice);
		return builder;
	}

	public static CleanPriceBuilder builder() {
		return new CleanPriceBuilder();
	}

	public static  class CleanPriceBuilder extends RosettaModelObjectBuilder {
	
		protected BigDecimal accruals;
		protected BigDecimal cleanPrice;
		protected String dirtyPrice;
	
		public CleanPriceBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends CleanPrice> metaData() {
			return metaData;
		} 
	
		
		public BigDecimal getAccruals() {
			return accruals;
		}
		
		
		public BigDecimal getCleanPrice() {
			return cleanPrice;
		}
		
		
		public String getDirtyPrice() {
			return dirtyPrice;
		}
		
	
		public CleanPriceBuilder setAccruals(BigDecimal accruals) {
			this.accruals = accruals;
			return this;
		}
		public CleanPriceBuilder setCleanPrice(BigDecimal cleanPrice) {
			this.cleanPrice = cleanPrice;
			return this;
		}
		public CleanPriceBuilder setDirtyPrice(String dirtyPrice) {
			this.dirtyPrice = dirtyPrice;
			return this;
		}
		
		public CleanPrice build() {
			return new CleanPrice(this);
		}
	
		@Override
		public CleanPriceBuilder prune() {
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getAccruals()!=null) return true;
			if (getCleanPrice()!=null) return true;
			if (getDirtyPrice()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("accruals"), BigDecimal.class, accruals, this);
			processor.processBasic(path.newSubPath("cleanPrice"), BigDecimal.class, cleanPrice, this);
			processor.processBasic(path.newSubPath("dirtyPrice"), String.class, dirtyPrice, this);
			
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			CleanPriceBuilder _that = (CleanPriceBuilder) o;
		
			if (accruals != null ? !accruals.equals(_that.accruals) : _that.accruals != null) return false;
			if (cleanPrice != null ? !cleanPrice.equals(_that.cleanPrice) : _that.cleanPrice != null) return false;
			if (dirtyPrice != null ? !dirtyPrice.equals(_that.dirtyPrice) : _that.dirtyPrice != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (accruals != null ? accruals.hashCode() : 0);
			_result = 31 * _result + (cleanPrice != null ? cleanPrice.hashCode() : 0);
			_result = 31 * _result + (dirtyPrice != null ? dirtyPrice.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "CleanPriceBuilder {" +
				"accruals=" + this.accruals + ", " +
				"cleanPrice=" + this.cleanPrice + ", " +
				"dirtyPrice=" + this.dirtyPrice +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("accruals"), BigDecimal.class, accruals, this);
		processor.processBasic(path.newSubPath("cleanPrice"), BigDecimal.class, cleanPrice, this);
		processor.processBasic(path.newSubPath("dirtyPrice"), String.class, dirtyPrice, this);
		
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		CleanPrice _that = (CleanPrice) o;
	
		if (accruals != null ? !accruals.equals(_that.accruals) : _that.accruals != null) return false;
		if (cleanPrice != null ? !cleanPrice.equals(_that.cleanPrice) : _that.cleanPrice != null) return false;
		if (dirtyPrice != null ? !dirtyPrice.equals(_that.dirtyPrice) : _that.dirtyPrice != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (accruals != null ? accruals.hashCode() : 0);
		_result = 31 * _result + (cleanPrice != null ? cleanPrice.hashCode() : 0);
		_result = 31 * _result + (dirtyPrice != null ? dirtyPrice.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "CleanPrice {" +
			"accruals=" + this.accruals + ", " +
			"cleanPrice=" + this.cleanPrice + ", " +
			"dirtyPrice=" + this.dirtyPrice +
		'}';
	}
}
