package org.isda.cdm;

import com.rosetta.model.lib.records.Date;
import org.isda.cdm.ClosedStateEnum;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.ClosedStateMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import com.rosetta.model.lib.records.Date;
		import static java.util.Optional.ofNullable;

/**
 * A class to qualify the closed state of an execution or a contract through the combination or a state (e.g. terminated, novated) and a set of dates: activity date, effective date and, when relevant, last payment date.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class ClosedState extends RosettaModelObject {
	private final Date activityDate;
	private final Date effectiveDate;
	private final Date lastPaymentDate;
	private final ClosedStateEnum state;
	private static ClosedStateMeta metaData = new ClosedStateMeta();
	
	ClosedState(ClosedStateBuilder builder) {
		this.activityDate = builder.getActivityDate();
		this.effectiveDate = builder.getEffectiveDate();
		this.lastPaymentDate = builder.getLastPaymentDate();
		this.state = builder.getState();
	}
	
	/**
	 * The activity date on which the closing state took place, i.e. either the event date of the closing event (e.g. option exercise, contract early termination) or the contractual termination date.
	 */
	@RosettaSynonym(value="activityDate", source="Rosetta_Workbench")
	public final Date getActivityDate() {
		return activityDate;
	}
	
	/**
	 * The date on which the closing event contractually takes effect, when different from the activity date. When an explicit event effective date attribute is associated with the closing event, it will be that date. In the case of a cancellation event, it will be the date on which the cancelled event took place.
	 */
	@RosettaSynonym(value="effectiveDate", source="Rosetta_Workbench")
	public final Date getEffectiveDate() {
		return effectiveDate;
	}
	
	/**
	 * The date associated with the last payment in relation to the artefact (e.g. contract) to which this closed state applies. As an example, in the case of an early termination event, it would be the settlement date of the associated fee, if applicable.
	 */
	@RosettaSynonym(value="lastPaymentDate", source="Rosetta_Workbench")
	public final Date getLastPaymentDate() {
		return lastPaymentDate;
	}
	
	/**
	 * The qualification of what gave way to the contract or execution closure, e.g. allocation, termination, ...
	 */
	@RosettaSynonym(value="state", source="Rosetta_Workbench")
	public final ClosedStateEnum getState() {
		return state;
	}
	
	
	@Override
	public RosettaMetaData<? extends ClosedState> metaData() {
		return metaData;
	} 
	
	public ClosedStateBuilder toBuilder() {
		ClosedStateBuilder builder = new ClosedStateBuilder();
		ofNullable(getActivityDate()).ifPresent(builder::setActivityDate);
		ofNullable(getEffectiveDate()).ifPresent(builder::setEffectiveDate);
		ofNullable(getLastPaymentDate()).ifPresent(builder::setLastPaymentDate);
		ofNullable(getState()).ifPresent(builder::setState);
		return builder;
	}

	public static ClosedStateBuilder builder() {
		return new ClosedStateBuilder();
	}

	public static  class ClosedStateBuilder extends RosettaModelObjectBuilder {
	
		protected Date activityDate;
		protected Date effectiveDate;
		protected Date lastPaymentDate;
		protected ClosedStateEnum state;
	
		public ClosedStateBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends ClosedState> metaData() {
			return metaData;
		} 
	
		
		public Date getActivityDate() {
			return activityDate;
		}
		
		
		public Date getEffectiveDate() {
			return effectiveDate;
		}
		
		
		public Date getLastPaymentDate() {
			return lastPaymentDate;
		}
		
		
		public ClosedStateEnum getState() {
			return state;
		}
		
	
		public ClosedStateBuilder setActivityDate(Date activityDate) {
			this.activityDate = activityDate;
			return this;
		}
		public ClosedStateBuilder setEffectiveDate(Date effectiveDate) {
			this.effectiveDate = effectiveDate;
			return this;
		}
		public ClosedStateBuilder setLastPaymentDate(Date lastPaymentDate) {
			this.lastPaymentDate = lastPaymentDate;
			return this;
		}
		public ClosedStateBuilder setState(ClosedStateEnum state) {
			this.state = state;
			return this;
		}
		
		public ClosedState build() {
			return new ClosedState(this);
		}
	
		@Override
		public ClosedStateBuilder prune() {
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getActivityDate()!=null) return true;
			if (getEffectiveDate()!=null) return true;
			if (getLastPaymentDate()!=null) return true;
			if (getState()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("activityDate"), Date.class, activityDate, this);
			processor.processBasic(path.newSubPath("effectiveDate"), Date.class, effectiveDate, this);
			processor.processBasic(path.newSubPath("lastPaymentDate"), Date.class, lastPaymentDate, this);
			processor.processBasic(path.newSubPath("state"), ClosedStateEnum.class, state, this);
			
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			ClosedStateBuilder _that = (ClosedStateBuilder) o;
		
			if (activityDate != null ? !activityDate.equals(_that.activityDate) : _that.activityDate != null) return false;
			if (effectiveDate != null ? !effectiveDate.equals(_that.effectiveDate) : _that.effectiveDate != null) return false;
			if (lastPaymentDate != null ? !lastPaymentDate.equals(_that.lastPaymentDate) : _that.lastPaymentDate != null) return false;
			if (state != null ? !state.equals(_that.state) : _that.state != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (activityDate != null ? activityDate.hashCode() : 0);
			_result = 31 * _result + (effectiveDate != null ? effectiveDate.hashCode() : 0);
			_result = 31 * _result + (lastPaymentDate != null ? lastPaymentDate.hashCode() : 0);
			_result = 31 * _result + (state != null ? state.getClass().getName().hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "ClosedStateBuilder {" +
				"activityDate=" + this.activityDate + ", " +
				"effectiveDate=" + this.effectiveDate + ", " +
				"lastPaymentDate=" + this.lastPaymentDate + ", " +
				"state=" + this.state +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("activityDate"), Date.class, activityDate, this);
		processor.processBasic(path.newSubPath("effectiveDate"), Date.class, effectiveDate, this);
		processor.processBasic(path.newSubPath("lastPaymentDate"), Date.class, lastPaymentDate, this);
		processor.processBasic(path.newSubPath("state"), ClosedStateEnum.class, state, this);
		
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		ClosedState _that = (ClosedState) o;
	
		if (activityDate != null ? !activityDate.equals(_that.activityDate) : _that.activityDate != null) return false;
		if (effectiveDate != null ? !effectiveDate.equals(_that.effectiveDate) : _that.effectiveDate != null) return false;
		if (lastPaymentDate != null ? !lastPaymentDate.equals(_that.lastPaymentDate) : _that.lastPaymentDate != null) return false;
		if (state != null ? !state.equals(_that.state) : _that.state != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (activityDate != null ? activityDate.hashCode() : 0);
		_result = 31 * _result + (effectiveDate != null ? effectiveDate.hashCode() : 0);
		_result = 31 * _result + (lastPaymentDate != null ? lastPaymentDate.hashCode() : 0);
		_result = 31 * _result + (state != null ? state.getClass().getName().hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "ClosedState {" +
			"activityDate=" + this.activityDate + ", " +
			"effectiveDate=" + this.effectiveDate + ", " +
			"lastPaymentDate=" + this.lastPaymentDate + ", " +
			"state=" + this.state +
		'}';
	}
}
