package org.isda.cdm;

import org.isda.cdm.IndependentAmount;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.CollateralMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A type for defining the obligations of the counterparty subject to credit support requirements.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="Collateral", source="FpML_5_10")
public class Collateral extends RosettaModelObject {
	private final IndependentAmount independentAmount;
	private static CollateralMeta metaData = new CollateralMeta();
	
	Collateral(CollateralBuilder builder) {
		this.independentAmount = ofNullable(builder.getIndependentAmount()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * Independent Amount is an amount that usually less creditworthy counterparties are asked to provide. It can either be a fixed amount or a percentage of the Transaction&#39;s value. The Independent Amount can be: (i) transferred before any trading between the parties occurs (as a deposit at a third party&#39;s account or with the counterparty) or (ii) callable after trading has occurred (typically because a downgrade has occurred). In situation (i), the Independent Amount is not included in the calculation of Exposure, but in situation (ii), it is included in the calculation of Exposure. Thus, for situation (ii), the Independent Amount may be transferred along with any collateral call. Independent Amount is a defined term in the ISDA Credit Support Annex. (&#39;with respect to a party, the amount specified as such for that party in Paragraph 13; if no amount is specified, zero&#39;).
	 */
	@RosettaSynonym(value="independentAmount", source="FpML_5_10")
	@RosettaSynonym(value="independentAmount", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="independentAmount", source="DTCC_11_0")
	@RosettaSynonym(value="independentAmount", source="DTCC_9_0")
	@RosettaSynonym(value="independentAmount", source="CME_ClearedConfirm_1_17")
	public final IndependentAmount getIndependentAmount() {
		return independentAmount;
	}
	
	
	@Override
	public RosettaMetaData<? extends Collateral> metaData() {
		return metaData;
	} 
	
	public CollateralBuilder toBuilder() {
		CollateralBuilder builder = new CollateralBuilder();
		ofNullable(getIndependentAmount()).ifPresent(builder::setIndependentAmount);
		return builder;
	}

	public static CollateralBuilder builder() {
		return new CollateralBuilder();
	}

	public static  class CollateralBuilder extends RosettaModelObjectBuilder {
	
		protected IndependentAmount.IndependentAmountBuilder independentAmount;
	
		public CollateralBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends Collateral> metaData() {
			return metaData;
		} 
	
		
		public IndependentAmount.IndependentAmountBuilder getIndependentAmount() {
			return independentAmount;
		}
		
		public IndependentAmount.IndependentAmountBuilder getOrCreateIndependentAmount() {
			if (independentAmount!=null) {
				return independentAmount;
			}
			else return independentAmount = new IndependentAmount.IndependentAmountBuilder();
		}
		
	
		public CollateralBuilder setIndependentAmount(IndependentAmount independentAmount) {
			this.independentAmount = independentAmount.toBuilder();
			return this;
		}
		public CollateralBuilder setIndependentAmountBuilder(IndependentAmount.IndependentAmountBuilder independentAmount) {
			this.independentAmount = independentAmount;
			return this;
		}
		
		public Collateral build() {
			return new Collateral(this);
		}
	
		@Override
		public CollateralBuilder prune() {
			if (independentAmount!=null && !independentAmount.prune().hasData()) independentAmount = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getIndependentAmount()!=null && getIndependentAmount().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("independentAmount"), processor, IndependentAmount.class, independentAmount);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			CollateralBuilder _that = (CollateralBuilder) o;
		
			if (independentAmount != null ? !independentAmount.equals(_that.independentAmount) : _that.independentAmount != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (independentAmount != null ? independentAmount.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "CollateralBuilder {" +
				"independentAmount=" + this.independentAmount +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("independentAmount"), processor, IndependentAmount.class, independentAmount);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		Collateral _that = (Collateral) o;
	
		if (independentAmount != null ? !independentAmount.equals(_that.independentAmount) : _that.independentAmount != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (independentAmount != null ? independentAmount.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "Collateral {" +
			"independentAmount=" + this.independentAmount +
		'}';
	}
}
