package org.isda.cdm;

import java.lang.String;
import org.isda.cdm.metafields.ReferenceWithMetaParty;
import org.isda.cdm.metafields.ReferenceWithMetaParty.ReferenceWithMetaPartyBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.CollateralManagementAgreementElectionMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.stream.Collectors;
		import static java.util.Optional.ofNullable;

/**
 * A class to specify the Collateral Management Agreement election by each party as the Obligee. ISDA 2016 Credit Support Annex for Initial Margin, paragraph 13, General Principles, (b(i): Collateral Management Agreement.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class CollateralManagementAgreementElection extends RosettaModelObject {
	private final String collateralManagementAgreement;
	private final ReferenceWithMetaParty party;
	private static CollateralManagementAgreementElectionMeta metaData = new CollateralManagementAgreementElectionMeta();
	
	CollateralManagementAgreementElection(CollateralManagementAgreementElectionBuilder builder) {
		this.collateralManagementAgreement = builder.getCollateralManagementAgreement();
		this.party = ofNullable(builder.getParty()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The designated Collateral Management Agreement with respect to the elective party as the Obligee.
	 */
	@RosettaSynonym(value="partyA_specify,partyB_specify", source="ISDA_Create_1_0")
	public final String getCollateralManagementAgreement() {
		return collateralManagementAgreement;
	}
	
	/**
	 * The elective party.
	 */
	public final ReferenceWithMetaParty getParty() {
		return party;
	}
	
	
	@Override
	public RosettaMetaData<? extends CollateralManagementAgreementElection> metaData() {
		return metaData;
	} 
	
	public CollateralManagementAgreementElectionBuilder toBuilder() {
		CollateralManagementAgreementElectionBuilder builder = new CollateralManagementAgreementElectionBuilder();
		ofNullable(getCollateralManagementAgreement()).ifPresent(builder::setCollateralManagementAgreement);
		ofNullable(getParty()).ifPresent(builder::setParty);
		return builder;
	}

	public static CollateralManagementAgreementElectionBuilder builder() {
		return new CollateralManagementAgreementElectionBuilder();
	}

	public static  class CollateralManagementAgreementElectionBuilder extends RosettaModelObjectBuilder {
	
		protected String collateralManagementAgreement;
		protected ReferenceWithMetaPartyBuilder party;
	
		public CollateralManagementAgreementElectionBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends CollateralManagementAgreementElection> metaData() {
			return metaData;
		} 
	
		
		public String getCollateralManagementAgreement() {
			return collateralManagementAgreement;
		}
		
		
		public ReferenceWithMetaPartyBuilder getParty() {
			return party;
		}
		
		public ReferenceWithMetaPartyBuilder getOrCreateParty() {
			if (party!=null) {
				return party;
			}
			else return party = new ReferenceWithMetaPartyBuilder();
		}
		
	
		public CollateralManagementAgreementElectionBuilder setCollateralManagementAgreement(String collateralManagementAgreement) {
			this.collateralManagementAgreement = collateralManagementAgreement;
			return this;
		}
		public CollateralManagementAgreementElectionBuilder setParty(ReferenceWithMetaParty party) {
			this.party = party.toBuilder();
			return this;
		}
		public CollateralManagementAgreementElectionBuilder setPartyBuilder(ReferenceWithMetaPartyBuilder party) {
			this.party = party;
			return this;
		}
		public CollateralManagementAgreementElectionBuilder setPartyRef(Party.PartyBuilder party) {
			return setParty(ReferenceWithMetaParty.builder().setValueBuilder(party).build());
		}
		public CollateralManagementAgreementElectionBuilder setPartyRef(Party party) {
			return setPartyRef(Party.builder());
		}
		
		public CollateralManagementAgreementElection build() {
			return new CollateralManagementAgreementElection(this);
		}
	
		@Override
		public CollateralManagementAgreementElectionBuilder prune() {
			if (party!=null && !party.prune().hasData()) party = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getCollateralManagementAgreement()!=null) return true;
			if (getParty()!=null && getParty().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("collateralManagementAgreement"), String.class, collateralManagementAgreement, this);
			
			processRosetta(path.newSubPath("party"), processor, ReferenceWithMetaParty.class, party);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			CollateralManagementAgreementElectionBuilder _that = (CollateralManagementAgreementElectionBuilder) o;
		
			if (collateralManagementAgreement != null ? !collateralManagementAgreement.equals(_that.collateralManagementAgreement) : _that.collateralManagementAgreement != null) return false;
			if (party != null ? !party.equals(_that.party) : _that.party != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (collateralManagementAgreement != null ? collateralManagementAgreement.hashCode() : 0);
			_result = 31 * _result + (party != null ? party.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "CollateralManagementAgreementElectionBuilder {" +
				"collateralManagementAgreement=" + this.collateralManagementAgreement + ", " +
				"party=" + this.party +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("collateralManagementAgreement"), String.class, collateralManagementAgreement, this);
		
		processRosetta(path.newSubPath("party"), processor, ReferenceWithMetaParty.class, party);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		CollateralManagementAgreementElection _that = (CollateralManagementAgreementElection) o;
	
		if (collateralManagementAgreement != null ? !collateralManagementAgreement.equals(_that.collateralManagementAgreement) : _that.collateralManagementAgreement != null) return false;
		if (party != null ? !party.equals(_that.party) : _that.party != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (collateralManagementAgreement != null ? collateralManagementAgreement.hashCode() : 0);
		_result = 31 * _result + (party != null ? party.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "CollateralManagementAgreementElection {" +
			"collateralManagementAgreement=" + this.collateralManagementAgreement + ", " +
			"party=" + this.party +
		'}';
	}
}
