package org.isda.cdm;

import java.lang.Boolean;
import org.isda.cdm.CollateralManagementArrangementElection;
import org.isda.cdm.CustodianEvent;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.CollateralManagementArrangementMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class to specify the Collateral Management Arrangement elections in accordance with the ISDA Japanese Law CSA. ISDA 2016 Japanese Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (m): Collateral Management Arrangement.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class CollateralManagementArrangement extends RosettaModelObject {
	private final CustodianEvent collateralManagerEvent;
	private final Boolean hasCollateralManagementLanguage;
	private final Boolean isCreditSupportDocument;
	private final CollateralManagementArrangementElection partyElection;
	private static CollateralManagementArrangementMeta metaData = new CollateralManagementArrangementMeta();
	
	CollateralManagementArrangement(CollateralManagementArrangementBuilder builder) {
		this.collateralManagerEvent = ofNullable(builder.getCollateralManagerEvent()).map(f->f.build()).orElse(null);
		this.hasCollateralManagementLanguage = builder.getHasCollateralManagementLanguage();
		this.isCreditSupportDocument = builder.getIsCreditSupportDocument();
		this.partyElection = ofNullable(builder.getPartyElection()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The Collateral Management Event applicability and end-date terms. ISDA 2016 Japanese Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (m)(ii): Collateral Manager Event.
	 */
	public final CustodianEvent getCollateralManagerEvent() {
		return collateralManagerEvent;
	}
	
	/**
	 * Collateral Management Agreement language is specified when True, not specified when False.
	 */
	@RosettaSynonym(value="", source="ISDA_Create_1_0")
	public final Boolean getHasCollateralManagementLanguage() {
		return hasCollateralManagementLanguage;
	}
	
	/**
	 * Unless specified as True, the Collateral Management Agreement is not a Credit Support Document under the agreement with respect to a party. ISDA 2016 Japanese Law Credit Support Annex for Initial Margin, paragraph 6 (e): The Collateral Management Agreement as a Credit Support Document.
	 */
	@RosettaSynonym(value="", source="ISDA_Create_1_0")
	public final Boolean getIsCreditSupportDocument() {
		return isCreditSupportDocument;
	}
	
	/**
	 * The party-specific election with respect to the Custodian and Segregated Account details and the Custodian Risk provision.
	 */
	public final CollateralManagementArrangementElection getPartyElection() {
		return partyElection;
	}
	
	
	@Override
	public RosettaMetaData<? extends CollateralManagementArrangement> metaData() {
		return metaData;
	} 
	
	public CollateralManagementArrangementBuilder toBuilder() {
		CollateralManagementArrangementBuilder builder = new CollateralManagementArrangementBuilder();
		ofNullable(getCollateralManagerEvent()).ifPresent(builder::setCollateralManagerEvent);
		ofNullable(getHasCollateralManagementLanguage()).ifPresent(builder::setHasCollateralManagementLanguage);
		ofNullable(getIsCreditSupportDocument()).ifPresent(builder::setIsCreditSupportDocument);
		ofNullable(getPartyElection()).ifPresent(builder::setPartyElection);
		return builder;
	}

	public static CollateralManagementArrangementBuilder builder() {
		return new CollateralManagementArrangementBuilder();
	}

	public static  class CollateralManagementArrangementBuilder extends RosettaModelObjectBuilder {
	
		protected CustodianEvent.CustodianEventBuilder collateralManagerEvent;
		protected Boolean hasCollateralManagementLanguage;
		protected Boolean isCreditSupportDocument;
		protected CollateralManagementArrangementElection.CollateralManagementArrangementElectionBuilder partyElection;
	
		public CollateralManagementArrangementBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends CollateralManagementArrangement> metaData() {
			return metaData;
		} 
	
		
		public CustodianEvent.CustodianEventBuilder getCollateralManagerEvent() {
			return collateralManagerEvent;
		}
		
		public CustodianEvent.CustodianEventBuilder getOrCreateCollateralManagerEvent() {
			if (collateralManagerEvent!=null) {
				return collateralManagerEvent;
			}
			else return collateralManagerEvent = new CustodianEvent.CustodianEventBuilder();
		}
		
		
		public Boolean getHasCollateralManagementLanguage() {
			return hasCollateralManagementLanguage;
		}
		
		
		public Boolean getIsCreditSupportDocument() {
			return isCreditSupportDocument;
		}
		
		
		public CollateralManagementArrangementElection.CollateralManagementArrangementElectionBuilder getPartyElection() {
			return partyElection;
		}
		
		public CollateralManagementArrangementElection.CollateralManagementArrangementElectionBuilder getOrCreatePartyElection() {
			if (partyElection!=null) {
				return partyElection;
			}
			else return partyElection = new CollateralManagementArrangementElection.CollateralManagementArrangementElectionBuilder();
		}
		
	
		public CollateralManagementArrangementBuilder setCollateralManagerEvent(CustodianEvent collateralManagerEvent) {
			this.collateralManagerEvent = collateralManagerEvent.toBuilder();
			return this;
		}
		public CollateralManagementArrangementBuilder setCollateralManagerEventBuilder(CustodianEvent.CustodianEventBuilder collateralManagerEvent) {
			this.collateralManagerEvent = collateralManagerEvent;
			return this;
		}
		public CollateralManagementArrangementBuilder setHasCollateralManagementLanguage(Boolean hasCollateralManagementLanguage) {
			this.hasCollateralManagementLanguage = hasCollateralManagementLanguage;
			return this;
		}
		public CollateralManagementArrangementBuilder setIsCreditSupportDocument(Boolean isCreditSupportDocument) {
			this.isCreditSupportDocument = isCreditSupportDocument;
			return this;
		}
		public CollateralManagementArrangementBuilder setPartyElection(CollateralManagementArrangementElection partyElection) {
			this.partyElection = partyElection.toBuilder();
			return this;
		}
		public CollateralManagementArrangementBuilder setPartyElectionBuilder(CollateralManagementArrangementElection.CollateralManagementArrangementElectionBuilder partyElection) {
			this.partyElection = partyElection;
			return this;
		}
		
		public CollateralManagementArrangement build() {
			return new CollateralManagementArrangement(this);
		}
	
		@Override
		public CollateralManagementArrangementBuilder prune() {
			if (collateralManagerEvent!=null && !collateralManagerEvent.prune().hasData()) collateralManagerEvent = null;
			if (partyElection!=null && !partyElection.prune().hasData()) partyElection = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getCollateralManagerEvent()!=null && getCollateralManagerEvent().hasData()) return true;
			if (getHasCollateralManagementLanguage()!=null) return true;
			if (getIsCreditSupportDocument()!=null) return true;
			if (getPartyElection()!=null && getPartyElection().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("hasCollateralManagementLanguage"), Boolean.class, hasCollateralManagementLanguage, this);
			processor.processBasic(path.newSubPath("isCreditSupportDocument"), Boolean.class, isCreditSupportDocument, this);
			
			processRosetta(path.newSubPath("collateralManagerEvent"), processor, CustodianEvent.class, collateralManagerEvent);
			processRosetta(path.newSubPath("partyElection"), processor, CollateralManagementArrangementElection.class, partyElection);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			CollateralManagementArrangementBuilder _that = (CollateralManagementArrangementBuilder) o;
		
			if (collateralManagerEvent != null ? !collateralManagerEvent.equals(_that.collateralManagerEvent) : _that.collateralManagerEvent != null) return false;
			if (hasCollateralManagementLanguage != null ? !hasCollateralManagementLanguage.equals(_that.hasCollateralManagementLanguage) : _that.hasCollateralManagementLanguage != null) return false;
			if (isCreditSupportDocument != null ? !isCreditSupportDocument.equals(_that.isCreditSupportDocument) : _that.isCreditSupportDocument != null) return false;
			if (partyElection != null ? !partyElection.equals(_that.partyElection) : _that.partyElection != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (collateralManagerEvent != null ? collateralManagerEvent.hashCode() : 0);
			_result = 31 * _result + (hasCollateralManagementLanguage != null ? hasCollateralManagementLanguage.hashCode() : 0);
			_result = 31 * _result + (isCreditSupportDocument != null ? isCreditSupportDocument.hashCode() : 0);
			_result = 31 * _result + (partyElection != null ? partyElection.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "CollateralManagementArrangementBuilder {" +
				"collateralManagerEvent=" + this.collateralManagerEvent + ", " +
				"hasCollateralManagementLanguage=" + this.hasCollateralManagementLanguage + ", " +
				"isCreditSupportDocument=" + this.isCreditSupportDocument + ", " +
				"partyElection=" + this.partyElection +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("hasCollateralManagementLanguage"), Boolean.class, hasCollateralManagementLanguage, this);
		processor.processBasic(path.newSubPath("isCreditSupportDocument"), Boolean.class, isCreditSupportDocument, this);
		
		processRosetta(path.newSubPath("collateralManagerEvent"), processor, CustodianEvent.class, collateralManagerEvent);
		processRosetta(path.newSubPath("partyElection"), processor, CollateralManagementArrangementElection.class, partyElection);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		CollateralManagementArrangement _that = (CollateralManagementArrangement) o;
	
		if (collateralManagerEvent != null ? !collateralManagerEvent.equals(_that.collateralManagerEvent) : _that.collateralManagerEvent != null) return false;
		if (hasCollateralManagementLanguage != null ? !hasCollateralManagementLanguage.equals(_that.hasCollateralManagementLanguage) : _that.hasCollateralManagementLanguage != null) return false;
		if (isCreditSupportDocument != null ? !isCreditSupportDocument.equals(_that.isCreditSupportDocument) : _that.isCreditSupportDocument != null) return false;
		if (partyElection != null ? !partyElection.equals(_that.partyElection) : _that.partyElection != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (collateralManagerEvent != null ? collateralManagerEvent.hashCode() : 0);
		_result = 31 * _result + (hasCollateralManagementLanguage != null ? hasCollateralManagementLanguage.hashCode() : 0);
		_result = 31 * _result + (isCreditSupportDocument != null ? isCreditSupportDocument.hashCode() : 0);
		_result = 31 * _result + (partyElection != null ? partyElection.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "CollateralManagementArrangement {" +
			"collateralManagerEvent=" + this.collateralManagerEvent + ", " +
			"hasCollateralManagementLanguage=" + this.hasCollateralManagementLanguage + ", " +
			"isCreditSupportDocument=" + this.isCreditSupportDocument + ", " +
			"partyElection=" + this.partyElection +
		'}';
	}
}
