package org.isda.cdm;

import org.isda.cdm.CustodianRisk;
import org.isda.cdm.metafields.ReferenceWithMetaParty;
import org.isda.cdm.metafields.ReferenceWithMetaParty.ReferenceWithMetaPartyBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.CollateralManagementArrangementElectionMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.stream.Collectors;
		import static java.util.Optional.ofNullable;

/**
 * A class to specify the election to specify the Collateral Management Arrangement.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class CollateralManagementArrangementElection extends RosettaModelObject {
	private final CustodianRisk collateralManagerRisk;
	private final ReferenceWithMetaParty party;
	private static CollateralManagementArrangementElectionMeta metaData = new CollateralManagementArrangementElectionMeta();
	
	CollateralManagementArrangementElection(CollateralManagementArrangementElectionBuilder builder) {
		this.collateralManagerRisk = ofNullable(builder.getCollateralManagerRisk()).map(f->f.build()).orElse(null);
		this.party = ofNullable(builder.getParty()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The qualification of the Collateral Manager Risk. ISDA 2016 Japanese Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (m)(i): Collateral Manager Risk.
	 */
	@RosettaSynonym(value="collateral_manager_risk", source="ISDA_Create_1_0")
	public final CustodianRisk getCollateralManagerRisk() {
		return collateralManagerRisk;
	}
	
	/**
	 * The elective party.
	 */
	public final ReferenceWithMetaParty getParty() {
		return party;
	}
	
	
	@Override
	public RosettaMetaData<? extends CollateralManagementArrangementElection> metaData() {
		return metaData;
	} 
	
	public CollateralManagementArrangementElectionBuilder toBuilder() {
		CollateralManagementArrangementElectionBuilder builder = new CollateralManagementArrangementElectionBuilder();
		ofNullable(getCollateralManagerRisk()).ifPresent(builder::setCollateralManagerRisk);
		ofNullable(getParty()).ifPresent(builder::setParty);
		return builder;
	}

	public static CollateralManagementArrangementElectionBuilder builder() {
		return new CollateralManagementArrangementElectionBuilder();
	}

	public static  class CollateralManagementArrangementElectionBuilder extends RosettaModelObjectBuilder {
	
		protected CustodianRisk.CustodianRiskBuilder collateralManagerRisk;
		protected ReferenceWithMetaPartyBuilder party;
	
		public CollateralManagementArrangementElectionBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends CollateralManagementArrangementElection> metaData() {
			return metaData;
		} 
	
		
		public CustodianRisk.CustodianRiskBuilder getCollateralManagerRisk() {
			return collateralManagerRisk;
		}
		
		public CustodianRisk.CustodianRiskBuilder getOrCreateCollateralManagerRisk() {
			if (collateralManagerRisk!=null) {
				return collateralManagerRisk;
			}
			else return collateralManagerRisk = new CustodianRisk.CustodianRiskBuilder();
		}
		
		
		public ReferenceWithMetaPartyBuilder getParty() {
			return party;
		}
		
		public ReferenceWithMetaPartyBuilder getOrCreateParty() {
			if (party!=null) {
				return party;
			}
			else return party = new ReferenceWithMetaPartyBuilder();
		}
		
	
		public CollateralManagementArrangementElectionBuilder setCollateralManagerRisk(CustodianRisk collateralManagerRisk) {
			this.collateralManagerRisk = collateralManagerRisk.toBuilder();
			return this;
		}
		public CollateralManagementArrangementElectionBuilder setCollateralManagerRiskBuilder(CustodianRisk.CustodianRiskBuilder collateralManagerRisk) {
			this.collateralManagerRisk = collateralManagerRisk;
			return this;
		}
		public CollateralManagementArrangementElectionBuilder setParty(ReferenceWithMetaParty party) {
			this.party = party.toBuilder();
			return this;
		}
		public CollateralManagementArrangementElectionBuilder setPartyBuilder(ReferenceWithMetaPartyBuilder party) {
			this.party = party;
			return this;
		}
		public CollateralManagementArrangementElectionBuilder setPartyRef(Party.PartyBuilder party) {
			return setParty(ReferenceWithMetaParty.builder().setValueBuilder(party).build());
		}
		public CollateralManagementArrangementElectionBuilder setPartyRef(Party party) {
			return setPartyRef(Party.builder());
		}
		
		public CollateralManagementArrangementElection build() {
			return new CollateralManagementArrangementElection(this);
		}
	
		@Override
		public CollateralManagementArrangementElectionBuilder prune() {
			if (collateralManagerRisk!=null && !collateralManagerRisk.prune().hasData()) collateralManagerRisk = null;
			if (party!=null && !party.prune().hasData()) party = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getCollateralManagerRisk()!=null && getCollateralManagerRisk().hasData()) return true;
			if (getParty()!=null && getParty().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("collateralManagerRisk"), processor, CustodianRisk.class, collateralManagerRisk);
			processRosetta(path.newSubPath("party"), processor, ReferenceWithMetaParty.class, party);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			CollateralManagementArrangementElectionBuilder _that = (CollateralManagementArrangementElectionBuilder) o;
		
			if (collateralManagerRisk != null ? !collateralManagerRisk.equals(_that.collateralManagerRisk) : _that.collateralManagerRisk != null) return false;
			if (party != null ? !party.equals(_that.party) : _that.party != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (collateralManagerRisk != null ? collateralManagerRisk.hashCode() : 0);
			_result = 31 * _result + (party != null ? party.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "CollateralManagementArrangementElectionBuilder {" +
				"collateralManagerRisk=" + this.collateralManagerRisk + ", " +
				"party=" + this.party +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("collateralManagerRisk"), processor, CustodianRisk.class, collateralManagerRisk);
		processRosetta(path.newSubPath("party"), processor, ReferenceWithMetaParty.class, party);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		CollateralManagementArrangementElection _that = (CollateralManagementArrangementElection) o;
	
		if (collateralManagerRisk != null ? !collateralManagerRisk.equals(_that.collateralManagerRisk) : _that.collateralManagerRisk != null) return false;
		if (party != null ? !party.equals(_that.party) : _that.party != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (collateralManagerRisk != null ? collateralManagerRisk.hashCode() : 0);
		_result = 31 * _result + (party != null ? party.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "CollateralManagementArrangementElection {" +
			"collateralManagerRisk=" + this.collateralManagerRisk + ", " +
			"party=" + this.party +
		'}';
	}
}
