package org.isda.cdm;

import java.lang.String;
import org.isda.cdm.metafields.ReferenceWithMetaParty;
import org.isda.cdm.metafields.ReferenceWithMetaParty.ReferenceWithMetaPartyBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.CollateralManagerElectionMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.stream.Collectors;
		import static java.util.Optional.ofNullable;

/**
 * A class to specify the Collateral Manager election by each party as the Obligee. ISDA 2016 Credit Support Annex for Initial Margin, paragraph 13, General Principles, (b(ii): Collateral Manager.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class CollateralManagerElection extends RosettaModelObject {
	private final String collateralManager;
	private final ReferenceWithMetaParty party;
	private static CollateralManagerElectionMeta metaData = new CollateralManagerElectionMeta();
	
	CollateralManagerElection(CollateralManagerElectionBuilder builder) {
		this.collateralManager = builder.getCollateralManager();
		this.party = ofNullable(builder.getParty()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The designated Collateral Manager with respect to the elective party as the Obligee.
	 */
	@RosettaSynonym(value="partyA_specify,partyB_specify", source="ISDA_Create_1_0")
	public final String getCollateralManager() {
		return collateralManager;
	}
	
	/**
	 * The elective party.
	 */
	public final ReferenceWithMetaParty getParty() {
		return party;
	}
	
	
	@Override
	public RosettaMetaData<? extends CollateralManagerElection> metaData() {
		return metaData;
	} 
	
	public CollateralManagerElectionBuilder toBuilder() {
		CollateralManagerElectionBuilder builder = new CollateralManagerElectionBuilder();
		ofNullable(getCollateralManager()).ifPresent(builder::setCollateralManager);
		ofNullable(getParty()).ifPresent(builder::setParty);
		return builder;
	}

	public static CollateralManagerElectionBuilder builder() {
		return new CollateralManagerElectionBuilder();
	}

	public static  class CollateralManagerElectionBuilder extends RosettaModelObjectBuilder {
	
		protected String collateralManager;
		protected ReferenceWithMetaPartyBuilder party;
	
		public CollateralManagerElectionBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends CollateralManagerElection> metaData() {
			return metaData;
		} 
	
		
		public String getCollateralManager() {
			return collateralManager;
		}
		
		
		public ReferenceWithMetaPartyBuilder getParty() {
			return party;
		}
		
		public ReferenceWithMetaPartyBuilder getOrCreateParty() {
			if (party!=null) {
				return party;
			}
			else return party = new ReferenceWithMetaPartyBuilder();
		}
		
	
		public CollateralManagerElectionBuilder setCollateralManager(String collateralManager) {
			this.collateralManager = collateralManager;
			return this;
		}
		public CollateralManagerElectionBuilder setParty(ReferenceWithMetaParty party) {
			this.party = party.toBuilder();
			return this;
		}
		public CollateralManagerElectionBuilder setPartyBuilder(ReferenceWithMetaPartyBuilder party) {
			this.party = party;
			return this;
		}
		public CollateralManagerElectionBuilder setPartyRef(Party.PartyBuilder party) {
			return setParty(ReferenceWithMetaParty.builder().setValueBuilder(party).build());
		}
		public CollateralManagerElectionBuilder setPartyRef(Party party) {
			return setPartyRef(Party.builder());
		}
		
		public CollateralManagerElection build() {
			return new CollateralManagerElection(this);
		}
	
		@Override
		public CollateralManagerElectionBuilder prune() {
			if (party!=null && !party.prune().hasData()) party = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getCollateralManager()!=null) return true;
			if (getParty()!=null && getParty().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("collateralManager"), String.class, collateralManager, this);
			
			processRosetta(path.newSubPath("party"), processor, ReferenceWithMetaParty.class, party);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			CollateralManagerElectionBuilder _that = (CollateralManagerElectionBuilder) o;
		
			if (collateralManager != null ? !collateralManager.equals(_that.collateralManager) : _that.collateralManager != null) return false;
			if (party != null ? !party.equals(_that.party) : _that.party != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (collateralManager != null ? collateralManager.hashCode() : 0);
			_result = 31 * _result + (party != null ? party.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "CollateralManagerElectionBuilder {" +
				"collateralManager=" + this.collateralManager + ", " +
				"party=" + this.party +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("collateralManager"), String.class, collateralManager, this);
		
		processRosetta(path.newSubPath("party"), processor, ReferenceWithMetaParty.class, party);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		CollateralManagerElection _that = (CollateralManagerElection) o;
	
		if (collateralManager != null ? !collateralManager.equals(_that.collateralManager) : _that.collateralManager != null) return false;
		if (party != null ? !party.equals(_that.party) : _that.party != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (collateralManager != null ? collateralManager.hashCode() : 0);
		_result = 31 * _result + (party != null ? party.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "CollateralManagerElection {" +
			"collateralManager=" + this.collateralManager + ", " +
			"party=" + this.party +
		'}';
	}
}
