package org.isda.cdm;

import java.math.BigDecimal;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.CollateralRoundingMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.math.BigDecimal;
		import static java.util.Optional.ofNullable;

/**
 * A class to specify the rounding methodology applicable to the Delivery Amount and the Return Amount in terms of nearest integral multiple of Base Currency units. ISDA 2016 English Law Credit Support Deed for Initial Margin, paragraph 13, General Principles, (c)(vi)(C): Rounding. | ISDA 2016 Japanese Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (d)(vi)(C): Rounding. | ISDA 2016 New York Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (c)(vi)(C): Rounding.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class CollateralRounding extends RosettaModelObject {
	private final BigDecimal deliveryAmount;
	private final BigDecimal returnAmount;
	private static CollateralRoundingMeta metaData = new CollateralRoundingMeta();
	
	CollateralRounding(CollateralRoundingBuilder builder) {
		this.deliveryAmount = builder.getDeliveryAmount();
		this.returnAmount = builder.getReturnAmount();
	}
	
	/**
	 * The rounding methodology applicable to the Delivery Amount in terms of nearest integral multiple of Base Currency units.
	 */
	@RosettaSynonym(value="delivery_amount", source="ISDA_Create_1_0")
	public final BigDecimal getDeliveryAmount() {
		return deliveryAmount;
	}
	
	/**
	 * The rounding methodology applicable to the Return Amount in terms of nearest integral multiple of Base Currency units.
	 */
	@RosettaSynonym(value="return_amount", source="ISDA_Create_1_0")
	public final BigDecimal getReturnAmount() {
		return returnAmount;
	}
	
	
	@Override
	public RosettaMetaData<? extends CollateralRounding> metaData() {
		return metaData;
	} 
	
	public CollateralRoundingBuilder toBuilder() {
		CollateralRoundingBuilder builder = new CollateralRoundingBuilder();
		ofNullable(getDeliveryAmount()).ifPresent(builder::setDeliveryAmount);
		ofNullable(getReturnAmount()).ifPresent(builder::setReturnAmount);
		return builder;
	}

	public static CollateralRoundingBuilder builder() {
		return new CollateralRoundingBuilder();
	}

	public static  class CollateralRoundingBuilder extends RosettaModelObjectBuilder {
	
		protected BigDecimal deliveryAmount;
		protected BigDecimal returnAmount;
	
		public CollateralRoundingBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends CollateralRounding> metaData() {
			return metaData;
		} 
	
		
		public BigDecimal getDeliveryAmount() {
			return deliveryAmount;
		}
		
		
		public BigDecimal getReturnAmount() {
			return returnAmount;
		}
		
	
		public CollateralRoundingBuilder setDeliveryAmount(BigDecimal deliveryAmount) {
			this.deliveryAmount = deliveryAmount;
			return this;
		}
		public CollateralRoundingBuilder setReturnAmount(BigDecimal returnAmount) {
			this.returnAmount = returnAmount;
			return this;
		}
		
		public CollateralRounding build() {
			return new CollateralRounding(this);
		}
	
		@Override
		public CollateralRoundingBuilder prune() {
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getDeliveryAmount()!=null) return true;
			if (getReturnAmount()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("deliveryAmount"), BigDecimal.class, deliveryAmount, this);
			processor.processBasic(path.newSubPath("returnAmount"), BigDecimal.class, returnAmount, this);
			
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			CollateralRoundingBuilder _that = (CollateralRoundingBuilder) o;
		
			if (deliveryAmount != null ? !deliveryAmount.equals(_that.deliveryAmount) : _that.deliveryAmount != null) return false;
			if (returnAmount != null ? !returnAmount.equals(_that.returnAmount) : _that.returnAmount != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (deliveryAmount != null ? deliveryAmount.hashCode() : 0);
			_result = 31 * _result + (returnAmount != null ? returnAmount.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "CollateralRoundingBuilder {" +
				"deliveryAmount=" + this.deliveryAmount + ", " +
				"returnAmount=" + this.returnAmount +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("deliveryAmount"), BigDecimal.class, deliveryAmount, this);
		processor.processBasic(path.newSubPath("returnAmount"), BigDecimal.class, returnAmount, this);
		
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		CollateralRounding _that = (CollateralRounding) o;
	
		if (deliveryAmount != null ? !deliveryAmount.equals(_that.deliveryAmount) : _that.deliveryAmount != null) return false;
		if (returnAmount != null ? !returnAmount.equals(_that.returnAmount) : _that.returnAmount != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (deliveryAmount != null ? deliveryAmount.hashCode() : 0);
		_result = 31 * _result + (returnAmount != null ? returnAmount.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "CollateralRounding {" +
			"deliveryAmount=" + this.deliveryAmount + ", " +
			"returnAmount=" + this.returnAmount +
		'}';
	}
}
