package org.isda.cdm;

import java.math.BigDecimal;
import org.isda.cdm.Commodity;
import org.isda.cdm.TimeUnitEnum;
import org.isda.cdm.UnitEnum;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.CommoditySetMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;
		import java.math.BigDecimal;

/**
 * @version 2.5.4
 */
@RosettaClass
public class CommoditySet extends RosettaModelObject {
	private final Commodity commodity;
	private final BigDecimal quantity;
	private final TimeUnitEnum timeUnit;
	private final UnitEnum unit;
	private static CommoditySetMeta metaData = new CommoditySetMeta();
	
	CommoditySet(CommoditySetBuilder builder) {
		this.commodity = ofNullable(builder.getCommodity()).map(f->f.build()).orElse(null);
		this.quantity = builder.getQuantity();
		this.timeUnit = builder.getTimeUnit();
		this.unit = builder.getUnit();
	}
	
	public final Commodity getCommodity() {
		return commodity;
	}
	
	public final BigDecimal getQuantity() {
		return quantity;
	}
	
	public final TimeUnitEnum getTimeUnit() {
		return timeUnit;
	}
	
	/**
	 * The unit of measure, applicable to physical assets.
	 */
	@RosettaSynonym(value="unit", source="Rosetta_Workbench")
	public final UnitEnum getUnit() {
		return unit;
	}
	
	
	@Override
	public RosettaMetaData<? extends CommoditySet> metaData() {
		return metaData;
	} 
	
	public CommoditySetBuilder toBuilder() {
		CommoditySetBuilder builder = new CommoditySetBuilder();
		ofNullable(getCommodity()).ifPresent(builder::setCommodity);
		ofNullable(getQuantity()).ifPresent(builder::setQuantity);
		ofNullable(getTimeUnit()).ifPresent(builder::setTimeUnit);
		ofNullable(getUnit()).ifPresent(builder::setUnit);
		return builder;
	}

	public static CommoditySetBuilder builder() {
		return new CommoditySetBuilder();
	}

	public static  class CommoditySetBuilder extends RosettaModelObjectBuilder {
	
		protected Commodity.CommodityBuilder commodity;
		protected BigDecimal quantity;
		protected TimeUnitEnum timeUnit;
		protected UnitEnum unit;
	
		public CommoditySetBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends CommoditySet> metaData() {
			return metaData;
		} 
	
		
		public Commodity.CommodityBuilder getCommodity() {
			return commodity;
		}
		
		public Commodity.CommodityBuilder getOrCreateCommodity() {
			if (commodity!=null) {
				return commodity;
			}
			else return commodity = new Commodity.CommodityBuilder();
		}
		
		
		public BigDecimal getQuantity() {
			return quantity;
		}
		
		
		public TimeUnitEnum getTimeUnit() {
			return timeUnit;
		}
		
		
		public UnitEnum getUnit() {
			return unit;
		}
		
	
		public CommoditySetBuilder setCommodity(Commodity commodity) {
			this.commodity = commodity.toBuilder();
			return this;
		}
		public CommoditySetBuilder setCommodityBuilder(Commodity.CommodityBuilder commodity) {
			this.commodity = commodity;
			return this;
		}
		public CommoditySetBuilder setQuantity(BigDecimal quantity) {
			this.quantity = quantity;
			return this;
		}
		public CommoditySetBuilder setTimeUnit(TimeUnitEnum timeUnit) {
			this.timeUnit = timeUnit;
			return this;
		}
		public CommoditySetBuilder setUnit(UnitEnum unit) {
			this.unit = unit;
			return this;
		}
		
		public CommoditySet build() {
			return new CommoditySet(this);
		}
	
		@Override
		public CommoditySetBuilder prune() {
			if (commodity!=null && !commodity.prune().hasData()) commodity = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getCommodity()!=null && getCommodity().hasData()) return true;
			if (getQuantity()!=null) return true;
			if (getTimeUnit()!=null) return true;
			if (getUnit()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("quantity"), BigDecimal.class, quantity, this);
			processor.processBasic(path.newSubPath("timeUnit"), TimeUnitEnum.class, timeUnit, this);
			processor.processBasic(path.newSubPath("unit"), UnitEnum.class, unit, this);
			
			processRosetta(path.newSubPath("commodity"), processor, Commodity.class, commodity);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			CommoditySetBuilder _that = (CommoditySetBuilder) o;
		
			if (commodity != null ? !commodity.equals(_that.commodity) : _that.commodity != null) return false;
			if (quantity != null ? !quantity.equals(_that.quantity) : _that.quantity != null) return false;
			if (timeUnit != null ? !timeUnit.equals(_that.timeUnit) : _that.timeUnit != null) return false;
			if (unit != null ? !unit.equals(_that.unit) : _that.unit != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (commodity != null ? commodity.hashCode() : 0);
			_result = 31 * _result + (quantity != null ? quantity.hashCode() : 0);
			_result = 31 * _result + (timeUnit != null ? timeUnit.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (unit != null ? unit.getClass().getName().hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "CommoditySetBuilder {" +
				"commodity=" + this.commodity + ", " +
				"quantity=" + this.quantity + ", " +
				"timeUnit=" + this.timeUnit + ", " +
				"unit=" + this.unit +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("quantity"), BigDecimal.class, quantity, this);
		processor.processBasic(path.newSubPath("timeUnit"), TimeUnitEnum.class, timeUnit, this);
		processor.processBasic(path.newSubPath("unit"), UnitEnum.class, unit, this);
		
		processRosetta(path.newSubPath("commodity"), processor, Commodity.class, commodity);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		CommoditySet _that = (CommoditySet) o;
	
		if (commodity != null ? !commodity.equals(_that.commodity) : _that.commodity != null) return false;
		if (quantity != null ? !quantity.equals(_that.quantity) : _that.quantity != null) return false;
		if (timeUnit != null ? !timeUnit.equals(_that.timeUnit) : _that.timeUnit != null) return false;
		if (unit != null ? !unit.equals(_that.unit) : _that.unit != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (commodity != null ? commodity.hashCode() : 0);
		_result = 31 * _result + (quantity != null ? quantity.hashCode() : 0);
		_result = 31 * _result + (timeUnit != null ? timeUnit.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (unit != null ? unit.getClass().getName().hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "CommoditySet {" +
			"commodity=" + this.commodity + ", " +
			"quantity=" + this.quantity + ", " +
			"timeUnit=" + this.timeUnit + ", " +
			"unit=" + this.unit +
		'}';
	}
}
