package org.isda.cdm;

import org.isda.cdm.DeterminationMethodEnum;
import org.isda.cdm.FxSpotRateSource;
import org.isda.cdm.RelativeDateOffset;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.CompositeMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * Specifies the conditions to be applied for converting into a reference currency when the actual currency rate is not determined upfront.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="Composite", source="FpML_5_10")
public class Composite extends RosettaModelObject {
	private final DeterminationMethodEnum determinationMethod;
	private final FxSpotRateSource fxSpotRateSource;
	private final RelativeDateOffset relativeDate;
	private static CompositeMeta metaData = new CompositeMeta();
	
	Composite(CompositeBuilder builder) {
		this.determinationMethod = builder.getDeterminationMethod();
		this.fxSpotRateSource = ofNullable(builder.getFxSpotRateSource()).map(f->f.build()).orElse(null);
		this.relativeDate = ofNullable(builder.getRelativeDate()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * Specifies the method according to which an amount or a date is determined.
	 */
	@RosettaSynonym(value="determinationMethod", source="FpML_5_10")
	@RosettaSynonym(value="determinationMethod", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="determinationMethod", source="DTCC_11_0")
	@RosettaSynonym(value="determinationMethod", source="DTCC_9_0")
	@RosettaSynonym(value="determinationMethod", source="CME_ClearedConfirm_1_17")
	public final DeterminationMethodEnum getDeterminationMethod() {
		return determinationMethod;
	}
	
	/**
	 * Specifies the methodology (reference source and, optionally, fixing time) to be used for determining a currency conversion rate.
	 */
	@RosettaSynonym(value="fxSpotRateSource", source="FpML_5_10")
	@RosettaSynonym(value="fxSpotRateSource", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="fxSpotRateSource", source="DTCC_11_0")
	@RosettaSynonym(value="fxSpotRateSource", source="DTCC_9_0")
	@RosettaSynonym(value="fxSpotRateSource", source="CME_ClearedConfirm_1_17")
	public final FxSpotRateSource getFxSpotRateSource() {
		return fxSpotRateSource;
	}
	
	/**
	 * A date specified as some offset to another date (the anchor date).
	 */
	@RosettaSynonym(value="relativeDate", source="FpML_5_10")
	@RosettaSynonym(value="relativeDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="relativeDate", source="DTCC_11_0")
	@RosettaSynonym(value="relativeDate", source="DTCC_9_0")
	@RosettaSynonym(value="relativeDate", source="CME_ClearedConfirm_1_17")
	public final RelativeDateOffset getRelativeDate() {
		return relativeDate;
	}
	
	
	@Override
	public RosettaMetaData<? extends Composite> metaData() {
		return metaData;
	} 
	
	public CompositeBuilder toBuilder() {
		CompositeBuilder builder = new CompositeBuilder();
		ofNullable(getDeterminationMethod()).ifPresent(builder::setDeterminationMethod);
		ofNullable(getFxSpotRateSource()).ifPresent(builder::setFxSpotRateSource);
		ofNullable(getRelativeDate()).ifPresent(builder::setRelativeDate);
		return builder;
	}

	public static CompositeBuilder builder() {
		return new CompositeBuilder();
	}

	public static  class CompositeBuilder extends RosettaModelObjectBuilder {
	
		protected DeterminationMethodEnum determinationMethod;
		protected FxSpotRateSource.FxSpotRateSourceBuilder fxSpotRateSource;
		protected RelativeDateOffset.RelativeDateOffsetBuilder relativeDate;
	
		public CompositeBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends Composite> metaData() {
			return metaData;
		} 
	
		
		public DeterminationMethodEnum getDeterminationMethod() {
			return determinationMethod;
		}
		
		
		public FxSpotRateSource.FxSpotRateSourceBuilder getFxSpotRateSource() {
			return fxSpotRateSource;
		}
		
		public FxSpotRateSource.FxSpotRateSourceBuilder getOrCreateFxSpotRateSource() {
			if (fxSpotRateSource!=null) {
				return fxSpotRateSource;
			}
			else return fxSpotRateSource = new FxSpotRateSource.FxSpotRateSourceBuilder();
		}
		
		
		public RelativeDateOffset.RelativeDateOffsetBuilder getRelativeDate() {
			return relativeDate;
		}
		
		public RelativeDateOffset.RelativeDateOffsetBuilder getOrCreateRelativeDate() {
			if (relativeDate!=null) {
				return relativeDate;
			}
			else return relativeDate = new RelativeDateOffset.RelativeDateOffsetBuilder();
		}
		
	
		public CompositeBuilder setDeterminationMethod(DeterminationMethodEnum determinationMethod) {
			this.determinationMethod = determinationMethod;
			return this;
		}
		public CompositeBuilder setFxSpotRateSource(FxSpotRateSource fxSpotRateSource) {
			this.fxSpotRateSource = fxSpotRateSource.toBuilder();
			return this;
		}
		public CompositeBuilder setFxSpotRateSourceBuilder(FxSpotRateSource.FxSpotRateSourceBuilder fxSpotRateSource) {
			this.fxSpotRateSource = fxSpotRateSource;
			return this;
		}
		public CompositeBuilder setRelativeDate(RelativeDateOffset relativeDate) {
			this.relativeDate = relativeDate.toBuilder();
			return this;
		}
		public CompositeBuilder setRelativeDateBuilder(RelativeDateOffset.RelativeDateOffsetBuilder relativeDate) {
			this.relativeDate = relativeDate;
			return this;
		}
		
		public Composite build() {
			return new Composite(this);
		}
	
		@Override
		public CompositeBuilder prune() {
			if (fxSpotRateSource!=null && !fxSpotRateSource.prune().hasData()) fxSpotRateSource = null;
			if (relativeDate!=null && !relativeDate.prune().hasData()) relativeDate = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getDeterminationMethod()!=null) return true;
			if (getFxSpotRateSource()!=null && getFxSpotRateSource().hasData()) return true;
			if (getRelativeDate()!=null && getRelativeDate().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("determinationMethod"), DeterminationMethodEnum.class, determinationMethod, this);
			
			processRosetta(path.newSubPath("fxSpotRateSource"), processor, FxSpotRateSource.class, fxSpotRateSource);
			processRosetta(path.newSubPath("relativeDate"), processor, RelativeDateOffset.class, relativeDate);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			CompositeBuilder _that = (CompositeBuilder) o;
		
			if (determinationMethod != null ? !determinationMethod.equals(_that.determinationMethod) : _that.determinationMethod != null) return false;
			if (fxSpotRateSource != null ? !fxSpotRateSource.equals(_that.fxSpotRateSource) : _that.fxSpotRateSource != null) return false;
			if (relativeDate != null ? !relativeDate.equals(_that.relativeDate) : _that.relativeDate != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (determinationMethod != null ? determinationMethod.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (fxSpotRateSource != null ? fxSpotRateSource.hashCode() : 0);
			_result = 31 * _result + (relativeDate != null ? relativeDate.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "CompositeBuilder {" +
				"determinationMethod=" + this.determinationMethod + ", " +
				"fxSpotRateSource=" + this.fxSpotRateSource + ", " +
				"relativeDate=" + this.relativeDate +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("determinationMethod"), DeterminationMethodEnum.class, determinationMethod, this);
		
		processRosetta(path.newSubPath("fxSpotRateSource"), processor, FxSpotRateSource.class, fxSpotRateSource);
		processRosetta(path.newSubPath("relativeDate"), processor, RelativeDateOffset.class, relativeDate);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		Composite _that = (Composite) o;
	
		if (determinationMethod != null ? !determinationMethod.equals(_that.determinationMethod) : _that.determinationMethod != null) return false;
		if (fxSpotRateSource != null ? !fxSpotRateSource.equals(_that.fxSpotRateSource) : _that.fxSpotRateSource != null) return false;
		if (relativeDate != null ? !relativeDate.equals(_that.relativeDate) : _that.relativeDate != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (determinationMethod != null ? determinationMethod.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (fxSpotRateSource != null ? fxSpotRateSource.hashCode() : 0);
		_result = 31 * _result + (relativeDate != null ? relativeDate.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "Composite {" +
			"determinationMethod=" + this.determinationMethod + ", " +
			"fxSpotRateSource=" + this.fxSpotRateSource + ", " +
			"relativeDate=" + this.relativeDate +
		'}';
	}
}
