package org.isda.cdm;

import java.lang.String;
import java.math.BigDecimal;
import org.isda.cdm.metafields.FieldWithMetaString;
import org.isda.cdm.metafields.FieldWithMetaString.FieldWithMetaStringBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.ComputedAmountMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.math.BigDecimal;
		import static java.util.Optional.ofNullable;

/**
 * A class to specify the outcome of a computed amount, for testing purposes.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class ComputedAmount extends RosettaModelObject {
	private final BigDecimal amount;
	private final String callFunction;
	private final FieldWithMetaString currency;
	private static ComputedAmountMeta metaData = new ComputedAmountMeta();
	
	ComputedAmount(ComputedAmountBuilder builder) {
		this.amount = builder.getAmount();
		this.callFunction = builder.getCallFunction();
		this.currency = ofNullable(builder.getCurrency()).map(f->f.build()).orElse(null);
	}
	
	public final BigDecimal getAmount() {
		return amount;
	}
	
	public final String getCallFunction() {
		return callFunction;
	}
	
	/**
	 * The currency in which the computed amount is denominated. The list of valid currencies is not presently positioned as an enumeration as part of the CDM because that scope is limited to the values specified by ISDA and FpML. As a result, implementers have to make reference to the relevant standard, such as the ISO 4217 standard for currency codes.
	 */
	@RosettaSynonym(value="currency", source="Rosetta_Workbench")
	public final FieldWithMetaString getCurrency() {
		return currency;
	}
	
	
	@Override
	public RosettaMetaData<? extends ComputedAmount> metaData() {
		return metaData;
	} 
	
	public ComputedAmountBuilder toBuilder() {
		ComputedAmountBuilder builder = new ComputedAmountBuilder();
		ofNullable(getAmount()).ifPresent(builder::setAmount);
		ofNullable(getCallFunction()).ifPresent(builder::setCallFunction);
		ofNullable(getCurrency()).ifPresent(builder::setCurrency);
		return builder;
	}

	public static ComputedAmountBuilder builder() {
		return new ComputedAmountBuilder();
	}

	public static  class ComputedAmountBuilder extends RosettaModelObjectBuilder {
	
		protected BigDecimal amount;
		protected String callFunction;
		protected FieldWithMetaStringBuilder currency;
	
		public ComputedAmountBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends ComputedAmount> metaData() {
			return metaData;
		} 
	
		
		public BigDecimal getAmount() {
			return amount;
		}
		
		
		public String getCallFunction() {
			return callFunction;
		}
		
		
		public FieldWithMetaStringBuilder getCurrency() {
			return currency;
		}
		
		public FieldWithMetaStringBuilder getOrCreateCurrency() {
			if (currency!=null) {
				return currency;
			}
			else return currency = new FieldWithMetaStringBuilder();
		}
		
	
		public ComputedAmountBuilder setAmount(BigDecimal amount) {
			this.amount = amount;
			return this;
		}
		public ComputedAmountBuilder setCallFunction(String callFunction) {
			this.callFunction = callFunction;
			return this;
		}
		public ComputedAmountBuilder setCurrency(FieldWithMetaString currency) {
			this.currency = currency.toBuilder();
			return this;
		}
		public ComputedAmountBuilder setCurrencyRef(String currency) {
			return setCurrency(FieldWithMetaString.builder().setValue(currency).build());
		}
		
		public ComputedAmount build() {
			return new ComputedAmount(this);
		}
	
		@Override
		public ComputedAmountBuilder prune() {
			if (currency!=null && !currency.prune().hasData()) currency = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getAmount()!=null) return true;
			if (getCallFunction()!=null) return true;
			if (getCurrency()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("amount"), BigDecimal.class, amount, this);
			processor.processBasic(path.newSubPath("callFunction"), String.class, callFunction, this);
			
			processRosetta(path.newSubPath("currency"), processor, FieldWithMetaString.class, currency);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			ComputedAmountBuilder _that = (ComputedAmountBuilder) o;
		
			if (amount != null ? !amount.equals(_that.amount) : _that.amount != null) return false;
			if (callFunction != null ? !callFunction.equals(_that.callFunction) : _that.callFunction != null) return false;
			if (currency != null ? !currency.equals(_that.currency) : _that.currency != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (amount != null ? amount.hashCode() : 0);
			_result = 31 * _result + (callFunction != null ? callFunction.hashCode() : 0);
			_result = 31 * _result + (currency != null ? currency.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "ComputedAmountBuilder {" +
				"amount=" + this.amount + ", " +
				"callFunction=" + this.callFunction + ", " +
				"currency=" + this.currency +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("amount"), BigDecimal.class, amount, this);
		processor.processBasic(path.newSubPath("callFunction"), String.class, callFunction, this);
		
		processRosetta(path.newSubPath("currency"), processor, FieldWithMetaString.class, currency);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		ComputedAmount _that = (ComputedAmount) o;
	
		if (amount != null ? !amount.equals(_that.amount) : _that.amount != null) return false;
		if (callFunction != null ? !callFunction.equals(_that.callFunction) : _that.callFunction != null) return false;
		if (currency != null ? !currency.equals(_that.currency) : _that.currency != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (amount != null ? amount.hashCode() : 0);
		_result = 31 * _result + (callFunction != null ? callFunction.hashCode() : 0);
		_result = 31 * _result + (currency != null ? currency.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "ComputedAmount {" +
			"amount=" + this.amount + ", " +
			"callFunction=" + this.callFunction + ", " +
			"currency=" + this.currency +
		'}';
	}
}
