package org.isda.cdm;

import java.lang.Boolean;
import java.lang.String;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.ConditionsElectionsMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class to specify the Termination Events that will be deemed an Access Condition (Initial Margin CSA) or a Specified Condition (Variation Margin CSA). ISDA 2016 English Law Credit Support Deed for Initial Margin, paragraph 13, General Principles, (e)(ii). | ISDA 2016 Japanese Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (f)(ii). | ISDA 2016 New York Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (e)(ii). | ISDA 2016 Credit Support Annex for Variation Margin, paragraph 13, (e): Conditions Precedent and Secured Party’s Rights and Remedies.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class ConditionsElections extends RosettaModelObject {
	private final String additionalLanguage;
	private final Boolean additionalTerminationEvents;
	private final Boolean creditEventUponMerger;
	private final Boolean forceMajeure;
	private final Boolean illegality;
	private final String party;
	private final Boolean taxEvent;
	private final Boolean taxEventUponMerger;
	private static ConditionsElectionsMeta metaData = new ConditionsElectionsMeta();
	
	ConditionsElections(ConditionsElectionsBuilder builder) {
		this.additionalLanguage = builder.getAdditionalLanguage();
		this.additionalTerminationEvents = builder.getAdditionalTerminationEvents();
		this.creditEventUponMerger = builder.getCreditEventUponMerger();
		this.forceMajeure = builder.getForceMajeure();
		this.illegality = builder.getIllegality();
		this.party = builder.getParty();
		this.taxEvent = builder.getTaxEvent();
		this.taxEventUponMerger = builder.getTaxEventUponMerger();
	}
	
	@RosettaSynonym(value="specify", source="ISDA_Create_1_0")
	public final String getAdditionalLanguage() {
		return additionalLanguage;
	}
	
	@RosettaSynonym(value="", source="ISDA_Create_1_0")
	@RosettaSynonym(value="", source="ISDA_Create_1_0")
	public final Boolean getAdditionalTerminationEvents() {
		return additionalTerminationEvents;
	}
	
	@RosettaSynonym(value="", source="ISDA_Create_1_0")
	@RosettaSynonym(value="", source="ISDA_Create_1_0")
	public final Boolean getCreditEventUponMerger() {
		return creditEventUponMerger;
	}
	
	@RosettaSynonym(value="", source="ISDA_Create_1_0")
	@RosettaSynonym(value="", source="ISDA_Create_1_0")
	public final Boolean getForceMajeure() {
		return forceMajeure;
	}
	
	@RosettaSynonym(value="", source="ISDA_Create_1_0")
	@RosettaSynonym(value="", source="ISDA_Create_1_0")
	public final Boolean getIllegality() {
		return illegality;
	}
	
	/**
	 * The elective party.
	 */
	@RosettaSynonym(value="", source="ISDA_Create_1_0")
	@RosettaSynonym(value="", source="ISDA_Create_1_0")
	public final String getParty() {
		return party;
	}
	
	@RosettaSynonym(value="", source="ISDA_Create_1_0")
	@RosettaSynonym(value="", source="ISDA_Create_1_0")
	public final Boolean getTaxEvent() {
		return taxEvent;
	}
	
	@RosettaSynonym(value="", source="ISDA_Create_1_0")
	@RosettaSynonym(value="", source="ISDA_Create_1_0")
	public final Boolean getTaxEventUponMerger() {
		return taxEventUponMerger;
	}
	
	
	@Override
	public RosettaMetaData<? extends ConditionsElections> metaData() {
		return metaData;
	} 
	
	public ConditionsElectionsBuilder toBuilder() {
		ConditionsElectionsBuilder builder = new ConditionsElectionsBuilder();
		ofNullable(getAdditionalLanguage()).ifPresent(builder::setAdditionalLanguage);
		ofNullable(getAdditionalTerminationEvents()).ifPresent(builder::setAdditionalTerminationEvents);
		ofNullable(getCreditEventUponMerger()).ifPresent(builder::setCreditEventUponMerger);
		ofNullable(getForceMajeure()).ifPresent(builder::setForceMajeure);
		ofNullable(getIllegality()).ifPresent(builder::setIllegality);
		ofNullable(getParty()).ifPresent(builder::setParty);
		ofNullable(getTaxEvent()).ifPresent(builder::setTaxEvent);
		ofNullable(getTaxEventUponMerger()).ifPresent(builder::setTaxEventUponMerger);
		return builder;
	}

	public static ConditionsElectionsBuilder builder() {
		return new ConditionsElectionsBuilder();
	}

	public static  class ConditionsElectionsBuilder extends RosettaModelObjectBuilder {
	
		protected String additionalLanguage;
		protected Boolean additionalTerminationEvents;
		protected Boolean creditEventUponMerger;
		protected Boolean forceMajeure;
		protected Boolean illegality;
		protected String party;
		protected Boolean taxEvent;
		protected Boolean taxEventUponMerger;
	
		public ConditionsElectionsBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends ConditionsElections> metaData() {
			return metaData;
		} 
	
		
		public String getAdditionalLanguage() {
			return additionalLanguage;
		}
		
		
		public Boolean getAdditionalTerminationEvents() {
			return additionalTerminationEvents;
		}
		
		
		public Boolean getCreditEventUponMerger() {
			return creditEventUponMerger;
		}
		
		
		public Boolean getForceMajeure() {
			return forceMajeure;
		}
		
		
		public Boolean getIllegality() {
			return illegality;
		}
		
		
		public String getParty() {
			return party;
		}
		
		
		public Boolean getTaxEvent() {
			return taxEvent;
		}
		
		
		public Boolean getTaxEventUponMerger() {
			return taxEventUponMerger;
		}
		
	
		public ConditionsElectionsBuilder setAdditionalLanguage(String additionalLanguage) {
			this.additionalLanguage = additionalLanguage;
			return this;
		}
		public ConditionsElectionsBuilder setAdditionalTerminationEvents(Boolean additionalTerminationEvents) {
			this.additionalTerminationEvents = additionalTerminationEvents;
			return this;
		}
		public ConditionsElectionsBuilder setCreditEventUponMerger(Boolean creditEventUponMerger) {
			this.creditEventUponMerger = creditEventUponMerger;
			return this;
		}
		public ConditionsElectionsBuilder setForceMajeure(Boolean forceMajeure) {
			this.forceMajeure = forceMajeure;
			return this;
		}
		public ConditionsElectionsBuilder setIllegality(Boolean illegality) {
			this.illegality = illegality;
			return this;
		}
		public ConditionsElectionsBuilder setParty(String party) {
			this.party = party;
			return this;
		}
		public ConditionsElectionsBuilder setTaxEvent(Boolean taxEvent) {
			this.taxEvent = taxEvent;
			return this;
		}
		public ConditionsElectionsBuilder setTaxEventUponMerger(Boolean taxEventUponMerger) {
			this.taxEventUponMerger = taxEventUponMerger;
			return this;
		}
		
		public ConditionsElections build() {
			return new ConditionsElections(this);
		}
	
		@Override
		public ConditionsElectionsBuilder prune() {
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getAdditionalLanguage()!=null) return true;
			if (getAdditionalTerminationEvents()!=null) return true;
			if (getCreditEventUponMerger()!=null) return true;
			if (getForceMajeure()!=null) return true;
			if (getIllegality()!=null) return true;
			if (getParty()!=null) return true;
			if (getTaxEvent()!=null) return true;
			if (getTaxEventUponMerger()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("additionalLanguage"), String.class, additionalLanguage, this);
			processor.processBasic(path.newSubPath("additionalTerminationEvents"), Boolean.class, additionalTerminationEvents, this);
			processor.processBasic(path.newSubPath("creditEventUponMerger"), Boolean.class, creditEventUponMerger, this);
			processor.processBasic(path.newSubPath("forceMajeure"), Boolean.class, forceMajeure, this);
			processor.processBasic(path.newSubPath("illegality"), Boolean.class, illegality, this);
			processor.processBasic(path.newSubPath("party"), String.class, party, this);
			processor.processBasic(path.newSubPath("taxEvent"), Boolean.class, taxEvent, this);
			processor.processBasic(path.newSubPath("taxEventUponMerger"), Boolean.class, taxEventUponMerger, this);
			
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			ConditionsElectionsBuilder _that = (ConditionsElectionsBuilder) o;
		
			if (additionalLanguage != null ? !additionalLanguage.equals(_that.additionalLanguage) : _that.additionalLanguage != null) return false;
			if (additionalTerminationEvents != null ? !additionalTerminationEvents.equals(_that.additionalTerminationEvents) : _that.additionalTerminationEvents != null) return false;
			if (creditEventUponMerger != null ? !creditEventUponMerger.equals(_that.creditEventUponMerger) : _that.creditEventUponMerger != null) return false;
			if (forceMajeure != null ? !forceMajeure.equals(_that.forceMajeure) : _that.forceMajeure != null) return false;
			if (illegality != null ? !illegality.equals(_that.illegality) : _that.illegality != null) return false;
			if (party != null ? !party.equals(_that.party) : _that.party != null) return false;
			if (taxEvent != null ? !taxEvent.equals(_that.taxEvent) : _that.taxEvent != null) return false;
			if (taxEventUponMerger != null ? !taxEventUponMerger.equals(_that.taxEventUponMerger) : _that.taxEventUponMerger != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (additionalLanguage != null ? additionalLanguage.hashCode() : 0);
			_result = 31 * _result + (additionalTerminationEvents != null ? additionalTerminationEvents.hashCode() : 0);
			_result = 31 * _result + (creditEventUponMerger != null ? creditEventUponMerger.hashCode() : 0);
			_result = 31 * _result + (forceMajeure != null ? forceMajeure.hashCode() : 0);
			_result = 31 * _result + (illegality != null ? illegality.hashCode() : 0);
			_result = 31 * _result + (party != null ? party.hashCode() : 0);
			_result = 31 * _result + (taxEvent != null ? taxEvent.hashCode() : 0);
			_result = 31 * _result + (taxEventUponMerger != null ? taxEventUponMerger.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "ConditionsElectionsBuilder {" +
				"additionalLanguage=" + this.additionalLanguage + ", " +
				"additionalTerminationEvents=" + this.additionalTerminationEvents + ", " +
				"creditEventUponMerger=" + this.creditEventUponMerger + ", " +
				"forceMajeure=" + this.forceMajeure + ", " +
				"illegality=" + this.illegality + ", " +
				"party=" + this.party + ", " +
				"taxEvent=" + this.taxEvent + ", " +
				"taxEventUponMerger=" + this.taxEventUponMerger +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("additionalLanguage"), String.class, additionalLanguage, this);
		processor.processBasic(path.newSubPath("additionalTerminationEvents"), Boolean.class, additionalTerminationEvents, this);
		processor.processBasic(path.newSubPath("creditEventUponMerger"), Boolean.class, creditEventUponMerger, this);
		processor.processBasic(path.newSubPath("forceMajeure"), Boolean.class, forceMajeure, this);
		processor.processBasic(path.newSubPath("illegality"), Boolean.class, illegality, this);
		processor.processBasic(path.newSubPath("party"), String.class, party, this);
		processor.processBasic(path.newSubPath("taxEvent"), Boolean.class, taxEvent, this);
		processor.processBasic(path.newSubPath("taxEventUponMerger"), Boolean.class, taxEventUponMerger, this);
		
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		ConditionsElections _that = (ConditionsElections) o;
	
		if (additionalLanguage != null ? !additionalLanguage.equals(_that.additionalLanguage) : _that.additionalLanguage != null) return false;
		if (additionalTerminationEvents != null ? !additionalTerminationEvents.equals(_that.additionalTerminationEvents) : _that.additionalTerminationEvents != null) return false;
		if (creditEventUponMerger != null ? !creditEventUponMerger.equals(_that.creditEventUponMerger) : _that.creditEventUponMerger != null) return false;
		if (forceMajeure != null ? !forceMajeure.equals(_that.forceMajeure) : _that.forceMajeure != null) return false;
		if (illegality != null ? !illegality.equals(_that.illegality) : _that.illegality != null) return false;
		if (party != null ? !party.equals(_that.party) : _that.party != null) return false;
		if (taxEvent != null ? !taxEvent.equals(_that.taxEvent) : _that.taxEvent != null) return false;
		if (taxEventUponMerger != null ? !taxEventUponMerger.equals(_that.taxEventUponMerger) : _that.taxEventUponMerger != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (additionalLanguage != null ? additionalLanguage.hashCode() : 0);
		_result = 31 * _result + (additionalTerminationEvents != null ? additionalTerminationEvents.hashCode() : 0);
		_result = 31 * _result + (creditEventUponMerger != null ? creditEventUponMerger.hashCode() : 0);
		_result = 31 * _result + (forceMajeure != null ? forceMajeure.hashCode() : 0);
		_result = 31 * _result + (illegality != null ? illegality.hashCode() : 0);
		_result = 31 * _result + (party != null ? party.hashCode() : 0);
		_result = 31 * _result + (taxEvent != null ? taxEvent.hashCode() : 0);
		_result = 31 * _result + (taxEventUponMerger != null ? taxEventUponMerger.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "ConditionsElections {" +
			"additionalLanguage=" + this.additionalLanguage + ", " +
			"additionalTerminationEvents=" + this.additionalTerminationEvents + ", " +
			"creditEventUponMerger=" + this.creditEventUponMerger + ", " +
			"forceMajeure=" + this.forceMajeure + ", " +
			"illegality=" + this.illegality + ", " +
			"party=" + this.party + ", " +
			"taxEvent=" + this.taxEvent + ", " +
			"taxEventUponMerger=" + this.taxEventUponMerger +
		'}';
	}
}
