package org.isda.cdm;

import java.lang.String;
import org.isda.cdm.Conditions;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.ConditionsPrecedentMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class to specify the two set of elections that may overwrite the default Condition Precedent provision as specified in Paragraph 4, (a) of the ISDA 2016 Credit Support Annex for Initial Margin and the ISDA 2016 Credit Support Annex for Variation Margin. ISDA 2016 English Law Credit Support Deed for Initial Margin, paragraph 13, General Principles, (e): Conditions Precedent. | ISDA 2016 Japanese Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (f): Conditions Precedent. | ISDA 2016 New York Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (e): Conditions Precedent. | ISDA 2016 Credit Support Annex for Variation Margin, paragraph 13, (e): Conditions Precedent and Secured Party’s Rights and Remedies.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class ConditionsPrecedent extends RosettaModelObject {
	private final Conditions conditions;
	private final String customProvisions;
	private static ConditionsPrecedentMeta metaData = new ConditionsPrecedentMeta();
	
	ConditionsPrecedent(ConditionsPrecedentBuilder builder) {
		this.conditions = ofNullable(builder.getConditions()).map(f->f.build()).orElse(null);
		this.customProvisions = builder.getCustomProvisions();
	}
	
	/**
	 * The parties&#39; election with respect to the Termination Events that will be deemed an Access Condition (Initial Margin CSA) or a Specified Condition (Variation Margin CSA). ISDA 2016 English Law Credit Support Deed for Initial Margin, paragraph 13, General Principles, (e)(ii). | ISDA 2016 Japanese Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (f)(ii). | ISDA 2016 New York Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (e)(ii). | ISDA 2016 Credit Support Annex for Variation Margin, paragraph 13, (e): Conditions Precedent and Secured Party’s Rights and Remedies.
	 */
	@RosettaSynonym(value="access_conditions", source="ISDA_Create_1_0")
	public final Conditions getConditions() {
		return conditions;
	}
	
	/**
	 * The custom provisions that might be specified by the parties to the agreement for the purpose of overwriting the default Condition Precedent provision as specified in ISDA 2016 Credit Support Annex for Initial Margin and the ISDA 2016 Credit Support Annex for Variation Margin, Paragraph 4, (a). ISDA 2016 English Law Credit Support Deed for Initial Margin, paragraph 13, General Principles, (e)(i). | ISDA 2016 Japanese Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (f)(i). | ISDA 2016 New York Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (e)(i). | ISDA 2016 Credit Support Annex for Variation Margin, paragraph 13, : Conditions Precedent and Secured Party’s Rights and Remedies.
	 */
	@RosettaSynonym(value="specify", source="ISDA_Create_1_0", path="conditions_precedent")
	public final String getCustomProvisions() {
		return customProvisions;
	}
	
	
	@Override
	public RosettaMetaData<? extends ConditionsPrecedent> metaData() {
		return metaData;
	} 
	
	public ConditionsPrecedentBuilder toBuilder() {
		ConditionsPrecedentBuilder builder = new ConditionsPrecedentBuilder();
		ofNullable(getConditions()).ifPresent(builder::setConditions);
		ofNullable(getCustomProvisions()).ifPresent(builder::setCustomProvisions);
		return builder;
	}

	public static ConditionsPrecedentBuilder builder() {
		return new ConditionsPrecedentBuilder();
	}

	public static  class ConditionsPrecedentBuilder extends RosettaModelObjectBuilder {
	
		protected Conditions.ConditionsBuilder conditions;
		protected String customProvisions;
	
		public ConditionsPrecedentBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends ConditionsPrecedent> metaData() {
			return metaData;
		} 
	
		
		public Conditions.ConditionsBuilder getConditions() {
			return conditions;
		}
		
		public Conditions.ConditionsBuilder getOrCreateConditions() {
			if (conditions!=null) {
				return conditions;
			}
			else return conditions = new Conditions.ConditionsBuilder();
		}
		
		
		public String getCustomProvisions() {
			return customProvisions;
		}
		
	
		public ConditionsPrecedentBuilder setConditions(Conditions conditions) {
			this.conditions = conditions.toBuilder();
			return this;
		}
		public ConditionsPrecedentBuilder setConditionsBuilder(Conditions.ConditionsBuilder conditions) {
			this.conditions = conditions;
			return this;
		}
		public ConditionsPrecedentBuilder setCustomProvisions(String customProvisions) {
			this.customProvisions = customProvisions;
			return this;
		}
		
		public ConditionsPrecedent build() {
			return new ConditionsPrecedent(this);
		}
	
		@Override
		public ConditionsPrecedentBuilder prune() {
			if (conditions!=null && !conditions.prune().hasData()) conditions = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getConditions()!=null && getConditions().hasData()) return true;
			if (getCustomProvisions()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("customProvisions"), String.class, customProvisions, this);
			
			processRosetta(path.newSubPath("conditions"), processor, Conditions.class, conditions);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			ConditionsPrecedentBuilder _that = (ConditionsPrecedentBuilder) o;
		
			if (conditions != null ? !conditions.equals(_that.conditions) : _that.conditions != null) return false;
			if (customProvisions != null ? !customProvisions.equals(_that.customProvisions) : _that.customProvisions != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (conditions != null ? conditions.hashCode() : 0);
			_result = 31 * _result + (customProvisions != null ? customProvisions.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "ConditionsPrecedentBuilder {" +
				"conditions=" + this.conditions + ", " +
				"customProvisions=" + this.customProvisions +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("customProvisions"), String.class, customProvisions, this);
		
		processRosetta(path.newSubPath("conditions"), processor, Conditions.class, conditions);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		ConditionsPrecedent _that = (ConditionsPrecedent) o;
	
		if (conditions != null ? !conditions.equals(_that.conditions) : _that.conditions != null) return false;
		if (customProvisions != null ? !customProvisions.equals(_that.customProvisions) : _that.customProvisions != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (conditions != null ? conditions.hashCode() : 0);
		_result = 31 * _result + (customProvisions != null ? customProvisions.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "ConditionsPrecedent {" +
			"conditions=" + this.conditions + ", " +
			"customProvisions=" + this.customProvisions +
		'}';
	}
}
