package org.isda.cdm;

import java.math.BigDecimal;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.ConstituentWeightMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.math.BigDecimal;
		import static java.util.Optional.ofNullable;

/**
 * A class describing the weight of each of the underlier constituent within the basket, either in absolute or relative terms.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="ConstituentWeight", source="FpML_5_10")
public class ConstituentWeight extends RosettaModelObject {
	private final BigDecimal basketPercentage;
	private final BigDecimal openUnits;
	private static ConstituentWeightMeta metaData = new ConstituentWeightMeta();
	
	ConstituentWeight(ConstituentWeightBuilder builder) {
		this.basketPercentage = builder.getBasketPercentage();
		this.openUnits = builder.getOpenUnits();
	}
	
	/**
	 * The relative weight of each respective basket constituent, expressed in percentage. A basket percentage of 5% would be represented as 0.05.
	 */
	@RosettaSynonym(value="basketPercentage", source="FpML_5_10")
	@RosettaSynonym(value="basketPercentage", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="basketPercentage", source="DTCC_11_0")
	@RosettaSynonym(value="basketPercentage", source="DTCC_9_0")
	@RosettaSynonym(value="basketPercentage", source="CME_ClearedConfirm_1_17")
	public final BigDecimal getBasketPercentage() {
		return basketPercentage;
	}
	
	/**
	 * The number of units (index or securities) that constitute the underlier of the swap. In the case of a basket swap, this element is used to reference both the number of basket units, and the number of each asset components of the basket when these are expressed in absolute terms.
	 */
	@RosettaSynonym(value="openUnits", source="FpML_5_10")
	@RosettaSynonym(value="openUnits", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="openUnits", source="DTCC_11_0")
	@RosettaSynonym(value="openUnits", source="DTCC_9_0")
	@RosettaSynonym(value="openUnits", source="CME_ClearedConfirm_1_17")
	public final BigDecimal getOpenUnits() {
		return openUnits;
	}
	
	
	@Override
	public RosettaMetaData<? extends ConstituentWeight> metaData() {
		return metaData;
	} 
	
	public ConstituentWeightBuilder toBuilder() {
		ConstituentWeightBuilder builder = new ConstituentWeightBuilder();
		ofNullable(getBasketPercentage()).ifPresent(builder::setBasketPercentage);
		ofNullable(getOpenUnits()).ifPresent(builder::setOpenUnits);
		return builder;
	}

	public static ConstituentWeightBuilder builder() {
		return new ConstituentWeightBuilder();
	}

	public static  class ConstituentWeightBuilder extends RosettaModelObjectBuilder {
	
		protected BigDecimal basketPercentage;
		protected BigDecimal openUnits;
	
		public ConstituentWeightBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends ConstituentWeight> metaData() {
			return metaData;
		} 
	
		
		public BigDecimal getBasketPercentage() {
			return basketPercentage;
		}
		
		
		public BigDecimal getOpenUnits() {
			return openUnits;
		}
		
	
		public ConstituentWeightBuilder setBasketPercentage(BigDecimal basketPercentage) {
			this.basketPercentage = basketPercentage;
			return this;
		}
		public ConstituentWeightBuilder setOpenUnits(BigDecimal openUnits) {
			this.openUnits = openUnits;
			return this;
		}
		
		public ConstituentWeight build() {
			return new ConstituentWeight(this);
		}
	
		@Override
		public ConstituentWeightBuilder prune() {
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getBasketPercentage()!=null) return true;
			if (getOpenUnits()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("basketPercentage"), BigDecimal.class, basketPercentage, this);
			processor.processBasic(path.newSubPath("openUnits"), BigDecimal.class, openUnits, this);
			
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			ConstituentWeightBuilder _that = (ConstituentWeightBuilder) o;
		
			if (basketPercentage != null ? !basketPercentage.equals(_that.basketPercentage) : _that.basketPercentage != null) return false;
			if (openUnits != null ? !openUnits.equals(_that.openUnits) : _that.openUnits != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (basketPercentage != null ? basketPercentage.hashCode() : 0);
			_result = 31 * _result + (openUnits != null ? openUnits.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "ConstituentWeightBuilder {" +
				"basketPercentage=" + this.basketPercentage + ", " +
				"openUnits=" + this.openUnits +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("basketPercentage"), BigDecimal.class, basketPercentage, this);
		processor.processBasic(path.newSubPath("openUnits"), BigDecimal.class, openUnits, this);
		
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		ConstituentWeight _that = (ConstituentWeight) o;
	
		if (basketPercentage != null ? !basketPercentage.equals(_that.basketPercentage) : _that.basketPercentage != null) return false;
		if (openUnits != null ? !openUnits.equals(_that.openUnits) : _that.openUnits != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (basketPercentage != null ? basketPercentage.hashCode() : 0);
		_result = 31 * _result + (openUnits != null ? openUnits.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "ConstituentWeight {" +
			"basketPercentage=" + this.basketPercentage + ", " +
			"openUnits=" + this.openUnits +
		'}';
	}
}
