package org.isda.cdm;

import org.isda.cdm.ExecutionState;
import org.isda.cdm.PostInceptionState;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.ContractFormationMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * Specification of the primitive event for the formation of a contract, with &#39;before&#39; state being an &#39;ExecutionState&#39; and &#39;after&#39; state being a &#39;PostInceptionState&#39;. This primitive combined with the &#39;ExecutionPrimitive&#39; supersedes the earlier &#39;Inception&#39; primitive, which had both the the execution and the contract formation bundled together, and as such was only applicable to contractual products. This intends to provide for properly atomic primitives.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class ContractFormation extends RosettaModelObject {
	private final PostInceptionState after;
	private final ExecutionState before;
	private static ContractFormationMeta metaData = new ContractFormationMeta();
	
	ContractFormation(ContractFormationBuilder builder) {
		this.after = ofNullable(builder.getAfter()).map(f->f.build()).orElse(null);
		this.before = ofNullable(builder.getBefore()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The after state corresponds to the new contract being formed between the parties, plus some additional trade workflow information.
	 */
	@RosettaSynonym(value="after", source="Rosetta_Workbench")
	public final PostInceptionState getAfter() {
		return after;
	}
	
	/**
	 * The before state corresponds to the output of an execution between the parties.
	 */
	@RosettaSynonym(value="before", source="Rosetta_Workbench")
	public final ExecutionState getBefore() {
		return before;
	}
	
	
	@Override
	public RosettaMetaData<? extends ContractFormation> metaData() {
		return metaData;
	} 
	
	public ContractFormationBuilder toBuilder() {
		ContractFormationBuilder builder = new ContractFormationBuilder();
		ofNullable(getAfter()).ifPresent(builder::setAfter);
		ofNullable(getBefore()).ifPresent(builder::setBefore);
		return builder;
	}

	public static ContractFormationBuilder builder() {
		return new ContractFormationBuilder();
	}

	public static  class ContractFormationBuilder extends RosettaModelObjectBuilder {
	
		protected PostInceptionState.PostInceptionStateBuilder after;
		protected ExecutionState.ExecutionStateBuilder before;
	
		public ContractFormationBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends ContractFormation> metaData() {
			return metaData;
		} 
	
		
		public PostInceptionState.PostInceptionStateBuilder getAfter() {
			return after;
		}
		
		public PostInceptionState.PostInceptionStateBuilder getOrCreateAfter() {
			if (after!=null) {
				return after;
			}
			else return after = new PostInceptionState.PostInceptionStateBuilder();
		}
		
		
		public ExecutionState.ExecutionStateBuilder getBefore() {
			return before;
		}
		
		public ExecutionState.ExecutionStateBuilder getOrCreateBefore() {
			if (before!=null) {
				return before;
			}
			else return before = new ExecutionState.ExecutionStateBuilder();
		}
		
	
		public ContractFormationBuilder setAfter(PostInceptionState after) {
			this.after = after.toBuilder();
			return this;
		}
		public ContractFormationBuilder setAfterBuilder(PostInceptionState.PostInceptionStateBuilder after) {
			this.after = after;
			return this;
		}
		public ContractFormationBuilder setBefore(ExecutionState before) {
			this.before = before.toBuilder();
			return this;
		}
		public ContractFormationBuilder setBeforeBuilder(ExecutionState.ExecutionStateBuilder before) {
			this.before = before;
			return this;
		}
		
		public ContractFormation build() {
			return new ContractFormation(this);
		}
	
		@Override
		public ContractFormationBuilder prune() {
			if (after!=null && !after.prune().hasData()) after = null;
			if (before!=null && !before.prune().hasData()) before = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getAfter()!=null && getAfter().hasData()) return true;
			if (getBefore()!=null && getBefore().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("after"), processor, PostInceptionState.class, after);
			processRosetta(path.newSubPath("before"), processor, ExecutionState.class, before);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			ContractFormationBuilder _that = (ContractFormationBuilder) o;
		
			if (after != null ? !after.equals(_that.after) : _that.after != null) return false;
			if (before != null ? !before.equals(_that.before) : _that.before != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (after != null ? after.hashCode() : 0);
			_result = 31 * _result + (before != null ? before.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "ContractFormationBuilder {" +
				"after=" + this.after + ", " +
				"before=" + this.before +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("after"), processor, PostInceptionState.class, after);
		processRosetta(path.newSubPath("before"), processor, ExecutionState.class, before);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		ContractFormation _that = (ContractFormation) o;
	
		if (after != null ? !after.equals(_that.after) : _that.after != null) return false;
		if (before != null ? !before.equals(_that.before) : _that.before != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (after != null ? after.hashCode() : 0);
		_result = 31 * _result + (before != null ? before.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "ContractFormation {" +
			"after=" + this.after + ", " +
			"before=" + this.before +
		'}';
	}
}
