package org.isda.cdm;

import org.isda.cdm.Contract;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.ContractStateMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class to specify a contract state instantiation with respect to the before and/or after state of lifecycle events.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class ContractState extends RosettaModelObject {
	private final Contract contract;
	private final Contract updatedContract;
	private static ContractStateMeta metaData = new ContractStateMeta();
	
	ContractState(ContractStateBuilder builder) {
		this.contract = ofNullable(builder.getContract()).map(f->f.build()).orElse(null);
		this.updatedContract = ofNullable(builder.getUpdatedContract()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * Reference to the original contract, such that the contract state can be resolved by super-imposing the updated values on top of the original contract.
	 */
	@RosettaSynonym(value="trade", source="Rosetta_Workbench")
	@RosettaSynonym(value="FpML", source="DTCC_11_0", path="Trade")
	@RosettaSynonym(value="FpML", source="DTCC_9_0", path="Trade")
	@RosettaSynonym(value="TradeMsg", source="DTCC_11_0", path="Manifest")
	@RosettaSynonym(value="TradeMsg", source="DTCC_9_0", path="Manifest")
	@RosettaSynonym(value="ReportingJurisdiction", source="DTCC_11_0", path="ReportingData.ReportingHeader")
	@RosettaSynonym(value="ReportingJurisdiction", source="DTCC_9_0", path="ReportingData.ReportingHeader")
	@RosettaSynonym(value="FpML", source="CME_SubmissionIRS_1_0", path="Instrmt.SecXML")
	public final Contract getContract() {
		return contract;
	}
	
	/**
	 * The state of the contract, represented as a replica of the original contract with updated values where applicable, e.g. in the case of resets.
	 */
	@RosettaSynonym(value="ignore", source="FpML_5_10")
	@RosettaSynonym(value="ignore", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="ignore", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="ignore", source="DTCC_11_0")
	@RosettaSynonym(value="ignore", source="DTCC_9_0")
	public final Contract getUpdatedContract() {
		return updatedContract;
	}
	
	
	@Override
	public RosettaMetaData<? extends ContractState> metaData() {
		return metaData;
	} 
	
	public ContractStateBuilder toBuilder() {
		ContractStateBuilder builder = new ContractStateBuilder();
		ofNullable(getContract()).ifPresent(builder::setContract);
		ofNullable(getUpdatedContract()).ifPresent(builder::setUpdatedContract);
		return builder;
	}

	public static ContractStateBuilder builder() {
		return new ContractStateBuilder();
	}

	public static  class ContractStateBuilder extends RosettaModelObjectBuilder {
	
		protected Contract.ContractBuilder contract;
		protected Contract.ContractBuilder updatedContract;
	
		public ContractStateBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends ContractState> metaData() {
			return metaData;
		} 
	
		
		public Contract.ContractBuilder getContract() {
			return contract;
		}
		
		public Contract.ContractBuilder getOrCreateContract() {
			if (contract!=null) {
				return contract;
			}
			else return contract = new Contract.ContractBuilder();
		}
		
		
		public Contract.ContractBuilder getUpdatedContract() {
			return updatedContract;
		}
		
		public Contract.ContractBuilder getOrCreateUpdatedContract() {
			if (updatedContract!=null) {
				return updatedContract;
			}
			else return updatedContract = new Contract.ContractBuilder();
		}
		
	
		public ContractStateBuilder setContract(Contract contract) {
			this.contract = contract.toBuilder();
			return this;
		}
		public ContractStateBuilder setContractBuilder(Contract.ContractBuilder contract) {
			this.contract = contract;
			return this;
		}
		public ContractStateBuilder setUpdatedContract(Contract updatedContract) {
			this.updatedContract = updatedContract.toBuilder();
			return this;
		}
		public ContractStateBuilder setUpdatedContractBuilder(Contract.ContractBuilder updatedContract) {
			this.updatedContract = updatedContract;
			return this;
		}
		
		public ContractState build() {
			return new ContractState(this);
		}
	
		@Override
		public ContractStateBuilder prune() {
			if (contract!=null && !contract.prune().hasData()) contract = null;
			if (updatedContract!=null && !updatedContract.prune().hasData()) updatedContract = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getContract()!=null && getContract().hasData()) return true;
			if (getUpdatedContract()!=null && getUpdatedContract().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("contract"), processor, Contract.class, contract);
			processRosetta(path.newSubPath("updatedContract"), processor, Contract.class, updatedContract);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			ContractStateBuilder _that = (ContractStateBuilder) o;
		
			if (contract != null ? !contract.equals(_that.contract) : _that.contract != null) return false;
			if (updatedContract != null ? !updatedContract.equals(_that.updatedContract) : _that.updatedContract != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (contract != null ? contract.hashCode() : 0);
			_result = 31 * _result + (updatedContract != null ? updatedContract.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "ContractStateBuilder {" +
				"contract=" + this.contract + ", " +
				"updatedContract=" + this.updatedContract +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("contract"), processor, Contract.class, contract);
		processRosetta(path.newSubPath("updatedContract"), processor, Contract.class, updatedContract);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		ContractState _that = (ContractState) o;
	
		if (contract != null ? !contract.equals(_that.contract) : _that.contract != null) return false;
		if (updatedContract != null ? !updatedContract.equals(_that.updatedContract) : _that.updatedContract != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (contract != null ? contract.hashCode() : 0);
		_result = 31 * _result + (updatedContract != null ? updatedContract.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "ContractState {" +
			"contract=" + this.contract + ", " +
			"updatedContract=" + this.updatedContract +
		'}';
	}
}
