package org.isda.cdm;

import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.EconomicTerms;
import org.isda.cdm.ProductIdentification;
import org.isda.cdm.ProductTaxonomy;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.ContractualProductMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;
		import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;

/**
 * A class to specify the contractual products&#39; economic terms, alongside their product identification and product taxonomy. The contractual product class is meant to be used across the pre-execution, execution and (as part of the Contract) post-execution lifecycle contexts.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class ContractualProduct extends RosettaModelObject {
	private final EconomicTerms economicTerms;
	private final ProductIdentification productIdentification;
	private final List<ProductTaxonomy> productTaxonomy;
	private static ContractualProductMeta metaData = new ContractualProductMeta();
	
	ContractualProduct(ContractualProductBuilder builder) {
		this.economicTerms = ofNullable(builder.getEconomicTerms()).map(f->f.build()).orElse(null);
		this.productIdentification = ofNullable(builder.getProductIdentification()).map(f->f.build()).orElse(null);
		this.productTaxonomy = ofNullable(builder.getProductTaxonomy()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
	}
	
	/**
	 * The economic terms associated with a contractual product, i.e. the set of features that are price-forming.
	 */
	public final EconomicTerms getEconomicTerms() {
		return economicTerms;
	}
	
	/**
	 * The product identification value(s) that might be associated with a contractual product. The CDM provides the ability to associate several product identification methods with a product.
	 */
	public final ProductIdentification getProductIdentification() {
		return productIdentification;
	}
	
	/**
	 * The product taxonomy value(s) associated with a contractual product.
	 */
	public final List<ProductTaxonomy> getProductTaxonomy() {
		return productTaxonomy;
	}
	
	
	@Override
	public RosettaMetaData<? extends ContractualProduct> metaData() {
		return metaData;
	} 
	
	public ContractualProductBuilder toBuilder() {
		ContractualProductBuilder builder = new ContractualProductBuilder();
		ofNullable(getEconomicTerms()).ifPresent(builder::setEconomicTerms);
		ofNullable(getProductIdentification()).ifPresent(builder::setProductIdentification);
		ofNullable(getProductTaxonomy()).ifPresent(productTaxonomy -> productTaxonomy.forEach(builder::addProductTaxonomy));
		return builder;
	}

	public static ContractualProductBuilder builder() {
		return new ContractualProductBuilder();
	}

	public static  class ContractualProductBuilder extends RosettaModelObjectBuilder implements Qualified{
	
		protected EconomicTerms.EconomicTermsBuilder economicTerms;
		protected ProductIdentification.ProductIdentificationBuilder productIdentification;
		protected List<ProductTaxonomy.ProductTaxonomyBuilder> productTaxonomy;
	
		public ContractualProductBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends ContractualProduct> metaData() {
			return metaData;
		} 
	
		
		public EconomicTerms.EconomicTermsBuilder getEconomicTerms() {
			return economicTerms;
		}
		
		public EconomicTerms.EconomicTermsBuilder getOrCreateEconomicTerms() {
			if (economicTerms!=null) {
				return economicTerms;
			}
			else return economicTerms = new EconomicTerms.EconomicTermsBuilder();
		}
		
		
		public ProductIdentification.ProductIdentificationBuilder getProductIdentification() {
			return productIdentification;
		}
		
		public ProductIdentification.ProductIdentificationBuilder getOrCreateProductIdentification() {
			if (productIdentification!=null) {
				return productIdentification;
			}
			else return productIdentification = new ProductIdentification.ProductIdentificationBuilder();
		}
		
		
		public List<ProductTaxonomy.ProductTaxonomyBuilder> getProductTaxonomy() {
			return productTaxonomy;
		}
		
		public ProductTaxonomy.ProductTaxonomyBuilder getOrCreateProductTaxonomy(int index) {
			if (productTaxonomy==null) {
				this.productTaxonomy = new ArrayList<>();
			}
			return getIndex(productTaxonomy, index, ()->new ProductTaxonomy.ProductTaxonomyBuilder());
		}
		
	
		public ContractualProductBuilder setEconomicTerms(EconomicTerms economicTerms) {
			this.economicTerms = economicTerms.toBuilder();
			return this;
		}
		public ContractualProductBuilder setEconomicTermsBuilder(EconomicTerms.EconomicTermsBuilder economicTerms) {
			this.economicTerms = economicTerms;
			return this;
		}
		public ContractualProductBuilder setProductIdentification(ProductIdentification productIdentification) {
			this.productIdentification = productIdentification.toBuilder();
			return this;
		}
		public ContractualProductBuilder setProductIdentificationBuilder(ProductIdentification.ProductIdentificationBuilder productIdentification) {
			this.productIdentification = productIdentification;
			return this;
		}
		public ContractualProductBuilder addProductTaxonomy(ProductTaxonomy productTaxonomy) {
			if(this.productTaxonomy == null){
				this.productTaxonomy = new ArrayList<>();
			}
			this.productTaxonomy.add(productTaxonomy.toBuilder());
			return this;
		}
		public ContractualProductBuilder addProductTaxonomy(ProductTaxonomy productTaxonomy, int _idx) {
			if(this.productTaxonomy == null){
				this.productTaxonomy = new ArrayList<>();
			}
			getIndex(this.productTaxonomy, _idx, () -> productTaxonomy.toBuilder());
			this.productTaxonomy.set(_idx, productTaxonomy.toBuilder());
			return this;
		}
		public ContractualProductBuilder addProductTaxonomy(List<ProductTaxonomy> productTaxonomys) {
			if(this.productTaxonomy == null){
				this.productTaxonomy = new ArrayList<>();
			}
			for (ProductTaxonomy toAdd : productTaxonomys) {
				this.productTaxonomy.add(toAdd.toBuilder());
			}
			return this;
		}
		public ContractualProductBuilder addProductTaxonomyBuilder(ProductTaxonomy.ProductTaxonomyBuilder productTaxonomy) {
			if(this.productTaxonomy == null){
				this.productTaxonomy = new ArrayList<>();
			}
			this.productTaxonomy.add(productTaxonomy);
			return this;
		}
		
		public ContractualProductBuilder clearProductTaxonomy() {
			this.productTaxonomy = null;
			return this;
		}
	public void setQualification(String qualification) {
		this.getOrCreateProductIdentification().setProductQualifier(qualification);
	}
		
		public ContractualProduct build() {
			return new ContractualProduct(this);
		}
	
		@Override
		public ContractualProductBuilder prune() {
			if (economicTerms!=null && !economicTerms.prune().hasData()) economicTerms = null;
			if (productIdentification!=null && !productIdentification.prune().hasData()) productIdentification = null;
			if (productTaxonomy!=null) productTaxonomy = productTaxonomy.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getEconomicTerms()!=null && getEconomicTerms().hasData()) return true;
			if (getProductIdentification()!=null && getProductIdentification().hasData()) return true;
			if (getProductTaxonomy()!=null && getProductTaxonomy().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("economicTerms"), processor, EconomicTerms.class, economicTerms);
			processRosetta(path.newSubPath("productIdentification"), processor, ProductIdentification.class, productIdentification);
			processRosetta(path.newSubPath("productTaxonomy"), processor, ProductTaxonomy.class, productTaxonomy);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			ContractualProductBuilder _that = (ContractualProductBuilder) o;
		
			if (economicTerms != null ? !economicTerms.equals(_that.economicTerms) : _that.economicTerms != null) return false;
			if (productIdentification != null ? !productIdentification.equals(_that.productIdentification) : _that.productIdentification != null) return false;
			if (!ListEquals.listEquals(productTaxonomy, _that.productTaxonomy)) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (economicTerms != null ? economicTerms.hashCode() : 0);
			_result = 31 * _result + (productIdentification != null ? productIdentification.hashCode() : 0);
			_result = 31 * _result + (productTaxonomy != null ? productTaxonomy.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "ContractualProductBuilder {" +
				"economicTerms=" + this.economicTerms + ", " +
				"productIdentification=" + this.productIdentification + ", " +
				"productTaxonomy=" + this.productTaxonomy +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("economicTerms"), processor, EconomicTerms.class, economicTerms);
		processRosetta(path.newSubPath("productIdentification"), processor, ProductIdentification.class, productIdentification);
		processRosetta(path.newSubPath("productTaxonomy"), processor, ProductTaxonomy.class, productTaxonomy);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		ContractualProduct _that = (ContractualProduct) o;
	
		if (economicTerms != null ? !economicTerms.equals(_that.economicTerms) : _that.economicTerms != null) return false;
		if (productIdentification != null ? !productIdentification.equals(_that.productIdentification) : _that.productIdentification != null) return false;
		if (!ListEquals.listEquals(productTaxonomy, _that.productTaxonomy)) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (economicTerms != null ? economicTerms.hashCode() : 0);
		_result = 31 * _result + (productIdentification != null ? productIdentification.hashCode() : 0);
		_result = 31 * _result + (productTaxonomy != null ? productTaxonomy.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "ContractualProduct {" +
			"economicTerms=" + this.economicTerms + ", " +
			"productIdentification=" + this.productIdentification + ", " +
			"productTaxonomy=" + this.productTaxonomy +
		'}';
	}
}
