package org.isda.cdm;

import org.isda.cdm.FutureValueAmount;
import org.isda.cdm.FxLinkedNotionalSchedule;
import org.isda.cdm.NotionalSchedule;
import org.isda.cdm.Quantity;
import org.isda.cdm.metafields.ReferenceWithMetaMoney;
import org.isda.cdm.metafields.ReferenceWithMetaMoney.ReferenceWithMetaMoneyBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.ContractualQuantityMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;
		import java.util.stream.Collectors;

/**
 * [SCHEDULED FOR DEPRECATION: Class to be emptied-out as products get progressively migrated to the restructured quantity mechanism.] A class to specify the quantity or notional amount that is associated with a contractual product and that is the base for the payout calculation. The quantity attribute applies to products relating to securities or tangible assets (such as equities or commodities), while the notional amount applies to products pertaining to interest rate, FX or credit products.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class ContractualQuantity extends RosettaModelObject {
	private final FutureValueAmount futureValueNotional;
	private final FxLinkedNotionalSchedule fxLinkedNotional;
	private final ReferenceWithMetaMoney notionalAmount;
	private final NotionalSchedule notionalSchedule;
	private final Quantity quantity;
	private static ContractualQuantityMeta metaData = new ContractualQuantityMeta();
	
	ContractualQuantity(ContractualQuantityBuilder builder) {
		this.futureValueNotional = ofNullable(builder.getFutureValueNotional()).map(f->f.build()).orElse(null);
		this.fxLinkedNotional = ofNullable(builder.getFxLinkedNotional()).map(f->f.build()).orElse(null);
		this.notionalAmount = ofNullable(builder.getNotionalAmount()).map(f->f.build()).orElse(null);
		this.notionalSchedule = ofNullable(builder.getNotionalSchedule()).map(f->f.build()).orElse(null);
		this.quantity = ofNullable(builder.getQuantity()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The future value notional is specific to BRL CDI swaps, and is specified alongside the notional amount. The value is calculated as follows: Future Value Notional = Notional Amount * (1 + Fixed Rate) ^ (Fixed Rate Day Count Fraction). The currency should always match that expressed in the notional schedule. The value date should match the adjusted termination date.
	 */
	@RosettaSynonym(value="futureValueNotional", source="FpML_5_10", path="calculationPeriodAmount.calculation")
	@RosettaSynonym(value="futureValueNotional", source="CME_SubmissionIRS_1_0", path="calculationPeriodAmount.calculation")
	@RosettaSynonym(value="futureValueNotional", source="DTCC_11_0", path="calculationPeriodAmount.calculation")
	@RosettaSynonym(value="futureValueNotional", source="DTCC_9_0", path="calculationPeriodAmount.calculation")
	@RosettaSynonym(value="futureValueNotional", source="CME_ClearedConfirm_1_17", path="calculationPeriodAmount.calculation")
	public final FutureValueAmount getFutureValueNotional() {
		return futureValueNotional;
	}
	
	/**
	 * The contractual quantity when specified as a notional amount which value is a function of FX parameters.
	 */
	@RosettaSynonym(value="fxLinkedNotionalSchedule", source="FpML_5_10", path="calculationPeriodAmount.calculation")
	@RosettaSynonym(value="fxLinkedNotionalSchedule", source="CME_SubmissionIRS_1_0", path="calculationPeriodAmount.calculation")
	@RosettaSynonym(value="fxLinkedNotionalSchedule", source="DTCC_11_0", path="calculationPeriodAmount.calculation")
	@RosettaSynonym(value="fxLinkedNotionalSchedule", source="DTCC_9_0", path="calculationPeriodAmount.calculation")
	@RosettaSynonym(value="fxLinkedNotionalSchedule", source="CME_ClearedConfirm_1_17", path="calculationPeriodAmount.calculation")
	public final FxLinkedNotionalSchedule getFxLinkedNotional() {
		return fxLinkedNotional;
	}
	
	/**
	 * The contractual quantity when specified as an amount and a currency units without associated schedule or FX terms. When applied to Credit default Swaps, this is the notional amount of protection coverage. ISDA 2003 Term: Floating Rate Payer Calculation Amount.
	 */
	@RosettaSynonym(value="initialValue", source="FpML_5_10", path="calculationPeriodAmount.calculation")
	@RosettaSynonym(value="initialValue", source="CME_SubmissionIRS_1_0", path="calculationPeriodAmount.calculation")
	@RosettaSynonym(value="initialValue", source="DTCC_11_0", path="calculationPeriodAmount.calculation")
	@RosettaSynonym(value="initialValue", source="DTCC_9_0", path="calculationPeriodAmount.calculation")
	@RosettaSynonym(value="initialValue", source="CME_ClearedConfirm_1_17", path="calculationPeriodAmount.calculation")
	@RosettaSynonym(value="calculationAmount", source="FpML_5_10", path="periodicPayment.fixedAmountCalculation")
	@RosettaSynonym(value="calculationAmount", source="CME_SubmissionIRS_1_0", path="periodicPayment.fixedAmountCalculation")
	@RosettaSynonym(value="calculationAmount", source="DTCC_11_0", path="periodicPayment.fixedAmountCalculation")
	@RosettaSynonym(value="calculationAmount", source="DTCC_9_0", path="periodicPayment.fixedAmountCalculation")
	@RosettaSynonym(value="calculationAmount", source="CME_ClearedConfirm_1_17", path="periodicPayment.fixedAmountCalculation")
	@RosettaSynonym(value="calculationAmount", source="FpML_5_10", path="periodicPayment.floatingAmountCalculation")
	@RosettaSynonym(value="calculationAmount", source="CME_SubmissionIRS_1_0", path="periodicPayment.floatingAmountCalculation")
	@RosettaSynonym(value="calculationAmount", source="DTCC_11_0", path="periodicPayment.floatingAmountCalculation")
	@RosettaSynonym(value="calculationAmount", source="DTCC_9_0", path="periodicPayment.floatingAmountCalculation")
	@RosettaSynonym(value="calculationAmount", source="CME_ClearedConfirm_1_17", path="periodicPayment.floatingAmountCalculation")
	@RosettaSynonym(value="notionalAmount", source="Rosetta_Workbench")
	@RosettaSynonym(value="notionalAmount", source="FpML_5_10")
	@RosettaSynonym(value="notionalAmount", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="notionalAmount", source="DTCC_11_0")
	@RosettaSynonym(value="notionalAmount", source="DTCC_9_0")
	@RosettaSynonym(value="notionalAmount", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="notional", source="FpML_5_10")
	@RosettaSynonym(value="notional", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="notional", source="DTCC_11_0")
	@RosettaSynonym(value="notional", source="DTCC_9_0")
	@RosettaSynonym(value="notional", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="notionalReference", source="FpML_5_10")
	@RosettaSynonym(value="notionalReference", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="notionalReference", source="DTCC_11_0")
	@RosettaSynonym(value="notionalReference", source="DTCC_9_0")
	@RosettaSynonym(value="notionalReference", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="settlementAmount", source="FpML_5_10", path="nearLeg")
	public final ReferenceWithMetaMoney getNotionalAmount() {
		return notionalAmount;
	}
	
	/**
	 * The contractual quantity when specified as a notional amount with an associated schedule, as used in the case of an amortizing swap where the notional amount decreases over time.
	 */
	@RosettaSynonym(value="calculationElements", source="DTCC_11_0")
	@RosettaSynonym(value="calculationElements", source="DTCC_9_0")
	public final NotionalSchedule getNotionalSchedule() {
		return notionalSchedule;
	}
	
	/**
	 * The contractual quantity when specified as an amount/number of securities or tangible assets. The quantity attribute is typical used in the equity and commodity asset classes.
	 */
	public final Quantity getQuantity() {
		return quantity;
	}
	
	
	@Override
	public RosettaMetaData<? extends ContractualQuantity> metaData() {
		return metaData;
	} 
	
	public ContractualQuantityBuilder toBuilder() {
		ContractualQuantityBuilder builder = new ContractualQuantityBuilder();
		ofNullable(getFutureValueNotional()).ifPresent(builder::setFutureValueNotional);
		ofNullable(getFxLinkedNotional()).ifPresent(builder::setFxLinkedNotional);
		ofNullable(getNotionalAmount()).ifPresent(builder::setNotionalAmount);
		ofNullable(getNotionalSchedule()).ifPresent(builder::setNotionalSchedule);
		ofNullable(getQuantity()).ifPresent(builder::setQuantity);
		return builder;
	}

	public static ContractualQuantityBuilder builder() {
		return new ContractualQuantityBuilder();
	}

	public static  class ContractualQuantityBuilder extends RosettaModelObjectBuilder {
	
		protected FutureValueAmount.FutureValueAmountBuilder futureValueNotional;
		protected FxLinkedNotionalSchedule.FxLinkedNotionalScheduleBuilder fxLinkedNotional;
		protected ReferenceWithMetaMoneyBuilder notionalAmount;
		protected NotionalSchedule.NotionalScheduleBuilder notionalSchedule;
		protected Quantity.QuantityBuilder quantity;
	
		public ContractualQuantityBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends ContractualQuantity> metaData() {
			return metaData;
		} 
	
		
		public FutureValueAmount.FutureValueAmountBuilder getFutureValueNotional() {
			return futureValueNotional;
		}
		
		public FutureValueAmount.FutureValueAmountBuilder getOrCreateFutureValueNotional() {
			if (futureValueNotional!=null) {
				return futureValueNotional;
			}
			else return futureValueNotional = new FutureValueAmount.FutureValueAmountBuilder();
		}
		
		
		public FxLinkedNotionalSchedule.FxLinkedNotionalScheduleBuilder getFxLinkedNotional() {
			return fxLinkedNotional;
		}
		
		public FxLinkedNotionalSchedule.FxLinkedNotionalScheduleBuilder getOrCreateFxLinkedNotional() {
			if (fxLinkedNotional!=null) {
				return fxLinkedNotional;
			}
			else return fxLinkedNotional = new FxLinkedNotionalSchedule.FxLinkedNotionalScheduleBuilder();
		}
		
		
		public ReferenceWithMetaMoneyBuilder getNotionalAmount() {
			return notionalAmount;
		}
		
		public ReferenceWithMetaMoneyBuilder getOrCreateNotionalAmount() {
			if (notionalAmount!=null) {
				return notionalAmount;
			}
			else return notionalAmount = new ReferenceWithMetaMoneyBuilder();
		}
		
		
		public NotionalSchedule.NotionalScheduleBuilder getNotionalSchedule() {
			return notionalSchedule;
		}
		
		public NotionalSchedule.NotionalScheduleBuilder getOrCreateNotionalSchedule() {
			if (notionalSchedule!=null) {
				return notionalSchedule;
			}
			else return notionalSchedule = new NotionalSchedule.NotionalScheduleBuilder();
		}
		
		
		public Quantity.QuantityBuilder getQuantity() {
			return quantity;
		}
		
		public Quantity.QuantityBuilder getOrCreateQuantity() {
			if (quantity!=null) {
				return quantity;
			}
			else return quantity = new Quantity.QuantityBuilder();
		}
		
	
		public ContractualQuantityBuilder setFutureValueNotional(FutureValueAmount futureValueNotional) {
			this.futureValueNotional = futureValueNotional.toBuilder();
			return this;
		}
		public ContractualQuantityBuilder setFutureValueNotionalBuilder(FutureValueAmount.FutureValueAmountBuilder futureValueNotional) {
			this.futureValueNotional = futureValueNotional;
			return this;
		}
		public ContractualQuantityBuilder setFxLinkedNotional(FxLinkedNotionalSchedule fxLinkedNotional) {
			this.fxLinkedNotional = fxLinkedNotional.toBuilder();
			return this;
		}
		public ContractualQuantityBuilder setFxLinkedNotionalBuilder(FxLinkedNotionalSchedule.FxLinkedNotionalScheduleBuilder fxLinkedNotional) {
			this.fxLinkedNotional = fxLinkedNotional;
			return this;
		}
		public ContractualQuantityBuilder setNotionalAmount(ReferenceWithMetaMoney notionalAmount) {
			this.notionalAmount = notionalAmount.toBuilder();
			return this;
		}
		public ContractualQuantityBuilder setNotionalAmountBuilder(ReferenceWithMetaMoneyBuilder notionalAmount) {
			this.notionalAmount = notionalAmount;
			return this;
		}
		public ContractualQuantityBuilder setNotionalAmountRef(Money.MoneyBuilder notionalAmount) {
			return setNotionalAmount(ReferenceWithMetaMoney.builder().setValueBuilder(notionalAmount).build());
		}
		public ContractualQuantityBuilder setNotionalAmountRef(Money notionalAmount) {
			return setNotionalAmountRef(Money.builder());
		}
		public ContractualQuantityBuilder setNotionalSchedule(NotionalSchedule notionalSchedule) {
			this.notionalSchedule = notionalSchedule.toBuilder();
			return this;
		}
		public ContractualQuantityBuilder setNotionalScheduleBuilder(NotionalSchedule.NotionalScheduleBuilder notionalSchedule) {
			this.notionalSchedule = notionalSchedule;
			return this;
		}
		public ContractualQuantityBuilder setQuantity(Quantity quantity) {
			this.quantity = quantity.toBuilder();
			return this;
		}
		public ContractualQuantityBuilder setQuantityBuilder(Quantity.QuantityBuilder quantity) {
			this.quantity = quantity;
			return this;
		}
		
		public ContractualQuantity build() {
			return new ContractualQuantity(this);
		}
	
		@Override
		public ContractualQuantityBuilder prune() {
			if (futureValueNotional!=null && !futureValueNotional.prune().hasData()) futureValueNotional = null;
			if (fxLinkedNotional!=null && !fxLinkedNotional.prune().hasData()) fxLinkedNotional = null;
			if (notionalAmount!=null && !notionalAmount.prune().hasData()) notionalAmount = null;
			if (notionalSchedule!=null && !notionalSchedule.prune().hasData()) notionalSchedule = null;
			if (quantity!=null && !quantity.prune().hasData()) quantity = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getFutureValueNotional()!=null && getFutureValueNotional().hasData()) return true;
			if (getFxLinkedNotional()!=null && getFxLinkedNotional().hasData()) return true;
			if (getNotionalAmount()!=null && getNotionalAmount().hasData()) return true;
			if (getNotionalSchedule()!=null && getNotionalSchedule().hasData()) return true;
			if (getQuantity()!=null && getQuantity().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("futureValueNotional"), processor, FutureValueAmount.class, futureValueNotional);
			processRosetta(path.newSubPath("fxLinkedNotional"), processor, FxLinkedNotionalSchedule.class, fxLinkedNotional);
			processRosetta(path.newSubPath("notionalAmount"), processor, ReferenceWithMetaMoney.class, notionalAmount);
			processRosetta(path.newSubPath("notionalSchedule"), processor, NotionalSchedule.class, notionalSchedule);
			processRosetta(path.newSubPath("quantity"), processor, Quantity.class, quantity);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			ContractualQuantityBuilder _that = (ContractualQuantityBuilder) o;
		
			if (futureValueNotional != null ? !futureValueNotional.equals(_that.futureValueNotional) : _that.futureValueNotional != null) return false;
			if (fxLinkedNotional != null ? !fxLinkedNotional.equals(_that.fxLinkedNotional) : _that.fxLinkedNotional != null) return false;
			if (notionalAmount != null ? !notionalAmount.equals(_that.notionalAmount) : _that.notionalAmount != null) return false;
			if (notionalSchedule != null ? !notionalSchedule.equals(_that.notionalSchedule) : _that.notionalSchedule != null) return false;
			if (quantity != null ? !quantity.equals(_that.quantity) : _that.quantity != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (futureValueNotional != null ? futureValueNotional.hashCode() : 0);
			_result = 31 * _result + (fxLinkedNotional != null ? fxLinkedNotional.hashCode() : 0);
			_result = 31 * _result + (notionalAmount != null ? notionalAmount.hashCode() : 0);
			_result = 31 * _result + (notionalSchedule != null ? notionalSchedule.hashCode() : 0);
			_result = 31 * _result + (quantity != null ? quantity.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "ContractualQuantityBuilder {" +
				"futureValueNotional=" + this.futureValueNotional + ", " +
				"fxLinkedNotional=" + this.fxLinkedNotional + ", " +
				"notionalAmount=" + this.notionalAmount + ", " +
				"notionalSchedule=" + this.notionalSchedule + ", " +
				"quantity=" + this.quantity +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("futureValueNotional"), processor, FutureValueAmount.class, futureValueNotional);
		processRosetta(path.newSubPath("fxLinkedNotional"), processor, FxLinkedNotionalSchedule.class, fxLinkedNotional);
		processRosetta(path.newSubPath("notionalAmount"), processor, ReferenceWithMetaMoney.class, notionalAmount);
		processRosetta(path.newSubPath("notionalSchedule"), processor, NotionalSchedule.class, notionalSchedule);
		processRosetta(path.newSubPath("quantity"), processor, Quantity.class, quantity);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		ContractualQuantity _that = (ContractualQuantity) o;
	
		if (futureValueNotional != null ? !futureValueNotional.equals(_that.futureValueNotional) : _that.futureValueNotional != null) return false;
		if (fxLinkedNotional != null ? !fxLinkedNotional.equals(_that.fxLinkedNotional) : _that.fxLinkedNotional != null) return false;
		if (notionalAmount != null ? !notionalAmount.equals(_that.notionalAmount) : _that.notionalAmount != null) return false;
		if (notionalSchedule != null ? !notionalSchedule.equals(_that.notionalSchedule) : _that.notionalSchedule != null) return false;
		if (quantity != null ? !quantity.equals(_that.quantity) : _that.quantity != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (futureValueNotional != null ? futureValueNotional.hashCode() : 0);
		_result = 31 * _result + (fxLinkedNotional != null ? fxLinkedNotional.hashCode() : 0);
		_result = 31 * _result + (notionalAmount != null ? notionalAmount.hashCode() : 0);
		_result = 31 * _result + (notionalSchedule != null ? notionalSchedule.hashCode() : 0);
		_result = 31 * _result + (quantity != null ? quantity.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "ContractualQuantity {" +
			"futureValueNotional=" + this.futureValueNotional + ", " +
			"fxLinkedNotional=" + this.fxLinkedNotional + ", " +
			"notionalAmount=" + this.notionalAmount + ", " +
			"notionalSchedule=" + this.notionalSchedule + ", " +
			"quantity=" + this.quantity +
		'}';
	}
}
