package org.isda.cdm;

import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.CashSettlementTerms;
import org.isda.cdm.GeneralTerms;
import org.isda.cdm.PhysicalSettlementTerms;
import org.isda.cdm.ProtectionTerms;
import org.isda.cdm.TransactedPrice;
import org.isda.cdm.metafields.MetaFields;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.CreditDefaultPayoutMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;
		import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;

/**
 * The credit default payout specification provides the details necessary for determining when a credit payout will be triggered as well as the parameters for calculating the payout and the settlement terms. The associated rosettaKey denotes the ability to associate a hash value to the CreditDefaultPayout instantiations for the purpose of model cross-referencing, in support of functionality such as the event effect and the lineage.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class CreditDefaultPayout extends RosettaModelObject implements GlobalKey {
	private final List<CashSettlementTerms> cashSettlementTerms;
	private final GeneralTerms generalTerms;
	private final MetaFields meta;
	private final List<PhysicalSettlementTerms> physicalSettlementTerms;
	private final List<ProtectionTerms> protectionTerms;
	private final TransactedPrice transactedPrice;
	private static CreditDefaultPayoutMeta metaData = new CreditDefaultPayoutMeta();
	
	CreditDefaultPayout(CreditDefaultPayoutBuilder builder) {
		this.cashSettlementTerms = ofNullable(builder.getCashSettlementTerms()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.generalTerms = ofNullable(builder.getGeneralTerms()).map(f->f.build()).orElse(null);
		this.meta = ofNullable(builder.getMeta()).map(f->f.build()).orElse(null);
		this.physicalSettlementTerms = ofNullable(builder.getPhysicalSettlementTerms()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.protectionTerms = ofNullable(builder.getProtectionTerms()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.transactedPrice = ofNullable(builder.getTransactedPrice()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * Specifies the terms applicable to the cash settlement of a credit event.
	 */
	@RosettaSynonym(value="cashSettlementTerms", source="FpML_5_10", path="trade.creditDefaultSwap")
	@RosettaSynonym(value="cashSettlementTerms", source="CME_SubmissionIRS_1_0", path="trade.creditDefaultSwap")
	@RosettaSynonym(value="cashSettlementTerms", source="DTCC_11_0", path="trade.creditDefaultSwap")
	@RosettaSynonym(value="cashSettlementTerms", source="DTCC_9_0", path="trade.creditDefaultSwap")
	@RosettaSynonym(value="cashSettlementTerms", source="CME_ClearedConfirm_1_17", path="trade.creditDefaultSwap")
	@RosettaSynonym(value="cashSettlementTerms", source="FpML_5_10", path="creditDefaultSwap")
	@RosettaSynonym(value="cashSettlementTerms", source="CME_SubmissionIRS_1_0", path="creditDefaultSwap")
	@RosettaSynonym(value="cashSettlementTerms", source="DTCC_11_0", path="creditDefaultSwap")
	@RosettaSynonym(value="cashSettlementTerms", source="DTCC_9_0", path="creditDefaultSwap")
	@RosettaSynonym(value="cashSettlementTerms", source="CME_ClearedConfirm_1_17", path="creditDefaultSwap")
	@RosettaSynonym(value="cashSettlementTerms", source="FpML_5_10")
	@RosettaSynonym(value="cashSettlementTerms", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="cashSettlementTerms", source="DTCC_11_0")
	@RosettaSynonym(value="cashSettlementTerms", source="DTCC_9_0")
	@RosettaSynonym(value="cashSettlementTerms", source="CME_ClearedConfirm_1_17")
	public final List<CashSettlementTerms> getCashSettlementTerms() {
		return cashSettlementTerms;
	}
	
	/**
	 * The specification of the non-monetary terms for the Credit Derivative Transaction, including the buyer and seller and selected items from the ISDA 2014 Credit Definition article II, such as the reference obligation and related terms.
	 */
	@RosettaSynonym(value="generalTerms", source="FpML_5_10", path="trade.creditDefaultSwap")
	@RosettaSynonym(value="generalTerms", source="CME_SubmissionIRS_1_0", path="trade.creditDefaultSwap")
	@RosettaSynonym(value="generalTerms", source="DTCC_11_0", path="trade.creditDefaultSwap")
	@RosettaSynonym(value="generalTerms", source="DTCC_9_0", path="trade.creditDefaultSwap")
	@RosettaSynonym(value="generalTerms", source="CME_ClearedConfirm_1_17", path="trade.creditDefaultSwap")
	@RosettaSynonym(value="generalTerms", source="FpML_5_10", path="creditDefaultSwap")
	@RosettaSynonym(value="generalTerms", source="CME_SubmissionIRS_1_0", path="creditDefaultSwap")
	@RosettaSynonym(value="generalTerms", source="DTCC_11_0", path="creditDefaultSwap")
	@RosettaSynonym(value="generalTerms", source="DTCC_9_0", path="creditDefaultSwap")
	@RosettaSynonym(value="generalTerms", source="CME_ClearedConfirm_1_17", path="creditDefaultSwap")
	@RosettaSynonym(value="generalTerms", source="FpML_5_10")
	@RosettaSynonym(value="generalTerms", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="generalTerms", source="DTCC_11_0")
	@RosettaSynonym(value="generalTerms", source="DTCC_9_0")
	@RosettaSynonym(value="generalTerms", source="CME_ClearedConfirm_1_17")
	public final GeneralTerms getGeneralTerms() {
		return generalTerms;
	}
	
	public final MetaFields getMeta() {
		return meta;
	}
	
	/**
	 * Specifies the terms applicable to the physical settlement of a credit event.
	 */
	@RosettaSynonym(value="physicalSettlementTerms", source="FpML_5_10", path="trade.creditDefaultSwap")
	@RosettaSynonym(value="physicalSettlementTerms", source="CME_SubmissionIRS_1_0", path="trade.creditDefaultSwap")
	@RosettaSynonym(value="physicalSettlementTerms", source="DTCC_11_0", path="trade.creditDefaultSwap")
	@RosettaSynonym(value="physicalSettlementTerms", source="DTCC_9_0", path="trade.creditDefaultSwap")
	@RosettaSynonym(value="physicalSettlementTerms", source="CME_ClearedConfirm_1_17", path="trade.creditDefaultSwap")
	@RosettaSynonym(value="physicalSettlementTerms", source="FpML_5_10", path="creditDefaultSwap")
	@RosettaSynonym(value="physicalSettlementTerms", source="CME_SubmissionIRS_1_0", path="creditDefaultSwap")
	@RosettaSynonym(value="physicalSettlementTerms", source="DTCC_11_0", path="creditDefaultSwap")
	@RosettaSynonym(value="physicalSettlementTerms", source="DTCC_9_0", path="creditDefaultSwap")
	@RosettaSynonym(value="physicalSettlementTerms", source="CME_ClearedConfirm_1_17", path="creditDefaultSwap")
	@RosettaSynonym(value="physicalSettlementTerms", source="FpML_5_10")
	@RosettaSynonym(value="physicalSettlementTerms", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="physicalSettlementTerms", source="DTCC_11_0")
	@RosettaSynonym(value="physicalSettlementTerms", source="DTCC_9_0")
	@RosettaSynonym(value="physicalSettlementTerms", source="CME_ClearedConfirm_1_17")
	public final List<PhysicalSettlementTerms> getPhysicalSettlementTerms() {
		return physicalSettlementTerms;
	}
	
	/**
	 * Specifies the terms for calculating a payout to protect the buyer of the swap in the case of a qualified credit event. These terms include the notional amount, the applicable credit events, the reference obligation, and in the case of a CDS on mortgage-backed securities, the floatingAmountEvents.
	 */
	@RosettaSynonym(value="protectionTerms", source="FpML_5_10", path="trade.creditDefaultSwap")
	@RosettaSynonym(value="protectionTerms", source="CME_SubmissionIRS_1_0", path="trade.creditDefaultSwap")
	@RosettaSynonym(value="protectionTerms", source="DTCC_11_0", path="trade.creditDefaultSwap")
	@RosettaSynonym(value="protectionTerms", source="DTCC_9_0", path="trade.creditDefaultSwap")
	@RosettaSynonym(value="protectionTerms", source="CME_ClearedConfirm_1_17", path="trade.creditDefaultSwap")
	@RosettaSynonym(value="protectionTerms", source="FpML_5_10", path="creditDefaultSwap")
	@RosettaSynonym(value="protectionTerms", source="CME_SubmissionIRS_1_0", path="creditDefaultSwap")
	@RosettaSynonym(value="protectionTerms", source="DTCC_11_0", path="creditDefaultSwap")
	@RosettaSynonym(value="protectionTerms", source="DTCC_9_0", path="creditDefaultSwap")
	@RosettaSynonym(value="protectionTerms", source="CME_ClearedConfirm_1_17", path="creditDefaultSwap")
	@RosettaSynonym(value="protectionTerms", source="FpML_5_10")
	@RosettaSynonym(value="protectionTerms", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="protectionTerms", source="DTCC_11_0")
	@RosettaSynonym(value="protectionTerms", source="DTCC_9_0")
	@RosettaSynonym(value="protectionTerms", source="CME_ClearedConfirm_1_17")
	public final List<ProtectionTerms> getProtectionTerms() {
		return protectionTerms;
	}
	
	/**
	 * The qualification of the price at which the contract has been transacted, in terms of market fixed rate, initial points, market price and/or quotation style. In FpML, those attributes are positioned as part of the fee leg.
	 */
	public final TransactedPrice getTransactedPrice() {
		return transactedPrice;
	}
	
	
	@Override
	public RosettaMetaData<? extends CreditDefaultPayout> metaData() {
		return metaData;
	} 
	
	public CreditDefaultPayoutBuilder toBuilder() {
		CreditDefaultPayoutBuilder builder = new CreditDefaultPayoutBuilder();
		ofNullable(getCashSettlementTerms()).ifPresent(cashSettlementTerms -> cashSettlementTerms.forEach(builder::addCashSettlementTerms));
		ofNullable(getGeneralTerms()).ifPresent(builder::setGeneralTerms);
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		ofNullable(getPhysicalSettlementTerms()).ifPresent(physicalSettlementTerms -> physicalSettlementTerms.forEach(builder::addPhysicalSettlementTerms));
		ofNullable(getProtectionTerms()).ifPresent(protectionTerms -> protectionTerms.forEach(builder::addProtectionTerms));
		ofNullable(getTransactedPrice()).ifPresent(builder::setTransactedPrice);
		return builder;
	}

	public static CreditDefaultPayoutBuilder builder() {
		return new CreditDefaultPayoutBuilder();
	}

	public static  class CreditDefaultPayoutBuilder extends RosettaModelObjectBuilder implements GlobalKeyBuilder<CreditDefaultPayoutBuilder> {
	
		protected List<CashSettlementTerms.CashSettlementTermsBuilder> cashSettlementTerms;
		protected GeneralTerms.GeneralTermsBuilder generalTerms;
		protected MetaFields.MetaFieldsBuilder meta;
		protected List<PhysicalSettlementTerms.PhysicalSettlementTermsBuilder> physicalSettlementTerms;
		protected List<ProtectionTerms.ProtectionTermsBuilder> protectionTerms;
		protected TransactedPrice.TransactedPriceBuilder transactedPrice;
	
		public CreditDefaultPayoutBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends CreditDefaultPayout> metaData() {
			return metaData;
		} 
	
		
		public List<CashSettlementTerms.CashSettlementTermsBuilder> getCashSettlementTerms() {
			return cashSettlementTerms;
		}
		
		public CashSettlementTerms.CashSettlementTermsBuilder getOrCreateCashSettlementTerms(int index) {
			if (cashSettlementTerms==null) {
				this.cashSettlementTerms = new ArrayList<>();
			}
			return getIndex(cashSettlementTerms, index, ()->new CashSettlementTerms.CashSettlementTermsBuilder());
		}
		
		
		public GeneralTerms.GeneralTermsBuilder getGeneralTerms() {
			return generalTerms;
		}
		
		public GeneralTerms.GeneralTermsBuilder getOrCreateGeneralTerms() {
			if (generalTerms!=null) {
				return generalTerms;
			}
			else return generalTerms = new GeneralTerms.GeneralTermsBuilder();
		}
		
		
		public MetaFields.MetaFieldsBuilder getMeta() {
			return meta;
		}
		
		public MetaFields.MetaFieldsBuilder getOrCreateMeta() {
			if (meta!=null) {
				return meta;
			}
			else return meta = new MetaFields.MetaFieldsBuilder();
		}
		
		
		public List<PhysicalSettlementTerms.PhysicalSettlementTermsBuilder> getPhysicalSettlementTerms() {
			return physicalSettlementTerms;
		}
		
		public PhysicalSettlementTerms.PhysicalSettlementTermsBuilder getOrCreatePhysicalSettlementTerms(int index) {
			if (physicalSettlementTerms==null) {
				this.physicalSettlementTerms = new ArrayList<>();
			}
			return getIndex(physicalSettlementTerms, index, ()->new PhysicalSettlementTerms.PhysicalSettlementTermsBuilder());
		}
		
		
		public List<ProtectionTerms.ProtectionTermsBuilder> getProtectionTerms() {
			return protectionTerms;
		}
		
		public ProtectionTerms.ProtectionTermsBuilder getOrCreateProtectionTerms(int index) {
			if (protectionTerms==null) {
				this.protectionTerms = new ArrayList<>();
			}
			return getIndex(protectionTerms, index, ()->new ProtectionTerms.ProtectionTermsBuilder());
		}
		
		
		public TransactedPrice.TransactedPriceBuilder getTransactedPrice() {
			return transactedPrice;
		}
		
		public TransactedPrice.TransactedPriceBuilder getOrCreateTransactedPrice() {
			if (transactedPrice!=null) {
				return transactedPrice;
			}
			else return transactedPrice = new TransactedPrice.TransactedPriceBuilder();
		}
		
	
		public CreditDefaultPayoutBuilder addCashSettlementTerms(CashSettlementTerms cashSettlementTerms) {
			if(this.cashSettlementTerms == null){
				this.cashSettlementTerms = new ArrayList<>();
			}
			this.cashSettlementTerms.add(cashSettlementTerms.toBuilder());
			return this;
		}
		public CreditDefaultPayoutBuilder addCashSettlementTerms(CashSettlementTerms cashSettlementTerms, int _idx) {
			if(this.cashSettlementTerms == null){
				this.cashSettlementTerms = new ArrayList<>();
			}
			getIndex(this.cashSettlementTerms, _idx, () -> cashSettlementTerms.toBuilder());
			this.cashSettlementTerms.set(_idx, cashSettlementTerms.toBuilder());
			return this;
		}
		public CreditDefaultPayoutBuilder addCashSettlementTerms(List<CashSettlementTerms> cashSettlementTermss) {
			if(this.cashSettlementTerms == null){
				this.cashSettlementTerms = new ArrayList<>();
			}
			for (CashSettlementTerms toAdd : cashSettlementTermss) {
				this.cashSettlementTerms.add(toAdd.toBuilder());
			}
			return this;
		}
		public CreditDefaultPayoutBuilder addCashSettlementTermsBuilder(CashSettlementTerms.CashSettlementTermsBuilder cashSettlementTerms) {
			if(this.cashSettlementTerms == null){
				this.cashSettlementTerms = new ArrayList<>();
			}
			this.cashSettlementTerms.add(cashSettlementTerms);
			return this;
		}
		
		public CreditDefaultPayoutBuilder clearCashSettlementTerms() {
			this.cashSettlementTerms = null;
			return this;
		}
		public CreditDefaultPayoutBuilder setGeneralTerms(GeneralTerms generalTerms) {
			this.generalTerms = generalTerms.toBuilder();
			return this;
		}
		public CreditDefaultPayoutBuilder setGeneralTermsBuilder(GeneralTerms.GeneralTermsBuilder generalTerms) {
			this.generalTerms = generalTerms;
			return this;
		}
		public CreditDefaultPayoutBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		public CreditDefaultPayoutBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		public CreditDefaultPayoutBuilder addPhysicalSettlementTerms(PhysicalSettlementTerms physicalSettlementTerms) {
			if(this.physicalSettlementTerms == null){
				this.physicalSettlementTerms = new ArrayList<>();
			}
			this.physicalSettlementTerms.add(physicalSettlementTerms.toBuilder());
			return this;
		}
		public CreditDefaultPayoutBuilder addPhysicalSettlementTerms(PhysicalSettlementTerms physicalSettlementTerms, int _idx) {
			if(this.physicalSettlementTerms == null){
				this.physicalSettlementTerms = new ArrayList<>();
			}
			getIndex(this.physicalSettlementTerms, _idx, () -> physicalSettlementTerms.toBuilder());
			this.physicalSettlementTerms.set(_idx, physicalSettlementTerms.toBuilder());
			return this;
		}
		public CreditDefaultPayoutBuilder addPhysicalSettlementTerms(List<PhysicalSettlementTerms> physicalSettlementTermss) {
			if(this.physicalSettlementTerms == null){
				this.physicalSettlementTerms = new ArrayList<>();
			}
			for (PhysicalSettlementTerms toAdd : physicalSettlementTermss) {
				this.physicalSettlementTerms.add(toAdd.toBuilder());
			}
			return this;
		}
		public CreditDefaultPayoutBuilder addPhysicalSettlementTermsBuilder(PhysicalSettlementTerms.PhysicalSettlementTermsBuilder physicalSettlementTerms) {
			if(this.physicalSettlementTerms == null){
				this.physicalSettlementTerms = new ArrayList<>();
			}
			this.physicalSettlementTerms.add(physicalSettlementTerms);
			return this;
		}
		
		public CreditDefaultPayoutBuilder clearPhysicalSettlementTerms() {
			this.physicalSettlementTerms = null;
			return this;
		}
		public CreditDefaultPayoutBuilder addProtectionTerms(ProtectionTerms protectionTerms) {
			if(this.protectionTerms == null){
				this.protectionTerms = new ArrayList<>();
			}
			this.protectionTerms.add(protectionTerms.toBuilder());
			return this;
		}
		public CreditDefaultPayoutBuilder addProtectionTerms(ProtectionTerms protectionTerms, int _idx) {
			if(this.protectionTerms == null){
				this.protectionTerms = new ArrayList<>();
			}
			getIndex(this.protectionTerms, _idx, () -> protectionTerms.toBuilder());
			this.protectionTerms.set(_idx, protectionTerms.toBuilder());
			return this;
		}
		public CreditDefaultPayoutBuilder addProtectionTerms(List<ProtectionTerms> protectionTermss) {
			if(this.protectionTerms == null){
				this.protectionTerms = new ArrayList<>();
			}
			for (ProtectionTerms toAdd : protectionTermss) {
				this.protectionTerms.add(toAdd.toBuilder());
			}
			return this;
		}
		public CreditDefaultPayoutBuilder addProtectionTermsBuilder(ProtectionTerms.ProtectionTermsBuilder protectionTerms) {
			if(this.protectionTerms == null){
				this.protectionTerms = new ArrayList<>();
			}
			this.protectionTerms.add(protectionTerms);
			return this;
		}
		
		public CreditDefaultPayoutBuilder clearProtectionTerms() {
			this.protectionTerms = null;
			return this;
		}
		public CreditDefaultPayoutBuilder setTransactedPrice(TransactedPrice transactedPrice) {
			this.transactedPrice = transactedPrice.toBuilder();
			return this;
		}
		public CreditDefaultPayoutBuilder setTransactedPriceBuilder(TransactedPrice.TransactedPriceBuilder transactedPrice) {
			this.transactedPrice = transactedPrice;
			return this;
		}
		
		public CreditDefaultPayout build() {
			return new CreditDefaultPayout(this);
		}
	
		@Override
		public CreditDefaultPayoutBuilder prune() {
			if (cashSettlementTerms!=null) cashSettlementTerms = cashSettlementTerms.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (generalTerms!=null && !generalTerms.prune().hasData()) generalTerms = null;
			if (meta!=null && !meta.prune().hasData()) meta = null;
			if (physicalSettlementTerms!=null) physicalSettlementTerms = physicalSettlementTerms.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (protectionTerms!=null) protectionTerms = protectionTerms.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (transactedPrice!=null && !transactedPrice.prune().hasData()) transactedPrice = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getCashSettlementTerms()!=null && getCashSettlementTerms().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getGeneralTerms()!=null && getGeneralTerms().hasData()) return true;
			if (getMeta()!=null && getMeta().hasData()) return true;
			if (getPhysicalSettlementTerms()!=null && getPhysicalSettlementTerms().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getProtectionTerms()!=null && getProtectionTerms().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getTransactedPrice()!=null && getTransactedPrice().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("cashSettlementTerms"), processor, CashSettlementTerms.class, cashSettlementTerms);
			processRosetta(path.newSubPath("generalTerms"), processor, GeneralTerms.class, generalTerms);
			processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
			processRosetta(path.newSubPath("physicalSettlementTerms"), processor, PhysicalSettlementTerms.class, physicalSettlementTerms);
			processRosetta(path.newSubPath("protectionTerms"), processor, ProtectionTerms.class, protectionTerms);
			processRosetta(path.newSubPath("transactedPrice"), processor, TransactedPrice.class, transactedPrice);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			CreditDefaultPayoutBuilder _that = (CreditDefaultPayoutBuilder) o;
		
			if (!ListEquals.listEquals(cashSettlementTerms, _that.cashSettlementTerms)) return false;
			if (generalTerms != null ? !generalTerms.equals(_that.generalTerms) : _that.generalTerms != null) return false;
			if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
			if (!ListEquals.listEquals(physicalSettlementTerms, _that.physicalSettlementTerms)) return false;
			if (!ListEquals.listEquals(protectionTerms, _that.protectionTerms)) return false;
			if (transactedPrice != null ? !transactedPrice.equals(_that.transactedPrice) : _that.transactedPrice != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (cashSettlementTerms != null ? cashSettlementTerms.hashCode() : 0);
			_result = 31 * _result + (generalTerms != null ? generalTerms.hashCode() : 0);
			_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
			_result = 31 * _result + (physicalSettlementTerms != null ? physicalSettlementTerms.hashCode() : 0);
			_result = 31 * _result + (protectionTerms != null ? protectionTerms.hashCode() : 0);
			_result = 31 * _result + (transactedPrice != null ? transactedPrice.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "CreditDefaultPayoutBuilder {" +
				"cashSettlementTerms=" + this.cashSettlementTerms + ", " +
				"generalTerms=" + this.generalTerms + ", " +
				"meta=" + this.meta + ", " +
				"physicalSettlementTerms=" + this.physicalSettlementTerms + ", " +
				"protectionTerms=" + this.protectionTerms + ", " +
				"transactedPrice=" + this.transactedPrice +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("cashSettlementTerms"), processor, CashSettlementTerms.class, cashSettlementTerms);
		processRosetta(path.newSubPath("generalTerms"), processor, GeneralTerms.class, generalTerms);
		processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
		processRosetta(path.newSubPath("physicalSettlementTerms"), processor, PhysicalSettlementTerms.class, physicalSettlementTerms);
		processRosetta(path.newSubPath("protectionTerms"), processor, ProtectionTerms.class, protectionTerms);
		processRosetta(path.newSubPath("transactedPrice"), processor, TransactedPrice.class, transactedPrice);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		CreditDefaultPayout _that = (CreditDefaultPayout) o;
	
		if (!ListEquals.listEquals(cashSettlementTerms, _that.cashSettlementTerms)) return false;
		if (generalTerms != null ? !generalTerms.equals(_that.generalTerms) : _that.generalTerms != null) return false;
		if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
		if (!ListEquals.listEquals(physicalSettlementTerms, _that.physicalSettlementTerms)) return false;
		if (!ListEquals.listEquals(protectionTerms, _that.protectionTerms)) return false;
		if (transactedPrice != null ? !transactedPrice.equals(_that.transactedPrice) : _that.transactedPrice != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (cashSettlementTerms != null ? cashSettlementTerms.hashCode() : 0);
		_result = 31 * _result + (generalTerms != null ? generalTerms.hashCode() : 0);
		_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
		_result = 31 * _result + (physicalSettlementTerms != null ? physicalSettlementTerms.hashCode() : 0);
		_result = 31 * _result + (protectionTerms != null ? protectionTerms.hashCode() : 0);
		_result = 31 * _result + (transactedPrice != null ? transactedPrice.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "CreditDefaultPayout {" +
			"cashSettlementTerms=" + this.cashSettlementTerms + ", " +
			"generalTerms=" + this.generalTerms + ", " +
			"meta=" + this.meta + ", " +
			"physicalSettlementTerms=" + this.physicalSettlementTerms + ", " +
			"protectionTerms=" + this.protectionTerms + ", " +
			"transactedPrice=" + this.transactedPrice +
		'}';
	}
}
