package org.isda.cdm;

import org.isda.cdm.BusinessCenterEnum;
import org.isda.cdm.NotifyingParty;
import org.isda.cdm.PubliclyAvailableInformation;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.CreditEventNoticeMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="CreditEventNotice", source="FpML_5_10")
public class CreditEventNotice extends RosettaModelObject {
	private final BusinessCenterEnum businessCenter;
	private final NotifyingParty notifyingParty;
	private final PubliclyAvailableInformation publiclyAvailableInformation;
	private static CreditEventNoticeMeta metaData = new CreditEventNoticeMeta();
	
	CreditEventNotice(CreditEventNoticeBuilder builder) {
		this.businessCenter = builder.getBusinessCenter();
		this.notifyingParty = ofNullable(builder.getNotifyingParty()).map(f->f.build()).orElse(null);
		this.publiclyAvailableInformation = ofNullable(builder.getPubliclyAvailableInformation()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * Inclusion of this business center element implies that Greenwich Mean Time in Section 3.3 of the 2003 ISDA Credit Derivatives Definitions is replaced by the local time of the city indicated by the businessCenter element value.
	 */
	@RosettaSynonym(value="businessCenter", source="FpML_5_10")
	@RosettaSynonym(value="businessCenter", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="businessCenter", source="DTCC_11_0")
	@RosettaSynonym(value="businessCenter", source="DTCC_9_0")
	@RosettaSynonym(value="businessCenter", source="CME_ClearedConfirm_1_17")
	public final BusinessCenterEnum getBusinessCenter() {
		return businessCenter;
	}
	
	/**
	 * The notifying party is the party that notifies the other party when a credit event has occurred by means of a credit event notice. If more than one party is referenced as being the notifying party then either party may notify the other of a credit event occurring. ISDA 2003 Term: Notifying Party.
	 */
	@RosettaSynonym(value="notifyingParty", source="FpML_5_10")
	@RosettaSynonym(value="notifyingParty", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="notifyingParty", source="DTCC_11_0")
	@RosettaSynonym(value="notifyingParty", source="DTCC_9_0")
	@RosettaSynonym(value="notifyingParty", source="CME_ClearedConfirm_1_17")
	public final NotifyingParty getNotifyingParty() {
		return notifyingParty;
	}
	
	/**
	 * A specified condition to settlement. Publicly available information means information that reasonably confirms any of the facts relevant to determining that a credit event or potential repudiation/moratorium, as applicable, has occurred. The ISDA defined list (2003) is the market standard and is considered comprehensive, and a minimum of two differing public sources must have published the relevant information, to declare a Credit Event. ISDA 2003 Term: Notice of Publicly Available Information Applicable.
	 */
	@RosettaSynonym(value="publiclyAvailableInformation", source="FpML_5_10")
	@RosettaSynonym(value="publiclyAvailableInformation", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="publiclyAvailableInformation", source="DTCC_11_0")
	@RosettaSynonym(value="publiclyAvailableInformation", source="DTCC_9_0")
	@RosettaSynonym(value="publiclyAvailableInformation", source="CME_ClearedConfirm_1_17")
	public final PubliclyAvailableInformation getPubliclyAvailableInformation() {
		return publiclyAvailableInformation;
	}
	
	
	@Override
	public RosettaMetaData<? extends CreditEventNotice> metaData() {
		return metaData;
	} 
	
	public CreditEventNoticeBuilder toBuilder() {
		CreditEventNoticeBuilder builder = new CreditEventNoticeBuilder();
		ofNullable(getBusinessCenter()).ifPresent(builder::setBusinessCenter);
		ofNullable(getNotifyingParty()).ifPresent(builder::setNotifyingParty);
		ofNullable(getPubliclyAvailableInformation()).ifPresent(builder::setPubliclyAvailableInformation);
		return builder;
	}

	public static CreditEventNoticeBuilder builder() {
		return new CreditEventNoticeBuilder();
	}

	public static  class CreditEventNoticeBuilder extends RosettaModelObjectBuilder {
	
		protected BusinessCenterEnum businessCenter;
		protected NotifyingParty.NotifyingPartyBuilder notifyingParty;
		protected PubliclyAvailableInformation.PubliclyAvailableInformationBuilder publiclyAvailableInformation;
	
		public CreditEventNoticeBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends CreditEventNotice> metaData() {
			return metaData;
		} 
	
		
		public BusinessCenterEnum getBusinessCenter() {
			return businessCenter;
		}
		
		
		public NotifyingParty.NotifyingPartyBuilder getNotifyingParty() {
			return notifyingParty;
		}
		
		public NotifyingParty.NotifyingPartyBuilder getOrCreateNotifyingParty() {
			if (notifyingParty!=null) {
				return notifyingParty;
			}
			else return notifyingParty = new NotifyingParty.NotifyingPartyBuilder();
		}
		
		
		public PubliclyAvailableInformation.PubliclyAvailableInformationBuilder getPubliclyAvailableInformation() {
			return publiclyAvailableInformation;
		}
		
		public PubliclyAvailableInformation.PubliclyAvailableInformationBuilder getOrCreatePubliclyAvailableInformation() {
			if (publiclyAvailableInformation!=null) {
				return publiclyAvailableInformation;
			}
			else return publiclyAvailableInformation = new PubliclyAvailableInformation.PubliclyAvailableInformationBuilder();
		}
		
	
		public CreditEventNoticeBuilder setBusinessCenter(BusinessCenterEnum businessCenter) {
			this.businessCenter = businessCenter;
			return this;
		}
		public CreditEventNoticeBuilder setNotifyingParty(NotifyingParty notifyingParty) {
			this.notifyingParty = notifyingParty.toBuilder();
			return this;
		}
		public CreditEventNoticeBuilder setNotifyingPartyBuilder(NotifyingParty.NotifyingPartyBuilder notifyingParty) {
			this.notifyingParty = notifyingParty;
			return this;
		}
		public CreditEventNoticeBuilder setPubliclyAvailableInformation(PubliclyAvailableInformation publiclyAvailableInformation) {
			this.publiclyAvailableInformation = publiclyAvailableInformation.toBuilder();
			return this;
		}
		public CreditEventNoticeBuilder setPubliclyAvailableInformationBuilder(PubliclyAvailableInformation.PubliclyAvailableInformationBuilder publiclyAvailableInformation) {
			this.publiclyAvailableInformation = publiclyAvailableInformation;
			return this;
		}
		
		public CreditEventNotice build() {
			return new CreditEventNotice(this);
		}
	
		@Override
		public CreditEventNoticeBuilder prune() {
			if (notifyingParty!=null && !notifyingParty.prune().hasData()) notifyingParty = null;
			if (publiclyAvailableInformation!=null && !publiclyAvailableInformation.prune().hasData()) publiclyAvailableInformation = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getBusinessCenter()!=null) return true;
			if (getNotifyingParty()!=null && getNotifyingParty().hasData()) return true;
			if (getPubliclyAvailableInformation()!=null && getPubliclyAvailableInformation().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("businessCenter"), BusinessCenterEnum.class, businessCenter, this);
			
			processRosetta(path.newSubPath("notifyingParty"), processor, NotifyingParty.class, notifyingParty);
			processRosetta(path.newSubPath("publiclyAvailableInformation"), processor, PubliclyAvailableInformation.class, publiclyAvailableInformation);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			CreditEventNoticeBuilder _that = (CreditEventNoticeBuilder) o;
		
			if (businessCenter != null ? !businessCenter.equals(_that.businessCenter) : _that.businessCenter != null) return false;
			if (notifyingParty != null ? !notifyingParty.equals(_that.notifyingParty) : _that.notifyingParty != null) return false;
			if (publiclyAvailableInformation != null ? !publiclyAvailableInformation.equals(_that.publiclyAvailableInformation) : _that.publiclyAvailableInformation != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (businessCenter != null ? businessCenter.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (notifyingParty != null ? notifyingParty.hashCode() : 0);
			_result = 31 * _result + (publiclyAvailableInformation != null ? publiclyAvailableInformation.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "CreditEventNoticeBuilder {" +
				"businessCenter=" + this.businessCenter + ", " +
				"notifyingParty=" + this.notifyingParty + ", " +
				"publiclyAvailableInformation=" + this.publiclyAvailableInformation +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("businessCenter"), BusinessCenterEnum.class, businessCenter, this);
		
		processRosetta(path.newSubPath("notifyingParty"), processor, NotifyingParty.class, notifyingParty);
		processRosetta(path.newSubPath("publiclyAvailableInformation"), processor, PubliclyAvailableInformation.class, publiclyAvailableInformation);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		CreditEventNotice _that = (CreditEventNotice) o;
	
		if (businessCenter != null ? !businessCenter.equals(_that.businessCenter) : _that.businessCenter != null) return false;
		if (notifyingParty != null ? !notifyingParty.equals(_that.notifyingParty) : _that.notifyingParty != null) return false;
		if (publiclyAvailableInformation != null ? !publiclyAvailableInformation.equals(_that.publiclyAvailableInformation) : _that.publiclyAvailableInformation != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (businessCenter != null ? businessCenter.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (notifyingParty != null ? notifyingParty.hashCode() : 0);
		_result = 31 * _result + (publiclyAvailableInformation != null ? publiclyAvailableInformation.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "CreditEventNotice {" +
			"businessCenter=" + this.businessCenter + ", " +
			"notifyingParty=" + this.notifyingParty + ", " +
			"publiclyAvailableInformation=" + this.publiclyAvailableInformation +
		'}';
	}
}
