package org.isda.cdm;

import java.lang.Boolean;
import org.isda.cdm.CreditEventNotice;
import org.isda.cdm.FailureToPay;
import org.isda.cdm.Money;
import org.isda.cdm.Restructuring;
import org.isda.cdm.metafields.MetaFields;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.CreditEventsMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class to specify the applicable Credit Events that would trigger a settlement, as specified in the related Confirmation and defined in the ISDA 2014 Credit Definition article IV section 4.1.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="CreditEvents", source="FpML_5_10")
@RosettaSynonym(value="CreditEvents", source="CME_SubmissionIRS_1_0")
@RosettaSynonym(value="CreditEvents", source="DTCC_11_0")
@RosettaSynonym(value="CreditEvents", source="DTCC_9_0")
@RosettaSynonym(value="CreditEvents", source="CME_ClearedConfirm_1_17")
public class CreditEvents extends RosettaModelObject implements GlobalKey {
	private final Boolean bankruptcy;
	private final CreditEventNotice creditEventNotice;
	private final Money defaultRequirement;
	private final Boolean distressedRatingsDowngrade;
	private final FailureToPay failureToPay;
	private final Boolean failureToPayInterest;
	private final Boolean failureToPayPrincipal;
	private final Boolean governmentalIntervention;
	private final Boolean impliedWritedown;
	private final Boolean maturityExtension;
	private final MetaFields meta;
	private final Boolean obligationAcceleration;
	private final Boolean obligationDefault;
	private final Boolean repudiationMoratorium;
	private final Restructuring restructuring;
	private final Boolean writedown;
	private static CreditEventsMeta metaData = new CreditEventsMeta();
	
	CreditEvents(CreditEventsBuilder builder) {
		this.bankruptcy = builder.getBankruptcy();
		this.creditEventNotice = ofNullable(builder.getCreditEventNotice()).map(f->f.build()).orElse(null);
		this.defaultRequirement = ofNullable(builder.getDefaultRequirement()).map(f->f.build()).orElse(null);
		this.distressedRatingsDowngrade = builder.getDistressedRatingsDowngrade();
		this.failureToPay = ofNullable(builder.getFailureToPay()).map(f->f.build()).orElse(null);
		this.failureToPayInterest = builder.getFailureToPayInterest();
		this.failureToPayPrincipal = builder.getFailureToPayPrincipal();
		this.governmentalIntervention = builder.getGovernmentalIntervention();
		this.impliedWritedown = builder.getImpliedWritedown();
		this.maturityExtension = builder.getMaturityExtension();
		this.meta = ofNullable(builder.getMeta()).map(f->f.build()).orElse(null);
		this.obligationAcceleration = builder.getObligationAcceleration();
		this.obligationDefault = builder.getObligationDefault();
		this.repudiationMoratorium = builder.getRepudiationMoratorium();
		this.restructuring = ofNullable(builder.getRestructuring()).map(f->f.build()).orElse(null);
		this.writedown = builder.getWritedown();
	}
	
	/**
	 * A credit event. The reference entity has been dissolved or has become insolvent. It also covers events that may be a precursor to insolvency such as instigation of bankruptcy or insolvency proceedings. Sovereign trades are not subject to Bankruptcy as &#39;technically&#39; a Sovereign cannot become bankrupt. ISDA 2003 Term: Bankruptcy.
	 */
	@RosettaSynonym(value="bankruptcy", source="FpML_5_10")
	@RosettaSynonym(value="bankruptcy", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="bankruptcy", source="DTCC_11_0")
	@RosettaSynonym(value="bankruptcy", source="DTCC_9_0")
	@RosettaSynonym(value="bankruptcy", source="CME_ClearedConfirm_1_17")
	public final Boolean getBankruptcy() {
		return bankruptcy;
	}
	
	/**
	 * A specified condition to settlement. An irrevocable written or verbal notice that describes a credit event that has occurred. The notice is sent from the notifying party (either the buyer or the seller) to the counterparty. It provides information relevant to determining that a credit event has occurred. This is typically accompanied by Publicly Available Information. ISDA 2003 Term: Credit Event Notice.
	 */
	@RosettaSynonym(value="creditEventNotice", source="FpML_5_10")
	@RosettaSynonym(value="creditEventNotice", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="creditEventNotice", source="DTCC_11_0")
	@RosettaSynonym(value="creditEventNotice", source="DTCC_9_0")
	@RosettaSynonym(value="creditEventNotice", source="CME_ClearedConfirm_1_17")
	public final CreditEventNotice getCreditEventNotice() {
		return creditEventNotice;
	}
	
	/**
	 * In relation to certain credit events, serves as a threshold for Obligation Acceleration, Obligation Default, Repudiation/Moratorium and Restructuring. Market standard is USD 10,000,000 (JPY 1,000,000,000 for all Japanese Yen trades). This is applied on an aggregate or total basis across all Obligations of the Reference Entity. Used to prevent technical/operational errors from triggering credit events. ISDA 2003 Term: Default Requirement.
	 */
	@RosettaSynonym(value="defaultRequirement", source="FpML_5_10")
	@RosettaSynonym(value="defaultRequirement", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="defaultRequirement", source="DTCC_11_0")
	@RosettaSynonym(value="defaultRequirement", source="DTCC_9_0")
	@RosettaSynonym(value="defaultRequirement", source="CME_ClearedConfirm_1_17")
	public final Money getDefaultRequirement() {
		return defaultRequirement;
	}
	
	/**
	 * A credit event. Results from the fact that the rating of the reference obligation is down-graded to a distressed rating level. From a usage standpoint, this credit event is typically not applicable in case of RMBS trades.
	 */
	@RosettaSynonym(value="distressedRatingsDowngrade", source="FpML_5_10")
	@RosettaSynonym(value="distressedRatingsDowngrade", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="distressedRatingsDowngrade", source="DTCC_11_0")
	@RosettaSynonym(value="distressedRatingsDowngrade", source="DTCC_9_0")
	@RosettaSynonym(value="distressedRatingsDowngrade", source="CME_ClearedConfirm_1_17")
	public final Boolean getDistressedRatingsDowngrade() {
		return distressedRatingsDowngrade;
	}
	
	/**
	 * A credit event. This credit event triggers, after the expiration of any applicable grace period, if the reference entity fails to make due payments in an aggregate amount of not less than the payment requirement on one or more obligations (e.g. a missed coupon payment). ISDA 2003 Term: Failure to Pay.
	 */
	@RosettaSynonym(value="failureToPay", source="FpML_5_10")
	@RosettaSynonym(value="failureToPay", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="failureToPay", source="DTCC_11_0")
	@RosettaSynonym(value="failureToPay", source="DTCC_9_0")
	@RosettaSynonym(value="failureToPay", source="CME_ClearedConfirm_1_17")
	public final FailureToPay getFailureToPay() {
		return failureToPay;
	}
	
	/**
	 * A credit event. Corresponds to the failure by the Reference Entity to pay an expected interest amount or the payment of an actual interest amount that is less than the expected interest amount. ISDA 2003 Term: Failure to Pay Interest.
	 */
	@RosettaSynonym(value="failureToPayInterest", source="FpML_5_10")
	@RosettaSynonym(value="failureToPayInterest", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="failureToPayInterest", source="DTCC_11_0")
	@RosettaSynonym(value="failureToPayInterest", source="DTCC_9_0")
	@RosettaSynonym(value="failureToPayInterest", source="CME_ClearedConfirm_1_17")
	public final Boolean getFailureToPayInterest() {
		return failureToPayInterest;
	}
	
	/**
	 * A credit event. Corresponds to the failure by the Reference Entity to pay an expected principal amount or the payment of an actual principal amount that is less than the expected principal amount. ISDA 2003 Term: Failure to Pay Principal.
	 */
	@RosettaSynonym(value="failureToPayPrincipal", source="FpML_5_10")
	@RosettaSynonym(value="failureToPayPrincipal", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="failureToPayPrincipal", source="DTCC_11_0")
	@RosettaSynonym(value="failureToPayPrincipal", source="DTCC_9_0")
	@RosettaSynonym(value="failureToPayPrincipal", source="CME_ClearedConfirm_1_17")
	public final Boolean getFailureToPayPrincipal() {
		return failureToPayPrincipal;
	}
	
	/**
	 * A credit event. A governmental intervention is an event resulting from an action by a governmental authority that materially impacts the reference entity&#39;s obligations, such as an interest rate reduction, principal reduction, deferral of interest or principal, change in priority ranking, or change in currency or composition of payment. ISDA 2014 Term: Governmental Intervention.
	 */
	@RosettaSynonym(value="governmentalIntervention", source="FpML_5_10")
	@RosettaSynonym(value="governmentalIntervention", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="governmentalIntervention", source="DTCC_11_0")
	@RosettaSynonym(value="governmentalIntervention", source="DTCC_9_0")
	@RosettaSynonym(value="governmentalIntervention", source="CME_ClearedConfirm_1_17")
	public final Boolean getGovernmentalIntervention() {
		return governmentalIntervention;
	}
	
	/**
	 * A credit event. Results from the fact that losses occur to the underlying instruments that do not result in reductions of the outstanding principal of the reference obligation.
	 */
	@RosettaSynonym(value="impliedWritedown", source="FpML_5_10")
	@RosettaSynonym(value="impliedWritedown", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="impliedWritedown", source="DTCC_11_0")
	@RosettaSynonym(value="impliedWritedown", source="DTCC_9_0")
	@RosettaSynonym(value="impliedWritedown", source="CME_ClearedConfirm_1_17")
	public final Boolean getImpliedWritedown() {
		return impliedWritedown;
	}
	
	/**
	 * A credit event. Results from the fact that the underlier fails to make principal payments as expected.
	 */
	@RosettaSynonym(value="maturityExtension", source="FpML_5_10")
	@RosettaSynonym(value="maturityExtension", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="maturityExtension", source="DTCC_11_0")
	@RosettaSynonym(value="maturityExtension", source="DTCC_9_0")
	@RosettaSynonym(value="maturityExtension", source="CME_ClearedConfirm_1_17")
	public final Boolean getMaturityExtension() {
		return maturityExtension;
	}
	
	public final MetaFields getMeta() {
		return meta;
	}
	
	/**
	 * A credit event. One or more of the obligations have been declared due and payable before they would otherwise have been due and payable as a result of, or on the basis of, the occurrence of a default, event of default or other similar condition or event other than failure to pay (preferred by the market over Obligation Default, because more definitive and encompasses the definition of Obligation Default - this is more favorable to the Seller). Subject to the default requirement amount. ISDA 2003 Term: Obligation Acceleration.
	 */
	@RosettaSynonym(value="obligationAcceleration", source="FpML_5_10")
	@RosettaSynonym(value="obligationAcceleration", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="obligationAcceleration", source="DTCC_11_0")
	@RosettaSynonym(value="obligationAcceleration", source="DTCC_9_0")
	@RosettaSynonym(value="obligationAcceleration", source="CME_ClearedConfirm_1_17")
	public final Boolean getObligationAcceleration() {
		return obligationAcceleration;
	}
	
	/**
	 * A credit event. One or more of the obligations have become capable of being declared due and payable before they would otherwise have been due and payable as a result of, or on the basis of, the occurrence of a default, event of default or other similar condition or event other than failure to pay. ISDA 2003 Term: Obligation Default.
	 */
	@RosettaSynonym(value="obligationDefault", source="FpML_5_10")
	@RosettaSynonym(value="obligationDefault", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="obligationDefault", source="DTCC_11_0")
	@RosettaSynonym(value="obligationDefault", source="DTCC_9_0")
	@RosettaSynonym(value="obligationDefault", source="CME_ClearedConfirm_1_17")
	public final Boolean getObligationDefault() {
		return obligationDefault;
	}
	
	/**
	 * A credit event. The reference entity, or a governmental authority, either refuses to recognise or challenges the validity of one or more obligations of the reference entity, or imposes a moratorium thereby postponing payments on one or more of the obligations of the reference entity. Subject to the default requirement amount. ISDA 2003 Term: Repudiation/Moratorium.
	 */
	@RosettaSynonym(value="repudiationMoratorium", source="FpML_5_10")
	@RosettaSynonym(value="repudiationMoratorium", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="repudiationMoratorium", source="DTCC_11_0")
	@RosettaSynonym(value="repudiationMoratorium", source="DTCC_9_0")
	@RosettaSynonym(value="repudiationMoratorium", source="CME_ClearedConfirm_1_17")
	public final Boolean getRepudiationMoratorium() {
		return repudiationMoratorium;
	}
	
	/**
	 * A credit event. A restructuring is an event that materially impacts the reference entity&#39;s obligations, such as an interest rate reduction, principal reduction, deferral of interest or principal, change in priority ranking, or change in currency or composition of payment. ISDA 2003 Term: Restructuring.
	 */
	@RosettaSynonym(value="restructuring", source="FpML_5_10")
	@RosettaSynonym(value="restructuring", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="restructuring", source="DTCC_11_0")
	@RosettaSynonym(value="restructuring", source="DTCC_9_0")
	@RosettaSynonym(value="restructuring", source="CME_ClearedConfirm_1_17")
	public final Restructuring getRestructuring() {
		return restructuring;
	}
	
	/**
	 * A credit event. Results from the fact that the underlier writes down its outstanding principal amount.
	 */
	@RosettaSynonym(value="writedown", source="FpML_5_10")
	@RosettaSynonym(value="writedown", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="writedown", source="DTCC_11_0")
	@RosettaSynonym(value="writedown", source="DTCC_9_0")
	@RosettaSynonym(value="writedown", source="CME_ClearedConfirm_1_17")
	public final Boolean getWritedown() {
		return writedown;
	}
	
	
	@Override
	public RosettaMetaData<? extends CreditEvents> metaData() {
		return metaData;
	} 
	
	public CreditEventsBuilder toBuilder() {
		CreditEventsBuilder builder = new CreditEventsBuilder();
		ofNullable(getBankruptcy()).ifPresent(builder::setBankruptcy);
		ofNullable(getCreditEventNotice()).ifPresent(builder::setCreditEventNotice);
		ofNullable(getDefaultRequirement()).ifPresent(builder::setDefaultRequirement);
		ofNullable(getDistressedRatingsDowngrade()).ifPresent(builder::setDistressedRatingsDowngrade);
		ofNullable(getFailureToPay()).ifPresent(builder::setFailureToPay);
		ofNullable(getFailureToPayInterest()).ifPresent(builder::setFailureToPayInterest);
		ofNullable(getFailureToPayPrincipal()).ifPresent(builder::setFailureToPayPrincipal);
		ofNullable(getGovernmentalIntervention()).ifPresent(builder::setGovernmentalIntervention);
		ofNullable(getImpliedWritedown()).ifPresent(builder::setImpliedWritedown);
		ofNullable(getMaturityExtension()).ifPresent(builder::setMaturityExtension);
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		ofNullable(getObligationAcceleration()).ifPresent(builder::setObligationAcceleration);
		ofNullable(getObligationDefault()).ifPresent(builder::setObligationDefault);
		ofNullable(getRepudiationMoratorium()).ifPresent(builder::setRepudiationMoratorium);
		ofNullable(getRestructuring()).ifPresent(builder::setRestructuring);
		ofNullable(getWritedown()).ifPresent(builder::setWritedown);
		return builder;
	}

	public static CreditEventsBuilder builder() {
		return new CreditEventsBuilder();
	}

	public static  class CreditEventsBuilder extends RosettaModelObjectBuilder implements GlobalKeyBuilder<CreditEventsBuilder> {
	
		protected Boolean bankruptcy;
		protected CreditEventNotice.CreditEventNoticeBuilder creditEventNotice;
		protected Money.MoneyBuilder defaultRequirement;
		protected Boolean distressedRatingsDowngrade;
		protected FailureToPay.FailureToPayBuilder failureToPay;
		protected Boolean failureToPayInterest;
		protected Boolean failureToPayPrincipal;
		protected Boolean governmentalIntervention;
		protected Boolean impliedWritedown;
		protected Boolean maturityExtension;
		protected MetaFields.MetaFieldsBuilder meta;
		protected Boolean obligationAcceleration;
		protected Boolean obligationDefault;
		protected Boolean repudiationMoratorium;
		protected Restructuring.RestructuringBuilder restructuring;
		protected Boolean writedown;
	
		public CreditEventsBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends CreditEvents> metaData() {
			return metaData;
		} 
	
		
		public Boolean getBankruptcy() {
			return bankruptcy;
		}
		
		
		public CreditEventNotice.CreditEventNoticeBuilder getCreditEventNotice() {
			return creditEventNotice;
		}
		
		public CreditEventNotice.CreditEventNoticeBuilder getOrCreateCreditEventNotice() {
			if (creditEventNotice!=null) {
				return creditEventNotice;
			}
			else return creditEventNotice = new CreditEventNotice.CreditEventNoticeBuilder();
		}
		
		
		public Money.MoneyBuilder getDefaultRequirement() {
			return defaultRequirement;
		}
		
		public Money.MoneyBuilder getOrCreateDefaultRequirement() {
			if (defaultRequirement!=null) {
				return defaultRequirement;
			}
			else return defaultRequirement = new Money.MoneyBuilder();
		}
		
		
		public Boolean getDistressedRatingsDowngrade() {
			return distressedRatingsDowngrade;
		}
		
		
		public FailureToPay.FailureToPayBuilder getFailureToPay() {
			return failureToPay;
		}
		
		public FailureToPay.FailureToPayBuilder getOrCreateFailureToPay() {
			if (failureToPay!=null) {
				return failureToPay;
			}
			else return failureToPay = new FailureToPay.FailureToPayBuilder();
		}
		
		
		public Boolean getFailureToPayInterest() {
			return failureToPayInterest;
		}
		
		
		public Boolean getFailureToPayPrincipal() {
			return failureToPayPrincipal;
		}
		
		
		public Boolean getGovernmentalIntervention() {
			return governmentalIntervention;
		}
		
		
		public Boolean getImpliedWritedown() {
			return impliedWritedown;
		}
		
		
		public Boolean getMaturityExtension() {
			return maturityExtension;
		}
		
		
		public MetaFields.MetaFieldsBuilder getMeta() {
			return meta;
		}
		
		public MetaFields.MetaFieldsBuilder getOrCreateMeta() {
			if (meta!=null) {
				return meta;
			}
			else return meta = new MetaFields.MetaFieldsBuilder();
		}
		
		
		public Boolean getObligationAcceleration() {
			return obligationAcceleration;
		}
		
		
		public Boolean getObligationDefault() {
			return obligationDefault;
		}
		
		
		public Boolean getRepudiationMoratorium() {
			return repudiationMoratorium;
		}
		
		
		public Restructuring.RestructuringBuilder getRestructuring() {
			return restructuring;
		}
		
		public Restructuring.RestructuringBuilder getOrCreateRestructuring() {
			if (restructuring!=null) {
				return restructuring;
			}
			else return restructuring = new Restructuring.RestructuringBuilder();
		}
		
		
		public Boolean getWritedown() {
			return writedown;
		}
		
	
		public CreditEventsBuilder setBankruptcy(Boolean bankruptcy) {
			this.bankruptcy = bankruptcy;
			return this;
		}
		public CreditEventsBuilder setCreditEventNotice(CreditEventNotice creditEventNotice) {
			this.creditEventNotice = creditEventNotice.toBuilder();
			return this;
		}
		public CreditEventsBuilder setCreditEventNoticeBuilder(CreditEventNotice.CreditEventNoticeBuilder creditEventNotice) {
			this.creditEventNotice = creditEventNotice;
			return this;
		}
		public CreditEventsBuilder setDefaultRequirement(Money defaultRequirement) {
			this.defaultRequirement = defaultRequirement.toBuilder();
			return this;
		}
		public CreditEventsBuilder setDefaultRequirementBuilder(Money.MoneyBuilder defaultRequirement) {
			this.defaultRequirement = defaultRequirement;
			return this;
		}
		public CreditEventsBuilder setDistressedRatingsDowngrade(Boolean distressedRatingsDowngrade) {
			this.distressedRatingsDowngrade = distressedRatingsDowngrade;
			return this;
		}
		public CreditEventsBuilder setFailureToPay(FailureToPay failureToPay) {
			this.failureToPay = failureToPay.toBuilder();
			return this;
		}
		public CreditEventsBuilder setFailureToPayBuilder(FailureToPay.FailureToPayBuilder failureToPay) {
			this.failureToPay = failureToPay;
			return this;
		}
		public CreditEventsBuilder setFailureToPayInterest(Boolean failureToPayInterest) {
			this.failureToPayInterest = failureToPayInterest;
			return this;
		}
		public CreditEventsBuilder setFailureToPayPrincipal(Boolean failureToPayPrincipal) {
			this.failureToPayPrincipal = failureToPayPrincipal;
			return this;
		}
		public CreditEventsBuilder setGovernmentalIntervention(Boolean governmentalIntervention) {
			this.governmentalIntervention = governmentalIntervention;
			return this;
		}
		public CreditEventsBuilder setImpliedWritedown(Boolean impliedWritedown) {
			this.impliedWritedown = impliedWritedown;
			return this;
		}
		public CreditEventsBuilder setMaturityExtension(Boolean maturityExtension) {
			this.maturityExtension = maturityExtension;
			return this;
		}
		public CreditEventsBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		public CreditEventsBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		public CreditEventsBuilder setObligationAcceleration(Boolean obligationAcceleration) {
			this.obligationAcceleration = obligationAcceleration;
			return this;
		}
		public CreditEventsBuilder setObligationDefault(Boolean obligationDefault) {
			this.obligationDefault = obligationDefault;
			return this;
		}
		public CreditEventsBuilder setRepudiationMoratorium(Boolean repudiationMoratorium) {
			this.repudiationMoratorium = repudiationMoratorium;
			return this;
		}
		public CreditEventsBuilder setRestructuring(Restructuring restructuring) {
			this.restructuring = restructuring.toBuilder();
			return this;
		}
		public CreditEventsBuilder setRestructuringBuilder(Restructuring.RestructuringBuilder restructuring) {
			this.restructuring = restructuring;
			return this;
		}
		public CreditEventsBuilder setWritedown(Boolean writedown) {
			this.writedown = writedown;
			return this;
		}
		
		public CreditEvents build() {
			return new CreditEvents(this);
		}
	
		@Override
		public CreditEventsBuilder prune() {
			if (creditEventNotice!=null && !creditEventNotice.prune().hasData()) creditEventNotice = null;
			if (defaultRequirement!=null && !defaultRequirement.prune().hasData()) defaultRequirement = null;
			if (failureToPay!=null && !failureToPay.prune().hasData()) failureToPay = null;
			if (meta!=null && !meta.prune().hasData()) meta = null;
			if (restructuring!=null && !restructuring.prune().hasData()) restructuring = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getBankruptcy()!=null) return true;
			if (getCreditEventNotice()!=null && getCreditEventNotice().hasData()) return true;
			if (getDefaultRequirement()!=null && getDefaultRequirement().hasData()) return true;
			if (getDistressedRatingsDowngrade()!=null) return true;
			if (getFailureToPay()!=null && getFailureToPay().hasData()) return true;
			if (getFailureToPayInterest()!=null) return true;
			if (getFailureToPayPrincipal()!=null) return true;
			if (getGovernmentalIntervention()!=null) return true;
			if (getImpliedWritedown()!=null) return true;
			if (getMaturityExtension()!=null) return true;
			if (getMeta()!=null && getMeta().hasData()) return true;
			if (getObligationAcceleration()!=null) return true;
			if (getObligationDefault()!=null) return true;
			if (getRepudiationMoratorium()!=null) return true;
			if (getRestructuring()!=null && getRestructuring().hasData()) return true;
			if (getWritedown()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("bankruptcy"), Boolean.class, bankruptcy, this);
			processor.processBasic(path.newSubPath("distressedRatingsDowngrade"), Boolean.class, distressedRatingsDowngrade, this);
			processor.processBasic(path.newSubPath("failureToPayInterest"), Boolean.class, failureToPayInterest, this);
			processor.processBasic(path.newSubPath("failureToPayPrincipal"), Boolean.class, failureToPayPrincipal, this);
			processor.processBasic(path.newSubPath("governmentalIntervention"), Boolean.class, governmentalIntervention, this);
			processor.processBasic(path.newSubPath("impliedWritedown"), Boolean.class, impliedWritedown, this);
			processor.processBasic(path.newSubPath("maturityExtension"), Boolean.class, maturityExtension, this);
			processor.processBasic(path.newSubPath("obligationAcceleration"), Boolean.class, obligationAcceleration, this);
			processor.processBasic(path.newSubPath("obligationDefault"), Boolean.class, obligationDefault, this);
			processor.processBasic(path.newSubPath("repudiationMoratorium"), Boolean.class, repudiationMoratorium, this);
			processor.processBasic(path.newSubPath("writedown"), Boolean.class, writedown, this);
			
			processRosetta(path.newSubPath("creditEventNotice"), processor, CreditEventNotice.class, creditEventNotice);
			processRosetta(path.newSubPath("defaultRequirement"), processor, Money.class, defaultRequirement);
			processRosetta(path.newSubPath("failureToPay"), processor, FailureToPay.class, failureToPay);
			processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
			processRosetta(path.newSubPath("restructuring"), processor, Restructuring.class, restructuring);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			CreditEventsBuilder _that = (CreditEventsBuilder) o;
		
			if (bankruptcy != null ? !bankruptcy.equals(_that.bankruptcy) : _that.bankruptcy != null) return false;
			if (creditEventNotice != null ? !creditEventNotice.equals(_that.creditEventNotice) : _that.creditEventNotice != null) return false;
			if (defaultRequirement != null ? !defaultRequirement.equals(_that.defaultRequirement) : _that.defaultRequirement != null) return false;
			if (distressedRatingsDowngrade != null ? !distressedRatingsDowngrade.equals(_that.distressedRatingsDowngrade) : _that.distressedRatingsDowngrade != null) return false;
			if (failureToPay != null ? !failureToPay.equals(_that.failureToPay) : _that.failureToPay != null) return false;
			if (failureToPayInterest != null ? !failureToPayInterest.equals(_that.failureToPayInterest) : _that.failureToPayInterest != null) return false;
			if (failureToPayPrincipal != null ? !failureToPayPrincipal.equals(_that.failureToPayPrincipal) : _that.failureToPayPrincipal != null) return false;
			if (governmentalIntervention != null ? !governmentalIntervention.equals(_that.governmentalIntervention) : _that.governmentalIntervention != null) return false;
			if (impliedWritedown != null ? !impliedWritedown.equals(_that.impliedWritedown) : _that.impliedWritedown != null) return false;
			if (maturityExtension != null ? !maturityExtension.equals(_that.maturityExtension) : _that.maturityExtension != null) return false;
			if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
			if (obligationAcceleration != null ? !obligationAcceleration.equals(_that.obligationAcceleration) : _that.obligationAcceleration != null) return false;
			if (obligationDefault != null ? !obligationDefault.equals(_that.obligationDefault) : _that.obligationDefault != null) return false;
			if (repudiationMoratorium != null ? !repudiationMoratorium.equals(_that.repudiationMoratorium) : _that.repudiationMoratorium != null) return false;
			if (restructuring != null ? !restructuring.equals(_that.restructuring) : _that.restructuring != null) return false;
			if (writedown != null ? !writedown.equals(_that.writedown) : _that.writedown != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (bankruptcy != null ? bankruptcy.hashCode() : 0);
			_result = 31 * _result + (creditEventNotice != null ? creditEventNotice.hashCode() : 0);
			_result = 31 * _result + (defaultRequirement != null ? defaultRequirement.hashCode() : 0);
			_result = 31 * _result + (distressedRatingsDowngrade != null ? distressedRatingsDowngrade.hashCode() : 0);
			_result = 31 * _result + (failureToPay != null ? failureToPay.hashCode() : 0);
			_result = 31 * _result + (failureToPayInterest != null ? failureToPayInterest.hashCode() : 0);
			_result = 31 * _result + (failureToPayPrincipal != null ? failureToPayPrincipal.hashCode() : 0);
			_result = 31 * _result + (governmentalIntervention != null ? governmentalIntervention.hashCode() : 0);
			_result = 31 * _result + (impliedWritedown != null ? impliedWritedown.hashCode() : 0);
			_result = 31 * _result + (maturityExtension != null ? maturityExtension.hashCode() : 0);
			_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
			_result = 31 * _result + (obligationAcceleration != null ? obligationAcceleration.hashCode() : 0);
			_result = 31 * _result + (obligationDefault != null ? obligationDefault.hashCode() : 0);
			_result = 31 * _result + (repudiationMoratorium != null ? repudiationMoratorium.hashCode() : 0);
			_result = 31 * _result + (restructuring != null ? restructuring.hashCode() : 0);
			_result = 31 * _result + (writedown != null ? writedown.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "CreditEventsBuilder {" +
				"bankruptcy=" + this.bankruptcy + ", " +
				"creditEventNotice=" + this.creditEventNotice + ", " +
				"defaultRequirement=" + this.defaultRequirement + ", " +
				"distressedRatingsDowngrade=" + this.distressedRatingsDowngrade + ", " +
				"failureToPay=" + this.failureToPay + ", " +
				"failureToPayInterest=" + this.failureToPayInterest + ", " +
				"failureToPayPrincipal=" + this.failureToPayPrincipal + ", " +
				"governmentalIntervention=" + this.governmentalIntervention + ", " +
				"impliedWritedown=" + this.impliedWritedown + ", " +
				"maturityExtension=" + this.maturityExtension + ", " +
				"meta=" + this.meta + ", " +
				"obligationAcceleration=" + this.obligationAcceleration + ", " +
				"obligationDefault=" + this.obligationDefault + ", " +
				"repudiationMoratorium=" + this.repudiationMoratorium + ", " +
				"restructuring=" + this.restructuring + ", " +
				"writedown=" + this.writedown +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("bankruptcy"), Boolean.class, bankruptcy, this);
		processor.processBasic(path.newSubPath("distressedRatingsDowngrade"), Boolean.class, distressedRatingsDowngrade, this);
		processor.processBasic(path.newSubPath("failureToPayInterest"), Boolean.class, failureToPayInterest, this);
		processor.processBasic(path.newSubPath("failureToPayPrincipal"), Boolean.class, failureToPayPrincipal, this);
		processor.processBasic(path.newSubPath("governmentalIntervention"), Boolean.class, governmentalIntervention, this);
		processor.processBasic(path.newSubPath("impliedWritedown"), Boolean.class, impliedWritedown, this);
		processor.processBasic(path.newSubPath("maturityExtension"), Boolean.class, maturityExtension, this);
		processor.processBasic(path.newSubPath("obligationAcceleration"), Boolean.class, obligationAcceleration, this);
		processor.processBasic(path.newSubPath("obligationDefault"), Boolean.class, obligationDefault, this);
		processor.processBasic(path.newSubPath("repudiationMoratorium"), Boolean.class, repudiationMoratorium, this);
		processor.processBasic(path.newSubPath("writedown"), Boolean.class, writedown, this);
		
		processRosetta(path.newSubPath("creditEventNotice"), processor, CreditEventNotice.class, creditEventNotice);
		processRosetta(path.newSubPath("defaultRequirement"), processor, Money.class, defaultRequirement);
		processRosetta(path.newSubPath("failureToPay"), processor, FailureToPay.class, failureToPay);
		processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
		processRosetta(path.newSubPath("restructuring"), processor, Restructuring.class, restructuring);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		CreditEvents _that = (CreditEvents) o;
	
		if (bankruptcy != null ? !bankruptcy.equals(_that.bankruptcy) : _that.bankruptcy != null) return false;
		if (creditEventNotice != null ? !creditEventNotice.equals(_that.creditEventNotice) : _that.creditEventNotice != null) return false;
		if (defaultRequirement != null ? !defaultRequirement.equals(_that.defaultRequirement) : _that.defaultRequirement != null) return false;
		if (distressedRatingsDowngrade != null ? !distressedRatingsDowngrade.equals(_that.distressedRatingsDowngrade) : _that.distressedRatingsDowngrade != null) return false;
		if (failureToPay != null ? !failureToPay.equals(_that.failureToPay) : _that.failureToPay != null) return false;
		if (failureToPayInterest != null ? !failureToPayInterest.equals(_that.failureToPayInterest) : _that.failureToPayInterest != null) return false;
		if (failureToPayPrincipal != null ? !failureToPayPrincipal.equals(_that.failureToPayPrincipal) : _that.failureToPayPrincipal != null) return false;
		if (governmentalIntervention != null ? !governmentalIntervention.equals(_that.governmentalIntervention) : _that.governmentalIntervention != null) return false;
		if (impliedWritedown != null ? !impliedWritedown.equals(_that.impliedWritedown) : _that.impliedWritedown != null) return false;
		if (maturityExtension != null ? !maturityExtension.equals(_that.maturityExtension) : _that.maturityExtension != null) return false;
		if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
		if (obligationAcceleration != null ? !obligationAcceleration.equals(_that.obligationAcceleration) : _that.obligationAcceleration != null) return false;
		if (obligationDefault != null ? !obligationDefault.equals(_that.obligationDefault) : _that.obligationDefault != null) return false;
		if (repudiationMoratorium != null ? !repudiationMoratorium.equals(_that.repudiationMoratorium) : _that.repudiationMoratorium != null) return false;
		if (restructuring != null ? !restructuring.equals(_that.restructuring) : _that.restructuring != null) return false;
		if (writedown != null ? !writedown.equals(_that.writedown) : _that.writedown != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (bankruptcy != null ? bankruptcy.hashCode() : 0);
		_result = 31 * _result + (creditEventNotice != null ? creditEventNotice.hashCode() : 0);
		_result = 31 * _result + (defaultRequirement != null ? defaultRequirement.hashCode() : 0);
		_result = 31 * _result + (distressedRatingsDowngrade != null ? distressedRatingsDowngrade.hashCode() : 0);
		_result = 31 * _result + (failureToPay != null ? failureToPay.hashCode() : 0);
		_result = 31 * _result + (failureToPayInterest != null ? failureToPayInterest.hashCode() : 0);
		_result = 31 * _result + (failureToPayPrincipal != null ? failureToPayPrincipal.hashCode() : 0);
		_result = 31 * _result + (governmentalIntervention != null ? governmentalIntervention.hashCode() : 0);
		_result = 31 * _result + (impliedWritedown != null ? impliedWritedown.hashCode() : 0);
		_result = 31 * _result + (maturityExtension != null ? maturityExtension.hashCode() : 0);
		_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
		_result = 31 * _result + (obligationAcceleration != null ? obligationAcceleration.hashCode() : 0);
		_result = 31 * _result + (obligationDefault != null ? obligationDefault.hashCode() : 0);
		_result = 31 * _result + (repudiationMoratorium != null ? repudiationMoratorium.hashCode() : 0);
		_result = 31 * _result + (restructuring != null ? restructuring.hashCode() : 0);
		_result = 31 * _result + (writedown != null ? writedown.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "CreditEvents {" +
			"bankruptcy=" + this.bankruptcy + ", " +
			"creditEventNotice=" + this.creditEventNotice + ", " +
			"defaultRequirement=" + this.defaultRequirement + ", " +
			"distressedRatingsDowngrade=" + this.distressedRatingsDowngrade + ", " +
			"failureToPay=" + this.failureToPay + ", " +
			"failureToPayInterest=" + this.failureToPayInterest + ", " +
			"failureToPayPrincipal=" + this.failureToPayPrincipal + ", " +
			"governmentalIntervention=" + this.governmentalIntervention + ", " +
			"impliedWritedown=" + this.impliedWritedown + ", " +
			"maturityExtension=" + this.maturityExtension + ", " +
			"meta=" + this.meta + ", " +
			"obligationAcceleration=" + this.obligationAcceleration + ", " +
			"obligationDefault=" + this.obligationDefault + ", " +
			"repudiationMoratorium=" + this.repudiationMoratorium + ", " +
			"restructuring=" + this.restructuring + ", " +
			"writedown=" + this.writedown +
		'}';
	}
}
