package org.isda.cdm;

import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.BusinessCenterTime;
import org.isda.cdm.LimitApplicable;
import org.isda.cdm.Period;
import org.isda.cdm.ProductIdentification;
import org.isda.cdm.metafields.FieldWithMetaString;
import org.isda.cdm.metafields.FieldWithMetaString.FieldWithMetaStringBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.CreditLimitMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;
		import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;

/**
 * A class to specify the credit limit, to be used in the context of the clearing workflow.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="CreditLimit", source="FpML_5_10")
public class CreditLimit extends RosettaModelObject {
	private final ProductIdentification applicableProduct;
	private final FieldWithMetaString currency;
	private final BusinessCenterTime expirationDateTime;
	private final List<LimitApplicable> limitApplicable;
	private final FieldWithMetaString limitId;
	private final Period tenor;
	private static CreditLimitMeta metaData = new CreditLimitMeta();
	
	CreditLimit(CreditLimitBuilder builder) {
		this.applicableProduct = ofNullable(builder.getApplicableProduct()).map(f->f.build()).orElse(null);
		this.currency = ofNullable(builder.getCurrency()).map(f->f.build()).orElse(null);
		this.expirationDateTime = ofNullable(builder.getExpirationDateTime()).map(f->f.build()).orElse(null);
		this.limitApplicable = ofNullable(builder.getLimitApplicable()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.limitId = ofNullable(builder.getLimitId()).map(f->f.build()).orElse(null);
		this.tenor = ofNullable(builder.getTenor()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * Further to the FpML standard, the CDM ProductIdentification provides the ability to associate a CDM qualified product.
	 */
	public final ProductIdentification getApplicableProduct() {
		return applicableProduct;
	}
	
	/**
	 * The currency in which the credit limit is denominated. The list of valid currencies is not presently positioned as an enumeration as part of the CDM because that scope is limited to the values specified by ISDA and FpML. As a result, implementers have to make reference to the relevant standard, such as the ISO 4217 standard for currency codes.
	 */
	@RosettaSynonym(value="currency", source="FpML_5_10")
	public final FieldWithMetaString getCurrency() {
		return currency;
	}
	
	/**
	 * The CDM naming include the time suffix because the FpML element is actually of type dateTime. Also, the attribute is of type BusinessCenterTime because of the design decision to only have time that are specified by reference to a timezone, with that class being deemed the proper implementation.
	 */
	@RosettaSynonym(value="expirationDate", source="FpML_5_10")
	public final BusinessCenterTime getExpirationDateTime() {
		return expirationDateTime;
	}
	
	@RosettaSynonym(value="limitApplicable", source="FpML_5_10")
	public final List<LimitApplicable> getLimitApplicable() {
		return limitApplicable;
	}
	
	@RosettaSynonym(value="limitId", source="FpML_5_10")
	public final FieldWithMetaString getLimitId() {
		return limitId;
	}
	
	/**
	 * The maximum allowed tenor for a trade under this limit. When this structure is used as part of a suspendCredit message, this tenor imposes a temporary tenor limit on allowed trade types.
	 */
	@RosettaSynonym(value="tenor", source="FpML_5_10")
	public final Period getTenor() {
		return tenor;
	}
	
	
	@Override
	public RosettaMetaData<? extends CreditLimit> metaData() {
		return metaData;
	} 
	
	public CreditLimitBuilder toBuilder() {
		CreditLimitBuilder builder = new CreditLimitBuilder();
		ofNullable(getApplicableProduct()).ifPresent(builder::setApplicableProduct);
		ofNullable(getCurrency()).ifPresent(builder::setCurrency);
		ofNullable(getExpirationDateTime()).ifPresent(builder::setExpirationDateTime);
		ofNullable(getLimitApplicable()).ifPresent(limitApplicable -> limitApplicable.forEach(builder::addLimitApplicable));
		ofNullable(getLimitId()).ifPresent(builder::setLimitId);
		ofNullable(getTenor()).ifPresent(builder::setTenor);
		return builder;
	}

	public static CreditLimitBuilder builder() {
		return new CreditLimitBuilder();
	}

	public static  class CreditLimitBuilder extends RosettaModelObjectBuilder {
	
		protected ProductIdentification.ProductIdentificationBuilder applicableProduct;
		protected FieldWithMetaStringBuilder currency;
		protected BusinessCenterTime.BusinessCenterTimeBuilder expirationDateTime;
		protected List<LimitApplicable.LimitApplicableBuilder> limitApplicable;
		protected FieldWithMetaStringBuilder limitId;
		protected Period.PeriodBuilder tenor;
	
		public CreditLimitBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends CreditLimit> metaData() {
			return metaData;
		} 
	
		
		public ProductIdentification.ProductIdentificationBuilder getApplicableProduct() {
			return applicableProduct;
		}
		
		public ProductIdentification.ProductIdentificationBuilder getOrCreateApplicableProduct() {
			if (applicableProduct!=null) {
				return applicableProduct;
			}
			else return applicableProduct = new ProductIdentification.ProductIdentificationBuilder();
		}
		
		
		public FieldWithMetaStringBuilder getCurrency() {
			return currency;
		}
		
		public FieldWithMetaStringBuilder getOrCreateCurrency() {
			if (currency!=null) {
				return currency;
			}
			else return currency = new FieldWithMetaStringBuilder();
		}
		
		
		public BusinessCenterTime.BusinessCenterTimeBuilder getExpirationDateTime() {
			return expirationDateTime;
		}
		
		public BusinessCenterTime.BusinessCenterTimeBuilder getOrCreateExpirationDateTime() {
			if (expirationDateTime!=null) {
				return expirationDateTime;
			}
			else return expirationDateTime = new BusinessCenterTime.BusinessCenterTimeBuilder();
		}
		
		
		public List<LimitApplicable.LimitApplicableBuilder> getLimitApplicable() {
			return limitApplicable;
		}
		
		public LimitApplicable.LimitApplicableBuilder getOrCreateLimitApplicable(int index) {
			if (limitApplicable==null) {
				this.limitApplicable = new ArrayList<>();
			}
			return getIndex(limitApplicable, index, ()->new LimitApplicable.LimitApplicableBuilder());
		}
		
		
		public FieldWithMetaStringBuilder getLimitId() {
			return limitId;
		}
		
		public FieldWithMetaStringBuilder getOrCreateLimitId() {
			if (limitId!=null) {
				return limitId;
			}
			else return limitId = new FieldWithMetaStringBuilder();
		}
		
		
		public Period.PeriodBuilder getTenor() {
			return tenor;
		}
		
		public Period.PeriodBuilder getOrCreateTenor() {
			if (tenor!=null) {
				return tenor;
			}
			else return tenor = new Period.PeriodBuilder();
		}
		
	
		public CreditLimitBuilder setApplicableProduct(ProductIdentification applicableProduct) {
			this.applicableProduct = applicableProduct.toBuilder();
			return this;
		}
		public CreditLimitBuilder setApplicableProductBuilder(ProductIdentification.ProductIdentificationBuilder applicableProduct) {
			this.applicableProduct = applicableProduct;
			return this;
		}
		public CreditLimitBuilder setCurrency(FieldWithMetaString currency) {
			this.currency = currency.toBuilder();
			return this;
		}
		public CreditLimitBuilder setCurrencyRef(String currency) {
			return setCurrency(FieldWithMetaString.builder().setValue(currency).build());
		}
		public CreditLimitBuilder setExpirationDateTime(BusinessCenterTime expirationDateTime) {
			this.expirationDateTime = expirationDateTime.toBuilder();
			return this;
		}
		public CreditLimitBuilder setExpirationDateTimeBuilder(BusinessCenterTime.BusinessCenterTimeBuilder expirationDateTime) {
			this.expirationDateTime = expirationDateTime;
			return this;
		}
		public CreditLimitBuilder addLimitApplicable(LimitApplicable limitApplicable) {
			if(this.limitApplicable == null){
				this.limitApplicable = new ArrayList<>();
			}
			this.limitApplicable.add(limitApplicable.toBuilder());
			return this;
		}
		public CreditLimitBuilder addLimitApplicable(LimitApplicable limitApplicable, int _idx) {
			if(this.limitApplicable == null){
				this.limitApplicable = new ArrayList<>();
			}
			getIndex(this.limitApplicable, _idx, () -> limitApplicable.toBuilder());
			this.limitApplicable.set(_idx, limitApplicable.toBuilder());
			return this;
		}
		public CreditLimitBuilder addLimitApplicable(List<LimitApplicable> limitApplicables) {
			if(this.limitApplicable == null){
				this.limitApplicable = new ArrayList<>();
			}
			for (LimitApplicable toAdd : limitApplicables) {
				this.limitApplicable.add(toAdd.toBuilder());
			}
			return this;
		}
		public CreditLimitBuilder addLimitApplicableBuilder(LimitApplicable.LimitApplicableBuilder limitApplicable) {
			if(this.limitApplicable == null){
				this.limitApplicable = new ArrayList<>();
			}
			this.limitApplicable.add(limitApplicable);
			return this;
		}
		
		public CreditLimitBuilder clearLimitApplicable() {
			this.limitApplicable = null;
			return this;
		}
		public CreditLimitBuilder setLimitId(FieldWithMetaString limitId) {
			this.limitId = limitId.toBuilder();
			return this;
		}
		public CreditLimitBuilder setLimitIdRef(String limitId) {
			return setLimitId(FieldWithMetaString.builder().setValue(limitId).build());
		}
		public CreditLimitBuilder setTenor(Period tenor) {
			this.tenor = tenor.toBuilder();
			return this;
		}
		public CreditLimitBuilder setTenorBuilder(Period.PeriodBuilder tenor) {
			this.tenor = tenor;
			return this;
		}
		
		public CreditLimit build() {
			return new CreditLimit(this);
		}
	
		@Override
		public CreditLimitBuilder prune() {
			if (applicableProduct!=null && !applicableProduct.prune().hasData()) applicableProduct = null;
			if (currency!=null && !currency.prune().hasData()) currency = null;
			if (expirationDateTime!=null && !expirationDateTime.prune().hasData()) expirationDateTime = null;
			if (limitApplicable!=null) limitApplicable = limitApplicable.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (limitId!=null && !limitId.prune().hasData()) limitId = null;
			if (tenor!=null && !tenor.prune().hasData()) tenor = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getApplicableProduct()!=null && getApplicableProduct().hasData()) return true;
			if (getCurrency()!=null) return true;
			if (getExpirationDateTime()!=null && getExpirationDateTime().hasData()) return true;
			if (getLimitApplicable()!=null && getLimitApplicable().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getLimitId()!=null) return true;
			if (getTenor()!=null && getTenor().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("applicableProduct"), processor, ProductIdentification.class, applicableProduct);
			processRosetta(path.newSubPath("currency"), processor, FieldWithMetaString.class, currency);
			processRosetta(path.newSubPath("expirationDateTime"), processor, BusinessCenterTime.class, expirationDateTime);
			processRosetta(path.newSubPath("limitApplicable"), processor, LimitApplicable.class, limitApplicable);
			processRosetta(path.newSubPath("limitId"), processor, FieldWithMetaString.class, limitId);
			processRosetta(path.newSubPath("tenor"), processor, Period.class, tenor);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			CreditLimitBuilder _that = (CreditLimitBuilder) o;
		
			if (applicableProduct != null ? !applicableProduct.equals(_that.applicableProduct) : _that.applicableProduct != null) return false;
			if (currency != null ? !currency.equals(_that.currency) : _that.currency != null) return false;
			if (expirationDateTime != null ? !expirationDateTime.equals(_that.expirationDateTime) : _that.expirationDateTime != null) return false;
			if (!ListEquals.listEquals(limitApplicable, _that.limitApplicable)) return false;
			if (limitId != null ? !limitId.equals(_that.limitId) : _that.limitId != null) return false;
			if (tenor != null ? !tenor.equals(_that.tenor) : _that.tenor != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (applicableProduct != null ? applicableProduct.hashCode() : 0);
			_result = 31 * _result + (currency != null ? currency.hashCode() : 0);
			_result = 31 * _result + (expirationDateTime != null ? expirationDateTime.hashCode() : 0);
			_result = 31 * _result + (limitApplicable != null ? limitApplicable.hashCode() : 0);
			_result = 31 * _result + (limitId != null ? limitId.hashCode() : 0);
			_result = 31 * _result + (tenor != null ? tenor.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "CreditLimitBuilder {" +
				"applicableProduct=" + this.applicableProduct + ", " +
				"currency=" + this.currency + ", " +
				"expirationDateTime=" + this.expirationDateTime + ", " +
				"limitApplicable=" + this.limitApplicable + ", " +
				"limitId=" + this.limitId + ", " +
				"tenor=" + this.tenor +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("applicableProduct"), processor, ProductIdentification.class, applicableProduct);
		processRosetta(path.newSubPath("currency"), processor, FieldWithMetaString.class, currency);
		processRosetta(path.newSubPath("expirationDateTime"), processor, BusinessCenterTime.class, expirationDateTime);
		processRosetta(path.newSubPath("limitApplicable"), processor, LimitApplicable.class, limitApplicable);
		processRosetta(path.newSubPath("limitId"), processor, FieldWithMetaString.class, limitId);
		processRosetta(path.newSubPath("tenor"), processor, Period.class, tenor);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		CreditLimit _that = (CreditLimit) o;
	
		if (applicableProduct != null ? !applicableProduct.equals(_that.applicableProduct) : _that.applicableProduct != null) return false;
		if (currency != null ? !currency.equals(_that.currency) : _that.currency != null) return false;
		if (expirationDateTime != null ? !expirationDateTime.equals(_that.expirationDateTime) : _that.expirationDateTime != null) return false;
		if (!ListEquals.listEquals(limitApplicable, _that.limitApplicable)) return false;
		if (limitId != null ? !limitId.equals(_that.limitId) : _that.limitId != null) return false;
		if (tenor != null ? !tenor.equals(_that.tenor) : _that.tenor != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (applicableProduct != null ? applicableProduct.hashCode() : 0);
		_result = 31 * _result + (currency != null ? currency.hashCode() : 0);
		_result = 31 * _result + (expirationDateTime != null ? expirationDateTime.hashCode() : 0);
		_result = 31 * _result + (limitApplicable != null ? limitApplicable.hashCode() : 0);
		_result = 31 * _result + (limitId != null ? limitId.hashCode() : 0);
		_result = 31 * _result + (tenor != null ? tenor.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "CreditLimit {" +
			"applicableProduct=" + this.applicableProduct + ", " +
			"currency=" + this.currency + ", " +
			"expirationDateTime=" + this.expirationDateTime + ", " +
			"limitApplicable=" + this.limitApplicable + ", " +
			"limitId=" + this.limitId + ", " +
			"tenor=" + this.tenor +
		'}';
	}
}
