package org.isda.cdm;

import org.isda.cdm.CreditLimitUtilisationPosition;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.CreditLimitUtilisationMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * Credit limit utilisation breakdown by executed trades and pending orders.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="CreditLimitUtilization", source="FpML_5_10")
public class CreditLimitUtilisation extends RosettaModelObject {
	private final CreditLimitUtilisationPosition executed;
	private final CreditLimitUtilisationPosition pending;
	private static CreditLimitUtilisationMeta metaData = new CreditLimitUtilisationMeta();
	
	CreditLimitUtilisation(CreditLimitUtilisationBuilder builder) {
		this.executed = ofNullable(builder.getExecuted()).map(f->f.build()).orElse(null);
		this.pending = ofNullable(builder.getPending()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * Credit limit utilisation attributable to executed trades.
	 */
	@RosettaSynonym(value="executed", source="FpML_5_10")
	public final CreditLimitUtilisationPosition getExecuted() {
		return executed;
	}
	
	/**
	 * Credit limit utilisation attributable to pending unexecuted orders.
	 */
	@RosettaSynonym(value="pending", source="FpML_5_10")
	public final CreditLimitUtilisationPosition getPending() {
		return pending;
	}
	
	
	@Override
	public RosettaMetaData<? extends CreditLimitUtilisation> metaData() {
		return metaData;
	} 
	
	public CreditLimitUtilisationBuilder toBuilder() {
		CreditLimitUtilisationBuilder builder = new CreditLimitUtilisationBuilder();
		ofNullable(getExecuted()).ifPresent(builder::setExecuted);
		ofNullable(getPending()).ifPresent(builder::setPending);
		return builder;
	}

	public static CreditLimitUtilisationBuilder builder() {
		return new CreditLimitUtilisationBuilder();
	}

	public static  class CreditLimitUtilisationBuilder extends RosettaModelObjectBuilder {
	
		protected CreditLimitUtilisationPosition.CreditLimitUtilisationPositionBuilder executed;
		protected CreditLimitUtilisationPosition.CreditLimitUtilisationPositionBuilder pending;
	
		public CreditLimitUtilisationBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends CreditLimitUtilisation> metaData() {
			return metaData;
		} 
	
		
		public CreditLimitUtilisationPosition.CreditLimitUtilisationPositionBuilder getExecuted() {
			return executed;
		}
		
		public CreditLimitUtilisationPosition.CreditLimitUtilisationPositionBuilder getOrCreateExecuted() {
			if (executed!=null) {
				return executed;
			}
			else return executed = new CreditLimitUtilisationPosition.CreditLimitUtilisationPositionBuilder();
		}
		
		
		public CreditLimitUtilisationPosition.CreditLimitUtilisationPositionBuilder getPending() {
			return pending;
		}
		
		public CreditLimitUtilisationPosition.CreditLimitUtilisationPositionBuilder getOrCreatePending() {
			if (pending!=null) {
				return pending;
			}
			else return pending = new CreditLimitUtilisationPosition.CreditLimitUtilisationPositionBuilder();
		}
		
	
		public CreditLimitUtilisationBuilder setExecuted(CreditLimitUtilisationPosition executed) {
			this.executed = executed.toBuilder();
			return this;
		}
		public CreditLimitUtilisationBuilder setExecutedBuilder(CreditLimitUtilisationPosition.CreditLimitUtilisationPositionBuilder executed) {
			this.executed = executed;
			return this;
		}
		public CreditLimitUtilisationBuilder setPending(CreditLimitUtilisationPosition pending) {
			this.pending = pending.toBuilder();
			return this;
		}
		public CreditLimitUtilisationBuilder setPendingBuilder(CreditLimitUtilisationPosition.CreditLimitUtilisationPositionBuilder pending) {
			this.pending = pending;
			return this;
		}
		
		public CreditLimitUtilisation build() {
			return new CreditLimitUtilisation(this);
		}
	
		@Override
		public CreditLimitUtilisationBuilder prune() {
			if (executed!=null && !executed.prune().hasData()) executed = null;
			if (pending!=null && !pending.prune().hasData()) pending = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getExecuted()!=null && getExecuted().hasData()) return true;
			if (getPending()!=null && getPending().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("executed"), processor, CreditLimitUtilisationPosition.class, executed);
			processRosetta(path.newSubPath("pending"), processor, CreditLimitUtilisationPosition.class, pending);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			CreditLimitUtilisationBuilder _that = (CreditLimitUtilisationBuilder) o;
		
			if (executed != null ? !executed.equals(_that.executed) : _that.executed != null) return false;
			if (pending != null ? !pending.equals(_that.pending) : _that.pending != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (executed != null ? executed.hashCode() : 0);
			_result = 31 * _result + (pending != null ? pending.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "CreditLimitUtilisationBuilder {" +
				"executed=" + this.executed + ", " +
				"pending=" + this.pending +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("executed"), processor, CreditLimitUtilisationPosition.class, executed);
		processRosetta(path.newSubPath("pending"), processor, CreditLimitUtilisationPosition.class, pending);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		CreditLimitUtilisation _that = (CreditLimitUtilisation) o;
	
		if (executed != null ? !executed.equals(_that.executed) : _that.executed != null) return false;
		if (pending != null ? !pending.equals(_that.pending) : _that.pending != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (executed != null ? executed.hashCode() : 0);
		_result = 31 * _result + (pending != null ? pending.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "CreditLimitUtilisation {" +
			"executed=" + this.executed + ", " +
			"pending=" + this.pending +
		'}';
	}
}
