package org.isda.cdm;

import org.isda.cdm.CreditRatingDebt;
import org.isda.cdm.metafields.FieldWithMetaCreditRatingAgencyEnum;
import org.isda.cdm.metafields.FieldWithMetaCreditRatingAgencyEnum.FieldWithMetaCreditRatingAgencyEnumBuilder;
import org.isda.cdm.metafields.FieldWithMetaString;
import org.isda.cdm.metafields.FieldWithMetaString.FieldWithMetaStringBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.CreditNotationMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class to specify the credit notation as the combination of agency, notation, scale and debt type qualifications.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="CreditNotation", source="FpML_5_10")
public class CreditNotation extends RosettaModelObject {
	private final FieldWithMetaCreditRatingAgencyEnum agency;
	private final CreditRatingDebt debt;
	private final FieldWithMetaString notation;
	private final FieldWithMetaString scale;
	private static CreditNotationMeta metaData = new CreditNotationMeta();
	
	CreditNotation(CreditNotationBuilder builder) {
		this.agency = ofNullable(builder.getAgency()).map(f->f.build()).orElse(null);
		this.debt = ofNullable(builder.getDebt()).map(f->f.build()).orElse(null);
		this.notation = ofNullable(builder.getNotation()).map(f->f.build()).orElse(null);
		this.scale = ofNullable(builder.getScale()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The credit agency to which the other variables (notation, scale, debt type) refer to.
	 */
	@RosettaSynonym(value="agency", source="FpML_5_10")
	public final FieldWithMetaCreditRatingAgencyEnum getAgency() {
		return agency;
	}
	
	/**
	 * The credit rating debt type (e.g. long term, high yield, deposits, ...) associated with the credit rating notation and scale.
	 */
	@RosettaSynonym(value="debt", source="FpML_5_10")
	public final CreditRatingDebt getDebt() {
		return debt;
	}
	
	/**
	 * The credit rating notation. As it varies among credit rating agencies, FpML doesn&#39;t specify a default scheme.
	 */
	@RosettaSynonym(value="notation", source="FpML_5_10")
	public final FieldWithMetaString getNotation() {
		return notation;
	}
	
	/**
	 * The credit rating scale, with a typical distinction between short term, long term. FpML doesn&#39;t specify a default scheme, which is hence not specified as an enumeration as part of the CDM.
	 */
	@RosettaSynonym(value="scale", source="FpML_5_10")
	public final FieldWithMetaString getScale() {
		return scale;
	}
	
	
	@Override
	public RosettaMetaData<? extends CreditNotation> metaData() {
		return metaData;
	} 
	
	public CreditNotationBuilder toBuilder() {
		CreditNotationBuilder builder = new CreditNotationBuilder();
		ofNullable(getAgency()).ifPresent(builder::setAgency);
		ofNullable(getDebt()).ifPresent(builder::setDebt);
		ofNullable(getNotation()).ifPresent(builder::setNotation);
		ofNullable(getScale()).ifPresent(builder::setScale);
		return builder;
	}

	public static CreditNotationBuilder builder() {
		return new CreditNotationBuilder();
	}

	public static  class CreditNotationBuilder extends RosettaModelObjectBuilder {
	
		protected FieldWithMetaCreditRatingAgencyEnumBuilder agency;
		protected CreditRatingDebt.CreditRatingDebtBuilder debt;
		protected FieldWithMetaStringBuilder notation;
		protected FieldWithMetaStringBuilder scale;
	
		public CreditNotationBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends CreditNotation> metaData() {
			return metaData;
		} 
	
		
		public FieldWithMetaCreditRatingAgencyEnumBuilder getAgency() {
			return agency;
		}
		
		public FieldWithMetaCreditRatingAgencyEnumBuilder getOrCreateAgency() {
			if (agency!=null) {
				return agency;
			}
			else return agency = new FieldWithMetaCreditRatingAgencyEnumBuilder();
		}
		
		
		public CreditRatingDebt.CreditRatingDebtBuilder getDebt() {
			return debt;
		}
		
		public CreditRatingDebt.CreditRatingDebtBuilder getOrCreateDebt() {
			if (debt!=null) {
				return debt;
			}
			else return debt = new CreditRatingDebt.CreditRatingDebtBuilder();
		}
		
		
		public FieldWithMetaStringBuilder getNotation() {
			return notation;
		}
		
		public FieldWithMetaStringBuilder getOrCreateNotation() {
			if (notation!=null) {
				return notation;
			}
			else return notation = new FieldWithMetaStringBuilder();
		}
		
		
		public FieldWithMetaStringBuilder getScale() {
			return scale;
		}
		
		public FieldWithMetaStringBuilder getOrCreateScale() {
			if (scale!=null) {
				return scale;
			}
			else return scale = new FieldWithMetaStringBuilder();
		}
		
	
		public CreditNotationBuilder setAgency(FieldWithMetaCreditRatingAgencyEnum agency) {
			this.agency = agency.toBuilder();
			return this;
		}
		public CreditNotationBuilder setAgencyRef(CreditRatingAgencyEnum agency) {
			return setAgency(FieldWithMetaCreditRatingAgencyEnum.builder().setValue(agency).build());
		}
		public CreditNotationBuilder setDebt(CreditRatingDebt debt) {
			this.debt = debt.toBuilder();
			return this;
		}
		public CreditNotationBuilder setDebtBuilder(CreditRatingDebt.CreditRatingDebtBuilder debt) {
			this.debt = debt;
			return this;
		}
		public CreditNotationBuilder setNotation(FieldWithMetaString notation) {
			this.notation = notation.toBuilder();
			return this;
		}
		public CreditNotationBuilder setNotationRef(String notation) {
			return setNotation(FieldWithMetaString.builder().setValue(notation).build());
		}
		public CreditNotationBuilder setScale(FieldWithMetaString scale) {
			this.scale = scale.toBuilder();
			return this;
		}
		public CreditNotationBuilder setScaleRef(String scale) {
			return setScale(FieldWithMetaString.builder().setValue(scale).build());
		}
		
		public CreditNotation build() {
			return new CreditNotation(this);
		}
	
		@Override
		public CreditNotationBuilder prune() {
			if (agency!=null && !agency.prune().hasData()) agency = null;
			if (debt!=null && !debt.prune().hasData()) debt = null;
			if (notation!=null && !notation.prune().hasData()) notation = null;
			if (scale!=null && !scale.prune().hasData()) scale = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getAgency()!=null) return true;
			if (getDebt()!=null && getDebt().hasData()) return true;
			if (getNotation()!=null) return true;
			if (getScale()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("agency"), processor, FieldWithMetaCreditRatingAgencyEnum.class, agency);
			processRosetta(path.newSubPath("debt"), processor, CreditRatingDebt.class, debt);
			processRosetta(path.newSubPath("notation"), processor, FieldWithMetaString.class, notation);
			processRosetta(path.newSubPath("scale"), processor, FieldWithMetaString.class, scale);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			CreditNotationBuilder _that = (CreditNotationBuilder) o;
		
			if (agency != null ? !agency.equals(_that.agency) : _that.agency != null) return false;
			if (debt != null ? !debt.equals(_that.debt) : _that.debt != null) return false;
			if (notation != null ? !notation.equals(_that.notation) : _that.notation != null) return false;
			if (scale != null ? !scale.equals(_that.scale) : _that.scale != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (agency != null ? agency.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (debt != null ? debt.hashCode() : 0);
			_result = 31 * _result + (notation != null ? notation.hashCode() : 0);
			_result = 31 * _result + (scale != null ? scale.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "CreditNotationBuilder {" +
				"agency=" + this.agency + ", " +
				"debt=" + this.debt + ", " +
				"notation=" + this.notation + ", " +
				"scale=" + this.scale +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("agency"), processor, FieldWithMetaCreditRatingAgencyEnum.class, agency);
		processRosetta(path.newSubPath("debt"), processor, CreditRatingDebt.class, debt);
		processRosetta(path.newSubPath("notation"), processor, FieldWithMetaString.class, notation);
		processRosetta(path.newSubPath("scale"), processor, FieldWithMetaString.class, scale);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		CreditNotation _that = (CreditNotation) o;
	
		if (agency != null ? !agency.equals(_that.agency) : _that.agency != null) return false;
		if (debt != null ? !debt.equals(_that.debt) : _that.debt != null) return false;
		if (notation != null ? !notation.equals(_that.notation) : _that.notation != null) return false;
		if (scale != null ? !scale.equals(_that.scale) : _that.scale != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (agency != null ? agency.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (debt != null ? debt.hashCode() : 0);
		_result = 31 * _result + (notation != null ? notation.hashCode() : 0);
		_result = 31 * _result + (scale != null ? scale.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "CreditNotation {" +
			"agency=" + this.agency + ", " +
			"debt=" + this.debt + ", " +
			"notation=" + this.notation + ", " +
			"scale=" + this.scale +
		'}';
	}
}
