package org.isda.cdm;

import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.CreditNotation;
import org.isda.cdm.MultipleCreditNotations;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.CreditNotationsMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;
		import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;

/**
 * The credit rating notation higher level construct, which provides the ability to specify multiple rating notations.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="CreditNotations", source="FpML_5_10")
public class CreditNotations extends RosettaModelObject {
	private final CreditNotation creditNotation;
	private final List<MultipleCreditNotations> creditNotations;
	private static CreditNotationsMeta metaData = new CreditNotationsMeta();
	
	CreditNotations(CreditNotationsBuilder builder) {
		this.creditNotation = ofNullable(builder.getCreditNotation()).map(f->f.build()).orElse(null);
		this.creditNotations = ofNullable(builder.getCreditNotations()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
	}
	
	/**
	 * This attribute is specified when only one credit notation is determined.
	 */
	@RosettaSynonym(value="creditNotation", source="FpML_5_10")
	public final CreditNotation getCreditNotation() {
		return creditNotation;
	}
	
	/**
	 * This attribute provides the ability to specify several credit notations, alongside an &#39;any&#39; or &#39;all&#39; or all condition.
	 */
	public final List<MultipleCreditNotations> getCreditNotations() {
		return creditNotations;
	}
	
	
	@Override
	public RosettaMetaData<? extends CreditNotations> metaData() {
		return metaData;
	} 
	
	public CreditNotationsBuilder toBuilder() {
		CreditNotationsBuilder builder = new CreditNotationsBuilder();
		ofNullable(getCreditNotation()).ifPresent(builder::setCreditNotation);
		ofNullable(getCreditNotations()).ifPresent(creditNotations -> creditNotations.forEach(builder::addCreditNotations));
		return builder;
	}

	public static CreditNotationsBuilder builder() {
		return new CreditNotationsBuilder();
	}

	public static  class CreditNotationsBuilder extends RosettaModelObjectBuilder {
	
		protected CreditNotation.CreditNotationBuilder creditNotation;
		protected List<MultipleCreditNotations.MultipleCreditNotationsBuilder> creditNotations;
	
		public CreditNotationsBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends CreditNotations> metaData() {
			return metaData;
		} 
	
		
		public CreditNotation.CreditNotationBuilder getCreditNotation() {
			return creditNotation;
		}
		
		public CreditNotation.CreditNotationBuilder getOrCreateCreditNotation() {
			if (creditNotation!=null) {
				return creditNotation;
			}
			else return creditNotation = new CreditNotation.CreditNotationBuilder();
		}
		
		
		public List<MultipleCreditNotations.MultipleCreditNotationsBuilder> getCreditNotations() {
			return creditNotations;
		}
		
		public MultipleCreditNotations.MultipleCreditNotationsBuilder getOrCreateCreditNotations(int index) {
			if (creditNotations==null) {
				this.creditNotations = new ArrayList<>();
			}
			return getIndex(creditNotations, index, ()->new MultipleCreditNotations.MultipleCreditNotationsBuilder());
		}
		
	
		public CreditNotationsBuilder setCreditNotation(CreditNotation creditNotation) {
			this.creditNotation = creditNotation.toBuilder();
			return this;
		}
		public CreditNotationsBuilder setCreditNotationBuilder(CreditNotation.CreditNotationBuilder creditNotation) {
			this.creditNotation = creditNotation;
			return this;
		}
		public CreditNotationsBuilder addCreditNotations(MultipleCreditNotations creditNotations) {
			if(this.creditNotations == null){
				this.creditNotations = new ArrayList<>();
			}
			this.creditNotations.add(creditNotations.toBuilder());
			return this;
		}
		public CreditNotationsBuilder addCreditNotations(MultipleCreditNotations creditNotations, int _idx) {
			if(this.creditNotations == null){
				this.creditNotations = new ArrayList<>();
			}
			getIndex(this.creditNotations, _idx, () -> creditNotations.toBuilder());
			this.creditNotations.set(_idx, creditNotations.toBuilder());
			return this;
		}
		public CreditNotationsBuilder addCreditNotations(List<MultipleCreditNotations> creditNotationss) {
			if(this.creditNotations == null){
				this.creditNotations = new ArrayList<>();
			}
			for (MultipleCreditNotations toAdd : creditNotationss) {
				this.creditNotations.add(toAdd.toBuilder());
			}
			return this;
		}
		public CreditNotationsBuilder addCreditNotationsBuilder(MultipleCreditNotations.MultipleCreditNotationsBuilder creditNotations) {
			if(this.creditNotations == null){
				this.creditNotations = new ArrayList<>();
			}
			this.creditNotations.add(creditNotations);
			return this;
		}
		
		public CreditNotationsBuilder clearCreditNotations() {
			this.creditNotations = null;
			return this;
		}
		
		public CreditNotations build() {
			return new CreditNotations(this);
		}
	
		@Override
		public CreditNotationsBuilder prune() {
			if (creditNotation!=null && !creditNotation.prune().hasData()) creditNotation = null;
			if (creditNotations!=null) creditNotations = creditNotations.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getCreditNotation()!=null && getCreditNotation().hasData()) return true;
			if (getCreditNotations()!=null && getCreditNotations().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("creditNotation"), processor, CreditNotation.class, creditNotation);
			processRosetta(path.newSubPath("creditNotations"), processor, MultipleCreditNotations.class, creditNotations);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			CreditNotationsBuilder _that = (CreditNotationsBuilder) o;
		
			if (creditNotation != null ? !creditNotation.equals(_that.creditNotation) : _that.creditNotation != null) return false;
			if (!ListEquals.listEquals(creditNotations, _that.creditNotations)) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (creditNotation != null ? creditNotation.hashCode() : 0);
			_result = 31 * _result + (creditNotations != null ? creditNotations.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "CreditNotationsBuilder {" +
				"creditNotation=" + this.creditNotation + ", " +
				"creditNotations=" + this.creditNotations +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("creditNotation"), processor, CreditNotation.class, creditNotation);
		processRosetta(path.newSubPath("creditNotations"), processor, MultipleCreditNotations.class, creditNotations);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		CreditNotations _that = (CreditNotations) o;
	
		if (creditNotation != null ? !creditNotation.equals(_that.creditNotation) : _that.creditNotation != null) return false;
		if (!ListEquals.listEquals(creditNotations, _that.creditNotations)) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (creditNotation != null ? creditNotation.hashCode() : 0);
		_result = 31 * _result + (creditNotations != null ? creditNotations.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "CreditNotations {" +
			"creditNotation=" + this.creditNotation + ", " +
			"creditNotations=" + this.creditNotations +
		'}';
	}
}
