package org.isda.cdm;

import org.isda.cdm.CollateralRounding;
import org.isda.cdm.MinimumTransferAmount;
import org.isda.cdm.Threshold;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.CreditSupportObligationsInitialMarginMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class to specify the Credit Support Obligations applicable to the Initial Margin Credit Support Annex and which are common among the English, Japanese and New York governing laws. This excludes the Other Eligible Support election (which only applies to the Japanese Law and New York Law agreements) and the Transfer Timing election (which only applies to the English Law and the New York Law agreements). ISDA 2016 English Law Credit Support Deed for Initial Margin, paragraph 13, General Principles, (c): Credit Support Obligations. | ISDA 2016 Japanese Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (d): Credit Support Obligations. | ISDA 2016 New York Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (c): Credit Support Obligations.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class CreditSupportObligationsInitialMargin extends RosettaModelObject {
	private final MinimumTransferAmount minimumTransferAmount;
	private final CollateralRounding rounding;
	private final Threshold threshold;
	private static CreditSupportObligationsInitialMarginMeta metaData = new CreditSupportObligationsInitialMarginMeta();
	
	CreditSupportObligationsInitialMargin(CreditSupportObligationsInitialMarginBuilder builder) {
		this.minimumTransferAmount = ofNullable(builder.getMinimumTransferAmount()).map(f->f.build()).orElse(null);
		this.rounding = ofNullable(builder.getRounding()).map(f->f.build()).orElse(null);
		this.threshold = ofNullable(builder.getThreshold()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The net amount of exposure reached before collateral has to be posted or returned. ISDA 2016 English Law Credit Support Deed for Initial Margin, paragraph 13, General Principles, (c)(vi)(B): Minimum Transfer Amount. | ISDA 2016 Japanese Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (d)(vi)(B): Minimum Transfer Amount. | ISDA 2016 New York Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (c)(vi)(B): Minimum Transfer Amount.
	 */
	@RosettaSynonym(value="minimum_transfer_amount", source="ISDA_Create_1_0")
	public final MinimumTransferAmount getMinimumTransferAmount() {
		return minimumTransferAmount;
	}
	
	/**
	 * The rounding methodology applicable to the Delivery Amount and the Return Amount in terms of nearest integral multiple of Base Currency units. ISDA 2016 English Law Credit Support Deed for Initial Margin, paragraph 13, General Principles, (c)(vi)(C): Rounding. | ISDA 2016 Japanese Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (d)(vi)(C): Rounding. | ISDA 2016 New York Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (c)(vi)(C): Rounding.
	 */
	@RosettaSynonym(value="rounding", source="ISDA_Create_1_0")
	public final CollateralRounding getRounding() {
		return rounding;
	}
	
	/**
	 * The amount of net exposure that a party is willing to bear in relation to the other party before it requires asking for collateral. ISDA 2016 English Law Credit Support Deed for Initial Margin, paragraph 13, General Principles, (c)(vi)(A): Threshold. | ISDA 2016 Japanese Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (d)(vi)(A): Threshold. | ISDA 2016 New York Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (c)(vi)(A): Threshold.
	 */
	@RosettaSynonym(value="threshold", source="ISDA_Create_1_0")
	public final Threshold getThreshold() {
		return threshold;
	}
	
	
	@Override
	public RosettaMetaData<? extends CreditSupportObligationsInitialMargin> metaData() {
		return metaData;
	} 
	
	public CreditSupportObligationsInitialMarginBuilder toBuilder() {
		CreditSupportObligationsInitialMarginBuilder builder = new CreditSupportObligationsInitialMarginBuilder();
		ofNullable(getMinimumTransferAmount()).ifPresent(builder::setMinimumTransferAmount);
		ofNullable(getRounding()).ifPresent(builder::setRounding);
		ofNullable(getThreshold()).ifPresent(builder::setThreshold);
		return builder;
	}

	public static CreditSupportObligationsInitialMarginBuilder builder() {
		return new CreditSupportObligationsInitialMarginBuilder();
	}

	public static  class CreditSupportObligationsInitialMarginBuilder extends RosettaModelObjectBuilder {
	
		protected MinimumTransferAmount.MinimumTransferAmountBuilder minimumTransferAmount;
		protected CollateralRounding.CollateralRoundingBuilder rounding;
		protected Threshold.ThresholdBuilder threshold;
	
		public CreditSupportObligationsInitialMarginBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends CreditSupportObligationsInitialMargin> metaData() {
			return metaData;
		} 
	
		
		public MinimumTransferAmount.MinimumTransferAmountBuilder getMinimumTransferAmount() {
			return minimumTransferAmount;
		}
		
		public MinimumTransferAmount.MinimumTransferAmountBuilder getOrCreateMinimumTransferAmount() {
			if (minimumTransferAmount!=null) {
				return minimumTransferAmount;
			}
			else return minimumTransferAmount = new MinimumTransferAmount.MinimumTransferAmountBuilder();
		}
		
		
		public CollateralRounding.CollateralRoundingBuilder getRounding() {
			return rounding;
		}
		
		public CollateralRounding.CollateralRoundingBuilder getOrCreateRounding() {
			if (rounding!=null) {
				return rounding;
			}
			else return rounding = new CollateralRounding.CollateralRoundingBuilder();
		}
		
		
		public Threshold.ThresholdBuilder getThreshold() {
			return threshold;
		}
		
		public Threshold.ThresholdBuilder getOrCreateThreshold() {
			if (threshold!=null) {
				return threshold;
			}
			else return threshold = new Threshold.ThresholdBuilder();
		}
		
	
		public CreditSupportObligationsInitialMarginBuilder setMinimumTransferAmount(MinimumTransferAmount minimumTransferAmount) {
			this.minimumTransferAmount = minimumTransferAmount.toBuilder();
			return this;
		}
		public CreditSupportObligationsInitialMarginBuilder setMinimumTransferAmountBuilder(MinimumTransferAmount.MinimumTransferAmountBuilder minimumTransferAmount) {
			this.minimumTransferAmount = minimumTransferAmount;
			return this;
		}
		public CreditSupportObligationsInitialMarginBuilder setRounding(CollateralRounding rounding) {
			this.rounding = rounding.toBuilder();
			return this;
		}
		public CreditSupportObligationsInitialMarginBuilder setRoundingBuilder(CollateralRounding.CollateralRoundingBuilder rounding) {
			this.rounding = rounding;
			return this;
		}
		public CreditSupportObligationsInitialMarginBuilder setThreshold(Threshold threshold) {
			this.threshold = threshold.toBuilder();
			return this;
		}
		public CreditSupportObligationsInitialMarginBuilder setThresholdBuilder(Threshold.ThresholdBuilder threshold) {
			this.threshold = threshold;
			return this;
		}
		
		public CreditSupportObligationsInitialMargin build() {
			return new CreditSupportObligationsInitialMargin(this);
		}
	
		@Override
		public CreditSupportObligationsInitialMarginBuilder prune() {
			if (minimumTransferAmount!=null && !minimumTransferAmount.prune().hasData()) minimumTransferAmount = null;
			if (rounding!=null && !rounding.prune().hasData()) rounding = null;
			if (threshold!=null && !threshold.prune().hasData()) threshold = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getMinimumTransferAmount()!=null && getMinimumTransferAmount().hasData()) return true;
			if (getRounding()!=null && getRounding().hasData()) return true;
			if (getThreshold()!=null && getThreshold().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("minimumTransferAmount"), processor, MinimumTransferAmount.class, minimumTransferAmount);
			processRosetta(path.newSubPath("rounding"), processor, CollateralRounding.class, rounding);
			processRosetta(path.newSubPath("threshold"), processor, Threshold.class, threshold);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			CreditSupportObligationsInitialMarginBuilder _that = (CreditSupportObligationsInitialMarginBuilder) o;
		
			if (minimumTransferAmount != null ? !minimumTransferAmount.equals(_that.minimumTransferAmount) : _that.minimumTransferAmount != null) return false;
			if (rounding != null ? !rounding.equals(_that.rounding) : _that.rounding != null) return false;
			if (threshold != null ? !threshold.equals(_that.threshold) : _that.threshold != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (minimumTransferAmount != null ? minimumTransferAmount.hashCode() : 0);
			_result = 31 * _result + (rounding != null ? rounding.hashCode() : 0);
			_result = 31 * _result + (threshold != null ? threshold.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "CreditSupportObligationsInitialMarginBuilder {" +
				"minimumTransferAmount=" + this.minimumTransferAmount + ", " +
				"rounding=" + this.rounding + ", " +
				"threshold=" + this.threshold +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("minimumTransferAmount"), processor, MinimumTransferAmount.class, minimumTransferAmount);
		processRosetta(path.newSubPath("rounding"), processor, CollateralRounding.class, rounding);
		processRosetta(path.newSubPath("threshold"), processor, Threshold.class, threshold);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		CreditSupportObligationsInitialMargin _that = (CreditSupportObligationsInitialMargin) o;
	
		if (minimumTransferAmount != null ? !minimumTransferAmount.equals(_that.minimumTransferAmount) : _that.minimumTransferAmount != null) return false;
		if (rounding != null ? !rounding.equals(_that.rounding) : _that.rounding != null) return false;
		if (threshold != null ? !threshold.equals(_that.threshold) : _that.threshold != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (minimumTransferAmount != null ? minimumTransferAmount.hashCode() : 0);
		_result = 31 * _result + (rounding != null ? rounding.hashCode() : 0);
		_result = 31 * _result + (threshold != null ? threshold.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "CreditSupportObligationsInitialMargin {" +
			"minimumTransferAmount=" + this.minimumTransferAmount + ", " +
			"rounding=" + this.rounding + ", " +
			"threshold=" + this.threshold +
		'}';
	}
}
