package org.isda.cdm;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.CollateralRounding;
import org.isda.cdm.EligibleCollateralVariationMargin;
import org.isda.cdm.IneligibleCreditSupport;
import org.isda.cdm.MinimumTransferAmount;
import org.isda.cdm.metafields.FieldWithMetaString;
import org.isda.cdm.metafields.FieldWithMetaString.FieldWithMetaStringBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.CreditSupportObligationsVariationMarginMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;
		import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import java.math.BigDecimal;

/**
 * A class to specify the Credit Support Obligations applicable to the Variation Margin Credit Support Annex and which are common among the English, Japanese and New York governing laws. ISDA 2016 Credit Support Annex for Variation Margin, paragraph 13, (c): Credit Support Obligations.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class CreditSupportObligationsVariationMargin extends RosettaModelObject {
	private final EligibleCollateralVariationMargin eligibleCollateral;
	private final BigDecimal fxHaircut;
	private final IneligibleCreditSupport ineligibleCreditSupport;
	private final List<FieldWithMetaString> majorCurrency;
	private final MinimumTransferAmount minimumTransferAmount;
	private final CollateralRounding rounding;
	private static CreditSupportObligationsVariationMarginMeta metaData = new CreditSupportObligationsVariationMarginMeta();
	
	CreditSupportObligationsVariationMargin(CreditSupportObligationsVariationMarginBuilder builder) {
		this.eligibleCollateral = ofNullable(builder.getEligibleCollateral()).map(f->f.build()).orElse(null);
		this.fxHaircut = builder.getFxHaircut();
		this.ineligibleCreditSupport = ofNullable(builder.getIneligibleCreditSupport()).map(f->f.build()).orElse(null);
		this.majorCurrency = ofNullable(builder.getMajorCurrency()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.minimumTransferAmount = ofNullable(builder.getMinimumTransferAmount()).map(f->f.build()).orElse(null);
		this.rounding = ofNullable(builder.getRounding()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The eligible collateral elections by the parties to the Credit Support Annex for Variation Margin. ISDA 2016 Credit Support Annex for Variation Margin, paragraph 13, (c)(ii): Eligible Collateral (VM).
	 */
	public final EligibleCollateralVariationMargin getEligibleCollateral() {
		return eligibleCollateral;
	}
	
	/**
	 * The FX haircut percentage that applies to each party (as the pledgor/chargor/obligor) and item of Eligible Collateral unless this item is denominated in a Major Currency or in the Base Currency. ISDA 2016 Credit Support Annex for Variation Margin, paragraph 13, (c)(v)(B): FX Haircut Percentage.
	 */
	public final BigDecimal getFxHaircut() {
		return fxHaircut;
	}
	
	/**
	 * The parties to which the provisions of Paragraph 11(g) of the ISDA 2016 Credit Support Annex for Variation Margin will apply to. ISDA 2016 Credit Support Annex for Variation Margin, paragraph 13, (c)(iii): Legally Ineligible Credit Support (VM).
	 */
	public final IneligibleCreditSupport getIneligibleCreditSupport() {
		return ineligibleCreditSupport;
	}
	
	/**
	 * The set of currencies that are specified as Major Currency for the purpose of applying the FX Haircut Percentage. ISDA 2016 Credit Support Annex for Variation Margin, paragraph 13, (c)(v)(B): FX Haircut Percentage.
	 */
	public final List<FieldWithMetaString> getMajorCurrency() {
		return majorCurrency;
	}
	
	/**
	 * The net amount of exposure reached before collateral has to be posted or returned. ISDA 2016 New York Law Credit Support Annex for Variation Margin, paragraph 13, (c)(vii)(A): Minimum Transfer Amount.
	 */
	public final MinimumTransferAmount getMinimumTransferAmount() {
		return minimumTransferAmount;
	}
	
	/**
	 * The rounding methodology applicable to the Delivery Amount and the Return Amount in terms of nearest integral multiple of Base Currency units. ISDA 2016 New York Law Credit Support Annex for Variation Margin, paragraph 13, (c)(vii)(B): Rounding.
	 */
	public final CollateralRounding getRounding() {
		return rounding;
	}
	
	
	@Override
	public RosettaMetaData<? extends CreditSupportObligationsVariationMargin> metaData() {
		return metaData;
	} 
	
	public CreditSupportObligationsVariationMarginBuilder toBuilder() {
		CreditSupportObligationsVariationMarginBuilder builder = new CreditSupportObligationsVariationMarginBuilder();
		ofNullable(getEligibleCollateral()).ifPresent(builder::setEligibleCollateral);
		ofNullable(getFxHaircut()).ifPresent(builder::setFxHaircut);
		ofNullable(getIneligibleCreditSupport()).ifPresent(builder::setIneligibleCreditSupport);
		ofNullable(getMajorCurrency()).ifPresent(majorCurrency -> majorCurrency.forEach(builder::addMajorCurrency));
		ofNullable(getMinimumTransferAmount()).ifPresent(builder::setMinimumTransferAmount);
		ofNullable(getRounding()).ifPresent(builder::setRounding);
		return builder;
	}

	public static CreditSupportObligationsVariationMarginBuilder builder() {
		return new CreditSupportObligationsVariationMarginBuilder();
	}

	public static  class CreditSupportObligationsVariationMarginBuilder extends RosettaModelObjectBuilder {
	
		protected EligibleCollateralVariationMargin.EligibleCollateralVariationMarginBuilder eligibleCollateral;
		protected BigDecimal fxHaircut;
		protected IneligibleCreditSupport.IneligibleCreditSupportBuilder ineligibleCreditSupport;
		protected List<FieldWithMetaStringBuilder> majorCurrency;
		protected MinimumTransferAmount.MinimumTransferAmountBuilder minimumTransferAmount;
		protected CollateralRounding.CollateralRoundingBuilder rounding;
	
		public CreditSupportObligationsVariationMarginBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends CreditSupportObligationsVariationMargin> metaData() {
			return metaData;
		} 
	
		
		public EligibleCollateralVariationMargin.EligibleCollateralVariationMarginBuilder getEligibleCollateral() {
			return eligibleCollateral;
		}
		
		public EligibleCollateralVariationMargin.EligibleCollateralVariationMarginBuilder getOrCreateEligibleCollateral() {
			if (eligibleCollateral!=null) {
				return eligibleCollateral;
			}
			else return eligibleCollateral = new EligibleCollateralVariationMargin.EligibleCollateralVariationMarginBuilder();
		}
		
		
		public BigDecimal getFxHaircut() {
			return fxHaircut;
		}
		
		
		public IneligibleCreditSupport.IneligibleCreditSupportBuilder getIneligibleCreditSupport() {
			return ineligibleCreditSupport;
		}
		
		public IneligibleCreditSupport.IneligibleCreditSupportBuilder getOrCreateIneligibleCreditSupport() {
			if (ineligibleCreditSupport!=null) {
				return ineligibleCreditSupport;
			}
			else return ineligibleCreditSupport = new IneligibleCreditSupport.IneligibleCreditSupportBuilder();
		}
		
		
		public List<FieldWithMetaStringBuilder> getMajorCurrency() {
			return majorCurrency;
		}
		
		public FieldWithMetaStringBuilder getOrCreateMajorCurrency(int index) {
			if (majorCurrency==null) {
				this.majorCurrency = new ArrayList<>();
			}
			return getIndex(majorCurrency, index, ()->new FieldWithMetaStringBuilder());
		}
		
		
		public MinimumTransferAmount.MinimumTransferAmountBuilder getMinimumTransferAmount() {
			return minimumTransferAmount;
		}
		
		public MinimumTransferAmount.MinimumTransferAmountBuilder getOrCreateMinimumTransferAmount() {
			if (minimumTransferAmount!=null) {
				return minimumTransferAmount;
			}
			else return minimumTransferAmount = new MinimumTransferAmount.MinimumTransferAmountBuilder();
		}
		
		
		public CollateralRounding.CollateralRoundingBuilder getRounding() {
			return rounding;
		}
		
		public CollateralRounding.CollateralRoundingBuilder getOrCreateRounding() {
			if (rounding!=null) {
				return rounding;
			}
			else return rounding = new CollateralRounding.CollateralRoundingBuilder();
		}
		
	
		public CreditSupportObligationsVariationMarginBuilder setEligibleCollateral(EligibleCollateralVariationMargin eligibleCollateral) {
			this.eligibleCollateral = eligibleCollateral.toBuilder();
			return this;
		}
		public CreditSupportObligationsVariationMarginBuilder setEligibleCollateralBuilder(EligibleCollateralVariationMargin.EligibleCollateralVariationMarginBuilder eligibleCollateral) {
			this.eligibleCollateral = eligibleCollateral;
			return this;
		}
		public CreditSupportObligationsVariationMarginBuilder setFxHaircut(BigDecimal fxHaircut) {
			this.fxHaircut = fxHaircut;
			return this;
		}
		public CreditSupportObligationsVariationMarginBuilder setIneligibleCreditSupport(IneligibleCreditSupport ineligibleCreditSupport) {
			this.ineligibleCreditSupport = ineligibleCreditSupport.toBuilder();
			return this;
		}
		public CreditSupportObligationsVariationMarginBuilder setIneligibleCreditSupportBuilder(IneligibleCreditSupport.IneligibleCreditSupportBuilder ineligibleCreditSupport) {
			this.ineligibleCreditSupport = ineligibleCreditSupport;
			return this;
		}
		public CreditSupportObligationsVariationMarginBuilder addMajorCurrency(FieldWithMetaString majorCurrency) {
			if(this.majorCurrency == null){
				this.majorCurrency = new ArrayList<>();
			}
			this.majorCurrency.add(majorCurrency.toBuilder());
			return this;
		}
		public CreditSupportObligationsVariationMarginBuilder addMajorCurrency(FieldWithMetaString majorCurrency, int _idx) {
			if(this.majorCurrency == null){
				this.majorCurrency = new ArrayList<>();
			}
			getIndex(this.majorCurrency, _idx, () -> majorCurrency.toBuilder());
			this.majorCurrency.set(_idx, majorCurrency.toBuilder());
			return this;
		}
		public CreditSupportObligationsVariationMarginBuilder addMajorCurrency(List<FieldWithMetaString> majorCurrencys) {
			if(this.majorCurrency == null){
				this.majorCurrency = new ArrayList<>();
			}
			for (FieldWithMetaString toAdd : majorCurrencys) {
				this.majorCurrency.add(toAdd.toBuilder());
			}
			return this;
		}
		
		public CreditSupportObligationsVariationMarginBuilder clearMajorCurrency() {
			this.majorCurrency = null;
			return this;
		}
		public CreditSupportObligationsVariationMarginBuilder setMinimumTransferAmount(MinimumTransferAmount minimumTransferAmount) {
			this.minimumTransferAmount = minimumTransferAmount.toBuilder();
			return this;
		}
		public CreditSupportObligationsVariationMarginBuilder setMinimumTransferAmountBuilder(MinimumTransferAmount.MinimumTransferAmountBuilder minimumTransferAmount) {
			this.minimumTransferAmount = minimumTransferAmount;
			return this;
		}
		public CreditSupportObligationsVariationMarginBuilder setRounding(CollateralRounding rounding) {
			this.rounding = rounding.toBuilder();
			return this;
		}
		public CreditSupportObligationsVariationMarginBuilder setRoundingBuilder(CollateralRounding.CollateralRoundingBuilder rounding) {
			this.rounding = rounding;
			return this;
		}
		
		public CreditSupportObligationsVariationMargin build() {
			return new CreditSupportObligationsVariationMargin(this);
		}
	
		@Override
		public CreditSupportObligationsVariationMarginBuilder prune() {
			if (eligibleCollateral!=null && !eligibleCollateral.prune().hasData()) eligibleCollateral = null;
			if (ineligibleCreditSupport!=null && !ineligibleCreditSupport.prune().hasData()) ineligibleCreditSupport = null;
			if (majorCurrency!=null) majorCurrency = majorCurrency.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (minimumTransferAmount!=null && !minimumTransferAmount.prune().hasData()) minimumTransferAmount = null;
			if (rounding!=null && !rounding.prune().hasData()) rounding = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getEligibleCollateral()!=null && getEligibleCollateral().hasData()) return true;
			if (getFxHaircut()!=null) return true;
			if (getIneligibleCreditSupport()!=null && getIneligibleCreditSupport().hasData()) return true;
			if (getMajorCurrency()!=null && !getMajorCurrency().isEmpty()) return true;
			if (getMinimumTransferAmount()!=null && getMinimumTransferAmount().hasData()) return true;
			if (getRounding()!=null && getRounding().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("fxHaircut"), BigDecimal.class, fxHaircut, this);
			
			processRosetta(path.newSubPath("eligibleCollateral"), processor, EligibleCollateralVariationMargin.class, eligibleCollateral);
			processRosetta(path.newSubPath("ineligibleCreditSupport"), processor, IneligibleCreditSupport.class, ineligibleCreditSupport);
			processRosetta(path.newSubPath("majorCurrency"), processor, FieldWithMetaString.class, majorCurrency);
			processRosetta(path.newSubPath("minimumTransferAmount"), processor, MinimumTransferAmount.class, minimumTransferAmount);
			processRosetta(path.newSubPath("rounding"), processor, CollateralRounding.class, rounding);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			CreditSupportObligationsVariationMarginBuilder _that = (CreditSupportObligationsVariationMarginBuilder) o;
		
			if (eligibleCollateral != null ? !eligibleCollateral.equals(_that.eligibleCollateral) : _that.eligibleCollateral != null) return false;
			if (fxHaircut != null ? !fxHaircut.equals(_that.fxHaircut) : _that.fxHaircut != null) return false;
			if (ineligibleCreditSupport != null ? !ineligibleCreditSupport.equals(_that.ineligibleCreditSupport) : _that.ineligibleCreditSupport != null) return false;
			if (!ListEquals.listEquals(majorCurrency, _that.majorCurrency)) return false;
			if (minimumTransferAmount != null ? !minimumTransferAmount.equals(_that.minimumTransferAmount) : _that.minimumTransferAmount != null) return false;
			if (rounding != null ? !rounding.equals(_that.rounding) : _that.rounding != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (eligibleCollateral != null ? eligibleCollateral.hashCode() : 0);
			_result = 31 * _result + (fxHaircut != null ? fxHaircut.hashCode() : 0);
			_result = 31 * _result + (ineligibleCreditSupport != null ? ineligibleCreditSupport.hashCode() : 0);
			_result = 31 * _result + (majorCurrency != null ? majorCurrency.hashCode() : 0);
			_result = 31 * _result + (minimumTransferAmount != null ? minimumTransferAmount.hashCode() : 0);
			_result = 31 * _result + (rounding != null ? rounding.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "CreditSupportObligationsVariationMarginBuilder {" +
				"eligibleCollateral=" + this.eligibleCollateral + ", " +
				"fxHaircut=" + this.fxHaircut + ", " +
				"ineligibleCreditSupport=" + this.ineligibleCreditSupport + ", " +
				"majorCurrency=" + this.majorCurrency + ", " +
				"minimumTransferAmount=" + this.minimumTransferAmount + ", " +
				"rounding=" + this.rounding +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("fxHaircut"), BigDecimal.class, fxHaircut, this);
		
		processRosetta(path.newSubPath("eligibleCollateral"), processor, EligibleCollateralVariationMargin.class, eligibleCollateral);
		processRosetta(path.newSubPath("ineligibleCreditSupport"), processor, IneligibleCreditSupport.class, ineligibleCreditSupport);
		processRosetta(path.newSubPath("majorCurrency"), processor, FieldWithMetaString.class, majorCurrency);
		processRosetta(path.newSubPath("minimumTransferAmount"), processor, MinimumTransferAmount.class, minimumTransferAmount);
		processRosetta(path.newSubPath("rounding"), processor, CollateralRounding.class, rounding);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		CreditSupportObligationsVariationMargin _that = (CreditSupportObligationsVariationMargin) o;
	
		if (eligibleCollateral != null ? !eligibleCollateral.equals(_that.eligibleCollateral) : _that.eligibleCollateral != null) return false;
		if (fxHaircut != null ? !fxHaircut.equals(_that.fxHaircut) : _that.fxHaircut != null) return false;
		if (ineligibleCreditSupport != null ? !ineligibleCreditSupport.equals(_that.ineligibleCreditSupport) : _that.ineligibleCreditSupport != null) return false;
		if (!ListEquals.listEquals(majorCurrency, _that.majorCurrency)) return false;
		if (minimumTransferAmount != null ? !minimumTransferAmount.equals(_that.minimumTransferAmount) : _that.minimumTransferAmount != null) return false;
		if (rounding != null ? !rounding.equals(_that.rounding) : _that.rounding != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (eligibleCollateral != null ? eligibleCollateral.hashCode() : 0);
		_result = 31 * _result + (fxHaircut != null ? fxHaircut.hashCode() : 0);
		_result = 31 * _result + (ineligibleCreditSupport != null ? ineligibleCreditSupport.hashCode() : 0);
		_result = 31 * _result + (majorCurrency != null ? majorCurrency.hashCode() : 0);
		_result = 31 * _result + (minimumTransferAmount != null ? minimumTransferAmount.hashCode() : 0);
		_result = 31 * _result + (rounding != null ? rounding.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "CreditSupportObligationsVariationMargin {" +
			"eligibleCollateral=" + this.eligibleCollateral + ", " +
			"fxHaircut=" + this.fxHaircut + ", " +
			"ineligibleCreditSupport=" + this.ineligibleCreditSupport + ", " +
			"majorCurrency=" + this.majorCurrency + ", " +
			"minimumTransferAmount=" + this.minimumTransferAmount + ", " +
			"rounding=" + this.rounding +
		'}';
	}
}
