package org.isda.cdm;

import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.CashSettlementReferenceBanks;
import org.isda.cdm.QuotationRateTypeEnum;
import org.isda.cdm.metafields.FieldWithMetaString;
import org.isda.cdm.metafields.FieldWithMetaString.FieldWithMetaStringBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.CrossCurrencyMethodMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;

/**
 * A class to represent the cash settlement method defined in the 2006 ISDA Definitions, Section 18.3. Cash Settlement Methods, paragraph (g) (published in Supplement number 28).
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="CrossCurrencyMethod", source="FpML_5_10")
public class CrossCurrencyMethod extends RosettaModelObject {
	private final List<FieldWithMetaString> cashSettlementCurrency;
	private final List<CashSettlementReferenceBanks> cashSettlementReferenceBanks;
	private final QuotationRateTypeEnum quotationRateType;
	private static CrossCurrencyMethodMeta metaData = new CrossCurrencyMethodMeta();
	
	CrossCurrencyMethod(CrossCurrencyMethodBuilder builder) {
		this.cashSettlementCurrency = ofNullable(builder.getCashSettlementCurrency()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.cashSettlementReferenceBanks = ofNullable(builder.getCashSettlementReferenceBanks()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.quotationRateType = builder.getQuotationRateType();
	}
	
	/**
	 * The currency, or currencies, in which the cash settlement amount(s) will be calculated and settled. While the order in which the currencies are stated is unimportant, the cash settlement currency or currencies must correspond to one or both of the constituent currencies of the swap transaction. The list of valid currencies is not presently positioned as an enumeration as part of the CDM because that scope is limited to the values specified by ISDA and FpML. As a result, implementers have to make reference to the relevant standard, such as the ISO 4217 standard for currency codes.
	 */
	@RosettaSynonym(value="cashSettlementCurrency", source="FpML_5_10")
	@RosettaSynonym(value="cashSettlementCurrency", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="cashSettlementCurrency", source="DTCC_11_0")
	@RosettaSynonym(value="cashSettlementCurrency", source="DTCC_9_0")
	@RosettaSynonym(value="cashSettlementCurrency", source="CME_ClearedConfirm_1_17")
	public final List<FieldWithMetaString> getCashSettlementCurrency() {
		return cashSettlementCurrency;
	}
	
	/**
	 * A container for a set of institutions that may be called upon to provide rate quotations as part of the method to determine the applicable cash settlement amount. If institutions are not specified, it is assumed that reference institutions will be agreed between the parties on the exercise date, or in the case of swap transaction to which mandatory early termination is applicable, the cash settlement valuation date.
	 */
	@RosettaSynonym(value="cashSettlementReferenceBanks", source="FpML_5_10")
	@RosettaSynonym(value="cashSettlementReferenceBanks", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="cashSettlementReferenceBanks", source="DTCC_11_0")
	@RosettaSynonym(value="cashSettlementReferenceBanks", source="DTCC_9_0")
	@RosettaSynonym(value="cashSettlementReferenceBanks", source="CME_ClearedConfirm_1_17")
	public final List<CashSettlementReferenceBanks> getCashSettlementReferenceBanks() {
		return cashSettlementReferenceBanks;
	}
	
	/**
	 * Which rate quote is to be observed, either Bid, Mid, Offer or Exercising Party Pays. The meaning of Exercising Party Pays is defined in the 2000 ISDA Definitions, Section 17.2. Certain Definitions Relating to Cash Settlement, paragraph (j).
	 */
	@RosettaSynonym(value="quotationRateType", source="FpML_5_10")
	@RosettaSynonym(value="quotationRateType", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="quotationRateType", source="DTCC_11_0")
	@RosettaSynonym(value="quotationRateType", source="DTCC_9_0")
	@RosettaSynonym(value="quotationRateType", source="CME_ClearedConfirm_1_17")
	public final QuotationRateTypeEnum getQuotationRateType() {
		return quotationRateType;
	}
	
	
	@Override
	public RosettaMetaData<? extends CrossCurrencyMethod> metaData() {
		return metaData;
	} 
	
	public CrossCurrencyMethodBuilder toBuilder() {
		CrossCurrencyMethodBuilder builder = new CrossCurrencyMethodBuilder();
		ofNullable(getCashSettlementCurrency()).ifPresent(cashSettlementCurrency -> cashSettlementCurrency.forEach(builder::addCashSettlementCurrency));
		ofNullable(getCashSettlementReferenceBanks()).ifPresent(cashSettlementReferenceBanks -> cashSettlementReferenceBanks.forEach(builder::addCashSettlementReferenceBanks));
		ofNullable(getQuotationRateType()).ifPresent(builder::setQuotationRateType);
		return builder;
	}

	public static CrossCurrencyMethodBuilder builder() {
		return new CrossCurrencyMethodBuilder();
	}

	public static  class CrossCurrencyMethodBuilder extends RosettaModelObjectBuilder {
	
		protected List<FieldWithMetaStringBuilder> cashSettlementCurrency;
		protected List<CashSettlementReferenceBanks.CashSettlementReferenceBanksBuilder> cashSettlementReferenceBanks;
		protected QuotationRateTypeEnum quotationRateType;
	
		public CrossCurrencyMethodBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends CrossCurrencyMethod> metaData() {
			return metaData;
		} 
	
		
		public List<FieldWithMetaStringBuilder> getCashSettlementCurrency() {
			return cashSettlementCurrency;
		}
		
		public FieldWithMetaStringBuilder getOrCreateCashSettlementCurrency(int index) {
			if (cashSettlementCurrency==null) {
				this.cashSettlementCurrency = new ArrayList<>();
			}
			return getIndex(cashSettlementCurrency, index, ()->new FieldWithMetaStringBuilder());
		}
		
		
		public List<CashSettlementReferenceBanks.CashSettlementReferenceBanksBuilder> getCashSettlementReferenceBanks() {
			return cashSettlementReferenceBanks;
		}
		
		public CashSettlementReferenceBanks.CashSettlementReferenceBanksBuilder getOrCreateCashSettlementReferenceBanks(int index) {
			if (cashSettlementReferenceBanks==null) {
				this.cashSettlementReferenceBanks = new ArrayList<>();
			}
			return getIndex(cashSettlementReferenceBanks, index, ()->new CashSettlementReferenceBanks.CashSettlementReferenceBanksBuilder());
		}
		
		
		public QuotationRateTypeEnum getQuotationRateType() {
			return quotationRateType;
		}
		
	
		public CrossCurrencyMethodBuilder addCashSettlementCurrency(FieldWithMetaString cashSettlementCurrency) {
			if(this.cashSettlementCurrency == null){
				this.cashSettlementCurrency = new ArrayList<>();
			}
			this.cashSettlementCurrency.add(cashSettlementCurrency.toBuilder());
			return this;
		}
		public CrossCurrencyMethodBuilder addCashSettlementCurrency(FieldWithMetaString cashSettlementCurrency, int _idx) {
			if(this.cashSettlementCurrency == null){
				this.cashSettlementCurrency = new ArrayList<>();
			}
			getIndex(this.cashSettlementCurrency, _idx, () -> cashSettlementCurrency.toBuilder());
			this.cashSettlementCurrency.set(_idx, cashSettlementCurrency.toBuilder());
			return this;
		}
		public CrossCurrencyMethodBuilder addCashSettlementCurrency(List<FieldWithMetaString> cashSettlementCurrencys) {
			if(this.cashSettlementCurrency == null){
				this.cashSettlementCurrency = new ArrayList<>();
			}
			for (FieldWithMetaString toAdd : cashSettlementCurrencys) {
				this.cashSettlementCurrency.add(toAdd.toBuilder());
			}
			return this;
		}
		
		public CrossCurrencyMethodBuilder clearCashSettlementCurrency() {
			this.cashSettlementCurrency = null;
			return this;
		}
		public CrossCurrencyMethodBuilder addCashSettlementReferenceBanks(CashSettlementReferenceBanks cashSettlementReferenceBanks) {
			if(this.cashSettlementReferenceBanks == null){
				this.cashSettlementReferenceBanks = new ArrayList<>();
			}
			this.cashSettlementReferenceBanks.add(cashSettlementReferenceBanks.toBuilder());
			return this;
		}
		public CrossCurrencyMethodBuilder addCashSettlementReferenceBanks(CashSettlementReferenceBanks cashSettlementReferenceBanks, int _idx) {
			if(this.cashSettlementReferenceBanks == null){
				this.cashSettlementReferenceBanks = new ArrayList<>();
			}
			getIndex(this.cashSettlementReferenceBanks, _idx, () -> cashSettlementReferenceBanks.toBuilder());
			this.cashSettlementReferenceBanks.set(_idx, cashSettlementReferenceBanks.toBuilder());
			return this;
		}
		public CrossCurrencyMethodBuilder addCashSettlementReferenceBanks(List<CashSettlementReferenceBanks> cashSettlementReferenceBankss) {
			if(this.cashSettlementReferenceBanks == null){
				this.cashSettlementReferenceBanks = new ArrayList<>();
			}
			for (CashSettlementReferenceBanks toAdd : cashSettlementReferenceBankss) {
				this.cashSettlementReferenceBanks.add(toAdd.toBuilder());
			}
			return this;
		}
		public CrossCurrencyMethodBuilder addCashSettlementReferenceBanksBuilder(CashSettlementReferenceBanks.CashSettlementReferenceBanksBuilder cashSettlementReferenceBanks) {
			if(this.cashSettlementReferenceBanks == null){
				this.cashSettlementReferenceBanks = new ArrayList<>();
			}
			this.cashSettlementReferenceBanks.add(cashSettlementReferenceBanks);
			return this;
		}
		
		public CrossCurrencyMethodBuilder clearCashSettlementReferenceBanks() {
			this.cashSettlementReferenceBanks = null;
			return this;
		}
		public CrossCurrencyMethodBuilder setQuotationRateType(QuotationRateTypeEnum quotationRateType) {
			this.quotationRateType = quotationRateType;
			return this;
		}
		
		public CrossCurrencyMethod build() {
			return new CrossCurrencyMethod(this);
		}
	
		@Override
		public CrossCurrencyMethodBuilder prune() {
			if (cashSettlementCurrency!=null) cashSettlementCurrency = cashSettlementCurrency.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (cashSettlementReferenceBanks!=null) cashSettlementReferenceBanks = cashSettlementReferenceBanks.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getCashSettlementCurrency()!=null && !getCashSettlementCurrency().isEmpty()) return true;
			if (getCashSettlementReferenceBanks()!=null && getCashSettlementReferenceBanks().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getQuotationRateType()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("quotationRateType"), QuotationRateTypeEnum.class, quotationRateType, this);
			
			processRosetta(path.newSubPath("cashSettlementCurrency"), processor, FieldWithMetaString.class, cashSettlementCurrency);
			processRosetta(path.newSubPath("cashSettlementReferenceBanks"), processor, CashSettlementReferenceBanks.class, cashSettlementReferenceBanks);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			CrossCurrencyMethodBuilder _that = (CrossCurrencyMethodBuilder) o;
		
			if (!ListEquals.listEquals(cashSettlementCurrency, _that.cashSettlementCurrency)) return false;
			if (!ListEquals.listEquals(cashSettlementReferenceBanks, _that.cashSettlementReferenceBanks)) return false;
			if (quotationRateType != null ? !quotationRateType.equals(_that.quotationRateType) : _that.quotationRateType != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (cashSettlementCurrency != null ? cashSettlementCurrency.hashCode() : 0);
			_result = 31 * _result + (cashSettlementReferenceBanks != null ? cashSettlementReferenceBanks.hashCode() : 0);
			_result = 31 * _result + (quotationRateType != null ? quotationRateType.getClass().getName().hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "CrossCurrencyMethodBuilder {" +
				"cashSettlementCurrency=" + this.cashSettlementCurrency + ", " +
				"cashSettlementReferenceBanks=" + this.cashSettlementReferenceBanks + ", " +
				"quotationRateType=" + this.quotationRateType +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("quotationRateType"), QuotationRateTypeEnum.class, quotationRateType, this);
		
		processRosetta(path.newSubPath("cashSettlementCurrency"), processor, FieldWithMetaString.class, cashSettlementCurrency);
		processRosetta(path.newSubPath("cashSettlementReferenceBanks"), processor, CashSettlementReferenceBanks.class, cashSettlementReferenceBanks);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		CrossCurrencyMethod _that = (CrossCurrencyMethod) o;
	
		if (!ListEquals.listEquals(cashSettlementCurrency, _that.cashSettlementCurrency)) return false;
		if (!ListEquals.listEquals(cashSettlementReferenceBanks, _that.cashSettlementReferenceBanks)) return false;
		if (quotationRateType != null ? !quotationRateType.equals(_that.quotationRateType) : _that.quotationRateType != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (cashSettlementCurrency != null ? cashSettlementCurrency.hashCode() : 0);
		_result = 31 * _result + (cashSettlementReferenceBanks != null ? cashSettlementReferenceBanks.hashCode() : 0);
		_result = 31 * _result + (quotationRateType != null ? quotationRateType.getClass().getName().hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "CrossCurrencyMethod {" +
			"cashSettlementCurrency=" + this.cashSettlementCurrency + ", " +
			"cashSettlementReferenceBanks=" + this.cashSettlementReferenceBanks + ", " +
			"quotationRateType=" + this.quotationRateType +
		'}';
	}
}
