package org.isda.cdm;

import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.meta.RosettaMetaData;
import java.math.BigDecimal;
import org.isda.cdm.meta.CrossRateMeta;
import org.isda.cdm.metafields.FieldWithMetaString;

import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.BuilderProcessor;
import com.rosetta.model.lib.process.Processor;

import static java.util.Optional.ofNullable;

/**
 * A class that is used for including the currency exchange rates used to cross between the traded currencies for non-base currency FX contracts.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="CrossRate", source="FpML_5_10")
public class CrossRate extends QuotedCurrencyPair {
	private final BigDecimal forwardPoints;
	private final BigDecimal rate;
	private final BigDecimal spotRate;
	private static CrossRateMeta metaData = new CrossRateMeta();
	
	CrossRate(CrossRateBuilder builder) {
		super(builder);
		this.forwardPoints = builder.getForwardPoints();
		this.rate = builder.getRate();
		this.spotRate = builder.getSpotRate();
	}
	
	/**
	 * An optional element used for deals consummated in the FX Forwards market. Forward points represent the interest rate differential between the two currencies traded and are quoted as a premium or a discount. Forward points are added to, or subtracted from, the spot rate to create the rate of the forward trade.
	 */
	@RosettaSynonym(value="forwardPoints", source="FpML_5_10")
	public final BigDecimal getForwardPoints() {
		return forwardPoints;
	}
	
	/**
	 * The exchange rate used to cross between the traded currencies.
	 */
	@RosettaSynonym(value="rate", source="FpML_5_10")
	public final BigDecimal getRate() {
		return rate;
	}
	
	/**
	 * An optional element used for FX forwards and certain types of FX OTC options. For deals consummated in the FX Forwards Market, this represents the current market rate for a particular currency pair.
	 */
	@RosettaSynonym(value="spotRate", source="FpML_5_10")
	public final BigDecimal getSpotRate() {
		return spotRate;
	}
	
	
	@Override
	public RosettaMetaData<? extends CrossRate> metaData() {
		return metaData;
	} 
	
	public CrossRateBuilder toBuilder() {
		CrossRateBuilder builder = new CrossRateBuilder();
		ofNullable(getForwardPoints()).ifPresent(builder::setForwardPoints);
		ofNullable(getRate()).ifPresent(builder::setRate);
		ofNullable(getSpotRate()).ifPresent(builder::setSpotRate);
		ofNullable(getCurrency1()).ifPresent(builder::setCurrency1);
		ofNullable(getCurrency2()).ifPresent(builder::setCurrency2);
		ofNullable(getQuoteBasis()).ifPresent(builder::setQuoteBasis);
		return builder;
	}

	public static CrossRateBuilder builder() {
		return new CrossRateBuilder();
	}

	public static class CrossRateBuilder extends QuotedCurrencyPairBuilder{
	
		protected BigDecimal forwardPoints;
		protected BigDecimal rate;
		protected BigDecimal spotRate;
	
		public CrossRateBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends CrossRate> metaData() {
			return metaData;
		} 
	
		
		public BigDecimal getForwardPoints() {
			return forwardPoints;
		}
		
		
		public BigDecimal getRate() {
			return rate;
		}
		
		
		public BigDecimal getSpotRate() {
			return spotRate;
		}
		
	
		public CrossRateBuilder setForwardPoints(BigDecimal forwardPoints) {
			this.forwardPoints = forwardPoints;
			return this;
		}
		public CrossRateBuilder setRate(BigDecimal rate) {
			this.rate = rate;
			return this;
		}
		public CrossRateBuilder setSpotRate(BigDecimal spotRate) {
			this.spotRate = spotRate;
			return this;
		}
		@Override public CrossRateBuilder setCurrency1(FieldWithMetaString currency1) {
			this.currency1 = currency1.toBuilder();
			return this;
		}
		@Override public CrossRateBuilder setCurrency1Ref(String currency1) {
			return setCurrency1(FieldWithMetaString.builder().setValue(currency1).build());
		}
		@Override public CrossRateBuilder setCurrency2(FieldWithMetaString currency2) {
			this.currency2 = currency2.toBuilder();
			return this;
		}
		@Override public CrossRateBuilder setCurrency2Ref(String currency2) {
			return setCurrency2(FieldWithMetaString.builder().setValue(currency2).build());
		}
		@Override public CrossRateBuilder setQuoteBasis(QuoteBasisEnum quoteBasis) {
			this.quoteBasis = quoteBasis;
			return this;
		}
	
		public CrossRate build() {
			return new CrossRate(this);
		}
	
		@Override
		public CrossRateBuilder prune() {
			super.prune();
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (super.hasData()) return true;
			if (getForwardPoints()!=null) return true;
			if (getRate()!=null) return true;
			if (getSpotRate()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
			super.process(path, processor);
		
			processor.processBasic(path.newSubPath("forwardPoints"), BigDecimal.class, forwardPoints, this);
			processor.processBasic(path.newSubPath("rate"), BigDecimal.class, rate, this);
			processor.processBasic(path.newSubPath("spotRate"), BigDecimal.class, spotRate, this);
			
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
			if (!super.equals(o)) return false;
		
			CrossRateBuilder _that = (CrossRateBuilder) o;
		
			if (forwardPoints != null ? !forwardPoints.equals(_that.forwardPoints) : _that.forwardPoints != null) return false;
			if (rate != null ? !rate.equals(_that.rate) : _that.rate != null) return false;
			if (spotRate != null ? !spotRate.equals(_that.spotRate) : _that.spotRate != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = super.hashCode();
			_result = 31 * _result + (forwardPoints != null ? forwardPoints.hashCode() : 0);
			_result = 31 * _result + (rate != null ? rate.hashCode() : 0);
			_result = 31 * _result + (spotRate != null ? spotRate.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "CrossRateBuilder {" +
				"forwardPoints=" + this.forwardPoints + ", " +
				"rate=" + this.rate + ", " +
				"spotRate=" + this.spotRate +
			'}' + " " + super.toString();
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		super.process(path, processor);
		
		processor.processBasic(path.newSubPath("forwardPoints"), BigDecimal.class, forwardPoints, this);
		processor.processBasic(path.newSubPath("rate"), BigDecimal.class, rate, this);
		processor.processBasic(path.newSubPath("spotRate"), BigDecimal.class, spotRate, this);
		
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
		if (!super.equals(o)) return false;
	
		CrossRate _that = (CrossRate) o;
	
		if (forwardPoints != null ? !forwardPoints.equals(_that.forwardPoints) : _that.forwardPoints != null) return false;
		if (rate != null ? !rate.equals(_that.rate) : _that.rate != null) return false;
		if (spotRate != null ? !spotRate.equals(_that.spotRate) : _that.spotRate != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = super.hashCode();
		_result = 31 * _result + (forwardPoints != null ? forwardPoints.hashCode() : 0);
		_result = 31 * _result + (rate != null ? rate.hashCode() : 0);
		_result = 31 * _result + (spotRate != null ? spotRate.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "CrossRate {" +
			"forwardPoints=" + this.forwardPoints + ", " +
			"rate=" + this.rate + ", " +
			"spotRate=" + this.spotRate +
		'}' + " " + super.toString();
	}
}
