package org.isda.cdm;

import java.lang.String;
import org.isda.cdm.AdditionalRepresentation;
import org.isda.cdm.ConditionsPrecedent;
import org.isda.cdm.ContactElection;
import org.isda.cdm.DisputeResolution;
import org.isda.cdm.Substitution;
import org.isda.cdm.metafields.FieldWithMetaString;
import org.isda.cdm.metafields.FieldWithMetaString.FieldWithMetaStringBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.Csa2016Meta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * An abstract class to specify the provisions for the 2016 ISDA Credit Support Annex that are common among governing laws and across Initial and Variation Margin.
 * 
 * @version 2.5.4
 */
@RosettaClass
public abstract class Csa2016 extends RosettaModelObject {
	private final String additionalObligations;
	private final AdditionalRepresentation additionalRepresentation;
	private final ContactElection addressesForTransfer;
	private final FieldWithMetaString baseCurrency;
	private final String bespokeProvision;
	private final ConditionsPrecedent conditionsPrecedent;
	private final ContactElection demandsAndNotices;
	private final DisputeResolution disputeResolution;
	private final Substitution substitution;
	private static Csa2016Meta metaData = new Csa2016Meta();
	
	Csa2016(Csa2016Builder builder) {
		this.additionalObligations = builder.getAdditionalObligations();
		this.additionalRepresentation = ofNullable(builder.getAdditionalRepresentation()).map(f->f.build()).orElse(null);
		this.addressesForTransfer = ofNullable(builder.getAddressesForTransfer()).map(f->f.build()).orElse(null);
		this.baseCurrency = ofNullable(builder.getBaseCurrency()).map(f->f.build()).orElse(null);
		this.bespokeProvision = builder.getBespokeProvision();
		this.conditionsPrecedent = ofNullable(builder.getConditionsPrecedent()).map(f->f.build()).orElse(null);
		this.demandsAndNotices = ofNullable(builder.getDemandsAndNotices()).map(f->f.build()).orElse(null);
		this.disputeResolution = ofNullable(builder.getDisputeResolution()).map(f->f.build()).orElse(null);
		this.substitution = ofNullable(builder.getSubstitution()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The additional obligations that might be specified by the parties to an ISDA CSA. ISDA 2016 English Law Credit Support Deed for Initial Margin, paragraph 13, General Principles, (b)(ii). | ISDA 2016 Japanese Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (c)(ii). | ISDA 2016 New York Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (b)(ii). | ISDA 2016 New York Law Credit Support Annex for Variation Margin, paragraph 13, (b)(ii).
	 */
	@RosettaSynonym(value="additional_obligations_specify", source="ISDA_Create_1_0", path="additional_obligations")
	public final String getAdditionalObligations() {
		return additionalObligations;
	}
	
	/**
	 * ISDA 2016 Credit Support Annex for Initial Margin, paragraph 13, General Principles, (o): Additional Representation(s). | ISDA 2016 Credit Support Annex for Variation Margin, paragraph 13, (k): Additional Representation(s).
	 */
	@RosettaSynonym(value="additional_representations", source="ISDA_Create_1_0")
	public final AdditionalRepresentation getAdditionalRepresentation() {
		return additionalRepresentation;
	}
	
	/**
	 * The address for transfer as specified by the respective parties to the agreement. ISDA 2016 Credit Support Annex for Initial Margin, paragraph 13, General Principles, (r): Addresses for Transfers. | ISDA 2016 Credit Support Annex for Variation Margin, paragraph 13, (n): Addresses for Transfers.
	 */
	@RosettaSynonym(value="addresses_for_transfers", source="ISDA_Create_1_0")
	public final ContactElection getAddressesForTransfer() {
		return addressesForTransfer;
	}
	
	/**
	 * The base currency for the Credit Support Annex document as elected by the parties to the agreement. The list of valid currencies is not presently positioned as an enumeration as part of the CDM because that scope is limited to the values specified by ISDA and FpML. As a result, implementers have to make reference to the relevant standard, such as the ISO 4217 standard for currency codes. ISDA 2016 CSA for Initial Margin, Paragraph 13 (a) | ISDA 2016 CSA for Variation Margin, Paragraph 13 (a).
	 */
	@RosettaSynonym(value="currency", source="ISDA_Create_1_0", path="base_currency")
	public final FieldWithMetaString getBaseCurrency() {
		return baseCurrency;
	}
	
	/**
	 * The bespoke provision that might be specified by the parties to the agreement. ISDA 2016 Credit Support Annex for Initial Margin, paragraph 13, General Principles, (u)(iii): Other Provisions. | ISDA 2016 Credit Support Annex for Variation Margin, paragraph 13, (p): Other Provisions.
	 */
	@RosettaSynonym(value="specify", source="ISDA_Create_1_0", path="bespoke_provisions")
	public final String getBespokeProvision() {
		return bespokeProvision;
	}
	
	/**
	 * The set of elections specified in Paragraph 13 that may overwrite the default Condition Precedent provision as specified in ISDA 2016 Credit Support Annex for Initial Margin, Paragraph 4, (a), and the set of provisions that are deemed Access Condition (Initial Margin) and Specified Condition (Variation Margin). ISDA 2016 English Law Credit Support Deed for Initial Margin, paragraph 13, General Principles, (e): Conditions Precedent. | ISDA 2016 Japanese Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (f): Conditions Precedent. | ISDA 2016 New York Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (e): Conditions Precedent. | ISDA 2016 Credit Support Annex for Variation Margin, paragraph 13, (e): Conditions Precedent and Secured Party’s Rights and Remedies.
	 */
	public final ConditionsPrecedent getConditionsPrecedent() {
		return conditionsPrecedent;
	}
	
	/**
	 * The address where the demands, specifications and notices will be communicated to within each of the parties to the agreement. ISDA 2016 Credit Support Annex for Initial Margin, paragraph 13, General Principles, (q): Demands and Notices. | ISDA 2016 Credit Support Annex for Variation Margin, paragraph 13, (m): Demands and Notices.
	 */
	@RosettaSynonym(value="demands_and_notices", source="ISDA_Create_1_0")
	public final ContactElection getDemandsAndNotices() {
		return demandsAndNotices;
	}
	
	/**
	 * The election terms under which a party disputes (i) the Calculation Agent’s calculation of a Delivery Amount or a Return Amount, or (ii) the Value of any Transfer of Eligible Credit Support or Posted Credit Support. ISDA 2016 English Law Credit Support Deed for Initial Margin, paragraph 13, General Principles, (g): Dispute Resolution. | ISDA 2016 Japanese Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (h): Dispute Resolution. | ISDA 2016 New York Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (g): Dispute Resolution. | ISDA 2016 Credit Support Annex for Variation Margin, paragraph 13, (g): Dispute Resolution.
	 */
	@RosettaSynonym(value="resolution_time", source="ISDA_Create_1_0")
	public final DisputeResolution getDisputeResolution() {
		return disputeResolution;
	}
	
	/**
	 * The conditions under which the Pledgor/Obligor/Chargor can substitute posted collateral. ISDA 2016 English Law Credit Support Deed for Initial Margin, paragraph 13, General Principles, (f): Substitution. | ISDA 2016 Japanese Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (g): Substitution. | ISDA 2016 New York Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (f): Substitution. | ISDA 2016 Credit Support Annex for Variation Margin, paragraph 13, (f): Substitution.
	 */
	@RosettaSynonym(value="consent", source="ISDA_Create_1_0")
	public final Substitution getSubstitution() {
		return substitution;
	}
	
	
	@Override
	public RosettaMetaData<? extends Csa2016> metaData() {
		return metaData;
	} 
	
	public abstract Csa2016Builder toBuilder();


	public static abstract
	 class Csa2016Builder extends RosettaModelObjectBuilder {
	
		protected String additionalObligations;
		protected AdditionalRepresentation.AdditionalRepresentationBuilder additionalRepresentation;
		protected ContactElection.ContactElectionBuilder addressesForTransfer;
		protected FieldWithMetaStringBuilder baseCurrency;
		protected String bespokeProvision;
		protected ConditionsPrecedent.ConditionsPrecedentBuilder conditionsPrecedent;
		protected ContactElection.ContactElectionBuilder demandsAndNotices;
		protected DisputeResolution.DisputeResolutionBuilder disputeResolution;
		protected Substitution.SubstitutionBuilder substitution;
	
		public Csa2016Builder() {
		}
				
		@Override
		public RosettaMetaData<? extends Csa2016> metaData() {
			return metaData;
		} 
	
		
		public String getAdditionalObligations() {
			return additionalObligations;
		}
		
		
		public AdditionalRepresentation.AdditionalRepresentationBuilder getAdditionalRepresentation() {
			return additionalRepresentation;
		}
		
		public AdditionalRepresentation.AdditionalRepresentationBuilder getOrCreateAdditionalRepresentation() {
			if (additionalRepresentation!=null) {
				return additionalRepresentation;
			}
			else return additionalRepresentation = new AdditionalRepresentation.AdditionalRepresentationBuilder();
		}
		
		
		public ContactElection.ContactElectionBuilder getAddressesForTransfer() {
			return addressesForTransfer;
		}
		
		public ContactElection.ContactElectionBuilder getOrCreateAddressesForTransfer() {
			if (addressesForTransfer!=null) {
				return addressesForTransfer;
			}
			else return addressesForTransfer = new ContactElection.ContactElectionBuilder();
		}
		
		
		public FieldWithMetaStringBuilder getBaseCurrency() {
			return baseCurrency;
		}
		
		public FieldWithMetaStringBuilder getOrCreateBaseCurrency() {
			if (baseCurrency!=null) {
				return baseCurrency;
			}
			else return baseCurrency = new FieldWithMetaStringBuilder();
		}
		
		
		public String getBespokeProvision() {
			return bespokeProvision;
		}
		
		
		public ConditionsPrecedent.ConditionsPrecedentBuilder getConditionsPrecedent() {
			return conditionsPrecedent;
		}
		
		public ConditionsPrecedent.ConditionsPrecedentBuilder getOrCreateConditionsPrecedent() {
			if (conditionsPrecedent!=null) {
				return conditionsPrecedent;
			}
			else return conditionsPrecedent = new ConditionsPrecedent.ConditionsPrecedentBuilder();
		}
		
		
		public ContactElection.ContactElectionBuilder getDemandsAndNotices() {
			return demandsAndNotices;
		}
		
		public ContactElection.ContactElectionBuilder getOrCreateDemandsAndNotices() {
			if (demandsAndNotices!=null) {
				return demandsAndNotices;
			}
			else return demandsAndNotices = new ContactElection.ContactElectionBuilder();
		}
		
		
		public DisputeResolution.DisputeResolutionBuilder getDisputeResolution() {
			return disputeResolution;
		}
		
		public DisputeResolution.DisputeResolutionBuilder getOrCreateDisputeResolution() {
			if (disputeResolution!=null) {
				return disputeResolution;
			}
			else return disputeResolution = new DisputeResolution.DisputeResolutionBuilder();
		}
		
		
		public Substitution.SubstitutionBuilder getSubstitution() {
			return substitution;
		}
		
		public Substitution.SubstitutionBuilder getOrCreateSubstitution() {
			if (substitution!=null) {
				return substitution;
			}
			else return substitution = new Substitution.SubstitutionBuilder();
		}
		
	
		public Csa2016Builder setAdditionalObligations(String additionalObligations) {
			this.additionalObligations = additionalObligations;
			return this;
		}
		public Csa2016Builder setAdditionalRepresentation(AdditionalRepresentation additionalRepresentation) {
			this.additionalRepresentation = additionalRepresentation.toBuilder();
			return this;
		}
		public Csa2016Builder setAdditionalRepresentationBuilder(AdditionalRepresentation.AdditionalRepresentationBuilder additionalRepresentation) {
			this.additionalRepresentation = additionalRepresentation;
			return this;
		}
		public Csa2016Builder setAddressesForTransfer(ContactElection addressesForTransfer) {
			this.addressesForTransfer = addressesForTransfer.toBuilder();
			return this;
		}
		public Csa2016Builder setAddressesForTransferBuilder(ContactElection.ContactElectionBuilder addressesForTransfer) {
			this.addressesForTransfer = addressesForTransfer;
			return this;
		}
		public Csa2016Builder setBaseCurrency(FieldWithMetaString baseCurrency) {
			this.baseCurrency = baseCurrency.toBuilder();
			return this;
		}
		public Csa2016Builder setBaseCurrencyRef(String baseCurrency) {
			return setBaseCurrency(FieldWithMetaString.builder().setValue(baseCurrency).build());
		}
		public Csa2016Builder setBespokeProvision(String bespokeProvision) {
			this.bespokeProvision = bespokeProvision;
			return this;
		}
		public Csa2016Builder setConditionsPrecedent(ConditionsPrecedent conditionsPrecedent) {
			this.conditionsPrecedent = conditionsPrecedent.toBuilder();
			return this;
		}
		public Csa2016Builder setConditionsPrecedentBuilder(ConditionsPrecedent.ConditionsPrecedentBuilder conditionsPrecedent) {
			this.conditionsPrecedent = conditionsPrecedent;
			return this;
		}
		public Csa2016Builder setDemandsAndNotices(ContactElection demandsAndNotices) {
			this.demandsAndNotices = demandsAndNotices.toBuilder();
			return this;
		}
		public Csa2016Builder setDemandsAndNoticesBuilder(ContactElection.ContactElectionBuilder demandsAndNotices) {
			this.demandsAndNotices = demandsAndNotices;
			return this;
		}
		public Csa2016Builder setDisputeResolution(DisputeResolution disputeResolution) {
			this.disputeResolution = disputeResolution.toBuilder();
			return this;
		}
		public Csa2016Builder setDisputeResolutionBuilder(DisputeResolution.DisputeResolutionBuilder disputeResolution) {
			this.disputeResolution = disputeResolution;
			return this;
		}
		public Csa2016Builder setSubstitution(Substitution substitution) {
			this.substitution = substitution.toBuilder();
			return this;
		}
		public Csa2016Builder setSubstitutionBuilder(Substitution.SubstitutionBuilder substitution) {
			this.substitution = substitution;
			return this;
		}
		
		public abstract Csa2016 build();
	
		@Override
		public Csa2016Builder prune() {
			if (additionalRepresentation!=null && !additionalRepresentation.prune().hasData()) additionalRepresentation = null;
			if (addressesForTransfer!=null && !addressesForTransfer.prune().hasData()) addressesForTransfer = null;
			if (baseCurrency!=null && !baseCurrency.prune().hasData()) baseCurrency = null;
			if (conditionsPrecedent!=null && !conditionsPrecedent.prune().hasData()) conditionsPrecedent = null;
			if (demandsAndNotices!=null && !demandsAndNotices.prune().hasData()) demandsAndNotices = null;
			if (disputeResolution!=null && !disputeResolution.prune().hasData()) disputeResolution = null;
			if (substitution!=null && !substitution.prune().hasData()) substitution = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getAdditionalObligations()!=null) return true;
			if (getAdditionalRepresentation()!=null && getAdditionalRepresentation().hasData()) return true;
			if (getAddressesForTransfer()!=null && getAddressesForTransfer().hasData()) return true;
			if (getBaseCurrency()!=null) return true;
			if (getBespokeProvision()!=null) return true;
			if (getConditionsPrecedent()!=null && getConditionsPrecedent().hasData()) return true;
			if (getDemandsAndNotices()!=null && getDemandsAndNotices().hasData()) return true;
			if (getDisputeResolution()!=null && getDisputeResolution().hasData()) return true;
			if (getSubstitution()!=null && getSubstitution().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("additionalObligations"), String.class, additionalObligations, this);
			processor.processBasic(path.newSubPath("bespokeProvision"), String.class, bespokeProvision, this);
			
			processRosetta(path.newSubPath("additionalRepresentation"), processor, AdditionalRepresentation.class, additionalRepresentation);
			processRosetta(path.newSubPath("addressesForTransfer"), processor, ContactElection.class, addressesForTransfer);
			processRosetta(path.newSubPath("baseCurrency"), processor, FieldWithMetaString.class, baseCurrency);
			processRosetta(path.newSubPath("conditionsPrecedent"), processor, ConditionsPrecedent.class, conditionsPrecedent);
			processRosetta(path.newSubPath("demandsAndNotices"), processor, ContactElection.class, demandsAndNotices);
			processRosetta(path.newSubPath("disputeResolution"), processor, DisputeResolution.class, disputeResolution);
			processRosetta(path.newSubPath("substitution"), processor, Substitution.class, substitution);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			Csa2016Builder _that = (Csa2016Builder) o;
		
			if (additionalObligations != null ? !additionalObligations.equals(_that.additionalObligations) : _that.additionalObligations != null) return false;
			if (additionalRepresentation != null ? !additionalRepresentation.equals(_that.additionalRepresentation) : _that.additionalRepresentation != null) return false;
			if (addressesForTransfer != null ? !addressesForTransfer.equals(_that.addressesForTransfer) : _that.addressesForTransfer != null) return false;
			if (baseCurrency != null ? !baseCurrency.equals(_that.baseCurrency) : _that.baseCurrency != null) return false;
			if (bespokeProvision != null ? !bespokeProvision.equals(_that.bespokeProvision) : _that.bespokeProvision != null) return false;
			if (conditionsPrecedent != null ? !conditionsPrecedent.equals(_that.conditionsPrecedent) : _that.conditionsPrecedent != null) return false;
			if (demandsAndNotices != null ? !demandsAndNotices.equals(_that.demandsAndNotices) : _that.demandsAndNotices != null) return false;
			if (disputeResolution != null ? !disputeResolution.equals(_that.disputeResolution) : _that.disputeResolution != null) return false;
			if (substitution != null ? !substitution.equals(_that.substitution) : _that.substitution != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (additionalObligations != null ? additionalObligations.hashCode() : 0);
			_result = 31 * _result + (additionalRepresentation != null ? additionalRepresentation.hashCode() : 0);
			_result = 31 * _result + (addressesForTransfer != null ? addressesForTransfer.hashCode() : 0);
			_result = 31 * _result + (baseCurrency != null ? baseCurrency.hashCode() : 0);
			_result = 31 * _result + (bespokeProvision != null ? bespokeProvision.hashCode() : 0);
			_result = 31 * _result + (conditionsPrecedent != null ? conditionsPrecedent.hashCode() : 0);
			_result = 31 * _result + (demandsAndNotices != null ? demandsAndNotices.hashCode() : 0);
			_result = 31 * _result + (disputeResolution != null ? disputeResolution.hashCode() : 0);
			_result = 31 * _result + (substitution != null ? substitution.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "Csa2016Builder {" +
				"additionalObligations=" + this.additionalObligations + ", " +
				"additionalRepresentation=" + this.additionalRepresentation + ", " +
				"addressesForTransfer=" + this.addressesForTransfer + ", " +
				"baseCurrency=" + this.baseCurrency + ", " +
				"bespokeProvision=" + this.bespokeProvision + ", " +
				"conditionsPrecedent=" + this.conditionsPrecedent + ", " +
				"demandsAndNotices=" + this.demandsAndNotices + ", " +
				"disputeResolution=" + this.disputeResolution + ", " +
				"substitution=" + this.substitution +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("additionalObligations"), String.class, additionalObligations, this);
		processor.processBasic(path.newSubPath("bespokeProvision"), String.class, bespokeProvision, this);
		
		processRosetta(path.newSubPath("additionalRepresentation"), processor, AdditionalRepresentation.class, additionalRepresentation);
		processRosetta(path.newSubPath("addressesForTransfer"), processor, ContactElection.class, addressesForTransfer);
		processRosetta(path.newSubPath("baseCurrency"), processor, FieldWithMetaString.class, baseCurrency);
		processRosetta(path.newSubPath("conditionsPrecedent"), processor, ConditionsPrecedent.class, conditionsPrecedent);
		processRosetta(path.newSubPath("demandsAndNotices"), processor, ContactElection.class, demandsAndNotices);
		processRosetta(path.newSubPath("disputeResolution"), processor, DisputeResolution.class, disputeResolution);
		processRosetta(path.newSubPath("substitution"), processor, Substitution.class, substitution);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		Csa2016 _that = (Csa2016) o;
	
		if (additionalObligations != null ? !additionalObligations.equals(_that.additionalObligations) : _that.additionalObligations != null) return false;
		if (additionalRepresentation != null ? !additionalRepresentation.equals(_that.additionalRepresentation) : _that.additionalRepresentation != null) return false;
		if (addressesForTransfer != null ? !addressesForTransfer.equals(_that.addressesForTransfer) : _that.addressesForTransfer != null) return false;
		if (baseCurrency != null ? !baseCurrency.equals(_that.baseCurrency) : _that.baseCurrency != null) return false;
		if (bespokeProvision != null ? !bespokeProvision.equals(_that.bespokeProvision) : _that.bespokeProvision != null) return false;
		if (conditionsPrecedent != null ? !conditionsPrecedent.equals(_that.conditionsPrecedent) : _that.conditionsPrecedent != null) return false;
		if (demandsAndNotices != null ? !demandsAndNotices.equals(_that.demandsAndNotices) : _that.demandsAndNotices != null) return false;
		if (disputeResolution != null ? !disputeResolution.equals(_that.disputeResolution) : _that.disputeResolution != null) return false;
		if (substitution != null ? !substitution.equals(_that.substitution) : _that.substitution != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (additionalObligations != null ? additionalObligations.hashCode() : 0);
		_result = 31 * _result + (additionalRepresentation != null ? additionalRepresentation.hashCode() : 0);
		_result = 31 * _result + (addressesForTransfer != null ? addressesForTransfer.hashCode() : 0);
		_result = 31 * _result + (baseCurrency != null ? baseCurrency.hashCode() : 0);
		_result = 31 * _result + (bespokeProvision != null ? bespokeProvision.hashCode() : 0);
		_result = 31 * _result + (conditionsPrecedent != null ? conditionsPrecedent.hashCode() : 0);
		_result = 31 * _result + (demandsAndNotices != null ? demandsAndNotices.hashCode() : 0);
		_result = 31 * _result + (disputeResolution != null ? disputeResolution.hashCode() : 0);
		_result = 31 * _result + (substitution != null ? substitution.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "Csa2016 {" +
			"additionalObligations=" + this.additionalObligations + ", " +
			"additionalRepresentation=" + this.additionalRepresentation + ", " +
			"addressesForTransfer=" + this.addressesForTransfer + ", " +
			"baseCurrency=" + this.baseCurrency + ", " +
			"bespokeProvision=" + this.bespokeProvision + ", " +
			"conditionsPrecedent=" + this.conditionsPrecedent + ", " +
			"demandsAndNotices=" + this.demandsAndNotices + ", " +
			"disputeResolution=" + this.disputeResolution + ", " +
			"substitution=" + this.substitution +
		'}';
	}
}
