package org.isda.cdm;

import java.lang.Boolean;
import org.isda.cdm.CalculationDateLocation;
import org.isda.cdm.CreditSupportObligationsInitialMargin;
import org.isda.cdm.FxHaircutCurrency;
import org.isda.cdm.Method;
import org.isda.cdm.NotificationTime;
import org.isda.cdm.OneWayProvisions;
import org.isda.cdm.Regime;
import org.isda.cdm.SensitivityMethodology;
import org.isda.cdm.TerminationCurrencyAmendment;
import org.isda.cdm.metafields.FieldWithMetaString;


import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.CsaInitialMargin2016Meta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * An abstract class to specify the provisions for the 2016 ISDA Credit Support Annex for Initial Margin that are common among governing laws.
 * 
 * @version 2.5.4
 */
@RosettaClass
public abstract class CsaInitialMargin2016 extends Csa2016 {
	private final CalculationDateLocation calculationDateLocation;
	private final CreditSupportObligationsInitialMargin creditSupportObligations;
	private final FxHaircutCurrency fxHaircutCurrency;
	private final Boolean identifiedCrossCurrencySwap;
	private final Method method;
	private final NotificationTime notificationTime;
	private final OneWayProvisions oneWayProvisions;
	private final Regime regime;
	private final SensitivityMethodology sensitivityToCommodity;
	private final SensitivityMethodology sensitivityToEquity;
	private final TerminationCurrencyAmendment terminationCurrency;
	private static CsaInitialMargin2016Meta metaData = new CsaInitialMargin2016Meta();
	
	CsaInitialMargin2016(CsaInitialMargin2016Builder builder) {
		super(builder);
		this.calculationDateLocation = ofNullable(builder.getCalculationDateLocation()).map(f->f.build()).orElse(null);
		this.creditSupportObligations = ofNullable(builder.getCreditSupportObligations()).map(f->f.build()).orElse(null);
		this.fxHaircutCurrency = ofNullable(builder.getFxHaircutCurrency()).map(f->f.build()).orElse(null);
		this.identifiedCrossCurrencySwap = builder.getIdentifiedCrossCurrencySwap();
		this.method = ofNullable(builder.getMethod()).map(f->f.build()).orElse(null);
		this.notificationTime = ofNullable(builder.getNotificationTime()).map(f->f.build()).orElse(null);
		this.oneWayProvisions = ofNullable(builder.getOneWayProvisions()).map(f->f.build()).orElse(null);
		this.regime = ofNullable(builder.getRegime()).map(f->f.build()).orElse(null);
		this.sensitivityToCommodity = ofNullable(builder.getSensitivityToCommodity()).map(f->f.build()).orElse(null);
		this.sensitivityToEquity = ofNullable(builder.getSensitivityToEquity()).map(f->f.build()).orElse(null);
		this.terminationCurrency = ofNullable(builder.getTerminationCurrency()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The specified location where the credit exposure will be calculated by the respective parties. ISDA 2016 English Law Credit Support Deed for Initial Margin, paragraph 13, General Principles, (d)(i): Calculation Date Location. | ISDA 2016 Japanese Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (e)(i): Calculation Date Location. | ISDA 2016 New York Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (d)(i): Calculation Date Location.
	 */
	@RosettaSynonym(value="calculation_date_location", source="ISDA_Create_1_0")
	public final CalculationDateLocation getCalculationDateLocation() {
		return calculationDateLocation;
	}
	
	/**
	 * The Credit Support Obligations applicable to the Initial Margin Credit Support Annex and which are common among the English, Japanese and New York governing laws. This excludes the Other Eligible Support election (which only applies to the Japanese Law and New York Law agreements) and the Transfer Timing election (which only applies to the English Law and the New York Law agreements). ISDA 2016 English Law Credit Support Deed for Initial Margin, paragraph 13, General Principles, (c): Credit Support Obligations. | ISDA 2016 Japanese Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (c): Credit Support Obligations. | ISDA 2016 New York Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (c): Credit Support Obligations. 
	 */
	public final CreditSupportObligationsInitialMargin getCreditSupportObligations() {
		return creditSupportObligations;
	}
	
	/**
	 * The reference currency for the purpose of specifying the FX Haircut relating to a posting obligation, as being either the Termination Currency or an FX Designated Currency.
	 */
	@RosettaSynonym(value="fx_haircut_percentage", source="ISDA_Create_1_0")
	public final FxHaircutCurrency getFxHaircutCurrency() {
		return fxHaircutCurrency;
	}
	
	/**
	 * The qualification of whether cross-currency swaps need to be identified in the Confirmation so that the obligations to exchange principal be disregarded for the purpose of determining the Delivery Amount or Return Amount. ISDA 2016 Credit Support Annex for Initial Margin, paragraph 13, General Principles, (gg)(1).
	 */
	@RosettaSynonym(value="", source="ISDA_Create_1_0")
	public final Boolean getIdentifiedCrossCurrencySwap() {
		return identifiedCrossCurrencySwap;
	}
	
	/**
	 * The specification of the ISDA SIMM Method for all Covered Transactions with respect to all Regimes. ISDA 2016 Credit Support Annex for Initial Margin, paragraph 13, General Principles, (ee).
	 */
	public final Method getMethod() {
		return method;
	}
	
	/**
	 * The time by which a demand for the Transfer of Eligible Credit Support (IM) or Posted Credit Support (IM) needs to be made in order for the transfer to take place in accordance with the Transfer Timing provisions. ISDA 2016 Credit Support Annex for Initial Margin, paragraph 13, General Principles, (d)(iii).
	 */
	@RosettaSynonym(value="notification_time", source="ISDA_Create_1_0")
	public final NotificationTime getNotificationTime() {
		return notificationTime;
	}
	
	/**
	 * The determination of whether the One Way Provisions are applicable (true) or not applicable (false) as specified by ISDA 2016 CSA for Initial Margin, Paragraph 13, (aa).
	 */
	@RosettaSynonym(value="one_way_provisions", source="ISDA_Create_1_0")
	public final OneWayProvisions getOneWayProvisions() {
		return oneWayProvisions;
	}
	
	/**
	 * The Regime Table provision as specified by the ISDA 2016 CSA for Initial Margin, Paragraph 13 General Principles, which determines the regulatory regime(s) applicable to each of the parties to the CSA in their capacity as Secured Party (English Law &amp; New York Law) or Obligee (Japanese Law), with one set of values per counterparty. As specified in the CSA, the applicability of a regime shall not be construed as a representation, admission or acknowledgement that either party is actually regulated under such regime.
	 */
	public final Regime getRegime() {
		return regime;
	}
	
	/**
	 * The methodology to compute sensitivities to commodity indices for the purpose of the ISDA 2016 and 2018 CSA for Initial Margin. ISDA 2016 Credit Support Annex for Initial Margin, paragraph 13, General Principles (gg)(2)(B).
	 */
	@RosettaSynonym(value="sensitivities_to_commodity_indices", source="ISDA_Create_1_0")
	public final SensitivityMethodology getSensitivityToCommodity() {
		return sensitivityToCommodity;
	}
	
	/**
	 * The methodology to compute sensitivities to equity indices, funds and ETFs for the purpose of the ISDA 2016 and 2018 CSA for Initial Margin. ISDA 2016 Credit Support Annex for Initial Margin, paragraph 13, General Principles (gg)(2)(A).
	 */
	@RosettaSynonym(value="sensitivities_to_equity_indices_funds_etfs", source="ISDA_Create_1_0")
	public final SensitivityMethodology getSensitivityToEquity() {
		return sensitivityToEquity;
	}
	
	/**
	 * The Amendment to Termination Currency elections by the parties to the agreement. ISDA 2016 English Law Credit Support Deed for Initial Margin, paragraph 13, General Principles, (v): Amendment to Termination Currency. | ISDA 2016 Japanese Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (t): Amendment to Termination Currency. | ISDA 2016 New York Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (t): Amendment to Termination Currency.
	 */
	@RosettaSynonym(value="amendment_to_termination_currency", source="ISDA_Create_1_0")
	public final TerminationCurrencyAmendment getTerminationCurrency() {
		return terminationCurrency;
	}
	
	
	@Override
	public RosettaMetaData<? extends CsaInitialMargin2016> metaData() {
		return metaData;
	} 
	
	public abstract CsaInitialMargin2016Builder toBuilder();


	public static abstract
	 class CsaInitialMargin2016Builder extends Csa2016Builder {
	
		protected CalculationDateLocation.CalculationDateLocationBuilder calculationDateLocation;
		protected CreditSupportObligationsInitialMargin.CreditSupportObligationsInitialMarginBuilder creditSupportObligations;
		protected FxHaircutCurrency.FxHaircutCurrencyBuilder fxHaircutCurrency;
		protected Boolean identifiedCrossCurrencySwap;
		protected Method.MethodBuilder method;
		protected NotificationTime.NotificationTimeBuilder notificationTime;
		protected OneWayProvisions.OneWayProvisionsBuilder oneWayProvisions;
		protected Regime.RegimeBuilder regime;
		protected SensitivityMethodology.SensitivityMethodologyBuilder sensitivityToCommodity;
		protected SensitivityMethodology.SensitivityMethodologyBuilder sensitivityToEquity;
		protected TerminationCurrencyAmendment.TerminationCurrencyAmendmentBuilder terminationCurrency;
	
		public CsaInitialMargin2016Builder() {
		}
				
		@Override
		public RosettaMetaData<? extends CsaInitialMargin2016> metaData() {
			return metaData;
		} 
	
		
		public CalculationDateLocation.CalculationDateLocationBuilder getCalculationDateLocation() {
			return calculationDateLocation;
		}
		
		public CalculationDateLocation.CalculationDateLocationBuilder getOrCreateCalculationDateLocation() {
			if (calculationDateLocation!=null) {
				return calculationDateLocation;
			}
			else return calculationDateLocation = new CalculationDateLocation.CalculationDateLocationBuilder();
		}
		
		
		public CreditSupportObligationsInitialMargin.CreditSupportObligationsInitialMarginBuilder getCreditSupportObligations() {
			return creditSupportObligations;
		}
		
		public CreditSupportObligationsInitialMargin.CreditSupportObligationsInitialMarginBuilder getOrCreateCreditSupportObligations() {
			if (creditSupportObligations!=null) {
				return creditSupportObligations;
			}
			else return creditSupportObligations = new CreditSupportObligationsInitialMargin.CreditSupportObligationsInitialMarginBuilder();
		}
		
		
		public FxHaircutCurrency.FxHaircutCurrencyBuilder getFxHaircutCurrency() {
			return fxHaircutCurrency;
		}
		
		public FxHaircutCurrency.FxHaircutCurrencyBuilder getOrCreateFxHaircutCurrency() {
			if (fxHaircutCurrency!=null) {
				return fxHaircutCurrency;
			}
			else return fxHaircutCurrency = new FxHaircutCurrency.FxHaircutCurrencyBuilder();
		}
		
		
		public Boolean getIdentifiedCrossCurrencySwap() {
			return identifiedCrossCurrencySwap;
		}
		
		
		public Method.MethodBuilder getMethod() {
			return method;
		}
		
		public Method.MethodBuilder getOrCreateMethod() {
			if (method!=null) {
				return method;
			}
			else return method = new Method.MethodBuilder();
		}
		
		
		public NotificationTime.NotificationTimeBuilder getNotificationTime() {
			return notificationTime;
		}
		
		public NotificationTime.NotificationTimeBuilder getOrCreateNotificationTime() {
			if (notificationTime!=null) {
				return notificationTime;
			}
			else return notificationTime = new NotificationTime.NotificationTimeBuilder();
		}
		
		
		public OneWayProvisions.OneWayProvisionsBuilder getOneWayProvisions() {
			return oneWayProvisions;
		}
		
		public OneWayProvisions.OneWayProvisionsBuilder getOrCreateOneWayProvisions() {
			if (oneWayProvisions!=null) {
				return oneWayProvisions;
			}
			else return oneWayProvisions = new OneWayProvisions.OneWayProvisionsBuilder();
		}
		
		
		public Regime.RegimeBuilder getRegime() {
			return regime;
		}
		
		public Regime.RegimeBuilder getOrCreateRegime() {
			if (regime!=null) {
				return regime;
			}
			else return regime = new Regime.RegimeBuilder();
		}
		
		
		public SensitivityMethodology.SensitivityMethodologyBuilder getSensitivityToCommodity() {
			return sensitivityToCommodity;
		}
		
		public SensitivityMethodology.SensitivityMethodologyBuilder getOrCreateSensitivityToCommodity() {
			if (sensitivityToCommodity!=null) {
				return sensitivityToCommodity;
			}
			else return sensitivityToCommodity = new SensitivityMethodology.SensitivityMethodologyBuilder();
		}
		
		
		public SensitivityMethodology.SensitivityMethodologyBuilder getSensitivityToEquity() {
			return sensitivityToEquity;
		}
		
		public SensitivityMethodology.SensitivityMethodologyBuilder getOrCreateSensitivityToEquity() {
			if (sensitivityToEquity!=null) {
				return sensitivityToEquity;
			}
			else return sensitivityToEquity = new SensitivityMethodology.SensitivityMethodologyBuilder();
		}
		
		
		public TerminationCurrencyAmendment.TerminationCurrencyAmendmentBuilder getTerminationCurrency() {
			return terminationCurrency;
		}
		
		public TerminationCurrencyAmendment.TerminationCurrencyAmendmentBuilder getOrCreateTerminationCurrency() {
			if (terminationCurrency!=null) {
				return terminationCurrency;
			}
			else return terminationCurrency = new TerminationCurrencyAmendment.TerminationCurrencyAmendmentBuilder();
		}
		
	
		public CsaInitialMargin2016Builder setCalculationDateLocation(CalculationDateLocation calculationDateLocation) {
			this.calculationDateLocation = calculationDateLocation.toBuilder();
			return this;
		}
		public CsaInitialMargin2016Builder setCalculationDateLocationBuilder(CalculationDateLocation.CalculationDateLocationBuilder calculationDateLocation) {
			this.calculationDateLocation = calculationDateLocation;
			return this;
		}
		public CsaInitialMargin2016Builder setCreditSupportObligations(CreditSupportObligationsInitialMargin creditSupportObligations) {
			this.creditSupportObligations = creditSupportObligations.toBuilder();
			return this;
		}
		public CsaInitialMargin2016Builder setCreditSupportObligationsBuilder(CreditSupportObligationsInitialMargin.CreditSupportObligationsInitialMarginBuilder creditSupportObligations) {
			this.creditSupportObligations = creditSupportObligations;
			return this;
		}
		public CsaInitialMargin2016Builder setFxHaircutCurrency(FxHaircutCurrency fxHaircutCurrency) {
			this.fxHaircutCurrency = fxHaircutCurrency.toBuilder();
			return this;
		}
		public CsaInitialMargin2016Builder setFxHaircutCurrencyBuilder(FxHaircutCurrency.FxHaircutCurrencyBuilder fxHaircutCurrency) {
			this.fxHaircutCurrency = fxHaircutCurrency;
			return this;
		}
		public CsaInitialMargin2016Builder setIdentifiedCrossCurrencySwap(Boolean identifiedCrossCurrencySwap) {
			this.identifiedCrossCurrencySwap = identifiedCrossCurrencySwap;
			return this;
		}
		public CsaInitialMargin2016Builder setMethod(Method method) {
			this.method = method.toBuilder();
			return this;
		}
		public CsaInitialMargin2016Builder setMethodBuilder(Method.MethodBuilder method) {
			this.method = method;
			return this;
		}
		public CsaInitialMargin2016Builder setNotificationTime(NotificationTime notificationTime) {
			this.notificationTime = notificationTime.toBuilder();
			return this;
		}
		public CsaInitialMargin2016Builder setNotificationTimeBuilder(NotificationTime.NotificationTimeBuilder notificationTime) {
			this.notificationTime = notificationTime;
			return this;
		}
		public CsaInitialMargin2016Builder setOneWayProvisions(OneWayProvisions oneWayProvisions) {
			this.oneWayProvisions = oneWayProvisions.toBuilder();
			return this;
		}
		public CsaInitialMargin2016Builder setOneWayProvisionsBuilder(OneWayProvisions.OneWayProvisionsBuilder oneWayProvisions) {
			this.oneWayProvisions = oneWayProvisions;
			return this;
		}
		public CsaInitialMargin2016Builder setRegime(Regime regime) {
			this.regime = regime.toBuilder();
			return this;
		}
		public CsaInitialMargin2016Builder setRegimeBuilder(Regime.RegimeBuilder regime) {
			this.regime = regime;
			return this;
		}
		public CsaInitialMargin2016Builder setSensitivityToCommodity(SensitivityMethodology sensitivityToCommodity) {
			this.sensitivityToCommodity = sensitivityToCommodity.toBuilder();
			return this;
		}
		public CsaInitialMargin2016Builder setSensitivityToCommodityBuilder(SensitivityMethodology.SensitivityMethodologyBuilder sensitivityToCommodity) {
			this.sensitivityToCommodity = sensitivityToCommodity;
			return this;
		}
		public CsaInitialMargin2016Builder setSensitivityToEquity(SensitivityMethodology sensitivityToEquity) {
			this.sensitivityToEquity = sensitivityToEquity.toBuilder();
			return this;
		}
		public CsaInitialMargin2016Builder setSensitivityToEquityBuilder(SensitivityMethodology.SensitivityMethodologyBuilder sensitivityToEquity) {
			this.sensitivityToEquity = sensitivityToEquity;
			return this;
		}
		public CsaInitialMargin2016Builder setTerminationCurrency(TerminationCurrencyAmendment terminationCurrency) {
			this.terminationCurrency = terminationCurrency.toBuilder();
			return this;
		}
		public CsaInitialMargin2016Builder setTerminationCurrencyBuilder(TerminationCurrencyAmendment.TerminationCurrencyAmendmentBuilder terminationCurrency) {
			this.terminationCurrency = terminationCurrency;
			return this;
		}
		@Override public CsaInitialMargin2016Builder setAdditionalObligations(String additionalObligations) {
			this.additionalObligations = additionalObligations;
			return this;
		}
		@Override public CsaInitialMargin2016Builder setAdditionalRepresentation(AdditionalRepresentation additionalRepresentation) {
			this.additionalRepresentation = additionalRepresentation.toBuilder();
			return this;
		}
		@Override public CsaInitialMargin2016Builder setAdditionalRepresentationBuilder(AdditionalRepresentation.AdditionalRepresentationBuilder additionalRepresentation) {
			this.additionalRepresentation = additionalRepresentation;
			return this;
		}
		@Override public CsaInitialMargin2016Builder setAddressesForTransfer(ContactElection addressesForTransfer) {
			this.addressesForTransfer = addressesForTransfer.toBuilder();
			return this;
		}
		@Override public CsaInitialMargin2016Builder setAddressesForTransferBuilder(ContactElection.ContactElectionBuilder addressesForTransfer) {
			this.addressesForTransfer = addressesForTransfer;
			return this;
		}
		@Override public CsaInitialMargin2016Builder setBaseCurrency(FieldWithMetaString baseCurrency) {
			this.baseCurrency = baseCurrency.toBuilder();
			return this;
		}
		@Override public CsaInitialMargin2016Builder setBaseCurrencyRef(String baseCurrency) {
			return setBaseCurrency(FieldWithMetaString.builder().setValue(baseCurrency).build());
		}
		@Override public CsaInitialMargin2016Builder setBespokeProvision(String bespokeProvision) {
			this.bespokeProvision = bespokeProvision;
			return this;
		}
		@Override public CsaInitialMargin2016Builder setConditionsPrecedent(ConditionsPrecedent conditionsPrecedent) {
			this.conditionsPrecedent = conditionsPrecedent.toBuilder();
			return this;
		}
		@Override public CsaInitialMargin2016Builder setConditionsPrecedentBuilder(ConditionsPrecedent.ConditionsPrecedentBuilder conditionsPrecedent) {
			this.conditionsPrecedent = conditionsPrecedent;
			return this;
		}
		@Override public CsaInitialMargin2016Builder setDemandsAndNotices(ContactElection demandsAndNotices) {
			this.demandsAndNotices = demandsAndNotices.toBuilder();
			return this;
		}
		@Override public CsaInitialMargin2016Builder setDemandsAndNoticesBuilder(ContactElection.ContactElectionBuilder demandsAndNotices) {
			this.demandsAndNotices = demandsAndNotices;
			return this;
		}
		@Override public CsaInitialMargin2016Builder setDisputeResolution(DisputeResolution disputeResolution) {
			this.disputeResolution = disputeResolution.toBuilder();
			return this;
		}
		@Override public CsaInitialMargin2016Builder setDisputeResolutionBuilder(DisputeResolution.DisputeResolutionBuilder disputeResolution) {
			this.disputeResolution = disputeResolution;
			return this;
		}
		@Override public CsaInitialMargin2016Builder setSubstitution(Substitution substitution) {
			this.substitution = substitution.toBuilder();
			return this;
		}
		@Override public CsaInitialMargin2016Builder setSubstitutionBuilder(Substitution.SubstitutionBuilder substitution) {
			this.substitution = substitution;
			return this;
		}
		
		public abstract CsaInitialMargin2016 build();
	
		@Override
		public CsaInitialMargin2016Builder prune() {
			super.prune();
			if (calculationDateLocation!=null && !calculationDateLocation.prune().hasData()) calculationDateLocation = null;
			if (creditSupportObligations!=null && !creditSupportObligations.prune().hasData()) creditSupportObligations = null;
			if (fxHaircutCurrency!=null && !fxHaircutCurrency.prune().hasData()) fxHaircutCurrency = null;
			if (method!=null && !method.prune().hasData()) method = null;
			if (notificationTime!=null && !notificationTime.prune().hasData()) notificationTime = null;
			if (oneWayProvisions!=null && !oneWayProvisions.prune().hasData()) oneWayProvisions = null;
			if (regime!=null && !regime.prune().hasData()) regime = null;
			if (sensitivityToCommodity!=null && !sensitivityToCommodity.prune().hasData()) sensitivityToCommodity = null;
			if (sensitivityToEquity!=null && !sensitivityToEquity.prune().hasData()) sensitivityToEquity = null;
			if (terminationCurrency!=null && !terminationCurrency.prune().hasData()) terminationCurrency = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (super.hasData()) return true;
			if (getCalculationDateLocation()!=null && getCalculationDateLocation().hasData()) return true;
			if (getCreditSupportObligations()!=null && getCreditSupportObligations().hasData()) return true;
			if (getFxHaircutCurrency()!=null && getFxHaircutCurrency().hasData()) return true;
			if (getIdentifiedCrossCurrencySwap()!=null) return true;
			if (getMethod()!=null && getMethod().hasData()) return true;
			if (getNotificationTime()!=null && getNotificationTime().hasData()) return true;
			if (getOneWayProvisions()!=null && getOneWayProvisions().hasData()) return true;
			if (getRegime()!=null && getRegime().hasData()) return true;
			if (getSensitivityToCommodity()!=null && getSensitivityToCommodity().hasData()) return true;
			if (getSensitivityToEquity()!=null && getSensitivityToEquity().hasData()) return true;
			if (getTerminationCurrency()!=null && getTerminationCurrency().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
			super.process(path, processor);
		
			processor.processBasic(path.newSubPath("identifiedCrossCurrencySwap"), Boolean.class, identifiedCrossCurrencySwap, this);
			
			processRosetta(path.newSubPath("calculationDateLocation"), processor, CalculationDateLocation.class, calculationDateLocation);
			processRosetta(path.newSubPath("creditSupportObligations"), processor, CreditSupportObligationsInitialMargin.class, creditSupportObligations);
			processRosetta(path.newSubPath("fxHaircutCurrency"), processor, FxHaircutCurrency.class, fxHaircutCurrency);
			processRosetta(path.newSubPath("method"), processor, Method.class, method);
			processRosetta(path.newSubPath("notificationTime"), processor, NotificationTime.class, notificationTime);
			processRosetta(path.newSubPath("oneWayProvisions"), processor, OneWayProvisions.class, oneWayProvisions);
			processRosetta(path.newSubPath("regime"), processor, Regime.class, regime);
			processRosetta(path.newSubPath("sensitivityToCommodity"), processor, SensitivityMethodology.class, sensitivityToCommodity);
			processRosetta(path.newSubPath("sensitivityToEquity"), processor, SensitivityMethodology.class, sensitivityToEquity);
			processRosetta(path.newSubPath("terminationCurrency"), processor, TerminationCurrencyAmendment.class, terminationCurrency);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
			if (!super.equals(o)) return false;
		
			CsaInitialMargin2016Builder _that = (CsaInitialMargin2016Builder) o;
		
			if (calculationDateLocation != null ? !calculationDateLocation.equals(_that.calculationDateLocation) : _that.calculationDateLocation != null) return false;
			if (creditSupportObligations != null ? !creditSupportObligations.equals(_that.creditSupportObligations) : _that.creditSupportObligations != null) return false;
			if (fxHaircutCurrency != null ? !fxHaircutCurrency.equals(_that.fxHaircutCurrency) : _that.fxHaircutCurrency != null) return false;
			if (identifiedCrossCurrencySwap != null ? !identifiedCrossCurrencySwap.equals(_that.identifiedCrossCurrencySwap) : _that.identifiedCrossCurrencySwap != null) return false;
			if (method != null ? !method.equals(_that.method) : _that.method != null) return false;
			if (notificationTime != null ? !notificationTime.equals(_that.notificationTime) : _that.notificationTime != null) return false;
			if (oneWayProvisions != null ? !oneWayProvisions.equals(_that.oneWayProvisions) : _that.oneWayProvisions != null) return false;
			if (regime != null ? !regime.equals(_that.regime) : _that.regime != null) return false;
			if (sensitivityToCommodity != null ? !sensitivityToCommodity.equals(_that.sensitivityToCommodity) : _that.sensitivityToCommodity != null) return false;
			if (sensitivityToEquity != null ? !sensitivityToEquity.equals(_that.sensitivityToEquity) : _that.sensitivityToEquity != null) return false;
			if (terminationCurrency != null ? !terminationCurrency.equals(_that.terminationCurrency) : _that.terminationCurrency != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = super.hashCode();
			_result = 31 * _result + (calculationDateLocation != null ? calculationDateLocation.hashCode() : 0);
			_result = 31 * _result + (creditSupportObligations != null ? creditSupportObligations.hashCode() : 0);
			_result = 31 * _result + (fxHaircutCurrency != null ? fxHaircutCurrency.hashCode() : 0);
			_result = 31 * _result + (identifiedCrossCurrencySwap != null ? identifiedCrossCurrencySwap.hashCode() : 0);
			_result = 31 * _result + (method != null ? method.hashCode() : 0);
			_result = 31 * _result + (notificationTime != null ? notificationTime.hashCode() : 0);
			_result = 31 * _result + (oneWayProvisions != null ? oneWayProvisions.hashCode() : 0);
			_result = 31 * _result + (regime != null ? regime.hashCode() : 0);
			_result = 31 * _result + (sensitivityToCommodity != null ? sensitivityToCommodity.hashCode() : 0);
			_result = 31 * _result + (sensitivityToEquity != null ? sensitivityToEquity.hashCode() : 0);
			_result = 31 * _result + (terminationCurrency != null ? terminationCurrency.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "CsaInitialMargin2016Builder {" +
				"calculationDateLocation=" + this.calculationDateLocation + ", " +
				"creditSupportObligations=" + this.creditSupportObligations + ", " +
				"fxHaircutCurrency=" + this.fxHaircutCurrency + ", " +
				"identifiedCrossCurrencySwap=" + this.identifiedCrossCurrencySwap + ", " +
				"method=" + this.method + ", " +
				"notificationTime=" + this.notificationTime + ", " +
				"oneWayProvisions=" + this.oneWayProvisions + ", " +
				"regime=" + this.regime + ", " +
				"sensitivityToCommodity=" + this.sensitivityToCommodity + ", " +
				"sensitivityToEquity=" + this.sensitivityToEquity + ", " +
				"terminationCurrency=" + this.terminationCurrency +
			'}' + " " + super.toString();
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		super.process(path, processor);
		
		processor.processBasic(path.newSubPath("identifiedCrossCurrencySwap"), Boolean.class, identifiedCrossCurrencySwap, this);
		
		processRosetta(path.newSubPath("calculationDateLocation"), processor, CalculationDateLocation.class, calculationDateLocation);
		processRosetta(path.newSubPath("creditSupportObligations"), processor, CreditSupportObligationsInitialMargin.class, creditSupportObligations);
		processRosetta(path.newSubPath("fxHaircutCurrency"), processor, FxHaircutCurrency.class, fxHaircutCurrency);
		processRosetta(path.newSubPath("method"), processor, Method.class, method);
		processRosetta(path.newSubPath("notificationTime"), processor, NotificationTime.class, notificationTime);
		processRosetta(path.newSubPath("oneWayProvisions"), processor, OneWayProvisions.class, oneWayProvisions);
		processRosetta(path.newSubPath("regime"), processor, Regime.class, regime);
		processRosetta(path.newSubPath("sensitivityToCommodity"), processor, SensitivityMethodology.class, sensitivityToCommodity);
		processRosetta(path.newSubPath("sensitivityToEquity"), processor, SensitivityMethodology.class, sensitivityToEquity);
		processRosetta(path.newSubPath("terminationCurrency"), processor, TerminationCurrencyAmendment.class, terminationCurrency);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
		if (!super.equals(o)) return false;
	
		CsaInitialMargin2016 _that = (CsaInitialMargin2016) o;
	
		if (calculationDateLocation != null ? !calculationDateLocation.equals(_that.calculationDateLocation) : _that.calculationDateLocation != null) return false;
		if (creditSupportObligations != null ? !creditSupportObligations.equals(_that.creditSupportObligations) : _that.creditSupportObligations != null) return false;
		if (fxHaircutCurrency != null ? !fxHaircutCurrency.equals(_that.fxHaircutCurrency) : _that.fxHaircutCurrency != null) return false;
		if (identifiedCrossCurrencySwap != null ? !identifiedCrossCurrencySwap.equals(_that.identifiedCrossCurrencySwap) : _that.identifiedCrossCurrencySwap != null) return false;
		if (method != null ? !method.equals(_that.method) : _that.method != null) return false;
		if (notificationTime != null ? !notificationTime.equals(_that.notificationTime) : _that.notificationTime != null) return false;
		if (oneWayProvisions != null ? !oneWayProvisions.equals(_that.oneWayProvisions) : _that.oneWayProvisions != null) return false;
		if (regime != null ? !regime.equals(_that.regime) : _that.regime != null) return false;
		if (sensitivityToCommodity != null ? !sensitivityToCommodity.equals(_that.sensitivityToCommodity) : _that.sensitivityToCommodity != null) return false;
		if (sensitivityToEquity != null ? !sensitivityToEquity.equals(_that.sensitivityToEquity) : _that.sensitivityToEquity != null) return false;
		if (terminationCurrency != null ? !terminationCurrency.equals(_that.terminationCurrency) : _that.terminationCurrency != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = super.hashCode();
		_result = 31 * _result + (calculationDateLocation != null ? calculationDateLocation.hashCode() : 0);
		_result = 31 * _result + (creditSupportObligations != null ? creditSupportObligations.hashCode() : 0);
		_result = 31 * _result + (fxHaircutCurrency != null ? fxHaircutCurrency.hashCode() : 0);
		_result = 31 * _result + (identifiedCrossCurrencySwap != null ? identifiedCrossCurrencySwap.hashCode() : 0);
		_result = 31 * _result + (method != null ? method.hashCode() : 0);
		_result = 31 * _result + (notificationTime != null ? notificationTime.hashCode() : 0);
		_result = 31 * _result + (oneWayProvisions != null ? oneWayProvisions.hashCode() : 0);
		_result = 31 * _result + (regime != null ? regime.hashCode() : 0);
		_result = 31 * _result + (sensitivityToCommodity != null ? sensitivityToCommodity.hashCode() : 0);
		_result = 31 * _result + (sensitivityToEquity != null ? sensitivityToEquity.hashCode() : 0);
		_result = 31 * _result + (terminationCurrency != null ? terminationCurrency.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "CsaInitialMargin2016 {" +
			"calculationDateLocation=" + this.calculationDateLocation + ", " +
			"creditSupportObligations=" + this.creditSupportObligations + ", " +
			"fxHaircutCurrency=" + this.fxHaircutCurrency + ", " +
			"identifiedCrossCurrencySwap=" + this.identifiedCrossCurrencySwap + ", " +
			"method=" + this.method + ", " +
			"notificationTime=" + this.notificationTime + ", " +
			"oneWayProvisions=" + this.oneWayProvisions + ", " +
			"regime=" + this.regime + ", " +
			"sensitivityToCommodity=" + this.sensitivityToCommodity + ", " +
			"sensitivityToEquity=" + this.sensitivityToEquity + ", " +
			"terminationCurrency=" + this.terminationCurrency +
		'}' + " " + super.toString();
	}
}
