package org.isda.cdm;

import java.lang.Boolean;
import java.lang.String;
import org.isda.cdm.AdditionalRightsEvent;
import org.isda.cdm.CollateralManagementAgreement;
import org.isda.cdm.CollateralManagementArrangement;
import org.isda.cdm.CollateralManager;
import org.isda.cdm.DistributionAndInterestPayment;
import org.isda.cdm.ObligorPostingObligations;
import org.isda.cdm.OtherEligibleAndPostedSupport;
import org.isda.cdm.RelatedAgreement;
import org.isda.cdm.RightsEvent;
import org.isda.cdm.metafields.FieldWithMetaString;


import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.CsaInitialMargin2016JapaneseLawMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class to specify the provisions that are specific to the Japanese Law version of the ISDA 2016 Credit Support Annex for Initial Margin.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class CsaInitialMargin2016JapaneseLaw extends CsaInitialMargin2016 {
	private final Boolean automaticSetOff;
	private final String cashSettlementDay;
	private final CollateralManagementAgreement collateralManagementAgreeement;
	private final CollateralManagementArrangement collateralManagementArrangement;
	private final CollateralManager collateralManager;
	private final DistributionAndInterestPayment distributionAndInterestPayment;
	private final RelatedAgreement japaneseLawCsa;
	private final AdditionalRightsEvent obligorAdditionalRightsEvent;
	private final ObligorPostingObligations obligorPostingObligations;
	private final RightsEvent obligorRightsEvent;
	private final RelatedAgreement otherCsa;
	private final OtherEligibleAndPostedSupport otherEligibleAndPostedSupport;
	private final String otherEligibleSupport;
	private final String securitiesSettlementDay;
	private final Boolean trustSchemeAddendum;
	private static CsaInitialMargin2016JapaneseLawMeta metaData = new CsaInitialMargin2016JapaneseLawMeta();
	
	CsaInitialMargin2016JapaneseLaw(CsaInitialMargin2016JapaneseLawBuilder builder) {
		super(builder);
		this.automaticSetOff = builder.getAutomaticSetOff();
		this.cashSettlementDay = builder.getCashSettlementDay();
		this.collateralManagementAgreeement = ofNullable(builder.getCollateralManagementAgreeement()).map(f->f.build()).orElse(null);
		this.collateralManagementArrangement = ofNullable(builder.getCollateralManagementArrangement()).map(f->f.build()).orElse(null);
		this.collateralManager = ofNullable(builder.getCollateralManager()).map(f->f.build()).orElse(null);
		this.distributionAndInterestPayment = ofNullable(builder.getDistributionAndInterestPayment()).map(f->f.build()).orElse(null);
		this.japaneseLawCsa = ofNullable(builder.getJapaneseLawCsa()).map(f->f.build()).orElse(null);
		this.obligorAdditionalRightsEvent = ofNullable(builder.getObligorAdditionalRightsEvent()).map(f->f.build()).orElse(null);
		this.obligorPostingObligations = ofNullable(builder.getObligorPostingObligations()).map(f->f.build()).orElse(null);
		this.obligorRightsEvent = ofNullable(builder.getObligorRightsEvent()).map(f->f.build()).orElse(null);
		this.otherCsa = ofNullable(builder.getOtherCsa()).map(f->f.build()).orElse(null);
		this.otherEligibleAndPostedSupport = ofNullable(builder.getOtherEligibleAndPostedSupport()).map(f->f.build()).orElse(null);
		this.otherEligibleSupport = builder.getOtherEligibleSupport();
		this.securitiesSettlementDay = builder.getSecuritiesSettlementDay();
		this.trustSchemeAddendum = builder.getTrustSchemeAddendum();
	}
	
	/**
	 * The Automatic Set-Off provision applies when the value is set to True. ISDA 2016 Japanese Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (l): Modification to Obligor’s Rights and Remedies.
	 */
	@RosettaSynonym(value="", source="ISDA_Create_1_0")
	public final Boolean getAutomaticSetOff() {
		return automaticSetOff;
	}
	
	/**
	 * Cash Settlement Day has the meaning specified in ISDA 2016 Japanese Law Credit Support Annex for Initial Margin, Paragraph 4(b)(i), unless otherwise specified as part of this attribute. ISDA 2016 Japanese Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (e)(iv): Cash Settlement Day.
	 */
	@RosettaSynonym(value="specify", source="ISDA_Create_1_0", path="cash_settlement_day")
	public final String getCashSettlementDay() {
		return cashSettlementDay;
	}
	
	/**
	 * ISDA 2016 Japanese Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (b)(i): Collateral Management Agreement.
	 */
	@RosettaSynonym(value="collateral_management_agreement", source="ISDA_Create_1_0")
	public final CollateralManagementAgreement getCollateralManagementAgreeement() {
		return collateralManagementAgreeement;
	}
	
	/**
	 * ISDA 2016 Japanese Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (m): Collateral Management Arrangement.
	 */
	public final CollateralManagementArrangement getCollateralManagementArrangement() {
		return collateralManagementArrangement;
	}
	
	/**
	 * ISDA 2016 Japanese Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (b)(ii): Collateral Manager.
	 */
	@RosettaSynonym(value="collateral_manager", source="ISDA_Create_1_0")
	public final CollateralManager getCollateralManager() {
		return collateralManager;
	}
	
	/**
	 * The Distributions and Interest Payment as part of the Japanese Law ISDA CSA. ISDA 2016 Credit Support Annex for Initial Margin, paragraph 13, General Principles, (n): Distributions and Interest Payment (IM).
	 */
	public final DistributionAndInterestPayment getDistributionAndInterestPayment() {
		return distributionAndInterestPayment;
	}
	
	/**
	 * The qualification of whether Japanese Law CSA (VM) are specified by the parties to the agreement. ISDA 2016 Credit Support Annex for Initial Margin, paragraph 13, General Principles, (s)(ii): Japanese Law CSA (VM).
	 */
	@RosettaSynonym(value="japanese_law_csa_vm", source="ISDA_Create_1_0")
	public final RelatedAgreement getJapaneseLawCsa() {
		return japaneseLawCsa;
	}
	
	/**
	 * The Obligor Additional Rights Event election. ISDA 2016 Japanese Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (k): Obligor Additional Rights Event.
	 */
	@RosettaSynonym(value="obligor_additional_rights_event", source="ISDA_Create_1_0")
	public final AdditionalRightsEvent getObligorAdditionalRightsEvent() {
		return obligorAdditionalRightsEvent;
	}
	
	/**
	 * The obligor&#39;s posting obligations. This term of obligor is the equivalent of the chargor in the English Law CSA and the pledgor in the New York Law CSA, i.e. it&#39;s the party that will post/pledge collateral to the other party. ISDA 2016 Japanese Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (ii).
	 */
	@RosettaSynonym(value="obligor_posting_obligations", source="ISDA_Create_1_0")
	public final ObligorPostingObligations getObligorPostingObligations() {
		return obligorPostingObligations;
	}
	
	/**
	 * The Obligor Rights Event election. ISDA 2016 Japanese Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (k): Obligor Rights Event.
	 */
	@RosettaSynonym(value="obligor_rights_event", source="ISDA_Create_1_0")
	public final RightsEvent getObligorRightsEvent() {
		return obligorRightsEvent;
	}
	
	/**
	 * The qualification of whether Other CSA are specified by the parties to the agreement. ISDA 2016 Credit Support Annex for Initial Margin, paragraph 13, General Principles, (s)(i): Other CSA.
	 */
	@RosettaSynonym(value="other_csa", source="ISDA_Create_1_0")
	public final RelatedAgreement getOtherCsa() {
		return otherCsa;
	}
	
	/**
	 * The Other Eligible Support elections associated with margin agreements. ISDA 2016 Japanese Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (p): Other Eligible Support (IM) and Other Posted Support (IM).
	 */
	public final OtherEligibleAndPostedSupport getOtherEligibleAndPostedSupport() {
		return otherEligibleAndPostedSupport;
	}
	
	/**
	 * The Other Eligible Support election. If not specified, this election is deemed as not applicable. ISDA 2016 Japanese Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (d)(iv).
	 */
	public final String getOtherEligibleSupport() {
		return otherEligibleSupport;
	}
	
	/**
	 * Securities Settlement Day has the meaning specified in ISDA 2016 Japanese Law Credit Support Annex for Initial Margin, Paragraph 12, unless otherwise specified as part of this attribute. ISDA 2016 Japanese Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (e)(v): Securities Settlement Day.
	 */
	@RosettaSynonym(value="specify", source="ISDA_Create_1_0", path="securities_settlement_day")
	public final String getSecuritiesSettlementDay() {
		return securitiesSettlementDay;
	}
	
	/**
	 * The qualification of whether Trust Scheme Addendum is applicable (True) or not applicable (False).
	 */
	@RosettaSynonym(value="", source="ISDA_Create_1_0")
	public final Boolean getTrustSchemeAddendum() {
		return trustSchemeAddendum;
	}
	
	
	@Override
	public RosettaMetaData<? extends CsaInitialMargin2016JapaneseLaw> metaData() {
		return metaData;
	} 
	
	public CsaInitialMargin2016JapaneseLawBuilder toBuilder() {
		CsaInitialMargin2016JapaneseLawBuilder builder = new CsaInitialMargin2016JapaneseLawBuilder();
		ofNullable(getAutomaticSetOff()).ifPresent(builder::setAutomaticSetOff);
		ofNullable(getCashSettlementDay()).ifPresent(builder::setCashSettlementDay);
		ofNullable(getCollateralManagementAgreeement()).ifPresent(builder::setCollateralManagementAgreeement);
		ofNullable(getCollateralManagementArrangement()).ifPresent(builder::setCollateralManagementArrangement);
		ofNullable(getCollateralManager()).ifPresent(builder::setCollateralManager);
		ofNullable(getDistributionAndInterestPayment()).ifPresent(builder::setDistributionAndInterestPayment);
		ofNullable(getJapaneseLawCsa()).ifPresent(builder::setJapaneseLawCsa);
		ofNullable(getObligorAdditionalRightsEvent()).ifPresent(builder::setObligorAdditionalRightsEvent);
		ofNullable(getObligorPostingObligations()).ifPresent(builder::setObligorPostingObligations);
		ofNullable(getObligorRightsEvent()).ifPresent(builder::setObligorRightsEvent);
		ofNullable(getOtherCsa()).ifPresent(builder::setOtherCsa);
		ofNullable(getOtherEligibleAndPostedSupport()).ifPresent(builder::setOtherEligibleAndPostedSupport);
		ofNullable(getOtherEligibleSupport()).ifPresent(builder::setOtherEligibleSupport);
		ofNullable(getSecuritiesSettlementDay()).ifPresent(builder::setSecuritiesSettlementDay);
		ofNullable(getTrustSchemeAddendum()).ifPresent(builder::setTrustSchemeAddendum);
		ofNullable(getCalculationDateLocation()).ifPresent(builder::setCalculationDateLocation);
		ofNullable(getCreditSupportObligations()).ifPresent(builder::setCreditSupportObligations);
		ofNullable(getFxHaircutCurrency()).ifPresent(builder::setFxHaircutCurrency);
		ofNullable(getIdentifiedCrossCurrencySwap()).ifPresent(builder::setIdentifiedCrossCurrencySwap);
		ofNullable(getMethod()).ifPresent(builder::setMethod);
		ofNullable(getNotificationTime()).ifPresent(builder::setNotificationTime);
		ofNullable(getOneWayProvisions()).ifPresent(builder::setOneWayProvisions);
		ofNullable(getRegime()).ifPresent(builder::setRegime);
		ofNullable(getSensitivityToCommodity()).ifPresent(builder::setSensitivityToCommodity);
		ofNullable(getSensitivityToEquity()).ifPresent(builder::setSensitivityToEquity);
		ofNullable(getTerminationCurrency()).ifPresent(builder::setTerminationCurrency);
		ofNullable(getAdditionalObligations()).ifPresent(builder::setAdditionalObligations);
		ofNullable(getAdditionalRepresentation()).ifPresent(builder::setAdditionalRepresentation);
		ofNullable(getAddressesForTransfer()).ifPresent(builder::setAddressesForTransfer);
		ofNullable(getBaseCurrency()).ifPresent(builder::setBaseCurrency);
		ofNullable(getBespokeProvision()).ifPresent(builder::setBespokeProvision);
		ofNullable(getConditionsPrecedent()).ifPresent(builder::setConditionsPrecedent);
		ofNullable(getDemandsAndNotices()).ifPresent(builder::setDemandsAndNotices);
		ofNullable(getDisputeResolution()).ifPresent(builder::setDisputeResolution);
		ofNullable(getSubstitution()).ifPresent(builder::setSubstitution);
		return builder;
	}

	public static CsaInitialMargin2016JapaneseLawBuilder builder() {
		return new CsaInitialMargin2016JapaneseLawBuilder();
	}

	public static  class CsaInitialMargin2016JapaneseLawBuilder extends CsaInitialMargin2016Builder {
	
		protected Boolean automaticSetOff;
		protected String cashSettlementDay;
		protected CollateralManagementAgreement.CollateralManagementAgreementBuilder collateralManagementAgreeement;
		protected CollateralManagementArrangement.CollateralManagementArrangementBuilder collateralManagementArrangement;
		protected CollateralManager.CollateralManagerBuilder collateralManager;
		protected DistributionAndInterestPayment.DistributionAndInterestPaymentBuilder distributionAndInterestPayment;
		protected RelatedAgreement.RelatedAgreementBuilder japaneseLawCsa;
		protected AdditionalRightsEvent.AdditionalRightsEventBuilder obligorAdditionalRightsEvent;
		protected ObligorPostingObligations.ObligorPostingObligationsBuilder obligorPostingObligations;
		protected RightsEvent.RightsEventBuilder obligorRightsEvent;
		protected RelatedAgreement.RelatedAgreementBuilder otherCsa;
		protected OtherEligibleAndPostedSupport.OtherEligibleAndPostedSupportBuilder otherEligibleAndPostedSupport;
		protected String otherEligibleSupport;
		protected String securitiesSettlementDay;
		protected Boolean trustSchemeAddendum;
	
		public CsaInitialMargin2016JapaneseLawBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends CsaInitialMargin2016JapaneseLaw> metaData() {
			return metaData;
		} 
	
		
		public Boolean getAutomaticSetOff() {
			return automaticSetOff;
		}
		
		
		public String getCashSettlementDay() {
			return cashSettlementDay;
		}
		
		
		public CollateralManagementAgreement.CollateralManagementAgreementBuilder getCollateralManagementAgreeement() {
			return collateralManagementAgreeement;
		}
		
		public CollateralManagementAgreement.CollateralManagementAgreementBuilder getOrCreateCollateralManagementAgreeement() {
			if (collateralManagementAgreeement!=null) {
				return collateralManagementAgreeement;
			}
			else return collateralManagementAgreeement = new CollateralManagementAgreement.CollateralManagementAgreementBuilder();
		}
		
		
		public CollateralManagementArrangement.CollateralManagementArrangementBuilder getCollateralManagementArrangement() {
			return collateralManagementArrangement;
		}
		
		public CollateralManagementArrangement.CollateralManagementArrangementBuilder getOrCreateCollateralManagementArrangement() {
			if (collateralManagementArrangement!=null) {
				return collateralManagementArrangement;
			}
			else return collateralManagementArrangement = new CollateralManagementArrangement.CollateralManagementArrangementBuilder();
		}
		
		
		public CollateralManager.CollateralManagerBuilder getCollateralManager() {
			return collateralManager;
		}
		
		public CollateralManager.CollateralManagerBuilder getOrCreateCollateralManager() {
			if (collateralManager!=null) {
				return collateralManager;
			}
			else return collateralManager = new CollateralManager.CollateralManagerBuilder();
		}
		
		
		public DistributionAndInterestPayment.DistributionAndInterestPaymentBuilder getDistributionAndInterestPayment() {
			return distributionAndInterestPayment;
		}
		
		public DistributionAndInterestPayment.DistributionAndInterestPaymentBuilder getOrCreateDistributionAndInterestPayment() {
			if (distributionAndInterestPayment!=null) {
				return distributionAndInterestPayment;
			}
			else return distributionAndInterestPayment = new DistributionAndInterestPayment.DistributionAndInterestPaymentBuilder();
		}
		
		
		public RelatedAgreement.RelatedAgreementBuilder getJapaneseLawCsa() {
			return japaneseLawCsa;
		}
		
		public RelatedAgreement.RelatedAgreementBuilder getOrCreateJapaneseLawCsa() {
			if (japaneseLawCsa!=null) {
				return japaneseLawCsa;
			}
			else return japaneseLawCsa = new RelatedAgreement.RelatedAgreementBuilder();
		}
		
		
		public AdditionalRightsEvent.AdditionalRightsEventBuilder getObligorAdditionalRightsEvent() {
			return obligorAdditionalRightsEvent;
		}
		
		public AdditionalRightsEvent.AdditionalRightsEventBuilder getOrCreateObligorAdditionalRightsEvent() {
			if (obligorAdditionalRightsEvent!=null) {
				return obligorAdditionalRightsEvent;
			}
			else return obligorAdditionalRightsEvent = new AdditionalRightsEvent.AdditionalRightsEventBuilder();
		}
		
		
		public ObligorPostingObligations.ObligorPostingObligationsBuilder getObligorPostingObligations() {
			return obligorPostingObligations;
		}
		
		public ObligorPostingObligations.ObligorPostingObligationsBuilder getOrCreateObligorPostingObligations() {
			if (obligorPostingObligations!=null) {
				return obligorPostingObligations;
			}
			else return obligorPostingObligations = new ObligorPostingObligations.ObligorPostingObligationsBuilder();
		}
		
		
		public RightsEvent.RightsEventBuilder getObligorRightsEvent() {
			return obligorRightsEvent;
		}
		
		public RightsEvent.RightsEventBuilder getOrCreateObligorRightsEvent() {
			if (obligorRightsEvent!=null) {
				return obligorRightsEvent;
			}
			else return obligorRightsEvent = new RightsEvent.RightsEventBuilder();
		}
		
		
		public RelatedAgreement.RelatedAgreementBuilder getOtherCsa() {
			return otherCsa;
		}
		
		public RelatedAgreement.RelatedAgreementBuilder getOrCreateOtherCsa() {
			if (otherCsa!=null) {
				return otherCsa;
			}
			else return otherCsa = new RelatedAgreement.RelatedAgreementBuilder();
		}
		
		
		public OtherEligibleAndPostedSupport.OtherEligibleAndPostedSupportBuilder getOtherEligibleAndPostedSupport() {
			return otherEligibleAndPostedSupport;
		}
		
		public OtherEligibleAndPostedSupport.OtherEligibleAndPostedSupportBuilder getOrCreateOtherEligibleAndPostedSupport() {
			if (otherEligibleAndPostedSupport!=null) {
				return otherEligibleAndPostedSupport;
			}
			else return otherEligibleAndPostedSupport = new OtherEligibleAndPostedSupport.OtherEligibleAndPostedSupportBuilder();
		}
		
		
		public String getOtherEligibleSupport() {
			return otherEligibleSupport;
		}
		
		
		public String getSecuritiesSettlementDay() {
			return securitiesSettlementDay;
		}
		
		
		public Boolean getTrustSchemeAddendum() {
			return trustSchemeAddendum;
		}
		
	
		public CsaInitialMargin2016JapaneseLawBuilder setAutomaticSetOff(Boolean automaticSetOff) {
			this.automaticSetOff = automaticSetOff;
			return this;
		}
		public CsaInitialMargin2016JapaneseLawBuilder setCashSettlementDay(String cashSettlementDay) {
			this.cashSettlementDay = cashSettlementDay;
			return this;
		}
		public CsaInitialMargin2016JapaneseLawBuilder setCollateralManagementAgreeement(CollateralManagementAgreement collateralManagementAgreeement) {
			this.collateralManagementAgreeement = collateralManagementAgreeement.toBuilder();
			return this;
		}
		public CsaInitialMargin2016JapaneseLawBuilder setCollateralManagementAgreeementBuilder(CollateralManagementAgreement.CollateralManagementAgreementBuilder collateralManagementAgreeement) {
			this.collateralManagementAgreeement = collateralManagementAgreeement;
			return this;
		}
		public CsaInitialMargin2016JapaneseLawBuilder setCollateralManagementArrangement(CollateralManagementArrangement collateralManagementArrangement) {
			this.collateralManagementArrangement = collateralManagementArrangement.toBuilder();
			return this;
		}
		public CsaInitialMargin2016JapaneseLawBuilder setCollateralManagementArrangementBuilder(CollateralManagementArrangement.CollateralManagementArrangementBuilder collateralManagementArrangement) {
			this.collateralManagementArrangement = collateralManagementArrangement;
			return this;
		}
		public CsaInitialMargin2016JapaneseLawBuilder setCollateralManager(CollateralManager collateralManager) {
			this.collateralManager = collateralManager.toBuilder();
			return this;
		}
		public CsaInitialMargin2016JapaneseLawBuilder setCollateralManagerBuilder(CollateralManager.CollateralManagerBuilder collateralManager) {
			this.collateralManager = collateralManager;
			return this;
		}
		public CsaInitialMargin2016JapaneseLawBuilder setDistributionAndInterestPayment(DistributionAndInterestPayment distributionAndInterestPayment) {
			this.distributionAndInterestPayment = distributionAndInterestPayment.toBuilder();
			return this;
		}
		public CsaInitialMargin2016JapaneseLawBuilder setDistributionAndInterestPaymentBuilder(DistributionAndInterestPayment.DistributionAndInterestPaymentBuilder distributionAndInterestPayment) {
			this.distributionAndInterestPayment = distributionAndInterestPayment;
			return this;
		}
		public CsaInitialMargin2016JapaneseLawBuilder setJapaneseLawCsa(RelatedAgreement japaneseLawCsa) {
			this.japaneseLawCsa = japaneseLawCsa.toBuilder();
			return this;
		}
		public CsaInitialMargin2016JapaneseLawBuilder setJapaneseLawCsaBuilder(RelatedAgreement.RelatedAgreementBuilder japaneseLawCsa) {
			this.japaneseLawCsa = japaneseLawCsa;
			return this;
		}
		public CsaInitialMargin2016JapaneseLawBuilder setObligorAdditionalRightsEvent(AdditionalRightsEvent obligorAdditionalRightsEvent) {
			this.obligorAdditionalRightsEvent = obligorAdditionalRightsEvent.toBuilder();
			return this;
		}
		public CsaInitialMargin2016JapaneseLawBuilder setObligorAdditionalRightsEventBuilder(AdditionalRightsEvent.AdditionalRightsEventBuilder obligorAdditionalRightsEvent) {
			this.obligorAdditionalRightsEvent = obligorAdditionalRightsEvent;
			return this;
		}
		public CsaInitialMargin2016JapaneseLawBuilder setObligorPostingObligations(ObligorPostingObligations obligorPostingObligations) {
			this.obligorPostingObligations = obligorPostingObligations.toBuilder();
			return this;
		}
		public CsaInitialMargin2016JapaneseLawBuilder setObligorPostingObligationsBuilder(ObligorPostingObligations.ObligorPostingObligationsBuilder obligorPostingObligations) {
			this.obligorPostingObligations = obligorPostingObligations;
			return this;
		}
		public CsaInitialMargin2016JapaneseLawBuilder setObligorRightsEvent(RightsEvent obligorRightsEvent) {
			this.obligorRightsEvent = obligorRightsEvent.toBuilder();
			return this;
		}
		public CsaInitialMargin2016JapaneseLawBuilder setObligorRightsEventBuilder(RightsEvent.RightsEventBuilder obligorRightsEvent) {
			this.obligorRightsEvent = obligorRightsEvent;
			return this;
		}
		public CsaInitialMargin2016JapaneseLawBuilder setOtherCsa(RelatedAgreement otherCsa) {
			this.otherCsa = otherCsa.toBuilder();
			return this;
		}
		public CsaInitialMargin2016JapaneseLawBuilder setOtherCsaBuilder(RelatedAgreement.RelatedAgreementBuilder otherCsa) {
			this.otherCsa = otherCsa;
			return this;
		}
		public CsaInitialMargin2016JapaneseLawBuilder setOtherEligibleAndPostedSupport(OtherEligibleAndPostedSupport otherEligibleAndPostedSupport) {
			this.otherEligibleAndPostedSupport = otherEligibleAndPostedSupport.toBuilder();
			return this;
		}
		public CsaInitialMargin2016JapaneseLawBuilder setOtherEligibleAndPostedSupportBuilder(OtherEligibleAndPostedSupport.OtherEligibleAndPostedSupportBuilder otherEligibleAndPostedSupport) {
			this.otherEligibleAndPostedSupport = otherEligibleAndPostedSupport;
			return this;
		}
		public CsaInitialMargin2016JapaneseLawBuilder setOtherEligibleSupport(String otherEligibleSupport) {
			this.otherEligibleSupport = otherEligibleSupport;
			return this;
		}
		public CsaInitialMargin2016JapaneseLawBuilder setSecuritiesSettlementDay(String securitiesSettlementDay) {
			this.securitiesSettlementDay = securitiesSettlementDay;
			return this;
		}
		public CsaInitialMargin2016JapaneseLawBuilder setTrustSchemeAddendum(Boolean trustSchemeAddendum) {
			this.trustSchemeAddendum = trustSchemeAddendum;
			return this;
		}
		@Override public CsaInitialMargin2016JapaneseLawBuilder setCalculationDateLocation(CalculationDateLocation calculationDateLocation) {
			this.calculationDateLocation = calculationDateLocation.toBuilder();
			return this;
		}
		@Override public CsaInitialMargin2016JapaneseLawBuilder setCalculationDateLocationBuilder(CalculationDateLocation.CalculationDateLocationBuilder calculationDateLocation) {
			this.calculationDateLocation = calculationDateLocation;
			return this;
		}
		@Override public CsaInitialMargin2016JapaneseLawBuilder setCreditSupportObligations(CreditSupportObligationsInitialMargin creditSupportObligations) {
			this.creditSupportObligations = creditSupportObligations.toBuilder();
			return this;
		}
		@Override public CsaInitialMargin2016JapaneseLawBuilder setCreditSupportObligationsBuilder(CreditSupportObligationsInitialMargin.CreditSupportObligationsInitialMarginBuilder creditSupportObligations) {
			this.creditSupportObligations = creditSupportObligations;
			return this;
		}
		@Override public CsaInitialMargin2016JapaneseLawBuilder setFxHaircutCurrency(FxHaircutCurrency fxHaircutCurrency) {
			this.fxHaircutCurrency = fxHaircutCurrency.toBuilder();
			return this;
		}
		@Override public CsaInitialMargin2016JapaneseLawBuilder setFxHaircutCurrencyBuilder(FxHaircutCurrency.FxHaircutCurrencyBuilder fxHaircutCurrency) {
			this.fxHaircutCurrency = fxHaircutCurrency;
			return this;
		}
		@Override public CsaInitialMargin2016JapaneseLawBuilder setIdentifiedCrossCurrencySwap(Boolean identifiedCrossCurrencySwap) {
			this.identifiedCrossCurrencySwap = identifiedCrossCurrencySwap;
			return this;
		}
		@Override public CsaInitialMargin2016JapaneseLawBuilder setMethod(Method method) {
			this.method = method.toBuilder();
			return this;
		}
		@Override public CsaInitialMargin2016JapaneseLawBuilder setMethodBuilder(Method.MethodBuilder method) {
			this.method = method;
			return this;
		}
		@Override public CsaInitialMargin2016JapaneseLawBuilder setNotificationTime(NotificationTime notificationTime) {
			this.notificationTime = notificationTime.toBuilder();
			return this;
		}
		@Override public CsaInitialMargin2016JapaneseLawBuilder setNotificationTimeBuilder(NotificationTime.NotificationTimeBuilder notificationTime) {
			this.notificationTime = notificationTime;
			return this;
		}
		@Override public CsaInitialMargin2016JapaneseLawBuilder setOneWayProvisions(OneWayProvisions oneWayProvisions) {
			this.oneWayProvisions = oneWayProvisions.toBuilder();
			return this;
		}
		@Override public CsaInitialMargin2016JapaneseLawBuilder setOneWayProvisionsBuilder(OneWayProvisions.OneWayProvisionsBuilder oneWayProvisions) {
			this.oneWayProvisions = oneWayProvisions;
			return this;
		}
		@Override public CsaInitialMargin2016JapaneseLawBuilder setRegime(Regime regime) {
			this.regime = regime.toBuilder();
			return this;
		}
		@Override public CsaInitialMargin2016JapaneseLawBuilder setRegimeBuilder(Regime.RegimeBuilder regime) {
			this.regime = regime;
			return this;
		}
		@Override public CsaInitialMargin2016JapaneseLawBuilder setSensitivityToCommodity(SensitivityMethodology sensitivityToCommodity) {
			this.sensitivityToCommodity = sensitivityToCommodity.toBuilder();
			return this;
		}
		@Override public CsaInitialMargin2016JapaneseLawBuilder setSensitivityToCommodityBuilder(SensitivityMethodology.SensitivityMethodologyBuilder sensitivityToCommodity) {
			this.sensitivityToCommodity = sensitivityToCommodity;
			return this;
		}
		@Override public CsaInitialMargin2016JapaneseLawBuilder setSensitivityToEquity(SensitivityMethodology sensitivityToEquity) {
			this.sensitivityToEquity = sensitivityToEquity.toBuilder();
			return this;
		}
		@Override public CsaInitialMargin2016JapaneseLawBuilder setSensitivityToEquityBuilder(SensitivityMethodology.SensitivityMethodologyBuilder sensitivityToEquity) {
			this.sensitivityToEquity = sensitivityToEquity;
			return this;
		}
		@Override public CsaInitialMargin2016JapaneseLawBuilder setTerminationCurrency(TerminationCurrencyAmendment terminationCurrency) {
			this.terminationCurrency = terminationCurrency.toBuilder();
			return this;
		}
		@Override public CsaInitialMargin2016JapaneseLawBuilder setTerminationCurrencyBuilder(TerminationCurrencyAmendment.TerminationCurrencyAmendmentBuilder terminationCurrency) {
			this.terminationCurrency = terminationCurrency;
			return this;
		}
		@Override public CsaInitialMargin2016JapaneseLawBuilder setAdditionalObligations(String additionalObligations) {
			this.additionalObligations = additionalObligations;
			return this;
		}
		@Override public CsaInitialMargin2016JapaneseLawBuilder setAdditionalRepresentation(AdditionalRepresentation additionalRepresentation) {
			this.additionalRepresentation = additionalRepresentation.toBuilder();
			return this;
		}
		@Override public CsaInitialMargin2016JapaneseLawBuilder setAdditionalRepresentationBuilder(AdditionalRepresentation.AdditionalRepresentationBuilder additionalRepresentation) {
			this.additionalRepresentation = additionalRepresentation;
			return this;
		}
		@Override public CsaInitialMargin2016JapaneseLawBuilder setAddressesForTransfer(ContactElection addressesForTransfer) {
			this.addressesForTransfer = addressesForTransfer.toBuilder();
			return this;
		}
		@Override public CsaInitialMargin2016JapaneseLawBuilder setAddressesForTransferBuilder(ContactElection.ContactElectionBuilder addressesForTransfer) {
			this.addressesForTransfer = addressesForTransfer;
			return this;
		}
		@Override public CsaInitialMargin2016JapaneseLawBuilder setBaseCurrency(FieldWithMetaString baseCurrency) {
			this.baseCurrency = baseCurrency.toBuilder();
			return this;
		}
		@Override public CsaInitialMargin2016JapaneseLawBuilder setBaseCurrencyRef(String baseCurrency) {
			return setBaseCurrency(FieldWithMetaString.builder().setValue(baseCurrency).build());
		}
		@Override public CsaInitialMargin2016JapaneseLawBuilder setBespokeProvision(String bespokeProvision) {
			this.bespokeProvision = bespokeProvision;
			return this;
		}
		@Override public CsaInitialMargin2016JapaneseLawBuilder setConditionsPrecedent(ConditionsPrecedent conditionsPrecedent) {
			this.conditionsPrecedent = conditionsPrecedent.toBuilder();
			return this;
		}
		@Override public CsaInitialMargin2016JapaneseLawBuilder setConditionsPrecedentBuilder(ConditionsPrecedent.ConditionsPrecedentBuilder conditionsPrecedent) {
			this.conditionsPrecedent = conditionsPrecedent;
			return this;
		}
		@Override public CsaInitialMargin2016JapaneseLawBuilder setDemandsAndNotices(ContactElection demandsAndNotices) {
			this.demandsAndNotices = demandsAndNotices.toBuilder();
			return this;
		}
		@Override public CsaInitialMargin2016JapaneseLawBuilder setDemandsAndNoticesBuilder(ContactElection.ContactElectionBuilder demandsAndNotices) {
			this.demandsAndNotices = demandsAndNotices;
			return this;
		}
		@Override public CsaInitialMargin2016JapaneseLawBuilder setDisputeResolution(DisputeResolution disputeResolution) {
			this.disputeResolution = disputeResolution.toBuilder();
			return this;
		}
		@Override public CsaInitialMargin2016JapaneseLawBuilder setDisputeResolutionBuilder(DisputeResolution.DisputeResolutionBuilder disputeResolution) {
			this.disputeResolution = disputeResolution;
			return this;
		}
		@Override public CsaInitialMargin2016JapaneseLawBuilder setSubstitution(Substitution substitution) {
			this.substitution = substitution.toBuilder();
			return this;
		}
		@Override public CsaInitialMargin2016JapaneseLawBuilder setSubstitutionBuilder(Substitution.SubstitutionBuilder substitution) {
			this.substitution = substitution;
			return this;
		}
		
		public CsaInitialMargin2016JapaneseLaw build() {
			return new CsaInitialMargin2016JapaneseLaw(this);
		}
	
		@Override
		public CsaInitialMargin2016JapaneseLawBuilder prune() {
			super.prune();
			if (collateralManagementAgreeement!=null && !collateralManagementAgreeement.prune().hasData()) collateralManagementAgreeement = null;
			if (collateralManagementArrangement!=null && !collateralManagementArrangement.prune().hasData()) collateralManagementArrangement = null;
			if (collateralManager!=null && !collateralManager.prune().hasData()) collateralManager = null;
			if (distributionAndInterestPayment!=null && !distributionAndInterestPayment.prune().hasData()) distributionAndInterestPayment = null;
			if (japaneseLawCsa!=null && !japaneseLawCsa.prune().hasData()) japaneseLawCsa = null;
			if (obligorAdditionalRightsEvent!=null && !obligorAdditionalRightsEvent.prune().hasData()) obligorAdditionalRightsEvent = null;
			if (obligorPostingObligations!=null && !obligorPostingObligations.prune().hasData()) obligorPostingObligations = null;
			if (obligorRightsEvent!=null && !obligorRightsEvent.prune().hasData()) obligorRightsEvent = null;
			if (otherCsa!=null && !otherCsa.prune().hasData()) otherCsa = null;
			if (otherEligibleAndPostedSupport!=null && !otherEligibleAndPostedSupport.prune().hasData()) otherEligibleAndPostedSupport = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (super.hasData()) return true;
			if (getAutomaticSetOff()!=null) return true;
			if (getCashSettlementDay()!=null) return true;
			if (getCollateralManagementAgreeement()!=null && getCollateralManagementAgreeement().hasData()) return true;
			if (getCollateralManagementArrangement()!=null && getCollateralManagementArrangement().hasData()) return true;
			if (getCollateralManager()!=null && getCollateralManager().hasData()) return true;
			if (getDistributionAndInterestPayment()!=null && getDistributionAndInterestPayment().hasData()) return true;
			if (getJapaneseLawCsa()!=null && getJapaneseLawCsa().hasData()) return true;
			if (getObligorAdditionalRightsEvent()!=null && getObligorAdditionalRightsEvent().hasData()) return true;
			if (getObligorPostingObligations()!=null && getObligorPostingObligations().hasData()) return true;
			if (getObligorRightsEvent()!=null && getObligorRightsEvent().hasData()) return true;
			if (getOtherCsa()!=null && getOtherCsa().hasData()) return true;
			if (getOtherEligibleAndPostedSupport()!=null && getOtherEligibleAndPostedSupport().hasData()) return true;
			if (getOtherEligibleSupport()!=null) return true;
			if (getSecuritiesSettlementDay()!=null) return true;
			if (getTrustSchemeAddendum()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
			super.process(path, processor);
		
			processor.processBasic(path.newSubPath("automaticSetOff"), Boolean.class, automaticSetOff, this);
			processor.processBasic(path.newSubPath("cashSettlementDay"), String.class, cashSettlementDay, this);
			processor.processBasic(path.newSubPath("otherEligibleSupport"), String.class, otherEligibleSupport, this);
			processor.processBasic(path.newSubPath("securitiesSettlementDay"), String.class, securitiesSettlementDay, this);
			processor.processBasic(path.newSubPath("trustSchemeAddendum"), Boolean.class, trustSchemeAddendum, this);
			
			processRosetta(path.newSubPath("collateralManagementAgreeement"), processor, CollateralManagementAgreement.class, collateralManagementAgreeement);
			processRosetta(path.newSubPath("collateralManagementArrangement"), processor, CollateralManagementArrangement.class, collateralManagementArrangement);
			processRosetta(path.newSubPath("collateralManager"), processor, CollateralManager.class, collateralManager);
			processRosetta(path.newSubPath("distributionAndInterestPayment"), processor, DistributionAndInterestPayment.class, distributionAndInterestPayment);
			processRosetta(path.newSubPath("japaneseLawCsa"), processor, RelatedAgreement.class, japaneseLawCsa);
			processRosetta(path.newSubPath("obligorAdditionalRightsEvent"), processor, AdditionalRightsEvent.class, obligorAdditionalRightsEvent);
			processRosetta(path.newSubPath("obligorPostingObligations"), processor, ObligorPostingObligations.class, obligorPostingObligations);
			processRosetta(path.newSubPath("obligorRightsEvent"), processor, RightsEvent.class, obligorRightsEvent);
			processRosetta(path.newSubPath("otherCsa"), processor, RelatedAgreement.class, otherCsa);
			processRosetta(path.newSubPath("otherEligibleAndPostedSupport"), processor, OtherEligibleAndPostedSupport.class, otherEligibleAndPostedSupport);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
			if (!super.equals(o)) return false;
		
			CsaInitialMargin2016JapaneseLawBuilder _that = (CsaInitialMargin2016JapaneseLawBuilder) o;
		
			if (automaticSetOff != null ? !automaticSetOff.equals(_that.automaticSetOff) : _that.automaticSetOff != null) return false;
			if (cashSettlementDay != null ? !cashSettlementDay.equals(_that.cashSettlementDay) : _that.cashSettlementDay != null) return false;
			if (collateralManagementAgreeement != null ? !collateralManagementAgreeement.equals(_that.collateralManagementAgreeement) : _that.collateralManagementAgreeement != null) return false;
			if (collateralManagementArrangement != null ? !collateralManagementArrangement.equals(_that.collateralManagementArrangement) : _that.collateralManagementArrangement != null) return false;
			if (collateralManager != null ? !collateralManager.equals(_that.collateralManager) : _that.collateralManager != null) return false;
			if (distributionAndInterestPayment != null ? !distributionAndInterestPayment.equals(_that.distributionAndInterestPayment) : _that.distributionAndInterestPayment != null) return false;
			if (japaneseLawCsa != null ? !japaneseLawCsa.equals(_that.japaneseLawCsa) : _that.japaneseLawCsa != null) return false;
			if (obligorAdditionalRightsEvent != null ? !obligorAdditionalRightsEvent.equals(_that.obligorAdditionalRightsEvent) : _that.obligorAdditionalRightsEvent != null) return false;
			if (obligorPostingObligations != null ? !obligorPostingObligations.equals(_that.obligorPostingObligations) : _that.obligorPostingObligations != null) return false;
			if (obligorRightsEvent != null ? !obligorRightsEvent.equals(_that.obligorRightsEvent) : _that.obligorRightsEvent != null) return false;
			if (otherCsa != null ? !otherCsa.equals(_that.otherCsa) : _that.otherCsa != null) return false;
			if (otherEligibleAndPostedSupport != null ? !otherEligibleAndPostedSupport.equals(_that.otherEligibleAndPostedSupport) : _that.otherEligibleAndPostedSupport != null) return false;
			if (otherEligibleSupport != null ? !otherEligibleSupport.equals(_that.otherEligibleSupport) : _that.otherEligibleSupport != null) return false;
			if (securitiesSettlementDay != null ? !securitiesSettlementDay.equals(_that.securitiesSettlementDay) : _that.securitiesSettlementDay != null) return false;
			if (trustSchemeAddendum != null ? !trustSchemeAddendum.equals(_that.trustSchemeAddendum) : _that.trustSchemeAddendum != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = super.hashCode();
			_result = 31 * _result + (automaticSetOff != null ? automaticSetOff.hashCode() : 0);
			_result = 31 * _result + (cashSettlementDay != null ? cashSettlementDay.hashCode() : 0);
			_result = 31 * _result + (collateralManagementAgreeement != null ? collateralManagementAgreeement.hashCode() : 0);
			_result = 31 * _result + (collateralManagementArrangement != null ? collateralManagementArrangement.hashCode() : 0);
			_result = 31 * _result + (collateralManager != null ? collateralManager.hashCode() : 0);
			_result = 31 * _result + (distributionAndInterestPayment != null ? distributionAndInterestPayment.hashCode() : 0);
			_result = 31 * _result + (japaneseLawCsa != null ? japaneseLawCsa.hashCode() : 0);
			_result = 31 * _result + (obligorAdditionalRightsEvent != null ? obligorAdditionalRightsEvent.hashCode() : 0);
			_result = 31 * _result + (obligorPostingObligations != null ? obligorPostingObligations.hashCode() : 0);
			_result = 31 * _result + (obligorRightsEvent != null ? obligorRightsEvent.hashCode() : 0);
			_result = 31 * _result + (otherCsa != null ? otherCsa.hashCode() : 0);
			_result = 31 * _result + (otherEligibleAndPostedSupport != null ? otherEligibleAndPostedSupport.hashCode() : 0);
			_result = 31 * _result + (otherEligibleSupport != null ? otherEligibleSupport.hashCode() : 0);
			_result = 31 * _result + (securitiesSettlementDay != null ? securitiesSettlementDay.hashCode() : 0);
			_result = 31 * _result + (trustSchemeAddendum != null ? trustSchemeAddendum.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "CsaInitialMargin2016JapaneseLawBuilder {" +
				"automaticSetOff=" + this.automaticSetOff + ", " +
				"cashSettlementDay=" + this.cashSettlementDay + ", " +
				"collateralManagementAgreeement=" + this.collateralManagementAgreeement + ", " +
				"collateralManagementArrangement=" + this.collateralManagementArrangement + ", " +
				"collateralManager=" + this.collateralManager + ", " +
				"distributionAndInterestPayment=" + this.distributionAndInterestPayment + ", " +
				"japaneseLawCsa=" + this.japaneseLawCsa + ", " +
				"obligorAdditionalRightsEvent=" + this.obligorAdditionalRightsEvent + ", " +
				"obligorPostingObligations=" + this.obligorPostingObligations + ", " +
				"obligorRightsEvent=" + this.obligorRightsEvent + ", " +
				"otherCsa=" + this.otherCsa + ", " +
				"otherEligibleAndPostedSupport=" + this.otherEligibleAndPostedSupport + ", " +
				"otherEligibleSupport=" + this.otherEligibleSupport + ", " +
				"securitiesSettlementDay=" + this.securitiesSettlementDay + ", " +
				"trustSchemeAddendum=" + this.trustSchemeAddendum +
			'}' + " " + super.toString();
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		super.process(path, processor);
		
		processor.processBasic(path.newSubPath("automaticSetOff"), Boolean.class, automaticSetOff, this);
		processor.processBasic(path.newSubPath("cashSettlementDay"), String.class, cashSettlementDay, this);
		processor.processBasic(path.newSubPath("otherEligibleSupport"), String.class, otherEligibleSupport, this);
		processor.processBasic(path.newSubPath("securitiesSettlementDay"), String.class, securitiesSettlementDay, this);
		processor.processBasic(path.newSubPath("trustSchemeAddendum"), Boolean.class, trustSchemeAddendum, this);
		
		processRosetta(path.newSubPath("collateralManagementAgreeement"), processor, CollateralManagementAgreement.class, collateralManagementAgreeement);
		processRosetta(path.newSubPath("collateralManagementArrangement"), processor, CollateralManagementArrangement.class, collateralManagementArrangement);
		processRosetta(path.newSubPath("collateralManager"), processor, CollateralManager.class, collateralManager);
		processRosetta(path.newSubPath("distributionAndInterestPayment"), processor, DistributionAndInterestPayment.class, distributionAndInterestPayment);
		processRosetta(path.newSubPath("japaneseLawCsa"), processor, RelatedAgreement.class, japaneseLawCsa);
		processRosetta(path.newSubPath("obligorAdditionalRightsEvent"), processor, AdditionalRightsEvent.class, obligorAdditionalRightsEvent);
		processRosetta(path.newSubPath("obligorPostingObligations"), processor, ObligorPostingObligations.class, obligorPostingObligations);
		processRosetta(path.newSubPath("obligorRightsEvent"), processor, RightsEvent.class, obligorRightsEvent);
		processRosetta(path.newSubPath("otherCsa"), processor, RelatedAgreement.class, otherCsa);
		processRosetta(path.newSubPath("otherEligibleAndPostedSupport"), processor, OtherEligibleAndPostedSupport.class, otherEligibleAndPostedSupport);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
		if (!super.equals(o)) return false;
	
		CsaInitialMargin2016JapaneseLaw _that = (CsaInitialMargin2016JapaneseLaw) o;
	
		if (automaticSetOff != null ? !automaticSetOff.equals(_that.automaticSetOff) : _that.automaticSetOff != null) return false;
		if (cashSettlementDay != null ? !cashSettlementDay.equals(_that.cashSettlementDay) : _that.cashSettlementDay != null) return false;
		if (collateralManagementAgreeement != null ? !collateralManagementAgreeement.equals(_that.collateralManagementAgreeement) : _that.collateralManagementAgreeement != null) return false;
		if (collateralManagementArrangement != null ? !collateralManagementArrangement.equals(_that.collateralManagementArrangement) : _that.collateralManagementArrangement != null) return false;
		if (collateralManager != null ? !collateralManager.equals(_that.collateralManager) : _that.collateralManager != null) return false;
		if (distributionAndInterestPayment != null ? !distributionAndInterestPayment.equals(_that.distributionAndInterestPayment) : _that.distributionAndInterestPayment != null) return false;
		if (japaneseLawCsa != null ? !japaneseLawCsa.equals(_that.japaneseLawCsa) : _that.japaneseLawCsa != null) return false;
		if (obligorAdditionalRightsEvent != null ? !obligorAdditionalRightsEvent.equals(_that.obligorAdditionalRightsEvent) : _that.obligorAdditionalRightsEvent != null) return false;
		if (obligorPostingObligations != null ? !obligorPostingObligations.equals(_that.obligorPostingObligations) : _that.obligorPostingObligations != null) return false;
		if (obligorRightsEvent != null ? !obligorRightsEvent.equals(_that.obligorRightsEvent) : _that.obligorRightsEvent != null) return false;
		if (otherCsa != null ? !otherCsa.equals(_that.otherCsa) : _that.otherCsa != null) return false;
		if (otherEligibleAndPostedSupport != null ? !otherEligibleAndPostedSupport.equals(_that.otherEligibleAndPostedSupport) : _that.otherEligibleAndPostedSupport != null) return false;
		if (otherEligibleSupport != null ? !otherEligibleSupport.equals(_that.otherEligibleSupport) : _that.otherEligibleSupport != null) return false;
		if (securitiesSettlementDay != null ? !securitiesSettlementDay.equals(_that.securitiesSettlementDay) : _that.securitiesSettlementDay != null) return false;
		if (trustSchemeAddendum != null ? !trustSchemeAddendum.equals(_that.trustSchemeAddendum) : _that.trustSchemeAddendum != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = super.hashCode();
		_result = 31 * _result + (automaticSetOff != null ? automaticSetOff.hashCode() : 0);
		_result = 31 * _result + (cashSettlementDay != null ? cashSettlementDay.hashCode() : 0);
		_result = 31 * _result + (collateralManagementAgreeement != null ? collateralManagementAgreeement.hashCode() : 0);
		_result = 31 * _result + (collateralManagementArrangement != null ? collateralManagementArrangement.hashCode() : 0);
		_result = 31 * _result + (collateralManager != null ? collateralManager.hashCode() : 0);
		_result = 31 * _result + (distributionAndInterestPayment != null ? distributionAndInterestPayment.hashCode() : 0);
		_result = 31 * _result + (japaneseLawCsa != null ? japaneseLawCsa.hashCode() : 0);
		_result = 31 * _result + (obligorAdditionalRightsEvent != null ? obligorAdditionalRightsEvent.hashCode() : 0);
		_result = 31 * _result + (obligorPostingObligations != null ? obligorPostingObligations.hashCode() : 0);
		_result = 31 * _result + (obligorRightsEvent != null ? obligorRightsEvent.hashCode() : 0);
		_result = 31 * _result + (otherCsa != null ? otherCsa.hashCode() : 0);
		_result = 31 * _result + (otherEligibleAndPostedSupport != null ? otherEligibleAndPostedSupport.hashCode() : 0);
		_result = 31 * _result + (otherEligibleSupport != null ? otherEligibleSupport.hashCode() : 0);
		_result = 31 * _result + (securitiesSettlementDay != null ? securitiesSettlementDay.hashCode() : 0);
		_result = 31 * _result + (trustSchemeAddendum != null ? trustSchemeAddendum.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "CsaInitialMargin2016JapaneseLaw {" +
			"automaticSetOff=" + this.automaticSetOff + ", " +
			"cashSettlementDay=" + this.cashSettlementDay + ", " +
			"collateralManagementAgreeement=" + this.collateralManagementAgreeement + ", " +
			"collateralManagementArrangement=" + this.collateralManagementArrangement + ", " +
			"collateralManager=" + this.collateralManager + ", " +
			"distributionAndInterestPayment=" + this.distributionAndInterestPayment + ", " +
			"japaneseLawCsa=" + this.japaneseLawCsa + ", " +
			"obligorAdditionalRightsEvent=" + this.obligorAdditionalRightsEvent + ", " +
			"obligorPostingObligations=" + this.obligorPostingObligations + ", " +
			"obligorRightsEvent=" + this.obligorRightsEvent + ", " +
			"otherCsa=" + this.otherCsa + ", " +
			"otherEligibleAndPostedSupport=" + this.otherEligibleAndPostedSupport + ", " +
			"otherEligibleSupport=" + this.otherEligibleSupport + ", " +
			"securitiesSettlementDay=" + this.securitiesSettlementDay + ", " +
			"trustSchemeAddendum=" + this.trustSchemeAddendum +
		'}' + " " + super.toString();
	}
}
