package org.isda.cdm;

import java.lang.Boolean;
import java.lang.String;
import org.isda.cdm.AdditionalRightsEvent;
import org.isda.cdm.CustodyArrangements;
import org.isda.cdm.OtherEligibleAndPostedSupport;
import org.isda.cdm.PledgorPostingObligations;
import org.isda.cdm.RightsEvent;
import org.isda.cdm.metafields.FieldWithMetaString;


import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.CsaInitialMargin2016NewYorkLawMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class to specify the provisions that are specific to the New York Law version of the ISDA 2016 Credit Support Annex for Initial Margin.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class CsaInitialMargin2016NewYorkLaw extends CsaInitialMargin2016 {
	private final CustodyArrangements custodyArrangements;
	private final Boolean deliveryInLieuRight;
	private final Boolean japaneseSecuritiesProvisions;
	private final OtherEligibleAndPostedSupport otherEligibleAndPostedSupport;
	private final String otherEligibleSupport;
	private final AdditionalRightsEvent pledgorAdditionalRightsEvent;
	private final PledgorPostingObligations pledgorPostingObligations;
	private final RightsEvent pledgorRightsEvent;
	private final String transferTiming;
	private static CsaInitialMargin2016NewYorkLawMeta metaData = new CsaInitialMargin2016NewYorkLawMeta();
	
	CsaInitialMargin2016NewYorkLaw(CsaInitialMargin2016NewYorkLawBuilder builder) {
		super(builder);
		this.custodyArrangements = ofNullable(builder.getCustodyArrangements()).map(f->f.build()).orElse(null);
		this.deliveryInLieuRight = builder.getDeliveryInLieuRight();
		this.japaneseSecuritiesProvisions = builder.getJapaneseSecuritiesProvisions();
		this.otherEligibleAndPostedSupport = ofNullable(builder.getOtherEligibleAndPostedSupport()).map(f->f.build()).orElse(null);
		this.otherEligibleSupport = builder.getOtherEligibleSupport();
		this.pledgorAdditionalRightsEvent = ofNullable(builder.getPledgorAdditionalRightsEvent()).map(f->f.build()).orElse(null);
		this.pledgorPostingObligations = ofNullable(builder.getPledgorPostingObligations()).map(f->f.build()).orElse(null);
		this.pledgorRightsEvent = ofNullable(builder.getPledgorRightsEvent()).map(f->f.build()).orElse(null);
		this.transferTiming = builder.getTransferTiming();
	}
	
	/**
	 * The Custodian and Segregated Account details in respect of each party as the Pledgor. ISDA 2016 New York Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (n): Custody Arrangements.
	 */
	@RosettaSynonym(value="control_agreement_as_a_credit_support_document", source="ISDA_Create_1_0", path="answers")
	public final CustodyArrangements getCustodyArrangements() {
		return custodyArrangements;
	}
	
	/**
	 * The Delivery in Lieu Right provision applies when the value is set to True. ISDA 2016 New York Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (m): Modification to Pledgor’s Rights and Remedies.
	 */
	@RosettaSynonym(value="", source="ISDA_Create_1_0")
	public final Boolean getDeliveryInLieuRight() {
		return deliveryInLieuRight;
	}
	
	/**
	 * ISDA 2016 New York Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (v): Japanese Securities Provisions (Shichiken).
	 */
	@RosettaSynonym(value="", source="ISDA_Create_1_0")
	public final Boolean getJapaneseSecuritiesProvisions() {
		return japaneseSecuritiesProvisions;
	}
	
	/**
	 * The Other Eligible Support elections associated with margin agreements. ISDA 2016 New York Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (p): Other Eligible Support (IM) and Other Posted Support (IM).
	 */
	public final OtherEligibleAndPostedSupport getOtherEligibleAndPostedSupport() {
		return otherEligibleAndPostedSupport;
	}
	
	/**
	 * The Other Eligible Support election. If not specified, this election is deemed as not applicable. ISDA 2016 New York Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (c)(iv).
	 */
	@RosettaSynonym(value="other_eligible_support", source="ISDA_Create_1_0", path="other_eligible_support")
	public final String getOtherEligibleSupport() {
		return otherEligibleSupport;
	}
	
	/**
	 * The Pledgor Additional Rights Event election. ISDA 2016 New York Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (k): Pledgor Additional Rights Event.
	 */
	@RosettaSynonym(value="pledgor_additional_rights_event", source="ISDA_Create_1_0")
	public final AdditionalRightsEvent getPledgorAdditionalRightsEvent() {
		return pledgorAdditionalRightsEvent;
	}
	
	/**
	 * The pledgor&#39;s posting obligations. This term of pledgor is the equivalent of the chargor in the English Law CSA and the obligor in the Japanese Law CSA, i.e. it&#39;s the party that will post/pledge collateral to the other party. ISDA 2016 New York Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (ii).
	 */
	@RosettaSynonym(value="pledgor_posting_obligations", source="ISDA_Create_1_0")
	public final PledgorPostingObligations getPledgorPostingObligations() {
		return pledgorPostingObligations;
	}
	
	/**
	 * The Pledgor Rights Event election. ISDA 2016 New York Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (j): Pledgor Rights Event.
	 */
	@RosettaSynonym(value="pledgor_rights_event", source="ISDA_Create_1_0")
	public final RightsEvent getPledgorRightsEvent() {
		return pledgorRightsEvent;
	}
	
	/**
	 * The time by which the transfer of collateral must take place when different from the Regular Settlement Day as a result of parties&#39; election. ISDA 2016 New York Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (c)(vii): Transfer Timing.
	 */
	@RosettaSynonym(value="settlement_day_specify", source="ISDA_Create_1_0", path="transfer_timing")
	public final String getTransferTiming() {
		return transferTiming;
	}
	
	
	@Override
	public RosettaMetaData<? extends CsaInitialMargin2016NewYorkLaw> metaData() {
		return metaData;
	} 
	
	public CsaInitialMargin2016NewYorkLawBuilder toBuilder() {
		CsaInitialMargin2016NewYorkLawBuilder builder = new CsaInitialMargin2016NewYorkLawBuilder();
		ofNullable(getCustodyArrangements()).ifPresent(builder::setCustodyArrangements);
		ofNullable(getDeliveryInLieuRight()).ifPresent(builder::setDeliveryInLieuRight);
		ofNullable(getJapaneseSecuritiesProvisions()).ifPresent(builder::setJapaneseSecuritiesProvisions);
		ofNullable(getOtherEligibleAndPostedSupport()).ifPresent(builder::setOtherEligibleAndPostedSupport);
		ofNullable(getOtherEligibleSupport()).ifPresent(builder::setOtherEligibleSupport);
		ofNullable(getPledgorAdditionalRightsEvent()).ifPresent(builder::setPledgorAdditionalRightsEvent);
		ofNullable(getPledgorPostingObligations()).ifPresent(builder::setPledgorPostingObligations);
		ofNullable(getPledgorRightsEvent()).ifPresent(builder::setPledgorRightsEvent);
		ofNullable(getTransferTiming()).ifPresent(builder::setTransferTiming);
		ofNullable(getCalculationDateLocation()).ifPresent(builder::setCalculationDateLocation);
		ofNullable(getCreditSupportObligations()).ifPresent(builder::setCreditSupportObligations);
		ofNullable(getFxHaircutCurrency()).ifPresent(builder::setFxHaircutCurrency);
		ofNullable(getIdentifiedCrossCurrencySwap()).ifPresent(builder::setIdentifiedCrossCurrencySwap);
		ofNullable(getMethod()).ifPresent(builder::setMethod);
		ofNullable(getNotificationTime()).ifPresent(builder::setNotificationTime);
		ofNullable(getOneWayProvisions()).ifPresent(builder::setOneWayProvisions);
		ofNullable(getRegime()).ifPresent(builder::setRegime);
		ofNullable(getSensitivityToCommodity()).ifPresent(builder::setSensitivityToCommodity);
		ofNullable(getSensitivityToEquity()).ifPresent(builder::setSensitivityToEquity);
		ofNullable(getTerminationCurrency()).ifPresent(builder::setTerminationCurrency);
		ofNullable(getAdditionalObligations()).ifPresent(builder::setAdditionalObligations);
		ofNullable(getAdditionalRepresentation()).ifPresent(builder::setAdditionalRepresentation);
		ofNullable(getAddressesForTransfer()).ifPresent(builder::setAddressesForTransfer);
		ofNullable(getBaseCurrency()).ifPresent(builder::setBaseCurrency);
		ofNullable(getBespokeProvision()).ifPresent(builder::setBespokeProvision);
		ofNullable(getConditionsPrecedent()).ifPresent(builder::setConditionsPrecedent);
		ofNullable(getDemandsAndNotices()).ifPresent(builder::setDemandsAndNotices);
		ofNullable(getDisputeResolution()).ifPresent(builder::setDisputeResolution);
		ofNullable(getSubstitution()).ifPresent(builder::setSubstitution);
		return builder;
	}

	public static CsaInitialMargin2016NewYorkLawBuilder builder() {
		return new CsaInitialMargin2016NewYorkLawBuilder();
	}

	public static  class CsaInitialMargin2016NewYorkLawBuilder extends CsaInitialMargin2016Builder {
	
		protected CustodyArrangements.CustodyArrangementsBuilder custodyArrangements;
		protected Boolean deliveryInLieuRight;
		protected Boolean japaneseSecuritiesProvisions;
		protected OtherEligibleAndPostedSupport.OtherEligibleAndPostedSupportBuilder otherEligibleAndPostedSupport;
		protected String otherEligibleSupport;
		protected AdditionalRightsEvent.AdditionalRightsEventBuilder pledgorAdditionalRightsEvent;
		protected PledgorPostingObligations.PledgorPostingObligationsBuilder pledgorPostingObligations;
		protected RightsEvent.RightsEventBuilder pledgorRightsEvent;
		protected String transferTiming;
	
		public CsaInitialMargin2016NewYorkLawBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends CsaInitialMargin2016NewYorkLaw> metaData() {
			return metaData;
		} 
	
		
		public CustodyArrangements.CustodyArrangementsBuilder getCustodyArrangements() {
			return custodyArrangements;
		}
		
		public CustodyArrangements.CustodyArrangementsBuilder getOrCreateCustodyArrangements() {
			if (custodyArrangements!=null) {
				return custodyArrangements;
			}
			else return custodyArrangements = new CustodyArrangements.CustodyArrangementsBuilder();
		}
		
		
		public Boolean getDeliveryInLieuRight() {
			return deliveryInLieuRight;
		}
		
		
		public Boolean getJapaneseSecuritiesProvisions() {
			return japaneseSecuritiesProvisions;
		}
		
		
		public OtherEligibleAndPostedSupport.OtherEligibleAndPostedSupportBuilder getOtherEligibleAndPostedSupport() {
			return otherEligibleAndPostedSupport;
		}
		
		public OtherEligibleAndPostedSupport.OtherEligibleAndPostedSupportBuilder getOrCreateOtherEligibleAndPostedSupport() {
			if (otherEligibleAndPostedSupport!=null) {
				return otherEligibleAndPostedSupport;
			}
			else return otherEligibleAndPostedSupport = new OtherEligibleAndPostedSupport.OtherEligibleAndPostedSupportBuilder();
		}
		
		
		public String getOtherEligibleSupport() {
			return otherEligibleSupport;
		}
		
		
		public AdditionalRightsEvent.AdditionalRightsEventBuilder getPledgorAdditionalRightsEvent() {
			return pledgorAdditionalRightsEvent;
		}
		
		public AdditionalRightsEvent.AdditionalRightsEventBuilder getOrCreatePledgorAdditionalRightsEvent() {
			if (pledgorAdditionalRightsEvent!=null) {
				return pledgorAdditionalRightsEvent;
			}
			else return pledgorAdditionalRightsEvent = new AdditionalRightsEvent.AdditionalRightsEventBuilder();
		}
		
		
		public PledgorPostingObligations.PledgorPostingObligationsBuilder getPledgorPostingObligations() {
			return pledgorPostingObligations;
		}
		
		public PledgorPostingObligations.PledgorPostingObligationsBuilder getOrCreatePledgorPostingObligations() {
			if (pledgorPostingObligations!=null) {
				return pledgorPostingObligations;
			}
			else return pledgorPostingObligations = new PledgorPostingObligations.PledgorPostingObligationsBuilder();
		}
		
		
		public RightsEvent.RightsEventBuilder getPledgorRightsEvent() {
			return pledgorRightsEvent;
		}
		
		public RightsEvent.RightsEventBuilder getOrCreatePledgorRightsEvent() {
			if (pledgorRightsEvent!=null) {
				return pledgorRightsEvent;
			}
			else return pledgorRightsEvent = new RightsEvent.RightsEventBuilder();
		}
		
		
		public String getTransferTiming() {
			return transferTiming;
		}
		
	
		public CsaInitialMargin2016NewYorkLawBuilder setCustodyArrangements(CustodyArrangements custodyArrangements) {
			this.custodyArrangements = custodyArrangements.toBuilder();
			return this;
		}
		public CsaInitialMargin2016NewYorkLawBuilder setCustodyArrangementsBuilder(CustodyArrangements.CustodyArrangementsBuilder custodyArrangements) {
			this.custodyArrangements = custodyArrangements;
			return this;
		}
		public CsaInitialMargin2016NewYorkLawBuilder setDeliveryInLieuRight(Boolean deliveryInLieuRight) {
			this.deliveryInLieuRight = deliveryInLieuRight;
			return this;
		}
		public CsaInitialMargin2016NewYorkLawBuilder setJapaneseSecuritiesProvisions(Boolean japaneseSecuritiesProvisions) {
			this.japaneseSecuritiesProvisions = japaneseSecuritiesProvisions;
			return this;
		}
		public CsaInitialMargin2016NewYorkLawBuilder setOtherEligibleAndPostedSupport(OtherEligibleAndPostedSupport otherEligibleAndPostedSupport) {
			this.otherEligibleAndPostedSupport = otherEligibleAndPostedSupport.toBuilder();
			return this;
		}
		public CsaInitialMargin2016NewYorkLawBuilder setOtherEligibleAndPostedSupportBuilder(OtherEligibleAndPostedSupport.OtherEligibleAndPostedSupportBuilder otherEligibleAndPostedSupport) {
			this.otherEligibleAndPostedSupport = otherEligibleAndPostedSupport;
			return this;
		}
		public CsaInitialMargin2016NewYorkLawBuilder setOtherEligibleSupport(String otherEligibleSupport) {
			this.otherEligibleSupport = otherEligibleSupport;
			return this;
		}
		public CsaInitialMargin2016NewYorkLawBuilder setPledgorAdditionalRightsEvent(AdditionalRightsEvent pledgorAdditionalRightsEvent) {
			this.pledgorAdditionalRightsEvent = pledgorAdditionalRightsEvent.toBuilder();
			return this;
		}
		public CsaInitialMargin2016NewYorkLawBuilder setPledgorAdditionalRightsEventBuilder(AdditionalRightsEvent.AdditionalRightsEventBuilder pledgorAdditionalRightsEvent) {
			this.pledgorAdditionalRightsEvent = pledgorAdditionalRightsEvent;
			return this;
		}
		public CsaInitialMargin2016NewYorkLawBuilder setPledgorPostingObligations(PledgorPostingObligations pledgorPostingObligations) {
			this.pledgorPostingObligations = pledgorPostingObligations.toBuilder();
			return this;
		}
		public CsaInitialMargin2016NewYorkLawBuilder setPledgorPostingObligationsBuilder(PledgorPostingObligations.PledgorPostingObligationsBuilder pledgorPostingObligations) {
			this.pledgorPostingObligations = pledgorPostingObligations;
			return this;
		}
		public CsaInitialMargin2016NewYorkLawBuilder setPledgorRightsEvent(RightsEvent pledgorRightsEvent) {
			this.pledgorRightsEvent = pledgorRightsEvent.toBuilder();
			return this;
		}
		public CsaInitialMargin2016NewYorkLawBuilder setPledgorRightsEventBuilder(RightsEvent.RightsEventBuilder pledgorRightsEvent) {
			this.pledgorRightsEvent = pledgorRightsEvent;
			return this;
		}
		public CsaInitialMargin2016NewYorkLawBuilder setTransferTiming(String transferTiming) {
			this.transferTiming = transferTiming;
			return this;
		}
		@Override public CsaInitialMargin2016NewYorkLawBuilder setCalculationDateLocation(CalculationDateLocation calculationDateLocation) {
			this.calculationDateLocation = calculationDateLocation.toBuilder();
			return this;
		}
		@Override public CsaInitialMargin2016NewYorkLawBuilder setCalculationDateLocationBuilder(CalculationDateLocation.CalculationDateLocationBuilder calculationDateLocation) {
			this.calculationDateLocation = calculationDateLocation;
			return this;
		}
		@Override public CsaInitialMargin2016NewYorkLawBuilder setCreditSupportObligations(CreditSupportObligationsInitialMargin creditSupportObligations) {
			this.creditSupportObligations = creditSupportObligations.toBuilder();
			return this;
		}
		@Override public CsaInitialMargin2016NewYorkLawBuilder setCreditSupportObligationsBuilder(CreditSupportObligationsInitialMargin.CreditSupportObligationsInitialMarginBuilder creditSupportObligations) {
			this.creditSupportObligations = creditSupportObligations;
			return this;
		}
		@Override public CsaInitialMargin2016NewYorkLawBuilder setFxHaircutCurrency(FxHaircutCurrency fxHaircutCurrency) {
			this.fxHaircutCurrency = fxHaircutCurrency.toBuilder();
			return this;
		}
		@Override public CsaInitialMargin2016NewYorkLawBuilder setFxHaircutCurrencyBuilder(FxHaircutCurrency.FxHaircutCurrencyBuilder fxHaircutCurrency) {
			this.fxHaircutCurrency = fxHaircutCurrency;
			return this;
		}
		@Override public CsaInitialMargin2016NewYorkLawBuilder setIdentifiedCrossCurrencySwap(Boolean identifiedCrossCurrencySwap) {
			this.identifiedCrossCurrencySwap = identifiedCrossCurrencySwap;
			return this;
		}
		@Override public CsaInitialMargin2016NewYorkLawBuilder setMethod(Method method) {
			this.method = method.toBuilder();
			return this;
		}
		@Override public CsaInitialMargin2016NewYorkLawBuilder setMethodBuilder(Method.MethodBuilder method) {
			this.method = method;
			return this;
		}
		@Override public CsaInitialMargin2016NewYorkLawBuilder setNotificationTime(NotificationTime notificationTime) {
			this.notificationTime = notificationTime.toBuilder();
			return this;
		}
		@Override public CsaInitialMargin2016NewYorkLawBuilder setNotificationTimeBuilder(NotificationTime.NotificationTimeBuilder notificationTime) {
			this.notificationTime = notificationTime;
			return this;
		}
		@Override public CsaInitialMargin2016NewYorkLawBuilder setOneWayProvisions(OneWayProvisions oneWayProvisions) {
			this.oneWayProvisions = oneWayProvisions.toBuilder();
			return this;
		}
		@Override public CsaInitialMargin2016NewYorkLawBuilder setOneWayProvisionsBuilder(OneWayProvisions.OneWayProvisionsBuilder oneWayProvisions) {
			this.oneWayProvisions = oneWayProvisions;
			return this;
		}
		@Override public CsaInitialMargin2016NewYorkLawBuilder setRegime(Regime regime) {
			this.regime = regime.toBuilder();
			return this;
		}
		@Override public CsaInitialMargin2016NewYorkLawBuilder setRegimeBuilder(Regime.RegimeBuilder regime) {
			this.regime = regime;
			return this;
		}
		@Override public CsaInitialMargin2016NewYorkLawBuilder setSensitivityToCommodity(SensitivityMethodology sensitivityToCommodity) {
			this.sensitivityToCommodity = sensitivityToCommodity.toBuilder();
			return this;
		}
		@Override public CsaInitialMargin2016NewYorkLawBuilder setSensitivityToCommodityBuilder(SensitivityMethodology.SensitivityMethodologyBuilder sensitivityToCommodity) {
			this.sensitivityToCommodity = sensitivityToCommodity;
			return this;
		}
		@Override public CsaInitialMargin2016NewYorkLawBuilder setSensitivityToEquity(SensitivityMethodology sensitivityToEquity) {
			this.sensitivityToEquity = sensitivityToEquity.toBuilder();
			return this;
		}
		@Override public CsaInitialMargin2016NewYorkLawBuilder setSensitivityToEquityBuilder(SensitivityMethodology.SensitivityMethodologyBuilder sensitivityToEquity) {
			this.sensitivityToEquity = sensitivityToEquity;
			return this;
		}
		@Override public CsaInitialMargin2016NewYorkLawBuilder setTerminationCurrency(TerminationCurrencyAmendment terminationCurrency) {
			this.terminationCurrency = terminationCurrency.toBuilder();
			return this;
		}
		@Override public CsaInitialMargin2016NewYorkLawBuilder setTerminationCurrencyBuilder(TerminationCurrencyAmendment.TerminationCurrencyAmendmentBuilder terminationCurrency) {
			this.terminationCurrency = terminationCurrency;
			return this;
		}
		@Override public CsaInitialMargin2016NewYorkLawBuilder setAdditionalObligations(String additionalObligations) {
			this.additionalObligations = additionalObligations;
			return this;
		}
		@Override public CsaInitialMargin2016NewYorkLawBuilder setAdditionalRepresentation(AdditionalRepresentation additionalRepresentation) {
			this.additionalRepresentation = additionalRepresentation.toBuilder();
			return this;
		}
		@Override public CsaInitialMargin2016NewYorkLawBuilder setAdditionalRepresentationBuilder(AdditionalRepresentation.AdditionalRepresentationBuilder additionalRepresentation) {
			this.additionalRepresentation = additionalRepresentation;
			return this;
		}
		@Override public CsaInitialMargin2016NewYorkLawBuilder setAddressesForTransfer(ContactElection addressesForTransfer) {
			this.addressesForTransfer = addressesForTransfer.toBuilder();
			return this;
		}
		@Override public CsaInitialMargin2016NewYorkLawBuilder setAddressesForTransferBuilder(ContactElection.ContactElectionBuilder addressesForTransfer) {
			this.addressesForTransfer = addressesForTransfer;
			return this;
		}
		@Override public CsaInitialMargin2016NewYorkLawBuilder setBaseCurrency(FieldWithMetaString baseCurrency) {
			this.baseCurrency = baseCurrency.toBuilder();
			return this;
		}
		@Override public CsaInitialMargin2016NewYorkLawBuilder setBaseCurrencyRef(String baseCurrency) {
			return setBaseCurrency(FieldWithMetaString.builder().setValue(baseCurrency).build());
		}
		@Override public CsaInitialMargin2016NewYorkLawBuilder setBespokeProvision(String bespokeProvision) {
			this.bespokeProvision = bespokeProvision;
			return this;
		}
		@Override public CsaInitialMargin2016NewYorkLawBuilder setConditionsPrecedent(ConditionsPrecedent conditionsPrecedent) {
			this.conditionsPrecedent = conditionsPrecedent.toBuilder();
			return this;
		}
		@Override public CsaInitialMargin2016NewYorkLawBuilder setConditionsPrecedentBuilder(ConditionsPrecedent.ConditionsPrecedentBuilder conditionsPrecedent) {
			this.conditionsPrecedent = conditionsPrecedent;
			return this;
		}
		@Override public CsaInitialMargin2016NewYorkLawBuilder setDemandsAndNotices(ContactElection demandsAndNotices) {
			this.demandsAndNotices = demandsAndNotices.toBuilder();
			return this;
		}
		@Override public CsaInitialMargin2016NewYorkLawBuilder setDemandsAndNoticesBuilder(ContactElection.ContactElectionBuilder demandsAndNotices) {
			this.demandsAndNotices = demandsAndNotices;
			return this;
		}
		@Override public CsaInitialMargin2016NewYorkLawBuilder setDisputeResolution(DisputeResolution disputeResolution) {
			this.disputeResolution = disputeResolution.toBuilder();
			return this;
		}
		@Override public CsaInitialMargin2016NewYorkLawBuilder setDisputeResolutionBuilder(DisputeResolution.DisputeResolutionBuilder disputeResolution) {
			this.disputeResolution = disputeResolution;
			return this;
		}
		@Override public CsaInitialMargin2016NewYorkLawBuilder setSubstitution(Substitution substitution) {
			this.substitution = substitution.toBuilder();
			return this;
		}
		@Override public CsaInitialMargin2016NewYorkLawBuilder setSubstitutionBuilder(Substitution.SubstitutionBuilder substitution) {
			this.substitution = substitution;
			return this;
		}
		
		public CsaInitialMargin2016NewYorkLaw build() {
			return new CsaInitialMargin2016NewYorkLaw(this);
		}
	
		@Override
		public CsaInitialMargin2016NewYorkLawBuilder prune() {
			super.prune();
			if (custodyArrangements!=null && !custodyArrangements.prune().hasData()) custodyArrangements = null;
			if (otherEligibleAndPostedSupport!=null && !otherEligibleAndPostedSupport.prune().hasData()) otherEligibleAndPostedSupport = null;
			if (pledgorAdditionalRightsEvent!=null && !pledgorAdditionalRightsEvent.prune().hasData()) pledgorAdditionalRightsEvent = null;
			if (pledgorPostingObligations!=null && !pledgorPostingObligations.prune().hasData()) pledgorPostingObligations = null;
			if (pledgorRightsEvent!=null && !pledgorRightsEvent.prune().hasData()) pledgorRightsEvent = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (super.hasData()) return true;
			if (getCustodyArrangements()!=null && getCustodyArrangements().hasData()) return true;
			if (getDeliveryInLieuRight()!=null) return true;
			if (getJapaneseSecuritiesProvisions()!=null) return true;
			if (getOtherEligibleAndPostedSupport()!=null && getOtherEligibleAndPostedSupport().hasData()) return true;
			if (getOtherEligibleSupport()!=null) return true;
			if (getPledgorAdditionalRightsEvent()!=null && getPledgorAdditionalRightsEvent().hasData()) return true;
			if (getPledgorPostingObligations()!=null && getPledgorPostingObligations().hasData()) return true;
			if (getPledgorRightsEvent()!=null && getPledgorRightsEvent().hasData()) return true;
			if (getTransferTiming()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
			super.process(path, processor);
		
			processor.processBasic(path.newSubPath("deliveryInLieuRight"), Boolean.class, deliveryInLieuRight, this);
			processor.processBasic(path.newSubPath("japaneseSecuritiesProvisions"), Boolean.class, japaneseSecuritiesProvisions, this);
			processor.processBasic(path.newSubPath("otherEligibleSupport"), String.class, otherEligibleSupport, this);
			processor.processBasic(path.newSubPath("transferTiming"), String.class, transferTiming, this);
			
			processRosetta(path.newSubPath("custodyArrangements"), processor, CustodyArrangements.class, custodyArrangements);
			processRosetta(path.newSubPath("otherEligibleAndPostedSupport"), processor, OtherEligibleAndPostedSupport.class, otherEligibleAndPostedSupport);
			processRosetta(path.newSubPath("pledgorAdditionalRightsEvent"), processor, AdditionalRightsEvent.class, pledgorAdditionalRightsEvent);
			processRosetta(path.newSubPath("pledgorPostingObligations"), processor, PledgorPostingObligations.class, pledgorPostingObligations);
			processRosetta(path.newSubPath("pledgorRightsEvent"), processor, RightsEvent.class, pledgorRightsEvent);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
			if (!super.equals(o)) return false;
		
			CsaInitialMargin2016NewYorkLawBuilder _that = (CsaInitialMargin2016NewYorkLawBuilder) o;
		
			if (custodyArrangements != null ? !custodyArrangements.equals(_that.custodyArrangements) : _that.custodyArrangements != null) return false;
			if (deliveryInLieuRight != null ? !deliveryInLieuRight.equals(_that.deliveryInLieuRight) : _that.deliveryInLieuRight != null) return false;
			if (japaneseSecuritiesProvisions != null ? !japaneseSecuritiesProvisions.equals(_that.japaneseSecuritiesProvisions) : _that.japaneseSecuritiesProvisions != null) return false;
			if (otherEligibleAndPostedSupport != null ? !otherEligibleAndPostedSupport.equals(_that.otherEligibleAndPostedSupport) : _that.otherEligibleAndPostedSupport != null) return false;
			if (otherEligibleSupport != null ? !otherEligibleSupport.equals(_that.otherEligibleSupport) : _that.otherEligibleSupport != null) return false;
			if (pledgorAdditionalRightsEvent != null ? !pledgorAdditionalRightsEvent.equals(_that.pledgorAdditionalRightsEvent) : _that.pledgorAdditionalRightsEvent != null) return false;
			if (pledgorPostingObligations != null ? !pledgorPostingObligations.equals(_that.pledgorPostingObligations) : _that.pledgorPostingObligations != null) return false;
			if (pledgorRightsEvent != null ? !pledgorRightsEvent.equals(_that.pledgorRightsEvent) : _that.pledgorRightsEvent != null) return false;
			if (transferTiming != null ? !transferTiming.equals(_that.transferTiming) : _that.transferTiming != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = super.hashCode();
			_result = 31 * _result + (custodyArrangements != null ? custodyArrangements.hashCode() : 0);
			_result = 31 * _result + (deliveryInLieuRight != null ? deliveryInLieuRight.hashCode() : 0);
			_result = 31 * _result + (japaneseSecuritiesProvisions != null ? japaneseSecuritiesProvisions.hashCode() : 0);
			_result = 31 * _result + (otherEligibleAndPostedSupport != null ? otherEligibleAndPostedSupport.hashCode() : 0);
			_result = 31 * _result + (otherEligibleSupport != null ? otherEligibleSupport.hashCode() : 0);
			_result = 31 * _result + (pledgorAdditionalRightsEvent != null ? pledgorAdditionalRightsEvent.hashCode() : 0);
			_result = 31 * _result + (pledgorPostingObligations != null ? pledgorPostingObligations.hashCode() : 0);
			_result = 31 * _result + (pledgorRightsEvent != null ? pledgorRightsEvent.hashCode() : 0);
			_result = 31 * _result + (transferTiming != null ? transferTiming.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "CsaInitialMargin2016NewYorkLawBuilder {" +
				"custodyArrangements=" + this.custodyArrangements + ", " +
				"deliveryInLieuRight=" + this.deliveryInLieuRight + ", " +
				"japaneseSecuritiesProvisions=" + this.japaneseSecuritiesProvisions + ", " +
				"otherEligibleAndPostedSupport=" + this.otherEligibleAndPostedSupport + ", " +
				"otherEligibleSupport=" + this.otherEligibleSupport + ", " +
				"pledgorAdditionalRightsEvent=" + this.pledgorAdditionalRightsEvent + ", " +
				"pledgorPostingObligations=" + this.pledgorPostingObligations + ", " +
				"pledgorRightsEvent=" + this.pledgorRightsEvent + ", " +
				"transferTiming=" + this.transferTiming +
			'}' + " " + super.toString();
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		super.process(path, processor);
		
		processor.processBasic(path.newSubPath("deliveryInLieuRight"), Boolean.class, deliveryInLieuRight, this);
		processor.processBasic(path.newSubPath("japaneseSecuritiesProvisions"), Boolean.class, japaneseSecuritiesProvisions, this);
		processor.processBasic(path.newSubPath("otherEligibleSupport"), String.class, otherEligibleSupport, this);
		processor.processBasic(path.newSubPath("transferTiming"), String.class, transferTiming, this);
		
		processRosetta(path.newSubPath("custodyArrangements"), processor, CustodyArrangements.class, custodyArrangements);
		processRosetta(path.newSubPath("otherEligibleAndPostedSupport"), processor, OtherEligibleAndPostedSupport.class, otherEligibleAndPostedSupport);
		processRosetta(path.newSubPath("pledgorAdditionalRightsEvent"), processor, AdditionalRightsEvent.class, pledgorAdditionalRightsEvent);
		processRosetta(path.newSubPath("pledgorPostingObligations"), processor, PledgorPostingObligations.class, pledgorPostingObligations);
		processRosetta(path.newSubPath("pledgorRightsEvent"), processor, RightsEvent.class, pledgorRightsEvent);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
		if (!super.equals(o)) return false;
	
		CsaInitialMargin2016NewYorkLaw _that = (CsaInitialMargin2016NewYorkLaw) o;
	
		if (custodyArrangements != null ? !custodyArrangements.equals(_that.custodyArrangements) : _that.custodyArrangements != null) return false;
		if (deliveryInLieuRight != null ? !deliveryInLieuRight.equals(_that.deliveryInLieuRight) : _that.deliveryInLieuRight != null) return false;
		if (japaneseSecuritiesProvisions != null ? !japaneseSecuritiesProvisions.equals(_that.japaneseSecuritiesProvisions) : _that.japaneseSecuritiesProvisions != null) return false;
		if (otherEligibleAndPostedSupport != null ? !otherEligibleAndPostedSupport.equals(_that.otherEligibleAndPostedSupport) : _that.otherEligibleAndPostedSupport != null) return false;
		if (otherEligibleSupport != null ? !otherEligibleSupport.equals(_that.otherEligibleSupport) : _that.otherEligibleSupport != null) return false;
		if (pledgorAdditionalRightsEvent != null ? !pledgorAdditionalRightsEvent.equals(_that.pledgorAdditionalRightsEvent) : _that.pledgorAdditionalRightsEvent != null) return false;
		if (pledgorPostingObligations != null ? !pledgorPostingObligations.equals(_that.pledgorPostingObligations) : _that.pledgorPostingObligations != null) return false;
		if (pledgorRightsEvent != null ? !pledgorRightsEvent.equals(_that.pledgorRightsEvent) : _that.pledgorRightsEvent != null) return false;
		if (transferTiming != null ? !transferTiming.equals(_that.transferTiming) : _that.transferTiming != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = super.hashCode();
		_result = 31 * _result + (custodyArrangements != null ? custodyArrangements.hashCode() : 0);
		_result = 31 * _result + (deliveryInLieuRight != null ? deliveryInLieuRight.hashCode() : 0);
		_result = 31 * _result + (japaneseSecuritiesProvisions != null ? japaneseSecuritiesProvisions.hashCode() : 0);
		_result = 31 * _result + (otherEligibleAndPostedSupport != null ? otherEligibleAndPostedSupport.hashCode() : 0);
		_result = 31 * _result + (otherEligibleSupport != null ? otherEligibleSupport.hashCode() : 0);
		_result = 31 * _result + (pledgorAdditionalRightsEvent != null ? pledgorAdditionalRightsEvent.hashCode() : 0);
		_result = 31 * _result + (pledgorPostingObligations != null ? pledgorPostingObligations.hashCode() : 0);
		_result = 31 * _result + (pledgorRightsEvent != null ? pledgorRightsEvent.hashCode() : 0);
		_result = 31 * _result + (transferTiming != null ? transferTiming.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "CsaInitialMargin2016NewYorkLaw {" +
			"custodyArrangements=" + this.custodyArrangements + ", " +
			"deliveryInLieuRight=" + this.deliveryInLieuRight + ", " +
			"japaneseSecuritiesProvisions=" + this.japaneseSecuritiesProvisions + ", " +
			"otherEligibleAndPostedSupport=" + this.otherEligibleAndPostedSupport + ", " +
			"otherEligibleSupport=" + this.otherEligibleSupport + ", " +
			"pledgorAdditionalRightsEvent=" + this.pledgorAdditionalRightsEvent + ", " +
			"pledgorPostingObligations=" + this.pledgorPostingObligations + ", " +
			"pledgorRightsEvent=" + this.pledgorRightsEvent + ", " +
			"transferTiming=" + this.transferTiming +
		'}' + " " + super.toString();
	}
}
