package org.isda.cdm;

import java.lang.Boolean;
import java.lang.Integer;
import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.BusinessCenterTime;
import org.isda.cdm.CalculationDateLocation;
import org.isda.cdm.CreditSupportObligationsVariationMargin;
import org.isda.cdm.HoldingAndUsingPostedCollateral;
import org.isda.cdm.RelatedAgreement;
import org.isda.cdm.metafields.FieldWithMetaString;
import org.isda.cdm.metafields.ReferenceWithMetaParty;
import org.isda.cdm.metafields.ReferenceWithMetaParty.ReferenceWithMetaPartyBuilder;


import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.CsaVariationMargin2016Meta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;
		import java.util.stream.Collectors;
		import java.util.List;
		import java.util.ArrayList;
		import java.util.Objects;

/**
 * An abstract class to specify the provisions for the 2016 ISDA Credit Support Annex for Variation Margin that are common among governing laws.
 * 
 * @version 2.5.4
 */
@RosettaClass
public abstract class CsaVariationMargin2016 extends Csa2016 {
	private final CreditSupportObligationsVariationMargin creditSupportObligations;
	private final Boolean creditSupportOffsets;
	private final HoldingAndUsingPostedCollateral holdingAndUsingPostedCollateral;
	private final Integer notificationTime;
	private final RelatedAgreement otherCsa;
	private final ReferenceWithMetaParty valuationAgent;
	private final CalculationDateLocation valuationDateLocation;
	private final List<BusinessCenterTime> valuationTime;
	private static CsaVariationMargin2016Meta metaData = new CsaVariationMargin2016Meta();
	
	CsaVariationMargin2016(CsaVariationMargin2016Builder builder) {
		super(builder);
		this.creditSupportObligations = ofNullable(builder.getCreditSupportObligations()).map(f->f.build()).orElse(null);
		this.creditSupportOffsets = builder.getCreditSupportOffsets();
		this.holdingAndUsingPostedCollateral = ofNullable(builder.getHoldingAndUsingPostedCollateral()).map(f->f.build()).orElse(null);
		this.notificationTime = builder.getNotificationTime();
		this.otherCsa = ofNullable(builder.getOtherCsa()).map(f->f.build()).orElse(null);
		this.valuationAgent = ofNullable(builder.getValuationAgent()).map(f->f.build()).orElse(null);
		this.valuationDateLocation = ofNullable(builder.getValuationDateLocation()).map(f->f.build()).orElse(null);
		this.valuationTime = ofNullable(builder.getValuationTime()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
	}
	
	/**
	 * The Credit Support Obligations applicable to the Variation Margin Credit Support Annex. ISDA 2016 Credit Support Annex for Variation Margin, paragraph 13, (c): Credit Support Obligations.
	 */
	public final CreditSupportObligationsVariationMargin getCreditSupportObligations() {
		return creditSupportObligations;
	}
	
	/**
	 * When True the Credit Support Offsets provisions in Paragraph 11(j) of the ISDA 2016 Credit Support Annex for Variation Margin apply. They don&#39;t apply when False. ISDA 2016 Credit Support Annex for Variation Margin, paragraph 13, (j): Credit Support Offsets.
	 */
	public final Boolean getCreditSupportOffsets() {
		return creditSupportOffsets;
	}
	
	/**
	 * The elections for the holding and using of posted collateral by the respective parties to the Credit Support Annex for Variation Margin. ISDA 2016 Credit Support Annex for Variation Margin, paragraph 13, (h): Holding and Using Posted Collateral (VM).
	 */
	public final HoldingAndUsingPostedCollateral getHoldingAndUsingPostedCollateral() {
		return holdingAndUsingPostedCollateral;
	}
	
	/**
	 * The extend of time by which a demand for the Transfer of Eligible Credit Support (VM) or Posted Credit Support (VM) needs to be made after the relevant Valuation Time in order for the transfer to take place. This time is expressed in a number of minutes. ISDA 2016 Credit Support Annex for Variation Margin, paragraph 13, (d)(iv): Notification Time.
	 */
	public final Integer getNotificationTime() {
		return notificationTime;
	}
	
	/**
	 * The qualification of whether Other CSA are specified by the parties to the agreement. ISDA 2016 Credit Support Annex for Variation Margin, paragraph 13, (o): Other CSA.
	 */
	public final RelatedAgreement getOtherCsa() {
		return otherCsa;
	}
	
	/**
	 * The Valuation Agent as specified by the parties to the agreement. ISDA 2016 Credit Support Annex for Variation Margin, paragraph 13, (d)(i): Valuation Agent.
	 */
	public final ReferenceWithMetaParty getValuationAgent() {
		return valuationAgent;
	}
	
	/**
	 * The business location elected by the respective parties to the agreement as the Valuation Date Location. ISDA 2016 Credit Support Annex for Variation Margin, paragraph 13, (d)(ii): Valuation Date Location.
	 */
	public final CalculationDateLocation getValuationDateLocation() {
		return valuationDateLocation;
	}
	
	/**
	 * ISDA 2016 Credit Support Annex for Variation Margin, paragraph 13, (d)(iii): Valuation Time.
	 */
	public final List<BusinessCenterTime> getValuationTime() {
		return valuationTime;
	}
	
	
	@Override
	public RosettaMetaData<? extends CsaVariationMargin2016> metaData() {
		return metaData;
	} 
	
	public abstract CsaVariationMargin2016Builder toBuilder();


	public static abstract
	 class CsaVariationMargin2016Builder extends Csa2016Builder {
	
		protected CreditSupportObligationsVariationMargin.CreditSupportObligationsVariationMarginBuilder creditSupportObligations;
		protected Boolean creditSupportOffsets;
		protected HoldingAndUsingPostedCollateral.HoldingAndUsingPostedCollateralBuilder holdingAndUsingPostedCollateral;
		protected Integer notificationTime;
		protected RelatedAgreement.RelatedAgreementBuilder otherCsa;
		protected ReferenceWithMetaPartyBuilder valuationAgent;
		protected CalculationDateLocation.CalculationDateLocationBuilder valuationDateLocation;
		protected List<BusinessCenterTime.BusinessCenterTimeBuilder> valuationTime;
	
		public CsaVariationMargin2016Builder() {
		}
				
		@Override
		public RosettaMetaData<? extends CsaVariationMargin2016> metaData() {
			return metaData;
		} 
	
		
		public CreditSupportObligationsVariationMargin.CreditSupportObligationsVariationMarginBuilder getCreditSupportObligations() {
			return creditSupportObligations;
		}
		
		public CreditSupportObligationsVariationMargin.CreditSupportObligationsVariationMarginBuilder getOrCreateCreditSupportObligations() {
			if (creditSupportObligations!=null) {
				return creditSupportObligations;
			}
			else return creditSupportObligations = new CreditSupportObligationsVariationMargin.CreditSupportObligationsVariationMarginBuilder();
		}
		
		
		public Boolean getCreditSupportOffsets() {
			return creditSupportOffsets;
		}
		
		
		public HoldingAndUsingPostedCollateral.HoldingAndUsingPostedCollateralBuilder getHoldingAndUsingPostedCollateral() {
			return holdingAndUsingPostedCollateral;
		}
		
		public HoldingAndUsingPostedCollateral.HoldingAndUsingPostedCollateralBuilder getOrCreateHoldingAndUsingPostedCollateral() {
			if (holdingAndUsingPostedCollateral!=null) {
				return holdingAndUsingPostedCollateral;
			}
			else return holdingAndUsingPostedCollateral = new HoldingAndUsingPostedCollateral.HoldingAndUsingPostedCollateralBuilder();
		}
		
		
		public Integer getNotificationTime() {
			return notificationTime;
		}
		
		
		public RelatedAgreement.RelatedAgreementBuilder getOtherCsa() {
			return otherCsa;
		}
		
		public RelatedAgreement.RelatedAgreementBuilder getOrCreateOtherCsa() {
			if (otherCsa!=null) {
				return otherCsa;
			}
			else return otherCsa = new RelatedAgreement.RelatedAgreementBuilder();
		}
		
		
		public ReferenceWithMetaPartyBuilder getValuationAgent() {
			return valuationAgent;
		}
		
		public ReferenceWithMetaPartyBuilder getOrCreateValuationAgent() {
			if (valuationAgent!=null) {
				return valuationAgent;
			}
			else return valuationAgent = new ReferenceWithMetaPartyBuilder();
		}
		
		
		public CalculationDateLocation.CalculationDateLocationBuilder getValuationDateLocation() {
			return valuationDateLocation;
		}
		
		public CalculationDateLocation.CalculationDateLocationBuilder getOrCreateValuationDateLocation() {
			if (valuationDateLocation!=null) {
				return valuationDateLocation;
			}
			else return valuationDateLocation = new CalculationDateLocation.CalculationDateLocationBuilder();
		}
		
		
		public List<BusinessCenterTime.BusinessCenterTimeBuilder> getValuationTime() {
			return valuationTime;
		}
		
		public BusinessCenterTime.BusinessCenterTimeBuilder getOrCreateValuationTime(int index) {
			if (valuationTime==null) {
				this.valuationTime = new ArrayList<>();
			}
			return getIndex(valuationTime, index, ()->new BusinessCenterTime.BusinessCenterTimeBuilder());
		}
		
	
		public CsaVariationMargin2016Builder setCreditSupportObligations(CreditSupportObligationsVariationMargin creditSupportObligations) {
			this.creditSupportObligations = creditSupportObligations.toBuilder();
			return this;
		}
		public CsaVariationMargin2016Builder setCreditSupportObligationsBuilder(CreditSupportObligationsVariationMargin.CreditSupportObligationsVariationMarginBuilder creditSupportObligations) {
			this.creditSupportObligations = creditSupportObligations;
			return this;
		}
		public CsaVariationMargin2016Builder setCreditSupportOffsets(Boolean creditSupportOffsets) {
			this.creditSupportOffsets = creditSupportOffsets;
			return this;
		}
		public CsaVariationMargin2016Builder setHoldingAndUsingPostedCollateral(HoldingAndUsingPostedCollateral holdingAndUsingPostedCollateral) {
			this.holdingAndUsingPostedCollateral = holdingAndUsingPostedCollateral.toBuilder();
			return this;
		}
		public CsaVariationMargin2016Builder setHoldingAndUsingPostedCollateralBuilder(HoldingAndUsingPostedCollateral.HoldingAndUsingPostedCollateralBuilder holdingAndUsingPostedCollateral) {
			this.holdingAndUsingPostedCollateral = holdingAndUsingPostedCollateral;
			return this;
		}
		public CsaVariationMargin2016Builder setNotificationTime(Integer notificationTime) {
			this.notificationTime = notificationTime;
			return this;
		}
		public CsaVariationMargin2016Builder setOtherCsa(RelatedAgreement otherCsa) {
			this.otherCsa = otherCsa.toBuilder();
			return this;
		}
		public CsaVariationMargin2016Builder setOtherCsaBuilder(RelatedAgreement.RelatedAgreementBuilder otherCsa) {
			this.otherCsa = otherCsa;
			return this;
		}
		public CsaVariationMargin2016Builder setValuationAgent(ReferenceWithMetaParty valuationAgent) {
			this.valuationAgent = valuationAgent.toBuilder();
			return this;
		}
		public CsaVariationMargin2016Builder setValuationAgentBuilder(ReferenceWithMetaPartyBuilder valuationAgent) {
			this.valuationAgent = valuationAgent;
			return this;
		}
		public CsaVariationMargin2016Builder setValuationAgentRef(Party.PartyBuilder valuationAgent) {
			return setValuationAgent(ReferenceWithMetaParty.builder().setValueBuilder(valuationAgent).build());
		}
		public CsaVariationMargin2016Builder setValuationAgentRef(Party valuationAgent) {
			return setValuationAgentRef(Party.builder());
		}
		public CsaVariationMargin2016Builder setValuationDateLocation(CalculationDateLocation valuationDateLocation) {
			this.valuationDateLocation = valuationDateLocation.toBuilder();
			return this;
		}
		public CsaVariationMargin2016Builder setValuationDateLocationBuilder(CalculationDateLocation.CalculationDateLocationBuilder valuationDateLocation) {
			this.valuationDateLocation = valuationDateLocation;
			return this;
		}
		public CsaVariationMargin2016Builder addValuationTime(BusinessCenterTime valuationTime) {
			if(this.valuationTime == null){
				this.valuationTime = new ArrayList<>();
			}
			this.valuationTime.add(valuationTime.toBuilder());
			return this;
		}
		public CsaVariationMargin2016Builder addValuationTime(BusinessCenterTime valuationTime, int _idx) {
			if(this.valuationTime == null){
				this.valuationTime = new ArrayList<>();
			}
			getIndex(this.valuationTime, _idx, () -> valuationTime.toBuilder());
			this.valuationTime.set(_idx, valuationTime.toBuilder());
			return this;
		}
		public CsaVariationMargin2016Builder addValuationTime(List<BusinessCenterTime> valuationTimes) {
			if(this.valuationTime == null){
				this.valuationTime = new ArrayList<>();
			}
			for (BusinessCenterTime toAdd : valuationTimes) {
				this.valuationTime.add(toAdd.toBuilder());
			}
			return this;
		}
		public CsaVariationMargin2016Builder addValuationTimeBuilder(BusinessCenterTime.BusinessCenterTimeBuilder valuationTime) {
			if(this.valuationTime == null){
				this.valuationTime = new ArrayList<>();
			}
			this.valuationTime.add(valuationTime);
			return this;
		}
		
		public CsaVariationMargin2016Builder clearValuationTime() {
			this.valuationTime = null;
			return this;
		}
		@Override public CsaVariationMargin2016Builder setAdditionalObligations(String additionalObligations) {
			this.additionalObligations = additionalObligations;
			return this;
		}
		@Override public CsaVariationMargin2016Builder setAdditionalRepresentation(AdditionalRepresentation additionalRepresentation) {
			this.additionalRepresentation = additionalRepresentation.toBuilder();
			return this;
		}
		@Override public CsaVariationMargin2016Builder setAdditionalRepresentationBuilder(AdditionalRepresentation.AdditionalRepresentationBuilder additionalRepresentation) {
			this.additionalRepresentation = additionalRepresentation;
			return this;
		}
		@Override public CsaVariationMargin2016Builder setAddressesForTransfer(ContactElection addressesForTransfer) {
			this.addressesForTransfer = addressesForTransfer.toBuilder();
			return this;
		}
		@Override public CsaVariationMargin2016Builder setAddressesForTransferBuilder(ContactElection.ContactElectionBuilder addressesForTransfer) {
			this.addressesForTransfer = addressesForTransfer;
			return this;
		}
		@Override public CsaVariationMargin2016Builder setBaseCurrency(FieldWithMetaString baseCurrency) {
			this.baseCurrency = baseCurrency.toBuilder();
			return this;
		}
		@Override public CsaVariationMargin2016Builder setBaseCurrencyRef(String baseCurrency) {
			return setBaseCurrency(FieldWithMetaString.builder().setValue(baseCurrency).build());
		}
		@Override public CsaVariationMargin2016Builder setBespokeProvision(String bespokeProvision) {
			this.bespokeProvision = bespokeProvision;
			return this;
		}
		@Override public CsaVariationMargin2016Builder setConditionsPrecedent(ConditionsPrecedent conditionsPrecedent) {
			this.conditionsPrecedent = conditionsPrecedent.toBuilder();
			return this;
		}
		@Override public CsaVariationMargin2016Builder setConditionsPrecedentBuilder(ConditionsPrecedent.ConditionsPrecedentBuilder conditionsPrecedent) {
			this.conditionsPrecedent = conditionsPrecedent;
			return this;
		}
		@Override public CsaVariationMargin2016Builder setDemandsAndNotices(ContactElection demandsAndNotices) {
			this.demandsAndNotices = demandsAndNotices.toBuilder();
			return this;
		}
		@Override public CsaVariationMargin2016Builder setDemandsAndNoticesBuilder(ContactElection.ContactElectionBuilder demandsAndNotices) {
			this.demandsAndNotices = demandsAndNotices;
			return this;
		}
		@Override public CsaVariationMargin2016Builder setDisputeResolution(DisputeResolution disputeResolution) {
			this.disputeResolution = disputeResolution.toBuilder();
			return this;
		}
		@Override public CsaVariationMargin2016Builder setDisputeResolutionBuilder(DisputeResolution.DisputeResolutionBuilder disputeResolution) {
			this.disputeResolution = disputeResolution;
			return this;
		}
		@Override public CsaVariationMargin2016Builder setSubstitution(Substitution substitution) {
			this.substitution = substitution.toBuilder();
			return this;
		}
		@Override public CsaVariationMargin2016Builder setSubstitutionBuilder(Substitution.SubstitutionBuilder substitution) {
			this.substitution = substitution;
			return this;
		}
		
		public abstract CsaVariationMargin2016 build();
	
		@Override
		public CsaVariationMargin2016Builder prune() {
			super.prune();
			if (creditSupportObligations!=null && !creditSupportObligations.prune().hasData()) creditSupportObligations = null;
			if (holdingAndUsingPostedCollateral!=null && !holdingAndUsingPostedCollateral.prune().hasData()) holdingAndUsingPostedCollateral = null;
			if (otherCsa!=null && !otherCsa.prune().hasData()) otherCsa = null;
			if (valuationAgent!=null && !valuationAgent.prune().hasData()) valuationAgent = null;
			if (valuationDateLocation!=null && !valuationDateLocation.prune().hasData()) valuationDateLocation = null;
			if (valuationTime!=null) valuationTime = valuationTime.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (super.hasData()) return true;
			if (getCreditSupportObligations()!=null && getCreditSupportObligations().hasData()) return true;
			if (getCreditSupportOffsets()!=null) return true;
			if (getHoldingAndUsingPostedCollateral()!=null && getHoldingAndUsingPostedCollateral().hasData()) return true;
			if (getNotificationTime()!=null) return true;
			if (getOtherCsa()!=null && getOtherCsa().hasData()) return true;
			if (getValuationAgent()!=null && getValuationAgent().hasData()) return true;
			if (getValuationDateLocation()!=null && getValuationDateLocation().hasData()) return true;
			if (getValuationTime()!=null && getValuationTime().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
			super.process(path, processor);
		
			processor.processBasic(path.newSubPath("creditSupportOffsets"), Boolean.class, creditSupportOffsets, this);
			processor.processBasic(path.newSubPath("notificationTime"), Integer.class, notificationTime, this);
			
			processRosetta(path.newSubPath("creditSupportObligations"), processor, CreditSupportObligationsVariationMargin.class, creditSupportObligations);
			processRosetta(path.newSubPath("holdingAndUsingPostedCollateral"), processor, HoldingAndUsingPostedCollateral.class, holdingAndUsingPostedCollateral);
			processRosetta(path.newSubPath("otherCsa"), processor, RelatedAgreement.class, otherCsa);
			processRosetta(path.newSubPath("valuationAgent"), processor, ReferenceWithMetaParty.class, valuationAgent);
			processRosetta(path.newSubPath("valuationDateLocation"), processor, CalculationDateLocation.class, valuationDateLocation);
			processRosetta(path.newSubPath("valuationTime"), processor, BusinessCenterTime.class, valuationTime);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
			if (!super.equals(o)) return false;
		
			CsaVariationMargin2016Builder _that = (CsaVariationMargin2016Builder) o;
		
			if (creditSupportObligations != null ? !creditSupportObligations.equals(_that.creditSupportObligations) : _that.creditSupportObligations != null) return false;
			if (creditSupportOffsets != null ? !creditSupportOffsets.equals(_that.creditSupportOffsets) : _that.creditSupportOffsets != null) return false;
			if (holdingAndUsingPostedCollateral != null ? !holdingAndUsingPostedCollateral.equals(_that.holdingAndUsingPostedCollateral) : _that.holdingAndUsingPostedCollateral != null) return false;
			if (notificationTime != null ? !notificationTime.equals(_that.notificationTime) : _that.notificationTime != null) return false;
			if (otherCsa != null ? !otherCsa.equals(_that.otherCsa) : _that.otherCsa != null) return false;
			if (valuationAgent != null ? !valuationAgent.equals(_that.valuationAgent) : _that.valuationAgent != null) return false;
			if (valuationDateLocation != null ? !valuationDateLocation.equals(_that.valuationDateLocation) : _that.valuationDateLocation != null) return false;
			if (!ListEquals.listEquals(valuationTime, _that.valuationTime)) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = super.hashCode();
			_result = 31 * _result + (creditSupportObligations != null ? creditSupportObligations.hashCode() : 0);
			_result = 31 * _result + (creditSupportOffsets != null ? creditSupportOffsets.hashCode() : 0);
			_result = 31 * _result + (holdingAndUsingPostedCollateral != null ? holdingAndUsingPostedCollateral.hashCode() : 0);
			_result = 31 * _result + (notificationTime != null ? notificationTime.hashCode() : 0);
			_result = 31 * _result + (otherCsa != null ? otherCsa.hashCode() : 0);
			_result = 31 * _result + (valuationAgent != null ? valuationAgent.hashCode() : 0);
			_result = 31 * _result + (valuationDateLocation != null ? valuationDateLocation.hashCode() : 0);
			_result = 31 * _result + (valuationTime != null ? valuationTime.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "CsaVariationMargin2016Builder {" +
				"creditSupportObligations=" + this.creditSupportObligations + ", " +
				"creditSupportOffsets=" + this.creditSupportOffsets + ", " +
				"holdingAndUsingPostedCollateral=" + this.holdingAndUsingPostedCollateral + ", " +
				"notificationTime=" + this.notificationTime + ", " +
				"otherCsa=" + this.otherCsa + ", " +
				"valuationAgent=" + this.valuationAgent + ", " +
				"valuationDateLocation=" + this.valuationDateLocation + ", " +
				"valuationTime=" + this.valuationTime +
			'}' + " " + super.toString();
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		super.process(path, processor);
		
		processor.processBasic(path.newSubPath("creditSupportOffsets"), Boolean.class, creditSupportOffsets, this);
		processor.processBasic(path.newSubPath("notificationTime"), Integer.class, notificationTime, this);
		
		processRosetta(path.newSubPath("creditSupportObligations"), processor, CreditSupportObligationsVariationMargin.class, creditSupportObligations);
		processRosetta(path.newSubPath("holdingAndUsingPostedCollateral"), processor, HoldingAndUsingPostedCollateral.class, holdingAndUsingPostedCollateral);
		processRosetta(path.newSubPath("otherCsa"), processor, RelatedAgreement.class, otherCsa);
		processRosetta(path.newSubPath("valuationAgent"), processor, ReferenceWithMetaParty.class, valuationAgent);
		processRosetta(path.newSubPath("valuationDateLocation"), processor, CalculationDateLocation.class, valuationDateLocation);
		processRosetta(path.newSubPath("valuationTime"), processor, BusinessCenterTime.class, valuationTime);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
		if (!super.equals(o)) return false;
	
		CsaVariationMargin2016 _that = (CsaVariationMargin2016) o;
	
		if (creditSupportObligations != null ? !creditSupportObligations.equals(_that.creditSupportObligations) : _that.creditSupportObligations != null) return false;
		if (creditSupportOffsets != null ? !creditSupportOffsets.equals(_that.creditSupportOffsets) : _that.creditSupportOffsets != null) return false;
		if (holdingAndUsingPostedCollateral != null ? !holdingAndUsingPostedCollateral.equals(_that.holdingAndUsingPostedCollateral) : _that.holdingAndUsingPostedCollateral != null) return false;
		if (notificationTime != null ? !notificationTime.equals(_that.notificationTime) : _that.notificationTime != null) return false;
		if (otherCsa != null ? !otherCsa.equals(_that.otherCsa) : _that.otherCsa != null) return false;
		if (valuationAgent != null ? !valuationAgent.equals(_that.valuationAgent) : _that.valuationAgent != null) return false;
		if (valuationDateLocation != null ? !valuationDateLocation.equals(_that.valuationDateLocation) : _that.valuationDateLocation != null) return false;
		if (!ListEquals.listEquals(valuationTime, _that.valuationTime)) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = super.hashCode();
		_result = 31 * _result + (creditSupportObligations != null ? creditSupportObligations.hashCode() : 0);
		_result = 31 * _result + (creditSupportOffsets != null ? creditSupportOffsets.hashCode() : 0);
		_result = 31 * _result + (holdingAndUsingPostedCollateral != null ? holdingAndUsingPostedCollateral.hashCode() : 0);
		_result = 31 * _result + (notificationTime != null ? notificationTime.hashCode() : 0);
		_result = 31 * _result + (otherCsa != null ? otherCsa.hashCode() : 0);
		_result = 31 * _result + (valuationAgent != null ? valuationAgent.hashCode() : 0);
		_result = 31 * _result + (valuationDateLocation != null ? valuationDateLocation.hashCode() : 0);
		_result = 31 * _result + (valuationTime != null ? valuationTime.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "CsaVariationMargin2016 {" +
			"creditSupportObligations=" + this.creditSupportObligations + ", " +
			"creditSupportOffsets=" + this.creditSupportOffsets + ", " +
			"holdingAndUsingPostedCollateral=" + this.holdingAndUsingPostedCollateral + ", " +
			"notificationTime=" + this.notificationTime + ", " +
			"otherCsa=" + this.otherCsa + ", " +
			"valuationAgent=" + this.valuationAgent + ", " +
			"valuationDateLocation=" + this.valuationDateLocation + ", " +
			"valuationTime=" + this.valuationTime +
		'}' + " " + super.toString();
	}
}
