package org.isda.cdm;

import java.lang.String;
import org.isda.cdm.OtherEligibleAndPostedSupport;
import org.isda.cdm.metafields.FieldWithMetaString;
import org.isda.cdm.metafields.ReferenceWithMetaParty;
import org.isda.cdm.metafields.ReferenceWithMetaParty.ReferenceWithMetaPartyBuilder;


import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.CsaVariationMargin2016NewYorkLawMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;
		import java.util.stream.Collectors;
		import java.util.List;
		import java.util.ArrayList;
		import java.util.Objects;

/**
 * A class to specify the provisions that are specific to the New York Law version of the ISDA 2016 Credit Support Annex for Variation Margin.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class CsaVariationMargin2016NewYorkLaw extends CsaVariationMargin2016 {
	private final OtherEligibleAndPostedSupport otherEligibleAndPostedSupport;
	private final String otherEligibleSupport;
	private final String transferTiming;
	private static CsaVariationMargin2016NewYorkLawMeta metaData = new CsaVariationMargin2016NewYorkLawMeta();
	
	CsaVariationMargin2016NewYorkLaw(CsaVariationMargin2016NewYorkLawBuilder builder) {
		super(builder);
		this.otherEligibleAndPostedSupport = ofNullable(builder.getOtherEligibleAndPostedSupport()).map(f->f.build()).orElse(null);
		this.otherEligibleSupport = builder.getOtherEligibleSupport();
		this.transferTiming = builder.getTransferTiming();
	}
	
	/**
	 * The Other Eligible Support elections associated with margin agreements. ISDA 2016 New York Law Credit Support Annex for Variation Margin, paragraph 13, (l): Other Eligible Support (VM) and Other Posted Support (VM).
	 */
	public final OtherEligibleAndPostedSupport getOtherEligibleAndPostedSupport() {
		return otherEligibleAndPostedSupport;
	}
	
	/**
	 * The Other Eligible Support election. If not specified, this election is deemed as not applicable. ISDA 2016 New York Law Credit Support Annex for Variation Margin, paragraph 13, (c)(vi): Other Eligible Support (VM).
	 */
	public final String getOtherEligibleSupport() {
		return otherEligibleSupport;
	}
	
	/**
	 * The time by which the transfer of collateral must take place after the Notification Time on the Regular Settlement Day. ISDA 2016 New York Law Credit Support Annex for Variation Margin, paragraph 13, (c)(viii): Transfer Timing.
	 */
	public final String getTransferTiming() {
		return transferTiming;
	}
	
	
	@Override
	public RosettaMetaData<? extends CsaVariationMargin2016NewYorkLaw> metaData() {
		return metaData;
	} 
	
	public CsaVariationMargin2016NewYorkLawBuilder toBuilder() {
		CsaVariationMargin2016NewYorkLawBuilder builder = new CsaVariationMargin2016NewYorkLawBuilder();
		ofNullable(getOtherEligibleAndPostedSupport()).ifPresent(builder::setOtherEligibleAndPostedSupport);
		ofNullable(getOtherEligibleSupport()).ifPresent(builder::setOtherEligibleSupport);
		ofNullable(getTransferTiming()).ifPresent(builder::setTransferTiming);
		ofNullable(getCreditSupportObligations()).ifPresent(builder::setCreditSupportObligations);
		ofNullable(getCreditSupportOffsets()).ifPresent(builder::setCreditSupportOffsets);
		ofNullable(getHoldingAndUsingPostedCollateral()).ifPresent(builder::setHoldingAndUsingPostedCollateral);
		ofNullable(getNotificationTime()).ifPresent(builder::setNotificationTime);
		ofNullable(getOtherCsa()).ifPresent(builder::setOtherCsa);
		ofNullable(getValuationAgent()).ifPresent(builder::setValuationAgent);
		ofNullable(getValuationDateLocation()).ifPresent(builder::setValuationDateLocation);
		ofNullable(getValuationTime()).ifPresent(valuationTime -> valuationTime.forEach(builder::addValuationTime));
		ofNullable(getAdditionalObligations()).ifPresent(builder::setAdditionalObligations);
		ofNullable(getAdditionalRepresentation()).ifPresent(builder::setAdditionalRepresentation);
		ofNullable(getAddressesForTransfer()).ifPresent(builder::setAddressesForTransfer);
		ofNullable(getBaseCurrency()).ifPresent(builder::setBaseCurrency);
		ofNullable(getBespokeProvision()).ifPresent(builder::setBespokeProvision);
		ofNullable(getConditionsPrecedent()).ifPresent(builder::setConditionsPrecedent);
		ofNullable(getDemandsAndNotices()).ifPresent(builder::setDemandsAndNotices);
		ofNullable(getDisputeResolution()).ifPresent(builder::setDisputeResolution);
		ofNullable(getSubstitution()).ifPresent(builder::setSubstitution);
		return builder;
	}

	public static CsaVariationMargin2016NewYorkLawBuilder builder() {
		return new CsaVariationMargin2016NewYorkLawBuilder();
	}

	public static  class CsaVariationMargin2016NewYorkLawBuilder extends CsaVariationMargin2016Builder {
	
		protected OtherEligibleAndPostedSupport.OtherEligibleAndPostedSupportBuilder otherEligibleAndPostedSupport;
		protected String otherEligibleSupport;
		protected String transferTiming;
	
		public CsaVariationMargin2016NewYorkLawBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends CsaVariationMargin2016NewYorkLaw> metaData() {
			return metaData;
		} 
	
		
		public OtherEligibleAndPostedSupport.OtherEligibleAndPostedSupportBuilder getOtherEligibleAndPostedSupport() {
			return otherEligibleAndPostedSupport;
		}
		
		public OtherEligibleAndPostedSupport.OtherEligibleAndPostedSupportBuilder getOrCreateOtherEligibleAndPostedSupport() {
			if (otherEligibleAndPostedSupport!=null) {
				return otherEligibleAndPostedSupport;
			}
			else return otherEligibleAndPostedSupport = new OtherEligibleAndPostedSupport.OtherEligibleAndPostedSupportBuilder();
		}
		
		
		public String getOtherEligibleSupport() {
			return otherEligibleSupport;
		}
		
		
		public String getTransferTiming() {
			return transferTiming;
		}
		
	
		public CsaVariationMargin2016NewYorkLawBuilder setOtherEligibleAndPostedSupport(OtherEligibleAndPostedSupport otherEligibleAndPostedSupport) {
			this.otherEligibleAndPostedSupport = otherEligibleAndPostedSupport.toBuilder();
			return this;
		}
		public CsaVariationMargin2016NewYorkLawBuilder setOtherEligibleAndPostedSupportBuilder(OtherEligibleAndPostedSupport.OtherEligibleAndPostedSupportBuilder otherEligibleAndPostedSupport) {
			this.otherEligibleAndPostedSupport = otherEligibleAndPostedSupport;
			return this;
		}
		public CsaVariationMargin2016NewYorkLawBuilder setOtherEligibleSupport(String otherEligibleSupport) {
			this.otherEligibleSupport = otherEligibleSupport;
			return this;
		}
		public CsaVariationMargin2016NewYorkLawBuilder setTransferTiming(String transferTiming) {
			this.transferTiming = transferTiming;
			return this;
		}
		@Override public CsaVariationMargin2016NewYorkLawBuilder setCreditSupportObligations(CreditSupportObligationsVariationMargin creditSupportObligations) {
			this.creditSupportObligations = creditSupportObligations.toBuilder();
			return this;
		}
		@Override public CsaVariationMargin2016NewYorkLawBuilder setCreditSupportObligationsBuilder(CreditSupportObligationsVariationMargin.CreditSupportObligationsVariationMarginBuilder creditSupportObligations) {
			this.creditSupportObligations = creditSupportObligations;
			return this;
		}
		@Override public CsaVariationMargin2016NewYorkLawBuilder setCreditSupportOffsets(Boolean creditSupportOffsets) {
			this.creditSupportOffsets = creditSupportOffsets;
			return this;
		}
		@Override public CsaVariationMargin2016NewYorkLawBuilder setHoldingAndUsingPostedCollateral(HoldingAndUsingPostedCollateral holdingAndUsingPostedCollateral) {
			this.holdingAndUsingPostedCollateral = holdingAndUsingPostedCollateral.toBuilder();
			return this;
		}
		@Override public CsaVariationMargin2016NewYorkLawBuilder setHoldingAndUsingPostedCollateralBuilder(HoldingAndUsingPostedCollateral.HoldingAndUsingPostedCollateralBuilder holdingAndUsingPostedCollateral) {
			this.holdingAndUsingPostedCollateral = holdingAndUsingPostedCollateral;
			return this;
		}
		@Override public CsaVariationMargin2016NewYorkLawBuilder setNotificationTime(Integer notificationTime) {
			this.notificationTime = notificationTime;
			return this;
		}
		@Override public CsaVariationMargin2016NewYorkLawBuilder setOtherCsa(RelatedAgreement otherCsa) {
			this.otherCsa = otherCsa.toBuilder();
			return this;
		}
		@Override public CsaVariationMargin2016NewYorkLawBuilder setOtherCsaBuilder(RelatedAgreement.RelatedAgreementBuilder otherCsa) {
			this.otherCsa = otherCsa;
			return this;
		}
		@Override public CsaVariationMargin2016NewYorkLawBuilder setValuationAgent(ReferenceWithMetaParty valuationAgent) {
			this.valuationAgent = valuationAgent.toBuilder();
			return this;
		}
		@Override public CsaVariationMargin2016NewYorkLawBuilder setValuationAgentBuilder(ReferenceWithMetaPartyBuilder valuationAgent) {
			this.valuationAgent = valuationAgent;
			return this;
		}
		@Override public CsaVariationMargin2016NewYorkLawBuilder setValuationAgentRef(Party.PartyBuilder valuationAgent) {
			return setValuationAgent(ReferenceWithMetaParty.builder().setValueBuilder(valuationAgent).build());
		}
		@Override public CsaVariationMargin2016NewYorkLawBuilder setValuationAgentRef(Party valuationAgent) {
			return setValuationAgentRef(Party.builder());
		}
		@Override public CsaVariationMargin2016NewYorkLawBuilder setValuationDateLocation(CalculationDateLocation valuationDateLocation) {
			this.valuationDateLocation = valuationDateLocation.toBuilder();
			return this;
		}
		@Override public CsaVariationMargin2016NewYorkLawBuilder setValuationDateLocationBuilder(CalculationDateLocation.CalculationDateLocationBuilder valuationDateLocation) {
			this.valuationDateLocation = valuationDateLocation;
			return this;
		}
		@Override public CsaVariationMargin2016NewYorkLawBuilder addValuationTime(BusinessCenterTime valuationTime) {
			if(this.valuationTime == null){
				this.valuationTime = new ArrayList<>();
			}
			this.valuationTime.add(valuationTime.toBuilder());
			return this;
		}
		@Override public CsaVariationMargin2016NewYorkLawBuilder addValuationTime(BusinessCenterTime valuationTime, int _idx) {
			if(this.valuationTime == null){
				this.valuationTime = new ArrayList<>();
			}
			getIndex(this.valuationTime, _idx, () -> valuationTime.toBuilder());
			this.valuationTime.set(_idx, valuationTime.toBuilder());
			return this;
		}
		@Override public CsaVariationMargin2016NewYorkLawBuilder addValuationTime(List<BusinessCenterTime> valuationTimes) {
			if(this.valuationTime == null){
				this.valuationTime = new ArrayList<>();
			}
			for (BusinessCenterTime toAdd : valuationTimes) {
				this.valuationTime.add(toAdd.toBuilder());
			}
			return this;
		}
		@Override public CsaVariationMargin2016NewYorkLawBuilder addValuationTimeBuilder(BusinessCenterTime.BusinessCenterTimeBuilder valuationTime) {
			if(this.valuationTime == null){
				this.valuationTime = new ArrayList<>();
			}
			this.valuationTime.add(valuationTime);
			return this;
		}
		
		@Override public CsaVariationMargin2016NewYorkLawBuilder clearValuationTime() {
			this.valuationTime = null;
			return this;
		}
		@Override public CsaVariationMargin2016NewYorkLawBuilder setAdditionalObligations(String additionalObligations) {
			this.additionalObligations = additionalObligations;
			return this;
		}
		@Override public CsaVariationMargin2016NewYorkLawBuilder setAdditionalRepresentation(AdditionalRepresentation additionalRepresentation) {
			this.additionalRepresentation = additionalRepresentation.toBuilder();
			return this;
		}
		@Override public CsaVariationMargin2016NewYorkLawBuilder setAdditionalRepresentationBuilder(AdditionalRepresentation.AdditionalRepresentationBuilder additionalRepresentation) {
			this.additionalRepresentation = additionalRepresentation;
			return this;
		}
		@Override public CsaVariationMargin2016NewYorkLawBuilder setAddressesForTransfer(ContactElection addressesForTransfer) {
			this.addressesForTransfer = addressesForTransfer.toBuilder();
			return this;
		}
		@Override public CsaVariationMargin2016NewYorkLawBuilder setAddressesForTransferBuilder(ContactElection.ContactElectionBuilder addressesForTransfer) {
			this.addressesForTransfer = addressesForTransfer;
			return this;
		}
		@Override public CsaVariationMargin2016NewYorkLawBuilder setBaseCurrency(FieldWithMetaString baseCurrency) {
			this.baseCurrency = baseCurrency.toBuilder();
			return this;
		}
		@Override public CsaVariationMargin2016NewYorkLawBuilder setBaseCurrencyRef(String baseCurrency) {
			return setBaseCurrency(FieldWithMetaString.builder().setValue(baseCurrency).build());
		}
		@Override public CsaVariationMargin2016NewYorkLawBuilder setBespokeProvision(String bespokeProvision) {
			this.bespokeProvision = bespokeProvision;
			return this;
		}
		@Override public CsaVariationMargin2016NewYorkLawBuilder setConditionsPrecedent(ConditionsPrecedent conditionsPrecedent) {
			this.conditionsPrecedent = conditionsPrecedent.toBuilder();
			return this;
		}
		@Override public CsaVariationMargin2016NewYorkLawBuilder setConditionsPrecedentBuilder(ConditionsPrecedent.ConditionsPrecedentBuilder conditionsPrecedent) {
			this.conditionsPrecedent = conditionsPrecedent;
			return this;
		}
		@Override public CsaVariationMargin2016NewYorkLawBuilder setDemandsAndNotices(ContactElection demandsAndNotices) {
			this.demandsAndNotices = demandsAndNotices.toBuilder();
			return this;
		}
		@Override public CsaVariationMargin2016NewYorkLawBuilder setDemandsAndNoticesBuilder(ContactElection.ContactElectionBuilder demandsAndNotices) {
			this.demandsAndNotices = demandsAndNotices;
			return this;
		}
		@Override public CsaVariationMargin2016NewYorkLawBuilder setDisputeResolution(DisputeResolution disputeResolution) {
			this.disputeResolution = disputeResolution.toBuilder();
			return this;
		}
		@Override public CsaVariationMargin2016NewYorkLawBuilder setDisputeResolutionBuilder(DisputeResolution.DisputeResolutionBuilder disputeResolution) {
			this.disputeResolution = disputeResolution;
			return this;
		}
		@Override public CsaVariationMargin2016NewYorkLawBuilder setSubstitution(Substitution substitution) {
			this.substitution = substitution.toBuilder();
			return this;
		}
		@Override public CsaVariationMargin2016NewYorkLawBuilder setSubstitutionBuilder(Substitution.SubstitutionBuilder substitution) {
			this.substitution = substitution;
			return this;
		}
		
		public CsaVariationMargin2016NewYorkLaw build() {
			return new CsaVariationMargin2016NewYorkLaw(this);
		}
	
		@Override
		public CsaVariationMargin2016NewYorkLawBuilder prune() {
			super.prune();
			if (otherEligibleAndPostedSupport!=null && !otherEligibleAndPostedSupport.prune().hasData()) otherEligibleAndPostedSupport = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (super.hasData()) return true;
			if (getOtherEligibleAndPostedSupport()!=null && getOtherEligibleAndPostedSupport().hasData()) return true;
			if (getOtherEligibleSupport()!=null) return true;
			if (getTransferTiming()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
			super.process(path, processor);
		
			processor.processBasic(path.newSubPath("otherEligibleSupport"), String.class, otherEligibleSupport, this);
			processor.processBasic(path.newSubPath("transferTiming"), String.class, transferTiming, this);
			
			processRosetta(path.newSubPath("otherEligibleAndPostedSupport"), processor, OtherEligibleAndPostedSupport.class, otherEligibleAndPostedSupport);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
			if (!super.equals(o)) return false;
		
			CsaVariationMargin2016NewYorkLawBuilder _that = (CsaVariationMargin2016NewYorkLawBuilder) o;
		
			if (otherEligibleAndPostedSupport != null ? !otherEligibleAndPostedSupport.equals(_that.otherEligibleAndPostedSupport) : _that.otherEligibleAndPostedSupport != null) return false;
			if (otherEligibleSupport != null ? !otherEligibleSupport.equals(_that.otherEligibleSupport) : _that.otherEligibleSupport != null) return false;
			if (transferTiming != null ? !transferTiming.equals(_that.transferTiming) : _that.transferTiming != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = super.hashCode();
			_result = 31 * _result + (otherEligibleAndPostedSupport != null ? otherEligibleAndPostedSupport.hashCode() : 0);
			_result = 31 * _result + (otherEligibleSupport != null ? otherEligibleSupport.hashCode() : 0);
			_result = 31 * _result + (transferTiming != null ? transferTiming.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "CsaVariationMargin2016NewYorkLawBuilder {" +
				"otherEligibleAndPostedSupport=" + this.otherEligibleAndPostedSupport + ", " +
				"otherEligibleSupport=" + this.otherEligibleSupport + ", " +
				"transferTiming=" + this.transferTiming +
			'}' + " " + super.toString();
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		super.process(path, processor);
		
		processor.processBasic(path.newSubPath("otherEligibleSupport"), String.class, otherEligibleSupport, this);
		processor.processBasic(path.newSubPath("transferTiming"), String.class, transferTiming, this);
		
		processRosetta(path.newSubPath("otherEligibleAndPostedSupport"), processor, OtherEligibleAndPostedSupport.class, otherEligibleAndPostedSupport);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
		if (!super.equals(o)) return false;
	
		CsaVariationMargin2016NewYorkLaw _that = (CsaVariationMargin2016NewYorkLaw) o;
	
		if (otherEligibleAndPostedSupport != null ? !otherEligibleAndPostedSupport.equals(_that.otherEligibleAndPostedSupport) : _that.otherEligibleAndPostedSupport != null) return false;
		if (otherEligibleSupport != null ? !otherEligibleSupport.equals(_that.otherEligibleSupport) : _that.otherEligibleSupport != null) return false;
		if (transferTiming != null ? !transferTiming.equals(_that.transferTiming) : _that.transferTiming != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = super.hashCode();
		_result = 31 * _result + (otherEligibleAndPostedSupport != null ? otherEligibleAndPostedSupport.hashCode() : 0);
		_result = 31 * _result + (otherEligibleSupport != null ? otherEligibleSupport.hashCode() : 0);
		_result = 31 * _result + (transferTiming != null ? transferTiming.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "CsaVariationMargin2016NewYorkLaw {" +
			"otherEligibleAndPostedSupport=" + this.otherEligibleAndPostedSupport + ", " +
			"otherEligibleSupport=" + this.otherEligibleSupport + ", " +
			"transferTiming=" + this.transferTiming +
		'}' + " " + super.toString();
	}
}
