package org.isda.cdm;

import java.lang.Boolean;
import java.lang.String;
import org.isda.cdm.AdditionalRightsEvent;
import org.isda.cdm.AppropriatedCollateralValuation;
import org.isda.cdm.ChargorPostingObligations;
import org.isda.cdm.CustodyArrangements;
import org.isda.cdm.ProcessAgent;
import org.isda.cdm.RightsEvent;
import org.isda.cdm.metafields.FieldWithMetaString;


import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.CsdInitialMargin2016EnglishLawMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class to specify the provisions that are specific to the English Law version of the ISDA 2016 Credit Support Deed for Initial Margin.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class CsdInitialMargin2016EnglishLaw extends CsaInitialMargin2016 {
	private final AppropriatedCollateralValuation appropriatedCollateralValuation;
	private final AdditionalRightsEvent chargorAdditionalRightsEvent;
	private final ChargorPostingObligations chargorPostingObligations;
	private final RightsEvent chargorRightsEvent;
	private final CustodyArrangements custodyArrangements;
	private final Boolean deliveryInLieuRight;
	private final Boolean japaneseSecuritiesProvisions;
	private final ProcessAgent processAgent;
	private final String transferTiming;
	private static CsdInitialMargin2016EnglishLawMeta metaData = new CsdInitialMargin2016EnglishLawMeta();
	
	CsdInitialMargin2016EnglishLaw(CsdInitialMargin2016EnglishLawBuilder builder) {
		super(builder);
		this.appropriatedCollateralValuation = ofNullable(builder.getAppropriatedCollateralValuation()).map(f->f.build()).orElse(null);
		this.chargorAdditionalRightsEvent = ofNullable(builder.getChargorAdditionalRightsEvent()).map(f->f.build()).orElse(null);
		this.chargorPostingObligations = ofNullable(builder.getChargorPostingObligations()).map(f->f.build()).orElse(null);
		this.chargorRightsEvent = ofNullable(builder.getChargorRightsEvent()).map(f->f.build()).orElse(null);
		this.custodyArrangements = ofNullable(builder.getCustodyArrangements()).map(f->f.build()).orElse(null);
		this.deliveryInLieuRight = builder.getDeliveryInLieuRight();
		this.japaneseSecuritiesProvisions = builder.getJapaneseSecuritiesProvisions();
		this.processAgent = ofNullable(builder.getProcessAgent()).map(f->f.build()).orElse(null);
		this.transferTiming = builder.getTransferTiming();
	}
	
	/**
	 * The election for the Valuation of Appropriate Collateral. ISDA 2016 English Law Credit Support Deed for Initial Margin, paragraph 13, General Principles, (u): Valuation of Appropriated Collateral.
	 */
	@RosettaSynonym(value="valuation_of_appropriated_collateral", source="ISDA_Create_1_0")
	public final AppropriatedCollateralValuation getAppropriatedCollateralValuation() {
		return appropriatedCollateralValuation;
	}
	
	/**
	 * The Chargor Additional Rights Event election. ISDA 2016 English Law Credit Support Deed for Initial Margin, paragraph 13, General Principles, (k): Chargor Additional Rights Event.
	 */
	@RosettaSynonym(value="chargor_additional_rights_event", source="ISDA_Create_1_0")
	public final AdditionalRightsEvent getChargorAdditionalRightsEvent() {
		return chargorAdditionalRightsEvent;
	}
	
	/**
	 * The chargor&#39;s posting obligations. This term of chargor is the equivalent of the pledgor in the New York Law CSA and the obligor in the Japanese Law CSA, i.e. it&#39;s the party that will post/pledge collateral to the other party. ISDA 2016 English Law Credit Support Deed for Initial Margin, paragraph 13, General Principles, (ii).
	 */
	@RosettaSynonym(value="chargor_posting_obligations", source="ISDA_Create_1_0")
	public final ChargorPostingObligations getChargorPostingObligations() {
		return chargorPostingObligations;
	}
	
	/**
	 * The Chargor Rights Event election. ISDA 2016 English Law Credit Support Deed for Initial Margin, paragraph 13, General Principles, (j): Chargor Rights Event.
	 */
	@RosettaSynonym(value="chargor_rights_event", source="ISDA_Create_1_0")
	public final RightsEvent getChargorRightsEvent() {
		return chargorRightsEvent;
	}
	
	/**
	 * The Custodian and Segregated Account details in respect of each party as the Pledgor. ISDA 2016 English Law Credit Support Deed for Initial Margin, paragraph 13, General Principles, (n): Custody Arrangements.
	 */
	public final CustodyArrangements getCustodyArrangements() {
		return custodyArrangements;
	}
	
	/**
	 * The Delivery in Lieu Right provision applies when the value is set to True. ISDA 2016 English Law Credit Support Deed for Initial Margin, paragraph 13, General Principles, (m): Modification to Chargor’s Rights and Remedies.
	 */
	@RosettaSynonym(value="", source="ISDA_Create_1_0")
	public final Boolean getDeliveryInLieuRight() {
		return deliveryInLieuRight;
	}
	
	/**
	 * ISDA 2016 English Credit Support Annex for Initial Margin, paragraph 13, General Principles, (x): Japanese Securities Provisions (Shichiken).
	 */
	@RosettaSynonym(value="", source="ISDA_Create_1_0")
	public final Boolean getJapaneseSecuritiesProvisions() {
		return japaneseSecuritiesProvisions;
	}
	
	/**
	 * The Process Agent that might be appointed by the parties to the agreement in accordance with the ISDA 2016 English Law CSA, paragraph 11(h). ISDA 2016 English Law Credit Support Deed for Initial Margin, paragraph 13, General Principles, (t): Process Agent.
	 */
	@RosettaSynonym(value="process_agent", source="ISDA_Create_1_0")
	public final ProcessAgent getProcessAgent() {
		return processAgent;
	}
	
	/**
	 * The time by which the transfer of collateral must take place when different from the Regular Settlement Day as a result of parties&#39; election. ISDA 2016 English Law Credit Support Deed for Initial Margin, paragraph 13, General Principles, (c)(vii): Transfer Timing.
	 */
	@RosettaSynonym(value="settlement_day_specify", source="ISDA_Create_1_0", path="transfer_timing")
	public final String getTransferTiming() {
		return transferTiming;
	}
	
	
	@Override
	public RosettaMetaData<? extends CsdInitialMargin2016EnglishLaw> metaData() {
		return metaData;
	} 
	
	public CsdInitialMargin2016EnglishLawBuilder toBuilder() {
		CsdInitialMargin2016EnglishLawBuilder builder = new CsdInitialMargin2016EnglishLawBuilder();
		ofNullable(getAppropriatedCollateralValuation()).ifPresent(builder::setAppropriatedCollateralValuation);
		ofNullable(getChargorAdditionalRightsEvent()).ifPresent(builder::setChargorAdditionalRightsEvent);
		ofNullable(getChargorPostingObligations()).ifPresent(builder::setChargorPostingObligations);
		ofNullable(getChargorRightsEvent()).ifPresent(builder::setChargorRightsEvent);
		ofNullable(getCustodyArrangements()).ifPresent(builder::setCustodyArrangements);
		ofNullable(getDeliveryInLieuRight()).ifPresent(builder::setDeliveryInLieuRight);
		ofNullable(getJapaneseSecuritiesProvisions()).ifPresent(builder::setJapaneseSecuritiesProvisions);
		ofNullable(getProcessAgent()).ifPresent(builder::setProcessAgent);
		ofNullable(getTransferTiming()).ifPresent(builder::setTransferTiming);
		ofNullable(getCalculationDateLocation()).ifPresent(builder::setCalculationDateLocation);
		ofNullable(getCreditSupportObligations()).ifPresent(builder::setCreditSupportObligations);
		ofNullable(getFxHaircutCurrency()).ifPresent(builder::setFxHaircutCurrency);
		ofNullable(getIdentifiedCrossCurrencySwap()).ifPresent(builder::setIdentifiedCrossCurrencySwap);
		ofNullable(getMethod()).ifPresent(builder::setMethod);
		ofNullable(getNotificationTime()).ifPresent(builder::setNotificationTime);
		ofNullable(getOneWayProvisions()).ifPresent(builder::setOneWayProvisions);
		ofNullable(getRegime()).ifPresent(builder::setRegime);
		ofNullable(getSensitivityToCommodity()).ifPresent(builder::setSensitivityToCommodity);
		ofNullable(getSensitivityToEquity()).ifPresent(builder::setSensitivityToEquity);
		ofNullable(getTerminationCurrency()).ifPresent(builder::setTerminationCurrency);
		ofNullable(getAdditionalObligations()).ifPresent(builder::setAdditionalObligations);
		ofNullable(getAdditionalRepresentation()).ifPresent(builder::setAdditionalRepresentation);
		ofNullable(getAddressesForTransfer()).ifPresent(builder::setAddressesForTransfer);
		ofNullable(getBaseCurrency()).ifPresent(builder::setBaseCurrency);
		ofNullable(getBespokeProvision()).ifPresent(builder::setBespokeProvision);
		ofNullable(getConditionsPrecedent()).ifPresent(builder::setConditionsPrecedent);
		ofNullable(getDemandsAndNotices()).ifPresent(builder::setDemandsAndNotices);
		ofNullable(getDisputeResolution()).ifPresent(builder::setDisputeResolution);
		ofNullable(getSubstitution()).ifPresent(builder::setSubstitution);
		return builder;
	}

	public static CsdInitialMargin2016EnglishLawBuilder builder() {
		return new CsdInitialMargin2016EnglishLawBuilder();
	}

	public static  class CsdInitialMargin2016EnglishLawBuilder extends CsaInitialMargin2016Builder {
	
		protected AppropriatedCollateralValuation.AppropriatedCollateralValuationBuilder appropriatedCollateralValuation;
		protected AdditionalRightsEvent.AdditionalRightsEventBuilder chargorAdditionalRightsEvent;
		protected ChargorPostingObligations.ChargorPostingObligationsBuilder chargorPostingObligations;
		protected RightsEvent.RightsEventBuilder chargorRightsEvent;
		protected CustodyArrangements.CustodyArrangementsBuilder custodyArrangements;
		protected Boolean deliveryInLieuRight;
		protected Boolean japaneseSecuritiesProvisions;
		protected ProcessAgent.ProcessAgentBuilder processAgent;
		protected String transferTiming;
	
		public CsdInitialMargin2016EnglishLawBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends CsdInitialMargin2016EnglishLaw> metaData() {
			return metaData;
		} 
	
		
		public AppropriatedCollateralValuation.AppropriatedCollateralValuationBuilder getAppropriatedCollateralValuation() {
			return appropriatedCollateralValuation;
		}
		
		public AppropriatedCollateralValuation.AppropriatedCollateralValuationBuilder getOrCreateAppropriatedCollateralValuation() {
			if (appropriatedCollateralValuation!=null) {
				return appropriatedCollateralValuation;
			}
			else return appropriatedCollateralValuation = new AppropriatedCollateralValuation.AppropriatedCollateralValuationBuilder();
		}
		
		
		public AdditionalRightsEvent.AdditionalRightsEventBuilder getChargorAdditionalRightsEvent() {
			return chargorAdditionalRightsEvent;
		}
		
		public AdditionalRightsEvent.AdditionalRightsEventBuilder getOrCreateChargorAdditionalRightsEvent() {
			if (chargorAdditionalRightsEvent!=null) {
				return chargorAdditionalRightsEvent;
			}
			else return chargorAdditionalRightsEvent = new AdditionalRightsEvent.AdditionalRightsEventBuilder();
		}
		
		
		public ChargorPostingObligations.ChargorPostingObligationsBuilder getChargorPostingObligations() {
			return chargorPostingObligations;
		}
		
		public ChargorPostingObligations.ChargorPostingObligationsBuilder getOrCreateChargorPostingObligations() {
			if (chargorPostingObligations!=null) {
				return chargorPostingObligations;
			}
			else return chargorPostingObligations = new ChargorPostingObligations.ChargorPostingObligationsBuilder();
		}
		
		
		public RightsEvent.RightsEventBuilder getChargorRightsEvent() {
			return chargorRightsEvent;
		}
		
		public RightsEvent.RightsEventBuilder getOrCreateChargorRightsEvent() {
			if (chargorRightsEvent!=null) {
				return chargorRightsEvent;
			}
			else return chargorRightsEvent = new RightsEvent.RightsEventBuilder();
		}
		
		
		public CustodyArrangements.CustodyArrangementsBuilder getCustodyArrangements() {
			return custodyArrangements;
		}
		
		public CustodyArrangements.CustodyArrangementsBuilder getOrCreateCustodyArrangements() {
			if (custodyArrangements!=null) {
				return custodyArrangements;
			}
			else return custodyArrangements = new CustodyArrangements.CustodyArrangementsBuilder();
		}
		
		
		public Boolean getDeliveryInLieuRight() {
			return deliveryInLieuRight;
		}
		
		
		public Boolean getJapaneseSecuritiesProvisions() {
			return japaneseSecuritiesProvisions;
		}
		
		
		public ProcessAgent.ProcessAgentBuilder getProcessAgent() {
			return processAgent;
		}
		
		public ProcessAgent.ProcessAgentBuilder getOrCreateProcessAgent() {
			if (processAgent!=null) {
				return processAgent;
			}
			else return processAgent = new ProcessAgent.ProcessAgentBuilder();
		}
		
		
		public String getTransferTiming() {
			return transferTiming;
		}
		
	
		public CsdInitialMargin2016EnglishLawBuilder setAppropriatedCollateralValuation(AppropriatedCollateralValuation appropriatedCollateralValuation) {
			this.appropriatedCollateralValuation = appropriatedCollateralValuation.toBuilder();
			return this;
		}
		public CsdInitialMargin2016EnglishLawBuilder setAppropriatedCollateralValuationBuilder(AppropriatedCollateralValuation.AppropriatedCollateralValuationBuilder appropriatedCollateralValuation) {
			this.appropriatedCollateralValuation = appropriatedCollateralValuation;
			return this;
		}
		public CsdInitialMargin2016EnglishLawBuilder setChargorAdditionalRightsEvent(AdditionalRightsEvent chargorAdditionalRightsEvent) {
			this.chargorAdditionalRightsEvent = chargorAdditionalRightsEvent.toBuilder();
			return this;
		}
		public CsdInitialMargin2016EnglishLawBuilder setChargorAdditionalRightsEventBuilder(AdditionalRightsEvent.AdditionalRightsEventBuilder chargorAdditionalRightsEvent) {
			this.chargorAdditionalRightsEvent = chargorAdditionalRightsEvent;
			return this;
		}
		public CsdInitialMargin2016EnglishLawBuilder setChargorPostingObligations(ChargorPostingObligations chargorPostingObligations) {
			this.chargorPostingObligations = chargorPostingObligations.toBuilder();
			return this;
		}
		public CsdInitialMargin2016EnglishLawBuilder setChargorPostingObligationsBuilder(ChargorPostingObligations.ChargorPostingObligationsBuilder chargorPostingObligations) {
			this.chargorPostingObligations = chargorPostingObligations;
			return this;
		}
		public CsdInitialMargin2016EnglishLawBuilder setChargorRightsEvent(RightsEvent chargorRightsEvent) {
			this.chargorRightsEvent = chargorRightsEvent.toBuilder();
			return this;
		}
		public CsdInitialMargin2016EnglishLawBuilder setChargorRightsEventBuilder(RightsEvent.RightsEventBuilder chargorRightsEvent) {
			this.chargorRightsEvent = chargorRightsEvent;
			return this;
		}
		public CsdInitialMargin2016EnglishLawBuilder setCustodyArrangements(CustodyArrangements custodyArrangements) {
			this.custodyArrangements = custodyArrangements.toBuilder();
			return this;
		}
		public CsdInitialMargin2016EnglishLawBuilder setCustodyArrangementsBuilder(CustodyArrangements.CustodyArrangementsBuilder custodyArrangements) {
			this.custodyArrangements = custodyArrangements;
			return this;
		}
		public CsdInitialMargin2016EnglishLawBuilder setDeliveryInLieuRight(Boolean deliveryInLieuRight) {
			this.deliveryInLieuRight = deliveryInLieuRight;
			return this;
		}
		public CsdInitialMargin2016EnglishLawBuilder setJapaneseSecuritiesProvisions(Boolean japaneseSecuritiesProvisions) {
			this.japaneseSecuritiesProvisions = japaneseSecuritiesProvisions;
			return this;
		}
		public CsdInitialMargin2016EnglishLawBuilder setProcessAgent(ProcessAgent processAgent) {
			this.processAgent = processAgent.toBuilder();
			return this;
		}
		public CsdInitialMargin2016EnglishLawBuilder setProcessAgentBuilder(ProcessAgent.ProcessAgentBuilder processAgent) {
			this.processAgent = processAgent;
			return this;
		}
		public CsdInitialMargin2016EnglishLawBuilder setTransferTiming(String transferTiming) {
			this.transferTiming = transferTiming;
			return this;
		}
		@Override public CsdInitialMargin2016EnglishLawBuilder setCalculationDateLocation(CalculationDateLocation calculationDateLocation) {
			this.calculationDateLocation = calculationDateLocation.toBuilder();
			return this;
		}
		@Override public CsdInitialMargin2016EnglishLawBuilder setCalculationDateLocationBuilder(CalculationDateLocation.CalculationDateLocationBuilder calculationDateLocation) {
			this.calculationDateLocation = calculationDateLocation;
			return this;
		}
		@Override public CsdInitialMargin2016EnglishLawBuilder setCreditSupportObligations(CreditSupportObligationsInitialMargin creditSupportObligations) {
			this.creditSupportObligations = creditSupportObligations.toBuilder();
			return this;
		}
		@Override public CsdInitialMargin2016EnglishLawBuilder setCreditSupportObligationsBuilder(CreditSupportObligationsInitialMargin.CreditSupportObligationsInitialMarginBuilder creditSupportObligations) {
			this.creditSupportObligations = creditSupportObligations;
			return this;
		}
		@Override public CsdInitialMargin2016EnglishLawBuilder setFxHaircutCurrency(FxHaircutCurrency fxHaircutCurrency) {
			this.fxHaircutCurrency = fxHaircutCurrency.toBuilder();
			return this;
		}
		@Override public CsdInitialMargin2016EnglishLawBuilder setFxHaircutCurrencyBuilder(FxHaircutCurrency.FxHaircutCurrencyBuilder fxHaircutCurrency) {
			this.fxHaircutCurrency = fxHaircutCurrency;
			return this;
		}
		@Override public CsdInitialMargin2016EnglishLawBuilder setIdentifiedCrossCurrencySwap(Boolean identifiedCrossCurrencySwap) {
			this.identifiedCrossCurrencySwap = identifiedCrossCurrencySwap;
			return this;
		}
		@Override public CsdInitialMargin2016EnglishLawBuilder setMethod(Method method) {
			this.method = method.toBuilder();
			return this;
		}
		@Override public CsdInitialMargin2016EnglishLawBuilder setMethodBuilder(Method.MethodBuilder method) {
			this.method = method;
			return this;
		}
		@Override public CsdInitialMargin2016EnglishLawBuilder setNotificationTime(NotificationTime notificationTime) {
			this.notificationTime = notificationTime.toBuilder();
			return this;
		}
		@Override public CsdInitialMargin2016EnglishLawBuilder setNotificationTimeBuilder(NotificationTime.NotificationTimeBuilder notificationTime) {
			this.notificationTime = notificationTime;
			return this;
		}
		@Override public CsdInitialMargin2016EnglishLawBuilder setOneWayProvisions(OneWayProvisions oneWayProvisions) {
			this.oneWayProvisions = oneWayProvisions.toBuilder();
			return this;
		}
		@Override public CsdInitialMargin2016EnglishLawBuilder setOneWayProvisionsBuilder(OneWayProvisions.OneWayProvisionsBuilder oneWayProvisions) {
			this.oneWayProvisions = oneWayProvisions;
			return this;
		}
		@Override public CsdInitialMargin2016EnglishLawBuilder setRegime(Regime regime) {
			this.regime = regime.toBuilder();
			return this;
		}
		@Override public CsdInitialMargin2016EnglishLawBuilder setRegimeBuilder(Regime.RegimeBuilder regime) {
			this.regime = regime;
			return this;
		}
		@Override public CsdInitialMargin2016EnglishLawBuilder setSensitivityToCommodity(SensitivityMethodology sensitivityToCommodity) {
			this.sensitivityToCommodity = sensitivityToCommodity.toBuilder();
			return this;
		}
		@Override public CsdInitialMargin2016EnglishLawBuilder setSensitivityToCommodityBuilder(SensitivityMethodology.SensitivityMethodologyBuilder sensitivityToCommodity) {
			this.sensitivityToCommodity = sensitivityToCommodity;
			return this;
		}
		@Override public CsdInitialMargin2016EnglishLawBuilder setSensitivityToEquity(SensitivityMethodology sensitivityToEquity) {
			this.sensitivityToEquity = sensitivityToEquity.toBuilder();
			return this;
		}
		@Override public CsdInitialMargin2016EnglishLawBuilder setSensitivityToEquityBuilder(SensitivityMethodology.SensitivityMethodologyBuilder sensitivityToEquity) {
			this.sensitivityToEquity = sensitivityToEquity;
			return this;
		}
		@Override public CsdInitialMargin2016EnglishLawBuilder setTerminationCurrency(TerminationCurrencyAmendment terminationCurrency) {
			this.terminationCurrency = terminationCurrency.toBuilder();
			return this;
		}
		@Override public CsdInitialMargin2016EnglishLawBuilder setTerminationCurrencyBuilder(TerminationCurrencyAmendment.TerminationCurrencyAmendmentBuilder terminationCurrency) {
			this.terminationCurrency = terminationCurrency;
			return this;
		}
		@Override public CsdInitialMargin2016EnglishLawBuilder setAdditionalObligations(String additionalObligations) {
			this.additionalObligations = additionalObligations;
			return this;
		}
		@Override public CsdInitialMargin2016EnglishLawBuilder setAdditionalRepresentation(AdditionalRepresentation additionalRepresentation) {
			this.additionalRepresentation = additionalRepresentation.toBuilder();
			return this;
		}
		@Override public CsdInitialMargin2016EnglishLawBuilder setAdditionalRepresentationBuilder(AdditionalRepresentation.AdditionalRepresentationBuilder additionalRepresentation) {
			this.additionalRepresentation = additionalRepresentation;
			return this;
		}
		@Override public CsdInitialMargin2016EnglishLawBuilder setAddressesForTransfer(ContactElection addressesForTransfer) {
			this.addressesForTransfer = addressesForTransfer.toBuilder();
			return this;
		}
		@Override public CsdInitialMargin2016EnglishLawBuilder setAddressesForTransferBuilder(ContactElection.ContactElectionBuilder addressesForTransfer) {
			this.addressesForTransfer = addressesForTransfer;
			return this;
		}
		@Override public CsdInitialMargin2016EnglishLawBuilder setBaseCurrency(FieldWithMetaString baseCurrency) {
			this.baseCurrency = baseCurrency.toBuilder();
			return this;
		}
		@Override public CsdInitialMargin2016EnglishLawBuilder setBaseCurrencyRef(String baseCurrency) {
			return setBaseCurrency(FieldWithMetaString.builder().setValue(baseCurrency).build());
		}
		@Override public CsdInitialMargin2016EnglishLawBuilder setBespokeProvision(String bespokeProvision) {
			this.bespokeProvision = bespokeProvision;
			return this;
		}
		@Override public CsdInitialMargin2016EnglishLawBuilder setConditionsPrecedent(ConditionsPrecedent conditionsPrecedent) {
			this.conditionsPrecedent = conditionsPrecedent.toBuilder();
			return this;
		}
		@Override public CsdInitialMargin2016EnglishLawBuilder setConditionsPrecedentBuilder(ConditionsPrecedent.ConditionsPrecedentBuilder conditionsPrecedent) {
			this.conditionsPrecedent = conditionsPrecedent;
			return this;
		}
		@Override public CsdInitialMargin2016EnglishLawBuilder setDemandsAndNotices(ContactElection demandsAndNotices) {
			this.demandsAndNotices = demandsAndNotices.toBuilder();
			return this;
		}
		@Override public CsdInitialMargin2016EnglishLawBuilder setDemandsAndNoticesBuilder(ContactElection.ContactElectionBuilder demandsAndNotices) {
			this.demandsAndNotices = demandsAndNotices;
			return this;
		}
		@Override public CsdInitialMargin2016EnglishLawBuilder setDisputeResolution(DisputeResolution disputeResolution) {
			this.disputeResolution = disputeResolution.toBuilder();
			return this;
		}
		@Override public CsdInitialMargin2016EnglishLawBuilder setDisputeResolutionBuilder(DisputeResolution.DisputeResolutionBuilder disputeResolution) {
			this.disputeResolution = disputeResolution;
			return this;
		}
		@Override public CsdInitialMargin2016EnglishLawBuilder setSubstitution(Substitution substitution) {
			this.substitution = substitution.toBuilder();
			return this;
		}
		@Override public CsdInitialMargin2016EnglishLawBuilder setSubstitutionBuilder(Substitution.SubstitutionBuilder substitution) {
			this.substitution = substitution;
			return this;
		}
		
		public CsdInitialMargin2016EnglishLaw build() {
			return new CsdInitialMargin2016EnglishLaw(this);
		}
	
		@Override
		public CsdInitialMargin2016EnglishLawBuilder prune() {
			super.prune();
			if (appropriatedCollateralValuation!=null && !appropriatedCollateralValuation.prune().hasData()) appropriatedCollateralValuation = null;
			if (chargorAdditionalRightsEvent!=null && !chargorAdditionalRightsEvent.prune().hasData()) chargorAdditionalRightsEvent = null;
			if (chargorPostingObligations!=null && !chargorPostingObligations.prune().hasData()) chargorPostingObligations = null;
			if (chargorRightsEvent!=null && !chargorRightsEvent.prune().hasData()) chargorRightsEvent = null;
			if (custodyArrangements!=null && !custodyArrangements.prune().hasData()) custodyArrangements = null;
			if (processAgent!=null && !processAgent.prune().hasData()) processAgent = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (super.hasData()) return true;
			if (getAppropriatedCollateralValuation()!=null && getAppropriatedCollateralValuation().hasData()) return true;
			if (getChargorAdditionalRightsEvent()!=null && getChargorAdditionalRightsEvent().hasData()) return true;
			if (getChargorPostingObligations()!=null && getChargorPostingObligations().hasData()) return true;
			if (getChargorRightsEvent()!=null && getChargorRightsEvent().hasData()) return true;
			if (getCustodyArrangements()!=null && getCustodyArrangements().hasData()) return true;
			if (getDeliveryInLieuRight()!=null) return true;
			if (getJapaneseSecuritiesProvisions()!=null) return true;
			if (getProcessAgent()!=null && getProcessAgent().hasData()) return true;
			if (getTransferTiming()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
			super.process(path, processor);
		
			processor.processBasic(path.newSubPath("deliveryInLieuRight"), Boolean.class, deliveryInLieuRight, this);
			processor.processBasic(path.newSubPath("japaneseSecuritiesProvisions"), Boolean.class, japaneseSecuritiesProvisions, this);
			processor.processBasic(path.newSubPath("transferTiming"), String.class, transferTiming, this);
			
			processRosetta(path.newSubPath("appropriatedCollateralValuation"), processor, AppropriatedCollateralValuation.class, appropriatedCollateralValuation);
			processRosetta(path.newSubPath("chargorAdditionalRightsEvent"), processor, AdditionalRightsEvent.class, chargorAdditionalRightsEvent);
			processRosetta(path.newSubPath("chargorPostingObligations"), processor, ChargorPostingObligations.class, chargorPostingObligations);
			processRosetta(path.newSubPath("chargorRightsEvent"), processor, RightsEvent.class, chargorRightsEvent);
			processRosetta(path.newSubPath("custodyArrangements"), processor, CustodyArrangements.class, custodyArrangements);
			processRosetta(path.newSubPath("processAgent"), processor, ProcessAgent.class, processAgent);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
			if (!super.equals(o)) return false;
		
			CsdInitialMargin2016EnglishLawBuilder _that = (CsdInitialMargin2016EnglishLawBuilder) o;
		
			if (appropriatedCollateralValuation != null ? !appropriatedCollateralValuation.equals(_that.appropriatedCollateralValuation) : _that.appropriatedCollateralValuation != null) return false;
			if (chargorAdditionalRightsEvent != null ? !chargorAdditionalRightsEvent.equals(_that.chargorAdditionalRightsEvent) : _that.chargorAdditionalRightsEvent != null) return false;
			if (chargorPostingObligations != null ? !chargorPostingObligations.equals(_that.chargorPostingObligations) : _that.chargorPostingObligations != null) return false;
			if (chargorRightsEvent != null ? !chargorRightsEvent.equals(_that.chargorRightsEvent) : _that.chargorRightsEvent != null) return false;
			if (custodyArrangements != null ? !custodyArrangements.equals(_that.custodyArrangements) : _that.custodyArrangements != null) return false;
			if (deliveryInLieuRight != null ? !deliveryInLieuRight.equals(_that.deliveryInLieuRight) : _that.deliveryInLieuRight != null) return false;
			if (japaneseSecuritiesProvisions != null ? !japaneseSecuritiesProvisions.equals(_that.japaneseSecuritiesProvisions) : _that.japaneseSecuritiesProvisions != null) return false;
			if (processAgent != null ? !processAgent.equals(_that.processAgent) : _that.processAgent != null) return false;
			if (transferTiming != null ? !transferTiming.equals(_that.transferTiming) : _that.transferTiming != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = super.hashCode();
			_result = 31 * _result + (appropriatedCollateralValuation != null ? appropriatedCollateralValuation.hashCode() : 0);
			_result = 31 * _result + (chargorAdditionalRightsEvent != null ? chargorAdditionalRightsEvent.hashCode() : 0);
			_result = 31 * _result + (chargorPostingObligations != null ? chargorPostingObligations.hashCode() : 0);
			_result = 31 * _result + (chargorRightsEvent != null ? chargorRightsEvent.hashCode() : 0);
			_result = 31 * _result + (custodyArrangements != null ? custodyArrangements.hashCode() : 0);
			_result = 31 * _result + (deliveryInLieuRight != null ? deliveryInLieuRight.hashCode() : 0);
			_result = 31 * _result + (japaneseSecuritiesProvisions != null ? japaneseSecuritiesProvisions.hashCode() : 0);
			_result = 31 * _result + (processAgent != null ? processAgent.hashCode() : 0);
			_result = 31 * _result + (transferTiming != null ? transferTiming.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "CsdInitialMargin2016EnglishLawBuilder {" +
				"appropriatedCollateralValuation=" + this.appropriatedCollateralValuation + ", " +
				"chargorAdditionalRightsEvent=" + this.chargorAdditionalRightsEvent + ", " +
				"chargorPostingObligations=" + this.chargorPostingObligations + ", " +
				"chargorRightsEvent=" + this.chargorRightsEvent + ", " +
				"custodyArrangements=" + this.custodyArrangements + ", " +
				"deliveryInLieuRight=" + this.deliveryInLieuRight + ", " +
				"japaneseSecuritiesProvisions=" + this.japaneseSecuritiesProvisions + ", " +
				"processAgent=" + this.processAgent + ", " +
				"transferTiming=" + this.transferTiming +
			'}' + " " + super.toString();
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		super.process(path, processor);
		
		processor.processBasic(path.newSubPath("deliveryInLieuRight"), Boolean.class, deliveryInLieuRight, this);
		processor.processBasic(path.newSubPath("japaneseSecuritiesProvisions"), Boolean.class, japaneseSecuritiesProvisions, this);
		processor.processBasic(path.newSubPath("transferTiming"), String.class, transferTiming, this);
		
		processRosetta(path.newSubPath("appropriatedCollateralValuation"), processor, AppropriatedCollateralValuation.class, appropriatedCollateralValuation);
		processRosetta(path.newSubPath("chargorAdditionalRightsEvent"), processor, AdditionalRightsEvent.class, chargorAdditionalRightsEvent);
		processRosetta(path.newSubPath("chargorPostingObligations"), processor, ChargorPostingObligations.class, chargorPostingObligations);
		processRosetta(path.newSubPath("chargorRightsEvent"), processor, RightsEvent.class, chargorRightsEvent);
		processRosetta(path.newSubPath("custodyArrangements"), processor, CustodyArrangements.class, custodyArrangements);
		processRosetta(path.newSubPath("processAgent"), processor, ProcessAgent.class, processAgent);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
		if (!super.equals(o)) return false;
	
		CsdInitialMargin2016EnglishLaw _that = (CsdInitialMargin2016EnglishLaw) o;
	
		if (appropriatedCollateralValuation != null ? !appropriatedCollateralValuation.equals(_that.appropriatedCollateralValuation) : _that.appropriatedCollateralValuation != null) return false;
		if (chargorAdditionalRightsEvent != null ? !chargorAdditionalRightsEvent.equals(_that.chargorAdditionalRightsEvent) : _that.chargorAdditionalRightsEvent != null) return false;
		if (chargorPostingObligations != null ? !chargorPostingObligations.equals(_that.chargorPostingObligations) : _that.chargorPostingObligations != null) return false;
		if (chargorRightsEvent != null ? !chargorRightsEvent.equals(_that.chargorRightsEvent) : _that.chargorRightsEvent != null) return false;
		if (custodyArrangements != null ? !custodyArrangements.equals(_that.custodyArrangements) : _that.custodyArrangements != null) return false;
		if (deliveryInLieuRight != null ? !deliveryInLieuRight.equals(_that.deliveryInLieuRight) : _that.deliveryInLieuRight != null) return false;
		if (japaneseSecuritiesProvisions != null ? !japaneseSecuritiesProvisions.equals(_that.japaneseSecuritiesProvisions) : _that.japaneseSecuritiesProvisions != null) return false;
		if (processAgent != null ? !processAgent.equals(_that.processAgent) : _that.processAgent != null) return false;
		if (transferTiming != null ? !transferTiming.equals(_that.transferTiming) : _that.transferTiming != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = super.hashCode();
		_result = 31 * _result + (appropriatedCollateralValuation != null ? appropriatedCollateralValuation.hashCode() : 0);
		_result = 31 * _result + (chargorAdditionalRightsEvent != null ? chargorAdditionalRightsEvent.hashCode() : 0);
		_result = 31 * _result + (chargorPostingObligations != null ? chargorPostingObligations.hashCode() : 0);
		_result = 31 * _result + (chargorRightsEvent != null ? chargorRightsEvent.hashCode() : 0);
		_result = 31 * _result + (custodyArrangements != null ? custodyArrangements.hashCode() : 0);
		_result = 31 * _result + (deliveryInLieuRight != null ? deliveryInLieuRight.hashCode() : 0);
		_result = 31 * _result + (japaneseSecuritiesProvisions != null ? japaneseSecuritiesProvisions.hashCode() : 0);
		_result = 31 * _result + (processAgent != null ? processAgent.hashCode() : 0);
		_result = 31 * _result + (transferTiming != null ? transferTiming.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "CsdInitialMargin2016EnglishLaw {" +
			"appropriatedCollateralValuation=" + this.appropriatedCollateralValuation + ", " +
			"chargorAdditionalRightsEvent=" + this.chargorAdditionalRightsEvent + ", " +
			"chargorPostingObligations=" + this.chargorPostingObligations + ", " +
			"chargorRightsEvent=" + this.chargorRightsEvent + ", " +
			"custodyArrangements=" + this.custodyArrangements + ", " +
			"deliveryInLieuRight=" + this.deliveryInLieuRight + ", " +
			"japaneseSecuritiesProvisions=" + this.japaneseSecuritiesProvisions + ", " +
			"processAgent=" + this.processAgent + ", " +
			"transferTiming=" + this.transferTiming +
		'}' + " " + super.toString();
	}
}
