package org.isda.cdm;

import java.lang.Boolean;
import org.isda.cdm.CustodianEventEndDate;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.CustodianEventMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class to specify the Custodian Event (English Law &amp; New York Law ISDA CSA) and the Collateral Manager Event (Japanese Law ISDA CSA) in terms of applicability and end-date. ISDA 2016 English Law Credit Support Deed for Initial Margin, paragraph 13, General Principles, (n)(iii): Custodian Event. | ISDA 2016 Japanese Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (m)(ii): Collateral Manager Event. | ISDA 2016 New York Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (n)(iii): Custodian Event.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class CustodianEvent extends RosettaModelObject {
	private final CustodianEventEndDate endDate;
	private final Boolean isApplicable;
	private static CustodianEventMeta metaData = new CustodianEventMeta();
	
	CustodianEvent(CustodianEventBuilder builder) {
		this.endDate = ofNullable(builder.getEndDate()).map(f->f.build()).orElse(null);
		this.isApplicable = builder.getIsApplicable();
	}
	
	/**
	 * The qualification of the Custodian Event (English Law &amp; New York Law ISDA CSA) or Collateral Manager Event (Japanese Law ISDA CSA) End Date.
	 */
	@RosettaSynonym(value="ce_end_date", source="ISDA_Create_1_0")
	public final CustodianEventEndDate getEndDate() {
		return endDate;
	}
	
	/**
	 * The qualification as to whether the Custodian Event (English Law &amp; New York Law ISDA CSA) or the Collateral Manager Event (Japanese Law ISDA CSA) is applicable.
	 */
	@RosettaSynonym(value="", source="ISDA_Create_1_0")
	public final Boolean getIsApplicable() {
		return isApplicable;
	}
	
	
	@Override
	public RosettaMetaData<? extends CustodianEvent> metaData() {
		return metaData;
	} 
	
	public CustodianEventBuilder toBuilder() {
		CustodianEventBuilder builder = new CustodianEventBuilder();
		ofNullable(getEndDate()).ifPresent(builder::setEndDate);
		ofNullable(getIsApplicable()).ifPresent(builder::setIsApplicable);
		return builder;
	}

	public static CustodianEventBuilder builder() {
		return new CustodianEventBuilder();
	}

	public static  class CustodianEventBuilder extends RosettaModelObjectBuilder {
	
		protected CustodianEventEndDate.CustodianEventEndDateBuilder endDate;
		protected Boolean isApplicable;
	
		public CustodianEventBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends CustodianEvent> metaData() {
			return metaData;
		} 
	
		
		public CustodianEventEndDate.CustodianEventEndDateBuilder getEndDate() {
			return endDate;
		}
		
		public CustodianEventEndDate.CustodianEventEndDateBuilder getOrCreateEndDate() {
			if (endDate!=null) {
				return endDate;
			}
			else return endDate = new CustodianEventEndDate.CustodianEventEndDateBuilder();
		}
		
		
		public Boolean getIsApplicable() {
			return isApplicable;
		}
		
	
		public CustodianEventBuilder setEndDate(CustodianEventEndDate endDate) {
			this.endDate = endDate.toBuilder();
			return this;
		}
		public CustodianEventBuilder setEndDateBuilder(CustodianEventEndDate.CustodianEventEndDateBuilder endDate) {
			this.endDate = endDate;
			return this;
		}
		public CustodianEventBuilder setIsApplicable(Boolean isApplicable) {
			this.isApplicable = isApplicable;
			return this;
		}
		
		public CustodianEvent build() {
			return new CustodianEvent(this);
		}
	
		@Override
		public CustodianEventBuilder prune() {
			if (endDate!=null && !endDate.prune().hasData()) endDate = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getEndDate()!=null && getEndDate().hasData()) return true;
			if (getIsApplicable()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("isApplicable"), Boolean.class, isApplicable, this);
			
			processRosetta(path.newSubPath("endDate"), processor, CustodianEventEndDate.class, endDate);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			CustodianEventBuilder _that = (CustodianEventBuilder) o;
		
			if (endDate != null ? !endDate.equals(_that.endDate) : _that.endDate != null) return false;
			if (isApplicable != null ? !isApplicable.equals(_that.isApplicable) : _that.isApplicable != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (endDate != null ? endDate.hashCode() : 0);
			_result = 31 * _result + (isApplicable != null ? isApplicable.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "CustodianEventBuilder {" +
				"endDate=" + this.endDate + ", " +
				"isApplicable=" + this.isApplicable +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("isApplicable"), Boolean.class, isApplicable, this);
		
		processRosetta(path.newSubPath("endDate"), processor, CustodianEventEndDate.class, endDate);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		CustodianEvent _that = (CustodianEvent) o;
	
		if (endDate != null ? !endDate.equals(_that.endDate) : _that.endDate != null) return false;
		if (isApplicable != null ? !isApplicable.equals(_that.isApplicable) : _that.isApplicable != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (endDate != null ? endDate.hashCode() : 0);
		_result = 31 * _result + (isApplicable != null ? isApplicable.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "CustodianEvent {" +
			"endDate=" + this.endDate + ", " +
			"isApplicable=" + this.isApplicable +
		'}';
	}
}
