package org.isda.cdm;

import org.isda.cdm.CustomisableOffset;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.CustodianEventEndDateMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class to specify the Custodian Event (English Law &amp; New York Law ISDA CSA) or Collateral Manager Event (Japanese Law ISDA CSA) End Date. Its qualification is function of three elective periods: either (i) a specified number of days after the occurrence of the Custodian Event (the daysAfterCustodianEvent attribute), or (ii) the number of days prior to the date on which the Control Agreement will terminate, with in this latter case (iii) the further qualification of the number of days prior to the Release Date if only one party has effectively provided the Timely Statement to the other party. ISDA 2016 English Law Credit Support Deed for Initial Margin, paragraph 13, General Principles, (n)(iii): CE End Date. | ISDA 2016 Japanese Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (m)(ii): CME End Date. | ISDA 2016 New York Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (n)(iii): CE End Date.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class CustodianEventEndDate extends RosettaModelObject {
	private final CustomisableOffset dateOfTimelyStatement;
	private final CustomisableOffset daysAfterCustodianEvent;
	private final CustomisableOffset releaseDate;
	private static CustodianEventEndDateMeta metaData = new CustodianEventEndDateMeta();
	
	CustodianEventEndDate(CustodianEventEndDateBuilder builder) {
		this.dateOfTimelyStatement = ofNullable(builder.getDateOfTimelyStatement()).map(f->f.build()).orElse(null);
		this.daysAfterCustodianEvent = ofNullable(builder.getDaysAfterCustodianEvent()).map(f->f.build()).orElse(null);
		this.releaseDate = ofNullable(builder.getReleaseDate()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The parties&#39; election to specify the number of days one party has effectively provided the Timely Statement to the other party.
	 */
	public final CustomisableOffset getDateOfTimelyStatement() {
		return dateOfTimelyStatement;
	}
	
	/**
	 * The parties&#39; election to specify the number of days after the occurrence of the Custodian Event (English Law &amp; New York Law ISDA CSA) or the Collateral Management Event (Japanese Law ISDA CSA) for the purpose of qualifying the CE/CME End Date.
	 */
	public final CustomisableOffset getDaysAfterCustodianEvent() {
		return daysAfterCustodianEvent;
	}
	
	/**
	 * The parties&#39; election to specify the number of days prior to the termination of the Control Agreement (English Law &amp; New York Law ISDA CSA) or the Collateral Management Event (Japanese Law ISDA CSA) for the purpose of qualifying the CE/CME End Date, in the case where advance notice is given.
	 */
	public final CustomisableOffset getReleaseDate() {
		return releaseDate;
	}
	
	
	@Override
	public RosettaMetaData<? extends CustodianEventEndDate> metaData() {
		return metaData;
	} 
	
	public CustodianEventEndDateBuilder toBuilder() {
		CustodianEventEndDateBuilder builder = new CustodianEventEndDateBuilder();
		ofNullable(getDateOfTimelyStatement()).ifPresent(builder::setDateOfTimelyStatement);
		ofNullable(getDaysAfterCustodianEvent()).ifPresent(builder::setDaysAfterCustodianEvent);
		ofNullable(getReleaseDate()).ifPresent(builder::setReleaseDate);
		return builder;
	}

	public static CustodianEventEndDateBuilder builder() {
		return new CustodianEventEndDateBuilder();
	}

	public static  class CustodianEventEndDateBuilder extends RosettaModelObjectBuilder {
	
		protected CustomisableOffset.CustomisableOffsetBuilder dateOfTimelyStatement;
		protected CustomisableOffset.CustomisableOffsetBuilder daysAfterCustodianEvent;
		protected CustomisableOffset.CustomisableOffsetBuilder releaseDate;
	
		public CustodianEventEndDateBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends CustodianEventEndDate> metaData() {
			return metaData;
		} 
	
		
		public CustomisableOffset.CustomisableOffsetBuilder getDateOfTimelyStatement() {
			return dateOfTimelyStatement;
		}
		
		public CustomisableOffset.CustomisableOffsetBuilder getOrCreateDateOfTimelyStatement() {
			if (dateOfTimelyStatement!=null) {
				return dateOfTimelyStatement;
			}
			else return dateOfTimelyStatement = new CustomisableOffset.CustomisableOffsetBuilder();
		}
		
		
		public CustomisableOffset.CustomisableOffsetBuilder getDaysAfterCustodianEvent() {
			return daysAfterCustodianEvent;
		}
		
		public CustomisableOffset.CustomisableOffsetBuilder getOrCreateDaysAfterCustodianEvent() {
			if (daysAfterCustodianEvent!=null) {
				return daysAfterCustodianEvent;
			}
			else return daysAfterCustodianEvent = new CustomisableOffset.CustomisableOffsetBuilder();
		}
		
		
		public CustomisableOffset.CustomisableOffsetBuilder getReleaseDate() {
			return releaseDate;
		}
		
		public CustomisableOffset.CustomisableOffsetBuilder getOrCreateReleaseDate() {
			if (releaseDate!=null) {
				return releaseDate;
			}
			else return releaseDate = new CustomisableOffset.CustomisableOffsetBuilder();
		}
		
	
		public CustodianEventEndDateBuilder setDateOfTimelyStatement(CustomisableOffset dateOfTimelyStatement) {
			this.dateOfTimelyStatement = dateOfTimelyStatement.toBuilder();
			return this;
		}
		public CustodianEventEndDateBuilder setDateOfTimelyStatementBuilder(CustomisableOffset.CustomisableOffsetBuilder dateOfTimelyStatement) {
			this.dateOfTimelyStatement = dateOfTimelyStatement;
			return this;
		}
		public CustodianEventEndDateBuilder setDaysAfterCustodianEvent(CustomisableOffset daysAfterCustodianEvent) {
			this.daysAfterCustodianEvent = daysAfterCustodianEvent.toBuilder();
			return this;
		}
		public CustodianEventEndDateBuilder setDaysAfterCustodianEventBuilder(CustomisableOffset.CustomisableOffsetBuilder daysAfterCustodianEvent) {
			this.daysAfterCustodianEvent = daysAfterCustodianEvent;
			return this;
		}
		public CustodianEventEndDateBuilder setReleaseDate(CustomisableOffset releaseDate) {
			this.releaseDate = releaseDate.toBuilder();
			return this;
		}
		public CustodianEventEndDateBuilder setReleaseDateBuilder(CustomisableOffset.CustomisableOffsetBuilder releaseDate) {
			this.releaseDate = releaseDate;
			return this;
		}
		
		public CustodianEventEndDate build() {
			return new CustodianEventEndDate(this);
		}
	
		@Override
		public CustodianEventEndDateBuilder prune() {
			if (dateOfTimelyStatement!=null && !dateOfTimelyStatement.prune().hasData()) dateOfTimelyStatement = null;
			if (daysAfterCustodianEvent!=null && !daysAfterCustodianEvent.prune().hasData()) daysAfterCustodianEvent = null;
			if (releaseDate!=null && !releaseDate.prune().hasData()) releaseDate = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getDateOfTimelyStatement()!=null && getDateOfTimelyStatement().hasData()) return true;
			if (getDaysAfterCustodianEvent()!=null && getDaysAfterCustodianEvent().hasData()) return true;
			if (getReleaseDate()!=null && getReleaseDate().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("dateOfTimelyStatement"), processor, CustomisableOffset.class, dateOfTimelyStatement);
			processRosetta(path.newSubPath("daysAfterCustodianEvent"), processor, CustomisableOffset.class, daysAfterCustodianEvent);
			processRosetta(path.newSubPath("releaseDate"), processor, CustomisableOffset.class, releaseDate);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			CustodianEventEndDateBuilder _that = (CustodianEventEndDateBuilder) o;
		
			if (dateOfTimelyStatement != null ? !dateOfTimelyStatement.equals(_that.dateOfTimelyStatement) : _that.dateOfTimelyStatement != null) return false;
			if (daysAfterCustodianEvent != null ? !daysAfterCustodianEvent.equals(_that.daysAfterCustodianEvent) : _that.daysAfterCustodianEvent != null) return false;
			if (releaseDate != null ? !releaseDate.equals(_that.releaseDate) : _that.releaseDate != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (dateOfTimelyStatement != null ? dateOfTimelyStatement.hashCode() : 0);
			_result = 31 * _result + (daysAfterCustodianEvent != null ? daysAfterCustodianEvent.hashCode() : 0);
			_result = 31 * _result + (releaseDate != null ? releaseDate.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "CustodianEventEndDateBuilder {" +
				"dateOfTimelyStatement=" + this.dateOfTimelyStatement + ", " +
				"daysAfterCustodianEvent=" + this.daysAfterCustodianEvent + ", " +
				"releaseDate=" + this.releaseDate +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("dateOfTimelyStatement"), processor, CustomisableOffset.class, dateOfTimelyStatement);
		processRosetta(path.newSubPath("daysAfterCustodianEvent"), processor, CustomisableOffset.class, daysAfterCustodianEvent);
		processRosetta(path.newSubPath("releaseDate"), processor, CustomisableOffset.class, releaseDate);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		CustodianEventEndDate _that = (CustodianEventEndDate) o;
	
		if (dateOfTimelyStatement != null ? !dateOfTimelyStatement.equals(_that.dateOfTimelyStatement) : _that.dateOfTimelyStatement != null) return false;
		if (daysAfterCustodianEvent != null ? !daysAfterCustodianEvent.equals(_that.daysAfterCustodianEvent) : _that.daysAfterCustodianEvent != null) return false;
		if (releaseDate != null ? !releaseDate.equals(_that.releaseDate) : _that.releaseDate != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (dateOfTimelyStatement != null ? dateOfTimelyStatement.hashCode() : 0);
		_result = 31 * _result + (daysAfterCustodianEvent != null ? daysAfterCustodianEvent.hashCode() : 0);
		_result = 31 * _result + (releaseDate != null ? releaseDate.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "CustodianEventEndDate {" +
			"dateOfTimelyStatement=" + this.dateOfTimelyStatement + ", " +
			"daysAfterCustodianEvent=" + this.daysAfterCustodianEvent + ", " +
			"releaseDate=" + this.releaseDate +
		'}';
	}
}
