package org.isda.cdm;

import java.lang.Boolean;
import java.lang.String;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.CustodianRiskMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class to specify the Custodian Risk (English Law and New York Law ISDA CSA) and the Collateral Manager Risk (Japanese Law ISDA CSA) election. ISDA 2016 English Law Credit Support Deed for Initial Margin, paragraph 13, General Principles, (n)(ii): Custodian (IM) Risk. | ISDA 2016 Japanese Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (m)(i): Collateral Manager Risk. | ISDA 2016 New York Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (n)(ii): Custodian (IM) Risk.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class CustodianRisk extends RosettaModelObject {
	private final Boolean isSpecified;
	private final String qualification;
	private static CustodianRiskMeta metaData = new CustodianRiskMeta();
	
	CustodianRisk(CustodianRiskBuilder builder) {
		this.isSpecified = builder.getIsSpecified();
		this.qualification = builder.getQualification();
	}
	
	/**
	 * The qualification as to whether the risk is deemed as Specified.
	 */
	@RosettaSynonym(value="", source="ISDA_Create_1_0")
	@RosettaSynonym(value="", source="ISDA_Create_1_0")
	public final Boolean getIsSpecified() {
		return isSpecified;
	}
	
	/**
	 * The Custodian Risk (English Law and New York Law ISDA CSA) or Collateral Manager Risk (Japanese Law ISDA CSA) qualification. This attribute is optional because the Custodian Risk provision can be deemed as &#39;Specified&#39;, although not be qualified through this attribute.
	 */
	@RosettaSynonym(value="partyA_specify", source="ISDA_Create_1_0")
	@RosettaSynonym(value="partyB_specify", source="ISDA_Create_1_0")
	public final String getQualification() {
		return qualification;
	}
	
	
	@Override
	public RosettaMetaData<? extends CustodianRisk> metaData() {
		return metaData;
	} 
	
	public CustodianRiskBuilder toBuilder() {
		CustodianRiskBuilder builder = new CustodianRiskBuilder();
		ofNullable(getIsSpecified()).ifPresent(builder::setIsSpecified);
		ofNullable(getQualification()).ifPresent(builder::setQualification);
		return builder;
	}

	public static CustodianRiskBuilder builder() {
		return new CustodianRiskBuilder();
	}

	public static  class CustodianRiskBuilder extends RosettaModelObjectBuilder {
	
		protected Boolean isSpecified;
		protected String qualification;
	
		public CustodianRiskBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends CustodianRisk> metaData() {
			return metaData;
		} 
	
		
		public Boolean getIsSpecified() {
			return isSpecified;
		}
		
		
		public String getQualification() {
			return qualification;
		}
		
	
		public CustodianRiskBuilder setIsSpecified(Boolean isSpecified) {
			this.isSpecified = isSpecified;
			return this;
		}
		public CustodianRiskBuilder setQualification(String qualification) {
			this.qualification = qualification;
			return this;
		}
		
		public CustodianRisk build() {
			return new CustodianRisk(this);
		}
	
		@Override
		public CustodianRiskBuilder prune() {
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getIsSpecified()!=null) return true;
			if (getQualification()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("isSpecified"), Boolean.class, isSpecified, this);
			processor.processBasic(path.newSubPath("qualification"), String.class, qualification, this);
			
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			CustodianRiskBuilder _that = (CustodianRiskBuilder) o;
		
			if (isSpecified != null ? !isSpecified.equals(_that.isSpecified) : _that.isSpecified != null) return false;
			if (qualification != null ? !qualification.equals(_that.qualification) : _that.qualification != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (isSpecified != null ? isSpecified.hashCode() : 0);
			_result = 31 * _result + (qualification != null ? qualification.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "CustodianRiskBuilder {" +
				"isSpecified=" + this.isSpecified + ", " +
				"qualification=" + this.qualification +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("isSpecified"), Boolean.class, isSpecified, this);
		processor.processBasic(path.newSubPath("qualification"), String.class, qualification, this);
		
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		CustodianRisk _that = (CustodianRisk) o;
	
		if (isSpecified != null ? !isSpecified.equals(_that.isSpecified) : _that.isSpecified != null) return false;
		if (qualification != null ? !qualification.equals(_that.qualification) : _that.qualification != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (isSpecified != null ? isSpecified.hashCode() : 0);
		_result = 31 * _result + (qualification != null ? qualification.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "CustodianRisk {" +
			"isSpecified=" + this.isSpecified + ", " +
			"qualification=" + this.qualification +
		'}';
	}
}
