package org.isda.cdm;

import org.isda.cdm.CreditNotations;
import org.isda.cdm.LegalEntity;
import org.isda.cdm.Money;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.CustodianTermsMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class to specify the requirements applicable to the custodian with respect to the holding of posted collateral. ISDA 2016 Credit Support Annex for Variation Margin, paragraph 13, (h)(i): Eligibility to Hold Posted Collateral (VM); Custodians (VM).
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="CustodianTerms", source="FpML_5_10")
public class CustodianTerms extends RosettaModelObject {
	private final LegalEntity initialDesignation;
	private final Money minimumAssets;
	private final CreditNotations minimumRating;
	private static CustodianTermsMeta metaData = new CustodianTermsMeta();
	
	CustodianTerms(CustodianTermsBuilder builder) {
		this.initialDesignation = ofNullable(builder.getInitialDesignation()).map(f->f.build()).orElse(null);
		this.minimumAssets = ofNullable(builder.getMinimumAssets()).map(f->f.build()).orElse(null);
		this.minimumRating = ofNullable(builder.getMinimumRating()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The 2016 ISDA CSA for Variation Margin provides the ability for the parties to specify the initial custodian.
	 */
	public final LegalEntity getInitialDesignation() {
		return initialDesignation;
	}
	
	/**
	 * The minimal level of assets requirement with respect to the custody agent.
	 */
	@RosettaSynonym(value="minimumAssets", source="FpML_5_10")
	public final Money getMinimumAssets() {
		return minimumAssets;
	}
	
	/**
	 * The minimal rating requirement with respect to the custody agent.
	 */
	@RosettaSynonym(value="minimumRating", source="FpML_5_10")
	public final CreditNotations getMinimumRating() {
		return minimumRating;
	}
	
	
	@Override
	public RosettaMetaData<? extends CustodianTerms> metaData() {
		return metaData;
	} 
	
	public CustodianTermsBuilder toBuilder() {
		CustodianTermsBuilder builder = new CustodianTermsBuilder();
		ofNullable(getInitialDesignation()).ifPresent(builder::setInitialDesignation);
		ofNullable(getMinimumAssets()).ifPresent(builder::setMinimumAssets);
		ofNullable(getMinimumRating()).ifPresent(builder::setMinimumRating);
		return builder;
	}

	public static CustodianTermsBuilder builder() {
		return new CustodianTermsBuilder();
	}

	public static  class CustodianTermsBuilder extends RosettaModelObjectBuilder {
	
		protected LegalEntity.LegalEntityBuilder initialDesignation;
		protected Money.MoneyBuilder minimumAssets;
		protected CreditNotations.CreditNotationsBuilder minimumRating;
	
		public CustodianTermsBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends CustodianTerms> metaData() {
			return metaData;
		} 
	
		
		public LegalEntity.LegalEntityBuilder getInitialDesignation() {
			return initialDesignation;
		}
		
		public LegalEntity.LegalEntityBuilder getOrCreateInitialDesignation() {
			if (initialDesignation!=null) {
				return initialDesignation;
			}
			else return initialDesignation = new LegalEntity.LegalEntityBuilder();
		}
		
		
		public Money.MoneyBuilder getMinimumAssets() {
			return minimumAssets;
		}
		
		public Money.MoneyBuilder getOrCreateMinimumAssets() {
			if (minimumAssets!=null) {
				return minimumAssets;
			}
			else return minimumAssets = new Money.MoneyBuilder();
		}
		
		
		public CreditNotations.CreditNotationsBuilder getMinimumRating() {
			return minimumRating;
		}
		
		public CreditNotations.CreditNotationsBuilder getOrCreateMinimumRating() {
			if (minimumRating!=null) {
				return minimumRating;
			}
			else return minimumRating = new CreditNotations.CreditNotationsBuilder();
		}
		
	
		public CustodianTermsBuilder setInitialDesignation(LegalEntity initialDesignation) {
			this.initialDesignation = initialDesignation.toBuilder();
			return this;
		}
		public CustodianTermsBuilder setInitialDesignationBuilder(LegalEntity.LegalEntityBuilder initialDesignation) {
			this.initialDesignation = initialDesignation;
			return this;
		}
		public CustodianTermsBuilder setMinimumAssets(Money minimumAssets) {
			this.minimumAssets = minimumAssets.toBuilder();
			return this;
		}
		public CustodianTermsBuilder setMinimumAssetsBuilder(Money.MoneyBuilder minimumAssets) {
			this.minimumAssets = minimumAssets;
			return this;
		}
		public CustodianTermsBuilder setMinimumRating(CreditNotations minimumRating) {
			this.minimumRating = minimumRating.toBuilder();
			return this;
		}
		public CustodianTermsBuilder setMinimumRatingBuilder(CreditNotations.CreditNotationsBuilder minimumRating) {
			this.minimumRating = minimumRating;
			return this;
		}
		
		public CustodianTerms build() {
			return new CustodianTerms(this);
		}
	
		@Override
		public CustodianTermsBuilder prune() {
			if (initialDesignation!=null && !initialDesignation.prune().hasData()) initialDesignation = null;
			if (minimumAssets!=null && !minimumAssets.prune().hasData()) minimumAssets = null;
			if (minimumRating!=null && !minimumRating.prune().hasData()) minimumRating = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getInitialDesignation()!=null && getInitialDesignation().hasData()) return true;
			if (getMinimumAssets()!=null && getMinimumAssets().hasData()) return true;
			if (getMinimumRating()!=null && getMinimumRating().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("initialDesignation"), processor, LegalEntity.class, initialDesignation);
			processRosetta(path.newSubPath("minimumAssets"), processor, Money.class, minimumAssets);
			processRosetta(path.newSubPath("minimumRating"), processor, CreditNotations.class, minimumRating);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			CustodianTermsBuilder _that = (CustodianTermsBuilder) o;
		
			if (initialDesignation != null ? !initialDesignation.equals(_that.initialDesignation) : _that.initialDesignation != null) return false;
			if (minimumAssets != null ? !minimumAssets.equals(_that.minimumAssets) : _that.minimumAssets != null) return false;
			if (minimumRating != null ? !minimumRating.equals(_that.minimumRating) : _that.minimumRating != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (initialDesignation != null ? initialDesignation.hashCode() : 0);
			_result = 31 * _result + (minimumAssets != null ? minimumAssets.hashCode() : 0);
			_result = 31 * _result + (minimumRating != null ? minimumRating.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "CustodianTermsBuilder {" +
				"initialDesignation=" + this.initialDesignation + ", " +
				"minimumAssets=" + this.minimumAssets + ", " +
				"minimumRating=" + this.minimumRating +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("initialDesignation"), processor, LegalEntity.class, initialDesignation);
		processRosetta(path.newSubPath("minimumAssets"), processor, Money.class, minimumAssets);
		processRosetta(path.newSubPath("minimumRating"), processor, CreditNotations.class, minimumRating);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		CustodianTerms _that = (CustodianTerms) o;
	
		if (initialDesignation != null ? !initialDesignation.equals(_that.initialDesignation) : _that.initialDesignation != null) return false;
		if (minimumAssets != null ? !minimumAssets.equals(_that.minimumAssets) : _that.minimumAssets != null) return false;
		if (minimumRating != null ? !minimumRating.equals(_that.minimumRating) : _that.minimumRating != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (initialDesignation != null ? initialDesignation.hashCode() : 0);
		_result = 31 * _result + (minimumAssets != null ? minimumAssets.hashCode() : 0);
		_result = 31 * _result + (minimumRating != null ? minimumRating.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "CustodianTerms {" +
			"initialDesignation=" + this.initialDesignation + ", " +
			"minimumAssets=" + this.minimumAssets + ", " +
			"minimumRating=" + this.minimumRating +
		'}';
	}
}
