package org.isda.cdm;

import java.lang.Boolean;
import java.lang.String;
import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.CustodianEvent;
import org.isda.cdm.CustodyArrangementsElection;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.CustodyArrangementsMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;

/**
 * A class to specify the Custody Arrangements for the English Law and New York Law ISDA Credit Support Annex. ISDA 2016 Credit Support Annex for Initial Margin, paragraph 13, General Principles, (n): Custody Arrangements.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class CustodyArrangements extends RosettaModelObject {
	private final CustodianEvent custodianEvent;
	private final Boolean hasControlAgreementLanguage;
	private final Boolean isCreditSupportDocument;
	private final String otherProvisions;
	private final List<CustodyArrangementsElection> partyElection;
	private static CustodyArrangementsMeta metaData = new CustodyArrangementsMeta();
	
	CustodyArrangements(CustodyArrangementsBuilder builder) {
		this.custodianEvent = ofNullable(builder.getCustodianEvent()).map(f->f.build()).orElse(null);
		this.hasControlAgreementLanguage = builder.getHasControlAgreementLanguage();
		this.isCreditSupportDocument = builder.getIsCreditSupportDocument();
		this.otherProvisions = builder.getOtherProvisions();
		this.partyElection = ofNullable(builder.getPartyElection()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
	}
	
	/**
	 * When specified as True, means that the Custodian Events specified in Paragraph 13 General Principles, (m)(iii) will constitute an Additional Termination Event. ISDA 2016 Credit Support Annex for Initial Margin, paragraph 13, General Principles, (m)(iii): Custodian Event.
	 */
	public final CustodianEvent getCustodianEvent() {
		return custodianEvent;
	}
	
	/**
	 * Control Agreement language is specified when True.
	 */
	@RosettaSynonym(value="", source="ISDA_Create_1_0")
	public final Boolean getHasControlAgreementLanguage() {
		return hasControlAgreementLanguage;
	}
	
	/**
	 * Unless specified as True, the Control Agreement is not a Credit Support Document under the agreement with respect to a party. ISDA 2016 Credit Support Annex for Initial Margin, paragraph 6 (e): The Control Agreement as a Credit Support Document.
	 */
	@RosettaSynonym(value="", source="ISDA_Create_1_0")
	public final Boolean getIsCreditSupportDocument() {
		return isCreditSupportDocument;
	}
	
	/**
	 * ISDA 2016 Credit Support Annex for Initial Margin, paragraph 13, General Principles, (n)(vii): Other Provisions.
	 */
	@RosettaSynonym(value="specify", source="ISDA_Create_1_0", path="other_provisions")
	public final String getOtherProvisions() {
		return otherProvisions;
	}
	
	/**
	 * The party-specific election with respect to the Custodian and Segregated Account details and the Custodian Risk provision.
	 */
	public final List<CustodyArrangementsElection> getPartyElection() {
		return partyElection;
	}
	
	
	@Override
	public RosettaMetaData<? extends CustodyArrangements> metaData() {
		return metaData;
	} 
	
	public CustodyArrangementsBuilder toBuilder() {
		CustodyArrangementsBuilder builder = new CustodyArrangementsBuilder();
		ofNullable(getCustodianEvent()).ifPresent(builder::setCustodianEvent);
		ofNullable(getHasControlAgreementLanguage()).ifPresent(builder::setHasControlAgreementLanguage);
		ofNullable(getIsCreditSupportDocument()).ifPresent(builder::setIsCreditSupportDocument);
		ofNullable(getOtherProvisions()).ifPresent(builder::setOtherProvisions);
		ofNullable(getPartyElection()).ifPresent(partyElection -> partyElection.forEach(builder::addPartyElection));
		return builder;
	}

	public static CustodyArrangementsBuilder builder() {
		return new CustodyArrangementsBuilder();
	}

	public static  class CustodyArrangementsBuilder extends RosettaModelObjectBuilder {
	
		protected CustodianEvent.CustodianEventBuilder custodianEvent;
		protected Boolean hasControlAgreementLanguage;
		protected Boolean isCreditSupportDocument;
		protected String otherProvisions;
		protected List<CustodyArrangementsElection.CustodyArrangementsElectionBuilder> partyElection;
	
		public CustodyArrangementsBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends CustodyArrangements> metaData() {
			return metaData;
		} 
	
		
		public CustodianEvent.CustodianEventBuilder getCustodianEvent() {
			return custodianEvent;
		}
		
		public CustodianEvent.CustodianEventBuilder getOrCreateCustodianEvent() {
			if (custodianEvent!=null) {
				return custodianEvent;
			}
			else return custodianEvent = new CustodianEvent.CustodianEventBuilder();
		}
		
		
		public Boolean getHasControlAgreementLanguage() {
			return hasControlAgreementLanguage;
		}
		
		
		public Boolean getIsCreditSupportDocument() {
			return isCreditSupportDocument;
		}
		
		
		public String getOtherProvisions() {
			return otherProvisions;
		}
		
		
		public List<CustodyArrangementsElection.CustodyArrangementsElectionBuilder> getPartyElection() {
			return partyElection;
		}
		
		public CustodyArrangementsElection.CustodyArrangementsElectionBuilder getOrCreatePartyElection(int index) {
			if (partyElection==null) {
				this.partyElection = new ArrayList<>();
			}
			return getIndex(partyElection, index, ()->new CustodyArrangementsElection.CustodyArrangementsElectionBuilder());
		}
		
	
		public CustodyArrangementsBuilder setCustodianEvent(CustodianEvent custodianEvent) {
			this.custodianEvent = custodianEvent.toBuilder();
			return this;
		}
		public CustodyArrangementsBuilder setCustodianEventBuilder(CustodianEvent.CustodianEventBuilder custodianEvent) {
			this.custodianEvent = custodianEvent;
			return this;
		}
		public CustodyArrangementsBuilder setHasControlAgreementLanguage(Boolean hasControlAgreementLanguage) {
			this.hasControlAgreementLanguage = hasControlAgreementLanguage;
			return this;
		}
		public CustodyArrangementsBuilder setIsCreditSupportDocument(Boolean isCreditSupportDocument) {
			this.isCreditSupportDocument = isCreditSupportDocument;
			return this;
		}
		public CustodyArrangementsBuilder setOtherProvisions(String otherProvisions) {
			this.otherProvisions = otherProvisions;
			return this;
		}
		public CustodyArrangementsBuilder addPartyElection(CustodyArrangementsElection partyElection) {
			if(this.partyElection == null){
				this.partyElection = new ArrayList<>();
			}
			this.partyElection.add(partyElection.toBuilder());
			return this;
		}
		public CustodyArrangementsBuilder addPartyElection(CustodyArrangementsElection partyElection, int _idx) {
			if(this.partyElection == null){
				this.partyElection = new ArrayList<>();
			}
			getIndex(this.partyElection, _idx, () -> partyElection.toBuilder());
			this.partyElection.set(_idx, partyElection.toBuilder());
			return this;
		}
		public CustodyArrangementsBuilder addPartyElection(List<CustodyArrangementsElection> partyElections) {
			if(this.partyElection == null){
				this.partyElection = new ArrayList<>();
			}
			for (CustodyArrangementsElection toAdd : partyElections) {
				this.partyElection.add(toAdd.toBuilder());
			}
			return this;
		}
		public CustodyArrangementsBuilder addPartyElectionBuilder(CustodyArrangementsElection.CustodyArrangementsElectionBuilder partyElection) {
			if(this.partyElection == null){
				this.partyElection = new ArrayList<>();
			}
			this.partyElection.add(partyElection);
			return this;
		}
		
		public CustodyArrangementsBuilder clearPartyElection() {
			this.partyElection = null;
			return this;
		}
		
		public CustodyArrangements build() {
			return new CustodyArrangements(this);
		}
	
		@Override
		public CustodyArrangementsBuilder prune() {
			if (custodianEvent!=null && !custodianEvent.prune().hasData()) custodianEvent = null;
			if (partyElection!=null) partyElection = partyElection.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getCustodianEvent()!=null && getCustodianEvent().hasData()) return true;
			if (getHasControlAgreementLanguage()!=null) return true;
			if (getIsCreditSupportDocument()!=null) return true;
			if (getOtherProvisions()!=null) return true;
			if (getPartyElection()!=null && getPartyElection().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("hasControlAgreementLanguage"), Boolean.class, hasControlAgreementLanguage, this);
			processor.processBasic(path.newSubPath("isCreditSupportDocument"), Boolean.class, isCreditSupportDocument, this);
			processor.processBasic(path.newSubPath("otherProvisions"), String.class, otherProvisions, this);
			
			processRosetta(path.newSubPath("custodianEvent"), processor, CustodianEvent.class, custodianEvent);
			processRosetta(path.newSubPath("partyElection"), processor, CustodyArrangementsElection.class, partyElection);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			CustodyArrangementsBuilder _that = (CustodyArrangementsBuilder) o;
		
			if (custodianEvent != null ? !custodianEvent.equals(_that.custodianEvent) : _that.custodianEvent != null) return false;
			if (hasControlAgreementLanguage != null ? !hasControlAgreementLanguage.equals(_that.hasControlAgreementLanguage) : _that.hasControlAgreementLanguage != null) return false;
			if (isCreditSupportDocument != null ? !isCreditSupportDocument.equals(_that.isCreditSupportDocument) : _that.isCreditSupportDocument != null) return false;
			if (otherProvisions != null ? !otherProvisions.equals(_that.otherProvisions) : _that.otherProvisions != null) return false;
			if (!ListEquals.listEquals(partyElection, _that.partyElection)) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (custodianEvent != null ? custodianEvent.hashCode() : 0);
			_result = 31 * _result + (hasControlAgreementLanguage != null ? hasControlAgreementLanguage.hashCode() : 0);
			_result = 31 * _result + (isCreditSupportDocument != null ? isCreditSupportDocument.hashCode() : 0);
			_result = 31 * _result + (otherProvisions != null ? otherProvisions.hashCode() : 0);
			_result = 31 * _result + (partyElection != null ? partyElection.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "CustodyArrangementsBuilder {" +
				"custodianEvent=" + this.custodianEvent + ", " +
				"hasControlAgreementLanguage=" + this.hasControlAgreementLanguage + ", " +
				"isCreditSupportDocument=" + this.isCreditSupportDocument + ", " +
				"otherProvisions=" + this.otherProvisions + ", " +
				"partyElection=" + this.partyElection +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("hasControlAgreementLanguage"), Boolean.class, hasControlAgreementLanguage, this);
		processor.processBasic(path.newSubPath("isCreditSupportDocument"), Boolean.class, isCreditSupportDocument, this);
		processor.processBasic(path.newSubPath("otherProvisions"), String.class, otherProvisions, this);
		
		processRosetta(path.newSubPath("custodianEvent"), processor, CustodianEvent.class, custodianEvent);
		processRosetta(path.newSubPath("partyElection"), processor, CustodyArrangementsElection.class, partyElection);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		CustodyArrangements _that = (CustodyArrangements) o;
	
		if (custodianEvent != null ? !custodianEvent.equals(_that.custodianEvent) : _that.custodianEvent != null) return false;
		if (hasControlAgreementLanguage != null ? !hasControlAgreementLanguage.equals(_that.hasControlAgreementLanguage) : _that.hasControlAgreementLanguage != null) return false;
		if (isCreditSupportDocument != null ? !isCreditSupportDocument.equals(_that.isCreditSupportDocument) : _that.isCreditSupportDocument != null) return false;
		if (otherProvisions != null ? !otherProvisions.equals(_that.otherProvisions) : _that.otherProvisions != null) return false;
		if (!ListEquals.listEquals(partyElection, _that.partyElection)) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (custodianEvent != null ? custodianEvent.hashCode() : 0);
		_result = 31 * _result + (hasControlAgreementLanguage != null ? hasControlAgreementLanguage.hashCode() : 0);
		_result = 31 * _result + (isCreditSupportDocument != null ? isCreditSupportDocument.hashCode() : 0);
		_result = 31 * _result + (otherProvisions != null ? otherProvisions.hashCode() : 0);
		_result = 31 * _result + (partyElection != null ? partyElection.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "CustodyArrangements {" +
			"custodianEvent=" + this.custodianEvent + ", " +
			"hasControlAgreementLanguage=" + this.hasControlAgreementLanguage + ", " +
			"isCreditSupportDocument=" + this.isCreditSupportDocument + ", " +
			"otherProvisions=" + this.otherProvisions + ", " +
			"partyElection=" + this.partyElection +
		'}';
	}
}
