package org.isda.cdm;

import org.isda.cdm.Account;
import org.isda.cdm.CustodianRisk;
import org.isda.cdm.LegalEntity;
import org.isda.cdm.metafields.ReferenceWithMetaParty;
import org.isda.cdm.metafields.ReferenceWithMetaParty.ReferenceWithMetaPartyBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.CustodyArrangementsElectionMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.stream.Collectors;
		import static java.util.Optional.ofNullable;

/**
 * A class to specify the Custody Arrangements election by each party to the agreement: custody agent and account(s) identification, as well as custodian risk qualification.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class CustodyArrangementsElection extends RosettaModelObject {
	private final LegalEntity custodian;
	private final CustodianRisk custodianRisk;
	private final ReferenceWithMetaParty party;
	private final Account segregatedCashAccount;
	private final Account segregatedSecurityAccount;
	private static CustodyArrangementsElectionMeta metaData = new CustodyArrangementsElectionMeta();
	
	CustodyArrangementsElection(CustodyArrangementsElectionBuilder builder) {
		this.custodian = ofNullable(builder.getCustodian()).map(f->f.build()).orElse(null);
		this.custodianRisk = ofNullable(builder.getCustodianRisk()).map(f->f.build()).orElse(null);
		this.party = ofNullable(builder.getParty()).map(f->f.build()).orElse(null);
		this.segregatedCashAccount = ofNullable(builder.getSegregatedCashAccount()).map(f->f.build()).orElse(null);
		this.segregatedSecurityAccount = ofNullable(builder.getSegregatedSecurityAccount()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The custody agent. While ISDA Create only specifies the custodian&#39;s name, specifying the legal entity as part of the CDM is deemed more appropriate, while this will still provide the ability to accommodate situations where only the entity name is available, as the entityId attribute is optional as part of the LegalEntity class.
	 */
	public final LegalEntity getCustodian() {
		return custodian;
	}
	
	/**
	 * The qualification of the Custodian Risk. ISDA 2016 Credit Support Annex for Initial Margin, paragraph 13, General Principles, (n)(ii): Custodian (IM) Risk.
	 */
	@RosettaSynonym(value="custodian_risk", source="ISDA_Create_1_0")
	public final CustodianRisk getCustodianRisk() {
		return custodianRisk;
	}
	
	/**
	 * The elective party.
	 */
	public final ReferenceWithMetaParty getParty() {
		return party;
	}
	
	/**
	 * The identification of the segregated cash account for the purpose of holding cash collateral.
	 */
	public final Account getSegregatedCashAccount() {
		return segregatedCashAccount;
	}
	
	/**
	 * The identification of the segregated security account for the purpose of holding security collateral.
	 */
	public final Account getSegregatedSecurityAccount() {
		return segregatedSecurityAccount;
	}
	
	
	@Override
	public RosettaMetaData<? extends CustodyArrangementsElection> metaData() {
		return metaData;
	} 
	
	public CustodyArrangementsElectionBuilder toBuilder() {
		CustodyArrangementsElectionBuilder builder = new CustodyArrangementsElectionBuilder();
		ofNullable(getCustodian()).ifPresent(builder::setCustodian);
		ofNullable(getCustodianRisk()).ifPresent(builder::setCustodianRisk);
		ofNullable(getParty()).ifPresent(builder::setParty);
		ofNullable(getSegregatedCashAccount()).ifPresent(builder::setSegregatedCashAccount);
		ofNullable(getSegregatedSecurityAccount()).ifPresent(builder::setSegregatedSecurityAccount);
		return builder;
	}

	public static CustodyArrangementsElectionBuilder builder() {
		return new CustodyArrangementsElectionBuilder();
	}

	public static  class CustodyArrangementsElectionBuilder extends RosettaModelObjectBuilder {
	
		protected LegalEntity.LegalEntityBuilder custodian;
		protected CustodianRisk.CustodianRiskBuilder custodianRisk;
		protected ReferenceWithMetaPartyBuilder party;
		protected Account.AccountBuilder segregatedCashAccount;
		protected Account.AccountBuilder segregatedSecurityAccount;
	
		public CustodyArrangementsElectionBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends CustodyArrangementsElection> metaData() {
			return metaData;
		} 
	
		
		public LegalEntity.LegalEntityBuilder getCustodian() {
			return custodian;
		}
		
		public LegalEntity.LegalEntityBuilder getOrCreateCustodian() {
			if (custodian!=null) {
				return custodian;
			}
			else return custodian = new LegalEntity.LegalEntityBuilder();
		}
		
		
		public CustodianRisk.CustodianRiskBuilder getCustodianRisk() {
			return custodianRisk;
		}
		
		public CustodianRisk.CustodianRiskBuilder getOrCreateCustodianRisk() {
			if (custodianRisk!=null) {
				return custodianRisk;
			}
			else return custodianRisk = new CustodianRisk.CustodianRiskBuilder();
		}
		
		
		public ReferenceWithMetaPartyBuilder getParty() {
			return party;
		}
		
		public ReferenceWithMetaPartyBuilder getOrCreateParty() {
			if (party!=null) {
				return party;
			}
			else return party = new ReferenceWithMetaPartyBuilder();
		}
		
		
		public Account.AccountBuilder getSegregatedCashAccount() {
			return segregatedCashAccount;
		}
		
		public Account.AccountBuilder getOrCreateSegregatedCashAccount() {
			if (segregatedCashAccount!=null) {
				return segregatedCashAccount;
			}
			else return segregatedCashAccount = new Account.AccountBuilder();
		}
		
		
		public Account.AccountBuilder getSegregatedSecurityAccount() {
			return segregatedSecurityAccount;
		}
		
		public Account.AccountBuilder getOrCreateSegregatedSecurityAccount() {
			if (segregatedSecurityAccount!=null) {
				return segregatedSecurityAccount;
			}
			else return segregatedSecurityAccount = new Account.AccountBuilder();
		}
		
	
		public CustodyArrangementsElectionBuilder setCustodian(LegalEntity custodian) {
			this.custodian = custodian.toBuilder();
			return this;
		}
		public CustodyArrangementsElectionBuilder setCustodianBuilder(LegalEntity.LegalEntityBuilder custodian) {
			this.custodian = custodian;
			return this;
		}
		public CustodyArrangementsElectionBuilder setCustodianRisk(CustodianRisk custodianRisk) {
			this.custodianRisk = custodianRisk.toBuilder();
			return this;
		}
		public CustodyArrangementsElectionBuilder setCustodianRiskBuilder(CustodianRisk.CustodianRiskBuilder custodianRisk) {
			this.custodianRisk = custodianRisk;
			return this;
		}
		public CustodyArrangementsElectionBuilder setParty(ReferenceWithMetaParty party) {
			this.party = party.toBuilder();
			return this;
		}
		public CustodyArrangementsElectionBuilder setPartyBuilder(ReferenceWithMetaPartyBuilder party) {
			this.party = party;
			return this;
		}
		public CustodyArrangementsElectionBuilder setPartyRef(Party.PartyBuilder party) {
			return setParty(ReferenceWithMetaParty.builder().setValueBuilder(party).build());
		}
		public CustodyArrangementsElectionBuilder setPartyRef(Party party) {
			return setPartyRef(Party.builder());
		}
		public CustodyArrangementsElectionBuilder setSegregatedCashAccount(Account segregatedCashAccount) {
			this.segregatedCashAccount = segregatedCashAccount.toBuilder();
			return this;
		}
		public CustodyArrangementsElectionBuilder setSegregatedCashAccountBuilder(Account.AccountBuilder segregatedCashAccount) {
			this.segregatedCashAccount = segregatedCashAccount;
			return this;
		}
		public CustodyArrangementsElectionBuilder setSegregatedSecurityAccount(Account segregatedSecurityAccount) {
			this.segregatedSecurityAccount = segregatedSecurityAccount.toBuilder();
			return this;
		}
		public CustodyArrangementsElectionBuilder setSegregatedSecurityAccountBuilder(Account.AccountBuilder segregatedSecurityAccount) {
			this.segregatedSecurityAccount = segregatedSecurityAccount;
			return this;
		}
		
		public CustodyArrangementsElection build() {
			return new CustodyArrangementsElection(this);
		}
	
		@Override
		public CustodyArrangementsElectionBuilder prune() {
			if (custodian!=null && !custodian.prune().hasData()) custodian = null;
			if (custodianRisk!=null && !custodianRisk.prune().hasData()) custodianRisk = null;
			if (party!=null && !party.prune().hasData()) party = null;
			if (segregatedCashAccount!=null && !segregatedCashAccount.prune().hasData()) segregatedCashAccount = null;
			if (segregatedSecurityAccount!=null && !segregatedSecurityAccount.prune().hasData()) segregatedSecurityAccount = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getCustodian()!=null && getCustodian().hasData()) return true;
			if (getCustodianRisk()!=null && getCustodianRisk().hasData()) return true;
			if (getParty()!=null && getParty().hasData()) return true;
			if (getSegregatedCashAccount()!=null && getSegregatedCashAccount().hasData()) return true;
			if (getSegregatedSecurityAccount()!=null && getSegregatedSecurityAccount().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("custodian"), processor, LegalEntity.class, custodian);
			processRosetta(path.newSubPath("custodianRisk"), processor, CustodianRisk.class, custodianRisk);
			processRosetta(path.newSubPath("party"), processor, ReferenceWithMetaParty.class, party);
			processRosetta(path.newSubPath("segregatedCashAccount"), processor, Account.class, segregatedCashAccount);
			processRosetta(path.newSubPath("segregatedSecurityAccount"), processor, Account.class, segregatedSecurityAccount);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			CustodyArrangementsElectionBuilder _that = (CustodyArrangementsElectionBuilder) o;
		
			if (custodian != null ? !custodian.equals(_that.custodian) : _that.custodian != null) return false;
			if (custodianRisk != null ? !custodianRisk.equals(_that.custodianRisk) : _that.custodianRisk != null) return false;
			if (party != null ? !party.equals(_that.party) : _that.party != null) return false;
			if (segregatedCashAccount != null ? !segregatedCashAccount.equals(_that.segregatedCashAccount) : _that.segregatedCashAccount != null) return false;
			if (segregatedSecurityAccount != null ? !segregatedSecurityAccount.equals(_that.segregatedSecurityAccount) : _that.segregatedSecurityAccount != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (custodian != null ? custodian.hashCode() : 0);
			_result = 31 * _result + (custodianRisk != null ? custodianRisk.hashCode() : 0);
			_result = 31 * _result + (party != null ? party.hashCode() : 0);
			_result = 31 * _result + (segregatedCashAccount != null ? segregatedCashAccount.hashCode() : 0);
			_result = 31 * _result + (segregatedSecurityAccount != null ? segregatedSecurityAccount.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "CustodyArrangementsElectionBuilder {" +
				"custodian=" + this.custodian + ", " +
				"custodianRisk=" + this.custodianRisk + ", " +
				"party=" + this.party + ", " +
				"segregatedCashAccount=" + this.segregatedCashAccount + ", " +
				"segregatedSecurityAccount=" + this.segregatedSecurityAccount +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("custodian"), processor, LegalEntity.class, custodian);
		processRosetta(path.newSubPath("custodianRisk"), processor, CustodianRisk.class, custodianRisk);
		processRosetta(path.newSubPath("party"), processor, ReferenceWithMetaParty.class, party);
		processRosetta(path.newSubPath("segregatedCashAccount"), processor, Account.class, segregatedCashAccount);
		processRosetta(path.newSubPath("segregatedSecurityAccount"), processor, Account.class, segregatedSecurityAccount);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		CustodyArrangementsElection _that = (CustodyArrangementsElection) o;
	
		if (custodian != null ? !custodian.equals(_that.custodian) : _that.custodian != null) return false;
		if (custodianRisk != null ? !custodianRisk.equals(_that.custodianRisk) : _that.custodianRisk != null) return false;
		if (party != null ? !party.equals(_that.party) : _that.party != null) return false;
		if (segregatedCashAccount != null ? !segregatedCashAccount.equals(_that.segregatedCashAccount) : _that.segregatedCashAccount != null) return false;
		if (segregatedSecurityAccount != null ? !segregatedSecurityAccount.equals(_that.segregatedSecurityAccount) : _that.segregatedSecurityAccount != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (custodian != null ? custodian.hashCode() : 0);
		_result = 31 * _result + (custodianRisk != null ? custodianRisk.hashCode() : 0);
		_result = 31 * _result + (party != null ? party.hashCode() : 0);
		_result = 31 * _result + (segregatedCashAccount != null ? segregatedCashAccount.hashCode() : 0);
		_result = 31 * _result + (segregatedSecurityAccount != null ? segregatedSecurityAccount.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "CustodyArrangementsElection {" +
			"custodian=" + this.custodian + ", " +
			"custodianRisk=" + this.custodianRisk + ", " +
			"party=" + this.party + ", " +
			"segregatedCashAccount=" + this.segregatedCashAccount + ", " +
			"segregatedSecurityAccount=" + this.segregatedSecurityAccount +
		'}';
	}
}
