package org.isda.cdm;

import java.lang.Boolean;
import java.lang.String;
import org.isda.cdm.LoanParticipation;
import org.isda.cdm.NotDomesticCurrency;
import org.isda.cdm.ObligationCategoryEnum;
import org.isda.cdm.PCDeliverableObligationCharac;
import org.isda.cdm.Period;
import org.isda.cdm.SpecifiedCurrency;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.DeliverableObligationsMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class to specify all the ISDA terms relevant to defining the deliverable obligations.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="DeliverableObligations", source="FpML_5_10")
public class DeliverableObligations extends RosettaModelObject {
	private final Boolean acceleratedOrMatured;
	private final Boolean accruedInterest;
	private final PCDeliverableObligationCharac assignableLoan;
	private final ObligationCategoryEnum category;
	private final PCDeliverableObligationCharac consentRequiredLoan;
	private final LoanParticipation directLoanParticipation;
	private final String excluded;
	private final Boolean fullFaithAndCreditObLiability;
	private final Boolean generalFundObligationLiability;
	private final LoanParticipation indirectLoanParticipation;
	private final Boolean listed;
	private final Period maximumMaturity;
	private final Boolean notBearer;
	private final Boolean notContingent;
	private final NotDomesticCurrency notDomesticCurrency;
	private final Boolean notDomesticIssuance;
	private final Boolean notDomesticLaw;
	private final Boolean notSovereignLender;
	private final Boolean notSubordinated;
	private final String othReferenceEntityObligations;
	private final Boolean revenueObligationLiability;
	private final SpecifiedCurrency specifiedCurrency;
	private final Boolean transferable;
	private static DeliverableObligationsMeta metaData = new DeliverableObligationsMeta();
	
	DeliverableObligations(DeliverableObligationsBuilder builder) {
		this.acceleratedOrMatured = builder.getAcceleratedOrMatured();
		this.accruedInterest = builder.getAccruedInterest();
		this.assignableLoan = ofNullable(builder.getAssignableLoan()).map(f->f.build()).orElse(null);
		this.category = builder.getCategory();
		this.consentRequiredLoan = ofNullable(builder.getConsentRequiredLoan()).map(f->f.build()).orElse(null);
		this.directLoanParticipation = ofNullable(builder.getDirectLoanParticipation()).map(f->f.build()).orElse(null);
		this.excluded = builder.getExcluded();
		this.fullFaithAndCreditObLiability = builder.getFullFaithAndCreditObLiability();
		this.generalFundObligationLiability = builder.getGeneralFundObligationLiability();
		this.indirectLoanParticipation = ofNullable(builder.getIndirectLoanParticipation()).map(f->f.build()).orElse(null);
		this.listed = builder.getListed();
		this.maximumMaturity = ofNullable(builder.getMaximumMaturity()).map(f->f.build()).orElse(null);
		this.notBearer = builder.getNotBearer();
		this.notContingent = builder.getNotContingent();
		this.notDomesticCurrency = ofNullable(builder.getNotDomesticCurrency()).map(f->f.build()).orElse(null);
		this.notDomesticIssuance = builder.getNotDomesticIssuance();
		this.notDomesticLaw = builder.getNotDomesticLaw();
		this.notSovereignLender = builder.getNotSovereignLender();
		this.notSubordinated = builder.getNotSubordinated();
		this.othReferenceEntityObligations = builder.getOthReferenceEntityObligations();
		this.revenueObligationLiability = builder.getRevenueObligationLiability();
		this.specifiedCurrency = ofNullable(builder.getSpecifiedCurrency()).map(f->f.build()).orElse(null);
		this.transferable = builder.getTransferable();
	}
	
	/**
	 * A deliverable obligation characteristic. An obligation at time of default is due to mature and due to be repaid, or as a result of downgrade/bankruptcy is due to be repaid as a result of an acceleration clause. ISDA 2003 Term: Accelerated or Matured.
	 */
	@RosettaSynonym(value="acceleratedOrMatured", source="FpML_5_10")
	@RosettaSynonym(value="acceleratedOrMatured", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="acceleratedOrMatured", source="DTCC_11_0")
	@RosettaSynonym(value="acceleratedOrMatured", source="DTCC_9_0")
	@RosettaSynonym(value="acceleratedOrMatured", source="CME_ClearedConfirm_1_17")
	public final Boolean getAcceleratedOrMatured() {
		return acceleratedOrMatured;
	}
	
	/**
	 * Indicates whether accrued interest is included (true) or not (false). For cash settlement this specifies whether quotations should be obtained inclusive or not of accrued interest. For physical settlement this specifies whether the buyer should deliver the obligation with an outstanding principal balance that includes or excludes accrued interest. ISDA 2003 Term: Include/Exclude Accrued Interest.
	 */
	@RosettaSynonym(value="accruedInterest", source="FpML_5_10")
	@RosettaSynonym(value="accruedInterest", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="accruedInterest", source="DTCC_11_0")
	@RosettaSynonym(value="accruedInterest", source="DTCC_9_0")
	@RosettaSynonym(value="accruedInterest", source="CME_ClearedConfirm_1_17")
	public final Boolean getAccruedInterest() {
		return accruedInterest;
	}
	
	/**
	 * A deliverable obligation characteristic. A loan that is freely assignable to a bank or financial institution without the consent of the Reference Entity or the guarantor, if any, of the loan (or the consent of the applicable borrower if a Reference Entity is guaranteeing the loan) or any agent. ISDA 2003 Term: Assignable Loan.
	 */
	@RosettaSynonym(value="assignableLoan", source="FpML_5_10")
	@RosettaSynonym(value="assignableLoan", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="assignableLoan", source="DTCC_11_0")
	@RosettaSynonym(value="assignableLoan", source="DTCC_9_0")
	@RosettaSynonym(value="assignableLoan", source="CME_ClearedConfirm_1_17")
	public final PCDeliverableObligationCharac getAssignableLoan() {
		return assignableLoan;
	}
	
	/**
	 * Used in both obligations and deliverable obligations to represent a class or type of securities which apply. ISDA 2003 Term: Obligation Category/Deliverable Obligation Category.
	 */
	@RosettaSynonym(value="category", source="FpML_5_10")
	@RosettaSynonym(value="category", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="category", source="DTCC_11_0")
	@RosettaSynonym(value="category", source="DTCC_9_0")
	@RosettaSynonym(value="category", source="CME_ClearedConfirm_1_17")
	public final ObligationCategoryEnum getCategory() {
		return category;
	}
	
	/**
	 * A deliverable obligation characteristic. A loan that is capable of being assigned with the consent of the Reference Entity or the guarantor, if any, of the loan or any agent. ISDA 2003 Term: Consent Required Loan.
	 */
	@RosettaSynonym(value="consentRequiredLoan", source="FpML_5_10")
	@RosettaSynonym(value="consentRequiredLoan", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="consentRequiredLoan", source="DTCC_11_0")
	@RosettaSynonym(value="consentRequiredLoan", source="DTCC_9_0")
	@RosettaSynonym(value="consentRequiredLoan", source="CME_ClearedConfirm_1_17")
	public final PCDeliverableObligationCharac getConsentRequiredLoan() {
		return consentRequiredLoan;
	}
	
	/**
	 * A deliverable obligation characteristic. A loan with a participation agreement whereby the buyer is capable of creating, or procuring the creation of, a contractual right in favour of the seller that provides the seller with recourse to the participation seller for a specified share in any payments due under the relevant loan which are received by the participation seller. ISDA 2003 Term: Direct Loan Participation.
	 */
	@RosettaSynonym(value="directLoanParticipation", source="FpML_5_10")
	@RosettaSynonym(value="directLoanParticipation", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="directLoanParticipation", source="DTCC_11_0")
	@RosettaSynonym(value="directLoanParticipation", source="DTCC_9_0")
	@RosettaSynonym(value="directLoanParticipation", source="CME_ClearedConfirm_1_17")
	public final LoanParticipation getDirectLoanParticipation() {
		return directLoanParticipation;
	}
	
	/**
	 * A free format string to specify any excluded obligations or deliverable obligations, as the case may be, of the reference entity or excluded types of obligations or deliverable obligations. ISDA 2003 Term: Excluded Obligations/Excluded Deliverable Obligations.
	 */
	@RosettaSynonym(value="excluded", source="FpML_5_10")
	@RosettaSynonym(value="excluded", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="excluded", source="DTCC_11_0")
	@RosettaSynonym(value="excluded", source="DTCC_9_0")
	@RosettaSynonym(value="excluded", source="CME_ClearedConfirm_1_17")
	public final String getExcluded() {
		return excluded;
	}
	
	/**
	 * An obligation and deliverable obligation characteristic. Defined in the ISDA published additional provisions for U.S. Municipal as Reference Entity. ISDA 2003 Term: Full Faith and Credit Obligation Liability.
	 */
	@RosettaSynonym(value="fullFaithAndCreditObLiability", source="FpML_5_10")
	@RosettaSynonym(value="fullFaithAndCreditObLiability", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="fullFaithAndCreditObLiability", source="DTCC_11_0")
	@RosettaSynonym(value="fullFaithAndCreditObLiability", source="DTCC_9_0")
	@RosettaSynonym(value="fullFaithAndCreditObLiability", source="CME_ClearedConfirm_1_17")
	public final Boolean getFullFaithAndCreditObLiability() {
		return fullFaithAndCreditObLiability;
	}
	
	/**
	 * An obligation and deliverable obligation characteristic. Defined in the ISDA published additional provisions for U.S. Municipal as Reference Entity. ISDA 2003 Term: General Fund Obligation Liability.
	 */
	@RosettaSynonym(value="generalFundObligationLiability", source="FpML_5_10")
	@RosettaSynonym(value="generalFundObligationLiability", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="generalFundObligationLiability", source="DTCC_11_0")
	@RosettaSynonym(value="generalFundObligationLiability", source="DTCC_9_0")
	@RosettaSynonym(value="generalFundObligationLiability", source="CME_ClearedConfirm_1_17")
	public final Boolean getGeneralFundObligationLiability() {
		return generalFundObligationLiability;
	}
	
	/**
	 * ISDA 1999 Term: Indirect Loan Participation. NOTE: Only applicable as a deliverable obligation under ISDA Credit 1999.
	 */
	@RosettaSynonym(value="indirectLoanParticipation", source="FpML_5_10")
	@RosettaSynonym(value="indirectLoanParticipation", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="indirectLoanParticipation", source="DTCC_11_0")
	@RosettaSynonym(value="indirectLoanParticipation", source="DTCC_9_0")
	@RosettaSynonym(value="indirectLoanParticipation", source="CME_ClearedConfirm_1_17")
	public final LoanParticipation getIndirectLoanParticipation() {
		return indirectLoanParticipation;
	}
	
	/**
	 * An obligation and deliverable obligation characteristic. Indicates whether or not the obligation is quoted, listed or ordinarily purchased and sold on an exchange. ISDA 2003 Term: Listed.
	 */
	@RosettaSynonym(value="listed", source="FpML_5_10")
	@RosettaSynonym(value="listed", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="listed", source="DTCC_11_0")
	@RosettaSynonym(value="listed", source="DTCC_9_0")
	@RosettaSynonym(value="listed", source="CME_ClearedConfirm_1_17")
	public final Boolean getListed() {
		return listed;
	}
	
	/**
	 * A deliverable obligation characteristic. An obligation that has a remaining maturity from the Physical Settlement Date of not greater than the period specified. ISDA 2003 Term: Maximum Maturity.
	 */
	@RosettaSynonym(value="maximumMaturity", source="FpML_5_10")
	@RosettaSynonym(value="maximumMaturity", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="maximumMaturity", source="DTCC_11_0")
	@RosettaSynonym(value="maximumMaturity", source="DTCC_9_0")
	@RosettaSynonym(value="maximumMaturity", source="CME_ClearedConfirm_1_17")
	public final Period getMaximumMaturity() {
		return maximumMaturity;
	}
	
	/**
	 * A deliverable obligation characteristic. Any obligation that is not a bearer instrument. This applies to Bonds only and is meant to avoid tax, fraud and security/delivery provisions that can potentially be associated with Bearer Bonds. ISDA 2003 Term: Not Bearer.
	 */
	@RosettaSynonym(value="notBearer", source="FpML_5_10")
	@RosettaSynonym(value="notBearer", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="notBearer", source="DTCC_11_0")
	@RosettaSynonym(value="notBearer", source="DTCC_9_0")
	@RosettaSynonym(value="notBearer", source="CME_ClearedConfirm_1_17")
	public final Boolean getNotBearer() {
		return notBearer;
	}
	
	/**
	 * A deliverable obligation characteristic. In essence Not Contingent means the repayment of principal cannot be dependant on a formula/index, i.e. to prevent the risk of being delivered an instrument that may never pay any element of principal, and to ensure that the obligation is interest bearing (on a regular schedule). ISDA 2003 Term: Not Contingent.
	 */
	@RosettaSynonym(value="notContingent", source="FpML_5_10")
	@RosettaSynonym(value="notContingent", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="notContingent", source="DTCC_11_0")
	@RosettaSynonym(value="notContingent", source="DTCC_9_0")
	@RosettaSynonym(value="notContingent", source="CME_ClearedConfirm_1_17")
	public final Boolean getNotContingent() {
		return notContingent;
	}
	
	/**
	 * An obligation and deliverable obligation characteristic. Any obligation that is payable in any currency other than the domestic currency. Domestic currency is either the currency so specified or, if no currency is specified, the currency of (a) the reference entity, if the reference entity is a sovereign, or (b) the jurisdiction in which the relevant reference entity is organised, if the reference entity is not a sovereign. ISDA 2003 Term: Not Domestic Currency.
	 */
	@RosettaSynonym(value="notDomesticCurrency", source="FpML_5_10")
	@RosettaSynonym(value="notDomesticCurrency", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="notDomesticCurrency", source="DTCC_11_0")
	@RosettaSynonym(value="notDomesticCurrency", source="DTCC_9_0")
	@RosettaSynonym(value="notDomesticCurrency", source="CME_ClearedConfirm_1_17")
	public final NotDomesticCurrency getNotDomesticCurrency() {
		return notDomesticCurrency;
	}
	
	/**
	 * An obligation and deliverable obligation characteristic. Any obligation other than an obligation that was intended to be offered for sale primarily in the domestic market of the relevant Reference Entity. This specifies that the obligation must be an internationally recognised bond. ISDA 2003 Term: Not Domestic Issuance.
	 */
	@RosettaSynonym(value="notDomesticIssuance", source="FpML_5_10")
	@RosettaSynonym(value="notDomesticIssuance", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="notDomesticIssuance", source="DTCC_11_0")
	@RosettaSynonym(value="notDomesticIssuance", source="DTCC_9_0")
	@RosettaSynonym(value="notDomesticIssuance", source="CME_ClearedConfirm_1_17")
	public final Boolean getNotDomesticIssuance() {
		return notDomesticIssuance;
	}
	
	/**
	 * An obligation and deliverable obligation characteristic. If the reference entity is a Sovereign, this means any obligation that is not subject to the laws of the reference entity. If the reference entity is not a sovereign, this means any obligation that is not subject to the laws of the jurisdiction of the reference entity. ISDA 2003 Term: Not Domestic Law.
	 */
	@RosettaSynonym(value="notDomesticLaw", source="FpML_5_10")
	@RosettaSynonym(value="notDomesticLaw", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="notDomesticLaw", source="DTCC_11_0")
	@RosettaSynonym(value="notDomesticLaw", source="DTCC_9_0")
	@RosettaSynonym(value="notDomesticLaw", source="CME_ClearedConfirm_1_17")
	public final Boolean getNotDomesticLaw() {
		return notDomesticLaw;
	}
	
	/**
	 * An obligation and deliverable obligation characteristic. Any obligation that is not primarily (majority) owed to a Sovereign or Supranational Organisation. ISDA 2003 Term: Not Sovereign Lender.
	 */
	@RosettaSynonym(value="notSovereignLender", source="FpML_5_10")
	@RosettaSynonym(value="notSovereignLender", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="notSovereignLender", source="DTCC_11_0")
	@RosettaSynonym(value="notSovereignLender", source="DTCC_9_0")
	@RosettaSynonym(value="notSovereignLender", source="CME_ClearedConfirm_1_17")
	public final Boolean getNotSovereignLender() {
		return notSovereignLender;
	}
	
	/**
	 * An obligation and deliverable obligation characteristic. An obligation that ranks at least equal with the most senior Reference Obligation in priority of payment or, if no Reference Obligation is specified in the related Confirmation, the obligations of the Reference Entity that are senior. ISDA 2003 Term: Not Subordinated.
	 */
	@RosettaSynonym(value="notSubordinated", source="FpML_5_10")
	@RosettaSynonym(value="notSubordinated", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="notSubordinated", source="DTCC_11_0")
	@RosettaSynonym(value="notSubordinated", source="DTCC_9_0")
	@RosettaSynonym(value="notSubordinated", source="CME_ClearedConfirm_1_17")
	public final Boolean getNotSubordinated() {
		return notSubordinated;
	}
	
	/**
	 * This element is used to specify any other obligations of a reference entity in both obligations and deliverable obligations. The obligations can be specified free-form. ISDA 2003 Term: Other Obligations of a Reference Entity.
	 */
	@RosettaSynonym(value="othReferenceEntityObligations", source="FpML_5_10")
	@RosettaSynonym(value="othReferenceEntityObligations", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="othReferenceEntityObligations", source="DTCC_11_0")
	@RosettaSynonym(value="othReferenceEntityObligations", source="DTCC_9_0")
	@RosettaSynonym(value="othReferenceEntityObligations", source="CME_ClearedConfirm_1_17")
	public final String getOthReferenceEntityObligations() {
		return othReferenceEntityObligations;
	}
	
	/**
	 * An obligation and deliverable obligation characteristic. Defined in the ISDA published additional provisions for U.S. Municipal as Reference Entity. ISDA 2003 Term: Revenue Obligation Liability.
	 */
	@RosettaSynonym(value="revenueObligationLiability", source="FpML_5_10")
	@RosettaSynonym(value="revenueObligationLiability", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="revenueObligationLiability", source="DTCC_11_0")
	@RosettaSynonym(value="revenueObligationLiability", source="DTCC_9_0")
	@RosettaSynonym(value="revenueObligationLiability", source="CME_ClearedConfirm_1_17")
	public final Boolean getRevenueObligationLiability() {
		return revenueObligationLiability;
	}
	
	/**
	 * An obligation and deliverable obligation characteristic. The currency or currencies in which an obligation or deliverable obligation must be payable. ISDA 2003 Term: Specified Currency.
	 */
	@RosettaSynonym(value="specifiedCurrency", source="FpML_5_10")
	@RosettaSynonym(value="specifiedCurrency", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="specifiedCurrency", source="DTCC_11_0")
	@RosettaSynonym(value="specifiedCurrency", source="DTCC_9_0")
	@RosettaSynonym(value="specifiedCurrency", source="CME_ClearedConfirm_1_17")
	public final SpecifiedCurrency getSpecifiedCurrency() {
		return specifiedCurrency;
	}
	
	/**
	 * A deliverable obligation characteristic. An obligation that is transferable to institutional investors without any contractual, statutory or regulatory restrictions. ISDA 2003 Term: Transferable.
	 */
	@RosettaSynonym(value="transferable", source="FpML_5_10")
	@RosettaSynonym(value="transferable", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="transferable", source="DTCC_11_0")
	@RosettaSynonym(value="transferable", source="DTCC_9_0")
	@RosettaSynonym(value="transferable", source="CME_ClearedConfirm_1_17")
	public final Boolean getTransferable() {
		return transferable;
	}
	
	
	@Override
	public RosettaMetaData<? extends DeliverableObligations> metaData() {
		return metaData;
	} 
	
	public DeliverableObligationsBuilder toBuilder() {
		DeliverableObligationsBuilder builder = new DeliverableObligationsBuilder();
		ofNullable(getAcceleratedOrMatured()).ifPresent(builder::setAcceleratedOrMatured);
		ofNullable(getAccruedInterest()).ifPresent(builder::setAccruedInterest);
		ofNullable(getAssignableLoan()).ifPresent(builder::setAssignableLoan);
		ofNullable(getCategory()).ifPresent(builder::setCategory);
		ofNullable(getConsentRequiredLoan()).ifPresent(builder::setConsentRequiredLoan);
		ofNullable(getDirectLoanParticipation()).ifPresent(builder::setDirectLoanParticipation);
		ofNullable(getExcluded()).ifPresent(builder::setExcluded);
		ofNullable(getFullFaithAndCreditObLiability()).ifPresent(builder::setFullFaithAndCreditObLiability);
		ofNullable(getGeneralFundObligationLiability()).ifPresent(builder::setGeneralFundObligationLiability);
		ofNullable(getIndirectLoanParticipation()).ifPresent(builder::setIndirectLoanParticipation);
		ofNullable(getListed()).ifPresent(builder::setListed);
		ofNullable(getMaximumMaturity()).ifPresent(builder::setMaximumMaturity);
		ofNullable(getNotBearer()).ifPresent(builder::setNotBearer);
		ofNullable(getNotContingent()).ifPresent(builder::setNotContingent);
		ofNullable(getNotDomesticCurrency()).ifPresent(builder::setNotDomesticCurrency);
		ofNullable(getNotDomesticIssuance()).ifPresent(builder::setNotDomesticIssuance);
		ofNullable(getNotDomesticLaw()).ifPresent(builder::setNotDomesticLaw);
		ofNullable(getNotSovereignLender()).ifPresent(builder::setNotSovereignLender);
		ofNullable(getNotSubordinated()).ifPresent(builder::setNotSubordinated);
		ofNullable(getOthReferenceEntityObligations()).ifPresent(builder::setOthReferenceEntityObligations);
		ofNullable(getRevenueObligationLiability()).ifPresent(builder::setRevenueObligationLiability);
		ofNullable(getSpecifiedCurrency()).ifPresent(builder::setSpecifiedCurrency);
		ofNullable(getTransferable()).ifPresent(builder::setTransferable);
		return builder;
	}

	public static DeliverableObligationsBuilder builder() {
		return new DeliverableObligationsBuilder();
	}

	public static  class DeliverableObligationsBuilder extends RosettaModelObjectBuilder {
	
		protected Boolean acceleratedOrMatured;
		protected Boolean accruedInterest;
		protected PCDeliverableObligationCharac.PCDeliverableObligationCharacBuilder assignableLoan;
		protected ObligationCategoryEnum category;
		protected PCDeliverableObligationCharac.PCDeliverableObligationCharacBuilder consentRequiredLoan;
		protected LoanParticipation.LoanParticipationBuilder directLoanParticipation;
		protected String excluded;
		protected Boolean fullFaithAndCreditObLiability;
		protected Boolean generalFundObligationLiability;
		protected LoanParticipation.LoanParticipationBuilder indirectLoanParticipation;
		protected Boolean listed;
		protected Period.PeriodBuilder maximumMaturity;
		protected Boolean notBearer;
		protected Boolean notContingent;
		protected NotDomesticCurrency.NotDomesticCurrencyBuilder notDomesticCurrency;
		protected Boolean notDomesticIssuance;
		protected Boolean notDomesticLaw;
		protected Boolean notSovereignLender;
		protected Boolean notSubordinated;
		protected String othReferenceEntityObligations;
		protected Boolean revenueObligationLiability;
		protected SpecifiedCurrency.SpecifiedCurrencyBuilder specifiedCurrency;
		protected Boolean transferable;
	
		public DeliverableObligationsBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends DeliverableObligations> metaData() {
			return metaData;
		} 
	
		
		public Boolean getAcceleratedOrMatured() {
			return acceleratedOrMatured;
		}
		
		
		public Boolean getAccruedInterest() {
			return accruedInterest;
		}
		
		
		public PCDeliverableObligationCharac.PCDeliverableObligationCharacBuilder getAssignableLoan() {
			return assignableLoan;
		}
		
		public PCDeliverableObligationCharac.PCDeliverableObligationCharacBuilder getOrCreateAssignableLoan() {
			if (assignableLoan!=null) {
				return assignableLoan;
			}
			else return assignableLoan = new PCDeliverableObligationCharac.PCDeliverableObligationCharacBuilder();
		}
		
		
		public ObligationCategoryEnum getCategory() {
			return category;
		}
		
		
		public PCDeliverableObligationCharac.PCDeliverableObligationCharacBuilder getConsentRequiredLoan() {
			return consentRequiredLoan;
		}
		
		public PCDeliverableObligationCharac.PCDeliverableObligationCharacBuilder getOrCreateConsentRequiredLoan() {
			if (consentRequiredLoan!=null) {
				return consentRequiredLoan;
			}
			else return consentRequiredLoan = new PCDeliverableObligationCharac.PCDeliverableObligationCharacBuilder();
		}
		
		
		public LoanParticipation.LoanParticipationBuilder getDirectLoanParticipation() {
			return directLoanParticipation;
		}
		
		public LoanParticipation.LoanParticipationBuilder getOrCreateDirectLoanParticipation() {
			if (directLoanParticipation!=null) {
				return directLoanParticipation;
			}
			else return directLoanParticipation = new LoanParticipation.LoanParticipationBuilder();
		}
		
		
		public String getExcluded() {
			return excluded;
		}
		
		
		public Boolean getFullFaithAndCreditObLiability() {
			return fullFaithAndCreditObLiability;
		}
		
		
		public Boolean getGeneralFundObligationLiability() {
			return generalFundObligationLiability;
		}
		
		
		public LoanParticipation.LoanParticipationBuilder getIndirectLoanParticipation() {
			return indirectLoanParticipation;
		}
		
		public LoanParticipation.LoanParticipationBuilder getOrCreateIndirectLoanParticipation() {
			if (indirectLoanParticipation!=null) {
				return indirectLoanParticipation;
			}
			else return indirectLoanParticipation = new LoanParticipation.LoanParticipationBuilder();
		}
		
		
		public Boolean getListed() {
			return listed;
		}
		
		
		public Period.PeriodBuilder getMaximumMaturity() {
			return maximumMaturity;
		}
		
		public Period.PeriodBuilder getOrCreateMaximumMaturity() {
			if (maximumMaturity!=null) {
				return maximumMaturity;
			}
			else return maximumMaturity = new Period.PeriodBuilder();
		}
		
		
		public Boolean getNotBearer() {
			return notBearer;
		}
		
		
		public Boolean getNotContingent() {
			return notContingent;
		}
		
		
		public NotDomesticCurrency.NotDomesticCurrencyBuilder getNotDomesticCurrency() {
			return notDomesticCurrency;
		}
		
		public NotDomesticCurrency.NotDomesticCurrencyBuilder getOrCreateNotDomesticCurrency() {
			if (notDomesticCurrency!=null) {
				return notDomesticCurrency;
			}
			else return notDomesticCurrency = new NotDomesticCurrency.NotDomesticCurrencyBuilder();
		}
		
		
		public Boolean getNotDomesticIssuance() {
			return notDomesticIssuance;
		}
		
		
		public Boolean getNotDomesticLaw() {
			return notDomesticLaw;
		}
		
		
		public Boolean getNotSovereignLender() {
			return notSovereignLender;
		}
		
		
		public Boolean getNotSubordinated() {
			return notSubordinated;
		}
		
		
		public String getOthReferenceEntityObligations() {
			return othReferenceEntityObligations;
		}
		
		
		public Boolean getRevenueObligationLiability() {
			return revenueObligationLiability;
		}
		
		
		public SpecifiedCurrency.SpecifiedCurrencyBuilder getSpecifiedCurrency() {
			return specifiedCurrency;
		}
		
		public SpecifiedCurrency.SpecifiedCurrencyBuilder getOrCreateSpecifiedCurrency() {
			if (specifiedCurrency!=null) {
				return specifiedCurrency;
			}
			else return specifiedCurrency = new SpecifiedCurrency.SpecifiedCurrencyBuilder();
		}
		
		
		public Boolean getTransferable() {
			return transferable;
		}
		
	
		public DeliverableObligationsBuilder setAcceleratedOrMatured(Boolean acceleratedOrMatured) {
			this.acceleratedOrMatured = acceleratedOrMatured;
			return this;
		}
		public DeliverableObligationsBuilder setAccruedInterest(Boolean accruedInterest) {
			this.accruedInterest = accruedInterest;
			return this;
		}
		public DeliverableObligationsBuilder setAssignableLoan(PCDeliverableObligationCharac assignableLoan) {
			this.assignableLoan = assignableLoan.toBuilder();
			return this;
		}
		public DeliverableObligationsBuilder setAssignableLoanBuilder(PCDeliverableObligationCharac.PCDeliverableObligationCharacBuilder assignableLoan) {
			this.assignableLoan = assignableLoan;
			return this;
		}
		public DeliverableObligationsBuilder setCategory(ObligationCategoryEnum category) {
			this.category = category;
			return this;
		}
		public DeliverableObligationsBuilder setConsentRequiredLoan(PCDeliverableObligationCharac consentRequiredLoan) {
			this.consentRequiredLoan = consentRequiredLoan.toBuilder();
			return this;
		}
		public DeliverableObligationsBuilder setConsentRequiredLoanBuilder(PCDeliverableObligationCharac.PCDeliverableObligationCharacBuilder consentRequiredLoan) {
			this.consentRequiredLoan = consentRequiredLoan;
			return this;
		}
		public DeliverableObligationsBuilder setDirectLoanParticipation(LoanParticipation directLoanParticipation) {
			this.directLoanParticipation = directLoanParticipation.toBuilder();
			return this;
		}
		public DeliverableObligationsBuilder setDirectLoanParticipationBuilder(LoanParticipation.LoanParticipationBuilder directLoanParticipation) {
			this.directLoanParticipation = directLoanParticipation;
			return this;
		}
		public DeliverableObligationsBuilder setExcluded(String excluded) {
			this.excluded = excluded;
			return this;
		}
		public DeliverableObligationsBuilder setFullFaithAndCreditObLiability(Boolean fullFaithAndCreditObLiability) {
			this.fullFaithAndCreditObLiability = fullFaithAndCreditObLiability;
			return this;
		}
		public DeliverableObligationsBuilder setGeneralFundObligationLiability(Boolean generalFundObligationLiability) {
			this.generalFundObligationLiability = generalFundObligationLiability;
			return this;
		}
		public DeliverableObligationsBuilder setIndirectLoanParticipation(LoanParticipation indirectLoanParticipation) {
			this.indirectLoanParticipation = indirectLoanParticipation.toBuilder();
			return this;
		}
		public DeliverableObligationsBuilder setIndirectLoanParticipationBuilder(LoanParticipation.LoanParticipationBuilder indirectLoanParticipation) {
			this.indirectLoanParticipation = indirectLoanParticipation;
			return this;
		}
		public DeliverableObligationsBuilder setListed(Boolean listed) {
			this.listed = listed;
			return this;
		}
		public DeliverableObligationsBuilder setMaximumMaturity(Period maximumMaturity) {
			this.maximumMaturity = maximumMaturity.toBuilder();
			return this;
		}
		public DeliverableObligationsBuilder setMaximumMaturityBuilder(Period.PeriodBuilder maximumMaturity) {
			this.maximumMaturity = maximumMaturity;
			return this;
		}
		public DeliverableObligationsBuilder setNotBearer(Boolean notBearer) {
			this.notBearer = notBearer;
			return this;
		}
		public DeliverableObligationsBuilder setNotContingent(Boolean notContingent) {
			this.notContingent = notContingent;
			return this;
		}
		public DeliverableObligationsBuilder setNotDomesticCurrency(NotDomesticCurrency notDomesticCurrency) {
			this.notDomesticCurrency = notDomesticCurrency.toBuilder();
			return this;
		}
		public DeliverableObligationsBuilder setNotDomesticCurrencyBuilder(NotDomesticCurrency.NotDomesticCurrencyBuilder notDomesticCurrency) {
			this.notDomesticCurrency = notDomesticCurrency;
			return this;
		}
		public DeliverableObligationsBuilder setNotDomesticIssuance(Boolean notDomesticIssuance) {
			this.notDomesticIssuance = notDomesticIssuance;
			return this;
		}
		public DeliverableObligationsBuilder setNotDomesticLaw(Boolean notDomesticLaw) {
			this.notDomesticLaw = notDomesticLaw;
			return this;
		}
		public DeliverableObligationsBuilder setNotSovereignLender(Boolean notSovereignLender) {
			this.notSovereignLender = notSovereignLender;
			return this;
		}
		public DeliverableObligationsBuilder setNotSubordinated(Boolean notSubordinated) {
			this.notSubordinated = notSubordinated;
			return this;
		}
		public DeliverableObligationsBuilder setOthReferenceEntityObligations(String othReferenceEntityObligations) {
			this.othReferenceEntityObligations = othReferenceEntityObligations;
			return this;
		}
		public DeliverableObligationsBuilder setRevenueObligationLiability(Boolean revenueObligationLiability) {
			this.revenueObligationLiability = revenueObligationLiability;
			return this;
		}
		public DeliverableObligationsBuilder setSpecifiedCurrency(SpecifiedCurrency specifiedCurrency) {
			this.specifiedCurrency = specifiedCurrency.toBuilder();
			return this;
		}
		public DeliverableObligationsBuilder setSpecifiedCurrencyBuilder(SpecifiedCurrency.SpecifiedCurrencyBuilder specifiedCurrency) {
			this.specifiedCurrency = specifiedCurrency;
			return this;
		}
		public DeliverableObligationsBuilder setTransferable(Boolean transferable) {
			this.transferable = transferable;
			return this;
		}
		
		public DeliverableObligations build() {
			return new DeliverableObligations(this);
		}
	
		@Override
		public DeliverableObligationsBuilder prune() {
			if (assignableLoan!=null && !assignableLoan.prune().hasData()) assignableLoan = null;
			if (consentRequiredLoan!=null && !consentRequiredLoan.prune().hasData()) consentRequiredLoan = null;
			if (directLoanParticipation!=null && !directLoanParticipation.prune().hasData()) directLoanParticipation = null;
			if (indirectLoanParticipation!=null && !indirectLoanParticipation.prune().hasData()) indirectLoanParticipation = null;
			if (maximumMaturity!=null && !maximumMaturity.prune().hasData()) maximumMaturity = null;
			if (notDomesticCurrency!=null && !notDomesticCurrency.prune().hasData()) notDomesticCurrency = null;
			if (specifiedCurrency!=null && !specifiedCurrency.prune().hasData()) specifiedCurrency = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getAcceleratedOrMatured()!=null) return true;
			if (getAccruedInterest()!=null) return true;
			if (getAssignableLoan()!=null && getAssignableLoan().hasData()) return true;
			if (getCategory()!=null) return true;
			if (getConsentRequiredLoan()!=null && getConsentRequiredLoan().hasData()) return true;
			if (getDirectLoanParticipation()!=null && getDirectLoanParticipation().hasData()) return true;
			if (getExcluded()!=null) return true;
			if (getFullFaithAndCreditObLiability()!=null) return true;
			if (getGeneralFundObligationLiability()!=null) return true;
			if (getIndirectLoanParticipation()!=null && getIndirectLoanParticipation().hasData()) return true;
			if (getListed()!=null) return true;
			if (getMaximumMaturity()!=null && getMaximumMaturity().hasData()) return true;
			if (getNotBearer()!=null) return true;
			if (getNotContingent()!=null) return true;
			if (getNotDomesticCurrency()!=null && getNotDomesticCurrency().hasData()) return true;
			if (getNotDomesticIssuance()!=null) return true;
			if (getNotDomesticLaw()!=null) return true;
			if (getNotSovereignLender()!=null) return true;
			if (getNotSubordinated()!=null) return true;
			if (getOthReferenceEntityObligations()!=null) return true;
			if (getRevenueObligationLiability()!=null) return true;
			if (getSpecifiedCurrency()!=null && getSpecifiedCurrency().hasData()) return true;
			if (getTransferable()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("acceleratedOrMatured"), Boolean.class, acceleratedOrMatured, this);
			processor.processBasic(path.newSubPath("accruedInterest"), Boolean.class, accruedInterest, this);
			processor.processBasic(path.newSubPath("category"), ObligationCategoryEnum.class, category, this);
			processor.processBasic(path.newSubPath("excluded"), String.class, excluded, this);
			processor.processBasic(path.newSubPath("fullFaithAndCreditObLiability"), Boolean.class, fullFaithAndCreditObLiability, this);
			processor.processBasic(path.newSubPath("generalFundObligationLiability"), Boolean.class, generalFundObligationLiability, this);
			processor.processBasic(path.newSubPath("listed"), Boolean.class, listed, this);
			processor.processBasic(path.newSubPath("notBearer"), Boolean.class, notBearer, this);
			processor.processBasic(path.newSubPath("notContingent"), Boolean.class, notContingent, this);
			processor.processBasic(path.newSubPath("notDomesticIssuance"), Boolean.class, notDomesticIssuance, this);
			processor.processBasic(path.newSubPath("notDomesticLaw"), Boolean.class, notDomesticLaw, this);
			processor.processBasic(path.newSubPath("notSovereignLender"), Boolean.class, notSovereignLender, this);
			processor.processBasic(path.newSubPath("notSubordinated"), Boolean.class, notSubordinated, this);
			processor.processBasic(path.newSubPath("othReferenceEntityObligations"), String.class, othReferenceEntityObligations, this);
			processor.processBasic(path.newSubPath("revenueObligationLiability"), Boolean.class, revenueObligationLiability, this);
			processor.processBasic(path.newSubPath("transferable"), Boolean.class, transferable, this);
			
			processRosetta(path.newSubPath("assignableLoan"), processor, PCDeliverableObligationCharac.class, assignableLoan);
			processRosetta(path.newSubPath("consentRequiredLoan"), processor, PCDeliverableObligationCharac.class, consentRequiredLoan);
			processRosetta(path.newSubPath("directLoanParticipation"), processor, LoanParticipation.class, directLoanParticipation);
			processRosetta(path.newSubPath("indirectLoanParticipation"), processor, LoanParticipation.class, indirectLoanParticipation);
			processRosetta(path.newSubPath("maximumMaturity"), processor, Period.class, maximumMaturity);
			processRosetta(path.newSubPath("notDomesticCurrency"), processor, NotDomesticCurrency.class, notDomesticCurrency);
			processRosetta(path.newSubPath("specifiedCurrency"), processor, SpecifiedCurrency.class, specifiedCurrency);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			DeliverableObligationsBuilder _that = (DeliverableObligationsBuilder) o;
		
			if (acceleratedOrMatured != null ? !acceleratedOrMatured.equals(_that.acceleratedOrMatured) : _that.acceleratedOrMatured != null) return false;
			if (accruedInterest != null ? !accruedInterest.equals(_that.accruedInterest) : _that.accruedInterest != null) return false;
			if (assignableLoan != null ? !assignableLoan.equals(_that.assignableLoan) : _that.assignableLoan != null) return false;
			if (category != null ? !category.equals(_that.category) : _that.category != null) return false;
			if (consentRequiredLoan != null ? !consentRequiredLoan.equals(_that.consentRequiredLoan) : _that.consentRequiredLoan != null) return false;
			if (directLoanParticipation != null ? !directLoanParticipation.equals(_that.directLoanParticipation) : _that.directLoanParticipation != null) return false;
			if (excluded != null ? !excluded.equals(_that.excluded) : _that.excluded != null) return false;
			if (fullFaithAndCreditObLiability != null ? !fullFaithAndCreditObLiability.equals(_that.fullFaithAndCreditObLiability) : _that.fullFaithAndCreditObLiability != null) return false;
			if (generalFundObligationLiability != null ? !generalFundObligationLiability.equals(_that.generalFundObligationLiability) : _that.generalFundObligationLiability != null) return false;
			if (indirectLoanParticipation != null ? !indirectLoanParticipation.equals(_that.indirectLoanParticipation) : _that.indirectLoanParticipation != null) return false;
			if (listed != null ? !listed.equals(_that.listed) : _that.listed != null) return false;
			if (maximumMaturity != null ? !maximumMaturity.equals(_that.maximumMaturity) : _that.maximumMaturity != null) return false;
			if (notBearer != null ? !notBearer.equals(_that.notBearer) : _that.notBearer != null) return false;
			if (notContingent != null ? !notContingent.equals(_that.notContingent) : _that.notContingent != null) return false;
			if (notDomesticCurrency != null ? !notDomesticCurrency.equals(_that.notDomesticCurrency) : _that.notDomesticCurrency != null) return false;
			if (notDomesticIssuance != null ? !notDomesticIssuance.equals(_that.notDomesticIssuance) : _that.notDomesticIssuance != null) return false;
			if (notDomesticLaw != null ? !notDomesticLaw.equals(_that.notDomesticLaw) : _that.notDomesticLaw != null) return false;
			if (notSovereignLender != null ? !notSovereignLender.equals(_that.notSovereignLender) : _that.notSovereignLender != null) return false;
			if (notSubordinated != null ? !notSubordinated.equals(_that.notSubordinated) : _that.notSubordinated != null) return false;
			if (othReferenceEntityObligations != null ? !othReferenceEntityObligations.equals(_that.othReferenceEntityObligations) : _that.othReferenceEntityObligations != null) return false;
			if (revenueObligationLiability != null ? !revenueObligationLiability.equals(_that.revenueObligationLiability) : _that.revenueObligationLiability != null) return false;
			if (specifiedCurrency != null ? !specifiedCurrency.equals(_that.specifiedCurrency) : _that.specifiedCurrency != null) return false;
			if (transferable != null ? !transferable.equals(_that.transferable) : _that.transferable != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (acceleratedOrMatured != null ? acceleratedOrMatured.hashCode() : 0);
			_result = 31 * _result + (accruedInterest != null ? accruedInterest.hashCode() : 0);
			_result = 31 * _result + (assignableLoan != null ? assignableLoan.hashCode() : 0);
			_result = 31 * _result + (category != null ? category.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (consentRequiredLoan != null ? consentRequiredLoan.hashCode() : 0);
			_result = 31 * _result + (directLoanParticipation != null ? directLoanParticipation.hashCode() : 0);
			_result = 31 * _result + (excluded != null ? excluded.hashCode() : 0);
			_result = 31 * _result + (fullFaithAndCreditObLiability != null ? fullFaithAndCreditObLiability.hashCode() : 0);
			_result = 31 * _result + (generalFundObligationLiability != null ? generalFundObligationLiability.hashCode() : 0);
			_result = 31 * _result + (indirectLoanParticipation != null ? indirectLoanParticipation.hashCode() : 0);
			_result = 31 * _result + (listed != null ? listed.hashCode() : 0);
			_result = 31 * _result + (maximumMaturity != null ? maximumMaturity.hashCode() : 0);
			_result = 31 * _result + (notBearer != null ? notBearer.hashCode() : 0);
			_result = 31 * _result + (notContingent != null ? notContingent.hashCode() : 0);
			_result = 31 * _result + (notDomesticCurrency != null ? notDomesticCurrency.hashCode() : 0);
			_result = 31 * _result + (notDomesticIssuance != null ? notDomesticIssuance.hashCode() : 0);
			_result = 31 * _result + (notDomesticLaw != null ? notDomesticLaw.hashCode() : 0);
			_result = 31 * _result + (notSovereignLender != null ? notSovereignLender.hashCode() : 0);
			_result = 31 * _result + (notSubordinated != null ? notSubordinated.hashCode() : 0);
			_result = 31 * _result + (othReferenceEntityObligations != null ? othReferenceEntityObligations.hashCode() : 0);
			_result = 31 * _result + (revenueObligationLiability != null ? revenueObligationLiability.hashCode() : 0);
			_result = 31 * _result + (specifiedCurrency != null ? specifiedCurrency.hashCode() : 0);
			_result = 31 * _result + (transferable != null ? transferable.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "DeliverableObligationsBuilder {" +
				"acceleratedOrMatured=" + this.acceleratedOrMatured + ", " +
				"accruedInterest=" + this.accruedInterest + ", " +
				"assignableLoan=" + this.assignableLoan + ", " +
				"category=" + this.category + ", " +
				"consentRequiredLoan=" + this.consentRequiredLoan + ", " +
				"directLoanParticipation=" + this.directLoanParticipation + ", " +
				"excluded=" + this.excluded + ", " +
				"fullFaithAndCreditObLiability=" + this.fullFaithAndCreditObLiability + ", " +
				"generalFundObligationLiability=" + this.generalFundObligationLiability + ", " +
				"indirectLoanParticipation=" + this.indirectLoanParticipation + ", " +
				"listed=" + this.listed + ", " +
				"maximumMaturity=" + this.maximumMaturity + ", " +
				"notBearer=" + this.notBearer + ", " +
				"notContingent=" + this.notContingent + ", " +
				"notDomesticCurrency=" + this.notDomesticCurrency + ", " +
				"notDomesticIssuance=" + this.notDomesticIssuance + ", " +
				"notDomesticLaw=" + this.notDomesticLaw + ", " +
				"notSovereignLender=" + this.notSovereignLender + ", " +
				"notSubordinated=" + this.notSubordinated + ", " +
				"othReferenceEntityObligations=" + this.othReferenceEntityObligations + ", " +
				"revenueObligationLiability=" + this.revenueObligationLiability + ", " +
				"specifiedCurrency=" + this.specifiedCurrency + ", " +
				"transferable=" + this.transferable +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("acceleratedOrMatured"), Boolean.class, acceleratedOrMatured, this);
		processor.processBasic(path.newSubPath("accruedInterest"), Boolean.class, accruedInterest, this);
		processor.processBasic(path.newSubPath("category"), ObligationCategoryEnum.class, category, this);
		processor.processBasic(path.newSubPath("excluded"), String.class, excluded, this);
		processor.processBasic(path.newSubPath("fullFaithAndCreditObLiability"), Boolean.class, fullFaithAndCreditObLiability, this);
		processor.processBasic(path.newSubPath("generalFundObligationLiability"), Boolean.class, generalFundObligationLiability, this);
		processor.processBasic(path.newSubPath("listed"), Boolean.class, listed, this);
		processor.processBasic(path.newSubPath("notBearer"), Boolean.class, notBearer, this);
		processor.processBasic(path.newSubPath("notContingent"), Boolean.class, notContingent, this);
		processor.processBasic(path.newSubPath("notDomesticIssuance"), Boolean.class, notDomesticIssuance, this);
		processor.processBasic(path.newSubPath("notDomesticLaw"), Boolean.class, notDomesticLaw, this);
		processor.processBasic(path.newSubPath("notSovereignLender"), Boolean.class, notSovereignLender, this);
		processor.processBasic(path.newSubPath("notSubordinated"), Boolean.class, notSubordinated, this);
		processor.processBasic(path.newSubPath("othReferenceEntityObligations"), String.class, othReferenceEntityObligations, this);
		processor.processBasic(path.newSubPath("revenueObligationLiability"), Boolean.class, revenueObligationLiability, this);
		processor.processBasic(path.newSubPath("transferable"), Boolean.class, transferable, this);
		
		processRosetta(path.newSubPath("assignableLoan"), processor, PCDeliverableObligationCharac.class, assignableLoan);
		processRosetta(path.newSubPath("consentRequiredLoan"), processor, PCDeliverableObligationCharac.class, consentRequiredLoan);
		processRosetta(path.newSubPath("directLoanParticipation"), processor, LoanParticipation.class, directLoanParticipation);
		processRosetta(path.newSubPath("indirectLoanParticipation"), processor, LoanParticipation.class, indirectLoanParticipation);
		processRosetta(path.newSubPath("maximumMaturity"), processor, Period.class, maximumMaturity);
		processRosetta(path.newSubPath("notDomesticCurrency"), processor, NotDomesticCurrency.class, notDomesticCurrency);
		processRosetta(path.newSubPath("specifiedCurrency"), processor, SpecifiedCurrency.class, specifiedCurrency);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		DeliverableObligations _that = (DeliverableObligations) o;
	
		if (acceleratedOrMatured != null ? !acceleratedOrMatured.equals(_that.acceleratedOrMatured) : _that.acceleratedOrMatured != null) return false;
		if (accruedInterest != null ? !accruedInterest.equals(_that.accruedInterest) : _that.accruedInterest != null) return false;
		if (assignableLoan != null ? !assignableLoan.equals(_that.assignableLoan) : _that.assignableLoan != null) return false;
		if (category != null ? !category.equals(_that.category) : _that.category != null) return false;
		if (consentRequiredLoan != null ? !consentRequiredLoan.equals(_that.consentRequiredLoan) : _that.consentRequiredLoan != null) return false;
		if (directLoanParticipation != null ? !directLoanParticipation.equals(_that.directLoanParticipation) : _that.directLoanParticipation != null) return false;
		if (excluded != null ? !excluded.equals(_that.excluded) : _that.excluded != null) return false;
		if (fullFaithAndCreditObLiability != null ? !fullFaithAndCreditObLiability.equals(_that.fullFaithAndCreditObLiability) : _that.fullFaithAndCreditObLiability != null) return false;
		if (generalFundObligationLiability != null ? !generalFundObligationLiability.equals(_that.generalFundObligationLiability) : _that.generalFundObligationLiability != null) return false;
		if (indirectLoanParticipation != null ? !indirectLoanParticipation.equals(_that.indirectLoanParticipation) : _that.indirectLoanParticipation != null) return false;
		if (listed != null ? !listed.equals(_that.listed) : _that.listed != null) return false;
		if (maximumMaturity != null ? !maximumMaturity.equals(_that.maximumMaturity) : _that.maximumMaturity != null) return false;
		if (notBearer != null ? !notBearer.equals(_that.notBearer) : _that.notBearer != null) return false;
		if (notContingent != null ? !notContingent.equals(_that.notContingent) : _that.notContingent != null) return false;
		if (notDomesticCurrency != null ? !notDomesticCurrency.equals(_that.notDomesticCurrency) : _that.notDomesticCurrency != null) return false;
		if (notDomesticIssuance != null ? !notDomesticIssuance.equals(_that.notDomesticIssuance) : _that.notDomesticIssuance != null) return false;
		if (notDomesticLaw != null ? !notDomesticLaw.equals(_that.notDomesticLaw) : _that.notDomesticLaw != null) return false;
		if (notSovereignLender != null ? !notSovereignLender.equals(_that.notSovereignLender) : _that.notSovereignLender != null) return false;
		if (notSubordinated != null ? !notSubordinated.equals(_that.notSubordinated) : _that.notSubordinated != null) return false;
		if (othReferenceEntityObligations != null ? !othReferenceEntityObligations.equals(_that.othReferenceEntityObligations) : _that.othReferenceEntityObligations != null) return false;
		if (revenueObligationLiability != null ? !revenueObligationLiability.equals(_that.revenueObligationLiability) : _that.revenueObligationLiability != null) return false;
		if (specifiedCurrency != null ? !specifiedCurrency.equals(_that.specifiedCurrency) : _that.specifiedCurrency != null) return false;
		if (transferable != null ? !transferable.equals(_that.transferable) : _that.transferable != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (acceleratedOrMatured != null ? acceleratedOrMatured.hashCode() : 0);
		_result = 31 * _result + (accruedInterest != null ? accruedInterest.hashCode() : 0);
		_result = 31 * _result + (assignableLoan != null ? assignableLoan.hashCode() : 0);
		_result = 31 * _result + (category != null ? category.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (consentRequiredLoan != null ? consentRequiredLoan.hashCode() : 0);
		_result = 31 * _result + (directLoanParticipation != null ? directLoanParticipation.hashCode() : 0);
		_result = 31 * _result + (excluded != null ? excluded.hashCode() : 0);
		_result = 31 * _result + (fullFaithAndCreditObLiability != null ? fullFaithAndCreditObLiability.hashCode() : 0);
		_result = 31 * _result + (generalFundObligationLiability != null ? generalFundObligationLiability.hashCode() : 0);
		_result = 31 * _result + (indirectLoanParticipation != null ? indirectLoanParticipation.hashCode() : 0);
		_result = 31 * _result + (listed != null ? listed.hashCode() : 0);
		_result = 31 * _result + (maximumMaturity != null ? maximumMaturity.hashCode() : 0);
		_result = 31 * _result + (notBearer != null ? notBearer.hashCode() : 0);
		_result = 31 * _result + (notContingent != null ? notContingent.hashCode() : 0);
		_result = 31 * _result + (notDomesticCurrency != null ? notDomesticCurrency.hashCode() : 0);
		_result = 31 * _result + (notDomesticIssuance != null ? notDomesticIssuance.hashCode() : 0);
		_result = 31 * _result + (notDomesticLaw != null ? notDomesticLaw.hashCode() : 0);
		_result = 31 * _result + (notSovereignLender != null ? notSovereignLender.hashCode() : 0);
		_result = 31 * _result + (notSubordinated != null ? notSubordinated.hashCode() : 0);
		_result = 31 * _result + (othReferenceEntityObligations != null ? othReferenceEntityObligations.hashCode() : 0);
		_result = 31 * _result + (revenueObligationLiability != null ? revenueObligationLiability.hashCode() : 0);
		_result = 31 * _result + (specifiedCurrency != null ? specifiedCurrency.hashCode() : 0);
		_result = 31 * _result + (transferable != null ? transferable.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "DeliverableObligations {" +
			"acceleratedOrMatured=" + this.acceleratedOrMatured + ", " +
			"accruedInterest=" + this.accruedInterest + ", " +
			"assignableLoan=" + this.assignableLoan + ", " +
			"category=" + this.category + ", " +
			"consentRequiredLoan=" + this.consentRequiredLoan + ", " +
			"directLoanParticipation=" + this.directLoanParticipation + ", " +
			"excluded=" + this.excluded + ", " +
			"fullFaithAndCreditObLiability=" + this.fullFaithAndCreditObLiability + ", " +
			"generalFundObligationLiability=" + this.generalFundObligationLiability + ", " +
			"indirectLoanParticipation=" + this.indirectLoanParticipation + ", " +
			"listed=" + this.listed + ", " +
			"maximumMaturity=" + this.maximumMaturity + ", " +
			"notBearer=" + this.notBearer + ", " +
			"notContingent=" + this.notContingent + ", " +
			"notDomesticCurrency=" + this.notDomesticCurrency + ", " +
			"notDomesticIssuance=" + this.notDomesticIssuance + ", " +
			"notDomesticLaw=" + this.notDomesticLaw + ", " +
			"notSovereignLender=" + this.notSovereignLender + ", " +
			"notSubordinated=" + this.notSubordinated + ", " +
			"othReferenceEntityObligations=" + this.othReferenceEntityObligations + ", " +
			"revenueObligationLiability=" + this.revenueObligationLiability + ", " +
			"specifiedCurrency=" + this.specifiedCurrency + ", " +
			"transferable=" + this.transferable +
		'}';
	}
}
