package org.isda.cdm;

import java.lang.String;
import org.isda.cdm.DeliveryAmountElectionEnum;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.DeliveryAmountMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class to specify the application of Interest Amount with respect the Delivery Amount. ISDA 2016 Japanese Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (n)(ii).
 * 
 * @version 2.5.4
 */
@RosettaClass
public class DeliveryAmount extends RosettaModelObject {
	private final String customElection;
	private final DeliveryAmountElectionEnum standardElection;
	private static DeliveryAmountMeta metaData = new DeliveryAmountMeta();
	
	DeliveryAmount(DeliveryAmountBuilder builder) {
		this.customElection = builder.getCustomElection();
		this.standardElection = builder.getStandardElection();
	}
	
	/**
	 * The custom election that might be specified by the parties to the agreement.
	 */
	@RosettaSynonym(value="specify_delivery_amount", source="ISDA_Create_1_0")
	public final String getCustomElection() {
		return customElection;
	}
	
	/**
	 * The standard election as specified by an enumeration.
	 */
	@RosettaSynonym(value="delivery_amount", source="ISDA_Create_1_0")
	public final DeliveryAmountElectionEnum getStandardElection() {
		return standardElection;
	}
	
	
	@Override
	public RosettaMetaData<? extends DeliveryAmount> metaData() {
		return metaData;
	} 
	
	public DeliveryAmountBuilder toBuilder() {
		DeliveryAmountBuilder builder = new DeliveryAmountBuilder();
		ofNullable(getCustomElection()).ifPresent(builder::setCustomElection);
		ofNullable(getStandardElection()).ifPresent(builder::setStandardElection);
		return builder;
	}

	public static DeliveryAmountBuilder builder() {
		return new DeliveryAmountBuilder();
	}

	public static  class DeliveryAmountBuilder extends RosettaModelObjectBuilder {
	
		protected String customElection;
		protected DeliveryAmountElectionEnum standardElection;
	
		public DeliveryAmountBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends DeliveryAmount> metaData() {
			return metaData;
		} 
	
		
		public String getCustomElection() {
			return customElection;
		}
		
		
		public DeliveryAmountElectionEnum getStandardElection() {
			return standardElection;
		}
		
	
		public DeliveryAmountBuilder setCustomElection(String customElection) {
			this.customElection = customElection;
			return this;
		}
		public DeliveryAmountBuilder setStandardElection(DeliveryAmountElectionEnum standardElection) {
			this.standardElection = standardElection;
			return this;
		}
		
		public DeliveryAmount build() {
			return new DeliveryAmount(this);
		}
	
		@Override
		public DeliveryAmountBuilder prune() {
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getCustomElection()!=null) return true;
			if (getStandardElection()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("customElection"), String.class, customElection, this);
			processor.processBasic(path.newSubPath("standardElection"), DeliveryAmountElectionEnum.class, standardElection, this);
			
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			DeliveryAmountBuilder _that = (DeliveryAmountBuilder) o;
		
			if (customElection != null ? !customElection.equals(_that.customElection) : _that.customElection != null) return false;
			if (standardElection != null ? !standardElection.equals(_that.standardElection) : _that.standardElection != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (customElection != null ? customElection.hashCode() : 0);
			_result = 31 * _result + (standardElection != null ? standardElection.getClass().getName().hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "DeliveryAmountBuilder {" +
				"customElection=" + this.customElection + ", " +
				"standardElection=" + this.standardElection +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("customElection"), String.class, customElection, this);
		processor.processBasic(path.newSubPath("standardElection"), DeliveryAmountElectionEnum.class, standardElection, this);
		
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		DeliveryAmount _that = (DeliveryAmount) o;
	
		if (customElection != null ? !customElection.equals(_that.customElection) : _that.customElection != null) return false;
		if (standardElection != null ? !standardElection.equals(_that.standardElection) : _that.standardElection != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (customElection != null ? customElection.hashCode() : 0);
		_result = 31 * _result + (standardElection != null ? standardElection.getClass().getName().hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "DeliveryAmount {" +
			"customElection=" + this.customElection + ", " +
			"standardElection=" + this.standardElection +
		'}';
	}
}
