package org.isda.cdm;

import java.math.BigDecimal;
import org.isda.cdm.DiscountingTypeEnum;
import org.isda.cdm.metafields.FieldWithMetaDayCountFractionEnum;
import org.isda.cdm.metafields.FieldWithMetaDayCountFractionEnum.FieldWithMetaDayCountFractionEnumBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.DiscountingMethodMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.math.BigDecimal;
		import static java.util.Optional.ofNullable;

/**
 * A class defining discounting information. The 2000 ISDA definitions, section 8.4. discounting (related to the calculation of a discounted fixed amount or floating amount) apply. This type must only be included if discounting applies.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="Discounting", source="FpML_5_10")
public class DiscountingMethod extends RosettaModelObject {
	private final BigDecimal discountRate;
	private final FieldWithMetaDayCountFractionEnum discountRateDayCountFraction;
	private final DiscountingTypeEnum discountingType;
	private static DiscountingMethodMeta metaData = new DiscountingMethodMeta();
	
	DiscountingMethod(DiscountingMethodBuilder builder) {
		this.discountRate = builder.getDiscountRate();
		this.discountRateDayCountFraction = ofNullable(builder.getDiscountRateDayCountFraction()).map(f->f.build()).orElse(null);
		this.discountingType = builder.getDiscountingType();
	}
	
	/**
	 * A discount rate, expressed as a decimal, to be used in the calculation of a discounted amount. A discount amount of 5% would be represented as 0.05.
	 */
	@RosettaSynonym(value="discountRate", source="FpML_5_10")
	@RosettaSynonym(value="discountRate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="discountRate", source="DTCC_11_0")
	@RosettaSynonym(value="discountRate", source="DTCC_9_0")
	@RosettaSynonym(value="discountRate", source="CME_ClearedConfirm_1_17")
	public final BigDecimal getDiscountRate() {
		return discountRate;
	}
	
	/**
	 * A discount day count fraction to be used in the calculation of a discounted amount.
	 */
	@RosettaSynonym(value="discountRateDayCountFraction", source="FpML_5_10")
	@RosettaSynonym(value="discountRateDayCountFraction", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="discountRateDayCountFraction", source="DTCC_11_0")
	@RosettaSynonym(value="discountRateDayCountFraction", source="DTCC_9_0")
	@RosettaSynonym(value="discountRateDayCountFraction", source="CME_ClearedConfirm_1_17")
	public final FieldWithMetaDayCountFractionEnum getDiscountRateDayCountFraction() {
		return discountRateDayCountFraction;
	}
	
	/**
	 * The discounting method that is applicable.
	 */
	@RosettaSynonym(value="discountingType", source="FpML_5_10")
	@RosettaSynonym(value="discountingType", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="discountingType", source="DTCC_11_0")
	@RosettaSynonym(value="discountingType", source="DTCC_9_0")
	@RosettaSynonym(value="discountingType", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="fraDiscounting", source="FpML_5_10")
	@RosettaSynonym(value="fraDiscounting", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="fraDiscounting", source="DTCC_11_0")
	@RosettaSynonym(value="fraDiscounting", source="DTCC_9_0")
	@RosettaSynonym(value="fraDiscounting", source="CME_ClearedConfirm_1_17")
	public final DiscountingTypeEnum getDiscountingType() {
		return discountingType;
	}
	
	
	@Override
	public RosettaMetaData<? extends DiscountingMethod> metaData() {
		return metaData;
	} 
	
	public DiscountingMethodBuilder toBuilder() {
		DiscountingMethodBuilder builder = new DiscountingMethodBuilder();
		ofNullable(getDiscountRate()).ifPresent(builder::setDiscountRate);
		ofNullable(getDiscountRateDayCountFraction()).ifPresent(builder::setDiscountRateDayCountFraction);
		ofNullable(getDiscountingType()).ifPresent(builder::setDiscountingType);
		return builder;
	}

	public static DiscountingMethodBuilder builder() {
		return new DiscountingMethodBuilder();
	}

	public static  class DiscountingMethodBuilder extends RosettaModelObjectBuilder {
	
		protected BigDecimal discountRate;
		protected FieldWithMetaDayCountFractionEnumBuilder discountRateDayCountFraction;
		protected DiscountingTypeEnum discountingType;
	
		public DiscountingMethodBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends DiscountingMethod> metaData() {
			return metaData;
		} 
	
		
		public BigDecimal getDiscountRate() {
			return discountRate;
		}
		
		
		public FieldWithMetaDayCountFractionEnumBuilder getDiscountRateDayCountFraction() {
			return discountRateDayCountFraction;
		}
		
		public FieldWithMetaDayCountFractionEnumBuilder getOrCreateDiscountRateDayCountFraction() {
			if (discountRateDayCountFraction!=null) {
				return discountRateDayCountFraction;
			}
			else return discountRateDayCountFraction = new FieldWithMetaDayCountFractionEnumBuilder();
		}
		
		
		public DiscountingTypeEnum getDiscountingType() {
			return discountingType;
		}
		
	
		public DiscountingMethodBuilder setDiscountRate(BigDecimal discountRate) {
			this.discountRate = discountRate;
			return this;
		}
		public DiscountingMethodBuilder setDiscountRateDayCountFraction(FieldWithMetaDayCountFractionEnum discountRateDayCountFraction) {
			this.discountRateDayCountFraction = discountRateDayCountFraction.toBuilder();
			return this;
		}
		public DiscountingMethodBuilder setDiscountRateDayCountFractionRef(DayCountFractionEnum discountRateDayCountFraction) {
			return setDiscountRateDayCountFraction(FieldWithMetaDayCountFractionEnum.builder().setValue(discountRateDayCountFraction).build());
		}
		public DiscountingMethodBuilder setDiscountingType(DiscountingTypeEnum discountingType) {
			this.discountingType = discountingType;
			return this;
		}
		
		public DiscountingMethod build() {
			return new DiscountingMethod(this);
		}
	
		@Override
		public DiscountingMethodBuilder prune() {
			if (discountRateDayCountFraction!=null && !discountRateDayCountFraction.prune().hasData()) discountRateDayCountFraction = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getDiscountRate()!=null) return true;
			if (getDiscountRateDayCountFraction()!=null) return true;
			if (getDiscountingType()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("discountRate"), BigDecimal.class, discountRate, this);
			processor.processBasic(path.newSubPath("discountingType"), DiscountingTypeEnum.class, discountingType, this);
			
			processRosetta(path.newSubPath("discountRateDayCountFraction"), processor, FieldWithMetaDayCountFractionEnum.class, discountRateDayCountFraction);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			DiscountingMethodBuilder _that = (DiscountingMethodBuilder) o;
		
			if (discountRate != null ? !discountRate.equals(_that.discountRate) : _that.discountRate != null) return false;
			if (discountRateDayCountFraction != null ? !discountRateDayCountFraction.equals(_that.discountRateDayCountFraction) : _that.discountRateDayCountFraction != null) return false;
			if (discountingType != null ? !discountingType.equals(_that.discountingType) : _that.discountingType != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (discountRate != null ? discountRate.hashCode() : 0);
			_result = 31 * _result + (discountRateDayCountFraction != null ? discountRateDayCountFraction.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (discountingType != null ? discountingType.getClass().getName().hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "DiscountingMethodBuilder {" +
				"discountRate=" + this.discountRate + ", " +
				"discountRateDayCountFraction=" + this.discountRateDayCountFraction + ", " +
				"discountingType=" + this.discountingType +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("discountRate"), BigDecimal.class, discountRate, this);
		processor.processBasic(path.newSubPath("discountingType"), DiscountingTypeEnum.class, discountingType, this);
		
		processRosetta(path.newSubPath("discountRateDayCountFraction"), processor, FieldWithMetaDayCountFractionEnum.class, discountRateDayCountFraction);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		DiscountingMethod _that = (DiscountingMethod) o;
	
		if (discountRate != null ? !discountRate.equals(_that.discountRate) : _that.discountRate != null) return false;
		if (discountRateDayCountFraction != null ? !discountRateDayCountFraction.equals(_that.discountRateDayCountFraction) : _that.discountRateDayCountFraction != null) return false;
		if (discountingType != null ? !discountingType.equals(_that.discountingType) : _that.discountingType != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (discountRate != null ? discountRate.hashCode() : 0);
		_result = 31 * _result + (discountRateDayCountFraction != null ? discountRateDayCountFraction.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (discountingType != null ? discountingType.getClass().getName().hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "DiscountingMethod {" +
			"discountRate=" + this.discountRate + ", " +
			"discountRateDayCountFraction=" + this.discountRateDayCountFraction + ", " +
			"discountingType=" + this.discountingType +
		'}';
	}
}
