package org.isda.cdm;

import java.lang.String;
import org.isda.cdm.BusinessCenterTime;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.DisputeResolutionMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class to specify the election terms under which a party disputes (i) the Calculation Agent’s calculation of a Delivery Amount or a Return Amount, or (ii) the Value of any Transfer of Eligible Credit Support or Posted Credit Support. Parties can specify such election either through a business center time or through a custom election. ISDA 2016 English Law Credit Support Deed for Initial Margin, paragraph 13, General Principles, (g): Dispute Resolution. | ISDA 2016 Japanese Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (h): Dispute Resolution. | ISDA 2016 New York Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (g): Dispute Resolution.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class DisputeResolution extends RosettaModelObject {
	private final String otherTerms;
	private final BusinessCenterTime resolutionTime;
	private static DisputeResolutionMeta metaData = new DisputeResolutionMeta();
	
	DisputeResolution(DisputeResolutionBuilder builder) {
		this.otherTerms = builder.getOtherTerms();
		this.resolutionTime = ofNullable(builder.getResolutionTime()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The custom Resolution Time election that might be specified by the parties.
	 */
	public final String getOtherTerms() {
		return otherTerms;
	}
	
	/**
	 * The time by which the dispute needs to be resolved, failure of which would trigger a recalculation alongside a process that is specified as part of the agreement. ISDA 2016 Credit Support Annex for Initial Margin, paragraph 13, General Principles, (g)(i): Resolution Time. | ISDA 2016 Credit Support Annex for Variation Margin, paragraph 13, (g)(i): Resolution Time.
	 */
	public final BusinessCenterTime getResolutionTime() {
		return resolutionTime;
	}
	
	
	@Override
	public RosettaMetaData<? extends DisputeResolution> metaData() {
		return metaData;
	} 
	
	public DisputeResolutionBuilder toBuilder() {
		DisputeResolutionBuilder builder = new DisputeResolutionBuilder();
		ofNullable(getOtherTerms()).ifPresent(builder::setOtherTerms);
		ofNullable(getResolutionTime()).ifPresent(builder::setResolutionTime);
		return builder;
	}

	public static DisputeResolutionBuilder builder() {
		return new DisputeResolutionBuilder();
	}

	public static  class DisputeResolutionBuilder extends RosettaModelObjectBuilder {
	
		protected String otherTerms;
		protected BusinessCenterTime.BusinessCenterTimeBuilder resolutionTime;
	
		public DisputeResolutionBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends DisputeResolution> metaData() {
			return metaData;
		} 
	
		
		public String getOtherTerms() {
			return otherTerms;
		}
		
		
		public BusinessCenterTime.BusinessCenterTimeBuilder getResolutionTime() {
			return resolutionTime;
		}
		
		public BusinessCenterTime.BusinessCenterTimeBuilder getOrCreateResolutionTime() {
			if (resolutionTime!=null) {
				return resolutionTime;
			}
			else return resolutionTime = new BusinessCenterTime.BusinessCenterTimeBuilder();
		}
		
	
		public DisputeResolutionBuilder setOtherTerms(String otherTerms) {
			this.otherTerms = otherTerms;
			return this;
		}
		public DisputeResolutionBuilder setResolutionTime(BusinessCenterTime resolutionTime) {
			this.resolutionTime = resolutionTime.toBuilder();
			return this;
		}
		public DisputeResolutionBuilder setResolutionTimeBuilder(BusinessCenterTime.BusinessCenterTimeBuilder resolutionTime) {
			this.resolutionTime = resolutionTime;
			return this;
		}
		
		public DisputeResolution build() {
			return new DisputeResolution(this);
		}
	
		@Override
		public DisputeResolutionBuilder prune() {
			if (resolutionTime!=null && !resolutionTime.prune().hasData()) resolutionTime = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getOtherTerms()!=null) return true;
			if (getResolutionTime()!=null && getResolutionTime().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("otherTerms"), String.class, otherTerms, this);
			
			processRosetta(path.newSubPath("resolutionTime"), processor, BusinessCenterTime.class, resolutionTime);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			DisputeResolutionBuilder _that = (DisputeResolutionBuilder) o;
		
			if (otherTerms != null ? !otherTerms.equals(_that.otherTerms) : _that.otherTerms != null) return false;
			if (resolutionTime != null ? !resolutionTime.equals(_that.resolutionTime) : _that.resolutionTime != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (otherTerms != null ? otherTerms.hashCode() : 0);
			_result = 31 * _result + (resolutionTime != null ? resolutionTime.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "DisputeResolutionBuilder {" +
				"otherTerms=" + this.otherTerms + ", " +
				"resolutionTime=" + this.resolutionTime +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("otherTerms"), String.class, otherTerms, this);
		
		processRosetta(path.newSubPath("resolutionTime"), processor, BusinessCenterTime.class, resolutionTime);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		DisputeResolution _that = (DisputeResolution) o;
	
		if (otherTerms != null ? !otherTerms.equals(_that.otherTerms) : _that.otherTerms != null) return false;
		if (resolutionTime != null ? !resolutionTime.equals(_that.resolutionTime) : _that.resolutionTime != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (otherTerms != null ? otherTerms.hashCode() : 0);
		_result = 31 * _result + (resolutionTime != null ? resolutionTime.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "DisputeResolution {" +
			"otherTerms=" + this.otherTerms + ", " +
			"resolutionTime=" + this.resolutionTime +
		'}';
	}
}
