package org.isda.cdm;

import java.lang.Boolean;
import java.lang.String;
import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.EligibleCurrencyInterestRate;
import org.isda.cdm.InterestAdjustment;
import org.isda.cdm.InterestAmount;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.DistributionAndInterestPaymentMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;

/**
 * A class to specify the Distributions and Interest Payment provisions applicable to the Japanese Law ISDA 2016 CSA for Initial Margin and the New York Law ISDA 2016 CSA for Variation Margin. ISDA 2016 Japanese Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (n): Distributions and Interest Payment (IM). | ISDA 2016 Credit Support Annex for Variation Margin, paragraph 13, (i): Distributions and Interest Payment (VM).
 * 
 * @version 2.5.4
 */
@RosettaClass
public class DistributionAndInterestPayment extends RosettaModelObject {
	private final String alternativeProvision;
	private final Boolean dailyInterestCompounding;
	private final InterestAdjustment interestAdjustment;
	private final InterestAmount interestAmount;
	private final Boolean interestPaymentNetting;
	private final Boolean interestPaymentTransfer;
	private final List<EligibleCurrencyInterestRate> interestRate;
	private final Boolean negativeInterest;
	private static DistributionAndInterestPaymentMeta metaData = new DistributionAndInterestPaymentMeta();
	
	DistributionAndInterestPayment(DistributionAndInterestPaymentBuilder builder) {
		this.alternativeProvision = builder.getAlternativeProvision();
		this.dailyInterestCompounding = builder.getDailyInterestCompounding();
		this.interestAdjustment = ofNullable(builder.getInterestAdjustment()).map(f->f.build()).orElse(null);
		this.interestAmount = ofNullable(builder.getInterestAmount()).map(f->f.build()).orElse(null);
		this.interestPaymentNetting = builder.getInterestPaymentNetting();
		this.interestPaymentTransfer = builder.getInterestPaymentTransfer();
		this.interestRate = ofNullable(builder.getInterestRate()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.negativeInterest = builder.getNegativeInterest();
	}
	
	/**
	 * When the alternative provision clause is specified, it means that the ISDA CSA Japanese Law provisions specified in Paragraph 6(c)(ii) don&#39;t apply and are overwritten by this election. When it is not specified, it means that the ISDA CSA Japanese Law provisions specified in Paragraph 6(c)(ii) apply. ISDA 2016 Japanese Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (n)(iv): Alternative to Interest Amount (IM) and Interest Payment (IM). | ISDA 2016 Credit Support Annex for Variation Margin, paragraph 13, (i)(iv): Alternative to Interest Amount (VM) and Interest Payment (VM).
	 */
	@RosettaSynonym(value="specify", source="ISDA_Create_1_0", path="alternative_to_interest_amounts_and_interest_payment")
	public final String getAlternativeProvision() {
		return alternativeProvision;
	}
	
	/**
	 * Daily interest compounding is applicable when True, and not applicable when False. ISDA 2016 Japanese Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (n)(iii): Other Interest Elections. | ISDA 2016 Credit Support Annex for Variation Margin, paragraph 13, (i)(iii): Other Interest Elections.
	 */
	@RosettaSynonym(value="", source="ISDA_Create_1_0")
	public final Boolean getDailyInterestCompounding() {
		return dailyInterestCompounding;
	}
	
	/**
	 * To election to specify whether the Interest Adjustment is applicable and what its periodicity is. ISDA 2016 Japanese Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (n)(ii). | ISDA 2016 Credit Support Annex for Variation Margin, paragraph 13, (i)(ii): Transfer of Interest Payment (VM) or application of Interest Amount (VM).
	 */
	@RosettaSynonym(value="interest_adjustment", source="ISDA_Create_1_0")
	public final InterestAdjustment getInterestAdjustment() {
		return interestAdjustment;
	}
	
	/**
	 * The application of Interest Amount with respect to the Delivery Amount and the Return Amount. ISDA 2016 Japanese Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (n)(ii).
	 */
	@RosettaSynonym(value="return_amount_delivery_amount", source="ISDA_Create_1_0")
	public final InterestAmount getInterestAmount() {
		return interestAmount;
	}
	
	/**
	 * The Interest Payment Netting is applicable when True. ISDA 2016 Japanese Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (n)(ii): Transfer of Interest Payment (IM) or application of Interest Amount (IM). | ISDA 2016 Credit Support Annex for Variation Margin, paragraph 13, (i)(ii): Transfer of Interest Payment (VM) or application of Interest Amount (VM).
	 */
	@RosettaSynonym(value="", source="ISDA_Create_1_0")
	public final Boolean getInterestPaymentNetting() {
		return interestPaymentNetting;
	}
	
	/**
	 * The Interest Payment Transfer is applicable when True. ISDA 2016 Japanese Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (n)(ii): Transfer of Interest Payment (IM) or application of Interest Amount (IM). | ISDA 2016 Credit Support Annex for Variation Margin, paragraph 13, (i)(ii): Transfer of Interest Payment (VM) or application of Interest Amount (VM).
	 */
	@RosettaSynonym(value="", source="ISDA_Create_1_0")
	public final Boolean getInterestPaymentTransfer() {
		return interestPaymentTransfer;
	}
	
	/**
	 * The interest rate associated with initial or variation margin collateral, depending upon the type of credit agreement that this election is associated with. ISDA 2016 Japanese Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (n)(i): Interest Rate (IM). | ISDA 2016 Credit Support Annex for Variation Margin, paragraph 13, (i)(i): Interest Rate (VM).
	 */
	@RosettaSynonym(value="interest_rate", source="ISDA_Create_1_0")
	public final List<EligibleCurrencyInterestRate> getInterestRate() {
		return interestRate;
	}
	
	/**
	 * Negative Interest is applicable when True, and not applicable when False. ISDA 2016 Japanese Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (n)(iii): Other Interest Elections. | ISDA 2016 Credit Support Annex for Variation Margin, paragraph 13, (i)(iii): Other Interest Elections.
	 */
	@RosettaSynonym(value="", source="ISDA_Create_1_0")
	public final Boolean getNegativeInterest() {
		return negativeInterest;
	}
	
	
	@Override
	public RosettaMetaData<? extends DistributionAndInterestPayment> metaData() {
		return metaData;
	} 
	
	public DistributionAndInterestPaymentBuilder toBuilder() {
		DistributionAndInterestPaymentBuilder builder = new DistributionAndInterestPaymentBuilder();
		ofNullable(getAlternativeProvision()).ifPresent(builder::setAlternativeProvision);
		ofNullable(getDailyInterestCompounding()).ifPresent(builder::setDailyInterestCompounding);
		ofNullable(getInterestAdjustment()).ifPresent(builder::setInterestAdjustment);
		ofNullable(getInterestAmount()).ifPresent(builder::setInterestAmount);
		ofNullable(getInterestPaymentNetting()).ifPresent(builder::setInterestPaymentNetting);
		ofNullable(getInterestPaymentTransfer()).ifPresent(builder::setInterestPaymentTransfer);
		ofNullable(getInterestRate()).ifPresent(interestRate -> interestRate.forEach(builder::addInterestRate));
		ofNullable(getNegativeInterest()).ifPresent(builder::setNegativeInterest);
		return builder;
	}

	public static DistributionAndInterestPaymentBuilder builder() {
		return new DistributionAndInterestPaymentBuilder();
	}

	public static  class DistributionAndInterestPaymentBuilder extends RosettaModelObjectBuilder {
	
		protected String alternativeProvision;
		protected Boolean dailyInterestCompounding;
		protected InterestAdjustment.InterestAdjustmentBuilder interestAdjustment;
		protected InterestAmount.InterestAmountBuilder interestAmount;
		protected Boolean interestPaymentNetting;
		protected Boolean interestPaymentTransfer;
		protected List<EligibleCurrencyInterestRate.EligibleCurrencyInterestRateBuilder> interestRate;
		protected Boolean negativeInterest;
	
		public DistributionAndInterestPaymentBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends DistributionAndInterestPayment> metaData() {
			return metaData;
		} 
	
		
		public String getAlternativeProvision() {
			return alternativeProvision;
		}
		
		
		public Boolean getDailyInterestCompounding() {
			return dailyInterestCompounding;
		}
		
		
		public InterestAdjustment.InterestAdjustmentBuilder getInterestAdjustment() {
			return interestAdjustment;
		}
		
		public InterestAdjustment.InterestAdjustmentBuilder getOrCreateInterestAdjustment() {
			if (interestAdjustment!=null) {
				return interestAdjustment;
			}
			else return interestAdjustment = new InterestAdjustment.InterestAdjustmentBuilder();
		}
		
		
		public InterestAmount.InterestAmountBuilder getInterestAmount() {
			return interestAmount;
		}
		
		public InterestAmount.InterestAmountBuilder getOrCreateInterestAmount() {
			if (interestAmount!=null) {
				return interestAmount;
			}
			else return interestAmount = new InterestAmount.InterestAmountBuilder();
		}
		
		
		public Boolean getInterestPaymentNetting() {
			return interestPaymentNetting;
		}
		
		
		public Boolean getInterestPaymentTransfer() {
			return interestPaymentTransfer;
		}
		
		
		public List<EligibleCurrencyInterestRate.EligibleCurrencyInterestRateBuilder> getInterestRate() {
			return interestRate;
		}
		
		public EligibleCurrencyInterestRate.EligibleCurrencyInterestRateBuilder getOrCreateInterestRate(int index) {
			if (interestRate==null) {
				this.interestRate = new ArrayList<>();
			}
			return getIndex(interestRate, index, ()->new EligibleCurrencyInterestRate.EligibleCurrencyInterestRateBuilder());
		}
		
		
		public Boolean getNegativeInterest() {
			return negativeInterest;
		}
		
	
		public DistributionAndInterestPaymentBuilder setAlternativeProvision(String alternativeProvision) {
			this.alternativeProvision = alternativeProvision;
			return this;
		}
		public DistributionAndInterestPaymentBuilder setDailyInterestCompounding(Boolean dailyInterestCompounding) {
			this.dailyInterestCompounding = dailyInterestCompounding;
			return this;
		}
		public DistributionAndInterestPaymentBuilder setInterestAdjustment(InterestAdjustment interestAdjustment) {
			this.interestAdjustment = interestAdjustment.toBuilder();
			return this;
		}
		public DistributionAndInterestPaymentBuilder setInterestAdjustmentBuilder(InterestAdjustment.InterestAdjustmentBuilder interestAdjustment) {
			this.interestAdjustment = interestAdjustment;
			return this;
		}
		public DistributionAndInterestPaymentBuilder setInterestAmount(InterestAmount interestAmount) {
			this.interestAmount = interestAmount.toBuilder();
			return this;
		}
		public DistributionAndInterestPaymentBuilder setInterestAmountBuilder(InterestAmount.InterestAmountBuilder interestAmount) {
			this.interestAmount = interestAmount;
			return this;
		}
		public DistributionAndInterestPaymentBuilder setInterestPaymentNetting(Boolean interestPaymentNetting) {
			this.interestPaymentNetting = interestPaymentNetting;
			return this;
		}
		public DistributionAndInterestPaymentBuilder setInterestPaymentTransfer(Boolean interestPaymentTransfer) {
			this.interestPaymentTransfer = interestPaymentTransfer;
			return this;
		}
		public DistributionAndInterestPaymentBuilder addInterestRate(EligibleCurrencyInterestRate interestRate) {
			if(this.interestRate == null){
				this.interestRate = new ArrayList<>();
			}
			this.interestRate.add(interestRate.toBuilder());
			return this;
		}
		public DistributionAndInterestPaymentBuilder addInterestRate(EligibleCurrencyInterestRate interestRate, int _idx) {
			if(this.interestRate == null){
				this.interestRate = new ArrayList<>();
			}
			getIndex(this.interestRate, _idx, () -> interestRate.toBuilder());
			this.interestRate.set(_idx, interestRate.toBuilder());
			return this;
		}
		public DistributionAndInterestPaymentBuilder addInterestRate(List<EligibleCurrencyInterestRate> interestRates) {
			if(this.interestRate == null){
				this.interestRate = new ArrayList<>();
			}
			for (EligibleCurrencyInterestRate toAdd : interestRates) {
				this.interestRate.add(toAdd.toBuilder());
			}
			return this;
		}
		public DistributionAndInterestPaymentBuilder addInterestRateBuilder(EligibleCurrencyInterestRate.EligibleCurrencyInterestRateBuilder interestRate) {
			if(this.interestRate == null){
				this.interestRate = new ArrayList<>();
			}
			this.interestRate.add(interestRate);
			return this;
		}
		
		public DistributionAndInterestPaymentBuilder clearInterestRate() {
			this.interestRate = null;
			return this;
		}
		public DistributionAndInterestPaymentBuilder setNegativeInterest(Boolean negativeInterest) {
			this.negativeInterest = negativeInterest;
			return this;
		}
		
		public DistributionAndInterestPayment build() {
			return new DistributionAndInterestPayment(this);
		}
	
		@Override
		public DistributionAndInterestPaymentBuilder prune() {
			if (interestAdjustment!=null && !interestAdjustment.prune().hasData()) interestAdjustment = null;
			if (interestAmount!=null && !interestAmount.prune().hasData()) interestAmount = null;
			if (interestRate!=null) interestRate = interestRate.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getAlternativeProvision()!=null) return true;
			if (getDailyInterestCompounding()!=null) return true;
			if (getInterestAdjustment()!=null && getInterestAdjustment().hasData()) return true;
			if (getInterestAmount()!=null && getInterestAmount().hasData()) return true;
			if (getInterestPaymentNetting()!=null) return true;
			if (getInterestPaymentTransfer()!=null) return true;
			if (getInterestRate()!=null && getInterestRate().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getNegativeInterest()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("alternativeProvision"), String.class, alternativeProvision, this);
			processor.processBasic(path.newSubPath("dailyInterestCompounding"), Boolean.class, dailyInterestCompounding, this);
			processor.processBasic(path.newSubPath("interestPaymentNetting"), Boolean.class, interestPaymentNetting, this);
			processor.processBasic(path.newSubPath("interestPaymentTransfer"), Boolean.class, interestPaymentTransfer, this);
			processor.processBasic(path.newSubPath("negativeInterest"), Boolean.class, negativeInterest, this);
			
			processRosetta(path.newSubPath("interestAdjustment"), processor, InterestAdjustment.class, interestAdjustment);
			processRosetta(path.newSubPath("interestAmount"), processor, InterestAmount.class, interestAmount);
			processRosetta(path.newSubPath("interestRate"), processor, EligibleCurrencyInterestRate.class, interestRate);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			DistributionAndInterestPaymentBuilder _that = (DistributionAndInterestPaymentBuilder) o;
		
			if (alternativeProvision != null ? !alternativeProvision.equals(_that.alternativeProvision) : _that.alternativeProvision != null) return false;
			if (dailyInterestCompounding != null ? !dailyInterestCompounding.equals(_that.dailyInterestCompounding) : _that.dailyInterestCompounding != null) return false;
			if (interestAdjustment != null ? !interestAdjustment.equals(_that.interestAdjustment) : _that.interestAdjustment != null) return false;
			if (interestAmount != null ? !interestAmount.equals(_that.interestAmount) : _that.interestAmount != null) return false;
			if (interestPaymentNetting != null ? !interestPaymentNetting.equals(_that.interestPaymentNetting) : _that.interestPaymentNetting != null) return false;
			if (interestPaymentTransfer != null ? !interestPaymentTransfer.equals(_that.interestPaymentTransfer) : _that.interestPaymentTransfer != null) return false;
			if (!ListEquals.listEquals(interestRate, _that.interestRate)) return false;
			if (negativeInterest != null ? !negativeInterest.equals(_that.negativeInterest) : _that.negativeInterest != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (alternativeProvision != null ? alternativeProvision.hashCode() : 0);
			_result = 31 * _result + (dailyInterestCompounding != null ? dailyInterestCompounding.hashCode() : 0);
			_result = 31 * _result + (interestAdjustment != null ? interestAdjustment.hashCode() : 0);
			_result = 31 * _result + (interestAmount != null ? interestAmount.hashCode() : 0);
			_result = 31 * _result + (interestPaymentNetting != null ? interestPaymentNetting.hashCode() : 0);
			_result = 31 * _result + (interestPaymentTransfer != null ? interestPaymentTransfer.hashCode() : 0);
			_result = 31 * _result + (interestRate != null ? interestRate.hashCode() : 0);
			_result = 31 * _result + (negativeInterest != null ? negativeInterest.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "DistributionAndInterestPaymentBuilder {" +
				"alternativeProvision=" + this.alternativeProvision + ", " +
				"dailyInterestCompounding=" + this.dailyInterestCompounding + ", " +
				"interestAdjustment=" + this.interestAdjustment + ", " +
				"interestAmount=" + this.interestAmount + ", " +
				"interestPaymentNetting=" + this.interestPaymentNetting + ", " +
				"interestPaymentTransfer=" + this.interestPaymentTransfer + ", " +
				"interestRate=" + this.interestRate + ", " +
				"negativeInterest=" + this.negativeInterest +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("alternativeProvision"), String.class, alternativeProvision, this);
		processor.processBasic(path.newSubPath("dailyInterestCompounding"), Boolean.class, dailyInterestCompounding, this);
		processor.processBasic(path.newSubPath("interestPaymentNetting"), Boolean.class, interestPaymentNetting, this);
		processor.processBasic(path.newSubPath("interestPaymentTransfer"), Boolean.class, interestPaymentTransfer, this);
		processor.processBasic(path.newSubPath("negativeInterest"), Boolean.class, negativeInterest, this);
		
		processRosetta(path.newSubPath("interestAdjustment"), processor, InterestAdjustment.class, interestAdjustment);
		processRosetta(path.newSubPath("interestAmount"), processor, InterestAmount.class, interestAmount);
		processRosetta(path.newSubPath("interestRate"), processor, EligibleCurrencyInterestRate.class, interestRate);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		DistributionAndInterestPayment _that = (DistributionAndInterestPayment) o;
	
		if (alternativeProvision != null ? !alternativeProvision.equals(_that.alternativeProvision) : _that.alternativeProvision != null) return false;
		if (dailyInterestCompounding != null ? !dailyInterestCompounding.equals(_that.dailyInterestCompounding) : _that.dailyInterestCompounding != null) return false;
		if (interestAdjustment != null ? !interestAdjustment.equals(_that.interestAdjustment) : _that.interestAdjustment != null) return false;
		if (interestAmount != null ? !interestAmount.equals(_that.interestAmount) : _that.interestAmount != null) return false;
		if (interestPaymentNetting != null ? !interestPaymentNetting.equals(_that.interestPaymentNetting) : _that.interestPaymentNetting != null) return false;
		if (interestPaymentTransfer != null ? !interestPaymentTransfer.equals(_that.interestPaymentTransfer) : _that.interestPaymentTransfer != null) return false;
		if (!ListEquals.listEquals(interestRate, _that.interestRate)) return false;
		if (negativeInterest != null ? !negativeInterest.equals(_that.negativeInterest) : _that.negativeInterest != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (alternativeProvision != null ? alternativeProvision.hashCode() : 0);
		_result = 31 * _result + (dailyInterestCompounding != null ? dailyInterestCompounding.hashCode() : 0);
		_result = 31 * _result + (interestAdjustment != null ? interestAdjustment.hashCode() : 0);
		_result = 31 * _result + (interestAmount != null ? interestAmount.hashCode() : 0);
		_result = 31 * _result + (interestPaymentNetting != null ? interestPaymentNetting.hashCode() : 0);
		_result = 31 * _result + (interestPaymentTransfer != null ? interestPaymentTransfer.hashCode() : 0);
		_result = 31 * _result + (interestRate != null ? interestRate.hashCode() : 0);
		_result = 31 * _result + (negativeInterest != null ? negativeInterest.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "DistributionAndInterestPayment {" +
			"alternativeProvision=" + this.alternativeProvision + ", " +
			"dailyInterestCompounding=" + this.dailyInterestCompounding + ", " +
			"interestAdjustment=" + this.interestAdjustment + ", " +
			"interestAmount=" + this.interestAmount + ", " +
			"interestPaymentNetting=" + this.interestPaymentNetting + ", " +
			"interestPaymentTransfer=" + this.interestPaymentTransfer + ", " +
			"interestRate=" + this.interestRate + ", " +
			"negativeInterest=" + this.negativeInterest +
		'}';
	}
}
