package org.isda.cdm;

import org.isda.cdm.DeterminationMethodEnum;
import org.isda.cdm.metafields.BasicReferenceWithMetaString;
import org.isda.cdm.metafields.BasicReferenceWithMetaString.BasicReferenceWithMetaStringBuilder;
import org.isda.cdm.metafields.FieldWithMetaString;
import org.isda.cdm.metafields.FieldWithMetaString.FieldWithMetaStringBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.DividendCurrencyMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class to specify the currency in which the dividends will be denominated, i.e. either in the dividend currency or in a currency specified as part of the contract.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class DividendCurrency extends RosettaModelObject {
	private final FieldWithMetaString currency;
	private final BasicReferenceWithMetaString currencyReference;
	private final DeterminationMethodEnum determinationMethod;
	private static DividendCurrencyMeta metaData = new DividendCurrencyMeta();
	
	DividendCurrency(DividendCurrencyBuilder builder) {
		this.currency = ofNullable(builder.getCurrency()).map(f->f.build()).orElse(null);
		this.currencyReference = ofNullable(builder.getCurrencyReference()).map(f->f.build()).orElse(null);
		this.determinationMethod = builder.getDeterminationMethod();
	}
	
	/**
	 * The currency in which the dividend is denominated. The list of valid currencies is not presently positioned as an enumeration as part of the CDM because that scope is limited to the values specified by ISDA and FpML. As a result, implementers have to make reference to the relevant standard, such as the ISO 4217 standard for currency codes.
	 */
	@RosettaSynonym(value="currency", source="FpML_5_10")
	public final FieldWithMetaString getCurrency() {
		return currency;
	}
	
	/**
	 * Reference to a currency specified elsewhere in the document
	 */
	@RosettaSynonym(value="currencyReference", source="FpML_5_10")
	public final BasicReferenceWithMetaString getCurrencyReference() {
		return currencyReference;
	}
	
	/**
	 * Specifies the method according to which the dividend is determined, e.g. the dividend currency.
	 */
	@RosettaSynonym(value="determinationMethod", source="FpML_5_10")
	public final DeterminationMethodEnum getDeterminationMethod() {
		return determinationMethod;
	}
	
	
	@Override
	public RosettaMetaData<? extends DividendCurrency> metaData() {
		return metaData;
	} 
	
	public DividendCurrencyBuilder toBuilder() {
		DividendCurrencyBuilder builder = new DividendCurrencyBuilder();
		ofNullable(getCurrency()).ifPresent(builder::setCurrency);
		ofNullable(getCurrencyReference()).ifPresent(builder::setCurrencyReference);
		ofNullable(getDeterminationMethod()).ifPresent(builder::setDeterminationMethod);
		return builder;
	}

	public static DividendCurrencyBuilder builder() {
		return new DividendCurrencyBuilder();
	}

	public static  class DividendCurrencyBuilder extends RosettaModelObjectBuilder {
	
		protected FieldWithMetaStringBuilder currency;
		protected BasicReferenceWithMetaStringBuilder currencyReference;
		protected DeterminationMethodEnum determinationMethod;
	
		public DividendCurrencyBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends DividendCurrency> metaData() {
			return metaData;
		} 
	
		
		public FieldWithMetaStringBuilder getCurrency() {
			return currency;
		}
		
		public FieldWithMetaStringBuilder getOrCreateCurrency() {
			if (currency!=null) {
				return currency;
			}
			else return currency = new FieldWithMetaStringBuilder();
		}
		
		
		public BasicReferenceWithMetaStringBuilder getCurrencyReference() {
			return currencyReference;
		}
		
		public BasicReferenceWithMetaStringBuilder getOrCreateCurrencyReference() {
			if (currencyReference!=null) {
				return currencyReference;
			}
			else return currencyReference = new BasicReferenceWithMetaStringBuilder();
		}
		
		
		public DeterminationMethodEnum getDeterminationMethod() {
			return determinationMethod;
		}
		
	
		public DividendCurrencyBuilder setCurrency(FieldWithMetaString currency) {
			this.currency = currency.toBuilder();
			return this;
		}
		public DividendCurrencyBuilder setCurrencyRef(String currency) {
			return setCurrency(FieldWithMetaString.builder().setValue(currency).build());
		}
		public DividendCurrencyBuilder setCurrencyReference(BasicReferenceWithMetaString currencyReference) {
			this.currencyReference = currencyReference.toBuilder();
			return this;
		}
		public DividendCurrencyBuilder setCurrencyReferenceRef(String currencyReference) {
			return setCurrencyReference(BasicReferenceWithMetaString.builder().setValue(currencyReference).build());
		}
		public DividendCurrencyBuilder setDeterminationMethod(DeterminationMethodEnum determinationMethod) {
			this.determinationMethod = determinationMethod;
			return this;
		}
		
		public DividendCurrency build() {
			return new DividendCurrency(this);
		}
	
		@Override
		public DividendCurrencyBuilder prune() {
			if (currency!=null && !currency.prune().hasData()) currency = null;
			if (currencyReference!=null && !currencyReference.prune().hasData()) currencyReference = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getCurrency()!=null) return true;
			if (getCurrencyReference()!=null) return true;
			if (getDeterminationMethod()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("determinationMethod"), DeterminationMethodEnum.class, determinationMethod, this);
			
			processRosetta(path.newSubPath("currency"), processor, FieldWithMetaString.class, currency);
			processRosetta(path.newSubPath("currencyReference"), processor, BasicReferenceWithMetaString.class, currencyReference);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			DividendCurrencyBuilder _that = (DividendCurrencyBuilder) o;
		
			if (currency != null ? !currency.equals(_that.currency) : _that.currency != null) return false;
			if (currencyReference != null ? !currencyReference.equals(_that.currencyReference) : _that.currencyReference != null) return false;
			if (determinationMethod != null ? !determinationMethod.equals(_that.determinationMethod) : _that.determinationMethod != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (currency != null ? currency.hashCode() : 0);
			_result = 31 * _result + (currencyReference != null ? currencyReference.hashCode() : 0);
			_result = 31 * _result + (determinationMethod != null ? determinationMethod.getClass().getName().hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "DividendCurrencyBuilder {" +
				"currency=" + this.currency + ", " +
				"currencyReference=" + this.currencyReference + ", " +
				"determinationMethod=" + this.determinationMethod +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("determinationMethod"), DeterminationMethodEnum.class, determinationMethod, this);
		
		processRosetta(path.newSubPath("currency"), processor, FieldWithMetaString.class, currency);
		processRosetta(path.newSubPath("currencyReference"), processor, BasicReferenceWithMetaString.class, currencyReference);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		DividendCurrency _that = (DividendCurrency) o;
	
		if (currency != null ? !currency.equals(_that.currency) : _that.currency != null) return false;
		if (currencyReference != null ? !currencyReference.equals(_that.currencyReference) : _that.currencyReference != null) return false;
		if (determinationMethod != null ? !determinationMethod.equals(_that.determinationMethod) : _that.determinationMethod != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (currency != null ? currency.hashCode() : 0);
		_result = 31 * _result + (currencyReference != null ? currencyReference.hashCode() : 0);
		_result = 31 * _result + (determinationMethod != null ? determinationMethod.getClass().getName().hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "DividendCurrency {" +
			"currency=" + this.currency + ", " +
			"currencyReference=" + this.currencyReference + ", " +
			"determinationMethod=" + this.determinationMethod +
		'}';
	}
}
