package org.isda.cdm;

import org.isda.cdm.DividendDateReferenceEnum;
import org.isda.cdm.Offset;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.DividendDateReferenceMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class to specify the dividend date by reference to another date, with the ability to apply and offset. This class doesn&#39;t exist in FpML and is meant to simplify the choice constraint associated with the DividendPaymentDate class.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class DividendDateReference extends RosettaModelObject {
	private final DividendDateReferenceEnum dateReference;
	private final Offset paymentDateOffset;
	private static DividendDateReferenceMeta metaData = new DividendDateReferenceMeta();
	
	DividendDateReference(DividendDateReferenceBuilder builder) {
		this.dateReference = builder.getDateReference();
		this.paymentDateOffset = ofNullable(builder.getPaymentDateOffset()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * Specification of the dividend date using an enumeration, with values such as the pay date, the ex-date or the record date.
	 */
	@RosettaSynonym(value="dateReference", source="FpML_5_10")
	public final DividendDateReferenceEnum getDateReference() {
		return dateReference;
	}
	
	/**
	 * Only to be used when SharePayment has been specified in the dividendDateReference element. The number of Currency Business Days following the day on which the Issuer of the Shares pays the relevant dividend to holders of record of the Shares.
	 */
	@RosettaSynonym(value="paymentDateOffset", source="FpML_5_10")
	public final Offset getPaymentDateOffset() {
		return paymentDateOffset;
	}
	
	
	@Override
	public RosettaMetaData<? extends DividendDateReference> metaData() {
		return metaData;
	} 
	
	public DividendDateReferenceBuilder toBuilder() {
		DividendDateReferenceBuilder builder = new DividendDateReferenceBuilder();
		ofNullable(getDateReference()).ifPresent(builder::setDateReference);
		ofNullable(getPaymentDateOffset()).ifPresent(builder::setPaymentDateOffset);
		return builder;
	}

	public static DividendDateReferenceBuilder builder() {
		return new DividendDateReferenceBuilder();
	}

	public static  class DividendDateReferenceBuilder extends RosettaModelObjectBuilder {
	
		protected DividendDateReferenceEnum dateReference;
		protected Offset.OffsetBuilder paymentDateOffset;
	
		public DividendDateReferenceBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends DividendDateReference> metaData() {
			return metaData;
		} 
	
		
		public DividendDateReferenceEnum getDateReference() {
			return dateReference;
		}
		
		
		public Offset.OffsetBuilder getPaymentDateOffset() {
			return paymentDateOffset;
		}
		
		public Offset.OffsetBuilder getOrCreatePaymentDateOffset() {
			if (paymentDateOffset!=null) {
				return paymentDateOffset;
			}
			else return paymentDateOffset = new Offset.OffsetBuilder();
		}
		
	
		public DividendDateReferenceBuilder setDateReference(DividendDateReferenceEnum dateReference) {
			this.dateReference = dateReference;
			return this;
		}
		public DividendDateReferenceBuilder setPaymentDateOffset(Offset paymentDateOffset) {
			this.paymentDateOffset = paymentDateOffset.toBuilder();
			return this;
		}
		public DividendDateReferenceBuilder setPaymentDateOffsetBuilder(Offset.OffsetBuilder paymentDateOffset) {
			this.paymentDateOffset = paymentDateOffset;
			return this;
		}
		
		public DividendDateReference build() {
			return new DividendDateReference(this);
		}
	
		@Override
		public DividendDateReferenceBuilder prune() {
			if (paymentDateOffset!=null && !paymentDateOffset.prune().hasData()) paymentDateOffset = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getDateReference()!=null) return true;
			if (getPaymentDateOffset()!=null && getPaymentDateOffset().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("dateReference"), DividendDateReferenceEnum.class, dateReference, this);
			
			processRosetta(path.newSubPath("paymentDateOffset"), processor, Offset.class, paymentDateOffset);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			DividendDateReferenceBuilder _that = (DividendDateReferenceBuilder) o;
		
			if (dateReference != null ? !dateReference.equals(_that.dateReference) : _that.dateReference != null) return false;
			if (paymentDateOffset != null ? !paymentDateOffset.equals(_that.paymentDateOffset) : _that.paymentDateOffset != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (dateReference != null ? dateReference.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (paymentDateOffset != null ? paymentDateOffset.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "DividendDateReferenceBuilder {" +
				"dateReference=" + this.dateReference + ", " +
				"paymentDateOffset=" + this.paymentDateOffset +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("dateReference"), DividendDateReferenceEnum.class, dateReference, this);
		
		processRosetta(path.newSubPath("paymentDateOffset"), processor, Offset.class, paymentDateOffset);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		DividendDateReference _that = (DividendDateReference) o;
	
		if (dateReference != null ? !dateReference.equals(_that.dateReference) : _that.dateReference != null) return false;
		if (paymentDateOffset != null ? !paymentDateOffset.equals(_that.paymentDateOffset) : _that.paymentDateOffset != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (dateReference != null ? dateReference.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (paymentDateOffset != null ? paymentDateOffset.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "DividendDateReference {" +
			"dateReference=" + this.dateReference + ", " +
			"paymentDateOffset=" + this.paymentDateOffset +
		'}';
	}
}
