package org.isda.cdm;

import java.math.BigDecimal;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.DividendPayoutMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.math.BigDecimal;
		import static java.util.Optional.ofNullable;

/**
 * A class describing the dividend payout ratio associated with an equity underlier. In certain cases the actual ratio is not known on trade inception, and only general conditions are then specified.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="DividendPayout", source="FpML_5_10")
public class DividendPayout extends RosettaModelObject {
	private final BigDecimal dividendPayoutRatio;
	private final BigDecimal dividendPayoutRatioCash;
	private final BigDecimal dividendPayoutRatioNonCash;
	private static DividendPayoutMeta metaData = new DividendPayoutMeta();
	
	DividendPayout(DividendPayoutBuilder builder) {
		this.dividendPayoutRatio = builder.getDividendPayoutRatio();
		this.dividendPayoutRatioCash = builder.getDividendPayoutRatioCash();
		this.dividendPayoutRatioNonCash = builder.getDividendPayoutRatioNonCash();
	}
	
	/**
	 * Specifies the total actual dividend payout ratio associated with the equity underlier. A ratio of 90% should be expressed at 0.90.
	 */
	@RosettaSynonym(value="dividendPayoutRatio", source="FpML_5_10")
	public final BigDecimal getDividendPayoutRatio() {
		return dividendPayoutRatio;
	}
	
	/**
	 * Specifies the cash actual dividend payout ratio associated with the equity underlier. A ratio of 90% should be expressed at 0.90.
	 */
	@RosettaSynonym(value="dividendPayoutRatioCash", source="FpML_5_10")
	public final BigDecimal getDividendPayoutRatioCash() {
		return dividendPayoutRatioCash;
	}
	
	/**
	 * Specifies the non cash actual dividend payout ratio associated with the equity underlier. A ratio of 90% should be expressed at 0.90.
	 */
	@RosettaSynonym(value="dividendPayoutRatioNonCash", source="FpML_5_10")
	public final BigDecimal getDividendPayoutRatioNonCash() {
		return dividendPayoutRatioNonCash;
	}
	
	
	@Override
	public RosettaMetaData<? extends DividendPayout> metaData() {
		return metaData;
	} 
	
	public DividendPayoutBuilder toBuilder() {
		DividendPayoutBuilder builder = new DividendPayoutBuilder();
		ofNullable(getDividendPayoutRatio()).ifPresent(builder::setDividendPayoutRatio);
		ofNullable(getDividendPayoutRatioCash()).ifPresent(builder::setDividendPayoutRatioCash);
		ofNullable(getDividendPayoutRatioNonCash()).ifPresent(builder::setDividendPayoutRatioNonCash);
		return builder;
	}

	public static DividendPayoutBuilder builder() {
		return new DividendPayoutBuilder();
	}

	public static  class DividendPayoutBuilder extends RosettaModelObjectBuilder {
	
		protected BigDecimal dividendPayoutRatio;
		protected BigDecimal dividendPayoutRatioCash;
		protected BigDecimal dividendPayoutRatioNonCash;
	
		public DividendPayoutBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends DividendPayout> metaData() {
			return metaData;
		} 
	
		
		public BigDecimal getDividendPayoutRatio() {
			return dividendPayoutRatio;
		}
		
		
		public BigDecimal getDividendPayoutRatioCash() {
			return dividendPayoutRatioCash;
		}
		
		
		public BigDecimal getDividendPayoutRatioNonCash() {
			return dividendPayoutRatioNonCash;
		}
		
	
		public DividendPayoutBuilder setDividendPayoutRatio(BigDecimal dividendPayoutRatio) {
			this.dividendPayoutRatio = dividendPayoutRatio;
			return this;
		}
		public DividendPayoutBuilder setDividendPayoutRatioCash(BigDecimal dividendPayoutRatioCash) {
			this.dividendPayoutRatioCash = dividendPayoutRatioCash;
			return this;
		}
		public DividendPayoutBuilder setDividendPayoutRatioNonCash(BigDecimal dividendPayoutRatioNonCash) {
			this.dividendPayoutRatioNonCash = dividendPayoutRatioNonCash;
			return this;
		}
		
		public DividendPayout build() {
			return new DividendPayout(this);
		}
	
		@Override
		public DividendPayoutBuilder prune() {
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getDividendPayoutRatio()!=null) return true;
			if (getDividendPayoutRatioCash()!=null) return true;
			if (getDividendPayoutRatioNonCash()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("dividendPayoutRatio"), BigDecimal.class, dividendPayoutRatio, this);
			processor.processBasic(path.newSubPath("dividendPayoutRatioCash"), BigDecimal.class, dividendPayoutRatioCash, this);
			processor.processBasic(path.newSubPath("dividendPayoutRatioNonCash"), BigDecimal.class, dividendPayoutRatioNonCash, this);
			
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			DividendPayoutBuilder _that = (DividendPayoutBuilder) o;
		
			if (dividendPayoutRatio != null ? !dividendPayoutRatio.equals(_that.dividendPayoutRatio) : _that.dividendPayoutRatio != null) return false;
			if (dividendPayoutRatioCash != null ? !dividendPayoutRatioCash.equals(_that.dividendPayoutRatioCash) : _that.dividendPayoutRatioCash != null) return false;
			if (dividendPayoutRatioNonCash != null ? !dividendPayoutRatioNonCash.equals(_that.dividendPayoutRatioNonCash) : _that.dividendPayoutRatioNonCash != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (dividendPayoutRatio != null ? dividendPayoutRatio.hashCode() : 0);
			_result = 31 * _result + (dividendPayoutRatioCash != null ? dividendPayoutRatioCash.hashCode() : 0);
			_result = 31 * _result + (dividendPayoutRatioNonCash != null ? dividendPayoutRatioNonCash.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "DividendPayoutBuilder {" +
				"dividendPayoutRatio=" + this.dividendPayoutRatio + ", " +
				"dividendPayoutRatioCash=" + this.dividendPayoutRatioCash + ", " +
				"dividendPayoutRatioNonCash=" + this.dividendPayoutRatioNonCash +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("dividendPayoutRatio"), BigDecimal.class, dividendPayoutRatio, this);
		processor.processBasic(path.newSubPath("dividendPayoutRatioCash"), BigDecimal.class, dividendPayoutRatioCash, this);
		processor.processBasic(path.newSubPath("dividendPayoutRatioNonCash"), BigDecimal.class, dividendPayoutRatioNonCash, this);
		
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		DividendPayout _that = (DividendPayout) o;
	
		if (dividendPayoutRatio != null ? !dividendPayoutRatio.equals(_that.dividendPayoutRatio) : _that.dividendPayoutRatio != null) return false;
		if (dividendPayoutRatioCash != null ? !dividendPayoutRatioCash.equals(_that.dividendPayoutRatioCash) : _that.dividendPayoutRatioCash != null) return false;
		if (dividendPayoutRatioNonCash != null ? !dividendPayoutRatioNonCash.equals(_that.dividendPayoutRatioNonCash) : _that.dividendPayoutRatioNonCash != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (dividendPayoutRatio != null ? dividendPayoutRatio.hashCode() : 0);
		_result = 31 * _result + (dividendPayoutRatioCash != null ? dividendPayoutRatioCash.hashCode() : 0);
		_result = 31 * _result + (dividendPayoutRatioNonCash != null ? dividendPayoutRatioNonCash.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "DividendPayout {" +
			"dividendPayoutRatio=" + this.dividendPayoutRatio + ", " +
			"dividendPayoutRatioCash=" + this.dividendPayoutRatioCash + ", " +
			"dividendPayoutRatioNonCash=" + this.dividendPayoutRatioNonCash +
		'}';
	}
}
