package org.isda.cdm;

import java.lang.Boolean;
import org.isda.cdm.DividendAmountTypeEnum;
import org.isda.cdm.DividendCurrency;
import org.isda.cdm.DividendEntitlementEnum;
import org.isda.cdm.DividendPaymentDate;
import org.isda.cdm.DividendPayout;
import org.isda.cdm.DividendPeriodEnum;
import org.isda.cdm.metafields.BasicReferenceWithMetaDate;
import org.isda.cdm.metafields.BasicReferenceWithMetaDate.BasicReferenceWithMetaDateBuilder;
import org.isda.cdm.metafields.ReferenceWithMetaParty;
import org.isda.cdm.metafields.ReferenceWithMetaParty.ReferenceWithMetaPartyBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.DividendReturnTermsMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;
		import com.rosetta.model.lib.records.Date;
		import java.util.stream.Collectors;

/**
 * A class describing the conditions governing the payment of dividends to the receiver of the equity return, with the exception of the dividend payout ratio, which is defined for each of the underlying components.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="DividendConditions", source="FpML_5_10")
public class DividendReturnTerms extends RosettaModelObject {
	private final DividendAmountTypeEnum dividendAmountType;
	private final DividendCurrency dividendCurrency;
	private final DividendEntitlementEnum dividendEntitlement;
	private final DividendPaymentDate dividendPaymentDate;
	private final DividendPayout dividendPayout;
	private final DividendPeriodEnum dividendPeriod;
	private final BasicReferenceWithMetaDate dividendPeriodEffectiveDate;
	private final BasicReferenceWithMetaDate dividendPeriodEndDate;
	private final Boolean dividendReinvestment;
	private final DividendAmountTypeEnum excessDividendAmount;
	private final ReferenceWithMetaParty extraordinaryDividendsParty;
	private static DividendReturnTermsMeta metaData = new DividendReturnTermsMeta();
	
	DividendReturnTerms(DividendReturnTermsBuilder builder) {
		this.dividendAmountType = builder.getDividendAmountType();
		this.dividendCurrency = ofNullable(builder.getDividendCurrency()).map(f->f.build()).orElse(null);
		this.dividendEntitlement = builder.getDividendEntitlement();
		this.dividendPaymentDate = ofNullable(builder.getDividendPaymentDate()).map(f->f.build()).orElse(null);
		this.dividendPayout = ofNullable(builder.getDividendPayout()).map(f->f.build()).orElse(null);
		this.dividendPeriod = builder.getDividendPeriod();
		this.dividendPeriodEffectiveDate = ofNullable(builder.getDividendPeriodEffectiveDate()).map(f->f.build()).orElse(null);
		this.dividendPeriodEndDate = ofNullable(builder.getDividendPeriodEndDate()).map(f->f.build()).orElse(null);
		this.dividendReinvestment = builder.getDividendReinvestment();
		this.excessDividendAmount = builder.getExcessDividendAmount();
		this.extraordinaryDividendsParty = ofNullable(builder.getExtraordinaryDividendsParty()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * Specifies whether the dividend is paid with respect to the Dividend Period.
	 */
	@RosettaSynonym(value="dividendAmount", source="FpML_5_10")
	public final DividendAmountTypeEnum getDividendAmountType() {
		return dividendAmountType;
	}
	
	/**
	 * Specifies the currency in which the dividend will be denominated, e.g. the dividend currency, or a specified currency. This class is not specified as such in FpML, which makes use of the CurrencyAndDeterminationMethod.model to specify such terms.
	 */
	public final DividendCurrency getDividendCurrency() {
		return dividendCurrency;
	}
	
	/**
	 * Defines the date on which the receiver of the equity return is entitled to the dividend.
	 */
	@RosettaSynonym(value="dividendEntitlement", source="FpML_5_10")
	public final DividendEntitlementEnum getDividendEntitlement() {
		return dividendEntitlement;
	}
	
	/**
	 * Specifies when the dividend will be paid to the receiver of the equity return. Has the meaning as defined in the ISDA 2002 Equity Derivatives Definitions. Is not applicable in the case of a dividend reinvestment election.
	 */
	@RosettaSynonym(value="dividendPaymentDate", source="FpML_5_10")
	public final DividendPaymentDate getDividendPaymentDate() {
		return dividendPaymentDate;
	}
	
	/**
	 * Specifies the dividend payout ratio associated with the underlier. In FpML 5.10 the payout is positioned at the underlier level, although there is an intent to reconsider this approach and position it at the leg level. This is approach adopted by the CDM.
	 */
	@RosettaSynonym(value="dividendPayout", source="FpML_5_10")
	public final DividendPayout getDividendPayout() {
		return dividendPayout;
	}
	
	/**
	 * 2002 ISDA Equity Derivatives Definitions: Dividend Period as either the First Period or the Second Period. | 
	 */
	@RosettaSynonym(value="dividendPeriod", source="FpML_5_10")
	public final DividendPeriodEnum getDividendPeriod() {
		return dividendPeriod;
	}
	
	/**
	 * 2002 ISDA Equity Derivatives Definitions: Dividend Period as such other period determined as provided in the related Confirmation. | 
	 */
	@RosettaSynonym(value="dividendPeriodEffectiveDate", source="FpML_5_10")
	public final BasicReferenceWithMetaDate getDividendPeriodEffectiveDate() {
		return dividendPeriodEffectiveDate;
	}
	
	/**
	 * 2002 ISDA Equity Derivatives Definitions: Dividend Period as such other period determined as provided in the related Confirmation.
	 */
	@RosettaSynonym(value="dividendPeriodEndDate", source="FpML_5_10")
	public final BasicReferenceWithMetaDate getDividendPeriodEndDate() {
		return dividendPeriodEndDate;
	}
	
	/**
	 * Boolean element that defines whether the dividend will be reinvested or not.
	 */
	@RosettaSynonym(value="dividendReinvestment", source="FpML_5_10")
	public final Boolean getDividendReinvestment() {
		return dividendReinvestment;
	}
	
	/**
	 * Determination of Gross Cash Dividend per Share.
	 */
	@RosettaSynonym(value="excessDividendAmount", source="FpML_5_10")
	public final DividendAmountTypeEnum getExcessDividendAmount() {
		return excessDividendAmount;
	}
	
	/**
	 * Reference to the party which determines if dividends are extraordinary in relation to normal levels.
	 */
	@RosettaSynonym(value="extraOrdinaryDividends", source="FpML_5_10")
	public final ReferenceWithMetaParty getExtraordinaryDividendsParty() {
		return extraordinaryDividendsParty;
	}
	
	
	@Override
	public RosettaMetaData<? extends DividendReturnTerms> metaData() {
		return metaData;
	} 
	
	public DividendReturnTermsBuilder toBuilder() {
		DividendReturnTermsBuilder builder = new DividendReturnTermsBuilder();
		ofNullable(getDividendAmountType()).ifPresent(builder::setDividendAmountType);
		ofNullable(getDividendCurrency()).ifPresent(builder::setDividendCurrency);
		ofNullable(getDividendEntitlement()).ifPresent(builder::setDividendEntitlement);
		ofNullable(getDividendPaymentDate()).ifPresent(builder::setDividendPaymentDate);
		ofNullable(getDividendPayout()).ifPresent(builder::setDividendPayout);
		ofNullable(getDividendPeriod()).ifPresent(builder::setDividendPeriod);
		ofNullable(getDividendPeriodEffectiveDate()).ifPresent(builder::setDividendPeriodEffectiveDate);
		ofNullable(getDividendPeriodEndDate()).ifPresent(builder::setDividendPeriodEndDate);
		ofNullable(getDividendReinvestment()).ifPresent(builder::setDividendReinvestment);
		ofNullable(getExcessDividendAmount()).ifPresent(builder::setExcessDividendAmount);
		ofNullable(getExtraordinaryDividendsParty()).ifPresent(builder::setExtraordinaryDividendsParty);
		return builder;
	}

	public static DividendReturnTermsBuilder builder() {
		return new DividendReturnTermsBuilder();
	}

	public static  class DividendReturnTermsBuilder extends RosettaModelObjectBuilder {
	
		protected DividendAmountTypeEnum dividendAmountType;
		protected DividendCurrency.DividendCurrencyBuilder dividendCurrency;
		protected DividendEntitlementEnum dividendEntitlement;
		protected DividendPaymentDate.DividendPaymentDateBuilder dividendPaymentDate;
		protected DividendPayout.DividendPayoutBuilder dividendPayout;
		protected DividendPeriodEnum dividendPeriod;
		protected BasicReferenceWithMetaDateBuilder dividendPeriodEffectiveDate;
		protected BasicReferenceWithMetaDateBuilder dividendPeriodEndDate;
		protected Boolean dividendReinvestment;
		protected DividendAmountTypeEnum excessDividendAmount;
		protected ReferenceWithMetaPartyBuilder extraordinaryDividendsParty;
	
		public DividendReturnTermsBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends DividendReturnTerms> metaData() {
			return metaData;
		} 
	
		
		public DividendAmountTypeEnum getDividendAmountType() {
			return dividendAmountType;
		}
		
		
		public DividendCurrency.DividendCurrencyBuilder getDividendCurrency() {
			return dividendCurrency;
		}
		
		public DividendCurrency.DividendCurrencyBuilder getOrCreateDividendCurrency() {
			if (dividendCurrency!=null) {
				return dividendCurrency;
			}
			else return dividendCurrency = new DividendCurrency.DividendCurrencyBuilder();
		}
		
		
		public DividendEntitlementEnum getDividendEntitlement() {
			return dividendEntitlement;
		}
		
		
		public DividendPaymentDate.DividendPaymentDateBuilder getDividendPaymentDate() {
			return dividendPaymentDate;
		}
		
		public DividendPaymentDate.DividendPaymentDateBuilder getOrCreateDividendPaymentDate() {
			if (dividendPaymentDate!=null) {
				return dividendPaymentDate;
			}
			else return dividendPaymentDate = new DividendPaymentDate.DividendPaymentDateBuilder();
		}
		
		
		public DividendPayout.DividendPayoutBuilder getDividendPayout() {
			return dividendPayout;
		}
		
		public DividendPayout.DividendPayoutBuilder getOrCreateDividendPayout() {
			if (dividendPayout!=null) {
				return dividendPayout;
			}
			else return dividendPayout = new DividendPayout.DividendPayoutBuilder();
		}
		
		
		public DividendPeriodEnum getDividendPeriod() {
			return dividendPeriod;
		}
		
		
		public BasicReferenceWithMetaDateBuilder getDividendPeriodEffectiveDate() {
			return dividendPeriodEffectiveDate;
		}
		
		public BasicReferenceWithMetaDateBuilder getOrCreateDividendPeriodEffectiveDate() {
			if (dividendPeriodEffectiveDate!=null) {
				return dividendPeriodEffectiveDate;
			}
			else return dividendPeriodEffectiveDate = new BasicReferenceWithMetaDateBuilder();
		}
		
		
		public BasicReferenceWithMetaDateBuilder getDividendPeriodEndDate() {
			return dividendPeriodEndDate;
		}
		
		public BasicReferenceWithMetaDateBuilder getOrCreateDividendPeriodEndDate() {
			if (dividendPeriodEndDate!=null) {
				return dividendPeriodEndDate;
			}
			else return dividendPeriodEndDate = new BasicReferenceWithMetaDateBuilder();
		}
		
		
		public Boolean getDividendReinvestment() {
			return dividendReinvestment;
		}
		
		
		public DividendAmountTypeEnum getExcessDividendAmount() {
			return excessDividendAmount;
		}
		
		
		public ReferenceWithMetaPartyBuilder getExtraordinaryDividendsParty() {
			return extraordinaryDividendsParty;
		}
		
		public ReferenceWithMetaPartyBuilder getOrCreateExtraordinaryDividendsParty() {
			if (extraordinaryDividendsParty!=null) {
				return extraordinaryDividendsParty;
			}
			else return extraordinaryDividendsParty = new ReferenceWithMetaPartyBuilder();
		}
		
	
		public DividendReturnTermsBuilder setDividendAmountType(DividendAmountTypeEnum dividendAmountType) {
			this.dividendAmountType = dividendAmountType;
			return this;
		}
		public DividendReturnTermsBuilder setDividendCurrency(DividendCurrency dividendCurrency) {
			this.dividendCurrency = dividendCurrency.toBuilder();
			return this;
		}
		public DividendReturnTermsBuilder setDividendCurrencyBuilder(DividendCurrency.DividendCurrencyBuilder dividendCurrency) {
			this.dividendCurrency = dividendCurrency;
			return this;
		}
		public DividendReturnTermsBuilder setDividendEntitlement(DividendEntitlementEnum dividendEntitlement) {
			this.dividendEntitlement = dividendEntitlement;
			return this;
		}
		public DividendReturnTermsBuilder setDividendPaymentDate(DividendPaymentDate dividendPaymentDate) {
			this.dividendPaymentDate = dividendPaymentDate.toBuilder();
			return this;
		}
		public DividendReturnTermsBuilder setDividendPaymentDateBuilder(DividendPaymentDate.DividendPaymentDateBuilder dividendPaymentDate) {
			this.dividendPaymentDate = dividendPaymentDate;
			return this;
		}
		public DividendReturnTermsBuilder setDividendPayout(DividendPayout dividendPayout) {
			this.dividendPayout = dividendPayout.toBuilder();
			return this;
		}
		public DividendReturnTermsBuilder setDividendPayoutBuilder(DividendPayout.DividendPayoutBuilder dividendPayout) {
			this.dividendPayout = dividendPayout;
			return this;
		}
		public DividendReturnTermsBuilder setDividendPeriod(DividendPeriodEnum dividendPeriod) {
			this.dividendPeriod = dividendPeriod;
			return this;
		}
		public DividendReturnTermsBuilder setDividendPeriodEffectiveDate(BasicReferenceWithMetaDate dividendPeriodEffectiveDate) {
			this.dividendPeriodEffectiveDate = dividendPeriodEffectiveDate.toBuilder();
			return this;
		}
		public DividendReturnTermsBuilder setDividendPeriodEffectiveDateRef(Date dividendPeriodEffectiveDate) {
			return setDividendPeriodEffectiveDate(BasicReferenceWithMetaDate.builder().setValue(dividendPeriodEffectiveDate).build());
		}
		public DividendReturnTermsBuilder setDividendPeriodEndDate(BasicReferenceWithMetaDate dividendPeriodEndDate) {
			this.dividendPeriodEndDate = dividendPeriodEndDate.toBuilder();
			return this;
		}
		public DividendReturnTermsBuilder setDividendPeriodEndDateRef(Date dividendPeriodEndDate) {
			return setDividendPeriodEndDate(BasicReferenceWithMetaDate.builder().setValue(dividendPeriodEndDate).build());
		}
		public DividendReturnTermsBuilder setDividendReinvestment(Boolean dividendReinvestment) {
			this.dividendReinvestment = dividendReinvestment;
			return this;
		}
		public DividendReturnTermsBuilder setExcessDividendAmount(DividendAmountTypeEnum excessDividendAmount) {
			this.excessDividendAmount = excessDividendAmount;
			return this;
		}
		public DividendReturnTermsBuilder setExtraordinaryDividendsParty(ReferenceWithMetaParty extraordinaryDividendsParty) {
			this.extraordinaryDividendsParty = extraordinaryDividendsParty.toBuilder();
			return this;
		}
		public DividendReturnTermsBuilder setExtraordinaryDividendsPartyBuilder(ReferenceWithMetaPartyBuilder extraordinaryDividendsParty) {
			this.extraordinaryDividendsParty = extraordinaryDividendsParty;
			return this;
		}
		public DividendReturnTermsBuilder setExtraordinaryDividendsPartyRef(Party.PartyBuilder extraordinaryDividendsParty) {
			return setExtraordinaryDividendsParty(ReferenceWithMetaParty.builder().setValueBuilder(extraordinaryDividendsParty).build());
		}
		public DividendReturnTermsBuilder setExtraordinaryDividendsPartyRef(Party extraordinaryDividendsParty) {
			return setExtraordinaryDividendsPartyRef(Party.builder());
		}
		
		public DividendReturnTerms build() {
			return new DividendReturnTerms(this);
		}
	
		@Override
		public DividendReturnTermsBuilder prune() {
			if (dividendCurrency!=null && !dividendCurrency.prune().hasData()) dividendCurrency = null;
			if (dividendPaymentDate!=null && !dividendPaymentDate.prune().hasData()) dividendPaymentDate = null;
			if (dividendPayout!=null && !dividendPayout.prune().hasData()) dividendPayout = null;
			if (dividendPeriodEffectiveDate!=null && !dividendPeriodEffectiveDate.prune().hasData()) dividendPeriodEffectiveDate = null;
			if (dividendPeriodEndDate!=null && !dividendPeriodEndDate.prune().hasData()) dividendPeriodEndDate = null;
			if (extraordinaryDividendsParty!=null && !extraordinaryDividendsParty.prune().hasData()) extraordinaryDividendsParty = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getDividendAmountType()!=null) return true;
			if (getDividendCurrency()!=null && getDividendCurrency().hasData()) return true;
			if (getDividendEntitlement()!=null) return true;
			if (getDividendPaymentDate()!=null && getDividendPaymentDate().hasData()) return true;
			if (getDividendPayout()!=null && getDividendPayout().hasData()) return true;
			if (getDividendPeriod()!=null) return true;
			if (getDividendPeriodEffectiveDate()!=null) return true;
			if (getDividendPeriodEndDate()!=null) return true;
			if (getDividendReinvestment()!=null) return true;
			if (getExcessDividendAmount()!=null) return true;
			if (getExtraordinaryDividendsParty()!=null && getExtraordinaryDividendsParty().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("dividendAmountType"), DividendAmountTypeEnum.class, dividendAmountType, this);
			processor.processBasic(path.newSubPath("dividendEntitlement"), DividendEntitlementEnum.class, dividendEntitlement, this);
			processor.processBasic(path.newSubPath("dividendPeriod"), DividendPeriodEnum.class, dividendPeriod, this);
			processor.processBasic(path.newSubPath("dividendReinvestment"), Boolean.class, dividendReinvestment, this);
			processor.processBasic(path.newSubPath("excessDividendAmount"), DividendAmountTypeEnum.class, excessDividendAmount, this);
			
			processRosetta(path.newSubPath("dividendCurrency"), processor, DividendCurrency.class, dividendCurrency);
			processRosetta(path.newSubPath("dividendPaymentDate"), processor, DividendPaymentDate.class, dividendPaymentDate);
			processRosetta(path.newSubPath("dividendPayout"), processor, DividendPayout.class, dividendPayout);
			processRosetta(path.newSubPath("dividendPeriodEffectiveDate"), processor, BasicReferenceWithMetaDate.class, dividendPeriodEffectiveDate);
			processRosetta(path.newSubPath("dividendPeriodEndDate"), processor, BasicReferenceWithMetaDate.class, dividendPeriodEndDate);
			processRosetta(path.newSubPath("extraordinaryDividendsParty"), processor, ReferenceWithMetaParty.class, extraordinaryDividendsParty);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			DividendReturnTermsBuilder _that = (DividendReturnTermsBuilder) o;
		
			if (dividendAmountType != null ? !dividendAmountType.equals(_that.dividendAmountType) : _that.dividendAmountType != null) return false;
			if (dividendCurrency != null ? !dividendCurrency.equals(_that.dividendCurrency) : _that.dividendCurrency != null) return false;
			if (dividendEntitlement != null ? !dividendEntitlement.equals(_that.dividendEntitlement) : _that.dividendEntitlement != null) return false;
			if (dividendPaymentDate != null ? !dividendPaymentDate.equals(_that.dividendPaymentDate) : _that.dividendPaymentDate != null) return false;
			if (dividendPayout != null ? !dividendPayout.equals(_that.dividendPayout) : _that.dividendPayout != null) return false;
			if (dividendPeriod != null ? !dividendPeriod.equals(_that.dividendPeriod) : _that.dividendPeriod != null) return false;
			if (dividendPeriodEffectiveDate != null ? !dividendPeriodEffectiveDate.equals(_that.dividendPeriodEffectiveDate) : _that.dividendPeriodEffectiveDate != null) return false;
			if (dividendPeriodEndDate != null ? !dividendPeriodEndDate.equals(_that.dividendPeriodEndDate) : _that.dividendPeriodEndDate != null) return false;
			if (dividendReinvestment != null ? !dividendReinvestment.equals(_that.dividendReinvestment) : _that.dividendReinvestment != null) return false;
			if (excessDividendAmount != null ? !excessDividendAmount.equals(_that.excessDividendAmount) : _that.excessDividendAmount != null) return false;
			if (extraordinaryDividendsParty != null ? !extraordinaryDividendsParty.equals(_that.extraordinaryDividendsParty) : _that.extraordinaryDividendsParty != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (dividendAmountType != null ? dividendAmountType.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (dividendCurrency != null ? dividendCurrency.hashCode() : 0);
			_result = 31 * _result + (dividendEntitlement != null ? dividendEntitlement.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (dividendPaymentDate != null ? dividendPaymentDate.hashCode() : 0);
			_result = 31 * _result + (dividendPayout != null ? dividendPayout.hashCode() : 0);
			_result = 31 * _result + (dividendPeriod != null ? dividendPeriod.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (dividendPeriodEffectiveDate != null ? dividendPeriodEffectiveDate.hashCode() : 0);
			_result = 31 * _result + (dividendPeriodEndDate != null ? dividendPeriodEndDate.hashCode() : 0);
			_result = 31 * _result + (dividendReinvestment != null ? dividendReinvestment.hashCode() : 0);
			_result = 31 * _result + (excessDividendAmount != null ? excessDividendAmount.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (extraordinaryDividendsParty != null ? extraordinaryDividendsParty.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "DividendReturnTermsBuilder {" +
				"dividendAmountType=" + this.dividendAmountType + ", " +
				"dividendCurrency=" + this.dividendCurrency + ", " +
				"dividendEntitlement=" + this.dividendEntitlement + ", " +
				"dividendPaymentDate=" + this.dividendPaymentDate + ", " +
				"dividendPayout=" + this.dividendPayout + ", " +
				"dividendPeriod=" + this.dividendPeriod + ", " +
				"dividendPeriodEffectiveDate=" + this.dividendPeriodEffectiveDate + ", " +
				"dividendPeriodEndDate=" + this.dividendPeriodEndDate + ", " +
				"dividendReinvestment=" + this.dividendReinvestment + ", " +
				"excessDividendAmount=" + this.excessDividendAmount + ", " +
				"extraordinaryDividendsParty=" + this.extraordinaryDividendsParty +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("dividendAmountType"), DividendAmountTypeEnum.class, dividendAmountType, this);
		processor.processBasic(path.newSubPath("dividendEntitlement"), DividendEntitlementEnum.class, dividendEntitlement, this);
		processor.processBasic(path.newSubPath("dividendPeriod"), DividendPeriodEnum.class, dividendPeriod, this);
		processor.processBasic(path.newSubPath("dividendReinvestment"), Boolean.class, dividendReinvestment, this);
		processor.processBasic(path.newSubPath("excessDividendAmount"), DividendAmountTypeEnum.class, excessDividendAmount, this);
		
		processRosetta(path.newSubPath("dividendCurrency"), processor, DividendCurrency.class, dividendCurrency);
		processRosetta(path.newSubPath("dividendPaymentDate"), processor, DividendPaymentDate.class, dividendPaymentDate);
		processRosetta(path.newSubPath("dividendPayout"), processor, DividendPayout.class, dividendPayout);
		processRosetta(path.newSubPath("dividendPeriodEffectiveDate"), processor, BasicReferenceWithMetaDate.class, dividendPeriodEffectiveDate);
		processRosetta(path.newSubPath("dividendPeriodEndDate"), processor, BasicReferenceWithMetaDate.class, dividendPeriodEndDate);
		processRosetta(path.newSubPath("extraordinaryDividendsParty"), processor, ReferenceWithMetaParty.class, extraordinaryDividendsParty);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		DividendReturnTerms _that = (DividendReturnTerms) o;
	
		if (dividendAmountType != null ? !dividendAmountType.equals(_that.dividendAmountType) : _that.dividendAmountType != null) return false;
		if (dividendCurrency != null ? !dividendCurrency.equals(_that.dividendCurrency) : _that.dividendCurrency != null) return false;
		if (dividendEntitlement != null ? !dividendEntitlement.equals(_that.dividendEntitlement) : _that.dividendEntitlement != null) return false;
		if (dividendPaymentDate != null ? !dividendPaymentDate.equals(_that.dividendPaymentDate) : _that.dividendPaymentDate != null) return false;
		if (dividendPayout != null ? !dividendPayout.equals(_that.dividendPayout) : _that.dividendPayout != null) return false;
		if (dividendPeriod != null ? !dividendPeriod.equals(_that.dividendPeriod) : _that.dividendPeriod != null) return false;
		if (dividendPeriodEffectiveDate != null ? !dividendPeriodEffectiveDate.equals(_that.dividendPeriodEffectiveDate) : _that.dividendPeriodEffectiveDate != null) return false;
		if (dividendPeriodEndDate != null ? !dividendPeriodEndDate.equals(_that.dividendPeriodEndDate) : _that.dividendPeriodEndDate != null) return false;
		if (dividendReinvestment != null ? !dividendReinvestment.equals(_that.dividendReinvestment) : _that.dividendReinvestment != null) return false;
		if (excessDividendAmount != null ? !excessDividendAmount.equals(_that.excessDividendAmount) : _that.excessDividendAmount != null) return false;
		if (extraordinaryDividendsParty != null ? !extraordinaryDividendsParty.equals(_that.extraordinaryDividendsParty) : _that.extraordinaryDividendsParty != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (dividendAmountType != null ? dividendAmountType.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (dividendCurrency != null ? dividendCurrency.hashCode() : 0);
		_result = 31 * _result + (dividendEntitlement != null ? dividendEntitlement.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (dividendPaymentDate != null ? dividendPaymentDate.hashCode() : 0);
		_result = 31 * _result + (dividendPayout != null ? dividendPayout.hashCode() : 0);
		_result = 31 * _result + (dividendPeriod != null ? dividendPeriod.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (dividendPeriodEffectiveDate != null ? dividendPeriodEffectiveDate.hashCode() : 0);
		_result = 31 * _result + (dividendPeriodEndDate != null ? dividendPeriodEndDate.hashCode() : 0);
		_result = 31 * _result + (dividendReinvestment != null ? dividendReinvestment.hashCode() : 0);
		_result = 31 * _result + (excessDividendAmount != null ? excessDividendAmount.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (extraordinaryDividendsParty != null ? extraordinaryDividendsParty.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "DividendReturnTerms {" +
			"dividendAmountType=" + this.dividendAmountType + ", " +
			"dividendCurrency=" + this.dividendCurrency + ", " +
			"dividendEntitlement=" + this.dividendEntitlement + ", " +
			"dividendPaymentDate=" + this.dividendPaymentDate + ", " +
			"dividendPayout=" + this.dividendPayout + ", " +
			"dividendPeriod=" + this.dividendPeriod + ", " +
			"dividendPeriodEffectiveDate=" + this.dividendPeriodEffectiveDate + ", " +
			"dividendPeriodEndDate=" + this.dividendPeriodEndDate + ", " +
			"dividendReinvestment=" + this.dividendReinvestment + ", " +
			"excessDividendAmount=" + this.excessDividendAmount + ", " +
			"extraordinaryDividendsParty=" + this.extraordinaryDividendsParty +
		'}';
	}
}
