package org.isda.cdm;

import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.DocumentationIdentification;
import org.isda.cdm.metafields.ReferenceWithMetaLegalAgreement;
import org.isda.cdm.metafields.ReferenceWithMetaLegalAgreement.ReferenceWithMetaLegalAgreementBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.DocumentationMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;

/**
 * A class for specifying the legal agreements that govern the contract, either as a reference to such agreements when specified as part of the CDM, or through identification of some of the key terms of those documents, such as the type of document, the document identifier, the publisher, the document vintage and the agreement date.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class Documentation extends RosettaModelObject {
	private final DocumentationIdentification documentationIdentification;
	private final List<ReferenceWithMetaLegalAgreement> legalAgreement;
	private static DocumentationMeta metaData = new DocumentationMeta();
	
	Documentation(DocumentationBuilder builder) {
		this.documentationIdentification = ofNullable(builder.getDocumentationIdentification()).map(f->f.build()).orElse(null);
		this.legalAgreement = ofNullable(builder.getLegalAgreement()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
	}
	
	/**
	 * The identification of the legal document(s) that govern the contract through some of the key terms of such document(s): type of document, the document identifier, the publisher, the document vintage and the agreement date.
	 */
	@RosettaSynonym(value="documentation", source="FpML_5_10", path="trade")
	@RosettaSynonym(value="documentation", source="DTCC_11_0", path="trade")
	@RosettaSynonym(value="documentation", source="DTCC_9_0", path="trade")
	@RosettaSynonym(value="documentation", source="CME_SubmissionIRS_1_0", path="trade")
	@RosettaSynonym(value="documentation", source="CME_ClearedConfirm_1_17", path="trade")
	@RosettaSynonym(value="documentation", source="FpML_5_10")
	@RosettaSynonym(value="documentation", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="documentation", source="CME_ClearedConfirm_1_17")
	public final DocumentationIdentification getDocumentationIdentification() {
		return documentationIdentification;
	}
	
	/**
	 * The legal agreement(s) that govern the contract, when such agreement is specified as a reference part of the CDM.
	 */
	public final List<ReferenceWithMetaLegalAgreement> getLegalAgreement() {
		return legalAgreement;
	}
	
	
	@Override
	public RosettaMetaData<? extends Documentation> metaData() {
		return metaData;
	} 
	
	public DocumentationBuilder toBuilder() {
		DocumentationBuilder builder = new DocumentationBuilder();
		ofNullable(getDocumentationIdentification()).ifPresent(builder::setDocumentationIdentification);
		ofNullable(getLegalAgreement()).ifPresent(legalAgreement -> legalAgreement.forEach(builder::addLegalAgreement));
		return builder;
	}

	public static DocumentationBuilder builder() {
		return new DocumentationBuilder();
	}

	public static  class DocumentationBuilder extends RosettaModelObjectBuilder {
	
		protected DocumentationIdentification.DocumentationIdentificationBuilder documentationIdentification;
		protected List<ReferenceWithMetaLegalAgreementBuilder> legalAgreement;
	
		public DocumentationBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends Documentation> metaData() {
			return metaData;
		} 
	
		
		public DocumentationIdentification.DocumentationIdentificationBuilder getDocumentationIdentification() {
			return documentationIdentification;
		}
		
		public DocumentationIdentification.DocumentationIdentificationBuilder getOrCreateDocumentationIdentification() {
			if (documentationIdentification!=null) {
				return documentationIdentification;
			}
			else return documentationIdentification = new DocumentationIdentification.DocumentationIdentificationBuilder();
		}
		
		
		public List<ReferenceWithMetaLegalAgreementBuilder> getLegalAgreement() {
			return legalAgreement;
		}
		
		public ReferenceWithMetaLegalAgreementBuilder getOrCreateLegalAgreement(int index) {
			if (legalAgreement==null) {
				this.legalAgreement = new ArrayList<>();
			}
			return getIndex(legalAgreement, index, ()->new ReferenceWithMetaLegalAgreementBuilder());
		}
		
	
		public DocumentationBuilder setDocumentationIdentification(DocumentationIdentification documentationIdentification) {
			this.documentationIdentification = documentationIdentification.toBuilder();
			return this;
		}
		public DocumentationBuilder setDocumentationIdentificationBuilder(DocumentationIdentification.DocumentationIdentificationBuilder documentationIdentification) {
			this.documentationIdentification = documentationIdentification;
			return this;
		}
		public DocumentationBuilder addLegalAgreement(ReferenceWithMetaLegalAgreement legalAgreement) {
			if(this.legalAgreement == null){
				this.legalAgreement = new ArrayList<>();
			}
			this.legalAgreement.add(legalAgreement.toBuilder());
			return this;
		}
		public DocumentationBuilder addLegalAgreement(ReferenceWithMetaLegalAgreement legalAgreement, int _idx) {
			if(this.legalAgreement == null){
				this.legalAgreement = new ArrayList<>();
			}
			getIndex(this.legalAgreement, _idx, () -> legalAgreement.toBuilder());
			this.legalAgreement.set(_idx, legalAgreement.toBuilder());
			return this;
		}
		public DocumentationBuilder addLegalAgreement(List<ReferenceWithMetaLegalAgreement> legalAgreements) {
			if(this.legalAgreement == null){
				this.legalAgreement = new ArrayList<>();
			}
			for (ReferenceWithMetaLegalAgreement toAdd : legalAgreements) {
				this.legalAgreement.add(toAdd.toBuilder());
			}
			return this;
		}
		public DocumentationBuilder addLegalAgreementBuilder(ReferenceWithMetaLegalAgreementBuilder legalAgreement) {
			if(this.legalAgreement == null){
				this.legalAgreement = new ArrayList<>();
			}
			this.legalAgreement.add(legalAgreement);
			return this;
		}
		
		public DocumentationBuilder addLegalAgreementRef(LegalAgreement.LegalAgreementBuilder legalAgreement) {
			return addLegalAgreement(ReferenceWithMetaLegalAgreement.builder().setValueBuilder(legalAgreement).build());
		}
		
		public DocumentationBuilder addLegalAgreementRef(LegalAgreement.LegalAgreementBuilder legalAgreement, int _idx) {
			return addLegalAgreement(ReferenceWithMetaLegalAgreement.builder().setValueBuilder(legalAgreement).build(), _idx);
		}
		
		public DocumentationBuilder addLegalAgreementRef(List<LegalAgreement> legalAgreements) {
			for (LegalAgreement toAdd : legalAgreements) {
				addLegalAgreementRef(toAdd);
			}
			return this;
		}
		
		public DocumentationBuilder addLegalAgreementRef(LegalAgreement legalAgreement) {
			return addLegalAgreementRef(legalAgreement.toBuilder());
		}
		
		public DocumentationBuilder clearLegalAgreement() {
			this.legalAgreement = null;
			return this;
		}
		
		public Documentation build() {
			return new Documentation(this);
		}
	
		@Override
		public DocumentationBuilder prune() {
			if (documentationIdentification!=null && !documentationIdentification.prune().hasData()) documentationIdentification = null;
			if (legalAgreement!=null) legalAgreement = legalAgreement.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getDocumentationIdentification()!=null && getDocumentationIdentification().hasData()) return true;
			if (getLegalAgreement()!=null && getLegalAgreement().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("documentationIdentification"), processor, DocumentationIdentification.class, documentationIdentification);
			processRosetta(path.newSubPath("legalAgreement"), processor, ReferenceWithMetaLegalAgreement.class, legalAgreement);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			DocumentationBuilder _that = (DocumentationBuilder) o;
		
			if (documentationIdentification != null ? !documentationIdentification.equals(_that.documentationIdentification) : _that.documentationIdentification != null) return false;
			if (!ListEquals.listEquals(legalAgreement, _that.legalAgreement)) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (documentationIdentification != null ? documentationIdentification.hashCode() : 0);
			_result = 31 * _result + (legalAgreement != null ? legalAgreement.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "DocumentationBuilder {" +
				"documentationIdentification=" + this.documentationIdentification + ", " +
				"legalAgreement=" + this.legalAgreement +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("documentationIdentification"), processor, DocumentationIdentification.class, documentationIdentification);
		processRosetta(path.newSubPath("legalAgreement"), processor, ReferenceWithMetaLegalAgreement.class, legalAgreement);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		Documentation _that = (Documentation) o;
	
		if (documentationIdentification != null ? !documentationIdentification.equals(_that.documentationIdentification) : _that.documentationIdentification != null) return false;
		if (!ListEquals.listEquals(legalAgreement, _that.legalAgreement)) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (documentationIdentification != null ? documentationIdentification.hashCode() : 0);
		_result = 31 * _result + (legalAgreement != null ? legalAgreement.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "Documentation {" +
			"documentationIdentification=" + this.documentationIdentification + ", " +
			"legalAgreement=" + this.legalAgreement +
		'}';
	}
}
