package org.isda.cdm;

import org.isda.cdm.ExercisePeriod;
import org.isda.cdm.MandatoryEarlyTermination;
import org.isda.cdm.OptionalEarlyTermination;
import org.isda.cdm.Period;
import org.isda.cdm.metafields.MetaFields;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.EarlyTerminationProvisionMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class defining an early termination provision for a swap. This early termination is at fair value, i.e. on termination the fair value of the product must be settled between the parties.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class EarlyTerminationProvision extends RosettaModelObject implements GlobalKey {
	private final MandatoryEarlyTermination mandatoryEarlyTermination;
	private final Period mandatoryEarlyTerminationDateTenor;
	private final MetaFields meta;
	private final OptionalEarlyTermination optionalEarlyTermination;
	private final ExercisePeriod optionalEarlyTerminationParameters;
	private static EarlyTerminationProvisionMeta metaData = new EarlyTerminationProvisionMeta();
	
	EarlyTerminationProvision(EarlyTerminationProvisionBuilder builder) {
		this.mandatoryEarlyTermination = ofNullable(builder.getMandatoryEarlyTermination()).map(f->f.build()).orElse(null);
		this.mandatoryEarlyTerminationDateTenor = ofNullable(builder.getMandatoryEarlyTerminationDateTenor()).map(f->f.build()).orElse(null);
		this.meta = ofNullable(builder.getMeta()).map(f->f.build()).orElse(null);
		this.optionalEarlyTermination = ofNullable(builder.getOptionalEarlyTermination()).map(f->f.build()).orElse(null);
		this.optionalEarlyTerminationParameters = ofNullable(builder.getOptionalEarlyTerminationParameters()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * A mandatory early termination provision to terminate the swap at fair value.
	 */
	@RosettaSynonym(value="mandatoryEarlyProvision", source="FpML_5_10")
	@RosettaSynonym(value="mandatoryEarlyProvision", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="mandatoryEarlyProvision", source="DTCC_11_0")
	@RosettaSynonym(value="mandatoryEarlyProvision", source="DTCC_9_0")
	@RosettaSynonym(value="mandatoryEarlyProvision", source="CME_ClearedConfirm_1_17")
	public final MandatoryEarlyTermination getMandatoryEarlyTermination() {
		return mandatoryEarlyTermination;
	}
	
	/**
	 * Period after trade date of the mandatory early termination date.
	 */
	@RosettaSynonym(value="mandatoryEarlyTerminationDateTenor", source="FpML_5_10")
	@RosettaSynonym(value="mandatoryEarlyTerminationDateTenor", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="mandatoryEarlyTerminationDateTenor", source="DTCC_11_0")
	@RosettaSynonym(value="mandatoryEarlyTerminationDateTenor", source="DTCC_9_0")
	@RosettaSynonym(value="mandatoryEarlyTerminationDateTenor", source="CME_ClearedConfirm_1_17")
	public final Period getMandatoryEarlyTerminationDateTenor() {
		return mandatoryEarlyTerminationDateTenor;
	}
	
	public final MetaFields getMeta() {
		return meta;
	}
	
	/**
	 * An option for either or both parties to terminate the swap at fair value.
	 */
	@RosettaSynonym(value="optionalEarlyTermination", source="FpML_5_10")
	@RosettaSynonym(value="optionalEarlyTermination", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="optionalEarlyTermination", source="DTCC_11_0")
	@RosettaSynonym(value="optionalEarlyTermination", source="DTCC_9_0")
	@RosettaSynonym(value="optionalEarlyTermination", source="CME_ClearedConfirm_1_17")
	public final OptionalEarlyTermination getOptionalEarlyTermination() {
		return optionalEarlyTermination;
	}
	
	/**
	 * Definition of the first early termination date and the frequency of the termination dates subsequent to that. American exercise is defined by having a frequency of one day.
	 */
	@RosettaSynonym(value="optionalEarlyTerminationParameters", source="FpML_5_10")
	@RosettaSynonym(value="optionalEarlyTerminationParameters", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="optionalEarlyTerminationParameters", source="DTCC_11_0")
	@RosettaSynonym(value="optionalEarlyTerminationParameters", source="DTCC_9_0")
	@RosettaSynonym(value="optionalEarlyTerminationParameters", source="CME_ClearedConfirm_1_17")
	public final ExercisePeriod getOptionalEarlyTerminationParameters() {
		return optionalEarlyTerminationParameters;
	}
	
	
	@Override
	public RosettaMetaData<? extends EarlyTerminationProvision> metaData() {
		return metaData;
	} 
	
	public EarlyTerminationProvisionBuilder toBuilder() {
		EarlyTerminationProvisionBuilder builder = new EarlyTerminationProvisionBuilder();
		ofNullable(getMandatoryEarlyTermination()).ifPresent(builder::setMandatoryEarlyTermination);
		ofNullable(getMandatoryEarlyTerminationDateTenor()).ifPresent(builder::setMandatoryEarlyTerminationDateTenor);
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		ofNullable(getOptionalEarlyTermination()).ifPresent(builder::setOptionalEarlyTermination);
		ofNullable(getOptionalEarlyTerminationParameters()).ifPresent(builder::setOptionalEarlyTerminationParameters);
		return builder;
	}

	public static EarlyTerminationProvisionBuilder builder() {
		return new EarlyTerminationProvisionBuilder();
	}

	public static  class EarlyTerminationProvisionBuilder extends RosettaModelObjectBuilder implements GlobalKeyBuilder<EarlyTerminationProvisionBuilder> {
	
		protected MandatoryEarlyTermination.MandatoryEarlyTerminationBuilder mandatoryEarlyTermination;
		protected Period.PeriodBuilder mandatoryEarlyTerminationDateTenor;
		protected MetaFields.MetaFieldsBuilder meta;
		protected OptionalEarlyTermination.OptionalEarlyTerminationBuilder optionalEarlyTermination;
		protected ExercisePeriod.ExercisePeriodBuilder optionalEarlyTerminationParameters;
	
		public EarlyTerminationProvisionBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends EarlyTerminationProvision> metaData() {
			return metaData;
		} 
	
		
		public MandatoryEarlyTermination.MandatoryEarlyTerminationBuilder getMandatoryEarlyTermination() {
			return mandatoryEarlyTermination;
		}
		
		public MandatoryEarlyTermination.MandatoryEarlyTerminationBuilder getOrCreateMandatoryEarlyTermination() {
			if (mandatoryEarlyTermination!=null) {
				return mandatoryEarlyTermination;
			}
			else return mandatoryEarlyTermination = new MandatoryEarlyTermination.MandatoryEarlyTerminationBuilder();
		}
		
		
		public Period.PeriodBuilder getMandatoryEarlyTerminationDateTenor() {
			return mandatoryEarlyTerminationDateTenor;
		}
		
		public Period.PeriodBuilder getOrCreateMandatoryEarlyTerminationDateTenor() {
			if (mandatoryEarlyTerminationDateTenor!=null) {
				return mandatoryEarlyTerminationDateTenor;
			}
			else return mandatoryEarlyTerminationDateTenor = new Period.PeriodBuilder();
		}
		
		
		public MetaFields.MetaFieldsBuilder getMeta() {
			return meta;
		}
		
		public MetaFields.MetaFieldsBuilder getOrCreateMeta() {
			if (meta!=null) {
				return meta;
			}
			else return meta = new MetaFields.MetaFieldsBuilder();
		}
		
		
		public OptionalEarlyTermination.OptionalEarlyTerminationBuilder getOptionalEarlyTermination() {
			return optionalEarlyTermination;
		}
		
		public OptionalEarlyTermination.OptionalEarlyTerminationBuilder getOrCreateOptionalEarlyTermination() {
			if (optionalEarlyTermination!=null) {
				return optionalEarlyTermination;
			}
			else return optionalEarlyTermination = new OptionalEarlyTermination.OptionalEarlyTerminationBuilder();
		}
		
		
		public ExercisePeriod.ExercisePeriodBuilder getOptionalEarlyTerminationParameters() {
			return optionalEarlyTerminationParameters;
		}
		
		public ExercisePeriod.ExercisePeriodBuilder getOrCreateOptionalEarlyTerminationParameters() {
			if (optionalEarlyTerminationParameters!=null) {
				return optionalEarlyTerminationParameters;
			}
			else return optionalEarlyTerminationParameters = new ExercisePeriod.ExercisePeriodBuilder();
		}
		
	
		public EarlyTerminationProvisionBuilder setMandatoryEarlyTermination(MandatoryEarlyTermination mandatoryEarlyTermination) {
			this.mandatoryEarlyTermination = mandatoryEarlyTermination.toBuilder();
			return this;
		}
		public EarlyTerminationProvisionBuilder setMandatoryEarlyTerminationBuilder(MandatoryEarlyTermination.MandatoryEarlyTerminationBuilder mandatoryEarlyTermination) {
			this.mandatoryEarlyTermination = mandatoryEarlyTermination;
			return this;
		}
		public EarlyTerminationProvisionBuilder setMandatoryEarlyTerminationDateTenor(Period mandatoryEarlyTerminationDateTenor) {
			this.mandatoryEarlyTerminationDateTenor = mandatoryEarlyTerminationDateTenor.toBuilder();
			return this;
		}
		public EarlyTerminationProvisionBuilder setMandatoryEarlyTerminationDateTenorBuilder(Period.PeriodBuilder mandatoryEarlyTerminationDateTenor) {
			this.mandatoryEarlyTerminationDateTenor = mandatoryEarlyTerminationDateTenor;
			return this;
		}
		public EarlyTerminationProvisionBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		public EarlyTerminationProvisionBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		public EarlyTerminationProvisionBuilder setOptionalEarlyTermination(OptionalEarlyTermination optionalEarlyTermination) {
			this.optionalEarlyTermination = optionalEarlyTermination.toBuilder();
			return this;
		}
		public EarlyTerminationProvisionBuilder setOptionalEarlyTerminationBuilder(OptionalEarlyTermination.OptionalEarlyTerminationBuilder optionalEarlyTermination) {
			this.optionalEarlyTermination = optionalEarlyTermination;
			return this;
		}
		public EarlyTerminationProvisionBuilder setOptionalEarlyTerminationParameters(ExercisePeriod optionalEarlyTerminationParameters) {
			this.optionalEarlyTerminationParameters = optionalEarlyTerminationParameters.toBuilder();
			return this;
		}
		public EarlyTerminationProvisionBuilder setOptionalEarlyTerminationParametersBuilder(ExercisePeriod.ExercisePeriodBuilder optionalEarlyTerminationParameters) {
			this.optionalEarlyTerminationParameters = optionalEarlyTerminationParameters;
			return this;
		}
		
		public EarlyTerminationProvision build() {
			return new EarlyTerminationProvision(this);
		}
	
		@Override
		public EarlyTerminationProvisionBuilder prune() {
			if (mandatoryEarlyTermination!=null && !mandatoryEarlyTermination.prune().hasData()) mandatoryEarlyTermination = null;
			if (mandatoryEarlyTerminationDateTenor!=null && !mandatoryEarlyTerminationDateTenor.prune().hasData()) mandatoryEarlyTerminationDateTenor = null;
			if (meta!=null && !meta.prune().hasData()) meta = null;
			if (optionalEarlyTermination!=null && !optionalEarlyTermination.prune().hasData()) optionalEarlyTermination = null;
			if (optionalEarlyTerminationParameters!=null && !optionalEarlyTerminationParameters.prune().hasData()) optionalEarlyTerminationParameters = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getMandatoryEarlyTermination()!=null && getMandatoryEarlyTermination().hasData()) return true;
			if (getMandatoryEarlyTerminationDateTenor()!=null && getMandatoryEarlyTerminationDateTenor().hasData()) return true;
			if (getMeta()!=null && getMeta().hasData()) return true;
			if (getOptionalEarlyTermination()!=null && getOptionalEarlyTermination().hasData()) return true;
			if (getOptionalEarlyTerminationParameters()!=null && getOptionalEarlyTerminationParameters().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("mandatoryEarlyTermination"), processor, MandatoryEarlyTermination.class, mandatoryEarlyTermination);
			processRosetta(path.newSubPath("mandatoryEarlyTerminationDateTenor"), processor, Period.class, mandatoryEarlyTerminationDateTenor);
			processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
			processRosetta(path.newSubPath("optionalEarlyTermination"), processor, OptionalEarlyTermination.class, optionalEarlyTermination);
			processRosetta(path.newSubPath("optionalEarlyTerminationParameters"), processor, ExercisePeriod.class, optionalEarlyTerminationParameters);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			EarlyTerminationProvisionBuilder _that = (EarlyTerminationProvisionBuilder) o;
		
			if (mandatoryEarlyTermination != null ? !mandatoryEarlyTermination.equals(_that.mandatoryEarlyTermination) : _that.mandatoryEarlyTermination != null) return false;
			if (mandatoryEarlyTerminationDateTenor != null ? !mandatoryEarlyTerminationDateTenor.equals(_that.mandatoryEarlyTerminationDateTenor) : _that.mandatoryEarlyTerminationDateTenor != null) return false;
			if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
			if (optionalEarlyTermination != null ? !optionalEarlyTermination.equals(_that.optionalEarlyTermination) : _that.optionalEarlyTermination != null) return false;
			if (optionalEarlyTerminationParameters != null ? !optionalEarlyTerminationParameters.equals(_that.optionalEarlyTerminationParameters) : _that.optionalEarlyTerminationParameters != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (mandatoryEarlyTermination != null ? mandatoryEarlyTermination.hashCode() : 0);
			_result = 31 * _result + (mandatoryEarlyTerminationDateTenor != null ? mandatoryEarlyTerminationDateTenor.hashCode() : 0);
			_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
			_result = 31 * _result + (optionalEarlyTermination != null ? optionalEarlyTermination.hashCode() : 0);
			_result = 31 * _result + (optionalEarlyTerminationParameters != null ? optionalEarlyTerminationParameters.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "EarlyTerminationProvisionBuilder {" +
				"mandatoryEarlyTermination=" + this.mandatoryEarlyTermination + ", " +
				"mandatoryEarlyTerminationDateTenor=" + this.mandatoryEarlyTerminationDateTenor + ", " +
				"meta=" + this.meta + ", " +
				"optionalEarlyTermination=" + this.optionalEarlyTermination + ", " +
				"optionalEarlyTerminationParameters=" + this.optionalEarlyTerminationParameters +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("mandatoryEarlyTermination"), processor, MandatoryEarlyTermination.class, mandatoryEarlyTermination);
		processRosetta(path.newSubPath("mandatoryEarlyTerminationDateTenor"), processor, Period.class, mandatoryEarlyTerminationDateTenor);
		processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
		processRosetta(path.newSubPath("optionalEarlyTermination"), processor, OptionalEarlyTermination.class, optionalEarlyTermination);
		processRosetta(path.newSubPath("optionalEarlyTerminationParameters"), processor, ExercisePeriod.class, optionalEarlyTerminationParameters);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		EarlyTerminationProvision _that = (EarlyTerminationProvision) o;
	
		if (mandatoryEarlyTermination != null ? !mandatoryEarlyTermination.equals(_that.mandatoryEarlyTermination) : _that.mandatoryEarlyTermination != null) return false;
		if (mandatoryEarlyTerminationDateTenor != null ? !mandatoryEarlyTerminationDateTenor.equals(_that.mandatoryEarlyTerminationDateTenor) : _that.mandatoryEarlyTerminationDateTenor != null) return false;
		if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
		if (optionalEarlyTermination != null ? !optionalEarlyTermination.equals(_that.optionalEarlyTermination) : _that.optionalEarlyTermination != null) return false;
		if (optionalEarlyTerminationParameters != null ? !optionalEarlyTerminationParameters.equals(_that.optionalEarlyTerminationParameters) : _that.optionalEarlyTerminationParameters != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (mandatoryEarlyTermination != null ? mandatoryEarlyTermination.hashCode() : 0);
		_result = 31 * _result + (mandatoryEarlyTerminationDateTenor != null ? mandatoryEarlyTerminationDateTenor.hashCode() : 0);
		_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
		_result = 31 * _result + (optionalEarlyTermination != null ? optionalEarlyTermination.hashCode() : 0);
		_result = 31 * _result + (optionalEarlyTerminationParameters != null ? optionalEarlyTerminationParameters.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "EarlyTerminationProvision {" +
			"mandatoryEarlyTermination=" + this.mandatoryEarlyTermination + ", " +
			"mandatoryEarlyTerminationDateTenor=" + this.mandatoryEarlyTerminationDateTenor + ", " +
			"meta=" + this.meta + ", " +
			"optionalEarlyTermination=" + this.optionalEarlyTermination + ", " +
			"optionalEarlyTerminationParameters=" + this.optionalEarlyTerminationParameters +
		'}';
	}
}
