package org.isda.cdm;

import java.lang.String;
import org.isda.cdm.AdjustableOrRelativeDate;
import org.isda.cdm.BusinessDayAdjustments;
import org.isda.cdm.CancelableProvision;
import org.isda.cdm.EarlyTerminationProvision;
import org.isda.cdm.ExtendibleProvision;
import org.isda.cdm.ExtraordinaryEvents;
import org.isda.cdm.Payout;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.EconomicTermsMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.RosettaKeyValue;
import com.rosetta.model.lib.RosettaKeyValueBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * This class represents the full set of price-forming features associated with a contractual product: the payout component, the notional/quantity, the effective and termination date and the date adjustment provisions when applying uniformily across the payout components. This class also includes the legal provisions which have valuation implications: cancelable provision, extendible provision, early termination provision and extraordinary events specification. A rosettaKeyValue is associated to the contractual product economic terms for the purpose of supporting hash-based reconciliations thanks to the fact that its computation doesn&#39;t include meta data, such as identifiers, references, schemes and other rosettaKey artefacts. The rosettaKeyValue default implementation is available as part of the generated code as org.isda.cdm.rosettakey.RosettaKeyValueHashFunction.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class EconomicTerms extends RosettaModelObject implements RosettaKeyValue {
	private final CancelableProvision cancelableProvision;
	private final BusinessDayAdjustments dateAdjustments;
	private final EarlyTerminationProvision earlyTerminationProvision;
	private final AdjustableOrRelativeDate effectiveDate;
	private final ExtendibleProvision extendibleProvision;
	private final ExtraordinaryEvents extraordinaryEvents;
	private final Payout payout;
	private final String rosettaKeyValue;
	private final AdjustableOrRelativeDate terminationDate;
	private static EconomicTermsMeta metaData = new EconomicTermsMeta();
	
	EconomicTerms(EconomicTermsBuilder builder) {
		this.cancelableProvision = ofNullable(builder.getCancelableProvision()).map(f->f.build()).orElse(null);
		this.dateAdjustments = ofNullable(builder.getDateAdjustments()).map(f->f.build()).orElse(null);
		this.earlyTerminationProvision = ofNullable(builder.getEarlyTerminationProvision()).map(f->f.build()).orElse(null);
		this.effectiveDate = ofNullable(builder.getEffectiveDate()).map(f->f.build()).orElse(null);
		this.extendibleProvision = ofNullable(builder.getExtendibleProvision()).map(f->f.build()).orElse(null);
		this.extraordinaryEvents = ofNullable(builder.getExtraordinaryEvents()).map(f->f.build()).orElse(null);
		this.payout = ofNullable(builder.getPayout()).map(f->f.build()).orElse(null);
		this.rosettaKeyValue = builder.getRosettaKeyValue();
		this.terminationDate = ofNullable(builder.getTerminationDate()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * A provision that allows the specification of an embedded option within a swap giving the buyer of the option the right to terminate the swap, in whole or in part, on the early termination date.
	 */
	@RosettaSynonym(value="cancelableProvision", source="FpML_5_10", path="trade.swap")
	@RosettaSynonym(value="cancelableProvision", source="CME_SubmissionIRS_1_0", path="trade.swap")
	@RosettaSynonym(value="cancelableProvision", source="DTCC_11_0", path="trade.swap")
	@RosettaSynonym(value="cancelableProvision", source="DTCC_9_0", path="trade.swap")
	@RosettaSynonym(value="cancelableProvision", source="CME_ClearedConfirm_1_17", path="trade.swap")
	public final CancelableProvision getCancelableProvision() {
		return cancelableProvision;
	}
	
	/**
	 * The business day adjustment convention when it applies across all the payout components. This specification of the business day convention and financial business centers is used for adjusting any calculation period date if it would otherwise fall on a day that is not a business day in the specified business center.
	 */
	@RosettaSynonym(value="dateAdjustments", source="FpML_5_10", path="trade.creditDefaultSwap.generalTerms")
	@RosettaSynonym(value="dateAdjustments", source="FpML_5_10", path="generalTerms")
	@RosettaSynonym(value="dateAdjustments", source="CME_SubmissionIRS_1_0", path="trade.creditDefaultSwap.generalTerms")
	@RosettaSynonym(value="dateAdjustments", source="CME_SubmissionIRS_1_0", path="generalTerms")
	@RosettaSynonym(value="dateAdjustments", source="DTCC_11_0", path="trade.creditDefaultSwap.generalTerms")
	@RosettaSynonym(value="dateAdjustments", source="DTCC_11_0", path="generalTerms")
	@RosettaSynonym(value="dateAdjustments", source="DTCC_9_0", path="trade.creditDefaultSwap.generalTerms")
	@RosettaSynonym(value="dateAdjustments", source="DTCC_9_0", path="generalTerms")
	@RosettaSynonym(value="dateAdjustments", source="CME_ClearedConfirm_1_17", path="trade.creditDefaultSwap.generalTerms")
	@RosettaSynonym(value="dateAdjustments", source="CME_ClearedConfirm_1_17", path="generalTerms")
	public final BusinessDayAdjustments getDateAdjustments() {
		return dateAdjustments;
	}
	
	/**
	 * Parameters specifying provisions relating to the optional and mandatory early termination of a swap transaction.
	 */
	@RosettaSynonym(value="earlyTerminationProvision", source="FpML_5_10", path="trade.swap")
	@RosettaSynonym(value="earlyTerminationProvision", source="FpML_5_10", path="trade.capFloor")
	@RosettaSynonym(value="earlyTerminationProvision", source="CME_SubmissionIRS_1_0", path="trade.swap")
	@RosettaSynonym(value="earlyTerminationProvision", source="CME_SubmissionIRS_1_0", path="trade.capFloor")
	@RosettaSynonym(value="earlyTerminationProvision", source="DTCC_11_0", path="trade.swap")
	@RosettaSynonym(value="earlyTerminationProvision", source="DTCC_11_0", path="trade.capFloor")
	@RosettaSynonym(value="earlyTerminationProvision", source="DTCC_9_0", path="trade.swap")
	@RosettaSynonym(value="earlyTerminationProvision", source="DTCC_9_0", path="trade.capFloor")
	@RosettaSynonym(value="earlyTerminationProvision", source="CME_ClearedConfirm_1_17", path="trade.swap")
	@RosettaSynonym(value="earlyTerminationProvision", source="CME_ClearedConfirm_1_17", path="trade.capFloor")
	public final EarlyTerminationProvision getEarlyTerminationProvision() {
		return earlyTerminationProvision;
	}
	
	/**
	 * The first day of the terms of the trade. This day may be subject to adjustment in accordance with a business day convention.
	 */
	@RosettaSynonym(value="effectiveDate", source="FpML_5_10", path="trade.creditDefaultSwap.generalTerms")
	@RosettaSynonym(value="effectiveDate", source="FpML_5_10", path="generalTerms")
	@RosettaSynonym(value="effectiveDate", source="CME_SubmissionIRS_1_0", path="trade.creditDefaultSwap.generalTerms")
	@RosettaSynonym(value="effectiveDate", source="CME_SubmissionIRS_1_0", path="generalTerms")
	@RosettaSynonym(value="effectiveDate", source="DTCC_11_0", path="trade.creditDefaultSwap.generalTerms")
	@RosettaSynonym(value="effectiveDate", source="DTCC_11_0", path="generalTerms")
	@RosettaSynonym(value="effectiveDate", source="DTCC_9_0", path="trade.creditDefaultSwap.generalTerms")
	@RosettaSynonym(value="effectiveDate", source="DTCC_9_0", path="generalTerms")
	@RosettaSynonym(value="effectiveDate", source="CME_ClearedConfirm_1_17", path="trade.creditDefaultSwap.generalTerms")
	@RosettaSynonym(value="effectiveDate", source="CME_ClearedConfirm_1_17", path="generalTerms")
	@RosettaSynonym(value="effectiveDate", source="Rosetta_Workbench", path="creditDefaultSwap.generalTerms")
	@RosettaSynonym(value="effectiveDate", source="Rosetta_Workbench", path="creditDefaultSwapOption.creditDefaultSwap.generalTerms")
	public final AdjustableOrRelativeDate getEffectiveDate() {
		return effectiveDate;
	}
	
	/**
	 * A provision that allows the specification of an embedded option with a swap giving the buyer of the option the right to extend the swap, in whole or in part, to the extended termination date.
	 */
	@RosettaSynonym(value="extendibleProvision", source="FpML_5_10", path="trade.swap")
	@RosettaSynonym(value="extendibleProvision", source="CME_SubmissionIRS_1_0", path="trade.swap")
	@RosettaSynonym(value="extendibleProvision", source="DTCC_11_0", path="trade.swap")
	@RosettaSynonym(value="extendibleProvision", source="DTCC_9_0", path="trade.swap")
	@RosettaSynonym(value="extendibleProvision", source="CME_ClearedConfirm_1_17", path="trade.swap")
	public final ExtendibleProvision getExtendibleProvision() {
		return extendibleProvision;
	}
	
	/**
	 * 2018 ISDA CDM Equity Confirmation for Security Equity Swap: Extraordinary Events.
	 */
	@RosettaSynonym(value="extraordinaryEvents", source="FpML_5_10", path="trade.returnSwap")
	public final ExtraordinaryEvents getExtraordinaryEvents() {
		return extraordinaryEvents;
	}
	
	/**
	 * The payout specifies the future cashflow computation methodology which characterizes a financial product.
	 */
	public final Payout getPayout() {
		return payout;
	}
	
	public final String getRosettaKeyValue() {
		return rosettaKeyValue;
	}
	
	/**
	 * The last day of the terms of the trade. This date may be subject to adjustments in accordance with the business day convention. It can also be specified in relation to another scheduled date (e.g. the last payment date).
	 */
	@RosettaSynonym(value="scheduledTerminationDate", source="FpML_5_10", path="trade.creditDefaultSwap.generalTerms")
	@RosettaSynonym(value="scheduledTerminationDate", source="FpML_5_10", path="generalTerms")
	@RosettaSynonym(value="scheduledTerminationDate", source="CME_SubmissionIRS_1_0", path="trade.creditDefaultSwap.generalTerms")
	@RosettaSynonym(value="scheduledTerminationDate", source="CME_SubmissionIRS_1_0", path="generalTerms")
	@RosettaSynonym(value="scheduledTerminationDate", source="DTCC_11_0", path="trade.creditDefaultSwap.generalTerms")
	@RosettaSynonym(value="scheduledTerminationDate", source="DTCC_11_0", path="generalTerms")
	@RosettaSynonym(value="scheduledTerminationDate", source="DTCC_9_0", path="trade.creditDefaultSwap.generalTerms")
	@RosettaSynonym(value="scheduledTerminationDate", source="DTCC_9_0", path="generalTerms")
	@RosettaSynonym(value="scheduledTerminationDate", source="CME_ClearedConfirm_1_17", path="trade.creditDefaultSwap.generalTerms")
	@RosettaSynonym(value="scheduledTerminationDate", source="CME_ClearedConfirm_1_17", path="generalTerms")
	@RosettaSynonym(value="scheduledTerminationDate", source="Rosetta_Workbench", path="creditDefaultSwap.generalTerms")
	@RosettaSynonym(value="scheduledTerminationDate", source="Rosetta_Workbench", path="creditDefaultSwapOption.creditDefaultSwap.generalTerms")
	public final AdjustableOrRelativeDate getTerminationDate() {
		return terminationDate;
	}
	
	
	@Override
	public RosettaMetaData<? extends EconomicTerms> metaData() {
		return metaData;
	} 
	
	public EconomicTermsBuilder toBuilder() {
		EconomicTermsBuilder builder = new EconomicTermsBuilder();
		ofNullable(getCancelableProvision()).ifPresent(builder::setCancelableProvision);
		ofNullable(getDateAdjustments()).ifPresent(builder::setDateAdjustments);
		ofNullable(getEarlyTerminationProvision()).ifPresent(builder::setEarlyTerminationProvision);
		ofNullable(getEffectiveDate()).ifPresent(builder::setEffectiveDate);
		ofNullable(getExtendibleProvision()).ifPresent(builder::setExtendibleProvision);
		ofNullable(getExtraordinaryEvents()).ifPresent(builder::setExtraordinaryEvents);
		ofNullable(getPayout()).ifPresent(builder::setPayout);
		ofNullable(getRosettaKeyValue()).ifPresent(builder::setRosettaKeyValue);
		ofNullable(getTerminationDate()).ifPresent(builder::setTerminationDate);
		return builder;
	}

	public static EconomicTermsBuilder builder() {
		return new EconomicTermsBuilder();
	}

	public static  class EconomicTermsBuilder extends RosettaModelObjectBuilder implements RosettaKeyValueBuilder<EconomicTermsBuilder> {
	
		protected CancelableProvision.CancelableProvisionBuilder cancelableProvision;
		protected BusinessDayAdjustments.BusinessDayAdjustmentsBuilder dateAdjustments;
		protected EarlyTerminationProvision.EarlyTerminationProvisionBuilder earlyTerminationProvision;
		protected AdjustableOrRelativeDate.AdjustableOrRelativeDateBuilder effectiveDate;
		protected ExtendibleProvision.ExtendibleProvisionBuilder extendibleProvision;
		protected ExtraordinaryEvents.ExtraordinaryEventsBuilder extraordinaryEvents;
		protected Payout.PayoutBuilder payout;
		protected String rosettaKeyValue;
		protected AdjustableOrRelativeDate.AdjustableOrRelativeDateBuilder terminationDate;
	
		public EconomicTermsBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends EconomicTerms> metaData() {
			return metaData;
		} 
	
		
		public CancelableProvision.CancelableProvisionBuilder getCancelableProvision() {
			return cancelableProvision;
		}
		
		public CancelableProvision.CancelableProvisionBuilder getOrCreateCancelableProvision() {
			if (cancelableProvision!=null) {
				return cancelableProvision;
			}
			else return cancelableProvision = new CancelableProvision.CancelableProvisionBuilder();
		}
		
		
		public BusinessDayAdjustments.BusinessDayAdjustmentsBuilder getDateAdjustments() {
			return dateAdjustments;
		}
		
		public BusinessDayAdjustments.BusinessDayAdjustmentsBuilder getOrCreateDateAdjustments() {
			if (dateAdjustments!=null) {
				return dateAdjustments;
			}
			else return dateAdjustments = new BusinessDayAdjustments.BusinessDayAdjustmentsBuilder();
		}
		
		
		public EarlyTerminationProvision.EarlyTerminationProvisionBuilder getEarlyTerminationProvision() {
			return earlyTerminationProvision;
		}
		
		public EarlyTerminationProvision.EarlyTerminationProvisionBuilder getOrCreateEarlyTerminationProvision() {
			if (earlyTerminationProvision!=null) {
				return earlyTerminationProvision;
			}
			else return earlyTerminationProvision = new EarlyTerminationProvision.EarlyTerminationProvisionBuilder();
		}
		
		
		public AdjustableOrRelativeDate.AdjustableOrRelativeDateBuilder getEffectiveDate() {
			return effectiveDate;
		}
		
		public AdjustableOrRelativeDate.AdjustableOrRelativeDateBuilder getOrCreateEffectiveDate() {
			if (effectiveDate!=null) {
				return effectiveDate;
			}
			else return effectiveDate = new AdjustableOrRelativeDate.AdjustableOrRelativeDateBuilder();
		}
		
		
		public ExtendibleProvision.ExtendibleProvisionBuilder getExtendibleProvision() {
			return extendibleProvision;
		}
		
		public ExtendibleProvision.ExtendibleProvisionBuilder getOrCreateExtendibleProvision() {
			if (extendibleProvision!=null) {
				return extendibleProvision;
			}
			else return extendibleProvision = new ExtendibleProvision.ExtendibleProvisionBuilder();
		}
		
		
		public ExtraordinaryEvents.ExtraordinaryEventsBuilder getExtraordinaryEvents() {
			return extraordinaryEvents;
		}
		
		public ExtraordinaryEvents.ExtraordinaryEventsBuilder getOrCreateExtraordinaryEvents() {
			if (extraordinaryEvents!=null) {
				return extraordinaryEvents;
			}
			else return extraordinaryEvents = new ExtraordinaryEvents.ExtraordinaryEventsBuilder();
		}
		
		
		public Payout.PayoutBuilder getPayout() {
			return payout;
		}
		
		public Payout.PayoutBuilder getOrCreatePayout() {
			if (payout!=null) {
				return payout;
			}
			else return payout = new Payout.PayoutBuilder();
		}
		
		
		public String getRosettaKeyValue() {
			return rosettaKeyValue;
		}
		
		
		public AdjustableOrRelativeDate.AdjustableOrRelativeDateBuilder getTerminationDate() {
			return terminationDate;
		}
		
		public AdjustableOrRelativeDate.AdjustableOrRelativeDateBuilder getOrCreateTerminationDate() {
			if (terminationDate!=null) {
				return terminationDate;
			}
			else return terminationDate = new AdjustableOrRelativeDate.AdjustableOrRelativeDateBuilder();
		}
		
	
		public EconomicTermsBuilder setCancelableProvision(CancelableProvision cancelableProvision) {
			this.cancelableProvision = cancelableProvision.toBuilder();
			return this;
		}
		public EconomicTermsBuilder setCancelableProvisionBuilder(CancelableProvision.CancelableProvisionBuilder cancelableProvision) {
			this.cancelableProvision = cancelableProvision;
			return this;
		}
		public EconomicTermsBuilder setDateAdjustments(BusinessDayAdjustments dateAdjustments) {
			this.dateAdjustments = dateAdjustments.toBuilder();
			return this;
		}
		public EconomicTermsBuilder setDateAdjustmentsBuilder(BusinessDayAdjustments.BusinessDayAdjustmentsBuilder dateAdjustments) {
			this.dateAdjustments = dateAdjustments;
			return this;
		}
		public EconomicTermsBuilder setEarlyTerminationProvision(EarlyTerminationProvision earlyTerminationProvision) {
			this.earlyTerminationProvision = earlyTerminationProvision.toBuilder();
			return this;
		}
		public EconomicTermsBuilder setEarlyTerminationProvisionBuilder(EarlyTerminationProvision.EarlyTerminationProvisionBuilder earlyTerminationProvision) {
			this.earlyTerminationProvision = earlyTerminationProvision;
			return this;
		}
		public EconomicTermsBuilder setEffectiveDate(AdjustableOrRelativeDate effectiveDate) {
			this.effectiveDate = effectiveDate.toBuilder();
			return this;
		}
		public EconomicTermsBuilder setEffectiveDateBuilder(AdjustableOrRelativeDate.AdjustableOrRelativeDateBuilder effectiveDate) {
			this.effectiveDate = effectiveDate;
			return this;
		}
		public EconomicTermsBuilder setExtendibleProvision(ExtendibleProvision extendibleProvision) {
			this.extendibleProvision = extendibleProvision.toBuilder();
			return this;
		}
		public EconomicTermsBuilder setExtendibleProvisionBuilder(ExtendibleProvision.ExtendibleProvisionBuilder extendibleProvision) {
			this.extendibleProvision = extendibleProvision;
			return this;
		}
		public EconomicTermsBuilder setExtraordinaryEvents(ExtraordinaryEvents extraordinaryEvents) {
			this.extraordinaryEvents = extraordinaryEvents.toBuilder();
			return this;
		}
		public EconomicTermsBuilder setExtraordinaryEventsBuilder(ExtraordinaryEvents.ExtraordinaryEventsBuilder extraordinaryEvents) {
			this.extraordinaryEvents = extraordinaryEvents;
			return this;
		}
		public EconomicTermsBuilder setPayout(Payout payout) {
			this.payout = payout.toBuilder();
			return this;
		}
		public EconomicTermsBuilder setPayoutBuilder(Payout.PayoutBuilder payout) {
			this.payout = payout;
			return this;
		}
		public EconomicTermsBuilder setRosettaKeyValue(String rosettaKeyValue) {
			this.rosettaKeyValue = rosettaKeyValue;
			return this;
		}
		public EconomicTermsBuilder setTerminationDate(AdjustableOrRelativeDate terminationDate) {
			this.terminationDate = terminationDate.toBuilder();
			return this;
		}
		public EconomicTermsBuilder setTerminationDateBuilder(AdjustableOrRelativeDate.AdjustableOrRelativeDateBuilder terminationDate) {
			this.terminationDate = terminationDate;
			return this;
		}
		
		public EconomicTerms build() {
			return new EconomicTerms(this);
		}
	
		@Override
		public EconomicTermsBuilder prune() {
			if (cancelableProvision!=null && !cancelableProvision.prune().hasData()) cancelableProvision = null;
			if (dateAdjustments!=null && !dateAdjustments.prune().hasData()) dateAdjustments = null;
			if (earlyTerminationProvision!=null && !earlyTerminationProvision.prune().hasData()) earlyTerminationProvision = null;
			if (effectiveDate!=null && !effectiveDate.prune().hasData()) effectiveDate = null;
			if (extendibleProvision!=null && !extendibleProvision.prune().hasData()) extendibleProvision = null;
			if (extraordinaryEvents!=null && !extraordinaryEvents.prune().hasData()) extraordinaryEvents = null;
			if (payout!=null && !payout.prune().hasData()) payout = null;
			if (terminationDate!=null && !terminationDate.prune().hasData()) terminationDate = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getCancelableProvision()!=null && getCancelableProvision().hasData()) return true;
			if (getDateAdjustments()!=null && getDateAdjustments().hasData()) return true;
			if (getEarlyTerminationProvision()!=null && getEarlyTerminationProvision().hasData()) return true;
			if (getEffectiveDate()!=null && getEffectiveDate().hasData()) return true;
			if (getExtendibleProvision()!=null && getExtendibleProvision().hasData()) return true;
			if (getExtraordinaryEvents()!=null && getExtraordinaryEvents().hasData()) return true;
			if (getPayout()!=null && getPayout().hasData()) return true;
			if (getRosettaKeyValue()!=null) return true;
			if (getTerminationDate()!=null && getTerminationDate().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("rosettaKeyValue"), String.class, rosettaKeyValue, this);
			
			processRosetta(path.newSubPath("cancelableProvision"), processor, CancelableProvision.class, cancelableProvision);
			processRosetta(path.newSubPath("dateAdjustments"), processor, BusinessDayAdjustments.class, dateAdjustments);
			processRosetta(path.newSubPath("earlyTerminationProvision"), processor, EarlyTerminationProvision.class, earlyTerminationProvision);
			processRosetta(path.newSubPath("effectiveDate"), processor, AdjustableOrRelativeDate.class, effectiveDate);
			processRosetta(path.newSubPath("extendibleProvision"), processor, ExtendibleProvision.class, extendibleProvision);
			processRosetta(path.newSubPath("extraordinaryEvents"), processor, ExtraordinaryEvents.class, extraordinaryEvents);
			processRosetta(path.newSubPath("payout"), processor, Payout.class, payout);
			processRosetta(path.newSubPath("terminationDate"), processor, AdjustableOrRelativeDate.class, terminationDate);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			EconomicTermsBuilder _that = (EconomicTermsBuilder) o;
		
			if (cancelableProvision != null ? !cancelableProvision.equals(_that.cancelableProvision) : _that.cancelableProvision != null) return false;
			if (dateAdjustments != null ? !dateAdjustments.equals(_that.dateAdjustments) : _that.dateAdjustments != null) return false;
			if (earlyTerminationProvision != null ? !earlyTerminationProvision.equals(_that.earlyTerminationProvision) : _that.earlyTerminationProvision != null) return false;
			if (effectiveDate != null ? !effectiveDate.equals(_that.effectiveDate) : _that.effectiveDate != null) return false;
			if (extendibleProvision != null ? !extendibleProvision.equals(_that.extendibleProvision) : _that.extendibleProvision != null) return false;
			if (extraordinaryEvents != null ? !extraordinaryEvents.equals(_that.extraordinaryEvents) : _that.extraordinaryEvents != null) return false;
			if (payout != null ? !payout.equals(_that.payout) : _that.payout != null) return false;
			if (rosettaKeyValue != null ? !rosettaKeyValue.equals(_that.rosettaKeyValue) : _that.rosettaKeyValue != null) return false;
			if (terminationDate != null ? !terminationDate.equals(_that.terminationDate) : _that.terminationDate != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (cancelableProvision != null ? cancelableProvision.hashCode() : 0);
			_result = 31 * _result + (dateAdjustments != null ? dateAdjustments.hashCode() : 0);
			_result = 31 * _result + (earlyTerminationProvision != null ? earlyTerminationProvision.hashCode() : 0);
			_result = 31 * _result + (effectiveDate != null ? effectiveDate.hashCode() : 0);
			_result = 31 * _result + (extendibleProvision != null ? extendibleProvision.hashCode() : 0);
			_result = 31 * _result + (extraordinaryEvents != null ? extraordinaryEvents.hashCode() : 0);
			_result = 31 * _result + (payout != null ? payout.hashCode() : 0);
			_result = 31 * _result + (rosettaKeyValue != null ? rosettaKeyValue.hashCode() : 0);
			_result = 31 * _result + (terminationDate != null ? terminationDate.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "EconomicTermsBuilder {" +
				"cancelableProvision=" + this.cancelableProvision + ", " +
				"dateAdjustments=" + this.dateAdjustments + ", " +
				"earlyTerminationProvision=" + this.earlyTerminationProvision + ", " +
				"effectiveDate=" + this.effectiveDate + ", " +
				"extendibleProvision=" + this.extendibleProvision + ", " +
				"extraordinaryEvents=" + this.extraordinaryEvents + ", " +
				"payout=" + this.payout + ", " +
				"rosettaKeyValue=" + this.rosettaKeyValue + ", " +
				"terminationDate=" + this.terminationDate +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("rosettaKeyValue"), String.class, rosettaKeyValue, this);
		
		processRosetta(path.newSubPath("cancelableProvision"), processor, CancelableProvision.class, cancelableProvision);
		processRosetta(path.newSubPath("dateAdjustments"), processor, BusinessDayAdjustments.class, dateAdjustments);
		processRosetta(path.newSubPath("earlyTerminationProvision"), processor, EarlyTerminationProvision.class, earlyTerminationProvision);
		processRosetta(path.newSubPath("effectiveDate"), processor, AdjustableOrRelativeDate.class, effectiveDate);
		processRosetta(path.newSubPath("extendibleProvision"), processor, ExtendibleProvision.class, extendibleProvision);
		processRosetta(path.newSubPath("extraordinaryEvents"), processor, ExtraordinaryEvents.class, extraordinaryEvents);
		processRosetta(path.newSubPath("payout"), processor, Payout.class, payout);
		processRosetta(path.newSubPath("terminationDate"), processor, AdjustableOrRelativeDate.class, terminationDate);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		EconomicTerms _that = (EconomicTerms) o;
	
		if (cancelableProvision != null ? !cancelableProvision.equals(_that.cancelableProvision) : _that.cancelableProvision != null) return false;
		if (dateAdjustments != null ? !dateAdjustments.equals(_that.dateAdjustments) : _that.dateAdjustments != null) return false;
		if (earlyTerminationProvision != null ? !earlyTerminationProvision.equals(_that.earlyTerminationProvision) : _that.earlyTerminationProvision != null) return false;
		if (effectiveDate != null ? !effectiveDate.equals(_that.effectiveDate) : _that.effectiveDate != null) return false;
		if (extendibleProvision != null ? !extendibleProvision.equals(_that.extendibleProvision) : _that.extendibleProvision != null) return false;
		if (extraordinaryEvents != null ? !extraordinaryEvents.equals(_that.extraordinaryEvents) : _that.extraordinaryEvents != null) return false;
		if (payout != null ? !payout.equals(_that.payout) : _that.payout != null) return false;
		if (rosettaKeyValue != null ? !rosettaKeyValue.equals(_that.rosettaKeyValue) : _that.rosettaKeyValue != null) return false;
		if (terminationDate != null ? !terminationDate.equals(_that.terminationDate) : _that.terminationDate != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (cancelableProvision != null ? cancelableProvision.hashCode() : 0);
		_result = 31 * _result + (dateAdjustments != null ? dateAdjustments.hashCode() : 0);
		_result = 31 * _result + (earlyTerminationProvision != null ? earlyTerminationProvision.hashCode() : 0);
		_result = 31 * _result + (effectiveDate != null ? effectiveDate.hashCode() : 0);
		_result = 31 * _result + (extendibleProvision != null ? extendibleProvision.hashCode() : 0);
		_result = 31 * _result + (extraordinaryEvents != null ? extraordinaryEvents.hashCode() : 0);
		_result = 31 * _result + (payout != null ? payout.hashCode() : 0);
		_result = 31 * _result + (rosettaKeyValue != null ? rosettaKeyValue.hashCode() : 0);
		_result = 31 * _result + (terminationDate != null ? terminationDate.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "EconomicTerms {" +
			"cancelableProvision=" + this.cancelableProvision + ", " +
			"dateAdjustments=" + this.dateAdjustments + ", " +
			"earlyTerminationProvision=" + this.earlyTerminationProvision + ", " +
			"effectiveDate=" + this.effectiveDate + ", " +
			"extendibleProvision=" + this.extendibleProvision + ", " +
			"extraordinaryEvents=" + this.extraordinaryEvents + ", " +
			"payout=" + this.payout + ", " +
			"rosettaKeyValue=" + this.rosettaKeyValue + ", " +
			"terminationDate=" + this.terminationDate +
		'}';
	}
}
