package org.isda.cdm;

import java.lang.Integer;
import java.lang.String;
import org.isda.cdm.Money;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.ElectiveAmountElectionMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class to specify the party elective amounts which can be used for the purpose of specifying elections such as the ISDA CSA Threshold and Minimum Transfer Amount.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class ElectiveAmountElection extends RosettaModelObject {
	private final Money amount;
	private final String customElection;
	private final Integer noAmount;
	private final String party;
	private static ElectiveAmountElectionMeta metaData = new ElectiveAmountElectionMeta();
	
	ElectiveAmountElection(ElectiveAmountElectionBuilder builder) {
		this.amount = ofNullable(builder.getAmount()).map(f->f.build()).orElse(null);
		this.customElection = builder.getCustomElection();
		this.noAmount = builder.getNoAmount();
		this.party = builder.getParty();
	}
	
	/**
	 * The elective amount when expressed as a currency amount. The associated PartyElectiveAmount_amount data rule enforces that the currency amount is actually greater than 0.
	 */
	public final Money getAmount() {
		return amount;
	}
	
	/**
	 * The elective amount when expressed as a custom election by the party.
	 */
	@RosettaSynonym(value="partyA_specify", source="ISDA_Create_1_0")
	@RosettaSynonym(value="partyB_specify", source="ISDA_Create_1_0")
	@RosettaSynonym(value="partyA_threshold", source="ISDA_Create_1_0")
	@RosettaSynonym(value="partyB_threshold", source="ISDA_Create_1_0")
	public final String getCustomElection() {
		return customElection;
	}
	
	/**
	 * The elective amount when specified as 0 with no associated currency. The associated PartyElectiveAmount_noAmount data rule enforces that the number is actually equal to 0.
	 */
	@RosettaSynonym(value="", source="ISDA_Create_1_0")
	@RosettaSynonym(value="", source="ISDA_Create_1_0")
	public final Integer getNoAmount() {
		return noAmount;
	}
	
	/**
	 * The elective party.
	 */
	@RosettaSynonym(value="", source="ISDA_Create_1_0")
	@RosettaSynonym(value="", source="ISDA_Create_1_0")
	public final String getParty() {
		return party;
	}
	
	
	@Override
	public RosettaMetaData<? extends ElectiveAmountElection> metaData() {
		return metaData;
	} 
	
	public ElectiveAmountElectionBuilder toBuilder() {
		ElectiveAmountElectionBuilder builder = new ElectiveAmountElectionBuilder();
		ofNullable(getAmount()).ifPresent(builder::setAmount);
		ofNullable(getCustomElection()).ifPresent(builder::setCustomElection);
		ofNullable(getNoAmount()).ifPresent(builder::setNoAmount);
		ofNullable(getParty()).ifPresent(builder::setParty);
		return builder;
	}

	public static ElectiveAmountElectionBuilder builder() {
		return new ElectiveAmountElectionBuilder();
	}

	public static  class ElectiveAmountElectionBuilder extends RosettaModelObjectBuilder {
	
		protected Money.MoneyBuilder amount;
		protected String customElection;
		protected Integer noAmount;
		protected String party;
	
		public ElectiveAmountElectionBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends ElectiveAmountElection> metaData() {
			return metaData;
		} 
	
		
		public Money.MoneyBuilder getAmount() {
			return amount;
		}
		
		public Money.MoneyBuilder getOrCreateAmount() {
			if (amount!=null) {
				return amount;
			}
			else return amount = new Money.MoneyBuilder();
		}
		
		
		public String getCustomElection() {
			return customElection;
		}
		
		
		public Integer getNoAmount() {
			return noAmount;
		}
		
		
		public String getParty() {
			return party;
		}
		
	
		public ElectiveAmountElectionBuilder setAmount(Money amount) {
			this.amount = amount.toBuilder();
			return this;
		}
		public ElectiveAmountElectionBuilder setAmountBuilder(Money.MoneyBuilder amount) {
			this.amount = amount;
			return this;
		}
		public ElectiveAmountElectionBuilder setCustomElection(String customElection) {
			this.customElection = customElection;
			return this;
		}
		public ElectiveAmountElectionBuilder setNoAmount(Integer noAmount) {
			this.noAmount = noAmount;
			return this;
		}
		public ElectiveAmountElectionBuilder setParty(String party) {
			this.party = party;
			return this;
		}
		
		public ElectiveAmountElection build() {
			return new ElectiveAmountElection(this);
		}
	
		@Override
		public ElectiveAmountElectionBuilder prune() {
			if (amount!=null && !amount.prune().hasData()) amount = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getAmount()!=null && getAmount().hasData()) return true;
			if (getCustomElection()!=null) return true;
			if (getNoAmount()!=null) return true;
			if (getParty()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("customElection"), String.class, customElection, this);
			processor.processBasic(path.newSubPath("noAmount"), Integer.class, noAmount, this);
			processor.processBasic(path.newSubPath("party"), String.class, party, this);
			
			processRosetta(path.newSubPath("amount"), processor, Money.class, amount);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			ElectiveAmountElectionBuilder _that = (ElectiveAmountElectionBuilder) o;
		
			if (amount != null ? !amount.equals(_that.amount) : _that.amount != null) return false;
			if (customElection != null ? !customElection.equals(_that.customElection) : _that.customElection != null) return false;
			if (noAmount != null ? !noAmount.equals(_that.noAmount) : _that.noAmount != null) return false;
			if (party != null ? !party.equals(_that.party) : _that.party != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (amount != null ? amount.hashCode() : 0);
			_result = 31 * _result + (customElection != null ? customElection.hashCode() : 0);
			_result = 31 * _result + (noAmount != null ? noAmount.hashCode() : 0);
			_result = 31 * _result + (party != null ? party.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "ElectiveAmountElectionBuilder {" +
				"amount=" + this.amount + ", " +
				"customElection=" + this.customElection + ", " +
				"noAmount=" + this.noAmount + ", " +
				"party=" + this.party +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("customElection"), String.class, customElection, this);
		processor.processBasic(path.newSubPath("noAmount"), Integer.class, noAmount, this);
		processor.processBasic(path.newSubPath("party"), String.class, party, this);
		
		processRosetta(path.newSubPath("amount"), processor, Money.class, amount);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		ElectiveAmountElection _that = (ElectiveAmountElection) o;
	
		if (amount != null ? !amount.equals(_that.amount) : _that.amount != null) return false;
		if (customElection != null ? !customElection.equals(_that.customElection) : _that.customElection != null) return false;
		if (noAmount != null ? !noAmount.equals(_that.noAmount) : _that.noAmount != null) return false;
		if (party != null ? !party.equals(_that.party) : _that.party != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (amount != null ? amount.hashCode() : 0);
		_result = 31 * _result + (customElection != null ? customElection.hashCode() : 0);
		_result = 31 * _result + (noAmount != null ? noAmount.hashCode() : 0);
		_result = 31 * _result + (party != null ? party.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "ElectiveAmountElection {" +
			"amount=" + this.amount + ", " +
			"customElection=" + this.customElection + ", " +
			"noAmount=" + this.noAmount + ", " +
			"party=" + this.party +
		'}';
	}
}
