package org.isda.cdm;

import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.CustodianTerms;
import org.isda.cdm.HoldingPostedCollateralEnum;
import org.isda.cdm.metafields.FieldWithMetaString;
import org.isda.cdm.metafields.FieldWithMetaString.FieldWithMetaStringBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.EligibilityToHoldCollateralMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;

/**
 * A class to specify the conditions under which a party and its custodian(s) are entitled to hold collateral. ISDA 2016 Credit Support Annex for Variation Margin, paragraph 13, (h)(i): Eligibility to Hold Posted Collateral (VM); Custodians (VM).
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="EligibilityToHoldCollateral", source="FpML_5_10")
public class EligibilityToHoldCollateral extends RosettaModelObject {
	private final CustodianTerms custodianTerms;
	private final List<FieldWithMetaString> eligibleCountry;
	private final List<HoldingPostedCollateralEnum> partyTerms;
	private static EligibilityToHoldCollateralMeta metaData = new EligibilityToHoldCollateralMeta();
	
	EligibilityToHoldCollateral(EligibilityToHoldCollateralBuilder builder) {
		this.custodianTerms = ofNullable(builder.getCustodianTerms()).map(f->f.build()).orElse(null);
		this.eligibleCountry = ofNullable(builder.getEligibleCountry()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.partyTerms = builder.getPartyTerms();
	}
	
	/**
	 * The restrictions that might be required by a party from the other party&#39;s custodian agent to hold its posted collateral.
	 */
	@RosettaSynonym(value="custodianTerms", source="FpML_5_10")
	public final CustodianTerms getCustodianTerms() {
		return custodianTerms;
	}
	
	/**
	 * The restrictions that might be required by a party from the other party in terms of country(ies) where collateral can be held.
	 */
	@RosettaSynonym(value="eligibleCountry", source="FpML_5_10")
	public final List<FieldWithMetaString> getEligibleCountry() {
		return eligibleCountry;
	}
	
	/**
	 * The condition(s) required by a party from the other party to hold its posted collateral.
	 */
	@RosettaSynonym(value="holdingPostedCollateral", source="FpML_5_10")
	public final List<HoldingPostedCollateralEnum> getPartyTerms() {
		return partyTerms;
	}
	
	
	@Override
	public RosettaMetaData<? extends EligibilityToHoldCollateral> metaData() {
		return metaData;
	} 
	
	public EligibilityToHoldCollateralBuilder toBuilder() {
		EligibilityToHoldCollateralBuilder builder = new EligibilityToHoldCollateralBuilder();
		ofNullable(getCustodianTerms()).ifPresent(builder::setCustodianTerms);
		ofNullable(getEligibleCountry()).ifPresent(eligibleCountry -> eligibleCountry.forEach(builder::addEligibleCountry));
		ofNullable(getPartyTerms()).ifPresent(partyTerms -> partyTerms.forEach(builder::addPartyTerms));
		return builder;
	}

	public static EligibilityToHoldCollateralBuilder builder() {
		return new EligibilityToHoldCollateralBuilder();
	}

	public static  class EligibilityToHoldCollateralBuilder extends RosettaModelObjectBuilder {
	
		protected CustodianTerms.CustodianTermsBuilder custodianTerms;
		protected List<FieldWithMetaStringBuilder> eligibleCountry;
		protected List<HoldingPostedCollateralEnum> partyTerms;
	
		public EligibilityToHoldCollateralBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends EligibilityToHoldCollateral> metaData() {
			return metaData;
		} 
	
		
		public CustodianTerms.CustodianTermsBuilder getCustodianTerms() {
			return custodianTerms;
		}
		
		public CustodianTerms.CustodianTermsBuilder getOrCreateCustodianTerms() {
			if (custodianTerms!=null) {
				return custodianTerms;
			}
			else return custodianTerms = new CustodianTerms.CustodianTermsBuilder();
		}
		
		
		public List<FieldWithMetaStringBuilder> getEligibleCountry() {
			return eligibleCountry;
		}
		
		public FieldWithMetaStringBuilder getOrCreateEligibleCountry(int index) {
			if (eligibleCountry==null) {
				this.eligibleCountry = new ArrayList<>();
			}
			return getIndex(eligibleCountry, index, ()->new FieldWithMetaStringBuilder());
		}
		
		
		public List<HoldingPostedCollateralEnum> getPartyTerms() {
			return partyTerms;
		}
		
	
		public EligibilityToHoldCollateralBuilder setCustodianTerms(CustodianTerms custodianTerms) {
			this.custodianTerms = custodianTerms.toBuilder();
			return this;
		}
		public EligibilityToHoldCollateralBuilder setCustodianTermsBuilder(CustodianTerms.CustodianTermsBuilder custodianTerms) {
			this.custodianTerms = custodianTerms;
			return this;
		}
		public EligibilityToHoldCollateralBuilder addEligibleCountry(FieldWithMetaString eligibleCountry) {
			if(this.eligibleCountry == null){
				this.eligibleCountry = new ArrayList<>();
			}
			this.eligibleCountry.add(eligibleCountry.toBuilder());
			return this;
		}
		public EligibilityToHoldCollateralBuilder addEligibleCountry(FieldWithMetaString eligibleCountry, int _idx) {
			if(this.eligibleCountry == null){
				this.eligibleCountry = new ArrayList<>();
			}
			getIndex(this.eligibleCountry, _idx, () -> eligibleCountry.toBuilder());
			this.eligibleCountry.set(_idx, eligibleCountry.toBuilder());
			return this;
		}
		public EligibilityToHoldCollateralBuilder addEligibleCountry(List<FieldWithMetaString> eligibleCountrys) {
			if(this.eligibleCountry == null){
				this.eligibleCountry = new ArrayList<>();
			}
			for (FieldWithMetaString toAdd : eligibleCountrys) {
				this.eligibleCountry.add(toAdd.toBuilder());
			}
			return this;
		}
		
		public EligibilityToHoldCollateralBuilder clearEligibleCountry() {
			this.eligibleCountry = null;
			return this;
		}
		public EligibilityToHoldCollateralBuilder addPartyTerms(HoldingPostedCollateralEnum partyTerms) {
			if(this.partyTerms == null){
				this.partyTerms = new ArrayList<>();
			}
			this.partyTerms.add(partyTerms);
			return this;
		}
		public EligibilityToHoldCollateralBuilder addPartyTerms(HoldingPostedCollateralEnum partyTerms, int _idx) {
			if(this.partyTerms == null){
				this.partyTerms = new ArrayList<>();
			}
			getIndex(this.partyTerms, _idx, () -> partyTerms);
			this.partyTerms.set(_idx, partyTerms);
			return this;
		}
		public EligibilityToHoldCollateralBuilder addPartyTerms(List<HoldingPostedCollateralEnum> partyTermss) {
			if(this.partyTerms == null){
				this.partyTerms = new ArrayList<>();
			}
			for (HoldingPostedCollateralEnum toAdd : partyTermss) {
				this.partyTerms.add(toAdd);
			}
			return this;
		}
		
		public EligibilityToHoldCollateralBuilder clearPartyTerms() {
			this.partyTerms = null;
			return this;
		}
		
		public EligibilityToHoldCollateral build() {
			return new EligibilityToHoldCollateral(this);
		}
	
		@Override
		public EligibilityToHoldCollateralBuilder prune() {
			if (custodianTerms!=null && !custodianTerms.prune().hasData()) custodianTerms = null;
			if (eligibleCountry!=null) eligibleCountry = eligibleCountry.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getCustodianTerms()!=null && getCustodianTerms().hasData()) return true;
			if (getEligibleCountry()!=null && !getEligibleCountry().isEmpty()) return true;
			if (getPartyTerms()!=null && !getPartyTerms().isEmpty()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("partyTerms"), HoldingPostedCollateralEnum.class, partyTerms, this);
			
			processRosetta(path.newSubPath("custodianTerms"), processor, CustodianTerms.class, custodianTerms);
			processRosetta(path.newSubPath("eligibleCountry"), processor, FieldWithMetaString.class, eligibleCountry);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			EligibilityToHoldCollateralBuilder _that = (EligibilityToHoldCollateralBuilder) o;
		
			if (custodianTerms != null ? !custodianTerms.equals(_that.custodianTerms) : _that.custodianTerms != null) return false;
			if (!ListEquals.listEquals(eligibleCountry, _that.eligibleCountry)) return false;
			if (!ListEquals.listEquals(partyTerms, _that.partyTerms)) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (custodianTerms != null ? custodianTerms.hashCode() : 0);
			_result = 31 * _result + (eligibleCountry != null ? eligibleCountry.hashCode() : 0);
			_result = 31 * _result + (partyTerms != null ? partyTerms.stream().map(Object::getClass).map(Class::getName).mapToInt(String::hashCode).sum() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "EligibilityToHoldCollateralBuilder {" +
				"custodianTerms=" + this.custodianTerms + ", " +
				"eligibleCountry=" + this.eligibleCountry + ", " +
				"partyTerms=" + this.partyTerms +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		partyTerms.stream().forEach(a->processor.processBasic(path.newSubPath("partyTerms"), HoldingPostedCollateralEnum.class, a, this));
		
		processRosetta(path.newSubPath("custodianTerms"), processor, CustodianTerms.class, custodianTerms);
		processRosetta(path.newSubPath("eligibleCountry"), processor, FieldWithMetaString.class, eligibleCountry);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		EligibilityToHoldCollateral _that = (EligibilityToHoldCollateral) o;
	
		if (custodianTerms != null ? !custodianTerms.equals(_that.custodianTerms) : _that.custodianTerms != null) return false;
		if (!ListEquals.listEquals(eligibleCountry, _that.eligibleCountry)) return false;
		if (!ListEquals.listEquals(partyTerms, _that.partyTerms)) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (custodianTerms != null ? custodianTerms.hashCode() : 0);
		_result = 31 * _result + (eligibleCountry != null ? eligibleCountry.hashCode() : 0);
		_result = 31 * _result + (partyTerms != null ? partyTerms.stream().map(Object::getClass).map(Class::getName).mapToInt(String::hashCode).sum() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "EligibilityToHoldCollateral {" +
			"custodianTerms=" + this.custodianTerms + ", " +
			"eligibleCountry=" + this.eligibleCountry + ", " +
			"partyTerms=" + this.partyTerms +
		'}';
	}
}
