package org.isda.cdm;

import java.math.BigDecimal;
import org.isda.cdm.CollateralAssetDefinitionsEnum;
import org.isda.cdm.CreditNotations;
import org.isda.cdm.Period;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.EligibleCollateralMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;
		import java.math.BigDecimal;

/**
 * The collateral eligibility as a function of the types of asset, the maturity and rating terms.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="EligibleCollateral", source="FpML_5_10")
public class EligibleCollateral extends RosettaModelObject {
	private final CollateralAssetDefinitionsEnum eligibleAsset;
	private final Period higherMaturity;
	private final Period lowerMaturity;
	private final CreditNotations minimumRating;
	private final BigDecimal valuationPercentage;
	private static EligibleCollateralMeta metaData = new EligibleCollateralMeta();
	
	EligibleCollateral(EligibleCollateralBuilder builder) {
		this.eligibleAsset = builder.getEligibleAsset();
		this.higherMaturity = ofNullable(builder.getHigherMaturity()).map(f->f.build()).orElse(null);
		this.lowerMaturity = ofNullable(builder.getLowerMaturity()).map(f->f.build()).orElse(null);
		this.minimumRating = ofNullable(builder.getMinimumRating()).map(f->f.build()).orElse(null);
		this.valuationPercentage = builder.getValuationPercentage();
	}
	
	/**
	 * The eligible asset is specified by reference to the ISDA 2003 Collateral Assets Definitions.
	 */
	@RosettaSynonym(value="eligibleAsset", source="FpML_5_10")
	@RosettaSynonym(value="items", source="ISDA_Create_1_0", path="partyA_eligible_currencies")
	@RosettaSynonym(value="items", source="ISDA_Create_1_0", path="partyB_eligible_currencies")
	public final CollateralAssetDefinitionsEnum getEligibleAsset() {
		return eligibleAsset;
	}
	
	/**
	 * The (optional) higher maturity band of the eligible collateral assets.
	 */
	@RosettaSynonym(value="higherMaturity", source="FpML_5_10")
	public final Period getHigherMaturity() {
		return higherMaturity;
	}
	
	/**
	 * The (optional) lower maturity band of the eligible collateral assets.
	 */
	@RosettaSynonym(value="lowerMaturity", source="FpML_5_10")
	public final Period getLowerMaturity() {
		return lowerMaturity;
	}
	
	/**
	 * The (optional) minimal rating terms that can be specified as part of the eligible collateral.
	 */
	@RosettaSynonym(value="minimumRating", source="FpML_5_10")
	public final CreditNotations getMinimumRating() {
		return minimumRating;
	}
	
	/**
	 * Specifies the haircut associated to the collateral asset, expressed as a value comprised between 0 and 1. A value of 0.9 means that 90% of the value of the applicable asset is considered as collateral. Absence of this element implies that the valuation percentage is 1, i.e. 100%.
	 */
	@RosettaSynonym(value="valuationPercentage", source="FpML_5_10")
	@RosettaSynonym(value="valuation_percentage", source="ISDA_Create_1_0", path="partyA_eligible_currencies")
	@RosettaSynonym(value="valuation_percentage", source="ISDA_Create_1_0", path="partyB_eligible_currencies")
	public final BigDecimal getValuationPercentage() {
		return valuationPercentage;
	}
	
	
	@Override
	public RosettaMetaData<? extends EligibleCollateral> metaData() {
		return metaData;
	} 
	
	public EligibleCollateralBuilder toBuilder() {
		EligibleCollateralBuilder builder = new EligibleCollateralBuilder();
		ofNullable(getEligibleAsset()).ifPresent(builder::setEligibleAsset);
		ofNullable(getHigherMaturity()).ifPresent(builder::setHigherMaturity);
		ofNullable(getLowerMaturity()).ifPresent(builder::setLowerMaturity);
		ofNullable(getMinimumRating()).ifPresent(builder::setMinimumRating);
		ofNullable(getValuationPercentage()).ifPresent(builder::setValuationPercentage);
		return builder;
	}

	public static EligibleCollateralBuilder builder() {
		return new EligibleCollateralBuilder();
	}

	public static  class EligibleCollateralBuilder extends RosettaModelObjectBuilder {
	
		protected CollateralAssetDefinitionsEnum eligibleAsset;
		protected Period.PeriodBuilder higherMaturity;
		protected Period.PeriodBuilder lowerMaturity;
		protected CreditNotations.CreditNotationsBuilder minimumRating;
		protected BigDecimal valuationPercentage;
	
		public EligibleCollateralBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends EligibleCollateral> metaData() {
			return metaData;
		} 
	
		
		public CollateralAssetDefinitionsEnum getEligibleAsset() {
			return eligibleAsset;
		}
		
		
		public Period.PeriodBuilder getHigherMaturity() {
			return higherMaturity;
		}
		
		public Period.PeriodBuilder getOrCreateHigherMaturity() {
			if (higherMaturity!=null) {
				return higherMaturity;
			}
			else return higherMaturity = new Period.PeriodBuilder();
		}
		
		
		public Period.PeriodBuilder getLowerMaturity() {
			return lowerMaturity;
		}
		
		public Period.PeriodBuilder getOrCreateLowerMaturity() {
			if (lowerMaturity!=null) {
				return lowerMaturity;
			}
			else return lowerMaturity = new Period.PeriodBuilder();
		}
		
		
		public CreditNotations.CreditNotationsBuilder getMinimumRating() {
			return minimumRating;
		}
		
		public CreditNotations.CreditNotationsBuilder getOrCreateMinimumRating() {
			if (minimumRating!=null) {
				return minimumRating;
			}
			else return minimumRating = new CreditNotations.CreditNotationsBuilder();
		}
		
		
		public BigDecimal getValuationPercentage() {
			return valuationPercentage;
		}
		
	
		public EligibleCollateralBuilder setEligibleAsset(CollateralAssetDefinitionsEnum eligibleAsset) {
			this.eligibleAsset = eligibleAsset;
			return this;
		}
		public EligibleCollateralBuilder setHigherMaturity(Period higherMaturity) {
			this.higherMaturity = higherMaturity.toBuilder();
			return this;
		}
		public EligibleCollateralBuilder setHigherMaturityBuilder(Period.PeriodBuilder higherMaturity) {
			this.higherMaturity = higherMaturity;
			return this;
		}
		public EligibleCollateralBuilder setLowerMaturity(Period lowerMaturity) {
			this.lowerMaturity = lowerMaturity.toBuilder();
			return this;
		}
		public EligibleCollateralBuilder setLowerMaturityBuilder(Period.PeriodBuilder lowerMaturity) {
			this.lowerMaturity = lowerMaturity;
			return this;
		}
		public EligibleCollateralBuilder setMinimumRating(CreditNotations minimumRating) {
			this.minimumRating = minimumRating.toBuilder();
			return this;
		}
		public EligibleCollateralBuilder setMinimumRatingBuilder(CreditNotations.CreditNotationsBuilder minimumRating) {
			this.minimumRating = minimumRating;
			return this;
		}
		public EligibleCollateralBuilder setValuationPercentage(BigDecimal valuationPercentage) {
			this.valuationPercentage = valuationPercentage;
			return this;
		}
		
		public EligibleCollateral build() {
			return new EligibleCollateral(this);
		}
	
		@Override
		public EligibleCollateralBuilder prune() {
			if (higherMaturity!=null && !higherMaturity.prune().hasData()) higherMaturity = null;
			if (lowerMaturity!=null && !lowerMaturity.prune().hasData()) lowerMaturity = null;
			if (minimumRating!=null && !minimumRating.prune().hasData()) minimumRating = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getEligibleAsset()!=null) return true;
			if (getHigherMaturity()!=null && getHigherMaturity().hasData()) return true;
			if (getLowerMaturity()!=null && getLowerMaturity().hasData()) return true;
			if (getMinimumRating()!=null && getMinimumRating().hasData()) return true;
			if (getValuationPercentage()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("eligibleAsset"), CollateralAssetDefinitionsEnum.class, eligibleAsset, this);
			processor.processBasic(path.newSubPath("valuationPercentage"), BigDecimal.class, valuationPercentage, this);
			
			processRosetta(path.newSubPath("higherMaturity"), processor, Period.class, higherMaturity);
			processRosetta(path.newSubPath("lowerMaturity"), processor, Period.class, lowerMaturity);
			processRosetta(path.newSubPath("minimumRating"), processor, CreditNotations.class, minimumRating);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			EligibleCollateralBuilder _that = (EligibleCollateralBuilder) o;
		
			if (eligibleAsset != null ? !eligibleAsset.equals(_that.eligibleAsset) : _that.eligibleAsset != null) return false;
			if (higherMaturity != null ? !higherMaturity.equals(_that.higherMaturity) : _that.higherMaturity != null) return false;
			if (lowerMaturity != null ? !lowerMaturity.equals(_that.lowerMaturity) : _that.lowerMaturity != null) return false;
			if (minimumRating != null ? !minimumRating.equals(_that.minimumRating) : _that.minimumRating != null) return false;
			if (valuationPercentage != null ? !valuationPercentage.equals(_that.valuationPercentage) : _that.valuationPercentage != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (eligibleAsset != null ? eligibleAsset.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (higherMaturity != null ? higherMaturity.hashCode() : 0);
			_result = 31 * _result + (lowerMaturity != null ? lowerMaturity.hashCode() : 0);
			_result = 31 * _result + (minimumRating != null ? minimumRating.hashCode() : 0);
			_result = 31 * _result + (valuationPercentage != null ? valuationPercentage.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "EligibleCollateralBuilder {" +
				"eligibleAsset=" + this.eligibleAsset + ", " +
				"higherMaturity=" + this.higherMaturity + ", " +
				"lowerMaturity=" + this.lowerMaturity + ", " +
				"minimumRating=" + this.minimumRating + ", " +
				"valuationPercentage=" + this.valuationPercentage +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("eligibleAsset"), CollateralAssetDefinitionsEnum.class, eligibleAsset, this);
		processor.processBasic(path.newSubPath("valuationPercentage"), BigDecimal.class, valuationPercentage, this);
		
		processRosetta(path.newSubPath("higherMaturity"), processor, Period.class, higherMaturity);
		processRosetta(path.newSubPath("lowerMaturity"), processor, Period.class, lowerMaturity);
		processRosetta(path.newSubPath("minimumRating"), processor, CreditNotations.class, minimumRating);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		EligibleCollateral _that = (EligibleCollateral) o;
	
		if (eligibleAsset != null ? !eligibleAsset.equals(_that.eligibleAsset) : _that.eligibleAsset != null) return false;
		if (higherMaturity != null ? !higherMaturity.equals(_that.higherMaturity) : _that.higherMaturity != null) return false;
		if (lowerMaturity != null ? !lowerMaturity.equals(_that.lowerMaturity) : _that.lowerMaturity != null) return false;
		if (minimumRating != null ? !minimumRating.equals(_that.minimumRating) : _that.minimumRating != null) return false;
		if (valuationPercentage != null ? !valuationPercentage.equals(_that.valuationPercentage) : _that.valuationPercentage != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (eligibleAsset != null ? eligibleAsset.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (higherMaturity != null ? higherMaturity.hashCode() : 0);
		_result = 31 * _result + (lowerMaturity != null ? lowerMaturity.hashCode() : 0);
		_result = 31 * _result + (minimumRating != null ? minimumRating.hashCode() : 0);
		_result = 31 * _result + (valuationPercentage != null ? valuationPercentage.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "EligibleCollateral {" +
			"eligibleAsset=" + this.eligibleAsset + ", " +
			"higherMaturity=" + this.higherMaturity + ", " +
			"lowerMaturity=" + this.lowerMaturity + ", " +
			"minimumRating=" + this.minimumRating + ", " +
			"valuationPercentage=" + this.valuationPercentage +
		'}';
	}
}
