package org.isda.cdm;

import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.EligibleCollateral;
import org.isda.cdm.metafields.ReferenceWithMetaParty;
import org.isda.cdm.metafields.ReferenceWithMetaParty.ReferenceWithMetaPartyBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.EligibleCollateralVariationMarginElectionMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.stream.Collectors;
		import static java.util.Optional.ofNullable;
		import java.util.List;
		import java.util.ArrayList;
		import java.util.Objects;

/**
 * A class to specify the parties&#39; elections with respect to the eligible collateral for each of the respective parties when acting as a pledgor/chargor/obligor. ISDA 2016 Credit Support Annex for Variation Margin, paragraph 13, (c)(ii): Eligible Collateral (VM).
 * 
 * @version 2.5.4
 */
@RosettaClass
public class EligibleCollateralVariationMarginElection extends RosettaModelObject {
	private final List<EligibleCollateral> eligibleCollateral;
	private final ReferenceWithMetaParty party;
	private static EligibleCollateralVariationMarginElectionMeta metaData = new EligibleCollateralVariationMarginElectionMeta();
	
	EligibleCollateralVariationMarginElection(EligibleCollateralVariationMarginElectionBuilder builder) {
		this.eligibleCollateral = ofNullable(builder.getEligibleCollateral()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.party = ofNullable(builder.getParty()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The eligible collateral for the specified party (acting as the Pledgor in the New York Law agreement, as the Chargor in the English Law deed and as the Obligor in the Japanese Law agreement). ISDA 2016 Credit Support Annex for Variation Margin, paragraph 13, (c)(ii): Eligible Collateral (VM).
	 */
	public final List<EligibleCollateral> getEligibleCollateral() {
		return eligibleCollateral;
	}
	
	/**
	 * The elective party.
	 */
	public final ReferenceWithMetaParty getParty() {
		return party;
	}
	
	
	@Override
	public RosettaMetaData<? extends EligibleCollateralVariationMarginElection> metaData() {
		return metaData;
	} 
	
	public EligibleCollateralVariationMarginElectionBuilder toBuilder() {
		EligibleCollateralVariationMarginElectionBuilder builder = new EligibleCollateralVariationMarginElectionBuilder();
		ofNullable(getEligibleCollateral()).ifPresent(eligibleCollateral -> eligibleCollateral.forEach(builder::addEligibleCollateral));
		ofNullable(getParty()).ifPresent(builder::setParty);
		return builder;
	}

	public static EligibleCollateralVariationMarginElectionBuilder builder() {
		return new EligibleCollateralVariationMarginElectionBuilder();
	}

	public static  class EligibleCollateralVariationMarginElectionBuilder extends RosettaModelObjectBuilder {
	
		protected List<EligibleCollateral.EligibleCollateralBuilder> eligibleCollateral;
		protected ReferenceWithMetaPartyBuilder party;
	
		public EligibleCollateralVariationMarginElectionBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends EligibleCollateralVariationMarginElection> metaData() {
			return metaData;
		} 
	
		
		public List<EligibleCollateral.EligibleCollateralBuilder> getEligibleCollateral() {
			return eligibleCollateral;
		}
		
		public EligibleCollateral.EligibleCollateralBuilder getOrCreateEligibleCollateral(int index) {
			if (eligibleCollateral==null) {
				this.eligibleCollateral = new ArrayList<>();
			}
			return getIndex(eligibleCollateral, index, ()->new EligibleCollateral.EligibleCollateralBuilder());
		}
		
		
		public ReferenceWithMetaPartyBuilder getParty() {
			return party;
		}
		
		public ReferenceWithMetaPartyBuilder getOrCreateParty() {
			if (party!=null) {
				return party;
			}
			else return party = new ReferenceWithMetaPartyBuilder();
		}
		
	
		public EligibleCollateralVariationMarginElectionBuilder addEligibleCollateral(EligibleCollateral eligibleCollateral) {
			if(this.eligibleCollateral == null){
				this.eligibleCollateral = new ArrayList<>();
			}
			this.eligibleCollateral.add(eligibleCollateral.toBuilder());
			return this;
		}
		public EligibleCollateralVariationMarginElectionBuilder addEligibleCollateral(EligibleCollateral eligibleCollateral, int _idx) {
			if(this.eligibleCollateral == null){
				this.eligibleCollateral = new ArrayList<>();
			}
			getIndex(this.eligibleCollateral, _idx, () -> eligibleCollateral.toBuilder());
			this.eligibleCollateral.set(_idx, eligibleCollateral.toBuilder());
			return this;
		}
		public EligibleCollateralVariationMarginElectionBuilder addEligibleCollateral(List<EligibleCollateral> eligibleCollaterals) {
			if(this.eligibleCollateral == null){
				this.eligibleCollateral = new ArrayList<>();
			}
			for (EligibleCollateral toAdd : eligibleCollaterals) {
				this.eligibleCollateral.add(toAdd.toBuilder());
			}
			return this;
		}
		public EligibleCollateralVariationMarginElectionBuilder addEligibleCollateralBuilder(EligibleCollateral.EligibleCollateralBuilder eligibleCollateral) {
			if(this.eligibleCollateral == null){
				this.eligibleCollateral = new ArrayList<>();
			}
			this.eligibleCollateral.add(eligibleCollateral);
			return this;
		}
		
		public EligibleCollateralVariationMarginElectionBuilder clearEligibleCollateral() {
			this.eligibleCollateral = null;
			return this;
		}
		public EligibleCollateralVariationMarginElectionBuilder setParty(ReferenceWithMetaParty party) {
			this.party = party.toBuilder();
			return this;
		}
		public EligibleCollateralVariationMarginElectionBuilder setPartyBuilder(ReferenceWithMetaPartyBuilder party) {
			this.party = party;
			return this;
		}
		public EligibleCollateralVariationMarginElectionBuilder setPartyRef(Party.PartyBuilder party) {
			return setParty(ReferenceWithMetaParty.builder().setValueBuilder(party).build());
		}
		public EligibleCollateralVariationMarginElectionBuilder setPartyRef(Party party) {
			return setPartyRef(Party.builder());
		}
		
		public EligibleCollateralVariationMarginElection build() {
			return new EligibleCollateralVariationMarginElection(this);
		}
	
		@Override
		public EligibleCollateralVariationMarginElectionBuilder prune() {
			if (eligibleCollateral!=null) eligibleCollateral = eligibleCollateral.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (party!=null && !party.prune().hasData()) party = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getEligibleCollateral()!=null && getEligibleCollateral().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getParty()!=null && getParty().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("eligibleCollateral"), processor, EligibleCollateral.class, eligibleCollateral);
			processRosetta(path.newSubPath("party"), processor, ReferenceWithMetaParty.class, party);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			EligibleCollateralVariationMarginElectionBuilder _that = (EligibleCollateralVariationMarginElectionBuilder) o;
		
			if (!ListEquals.listEquals(eligibleCollateral, _that.eligibleCollateral)) return false;
			if (party != null ? !party.equals(_that.party) : _that.party != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (eligibleCollateral != null ? eligibleCollateral.hashCode() : 0);
			_result = 31 * _result + (party != null ? party.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "EligibleCollateralVariationMarginElectionBuilder {" +
				"eligibleCollateral=" + this.eligibleCollateral + ", " +
				"party=" + this.party +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("eligibleCollateral"), processor, EligibleCollateral.class, eligibleCollateral);
		processRosetta(path.newSubPath("party"), processor, ReferenceWithMetaParty.class, party);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		EligibleCollateralVariationMarginElection _that = (EligibleCollateralVariationMarginElection) o;
	
		if (!ListEquals.listEquals(eligibleCollateral, _that.eligibleCollateral)) return false;
		if (party != null ? !party.equals(_that.party) : _that.party != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (eligibleCollateral != null ? eligibleCollateral.hashCode() : 0);
		_result = 31 * _result + (party != null ? party.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "EligibleCollateralVariationMarginElection {" +
			"eligibleCollateral=" + this.eligibleCollateral + ", " +
			"party=" + this.party +
		'}';
	}
}
