package org.isda.cdm;

import java.lang.String;
import java.math.BigDecimal;
import org.isda.cdm.metafields.FieldWithMetaString;
import org.isda.cdm.metafields.FieldWithMetaString.FieldWithMetaStringBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.EligibleCurrencyInterestRateMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.math.BigDecimal;
		import static java.util.Optional.ofNullable;

/**
 * A class to specify the interest rate associated with initial margin collateral. ISDA 2016 Japanese Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (n): Distributions and Interest Payment (IM).
 * 
 * @version 2.5.4
 */
@RosettaClass
public class EligibleCurrencyInterestRate extends RosettaModelObject {
	private final String actual365Currency;
	private final FieldWithMetaString currency;
	private final BigDecimal interestRate;
	private static EligibleCurrencyInterestRateMeta metaData = new EligibleCurrencyInterestRateMeta();
	
	EligibleCurrencyInterestRate(EligibleCurrencyInterestRateBuilder builder) {
		this.actual365Currency = builder.getActual365Currency();
		this.currency = ofNullable(builder.getCurrency()).map(f->f.build()).orElse(null);
		this.interestRate = builder.getInterestRate();
	}
	
	@RosettaSynonym(value="a365_currency", source="ISDA_Create_1_0")
	public final String getActual365Currency() {
		return actual365Currency;
	}
	
	/**
	 * The eligible currency.
	 */
	@RosettaSynonym(value="eligible_currency", source="ISDA_Create_1_0")
	public final FieldWithMetaString getCurrency() {
		return currency;
	}
	
	/**
	 * The interest rate associated with the eligible currency.
	 */
	@RosettaSynonym(value="interest_rate", source="ISDA_Create_1_0")
	public final BigDecimal getInterestRate() {
		return interestRate;
	}
	
	
	@Override
	public RosettaMetaData<? extends EligibleCurrencyInterestRate> metaData() {
		return metaData;
	} 
	
	public EligibleCurrencyInterestRateBuilder toBuilder() {
		EligibleCurrencyInterestRateBuilder builder = new EligibleCurrencyInterestRateBuilder();
		ofNullable(getActual365Currency()).ifPresent(builder::setActual365Currency);
		ofNullable(getCurrency()).ifPresent(builder::setCurrency);
		ofNullable(getInterestRate()).ifPresent(builder::setInterestRate);
		return builder;
	}

	public static EligibleCurrencyInterestRateBuilder builder() {
		return new EligibleCurrencyInterestRateBuilder();
	}

	public static  class EligibleCurrencyInterestRateBuilder extends RosettaModelObjectBuilder {
	
		protected String actual365Currency;
		protected FieldWithMetaStringBuilder currency;
		protected BigDecimal interestRate;
	
		public EligibleCurrencyInterestRateBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends EligibleCurrencyInterestRate> metaData() {
			return metaData;
		} 
	
		
		public String getActual365Currency() {
			return actual365Currency;
		}
		
		
		public FieldWithMetaStringBuilder getCurrency() {
			return currency;
		}
		
		public FieldWithMetaStringBuilder getOrCreateCurrency() {
			if (currency!=null) {
				return currency;
			}
			else return currency = new FieldWithMetaStringBuilder();
		}
		
		
		public BigDecimal getInterestRate() {
			return interestRate;
		}
		
	
		public EligibleCurrencyInterestRateBuilder setActual365Currency(String actual365Currency) {
			this.actual365Currency = actual365Currency;
			return this;
		}
		public EligibleCurrencyInterestRateBuilder setCurrency(FieldWithMetaString currency) {
			this.currency = currency.toBuilder();
			return this;
		}
		public EligibleCurrencyInterestRateBuilder setCurrencyRef(String currency) {
			return setCurrency(FieldWithMetaString.builder().setValue(currency).build());
		}
		public EligibleCurrencyInterestRateBuilder setInterestRate(BigDecimal interestRate) {
			this.interestRate = interestRate;
			return this;
		}
		
		public EligibleCurrencyInterestRate build() {
			return new EligibleCurrencyInterestRate(this);
		}
	
		@Override
		public EligibleCurrencyInterestRateBuilder prune() {
			if (currency!=null && !currency.prune().hasData()) currency = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getActual365Currency()!=null) return true;
			if (getCurrency()!=null) return true;
			if (getInterestRate()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("actual365Currency"), String.class, actual365Currency, this);
			processor.processBasic(path.newSubPath("interestRate"), BigDecimal.class, interestRate, this);
			
			processRosetta(path.newSubPath("currency"), processor, FieldWithMetaString.class, currency);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			EligibleCurrencyInterestRateBuilder _that = (EligibleCurrencyInterestRateBuilder) o;
		
			if (actual365Currency != null ? !actual365Currency.equals(_that.actual365Currency) : _that.actual365Currency != null) return false;
			if (currency != null ? !currency.equals(_that.currency) : _that.currency != null) return false;
			if (interestRate != null ? !interestRate.equals(_that.interestRate) : _that.interestRate != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (actual365Currency != null ? actual365Currency.hashCode() : 0);
			_result = 31 * _result + (currency != null ? currency.hashCode() : 0);
			_result = 31 * _result + (interestRate != null ? interestRate.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "EligibleCurrencyInterestRateBuilder {" +
				"actual365Currency=" + this.actual365Currency + ", " +
				"currency=" + this.currency + ", " +
				"interestRate=" + this.interestRate +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("actual365Currency"), String.class, actual365Currency, this);
		processor.processBasic(path.newSubPath("interestRate"), BigDecimal.class, interestRate, this);
		
		processRosetta(path.newSubPath("currency"), processor, FieldWithMetaString.class, currency);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		EligibleCurrencyInterestRate _that = (EligibleCurrencyInterestRate) o;
	
		if (actual365Currency != null ? !actual365Currency.equals(_that.actual365Currency) : _that.actual365Currency != null) return false;
		if (currency != null ? !currency.equals(_that.currency) : _that.currency != null) return false;
		if (interestRate != null ? !interestRate.equals(_that.interestRate) : _that.interestRate != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (actual365Currency != null ? actual365Currency.hashCode() : 0);
		_result = 31 * _result + (currency != null ? currency.hashCode() : 0);
		_result = 31 * _result + (interestRate != null ? interestRate.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "EligibleCurrencyInterestRate {" +
			"actual365Currency=" + this.actual365Currency + ", " +
			"currency=" + this.currency + ", " +
			"interestRate=" + this.interestRate +
		'}';
	}
}
