package org.isda.cdm;

import org.isda.cdm.ShareExtraordinaryEventEnum;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.EquityCorporateEventsMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class for defining the merger events and their treatment.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="EquityCorporateEvents", source="FpML_5_10")
public class EquityCorporateEvents extends RosettaModelObject {
	private final ShareExtraordinaryEventEnum shareForCombined;
	private final ShareExtraordinaryEventEnum shareForOther;
	private final ShareExtraordinaryEventEnum shareForShare;
	private static EquityCorporateEventsMeta metaData = new EquityCorporateEventsMeta();
	
	EquityCorporateEvents(EquityCorporateEventsBuilder builder) {
		this.shareForCombined = builder.getShareForCombined();
		this.shareForOther = builder.getShareForOther();
		this.shareForShare = builder.getShareForShare();
	}
	
	/**
	 * 2018 ISDA CDM Equity Confirmation for Security Equity Swap: Merger Event (S-F-C) shall occur if a Merger Event occurs and the consideration for the relevant Security consists solely of Combined Consideration. | The consideration paid for the original shares following the Merger Event consists of both cash/securities and new shares.
	 */
	@RosettaSynonym(value="shareForCombined", source="FpML_5_10")
	public final ShareExtraordinaryEventEnum getShareForCombined() {
		return shareForCombined;
	}
	
	/**
	 * 2018 ISDA CDM Equity Confirmation for Security Equity Swap: Merger Event (S-F-O) shall occur if a Merger Event occurs and the consideration for the relevant Security consists solely of Other Consideration. | The consideration paid for the original shares following the Merger Event consists wholly of cash/securities other than new shares.
	 */
	@RosettaSynonym(value="shareForOther", source="FpML_5_10")
	public final ShareExtraordinaryEventEnum getShareForOther() {
		return shareForOther;
	}
	
	/**
	 * 2018 ISDA CDM Equity Confirmation for Security Equity Swap: Merger Event (S-F-S) shall occur if a Merger Event occurs and the consideration for the relevant Security consists solely of Combined Consideration. | The consideration paid for the original shares following the Merger Event consists wholly of new shares.
	 */
	@RosettaSynonym(value="shareForShare", source="FpML_5_10")
	public final ShareExtraordinaryEventEnum getShareForShare() {
		return shareForShare;
	}
	
	
	@Override
	public RosettaMetaData<? extends EquityCorporateEvents> metaData() {
		return metaData;
	} 
	
	public EquityCorporateEventsBuilder toBuilder() {
		EquityCorporateEventsBuilder builder = new EquityCorporateEventsBuilder();
		ofNullable(getShareForCombined()).ifPresent(builder::setShareForCombined);
		ofNullable(getShareForOther()).ifPresent(builder::setShareForOther);
		ofNullable(getShareForShare()).ifPresent(builder::setShareForShare);
		return builder;
	}

	public static EquityCorporateEventsBuilder builder() {
		return new EquityCorporateEventsBuilder();
	}

	public static  class EquityCorporateEventsBuilder extends RosettaModelObjectBuilder {
	
		protected ShareExtraordinaryEventEnum shareForCombined;
		protected ShareExtraordinaryEventEnum shareForOther;
		protected ShareExtraordinaryEventEnum shareForShare;
	
		public EquityCorporateEventsBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends EquityCorporateEvents> metaData() {
			return metaData;
		} 
	
		
		public ShareExtraordinaryEventEnum getShareForCombined() {
			return shareForCombined;
		}
		
		
		public ShareExtraordinaryEventEnum getShareForOther() {
			return shareForOther;
		}
		
		
		public ShareExtraordinaryEventEnum getShareForShare() {
			return shareForShare;
		}
		
	
		public EquityCorporateEventsBuilder setShareForCombined(ShareExtraordinaryEventEnum shareForCombined) {
			this.shareForCombined = shareForCombined;
			return this;
		}
		public EquityCorporateEventsBuilder setShareForOther(ShareExtraordinaryEventEnum shareForOther) {
			this.shareForOther = shareForOther;
			return this;
		}
		public EquityCorporateEventsBuilder setShareForShare(ShareExtraordinaryEventEnum shareForShare) {
			this.shareForShare = shareForShare;
			return this;
		}
		
		public EquityCorporateEvents build() {
			return new EquityCorporateEvents(this);
		}
	
		@Override
		public EquityCorporateEventsBuilder prune() {
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getShareForCombined()!=null) return true;
			if (getShareForOther()!=null) return true;
			if (getShareForShare()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("shareForCombined"), ShareExtraordinaryEventEnum.class, shareForCombined, this);
			processor.processBasic(path.newSubPath("shareForOther"), ShareExtraordinaryEventEnum.class, shareForOther, this);
			processor.processBasic(path.newSubPath("shareForShare"), ShareExtraordinaryEventEnum.class, shareForShare, this);
			
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			EquityCorporateEventsBuilder _that = (EquityCorporateEventsBuilder) o;
		
			if (shareForCombined != null ? !shareForCombined.equals(_that.shareForCombined) : _that.shareForCombined != null) return false;
			if (shareForOther != null ? !shareForOther.equals(_that.shareForOther) : _that.shareForOther != null) return false;
			if (shareForShare != null ? !shareForShare.equals(_that.shareForShare) : _that.shareForShare != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (shareForCombined != null ? shareForCombined.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (shareForOther != null ? shareForOther.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (shareForShare != null ? shareForShare.getClass().getName().hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "EquityCorporateEventsBuilder {" +
				"shareForCombined=" + this.shareForCombined + ", " +
				"shareForOther=" + this.shareForOther + ", " +
				"shareForShare=" + this.shareForShare +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("shareForCombined"), ShareExtraordinaryEventEnum.class, shareForCombined, this);
		processor.processBasic(path.newSubPath("shareForOther"), ShareExtraordinaryEventEnum.class, shareForOther, this);
		processor.processBasic(path.newSubPath("shareForShare"), ShareExtraordinaryEventEnum.class, shareForShare, this);
		
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		EquityCorporateEvents _that = (EquityCorporateEvents) o;
	
		if (shareForCombined != null ? !shareForCombined.equals(_that.shareForCombined) : _that.shareForCombined != null) return false;
		if (shareForOther != null ? !shareForOther.equals(_that.shareForOther) : _that.shareForOther != null) return false;
		if (shareForShare != null ? !shareForShare.equals(_that.shareForShare) : _that.shareForShare != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (shareForCombined != null ? shareForCombined.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (shareForOther != null ? shareForOther.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (shareForShare != null ? shareForShare.getClass().getName().hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "EquityCorporateEvents {" +
			"shareForCombined=" + this.shareForCombined + ", " +
			"shareForOther=" + this.shareForOther + ", " +
			"shareForShare=" + this.shareForShare +
		'}';
	}
}
