package org.isda.cdm;

import java.math.BigDecimal;
import org.isda.cdm.CalculationPeriodDates;
import org.isda.cdm.DividendReturnTerms;
import org.isda.cdm.PayerReceiver;
import org.isda.cdm.PaymentDates;
import org.isda.cdm.PriceReturnTerms;
import org.isda.cdm.ReturnTypeEnum;
import org.isda.cdm.SettlementTerms;
import org.isda.cdm.Underlier;
import org.isda.cdm.metafields.MetaFields;


import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.EquityPayoutMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;
		import java.math.BigDecimal;

/**
 * The equity payout specification terms. The associated rosettaKey denotes the ability to associate a hash value to the EquityPayout instantiations for the purpose of model cross-referencing, in support of functionality such as the event effect and the lineage.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class EquityPayout extends PayoutBase implements GlobalKey {
	private final CalculationPeriodDates calculationPeriodDates;
	private final DividendReturnTerms dividendReturnTerms;
	private final MetaFields meta;
	private final PayerReceiver payerReceiver;
	private final PaymentDates paymentDates;
	private final BigDecimal performance;
	private final PriceReturnTerms priceReturnTerms;
	private final BigDecimal rateOfReturn;
	private final ReturnTypeEnum returnType;
	private final SettlementTerms settlementTerms;
	private final Underlier underlier;
	private static EquityPayoutMeta metaData = new EquityPayoutMeta();
	
	EquityPayout(EquityPayoutBuilder builder) {
		super(builder);
		this.calculationPeriodDates = ofNullable(builder.getCalculationPeriodDates()).map(f->f.build()).orElse(null);
		this.dividendReturnTerms = ofNullable(builder.getDividendReturnTerms()).map(f->f.build()).orElse(null);
		this.meta = ofNullable(builder.getMeta()).map(f->f.build()).orElse(null);
		this.payerReceiver = ofNullable(builder.getPayerReceiver()).map(f->f.build()).orElse(null);
		this.paymentDates = ofNullable(builder.getPaymentDates()).map(f->f.build()).orElse(null);
		this.performance = builder.getPerformance();
		this.priceReturnTerms = ofNullable(builder.getPriceReturnTerms()).map(f->f.build()).orElse(null);
		this.rateOfReturn = builder.getRateOfReturn();
		this.returnType = builder.getReturnType();
		this.settlementTerms = ofNullable(builder.getSettlementTerms()).map(f->f.build()).orElse(null);
		this.underlier = ofNullable(builder.getUnderlier()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The calculation period dates schedule.
	 */
	public final CalculationPeriodDates getCalculationPeriodDates() {
		return calculationPeriodDates;
	}
	
	/**
	 * 2018 ISDA CDM Equity Confirmation for Security Equity Swap: Dividend Obligations
	 */
	@RosettaSynonym(value="dividendConditions", source="FpML_5_10", path="return")
	@RosettaSynonym(value="singleUnderlyer", source="FpML_5_10", path="underlyer")
	public final DividendReturnTerms getDividendReturnTerms() {
		return dividendReturnTerms;
	}
	
	public final MetaFields getMeta() {
		return meta;
	}
	
	public final PayerReceiver getPayerReceiver() {
		return payerReceiver;
	}
	
	/**
	 * The payment date schedule, as defined by the parameters that are needed to specify it, either in a parametric way or by reference to another schedule of dates (e.g. the valuation dates).
	 */
	@RosettaSynonym(value="paymentDates", source="FpML_5_10", path="rateOfReturn")
	public final PaymentDates getPaymentDates() {
		return paymentDates;
	}
	
	/**
	 * Performance calculation, in accordance with Part 1 Section 12 of the 2018 ISDA CDM Equity Confirmation for Security Equity Swap, Para 75. &#39;Equity Performance&#39;. Cumulative performance is used as a notional multiplier factor on both legs of an Equity Swap.
	 */
	public final BigDecimal getPerformance() {
		return performance;
	}
	
	/**
	 * 2018 ISDA CDM Equity Confirmation for Security Equity Swap: Pricing
	 */
	public final PriceReturnTerms getPriceReturnTerms() {
		return priceReturnTerms;
	}
	
	/**
	 * Rate of Return calculation, in accordance with Part 1 Section 12 of the 2018 ISDA CDM Equity Confirmation for Security Equity Swap, Para 139. &#39;Rate Of Return&#39;.
	 */
	public final BigDecimal getRateOfReturn() {
		return rateOfReturn;
	}
	
	/**
	 * Specifies the type of return associated with the equity payout.
	 */
	@RosettaSynonym(value="returnType", source="FpML_5_10", path="return")
	public final ReturnTypeEnum getReturnType() {
		return returnType;
	}
	
	/**
	 * 2018 ISDA CDM Equity Confirmation for Security Equity Swap: Settlement.
	 */
	public final SettlementTerms getSettlementTerms() {
		return settlementTerms;
	}
	
	/**
	 * 2018 ISDA CDM Equity Confirmation for Security Equity Swap: Security
	 */
	@RosettaSynonym(value="singleUnderlyer", source="FpML_5_10", path="underlyer")
	@RosettaSynonym(value="basket", source="FpML_5_10", path="underlyer")
	public final Underlier getUnderlier() {
		return underlier;
	}
	
	
	@Override
	public RosettaMetaData<? extends EquityPayout> metaData() {
		return metaData;
	} 
	
	public EquityPayoutBuilder toBuilder() {
		EquityPayoutBuilder builder = new EquityPayoutBuilder();
		ofNullable(getCalculationPeriodDates()).ifPresent(builder::setCalculationPeriodDates);
		ofNullable(getDividendReturnTerms()).ifPresent(builder::setDividendReturnTerms);
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		ofNullable(getPayerReceiver()).ifPresent(builder::setPayerReceiver);
		ofNullable(getPaymentDates()).ifPresent(builder::setPaymentDates);
		ofNullable(getPerformance()).ifPresent(builder::setPerformance);
		ofNullable(getPriceReturnTerms()).ifPresent(builder::setPriceReturnTerms);
		ofNullable(getRateOfReturn()).ifPresent(builder::setRateOfReturn);
		ofNullable(getReturnType()).ifPresent(builder::setReturnType);
		ofNullable(getSettlementTerms()).ifPresent(builder::setSettlementTerms);
		ofNullable(getUnderlier()).ifPresent(builder::setUnderlier);
		ofNullable(getPayoutQuantity()).ifPresent(builder::setPayoutQuantity);
		return builder;
	}

	public static EquityPayoutBuilder builder() {
		return new EquityPayoutBuilder();
	}

	public static  class EquityPayoutBuilder extends PayoutBaseBuilder implements GlobalKeyBuilder<EquityPayoutBuilder> {
	
		protected CalculationPeriodDates.CalculationPeriodDatesBuilder calculationPeriodDates;
		protected DividendReturnTerms.DividendReturnTermsBuilder dividendReturnTerms;
		protected MetaFields.MetaFieldsBuilder meta;
		protected PayerReceiver.PayerReceiverBuilder payerReceiver;
		protected PaymentDates.PaymentDatesBuilder paymentDates;
		protected BigDecimal performance;
		protected PriceReturnTerms.PriceReturnTermsBuilder priceReturnTerms;
		protected BigDecimal rateOfReturn;
		protected ReturnTypeEnum returnType;
		protected SettlementTerms.SettlementTermsBuilder settlementTerms;
		protected Underlier.UnderlierBuilder underlier;
	
		public EquityPayoutBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends EquityPayout> metaData() {
			return metaData;
		} 
	
		
		public CalculationPeriodDates.CalculationPeriodDatesBuilder getCalculationPeriodDates() {
			return calculationPeriodDates;
		}
		
		public CalculationPeriodDates.CalculationPeriodDatesBuilder getOrCreateCalculationPeriodDates() {
			if (calculationPeriodDates!=null) {
				return calculationPeriodDates;
			}
			else return calculationPeriodDates = new CalculationPeriodDates.CalculationPeriodDatesBuilder();
		}
		
		
		public DividendReturnTerms.DividendReturnTermsBuilder getDividendReturnTerms() {
			return dividendReturnTerms;
		}
		
		public DividendReturnTerms.DividendReturnTermsBuilder getOrCreateDividendReturnTerms() {
			if (dividendReturnTerms!=null) {
				return dividendReturnTerms;
			}
			else return dividendReturnTerms = new DividendReturnTerms.DividendReturnTermsBuilder();
		}
		
		
		public MetaFields.MetaFieldsBuilder getMeta() {
			return meta;
		}
		
		public MetaFields.MetaFieldsBuilder getOrCreateMeta() {
			if (meta!=null) {
				return meta;
			}
			else return meta = new MetaFields.MetaFieldsBuilder();
		}
		
		
		public PayerReceiver.PayerReceiverBuilder getPayerReceiver() {
			return payerReceiver;
		}
		
		public PayerReceiver.PayerReceiverBuilder getOrCreatePayerReceiver() {
			if (payerReceiver!=null) {
				return payerReceiver;
			}
			else return payerReceiver = new PayerReceiver.PayerReceiverBuilder();
		}
		
		
		public PaymentDates.PaymentDatesBuilder getPaymentDates() {
			return paymentDates;
		}
		
		public PaymentDates.PaymentDatesBuilder getOrCreatePaymentDates() {
			if (paymentDates!=null) {
				return paymentDates;
			}
			else return paymentDates = new PaymentDates.PaymentDatesBuilder();
		}
		
		
		public BigDecimal getPerformance() {
			return performance;
		}
		
		
		public PriceReturnTerms.PriceReturnTermsBuilder getPriceReturnTerms() {
			return priceReturnTerms;
		}
		
		public PriceReturnTerms.PriceReturnTermsBuilder getOrCreatePriceReturnTerms() {
			if (priceReturnTerms!=null) {
				return priceReturnTerms;
			}
			else return priceReturnTerms = new PriceReturnTerms.PriceReturnTermsBuilder();
		}
		
		
		public BigDecimal getRateOfReturn() {
			return rateOfReturn;
		}
		
		
		public ReturnTypeEnum getReturnType() {
			return returnType;
		}
		
		
		public SettlementTerms.SettlementTermsBuilder getSettlementTerms() {
			return settlementTerms;
		}
		
		public SettlementTerms.SettlementTermsBuilder getOrCreateSettlementTerms() {
			if (settlementTerms!=null) {
				return settlementTerms;
			}
			else return settlementTerms = new SettlementTerms.SettlementTermsBuilder();
		}
		
		
		public Underlier.UnderlierBuilder getUnderlier() {
			return underlier;
		}
		
		public Underlier.UnderlierBuilder getOrCreateUnderlier() {
			if (underlier!=null) {
				return underlier;
			}
			else return underlier = new Underlier.UnderlierBuilder();
		}
		
	
		public EquityPayoutBuilder setCalculationPeriodDates(CalculationPeriodDates calculationPeriodDates) {
			this.calculationPeriodDates = calculationPeriodDates.toBuilder();
			return this;
		}
		public EquityPayoutBuilder setCalculationPeriodDatesBuilder(CalculationPeriodDates.CalculationPeriodDatesBuilder calculationPeriodDates) {
			this.calculationPeriodDates = calculationPeriodDates;
			return this;
		}
		public EquityPayoutBuilder setDividendReturnTerms(DividendReturnTerms dividendReturnTerms) {
			this.dividendReturnTerms = dividendReturnTerms.toBuilder();
			return this;
		}
		public EquityPayoutBuilder setDividendReturnTermsBuilder(DividendReturnTerms.DividendReturnTermsBuilder dividendReturnTerms) {
			this.dividendReturnTerms = dividendReturnTerms;
			return this;
		}
		public EquityPayoutBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		public EquityPayoutBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		public EquityPayoutBuilder setPayerReceiver(PayerReceiver payerReceiver) {
			this.payerReceiver = payerReceiver.toBuilder();
			return this;
		}
		public EquityPayoutBuilder setPayerReceiverBuilder(PayerReceiver.PayerReceiverBuilder payerReceiver) {
			this.payerReceiver = payerReceiver;
			return this;
		}
		public EquityPayoutBuilder setPaymentDates(PaymentDates paymentDates) {
			this.paymentDates = paymentDates.toBuilder();
			return this;
		}
		public EquityPayoutBuilder setPaymentDatesBuilder(PaymentDates.PaymentDatesBuilder paymentDates) {
			this.paymentDates = paymentDates;
			return this;
		}
		public EquityPayoutBuilder setPerformance(BigDecimal performance) {
			this.performance = performance;
			return this;
		}
		public EquityPayoutBuilder setPriceReturnTerms(PriceReturnTerms priceReturnTerms) {
			this.priceReturnTerms = priceReturnTerms.toBuilder();
			return this;
		}
		public EquityPayoutBuilder setPriceReturnTermsBuilder(PriceReturnTerms.PriceReturnTermsBuilder priceReturnTerms) {
			this.priceReturnTerms = priceReturnTerms;
			return this;
		}
		public EquityPayoutBuilder setRateOfReturn(BigDecimal rateOfReturn) {
			this.rateOfReturn = rateOfReturn;
			return this;
		}
		public EquityPayoutBuilder setReturnType(ReturnTypeEnum returnType) {
			this.returnType = returnType;
			return this;
		}
		public EquityPayoutBuilder setSettlementTerms(SettlementTerms settlementTerms) {
			this.settlementTerms = settlementTerms.toBuilder();
			return this;
		}
		public EquityPayoutBuilder setSettlementTermsBuilder(SettlementTerms.SettlementTermsBuilder settlementTerms) {
			this.settlementTerms = settlementTerms;
			return this;
		}
		public EquityPayoutBuilder setUnderlier(Underlier underlier) {
			this.underlier = underlier.toBuilder();
			return this;
		}
		public EquityPayoutBuilder setUnderlierBuilder(Underlier.UnderlierBuilder underlier) {
			this.underlier = underlier;
			return this;
		}
		@Override public EquityPayoutBuilder setPayoutQuantity(ResolvablePayoutQuantity payoutQuantity) {
			this.payoutQuantity = payoutQuantity.toBuilder();
			return this;
		}
		@Override public EquityPayoutBuilder setPayoutQuantityBuilder(ResolvablePayoutQuantity.ResolvablePayoutQuantityBuilder payoutQuantity) {
			this.payoutQuantity = payoutQuantity;
			return this;
		}
		
		public EquityPayout build() {
			return new EquityPayout(this);
		}
	
		@Override
		public EquityPayoutBuilder prune() {
			super.prune();
			if (calculationPeriodDates!=null && !calculationPeriodDates.prune().hasData()) calculationPeriodDates = null;
			if (dividendReturnTerms!=null && !dividendReturnTerms.prune().hasData()) dividendReturnTerms = null;
			if (meta!=null && !meta.prune().hasData()) meta = null;
			if (payerReceiver!=null && !payerReceiver.prune().hasData()) payerReceiver = null;
			if (paymentDates!=null && !paymentDates.prune().hasData()) paymentDates = null;
			if (priceReturnTerms!=null && !priceReturnTerms.prune().hasData()) priceReturnTerms = null;
			if (settlementTerms!=null && !settlementTerms.prune().hasData()) settlementTerms = null;
			if (underlier!=null && !underlier.prune().hasData()) underlier = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (super.hasData()) return true;
			if (getCalculationPeriodDates()!=null && getCalculationPeriodDates().hasData()) return true;
			if (getDividendReturnTerms()!=null && getDividendReturnTerms().hasData()) return true;
			if (getMeta()!=null && getMeta().hasData()) return true;
			if (getPayerReceiver()!=null && getPayerReceiver().hasData()) return true;
			if (getPaymentDates()!=null && getPaymentDates().hasData()) return true;
			if (getPerformance()!=null) return true;
			if (getPriceReturnTerms()!=null && getPriceReturnTerms().hasData()) return true;
			if (getRateOfReturn()!=null) return true;
			if (getReturnType()!=null) return true;
			if (getSettlementTerms()!=null && getSettlementTerms().hasData()) return true;
			if (getUnderlier()!=null && getUnderlier().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
			super.process(path, processor);
		
			processor.processBasic(path.newSubPath("performance"), BigDecimal.class, performance, this);
			processor.processBasic(path.newSubPath("rateOfReturn"), BigDecimal.class, rateOfReturn, this);
			processor.processBasic(path.newSubPath("returnType"), ReturnTypeEnum.class, returnType, this);
			
			processRosetta(path.newSubPath("calculationPeriodDates"), processor, CalculationPeriodDates.class, calculationPeriodDates);
			processRosetta(path.newSubPath("dividendReturnTerms"), processor, DividendReturnTerms.class, dividendReturnTerms);
			processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
			processRosetta(path.newSubPath("payerReceiver"), processor, PayerReceiver.class, payerReceiver);
			processRosetta(path.newSubPath("paymentDates"), processor, PaymentDates.class, paymentDates);
			processRosetta(path.newSubPath("priceReturnTerms"), processor, PriceReturnTerms.class, priceReturnTerms);
			processRosetta(path.newSubPath("settlementTerms"), processor, SettlementTerms.class, settlementTerms);
			processRosetta(path.newSubPath("underlier"), processor, Underlier.class, underlier);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
			if (!super.equals(o)) return false;
		
			EquityPayoutBuilder _that = (EquityPayoutBuilder) o;
		
			if (calculationPeriodDates != null ? !calculationPeriodDates.equals(_that.calculationPeriodDates) : _that.calculationPeriodDates != null) return false;
			if (dividendReturnTerms != null ? !dividendReturnTerms.equals(_that.dividendReturnTerms) : _that.dividendReturnTerms != null) return false;
			if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
			if (payerReceiver != null ? !payerReceiver.equals(_that.payerReceiver) : _that.payerReceiver != null) return false;
			if (paymentDates != null ? !paymentDates.equals(_that.paymentDates) : _that.paymentDates != null) return false;
			if (performance != null ? !performance.equals(_that.performance) : _that.performance != null) return false;
			if (priceReturnTerms != null ? !priceReturnTerms.equals(_that.priceReturnTerms) : _that.priceReturnTerms != null) return false;
			if (rateOfReturn != null ? !rateOfReturn.equals(_that.rateOfReturn) : _that.rateOfReturn != null) return false;
			if (returnType != null ? !returnType.equals(_that.returnType) : _that.returnType != null) return false;
			if (settlementTerms != null ? !settlementTerms.equals(_that.settlementTerms) : _that.settlementTerms != null) return false;
			if (underlier != null ? !underlier.equals(_that.underlier) : _that.underlier != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = super.hashCode();
			_result = 31 * _result + (calculationPeriodDates != null ? calculationPeriodDates.hashCode() : 0);
			_result = 31 * _result + (dividendReturnTerms != null ? dividendReturnTerms.hashCode() : 0);
			_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
			_result = 31 * _result + (payerReceiver != null ? payerReceiver.hashCode() : 0);
			_result = 31 * _result + (paymentDates != null ? paymentDates.hashCode() : 0);
			_result = 31 * _result + (performance != null ? performance.hashCode() : 0);
			_result = 31 * _result + (priceReturnTerms != null ? priceReturnTerms.hashCode() : 0);
			_result = 31 * _result + (rateOfReturn != null ? rateOfReturn.hashCode() : 0);
			_result = 31 * _result + (returnType != null ? returnType.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (settlementTerms != null ? settlementTerms.hashCode() : 0);
			_result = 31 * _result + (underlier != null ? underlier.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "EquityPayoutBuilder {" +
				"calculationPeriodDates=" + this.calculationPeriodDates + ", " +
				"dividendReturnTerms=" + this.dividendReturnTerms + ", " +
				"meta=" + this.meta + ", " +
				"payerReceiver=" + this.payerReceiver + ", " +
				"paymentDates=" + this.paymentDates + ", " +
				"performance=" + this.performance + ", " +
				"priceReturnTerms=" + this.priceReturnTerms + ", " +
				"rateOfReturn=" + this.rateOfReturn + ", " +
				"returnType=" + this.returnType + ", " +
				"settlementTerms=" + this.settlementTerms + ", " +
				"underlier=" + this.underlier +
			'}' + " " + super.toString();
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		super.process(path, processor);
		
		processor.processBasic(path.newSubPath("performance"), BigDecimal.class, performance, this);
		processor.processBasic(path.newSubPath("rateOfReturn"), BigDecimal.class, rateOfReturn, this);
		processor.processBasic(path.newSubPath("returnType"), ReturnTypeEnum.class, returnType, this);
		
		processRosetta(path.newSubPath("calculationPeriodDates"), processor, CalculationPeriodDates.class, calculationPeriodDates);
		processRosetta(path.newSubPath("dividendReturnTerms"), processor, DividendReturnTerms.class, dividendReturnTerms);
		processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
		processRosetta(path.newSubPath("payerReceiver"), processor, PayerReceiver.class, payerReceiver);
		processRosetta(path.newSubPath("paymentDates"), processor, PaymentDates.class, paymentDates);
		processRosetta(path.newSubPath("priceReturnTerms"), processor, PriceReturnTerms.class, priceReturnTerms);
		processRosetta(path.newSubPath("settlementTerms"), processor, SettlementTerms.class, settlementTerms);
		processRosetta(path.newSubPath("underlier"), processor, Underlier.class, underlier);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
		if (!super.equals(o)) return false;
	
		EquityPayout _that = (EquityPayout) o;
	
		if (calculationPeriodDates != null ? !calculationPeriodDates.equals(_that.calculationPeriodDates) : _that.calculationPeriodDates != null) return false;
		if (dividendReturnTerms != null ? !dividendReturnTerms.equals(_that.dividendReturnTerms) : _that.dividendReturnTerms != null) return false;
		if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
		if (payerReceiver != null ? !payerReceiver.equals(_that.payerReceiver) : _that.payerReceiver != null) return false;
		if (paymentDates != null ? !paymentDates.equals(_that.paymentDates) : _that.paymentDates != null) return false;
		if (performance != null ? !performance.equals(_that.performance) : _that.performance != null) return false;
		if (priceReturnTerms != null ? !priceReturnTerms.equals(_that.priceReturnTerms) : _that.priceReturnTerms != null) return false;
		if (rateOfReturn != null ? !rateOfReturn.equals(_that.rateOfReturn) : _that.rateOfReturn != null) return false;
		if (returnType != null ? !returnType.equals(_that.returnType) : _that.returnType != null) return false;
		if (settlementTerms != null ? !settlementTerms.equals(_that.settlementTerms) : _that.settlementTerms != null) return false;
		if (underlier != null ? !underlier.equals(_that.underlier) : _that.underlier != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = super.hashCode();
		_result = 31 * _result + (calculationPeriodDates != null ? calculationPeriodDates.hashCode() : 0);
		_result = 31 * _result + (dividendReturnTerms != null ? dividendReturnTerms.hashCode() : 0);
		_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
		_result = 31 * _result + (payerReceiver != null ? payerReceiver.hashCode() : 0);
		_result = 31 * _result + (paymentDates != null ? paymentDates.hashCode() : 0);
		_result = 31 * _result + (performance != null ? performance.hashCode() : 0);
		_result = 31 * _result + (priceReturnTerms != null ? priceReturnTerms.hashCode() : 0);
		_result = 31 * _result + (rateOfReturn != null ? rateOfReturn.hashCode() : 0);
		_result = 31 * _result + (returnType != null ? returnType.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (settlementTerms != null ? settlementTerms.hashCode() : 0);
		_result = 31 * _result + (underlier != null ? underlier.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "EquityPayout {" +
			"calculationPeriodDates=" + this.calculationPeriodDates + ", " +
			"dividendReturnTerms=" + this.dividendReturnTerms + ", " +
			"meta=" + this.meta + ", " +
			"payerReceiver=" + this.payerReceiver + ", " +
			"paymentDates=" + this.paymentDates + ", " +
			"performance=" + this.performance + ", " +
			"priceReturnTerms=" + this.priceReturnTerms + ", " +
			"rateOfReturn=" + this.rateOfReturn + ", " +
			"returnType=" + this.returnType + ", " +
			"settlementTerms=" + this.settlementTerms + ", " +
			"underlier=" + this.underlier +
		'}' + " " + super.toString();
	}
}
