package org.isda.cdm;

import org.isda.cdm.CalculationPeriodDates;
import org.isda.cdm.InterpolationMethodEnum;
import org.isda.cdm.PaymentDates;
import org.isda.cdm.PriceReturnTerms;
import org.isda.cdm.ReturnTypeEnum;
import org.isda.cdm.SettlementTerms;


import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.EquitySwapMasterConfirmation2018Meta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * Specification for the General Terms and Relationship Supplement Elections as provided in the 2018 ISDA CDM Equity Confirmation for Security Equity Swap.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class EquitySwapMasterConfirmation2018 extends EquityMasterConfirmation {
	private final CalculationPeriodDates equityCalculationPeriod;
	private final PaymentDates equityCashSettlementDates;
	private final InterpolationMethodEnum linearInterpolationElection;
	private final PriceReturnTerms pricingMethodElection;
	private final SettlementTerms settlementTerms;
	private final ReturnTypeEnum typeOfSwapElection;
	private static EquitySwapMasterConfirmation2018Meta metaData = new EquitySwapMasterConfirmation2018Meta();
	
	EquitySwapMasterConfirmation2018(EquitySwapMasterConfirmation2018Builder builder) {
		super(builder);
		this.equityCalculationPeriod = ofNullable(builder.getEquityCalculationPeriod()).map(f->f.build()).orElse(null);
		this.equityCashSettlementDates = ofNullable(builder.getEquityCashSettlementDates()).map(f->f.build()).orElse(null);
		this.linearInterpolationElection = builder.getLinearInterpolationElection();
		this.pricingMethodElection = ofNullable(builder.getPricingMethodElection()).map(f->f.build()).orElse(null);
		this.settlementTerms = ofNullable(builder.getSettlementTerms()).map(f->f.build()).orElse(null);
		this.typeOfSwapElection = builder.getTypeOfSwapElection();
	}
	
	/**
	 * The parameters used to generate the &#39;Equity Valuation Dates&#39; schedule, including the Effective Date and Termination Date for the Swap.
	 */
	public final CalculationPeriodDates getEquityCalculationPeriod() {
		return equityCalculationPeriod;
	}
	
	/**
	 * The parameters used to generate the payment date schedule, relative to the equityCalculationPeriod. Part 1 Section 12, &#39;Definitions&#39;, of the 2018 ISDA CDM Equity Confirmation. Para 73: &#39;Equity Cash Settlement Date&#39; means each date falling one Settlement Cycle after an Equity Valuation Date; provided that if any such date is not a Settlement Currency Business Day, then such date shall be adjusted per Following Day Adjustment.
	 */
	public final PaymentDates getEquityCashSettlementDates() {
		return equityCashSettlementDates;
	}
	
	/**
	 * Part 1 Section 3, &#39;Floating Obligations&#39;, of the 2018 ISDA CDM Equity Confirmation. Para 3.3, &#39;Linear Interpolation&#39;: If the initial Calculation Period is not equal to the Designated Maturity, then the Linear Interpolation Election shall be as specified in the Relationship Supplement, unless otherwise specified in the Transaction Supplement.
	 */
	public final InterpolationMethodEnum getLinearInterpolationElection() {
		return linearInterpolationElection;
	}
	
	/**
	 * Part 1 Section 5, &#39;Pricing&#39;, of the 2018 ISDA CDM Equity Confirmation, Para 5.1, &#39;Determining Prices&#39;: Each price in relation to a Pricing Date shall be determined pursuant to the specified Pricing Method. The relevant price specified under the column header &#39;Price&#39; for a corresponding Pricing Date specified under the column header &#39;Pricing Date&#39; shall be determined using the corresponding method specified under the column header &#39;Pricing Method&#39;. Pricing Method for the final Equity Valuation Date shall be specified by the Final EVD Pricing Election and Pricing Method for any other Equity Valaution Date shall be &#39;Securities Close Pricing (Official)&#39;
	 */
	public final PriceReturnTerms getPricingMethodElection() {
		return pricingMethodElection;
	}
	
	/**
	 * Part 1 Section 8, &#39;Settlement&#39;, of the 2018 ISDA CDM Equity Confirmation for Security Equity Swap. All Settlements are in Cash.
	 */
	public final SettlementTerms getSettlementTerms() {
		return settlementTerms;
	}
	
	/**
	 * Part 1 Section 4, &#39;Dividend Obligations&#39;, of the 2018 ISDA CDM Equity Confirmation, Para 4.2 &#39;Dividend Returns&#39;: The Type Of Swap Election shall be &#39;Total Return&#39;, unless otherwise specified (as alternative &#39;Price Return&#39;) in the Transaction Supplement.
	 */
	public final ReturnTypeEnum getTypeOfSwapElection() {
		return typeOfSwapElection;
	}
	
	
	@Override
	public RosettaMetaData<? extends EquitySwapMasterConfirmation2018> metaData() {
		return metaData;
	} 
	
	public EquitySwapMasterConfirmation2018Builder toBuilder() {
		EquitySwapMasterConfirmation2018Builder builder = new EquitySwapMasterConfirmation2018Builder();
		ofNullable(getEquityCalculationPeriod()).ifPresent(builder::setEquityCalculationPeriod);
		ofNullable(getEquityCashSettlementDates()).ifPresent(builder::setEquityCashSettlementDates);
		ofNullable(getLinearInterpolationElection()).ifPresent(builder::setLinearInterpolationElection);
		ofNullable(getPricingMethodElection()).ifPresent(builder::setPricingMethodElection);
		ofNullable(getSettlementTerms()).ifPresent(builder::setSettlementTerms);
		ofNullable(getTypeOfSwapElection()).ifPresent(builder::setTypeOfSwapElection);
		return builder;
	}

	public static EquitySwapMasterConfirmation2018Builder builder() {
		return new EquitySwapMasterConfirmation2018Builder();
	}

	public static  class EquitySwapMasterConfirmation2018Builder extends EquityMasterConfirmationBuilder {
	
		protected CalculationPeriodDates.CalculationPeriodDatesBuilder equityCalculationPeriod;
		protected PaymentDates.PaymentDatesBuilder equityCashSettlementDates;
		protected InterpolationMethodEnum linearInterpolationElection;
		protected PriceReturnTerms.PriceReturnTermsBuilder pricingMethodElection;
		protected SettlementTerms.SettlementTermsBuilder settlementTerms;
		protected ReturnTypeEnum typeOfSwapElection;
	
		public EquitySwapMasterConfirmation2018Builder() {
		}
				
		@Override
		public RosettaMetaData<? extends EquitySwapMasterConfirmation2018> metaData() {
			return metaData;
		} 
	
		
		public CalculationPeriodDates.CalculationPeriodDatesBuilder getEquityCalculationPeriod() {
			return equityCalculationPeriod;
		}
		
		public CalculationPeriodDates.CalculationPeriodDatesBuilder getOrCreateEquityCalculationPeriod() {
			if (equityCalculationPeriod!=null) {
				return equityCalculationPeriod;
			}
			else return equityCalculationPeriod = new CalculationPeriodDates.CalculationPeriodDatesBuilder();
		}
		
		
		public PaymentDates.PaymentDatesBuilder getEquityCashSettlementDates() {
			return equityCashSettlementDates;
		}
		
		public PaymentDates.PaymentDatesBuilder getOrCreateEquityCashSettlementDates() {
			if (equityCashSettlementDates!=null) {
				return equityCashSettlementDates;
			}
			else return equityCashSettlementDates = new PaymentDates.PaymentDatesBuilder();
		}
		
		
		public InterpolationMethodEnum getLinearInterpolationElection() {
			return linearInterpolationElection;
		}
		
		
		public PriceReturnTerms.PriceReturnTermsBuilder getPricingMethodElection() {
			return pricingMethodElection;
		}
		
		public PriceReturnTerms.PriceReturnTermsBuilder getOrCreatePricingMethodElection() {
			if (pricingMethodElection!=null) {
				return pricingMethodElection;
			}
			else return pricingMethodElection = new PriceReturnTerms.PriceReturnTermsBuilder();
		}
		
		
		public SettlementTerms.SettlementTermsBuilder getSettlementTerms() {
			return settlementTerms;
		}
		
		public SettlementTerms.SettlementTermsBuilder getOrCreateSettlementTerms() {
			if (settlementTerms!=null) {
				return settlementTerms;
			}
			else return settlementTerms = new SettlementTerms.SettlementTermsBuilder();
		}
		
		
		public ReturnTypeEnum getTypeOfSwapElection() {
			return typeOfSwapElection;
		}
		
	
		public EquitySwapMasterConfirmation2018Builder setEquityCalculationPeriod(CalculationPeriodDates equityCalculationPeriod) {
			this.equityCalculationPeriod = equityCalculationPeriod.toBuilder();
			return this;
		}
		public EquitySwapMasterConfirmation2018Builder setEquityCalculationPeriodBuilder(CalculationPeriodDates.CalculationPeriodDatesBuilder equityCalculationPeriod) {
			this.equityCalculationPeriod = equityCalculationPeriod;
			return this;
		}
		public EquitySwapMasterConfirmation2018Builder setEquityCashSettlementDates(PaymentDates equityCashSettlementDates) {
			this.equityCashSettlementDates = equityCashSettlementDates.toBuilder();
			return this;
		}
		public EquitySwapMasterConfirmation2018Builder setEquityCashSettlementDatesBuilder(PaymentDates.PaymentDatesBuilder equityCashSettlementDates) {
			this.equityCashSettlementDates = equityCashSettlementDates;
			return this;
		}
		public EquitySwapMasterConfirmation2018Builder setLinearInterpolationElection(InterpolationMethodEnum linearInterpolationElection) {
			this.linearInterpolationElection = linearInterpolationElection;
			return this;
		}
		public EquitySwapMasterConfirmation2018Builder setPricingMethodElection(PriceReturnTerms pricingMethodElection) {
			this.pricingMethodElection = pricingMethodElection.toBuilder();
			return this;
		}
		public EquitySwapMasterConfirmation2018Builder setPricingMethodElectionBuilder(PriceReturnTerms.PriceReturnTermsBuilder pricingMethodElection) {
			this.pricingMethodElection = pricingMethodElection;
			return this;
		}
		public EquitySwapMasterConfirmation2018Builder setSettlementTerms(SettlementTerms settlementTerms) {
			this.settlementTerms = settlementTerms.toBuilder();
			return this;
		}
		public EquitySwapMasterConfirmation2018Builder setSettlementTermsBuilder(SettlementTerms.SettlementTermsBuilder settlementTerms) {
			this.settlementTerms = settlementTerms;
			return this;
		}
		public EquitySwapMasterConfirmation2018Builder setTypeOfSwapElection(ReturnTypeEnum typeOfSwapElection) {
			this.typeOfSwapElection = typeOfSwapElection;
			return this;
		}
		
		public EquitySwapMasterConfirmation2018 build() {
			return new EquitySwapMasterConfirmation2018(this);
		}
	
		@Override
		public EquitySwapMasterConfirmation2018Builder prune() {
			super.prune();
			if (equityCalculationPeriod!=null && !equityCalculationPeriod.prune().hasData()) equityCalculationPeriod = null;
			if (equityCashSettlementDates!=null && !equityCashSettlementDates.prune().hasData()) equityCashSettlementDates = null;
			if (pricingMethodElection!=null && !pricingMethodElection.prune().hasData()) pricingMethodElection = null;
			if (settlementTerms!=null && !settlementTerms.prune().hasData()) settlementTerms = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (super.hasData()) return true;
			if (getEquityCalculationPeriod()!=null && getEquityCalculationPeriod().hasData()) return true;
			if (getEquityCashSettlementDates()!=null && getEquityCashSettlementDates().hasData()) return true;
			if (getLinearInterpolationElection()!=null) return true;
			if (getPricingMethodElection()!=null && getPricingMethodElection().hasData()) return true;
			if (getSettlementTerms()!=null && getSettlementTerms().hasData()) return true;
			if (getTypeOfSwapElection()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
			super.process(path, processor);
		
			processor.processBasic(path.newSubPath("linearInterpolationElection"), InterpolationMethodEnum.class, linearInterpolationElection, this);
			processor.processBasic(path.newSubPath("typeOfSwapElection"), ReturnTypeEnum.class, typeOfSwapElection, this);
			
			processRosetta(path.newSubPath("equityCalculationPeriod"), processor, CalculationPeriodDates.class, equityCalculationPeriod);
			processRosetta(path.newSubPath("equityCashSettlementDates"), processor, PaymentDates.class, equityCashSettlementDates);
			processRosetta(path.newSubPath("pricingMethodElection"), processor, PriceReturnTerms.class, pricingMethodElection);
			processRosetta(path.newSubPath("settlementTerms"), processor, SettlementTerms.class, settlementTerms);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
			if (!super.equals(o)) return false;
		
			EquitySwapMasterConfirmation2018Builder _that = (EquitySwapMasterConfirmation2018Builder) o;
		
			if (equityCalculationPeriod != null ? !equityCalculationPeriod.equals(_that.equityCalculationPeriod) : _that.equityCalculationPeriod != null) return false;
			if (equityCashSettlementDates != null ? !equityCashSettlementDates.equals(_that.equityCashSettlementDates) : _that.equityCashSettlementDates != null) return false;
			if (linearInterpolationElection != null ? !linearInterpolationElection.equals(_that.linearInterpolationElection) : _that.linearInterpolationElection != null) return false;
			if (pricingMethodElection != null ? !pricingMethodElection.equals(_that.pricingMethodElection) : _that.pricingMethodElection != null) return false;
			if (settlementTerms != null ? !settlementTerms.equals(_that.settlementTerms) : _that.settlementTerms != null) return false;
			if (typeOfSwapElection != null ? !typeOfSwapElection.equals(_that.typeOfSwapElection) : _that.typeOfSwapElection != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = super.hashCode();
			_result = 31 * _result + (equityCalculationPeriod != null ? equityCalculationPeriod.hashCode() : 0);
			_result = 31 * _result + (equityCashSettlementDates != null ? equityCashSettlementDates.hashCode() : 0);
			_result = 31 * _result + (linearInterpolationElection != null ? linearInterpolationElection.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (pricingMethodElection != null ? pricingMethodElection.hashCode() : 0);
			_result = 31 * _result + (settlementTerms != null ? settlementTerms.hashCode() : 0);
			_result = 31 * _result + (typeOfSwapElection != null ? typeOfSwapElection.getClass().getName().hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "EquitySwapMasterConfirmation2018Builder {" +
				"equityCalculationPeriod=" + this.equityCalculationPeriod + ", " +
				"equityCashSettlementDates=" + this.equityCashSettlementDates + ", " +
				"linearInterpolationElection=" + this.linearInterpolationElection + ", " +
				"pricingMethodElection=" + this.pricingMethodElection + ", " +
				"settlementTerms=" + this.settlementTerms + ", " +
				"typeOfSwapElection=" + this.typeOfSwapElection +
			'}' + " " + super.toString();
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		super.process(path, processor);
		
		processor.processBasic(path.newSubPath("linearInterpolationElection"), InterpolationMethodEnum.class, linearInterpolationElection, this);
		processor.processBasic(path.newSubPath("typeOfSwapElection"), ReturnTypeEnum.class, typeOfSwapElection, this);
		
		processRosetta(path.newSubPath("equityCalculationPeriod"), processor, CalculationPeriodDates.class, equityCalculationPeriod);
		processRosetta(path.newSubPath("equityCashSettlementDates"), processor, PaymentDates.class, equityCashSettlementDates);
		processRosetta(path.newSubPath("pricingMethodElection"), processor, PriceReturnTerms.class, pricingMethodElection);
		processRosetta(path.newSubPath("settlementTerms"), processor, SettlementTerms.class, settlementTerms);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
		if (!super.equals(o)) return false;
	
		EquitySwapMasterConfirmation2018 _that = (EquitySwapMasterConfirmation2018) o;
	
		if (equityCalculationPeriod != null ? !equityCalculationPeriod.equals(_that.equityCalculationPeriod) : _that.equityCalculationPeriod != null) return false;
		if (equityCashSettlementDates != null ? !equityCashSettlementDates.equals(_that.equityCashSettlementDates) : _that.equityCashSettlementDates != null) return false;
		if (linearInterpolationElection != null ? !linearInterpolationElection.equals(_that.linearInterpolationElection) : _that.linearInterpolationElection != null) return false;
		if (pricingMethodElection != null ? !pricingMethodElection.equals(_that.pricingMethodElection) : _that.pricingMethodElection != null) return false;
		if (settlementTerms != null ? !settlementTerms.equals(_that.settlementTerms) : _that.settlementTerms != null) return false;
		if (typeOfSwapElection != null ? !typeOfSwapElection.equals(_that.typeOfSwapElection) : _that.typeOfSwapElection != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = super.hashCode();
		_result = 31 * _result + (equityCalculationPeriod != null ? equityCalculationPeriod.hashCode() : 0);
		_result = 31 * _result + (equityCashSettlementDates != null ? equityCashSettlementDates.hashCode() : 0);
		_result = 31 * _result + (linearInterpolationElection != null ? linearInterpolationElection.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (pricingMethodElection != null ? pricingMethodElection.hashCode() : 0);
		_result = 31 * _result + (settlementTerms != null ? settlementTerms.hashCode() : 0);
		_result = 31 * _result + (typeOfSwapElection != null ? typeOfSwapElection.getClass().getName().hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "EquitySwapMasterConfirmation2018 {" +
			"equityCalculationPeriod=" + this.equityCalculationPeriod + ", " +
			"equityCashSettlementDates=" + this.equityCashSettlementDates + ", " +
			"linearInterpolationElection=" + this.linearInterpolationElection + ", " +
			"pricingMethodElection=" + this.pricingMethodElection + ", " +
			"settlementTerms=" + this.settlementTerms + ", " +
			"typeOfSwapElection=" + this.typeOfSwapElection +
		'}' + " " + super.toString();
	}
}
