package org.isda.cdm;

import org.isda.cdm.AdjustableOrRelativeDate;
import org.isda.cdm.AdjustableOrRelativeDates;
import org.isda.cdm.BusinessCenterTime;
import org.isda.cdm.DeterminationMethodEnum;
import org.isda.cdm.TimeTypeEnum;


import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.EquityValuationMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;
		import java.math.BigDecimal;

/**
 * A class for defining how and when an equity option or equity swap is to be valued.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="EquityValuation", source="FpML_5_10")
public class EquityValuation extends Price {
	private final DeterminationMethodEnum determinationMethod;
	private final AdjustableOrRelativeDate valuationDate;
	private final AdjustableOrRelativeDates valuationDates;
	private final BusinessCenterTime valuationTime;
	private final TimeTypeEnum valuationTimeType;
	private static EquityValuationMeta metaData = new EquityValuationMeta();
	
	EquityValuation(EquityValuationBuilder builder) {
		super(builder);
		this.determinationMethod = builder.getDeterminationMethod();
		this.valuationDate = ofNullable(builder.getValuationDate()).map(f->f.build()).orElse(null);
		this.valuationDates = ofNullable(builder.getValuationDates()).map(f->f.build()).orElse(null);
		this.valuationTime = ofNullable(builder.getValuationTime()).map(f->f.build()).orElse(null);
		this.valuationTimeType = builder.getValuationTimeType();
	}
	
	/**
	 * Specifies the method according to which an amount or a date is determined.
	 */
	@RosettaSynonym(value="determinationMethod", source="FpML_5_10")
	public final DeterminationMethodEnum getDeterminationMethod() {
		return determinationMethod;
	}
	
	/**
	 * 2018 ISDA CDM Equity Confirmation for Security Equity Swap: Pricing Date
	 */
	@RosettaSynonym(value="valuationDate", source="FpML_5_10", path="valuationRules")
	public final AdjustableOrRelativeDate getValuationDate() {
		return valuationDate;
	}
	
	/**
	 * 2018 ISDA CDM Equity Confirmation for Security Equity Swap: Pricing Date
	 */
	@RosettaSynonym(value="valuationDates", source="FpML_5_10", path="valuationRules")
	public final AdjustableOrRelativeDates getValuationDates() {
		return valuationDates;
	}
	
	/**
	 * The specific time of day at which the calculation agent values the underlying. The SpecificTime is the only case when the valuationTime (time + business center location – e.g. 10:00:00 USNY) should be provided. You should be able to provide just the valuationTime without valuationTimeType, which infer that this is a specific time.
	 */
	@RosettaSynonym(value="valuationTime", source="FpML_5_10", path="valuationRules")
	public final BusinessCenterTime getValuationTime() {
		return valuationTime;
	}
	
	/**
	 * The time of day at which the calculation agent values the underlying, for example the official closing time of the exchange.
	 */
	@RosettaSynonym(value="valuationTimeType", source="FpML_5_10", path="valuationRules")
	public final TimeTypeEnum getValuationTimeType() {
		return valuationTimeType;
	}
	
	
	@Override
	public RosettaMetaData<? extends EquityValuation> metaData() {
		return metaData;
	} 
	
	public EquityValuationBuilder toBuilder() {
		EquityValuationBuilder builder = new EquityValuationBuilder();
		ofNullable(getDeterminationMethod()).ifPresent(builder::setDeterminationMethod);
		ofNullable(getValuationDate()).ifPresent(builder::setValuationDate);
		ofNullable(getValuationDates()).ifPresent(builder::setValuationDates);
		ofNullable(getValuationTime()).ifPresent(builder::setValuationTime);
		ofNullable(getValuationTimeType()).ifPresent(builder::setValuationTimeType);
		ofNullable(getAccruedInterest()).ifPresent(builder::setAccruedInterest);
		ofNullable(getCleanNetPrice()).ifPresent(builder::setCleanNetPrice);
		ofNullable(getGrossPrice()).ifPresent(builder::setGrossPrice);
		ofNullable(getNetPrice()).ifPresent(builder::setNetPrice);
		return builder;
	}

	public static EquityValuationBuilder builder() {
		return new EquityValuationBuilder();
	}

	public static  class EquityValuationBuilder extends PriceBuilder {
	
		protected DeterminationMethodEnum determinationMethod;
		protected AdjustableOrRelativeDate.AdjustableOrRelativeDateBuilder valuationDate;
		protected AdjustableOrRelativeDates.AdjustableOrRelativeDatesBuilder valuationDates;
		protected BusinessCenterTime.BusinessCenterTimeBuilder valuationTime;
		protected TimeTypeEnum valuationTimeType;
	
		public EquityValuationBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends EquityValuation> metaData() {
			return metaData;
		} 
	
		
		public DeterminationMethodEnum getDeterminationMethod() {
			return determinationMethod;
		}
		
		
		public AdjustableOrRelativeDate.AdjustableOrRelativeDateBuilder getValuationDate() {
			return valuationDate;
		}
		
		public AdjustableOrRelativeDate.AdjustableOrRelativeDateBuilder getOrCreateValuationDate() {
			if (valuationDate!=null) {
				return valuationDate;
			}
			else return valuationDate = new AdjustableOrRelativeDate.AdjustableOrRelativeDateBuilder();
		}
		
		
		public AdjustableOrRelativeDates.AdjustableOrRelativeDatesBuilder getValuationDates() {
			return valuationDates;
		}
		
		public AdjustableOrRelativeDates.AdjustableOrRelativeDatesBuilder getOrCreateValuationDates() {
			if (valuationDates!=null) {
				return valuationDates;
			}
			else return valuationDates = new AdjustableOrRelativeDates.AdjustableOrRelativeDatesBuilder();
		}
		
		
		public BusinessCenterTime.BusinessCenterTimeBuilder getValuationTime() {
			return valuationTime;
		}
		
		public BusinessCenterTime.BusinessCenterTimeBuilder getOrCreateValuationTime() {
			if (valuationTime!=null) {
				return valuationTime;
			}
			else return valuationTime = new BusinessCenterTime.BusinessCenterTimeBuilder();
		}
		
		
		public TimeTypeEnum getValuationTimeType() {
			return valuationTimeType;
		}
		
	
		public EquityValuationBuilder setDeterminationMethod(DeterminationMethodEnum determinationMethod) {
			this.determinationMethod = determinationMethod;
			return this;
		}
		public EquityValuationBuilder setValuationDate(AdjustableOrRelativeDate valuationDate) {
			this.valuationDate = valuationDate.toBuilder();
			return this;
		}
		public EquityValuationBuilder setValuationDateBuilder(AdjustableOrRelativeDate.AdjustableOrRelativeDateBuilder valuationDate) {
			this.valuationDate = valuationDate;
			return this;
		}
		public EquityValuationBuilder setValuationDates(AdjustableOrRelativeDates valuationDates) {
			this.valuationDates = valuationDates.toBuilder();
			return this;
		}
		public EquityValuationBuilder setValuationDatesBuilder(AdjustableOrRelativeDates.AdjustableOrRelativeDatesBuilder valuationDates) {
			this.valuationDates = valuationDates;
			return this;
		}
		public EquityValuationBuilder setValuationTime(BusinessCenterTime valuationTime) {
			this.valuationTime = valuationTime.toBuilder();
			return this;
		}
		public EquityValuationBuilder setValuationTimeBuilder(BusinessCenterTime.BusinessCenterTimeBuilder valuationTime) {
			this.valuationTime = valuationTime;
			return this;
		}
		public EquityValuationBuilder setValuationTimeType(TimeTypeEnum valuationTimeType) {
			this.valuationTimeType = valuationTimeType;
			return this;
		}
		@Override public EquityValuationBuilder setAccruedInterest(BigDecimal accruedInterest) {
			this.accruedInterest = accruedInterest;
			return this;
		}
		@Override public EquityValuationBuilder setCleanNetPrice(ActualPrice cleanNetPrice) {
			this.cleanNetPrice = cleanNetPrice.toBuilder();
			return this;
		}
		@Override public EquityValuationBuilder setCleanNetPriceBuilder(ActualPrice.ActualPriceBuilder cleanNetPrice) {
			this.cleanNetPrice = cleanNetPrice;
			return this;
		}
		@Override public EquityValuationBuilder setGrossPrice(ActualPrice grossPrice) {
			this.grossPrice = grossPrice.toBuilder();
			return this;
		}
		@Override public EquityValuationBuilder setGrossPriceBuilder(ActualPrice.ActualPriceBuilder grossPrice) {
			this.grossPrice = grossPrice;
			return this;
		}
		@Override public EquityValuationBuilder setNetPrice(ActualPrice netPrice) {
			this.netPrice = netPrice.toBuilder();
			return this;
		}
		@Override public EquityValuationBuilder setNetPriceBuilder(ActualPrice.ActualPriceBuilder netPrice) {
			this.netPrice = netPrice;
			return this;
		}
		
		public EquityValuation build() {
			return new EquityValuation(this);
		}
	
		@Override
		public EquityValuationBuilder prune() {
			super.prune();
			if (valuationDate!=null && !valuationDate.prune().hasData()) valuationDate = null;
			if (valuationDates!=null && !valuationDates.prune().hasData()) valuationDates = null;
			if (valuationTime!=null && !valuationTime.prune().hasData()) valuationTime = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (super.hasData()) return true;
			if (getDeterminationMethod()!=null) return true;
			if (getValuationDate()!=null && getValuationDate().hasData()) return true;
			if (getValuationDates()!=null && getValuationDates().hasData()) return true;
			if (getValuationTime()!=null && getValuationTime().hasData()) return true;
			if (getValuationTimeType()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
			super.process(path, processor);
		
			processor.processBasic(path.newSubPath("determinationMethod"), DeterminationMethodEnum.class, determinationMethod, this);
			processor.processBasic(path.newSubPath("valuationTimeType"), TimeTypeEnum.class, valuationTimeType, this);
			
			processRosetta(path.newSubPath("valuationDate"), processor, AdjustableOrRelativeDate.class, valuationDate);
			processRosetta(path.newSubPath("valuationDates"), processor, AdjustableOrRelativeDates.class, valuationDates);
			processRosetta(path.newSubPath("valuationTime"), processor, BusinessCenterTime.class, valuationTime);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
			if (!super.equals(o)) return false;
		
			EquityValuationBuilder _that = (EquityValuationBuilder) o;
		
			if (determinationMethod != null ? !determinationMethod.equals(_that.determinationMethod) : _that.determinationMethod != null) return false;
			if (valuationDate != null ? !valuationDate.equals(_that.valuationDate) : _that.valuationDate != null) return false;
			if (valuationDates != null ? !valuationDates.equals(_that.valuationDates) : _that.valuationDates != null) return false;
			if (valuationTime != null ? !valuationTime.equals(_that.valuationTime) : _that.valuationTime != null) return false;
			if (valuationTimeType != null ? !valuationTimeType.equals(_that.valuationTimeType) : _that.valuationTimeType != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = super.hashCode();
			_result = 31 * _result + (determinationMethod != null ? determinationMethod.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (valuationDate != null ? valuationDate.hashCode() : 0);
			_result = 31 * _result + (valuationDates != null ? valuationDates.hashCode() : 0);
			_result = 31 * _result + (valuationTime != null ? valuationTime.hashCode() : 0);
			_result = 31 * _result + (valuationTimeType != null ? valuationTimeType.getClass().getName().hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "EquityValuationBuilder {" +
				"determinationMethod=" + this.determinationMethod + ", " +
				"valuationDate=" + this.valuationDate + ", " +
				"valuationDates=" + this.valuationDates + ", " +
				"valuationTime=" + this.valuationTime + ", " +
				"valuationTimeType=" + this.valuationTimeType +
			'}' + " " + super.toString();
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		super.process(path, processor);
		
		processor.processBasic(path.newSubPath("determinationMethod"), DeterminationMethodEnum.class, determinationMethod, this);
		processor.processBasic(path.newSubPath("valuationTimeType"), TimeTypeEnum.class, valuationTimeType, this);
		
		processRosetta(path.newSubPath("valuationDate"), processor, AdjustableOrRelativeDate.class, valuationDate);
		processRosetta(path.newSubPath("valuationDates"), processor, AdjustableOrRelativeDates.class, valuationDates);
		processRosetta(path.newSubPath("valuationTime"), processor, BusinessCenterTime.class, valuationTime);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
		if (!super.equals(o)) return false;
	
		EquityValuation _that = (EquityValuation) o;
	
		if (determinationMethod != null ? !determinationMethod.equals(_that.determinationMethod) : _that.determinationMethod != null) return false;
		if (valuationDate != null ? !valuationDate.equals(_that.valuationDate) : _that.valuationDate != null) return false;
		if (valuationDates != null ? !valuationDates.equals(_that.valuationDates) : _that.valuationDates != null) return false;
		if (valuationTime != null ? !valuationTime.equals(_that.valuationTime) : _that.valuationTime != null) return false;
		if (valuationTimeType != null ? !valuationTimeType.equals(_that.valuationTimeType) : _that.valuationTimeType != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = super.hashCode();
		_result = 31 * _result + (determinationMethod != null ? determinationMethod.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (valuationDate != null ? valuationDate.hashCode() : 0);
		_result = 31 * _result + (valuationDates != null ? valuationDates.hashCode() : 0);
		_result = 31 * _result + (valuationTime != null ? valuationTime.hashCode() : 0);
		_result = 31 * _result + (valuationTimeType != null ? valuationTimeType.getClass().getName().hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "EquityValuation {" +
			"determinationMethod=" + this.determinationMethod + ", " +
			"valuationDate=" + this.valuationDate + ", " +
			"valuationDates=" + this.valuationDates + ", " +
			"valuationTime=" + this.valuationTime + ", " +
			"valuationTimeType=" + this.valuationTimeType +
		'}' + " " + super.toString();
	}
}
