package org.isda.cdm;

import org.isda.cdm.AdjustableOrRelativeDate;
import org.isda.cdm.AdjustableOrRelativeDates;
import org.isda.cdm.BusinessCenterTime;
import org.isda.cdm.ExerciseFee;
import org.isda.cdm.PartialExercise;
import org.isda.cdm.metafields.MetaFields;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.EuropeanExerciseMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class defining the exercise period for a European style option together with any rules governing the notional amount of the underlying which can be exercised on any given exercise date and any associated exercise fees.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="EuropeanExercise", source="FpML_5_10")
@RosettaSynonym(value="EuropeanExercise", source="CME_SubmissionIRS_1_0")
@RosettaSynonym(value="EuropeanExercise", source="DTCC_11_0")
@RosettaSynonym(value="EuropeanExercise", source="DTCC_9_0")
@RosettaSynonym(value="EuropeanExercise", source="CME_ClearedConfirm_1_17")
public class EuropeanExercise extends RosettaModelObject implements GlobalKey {
	private final BusinessCenterTime earliestExerciseTime;
	private final ExerciseFee exerciseFee;
	private final AdjustableOrRelativeDate expirationDate;
	private final BusinessCenterTime expirationTime;
	private final MetaFields meta;
	private final PartialExercise partialExercise;
	private final AdjustableOrRelativeDates relevantUnderlyingDate;
	private static EuropeanExerciseMeta metaData = new EuropeanExerciseMeta();
	
	EuropeanExercise(EuropeanExerciseBuilder builder) {
		this.earliestExerciseTime = ofNullable(builder.getEarliestExerciseTime()).map(f->f.build()).orElse(null);
		this.exerciseFee = ofNullable(builder.getExerciseFee()).map(f->f.build()).orElse(null);
		this.expirationDate = ofNullable(builder.getExpirationDate()).map(f->f.build()).orElse(null);
		this.expirationTime = ofNullable(builder.getExpirationTime()).map(f->f.build()).orElse(null);
		this.meta = ofNullable(builder.getMeta()).map(f->f.build()).orElse(null);
		this.partialExercise = ofNullable(builder.getPartialExercise()).map(f->f.build()).orElse(null);
		this.relevantUnderlyingDate = ofNullable(builder.getRelevantUnderlyingDate()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The earliest time at which notice of exercise can be given by the buyer to the seller (or seller&#39;s agent) on the expiration date.
	 */
	@RosettaSynonym(value="earliestExerciseTime", source="FpML_5_10")
	@RosettaSynonym(value="earliestExerciseTime", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="earliestExerciseTime", source="DTCC_11_0")
	@RosettaSynonym(value="earliestExerciseTime", source="DTCC_9_0")
	@RosettaSynonym(value="earliestExerciseTime", source="CME_ClearedConfirm_1_17")
	public final BusinessCenterTime getEarliestExerciseTime() {
		return earliestExerciseTime;
	}
	
	/**
	 * A fee to be paid on exercise. This could be represented as an amount or a rate and notional reference on which to apply the rate.
	 */
	@RosettaSynonym(value="exerciseFee", source="FpML_5_10")
	@RosettaSynonym(value="exerciseFee", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="exerciseFee", source="DTCC_11_0")
	@RosettaSynonym(value="exerciseFee", source="DTCC_9_0")
	@RosettaSynonym(value="exerciseFee", source="CME_ClearedConfirm_1_17")
	public final ExerciseFee getExerciseFee() {
		return exerciseFee;
	}
	
	/**
	 * The last day within an exercise period for an American style option. For a European style option it is the only day within the exercise period.
	 */
	@RosettaSynonym(value="expirationDate", source="FpML_5_10")
	@RosettaSynonym(value="expirationDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="expirationDate", source="DTCC_11_0")
	@RosettaSynonym(value="expirationDate", source="DTCC_9_0")
	@RosettaSynonym(value="expirationDate", source="CME_ClearedConfirm_1_17")
	public final AdjustableOrRelativeDate getExpirationDate() {
		return expirationDate;
	}
	
	/**
	 * The latest time for exercise on expirationDate.
	 */
	@RosettaSynonym(value="expirationTime", source="FpML_5_10")
	@RosettaSynonym(value="expirationTime", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="expirationTime", source="DTCC_11_0")
	@RosettaSynonym(value="expirationTime", source="DTCC_9_0")
	@RosettaSynonym(value="expirationTime", source="CME_ClearedConfirm_1_17")
	public final BusinessCenterTime getExpirationTime() {
		return expirationTime;
	}
	
	public final MetaFields getMeta() {
		return meta;
	}
	
	/**
	 * As defined in the 2000 ISDA Definitions, Section 12.3. Partial Exercise, the buyer of the option has the right to exercise all or less than all the notional amount of the underlying swap on the expiration date, but may not exercise less than the minimum notional amount, and if an integral multiple amount is specified, the notional amount exercised must be equal to, or be an integral multiple of, the integral multiple amount.
	 */
	@RosettaSynonym(value="partialExercise", source="FpML_5_10")
	@RosettaSynonym(value="partialExercise", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="partialExercise", source="DTCC_11_0")
	@RosettaSynonym(value="partialExercise", source="DTCC_9_0")
	@RosettaSynonym(value="partialExercise", source="CME_ClearedConfirm_1_17")
	public final PartialExercise getPartialExercise() {
		return partialExercise;
	}
	
	/**
	 * The date on the underlying set by the exercise of an option. What this date is depends on the option (e.g. in a swaption it is the swap effective date, in an extendible/cancelable provision it is the swap termination date).
	 */
	@RosettaSynonym(value="relevantUnderlyingDate", source="FpML_5_10")
	@RosettaSynonym(value="relevantUnderlyingDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="relevantUnderlyingDate", source="DTCC_11_0")
	@RosettaSynonym(value="relevantUnderlyingDate", source="DTCC_9_0")
	@RosettaSynonym(value="relevantUnderlyingDate", source="CME_ClearedConfirm_1_17")
	public final AdjustableOrRelativeDates getRelevantUnderlyingDate() {
		return relevantUnderlyingDate;
	}
	
	
	@Override
	public RosettaMetaData<? extends EuropeanExercise> metaData() {
		return metaData;
	} 
	
	public EuropeanExerciseBuilder toBuilder() {
		EuropeanExerciseBuilder builder = new EuropeanExerciseBuilder();
		ofNullable(getEarliestExerciseTime()).ifPresent(builder::setEarliestExerciseTime);
		ofNullable(getExerciseFee()).ifPresent(builder::setExerciseFee);
		ofNullable(getExpirationDate()).ifPresent(builder::setExpirationDate);
		ofNullable(getExpirationTime()).ifPresent(builder::setExpirationTime);
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		ofNullable(getPartialExercise()).ifPresent(builder::setPartialExercise);
		ofNullable(getRelevantUnderlyingDate()).ifPresent(builder::setRelevantUnderlyingDate);
		return builder;
	}

	public static EuropeanExerciseBuilder builder() {
		return new EuropeanExerciseBuilder();
	}

	public static  class EuropeanExerciseBuilder extends RosettaModelObjectBuilder implements GlobalKeyBuilder<EuropeanExerciseBuilder> {
	
		protected BusinessCenterTime.BusinessCenterTimeBuilder earliestExerciseTime;
		protected ExerciseFee.ExerciseFeeBuilder exerciseFee;
		protected AdjustableOrRelativeDate.AdjustableOrRelativeDateBuilder expirationDate;
		protected BusinessCenterTime.BusinessCenterTimeBuilder expirationTime;
		protected MetaFields.MetaFieldsBuilder meta;
		protected PartialExercise.PartialExerciseBuilder partialExercise;
		protected AdjustableOrRelativeDates.AdjustableOrRelativeDatesBuilder relevantUnderlyingDate;
	
		public EuropeanExerciseBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends EuropeanExercise> metaData() {
			return metaData;
		} 
	
		
		public BusinessCenterTime.BusinessCenterTimeBuilder getEarliestExerciseTime() {
			return earliestExerciseTime;
		}
		
		public BusinessCenterTime.BusinessCenterTimeBuilder getOrCreateEarliestExerciseTime() {
			if (earliestExerciseTime!=null) {
				return earliestExerciseTime;
			}
			else return earliestExerciseTime = new BusinessCenterTime.BusinessCenterTimeBuilder();
		}
		
		
		public ExerciseFee.ExerciseFeeBuilder getExerciseFee() {
			return exerciseFee;
		}
		
		public ExerciseFee.ExerciseFeeBuilder getOrCreateExerciseFee() {
			if (exerciseFee!=null) {
				return exerciseFee;
			}
			else return exerciseFee = new ExerciseFee.ExerciseFeeBuilder();
		}
		
		
		public AdjustableOrRelativeDate.AdjustableOrRelativeDateBuilder getExpirationDate() {
			return expirationDate;
		}
		
		public AdjustableOrRelativeDate.AdjustableOrRelativeDateBuilder getOrCreateExpirationDate() {
			if (expirationDate!=null) {
				return expirationDate;
			}
			else return expirationDate = new AdjustableOrRelativeDate.AdjustableOrRelativeDateBuilder();
		}
		
		
		public BusinessCenterTime.BusinessCenterTimeBuilder getExpirationTime() {
			return expirationTime;
		}
		
		public BusinessCenterTime.BusinessCenterTimeBuilder getOrCreateExpirationTime() {
			if (expirationTime!=null) {
				return expirationTime;
			}
			else return expirationTime = new BusinessCenterTime.BusinessCenterTimeBuilder();
		}
		
		
		public MetaFields.MetaFieldsBuilder getMeta() {
			return meta;
		}
		
		public MetaFields.MetaFieldsBuilder getOrCreateMeta() {
			if (meta!=null) {
				return meta;
			}
			else return meta = new MetaFields.MetaFieldsBuilder();
		}
		
		
		public PartialExercise.PartialExerciseBuilder getPartialExercise() {
			return partialExercise;
		}
		
		public PartialExercise.PartialExerciseBuilder getOrCreatePartialExercise() {
			if (partialExercise!=null) {
				return partialExercise;
			}
			else return partialExercise = new PartialExercise.PartialExerciseBuilder();
		}
		
		
		public AdjustableOrRelativeDates.AdjustableOrRelativeDatesBuilder getRelevantUnderlyingDate() {
			return relevantUnderlyingDate;
		}
		
		public AdjustableOrRelativeDates.AdjustableOrRelativeDatesBuilder getOrCreateRelevantUnderlyingDate() {
			if (relevantUnderlyingDate!=null) {
				return relevantUnderlyingDate;
			}
			else return relevantUnderlyingDate = new AdjustableOrRelativeDates.AdjustableOrRelativeDatesBuilder();
		}
		
	
		public EuropeanExerciseBuilder setEarliestExerciseTime(BusinessCenterTime earliestExerciseTime) {
			this.earliestExerciseTime = earliestExerciseTime.toBuilder();
			return this;
		}
		public EuropeanExerciseBuilder setEarliestExerciseTimeBuilder(BusinessCenterTime.BusinessCenterTimeBuilder earliestExerciseTime) {
			this.earliestExerciseTime = earliestExerciseTime;
			return this;
		}
		public EuropeanExerciseBuilder setExerciseFee(ExerciseFee exerciseFee) {
			this.exerciseFee = exerciseFee.toBuilder();
			return this;
		}
		public EuropeanExerciseBuilder setExerciseFeeBuilder(ExerciseFee.ExerciseFeeBuilder exerciseFee) {
			this.exerciseFee = exerciseFee;
			return this;
		}
		public EuropeanExerciseBuilder setExpirationDate(AdjustableOrRelativeDate expirationDate) {
			this.expirationDate = expirationDate.toBuilder();
			return this;
		}
		public EuropeanExerciseBuilder setExpirationDateBuilder(AdjustableOrRelativeDate.AdjustableOrRelativeDateBuilder expirationDate) {
			this.expirationDate = expirationDate;
			return this;
		}
		public EuropeanExerciseBuilder setExpirationTime(BusinessCenterTime expirationTime) {
			this.expirationTime = expirationTime.toBuilder();
			return this;
		}
		public EuropeanExerciseBuilder setExpirationTimeBuilder(BusinessCenterTime.BusinessCenterTimeBuilder expirationTime) {
			this.expirationTime = expirationTime;
			return this;
		}
		public EuropeanExerciseBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		public EuropeanExerciseBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		public EuropeanExerciseBuilder setPartialExercise(PartialExercise partialExercise) {
			this.partialExercise = partialExercise.toBuilder();
			return this;
		}
		public EuropeanExerciseBuilder setPartialExerciseBuilder(PartialExercise.PartialExerciseBuilder partialExercise) {
			this.partialExercise = partialExercise;
			return this;
		}
		public EuropeanExerciseBuilder setRelevantUnderlyingDate(AdjustableOrRelativeDates relevantUnderlyingDate) {
			this.relevantUnderlyingDate = relevantUnderlyingDate.toBuilder();
			return this;
		}
		public EuropeanExerciseBuilder setRelevantUnderlyingDateBuilder(AdjustableOrRelativeDates.AdjustableOrRelativeDatesBuilder relevantUnderlyingDate) {
			this.relevantUnderlyingDate = relevantUnderlyingDate;
			return this;
		}
		
		public EuropeanExercise build() {
			return new EuropeanExercise(this);
		}
	
		@Override
		public EuropeanExerciseBuilder prune() {
			if (earliestExerciseTime!=null && !earliestExerciseTime.prune().hasData()) earliestExerciseTime = null;
			if (exerciseFee!=null && !exerciseFee.prune().hasData()) exerciseFee = null;
			if (expirationDate!=null && !expirationDate.prune().hasData()) expirationDate = null;
			if (expirationTime!=null && !expirationTime.prune().hasData()) expirationTime = null;
			if (meta!=null && !meta.prune().hasData()) meta = null;
			if (partialExercise!=null && !partialExercise.prune().hasData()) partialExercise = null;
			if (relevantUnderlyingDate!=null && !relevantUnderlyingDate.prune().hasData()) relevantUnderlyingDate = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getEarliestExerciseTime()!=null && getEarliestExerciseTime().hasData()) return true;
			if (getExerciseFee()!=null && getExerciseFee().hasData()) return true;
			if (getExpirationDate()!=null && getExpirationDate().hasData()) return true;
			if (getExpirationTime()!=null && getExpirationTime().hasData()) return true;
			if (getMeta()!=null && getMeta().hasData()) return true;
			if (getPartialExercise()!=null && getPartialExercise().hasData()) return true;
			if (getRelevantUnderlyingDate()!=null && getRelevantUnderlyingDate().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("earliestExerciseTime"), processor, BusinessCenterTime.class, earliestExerciseTime);
			processRosetta(path.newSubPath("exerciseFee"), processor, ExerciseFee.class, exerciseFee);
			processRosetta(path.newSubPath("expirationDate"), processor, AdjustableOrRelativeDate.class, expirationDate);
			processRosetta(path.newSubPath("expirationTime"), processor, BusinessCenterTime.class, expirationTime);
			processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
			processRosetta(path.newSubPath("partialExercise"), processor, PartialExercise.class, partialExercise);
			processRosetta(path.newSubPath("relevantUnderlyingDate"), processor, AdjustableOrRelativeDates.class, relevantUnderlyingDate);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			EuropeanExerciseBuilder _that = (EuropeanExerciseBuilder) o;
		
			if (earliestExerciseTime != null ? !earliestExerciseTime.equals(_that.earliestExerciseTime) : _that.earliestExerciseTime != null) return false;
			if (exerciseFee != null ? !exerciseFee.equals(_that.exerciseFee) : _that.exerciseFee != null) return false;
			if (expirationDate != null ? !expirationDate.equals(_that.expirationDate) : _that.expirationDate != null) return false;
			if (expirationTime != null ? !expirationTime.equals(_that.expirationTime) : _that.expirationTime != null) return false;
			if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
			if (partialExercise != null ? !partialExercise.equals(_that.partialExercise) : _that.partialExercise != null) return false;
			if (relevantUnderlyingDate != null ? !relevantUnderlyingDate.equals(_that.relevantUnderlyingDate) : _that.relevantUnderlyingDate != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (earliestExerciseTime != null ? earliestExerciseTime.hashCode() : 0);
			_result = 31 * _result + (exerciseFee != null ? exerciseFee.hashCode() : 0);
			_result = 31 * _result + (expirationDate != null ? expirationDate.hashCode() : 0);
			_result = 31 * _result + (expirationTime != null ? expirationTime.hashCode() : 0);
			_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
			_result = 31 * _result + (partialExercise != null ? partialExercise.hashCode() : 0);
			_result = 31 * _result + (relevantUnderlyingDate != null ? relevantUnderlyingDate.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "EuropeanExerciseBuilder {" +
				"earliestExerciseTime=" + this.earliestExerciseTime + ", " +
				"exerciseFee=" + this.exerciseFee + ", " +
				"expirationDate=" + this.expirationDate + ", " +
				"expirationTime=" + this.expirationTime + ", " +
				"meta=" + this.meta + ", " +
				"partialExercise=" + this.partialExercise + ", " +
				"relevantUnderlyingDate=" + this.relevantUnderlyingDate +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("earliestExerciseTime"), processor, BusinessCenterTime.class, earliestExerciseTime);
		processRosetta(path.newSubPath("exerciseFee"), processor, ExerciseFee.class, exerciseFee);
		processRosetta(path.newSubPath("expirationDate"), processor, AdjustableOrRelativeDate.class, expirationDate);
		processRosetta(path.newSubPath("expirationTime"), processor, BusinessCenterTime.class, expirationTime);
		processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
		processRosetta(path.newSubPath("partialExercise"), processor, PartialExercise.class, partialExercise);
		processRosetta(path.newSubPath("relevantUnderlyingDate"), processor, AdjustableOrRelativeDates.class, relevantUnderlyingDate);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		EuropeanExercise _that = (EuropeanExercise) o;
	
		if (earliestExerciseTime != null ? !earliestExerciseTime.equals(_that.earliestExerciseTime) : _that.earliestExerciseTime != null) return false;
		if (exerciseFee != null ? !exerciseFee.equals(_that.exerciseFee) : _that.exerciseFee != null) return false;
		if (expirationDate != null ? !expirationDate.equals(_that.expirationDate) : _that.expirationDate != null) return false;
		if (expirationTime != null ? !expirationTime.equals(_that.expirationTime) : _that.expirationTime != null) return false;
		if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
		if (partialExercise != null ? !partialExercise.equals(_that.partialExercise) : _that.partialExercise != null) return false;
		if (relevantUnderlyingDate != null ? !relevantUnderlyingDate.equals(_that.relevantUnderlyingDate) : _that.relevantUnderlyingDate != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (earliestExerciseTime != null ? earliestExerciseTime.hashCode() : 0);
		_result = 31 * _result + (exerciseFee != null ? exerciseFee.hashCode() : 0);
		_result = 31 * _result + (expirationDate != null ? expirationDate.hashCode() : 0);
		_result = 31 * _result + (expirationTime != null ? expirationTime.hashCode() : 0);
		_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
		_result = 31 * _result + (partialExercise != null ? partialExercise.hashCode() : 0);
		_result = 31 * _result + (relevantUnderlyingDate != null ? relevantUnderlyingDate.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "EuropeanExercise {" +
			"earliestExerciseTime=" + this.earliestExerciseTime + ", " +
			"exerciseFee=" + this.exerciseFee + ", " +
			"expirationDate=" + this.expirationDate + ", " +
			"expirationTime=" + this.expirationTime + ", " +
			"meta=" + this.meta + ", " +
			"partialExercise=" + this.partialExercise + ", " +
			"relevantUnderlyingDate=" + this.relevantUnderlyingDate +
		'}';
	}
}
