package org.isda.cdm;

import com.rosetta.model.lib.records.Date;
import java.lang.String;
import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.Account;
import org.isda.cdm.ActionEnum;
import org.isda.cdm.EventEffect;
import org.isda.cdm.EventTimestamp;
import org.isda.cdm.EventWorkflow;
import org.isda.cdm.Identifier;
import org.isda.cdm.IntentEnum;
import org.isda.cdm.Lineage;
import org.isda.cdm.MessageInformation;
import org.isda.cdm.Party;
import org.isda.cdm.PrimitiveEvent;
import org.isda.cdm.metafields.MetaFields;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.EventMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaQualified;
import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;
		import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import com.rosetta.model.lib.records.Date;

/**
 * A class to specify the lifecycle event. The associated rosettaKey denotes the ability to associate a hash value to the Event instantiation for the purpose of model cross-referencing, in support of functionality such as the event effect and the lineage.
 * 
 * @version 2.5.4
 */
@RosettaClass
@RosettaQualified(attribute="eventQualifier",qualifiedClass=Event.class)
public class Event extends RosettaModelObject implements GlobalKey {
	private final List<Account> account;
	private final ActionEnum action;
	private final Date effectiveDate;
	private final Date eventDate;
	private final EventEffect eventEffect;
	private final List<Identifier> eventIdentifier;
	private final String eventQualifier;
	private final EventWorkflow eventWorkflow;
	private final String functionCall;
	private final IntentEnum intent;
	private final Lineage lineage;
	private final MessageInformation messageInformation;
	private final MetaFields meta;
	private final List<Party> party;
	private final PrimitiveEvent primitive;
	private final List<EventTimestamp> timestamp;
	private static EventMeta metaData = new EventMeta();
	
	Event(EventBuilder builder) {
		this.account = ofNullable(builder.getAccount()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.action = builder.getAction();
		this.effectiveDate = builder.getEffectiveDate();
		this.eventDate = builder.getEventDate();
		this.eventEffect = ofNullable(builder.getEventEffect()).map(f->f.build()).orElse(null);
		this.eventIdentifier = ofNullable(builder.getEventIdentifier()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.eventQualifier = builder.getEventQualifier();
		this.eventWorkflow = ofNullable(builder.getEventWorkflow()).map(f->f.build()).orElse(null);
		this.functionCall = builder.getFunctionCall();
		this.intent = builder.getIntent();
		this.lineage = ofNullable(builder.getLineage()).map(f->f.build()).orElse(null);
		this.messageInformation = ofNullable(builder.getMessageInformation()).map(f->f.build()).orElse(null);
		this.meta = ofNullable(builder.getMeta()).map(f->f.build()).orElse(null);
		this.party = ofNullable(builder.getParty()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.primitive = ofNullable(builder.getPrimitive()).map(f->f.build()).orElse(null);
		this.timestamp = ofNullable(builder.getTimestamp()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
	}
	
	/**
	 * Optional account information that could be associated to the event.
	 */
	@RosettaSynonym(value="account", source="FpML_5_10")
	@RosettaSynonym(value="account", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="account", source="DTCC_11_0")
	@RosettaSynonym(value="account", source="DTCC_9_0")
	@RosettaSynonym(value="account", source="CME_ClearedConfirm_1_17", path="clearingConfirmed")
	public final List<Account> getAccount() {
		return account;
	}
	
	/**
	 * Specifies whether the event is a new, a correction or a cancellation.
	 */
	@RosettaSynonym(value="", source="Rosetta_Workbench")
	@RosettaSynonym(value="", source="FpML_5_10")
	@RosettaSynonym(value="Activity", source="DTCC_11_0", path="Header.OTC_RM.Manifest.TradeMsg")
	@RosettaSynonym(value="Activity", source="DTCC_9_0", path="Header.OTC_RM.Manifest.TradeMsg")
	@RosettaSynonym(value="TransTyp", source="CME_SubmissionIRS_1_0", path="TrdCaptRpt")
	public final ActionEnum getAction() {
		return action;
	}
	
	/**
	 * The date on which the event contractually takes effect, when different from the event date.
	 */
	@RosettaSynonym(value="effectiveDate", source="Rosetta_Workbench")
	public final Date getEffectiveDate() {
		return effectiveDate;
	}
	
	/**
	 * The date on which the event is taking place. This is the equivalent of the trade date in the case of an execution or a contract.
	 */
	@RosettaSynonym(value="eventDate", source="Rosetta_Workbench")
	@RosettaSynonym(value="clearedDate", source="CME_ClearedConfirm_1_17", path="clearingConfirmed.trade.tradeHeader", maps=2)
	public final Date getEventDate() {
		return eventDate;
	}
	
	/**
	 * The set of effects associated with the lifecycle event, i.e. generated cashflows, contracts (from, say, novation events), listed products (from, say, a bond option exercise event) values and more. Those are represented through a set of rosettaKey references. This attribute is optional in order to provide implementers with the ability not to make use of this feature.
	 */
	public final EventEffect getEventEffect() {
		return eventEffect;
	}
	
	/**
	 * The identifier(s) that uniquely identify a lifecycle event. The unbounded cardinality is meant to provide the ability to associate identifiers that are issued by distinct parties. As an example, each of the parties to the event may choose to associate their own identifiers to the event.
	 */
	@RosettaSynonym(value="eventIdentifier", source="Rosetta_Workbench")
	@RosettaSynonym(value="Submitter", source="DTCC_11_0", path="Header.OTC_RM.Manifest.TradeMsg")
	@RosettaSynonym(value="Submitter", source="DTCC_9_0", path="Header.OTC_RM.Manifest.TradeMsg")
	@RosettaSynonym(value="ContraTradeId", source="DTCC_11_0", path="Header.OTC_RM.Manifest.TradeMsg")
	@RosettaSynonym(value="ContraTradeId", source="DTCC_9_0", path="Header.OTC_RM.Manifest.TradeMsg")
	@RosettaSynonym(value="YourTradeId", source="DTCC_11_0", path="Header.OTC_RM.Manifest.TradeMsg")
	@RosettaSynonym(value="YourTradeId", source="DTCC_9_0", path="Header.OTC_RM.Manifest.TradeMsg")
	@RosettaSynonym(value="tradeIdentifyingItems", source="DTCC_11_0", path="Header.OTC_RM.Manifest.TradeMsg")
	@RosettaSynonym(value="tradeIdentifyingItems", source="DTCC_9_0", path="Header.OTC_RM.Manifest.TradeMsg")
	public final List<Identifier> getEventIdentifier() {
		return eventIdentifier;
	}
	
	/**
	 * The CDM event qualifier, which corresponds to the outcome of the isEvent qualification logic which qualifies the lifecycle event as a function of its features (e.g. PartialTermination, ClearingSubmission, Novation, ...).
	 */
	public final String getEventQualifier() {
		return eventQualifier;
	}
	
	/**
	 * The event workflow information, i.e. the workflow status, the associated comment and the partyCustomisedWorkflow which purpose is to provide the ability to associate custom workflow information to the CDM.
	 */
	public final EventWorkflow getEventWorkflow() {
		return eventWorkflow;
	}
	
	/**
	 * This is placeholder concept for a function call into a calculation that will return an outcome. This concept needs to be further firmed out.
	 */
	@RosettaSynonym(value="functionCall", source="Rosetta_Workbench")
	public final String getFunctionCall() {
		return functionCall;
	}
	
	/**
	 * The intent attribute is meant to be specified when the event qualification cannot be programmatically inferred from the event features. As a result it is only associated with those primitives that can give way to such ambiguity, the quantityChange being one of those. An example of such is a reduction in the trade notional, which could be interpreted as either a trade correction (unless a maximum period of time post-event is specified as part of the qualification), a partial termination or a portfolio rebalancing in the case of an equity swap. On the other hand, an event such as the exercise is not expected to have an associated intent as there should not be ambiguity.
	 */
	@RosettaSynonym(value="intent", source="Rosetta_Workbench")
	public final IntentEnum getIntent() {
		return intent;
	}
	
	/**
	 * The lineage attribute provides a linkage among lifecycle events through the rosettaKey hash value. One example is when a given lifecycle event is being corrected or cancelled. In such case, each subsequent event will have lineage into the prior version of that event. The second broad use case is when an event has a dependency upon either another event (e.g. the regular payment associated with a fix/float swap will have a lineage into the reset event, which will in turn have a lineage into the observation event for the floating rate and the contract) or a contract (e.g. the exercise of an option has a lineage into that option).
	 */
	@RosettaSynonym(value="lineage", source="Rosetta_Workbench")
	public final Lineage getLineage() {
		return lineage;
	}
	
	@RosettaSynonym(value="FpML", source="DTCC_11_0", path="Body.OTC_Matching.Trade")
	@RosettaSynonym(value="FpML", source="DTCC_9_0", path="Body.OTC_Matching.Trade")
	@RosettaSynonym(value="FpML", source="DTCC_11_0", path="Body.OTC_Matching.Payment")
	@RosettaSynonym(value="FpML", source="DTCC_9_0", path="Body.OTC_Matching.Payment")
	@RosettaSynonym(value="RouteInfo", source="DTCC_11_0", path="Header.OTC_RM.Delivery")
	@RosettaSynonym(value="RouteInfo", source="DTCC_9_0", path="Header.OTC_RM.Delivery")
	@RosettaSynonym(value="Manifest", source="DTCC_11_0", path="Header.OTC_RM")
	@RosettaSynonym(value="Manifest", source="DTCC_9_0", path="Header.OTC_RM")
	@RosettaSynonym(value="header", source="CME_ClearedConfirm_1_17", path="clearingConfirmed")
	public final MessageInformation getMessageInformation() {
		return messageInformation;
	}
	
	public final MetaFields getMeta() {
		return meta;
	}
	
	/**
	 * The specification of the event parties. This attribute is optional, as not applicable to certain events (e.g. most of the observations).
	 */
	@RosettaSynonym(value="party", source="FpML_5_10")
	@RosettaSynonym(value="party", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="party", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="party", source="DTCC_11_0", path="Body.OTC_Matching.Trade.FpML")
	@RosettaSynonym(value="party", source="DTCC_9_0", path="Body.OTC_Matching.Trade.FpML")
	@RosettaSynonym(value="party", source="DTCC_11_0", path="Body.OTC_Matching.Payment.FpML")
	@RosettaSynonym(value="party", source="DTCC_9_0", path="Body.OTC_Matching.Payment.FpML")
	public final List<Party> getParty() {
		return party;
	}
	
	/**
	 * The elemental component(s) that specify the lifecycle events. Each of the primitive/elemental components listed as part of the PrimitiveEvent class has distinctive features that allow to specify the lifecycle event, either by itself or in combination with some other of such components.
	 */
	@RosettaSynonym(value="primitive", source="Rosetta_Workbench")
	public final PrimitiveEvent getPrimitive() {
		return primitive;
	}
	
	/**
	 * The set of timestamp(s) associated with the event as a collection of [dateTime, qualifier].
	 */
	@RosettaSynonym(value="timestamp", source="Rosetta_Workbench")
	@RosettaSynonym(value="header", source="DTCC_11_0", path="Body.OTC_Matching.Trade.FpML")
	@RosettaSynonym(value="header", source="DTCC_9_0", path="Body.OTC_Matching.Trade.FpML")
	@RosettaSynonym(value="Route", source="DTCC_11_0", path="Header.OTC_RM.Delivery.RouteHist")
	@RosettaSynonym(value="Route", source="DTCC_9_0", path="Header.OTC_RM.Delivery.RouteHist")
	@RosettaSynonym(value="header", source="DTCC_11_0", path="Body.OTC_Matching.Payment.FpML")
	@RosettaSynonym(value="header", source="DTCC_9_0", path="Body.OTC_Matching.Payment.FpML")
	@RosettaSynonym(value="timestamps", source="CME_ClearedConfirm_1_17", path="clearingConfirmed.trade.tradeHeader")
	@RosettaSynonym(value="header", source="CME_ClearedConfirm_1_17", path="clearingConfirmed")
	public final List<EventTimestamp> getTimestamp() {
		return timestamp;
	}
	
	
	@Override
	public RosettaMetaData<? extends Event> metaData() {
		return metaData;
	} 
	
	public EventBuilder toBuilder() {
		EventBuilder builder = new EventBuilder();
		ofNullable(getAccount()).ifPresent(account -> account.forEach(builder::addAccount));
		ofNullable(getAction()).ifPresent(builder::setAction);
		ofNullable(getEffectiveDate()).ifPresent(builder::setEffectiveDate);
		ofNullable(getEventDate()).ifPresent(builder::setEventDate);
		ofNullable(getEventEffect()).ifPresent(builder::setEventEffect);
		ofNullable(getEventIdentifier()).ifPresent(eventIdentifier -> eventIdentifier.forEach(builder::addEventIdentifier));
		ofNullable(getEventQualifier()).ifPresent(builder::setEventQualifier);
		ofNullable(getEventWorkflow()).ifPresent(builder::setEventWorkflow);
		ofNullable(getFunctionCall()).ifPresent(builder::setFunctionCall);
		ofNullable(getIntent()).ifPresent(builder::setIntent);
		ofNullable(getLineage()).ifPresent(builder::setLineage);
		ofNullable(getMessageInformation()).ifPresent(builder::setMessageInformation);
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		ofNullable(getParty()).ifPresent(party -> party.forEach(builder::addParty));
		ofNullable(getPrimitive()).ifPresent(builder::setPrimitive);
		ofNullable(getTimestamp()).ifPresent(timestamp -> timestamp.forEach(builder::addTimestamp));
		return builder;
	}

	public static EventBuilder builder() {
		return new EventBuilder();
	}

	public static  class EventBuilder extends RosettaModelObjectBuilder implements GlobalKeyBuilder<EventBuilder> , Qualified{
	
		protected List<Account.AccountBuilder> account;
		protected ActionEnum action;
		protected Date effectiveDate;
		protected Date eventDate;
		protected EventEffect.EventEffectBuilder eventEffect;
		protected List<Identifier.IdentifierBuilder> eventIdentifier;
		protected String eventQualifier;
		protected EventWorkflow.EventWorkflowBuilder eventWorkflow;
		protected String functionCall;
		protected IntentEnum intent;
		protected Lineage.LineageBuilder lineage;
		protected MessageInformation.MessageInformationBuilder messageInformation;
		protected MetaFields.MetaFieldsBuilder meta;
		protected List<Party.PartyBuilder> party;
		protected PrimitiveEvent.PrimitiveEventBuilder primitive;
		protected List<EventTimestamp.EventTimestampBuilder> timestamp;
	
		public EventBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends Event> metaData() {
			return metaData;
		} 
	
		
		public List<Account.AccountBuilder> getAccount() {
			return account;
		}
		
		public Account.AccountBuilder getOrCreateAccount(int index) {
			if (account==null) {
				this.account = new ArrayList<>();
			}
			return getIndex(account, index, ()->new Account.AccountBuilder());
		}
		
		
		public ActionEnum getAction() {
			return action;
		}
		
		
		public Date getEffectiveDate() {
			return effectiveDate;
		}
		
		
		public Date getEventDate() {
			return eventDate;
		}
		
		
		public EventEffect.EventEffectBuilder getEventEffect() {
			return eventEffect;
		}
		
		public EventEffect.EventEffectBuilder getOrCreateEventEffect() {
			if (eventEffect!=null) {
				return eventEffect;
			}
			else return eventEffect = new EventEffect.EventEffectBuilder();
		}
		
		
		public List<Identifier.IdentifierBuilder> getEventIdentifier() {
			return eventIdentifier;
		}
		
		public Identifier.IdentifierBuilder getOrCreateEventIdentifier(int index) {
			if (eventIdentifier==null) {
				this.eventIdentifier = new ArrayList<>();
			}
			return getIndex(eventIdentifier, index, ()->new Identifier.IdentifierBuilder());
		}
		
		
		public String getEventQualifier() {
			return eventQualifier;
		}
		
		
		public EventWorkflow.EventWorkflowBuilder getEventWorkflow() {
			return eventWorkflow;
		}
		
		public EventWorkflow.EventWorkflowBuilder getOrCreateEventWorkflow() {
			if (eventWorkflow!=null) {
				return eventWorkflow;
			}
			else return eventWorkflow = new EventWorkflow.EventWorkflowBuilder();
		}
		
		
		public String getFunctionCall() {
			return functionCall;
		}
		
		
		public IntentEnum getIntent() {
			return intent;
		}
		
		
		public Lineage.LineageBuilder getLineage() {
			return lineage;
		}
		
		public Lineage.LineageBuilder getOrCreateLineage() {
			if (lineage!=null) {
				return lineage;
			}
			else return lineage = new Lineage.LineageBuilder();
		}
		
		
		public MessageInformation.MessageInformationBuilder getMessageInformation() {
			return messageInformation;
		}
		
		public MessageInformation.MessageInformationBuilder getOrCreateMessageInformation() {
			if (messageInformation!=null) {
				return messageInformation;
			}
			else return messageInformation = new MessageInformation.MessageInformationBuilder();
		}
		
		
		public MetaFields.MetaFieldsBuilder getMeta() {
			return meta;
		}
		
		public MetaFields.MetaFieldsBuilder getOrCreateMeta() {
			if (meta!=null) {
				return meta;
			}
			else return meta = new MetaFields.MetaFieldsBuilder();
		}
		
		
		public List<Party.PartyBuilder> getParty() {
			return party;
		}
		
		public Party.PartyBuilder getOrCreateParty(int index) {
			if (party==null) {
				this.party = new ArrayList<>();
			}
			return getIndex(party, index, ()->new Party.PartyBuilder());
		}
		
		
		public PrimitiveEvent.PrimitiveEventBuilder getPrimitive() {
			return primitive;
		}
		
		public PrimitiveEvent.PrimitiveEventBuilder getOrCreatePrimitive() {
			if (primitive!=null) {
				return primitive;
			}
			else return primitive = new PrimitiveEvent.PrimitiveEventBuilder();
		}
		
		
		public List<EventTimestamp.EventTimestampBuilder> getTimestamp() {
			return timestamp;
		}
		
		public EventTimestamp.EventTimestampBuilder getOrCreateTimestamp(int index) {
			if (timestamp==null) {
				this.timestamp = new ArrayList<>();
			}
			return getIndex(timestamp, index, ()->new EventTimestamp.EventTimestampBuilder());
		}
		
	
		public EventBuilder addAccount(Account account) {
			if(this.account == null){
				this.account = new ArrayList<>();
			}
			this.account.add(account.toBuilder());
			return this;
		}
		public EventBuilder addAccount(Account account, int _idx) {
			if(this.account == null){
				this.account = new ArrayList<>();
			}
			getIndex(this.account, _idx, () -> account.toBuilder());
			this.account.set(_idx, account.toBuilder());
			return this;
		}
		public EventBuilder addAccount(List<Account> accounts) {
			if(this.account == null){
				this.account = new ArrayList<>();
			}
			for (Account toAdd : accounts) {
				this.account.add(toAdd.toBuilder());
			}
			return this;
		}
		public EventBuilder addAccountBuilder(Account.AccountBuilder account) {
			if(this.account == null){
				this.account = new ArrayList<>();
			}
			this.account.add(account);
			return this;
		}
		
		public EventBuilder clearAccount() {
			this.account = null;
			return this;
		}
		public EventBuilder setAction(ActionEnum action) {
			this.action = action;
			return this;
		}
		public EventBuilder setEffectiveDate(Date effectiveDate) {
			this.effectiveDate = effectiveDate;
			return this;
		}
		public EventBuilder setEventDate(Date eventDate) {
			this.eventDate = eventDate;
			return this;
		}
		public EventBuilder setEventEffect(EventEffect eventEffect) {
			this.eventEffect = eventEffect.toBuilder();
			return this;
		}
		public EventBuilder setEventEffectBuilder(EventEffect.EventEffectBuilder eventEffect) {
			this.eventEffect = eventEffect;
			return this;
		}
		public EventBuilder addEventIdentifier(Identifier eventIdentifier) {
			if(this.eventIdentifier == null){
				this.eventIdentifier = new ArrayList<>();
			}
			this.eventIdentifier.add(eventIdentifier.toBuilder());
			return this;
		}
		public EventBuilder addEventIdentifier(Identifier eventIdentifier, int _idx) {
			if(this.eventIdentifier == null){
				this.eventIdentifier = new ArrayList<>();
			}
			getIndex(this.eventIdentifier, _idx, () -> eventIdentifier.toBuilder());
			this.eventIdentifier.set(_idx, eventIdentifier.toBuilder());
			return this;
		}
		public EventBuilder addEventIdentifier(List<Identifier> eventIdentifiers) {
			if(this.eventIdentifier == null){
				this.eventIdentifier = new ArrayList<>();
			}
			for (Identifier toAdd : eventIdentifiers) {
				this.eventIdentifier.add(toAdd.toBuilder());
			}
			return this;
		}
		public EventBuilder addEventIdentifierBuilder(Identifier.IdentifierBuilder eventIdentifier) {
			if(this.eventIdentifier == null){
				this.eventIdentifier = new ArrayList<>();
			}
			this.eventIdentifier.add(eventIdentifier);
			return this;
		}
		
		public EventBuilder clearEventIdentifier() {
			this.eventIdentifier = null;
			return this;
		}
		public EventBuilder setEventQualifier(String eventQualifier) {
			this.eventQualifier = eventQualifier;
			return this;
		}
		public EventBuilder setEventWorkflow(EventWorkflow eventWorkflow) {
			this.eventWorkflow = eventWorkflow.toBuilder();
			return this;
		}
		public EventBuilder setEventWorkflowBuilder(EventWorkflow.EventWorkflowBuilder eventWorkflow) {
			this.eventWorkflow = eventWorkflow;
			return this;
		}
		public EventBuilder setFunctionCall(String functionCall) {
			this.functionCall = functionCall;
			return this;
		}
		public EventBuilder setIntent(IntentEnum intent) {
			this.intent = intent;
			return this;
		}
		public EventBuilder setLineage(Lineage lineage) {
			this.lineage = lineage.toBuilder();
			return this;
		}
		public EventBuilder setLineageBuilder(Lineage.LineageBuilder lineage) {
			this.lineage = lineage;
			return this;
		}
		public EventBuilder setMessageInformation(MessageInformation messageInformation) {
			this.messageInformation = messageInformation.toBuilder();
			return this;
		}
		public EventBuilder setMessageInformationBuilder(MessageInformation.MessageInformationBuilder messageInformation) {
			this.messageInformation = messageInformation;
			return this;
		}
		public EventBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		public EventBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		public EventBuilder addParty(Party party) {
			if(this.party == null){
				this.party = new ArrayList<>();
			}
			this.party.add(party.toBuilder());
			return this;
		}
		public EventBuilder addParty(Party party, int _idx) {
			if(this.party == null){
				this.party = new ArrayList<>();
			}
			getIndex(this.party, _idx, () -> party.toBuilder());
			this.party.set(_idx, party.toBuilder());
			return this;
		}
		public EventBuilder addParty(List<Party> partys) {
			if(this.party == null){
				this.party = new ArrayList<>();
			}
			for (Party toAdd : partys) {
				this.party.add(toAdd.toBuilder());
			}
			return this;
		}
		public EventBuilder addPartyBuilder(Party.PartyBuilder party) {
			if(this.party == null){
				this.party = new ArrayList<>();
			}
			this.party.add(party);
			return this;
		}
		
		public EventBuilder clearParty() {
			this.party = null;
			return this;
		}
		public EventBuilder setPrimitive(PrimitiveEvent primitive) {
			this.primitive = primitive.toBuilder();
			return this;
		}
		public EventBuilder setPrimitiveBuilder(PrimitiveEvent.PrimitiveEventBuilder primitive) {
			this.primitive = primitive;
			return this;
		}
		public EventBuilder addTimestamp(EventTimestamp timestamp) {
			if(this.timestamp == null){
				this.timestamp = new ArrayList<>();
			}
			this.timestamp.add(timestamp.toBuilder());
			return this;
		}
		public EventBuilder addTimestamp(EventTimestamp timestamp, int _idx) {
			if(this.timestamp == null){
				this.timestamp = new ArrayList<>();
			}
			getIndex(this.timestamp, _idx, () -> timestamp.toBuilder());
			this.timestamp.set(_idx, timestamp.toBuilder());
			return this;
		}
		public EventBuilder addTimestamp(List<EventTimestamp> timestamps) {
			if(this.timestamp == null){
				this.timestamp = new ArrayList<>();
			}
			for (EventTimestamp toAdd : timestamps) {
				this.timestamp.add(toAdd.toBuilder());
			}
			return this;
		}
		public EventBuilder addTimestampBuilder(EventTimestamp.EventTimestampBuilder timestamp) {
			if(this.timestamp == null){
				this.timestamp = new ArrayList<>();
			}
			this.timestamp.add(timestamp);
			return this;
		}
		
		public EventBuilder clearTimestamp() {
			this.timestamp = null;
			return this;
		}
	public void setQualification(String qualification) {
		this.setEventQualifier(qualification);
	}
		
		public Event build() {
			return new Event(this);
		}
	
		@Override
		public EventBuilder prune() {
			if (account!=null) account = account.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (eventEffect!=null && !eventEffect.prune().hasData()) eventEffect = null;
			if (eventIdentifier!=null) eventIdentifier = eventIdentifier.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (eventWorkflow!=null && !eventWorkflow.prune().hasData()) eventWorkflow = null;
			if (lineage!=null && !lineage.prune().hasData()) lineage = null;
			if (messageInformation!=null && !messageInformation.prune().hasData()) messageInformation = null;
			if (meta!=null && !meta.prune().hasData()) meta = null;
			if (party!=null) party = party.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (primitive!=null && !primitive.prune().hasData()) primitive = null;
			if (timestamp!=null) timestamp = timestamp.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getAccount()!=null && getAccount().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getAction()!=null) return true;
			if (getEffectiveDate()!=null) return true;
			if (getEventDate()!=null) return true;
			if (getEventEffect()!=null && getEventEffect().hasData()) return true;
			if (getEventIdentifier()!=null && getEventIdentifier().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getEventQualifier()!=null) return true;
			if (getEventWorkflow()!=null && getEventWorkflow().hasData()) return true;
			if (getFunctionCall()!=null) return true;
			if (getIntent()!=null) return true;
			if (getLineage()!=null && getLineage().hasData()) return true;
			if (getMessageInformation()!=null && getMessageInformation().hasData()) return true;
			if (getMeta()!=null && getMeta().hasData()) return true;
			if (getParty()!=null && getParty().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getPrimitive()!=null && getPrimitive().hasData()) return true;
			if (getTimestamp()!=null && getTimestamp().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("action"), ActionEnum.class, action, this);
			processor.processBasic(path.newSubPath("effectiveDate"), Date.class, effectiveDate, this);
			processor.processBasic(path.newSubPath("eventDate"), Date.class, eventDate, this);
			processor.processBasic(path.newSubPath("eventQualifier"), String.class, eventQualifier, this);
			processor.processBasic(path.newSubPath("functionCall"), String.class, functionCall, this);
			processor.processBasic(path.newSubPath("intent"), IntentEnum.class, intent, this);
			
			processRosetta(path.newSubPath("account"), processor, Account.class, account);
			processRosetta(path.newSubPath("eventEffect"), processor, EventEffect.class, eventEffect);
			processRosetta(path.newSubPath("eventIdentifier"), processor, Identifier.class, eventIdentifier);
			processRosetta(path.newSubPath("eventWorkflow"), processor, EventWorkflow.class, eventWorkflow);
			processRosetta(path.newSubPath("lineage"), processor, Lineage.class, lineage);
			processRosetta(path.newSubPath("messageInformation"), processor, MessageInformation.class, messageInformation);
			processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
			processRosetta(path.newSubPath("party"), processor, Party.class, party);
			processRosetta(path.newSubPath("primitive"), processor, PrimitiveEvent.class, primitive);
			processRosetta(path.newSubPath("timestamp"), processor, EventTimestamp.class, timestamp);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			EventBuilder _that = (EventBuilder) o;
		
			if (!ListEquals.listEquals(account, _that.account)) return false;
			if (action != null ? !action.equals(_that.action) : _that.action != null) return false;
			if (effectiveDate != null ? !effectiveDate.equals(_that.effectiveDate) : _that.effectiveDate != null) return false;
			if (eventDate != null ? !eventDate.equals(_that.eventDate) : _that.eventDate != null) return false;
			if (!ListEquals.listEquals(eventIdentifier, _that.eventIdentifier)) return false;
			if (eventQualifier != null ? !eventQualifier.equals(_that.eventQualifier) : _that.eventQualifier != null) return false;
			if (eventWorkflow != null ? !eventWorkflow.equals(_that.eventWorkflow) : _that.eventWorkflow != null) return false;
			if (functionCall != null ? !functionCall.equals(_that.functionCall) : _that.functionCall != null) return false;
			if (intent != null ? !intent.equals(_that.intent) : _that.intent != null) return false;
			if (lineage != null ? !lineage.equals(_that.lineage) : _that.lineage != null) return false;
			if (messageInformation != null ? !messageInformation.equals(_that.messageInformation) : _that.messageInformation != null) return false;
			if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
			if (!ListEquals.listEquals(party, _that.party)) return false;
			if (primitive != null ? !primitive.equals(_that.primitive) : _that.primitive != null) return false;
			if (!ListEquals.listEquals(timestamp, _that.timestamp)) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (account != null ? account.hashCode() : 0);
			_result = 31 * _result + (action != null ? action.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (effectiveDate != null ? effectiveDate.hashCode() : 0);
			_result = 31 * _result + (eventDate != null ? eventDate.hashCode() : 0);
			_result = 31 * _result + (eventIdentifier != null ? eventIdentifier.hashCode() : 0);
			_result = 31 * _result + (eventQualifier != null ? eventQualifier.hashCode() : 0);
			_result = 31 * _result + (eventWorkflow != null ? eventWorkflow.hashCode() : 0);
			_result = 31 * _result + (functionCall != null ? functionCall.hashCode() : 0);
			_result = 31 * _result + (intent != null ? intent.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (lineage != null ? lineage.hashCode() : 0);
			_result = 31 * _result + (messageInformation != null ? messageInformation.hashCode() : 0);
			_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
			_result = 31 * _result + (party != null ? party.hashCode() : 0);
			_result = 31 * _result + (primitive != null ? primitive.hashCode() : 0);
			_result = 31 * _result + (timestamp != null ? timestamp.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "EventBuilder {" +
				"account=" + this.account + ", " +
				"action=" + this.action + ", " +
				"effectiveDate=" + this.effectiveDate + ", " +
				"eventDate=" + this.eventDate + ", " +
				"eventEffect=" + this.eventEffect + ", " +
				"eventIdentifier=" + this.eventIdentifier + ", " +
				"eventQualifier=" + this.eventQualifier + ", " +
				"eventWorkflow=" + this.eventWorkflow + ", " +
				"functionCall=" + this.functionCall + ", " +
				"intent=" + this.intent + ", " +
				"lineage=" + this.lineage + ", " +
				"messageInformation=" + this.messageInformation + ", " +
				"meta=" + this.meta + ", " +
				"party=" + this.party + ", " +
				"primitive=" + this.primitive + ", " +
				"timestamp=" + this.timestamp +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("action"), ActionEnum.class, action, this);
		processor.processBasic(path.newSubPath("effectiveDate"), Date.class, effectiveDate, this);
		processor.processBasic(path.newSubPath("eventDate"), Date.class, eventDate, this);
		processor.processBasic(path.newSubPath("eventQualifier"), String.class, eventQualifier, this);
		processor.processBasic(path.newSubPath("functionCall"), String.class, functionCall, this);
		processor.processBasic(path.newSubPath("intent"), IntentEnum.class, intent, this);
		
		processRosetta(path.newSubPath("account"), processor, Account.class, account);
		processRosetta(path.newSubPath("eventEffect"), processor, EventEffect.class, eventEffect);
		processRosetta(path.newSubPath("eventIdentifier"), processor, Identifier.class, eventIdentifier);
		processRosetta(path.newSubPath("eventWorkflow"), processor, EventWorkflow.class, eventWorkflow);
		processRosetta(path.newSubPath("lineage"), processor, Lineage.class, lineage);
		processRosetta(path.newSubPath("messageInformation"), processor, MessageInformation.class, messageInformation);
		processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
		processRosetta(path.newSubPath("party"), processor, Party.class, party);
		processRosetta(path.newSubPath("primitive"), processor, PrimitiveEvent.class, primitive);
		processRosetta(path.newSubPath("timestamp"), processor, EventTimestamp.class, timestamp);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		Event _that = (Event) o;
	
		if (!ListEquals.listEquals(account, _that.account)) return false;
		if (action != null ? !action.equals(_that.action) : _that.action != null) return false;
		if (effectiveDate != null ? !effectiveDate.equals(_that.effectiveDate) : _that.effectiveDate != null) return false;
		if (eventDate != null ? !eventDate.equals(_that.eventDate) : _that.eventDate != null) return false;
		if (!ListEquals.listEquals(eventIdentifier, _that.eventIdentifier)) return false;
		if (eventQualifier != null ? !eventQualifier.equals(_that.eventQualifier) : _that.eventQualifier != null) return false;
		if (eventWorkflow != null ? !eventWorkflow.equals(_that.eventWorkflow) : _that.eventWorkflow != null) return false;
		if (functionCall != null ? !functionCall.equals(_that.functionCall) : _that.functionCall != null) return false;
		if (intent != null ? !intent.equals(_that.intent) : _that.intent != null) return false;
		if (lineage != null ? !lineage.equals(_that.lineage) : _that.lineage != null) return false;
		if (messageInformation != null ? !messageInformation.equals(_that.messageInformation) : _that.messageInformation != null) return false;
		if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
		if (!ListEquals.listEquals(party, _that.party)) return false;
		if (primitive != null ? !primitive.equals(_that.primitive) : _that.primitive != null) return false;
		if (!ListEquals.listEquals(timestamp, _that.timestamp)) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (account != null ? account.hashCode() : 0);
		_result = 31 * _result + (action != null ? action.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (effectiveDate != null ? effectiveDate.hashCode() : 0);
		_result = 31 * _result + (eventDate != null ? eventDate.hashCode() : 0);
		_result = 31 * _result + (eventIdentifier != null ? eventIdentifier.hashCode() : 0);
		_result = 31 * _result + (eventQualifier != null ? eventQualifier.hashCode() : 0);
		_result = 31 * _result + (eventWorkflow != null ? eventWorkflow.hashCode() : 0);
		_result = 31 * _result + (functionCall != null ? functionCall.hashCode() : 0);
		_result = 31 * _result + (intent != null ? intent.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (lineage != null ? lineage.hashCode() : 0);
		_result = 31 * _result + (messageInformation != null ? messageInformation.hashCode() : 0);
		_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
		_result = 31 * _result + (party != null ? party.hashCode() : 0);
		_result = 31 * _result + (primitive != null ? primitive.hashCode() : 0);
		_result = 31 * _result + (timestamp != null ? timestamp.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "Event {" +
			"account=" + this.account + ", " +
			"action=" + this.action + ", " +
			"effectiveDate=" + this.effectiveDate + ", " +
			"eventDate=" + this.eventDate + ", " +
			"eventEffect=" + this.eventEffect + ", " +
			"eventIdentifier=" + this.eventIdentifier + ", " +
			"eventQualifier=" + this.eventQualifier + ", " +
			"eventWorkflow=" + this.eventWorkflow + ", " +
			"functionCall=" + this.functionCall + ", " +
			"intent=" + this.intent + ", " +
			"lineage=" + this.lineage + ", " +
			"messageInformation=" + this.messageInformation + ", " +
			"meta=" + this.meta + ", " +
			"party=" + this.party + ", " +
			"primitive=" + this.primitive + ", " +
			"timestamp=" + this.timestamp +
		'}';
	}
}
