package org.isda.cdm;

import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.metafields.ReferenceWithMetaContract;
import org.isda.cdm.metafields.ReferenceWithMetaContract.ReferenceWithMetaContractBuilder;
import org.isda.cdm.metafields.ReferenceWithMetaExecution;
import org.isda.cdm.metafields.ReferenceWithMetaExecution.ReferenceWithMetaExecutionBuilder;
import org.isda.cdm.metafields.ReferenceWithMetaProductIdentifier;
import org.isda.cdm.metafields.ReferenceWithMetaProductIdentifier.ReferenceWithMetaProductIdentifierBuilder;
import org.isda.cdm.metafields.ReferenceWithMetaTransferPrimitive;
import org.isda.cdm.metafields.ReferenceWithMetaTransferPrimitive.ReferenceWithMetaTransferPrimitiveBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.EventEffectMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;

/**
 * The set of operational and positional effects associated with a lifecycle event, alongside the reference to the contract reference(s) that is subject to the event (and is positioned in the before state of the event primitive).
 * 
 * @version 2.5.4
 */
@RosettaClass
public class EventEffect extends RosettaModelObject {
	private final List<ReferenceWithMetaContract> contract;
	private final List<ReferenceWithMetaContract> effectedContract;
	private final List<ReferenceWithMetaExecution> effectedExecution;
	private final List<ReferenceWithMetaExecution> execution;
	private final List<ReferenceWithMetaProductIdentifier> productIdentifier;
	private final List<ReferenceWithMetaTransferPrimitive> transfer;
	private static EventEffectMeta metaData = new EventEffectMeta();
	
	EventEffect(EventEffectBuilder builder) {
		this.contract = ofNullable(builder.getContract()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.effectedContract = ofNullable(builder.getEffectedContract()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.effectedExecution = ofNullable(builder.getEffectedExecution()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.execution = ofNullable(builder.getExecution()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.productIdentifier = ofNullable(builder.getProductIdentifier()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.transfer = ofNullable(builder.getTransfer()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
	}
	
	/**
	 * A pointer to the contract effect(s), an example of such being the outcome of a new trade, swaption exercise or novation event.
	 */
	public final List<ReferenceWithMetaContract> getContract() {
		return contract;
	}
	
	/**
	 * A pointer to the contract(s) to which the event effect(s) apply, i.e. in the before event state.
	 */
	public final List<ReferenceWithMetaContract> getEffectedContract() {
		return effectedContract;
	}
	
	/**
	 * A pointer to the execution(s) to which the event effect(s) apply, i.e. in the before event state.
	 */
	public final List<ReferenceWithMetaExecution> getEffectedExecution() {
		return effectedExecution;
	}
	
	/**
	 * A pointer to the execution effect(s), an example of such being a clearing submission event when taking place on the back of an execution.
	 */
	public final List<ReferenceWithMetaExecution> getExecution() {
		return execution;
	}
	
	/**
	 * A pointer to the product identifier effect(s), an example of such being the outcome of the physical exercise of a bond option.
	 */
	public final List<ReferenceWithMetaProductIdentifier> getProductIdentifier() {
		return productIdentifier;
	}
	
	/**
	 * A pointer to the transfer effect(s), either a cash, security or other asset.
	 */
	public final List<ReferenceWithMetaTransferPrimitive> getTransfer() {
		return transfer;
	}
	
	
	@Override
	public RosettaMetaData<? extends EventEffect> metaData() {
		return metaData;
	} 
	
	public EventEffectBuilder toBuilder() {
		EventEffectBuilder builder = new EventEffectBuilder();
		ofNullable(getContract()).ifPresent(contract -> contract.forEach(builder::addContract));
		ofNullable(getEffectedContract()).ifPresent(effectedContract -> effectedContract.forEach(builder::addEffectedContract));
		ofNullable(getEffectedExecution()).ifPresent(effectedExecution -> effectedExecution.forEach(builder::addEffectedExecution));
		ofNullable(getExecution()).ifPresent(execution -> execution.forEach(builder::addExecution));
		ofNullable(getProductIdentifier()).ifPresent(productIdentifier -> productIdentifier.forEach(builder::addProductIdentifier));
		ofNullable(getTransfer()).ifPresent(transfer -> transfer.forEach(builder::addTransfer));
		return builder;
	}

	public static EventEffectBuilder builder() {
		return new EventEffectBuilder();
	}

	public static  class EventEffectBuilder extends RosettaModelObjectBuilder {
	
		protected List<ReferenceWithMetaContractBuilder> contract;
		protected List<ReferenceWithMetaContractBuilder> effectedContract;
		protected List<ReferenceWithMetaExecutionBuilder> effectedExecution;
		protected List<ReferenceWithMetaExecutionBuilder> execution;
		protected List<ReferenceWithMetaProductIdentifierBuilder> productIdentifier;
		protected List<ReferenceWithMetaTransferPrimitiveBuilder> transfer;
	
		public EventEffectBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends EventEffect> metaData() {
			return metaData;
		} 
	
		
		public List<ReferenceWithMetaContractBuilder> getContract() {
			return contract;
		}
		
		public ReferenceWithMetaContractBuilder getOrCreateContract(int index) {
			if (contract==null) {
				this.contract = new ArrayList<>();
			}
			return getIndex(contract, index, ()->new ReferenceWithMetaContractBuilder());
		}
		
		
		public List<ReferenceWithMetaContractBuilder> getEffectedContract() {
			return effectedContract;
		}
		
		public ReferenceWithMetaContractBuilder getOrCreateEffectedContract(int index) {
			if (effectedContract==null) {
				this.effectedContract = new ArrayList<>();
			}
			return getIndex(effectedContract, index, ()->new ReferenceWithMetaContractBuilder());
		}
		
		
		public List<ReferenceWithMetaExecutionBuilder> getEffectedExecution() {
			return effectedExecution;
		}
		
		public ReferenceWithMetaExecutionBuilder getOrCreateEffectedExecution(int index) {
			if (effectedExecution==null) {
				this.effectedExecution = new ArrayList<>();
			}
			return getIndex(effectedExecution, index, ()->new ReferenceWithMetaExecutionBuilder());
		}
		
		
		public List<ReferenceWithMetaExecutionBuilder> getExecution() {
			return execution;
		}
		
		public ReferenceWithMetaExecutionBuilder getOrCreateExecution(int index) {
			if (execution==null) {
				this.execution = new ArrayList<>();
			}
			return getIndex(execution, index, ()->new ReferenceWithMetaExecutionBuilder());
		}
		
		
		public List<ReferenceWithMetaProductIdentifierBuilder> getProductIdentifier() {
			return productIdentifier;
		}
		
		public ReferenceWithMetaProductIdentifierBuilder getOrCreateProductIdentifier(int index) {
			if (productIdentifier==null) {
				this.productIdentifier = new ArrayList<>();
			}
			return getIndex(productIdentifier, index, ()->new ReferenceWithMetaProductIdentifierBuilder());
		}
		
		
		public List<ReferenceWithMetaTransferPrimitiveBuilder> getTransfer() {
			return transfer;
		}
		
		public ReferenceWithMetaTransferPrimitiveBuilder getOrCreateTransfer(int index) {
			if (transfer==null) {
				this.transfer = new ArrayList<>();
			}
			return getIndex(transfer, index, ()->new ReferenceWithMetaTransferPrimitiveBuilder());
		}
		
	
		public EventEffectBuilder addContract(ReferenceWithMetaContract contract) {
			if(this.contract == null){
				this.contract = new ArrayList<>();
			}
			this.contract.add(contract.toBuilder());
			return this;
		}
		public EventEffectBuilder addContract(ReferenceWithMetaContract contract, int _idx) {
			if(this.contract == null){
				this.contract = new ArrayList<>();
			}
			getIndex(this.contract, _idx, () -> contract.toBuilder());
			this.contract.set(_idx, contract.toBuilder());
			return this;
		}
		public EventEffectBuilder addContract(List<ReferenceWithMetaContract> contracts) {
			if(this.contract == null){
				this.contract = new ArrayList<>();
			}
			for (ReferenceWithMetaContract toAdd : contracts) {
				this.contract.add(toAdd.toBuilder());
			}
			return this;
		}
		public EventEffectBuilder addContractBuilder(ReferenceWithMetaContractBuilder contract) {
			if(this.contract == null){
				this.contract = new ArrayList<>();
			}
			this.contract.add(contract);
			return this;
		}
		
		public EventEffectBuilder addContractRef(Contract.ContractBuilder contract) {
			return addContract(ReferenceWithMetaContract.builder().setValueBuilder(contract).build());
		}
		
		public EventEffectBuilder addContractRef(Contract.ContractBuilder contract, int _idx) {
			return addContract(ReferenceWithMetaContract.builder().setValueBuilder(contract).build(), _idx);
		}
		
		public EventEffectBuilder addContractRef(List<Contract> contracts) {
			for (Contract toAdd : contracts) {
				addContractRef(toAdd);
			}
			return this;
		}
		
		public EventEffectBuilder addContractRef(Contract contract) {
			return addContractRef(contract.toBuilder());
		}
		
		public EventEffectBuilder clearContract() {
			this.contract = null;
			return this;
		}
		public EventEffectBuilder addEffectedContract(ReferenceWithMetaContract effectedContract) {
			if(this.effectedContract == null){
				this.effectedContract = new ArrayList<>();
			}
			this.effectedContract.add(effectedContract.toBuilder());
			return this;
		}
		public EventEffectBuilder addEffectedContract(ReferenceWithMetaContract effectedContract, int _idx) {
			if(this.effectedContract == null){
				this.effectedContract = new ArrayList<>();
			}
			getIndex(this.effectedContract, _idx, () -> effectedContract.toBuilder());
			this.effectedContract.set(_idx, effectedContract.toBuilder());
			return this;
		}
		public EventEffectBuilder addEffectedContract(List<ReferenceWithMetaContract> effectedContracts) {
			if(this.effectedContract == null){
				this.effectedContract = new ArrayList<>();
			}
			for (ReferenceWithMetaContract toAdd : effectedContracts) {
				this.effectedContract.add(toAdd.toBuilder());
			}
			return this;
		}
		public EventEffectBuilder addEffectedContractBuilder(ReferenceWithMetaContractBuilder effectedContract) {
			if(this.effectedContract == null){
				this.effectedContract = new ArrayList<>();
			}
			this.effectedContract.add(effectedContract);
			return this;
		}
		
		public EventEffectBuilder addEffectedContractRef(Contract.ContractBuilder effectedContract) {
			return addEffectedContract(ReferenceWithMetaContract.builder().setValueBuilder(effectedContract).build());
		}
		
		public EventEffectBuilder addEffectedContractRef(Contract.ContractBuilder effectedContract, int _idx) {
			return addEffectedContract(ReferenceWithMetaContract.builder().setValueBuilder(effectedContract).build(), _idx);
		}
		
		public EventEffectBuilder addEffectedContractRef(List<Contract> effectedContracts) {
			for (Contract toAdd : effectedContracts) {
				addEffectedContractRef(toAdd);
			}
			return this;
		}
		
		public EventEffectBuilder addEffectedContractRef(Contract effectedContract) {
			return addEffectedContractRef(effectedContract.toBuilder());
		}
		
		public EventEffectBuilder clearEffectedContract() {
			this.effectedContract = null;
			return this;
		}
		public EventEffectBuilder addEffectedExecution(ReferenceWithMetaExecution effectedExecution) {
			if(this.effectedExecution == null){
				this.effectedExecution = new ArrayList<>();
			}
			this.effectedExecution.add(effectedExecution.toBuilder());
			return this;
		}
		public EventEffectBuilder addEffectedExecution(ReferenceWithMetaExecution effectedExecution, int _idx) {
			if(this.effectedExecution == null){
				this.effectedExecution = new ArrayList<>();
			}
			getIndex(this.effectedExecution, _idx, () -> effectedExecution.toBuilder());
			this.effectedExecution.set(_idx, effectedExecution.toBuilder());
			return this;
		}
		public EventEffectBuilder addEffectedExecution(List<ReferenceWithMetaExecution> effectedExecutions) {
			if(this.effectedExecution == null){
				this.effectedExecution = new ArrayList<>();
			}
			for (ReferenceWithMetaExecution toAdd : effectedExecutions) {
				this.effectedExecution.add(toAdd.toBuilder());
			}
			return this;
		}
		public EventEffectBuilder addEffectedExecutionBuilder(ReferenceWithMetaExecutionBuilder effectedExecution) {
			if(this.effectedExecution == null){
				this.effectedExecution = new ArrayList<>();
			}
			this.effectedExecution.add(effectedExecution);
			return this;
		}
		
		public EventEffectBuilder addEffectedExecutionRef(Execution.ExecutionBuilder effectedExecution) {
			return addEffectedExecution(ReferenceWithMetaExecution.builder().setValueBuilder(effectedExecution).build());
		}
		
		public EventEffectBuilder addEffectedExecutionRef(Execution.ExecutionBuilder effectedExecution, int _idx) {
			return addEffectedExecution(ReferenceWithMetaExecution.builder().setValueBuilder(effectedExecution).build(), _idx);
		}
		
		public EventEffectBuilder addEffectedExecutionRef(List<Execution> effectedExecutions) {
			for (Execution toAdd : effectedExecutions) {
				addEffectedExecutionRef(toAdd);
			}
			return this;
		}
		
		public EventEffectBuilder addEffectedExecutionRef(Execution effectedExecution) {
			return addEffectedExecutionRef(effectedExecution.toBuilder());
		}
		
		public EventEffectBuilder clearEffectedExecution() {
			this.effectedExecution = null;
			return this;
		}
		public EventEffectBuilder addExecution(ReferenceWithMetaExecution execution) {
			if(this.execution == null){
				this.execution = new ArrayList<>();
			}
			this.execution.add(execution.toBuilder());
			return this;
		}
		public EventEffectBuilder addExecution(ReferenceWithMetaExecution execution, int _idx) {
			if(this.execution == null){
				this.execution = new ArrayList<>();
			}
			getIndex(this.execution, _idx, () -> execution.toBuilder());
			this.execution.set(_idx, execution.toBuilder());
			return this;
		}
		public EventEffectBuilder addExecution(List<ReferenceWithMetaExecution> executions) {
			if(this.execution == null){
				this.execution = new ArrayList<>();
			}
			for (ReferenceWithMetaExecution toAdd : executions) {
				this.execution.add(toAdd.toBuilder());
			}
			return this;
		}
		public EventEffectBuilder addExecutionBuilder(ReferenceWithMetaExecutionBuilder execution) {
			if(this.execution == null){
				this.execution = new ArrayList<>();
			}
			this.execution.add(execution);
			return this;
		}
		
		public EventEffectBuilder addExecutionRef(Execution.ExecutionBuilder execution) {
			return addExecution(ReferenceWithMetaExecution.builder().setValueBuilder(execution).build());
		}
		
		public EventEffectBuilder addExecutionRef(Execution.ExecutionBuilder execution, int _idx) {
			return addExecution(ReferenceWithMetaExecution.builder().setValueBuilder(execution).build(), _idx);
		}
		
		public EventEffectBuilder addExecutionRef(List<Execution> executions) {
			for (Execution toAdd : executions) {
				addExecutionRef(toAdd);
			}
			return this;
		}
		
		public EventEffectBuilder addExecutionRef(Execution execution) {
			return addExecutionRef(execution.toBuilder());
		}
		
		public EventEffectBuilder clearExecution() {
			this.execution = null;
			return this;
		}
		public EventEffectBuilder addProductIdentifier(ReferenceWithMetaProductIdentifier productIdentifier) {
			if(this.productIdentifier == null){
				this.productIdentifier = new ArrayList<>();
			}
			this.productIdentifier.add(productIdentifier.toBuilder());
			return this;
		}
		public EventEffectBuilder addProductIdentifier(ReferenceWithMetaProductIdentifier productIdentifier, int _idx) {
			if(this.productIdentifier == null){
				this.productIdentifier = new ArrayList<>();
			}
			getIndex(this.productIdentifier, _idx, () -> productIdentifier.toBuilder());
			this.productIdentifier.set(_idx, productIdentifier.toBuilder());
			return this;
		}
		public EventEffectBuilder addProductIdentifier(List<ReferenceWithMetaProductIdentifier> productIdentifiers) {
			if(this.productIdentifier == null){
				this.productIdentifier = new ArrayList<>();
			}
			for (ReferenceWithMetaProductIdentifier toAdd : productIdentifiers) {
				this.productIdentifier.add(toAdd.toBuilder());
			}
			return this;
		}
		public EventEffectBuilder addProductIdentifierBuilder(ReferenceWithMetaProductIdentifierBuilder productIdentifier) {
			if(this.productIdentifier == null){
				this.productIdentifier = new ArrayList<>();
			}
			this.productIdentifier.add(productIdentifier);
			return this;
		}
		
		public EventEffectBuilder addProductIdentifierRef(ProductIdentifier.ProductIdentifierBuilder productIdentifier) {
			return addProductIdentifier(ReferenceWithMetaProductIdentifier.builder().setValueBuilder(productIdentifier).build());
		}
		
		public EventEffectBuilder addProductIdentifierRef(ProductIdentifier.ProductIdentifierBuilder productIdentifier, int _idx) {
			return addProductIdentifier(ReferenceWithMetaProductIdentifier.builder().setValueBuilder(productIdentifier).build(), _idx);
		}
		
		public EventEffectBuilder addProductIdentifierRef(List<ProductIdentifier> productIdentifiers) {
			for (ProductIdentifier toAdd : productIdentifiers) {
				addProductIdentifierRef(toAdd);
			}
			return this;
		}
		
		public EventEffectBuilder addProductIdentifierRef(ProductIdentifier productIdentifier) {
			return addProductIdentifierRef(productIdentifier.toBuilder());
		}
		
		public EventEffectBuilder clearProductIdentifier() {
			this.productIdentifier = null;
			return this;
		}
		public EventEffectBuilder addTransfer(ReferenceWithMetaTransferPrimitive transfer) {
			if(this.transfer == null){
				this.transfer = new ArrayList<>();
			}
			this.transfer.add(transfer.toBuilder());
			return this;
		}
		public EventEffectBuilder addTransfer(ReferenceWithMetaTransferPrimitive transfer, int _idx) {
			if(this.transfer == null){
				this.transfer = new ArrayList<>();
			}
			getIndex(this.transfer, _idx, () -> transfer.toBuilder());
			this.transfer.set(_idx, transfer.toBuilder());
			return this;
		}
		public EventEffectBuilder addTransfer(List<ReferenceWithMetaTransferPrimitive> transfers) {
			if(this.transfer == null){
				this.transfer = new ArrayList<>();
			}
			for (ReferenceWithMetaTransferPrimitive toAdd : transfers) {
				this.transfer.add(toAdd.toBuilder());
			}
			return this;
		}
		public EventEffectBuilder addTransferBuilder(ReferenceWithMetaTransferPrimitiveBuilder transfer) {
			if(this.transfer == null){
				this.transfer = new ArrayList<>();
			}
			this.transfer.add(transfer);
			return this;
		}
		
		public EventEffectBuilder addTransferRef(TransferPrimitive.TransferPrimitiveBuilder transfer) {
			return addTransfer(ReferenceWithMetaTransferPrimitive.builder().setValueBuilder(transfer).build());
		}
		
		public EventEffectBuilder addTransferRef(TransferPrimitive.TransferPrimitiveBuilder transfer, int _idx) {
			return addTransfer(ReferenceWithMetaTransferPrimitive.builder().setValueBuilder(transfer).build(), _idx);
		}
		
		public EventEffectBuilder addTransferRef(List<TransferPrimitive> transfers) {
			for (TransferPrimitive toAdd : transfers) {
				addTransferRef(toAdd);
			}
			return this;
		}
		
		public EventEffectBuilder addTransferRef(TransferPrimitive transfer) {
			return addTransferRef(transfer.toBuilder());
		}
		
		public EventEffectBuilder clearTransfer() {
			this.transfer = null;
			return this;
		}
		
		public EventEffect build() {
			return new EventEffect(this);
		}
	
		@Override
		public EventEffectBuilder prune() {
			if (contract!=null) contract = contract.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (effectedContract!=null) effectedContract = effectedContract.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (effectedExecution!=null) effectedExecution = effectedExecution.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (execution!=null) execution = execution.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (productIdentifier!=null) productIdentifier = productIdentifier.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (transfer!=null) transfer = transfer.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getContract()!=null && getContract().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getEffectedContract()!=null && getEffectedContract().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getEffectedExecution()!=null && getEffectedExecution().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getExecution()!=null && getExecution().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getProductIdentifier()!=null && getProductIdentifier().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getTransfer()!=null && getTransfer().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("contract"), processor, ReferenceWithMetaContract.class, contract);
			processRosetta(path.newSubPath("effectedContract"), processor, ReferenceWithMetaContract.class, effectedContract);
			processRosetta(path.newSubPath("effectedExecution"), processor, ReferenceWithMetaExecution.class, effectedExecution);
			processRosetta(path.newSubPath("execution"), processor, ReferenceWithMetaExecution.class, execution);
			processRosetta(path.newSubPath("productIdentifier"), processor, ReferenceWithMetaProductIdentifier.class, productIdentifier);
			processRosetta(path.newSubPath("transfer"), processor, ReferenceWithMetaTransferPrimitive.class, transfer);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			EventEffectBuilder _that = (EventEffectBuilder) o;
		
			if (!ListEquals.listEquals(contract, _that.contract)) return false;
			if (!ListEquals.listEquals(effectedContract, _that.effectedContract)) return false;
			if (!ListEquals.listEquals(effectedExecution, _that.effectedExecution)) return false;
			if (!ListEquals.listEquals(execution, _that.execution)) return false;
			if (!ListEquals.listEquals(productIdentifier, _that.productIdentifier)) return false;
			if (!ListEquals.listEquals(transfer, _that.transfer)) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (contract != null ? contract.hashCode() : 0);
			_result = 31 * _result + (effectedContract != null ? effectedContract.hashCode() : 0);
			_result = 31 * _result + (effectedExecution != null ? effectedExecution.hashCode() : 0);
			_result = 31 * _result + (execution != null ? execution.hashCode() : 0);
			_result = 31 * _result + (productIdentifier != null ? productIdentifier.hashCode() : 0);
			_result = 31 * _result + (transfer != null ? transfer.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "EventEffectBuilder {" +
				"contract=" + this.contract + ", " +
				"effectedContract=" + this.effectedContract + ", " +
				"effectedExecution=" + this.effectedExecution + ", " +
				"execution=" + this.execution + ", " +
				"productIdentifier=" + this.productIdentifier + ", " +
				"transfer=" + this.transfer +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("contract"), processor, ReferenceWithMetaContract.class, contract);
		processRosetta(path.newSubPath("effectedContract"), processor, ReferenceWithMetaContract.class, effectedContract);
		processRosetta(path.newSubPath("effectedExecution"), processor, ReferenceWithMetaExecution.class, effectedExecution);
		processRosetta(path.newSubPath("execution"), processor, ReferenceWithMetaExecution.class, execution);
		processRosetta(path.newSubPath("productIdentifier"), processor, ReferenceWithMetaProductIdentifier.class, productIdentifier);
		processRosetta(path.newSubPath("transfer"), processor, ReferenceWithMetaTransferPrimitive.class, transfer);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		EventEffect _that = (EventEffect) o;
	
		if (!ListEquals.listEquals(contract, _that.contract)) return false;
		if (!ListEquals.listEquals(effectedContract, _that.effectedContract)) return false;
		if (!ListEquals.listEquals(effectedExecution, _that.effectedExecution)) return false;
		if (!ListEquals.listEquals(execution, _that.execution)) return false;
		if (!ListEquals.listEquals(productIdentifier, _that.productIdentifier)) return false;
		if (!ListEquals.listEquals(transfer, _that.transfer)) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (contract != null ? contract.hashCode() : 0);
		_result = 31 * _result + (effectedContract != null ? effectedContract.hashCode() : 0);
		_result = 31 * _result + (effectedExecution != null ? effectedExecution.hashCode() : 0);
		_result = 31 * _result + (execution != null ? execution.hashCode() : 0);
		_result = 31 * _result + (productIdentifier != null ? productIdentifier.hashCode() : 0);
		_result = 31 * _result + (transfer != null ? transfer.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "EventEffect {" +
			"contract=" + this.contract + ", " +
			"effectedContract=" + this.effectedContract + ", " +
			"effectedExecution=" + this.effectedExecution + ", " +
			"execution=" + this.execution + ", " +
			"productIdentifier=" + this.productIdentifier + ", " +
			"transfer=" + this.transfer +
		'}';
	}
}
