package org.isda.cdm;

import java.time.ZonedDateTime;
import org.isda.cdm.EventTimestampQualificationEnum;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.EventTimestampMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.time.ZonedDateTime;
		import static java.util.Optional.ofNullable;

/**
 * A class to represent the various set of timestamps that can be associated with lifecycle events, as a collection of [dateTime, qualifier].
 * 
 * @version 2.5.4
 */
@RosettaClass
public class EventTimestamp extends RosettaModelObject {
	private final ZonedDateTime dateTime;
	private final EventTimestampQualificationEnum qualification;
	private static EventTimestampMeta metaData = new EventTimestampMeta();
	
	EventTimestamp(EventTimestampBuilder builder) {
		this.dateTime = builder.getDateTime();
		this.qualification = builder.getQualification();
	}
	
	/**
	 * The CDM specifies that the zoned date time is to be expressed in accordance with ISO 8601, either as UTC as an offset to UTC.
	 */
	@RosettaSynonym(value="creationTimestamp", source="Rosetta_Workbench")
	@RosettaSynonym(value="creationTimestamp", source="DTCC_11_0")
	@RosettaSynonym(value="creationTimestamp", source="DTCC_9_0")
	@RosettaSynonym(value="creationTimestamp", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="expiryTimestamp", source="Rosetta_Workbench")
	@RosettaSynonym(value="expiryTimestamp", source="DTCC_11_0")
	@RosettaSynonym(value="expiryTimestamp", source="DTCC_9_0")
	@RosettaSynonym(value="expiryTimestamp", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="ReceiveTime", source="DTCC_11_0")
	@RosettaSynonym(value="ReceiveTime", source="DTCC_9_0")
	@RosettaSynonym(value="submittedForClearing", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="TxnTm", source="CME_SubmissionIRS_1_0", path="TrdCaptRpt")
	@RosettaSynonym(value="Snt", source="CME_SubmissionIRS_1_0", path="TrdCaptRpt.Hdr")
	@RosettaSynonym(value="TS", source="CME_SubmissionIRS_1_0", path="TrdCaptRpt.TrdRegTS")
	public final ZonedDateTime getDateTime() {
		return dateTime;
	}
	
	/**
	 * The timestamp qualifier is specified through an enumeration because the experience of integrating the DTCC and CME data representations suggests that a wide set of timestamps are currently utilized among service providers, while there is not at present an objective set of criteria that could help suggest a defined set of timestamps as part of the CDM. At some future point, one possible baseline could be developed from the review of the set of timestamps specified across regulatory regimes and regulations (incl. regulations such as high frequency trading). Also, the integration with a further set of implementations and the specification of business workflows such as clearing as part of the CDM development should help confirm the implementation approach in this respect.
	 */
	@RosettaSynonym(value="", source="Rosetta_Workbench")
	@RosettaSynonym(value="", source="DTCC_11_0")
	@RosettaSynonym(value="", source="DTCC_9_0")
	@RosettaSynonym(value="", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="", source="Rosetta_Workbench")
	@RosettaSynonym(value="", source="DTCC_11_0")
	@RosettaSynonym(value="", source="DTCC_9_0")
	@RosettaSynonym(value="", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="", source="Rosetta_Workbench")
	@RosettaSynonym(value="", source="DTCC_11_0")
	@RosettaSynonym(value="", source="DTCC_9_0")
	@RosettaSynonym(value="", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="", source="CME_SubmissionIRS_1_0")
	public final EventTimestampQualificationEnum getQualification() {
		return qualification;
	}
	
	
	@Override
	public RosettaMetaData<? extends EventTimestamp> metaData() {
		return metaData;
	} 
	
	public EventTimestampBuilder toBuilder() {
		EventTimestampBuilder builder = new EventTimestampBuilder();
		ofNullable(getDateTime()).ifPresent(builder::setDateTime);
		ofNullable(getQualification()).ifPresent(builder::setQualification);
		return builder;
	}

	public static EventTimestampBuilder builder() {
		return new EventTimestampBuilder();
	}

	public static  class EventTimestampBuilder extends RosettaModelObjectBuilder {
	
		protected ZonedDateTime dateTime;
		protected EventTimestampQualificationEnum qualification;
	
		public EventTimestampBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends EventTimestamp> metaData() {
			return metaData;
		} 
	
		
		public ZonedDateTime getDateTime() {
			return dateTime;
		}
		
		
		public EventTimestampQualificationEnum getQualification() {
			return qualification;
		}
		
	
		public EventTimestampBuilder setDateTime(ZonedDateTime dateTime) {
			this.dateTime = dateTime;
			return this;
		}
		public EventTimestampBuilder setQualification(EventTimestampQualificationEnum qualification) {
			this.qualification = qualification;
			return this;
		}
		
		public EventTimestamp build() {
			return new EventTimestamp(this);
		}
	
		@Override
		public EventTimestampBuilder prune() {
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getDateTime()!=null) return true;
			if (getQualification()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("dateTime"), ZonedDateTime.class, dateTime, this);
			processor.processBasic(path.newSubPath("qualification"), EventTimestampQualificationEnum.class, qualification, this);
			
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			EventTimestampBuilder _that = (EventTimestampBuilder) o;
		
			if (dateTime != null ? !dateTime.equals(_that.dateTime) : _that.dateTime != null) return false;
			if (qualification != null ? !qualification.equals(_that.qualification) : _that.qualification != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (dateTime != null ? dateTime.hashCode() : 0);
			_result = 31 * _result + (qualification != null ? qualification.getClass().getName().hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "EventTimestampBuilder {" +
				"dateTime=" + this.dateTime + ", " +
				"qualification=" + this.qualification +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("dateTime"), ZonedDateTime.class, dateTime, this);
		processor.processBasic(path.newSubPath("qualification"), EventTimestampQualificationEnum.class, qualification, this);
		
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		EventTimestamp _that = (EventTimestamp) o;
	
		if (dateTime != null ? !dateTime.equals(_that.dateTime) : _that.dateTime != null) return false;
		if (qualification != null ? !qualification.equals(_that.qualification) : _that.qualification != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (dateTime != null ? dateTime.hashCode() : 0);
		_result = 31 * _result + (qualification != null ? qualification.getClass().getName().hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "EventTimestamp {" +
			"dateTime=" + this.dateTime + ", " +
			"qualification=" + this.qualification +
		'}';
	}
}
