package org.isda.cdm;

import java.lang.String;
import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.PartyCustomisedWorkflow;
import org.isda.cdm.WorkflowStatusEnum;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.EventWorkflowMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;

/**
 * A class to specify workflow information, which is conceptually applicable to all lifecycle events.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class EventWorkflow extends RosettaModelObject {
	private final String comment;
	private final List<PartyCustomisedWorkflow> partyCustomisedWorkflow;
	private final List<WorkflowStatusEnum> workflowStatus;
	private static EventWorkflowMeta metaData = new EventWorkflowMeta();
	
	EventWorkflow(EventWorkflowBuilder builder) {
		this.comment = builder.getComment();
		this.partyCustomisedWorkflow = ofNullable(builder.getPartyCustomisedWorkflow()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.workflowStatus = builder.getWorkflowStatus();
	}
	
	/**
	 * A comment field to be associated with the workflow, e.g. to specify why a transaction event was rejected by a party.
	 */
	public final String getComment() {
		return comment;
	}
	
	/**
	 * Workflow data that is specific to certain market participants and is expressed as part of the CDM in a very generic manner, which can be party-specific. The initial use cases have been derived from the CME clearing and the DTCC TIW submissions.
	 */
	@RosettaSynonym(value="RptSide", source="CME_SubmissionIRS_1_0", path="TrdCaptRpt")
	public final List<PartyCustomisedWorkflow> getPartyCustomisedWorkflow() {
		return partyCustomisedWorkflow;
	}
	
	/**
	 * The workflow status indicator, e.g. Accepted, Rejected, ...
	 */
	@RosettaSynonym(value="RptTyp", source="CME_SubmissionIRS_1_0", path="TrdCaptRpt")
	@RosettaSynonym(value="workflowStatus", source="Rosetta_Workbench")
	@RosettaSynonym(value="status", source="CME_ClearedConfirm_1_17", path="clearingConfirmed.trade.tradeHeader")
	public final List<WorkflowStatusEnum> getWorkflowStatus() {
		return workflowStatus;
	}
	
	
	@Override
	public RosettaMetaData<? extends EventWorkflow> metaData() {
		return metaData;
	} 
	
	public EventWorkflowBuilder toBuilder() {
		EventWorkflowBuilder builder = new EventWorkflowBuilder();
		ofNullable(getComment()).ifPresent(builder::setComment);
		ofNullable(getPartyCustomisedWorkflow()).ifPresent(partyCustomisedWorkflow -> partyCustomisedWorkflow.forEach(builder::addPartyCustomisedWorkflow));
		ofNullable(getWorkflowStatus()).ifPresent(workflowStatus -> workflowStatus.forEach(builder::addWorkflowStatus));
		return builder;
	}

	public static EventWorkflowBuilder builder() {
		return new EventWorkflowBuilder();
	}

	public static  class EventWorkflowBuilder extends RosettaModelObjectBuilder {
	
		protected String comment;
		protected List<PartyCustomisedWorkflow.PartyCustomisedWorkflowBuilder> partyCustomisedWorkflow;
		protected List<WorkflowStatusEnum> workflowStatus;
	
		public EventWorkflowBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends EventWorkflow> metaData() {
			return metaData;
		} 
	
		
		public String getComment() {
			return comment;
		}
		
		
		public List<PartyCustomisedWorkflow.PartyCustomisedWorkflowBuilder> getPartyCustomisedWorkflow() {
			return partyCustomisedWorkflow;
		}
		
		public PartyCustomisedWorkflow.PartyCustomisedWorkflowBuilder getOrCreatePartyCustomisedWorkflow(int index) {
			if (partyCustomisedWorkflow==null) {
				this.partyCustomisedWorkflow = new ArrayList<>();
			}
			return getIndex(partyCustomisedWorkflow, index, ()->new PartyCustomisedWorkflow.PartyCustomisedWorkflowBuilder());
		}
		
		
		public List<WorkflowStatusEnum> getWorkflowStatus() {
			return workflowStatus;
		}
		
	
		public EventWorkflowBuilder setComment(String comment) {
			this.comment = comment;
			return this;
		}
		public EventWorkflowBuilder addPartyCustomisedWorkflow(PartyCustomisedWorkflow partyCustomisedWorkflow) {
			if(this.partyCustomisedWorkflow == null){
				this.partyCustomisedWorkflow = new ArrayList<>();
			}
			this.partyCustomisedWorkflow.add(partyCustomisedWorkflow.toBuilder());
			return this;
		}
		public EventWorkflowBuilder addPartyCustomisedWorkflow(PartyCustomisedWorkflow partyCustomisedWorkflow, int _idx) {
			if(this.partyCustomisedWorkflow == null){
				this.partyCustomisedWorkflow = new ArrayList<>();
			}
			getIndex(this.partyCustomisedWorkflow, _idx, () -> partyCustomisedWorkflow.toBuilder());
			this.partyCustomisedWorkflow.set(_idx, partyCustomisedWorkflow.toBuilder());
			return this;
		}
		public EventWorkflowBuilder addPartyCustomisedWorkflow(List<PartyCustomisedWorkflow> partyCustomisedWorkflows) {
			if(this.partyCustomisedWorkflow == null){
				this.partyCustomisedWorkflow = new ArrayList<>();
			}
			for (PartyCustomisedWorkflow toAdd : partyCustomisedWorkflows) {
				this.partyCustomisedWorkflow.add(toAdd.toBuilder());
			}
			return this;
		}
		public EventWorkflowBuilder addPartyCustomisedWorkflowBuilder(PartyCustomisedWorkflow.PartyCustomisedWorkflowBuilder partyCustomisedWorkflow) {
			if(this.partyCustomisedWorkflow == null){
				this.partyCustomisedWorkflow = new ArrayList<>();
			}
			this.partyCustomisedWorkflow.add(partyCustomisedWorkflow);
			return this;
		}
		
		public EventWorkflowBuilder clearPartyCustomisedWorkflow() {
			this.partyCustomisedWorkflow = null;
			return this;
		}
		public EventWorkflowBuilder addWorkflowStatus(WorkflowStatusEnum workflowStatus) {
			if(this.workflowStatus == null){
				this.workflowStatus = new ArrayList<>();
			}
			this.workflowStatus.add(workflowStatus);
			return this;
		}
		public EventWorkflowBuilder addWorkflowStatus(WorkflowStatusEnum workflowStatus, int _idx) {
			if(this.workflowStatus == null){
				this.workflowStatus = new ArrayList<>();
			}
			getIndex(this.workflowStatus, _idx, () -> workflowStatus);
			this.workflowStatus.set(_idx, workflowStatus);
			return this;
		}
		public EventWorkflowBuilder addWorkflowStatus(List<WorkflowStatusEnum> workflowStatuss) {
			if(this.workflowStatus == null){
				this.workflowStatus = new ArrayList<>();
			}
			for (WorkflowStatusEnum toAdd : workflowStatuss) {
				this.workflowStatus.add(toAdd);
			}
			return this;
		}
		
		public EventWorkflowBuilder clearWorkflowStatus() {
			this.workflowStatus = null;
			return this;
		}
		
		public EventWorkflow build() {
			return new EventWorkflow(this);
		}
	
		@Override
		public EventWorkflowBuilder prune() {
			if (partyCustomisedWorkflow!=null) partyCustomisedWorkflow = partyCustomisedWorkflow.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getComment()!=null) return true;
			if (getPartyCustomisedWorkflow()!=null && getPartyCustomisedWorkflow().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getWorkflowStatus()!=null && !getWorkflowStatus().isEmpty()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("comment"), String.class, comment, this);
			processor.processBasic(path.newSubPath("workflowStatus"), WorkflowStatusEnum.class, workflowStatus, this);
			
			processRosetta(path.newSubPath("partyCustomisedWorkflow"), processor, PartyCustomisedWorkflow.class, partyCustomisedWorkflow);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			EventWorkflowBuilder _that = (EventWorkflowBuilder) o;
		
			if (comment != null ? !comment.equals(_that.comment) : _that.comment != null) return false;
			if (!ListEquals.listEquals(partyCustomisedWorkflow, _that.partyCustomisedWorkflow)) return false;
			if (!ListEquals.listEquals(workflowStatus, _that.workflowStatus)) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (comment != null ? comment.hashCode() : 0);
			_result = 31 * _result + (partyCustomisedWorkflow != null ? partyCustomisedWorkflow.hashCode() : 0);
			_result = 31 * _result + (workflowStatus != null ? workflowStatus.stream().map(Object::getClass).map(Class::getName).mapToInt(String::hashCode).sum() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "EventWorkflowBuilder {" +
				"comment=" + this.comment + ", " +
				"partyCustomisedWorkflow=" + this.partyCustomisedWorkflow + ", " +
				"workflowStatus=" + this.workflowStatus +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("comment"), String.class, comment, this);
		workflowStatus.stream().forEach(a->processor.processBasic(path.newSubPath("workflowStatus"), WorkflowStatusEnum.class, a, this));
		
		processRosetta(path.newSubPath("partyCustomisedWorkflow"), processor, PartyCustomisedWorkflow.class, partyCustomisedWorkflow);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		EventWorkflow _that = (EventWorkflow) o;
	
		if (comment != null ? !comment.equals(_that.comment) : _that.comment != null) return false;
		if (!ListEquals.listEquals(partyCustomisedWorkflow, _that.partyCustomisedWorkflow)) return false;
		if (!ListEquals.listEquals(workflowStatus, _that.workflowStatus)) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (comment != null ? comment.hashCode() : 0);
		_result = 31 * _result + (partyCustomisedWorkflow != null ? partyCustomisedWorkflow.hashCode() : 0);
		_result = 31 * _result + (workflowStatus != null ? workflowStatus.stream().map(Object::getClass).map(Class::getName).mapToInt(String::hashCode).sum() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "EventWorkflow {" +
			"comment=" + this.comment + ", " +
			"partyCustomisedWorkflow=" + this.partyCustomisedWorkflow + ", " +
			"workflowStatus=" + this.workflowStatus +
		'}';
	}
}
