package org.isda.cdm;

import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.meta.RosettaMetaData;
import com.rosetta.util.ListEquals;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;
import org.isda.cdm.CrossRate;
import org.isda.cdm.QuotedCurrencyPair;
import org.isda.cdm.meta.ExchangeRateMeta;

import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.BuilderProcessor;
import com.rosetta.model.lib.process.Processor;

import static java.util.Optional.ofNullable;

/**
 * A class that is used for describing the exchange rate for a particular transaction.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="ExchangeRate", source="FpML_5_10")
public class ExchangeRate extends RosettaModelObject {
	private final List<CrossRate> crossRate;
	private final BigDecimal forwardPoints;
	private final BigDecimal pointValue;
	private final QuotedCurrencyPair quotedCurrencyPair;
	private final BigDecimal rate;
	private final BigDecimal spotRate;
	private static ExchangeRateMeta metaData = new ExchangeRateMeta();
	
	ExchangeRate(ExchangeRateBuilder builder) {
		this.crossRate = ofNullable(builder.getCrossRate()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.forwardPoints = builder.getForwardPoints();
		this.pointValue = builder.getPointValue();
		this.quotedCurrencyPair = ofNullable(builder.getQuotedCurrencyPair()).map(f->f.build()).orElse(null);
		this.rate = builder.getRate();
		this.spotRate = builder.getSpotRate();
	}
	
	/**
	 * An optional element that allow for definition of the currency exchange rates used to cross between the traded currencies for non-base currency FX contracts.
	 */
	@RosettaSynonym(value="crossRate", source="FpML_5_10")
	public final List<CrossRate> getCrossRate() {
		return crossRate;
	}
	
	/**
	 * An optional element used for deals consummated in the FX Forwards market. Forward points represent the interest rate differential between the two currencies traded and are quoted as a premium or a discount. Forward points are added to, or subtracted from, the spot rate to create the rate of the forward trade.
	 */
	@RosettaSynonym(value="forwardPoints", source="FpML_5_10")
	public final BigDecimal getForwardPoints() {
		return forwardPoints;
	}
	
	/**
	 * An optional element that documents the size of point (pip) in which a rate was quoted (or in this case, forwardPoints are calculated). Point (pip) size varies by currency pair: major currencies are all traded in points of 0.0001, with the exception of JPY which has a point size of 0.01. The FpML PointValue type is constrained such that the factor can only be values of 10^n, where n &lt;= 0, this constraint is not enforced here and is left to the implementor.
	 */
	@RosettaSynonym(value="pointValue", source="FpML_5_10")
	public final BigDecimal getPointValue() {
		return pointValue;
	}
	
	/**
	 * Defines the two currencies for an FX trade and the quotation relationship between the two currencies.
	 */
	@RosettaSynonym(value="quotedCurrencyPair", source="FpML_5_10")
	public final QuotedCurrencyPair getQuotedCurrencyPair() {
		return quotedCurrencyPair;
	}
	
	/**
	 * The rate of exchange between the two currencies of the leg of a deal. Must be specified with a quote basis.
	 */
	@RosettaSynonym(value="rate", source="FpML_5_10")
	public final BigDecimal getRate() {
		return rate;
	}
	
	/**
	 * An element used for FX forwards and certain types of FX OTC options. For deals consummated in the FX Forwards Market, this represents the current market rate for a particular currency pair. For barrier and digital/binary options, it can be useful to include the spot rate at the time the option was executed to make it easier to know whether the option needs to move &#39;up&#39; or &#39;down&#39; to be triggered.
	 */
	@RosettaSynonym(value="spotRate", source="FpML_5_10")
	public final BigDecimal getSpotRate() {
		return spotRate;
	}
	
	
	@Override
	public RosettaMetaData<? extends ExchangeRate> metaData() {
		return metaData;
	} 
	
	public ExchangeRateBuilder toBuilder() {
		ExchangeRateBuilder builder = new ExchangeRateBuilder();
		ofNullable(getCrossRate()).ifPresent(crossRate -> crossRate.forEach(builder::addCrossRate));
		ofNullable(getForwardPoints()).ifPresent(builder::setForwardPoints);
		ofNullable(getPointValue()).ifPresent(builder::setPointValue);
		ofNullable(getQuotedCurrencyPair()).ifPresent(builder::setQuotedCurrencyPair);
		ofNullable(getRate()).ifPresent(builder::setRate);
		ofNullable(getSpotRate()).ifPresent(builder::setSpotRate);
		return builder;
	}

	public static ExchangeRateBuilder builder() {
		return new ExchangeRateBuilder();
	}

	public static class ExchangeRateBuilder extends RosettaModelObjectBuilder{
	
		protected List<CrossRate.CrossRateBuilder> crossRate;
		protected BigDecimal forwardPoints;
		protected BigDecimal pointValue;
		protected QuotedCurrencyPair.QuotedCurrencyPairBuilder quotedCurrencyPair;
		protected BigDecimal rate;
		protected BigDecimal spotRate;
	
		public ExchangeRateBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends ExchangeRate> metaData() {
			return metaData;
		} 
	
		
		public List<CrossRate.CrossRateBuilder> getCrossRate() {
			return crossRate;
		}
		
		public CrossRate.CrossRateBuilder getOrCreateCrossRate(int index) {
			if (crossRate==null) {
				this.crossRate = new ArrayList<>();
			}
			return getIndex(crossRate, index, ()->new CrossRate.CrossRateBuilder());
		}
		
		
		public BigDecimal getForwardPoints() {
			return forwardPoints;
		}
		
		
		public BigDecimal getPointValue() {
			return pointValue;
		}
		
		
		public QuotedCurrencyPair.QuotedCurrencyPairBuilder getQuotedCurrencyPair() {
			return quotedCurrencyPair;
		}
		
		public QuotedCurrencyPair.QuotedCurrencyPairBuilder getOrCreateQuotedCurrencyPair() {
			if (quotedCurrencyPair!=null) {
				return quotedCurrencyPair;
			}
			else return quotedCurrencyPair = new QuotedCurrencyPair.QuotedCurrencyPairBuilder();
		}
		
		
		public BigDecimal getRate() {
			return rate;
		}
		
		
		public BigDecimal getSpotRate() {
			return spotRate;
		}
		
	
		public ExchangeRateBuilder addCrossRate(CrossRate crossRate) {
			if(this.crossRate == null){
				this.crossRate = new ArrayList<>();
			}
			this.crossRate.add(crossRate.toBuilder());
			return this;
		}
		public ExchangeRateBuilder addCrossRate(CrossRate crossRate, int _idx) {
			if(this.crossRate == null){
				this.crossRate = new ArrayList<>();
			}
			getIndex(this.crossRate, _idx, () -> crossRate.toBuilder());
			this.crossRate.set(_idx, crossRate.toBuilder());
			return this;
		}
		public ExchangeRateBuilder addCrossRate(List<CrossRate> crossRates) {
			if(this.crossRate == null){
				this.crossRate = new ArrayList<>();
			}
			for (CrossRate toAdd : crossRates) {
				this.crossRate.add(toAdd.toBuilder());
			}
			return this;
		}
		public ExchangeRateBuilder addCrossRateBuilder(CrossRate.CrossRateBuilder crossRate) {
			if(this.crossRate == null){
				this.crossRate = new ArrayList<>();
			}
			this.crossRate.add(crossRate);
			return this;
		}
		
		public ExchangeRateBuilder clearCrossRate() {
			this.crossRate = null;
			return this;
		}
		public ExchangeRateBuilder setForwardPoints(BigDecimal forwardPoints) {
			this.forwardPoints = forwardPoints;
			return this;
		}
		public ExchangeRateBuilder setPointValue(BigDecimal pointValue) {
			this.pointValue = pointValue;
			return this;
		}
		public ExchangeRateBuilder setQuotedCurrencyPair(QuotedCurrencyPair quotedCurrencyPair) {
			this.quotedCurrencyPair = quotedCurrencyPair.toBuilder();
			return this;
		}
		public ExchangeRateBuilder setQuotedCurrencyPairBuilder(QuotedCurrencyPair.QuotedCurrencyPairBuilder quotedCurrencyPair) {
			this.quotedCurrencyPair = quotedCurrencyPair;
			return this;
		}
		public ExchangeRateBuilder setRate(BigDecimal rate) {
			this.rate = rate;
			return this;
		}
		public ExchangeRateBuilder setSpotRate(BigDecimal spotRate) {
			this.spotRate = spotRate;
			return this;
		}
	
		public ExchangeRate build() {
			return new ExchangeRate(this);
		}
	
		@Override
		public ExchangeRateBuilder prune() {
			if (crossRate!=null) crossRate = crossRate.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (quotedCurrencyPair!=null && !quotedCurrencyPair.prune().hasData()) quotedCurrencyPair = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getCrossRate()!=null && getCrossRate().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getForwardPoints()!=null) return true;
			if (getPointValue()!=null) return true;
			if (getQuotedCurrencyPair()!=null && getQuotedCurrencyPair().hasData()) return true;
			if (getRate()!=null) return true;
			if (getSpotRate()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("forwardPoints"), BigDecimal.class, forwardPoints, this);
			processor.processBasic(path.newSubPath("pointValue"), BigDecimal.class, pointValue, this);
			processor.processBasic(path.newSubPath("rate"), BigDecimal.class, rate, this);
			processor.processBasic(path.newSubPath("spotRate"), BigDecimal.class, spotRate, this);
			
			processRosetta(path.newSubPath("crossRate"), processor, CrossRate.class, crossRate);
			processRosetta(path.newSubPath("quotedCurrencyPair"), processor, QuotedCurrencyPair.class, quotedCurrencyPair);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			ExchangeRateBuilder _that = (ExchangeRateBuilder) o;
		
			if (!ListEquals.listEquals(crossRate, _that.crossRate)) return false;
			if (forwardPoints != null ? !forwardPoints.equals(_that.forwardPoints) : _that.forwardPoints != null) return false;
			if (pointValue != null ? !pointValue.equals(_that.pointValue) : _that.pointValue != null) return false;
			if (quotedCurrencyPair != null ? !quotedCurrencyPair.equals(_that.quotedCurrencyPair) : _that.quotedCurrencyPair != null) return false;
			if (rate != null ? !rate.equals(_that.rate) : _that.rate != null) return false;
			if (spotRate != null ? !spotRate.equals(_that.spotRate) : _that.spotRate != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (crossRate != null ? crossRate.hashCode() : 0);
			_result = 31 * _result + (forwardPoints != null ? forwardPoints.hashCode() : 0);
			_result = 31 * _result + (pointValue != null ? pointValue.hashCode() : 0);
			_result = 31 * _result + (quotedCurrencyPair != null ? quotedCurrencyPair.hashCode() : 0);
			_result = 31 * _result + (rate != null ? rate.hashCode() : 0);
			_result = 31 * _result + (spotRate != null ? spotRate.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "ExchangeRateBuilder {" +
				"crossRate=" + this.crossRate + ", " +
				"forwardPoints=" + this.forwardPoints + ", " +
				"pointValue=" + this.pointValue + ", " +
				"quotedCurrencyPair=" + this.quotedCurrencyPair + ", " +
				"rate=" + this.rate + ", " +
				"spotRate=" + this.spotRate +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("forwardPoints"), BigDecimal.class, forwardPoints, this);
		processor.processBasic(path.newSubPath("pointValue"), BigDecimal.class, pointValue, this);
		processor.processBasic(path.newSubPath("rate"), BigDecimal.class, rate, this);
		processor.processBasic(path.newSubPath("spotRate"), BigDecimal.class, spotRate, this);
		
		processRosetta(path.newSubPath("crossRate"), processor, CrossRate.class, crossRate);
		processRosetta(path.newSubPath("quotedCurrencyPair"), processor, QuotedCurrencyPair.class, quotedCurrencyPair);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		ExchangeRate _that = (ExchangeRate) o;
	
		if (!ListEquals.listEquals(crossRate, _that.crossRate)) return false;
		if (forwardPoints != null ? !forwardPoints.equals(_that.forwardPoints) : _that.forwardPoints != null) return false;
		if (pointValue != null ? !pointValue.equals(_that.pointValue) : _that.pointValue != null) return false;
		if (quotedCurrencyPair != null ? !quotedCurrencyPair.equals(_that.quotedCurrencyPair) : _that.quotedCurrencyPair != null) return false;
		if (rate != null ? !rate.equals(_that.rate) : _that.rate != null) return false;
		if (spotRate != null ? !spotRate.equals(_that.spotRate) : _that.spotRate != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (crossRate != null ? crossRate.hashCode() : 0);
		_result = 31 * _result + (forwardPoints != null ? forwardPoints.hashCode() : 0);
		_result = 31 * _result + (pointValue != null ? pointValue.hashCode() : 0);
		_result = 31 * _result + (quotedCurrencyPair != null ? quotedCurrencyPair.hashCode() : 0);
		_result = 31 * _result + (rate != null ? rate.hashCode() : 0);
		_result = 31 * _result + (spotRate != null ? spotRate.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "ExchangeRate {" +
			"crossRate=" + this.crossRate + ", " +
			"forwardPoints=" + this.forwardPoints + ", " +
			"pointValue=" + this.pointValue + ", " +
			"quotedCurrencyPair=" + this.quotedCurrencyPair + ", " +
			"rate=" + this.rate + ", " +
			"spotRate=" + this.spotRate +
		'}';
	}
}
