package org.isda.cdm;

import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.ClosedState;
import org.isda.cdm.ExecutionQuantity;
import org.isda.cdm.ExecutionTypeEnum;
import org.isda.cdm.Identifier;
import org.isda.cdm.LegalEntity;
import org.isda.cdm.PartyRole;
import org.isda.cdm.Price;
import org.isda.cdm.Product;
import org.isda.cdm.Quantity;
import org.isda.cdm.SettlementTerms;
import org.isda.cdm.metafields.FieldWithMetaDate;
import org.isda.cdm.metafields.FieldWithMetaDate.FieldWithMetaDateBuilder;
import org.isda.cdm.metafields.MetaFields;
import org.isda.cdm.metafields.ReferenceWithMetaParty;
import org.isda.cdm.metafields.ReferenceWithMetaParty.ReferenceWithMetaPartyBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.ExecutionMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;
		import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import com.rosetta.model.lib.records.Date;

/**
 * A class to specify an execution, which consists essentially in the economic terms which are agreed between the parties, alongside with the qualification of the type of execution. The associated rosettaKey denotes the ability to associate a hash value to the respective Execution instantiations for the purpose of model cross-referencing, in support of functionality such as the event effect and the lineage.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class Execution extends RosettaModelObject implements GlobalKey {
	private final ClosedState closedState;
	private final ExecutionQuantity executionQuantity;
	private final ExecutionTypeEnum executionType;
	private final LegalEntity executionVenue;
	private final List<Identifier> identifier;
	private final MetaFields meta;
	private final List<ReferenceWithMetaParty> party;
	private final List<PartyRole> partyRole;
	private final Price price;
	private final Product product;
	private final Quantity quantity;
	private final SettlementTerms settlementTerms;
	private final FieldWithMetaDate tradeDate;
	private static ExecutionMeta metaData = new ExecutionMeta();
	
	Execution(ExecutionBuilder builder) {
		this.closedState = ofNullable(builder.getClosedState()).map(f->f.build()).orElse(null);
		this.executionQuantity = ofNullable(builder.getExecutionQuantity()).map(f->f.build()).orElse(null);
		this.executionType = builder.getExecutionType();
		this.executionVenue = ofNullable(builder.getExecutionVenue()).map(f->f.build()).orElse(null);
		this.identifier = ofNullable(builder.getIdentifier()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.meta = ofNullable(builder.getMeta()).map(f->f.build()).orElse(null);
		this.party = ofNullable(builder.getParty()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.partyRole = ofNullable(builder.getPartyRole()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.price = ofNullable(builder.getPrice()).map(f->f.build()).orElse(null);
		this.product = ofNullable(builder.getProduct()).map(f->f.build()).orElse(null);
		this.quantity = ofNullable(builder.getQuantity()).map(f->f.build()).orElse(null);
		this.settlementTerms = ofNullable(builder.getSettlementTerms()).map(f->f.build()).orElse(null);
		this.tradeDate = ofNullable(builder.getTradeDate()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The qualification of what led to the execution closure alongside with the dates on which this closure takes effect.
	 */
	@RosettaSynonym(value="closedState", source="Rosetta_Workbench")
	public final ClosedState getClosedState() {
		return closedState;
	}
	
	/**
	 * Complex attribute to specify the quantity of the contractual product that is being executed. This temporarily makes uses of the &#39;ExecutionQuantity&#39; class, until such time when the latter can be renamed as, and supersede, the existing &#39;ContractualQuantity&#39; class. To be later merged with the &#39;quantity&#39; attribute to have consistent mechanism between contractual and non-contractual products.
	 */
	@RosettaSynonym(value="swap", source="Rosetta_Workbench")
	@RosettaSynonym(value="swap", source="CME_SubmissionIRS_1_0", path="Instrmt.SecXML.FpML.trade")
	@RosettaSynonym(value="swap", source="CME_SubmissionIRS_1_0", path="Instrmt.SecXML.FpML.trade.swaption")
	public final ExecutionQuantity getExecutionQuantity() {
		return executionQuantity;
	}
	
	/**
	 * Specifies the type of execution, e.g. via voice, electronically...
	 */
	@RosettaSynonym(value="executionType", source="Rosetta_Workbench")
	@RosettaSynonym(value="VenuTyp", source="CME_SubmissionIRS_1_0")
	public final ExecutionTypeEnum getExecutionType() {
		return executionType;
	}
	
	/**
	 * The execution venue identification, when applicable.
	 */
	public final LegalEntity getExecutionVenue() {
		return executionVenue;
	}
	
	/**
	 * The identifier(s) associated with the execution.
	 */
	@RosettaSynonym(value="partyTradeIdentifier", source="FpML_5_10", path="tradeHeader")
	@RosettaSynonym(value="partyTradeIdentifier", source="DTCC_11_0", path="Submitter")
	@RosettaSynonym(value="partyTradeIdentifier", source="DTCC_9_0", path="Submitter")
	@RosettaSynonym(value="partyTradeIdentifier", source="CME_SubmissionIRS_1_0", path="Instrmt.SecXML.FpML.trade.tradeHeader")
	public final List<Identifier> getIdentifier() {
		return identifier;
	}
	
	public final MetaFields getMeta() {
		return meta;
	}
	
	/**
	 * The party reference is optional because positioned as part of the Event class when the execution is specified as part of such context.
	 */
	@RosettaSynonym(value="party", source="CME_SubmissionIRS_1_0", path="Instrmt.SecXML.FpML")
	public final List<ReferenceWithMetaParty> getParty() {
		return party;
	}
	
	/**
	 * The role(s) that party(ies) may have in relation to the execution, further to the principal parties (i.e payer/receive or buyer/seller) to it.
	 */
	public final List<PartyRole> getPartyRole() {
		return partyRole;
	}
	
	/**
	 * The transacted price of the execution, which is applicable for products such as securities.
	 */
	public final Price getPrice() {
		return price;
	}
	
	/**
	 * The product traded as part of this execution
	 */
	@RosettaSynonym(value="FpML", source="CME_SubmissionIRS_1_0", path="Instrmt.SecXML")
	public final Product getProduct() {
		return product;
	}
	
	/**
	 * The execution quantity when specified as an amount/number of securities or tangible assets.
	 */
	public final Quantity getQuantity() {
		return quantity;
	}
	
	/**
	 * The execution settlement terms, which is applicable for products such as securities
	 */
	public final SettlementTerms getSettlementTerms() {
		return settlementTerms;
	}
	
	/**
	 * The trade/execution date.
	 */
	@RosettaSynonym(value="tradeDate", source="FpML_5_10", path="tradeHeader")
	@RosettaSynonym(value="tradeDate", source="CME_SubmissionIRS_1_0", path="Instrmt.SecXML.FpML.trade.tradeHeader")
	public final FieldWithMetaDate getTradeDate() {
		return tradeDate;
	}
	
	
	@Override
	public RosettaMetaData<? extends Execution> metaData() {
		return metaData;
	} 
	
	public ExecutionBuilder toBuilder() {
		ExecutionBuilder builder = new ExecutionBuilder();
		ofNullable(getClosedState()).ifPresent(builder::setClosedState);
		ofNullable(getExecutionQuantity()).ifPresent(builder::setExecutionQuantity);
		ofNullable(getExecutionType()).ifPresent(builder::setExecutionType);
		ofNullable(getExecutionVenue()).ifPresent(builder::setExecutionVenue);
		ofNullable(getIdentifier()).ifPresent(identifier -> identifier.forEach(builder::addIdentifier));
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		ofNullable(getParty()).ifPresent(party -> party.forEach(builder::addParty));
		ofNullable(getPartyRole()).ifPresent(partyRole -> partyRole.forEach(builder::addPartyRole));
		ofNullable(getPrice()).ifPresent(builder::setPrice);
		ofNullable(getProduct()).ifPresent(builder::setProduct);
		ofNullable(getQuantity()).ifPresent(builder::setQuantity);
		ofNullable(getSettlementTerms()).ifPresent(builder::setSettlementTerms);
		ofNullable(getTradeDate()).ifPresent(builder::setTradeDate);
		return builder;
	}

	public static ExecutionBuilder builder() {
		return new ExecutionBuilder();
	}

	public static  class ExecutionBuilder extends RosettaModelObjectBuilder implements GlobalKeyBuilder<ExecutionBuilder> {
	
		protected ClosedState.ClosedStateBuilder closedState;
		protected ExecutionQuantity.ExecutionQuantityBuilder executionQuantity;
		protected ExecutionTypeEnum executionType;
		protected LegalEntity.LegalEntityBuilder executionVenue;
		protected List<Identifier.IdentifierBuilder> identifier;
		protected MetaFields.MetaFieldsBuilder meta;
		protected List<ReferenceWithMetaPartyBuilder> party;
		protected List<PartyRole.PartyRoleBuilder> partyRole;
		protected Price.PriceBuilder price;
		protected Product.ProductBuilder product;
		protected Quantity.QuantityBuilder quantity;
		protected SettlementTerms.SettlementTermsBuilder settlementTerms;
		protected FieldWithMetaDateBuilder tradeDate;
	
		public ExecutionBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends Execution> metaData() {
			return metaData;
		} 
	
		
		public ClosedState.ClosedStateBuilder getClosedState() {
			return closedState;
		}
		
		public ClosedState.ClosedStateBuilder getOrCreateClosedState() {
			if (closedState!=null) {
				return closedState;
			}
			else return closedState = new ClosedState.ClosedStateBuilder();
		}
		
		
		public ExecutionQuantity.ExecutionQuantityBuilder getExecutionQuantity() {
			return executionQuantity;
		}
		
		public ExecutionQuantity.ExecutionQuantityBuilder getOrCreateExecutionQuantity() {
			if (executionQuantity!=null) {
				return executionQuantity;
			}
			else return executionQuantity = new ExecutionQuantity.ExecutionQuantityBuilder();
		}
		
		
		public ExecutionTypeEnum getExecutionType() {
			return executionType;
		}
		
		
		public LegalEntity.LegalEntityBuilder getExecutionVenue() {
			return executionVenue;
		}
		
		public LegalEntity.LegalEntityBuilder getOrCreateExecutionVenue() {
			if (executionVenue!=null) {
				return executionVenue;
			}
			else return executionVenue = new LegalEntity.LegalEntityBuilder();
		}
		
		
		public List<Identifier.IdentifierBuilder> getIdentifier() {
			return identifier;
		}
		
		public Identifier.IdentifierBuilder getOrCreateIdentifier(int index) {
			if (identifier==null) {
				this.identifier = new ArrayList<>();
			}
			return getIndex(identifier, index, ()->new Identifier.IdentifierBuilder());
		}
		
		
		public MetaFields.MetaFieldsBuilder getMeta() {
			return meta;
		}
		
		public MetaFields.MetaFieldsBuilder getOrCreateMeta() {
			if (meta!=null) {
				return meta;
			}
			else return meta = new MetaFields.MetaFieldsBuilder();
		}
		
		
		public List<ReferenceWithMetaPartyBuilder> getParty() {
			return party;
		}
		
		public ReferenceWithMetaPartyBuilder getOrCreateParty(int index) {
			if (party==null) {
				this.party = new ArrayList<>();
			}
			return getIndex(party, index, ()->new ReferenceWithMetaPartyBuilder());
		}
		
		
		public List<PartyRole.PartyRoleBuilder> getPartyRole() {
			return partyRole;
		}
		
		public PartyRole.PartyRoleBuilder getOrCreatePartyRole(int index) {
			if (partyRole==null) {
				this.partyRole = new ArrayList<>();
			}
			return getIndex(partyRole, index, ()->new PartyRole.PartyRoleBuilder());
		}
		
		
		public Price.PriceBuilder getPrice() {
			return price;
		}
		
		public Price.PriceBuilder getOrCreatePrice() {
			if (price!=null) {
				return price;
			}
			else return price = new Price.PriceBuilder();
		}
		
		
		public Product.ProductBuilder getProduct() {
			return product;
		}
		
		public Product.ProductBuilder getOrCreateProduct() {
			if (product!=null) {
				return product;
			}
			else return product = new Product.ProductBuilder();
		}
		
		
		public Quantity.QuantityBuilder getQuantity() {
			return quantity;
		}
		
		public Quantity.QuantityBuilder getOrCreateQuantity() {
			if (quantity!=null) {
				return quantity;
			}
			else return quantity = new Quantity.QuantityBuilder();
		}
		
		
		public SettlementTerms.SettlementTermsBuilder getSettlementTerms() {
			return settlementTerms;
		}
		
		public SettlementTerms.SettlementTermsBuilder getOrCreateSettlementTerms() {
			if (settlementTerms!=null) {
				return settlementTerms;
			}
			else return settlementTerms = new SettlementTerms.SettlementTermsBuilder();
		}
		
		
		public FieldWithMetaDateBuilder getTradeDate() {
			return tradeDate;
		}
		
		public FieldWithMetaDateBuilder getOrCreateTradeDate() {
			if (tradeDate!=null) {
				return tradeDate;
			}
			else return tradeDate = new FieldWithMetaDateBuilder();
		}
		
	
		public ExecutionBuilder setClosedState(ClosedState closedState) {
			this.closedState = closedState.toBuilder();
			return this;
		}
		public ExecutionBuilder setClosedStateBuilder(ClosedState.ClosedStateBuilder closedState) {
			this.closedState = closedState;
			return this;
		}
		public ExecutionBuilder setExecutionQuantity(ExecutionQuantity executionQuantity) {
			this.executionQuantity = executionQuantity.toBuilder();
			return this;
		}
		public ExecutionBuilder setExecutionQuantityBuilder(ExecutionQuantity.ExecutionQuantityBuilder executionQuantity) {
			this.executionQuantity = executionQuantity;
			return this;
		}
		public ExecutionBuilder setExecutionType(ExecutionTypeEnum executionType) {
			this.executionType = executionType;
			return this;
		}
		public ExecutionBuilder setExecutionVenue(LegalEntity executionVenue) {
			this.executionVenue = executionVenue.toBuilder();
			return this;
		}
		public ExecutionBuilder setExecutionVenueBuilder(LegalEntity.LegalEntityBuilder executionVenue) {
			this.executionVenue = executionVenue;
			return this;
		}
		public ExecutionBuilder addIdentifier(Identifier identifier) {
			if(this.identifier == null){
				this.identifier = new ArrayList<>();
			}
			this.identifier.add(identifier.toBuilder());
			return this;
		}
		public ExecutionBuilder addIdentifier(Identifier identifier, int _idx) {
			if(this.identifier == null){
				this.identifier = new ArrayList<>();
			}
			getIndex(this.identifier, _idx, () -> identifier.toBuilder());
			this.identifier.set(_idx, identifier.toBuilder());
			return this;
		}
		public ExecutionBuilder addIdentifier(List<Identifier> identifiers) {
			if(this.identifier == null){
				this.identifier = new ArrayList<>();
			}
			for (Identifier toAdd : identifiers) {
				this.identifier.add(toAdd.toBuilder());
			}
			return this;
		}
		public ExecutionBuilder addIdentifierBuilder(Identifier.IdentifierBuilder identifier) {
			if(this.identifier == null){
				this.identifier = new ArrayList<>();
			}
			this.identifier.add(identifier);
			return this;
		}
		
		public ExecutionBuilder clearIdentifier() {
			this.identifier = null;
			return this;
		}
		public ExecutionBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		public ExecutionBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		public ExecutionBuilder addParty(ReferenceWithMetaParty party) {
			if(this.party == null){
				this.party = new ArrayList<>();
			}
			this.party.add(party.toBuilder());
			return this;
		}
		public ExecutionBuilder addParty(ReferenceWithMetaParty party, int _idx) {
			if(this.party == null){
				this.party = new ArrayList<>();
			}
			getIndex(this.party, _idx, () -> party.toBuilder());
			this.party.set(_idx, party.toBuilder());
			return this;
		}
		public ExecutionBuilder addParty(List<ReferenceWithMetaParty> partys) {
			if(this.party == null){
				this.party = new ArrayList<>();
			}
			for (ReferenceWithMetaParty toAdd : partys) {
				this.party.add(toAdd.toBuilder());
			}
			return this;
		}
		public ExecutionBuilder addPartyBuilder(ReferenceWithMetaPartyBuilder party) {
			if(this.party == null){
				this.party = new ArrayList<>();
			}
			this.party.add(party);
			return this;
		}
		
		public ExecutionBuilder addPartyRef(Party.PartyBuilder party) {
			return addParty(ReferenceWithMetaParty.builder().setValueBuilder(party).build());
		}
		
		public ExecutionBuilder addPartyRef(Party.PartyBuilder party, int _idx) {
			return addParty(ReferenceWithMetaParty.builder().setValueBuilder(party).build(), _idx);
		}
		
		public ExecutionBuilder addPartyRef(List<Party> partys) {
			for (Party toAdd : partys) {
				addPartyRef(toAdd);
			}
			return this;
		}
		
		public ExecutionBuilder addPartyRef(Party party) {
			return addPartyRef(party.toBuilder());
		}
		
		public ExecutionBuilder clearParty() {
			this.party = null;
			return this;
		}
		public ExecutionBuilder addPartyRole(PartyRole partyRole) {
			if(this.partyRole == null){
				this.partyRole = new ArrayList<>();
			}
			this.partyRole.add(partyRole.toBuilder());
			return this;
		}
		public ExecutionBuilder addPartyRole(PartyRole partyRole, int _idx) {
			if(this.partyRole == null){
				this.partyRole = new ArrayList<>();
			}
			getIndex(this.partyRole, _idx, () -> partyRole.toBuilder());
			this.partyRole.set(_idx, partyRole.toBuilder());
			return this;
		}
		public ExecutionBuilder addPartyRole(List<PartyRole> partyRoles) {
			if(this.partyRole == null){
				this.partyRole = new ArrayList<>();
			}
			for (PartyRole toAdd : partyRoles) {
				this.partyRole.add(toAdd.toBuilder());
			}
			return this;
		}
		public ExecutionBuilder addPartyRoleBuilder(PartyRole.PartyRoleBuilder partyRole) {
			if(this.partyRole == null){
				this.partyRole = new ArrayList<>();
			}
			this.partyRole.add(partyRole);
			return this;
		}
		
		public ExecutionBuilder clearPartyRole() {
			this.partyRole = null;
			return this;
		}
		public ExecutionBuilder setPrice(Price price) {
			this.price = price.toBuilder();
			return this;
		}
		public ExecutionBuilder setPriceBuilder(Price.PriceBuilder price) {
			this.price = price;
			return this;
		}
		public ExecutionBuilder setProduct(Product product) {
			this.product = product.toBuilder();
			return this;
		}
		public ExecutionBuilder setProductBuilder(Product.ProductBuilder product) {
			this.product = product;
			return this;
		}
		public ExecutionBuilder setQuantity(Quantity quantity) {
			this.quantity = quantity.toBuilder();
			return this;
		}
		public ExecutionBuilder setQuantityBuilder(Quantity.QuantityBuilder quantity) {
			this.quantity = quantity;
			return this;
		}
		public ExecutionBuilder setSettlementTerms(SettlementTerms settlementTerms) {
			this.settlementTerms = settlementTerms.toBuilder();
			return this;
		}
		public ExecutionBuilder setSettlementTermsBuilder(SettlementTerms.SettlementTermsBuilder settlementTerms) {
			this.settlementTerms = settlementTerms;
			return this;
		}
		public ExecutionBuilder setTradeDate(FieldWithMetaDate tradeDate) {
			this.tradeDate = tradeDate.toBuilder();
			return this;
		}
		public ExecutionBuilder setTradeDateRef(Date tradeDate) {
			return setTradeDate(FieldWithMetaDate.builder().setValue(tradeDate).build());
		}
		
		public Execution build() {
			return new Execution(this);
		}
	
		@Override
		public ExecutionBuilder prune() {
			if (closedState!=null && !closedState.prune().hasData()) closedState = null;
			if (executionQuantity!=null && !executionQuantity.prune().hasData()) executionQuantity = null;
			if (executionVenue!=null && !executionVenue.prune().hasData()) executionVenue = null;
			if (identifier!=null) identifier = identifier.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (meta!=null && !meta.prune().hasData()) meta = null;
			if (party!=null) party = party.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (partyRole!=null) partyRole = partyRole.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (price!=null && !price.prune().hasData()) price = null;
			if (product!=null && !product.prune().hasData()) product = null;
			if (quantity!=null && !quantity.prune().hasData()) quantity = null;
			if (settlementTerms!=null && !settlementTerms.prune().hasData()) settlementTerms = null;
			if (tradeDate!=null && !tradeDate.prune().hasData()) tradeDate = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getClosedState()!=null && getClosedState().hasData()) return true;
			if (getExecutionQuantity()!=null && getExecutionQuantity().hasData()) return true;
			if (getExecutionType()!=null) return true;
			if (getExecutionVenue()!=null && getExecutionVenue().hasData()) return true;
			if (getIdentifier()!=null && getIdentifier().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getMeta()!=null && getMeta().hasData()) return true;
			if (getParty()!=null && getParty().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getPartyRole()!=null && getPartyRole().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getPrice()!=null && getPrice().hasData()) return true;
			if (getProduct()!=null && getProduct().hasData()) return true;
			if (getQuantity()!=null && getQuantity().hasData()) return true;
			if (getSettlementTerms()!=null && getSettlementTerms().hasData()) return true;
			if (getTradeDate()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("executionType"), ExecutionTypeEnum.class, executionType, this);
			
			processRosetta(path.newSubPath("closedState"), processor, ClosedState.class, closedState);
			processRosetta(path.newSubPath("executionQuantity"), processor, ExecutionQuantity.class, executionQuantity);
			processRosetta(path.newSubPath("executionVenue"), processor, LegalEntity.class, executionVenue);
			processRosetta(path.newSubPath("identifier"), processor, Identifier.class, identifier);
			processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
			processRosetta(path.newSubPath("party"), processor, ReferenceWithMetaParty.class, party);
			processRosetta(path.newSubPath("partyRole"), processor, PartyRole.class, partyRole);
			processRosetta(path.newSubPath("price"), processor, Price.class, price);
			processRosetta(path.newSubPath("product"), processor, Product.class, product);
			processRosetta(path.newSubPath("quantity"), processor, Quantity.class, quantity);
			processRosetta(path.newSubPath("settlementTerms"), processor, SettlementTerms.class, settlementTerms);
			processRosetta(path.newSubPath("tradeDate"), processor, FieldWithMetaDate.class, tradeDate);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			ExecutionBuilder _that = (ExecutionBuilder) o;
		
			if (closedState != null ? !closedState.equals(_that.closedState) : _that.closedState != null) return false;
			if (executionQuantity != null ? !executionQuantity.equals(_that.executionQuantity) : _that.executionQuantity != null) return false;
			if (executionType != null ? !executionType.equals(_that.executionType) : _that.executionType != null) return false;
			if (executionVenue != null ? !executionVenue.equals(_that.executionVenue) : _that.executionVenue != null) return false;
			if (!ListEquals.listEquals(identifier, _that.identifier)) return false;
			if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
			if (!ListEquals.listEquals(party, _that.party)) return false;
			if (!ListEquals.listEquals(partyRole, _that.partyRole)) return false;
			if (price != null ? !price.equals(_that.price) : _that.price != null) return false;
			if (product != null ? !product.equals(_that.product) : _that.product != null) return false;
			if (quantity != null ? !quantity.equals(_that.quantity) : _that.quantity != null) return false;
			if (settlementTerms != null ? !settlementTerms.equals(_that.settlementTerms) : _that.settlementTerms != null) return false;
			if (tradeDate != null ? !tradeDate.equals(_that.tradeDate) : _that.tradeDate != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (closedState != null ? closedState.hashCode() : 0);
			_result = 31 * _result + (executionQuantity != null ? executionQuantity.hashCode() : 0);
			_result = 31 * _result + (executionType != null ? executionType.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (executionVenue != null ? executionVenue.hashCode() : 0);
			_result = 31 * _result + (identifier != null ? identifier.hashCode() : 0);
			_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
			_result = 31 * _result + (party != null ? party.hashCode() : 0);
			_result = 31 * _result + (partyRole != null ? partyRole.hashCode() : 0);
			_result = 31 * _result + (price != null ? price.hashCode() : 0);
			_result = 31 * _result + (product != null ? product.hashCode() : 0);
			_result = 31 * _result + (quantity != null ? quantity.hashCode() : 0);
			_result = 31 * _result + (settlementTerms != null ? settlementTerms.hashCode() : 0);
			_result = 31 * _result + (tradeDate != null ? tradeDate.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "ExecutionBuilder {" +
				"closedState=" + this.closedState + ", " +
				"executionQuantity=" + this.executionQuantity + ", " +
				"executionType=" + this.executionType + ", " +
				"executionVenue=" + this.executionVenue + ", " +
				"identifier=" + this.identifier + ", " +
				"meta=" + this.meta + ", " +
				"party=" + this.party + ", " +
				"partyRole=" + this.partyRole + ", " +
				"price=" + this.price + ", " +
				"product=" + this.product + ", " +
				"quantity=" + this.quantity + ", " +
				"settlementTerms=" + this.settlementTerms + ", " +
				"tradeDate=" + this.tradeDate +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("executionType"), ExecutionTypeEnum.class, executionType, this);
		
		processRosetta(path.newSubPath("closedState"), processor, ClosedState.class, closedState);
		processRosetta(path.newSubPath("executionQuantity"), processor, ExecutionQuantity.class, executionQuantity);
		processRosetta(path.newSubPath("executionVenue"), processor, LegalEntity.class, executionVenue);
		processRosetta(path.newSubPath("identifier"), processor, Identifier.class, identifier);
		processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
		processRosetta(path.newSubPath("party"), processor, ReferenceWithMetaParty.class, party);
		processRosetta(path.newSubPath("partyRole"), processor, PartyRole.class, partyRole);
		processRosetta(path.newSubPath("price"), processor, Price.class, price);
		processRosetta(path.newSubPath("product"), processor, Product.class, product);
		processRosetta(path.newSubPath("quantity"), processor, Quantity.class, quantity);
		processRosetta(path.newSubPath("settlementTerms"), processor, SettlementTerms.class, settlementTerms);
		processRosetta(path.newSubPath("tradeDate"), processor, FieldWithMetaDate.class, tradeDate);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		Execution _that = (Execution) o;
	
		if (closedState != null ? !closedState.equals(_that.closedState) : _that.closedState != null) return false;
		if (executionQuantity != null ? !executionQuantity.equals(_that.executionQuantity) : _that.executionQuantity != null) return false;
		if (executionType != null ? !executionType.equals(_that.executionType) : _that.executionType != null) return false;
		if (executionVenue != null ? !executionVenue.equals(_that.executionVenue) : _that.executionVenue != null) return false;
		if (!ListEquals.listEquals(identifier, _that.identifier)) return false;
		if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
		if (!ListEquals.listEquals(party, _that.party)) return false;
		if (!ListEquals.listEquals(partyRole, _that.partyRole)) return false;
		if (price != null ? !price.equals(_that.price) : _that.price != null) return false;
		if (product != null ? !product.equals(_that.product) : _that.product != null) return false;
		if (quantity != null ? !quantity.equals(_that.quantity) : _that.quantity != null) return false;
		if (settlementTerms != null ? !settlementTerms.equals(_that.settlementTerms) : _that.settlementTerms != null) return false;
		if (tradeDate != null ? !tradeDate.equals(_that.tradeDate) : _that.tradeDate != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (closedState != null ? closedState.hashCode() : 0);
		_result = 31 * _result + (executionQuantity != null ? executionQuantity.hashCode() : 0);
		_result = 31 * _result + (executionType != null ? executionType.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (executionVenue != null ? executionVenue.hashCode() : 0);
		_result = 31 * _result + (identifier != null ? identifier.hashCode() : 0);
		_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
		_result = 31 * _result + (party != null ? party.hashCode() : 0);
		_result = 31 * _result + (partyRole != null ? partyRole.hashCode() : 0);
		_result = 31 * _result + (price != null ? price.hashCode() : 0);
		_result = 31 * _result + (product != null ? product.hashCode() : 0);
		_result = 31 * _result + (quantity != null ? quantity.hashCode() : 0);
		_result = 31 * _result + (settlementTerms != null ? settlementTerms.hashCode() : 0);
		_result = 31 * _result + (tradeDate != null ? tradeDate.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "Execution {" +
			"closedState=" + this.closedState + ", " +
			"executionQuantity=" + this.executionQuantity + ", " +
			"executionType=" + this.executionType + ", " +
			"executionVenue=" + this.executionVenue + ", " +
			"identifier=" + this.identifier + ", " +
			"meta=" + this.meta + ", " +
			"party=" + this.party + ", " +
			"partyRole=" + this.partyRole + ", " +
			"price=" + this.price + ", " +
			"product=" + this.product + ", " +
			"quantity=" + this.quantity + ", " +
			"settlementTerms=" + this.settlementTerms + ", " +
			"tradeDate=" + this.tradeDate +
		'}';
	}
}
