package org.isda.cdm;

import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.ExecutionState;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.ExecutionPrimitiveMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;

/**
 * Specification of the primitive event for an execution, with &#39;after&#39; state being an ExecutionState and the &#39;before&#39; state being Null. It is expected that the &#39;before&#39; state will be adjusted or deprecated once the CDM scope is extended to the pre-execution space.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class ExecutionPrimitive extends RosettaModelObject {
	private final ExecutionState after;
	private final List<ExecutionState> before;
	private static ExecutionPrimitiveMeta metaData = new ExecutionPrimitiveMeta();
	
	ExecutionPrimitive(ExecutionPrimitiveBuilder builder) {
		this.after = ofNullable(builder.getAfter()).map(f->f.build()).orElse(null);
		this.before = ofNullable(builder.getBefore()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
	}
	
	/**
	 * The after state corresponds to the execution between the parties. In the case of an execution on a contractual product, some additional characteristics may need to be specified to get a fully-formed contract, for instance when the executing party acts as an agent, as is the case in an allocation scenario. This is the purpose of the &#39;ContractFormation&#39; primitive event.
	 */
	@RosettaSynonym(value="after", source="Rosetta_Workbench")
	public final ExecutionState getAfter() {
		return after;
	}
	
	/**
	 * The 0 cardinality reflects the fact that there is no execution in the before state of an execution primitive. As noted in the definition associated with the class, this is expected to change once the CDM scope is extended to the pre-execution space.
	 */
	@RosettaSynonym(value="before", source="Rosetta_Workbench")
	public final List<ExecutionState> getBefore() {
		return before;
	}
	
	
	@Override
	public RosettaMetaData<? extends ExecutionPrimitive> metaData() {
		return metaData;
	} 
	
	public ExecutionPrimitiveBuilder toBuilder() {
		ExecutionPrimitiveBuilder builder = new ExecutionPrimitiveBuilder();
		ofNullable(getAfter()).ifPresent(builder::setAfter);
		ofNullable(getBefore()).ifPresent(before -> before.forEach(builder::addBefore));
		return builder;
	}

	public static ExecutionPrimitiveBuilder builder() {
		return new ExecutionPrimitiveBuilder();
	}

	public static  class ExecutionPrimitiveBuilder extends RosettaModelObjectBuilder {
	
		protected ExecutionState.ExecutionStateBuilder after;
		protected List<ExecutionState.ExecutionStateBuilder> before;
	
		public ExecutionPrimitiveBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends ExecutionPrimitive> metaData() {
			return metaData;
		} 
	
		
		public ExecutionState.ExecutionStateBuilder getAfter() {
			return after;
		}
		
		public ExecutionState.ExecutionStateBuilder getOrCreateAfter() {
			if (after!=null) {
				return after;
			}
			else return after = new ExecutionState.ExecutionStateBuilder();
		}
		
		
		public List<ExecutionState.ExecutionStateBuilder> getBefore() {
			return before;
		}
		
		public ExecutionState.ExecutionStateBuilder getOrCreateBefore(int index) {
			if (before==null) {
				this.before = new ArrayList<>();
			}
			return getIndex(before, index, ()->new ExecutionState.ExecutionStateBuilder());
		}
		
	
		public ExecutionPrimitiveBuilder setAfter(ExecutionState after) {
			this.after = after.toBuilder();
			return this;
		}
		public ExecutionPrimitiveBuilder setAfterBuilder(ExecutionState.ExecutionStateBuilder after) {
			this.after = after;
			return this;
		}
		public ExecutionPrimitiveBuilder addBefore(ExecutionState before) {
			if(this.before == null){
				this.before = new ArrayList<>();
			}
			this.before.add(before.toBuilder());
			return this;
		}
		public ExecutionPrimitiveBuilder addBefore(ExecutionState before, int _idx) {
			if(this.before == null){
				this.before = new ArrayList<>();
			}
			getIndex(this.before, _idx, () -> before.toBuilder());
			this.before.set(_idx, before.toBuilder());
			return this;
		}
		public ExecutionPrimitiveBuilder addBefore(List<ExecutionState> befores) {
			if(this.before == null){
				this.before = new ArrayList<>();
			}
			for (ExecutionState toAdd : befores) {
				this.before.add(toAdd.toBuilder());
			}
			return this;
		}
		public ExecutionPrimitiveBuilder addBeforeBuilder(ExecutionState.ExecutionStateBuilder before) {
			if(this.before == null){
				this.before = new ArrayList<>();
			}
			this.before.add(before);
			return this;
		}
		
		public ExecutionPrimitiveBuilder clearBefore() {
			this.before = null;
			return this;
		}
		
		public ExecutionPrimitive build() {
			return new ExecutionPrimitive(this);
		}
	
		@Override
		public ExecutionPrimitiveBuilder prune() {
			if (after!=null && !after.prune().hasData()) after = null;
			if (before!=null) before = before.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getAfter()!=null && getAfter().hasData()) return true;
			if (getBefore()!=null && getBefore().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("after"), processor, ExecutionState.class, after);
			processRosetta(path.newSubPath("before"), processor, ExecutionState.class, before);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			ExecutionPrimitiveBuilder _that = (ExecutionPrimitiveBuilder) o;
		
			if (after != null ? !after.equals(_that.after) : _that.after != null) return false;
			if (!ListEquals.listEquals(before, _that.before)) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (after != null ? after.hashCode() : 0);
			_result = 31 * _result + (before != null ? before.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "ExecutionPrimitiveBuilder {" +
				"after=" + this.after + ", " +
				"before=" + this.before +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("after"), processor, ExecutionState.class, after);
		processRosetta(path.newSubPath("before"), processor, ExecutionState.class, before);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		ExecutionPrimitive _that = (ExecutionPrimitive) o;
	
		if (after != null ? !after.equals(_that.after) : _that.after != null) return false;
		if (!ListEquals.listEquals(before, _that.before)) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (after != null ? after.hashCode() : 0);
		_result = 31 * _result + (before != null ? before.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "ExecutionPrimitive {" +
			"after=" + this.after + ", " +
			"before=" + this.before +
		'}';
	}
}
