package org.isda.cdm;

import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.NotionalAdjustmentEnum;
import org.isda.cdm.QuantityNotation;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.ExecutionQuantityMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;

/**
 * Class to specify the executed quantity of a product, provided as (possibly multiple) quantity notations, plus additional information regarding potential future quantity adjustments. Should be remaned as, and supersede, &#39;ContractualQuantity&#39; once the latter can be deprecated.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class ExecutionQuantity extends RosettaModelObject {
	private final NotionalAdjustmentEnum adjustment;
	private final List<QuantityNotation> quantityNotation;
	private static ExecutionQuantityMeta metaData = new ExecutionQuantityMeta();
	
	ExecutionQuantity(ExecutionQuantityBuilder builder) {
		this.adjustment = builder.getAdjustment();
		this.quantityNotation = ofNullable(builder.getQuantityNotation()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
	}
	
	/**
	 * Specifies the conditions that govern the adjustment to the quantity of a contractual product being traded: e.g. execution, portfolio rebalancing etc. It is typically used in the context of Equity Swaps.
	 */
	@RosettaSynonym(value="notionalAdjustments", source="FpML_5_10")
	public final NotionalAdjustmentEnum getAdjustment() {
		return adjustment;
	}
	
	/**
	 * Quantity of the underlying contractual product that the contract is written on, defined as a single, non-negative number, a (mandatory) notation tag and an (optional) unit. Products, including contractual products, are designed to be defined as a unit in CDM (with quantity schedule and cross-referencing mechanisms handled as part of the underlying payout legs), so that the overall quantity can be abstracted away from the product definition. The notation tag (e.g. &#39;Notional&#39;) is meant to be referenced by the underlying payout legs and used to resolve the product, by fetching the actual amount from the appropriate quantity notation.
	 */
	@RosettaSynonym(value="notional", source="FpML_5_10")
	@RosettaSynonym(value="underlyer", source="FpML_5_10")
	@RosettaSynonym(value="notionalSchedule", source="FpML_5_10", path="swapStream.calculationPeriodAmount.calculation")
	@RosettaSynonym(value="notionalSchedule", source="CME_SubmissionIRS_1_0", path="swapStream.calculationPeriodAmount.calculation")
	@RosettaSynonym(value="notionalSchedule", source="DTCC_11_0", path="swapStream.calculationPeriodAmount.calculation")
	@RosettaSynonym(value="notionalSchedule", source="DTCC_9_0", path="swapStream.calculationPeriodAmount.calculation")
	@RosettaSynonym(value="notionalSchedule", source="CME_ClearedConfirm_1_17", path="swapStream.calculationPeriodAmount.calculation")
	@RosettaSynonym(value="notionalSchedule", source="FpML_5_10", path="capFloorStream.calculationPeriodAmount.calculation")
	@RosettaSynonym(value="notionalSchedule", source="CME_SubmissionIRS_1_0", path="capFloorStream.calculationPeriodAmount.calculation")
	@RosettaSynonym(value="notionalSchedule", source="DTCC_11_0", path="capFloorStream.calculationPeriodAmount.calculation")
	@RosettaSynonym(value="notionalSchedule", source="DTCC_9_0", path="capFloorStream.calculationPeriodAmount.calculation")
	@RosettaSynonym(value="notionalSchedule", source="CME_ClearedConfirm_1_17", path="capFloorStream.calculationPeriodAmount.calculation")
	public final List<QuantityNotation> getQuantityNotation() {
		return quantityNotation;
	}
	
	
	@Override
	public RosettaMetaData<? extends ExecutionQuantity> metaData() {
		return metaData;
	} 
	
	public ExecutionQuantityBuilder toBuilder() {
		ExecutionQuantityBuilder builder = new ExecutionQuantityBuilder();
		ofNullable(getAdjustment()).ifPresent(builder::setAdjustment);
		ofNullable(getQuantityNotation()).ifPresent(quantityNotation -> quantityNotation.forEach(builder::addQuantityNotation));
		return builder;
	}

	public static ExecutionQuantityBuilder builder() {
		return new ExecutionQuantityBuilder();
	}

	public static  class ExecutionQuantityBuilder extends RosettaModelObjectBuilder {
	
		protected NotionalAdjustmentEnum adjustment;
		protected List<QuantityNotation.QuantityNotationBuilder> quantityNotation;
	
		public ExecutionQuantityBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends ExecutionQuantity> metaData() {
			return metaData;
		} 
	
		
		public NotionalAdjustmentEnum getAdjustment() {
			return adjustment;
		}
		
		
		public List<QuantityNotation.QuantityNotationBuilder> getQuantityNotation() {
			return quantityNotation;
		}
		
		public QuantityNotation.QuantityNotationBuilder getOrCreateQuantityNotation(int index) {
			if (quantityNotation==null) {
				this.quantityNotation = new ArrayList<>();
			}
			return getIndex(quantityNotation, index, ()->new QuantityNotation.QuantityNotationBuilder());
		}
		
	
		public ExecutionQuantityBuilder setAdjustment(NotionalAdjustmentEnum adjustment) {
			this.adjustment = adjustment;
			return this;
		}
		public ExecutionQuantityBuilder addQuantityNotation(QuantityNotation quantityNotation) {
			if(this.quantityNotation == null){
				this.quantityNotation = new ArrayList<>();
			}
			this.quantityNotation.add(quantityNotation.toBuilder());
			return this;
		}
		public ExecutionQuantityBuilder addQuantityNotation(QuantityNotation quantityNotation, int _idx) {
			if(this.quantityNotation == null){
				this.quantityNotation = new ArrayList<>();
			}
			getIndex(this.quantityNotation, _idx, () -> quantityNotation.toBuilder());
			this.quantityNotation.set(_idx, quantityNotation.toBuilder());
			return this;
		}
		public ExecutionQuantityBuilder addQuantityNotation(List<QuantityNotation> quantityNotations) {
			if(this.quantityNotation == null){
				this.quantityNotation = new ArrayList<>();
			}
			for (QuantityNotation toAdd : quantityNotations) {
				this.quantityNotation.add(toAdd.toBuilder());
			}
			return this;
		}
		public ExecutionQuantityBuilder addQuantityNotationBuilder(QuantityNotation.QuantityNotationBuilder quantityNotation) {
			if(this.quantityNotation == null){
				this.quantityNotation = new ArrayList<>();
			}
			this.quantityNotation.add(quantityNotation);
			return this;
		}
		
		public ExecutionQuantityBuilder clearQuantityNotation() {
			this.quantityNotation = null;
			return this;
		}
		
		public ExecutionQuantity build() {
			return new ExecutionQuantity(this);
		}
	
		@Override
		public ExecutionQuantityBuilder prune() {
			if (quantityNotation!=null) quantityNotation = quantityNotation.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getAdjustment()!=null) return true;
			if (getQuantityNotation()!=null && getQuantityNotation().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("adjustment"), NotionalAdjustmentEnum.class, adjustment, this);
			
			processRosetta(path.newSubPath("quantityNotation"), processor, QuantityNotation.class, quantityNotation);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			ExecutionQuantityBuilder _that = (ExecutionQuantityBuilder) o;
		
			if (adjustment != null ? !adjustment.equals(_that.adjustment) : _that.adjustment != null) return false;
			if (!ListEquals.listEquals(quantityNotation, _that.quantityNotation)) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (adjustment != null ? adjustment.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (quantityNotation != null ? quantityNotation.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "ExecutionQuantityBuilder {" +
				"adjustment=" + this.adjustment + ", " +
				"quantityNotation=" + this.quantityNotation +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("adjustment"), NotionalAdjustmentEnum.class, adjustment, this);
		
		processRosetta(path.newSubPath("quantityNotation"), processor, QuantityNotation.class, quantityNotation);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		ExecutionQuantity _that = (ExecutionQuantity) o;
	
		if (adjustment != null ? !adjustment.equals(_that.adjustment) : _that.adjustment != null) return false;
		if (!ListEquals.listEquals(quantityNotation, _that.quantityNotation)) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (adjustment != null ? adjustment.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (quantityNotation != null ? quantityNotation.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "ExecutionQuantity {" +
			"adjustment=" + this.adjustment + ", " +
			"quantityNotation=" + this.quantityNotation +
		'}';
	}
}
