package org.isda.cdm;

import java.math.BigDecimal;
import org.isda.cdm.RelativeDateOffset;
import org.isda.cdm.metafields.ReferenceWithMetaAccount;
import org.isda.cdm.metafields.ReferenceWithMetaAccount.ReferenceWithMetaAccountBuilder;
import org.isda.cdm.metafields.ReferenceWithMetaMoney;
import org.isda.cdm.metafields.ReferenceWithMetaMoney.ReferenceWithMetaMoneyBuilder;
import org.isda.cdm.metafields.ReferenceWithMetaParty;
import org.isda.cdm.metafields.ReferenceWithMetaParty.ReferenceWithMetaPartyBuilder;


import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.ExerciseFeeMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.stream.Collectors;
		import static java.util.Optional.ofNullable;
		import java.math.BigDecimal;

/**
 * A class defining the fee payable on exercise of an option. This fee may be defined as an amount or a percentage of the notional exercised. As a difference with FpML, it extends the BuyerSeller class.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="ExerciseFee", source="FpML_5_10")
public class ExerciseFee extends BuyerSeller {
	private final BigDecimal feeAmount;
	private final RelativeDateOffset feePaymentDate;
	private final BigDecimal feeRate;
	private final ReferenceWithMetaMoney notionalReference;
	private static ExerciseFeeMeta metaData = new ExerciseFeeMeta();
	
	ExerciseFee(ExerciseFeeBuilder builder) {
		super(builder);
		this.feeAmount = builder.getFeeAmount();
		this.feePaymentDate = ofNullable(builder.getFeePaymentDate()).map(f->f.build()).orElse(null);
		this.feeRate = builder.getFeeRate();
		this.notionalReference = ofNullable(builder.getNotionalReference()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The amount of fee to be paid on exercise. The fee currency is that of the referenced notional.
	 */
	@RosettaSynonym(value="feeAmount", source="FpML_5_10")
	@RosettaSynonym(value="feeAmount", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="feeAmount", source="DTCC_11_0")
	@RosettaSynonym(value="feeAmount", source="DTCC_9_0")
	@RosettaSynonym(value="feeAmount", source="CME_ClearedConfirm_1_17")
	public final BigDecimal getFeeAmount() {
		return feeAmount;
	}
	
	/**
	 * The date on which exercise fee(s) will be paid. It is specified as a relative date.
	 */
	@RosettaSynonym(value="feePaymentDate", source="FpML_5_10")
	@RosettaSynonym(value="feePaymentDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="feePaymentDate", source="DTCC_11_0")
	@RosettaSynonym(value="feePaymentDate", source="DTCC_9_0")
	@RosettaSynonym(value="feePaymentDate", source="CME_ClearedConfirm_1_17")
	public final RelativeDateOffset getFeePaymentDate() {
		return feePaymentDate;
	}
	
	/**
	 * A fee represented as a percentage of some referenced notional. A percentage of 5% would be represented as 0.05.
	 */
	@RosettaSynonym(value="feeRate", source="FpML_5_10")
	@RosettaSynonym(value="feeRate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="feeRate", source="DTCC_11_0")
	@RosettaSynonym(value="feeRate", source="DTCC_9_0")
	@RosettaSynonym(value="feeRate", source="CME_ClearedConfirm_1_17")
	public final BigDecimal getFeeRate() {
		return feeRate;
	}
	
	/**
	 * A pointer style reference to the associated notional schedule defined elsewhere in the document.
	 */
	@RosettaSynonym(value="notionalReference", source="FpML_5_10")
	@RosettaSynonym(value="notionalReference", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="notionalReference", source="DTCC_11_0")
	@RosettaSynonym(value="notionalReference", source="DTCC_9_0")
	@RosettaSynonym(value="notionalReference", source="CME_ClearedConfirm_1_17")
	public final ReferenceWithMetaMoney getNotionalReference() {
		return notionalReference;
	}
	
	
	@Override
	public RosettaMetaData<? extends ExerciseFee> metaData() {
		return metaData;
	} 
	
	public ExerciseFeeBuilder toBuilder() {
		ExerciseFeeBuilder builder = new ExerciseFeeBuilder();
		ofNullable(getFeeAmount()).ifPresent(builder::setFeeAmount);
		ofNullable(getFeePaymentDate()).ifPresent(builder::setFeePaymentDate);
		ofNullable(getFeeRate()).ifPresent(builder::setFeeRate);
		ofNullable(getNotionalReference()).ifPresent(builder::setNotionalReference);
		ofNullable(getBuyerAccountReference()).ifPresent(builder::setBuyerAccountReference);
		ofNullable(getBuyerPartyReference()).ifPresent(builder::setBuyerPartyReference);
		ofNullable(getSellerAccountReference()).ifPresent(builder::setSellerAccountReference);
		ofNullable(getSellerPartyReference()).ifPresent(builder::setSellerPartyReference);
		return builder;
	}

	public static ExerciseFeeBuilder builder() {
		return new ExerciseFeeBuilder();
	}

	public static  class ExerciseFeeBuilder extends BuyerSellerBuilder {
	
		protected BigDecimal feeAmount;
		protected RelativeDateOffset.RelativeDateOffsetBuilder feePaymentDate;
		protected BigDecimal feeRate;
		protected ReferenceWithMetaMoneyBuilder notionalReference;
	
		public ExerciseFeeBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends ExerciseFee> metaData() {
			return metaData;
		} 
	
		
		public BigDecimal getFeeAmount() {
			return feeAmount;
		}
		
		
		public RelativeDateOffset.RelativeDateOffsetBuilder getFeePaymentDate() {
			return feePaymentDate;
		}
		
		public RelativeDateOffset.RelativeDateOffsetBuilder getOrCreateFeePaymentDate() {
			if (feePaymentDate!=null) {
				return feePaymentDate;
			}
			else return feePaymentDate = new RelativeDateOffset.RelativeDateOffsetBuilder();
		}
		
		
		public BigDecimal getFeeRate() {
			return feeRate;
		}
		
		
		public ReferenceWithMetaMoneyBuilder getNotionalReference() {
			return notionalReference;
		}
		
		public ReferenceWithMetaMoneyBuilder getOrCreateNotionalReference() {
			if (notionalReference!=null) {
				return notionalReference;
			}
			else return notionalReference = new ReferenceWithMetaMoneyBuilder();
		}
		
	
		public ExerciseFeeBuilder setFeeAmount(BigDecimal feeAmount) {
			this.feeAmount = feeAmount;
			return this;
		}
		public ExerciseFeeBuilder setFeePaymentDate(RelativeDateOffset feePaymentDate) {
			this.feePaymentDate = feePaymentDate.toBuilder();
			return this;
		}
		public ExerciseFeeBuilder setFeePaymentDateBuilder(RelativeDateOffset.RelativeDateOffsetBuilder feePaymentDate) {
			this.feePaymentDate = feePaymentDate;
			return this;
		}
		public ExerciseFeeBuilder setFeeRate(BigDecimal feeRate) {
			this.feeRate = feeRate;
			return this;
		}
		public ExerciseFeeBuilder setNotionalReference(ReferenceWithMetaMoney notionalReference) {
			this.notionalReference = notionalReference.toBuilder();
			return this;
		}
		public ExerciseFeeBuilder setNotionalReferenceBuilder(ReferenceWithMetaMoneyBuilder notionalReference) {
			this.notionalReference = notionalReference;
			return this;
		}
		public ExerciseFeeBuilder setNotionalReferenceRef(Money.MoneyBuilder notionalReference) {
			return setNotionalReference(ReferenceWithMetaMoney.builder().setValueBuilder(notionalReference).build());
		}
		public ExerciseFeeBuilder setNotionalReferenceRef(Money notionalReference) {
			return setNotionalReferenceRef(Money.builder());
		}
		@Override public ExerciseFeeBuilder setBuyerAccountReference(ReferenceWithMetaAccount buyerAccountReference) {
			this.buyerAccountReference = buyerAccountReference.toBuilder();
			return this;
		}
		@Override public ExerciseFeeBuilder setBuyerAccountReferenceBuilder(ReferenceWithMetaAccountBuilder buyerAccountReference) {
			this.buyerAccountReference = buyerAccountReference;
			return this;
		}
		@Override public ExerciseFeeBuilder setBuyerAccountReferenceRef(Account.AccountBuilder buyerAccountReference) {
			return setBuyerAccountReference(ReferenceWithMetaAccount.builder().setValueBuilder(buyerAccountReference).build());
		}
		@Override public ExerciseFeeBuilder setBuyerAccountReferenceRef(Account buyerAccountReference) {
			return setBuyerAccountReferenceRef(Account.builder());
		}
		@Override public ExerciseFeeBuilder setBuyerPartyReference(ReferenceWithMetaParty buyerPartyReference) {
			this.buyerPartyReference = buyerPartyReference.toBuilder();
			return this;
		}
		@Override public ExerciseFeeBuilder setBuyerPartyReferenceBuilder(ReferenceWithMetaPartyBuilder buyerPartyReference) {
			this.buyerPartyReference = buyerPartyReference;
			return this;
		}
		@Override public ExerciseFeeBuilder setBuyerPartyReferenceRef(Party.PartyBuilder buyerPartyReference) {
			return setBuyerPartyReference(ReferenceWithMetaParty.builder().setValueBuilder(buyerPartyReference).build());
		}
		@Override public ExerciseFeeBuilder setBuyerPartyReferenceRef(Party buyerPartyReference) {
			return setBuyerPartyReferenceRef(Party.builder());
		}
		@Override public ExerciseFeeBuilder setSellerAccountReference(ReferenceWithMetaAccount sellerAccountReference) {
			this.sellerAccountReference = sellerAccountReference.toBuilder();
			return this;
		}
		@Override public ExerciseFeeBuilder setSellerAccountReferenceBuilder(ReferenceWithMetaAccountBuilder sellerAccountReference) {
			this.sellerAccountReference = sellerAccountReference;
			return this;
		}
		@Override public ExerciseFeeBuilder setSellerAccountReferenceRef(Account.AccountBuilder sellerAccountReference) {
			return setSellerAccountReference(ReferenceWithMetaAccount.builder().setValueBuilder(sellerAccountReference).build());
		}
		@Override public ExerciseFeeBuilder setSellerAccountReferenceRef(Account sellerAccountReference) {
			return setSellerAccountReferenceRef(Account.builder());
		}
		@Override public ExerciseFeeBuilder setSellerPartyReference(ReferenceWithMetaParty sellerPartyReference) {
			this.sellerPartyReference = sellerPartyReference.toBuilder();
			return this;
		}
		@Override public ExerciseFeeBuilder setSellerPartyReferenceBuilder(ReferenceWithMetaPartyBuilder sellerPartyReference) {
			this.sellerPartyReference = sellerPartyReference;
			return this;
		}
		@Override public ExerciseFeeBuilder setSellerPartyReferenceRef(Party.PartyBuilder sellerPartyReference) {
			return setSellerPartyReference(ReferenceWithMetaParty.builder().setValueBuilder(sellerPartyReference).build());
		}
		@Override public ExerciseFeeBuilder setSellerPartyReferenceRef(Party sellerPartyReference) {
			return setSellerPartyReferenceRef(Party.builder());
		}
		
		public ExerciseFee build() {
			return new ExerciseFee(this);
		}
	
		@Override
		public ExerciseFeeBuilder prune() {
			super.prune();
			if (feePaymentDate!=null && !feePaymentDate.prune().hasData()) feePaymentDate = null;
			if (notionalReference!=null && !notionalReference.prune().hasData()) notionalReference = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (super.hasData()) return true;
			if (getFeeAmount()!=null) return true;
			if (getFeePaymentDate()!=null && getFeePaymentDate().hasData()) return true;
			if (getFeeRate()!=null) return true;
			if (getNotionalReference()!=null && getNotionalReference().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
			super.process(path, processor);
		
			processor.processBasic(path.newSubPath("feeAmount"), BigDecimal.class, feeAmount, this);
			processor.processBasic(path.newSubPath("feeRate"), BigDecimal.class, feeRate, this);
			
			processRosetta(path.newSubPath("feePaymentDate"), processor, RelativeDateOffset.class, feePaymentDate);
			processRosetta(path.newSubPath("notionalReference"), processor, ReferenceWithMetaMoney.class, notionalReference);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
			if (!super.equals(o)) return false;
		
			ExerciseFeeBuilder _that = (ExerciseFeeBuilder) o;
		
			if (feeAmount != null ? !feeAmount.equals(_that.feeAmount) : _that.feeAmount != null) return false;
			if (feePaymentDate != null ? !feePaymentDate.equals(_that.feePaymentDate) : _that.feePaymentDate != null) return false;
			if (feeRate != null ? !feeRate.equals(_that.feeRate) : _that.feeRate != null) return false;
			if (notionalReference != null ? !notionalReference.equals(_that.notionalReference) : _that.notionalReference != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = super.hashCode();
			_result = 31 * _result + (feeAmount != null ? feeAmount.hashCode() : 0);
			_result = 31 * _result + (feePaymentDate != null ? feePaymentDate.hashCode() : 0);
			_result = 31 * _result + (feeRate != null ? feeRate.hashCode() : 0);
			_result = 31 * _result + (notionalReference != null ? notionalReference.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "ExerciseFeeBuilder {" +
				"feeAmount=" + this.feeAmount + ", " +
				"feePaymentDate=" + this.feePaymentDate + ", " +
				"feeRate=" + this.feeRate + ", " +
				"notionalReference=" + this.notionalReference +
			'}' + " " + super.toString();
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		super.process(path, processor);
		
		processor.processBasic(path.newSubPath("feeAmount"), BigDecimal.class, feeAmount, this);
		processor.processBasic(path.newSubPath("feeRate"), BigDecimal.class, feeRate, this);
		
		processRosetta(path.newSubPath("feePaymentDate"), processor, RelativeDateOffset.class, feePaymentDate);
		processRosetta(path.newSubPath("notionalReference"), processor, ReferenceWithMetaMoney.class, notionalReference);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
		if (!super.equals(o)) return false;
	
		ExerciseFee _that = (ExerciseFee) o;
	
		if (feeAmount != null ? !feeAmount.equals(_that.feeAmount) : _that.feeAmount != null) return false;
		if (feePaymentDate != null ? !feePaymentDate.equals(_that.feePaymentDate) : _that.feePaymentDate != null) return false;
		if (feeRate != null ? !feeRate.equals(_that.feeRate) : _that.feeRate != null) return false;
		if (notionalReference != null ? !notionalReference.equals(_that.notionalReference) : _that.notionalReference != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = super.hashCode();
		_result = 31 * _result + (feeAmount != null ? feeAmount.hashCode() : 0);
		_result = 31 * _result + (feePaymentDate != null ? feePaymentDate.hashCode() : 0);
		_result = 31 * _result + (feeRate != null ? feeRate.hashCode() : 0);
		_result = 31 * _result + (notionalReference != null ? notionalReference.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "ExerciseFee {" +
			"feeAmount=" + this.feeAmount + ", " +
			"feePaymentDate=" + this.feePaymentDate + ", " +
			"feeRate=" + this.feeRate + ", " +
			"notionalReference=" + this.notionalReference +
		'}' + " " + super.toString();
	}
}
