package org.isda.cdm;

import org.isda.cdm.AmountSchedule;
import org.isda.cdm.RelativeDateOffset;
import org.isda.cdm.Schedule;
import org.isda.cdm.metafields.ReferenceWithMetaAccount;
import org.isda.cdm.metafields.ReferenceWithMetaAccount.ReferenceWithMetaAccountBuilder;
import org.isda.cdm.metafields.ReferenceWithMetaMoney;
import org.isda.cdm.metafields.ReferenceWithMetaMoney.ReferenceWithMetaMoneyBuilder;
import org.isda.cdm.metafields.ReferenceWithMetaParty;
import org.isda.cdm.metafields.ReferenceWithMetaParty.ReferenceWithMetaPartyBuilder;


import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.ExerciseFeeScheduleMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.stream.Collectors;
		import static java.util.Optional.ofNullable;

/**
 * A class to define a fee or schedule of fees to be payable on the exercise of an option. This fee may be defined as an amount or a percentage of the notional exercised. As a difference with FpML, it extends the BuyerSeller class.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="ExerciseFeeSchedule", source="FpML_5_10")
public class ExerciseFeeSchedule extends PayerReceiver {
	private final AmountSchedule feeAmountSchedule;
	private final RelativeDateOffset feePaymentDate;
	private final Schedule feeRateSchedule;
	private final ReferenceWithMetaMoney notionalReference;
	private static ExerciseFeeScheduleMeta metaData = new ExerciseFeeScheduleMeta();
	
	ExerciseFeeSchedule(ExerciseFeeScheduleBuilder builder) {
		super(builder);
		this.feeAmountSchedule = ofNullable(builder.getFeeAmountSchedule()).map(f->f.build()).orElse(null);
		this.feePaymentDate = ofNullable(builder.getFeePaymentDate()).map(f->f.build()).orElse(null);
		this.feeRateSchedule = ofNullable(builder.getFeeRateSchedule()).map(f->f.build()).orElse(null);
		this.notionalReference = ofNullable(builder.getNotionalReference()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The exercise fee amount schedule. The fees are expressed as currency amounts. The currency of the fee is assumed to be that of the notional schedule referenced.
	 */
	@RosettaSynonym(value="feeAmountSchedule", source="FpML_5_10")
	@RosettaSynonym(value="feeAmountSchedule", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="feeAmountSchedule", source="DTCC_11_0")
	@RosettaSynonym(value="feeAmountSchedule", source="DTCC_9_0")
	@RosettaSynonym(value="feeAmountSchedule", source="CME_ClearedConfirm_1_17")
	public final AmountSchedule getFeeAmountSchedule() {
		return feeAmountSchedule;
	}
	
	/**
	 * The date on which exercise fee(s) will be paid. It is specified as a relative date.
	 */
	@RosettaSynonym(value="feePaymentDate", source="FpML_5_10")
	@RosettaSynonym(value="feePaymentDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="feePaymentDate", source="DTCC_11_0")
	@RosettaSynonym(value="feePaymentDate", source="DTCC_9_0")
	@RosettaSynonym(value="feePaymentDate", source="CME_ClearedConfirm_1_17")
	public final RelativeDateOffset getFeePaymentDate() {
		return feePaymentDate;
	}
	
	/**
	 * The exercise free rate schedule. The fees are expressed as percentage rates of the notional being exercised. The currency of the fee is assumed to be that of the notional schedule referenced.
	 */
	@RosettaSynonym(value="feeRateSchedule", source="FpML_5_10")
	@RosettaSynonym(value="feeRateSchedule", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="feeRateSchedule", source="DTCC_11_0")
	@RosettaSynonym(value="feeRateSchedule", source="DTCC_9_0")
	@RosettaSynonym(value="feeRateSchedule", source="CME_ClearedConfirm_1_17")
	public final Schedule getFeeRateSchedule() {
		return feeRateSchedule;
	}
	
	/**
	 * A pointer style reference to the associated notional schedule defined elsewhere in the document.
	 */
	@RosettaSynonym(value="notionalReference", source="FpML_5_10")
	@RosettaSynonym(value="notionalReference", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="notionalReference", source="DTCC_11_0")
	@RosettaSynonym(value="notionalReference", source="DTCC_9_0")
	@RosettaSynonym(value="notionalReference", source="CME_ClearedConfirm_1_17")
	public final ReferenceWithMetaMoney getNotionalReference() {
		return notionalReference;
	}
	
	
	@Override
	public RosettaMetaData<? extends ExerciseFeeSchedule> metaData() {
		return metaData;
	} 
	
	public ExerciseFeeScheduleBuilder toBuilder() {
		ExerciseFeeScheduleBuilder builder = new ExerciseFeeScheduleBuilder();
		ofNullable(getFeeAmountSchedule()).ifPresent(builder::setFeeAmountSchedule);
		ofNullable(getFeePaymentDate()).ifPresent(builder::setFeePaymentDate);
		ofNullable(getFeeRateSchedule()).ifPresent(builder::setFeeRateSchedule);
		ofNullable(getNotionalReference()).ifPresent(builder::setNotionalReference);
		ofNullable(getPayerAccountReference()).ifPresent(builder::setPayerAccountReference);
		ofNullable(getPayerPartyReference()).ifPresent(builder::setPayerPartyReference);
		ofNullable(getReceiverAccountReference()).ifPresent(builder::setReceiverAccountReference);
		ofNullable(getReceiverPartyReference()).ifPresent(builder::setReceiverPartyReference);
		return builder;
	}

	public static ExerciseFeeScheduleBuilder builder() {
		return new ExerciseFeeScheduleBuilder();
	}

	public static  class ExerciseFeeScheduleBuilder extends PayerReceiverBuilder {
	
		protected AmountSchedule.AmountScheduleBuilder feeAmountSchedule;
		protected RelativeDateOffset.RelativeDateOffsetBuilder feePaymentDate;
		protected Schedule.ScheduleBuilder feeRateSchedule;
		protected ReferenceWithMetaMoneyBuilder notionalReference;
	
		public ExerciseFeeScheduleBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends ExerciseFeeSchedule> metaData() {
			return metaData;
		} 
	
		
		public AmountSchedule.AmountScheduleBuilder getFeeAmountSchedule() {
			return feeAmountSchedule;
		}
		
		public AmountSchedule.AmountScheduleBuilder getOrCreateFeeAmountSchedule() {
			if (feeAmountSchedule!=null) {
				return feeAmountSchedule;
			}
			else return feeAmountSchedule = new AmountSchedule.AmountScheduleBuilder();
		}
		
		
		public RelativeDateOffset.RelativeDateOffsetBuilder getFeePaymentDate() {
			return feePaymentDate;
		}
		
		public RelativeDateOffset.RelativeDateOffsetBuilder getOrCreateFeePaymentDate() {
			if (feePaymentDate!=null) {
				return feePaymentDate;
			}
			else return feePaymentDate = new RelativeDateOffset.RelativeDateOffsetBuilder();
		}
		
		
		public Schedule.ScheduleBuilder getFeeRateSchedule() {
			return feeRateSchedule;
		}
		
		public Schedule.ScheduleBuilder getOrCreateFeeRateSchedule() {
			if (feeRateSchedule!=null) {
				return feeRateSchedule;
			}
			else return feeRateSchedule = new Schedule.ScheduleBuilder();
		}
		
		
		public ReferenceWithMetaMoneyBuilder getNotionalReference() {
			return notionalReference;
		}
		
		public ReferenceWithMetaMoneyBuilder getOrCreateNotionalReference() {
			if (notionalReference!=null) {
				return notionalReference;
			}
			else return notionalReference = new ReferenceWithMetaMoneyBuilder();
		}
		
	
		public ExerciseFeeScheduleBuilder setFeeAmountSchedule(AmountSchedule feeAmountSchedule) {
			this.feeAmountSchedule = feeAmountSchedule.toBuilder();
			return this;
		}
		public ExerciseFeeScheduleBuilder setFeeAmountScheduleBuilder(AmountSchedule.AmountScheduleBuilder feeAmountSchedule) {
			this.feeAmountSchedule = feeAmountSchedule;
			return this;
		}
		public ExerciseFeeScheduleBuilder setFeePaymentDate(RelativeDateOffset feePaymentDate) {
			this.feePaymentDate = feePaymentDate.toBuilder();
			return this;
		}
		public ExerciseFeeScheduleBuilder setFeePaymentDateBuilder(RelativeDateOffset.RelativeDateOffsetBuilder feePaymentDate) {
			this.feePaymentDate = feePaymentDate;
			return this;
		}
		public ExerciseFeeScheduleBuilder setFeeRateSchedule(Schedule feeRateSchedule) {
			this.feeRateSchedule = feeRateSchedule.toBuilder();
			return this;
		}
		public ExerciseFeeScheduleBuilder setFeeRateScheduleBuilder(Schedule.ScheduleBuilder feeRateSchedule) {
			this.feeRateSchedule = feeRateSchedule;
			return this;
		}
		public ExerciseFeeScheduleBuilder setNotionalReference(ReferenceWithMetaMoney notionalReference) {
			this.notionalReference = notionalReference.toBuilder();
			return this;
		}
		public ExerciseFeeScheduleBuilder setNotionalReferenceBuilder(ReferenceWithMetaMoneyBuilder notionalReference) {
			this.notionalReference = notionalReference;
			return this;
		}
		public ExerciseFeeScheduleBuilder setNotionalReferenceRef(Money.MoneyBuilder notionalReference) {
			return setNotionalReference(ReferenceWithMetaMoney.builder().setValueBuilder(notionalReference).build());
		}
		public ExerciseFeeScheduleBuilder setNotionalReferenceRef(Money notionalReference) {
			return setNotionalReferenceRef(Money.builder());
		}
		@Override public ExerciseFeeScheduleBuilder setPayerAccountReference(ReferenceWithMetaAccount payerAccountReference) {
			this.payerAccountReference = payerAccountReference.toBuilder();
			return this;
		}
		@Override public ExerciseFeeScheduleBuilder setPayerAccountReferenceBuilder(ReferenceWithMetaAccountBuilder payerAccountReference) {
			this.payerAccountReference = payerAccountReference;
			return this;
		}
		@Override public ExerciseFeeScheduleBuilder setPayerAccountReferenceRef(Account.AccountBuilder payerAccountReference) {
			return setPayerAccountReference(ReferenceWithMetaAccount.builder().setValueBuilder(payerAccountReference).build());
		}
		@Override public ExerciseFeeScheduleBuilder setPayerAccountReferenceRef(Account payerAccountReference) {
			return setPayerAccountReferenceRef(Account.builder());
		}
		@Override public ExerciseFeeScheduleBuilder setPayerPartyReference(ReferenceWithMetaParty payerPartyReference) {
			this.payerPartyReference = payerPartyReference.toBuilder();
			return this;
		}
		@Override public ExerciseFeeScheduleBuilder setPayerPartyReferenceBuilder(ReferenceWithMetaPartyBuilder payerPartyReference) {
			this.payerPartyReference = payerPartyReference;
			return this;
		}
		@Override public ExerciseFeeScheduleBuilder setPayerPartyReferenceRef(Party.PartyBuilder payerPartyReference) {
			return setPayerPartyReference(ReferenceWithMetaParty.builder().setValueBuilder(payerPartyReference).build());
		}
		@Override public ExerciseFeeScheduleBuilder setPayerPartyReferenceRef(Party payerPartyReference) {
			return setPayerPartyReferenceRef(Party.builder());
		}
		@Override public ExerciseFeeScheduleBuilder setReceiverAccountReference(ReferenceWithMetaAccount receiverAccountReference) {
			this.receiverAccountReference = receiverAccountReference.toBuilder();
			return this;
		}
		@Override public ExerciseFeeScheduleBuilder setReceiverAccountReferenceBuilder(ReferenceWithMetaAccountBuilder receiverAccountReference) {
			this.receiverAccountReference = receiverAccountReference;
			return this;
		}
		@Override public ExerciseFeeScheduleBuilder setReceiverAccountReferenceRef(Account.AccountBuilder receiverAccountReference) {
			return setReceiverAccountReference(ReferenceWithMetaAccount.builder().setValueBuilder(receiverAccountReference).build());
		}
		@Override public ExerciseFeeScheduleBuilder setReceiverAccountReferenceRef(Account receiverAccountReference) {
			return setReceiverAccountReferenceRef(Account.builder());
		}
		@Override public ExerciseFeeScheduleBuilder setReceiverPartyReference(ReferenceWithMetaParty receiverPartyReference) {
			this.receiverPartyReference = receiverPartyReference.toBuilder();
			return this;
		}
		@Override public ExerciseFeeScheduleBuilder setReceiverPartyReferenceBuilder(ReferenceWithMetaPartyBuilder receiverPartyReference) {
			this.receiverPartyReference = receiverPartyReference;
			return this;
		}
		@Override public ExerciseFeeScheduleBuilder setReceiverPartyReferenceRef(Party.PartyBuilder receiverPartyReference) {
			return setReceiverPartyReference(ReferenceWithMetaParty.builder().setValueBuilder(receiverPartyReference).build());
		}
		@Override public ExerciseFeeScheduleBuilder setReceiverPartyReferenceRef(Party receiverPartyReference) {
			return setReceiverPartyReferenceRef(Party.builder());
		}
		
		public ExerciseFeeSchedule build() {
			return new ExerciseFeeSchedule(this);
		}
	
		@Override
		public ExerciseFeeScheduleBuilder prune() {
			super.prune();
			if (feeAmountSchedule!=null && !feeAmountSchedule.prune().hasData()) feeAmountSchedule = null;
			if (feePaymentDate!=null && !feePaymentDate.prune().hasData()) feePaymentDate = null;
			if (feeRateSchedule!=null && !feeRateSchedule.prune().hasData()) feeRateSchedule = null;
			if (notionalReference!=null && !notionalReference.prune().hasData()) notionalReference = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (super.hasData()) return true;
			if (getFeeAmountSchedule()!=null && getFeeAmountSchedule().hasData()) return true;
			if (getFeePaymentDate()!=null && getFeePaymentDate().hasData()) return true;
			if (getFeeRateSchedule()!=null && getFeeRateSchedule().hasData()) return true;
			if (getNotionalReference()!=null && getNotionalReference().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
			super.process(path, processor);
		
			
			processRosetta(path.newSubPath("feeAmountSchedule"), processor, AmountSchedule.class, feeAmountSchedule);
			processRosetta(path.newSubPath("feePaymentDate"), processor, RelativeDateOffset.class, feePaymentDate);
			processRosetta(path.newSubPath("feeRateSchedule"), processor, Schedule.class, feeRateSchedule);
			processRosetta(path.newSubPath("notionalReference"), processor, ReferenceWithMetaMoney.class, notionalReference);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
			if (!super.equals(o)) return false;
		
			ExerciseFeeScheduleBuilder _that = (ExerciseFeeScheduleBuilder) o;
		
			if (feeAmountSchedule != null ? !feeAmountSchedule.equals(_that.feeAmountSchedule) : _that.feeAmountSchedule != null) return false;
			if (feePaymentDate != null ? !feePaymentDate.equals(_that.feePaymentDate) : _that.feePaymentDate != null) return false;
			if (feeRateSchedule != null ? !feeRateSchedule.equals(_that.feeRateSchedule) : _that.feeRateSchedule != null) return false;
			if (notionalReference != null ? !notionalReference.equals(_that.notionalReference) : _that.notionalReference != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = super.hashCode();
			_result = 31 * _result + (feeAmountSchedule != null ? feeAmountSchedule.hashCode() : 0);
			_result = 31 * _result + (feePaymentDate != null ? feePaymentDate.hashCode() : 0);
			_result = 31 * _result + (feeRateSchedule != null ? feeRateSchedule.hashCode() : 0);
			_result = 31 * _result + (notionalReference != null ? notionalReference.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "ExerciseFeeScheduleBuilder {" +
				"feeAmountSchedule=" + this.feeAmountSchedule + ", " +
				"feePaymentDate=" + this.feePaymentDate + ", " +
				"feeRateSchedule=" + this.feeRateSchedule + ", " +
				"notionalReference=" + this.notionalReference +
			'}' + " " + super.toString();
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		super.process(path, processor);
		
		
		processRosetta(path.newSubPath("feeAmountSchedule"), processor, AmountSchedule.class, feeAmountSchedule);
		processRosetta(path.newSubPath("feePaymentDate"), processor, RelativeDateOffset.class, feePaymentDate);
		processRosetta(path.newSubPath("feeRateSchedule"), processor, Schedule.class, feeRateSchedule);
		processRosetta(path.newSubPath("notionalReference"), processor, ReferenceWithMetaMoney.class, notionalReference);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
		if (!super.equals(o)) return false;
	
		ExerciseFeeSchedule _that = (ExerciseFeeSchedule) o;
	
		if (feeAmountSchedule != null ? !feeAmountSchedule.equals(_that.feeAmountSchedule) : _that.feeAmountSchedule != null) return false;
		if (feePaymentDate != null ? !feePaymentDate.equals(_that.feePaymentDate) : _that.feePaymentDate != null) return false;
		if (feeRateSchedule != null ? !feeRateSchedule.equals(_that.feeRateSchedule) : _that.feeRateSchedule != null) return false;
		if (notionalReference != null ? !notionalReference.equals(_that.notionalReference) : _that.notionalReference != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = super.hashCode();
		_result = 31 * _result + (feeAmountSchedule != null ? feeAmountSchedule.hashCode() : 0);
		_result = 31 * _result + (feePaymentDate != null ? feePaymentDate.hashCode() : 0);
		_result = 31 * _result + (feeRateSchedule != null ? feeRateSchedule.hashCode() : 0);
		_result = 31 * _result + (notionalReference != null ? notionalReference.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "ExerciseFeeSchedule {" +
			"feeAmountSchedule=" + this.feeAmountSchedule + ", " +
			"feePaymentDate=" + this.feePaymentDate + ", " +
			"feeRateSchedule=" + this.feeRateSchedule + ", " +
			"notionalReference=" + this.notionalReference +
		'}' + " " + super.toString();
	}
}
