package org.isda.cdm;

import org.isda.cdm.Period;
import org.isda.cdm.metafields.MetaFields;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.ExercisePeriodMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * This defines the time interval to the start of the exercise period, i.e. the earliest exercise date, and the frequency of subsequent exercise dates (if any).
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="ExercisePeriod", source="FpML_5_10")
@RosettaSynonym(value="ExercisePeriod", source="CME_SubmissionIRS_1_0")
@RosettaSynonym(value="ExercisePeriod", source="DTCC_11_0")
@RosettaSynonym(value="ExercisePeriod", source="DTCC_9_0")
@RosettaSynonym(value="ExercisePeriod", source="CME_ClearedConfirm_1_17")
public class ExercisePeriod extends RosettaModelObject implements GlobalKey {
	private final Period earliestExerciseDateTenor;
	private final Period exerciseFrequency;
	private final MetaFields meta;
	private static ExercisePeriodMeta metaData = new ExercisePeriodMeta();
	
	ExercisePeriod(ExercisePeriodBuilder builder) {
		this.earliestExerciseDateTenor = ofNullable(builder.getEarliestExerciseDateTenor()).map(f->f.build()).orElse(null);
		this.exerciseFrequency = ofNullable(builder.getExerciseFrequency()).map(f->f.build()).orElse(null);
		this.meta = ofNullable(builder.getMeta()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The time interval to the first (and possibly only) exercise date in the exercise period.
	 */
	@RosettaSynonym(value="earliestExerciseDateTenor", source="FpML_5_10")
	@RosettaSynonym(value="earliestExerciseDateTenor", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="earliestExerciseDateTenor", source="DTCC_11_0")
	@RosettaSynonym(value="earliestExerciseDateTenor", source="DTCC_9_0")
	@RosettaSynonym(value="earliestExerciseDateTenor", source="CME_ClearedConfirm_1_17")
	public final Period getEarliestExerciseDateTenor() {
		return earliestExerciseDateTenor;
	}
	
	/**
	 * The frequency of subsequent exercise dates in the exercise period following the earliest exercise date. An interval of 1 day should be used to indicate an American style exercise period.
	 */
	@RosettaSynonym(value="exerciseFrequency", source="FpML_5_10")
	@RosettaSynonym(value="exerciseFrequency", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="exerciseFrequency", source="DTCC_11_0")
	@RosettaSynonym(value="exerciseFrequency", source="DTCC_9_0")
	@RosettaSynonym(value="exerciseFrequency", source="CME_ClearedConfirm_1_17")
	public final Period getExerciseFrequency() {
		return exerciseFrequency;
	}
	
	public final MetaFields getMeta() {
		return meta;
	}
	
	
	@Override
	public RosettaMetaData<? extends ExercisePeriod> metaData() {
		return metaData;
	} 
	
	public ExercisePeriodBuilder toBuilder() {
		ExercisePeriodBuilder builder = new ExercisePeriodBuilder();
		ofNullable(getEarliestExerciseDateTenor()).ifPresent(builder::setEarliestExerciseDateTenor);
		ofNullable(getExerciseFrequency()).ifPresent(builder::setExerciseFrequency);
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		return builder;
	}

	public static ExercisePeriodBuilder builder() {
		return new ExercisePeriodBuilder();
	}

	public static  class ExercisePeriodBuilder extends RosettaModelObjectBuilder implements GlobalKeyBuilder<ExercisePeriodBuilder> {
	
		protected Period.PeriodBuilder earliestExerciseDateTenor;
		protected Period.PeriodBuilder exerciseFrequency;
		protected MetaFields.MetaFieldsBuilder meta;
	
		public ExercisePeriodBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends ExercisePeriod> metaData() {
			return metaData;
		} 
	
		
		public Period.PeriodBuilder getEarliestExerciseDateTenor() {
			return earliestExerciseDateTenor;
		}
		
		public Period.PeriodBuilder getOrCreateEarliestExerciseDateTenor() {
			if (earliestExerciseDateTenor!=null) {
				return earliestExerciseDateTenor;
			}
			else return earliestExerciseDateTenor = new Period.PeriodBuilder();
		}
		
		
		public Period.PeriodBuilder getExerciseFrequency() {
			return exerciseFrequency;
		}
		
		public Period.PeriodBuilder getOrCreateExerciseFrequency() {
			if (exerciseFrequency!=null) {
				return exerciseFrequency;
			}
			else return exerciseFrequency = new Period.PeriodBuilder();
		}
		
		
		public MetaFields.MetaFieldsBuilder getMeta() {
			return meta;
		}
		
		public MetaFields.MetaFieldsBuilder getOrCreateMeta() {
			if (meta!=null) {
				return meta;
			}
			else return meta = new MetaFields.MetaFieldsBuilder();
		}
		
	
		public ExercisePeriodBuilder setEarliestExerciseDateTenor(Period earliestExerciseDateTenor) {
			this.earliestExerciseDateTenor = earliestExerciseDateTenor.toBuilder();
			return this;
		}
		public ExercisePeriodBuilder setEarliestExerciseDateTenorBuilder(Period.PeriodBuilder earliestExerciseDateTenor) {
			this.earliestExerciseDateTenor = earliestExerciseDateTenor;
			return this;
		}
		public ExercisePeriodBuilder setExerciseFrequency(Period exerciseFrequency) {
			this.exerciseFrequency = exerciseFrequency.toBuilder();
			return this;
		}
		public ExercisePeriodBuilder setExerciseFrequencyBuilder(Period.PeriodBuilder exerciseFrequency) {
			this.exerciseFrequency = exerciseFrequency;
			return this;
		}
		public ExercisePeriodBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		public ExercisePeriodBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		
		public ExercisePeriod build() {
			return new ExercisePeriod(this);
		}
	
		@Override
		public ExercisePeriodBuilder prune() {
			if (earliestExerciseDateTenor!=null && !earliestExerciseDateTenor.prune().hasData()) earliestExerciseDateTenor = null;
			if (exerciseFrequency!=null && !exerciseFrequency.prune().hasData()) exerciseFrequency = null;
			if (meta!=null && !meta.prune().hasData()) meta = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getEarliestExerciseDateTenor()!=null && getEarliestExerciseDateTenor().hasData()) return true;
			if (getExerciseFrequency()!=null && getExerciseFrequency().hasData()) return true;
			if (getMeta()!=null && getMeta().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("earliestExerciseDateTenor"), processor, Period.class, earliestExerciseDateTenor);
			processRosetta(path.newSubPath("exerciseFrequency"), processor, Period.class, exerciseFrequency);
			processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			ExercisePeriodBuilder _that = (ExercisePeriodBuilder) o;
		
			if (earliestExerciseDateTenor != null ? !earliestExerciseDateTenor.equals(_that.earliestExerciseDateTenor) : _that.earliestExerciseDateTenor != null) return false;
			if (exerciseFrequency != null ? !exerciseFrequency.equals(_that.exerciseFrequency) : _that.exerciseFrequency != null) return false;
			if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (earliestExerciseDateTenor != null ? earliestExerciseDateTenor.hashCode() : 0);
			_result = 31 * _result + (exerciseFrequency != null ? exerciseFrequency.hashCode() : 0);
			_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "ExercisePeriodBuilder {" +
				"earliestExerciseDateTenor=" + this.earliestExerciseDateTenor + ", " +
				"exerciseFrequency=" + this.exerciseFrequency + ", " +
				"meta=" + this.meta +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("earliestExerciseDateTenor"), processor, Period.class, earliestExerciseDateTenor);
		processRosetta(path.newSubPath("exerciseFrequency"), processor, Period.class, exerciseFrequency);
		processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		ExercisePeriod _that = (ExercisePeriod) o;
	
		if (earliestExerciseDateTenor != null ? !earliestExerciseDateTenor.equals(_that.earliestExerciseDateTenor) : _that.earliestExerciseDateTenor != null) return false;
		if (exerciseFrequency != null ? !exerciseFrequency.equals(_that.exerciseFrequency) : _that.exerciseFrequency != null) return false;
		if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (earliestExerciseDateTenor != null ? earliestExerciseDateTenor.hashCode() : 0);
		_result = 31 * _result + (exerciseFrequency != null ? exerciseFrequency.hashCode() : 0);
		_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "ExercisePeriod {" +
			"earliestExerciseDateTenor=" + this.earliestExerciseDateTenor + ", " +
			"exerciseFrequency=" + this.exerciseFrequency + ", " +
			"meta=" + this.meta +
		'}';
	}
}
