package org.isda.cdm;

import java.lang.Boolean;
import org.isda.cdm.AutomaticExercise;
import org.isda.cdm.ManualExercise;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.ExerciseProcedureMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class describing how notice of exercise should be given. This can be either manual or automatic.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="ExerciseProcedure", source="FpML_5_10")
public class ExerciseProcedure extends RosettaModelObject {
	private final AutomaticExercise automaticExercise;
	private final Boolean followUpConfirmation;
	private final Boolean limitedRightToConfirm;
	private final ManualExercise manualExercise;
	private final Boolean splitTicket;
	private static ExerciseProcedureMeta metaData = new ExerciseProcedureMeta();
	
	ExerciseProcedure(ExerciseProcedureBuilder builder) {
		this.automaticExercise = ofNullable(builder.getAutomaticExercise()).map(f->f.build()).orElse(null);
		this.followUpConfirmation = builder.getFollowUpConfirmation();
		this.limitedRightToConfirm = builder.getLimitedRightToConfirm();
		this.manualExercise = ofNullable(builder.getManualExercise()).map(f->f.build()).orElse(null);
		this.splitTicket = builder.getSplitTicket();
	}
	
	/**
	 * If automatic is specified, then the notional amount of the underlying swap not previously exercised under the swaption will be automatically exercised at the expiration time on the expiration date if at such time the buyer is in-the-money, provided that the difference between the settlement rate and the fixed rate under the relevant underlying swap is not less than the specified threshold rate. The term in-the-money is assumed to have the meaning defining in the 2000 ISDA Definitions, Section 17.4 In-the-money.
	 */
	@RosettaSynonym(value="automaticExercise", source="FpML_5_10")
	@RosettaSynonym(value="automaticExercise", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="automaticExercise", source="DTCC_11_0")
	@RosettaSynonym(value="automaticExercise", source="DTCC_9_0")
	@RosettaSynonym(value="automaticExercise", source="CME_ClearedConfirm_1_17")
	public final AutomaticExercise getAutomaticExercise() {
		return automaticExercise;
	}
	
	/**
	 * A flag to indicate whether follow-up confirmation of exercise (written or electronic) is required following telephonic notice by the buyer to the seller or seller&#39;s agent.
	 */
	@RosettaSynonym(value="followUpConfirmation", source="FpML_5_10")
	@RosettaSynonym(value="followUpConfirmation", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="followUpConfirmation", source="DTCC_11_0")
	@RosettaSynonym(value="followUpConfirmation", source="DTCC_9_0")
	@RosettaSynonym(value="followUpConfirmation", source="CME_ClearedConfirm_1_17")
	public final Boolean getFollowUpConfirmation() {
		return followUpConfirmation;
	}
	
	/**
	 * Has the meaning defined as part of the 1997 ISDA Government Bond Option Definitions, section 4.5 Limited Right to Confirm Exercise. If present, (i) the Seller may request the Buyer to confirm its intent if not done on or before the expiration time on the Expiration date (ii) specific rules will apply in relation to the settlement mode.
	 */
	@RosettaSynonym(value="limitedRightToConfirm", source="FpML_5_10")
	@RosettaSynonym(value="limitedRightToConfirm", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="limitedRightToConfirm", source="DTCC_11_0")
	@RosettaSynonym(value="limitedRightToConfirm", source="DTCC_9_0")
	@RosettaSynonym(value="limitedRightToConfirm", source="CME_ClearedConfirm_1_17")
	public final Boolean getLimitedRightToConfirm() {
		return limitedRightToConfirm;
	}
	
	/**
	 * Specifies that the notice of exercise must be given by the buyer to the seller or seller&#39;s agent.
	 */
	@RosettaSynonym(value="manualExercise", source="FpML_5_10")
	@RosettaSynonym(value="manualExercise", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="manualExercise", source="DTCC_11_0")
	@RosettaSynonym(value="manualExercise", source="DTCC_9_0")
	@RosettaSynonym(value="manualExercise", source="CME_ClearedConfirm_1_17")
	public final ManualExercise getManualExercise() {
		return manualExercise;
	}
	
	/**
	 * Typically applicable to the physical settlement of bond and convertible bond options. If present, means that the party required to deliver the bonds will divide those to be delivered as notifying party desires to facilitate delivery obligations.
	 */
	@RosettaSynonym(value="splitTicket", source="FpML_5_10")
	@RosettaSynonym(value="splitTicket", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="splitTicket", source="DTCC_11_0")
	@RosettaSynonym(value="splitTicket", source="DTCC_9_0")
	@RosettaSynonym(value="splitTicket", source="CME_ClearedConfirm_1_17")
	public final Boolean getSplitTicket() {
		return splitTicket;
	}
	
	
	@Override
	public RosettaMetaData<? extends ExerciseProcedure> metaData() {
		return metaData;
	} 
	
	public ExerciseProcedureBuilder toBuilder() {
		ExerciseProcedureBuilder builder = new ExerciseProcedureBuilder();
		ofNullable(getAutomaticExercise()).ifPresent(builder::setAutomaticExercise);
		ofNullable(getFollowUpConfirmation()).ifPresent(builder::setFollowUpConfirmation);
		ofNullable(getLimitedRightToConfirm()).ifPresent(builder::setLimitedRightToConfirm);
		ofNullable(getManualExercise()).ifPresent(builder::setManualExercise);
		ofNullable(getSplitTicket()).ifPresent(builder::setSplitTicket);
		return builder;
	}

	public static ExerciseProcedureBuilder builder() {
		return new ExerciseProcedureBuilder();
	}

	public static  class ExerciseProcedureBuilder extends RosettaModelObjectBuilder {
	
		protected AutomaticExercise.AutomaticExerciseBuilder automaticExercise;
		protected Boolean followUpConfirmation;
		protected Boolean limitedRightToConfirm;
		protected ManualExercise.ManualExerciseBuilder manualExercise;
		protected Boolean splitTicket;
	
		public ExerciseProcedureBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends ExerciseProcedure> metaData() {
			return metaData;
		} 
	
		
		public AutomaticExercise.AutomaticExerciseBuilder getAutomaticExercise() {
			return automaticExercise;
		}
		
		public AutomaticExercise.AutomaticExerciseBuilder getOrCreateAutomaticExercise() {
			if (automaticExercise!=null) {
				return automaticExercise;
			}
			else return automaticExercise = new AutomaticExercise.AutomaticExerciseBuilder();
		}
		
		
		public Boolean getFollowUpConfirmation() {
			return followUpConfirmation;
		}
		
		
		public Boolean getLimitedRightToConfirm() {
			return limitedRightToConfirm;
		}
		
		
		public ManualExercise.ManualExerciseBuilder getManualExercise() {
			return manualExercise;
		}
		
		public ManualExercise.ManualExerciseBuilder getOrCreateManualExercise() {
			if (manualExercise!=null) {
				return manualExercise;
			}
			else return manualExercise = new ManualExercise.ManualExerciseBuilder();
		}
		
		
		public Boolean getSplitTicket() {
			return splitTicket;
		}
		
	
		public ExerciseProcedureBuilder setAutomaticExercise(AutomaticExercise automaticExercise) {
			this.automaticExercise = automaticExercise.toBuilder();
			return this;
		}
		public ExerciseProcedureBuilder setAutomaticExerciseBuilder(AutomaticExercise.AutomaticExerciseBuilder automaticExercise) {
			this.automaticExercise = automaticExercise;
			return this;
		}
		public ExerciseProcedureBuilder setFollowUpConfirmation(Boolean followUpConfirmation) {
			this.followUpConfirmation = followUpConfirmation;
			return this;
		}
		public ExerciseProcedureBuilder setLimitedRightToConfirm(Boolean limitedRightToConfirm) {
			this.limitedRightToConfirm = limitedRightToConfirm;
			return this;
		}
		public ExerciseProcedureBuilder setManualExercise(ManualExercise manualExercise) {
			this.manualExercise = manualExercise.toBuilder();
			return this;
		}
		public ExerciseProcedureBuilder setManualExerciseBuilder(ManualExercise.ManualExerciseBuilder manualExercise) {
			this.manualExercise = manualExercise;
			return this;
		}
		public ExerciseProcedureBuilder setSplitTicket(Boolean splitTicket) {
			this.splitTicket = splitTicket;
			return this;
		}
		
		public ExerciseProcedure build() {
			return new ExerciseProcedure(this);
		}
	
		@Override
		public ExerciseProcedureBuilder prune() {
			if (automaticExercise!=null && !automaticExercise.prune().hasData()) automaticExercise = null;
			if (manualExercise!=null && !manualExercise.prune().hasData()) manualExercise = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getAutomaticExercise()!=null && getAutomaticExercise().hasData()) return true;
			if (getFollowUpConfirmation()!=null) return true;
			if (getLimitedRightToConfirm()!=null) return true;
			if (getManualExercise()!=null && getManualExercise().hasData()) return true;
			if (getSplitTicket()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("followUpConfirmation"), Boolean.class, followUpConfirmation, this);
			processor.processBasic(path.newSubPath("limitedRightToConfirm"), Boolean.class, limitedRightToConfirm, this);
			processor.processBasic(path.newSubPath("splitTicket"), Boolean.class, splitTicket, this);
			
			processRosetta(path.newSubPath("automaticExercise"), processor, AutomaticExercise.class, automaticExercise);
			processRosetta(path.newSubPath("manualExercise"), processor, ManualExercise.class, manualExercise);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			ExerciseProcedureBuilder _that = (ExerciseProcedureBuilder) o;
		
			if (automaticExercise != null ? !automaticExercise.equals(_that.automaticExercise) : _that.automaticExercise != null) return false;
			if (followUpConfirmation != null ? !followUpConfirmation.equals(_that.followUpConfirmation) : _that.followUpConfirmation != null) return false;
			if (limitedRightToConfirm != null ? !limitedRightToConfirm.equals(_that.limitedRightToConfirm) : _that.limitedRightToConfirm != null) return false;
			if (manualExercise != null ? !manualExercise.equals(_that.manualExercise) : _that.manualExercise != null) return false;
			if (splitTicket != null ? !splitTicket.equals(_that.splitTicket) : _that.splitTicket != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (automaticExercise != null ? automaticExercise.hashCode() : 0);
			_result = 31 * _result + (followUpConfirmation != null ? followUpConfirmation.hashCode() : 0);
			_result = 31 * _result + (limitedRightToConfirm != null ? limitedRightToConfirm.hashCode() : 0);
			_result = 31 * _result + (manualExercise != null ? manualExercise.hashCode() : 0);
			_result = 31 * _result + (splitTicket != null ? splitTicket.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "ExerciseProcedureBuilder {" +
				"automaticExercise=" + this.automaticExercise + ", " +
				"followUpConfirmation=" + this.followUpConfirmation + ", " +
				"limitedRightToConfirm=" + this.limitedRightToConfirm + ", " +
				"manualExercise=" + this.manualExercise + ", " +
				"splitTicket=" + this.splitTicket +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("followUpConfirmation"), Boolean.class, followUpConfirmation, this);
		processor.processBasic(path.newSubPath("limitedRightToConfirm"), Boolean.class, limitedRightToConfirm, this);
		processor.processBasic(path.newSubPath("splitTicket"), Boolean.class, splitTicket, this);
		
		processRosetta(path.newSubPath("automaticExercise"), processor, AutomaticExercise.class, automaticExercise);
		processRosetta(path.newSubPath("manualExercise"), processor, ManualExercise.class, manualExercise);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		ExerciseProcedure _that = (ExerciseProcedure) o;
	
		if (automaticExercise != null ? !automaticExercise.equals(_that.automaticExercise) : _that.automaticExercise != null) return false;
		if (followUpConfirmation != null ? !followUpConfirmation.equals(_that.followUpConfirmation) : _that.followUpConfirmation != null) return false;
		if (limitedRightToConfirm != null ? !limitedRightToConfirm.equals(_that.limitedRightToConfirm) : _that.limitedRightToConfirm != null) return false;
		if (manualExercise != null ? !manualExercise.equals(_that.manualExercise) : _that.manualExercise != null) return false;
		if (splitTicket != null ? !splitTicket.equals(_that.splitTicket) : _that.splitTicket != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (automaticExercise != null ? automaticExercise.hashCode() : 0);
		_result = 31 * _result + (followUpConfirmation != null ? followUpConfirmation.hashCode() : 0);
		_result = 31 * _result + (limitedRightToConfirm != null ? limitedRightToConfirm.hashCode() : 0);
		_result = 31 * _result + (manualExercise != null ? manualExercise.hashCode() : 0);
		_result = 31 * _result + (splitTicket != null ? splitTicket.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "ExerciseProcedure {" +
			"automaticExercise=" + this.automaticExercise + ", " +
			"followUpConfirmation=" + this.followUpConfirmation + ", " +
			"limitedRightToConfirm=" + this.limitedRightToConfirm + ", " +
			"manualExercise=" + this.manualExercise + ", " +
			"splitTicket=" + this.splitTicket +
		'}';
	}
}
