package org.isda.cdm;

import java.lang.Boolean;
import org.isda.cdm.AdditionalDisruptionEvents;
import org.isda.cdm.EquityCorporateEvents;
import org.isda.cdm.IndexAdjustmentEvents;
import org.isda.cdm.NationalizationOrInsolvencyOrDelistingEventEnum;
import org.isda.cdm.Representations;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.ExtraordinaryEventsMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * Where the underlying is shares, defines market events affecting the issuer of those shares that may require the terms of the transaction to be adjusted.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="ExtraordinaryEvents", source="FpML_5_10")
public class ExtraordinaryEvents extends RosettaModelObject {
	private final AdditionalDisruptionEvents additionalDisruptionEvents;
	private final Boolean compositionOfCombinedConsideration;
	private final NationalizationOrInsolvencyOrDelistingEventEnum delisting;
	private final Boolean failureToDeliver;
	private final IndexAdjustmentEvents indexAdjustmentEvents;
	private final EquityCorporateEvents mergerEvents;
	private final NationalizationOrInsolvencyOrDelistingEventEnum nationalizationOrInsolvency;
	private final Representations representations;
	private final EquityCorporateEvents tenderOfferEvents;
	private static ExtraordinaryEventsMeta metaData = new ExtraordinaryEventsMeta();
	
	ExtraordinaryEvents(ExtraordinaryEventsBuilder builder) {
		this.additionalDisruptionEvents = ofNullable(builder.getAdditionalDisruptionEvents()).map(f->f.build()).orElse(null);
		this.compositionOfCombinedConsideration = builder.getCompositionOfCombinedConsideration();
		this.delisting = builder.getDelisting();
		this.failureToDeliver = builder.getFailureToDeliver();
		this.indexAdjustmentEvents = ofNullable(builder.getIndexAdjustmentEvents()).map(f->f.build()).orElse(null);
		this.mergerEvents = ofNullable(builder.getMergerEvents()).map(f->f.build()).orElse(null);
		this.nationalizationOrInsolvency = builder.getNationalizationOrInsolvency();
		this.representations = ofNullable(builder.getRepresentations()).map(f->f.build()).orElse(null);
		this.tenderOfferEvents = ofNullable(builder.getTenderOfferEvents()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * 2002 ISDA Equity Derivatives Definitions: Additional Disruption Events | 2018 ISDA CDM Equity Confirmation for Security Equity Swap: Additional Disruption Events means each Additional Disruption Event specified in the Confirmation Side Letter. For the avoidance of doubt, each Additional Disruption Event shall be an Extraordinary Event.
	 */
	@RosettaSynonym(value="additionalDisruptionEvents", source="FpML_5_10")
	public final AdditionalDisruptionEvents getAdditionalDisruptionEvents() {
		return additionalDisruptionEvents;
	}
	
	/**
	 * 2002 ISDA Equity Derivatives Definitions: Composition of Combined Consideration | 2018 ISDA CDM Equity Confirmation for Security Equity Swap: Combined Consideration | If present and true, then composition of combined consideration is applicable.
	 */
	@RosettaSynonym(value="compositionOfCombinedConsideration", source="FpML_5_10")
	public final Boolean getCompositionOfCombinedConsideration() {
		return compositionOfCombinedConsideration;
	}
	
	/**
	 * 2002 ISDA Equity Derivatives Definitions: Delisting | 2018 ISDA CDM Equity Confirmation for Security Equity Swap: Delisting means Delisting (Broad Relisting) or Delisting (Narrow Relisting), as specified in the Relationship Supplement.
	 */
	@RosettaSynonym(value="delisting", source="FpML_5_10")
	public final NationalizationOrInsolvencyOrDelistingEventEnum getDelisting() {
		return delisting;
	}
	
	/**
	 * If true, failure to deliver is applicable.
	 */
	@RosettaSynonym(value="failureToDeliver", source="FpML_5_10")
	public final Boolean getFailureToDeliver() {
		return failureToDeliver;
	}
	
	/**
	 * 2002 ISDA Equity Derivatives Definitions: Adjustments to Indices | 
	 */
	@RosettaSynonym(value="indexAdjustmentEvents", source="FpML_5_10")
	public final IndexAdjustmentEvents getIndexAdjustmentEvents() {
		return indexAdjustmentEvents;
	}
	
	/**
	 * 2018 ISDA CDM Equity Confirmation for Security Equity Swap: Merger Event shall occur if a Merger occurs and the Merger Date is on or before the final Equity Valuation Date | Occurs when the underlying ceases to exist following a merger between the Issuer and another company.
	 */
	@RosettaSynonym(value="mergerEvents", source="FpML_5_10")
	public final EquityCorporateEvents getMergerEvents() {
		return mergerEvents;
	}
	
	/**
	 * 2002 ISDA Equity Derivatives Definitions: Nationalization and Insolvency | 2018 ISDA CDM Equity Confirmation for Security Equity Swap: Nationalization shall occur if all the Securities or all or substantially all the assets of an Issuer are nationalized, expropriated or are otherwise required to be transferred to any governmental agency, authority, entity or instrumentality thereof. Insolvency Filing means as defined in the Confirmation Side Letter.
	 */
	@RosettaSynonym(value="nationalisationOrInsolvency", source="FpML_5_10")
	public final NationalizationOrInsolvencyOrDelistingEventEnum getNationalizationOrInsolvency() {
		return nationalizationOrInsolvency;
	}
	
	@RosettaSynonym(value="representations", source="FpML_5_10")
	public final Representations getRepresentations() {
		return representations;
	}
	
	/**
	 * 2002 ISDA Equity Derivatives Definitions: Tender Offers | 2018 ISDA CDM Equity Confirmation for Security Equity Swap: Tender Offer Event
	 */
	@RosettaSynonym(value="tenderOfferEvents", source="FpML_5_10")
	public final EquityCorporateEvents getTenderOfferEvents() {
		return tenderOfferEvents;
	}
	
	
	@Override
	public RosettaMetaData<? extends ExtraordinaryEvents> metaData() {
		return metaData;
	} 
	
	public ExtraordinaryEventsBuilder toBuilder() {
		ExtraordinaryEventsBuilder builder = new ExtraordinaryEventsBuilder();
		ofNullable(getAdditionalDisruptionEvents()).ifPresent(builder::setAdditionalDisruptionEvents);
		ofNullable(getCompositionOfCombinedConsideration()).ifPresent(builder::setCompositionOfCombinedConsideration);
		ofNullable(getDelisting()).ifPresent(builder::setDelisting);
		ofNullable(getFailureToDeliver()).ifPresent(builder::setFailureToDeliver);
		ofNullable(getIndexAdjustmentEvents()).ifPresent(builder::setIndexAdjustmentEvents);
		ofNullable(getMergerEvents()).ifPresent(builder::setMergerEvents);
		ofNullable(getNationalizationOrInsolvency()).ifPresent(builder::setNationalizationOrInsolvency);
		ofNullable(getRepresentations()).ifPresent(builder::setRepresentations);
		ofNullable(getTenderOfferEvents()).ifPresent(builder::setTenderOfferEvents);
		return builder;
	}

	public static ExtraordinaryEventsBuilder builder() {
		return new ExtraordinaryEventsBuilder();
	}

	public static  class ExtraordinaryEventsBuilder extends RosettaModelObjectBuilder {
	
		protected AdditionalDisruptionEvents.AdditionalDisruptionEventsBuilder additionalDisruptionEvents;
		protected Boolean compositionOfCombinedConsideration;
		protected NationalizationOrInsolvencyOrDelistingEventEnum delisting;
		protected Boolean failureToDeliver;
		protected IndexAdjustmentEvents.IndexAdjustmentEventsBuilder indexAdjustmentEvents;
		protected EquityCorporateEvents.EquityCorporateEventsBuilder mergerEvents;
		protected NationalizationOrInsolvencyOrDelistingEventEnum nationalizationOrInsolvency;
		protected Representations.RepresentationsBuilder representations;
		protected EquityCorporateEvents.EquityCorporateEventsBuilder tenderOfferEvents;
	
		public ExtraordinaryEventsBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends ExtraordinaryEvents> metaData() {
			return metaData;
		} 
	
		
		public AdditionalDisruptionEvents.AdditionalDisruptionEventsBuilder getAdditionalDisruptionEvents() {
			return additionalDisruptionEvents;
		}
		
		public AdditionalDisruptionEvents.AdditionalDisruptionEventsBuilder getOrCreateAdditionalDisruptionEvents() {
			if (additionalDisruptionEvents!=null) {
				return additionalDisruptionEvents;
			}
			else return additionalDisruptionEvents = new AdditionalDisruptionEvents.AdditionalDisruptionEventsBuilder();
		}
		
		
		public Boolean getCompositionOfCombinedConsideration() {
			return compositionOfCombinedConsideration;
		}
		
		
		public NationalizationOrInsolvencyOrDelistingEventEnum getDelisting() {
			return delisting;
		}
		
		
		public Boolean getFailureToDeliver() {
			return failureToDeliver;
		}
		
		
		public IndexAdjustmentEvents.IndexAdjustmentEventsBuilder getIndexAdjustmentEvents() {
			return indexAdjustmentEvents;
		}
		
		public IndexAdjustmentEvents.IndexAdjustmentEventsBuilder getOrCreateIndexAdjustmentEvents() {
			if (indexAdjustmentEvents!=null) {
				return indexAdjustmentEvents;
			}
			else return indexAdjustmentEvents = new IndexAdjustmentEvents.IndexAdjustmentEventsBuilder();
		}
		
		
		public EquityCorporateEvents.EquityCorporateEventsBuilder getMergerEvents() {
			return mergerEvents;
		}
		
		public EquityCorporateEvents.EquityCorporateEventsBuilder getOrCreateMergerEvents() {
			if (mergerEvents!=null) {
				return mergerEvents;
			}
			else return mergerEvents = new EquityCorporateEvents.EquityCorporateEventsBuilder();
		}
		
		
		public NationalizationOrInsolvencyOrDelistingEventEnum getNationalizationOrInsolvency() {
			return nationalizationOrInsolvency;
		}
		
		
		public Representations.RepresentationsBuilder getRepresentations() {
			return representations;
		}
		
		public Representations.RepresentationsBuilder getOrCreateRepresentations() {
			if (representations!=null) {
				return representations;
			}
			else return representations = new Representations.RepresentationsBuilder();
		}
		
		
		public EquityCorporateEvents.EquityCorporateEventsBuilder getTenderOfferEvents() {
			return tenderOfferEvents;
		}
		
		public EquityCorporateEvents.EquityCorporateEventsBuilder getOrCreateTenderOfferEvents() {
			if (tenderOfferEvents!=null) {
				return tenderOfferEvents;
			}
			else return tenderOfferEvents = new EquityCorporateEvents.EquityCorporateEventsBuilder();
		}
		
	
		public ExtraordinaryEventsBuilder setAdditionalDisruptionEvents(AdditionalDisruptionEvents additionalDisruptionEvents) {
			this.additionalDisruptionEvents = additionalDisruptionEvents.toBuilder();
			return this;
		}
		public ExtraordinaryEventsBuilder setAdditionalDisruptionEventsBuilder(AdditionalDisruptionEvents.AdditionalDisruptionEventsBuilder additionalDisruptionEvents) {
			this.additionalDisruptionEvents = additionalDisruptionEvents;
			return this;
		}
		public ExtraordinaryEventsBuilder setCompositionOfCombinedConsideration(Boolean compositionOfCombinedConsideration) {
			this.compositionOfCombinedConsideration = compositionOfCombinedConsideration;
			return this;
		}
		public ExtraordinaryEventsBuilder setDelisting(NationalizationOrInsolvencyOrDelistingEventEnum delisting) {
			this.delisting = delisting;
			return this;
		}
		public ExtraordinaryEventsBuilder setFailureToDeliver(Boolean failureToDeliver) {
			this.failureToDeliver = failureToDeliver;
			return this;
		}
		public ExtraordinaryEventsBuilder setIndexAdjustmentEvents(IndexAdjustmentEvents indexAdjustmentEvents) {
			this.indexAdjustmentEvents = indexAdjustmentEvents.toBuilder();
			return this;
		}
		public ExtraordinaryEventsBuilder setIndexAdjustmentEventsBuilder(IndexAdjustmentEvents.IndexAdjustmentEventsBuilder indexAdjustmentEvents) {
			this.indexAdjustmentEvents = indexAdjustmentEvents;
			return this;
		}
		public ExtraordinaryEventsBuilder setMergerEvents(EquityCorporateEvents mergerEvents) {
			this.mergerEvents = mergerEvents.toBuilder();
			return this;
		}
		public ExtraordinaryEventsBuilder setMergerEventsBuilder(EquityCorporateEvents.EquityCorporateEventsBuilder mergerEvents) {
			this.mergerEvents = mergerEvents;
			return this;
		}
		public ExtraordinaryEventsBuilder setNationalizationOrInsolvency(NationalizationOrInsolvencyOrDelistingEventEnum nationalizationOrInsolvency) {
			this.nationalizationOrInsolvency = nationalizationOrInsolvency;
			return this;
		}
		public ExtraordinaryEventsBuilder setRepresentations(Representations representations) {
			this.representations = representations.toBuilder();
			return this;
		}
		public ExtraordinaryEventsBuilder setRepresentationsBuilder(Representations.RepresentationsBuilder representations) {
			this.representations = representations;
			return this;
		}
		public ExtraordinaryEventsBuilder setTenderOfferEvents(EquityCorporateEvents tenderOfferEvents) {
			this.tenderOfferEvents = tenderOfferEvents.toBuilder();
			return this;
		}
		public ExtraordinaryEventsBuilder setTenderOfferEventsBuilder(EquityCorporateEvents.EquityCorporateEventsBuilder tenderOfferEvents) {
			this.tenderOfferEvents = tenderOfferEvents;
			return this;
		}
		
		public ExtraordinaryEvents build() {
			return new ExtraordinaryEvents(this);
		}
	
		@Override
		public ExtraordinaryEventsBuilder prune() {
			if (additionalDisruptionEvents!=null && !additionalDisruptionEvents.prune().hasData()) additionalDisruptionEvents = null;
			if (indexAdjustmentEvents!=null && !indexAdjustmentEvents.prune().hasData()) indexAdjustmentEvents = null;
			if (mergerEvents!=null && !mergerEvents.prune().hasData()) mergerEvents = null;
			if (representations!=null && !representations.prune().hasData()) representations = null;
			if (tenderOfferEvents!=null && !tenderOfferEvents.prune().hasData()) tenderOfferEvents = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getAdditionalDisruptionEvents()!=null && getAdditionalDisruptionEvents().hasData()) return true;
			if (getCompositionOfCombinedConsideration()!=null) return true;
			if (getDelisting()!=null) return true;
			if (getFailureToDeliver()!=null) return true;
			if (getIndexAdjustmentEvents()!=null && getIndexAdjustmentEvents().hasData()) return true;
			if (getMergerEvents()!=null && getMergerEvents().hasData()) return true;
			if (getNationalizationOrInsolvency()!=null) return true;
			if (getRepresentations()!=null && getRepresentations().hasData()) return true;
			if (getTenderOfferEvents()!=null && getTenderOfferEvents().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("compositionOfCombinedConsideration"), Boolean.class, compositionOfCombinedConsideration, this);
			processor.processBasic(path.newSubPath("delisting"), NationalizationOrInsolvencyOrDelistingEventEnum.class, delisting, this);
			processor.processBasic(path.newSubPath("failureToDeliver"), Boolean.class, failureToDeliver, this);
			processor.processBasic(path.newSubPath("nationalizationOrInsolvency"), NationalizationOrInsolvencyOrDelistingEventEnum.class, nationalizationOrInsolvency, this);
			
			processRosetta(path.newSubPath("additionalDisruptionEvents"), processor, AdditionalDisruptionEvents.class, additionalDisruptionEvents);
			processRosetta(path.newSubPath("indexAdjustmentEvents"), processor, IndexAdjustmentEvents.class, indexAdjustmentEvents);
			processRosetta(path.newSubPath("mergerEvents"), processor, EquityCorporateEvents.class, mergerEvents);
			processRosetta(path.newSubPath("representations"), processor, Representations.class, representations);
			processRosetta(path.newSubPath("tenderOfferEvents"), processor, EquityCorporateEvents.class, tenderOfferEvents);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			ExtraordinaryEventsBuilder _that = (ExtraordinaryEventsBuilder) o;
		
			if (additionalDisruptionEvents != null ? !additionalDisruptionEvents.equals(_that.additionalDisruptionEvents) : _that.additionalDisruptionEvents != null) return false;
			if (compositionOfCombinedConsideration != null ? !compositionOfCombinedConsideration.equals(_that.compositionOfCombinedConsideration) : _that.compositionOfCombinedConsideration != null) return false;
			if (delisting != null ? !delisting.equals(_that.delisting) : _that.delisting != null) return false;
			if (failureToDeliver != null ? !failureToDeliver.equals(_that.failureToDeliver) : _that.failureToDeliver != null) return false;
			if (indexAdjustmentEvents != null ? !indexAdjustmentEvents.equals(_that.indexAdjustmentEvents) : _that.indexAdjustmentEvents != null) return false;
			if (mergerEvents != null ? !mergerEvents.equals(_that.mergerEvents) : _that.mergerEvents != null) return false;
			if (nationalizationOrInsolvency != null ? !nationalizationOrInsolvency.equals(_that.nationalizationOrInsolvency) : _that.nationalizationOrInsolvency != null) return false;
			if (representations != null ? !representations.equals(_that.representations) : _that.representations != null) return false;
			if (tenderOfferEvents != null ? !tenderOfferEvents.equals(_that.tenderOfferEvents) : _that.tenderOfferEvents != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (additionalDisruptionEvents != null ? additionalDisruptionEvents.hashCode() : 0);
			_result = 31 * _result + (compositionOfCombinedConsideration != null ? compositionOfCombinedConsideration.hashCode() : 0);
			_result = 31 * _result + (delisting != null ? delisting.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (failureToDeliver != null ? failureToDeliver.hashCode() : 0);
			_result = 31 * _result + (indexAdjustmentEvents != null ? indexAdjustmentEvents.hashCode() : 0);
			_result = 31 * _result + (mergerEvents != null ? mergerEvents.hashCode() : 0);
			_result = 31 * _result + (nationalizationOrInsolvency != null ? nationalizationOrInsolvency.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (representations != null ? representations.hashCode() : 0);
			_result = 31 * _result + (tenderOfferEvents != null ? tenderOfferEvents.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "ExtraordinaryEventsBuilder {" +
				"additionalDisruptionEvents=" + this.additionalDisruptionEvents + ", " +
				"compositionOfCombinedConsideration=" + this.compositionOfCombinedConsideration + ", " +
				"delisting=" + this.delisting + ", " +
				"failureToDeliver=" + this.failureToDeliver + ", " +
				"indexAdjustmentEvents=" + this.indexAdjustmentEvents + ", " +
				"mergerEvents=" + this.mergerEvents + ", " +
				"nationalizationOrInsolvency=" + this.nationalizationOrInsolvency + ", " +
				"representations=" + this.representations + ", " +
				"tenderOfferEvents=" + this.tenderOfferEvents +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("compositionOfCombinedConsideration"), Boolean.class, compositionOfCombinedConsideration, this);
		processor.processBasic(path.newSubPath("delisting"), NationalizationOrInsolvencyOrDelistingEventEnum.class, delisting, this);
		processor.processBasic(path.newSubPath("failureToDeliver"), Boolean.class, failureToDeliver, this);
		processor.processBasic(path.newSubPath("nationalizationOrInsolvency"), NationalizationOrInsolvencyOrDelistingEventEnum.class, nationalizationOrInsolvency, this);
		
		processRosetta(path.newSubPath("additionalDisruptionEvents"), processor, AdditionalDisruptionEvents.class, additionalDisruptionEvents);
		processRosetta(path.newSubPath("indexAdjustmentEvents"), processor, IndexAdjustmentEvents.class, indexAdjustmentEvents);
		processRosetta(path.newSubPath("mergerEvents"), processor, EquityCorporateEvents.class, mergerEvents);
		processRosetta(path.newSubPath("representations"), processor, Representations.class, representations);
		processRosetta(path.newSubPath("tenderOfferEvents"), processor, EquityCorporateEvents.class, tenderOfferEvents);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		ExtraordinaryEvents _that = (ExtraordinaryEvents) o;
	
		if (additionalDisruptionEvents != null ? !additionalDisruptionEvents.equals(_that.additionalDisruptionEvents) : _that.additionalDisruptionEvents != null) return false;
		if (compositionOfCombinedConsideration != null ? !compositionOfCombinedConsideration.equals(_that.compositionOfCombinedConsideration) : _that.compositionOfCombinedConsideration != null) return false;
		if (delisting != null ? !delisting.equals(_that.delisting) : _that.delisting != null) return false;
		if (failureToDeliver != null ? !failureToDeliver.equals(_that.failureToDeliver) : _that.failureToDeliver != null) return false;
		if (indexAdjustmentEvents != null ? !indexAdjustmentEvents.equals(_that.indexAdjustmentEvents) : _that.indexAdjustmentEvents != null) return false;
		if (mergerEvents != null ? !mergerEvents.equals(_that.mergerEvents) : _that.mergerEvents != null) return false;
		if (nationalizationOrInsolvency != null ? !nationalizationOrInsolvency.equals(_that.nationalizationOrInsolvency) : _that.nationalizationOrInsolvency != null) return false;
		if (representations != null ? !representations.equals(_that.representations) : _that.representations != null) return false;
		if (tenderOfferEvents != null ? !tenderOfferEvents.equals(_that.tenderOfferEvents) : _that.tenderOfferEvents != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (additionalDisruptionEvents != null ? additionalDisruptionEvents.hashCode() : 0);
		_result = 31 * _result + (compositionOfCombinedConsideration != null ? compositionOfCombinedConsideration.hashCode() : 0);
		_result = 31 * _result + (delisting != null ? delisting.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (failureToDeliver != null ? failureToDeliver.hashCode() : 0);
		_result = 31 * _result + (indexAdjustmentEvents != null ? indexAdjustmentEvents.hashCode() : 0);
		_result = 31 * _result + (mergerEvents != null ? mergerEvents.hashCode() : 0);
		_result = 31 * _result + (nationalizationOrInsolvency != null ? nationalizationOrInsolvency.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (representations != null ? representations.hashCode() : 0);
		_result = 31 * _result + (tenderOfferEvents != null ? tenderOfferEvents.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "ExtraordinaryEvents {" +
			"additionalDisruptionEvents=" + this.additionalDisruptionEvents + ", " +
			"compositionOfCombinedConsideration=" + this.compositionOfCombinedConsideration + ", " +
			"delisting=" + this.delisting + ", " +
			"failureToDeliver=" + this.failureToDeliver + ", " +
			"indexAdjustmentEvents=" + this.indexAdjustmentEvents + ", " +
			"mergerEvents=" + this.mergerEvents + ", " +
			"nationalizationOrInsolvency=" + this.nationalizationOrInsolvency + ", " +
			"representations=" + this.representations + ", " +
			"tenderOfferEvents=" + this.tenderOfferEvents +
		'}';
	}
}
