package org.isda.cdm;

import java.lang.Boolean;
import org.isda.cdm.AdditionalFixedPayments;
import org.isda.cdm.FloatingAmountProvisions;
import org.isda.cdm.InterestShortFall;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.FloatingAmountEventsMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class to specify the ISDA terms relating to the floating rate payment events and the implied additional fixed payments, applicable to the credit derivatives transactions on mortgage-backed securities with pay-as-you-go or physical settlement.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="FloatingAmountEvents", source="FpML_5_10")
public class FloatingAmountEvents extends RosettaModelObject {
	private final AdditionalFixedPayments additionalFixedPayments;
	private final Boolean failureToPayPrincipal;
	private final FloatingAmountProvisions floatingAmountProvisions;
	private final Boolean impliedWritedown;
	private final InterestShortFall interestShortfall;
	private final Boolean writedown;
	private static FloatingAmountEventsMeta metaData = new FloatingAmountEventsMeta();
	
	FloatingAmountEvents(FloatingAmountEventsBuilder builder) {
		this.additionalFixedPayments = ofNullable(builder.getAdditionalFixedPayments()).map(f->f.build()).orElse(null);
		this.failureToPayPrincipal = builder.getFailureToPayPrincipal();
		this.floatingAmountProvisions = ofNullable(builder.getFloatingAmountProvisions()).map(f->f.build()).orElse(null);
		this.impliedWritedown = builder.getImpliedWritedown();
		this.interestShortfall = ofNullable(builder.getInterestShortfall()).map(f->f.build()).orElse(null);
		this.writedown = builder.getWritedown();
	}
	
	/**
	 * Specifies the events that will give rise to the payment additional fixed payments.
	 */
	@RosettaSynonym(value="additionalFixedPayments", source="FpML_5_10")
	@RosettaSynonym(value="additionalFixedPayments", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="additionalFixedPayments", source="DTCC_11_0")
	@RosettaSynonym(value="additionalFixedPayments", source="DTCC_9_0")
	@RosettaSynonym(value="additionalFixedPayments", source="CME_ClearedConfirm_1_17")
	public final AdditionalFixedPayments getAdditionalFixedPayments() {
		return additionalFixedPayments;
	}
	
	/**
	 * A floating rate payment event. Corresponds to the failure by the Reference Entity to pay an expected principal amount or the payment of an actual principal amount that is less than the expected principal amount. ISDA 2003 Term: Failure to Pay Principal.
	 */
	@RosettaSynonym(value="failureToPayPrincipal", source="FpML_5_10")
	@RosettaSynonym(value="failureToPayPrincipal", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="failureToPayPrincipal", source="DTCC_11_0")
	@RosettaSynonym(value="failureToPayPrincipal", source="DTCC_9_0")
	@RosettaSynonym(value="failureToPayPrincipal", source="CME_ClearedConfirm_1_17")
	public final Boolean getFailureToPayPrincipal() {
		return failureToPayPrincipal;
	}
	
	/**
	 * Specifies the floating amount provisions associated with the floatingAmountEvents.
	 */
	@RosettaSynonym(value="floatingAmountProvisions", source="FpML_5_10")
	@RosettaSynonym(value="floatingAmountProvisions", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="floatingAmountProvisions", source="DTCC_11_0")
	@RosettaSynonym(value="floatingAmountProvisions", source="DTCC_9_0")
	@RosettaSynonym(value="floatingAmountProvisions", source="CME_ClearedConfirm_1_17")
	public final FloatingAmountProvisions getFloatingAmountProvisions() {
		return floatingAmountProvisions;
	}
	
	/**
	 * A floating rate payment event. Results from the fact that losses occur to the underlying instruments that do not result in reductions of the outstanding principal of the reference obligation.
	 */
	@RosettaSynonym(value="impliedWritedown", source="FpML_5_10")
	@RosettaSynonym(value="impliedWritedown", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="impliedWritedown", source="DTCC_11_0")
	@RosettaSynonym(value="impliedWritedown", source="DTCC_9_0")
	@RosettaSynonym(value="impliedWritedown", source="CME_ClearedConfirm_1_17")
	public final Boolean getImpliedWritedown() {
		return impliedWritedown;
	}
	
	/**
	 * A floating rate payment event. With respect to any Reference Obligation Payment Date, either (a) the non-payment of an Expected Interest Amount or (b) the payment of an Actual Interest Amount that is less than the Expected Interest Amount. ISDA 2003 Term: Interest Shortfall.
	 */
	@RosettaSynonym(value="interestShortfall", source="FpML_5_10")
	@RosettaSynonym(value="interestShortfall", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="interestShortfall", source="DTCC_11_0")
	@RosettaSynonym(value="interestShortfall", source="DTCC_9_0")
	@RosettaSynonym(value="interestShortfall", source="CME_ClearedConfirm_1_17")
	public final InterestShortFall getInterestShortfall() {
		return interestShortfall;
	}
	
	/**
	 * A floating rate payment event. Results from the fact that the underlier writes down its outstanding principal amount. ISDA 2003 Term: Writedown.
	 */
	@RosettaSynonym(value="writedown", source="FpML_5_10")
	@RosettaSynonym(value="writedown", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="writedown", source="DTCC_11_0")
	@RosettaSynonym(value="writedown", source="DTCC_9_0")
	@RosettaSynonym(value="writedown", source="CME_ClearedConfirm_1_17")
	public final Boolean getWritedown() {
		return writedown;
	}
	
	
	@Override
	public RosettaMetaData<? extends FloatingAmountEvents> metaData() {
		return metaData;
	} 
	
	public FloatingAmountEventsBuilder toBuilder() {
		FloatingAmountEventsBuilder builder = new FloatingAmountEventsBuilder();
		ofNullable(getAdditionalFixedPayments()).ifPresent(builder::setAdditionalFixedPayments);
		ofNullable(getFailureToPayPrincipal()).ifPresent(builder::setFailureToPayPrincipal);
		ofNullable(getFloatingAmountProvisions()).ifPresent(builder::setFloatingAmountProvisions);
		ofNullable(getImpliedWritedown()).ifPresent(builder::setImpliedWritedown);
		ofNullable(getInterestShortfall()).ifPresent(builder::setInterestShortfall);
		ofNullable(getWritedown()).ifPresent(builder::setWritedown);
		return builder;
	}

	public static FloatingAmountEventsBuilder builder() {
		return new FloatingAmountEventsBuilder();
	}

	public static  class FloatingAmountEventsBuilder extends RosettaModelObjectBuilder {
	
		protected AdditionalFixedPayments.AdditionalFixedPaymentsBuilder additionalFixedPayments;
		protected Boolean failureToPayPrincipal;
		protected FloatingAmountProvisions.FloatingAmountProvisionsBuilder floatingAmountProvisions;
		protected Boolean impliedWritedown;
		protected InterestShortFall.InterestShortFallBuilder interestShortfall;
		protected Boolean writedown;
	
		public FloatingAmountEventsBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends FloatingAmountEvents> metaData() {
			return metaData;
		} 
	
		
		public AdditionalFixedPayments.AdditionalFixedPaymentsBuilder getAdditionalFixedPayments() {
			return additionalFixedPayments;
		}
		
		public AdditionalFixedPayments.AdditionalFixedPaymentsBuilder getOrCreateAdditionalFixedPayments() {
			if (additionalFixedPayments!=null) {
				return additionalFixedPayments;
			}
			else return additionalFixedPayments = new AdditionalFixedPayments.AdditionalFixedPaymentsBuilder();
		}
		
		
		public Boolean getFailureToPayPrincipal() {
			return failureToPayPrincipal;
		}
		
		
		public FloatingAmountProvisions.FloatingAmountProvisionsBuilder getFloatingAmountProvisions() {
			return floatingAmountProvisions;
		}
		
		public FloatingAmountProvisions.FloatingAmountProvisionsBuilder getOrCreateFloatingAmountProvisions() {
			if (floatingAmountProvisions!=null) {
				return floatingAmountProvisions;
			}
			else return floatingAmountProvisions = new FloatingAmountProvisions.FloatingAmountProvisionsBuilder();
		}
		
		
		public Boolean getImpliedWritedown() {
			return impliedWritedown;
		}
		
		
		public InterestShortFall.InterestShortFallBuilder getInterestShortfall() {
			return interestShortfall;
		}
		
		public InterestShortFall.InterestShortFallBuilder getOrCreateInterestShortfall() {
			if (interestShortfall!=null) {
				return interestShortfall;
			}
			else return interestShortfall = new InterestShortFall.InterestShortFallBuilder();
		}
		
		
		public Boolean getWritedown() {
			return writedown;
		}
		
	
		public FloatingAmountEventsBuilder setAdditionalFixedPayments(AdditionalFixedPayments additionalFixedPayments) {
			this.additionalFixedPayments = additionalFixedPayments.toBuilder();
			return this;
		}
		public FloatingAmountEventsBuilder setAdditionalFixedPaymentsBuilder(AdditionalFixedPayments.AdditionalFixedPaymentsBuilder additionalFixedPayments) {
			this.additionalFixedPayments = additionalFixedPayments;
			return this;
		}
		public FloatingAmountEventsBuilder setFailureToPayPrincipal(Boolean failureToPayPrincipal) {
			this.failureToPayPrincipal = failureToPayPrincipal;
			return this;
		}
		public FloatingAmountEventsBuilder setFloatingAmountProvisions(FloatingAmountProvisions floatingAmountProvisions) {
			this.floatingAmountProvisions = floatingAmountProvisions.toBuilder();
			return this;
		}
		public FloatingAmountEventsBuilder setFloatingAmountProvisionsBuilder(FloatingAmountProvisions.FloatingAmountProvisionsBuilder floatingAmountProvisions) {
			this.floatingAmountProvisions = floatingAmountProvisions;
			return this;
		}
		public FloatingAmountEventsBuilder setImpliedWritedown(Boolean impliedWritedown) {
			this.impliedWritedown = impliedWritedown;
			return this;
		}
		public FloatingAmountEventsBuilder setInterestShortfall(InterestShortFall interestShortfall) {
			this.interestShortfall = interestShortfall.toBuilder();
			return this;
		}
		public FloatingAmountEventsBuilder setInterestShortfallBuilder(InterestShortFall.InterestShortFallBuilder interestShortfall) {
			this.interestShortfall = interestShortfall;
			return this;
		}
		public FloatingAmountEventsBuilder setWritedown(Boolean writedown) {
			this.writedown = writedown;
			return this;
		}
		
		public FloatingAmountEvents build() {
			return new FloatingAmountEvents(this);
		}
	
		@Override
		public FloatingAmountEventsBuilder prune() {
			if (additionalFixedPayments!=null && !additionalFixedPayments.prune().hasData()) additionalFixedPayments = null;
			if (floatingAmountProvisions!=null && !floatingAmountProvisions.prune().hasData()) floatingAmountProvisions = null;
			if (interestShortfall!=null && !interestShortfall.prune().hasData()) interestShortfall = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getAdditionalFixedPayments()!=null && getAdditionalFixedPayments().hasData()) return true;
			if (getFailureToPayPrincipal()!=null) return true;
			if (getFloatingAmountProvisions()!=null && getFloatingAmountProvisions().hasData()) return true;
			if (getImpliedWritedown()!=null) return true;
			if (getInterestShortfall()!=null && getInterestShortfall().hasData()) return true;
			if (getWritedown()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("failureToPayPrincipal"), Boolean.class, failureToPayPrincipal, this);
			processor.processBasic(path.newSubPath("impliedWritedown"), Boolean.class, impliedWritedown, this);
			processor.processBasic(path.newSubPath("writedown"), Boolean.class, writedown, this);
			
			processRosetta(path.newSubPath("additionalFixedPayments"), processor, AdditionalFixedPayments.class, additionalFixedPayments);
			processRosetta(path.newSubPath("floatingAmountProvisions"), processor, FloatingAmountProvisions.class, floatingAmountProvisions);
			processRosetta(path.newSubPath("interestShortfall"), processor, InterestShortFall.class, interestShortfall);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			FloatingAmountEventsBuilder _that = (FloatingAmountEventsBuilder) o;
		
			if (additionalFixedPayments != null ? !additionalFixedPayments.equals(_that.additionalFixedPayments) : _that.additionalFixedPayments != null) return false;
			if (failureToPayPrincipal != null ? !failureToPayPrincipal.equals(_that.failureToPayPrincipal) : _that.failureToPayPrincipal != null) return false;
			if (floatingAmountProvisions != null ? !floatingAmountProvisions.equals(_that.floatingAmountProvisions) : _that.floatingAmountProvisions != null) return false;
			if (impliedWritedown != null ? !impliedWritedown.equals(_that.impliedWritedown) : _that.impliedWritedown != null) return false;
			if (interestShortfall != null ? !interestShortfall.equals(_that.interestShortfall) : _that.interestShortfall != null) return false;
			if (writedown != null ? !writedown.equals(_that.writedown) : _that.writedown != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (additionalFixedPayments != null ? additionalFixedPayments.hashCode() : 0);
			_result = 31 * _result + (failureToPayPrincipal != null ? failureToPayPrincipal.hashCode() : 0);
			_result = 31 * _result + (floatingAmountProvisions != null ? floatingAmountProvisions.hashCode() : 0);
			_result = 31 * _result + (impliedWritedown != null ? impliedWritedown.hashCode() : 0);
			_result = 31 * _result + (interestShortfall != null ? interestShortfall.hashCode() : 0);
			_result = 31 * _result + (writedown != null ? writedown.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "FloatingAmountEventsBuilder {" +
				"additionalFixedPayments=" + this.additionalFixedPayments + ", " +
				"failureToPayPrincipal=" + this.failureToPayPrincipal + ", " +
				"floatingAmountProvisions=" + this.floatingAmountProvisions + ", " +
				"impliedWritedown=" + this.impliedWritedown + ", " +
				"interestShortfall=" + this.interestShortfall + ", " +
				"writedown=" + this.writedown +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("failureToPayPrincipal"), Boolean.class, failureToPayPrincipal, this);
		processor.processBasic(path.newSubPath("impliedWritedown"), Boolean.class, impliedWritedown, this);
		processor.processBasic(path.newSubPath("writedown"), Boolean.class, writedown, this);
		
		processRosetta(path.newSubPath("additionalFixedPayments"), processor, AdditionalFixedPayments.class, additionalFixedPayments);
		processRosetta(path.newSubPath("floatingAmountProvisions"), processor, FloatingAmountProvisions.class, floatingAmountProvisions);
		processRosetta(path.newSubPath("interestShortfall"), processor, InterestShortFall.class, interestShortfall);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		FloatingAmountEvents _that = (FloatingAmountEvents) o;
	
		if (additionalFixedPayments != null ? !additionalFixedPayments.equals(_that.additionalFixedPayments) : _that.additionalFixedPayments != null) return false;
		if (failureToPayPrincipal != null ? !failureToPayPrincipal.equals(_that.failureToPayPrincipal) : _that.failureToPayPrincipal != null) return false;
		if (floatingAmountProvisions != null ? !floatingAmountProvisions.equals(_that.floatingAmountProvisions) : _that.floatingAmountProvisions != null) return false;
		if (impliedWritedown != null ? !impliedWritedown.equals(_that.impliedWritedown) : _that.impliedWritedown != null) return false;
		if (interestShortfall != null ? !interestShortfall.equals(_that.interestShortfall) : _that.interestShortfall != null) return false;
		if (writedown != null ? !writedown.equals(_that.writedown) : _that.writedown != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (additionalFixedPayments != null ? additionalFixedPayments.hashCode() : 0);
		_result = 31 * _result + (failureToPayPrincipal != null ? failureToPayPrincipal.hashCode() : 0);
		_result = 31 * _result + (floatingAmountProvisions != null ? floatingAmountProvisions.hashCode() : 0);
		_result = 31 * _result + (impliedWritedown != null ? impliedWritedown.hashCode() : 0);
		_result = 31 * _result + (interestShortfall != null ? interestShortfall.hashCode() : 0);
		_result = 31 * _result + (writedown != null ? writedown.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "FloatingAmountEvents {" +
			"additionalFixedPayments=" + this.additionalFixedPayments + ", " +
			"failureToPayPrincipal=" + this.failureToPayPrincipal + ", " +
			"floatingAmountProvisions=" + this.floatingAmountProvisions + ", " +
			"impliedWritedown=" + this.impliedWritedown + ", " +
			"interestShortfall=" + this.interestShortfall + ", " +
			"writedown=" + this.writedown +
		'}';
	}
}
