package org.isda.cdm;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.RateObservation;
import org.isda.cdm.Strike;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.FloatingRateDefinitionMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.math.BigDecimal;
		import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;

/**
 * A class defining parameters associated with a floating rate reset. This class forms part of the cashflows representation of a stream.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="FloatingRateDefinition", source="FpML_5_10")
public class FloatingRateDefinition extends RosettaModelObject {
	private final BigDecimal calculatedRate;
	private final List<Strike> capRate;
	private final BigDecimal floatingRateMultiplier;
	private final List<Strike> floorRate;
	private final List<RateObservation> rateObservation;
	private final BigDecimal spread;
	private static FloatingRateDefinitionMeta metaData = new FloatingRateDefinitionMeta();
	
	FloatingRateDefinition(FloatingRateDefinitionBuilder builder) {
		this.calculatedRate = builder.getCalculatedRate();
		this.capRate = ofNullable(builder.getCapRate()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.floatingRateMultiplier = builder.getFloatingRateMultiplier();
		this.floorRate = ofNullable(builder.getFloorRate()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.rateObservation = ofNullable(builder.getRateObservation()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.spread = builder.getSpread();
	}
	
	/**
	 * The final calculated rate for a calculation period after any required averaging of rates A calculated rate of 5% would be represented as 0.05.
	 */
	@RosettaSynonym(value="calculatedRate", source="FpML_5_10")
	@RosettaSynonym(value="calculatedRate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="calculatedRate", source="DTCC_11_0")
	@RosettaSynonym(value="calculatedRate", source="DTCC_9_0")
	@RosettaSynonym(value="calculatedRate", source="CME_ClearedConfirm_1_17")
	public final BigDecimal getCalculatedRate() {
		return calculatedRate;
	}
	
	/**
	 * The cap rate, if any, which applies to the floating rate for the calculation period. The cap rate (strike) is only required where the floating rate on a swap stream is capped at a certain strike level. The cap rate is assumed to be exclusive of any spread and is a per annum rate, expressed as a decimal. A cap rate of 5% would be represented as 0.05.
	 */
	@RosettaSynonym(value="capRate", source="FpML_5_10")
	@RosettaSynonym(value="capRate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="capRate", source="DTCC_11_0")
	@RosettaSynonym(value="capRate", source="DTCC_9_0")
	@RosettaSynonym(value="capRate", source="CME_ClearedConfirm_1_17")
	public final List<Strike> getCapRate() {
		return capRate;
	}
	
	/**
	 * A rate multiplier to apply to the floating rate. The multiplier can be a positive or negative decimal. This element should only be included if the multiplier is not equal to 1 (one).
	 */
	@RosettaSynonym(value="floatingRateMultiplier", source="FpML_5_10")
	@RosettaSynonym(value="floatingRateMultiplier", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="floatingRateMultiplier", source="DTCC_11_0")
	@RosettaSynonym(value="floatingRateMultiplier", source="DTCC_9_0")
	@RosettaSynonym(value="floatingRateMultiplier", source="CME_ClearedConfirm_1_17")
	public final BigDecimal getFloatingRateMultiplier() {
		return floatingRateMultiplier;
	}
	
	/**
	 * The floor rate, if any, which applies to the floating rate for the calculation period. The floor rate (strike) is only required where the floating rate on a swap stream is floored at a certain strike level. The floor rate is assumed to be exclusive of any spread and is a per annum rate, expressed as a decimal. The floor rate of 5% would be represented as 0.05.
	 */
	@RosettaSynonym(value="floorRate", source="FpML_5_10")
	@RosettaSynonym(value="floorRate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="floorRate", source="DTCC_11_0")
	@RosettaSynonym(value="floorRate", source="DTCC_9_0")
	@RosettaSynonym(value="floorRate", source="CME_ClearedConfirm_1_17")
	public final List<Strike> getFloorRate() {
		return floorRate;
	}
	
	/**
	 * The details of a particular rate observation, including the fixing date and observed rate. A list of rate observation elements may be ordered in the document by ascending adjusted fixing date. An FpML document containing an unordered list of rate observations is still regarded as a conformant document.
	 */
	@RosettaSynonym(value="rateObservation", source="FpML_5_10")
	@RosettaSynonym(value="rateObservation", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="rateObservation", source="DTCC_11_0")
	@RosettaSynonym(value="rateObservation", source="DTCC_9_0")
	@RosettaSynonym(value="rateObservation", source="CME_ClearedConfirm_1_17")
	public final List<RateObservation> getRateObservation() {
		return rateObservation;
	}
	
	/**
	 * The ISDA Spread, if any, which applies for the calculation period. The spread is a per annum rate, expressed as a decimal. For purposes of determining a calculation period amount, if positive the spread will be added to the floating rate and if negative the spread will be subtracted from the floating rate. A positive 10 basis point (0.1%) spread would be represented as 0.001.
	 */
	@RosettaSynonym(value="spread", source="FpML_5_10")
	@RosettaSynonym(value="spread", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="spread", source="DTCC_11_0")
	@RosettaSynonym(value="spread", source="DTCC_9_0")
	@RosettaSynonym(value="spread", source="CME_ClearedConfirm_1_17")
	public final BigDecimal getSpread() {
		return spread;
	}
	
	
	@Override
	public RosettaMetaData<? extends FloatingRateDefinition> metaData() {
		return metaData;
	} 
	
	public FloatingRateDefinitionBuilder toBuilder() {
		FloatingRateDefinitionBuilder builder = new FloatingRateDefinitionBuilder();
		ofNullable(getCalculatedRate()).ifPresent(builder::setCalculatedRate);
		ofNullable(getCapRate()).ifPresent(capRate -> capRate.forEach(builder::addCapRate));
		ofNullable(getFloatingRateMultiplier()).ifPresent(builder::setFloatingRateMultiplier);
		ofNullable(getFloorRate()).ifPresent(floorRate -> floorRate.forEach(builder::addFloorRate));
		ofNullable(getRateObservation()).ifPresent(rateObservation -> rateObservation.forEach(builder::addRateObservation));
		ofNullable(getSpread()).ifPresent(builder::setSpread);
		return builder;
	}

	public static FloatingRateDefinitionBuilder builder() {
		return new FloatingRateDefinitionBuilder();
	}

	public static  class FloatingRateDefinitionBuilder extends RosettaModelObjectBuilder {
	
		protected BigDecimal calculatedRate;
		protected List<Strike.StrikeBuilder> capRate;
		protected BigDecimal floatingRateMultiplier;
		protected List<Strike.StrikeBuilder> floorRate;
		protected List<RateObservation.RateObservationBuilder> rateObservation;
		protected BigDecimal spread;
	
		public FloatingRateDefinitionBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends FloatingRateDefinition> metaData() {
			return metaData;
		} 
	
		
		public BigDecimal getCalculatedRate() {
			return calculatedRate;
		}
		
		
		public List<Strike.StrikeBuilder> getCapRate() {
			return capRate;
		}
		
		public Strike.StrikeBuilder getOrCreateCapRate(int index) {
			if (capRate==null) {
				this.capRate = new ArrayList<>();
			}
			return getIndex(capRate, index, ()->new Strike.StrikeBuilder());
		}
		
		
		public BigDecimal getFloatingRateMultiplier() {
			return floatingRateMultiplier;
		}
		
		
		public List<Strike.StrikeBuilder> getFloorRate() {
			return floorRate;
		}
		
		public Strike.StrikeBuilder getOrCreateFloorRate(int index) {
			if (floorRate==null) {
				this.floorRate = new ArrayList<>();
			}
			return getIndex(floorRate, index, ()->new Strike.StrikeBuilder());
		}
		
		
		public List<RateObservation.RateObservationBuilder> getRateObservation() {
			return rateObservation;
		}
		
		public RateObservation.RateObservationBuilder getOrCreateRateObservation(int index) {
			if (rateObservation==null) {
				this.rateObservation = new ArrayList<>();
			}
			return getIndex(rateObservation, index, ()->new RateObservation.RateObservationBuilder());
		}
		
		
		public BigDecimal getSpread() {
			return spread;
		}
		
	
		public FloatingRateDefinitionBuilder setCalculatedRate(BigDecimal calculatedRate) {
			this.calculatedRate = calculatedRate;
			return this;
		}
		public FloatingRateDefinitionBuilder addCapRate(Strike capRate) {
			if(this.capRate == null){
				this.capRate = new ArrayList<>();
			}
			this.capRate.add(capRate.toBuilder());
			return this;
		}
		public FloatingRateDefinitionBuilder addCapRate(Strike capRate, int _idx) {
			if(this.capRate == null){
				this.capRate = new ArrayList<>();
			}
			getIndex(this.capRate, _idx, () -> capRate.toBuilder());
			this.capRate.set(_idx, capRate.toBuilder());
			return this;
		}
		public FloatingRateDefinitionBuilder addCapRate(List<Strike> capRates) {
			if(this.capRate == null){
				this.capRate = new ArrayList<>();
			}
			for (Strike toAdd : capRates) {
				this.capRate.add(toAdd.toBuilder());
			}
			return this;
		}
		public FloatingRateDefinitionBuilder addCapRateBuilder(Strike.StrikeBuilder capRate) {
			if(this.capRate == null){
				this.capRate = new ArrayList<>();
			}
			this.capRate.add(capRate);
			return this;
		}
		
		public FloatingRateDefinitionBuilder clearCapRate() {
			this.capRate = null;
			return this;
		}
		public FloatingRateDefinitionBuilder setFloatingRateMultiplier(BigDecimal floatingRateMultiplier) {
			this.floatingRateMultiplier = floatingRateMultiplier;
			return this;
		}
		public FloatingRateDefinitionBuilder addFloorRate(Strike floorRate) {
			if(this.floorRate == null){
				this.floorRate = new ArrayList<>();
			}
			this.floorRate.add(floorRate.toBuilder());
			return this;
		}
		public FloatingRateDefinitionBuilder addFloorRate(Strike floorRate, int _idx) {
			if(this.floorRate == null){
				this.floorRate = new ArrayList<>();
			}
			getIndex(this.floorRate, _idx, () -> floorRate.toBuilder());
			this.floorRate.set(_idx, floorRate.toBuilder());
			return this;
		}
		public FloatingRateDefinitionBuilder addFloorRate(List<Strike> floorRates) {
			if(this.floorRate == null){
				this.floorRate = new ArrayList<>();
			}
			for (Strike toAdd : floorRates) {
				this.floorRate.add(toAdd.toBuilder());
			}
			return this;
		}
		public FloatingRateDefinitionBuilder addFloorRateBuilder(Strike.StrikeBuilder floorRate) {
			if(this.floorRate == null){
				this.floorRate = new ArrayList<>();
			}
			this.floorRate.add(floorRate);
			return this;
		}
		
		public FloatingRateDefinitionBuilder clearFloorRate() {
			this.floorRate = null;
			return this;
		}
		public FloatingRateDefinitionBuilder addRateObservation(RateObservation rateObservation) {
			if(this.rateObservation == null){
				this.rateObservation = new ArrayList<>();
			}
			this.rateObservation.add(rateObservation.toBuilder());
			return this;
		}
		public FloatingRateDefinitionBuilder addRateObservation(RateObservation rateObservation, int _idx) {
			if(this.rateObservation == null){
				this.rateObservation = new ArrayList<>();
			}
			getIndex(this.rateObservation, _idx, () -> rateObservation.toBuilder());
			this.rateObservation.set(_idx, rateObservation.toBuilder());
			return this;
		}
		public FloatingRateDefinitionBuilder addRateObservation(List<RateObservation> rateObservations) {
			if(this.rateObservation == null){
				this.rateObservation = new ArrayList<>();
			}
			for (RateObservation toAdd : rateObservations) {
				this.rateObservation.add(toAdd.toBuilder());
			}
			return this;
		}
		public FloatingRateDefinitionBuilder addRateObservationBuilder(RateObservation.RateObservationBuilder rateObservation) {
			if(this.rateObservation == null){
				this.rateObservation = new ArrayList<>();
			}
			this.rateObservation.add(rateObservation);
			return this;
		}
		
		public FloatingRateDefinitionBuilder clearRateObservation() {
			this.rateObservation = null;
			return this;
		}
		public FloatingRateDefinitionBuilder setSpread(BigDecimal spread) {
			this.spread = spread;
			return this;
		}
		
		public FloatingRateDefinition build() {
			return new FloatingRateDefinition(this);
		}
	
		@Override
		public FloatingRateDefinitionBuilder prune() {
			if (capRate!=null) capRate = capRate.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (floorRate!=null) floorRate = floorRate.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (rateObservation!=null) rateObservation = rateObservation.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getCalculatedRate()!=null) return true;
			if (getCapRate()!=null && getCapRate().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getFloatingRateMultiplier()!=null) return true;
			if (getFloorRate()!=null && getFloorRate().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getRateObservation()!=null && getRateObservation().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getSpread()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("calculatedRate"), BigDecimal.class, calculatedRate, this);
			processor.processBasic(path.newSubPath("floatingRateMultiplier"), BigDecimal.class, floatingRateMultiplier, this);
			processor.processBasic(path.newSubPath("spread"), BigDecimal.class, spread, this);
			
			processRosetta(path.newSubPath("capRate"), processor, Strike.class, capRate);
			processRosetta(path.newSubPath("floorRate"), processor, Strike.class, floorRate);
			processRosetta(path.newSubPath("rateObservation"), processor, RateObservation.class, rateObservation);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			FloatingRateDefinitionBuilder _that = (FloatingRateDefinitionBuilder) o;
		
			if (calculatedRate != null ? !calculatedRate.equals(_that.calculatedRate) : _that.calculatedRate != null) return false;
			if (!ListEquals.listEquals(capRate, _that.capRate)) return false;
			if (floatingRateMultiplier != null ? !floatingRateMultiplier.equals(_that.floatingRateMultiplier) : _that.floatingRateMultiplier != null) return false;
			if (!ListEquals.listEquals(floorRate, _that.floorRate)) return false;
			if (!ListEquals.listEquals(rateObservation, _that.rateObservation)) return false;
			if (spread != null ? !spread.equals(_that.spread) : _that.spread != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (calculatedRate != null ? calculatedRate.hashCode() : 0);
			_result = 31 * _result + (capRate != null ? capRate.hashCode() : 0);
			_result = 31 * _result + (floatingRateMultiplier != null ? floatingRateMultiplier.hashCode() : 0);
			_result = 31 * _result + (floorRate != null ? floorRate.hashCode() : 0);
			_result = 31 * _result + (rateObservation != null ? rateObservation.hashCode() : 0);
			_result = 31 * _result + (spread != null ? spread.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "FloatingRateDefinitionBuilder {" +
				"calculatedRate=" + this.calculatedRate + ", " +
				"capRate=" + this.capRate + ", " +
				"floatingRateMultiplier=" + this.floatingRateMultiplier + ", " +
				"floorRate=" + this.floorRate + ", " +
				"rateObservation=" + this.rateObservation + ", " +
				"spread=" + this.spread +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("calculatedRate"), BigDecimal.class, calculatedRate, this);
		processor.processBasic(path.newSubPath("floatingRateMultiplier"), BigDecimal.class, floatingRateMultiplier, this);
		processor.processBasic(path.newSubPath("spread"), BigDecimal.class, spread, this);
		
		processRosetta(path.newSubPath("capRate"), processor, Strike.class, capRate);
		processRosetta(path.newSubPath("floorRate"), processor, Strike.class, floorRate);
		processRosetta(path.newSubPath("rateObservation"), processor, RateObservation.class, rateObservation);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		FloatingRateDefinition _that = (FloatingRateDefinition) o;
	
		if (calculatedRate != null ? !calculatedRate.equals(_that.calculatedRate) : _that.calculatedRate != null) return false;
		if (!ListEquals.listEquals(capRate, _that.capRate)) return false;
		if (floatingRateMultiplier != null ? !floatingRateMultiplier.equals(_that.floatingRateMultiplier) : _that.floatingRateMultiplier != null) return false;
		if (!ListEquals.listEquals(floorRate, _that.floorRate)) return false;
		if (!ListEquals.listEquals(rateObservation, _that.rateObservation)) return false;
		if (spread != null ? !spread.equals(_that.spread) : _that.spread != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (calculatedRate != null ? calculatedRate.hashCode() : 0);
		_result = 31 * _result + (capRate != null ? capRate.hashCode() : 0);
		_result = 31 * _result + (floatingRateMultiplier != null ? floatingRateMultiplier.hashCode() : 0);
		_result = 31 * _result + (floorRate != null ? floorRate.hashCode() : 0);
		_result = 31 * _result + (rateObservation != null ? rateObservation.hashCode() : 0);
		_result = 31 * _result + (spread != null ? spread.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "FloatingRateDefinition {" +
			"calculatedRate=" + this.calculatedRate + ", " +
			"capRate=" + this.capRate + ", " +
			"floatingRateMultiplier=" + this.floatingRateMultiplier + ", " +
			"floorRate=" + this.floorRate + ", " +
			"rateObservation=" + this.rateObservation + ", " +
			"spread=" + this.spread +
		'}';
	}
}
