package org.isda.cdm;

import java.math.BigDecimal;
import org.isda.cdm.AveragingMethodEnum;
import org.isda.cdm.NegativeInterestRateTreatmentEnum;
import org.isda.cdm.Rounding;
import org.isda.cdm.metafields.FieldWithMetaFloatingRateIndexEnum;


import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.FloatingRateSpecificationMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.math.BigDecimal;
		import static java.util.Optional.ofNullable;
		import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;

/**
 * A class to specify the floating interest rate by extending the floating rate definition with a set of attributes that specify such rate: the initial value specified as part of the trade, the rounding convention, the averaging method and the negative interest rate treatment.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="FloatingRateCalculation", source="FpML_5_10")
public class FloatingRateSpecification extends FloatingRate {
	private final AveragingMethodEnum averagingMethod;
	private final Rounding finalRateRounding;
	private final BigDecimal initialRate;
	private final NegativeInterestRateTreatmentEnum negativeInterestRateTreatment;
	private static FloatingRateSpecificationMeta metaData = new FloatingRateSpecificationMeta();
	
	FloatingRateSpecification(FloatingRateSpecificationBuilder builder) {
		super(builder);
		this.averagingMethod = builder.getAveragingMethod();
		this.finalRateRounding = ofNullable(builder.getFinalRateRounding()).map(f->f.build()).orElse(null);
		this.initialRate = builder.getInitialRate();
		this.negativeInterestRateTreatment = builder.getNegativeInterestRateTreatment();
	}
	
	/**
	 * If averaging is applicable, this component specifies whether a weighted or unweighted average method of calculation is to be used. The component must only be included when averaging applies.
	 */
	@RosettaSynonym(value="averagingMethod", source="FpML_5_10", path="calculationPeriodAmount.calculation.floatingRateCalculation")
	@RosettaSynonym(value="averagingMethod", source="FpML_5_10", path="periodicPayment.floatingAmountCalculation.floatingRate")
	@RosettaSynonym(value="averagingMethod", source="CME_SubmissionIRS_1_0", path="calculationPeriodAmount.calculation.floatingRateCalculation")
	@RosettaSynonym(value="averagingMethod", source="CME_SubmissionIRS_1_0", path="periodicPayment.floatingAmountCalculation.floatingRate")
	@RosettaSynonym(value="averagingMethod", source="DTCC_11_0", path="calculationPeriodAmount.calculation.floatingRateCalculation")
	@RosettaSynonym(value="averagingMethod", source="DTCC_11_0", path="periodicPayment.floatingAmountCalculation.floatingRate")
	@RosettaSynonym(value="averagingMethod", source="DTCC_9_0", path="calculationPeriodAmount.calculation.floatingRateCalculation")
	@RosettaSynonym(value="averagingMethod", source="DTCC_9_0", path="periodicPayment.floatingAmountCalculation.floatingRate")
	@RosettaSynonym(value="averagingMethod", source="CME_ClearedConfirm_1_17", path="calculationPeriodAmount.calculation.floatingRateCalculation")
	@RosettaSynonym(value="averagingMethod", source="CME_ClearedConfirm_1_17", path="periodicPayment.floatingAmountCalculation.floatingRate")
	public final AveragingMethodEnum getAveragingMethod() {
		return averagingMethod;
	}
	
	/**
	 * The rounding convention to apply to the final rate used in determination of a calculation period amount.
	 */
	@RosettaSynonym(value="finalRateRounding", source="FpML_5_10", path="calculationPeriodAmount.calculation.floatingRateCalculation")
	@RosettaSynonym(value="finalRateRounding", source="FpML_5_10", path="periodicPayment.floatingAmountCalculation.floatingRate")
	@RosettaSynonym(value="finalRateRounding", source="CME_SubmissionIRS_1_0", path="calculationPeriodAmount.calculation.floatingRateCalculation")
	@RosettaSynonym(value="finalRateRounding", source="CME_SubmissionIRS_1_0", path="periodicPayment.floatingAmountCalculation.floatingRate")
	@RosettaSynonym(value="finalRateRounding", source="DTCC_11_0", path="calculationPeriodAmount.calculation.floatingRateCalculation")
	@RosettaSynonym(value="finalRateRounding", source="DTCC_11_0", path="periodicPayment.floatingAmountCalculation.floatingRate")
	@RosettaSynonym(value="finalRateRounding", source="DTCC_9_0", path="calculationPeriodAmount.calculation.floatingRateCalculation")
	@RosettaSynonym(value="finalRateRounding", source="DTCC_9_0", path="periodicPayment.floatingAmountCalculation.floatingRate")
	@RosettaSynonym(value="finalRateRounding", source="CME_ClearedConfirm_1_17", path="calculationPeriodAmount.calculation.floatingRateCalculation")
	@RosettaSynonym(value="finalRateRounding", source="CME_ClearedConfirm_1_17", path="periodicPayment.floatingAmountCalculation.floatingRate")
	public final Rounding getFinalRateRounding() {
		return finalRateRounding;
	}
	
	/**
	 * The initial floating rate reset agreed between the principal parties involved in the trade. This is assumed to be the first required reset rate for the first regular calculation period. It should only be included when the rate is not equal to the rate published on the source implied by the floating rate index. An initial rate of 5% would be represented as 0.05.
	 */
	@RosettaSynonym(value="initialRate", source="FpML_5_10", path="calculationPeriodAmount.calculation.floatingRateCalculation")
	@RosettaSynonym(value="initialRate", source="FpML_5_10", path="periodicPayment.floatingAmountCalculation.floatingRate")
	@RosettaSynonym(value="initialRate", source="CME_SubmissionIRS_1_0", path="calculationPeriodAmount.calculation.floatingRateCalculation")
	@RosettaSynonym(value="initialRate", source="CME_SubmissionIRS_1_0", path="periodicPayment.floatingAmountCalculation.floatingRate")
	@RosettaSynonym(value="initialRate", source="DTCC_11_0", path="calculationPeriodAmount.calculation.floatingRateCalculation")
	@RosettaSynonym(value="initialRate", source="DTCC_11_0", path="periodicPayment.floatingAmountCalculation.floatingRate")
	@RosettaSynonym(value="initialRate", source="DTCC_9_0", path="calculationPeriodAmount.calculation.floatingRateCalculation")
	@RosettaSynonym(value="initialRate", source="DTCC_9_0", path="periodicPayment.floatingAmountCalculation.floatingRate")
	@RosettaSynonym(value="initialRate", source="CME_ClearedConfirm_1_17", path="calculationPeriodAmount.calculation.floatingRateCalculation")
	@RosettaSynonym(value="initialRate", source="CME_ClearedConfirm_1_17", path="periodicPayment.floatingAmountCalculation.floatingRate")
	public final BigDecimal getInitialRate() {
		return initialRate;
	}
	
	/**
	 * The specification of any provisions for calculating payment obligations when a floating rate is negative (either due to a quoted negative floating rate or by operation of a spread that is subtracted from the floating rate).
	 */
	@RosettaSynonym(value="negativeInterestRateTreatment", source="FpML_5_10", path="calculationPeriodAmount.calculation.floatingRateCalculation")
	@RosettaSynonym(value="negativeInterestRateTreatment", source="FpML_5_10", path="periodicPayment.floatingAmountCalculation.floatingRate")
	@RosettaSynonym(value="negativeInterestRateTreatment", source="CME_SubmissionIRS_1_0", path="calculationPeriodAmount.calculation.floatingRateCalculation")
	@RosettaSynonym(value="negativeInterestRateTreatment", source="CME_SubmissionIRS_1_0", path="periodicPayment.floatingAmountCalculation.floatingRate")
	@RosettaSynonym(value="negativeInterestRateTreatment", source="DTCC_11_0", path="calculationPeriodAmount.calculation.floatingRateCalculation")
	@RosettaSynonym(value="negativeInterestRateTreatment", source="DTCC_11_0", path="periodicPayment.floatingAmountCalculation.floatingRate")
	@RosettaSynonym(value="negativeInterestRateTreatment", source="DTCC_9_0", path="calculationPeriodAmount.calculation.floatingRateCalculation")
	@RosettaSynonym(value="negativeInterestRateTreatment", source="DTCC_9_0", path="periodicPayment.floatingAmountCalculation.floatingRate")
	@RosettaSynonym(value="negativeInterestRateTreatment", source="CME_ClearedConfirm_1_17", path="calculationPeriodAmount.calculation.floatingRateCalculation")
	@RosettaSynonym(value="negativeInterestRateTreatment", source="CME_ClearedConfirm_1_17", path="periodicPayment.floatingAmountCalculation.floatingRate")
	public final NegativeInterestRateTreatmentEnum getNegativeInterestRateTreatment() {
		return negativeInterestRateTreatment;
	}
	
	
	@Override
	public RosettaMetaData<? extends FloatingRateSpecification> metaData() {
		return metaData;
	} 
	
	public FloatingRateSpecificationBuilder toBuilder() {
		FloatingRateSpecificationBuilder builder = new FloatingRateSpecificationBuilder();
		ofNullable(getAveragingMethod()).ifPresent(builder::setAveragingMethod);
		ofNullable(getFinalRateRounding()).ifPresent(builder::setFinalRateRounding);
		ofNullable(getInitialRate()).ifPresent(builder::setInitialRate);
		ofNullable(getNegativeInterestRateTreatment()).ifPresent(builder::setNegativeInterestRateTreatment);
		ofNullable(getCapRateSchedule()).ifPresent(capRateSchedule -> capRateSchedule.forEach(builder::addCapRateSchedule));
		ofNullable(getFloatingRateIndex()).ifPresent(builder::setFloatingRateIndex);
		ofNullable(getFloatingRateMultiplierSchedule()).ifPresent(builder::setFloatingRateMultiplierSchedule);
		ofNullable(getFloorRateSchedule()).ifPresent(floorRateSchedule -> floorRateSchedule.forEach(builder::addFloorRateSchedule));
		ofNullable(getIndexTenor()).ifPresent(builder::setIndexTenor);
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		ofNullable(getRateTreatment()).ifPresent(builder::setRateTreatment);
		ofNullable(getSpreadSchedule()).ifPresent(spreadSchedule -> spreadSchedule.forEach(builder::addSpreadSchedule));
		return builder;
	}

	public static FloatingRateSpecificationBuilder builder() {
		return new FloatingRateSpecificationBuilder();
	}

	public static  class FloatingRateSpecificationBuilder extends FloatingRateBuilder {
	
		protected AveragingMethodEnum averagingMethod;
		protected Rounding.RoundingBuilder finalRateRounding;
		protected BigDecimal initialRate;
		protected NegativeInterestRateTreatmentEnum negativeInterestRateTreatment;
	
		public FloatingRateSpecificationBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends FloatingRateSpecification> metaData() {
			return metaData;
		} 
	
		
		public AveragingMethodEnum getAveragingMethod() {
			return averagingMethod;
		}
		
		
		public Rounding.RoundingBuilder getFinalRateRounding() {
			return finalRateRounding;
		}
		
		public Rounding.RoundingBuilder getOrCreateFinalRateRounding() {
			if (finalRateRounding!=null) {
				return finalRateRounding;
			}
			else return finalRateRounding = new Rounding.RoundingBuilder();
		}
		
		
		public BigDecimal getInitialRate() {
			return initialRate;
		}
		
		
		public NegativeInterestRateTreatmentEnum getNegativeInterestRateTreatment() {
			return negativeInterestRateTreatment;
		}
		
	
		public FloatingRateSpecificationBuilder setAveragingMethod(AveragingMethodEnum averagingMethod) {
			this.averagingMethod = averagingMethod;
			return this;
		}
		public FloatingRateSpecificationBuilder setFinalRateRounding(Rounding finalRateRounding) {
			this.finalRateRounding = finalRateRounding.toBuilder();
			return this;
		}
		public FloatingRateSpecificationBuilder setFinalRateRoundingBuilder(Rounding.RoundingBuilder finalRateRounding) {
			this.finalRateRounding = finalRateRounding;
			return this;
		}
		public FloatingRateSpecificationBuilder setInitialRate(BigDecimal initialRate) {
			this.initialRate = initialRate;
			return this;
		}
		public FloatingRateSpecificationBuilder setNegativeInterestRateTreatment(NegativeInterestRateTreatmentEnum negativeInterestRateTreatment) {
			this.negativeInterestRateTreatment = negativeInterestRateTreatment;
			return this;
		}
		@Override public FloatingRateSpecificationBuilder addCapRateSchedule(StrikeSchedule capRateSchedule) {
			if(this.capRateSchedule == null){
				this.capRateSchedule = new ArrayList<>();
			}
			this.capRateSchedule.add(capRateSchedule.toBuilder());
			return this;
		}
		@Override public FloatingRateSpecificationBuilder addCapRateSchedule(StrikeSchedule capRateSchedule, int _idx) {
			if(this.capRateSchedule == null){
				this.capRateSchedule = new ArrayList<>();
			}
			getIndex(this.capRateSchedule, _idx, () -> capRateSchedule.toBuilder());
			this.capRateSchedule.set(_idx, capRateSchedule.toBuilder());
			return this;
		}
		@Override public FloatingRateSpecificationBuilder addCapRateSchedule(List<StrikeSchedule> capRateSchedules) {
			if(this.capRateSchedule == null){
				this.capRateSchedule = new ArrayList<>();
			}
			for (StrikeSchedule toAdd : capRateSchedules) {
				this.capRateSchedule.add(toAdd.toBuilder());
			}
			return this;
		}
		@Override public FloatingRateSpecificationBuilder addCapRateScheduleBuilder(StrikeSchedule.StrikeScheduleBuilder capRateSchedule) {
			if(this.capRateSchedule == null){
				this.capRateSchedule = new ArrayList<>();
			}
			this.capRateSchedule.add(capRateSchedule);
			return this;
		}
		
		@Override public FloatingRateSpecificationBuilder clearCapRateSchedule() {
			this.capRateSchedule = null;
			return this;
		}
		@Override public FloatingRateSpecificationBuilder setFloatingRateIndex(FieldWithMetaFloatingRateIndexEnum floatingRateIndex) {
			this.floatingRateIndex = floatingRateIndex.toBuilder();
			return this;
		}
		@Override public FloatingRateSpecificationBuilder setFloatingRateIndexRef(FloatingRateIndexEnum floatingRateIndex) {
			return setFloatingRateIndex(FieldWithMetaFloatingRateIndexEnum.builder().setValue(floatingRateIndex).build());
		}
		@Override public FloatingRateSpecificationBuilder setFloatingRateMultiplierSchedule(Schedule floatingRateMultiplierSchedule) {
			this.floatingRateMultiplierSchedule = floatingRateMultiplierSchedule.toBuilder();
			return this;
		}
		@Override public FloatingRateSpecificationBuilder setFloatingRateMultiplierScheduleBuilder(Schedule.ScheduleBuilder floatingRateMultiplierSchedule) {
			this.floatingRateMultiplierSchedule = floatingRateMultiplierSchedule;
			return this;
		}
		@Override public FloatingRateSpecificationBuilder addFloorRateSchedule(StrikeSchedule floorRateSchedule) {
			if(this.floorRateSchedule == null){
				this.floorRateSchedule = new ArrayList<>();
			}
			this.floorRateSchedule.add(floorRateSchedule.toBuilder());
			return this;
		}
		@Override public FloatingRateSpecificationBuilder addFloorRateSchedule(StrikeSchedule floorRateSchedule, int _idx) {
			if(this.floorRateSchedule == null){
				this.floorRateSchedule = new ArrayList<>();
			}
			getIndex(this.floorRateSchedule, _idx, () -> floorRateSchedule.toBuilder());
			this.floorRateSchedule.set(_idx, floorRateSchedule.toBuilder());
			return this;
		}
		@Override public FloatingRateSpecificationBuilder addFloorRateSchedule(List<StrikeSchedule> floorRateSchedules) {
			if(this.floorRateSchedule == null){
				this.floorRateSchedule = new ArrayList<>();
			}
			for (StrikeSchedule toAdd : floorRateSchedules) {
				this.floorRateSchedule.add(toAdd.toBuilder());
			}
			return this;
		}
		@Override public FloatingRateSpecificationBuilder addFloorRateScheduleBuilder(StrikeSchedule.StrikeScheduleBuilder floorRateSchedule) {
			if(this.floorRateSchedule == null){
				this.floorRateSchedule = new ArrayList<>();
			}
			this.floorRateSchedule.add(floorRateSchedule);
			return this;
		}
		
		@Override public FloatingRateSpecificationBuilder clearFloorRateSchedule() {
			this.floorRateSchedule = null;
			return this;
		}
		@Override public FloatingRateSpecificationBuilder setIndexTenor(Period indexTenor) {
			this.indexTenor = indexTenor.toBuilder();
			return this;
		}
		@Override public FloatingRateSpecificationBuilder setIndexTenorBuilder(Period.PeriodBuilder indexTenor) {
			this.indexTenor = indexTenor;
			return this;
		}
		@Override public FloatingRateSpecificationBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		@Override public FloatingRateSpecificationBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		@Override public FloatingRateSpecificationBuilder setRateTreatment(RateTreatmentEnum rateTreatment) {
			this.rateTreatment = rateTreatment;
			return this;
		}
		@Override public FloatingRateSpecificationBuilder addSpreadSchedule(SpreadSchedule spreadSchedule) {
			if(this.spreadSchedule == null){
				this.spreadSchedule = new ArrayList<>();
			}
			this.spreadSchedule.add(spreadSchedule.toBuilder());
			return this;
		}
		@Override public FloatingRateSpecificationBuilder addSpreadSchedule(SpreadSchedule spreadSchedule, int _idx) {
			if(this.spreadSchedule == null){
				this.spreadSchedule = new ArrayList<>();
			}
			getIndex(this.spreadSchedule, _idx, () -> spreadSchedule.toBuilder());
			this.spreadSchedule.set(_idx, spreadSchedule.toBuilder());
			return this;
		}
		@Override public FloatingRateSpecificationBuilder addSpreadSchedule(List<SpreadSchedule> spreadSchedules) {
			if(this.spreadSchedule == null){
				this.spreadSchedule = new ArrayList<>();
			}
			for (SpreadSchedule toAdd : spreadSchedules) {
				this.spreadSchedule.add(toAdd.toBuilder());
			}
			return this;
		}
		@Override public FloatingRateSpecificationBuilder addSpreadScheduleBuilder(SpreadSchedule.SpreadScheduleBuilder spreadSchedule) {
			if(this.spreadSchedule == null){
				this.spreadSchedule = new ArrayList<>();
			}
			this.spreadSchedule.add(spreadSchedule);
			return this;
		}
		
		@Override public FloatingRateSpecificationBuilder clearSpreadSchedule() {
			this.spreadSchedule = null;
			return this;
		}
		
		public FloatingRateSpecification build() {
			return new FloatingRateSpecification(this);
		}
	
		@Override
		public FloatingRateSpecificationBuilder prune() {
			super.prune();
			if (finalRateRounding!=null && !finalRateRounding.prune().hasData()) finalRateRounding = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (super.hasData()) return true;
			if (getAveragingMethod()!=null) return true;
			if (getFinalRateRounding()!=null && getFinalRateRounding().hasData()) return true;
			if (getInitialRate()!=null) return true;
			if (getNegativeInterestRateTreatment()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
			super.process(path, processor);
		
			processor.processBasic(path.newSubPath("averagingMethod"), AveragingMethodEnum.class, averagingMethod, this);
			processor.processBasic(path.newSubPath("initialRate"), BigDecimal.class, initialRate, this);
			processor.processBasic(path.newSubPath("negativeInterestRateTreatment"), NegativeInterestRateTreatmentEnum.class, negativeInterestRateTreatment, this);
			
			processRosetta(path.newSubPath("finalRateRounding"), processor, Rounding.class, finalRateRounding);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
			if (!super.equals(o)) return false;
		
			FloatingRateSpecificationBuilder _that = (FloatingRateSpecificationBuilder) o;
		
			if (averagingMethod != null ? !averagingMethod.equals(_that.averagingMethod) : _that.averagingMethod != null) return false;
			if (finalRateRounding != null ? !finalRateRounding.equals(_that.finalRateRounding) : _that.finalRateRounding != null) return false;
			if (initialRate != null ? !initialRate.equals(_that.initialRate) : _that.initialRate != null) return false;
			if (negativeInterestRateTreatment != null ? !negativeInterestRateTreatment.equals(_that.negativeInterestRateTreatment) : _that.negativeInterestRateTreatment != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = super.hashCode();
			_result = 31 * _result + (averagingMethod != null ? averagingMethod.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (finalRateRounding != null ? finalRateRounding.hashCode() : 0);
			_result = 31 * _result + (initialRate != null ? initialRate.hashCode() : 0);
			_result = 31 * _result + (negativeInterestRateTreatment != null ? negativeInterestRateTreatment.getClass().getName().hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "FloatingRateSpecificationBuilder {" +
				"averagingMethod=" + this.averagingMethod + ", " +
				"finalRateRounding=" + this.finalRateRounding + ", " +
				"initialRate=" + this.initialRate + ", " +
				"negativeInterestRateTreatment=" + this.negativeInterestRateTreatment +
			'}' + " " + super.toString();
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		super.process(path, processor);
		
		processor.processBasic(path.newSubPath("averagingMethod"), AveragingMethodEnum.class, averagingMethod, this);
		processor.processBasic(path.newSubPath("initialRate"), BigDecimal.class, initialRate, this);
		processor.processBasic(path.newSubPath("negativeInterestRateTreatment"), NegativeInterestRateTreatmentEnum.class, negativeInterestRateTreatment, this);
		
		processRosetta(path.newSubPath("finalRateRounding"), processor, Rounding.class, finalRateRounding);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
		if (!super.equals(o)) return false;
	
		FloatingRateSpecification _that = (FloatingRateSpecification) o;
	
		if (averagingMethod != null ? !averagingMethod.equals(_that.averagingMethod) : _that.averagingMethod != null) return false;
		if (finalRateRounding != null ? !finalRateRounding.equals(_that.finalRateRounding) : _that.finalRateRounding != null) return false;
		if (initialRate != null ? !initialRate.equals(_that.initialRate) : _that.initialRate != null) return false;
		if (negativeInterestRateTreatment != null ? !negativeInterestRateTreatment.equals(_that.negativeInterestRateTreatment) : _that.negativeInterestRateTreatment != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = super.hashCode();
		_result = 31 * _result + (averagingMethod != null ? averagingMethod.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (finalRateRounding != null ? finalRateRounding.hashCode() : 0);
		_result = 31 * _result + (initialRate != null ? initialRate.hashCode() : 0);
		_result = 31 * _result + (negativeInterestRateTreatment != null ? negativeInterestRateTreatment.getClass().getName().hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "FloatingRateSpecification {" +
			"averagingMethod=" + this.averagingMethod + ", " +
			"finalRateRounding=" + this.finalRateRounding + ", " +
			"initialRate=" + this.initialRate + ", " +
			"negativeInterestRateTreatment=" + this.negativeInterestRateTreatment +
		'}' + " " + super.toString();
	}
}
