package org.isda.cdm;

import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.Cashflow;
import org.isda.cdm.ExchangeRate;
import org.isda.cdm.Period;
import org.isda.cdm.meta.ForeignExchangeMeta;

import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.BuilderProcessor;
import com.rosetta.model.lib.process.Processor;

import static java.util.Optional.ofNullable;

/**
 * From FpML: A type defining either a spot or forward FX transactions.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="FxSingleLeg", source="FpML_5_10")
public class ForeignExchange extends RosettaModelObject {
	private final ExchangeRate exchangeRate;
	private final Cashflow exchangedCurrency1;
	private final Cashflow exchangedCurrency2;
	private final Period tenorPeriod;
	private static ForeignExchangeMeta metaData = new ForeignExchangeMeta();
	
	ForeignExchange(ForeignExchangeBuilder builder) {
		this.exchangeRate = ofNullable(builder.getExchangeRate()).map(f->f.build()).orElse(null);
		this.exchangedCurrency1 = ofNullable(builder.getExchangedCurrency1()).map(f->f.build()).orElse(null);
		this.exchangedCurrency2 = ofNullable(builder.getExchangedCurrency2()).map(f->f.build()).orElse(null);
		this.tenorPeriod = ofNullable(builder.getTenorPeriod()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The rate of exchange between the two currencies.
	 */
	@RosettaSynonym(value="exchangeRate", source="FpML_5_10")
	public final ExchangeRate getExchangeRate() {
		return exchangeRate;
	}
	
	/**
	 * This is the first of the two currency flows that define a single leg of a standard foreign exchange transaction.
	 */
	@RosettaSynonym(value="exchangedCurrency1", source="FpML_5_10")
	public final Cashflow getExchangedCurrency1() {
		return exchangedCurrency1;
	}
	
	/**
	 * This is the second of the two currency flows that define a single leg of a standard foreign exchange transaction.
	 */
	@RosettaSynonym(value="exchangedCurrency2", source="FpML_5_10")
	public final Cashflow getExchangedCurrency2() {
		return exchangedCurrency2;
	}
	
	/**
	 * A tenor expressed as a period type and multiplier (e.g. 1D, 1Y, etc.)
	 */
	@RosettaSynonym(value="tenorPeriod", source="FpML_5_10")
	public final Period getTenorPeriod() {
		return tenorPeriod;
	}
	
	
	@Override
	public RosettaMetaData<? extends ForeignExchange> metaData() {
		return metaData;
	} 
	
	public ForeignExchangeBuilder toBuilder() {
		ForeignExchangeBuilder builder = new ForeignExchangeBuilder();
		ofNullable(getExchangeRate()).ifPresent(builder::setExchangeRate);
		ofNullable(getExchangedCurrency1()).ifPresent(builder::setExchangedCurrency1);
		ofNullable(getExchangedCurrency2()).ifPresent(builder::setExchangedCurrency2);
		ofNullable(getTenorPeriod()).ifPresent(builder::setTenorPeriod);
		return builder;
	}

	public static ForeignExchangeBuilder builder() {
		return new ForeignExchangeBuilder();
	}

	public static class ForeignExchangeBuilder extends RosettaModelObjectBuilder{
	
		protected ExchangeRate.ExchangeRateBuilder exchangeRate;
		protected Cashflow.CashflowBuilder exchangedCurrency1;
		protected Cashflow.CashflowBuilder exchangedCurrency2;
		protected Period.PeriodBuilder tenorPeriod;
	
		public ForeignExchangeBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends ForeignExchange> metaData() {
			return metaData;
		} 
	
		
		public ExchangeRate.ExchangeRateBuilder getExchangeRate() {
			return exchangeRate;
		}
		
		public ExchangeRate.ExchangeRateBuilder getOrCreateExchangeRate() {
			if (exchangeRate!=null) {
				return exchangeRate;
			}
			else return exchangeRate = new ExchangeRate.ExchangeRateBuilder();
		}
		
		
		public Cashflow.CashflowBuilder getExchangedCurrency1() {
			return exchangedCurrency1;
		}
		
		public Cashflow.CashflowBuilder getOrCreateExchangedCurrency1() {
			if (exchangedCurrency1!=null) {
				return exchangedCurrency1;
			}
			else return exchangedCurrency1 = new Cashflow.CashflowBuilder();
		}
		
		
		public Cashflow.CashflowBuilder getExchangedCurrency2() {
			return exchangedCurrency2;
		}
		
		public Cashflow.CashflowBuilder getOrCreateExchangedCurrency2() {
			if (exchangedCurrency2!=null) {
				return exchangedCurrency2;
			}
			else return exchangedCurrency2 = new Cashflow.CashflowBuilder();
		}
		
		
		public Period.PeriodBuilder getTenorPeriod() {
			return tenorPeriod;
		}
		
		public Period.PeriodBuilder getOrCreateTenorPeriod() {
			if (tenorPeriod!=null) {
				return tenorPeriod;
			}
			else return tenorPeriod = new Period.PeriodBuilder();
		}
		
	
		public ForeignExchangeBuilder setExchangeRate(ExchangeRate exchangeRate) {
			this.exchangeRate = exchangeRate.toBuilder();
			return this;
		}
		public ForeignExchangeBuilder setExchangeRateBuilder(ExchangeRate.ExchangeRateBuilder exchangeRate) {
			this.exchangeRate = exchangeRate;
			return this;
		}
		public ForeignExchangeBuilder setExchangedCurrency1(Cashflow exchangedCurrency1) {
			this.exchangedCurrency1 = exchangedCurrency1.toBuilder();
			return this;
		}
		public ForeignExchangeBuilder setExchangedCurrency1Builder(Cashflow.CashflowBuilder exchangedCurrency1) {
			this.exchangedCurrency1 = exchangedCurrency1;
			return this;
		}
		public ForeignExchangeBuilder setExchangedCurrency2(Cashflow exchangedCurrency2) {
			this.exchangedCurrency2 = exchangedCurrency2.toBuilder();
			return this;
		}
		public ForeignExchangeBuilder setExchangedCurrency2Builder(Cashflow.CashflowBuilder exchangedCurrency2) {
			this.exchangedCurrency2 = exchangedCurrency2;
			return this;
		}
		public ForeignExchangeBuilder setTenorPeriod(Period tenorPeriod) {
			this.tenorPeriod = tenorPeriod.toBuilder();
			return this;
		}
		public ForeignExchangeBuilder setTenorPeriodBuilder(Period.PeriodBuilder tenorPeriod) {
			this.tenorPeriod = tenorPeriod;
			return this;
		}
	
		public ForeignExchange build() {
			return new ForeignExchange(this);
		}
	
		@Override
		public ForeignExchangeBuilder prune() {
			if (exchangeRate!=null && !exchangeRate.prune().hasData()) exchangeRate = null;
			if (exchangedCurrency1!=null && !exchangedCurrency1.prune().hasData()) exchangedCurrency1 = null;
			if (exchangedCurrency2!=null && !exchangedCurrency2.prune().hasData()) exchangedCurrency2 = null;
			if (tenorPeriod!=null && !tenorPeriod.prune().hasData()) tenorPeriod = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getExchangeRate()!=null && getExchangeRate().hasData()) return true;
			if (getExchangedCurrency1()!=null && getExchangedCurrency1().hasData()) return true;
			if (getExchangedCurrency2()!=null && getExchangedCurrency2().hasData()) return true;
			if (getTenorPeriod()!=null && getTenorPeriod().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("exchangeRate"), processor, ExchangeRate.class, exchangeRate);
			processRosetta(path.newSubPath("exchangedCurrency1"), processor, Cashflow.class, exchangedCurrency1);
			processRosetta(path.newSubPath("exchangedCurrency2"), processor, Cashflow.class, exchangedCurrency2);
			processRosetta(path.newSubPath("tenorPeriod"), processor, Period.class, tenorPeriod);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			ForeignExchangeBuilder _that = (ForeignExchangeBuilder) o;
		
			if (exchangeRate != null ? !exchangeRate.equals(_that.exchangeRate) : _that.exchangeRate != null) return false;
			if (exchangedCurrency1 != null ? !exchangedCurrency1.equals(_that.exchangedCurrency1) : _that.exchangedCurrency1 != null) return false;
			if (exchangedCurrency2 != null ? !exchangedCurrency2.equals(_that.exchangedCurrency2) : _that.exchangedCurrency2 != null) return false;
			if (tenorPeriod != null ? !tenorPeriod.equals(_that.tenorPeriod) : _that.tenorPeriod != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (exchangeRate != null ? exchangeRate.hashCode() : 0);
			_result = 31 * _result + (exchangedCurrency1 != null ? exchangedCurrency1.hashCode() : 0);
			_result = 31 * _result + (exchangedCurrency2 != null ? exchangedCurrency2.hashCode() : 0);
			_result = 31 * _result + (tenorPeriod != null ? tenorPeriod.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "ForeignExchangeBuilder {" +
				"exchangeRate=" + this.exchangeRate + ", " +
				"exchangedCurrency1=" + this.exchangedCurrency1 + ", " +
				"exchangedCurrency2=" + this.exchangedCurrency2 + ", " +
				"tenorPeriod=" + this.tenorPeriod +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("exchangeRate"), processor, ExchangeRate.class, exchangeRate);
		processRosetta(path.newSubPath("exchangedCurrency1"), processor, Cashflow.class, exchangedCurrency1);
		processRosetta(path.newSubPath("exchangedCurrency2"), processor, Cashflow.class, exchangedCurrency2);
		processRosetta(path.newSubPath("tenorPeriod"), processor, Period.class, tenorPeriod);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		ForeignExchange _that = (ForeignExchange) o;
	
		if (exchangeRate != null ? !exchangeRate.equals(_that.exchangeRate) : _that.exchangeRate != null) return false;
		if (exchangedCurrency1 != null ? !exchangedCurrency1.equals(_that.exchangedCurrency1) : _that.exchangedCurrency1 != null) return false;
		if (exchangedCurrency2 != null ? !exchangedCurrency2.equals(_that.exchangedCurrency2) : _that.exchangedCurrency2 != null) return false;
		if (tenorPeriod != null ? !tenorPeriod.equals(_that.tenorPeriod) : _that.tenorPeriod != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (exchangeRate != null ? exchangeRate.hashCode() : 0);
		_result = 31 * _result + (exchangedCurrency1 != null ? exchangedCurrency1.hashCode() : 0);
		_result = 31 * _result + (exchangedCurrency2 != null ? exchangedCurrency2.hashCode() : 0);
		_result = 31 * _result + (tenorPeriod != null ? tenorPeriod.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "ForeignExchange {" +
			"exchangeRate=" + this.exchangeRate + ", " +
			"exchangedCurrency1=" + this.exchangedCurrency1 + ", " +
			"exchangedCurrency2=" + this.exchangedCurrency2 + ", " +
			"tenorPeriod=" + this.tenorPeriod +
		'}';
	}
}
