package org.isda.cdm;

import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.OptionSettlement;
import org.isda.cdm.Underlier;
import org.isda.cdm.meta.ForwardPayoutMeta;

import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.BuilderProcessor;
import com.rosetta.model.lib.process.Processor;

import static java.util.Optional.ofNullable;

/**
 * Representation of a forward settling payout. The &#39;Underlier&#39; attribute captures the underlying payout, which is settled according to the &#39;SettlementTerms&#39; attribute. Both FX Spot and FX Forward should use this componen, while distinction between the two will be handled via product qualification.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class ForwardPayout extends RosettaModelObject {
	private final OptionSettlement settlementTerms;
	private final Underlier underlier;
	private static ForwardPayoutMeta metaData = new ForwardPayoutMeta();
	
	ForwardPayout(ForwardPayoutBuilder builder) {
		this.settlementTerms = ofNullable(builder.getSettlementTerms()).map(f->f.build()).orElse(null);
		this.underlier = ofNullable(builder.getUnderlier()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * Settlement terms for the underlier that include the settlement date, settlement method etc.
	 */
	public final OptionSettlement getSettlementTerms() {
		return settlementTerms;
	}
	
	/**
	 * Underlying product that the forward is written on, which can be of any type: FX, a contractual product, a security, etc.
	 */
	public final Underlier getUnderlier() {
		return underlier;
	}
	
	
	@Override
	public RosettaMetaData<? extends ForwardPayout> metaData() {
		return metaData;
	} 
	
	public ForwardPayoutBuilder toBuilder() {
		ForwardPayoutBuilder builder = new ForwardPayoutBuilder();
		ofNullable(getSettlementTerms()).ifPresent(builder::setSettlementTerms);
		ofNullable(getUnderlier()).ifPresent(builder::setUnderlier);
		return builder;
	}

	public static ForwardPayoutBuilder builder() {
		return new ForwardPayoutBuilder();
	}

	public static class ForwardPayoutBuilder extends RosettaModelObjectBuilder{
	
		protected OptionSettlement.OptionSettlementBuilder settlementTerms;
		protected Underlier.UnderlierBuilder underlier;
	
		public ForwardPayoutBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends ForwardPayout> metaData() {
			return metaData;
		} 
	
		
		public OptionSettlement.OptionSettlementBuilder getSettlementTerms() {
			return settlementTerms;
		}
		
		public OptionSettlement.OptionSettlementBuilder getOrCreateSettlementTerms() {
			if (settlementTerms!=null) {
				return settlementTerms;
			}
			else return settlementTerms = new OptionSettlement.OptionSettlementBuilder();
		}
		
		
		public Underlier.UnderlierBuilder getUnderlier() {
			return underlier;
		}
		
		public Underlier.UnderlierBuilder getOrCreateUnderlier() {
			if (underlier!=null) {
				return underlier;
			}
			else return underlier = new Underlier.UnderlierBuilder();
		}
		
	
		public ForwardPayoutBuilder setSettlementTerms(OptionSettlement settlementTerms) {
			this.settlementTerms = settlementTerms.toBuilder();
			return this;
		}
		public ForwardPayoutBuilder setSettlementTermsBuilder(OptionSettlement.OptionSettlementBuilder settlementTerms) {
			this.settlementTerms = settlementTerms;
			return this;
		}
		public ForwardPayoutBuilder setUnderlier(Underlier underlier) {
			this.underlier = underlier.toBuilder();
			return this;
		}
		public ForwardPayoutBuilder setUnderlierBuilder(Underlier.UnderlierBuilder underlier) {
			this.underlier = underlier;
			return this;
		}
	
		public ForwardPayout build() {
			return new ForwardPayout(this);
		}
	
		@Override
		public ForwardPayoutBuilder prune() {
			if (settlementTerms!=null && !settlementTerms.prune().hasData()) settlementTerms = null;
			if (underlier!=null && !underlier.prune().hasData()) underlier = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getSettlementTerms()!=null && getSettlementTerms().hasData()) return true;
			if (getUnderlier()!=null && getUnderlier().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("settlementTerms"), processor, OptionSettlement.class, settlementTerms);
			processRosetta(path.newSubPath("underlier"), processor, Underlier.class, underlier);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			ForwardPayoutBuilder _that = (ForwardPayoutBuilder) o;
		
			if (settlementTerms != null ? !settlementTerms.equals(_that.settlementTerms) : _that.settlementTerms != null) return false;
			if (underlier != null ? !underlier.equals(_that.underlier) : _that.underlier != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (settlementTerms != null ? settlementTerms.hashCode() : 0);
			_result = 31 * _result + (underlier != null ? underlier.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "ForwardPayoutBuilder {" +
				"settlementTerms=" + this.settlementTerms + ", " +
				"underlier=" + this.underlier +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("settlementTerms"), processor, OptionSettlement.class, settlementTerms);
		processRosetta(path.newSubPath("underlier"), processor, Underlier.class, underlier);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		ForwardPayout _that = (ForwardPayout) o;
	
		if (settlementTerms != null ? !settlementTerms.equals(_that.settlementTerms) : _that.settlementTerms != null) return false;
		if (underlier != null ? !underlier.equals(_that.underlier) : _that.underlier != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (settlementTerms != null ? settlementTerms.hashCode() : 0);
		_result = 31 * _result + (underlier != null ? underlier.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "ForwardPayout {" +
			"settlementTerms=" + this.settlementTerms + ", " +
			"underlier=" + this.underlier +
		'}';
	}
}
