package org.isda.cdm;

import java.lang.Integer;
import org.isda.cdm.PeriodExtendedEnum;
import org.isda.cdm.metafields.MetaFields;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.FrequencyMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class for defining a date frequency, e.g. one day, three months, through the combination of an integer value and a standardized period value that is specified as part of an enumeration.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="Frequency", source="FpML_5_10")
@RosettaSynonym(value="Frequency", source="CME_SubmissionIRS_1_0")
@RosettaSynonym(value="Frequency", source="DTCC_11_0")
@RosettaSynonym(value="Frequency", source="DTCC_9_0")
@RosettaSynonym(value="Frequency", source="CME_ClearedConfirm_1_17")
public class Frequency extends RosettaModelObject implements GlobalKey {
	private final MetaFields meta;
	private final PeriodExtendedEnum period;
	private final Integer periodMultiplier;
	private static FrequencyMeta metaData = new FrequencyMeta();
	
	Frequency(FrequencyBuilder builder) {
		this.meta = ofNullable(builder.getMeta()).map(f->f.build()).orElse(null);
		this.period = builder.getPeriod();
		this.periodMultiplier = builder.getPeriodMultiplier();
	}
	
	public final MetaFields getMeta() {
		return meta;
	}
	
	/**
	 * A time period, e.g. a day, week, month, year or term of the stream.
	 */
	@RosettaSynonym(value="period", source="FpML_5_10")
	@RosettaSynonym(value="period", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="period", source="DTCC_11_0")
	@RosettaSynonym(value="period", source="DTCC_9_0")
	@RosettaSynonym(value="period", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="period", source="FpML_5_10", path="paymentDatesInterim.relativeDates")
	public final PeriodExtendedEnum getPeriod() {
		return period;
	}
	
	/**
	 * A time period multiplier, e.g. 1, 2, or 3. If the period value is T (Term) then period multiplier must contain the value 1.
	 */
	@RosettaSynonym(value="periodMultiplier", source="FpML_5_10")
	@RosettaSynonym(value="periodMultiplier", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="periodMultiplier", source="DTCC_11_0")
	@RosettaSynonym(value="periodMultiplier", source="DTCC_9_0")
	@RosettaSynonym(value="periodMultiplier", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="periodMultiplier", source="FpML_5_10", path="paymentDatesInterim.relativeDates")
	public final Integer getPeriodMultiplier() {
		return periodMultiplier;
	}
	
	
	@Override
	public RosettaMetaData<? extends Frequency> metaData() {
		return metaData;
	} 
	
	public FrequencyBuilder toBuilder() {
		FrequencyBuilder builder = new FrequencyBuilder();
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		ofNullable(getPeriod()).ifPresent(builder::setPeriod);
		ofNullable(getPeriodMultiplier()).ifPresent(builder::setPeriodMultiplier);
		return builder;
	}

	public static FrequencyBuilder builder() {
		return new FrequencyBuilder();
	}

	public static  class FrequencyBuilder extends RosettaModelObjectBuilder implements GlobalKeyBuilder<FrequencyBuilder> {
	
		protected MetaFields.MetaFieldsBuilder meta;
		protected PeriodExtendedEnum period;
		protected Integer periodMultiplier;
	
		public FrequencyBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends Frequency> metaData() {
			return metaData;
		} 
	
		
		public MetaFields.MetaFieldsBuilder getMeta() {
			return meta;
		}
		
		public MetaFields.MetaFieldsBuilder getOrCreateMeta() {
			if (meta!=null) {
				return meta;
			}
			else return meta = new MetaFields.MetaFieldsBuilder();
		}
		
		
		public PeriodExtendedEnum getPeriod() {
			return period;
		}
		
		
		public Integer getPeriodMultiplier() {
			return periodMultiplier;
		}
		
	
		public FrequencyBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		public FrequencyBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		public FrequencyBuilder setPeriod(PeriodExtendedEnum period) {
			this.period = period;
			return this;
		}
		public FrequencyBuilder setPeriodMultiplier(Integer periodMultiplier) {
			this.periodMultiplier = periodMultiplier;
			return this;
		}
		
		public Frequency build() {
			return new Frequency(this);
		}
	
		@Override
		public FrequencyBuilder prune() {
			if (meta!=null && !meta.prune().hasData()) meta = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getMeta()!=null && getMeta().hasData()) return true;
			if (getPeriod()!=null) return true;
			if (getPeriodMultiplier()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("period"), PeriodExtendedEnum.class, period, this);
			processor.processBasic(path.newSubPath("periodMultiplier"), Integer.class, periodMultiplier, this);
			
			processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			FrequencyBuilder _that = (FrequencyBuilder) o;
		
			if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
			if (period != null ? !period.equals(_that.period) : _that.period != null) return false;
			if (periodMultiplier != null ? !periodMultiplier.equals(_that.periodMultiplier) : _that.periodMultiplier != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
			_result = 31 * _result + (period != null ? period.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (periodMultiplier != null ? periodMultiplier.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "FrequencyBuilder {" +
				"meta=" + this.meta + ", " +
				"period=" + this.period + ", " +
				"periodMultiplier=" + this.periodMultiplier +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("period"), PeriodExtendedEnum.class, period, this);
		processor.processBasic(path.newSubPath("periodMultiplier"), Integer.class, periodMultiplier, this);
		
		processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		Frequency _that = (Frequency) o;
	
		if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
		if (period != null ? !period.equals(_that.period) : _that.period != null) return false;
		if (periodMultiplier != null ? !periodMultiplier.equals(_that.periodMultiplier) : _that.periodMultiplier != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
		_result = 31 * _result + (period != null ? period.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (periodMultiplier != null ? periodMultiplier.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "Frequency {" +
			"meta=" + this.meta + ", " +
			"period=" + this.period + ", " +
			"periodMultiplier=" + this.periodMultiplier +
		'}';
	}
}
