package org.isda.cdm;

import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.FxFixing;
import org.isda.cdm.FxRateSourceFixing;
import org.isda.cdm.metafields.FieldWithMetaString;
import org.isda.cdm.metafields.FieldWithMetaString.FieldWithMetaStringBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.FxCashSettlementMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;

/**
 * A class that is used for describing cash settlement of an option / non deliverable forward. It includes the currency to settle into together with the fixings required to calculate the currency amount.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="FxCashSettlement", source="FpML_5_10")
public class FxCashSettlement extends RosettaModelObject {
	private final List<FxFixing> fixing;
	private final List<FxRateSourceFixing> rateSourceFixing;
	private final FieldWithMetaString referenceCurrency;
	private final FieldWithMetaString settlementCurrency;
	private static FxCashSettlementMeta metaData = new FxCashSettlementMeta();
	
	FxCashSettlement(FxCashSettlementBuilder builder) {
		this.fixing = ofNullable(builder.getFixing()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.rateSourceFixing = ofNullable(builder.getRateSourceFixing()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.referenceCurrency = ofNullable(builder.getReferenceCurrency()).map(f->f.build()).orElse(null);
		this.settlementCurrency = ofNullable(builder.getSettlementCurrency()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * Specifies the source for and timing of a fixing of an exchange rate. This is used in the agreement of non-deliverable forward trades as well as various types of FX OTC options that require observations against a particular rate. This element is optional, permitting it to be omitted where fixing details are unavailable at the point of message creation. It has multiple occurrence to support the case where fixing details must be specified for more than one currency pair e.g. on an option settled into a third currency (that is not one of the option currencies).
	 */
	@RosettaSynonym(value="fixing", source="FpML_5_10")
	public final List<FxFixing> getFixing() {
		return fixing;
	}
	
	/**
	 * Specifies the source for and timing of a fixing of an exchange rate. This is used in the agreement of non-deliverable forward trades as well as various types of FX OTC options that require observations against a particular rate. This element is optional, permitting it to be omitted where fixing details are unavailable at the point of message creation. It has multiple occurrence to support the case where fixing details must be specified for more than one currency pair e.g. on an option settled into a third currency (that is not one of the option currencies).
	 */
	@RosettaSynonym(value="rateSourceFixing", source="FpML_5_10")
	public final List<FxRateSourceFixing> getRateSourceFixing() {
		return rateSourceFixing;
	}
	
	@RosettaSynonym(value="referenceCurrency", source="FpML_5_10")
	public final FieldWithMetaString getReferenceCurrency() {
		return referenceCurrency;
	}
	
	/**
	 * The currency in which cash settlement occurs for non-deliverable forwards and cash-settled options (non-deliverable or otherwise).
	 */
	@RosettaSynonym(value="settlementCurrency", source="FpML_5_10")
	public final FieldWithMetaString getSettlementCurrency() {
		return settlementCurrency;
	}
	
	
	@Override
	public RosettaMetaData<? extends FxCashSettlement> metaData() {
		return metaData;
	} 
	
	public FxCashSettlementBuilder toBuilder() {
		FxCashSettlementBuilder builder = new FxCashSettlementBuilder();
		ofNullable(getFixing()).ifPresent(fixing -> fixing.forEach(builder::addFixing));
		ofNullable(getRateSourceFixing()).ifPresent(rateSourceFixing -> rateSourceFixing.forEach(builder::addRateSourceFixing));
		ofNullable(getReferenceCurrency()).ifPresent(builder::setReferenceCurrency);
		ofNullable(getSettlementCurrency()).ifPresent(builder::setSettlementCurrency);
		return builder;
	}

	public static FxCashSettlementBuilder builder() {
		return new FxCashSettlementBuilder();
	}

	public static  class FxCashSettlementBuilder extends RosettaModelObjectBuilder {
	
		protected List<FxFixing.FxFixingBuilder> fixing;
		protected List<FxRateSourceFixing.FxRateSourceFixingBuilder> rateSourceFixing;
		protected FieldWithMetaStringBuilder referenceCurrency;
		protected FieldWithMetaStringBuilder settlementCurrency;
	
		public FxCashSettlementBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends FxCashSettlement> metaData() {
			return metaData;
		} 
	
		
		public List<FxFixing.FxFixingBuilder> getFixing() {
			return fixing;
		}
		
		public FxFixing.FxFixingBuilder getOrCreateFixing(int index) {
			if (fixing==null) {
				this.fixing = new ArrayList<>();
			}
			return getIndex(fixing, index, ()->new FxFixing.FxFixingBuilder());
		}
		
		
		public List<FxRateSourceFixing.FxRateSourceFixingBuilder> getRateSourceFixing() {
			return rateSourceFixing;
		}
		
		public FxRateSourceFixing.FxRateSourceFixingBuilder getOrCreateRateSourceFixing(int index) {
			if (rateSourceFixing==null) {
				this.rateSourceFixing = new ArrayList<>();
			}
			return getIndex(rateSourceFixing, index, ()->new FxRateSourceFixing.FxRateSourceFixingBuilder());
		}
		
		
		public FieldWithMetaStringBuilder getReferenceCurrency() {
			return referenceCurrency;
		}
		
		public FieldWithMetaStringBuilder getOrCreateReferenceCurrency() {
			if (referenceCurrency!=null) {
				return referenceCurrency;
			}
			else return referenceCurrency = new FieldWithMetaStringBuilder();
		}
		
		
		public FieldWithMetaStringBuilder getSettlementCurrency() {
			return settlementCurrency;
		}
		
		public FieldWithMetaStringBuilder getOrCreateSettlementCurrency() {
			if (settlementCurrency!=null) {
				return settlementCurrency;
			}
			else return settlementCurrency = new FieldWithMetaStringBuilder();
		}
		
	
		public FxCashSettlementBuilder addFixing(FxFixing fixing) {
			if(this.fixing == null){
				this.fixing = new ArrayList<>();
			}
			this.fixing.add(fixing.toBuilder());
			return this;
		}
		public FxCashSettlementBuilder addFixing(FxFixing fixing, int _idx) {
			if(this.fixing == null){
				this.fixing = new ArrayList<>();
			}
			getIndex(this.fixing, _idx, () -> fixing.toBuilder());
			this.fixing.set(_idx, fixing.toBuilder());
			return this;
		}
		public FxCashSettlementBuilder addFixing(List<FxFixing> fixings) {
			if(this.fixing == null){
				this.fixing = new ArrayList<>();
			}
			for (FxFixing toAdd : fixings) {
				this.fixing.add(toAdd.toBuilder());
			}
			return this;
		}
		public FxCashSettlementBuilder addFixingBuilder(FxFixing.FxFixingBuilder fixing) {
			if(this.fixing == null){
				this.fixing = new ArrayList<>();
			}
			this.fixing.add(fixing);
			return this;
		}
		
		public FxCashSettlementBuilder clearFixing() {
			this.fixing = null;
			return this;
		}
		public FxCashSettlementBuilder addRateSourceFixing(FxRateSourceFixing rateSourceFixing) {
			if(this.rateSourceFixing == null){
				this.rateSourceFixing = new ArrayList<>();
			}
			this.rateSourceFixing.add(rateSourceFixing.toBuilder());
			return this;
		}
		public FxCashSettlementBuilder addRateSourceFixing(FxRateSourceFixing rateSourceFixing, int _idx) {
			if(this.rateSourceFixing == null){
				this.rateSourceFixing = new ArrayList<>();
			}
			getIndex(this.rateSourceFixing, _idx, () -> rateSourceFixing.toBuilder());
			this.rateSourceFixing.set(_idx, rateSourceFixing.toBuilder());
			return this;
		}
		public FxCashSettlementBuilder addRateSourceFixing(List<FxRateSourceFixing> rateSourceFixings) {
			if(this.rateSourceFixing == null){
				this.rateSourceFixing = new ArrayList<>();
			}
			for (FxRateSourceFixing toAdd : rateSourceFixings) {
				this.rateSourceFixing.add(toAdd.toBuilder());
			}
			return this;
		}
		public FxCashSettlementBuilder addRateSourceFixingBuilder(FxRateSourceFixing.FxRateSourceFixingBuilder rateSourceFixing) {
			if(this.rateSourceFixing == null){
				this.rateSourceFixing = new ArrayList<>();
			}
			this.rateSourceFixing.add(rateSourceFixing);
			return this;
		}
		
		public FxCashSettlementBuilder clearRateSourceFixing() {
			this.rateSourceFixing = null;
			return this;
		}
		public FxCashSettlementBuilder setReferenceCurrency(FieldWithMetaString referenceCurrency) {
			this.referenceCurrency = referenceCurrency.toBuilder();
			return this;
		}
		public FxCashSettlementBuilder setReferenceCurrencyRef(String referenceCurrency) {
			return setReferenceCurrency(FieldWithMetaString.builder().setValue(referenceCurrency).build());
		}
		public FxCashSettlementBuilder setSettlementCurrency(FieldWithMetaString settlementCurrency) {
			this.settlementCurrency = settlementCurrency.toBuilder();
			return this;
		}
		public FxCashSettlementBuilder setSettlementCurrencyRef(String settlementCurrency) {
			return setSettlementCurrency(FieldWithMetaString.builder().setValue(settlementCurrency).build());
		}
		
		public FxCashSettlement build() {
			return new FxCashSettlement(this);
		}
	
		@Override
		public FxCashSettlementBuilder prune() {
			if (fixing!=null) fixing = fixing.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (rateSourceFixing!=null) rateSourceFixing = rateSourceFixing.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (referenceCurrency!=null && !referenceCurrency.prune().hasData()) referenceCurrency = null;
			if (settlementCurrency!=null && !settlementCurrency.prune().hasData()) settlementCurrency = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getFixing()!=null && getFixing().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getRateSourceFixing()!=null && getRateSourceFixing().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getReferenceCurrency()!=null) return true;
			if (getSettlementCurrency()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("fixing"), processor, FxFixing.class, fixing);
			processRosetta(path.newSubPath("rateSourceFixing"), processor, FxRateSourceFixing.class, rateSourceFixing);
			processRosetta(path.newSubPath("referenceCurrency"), processor, FieldWithMetaString.class, referenceCurrency);
			processRosetta(path.newSubPath("settlementCurrency"), processor, FieldWithMetaString.class, settlementCurrency);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			FxCashSettlementBuilder _that = (FxCashSettlementBuilder) o;
		
			if (!ListEquals.listEquals(fixing, _that.fixing)) return false;
			if (!ListEquals.listEquals(rateSourceFixing, _that.rateSourceFixing)) return false;
			if (referenceCurrency != null ? !referenceCurrency.equals(_that.referenceCurrency) : _that.referenceCurrency != null) return false;
			if (settlementCurrency != null ? !settlementCurrency.equals(_that.settlementCurrency) : _that.settlementCurrency != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (fixing != null ? fixing.hashCode() : 0);
			_result = 31 * _result + (rateSourceFixing != null ? rateSourceFixing.hashCode() : 0);
			_result = 31 * _result + (referenceCurrency != null ? referenceCurrency.hashCode() : 0);
			_result = 31 * _result + (settlementCurrency != null ? settlementCurrency.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "FxCashSettlementBuilder {" +
				"fixing=" + this.fixing + ", " +
				"rateSourceFixing=" + this.rateSourceFixing + ", " +
				"referenceCurrency=" + this.referenceCurrency + ", " +
				"settlementCurrency=" + this.settlementCurrency +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("fixing"), processor, FxFixing.class, fixing);
		processRosetta(path.newSubPath("rateSourceFixing"), processor, FxRateSourceFixing.class, rateSourceFixing);
		processRosetta(path.newSubPath("referenceCurrency"), processor, FieldWithMetaString.class, referenceCurrency);
		processRosetta(path.newSubPath("settlementCurrency"), processor, FieldWithMetaString.class, settlementCurrency);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		FxCashSettlement _that = (FxCashSettlement) o;
	
		if (!ListEquals.listEquals(fixing, _that.fixing)) return false;
		if (!ListEquals.listEquals(rateSourceFixing, _that.rateSourceFixing)) return false;
		if (referenceCurrency != null ? !referenceCurrency.equals(_that.referenceCurrency) : _that.referenceCurrency != null) return false;
		if (settlementCurrency != null ? !settlementCurrency.equals(_that.settlementCurrency) : _that.settlementCurrency != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (fixing != null ? fixing.hashCode() : 0);
		_result = 31 * _result + (rateSourceFixing != null ? rateSourceFixing.hashCode() : 0);
		_result = 31 * _result + (referenceCurrency != null ? referenceCurrency.hashCode() : 0);
		_result = 31 * _result + (settlementCurrency != null ? settlementCurrency.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "FxCashSettlement {" +
			"fixing=" + this.fixing + ", " +
			"rateSourceFixing=" + this.rateSourceFixing + ", " +
			"referenceCurrency=" + this.referenceCurrency + ", " +
			"settlementCurrency=" + this.settlementCurrency +
		'}';
	}
}
