package org.isda.cdm;

import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.meta.RosettaMetaData;
import com.rosetta.model.lib.records.Date;
import org.isda.cdm.FxSpotRateSource;
import org.isda.cdm.QuotedCurrencyPair;
import org.isda.cdm.meta.FxFixingMeta;

import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.BuilderProcessor;
import com.rosetta.model.lib.process.Processor;

import static java.util.Optional.ofNullable;

/**
 * A class that specifies the source for and timing of a fixing of an exchange rate. This is used in the agreement of non-deliverable forward trades as well as various types of FX OTC options that require observations against a particular rate.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="FxFixing", source="FpML_5_10")
public class FxFixing extends RosettaModelObject {
	private final Date fixingDate;
	private final FxSpotRateSource fxSpotRateSource;
	private final QuotedCurrencyPair quotedCurrencyPair;
	private static FxFixingMeta metaData = new FxFixingMeta();
	
	FxFixing(FxFixingBuilder builder) {
		this.fixingDate = builder.getFixingDate();
		this.fxSpotRateSource = ofNullable(builder.getFxSpotRateSource()).map(f->f.build()).orElse(null);
		this.quotedCurrencyPair = ofNullable(builder.getQuotedCurrencyPair()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * Describes the specific date when a non-deliverable forward or cash-settled option will &#39;fix&#39; against a particular rate, which will be used to compute the ultimate cash settlement. This element should be omitted where a single, discrete fixing date cannot be identified e.g. on an american option, where fixing may occur at any date on a continuous range.
	 */
	@RosettaSynonym(value="fixingDate", source="FpML_5_10")
	public final Date getFixingDate() {
		return fixingDate;
	}
	
	/**
	 * Specifies the methodology (reference source and, optionally, fixing time) to be used for determining a currency conversion rate.
	 */
	@RosettaSynonym(value="fxSpotRateSource", source="FpML_5_10")
	public final FxSpotRateSource getFxSpotRateSource() {
		return fxSpotRateSource;
	}
	
	/**
	 * Defines the two currencies for an FX trade and the quotation relationship between the two currencies.
	 */
	@RosettaSynonym(value="quotedCurrencyPair", source="FpML_5_10")
	public final QuotedCurrencyPair getQuotedCurrencyPair() {
		return quotedCurrencyPair;
	}
	
	
	@Override
	public RosettaMetaData<? extends FxFixing> metaData() {
		return metaData;
	} 
	
	public FxFixingBuilder toBuilder() {
		FxFixingBuilder builder = new FxFixingBuilder();
		ofNullable(getFixingDate()).ifPresent(builder::setFixingDate);
		ofNullable(getFxSpotRateSource()).ifPresent(builder::setFxSpotRateSource);
		ofNullable(getQuotedCurrencyPair()).ifPresent(builder::setQuotedCurrencyPair);
		return builder;
	}

	public static FxFixingBuilder builder() {
		return new FxFixingBuilder();
	}

	public static class FxFixingBuilder extends RosettaModelObjectBuilder{
	
		protected Date fixingDate;
		protected FxSpotRateSource.FxSpotRateSourceBuilder fxSpotRateSource;
		protected QuotedCurrencyPair.QuotedCurrencyPairBuilder quotedCurrencyPair;
	
		public FxFixingBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends FxFixing> metaData() {
			return metaData;
		} 
	
		
		public Date getFixingDate() {
			return fixingDate;
		}
		
		
		public FxSpotRateSource.FxSpotRateSourceBuilder getFxSpotRateSource() {
			return fxSpotRateSource;
		}
		
		public FxSpotRateSource.FxSpotRateSourceBuilder getOrCreateFxSpotRateSource() {
			if (fxSpotRateSource!=null) {
				return fxSpotRateSource;
			}
			else return fxSpotRateSource = new FxSpotRateSource.FxSpotRateSourceBuilder();
		}
		
		
		public QuotedCurrencyPair.QuotedCurrencyPairBuilder getQuotedCurrencyPair() {
			return quotedCurrencyPair;
		}
		
		public QuotedCurrencyPair.QuotedCurrencyPairBuilder getOrCreateQuotedCurrencyPair() {
			if (quotedCurrencyPair!=null) {
				return quotedCurrencyPair;
			}
			else return quotedCurrencyPair = new QuotedCurrencyPair.QuotedCurrencyPairBuilder();
		}
		
	
		public FxFixingBuilder setFixingDate(Date fixingDate) {
			this.fixingDate = fixingDate;
			return this;
		}
		public FxFixingBuilder setFxSpotRateSource(FxSpotRateSource fxSpotRateSource) {
			this.fxSpotRateSource = fxSpotRateSource.toBuilder();
			return this;
		}
		public FxFixingBuilder setFxSpotRateSourceBuilder(FxSpotRateSource.FxSpotRateSourceBuilder fxSpotRateSource) {
			this.fxSpotRateSource = fxSpotRateSource;
			return this;
		}
		public FxFixingBuilder setQuotedCurrencyPair(QuotedCurrencyPair quotedCurrencyPair) {
			this.quotedCurrencyPair = quotedCurrencyPair.toBuilder();
			return this;
		}
		public FxFixingBuilder setQuotedCurrencyPairBuilder(QuotedCurrencyPair.QuotedCurrencyPairBuilder quotedCurrencyPair) {
			this.quotedCurrencyPair = quotedCurrencyPair;
			return this;
		}
	
		public FxFixing build() {
			return new FxFixing(this);
		}
	
		@Override
		public FxFixingBuilder prune() {
			if (fxSpotRateSource!=null && !fxSpotRateSource.prune().hasData()) fxSpotRateSource = null;
			if (quotedCurrencyPair!=null && !quotedCurrencyPair.prune().hasData()) quotedCurrencyPair = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getFixingDate()!=null) return true;
			if (getFxSpotRateSource()!=null && getFxSpotRateSource().hasData()) return true;
			if (getQuotedCurrencyPair()!=null && getQuotedCurrencyPair().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("fixingDate"), Date.class, fixingDate, this);
			
			processRosetta(path.newSubPath("fxSpotRateSource"), processor, FxSpotRateSource.class, fxSpotRateSource);
			processRosetta(path.newSubPath("quotedCurrencyPair"), processor, QuotedCurrencyPair.class, quotedCurrencyPair);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			FxFixingBuilder _that = (FxFixingBuilder) o;
		
			if (fixingDate != null ? !fixingDate.equals(_that.fixingDate) : _that.fixingDate != null) return false;
			if (fxSpotRateSource != null ? !fxSpotRateSource.equals(_that.fxSpotRateSource) : _that.fxSpotRateSource != null) return false;
			if (quotedCurrencyPair != null ? !quotedCurrencyPair.equals(_that.quotedCurrencyPair) : _that.quotedCurrencyPair != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (fixingDate != null ? fixingDate.hashCode() : 0);
			_result = 31 * _result + (fxSpotRateSource != null ? fxSpotRateSource.hashCode() : 0);
			_result = 31 * _result + (quotedCurrencyPair != null ? quotedCurrencyPair.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "FxFixingBuilder {" +
				"fixingDate=" + this.fixingDate + ", " +
				"fxSpotRateSource=" + this.fxSpotRateSource + ", " +
				"quotedCurrencyPair=" + this.quotedCurrencyPair +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("fixingDate"), Date.class, fixingDate, this);
		
		processRosetta(path.newSubPath("fxSpotRateSource"), processor, FxSpotRateSource.class, fxSpotRateSource);
		processRosetta(path.newSubPath("quotedCurrencyPair"), processor, QuotedCurrencyPair.class, quotedCurrencyPair);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		FxFixing _that = (FxFixing) o;
	
		if (fixingDate != null ? !fixingDate.equals(_that.fixingDate) : _that.fixingDate != null) return false;
		if (fxSpotRateSource != null ? !fxSpotRateSource.equals(_that.fxSpotRateSource) : _that.fxSpotRateSource != null) return false;
		if (quotedCurrencyPair != null ? !quotedCurrencyPair.equals(_that.quotedCurrencyPair) : _that.quotedCurrencyPair != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (fixingDate != null ? fixingDate.hashCode() : 0);
		_result = 31 * _result + (fxSpotRateSource != null ? fxSpotRateSource.hashCode() : 0);
		_result = 31 * _result + (quotedCurrencyPair != null ? quotedCurrencyPair.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "FxFixing {" +
			"fixingDate=" + this.fixingDate + ", " +
			"fxSpotRateSource=" + this.fxSpotRateSource + ", " +
			"quotedCurrencyPair=" + this.quotedCurrencyPair +
		'}';
	}
}
