package org.isda.cdm;

import org.isda.cdm.BusinessCenters;
import org.isda.cdm.BusinessDayConventionEnum;
import org.isda.cdm.DateRelativeToCalculationPeriodDates;
import org.isda.cdm.DateRelativeToPaymentDates;
import org.isda.cdm.metafields.ReferenceWithMetaBusinessCenters;
import org.isda.cdm.metafields.ReferenceWithMetaBusinessCenters.ReferenceWithMetaBusinessCentersBuilder;


import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.FxFixingDateMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;
		import java.util.stream.Collectors;

/**
 * A class that is extending the Offset structure for providing the ability to specify an FX fixing date as an offset to dates specified somewhere else in the document.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="FxFixingDate", source="FpML_5_10")
public class FxFixingDate extends Offset {
	private final BusinessCenters businessCenters;
	private final ReferenceWithMetaBusinessCenters businessCentersReference;
	private final BusinessDayConventionEnum businessDayConvention;
	private final DateRelativeToCalculationPeriodDates dateRelativeToCalculationPeriodDates;
	private final DateRelativeToPaymentDates dateRelativeToPaymentDates;
	private static FxFixingDateMeta metaData = new FxFixingDateMeta();
	
	FxFixingDate(FxFixingDateBuilder builder) {
		super(builder);
		this.businessCenters = ofNullable(builder.getBusinessCenters()).map(f->f.build()).orElse(null);
		this.businessCentersReference = ofNullable(builder.getBusinessCentersReference()).map(f->f.build()).orElse(null);
		this.businessDayConvention = builder.getBusinessDayConvention();
		this.dateRelativeToCalculationPeriodDates = ofNullable(builder.getDateRelativeToCalculationPeriodDates()).map(f->f.build()).orElse(null);
		this.dateRelativeToPaymentDates = ofNullable(builder.getDateRelativeToPaymentDates()).map(f->f.build()).orElse(null);
	}
	
	@RosettaSynonym(value="businessCenters", source="FpML_5_10")
	@RosettaSynonym(value="businessCenters", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="businessCenters", source="DTCC_11_0")
	@RosettaSynonym(value="businessCenters", source="DTCC_9_0")
	@RosettaSynonym(value="businessCenters", source="CME_ClearedConfirm_1_17")
	public final BusinessCenters getBusinessCenters() {
		return businessCenters;
	}
	
	/**
	 * A reference to a set of financial business centers defined elsewhere in the document. This set of business centers is used to determine whether a particular day is a business day or not.
	 */
	@RosettaSynonym(value="businessCentersReference", source="FpML_5_10")
	@RosettaSynonym(value="businessCentersReference", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="businessCentersReference", source="DTCC_11_0")
	@RosettaSynonym(value="businessCentersReference", source="DTCC_9_0")
	@RosettaSynonym(value="businessCentersReference", source="CME_ClearedConfirm_1_17")
	public final ReferenceWithMetaBusinessCenters getBusinessCentersReference() {
		return businessCentersReference;
	}
	
	/**
	 * The convention for adjusting a date if it would otherwise fall on a day that is not a business day, as specified by an ISDA convention (e.g. Following, Precedent).
	 */
	@RosettaSynonym(value="businessDayConvention", source="FpML_5_10")
	@RosettaSynonym(value="businessDayConvention", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="businessDayConvention", source="DTCC_11_0")
	@RosettaSynonym(value="businessDayConvention", source="DTCC_9_0")
	@RosettaSynonym(value="businessDayConvention", source="CME_ClearedConfirm_1_17")
	public final BusinessDayConventionEnum getBusinessDayConvention() {
		return businessDayConvention;
	}
	
	/**
	 * The calculation period references on which settlements in non-deliverable currency are due and will then have to be converted according to the terms specified through the other parts of the nonDeliverableSettlement structure. Implemented for Brazilian-CDI swaps where it will refer to the termination date of the appropriate leg.
	 */
	@RosettaSynonym(value="dateRelativeToCalculationPeriodDates", source="FpML_5_10")
	@RosettaSynonym(value="dateRelativeToCalculationPeriodDates", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="dateRelativeToCalculationPeriodDates", source="DTCC_11_0")
	@RosettaSynonym(value="dateRelativeToCalculationPeriodDates", source="DTCC_9_0")
	@RosettaSynonym(value="dateRelativeToCalculationPeriodDates", source="CME_ClearedConfirm_1_17")
	public final DateRelativeToCalculationPeriodDates getDateRelativeToCalculationPeriodDates() {
		return dateRelativeToCalculationPeriodDates;
	}
	
	/**
	 * The payment date references on which settlements in non-deliverable currency are due and will then have to be converted according to the terms specified through the other parts of the nonDeliverableSettlement structure.
	 */
	@RosettaSynonym(value="dateRelativeToPaymentDates", source="FpML_5_10")
	@RosettaSynonym(value="dateRelativeToPaymentDates", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="dateRelativeToPaymentDates", source="DTCC_11_0")
	@RosettaSynonym(value="dateRelativeToPaymentDates", source="DTCC_9_0")
	@RosettaSynonym(value="dateRelativeToPaymentDates", source="CME_ClearedConfirm_1_17")
	public final DateRelativeToPaymentDates getDateRelativeToPaymentDates() {
		return dateRelativeToPaymentDates;
	}
	
	
	@Override
	public RosettaMetaData<? extends FxFixingDate> metaData() {
		return metaData;
	} 
	
	public FxFixingDateBuilder toBuilder() {
		FxFixingDateBuilder builder = new FxFixingDateBuilder();
		ofNullable(getBusinessCenters()).ifPresent(builder::setBusinessCenters);
		ofNullable(getBusinessCentersReference()).ifPresent(builder::setBusinessCentersReference);
		ofNullable(getBusinessDayConvention()).ifPresent(builder::setBusinessDayConvention);
		ofNullable(getDateRelativeToCalculationPeriodDates()).ifPresent(builder::setDateRelativeToCalculationPeriodDates);
		ofNullable(getDateRelativeToPaymentDates()).ifPresent(builder::setDateRelativeToPaymentDates);
		ofNullable(getDayType()).ifPresent(builder::setDayType);
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		ofNullable(getPeriod()).ifPresent(builder::setPeriod);
		ofNullable(getPeriodMultiplier()).ifPresent(builder::setPeriodMultiplier);
		return builder;
	}

	public static FxFixingDateBuilder builder() {
		return new FxFixingDateBuilder();
	}

	public static  class FxFixingDateBuilder extends OffsetBuilder {
	
		protected BusinessCenters.BusinessCentersBuilder businessCenters;
		protected ReferenceWithMetaBusinessCentersBuilder businessCentersReference;
		protected BusinessDayConventionEnum businessDayConvention;
		protected DateRelativeToCalculationPeriodDates.DateRelativeToCalculationPeriodDatesBuilder dateRelativeToCalculationPeriodDates;
		protected DateRelativeToPaymentDates.DateRelativeToPaymentDatesBuilder dateRelativeToPaymentDates;
	
		public FxFixingDateBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends FxFixingDate> metaData() {
			return metaData;
		} 
	
		
		public BusinessCenters.BusinessCentersBuilder getBusinessCenters() {
			return businessCenters;
		}
		
		public BusinessCenters.BusinessCentersBuilder getOrCreateBusinessCenters() {
			if (businessCenters!=null) {
				return businessCenters;
			}
			else return businessCenters = new BusinessCenters.BusinessCentersBuilder();
		}
		
		
		public ReferenceWithMetaBusinessCentersBuilder getBusinessCentersReference() {
			return businessCentersReference;
		}
		
		public ReferenceWithMetaBusinessCentersBuilder getOrCreateBusinessCentersReference() {
			if (businessCentersReference!=null) {
				return businessCentersReference;
			}
			else return businessCentersReference = new ReferenceWithMetaBusinessCentersBuilder();
		}
		
		
		public BusinessDayConventionEnum getBusinessDayConvention() {
			return businessDayConvention;
		}
		
		
		public DateRelativeToCalculationPeriodDates.DateRelativeToCalculationPeriodDatesBuilder getDateRelativeToCalculationPeriodDates() {
			return dateRelativeToCalculationPeriodDates;
		}
		
		public DateRelativeToCalculationPeriodDates.DateRelativeToCalculationPeriodDatesBuilder getOrCreateDateRelativeToCalculationPeriodDates() {
			if (dateRelativeToCalculationPeriodDates!=null) {
				return dateRelativeToCalculationPeriodDates;
			}
			else return dateRelativeToCalculationPeriodDates = new DateRelativeToCalculationPeriodDates.DateRelativeToCalculationPeriodDatesBuilder();
		}
		
		
		public DateRelativeToPaymentDates.DateRelativeToPaymentDatesBuilder getDateRelativeToPaymentDates() {
			return dateRelativeToPaymentDates;
		}
		
		public DateRelativeToPaymentDates.DateRelativeToPaymentDatesBuilder getOrCreateDateRelativeToPaymentDates() {
			if (dateRelativeToPaymentDates!=null) {
				return dateRelativeToPaymentDates;
			}
			else return dateRelativeToPaymentDates = new DateRelativeToPaymentDates.DateRelativeToPaymentDatesBuilder();
		}
		
	
		public FxFixingDateBuilder setBusinessCenters(BusinessCenters businessCenters) {
			this.businessCenters = businessCenters.toBuilder();
			return this;
		}
		public FxFixingDateBuilder setBusinessCentersBuilder(BusinessCenters.BusinessCentersBuilder businessCenters) {
			this.businessCenters = businessCenters;
			return this;
		}
		public FxFixingDateBuilder setBusinessCentersReference(ReferenceWithMetaBusinessCenters businessCentersReference) {
			this.businessCentersReference = businessCentersReference.toBuilder();
			return this;
		}
		public FxFixingDateBuilder setBusinessCentersReferenceBuilder(ReferenceWithMetaBusinessCentersBuilder businessCentersReference) {
			this.businessCentersReference = businessCentersReference;
			return this;
		}
		public FxFixingDateBuilder setBusinessCentersReferenceRef(BusinessCenters.BusinessCentersBuilder businessCentersReference) {
			return setBusinessCentersReference(ReferenceWithMetaBusinessCenters.builder().setValueBuilder(businessCentersReference).build());
		}
		public FxFixingDateBuilder setBusinessCentersReferenceRef(BusinessCenters businessCentersReference) {
			return setBusinessCentersReferenceRef(BusinessCenters.builder());
		}
		public FxFixingDateBuilder setBusinessDayConvention(BusinessDayConventionEnum businessDayConvention) {
			this.businessDayConvention = businessDayConvention;
			return this;
		}
		public FxFixingDateBuilder setDateRelativeToCalculationPeriodDates(DateRelativeToCalculationPeriodDates dateRelativeToCalculationPeriodDates) {
			this.dateRelativeToCalculationPeriodDates = dateRelativeToCalculationPeriodDates.toBuilder();
			return this;
		}
		public FxFixingDateBuilder setDateRelativeToCalculationPeriodDatesBuilder(DateRelativeToCalculationPeriodDates.DateRelativeToCalculationPeriodDatesBuilder dateRelativeToCalculationPeriodDates) {
			this.dateRelativeToCalculationPeriodDates = dateRelativeToCalculationPeriodDates;
			return this;
		}
		public FxFixingDateBuilder setDateRelativeToPaymentDates(DateRelativeToPaymentDates dateRelativeToPaymentDates) {
			this.dateRelativeToPaymentDates = dateRelativeToPaymentDates.toBuilder();
			return this;
		}
		public FxFixingDateBuilder setDateRelativeToPaymentDatesBuilder(DateRelativeToPaymentDates.DateRelativeToPaymentDatesBuilder dateRelativeToPaymentDates) {
			this.dateRelativeToPaymentDates = dateRelativeToPaymentDates;
			return this;
		}
		@Override public FxFixingDateBuilder setDayType(DayTypeEnum dayType) {
			this.dayType = dayType;
			return this;
		}
		@Override public FxFixingDateBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		@Override public FxFixingDateBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		@Override public FxFixingDateBuilder setPeriod(PeriodEnum period) {
			this.period = period;
			return this;
		}
		@Override public FxFixingDateBuilder setPeriodMultiplier(Integer periodMultiplier) {
			this.periodMultiplier = periodMultiplier;
			return this;
		}
		
		public FxFixingDate build() {
			return new FxFixingDate(this);
		}
	
		@Override
		public FxFixingDateBuilder prune() {
			super.prune();
			if (businessCenters!=null && !businessCenters.prune().hasData()) businessCenters = null;
			if (businessCentersReference!=null && !businessCentersReference.prune().hasData()) businessCentersReference = null;
			if (dateRelativeToCalculationPeriodDates!=null && !dateRelativeToCalculationPeriodDates.prune().hasData()) dateRelativeToCalculationPeriodDates = null;
			if (dateRelativeToPaymentDates!=null && !dateRelativeToPaymentDates.prune().hasData()) dateRelativeToPaymentDates = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (super.hasData()) return true;
			if (getBusinessCenters()!=null && getBusinessCenters().hasData()) return true;
			if (getBusinessCentersReference()!=null && getBusinessCentersReference().hasData()) return true;
			if (getBusinessDayConvention()!=null) return true;
			if (getDateRelativeToCalculationPeriodDates()!=null && getDateRelativeToCalculationPeriodDates().hasData()) return true;
			if (getDateRelativeToPaymentDates()!=null && getDateRelativeToPaymentDates().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
			super.process(path, processor);
		
			processor.processBasic(path.newSubPath("businessDayConvention"), BusinessDayConventionEnum.class, businessDayConvention, this);
			
			processRosetta(path.newSubPath("businessCenters"), processor, BusinessCenters.class, businessCenters);
			processRosetta(path.newSubPath("businessCentersReference"), processor, ReferenceWithMetaBusinessCenters.class, businessCentersReference);
			processRosetta(path.newSubPath("dateRelativeToCalculationPeriodDates"), processor, DateRelativeToCalculationPeriodDates.class, dateRelativeToCalculationPeriodDates);
			processRosetta(path.newSubPath("dateRelativeToPaymentDates"), processor, DateRelativeToPaymentDates.class, dateRelativeToPaymentDates);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
			if (!super.equals(o)) return false;
		
			FxFixingDateBuilder _that = (FxFixingDateBuilder) o;
		
			if (businessCenters != null ? !businessCenters.equals(_that.businessCenters) : _that.businessCenters != null) return false;
			if (businessCentersReference != null ? !businessCentersReference.equals(_that.businessCentersReference) : _that.businessCentersReference != null) return false;
			if (businessDayConvention != null ? !businessDayConvention.equals(_that.businessDayConvention) : _that.businessDayConvention != null) return false;
			if (dateRelativeToCalculationPeriodDates != null ? !dateRelativeToCalculationPeriodDates.equals(_that.dateRelativeToCalculationPeriodDates) : _that.dateRelativeToCalculationPeriodDates != null) return false;
			if (dateRelativeToPaymentDates != null ? !dateRelativeToPaymentDates.equals(_that.dateRelativeToPaymentDates) : _that.dateRelativeToPaymentDates != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = super.hashCode();
			_result = 31 * _result + (businessCenters != null ? businessCenters.hashCode() : 0);
			_result = 31 * _result + (businessCentersReference != null ? businessCentersReference.hashCode() : 0);
			_result = 31 * _result + (businessDayConvention != null ? businessDayConvention.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (dateRelativeToCalculationPeriodDates != null ? dateRelativeToCalculationPeriodDates.hashCode() : 0);
			_result = 31 * _result + (dateRelativeToPaymentDates != null ? dateRelativeToPaymentDates.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "FxFixingDateBuilder {" +
				"businessCenters=" + this.businessCenters + ", " +
				"businessCentersReference=" + this.businessCentersReference + ", " +
				"businessDayConvention=" + this.businessDayConvention + ", " +
				"dateRelativeToCalculationPeriodDates=" + this.dateRelativeToCalculationPeriodDates + ", " +
				"dateRelativeToPaymentDates=" + this.dateRelativeToPaymentDates +
			'}' + " " + super.toString();
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		super.process(path, processor);
		
		processor.processBasic(path.newSubPath("businessDayConvention"), BusinessDayConventionEnum.class, businessDayConvention, this);
		
		processRosetta(path.newSubPath("businessCenters"), processor, BusinessCenters.class, businessCenters);
		processRosetta(path.newSubPath("businessCentersReference"), processor, ReferenceWithMetaBusinessCenters.class, businessCentersReference);
		processRosetta(path.newSubPath("dateRelativeToCalculationPeriodDates"), processor, DateRelativeToCalculationPeriodDates.class, dateRelativeToCalculationPeriodDates);
		processRosetta(path.newSubPath("dateRelativeToPaymentDates"), processor, DateRelativeToPaymentDates.class, dateRelativeToPaymentDates);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
		if (!super.equals(o)) return false;
	
		FxFixingDate _that = (FxFixingDate) o;
	
		if (businessCenters != null ? !businessCenters.equals(_that.businessCenters) : _that.businessCenters != null) return false;
		if (businessCentersReference != null ? !businessCentersReference.equals(_that.businessCentersReference) : _that.businessCentersReference != null) return false;
		if (businessDayConvention != null ? !businessDayConvention.equals(_that.businessDayConvention) : _that.businessDayConvention != null) return false;
		if (dateRelativeToCalculationPeriodDates != null ? !dateRelativeToCalculationPeriodDates.equals(_that.dateRelativeToCalculationPeriodDates) : _that.dateRelativeToCalculationPeriodDates != null) return false;
		if (dateRelativeToPaymentDates != null ? !dateRelativeToPaymentDates.equals(_that.dateRelativeToPaymentDates) : _that.dateRelativeToPaymentDates != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = super.hashCode();
		_result = 31 * _result + (businessCenters != null ? businessCenters.hashCode() : 0);
		_result = 31 * _result + (businessCentersReference != null ? businessCentersReference.hashCode() : 0);
		_result = 31 * _result + (businessDayConvention != null ? businessDayConvention.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (dateRelativeToCalculationPeriodDates != null ? dateRelativeToCalculationPeriodDates.hashCode() : 0);
		_result = 31 * _result + (dateRelativeToPaymentDates != null ? dateRelativeToPaymentDates.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "FxFixingDate {" +
			"businessCenters=" + this.businessCenters + ", " +
			"businessCentersReference=" + this.businessCentersReference + ", " +
			"businessDayConvention=" + this.businessDayConvention + ", " +
			"dateRelativeToCalculationPeriodDates=" + this.dateRelativeToCalculationPeriodDates + ", " +
			"dateRelativeToPaymentDates=" + this.dateRelativeToPaymentDates +
		'}' + " " + super.toString();
	}
}
