package org.isda.cdm;

import java.lang.Boolean;
import org.isda.cdm.metafields.FieldWithMetaString;
import org.isda.cdm.metafields.FieldWithMetaString.FieldWithMetaStringBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.FxHaircutCurrencyMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class to specify the reference currency for the purpose of specifying the FX Haircut relating to a posting obligation, as being either the Termination Currency or an FX Designated Currency.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class FxHaircutCurrency extends RosettaModelObject {
	private final FieldWithMetaString fxDesignatedCurrency;
	private final Boolean isTerminationCurrency;
	private static FxHaircutCurrencyMeta metaData = new FxHaircutCurrencyMeta();
	
	FxHaircutCurrency(FxHaircutCurrencyBuilder builder) {
		this.fxDesignatedCurrency = ofNullable(builder.getFxDesignatedCurrency()).map(f->f.build()).orElse(null);
		this.isTerminationCurrency = builder.getIsTerminationCurrency();
	}
	
	/**
	 * When specified, the reference currency for the purpose of specifying the FX Haircut relating to a posting obligation. The list of valid currencies is not presently positioned as an enumeration as part of the CDM because that scope is limited to the values specified by ISDA and FpML. As a result, implementers have to make reference to the relevant standard, such as the ISO 4217 standard for currency codes.
	 */
	@RosettaSynonym(value="fx_designated_currency", source="ISDA_Create_1_0")
	public final FieldWithMetaString getFxDesignatedCurrency() {
		return fxDesignatedCurrency;
	}
	
	/**
	 * The reference currency for the purpose of specifying the FX Haircut relating to a posting obligation is the Termination Currency when the Boolean value is set to True.
	 */
	@RosettaSynonym(value="", source="ISDA_Create_1_0")
	public final Boolean getIsTerminationCurrency() {
		return isTerminationCurrency;
	}
	
	
	@Override
	public RosettaMetaData<? extends FxHaircutCurrency> metaData() {
		return metaData;
	} 
	
	public FxHaircutCurrencyBuilder toBuilder() {
		FxHaircutCurrencyBuilder builder = new FxHaircutCurrencyBuilder();
		ofNullable(getFxDesignatedCurrency()).ifPresent(builder::setFxDesignatedCurrency);
		ofNullable(getIsTerminationCurrency()).ifPresent(builder::setIsTerminationCurrency);
		return builder;
	}

	public static FxHaircutCurrencyBuilder builder() {
		return new FxHaircutCurrencyBuilder();
	}

	public static  class FxHaircutCurrencyBuilder extends RosettaModelObjectBuilder {
	
		protected FieldWithMetaStringBuilder fxDesignatedCurrency;
		protected Boolean isTerminationCurrency;
	
		public FxHaircutCurrencyBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends FxHaircutCurrency> metaData() {
			return metaData;
		} 
	
		
		public FieldWithMetaStringBuilder getFxDesignatedCurrency() {
			return fxDesignatedCurrency;
		}
		
		public FieldWithMetaStringBuilder getOrCreateFxDesignatedCurrency() {
			if (fxDesignatedCurrency!=null) {
				return fxDesignatedCurrency;
			}
			else return fxDesignatedCurrency = new FieldWithMetaStringBuilder();
		}
		
		
		public Boolean getIsTerminationCurrency() {
			return isTerminationCurrency;
		}
		
	
		public FxHaircutCurrencyBuilder setFxDesignatedCurrency(FieldWithMetaString fxDesignatedCurrency) {
			this.fxDesignatedCurrency = fxDesignatedCurrency.toBuilder();
			return this;
		}
		public FxHaircutCurrencyBuilder setFxDesignatedCurrencyRef(String fxDesignatedCurrency) {
			return setFxDesignatedCurrency(FieldWithMetaString.builder().setValue(fxDesignatedCurrency).build());
		}
		public FxHaircutCurrencyBuilder setIsTerminationCurrency(Boolean isTerminationCurrency) {
			this.isTerminationCurrency = isTerminationCurrency;
			return this;
		}
		
		public FxHaircutCurrency build() {
			return new FxHaircutCurrency(this);
		}
	
		@Override
		public FxHaircutCurrencyBuilder prune() {
			if (fxDesignatedCurrency!=null && !fxDesignatedCurrency.prune().hasData()) fxDesignatedCurrency = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getFxDesignatedCurrency()!=null) return true;
			if (getIsTerminationCurrency()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("isTerminationCurrency"), Boolean.class, isTerminationCurrency, this);
			
			processRosetta(path.newSubPath("fxDesignatedCurrency"), processor, FieldWithMetaString.class, fxDesignatedCurrency);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			FxHaircutCurrencyBuilder _that = (FxHaircutCurrencyBuilder) o;
		
			if (fxDesignatedCurrency != null ? !fxDesignatedCurrency.equals(_that.fxDesignatedCurrency) : _that.fxDesignatedCurrency != null) return false;
			if (isTerminationCurrency != null ? !isTerminationCurrency.equals(_that.isTerminationCurrency) : _that.isTerminationCurrency != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (fxDesignatedCurrency != null ? fxDesignatedCurrency.hashCode() : 0);
			_result = 31 * _result + (isTerminationCurrency != null ? isTerminationCurrency.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "FxHaircutCurrencyBuilder {" +
				"fxDesignatedCurrency=" + this.fxDesignatedCurrency + ", " +
				"isTerminationCurrency=" + this.isTerminationCurrency +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("isTerminationCurrency"), Boolean.class, isTerminationCurrency, this);
		
		processRosetta(path.newSubPath("fxDesignatedCurrency"), processor, FieldWithMetaString.class, fxDesignatedCurrency);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		FxHaircutCurrency _that = (FxHaircutCurrency) o;
	
		if (fxDesignatedCurrency != null ? !fxDesignatedCurrency.equals(_that.fxDesignatedCurrency) : _that.fxDesignatedCurrency != null) return false;
		if (isTerminationCurrency != null ? !isTerminationCurrency.equals(_that.isTerminationCurrency) : _that.isTerminationCurrency != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (fxDesignatedCurrency != null ? fxDesignatedCurrency.hashCode() : 0);
		_result = 31 * _result + (isTerminationCurrency != null ? isTerminationCurrency.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "FxHaircutCurrency {" +
			"fxDesignatedCurrency=" + this.fxDesignatedCurrency + ", " +
			"isTerminationCurrency=" + this.isTerminationCurrency +
		'}';
	}
}
