package org.isda.cdm;

import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.BusinessCenterTime;
import org.isda.cdm.meta.FxInformationSourceMeta;
import org.isda.cdm.metafields.FieldWithMetaInformationProviderEnum;
import org.isda.cdm.metafields.FieldWithMetaString;

import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.BuilderProcessor;
import com.rosetta.model.lib.process.Processor;

import static java.util.Optional.ofNullable;

/**
 * Information source specific to Foreign Exchange products.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="FxInformationSource", source="FpML_5_10")
public class FxInformationSource extends InformationSource {
	private final BusinessCenterTime fixingTime;
	private static FxInformationSourceMeta metaData = new FxInformationSourceMeta();
	
	FxInformationSource(FxInformationSourceBuilder builder) {
		super(builder);
		this.fixingTime = ofNullable(builder.getFixingTime()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The time that the fixing will be taken along with a business center to define the time zone.
	 */
	public final BusinessCenterTime getFixingTime() {
		return fixingTime;
	}
	
	
	@Override
	public RosettaMetaData<? extends FxInformationSource> metaData() {
		return metaData;
	} 
	
	public FxInformationSourceBuilder toBuilder() {
		FxInformationSourceBuilder builder = new FxInformationSourceBuilder();
		ofNullable(getFixingTime()).ifPresent(builder::setFixingTime);
		ofNullable(getSourcePage()).ifPresent(builder::setSourcePage);
		ofNullable(getSourcePageHeading()).ifPresent(builder::setSourcePageHeading);
		ofNullable(getSourceProvider()).ifPresent(builder::setSourceProvider);
		return builder;
	}

	public static FxInformationSourceBuilder builder() {
		return new FxInformationSourceBuilder();
	}

	public static class FxInformationSourceBuilder extends InformationSourceBuilder{
	
		protected BusinessCenterTime.BusinessCenterTimeBuilder fixingTime;
	
		public FxInformationSourceBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends FxInformationSource> metaData() {
			return metaData;
		} 
	
		
		public BusinessCenterTime.BusinessCenterTimeBuilder getFixingTime() {
			return fixingTime;
		}
		
		public BusinessCenterTime.BusinessCenterTimeBuilder getOrCreateFixingTime() {
			if (fixingTime!=null) {
				return fixingTime;
			}
			else return fixingTime = new BusinessCenterTime.BusinessCenterTimeBuilder();
		}
		
	
		public FxInformationSourceBuilder setFixingTime(BusinessCenterTime fixingTime) {
			this.fixingTime = fixingTime.toBuilder();
			return this;
		}
		public FxInformationSourceBuilder setFixingTimeBuilder(BusinessCenterTime.BusinessCenterTimeBuilder fixingTime) {
			this.fixingTime = fixingTime;
			return this;
		}
		@Override public FxInformationSourceBuilder setSourcePage(FieldWithMetaString sourcePage) {
			this.sourcePage = sourcePage.toBuilder();
			return this;
		}
		@Override public FxInformationSourceBuilder setSourcePageRef(String sourcePage) {
			return setSourcePage(FieldWithMetaString.builder().setValue(sourcePage).build());
		}
		@Override public FxInformationSourceBuilder setSourcePageHeading(String sourcePageHeading) {
			this.sourcePageHeading = sourcePageHeading;
			return this;
		}
		@Override public FxInformationSourceBuilder setSourceProvider(FieldWithMetaInformationProviderEnum sourceProvider) {
			this.sourceProvider = sourceProvider.toBuilder();
			return this;
		}
		@Override public FxInformationSourceBuilder setSourceProviderRef(InformationProviderEnum sourceProvider) {
			return setSourceProvider(FieldWithMetaInformationProviderEnum.builder().setValue(sourceProvider).build());
		}
	
		public FxInformationSource build() {
			return new FxInformationSource(this);
		}
	
		@Override
		public FxInformationSourceBuilder prune() {
			super.prune();
			if (fixingTime!=null && !fixingTime.prune().hasData()) fixingTime = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (super.hasData()) return true;
			if (getFixingTime()!=null && getFixingTime().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
			super.process(path, processor);
		
			
			processRosetta(path.newSubPath("fixingTime"), processor, BusinessCenterTime.class, fixingTime);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
			if (!super.equals(o)) return false;
		
			FxInformationSourceBuilder _that = (FxInformationSourceBuilder) o;
		
			if (fixingTime != null ? !fixingTime.equals(_that.fixingTime) : _that.fixingTime != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = super.hashCode();
			_result = 31 * _result + (fixingTime != null ? fixingTime.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "FxInformationSourceBuilder {" +
				"fixingTime=" + this.fixingTime +
			'}' + " " + super.toString();
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		super.process(path, processor);
		
		
		processRosetta(path.newSubPath("fixingTime"), processor, BusinessCenterTime.class, fixingTime);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
		if (!super.equals(o)) return false;
	
		FxInformationSource _that = (FxInformationSource) o;
	
		if (fixingTime != null ? !fixingTime.equals(_that.fixingTime) : _that.fixingTime != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = super.hashCode();
		_result = 31 * _result + (fixingTime != null ? fixingTime.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "FxInformationSource {" +
			"fixingTime=" + this.fixingTime +
		'}' + " " + super.toString();
	}
}
