package org.isda.cdm;

import java.math.BigDecimal;
import org.isda.cdm.QuotedCurrencyPair;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.FxRateMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.math.BigDecimal;
		import static java.util.Optional.ofNullable;

/**
 * A class describing the rate of a currency conversion: pair of currency, quotation mode and exchange rate.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="FxRate", source="FpML_5_10")
public class FxRate extends RosettaModelObject {
	private final QuotedCurrencyPair quotedCurrencyPair;
	private final BigDecimal rate;
	private static FxRateMeta metaData = new FxRateMeta();
	
	FxRate(FxRateBuilder builder) {
		this.quotedCurrencyPair = ofNullable(builder.getQuotedCurrencyPair()).map(f->f.build()).orElse(null);
		this.rate = builder.getRate();
	}
	
	/**
	 * Defines the two currencies for an FX trade and the quotation relationship between the two currencies.
	 */
	@RosettaSynonym(value="quotedCurrencyPair", source="FpML_5_10")
	@RosettaSynonym(value="quotedCurrencyPair", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="quotedCurrencyPair", source="DTCC_11_0")
	@RosettaSynonym(value="quotedCurrencyPair", source="DTCC_9_0")
	@RosettaSynonym(value="quotedCurrencyPair", source="CME_ClearedConfirm_1_17")
	public final QuotedCurrencyPair getQuotedCurrencyPair() {
		return quotedCurrencyPair;
	}
	
	/**
	 * The rate of exchange between the two currencies of the leg of a deal. Must be specified with a quote basis.
	 */
	@RosettaSynonym(value="rate", source="FpML_5_10")
	@RosettaSynonym(value="rate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="rate", source="DTCC_11_0")
	@RosettaSynonym(value="rate", source="DTCC_9_0")
	@RosettaSynonym(value="rate", source="CME_ClearedConfirm_1_17")
	public final BigDecimal getRate() {
		return rate;
	}
	
	
	@Override
	public RosettaMetaData<? extends FxRate> metaData() {
		return metaData;
	} 
	
	public FxRateBuilder toBuilder() {
		FxRateBuilder builder = new FxRateBuilder();
		ofNullable(getQuotedCurrencyPair()).ifPresent(builder::setQuotedCurrencyPair);
		ofNullable(getRate()).ifPresent(builder::setRate);
		return builder;
	}

	public static FxRateBuilder builder() {
		return new FxRateBuilder();
	}

	public static  class FxRateBuilder extends RosettaModelObjectBuilder {
	
		protected QuotedCurrencyPair.QuotedCurrencyPairBuilder quotedCurrencyPair;
		protected BigDecimal rate;
	
		public FxRateBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends FxRate> metaData() {
			return metaData;
		} 
	
		
		public QuotedCurrencyPair.QuotedCurrencyPairBuilder getQuotedCurrencyPair() {
			return quotedCurrencyPair;
		}
		
		public QuotedCurrencyPair.QuotedCurrencyPairBuilder getOrCreateQuotedCurrencyPair() {
			if (quotedCurrencyPair!=null) {
				return quotedCurrencyPair;
			}
			else return quotedCurrencyPair = new QuotedCurrencyPair.QuotedCurrencyPairBuilder();
		}
		
		
		public BigDecimal getRate() {
			return rate;
		}
		
	
		public FxRateBuilder setQuotedCurrencyPair(QuotedCurrencyPair quotedCurrencyPair) {
			this.quotedCurrencyPair = quotedCurrencyPair.toBuilder();
			return this;
		}
		public FxRateBuilder setQuotedCurrencyPairBuilder(QuotedCurrencyPair.QuotedCurrencyPairBuilder quotedCurrencyPair) {
			this.quotedCurrencyPair = quotedCurrencyPair;
			return this;
		}
		public FxRateBuilder setRate(BigDecimal rate) {
			this.rate = rate;
			return this;
		}
		
		public FxRate build() {
			return new FxRate(this);
		}
	
		@Override
		public FxRateBuilder prune() {
			if (quotedCurrencyPair!=null && !quotedCurrencyPair.prune().hasData()) quotedCurrencyPair = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getQuotedCurrencyPair()!=null && getQuotedCurrencyPair().hasData()) return true;
			if (getRate()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("rate"), BigDecimal.class, rate, this);
			
			processRosetta(path.newSubPath("quotedCurrencyPair"), processor, QuotedCurrencyPair.class, quotedCurrencyPair);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			FxRateBuilder _that = (FxRateBuilder) o;
		
			if (quotedCurrencyPair != null ? !quotedCurrencyPair.equals(_that.quotedCurrencyPair) : _that.quotedCurrencyPair != null) return false;
			if (rate != null ? !rate.equals(_that.rate) : _that.rate != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (quotedCurrencyPair != null ? quotedCurrencyPair.hashCode() : 0);
			_result = 31 * _result + (rate != null ? rate.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "FxRateBuilder {" +
				"quotedCurrencyPair=" + this.quotedCurrencyPair + ", " +
				"rate=" + this.rate +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("rate"), BigDecimal.class, rate, this);
		
		processRosetta(path.newSubPath("quotedCurrencyPair"), processor, QuotedCurrencyPair.class, quotedCurrencyPair);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		FxRate _that = (FxRate) o;
	
		if (quotedCurrencyPair != null ? !quotedCurrencyPair.equals(_that.quotedCurrencyPair) : _that.quotedCurrencyPair != null) return false;
		if (rate != null ? !rate.equals(_that.rate) : _that.rate != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (quotedCurrencyPair != null ? quotedCurrencyPair.hashCode() : 0);
		_result = 31 * _result + (rate != null ? rate.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "FxRate {" +
			"quotedCurrencyPair=" + this.quotedCurrencyPair + ", " +
			"rate=" + this.rate +
		'}';
	}
}
