package org.isda.cdm;

import org.isda.cdm.FxInformationSource;
import org.isda.cdm.metafields.FieldWithMetaString;
import org.isda.cdm.metafields.FieldWithMetaString.FieldWithMetaStringBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.FxSettlementRateSourceMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * The source of the Foreign Exchange settlement rate.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="FxSettlementRateSource", source="FpML_5_10")
public class FxSettlementRateSource extends RosettaModelObject {
	private final FxInformationSource nonstandardSettlementRate;
	private final FieldWithMetaString settlementRateOption;
	private static FxSettlementRateSourceMeta metaData = new FxSettlementRateSourceMeta();
	
	FxSettlementRateSource(FxSettlementRateSourceBuilder builder) {
		this.nonstandardSettlementRate = ofNullable(builder.getNonstandardSettlementRate()).map(f->f.build()).orElse(null);
		this.settlementRateOption = ofNullable(builder.getSettlementRateOption()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * Indicates that a non-standard rate source will be used for the fixing.
	 */
	public final FxInformationSource getNonstandardSettlementRate() {
		return nonstandardSettlementRate;
	}
	
	/**
	 * Indicates that an officially defined rate settlement rate option will be the used for the fixing.
	 */
	@RosettaSynonym(value="settlementRateOption", source="FpML_5_10")
	public final FieldWithMetaString getSettlementRateOption() {
		return settlementRateOption;
	}
	
	
	@Override
	public RosettaMetaData<? extends FxSettlementRateSource> metaData() {
		return metaData;
	} 
	
	public FxSettlementRateSourceBuilder toBuilder() {
		FxSettlementRateSourceBuilder builder = new FxSettlementRateSourceBuilder();
		ofNullable(getNonstandardSettlementRate()).ifPresent(builder::setNonstandardSettlementRate);
		ofNullable(getSettlementRateOption()).ifPresent(builder::setSettlementRateOption);
		return builder;
	}

	public static FxSettlementRateSourceBuilder builder() {
		return new FxSettlementRateSourceBuilder();
	}

	public static  class FxSettlementRateSourceBuilder extends RosettaModelObjectBuilder {
	
		protected FxInformationSource.FxInformationSourceBuilder nonstandardSettlementRate;
		protected FieldWithMetaStringBuilder settlementRateOption;
	
		public FxSettlementRateSourceBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends FxSettlementRateSource> metaData() {
			return metaData;
		} 
	
		
		public FxInformationSource.FxInformationSourceBuilder getNonstandardSettlementRate() {
			return nonstandardSettlementRate;
		}
		
		public FxInformationSource.FxInformationSourceBuilder getOrCreateNonstandardSettlementRate() {
			if (nonstandardSettlementRate!=null) {
				return nonstandardSettlementRate;
			}
			else return nonstandardSettlementRate = new FxInformationSource.FxInformationSourceBuilder();
		}
		
		
		public FieldWithMetaStringBuilder getSettlementRateOption() {
			return settlementRateOption;
		}
		
		public FieldWithMetaStringBuilder getOrCreateSettlementRateOption() {
			if (settlementRateOption!=null) {
				return settlementRateOption;
			}
			else return settlementRateOption = new FieldWithMetaStringBuilder();
		}
		
	
		public FxSettlementRateSourceBuilder setNonstandardSettlementRate(FxInformationSource nonstandardSettlementRate) {
			this.nonstandardSettlementRate = nonstandardSettlementRate.toBuilder();
			return this;
		}
		public FxSettlementRateSourceBuilder setNonstandardSettlementRateBuilder(FxInformationSource.FxInformationSourceBuilder nonstandardSettlementRate) {
			this.nonstandardSettlementRate = nonstandardSettlementRate;
			return this;
		}
		public FxSettlementRateSourceBuilder setSettlementRateOption(FieldWithMetaString settlementRateOption) {
			this.settlementRateOption = settlementRateOption.toBuilder();
			return this;
		}
		public FxSettlementRateSourceBuilder setSettlementRateOptionRef(String settlementRateOption) {
			return setSettlementRateOption(FieldWithMetaString.builder().setValue(settlementRateOption).build());
		}
		
		public FxSettlementRateSource build() {
			return new FxSettlementRateSource(this);
		}
	
		@Override
		public FxSettlementRateSourceBuilder prune() {
			if (nonstandardSettlementRate!=null && !nonstandardSettlementRate.prune().hasData()) nonstandardSettlementRate = null;
			if (settlementRateOption!=null && !settlementRateOption.prune().hasData()) settlementRateOption = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getNonstandardSettlementRate()!=null && getNonstandardSettlementRate().hasData()) return true;
			if (getSettlementRateOption()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("nonstandardSettlementRate"), processor, FxInformationSource.class, nonstandardSettlementRate);
			processRosetta(path.newSubPath("settlementRateOption"), processor, FieldWithMetaString.class, settlementRateOption);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			FxSettlementRateSourceBuilder _that = (FxSettlementRateSourceBuilder) o;
		
			if (nonstandardSettlementRate != null ? !nonstandardSettlementRate.equals(_that.nonstandardSettlementRate) : _that.nonstandardSettlementRate != null) return false;
			if (settlementRateOption != null ? !settlementRateOption.equals(_that.settlementRateOption) : _that.settlementRateOption != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (nonstandardSettlementRate != null ? nonstandardSettlementRate.hashCode() : 0);
			_result = 31 * _result + (settlementRateOption != null ? settlementRateOption.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "FxSettlementRateSourceBuilder {" +
				"nonstandardSettlementRate=" + this.nonstandardSettlementRate + ", " +
				"settlementRateOption=" + this.settlementRateOption +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("nonstandardSettlementRate"), processor, FxInformationSource.class, nonstandardSettlementRate);
		processRosetta(path.newSubPath("settlementRateOption"), processor, FieldWithMetaString.class, settlementRateOption);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		FxSettlementRateSource _that = (FxSettlementRateSource) o;
	
		if (nonstandardSettlementRate != null ? !nonstandardSettlementRate.equals(_that.nonstandardSettlementRate) : _that.nonstandardSettlementRate != null) return false;
		if (settlementRateOption != null ? !settlementRateOption.equals(_that.settlementRateOption) : _that.settlementRateOption != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (nonstandardSettlementRate != null ? nonstandardSettlementRate.hashCode() : 0);
		_result = 31 * _result + (settlementRateOption != null ? settlementRateOption.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "FxSettlementRateSource {" +
			"nonstandardSettlementRate=" + this.nonstandardSettlementRate + ", " +
			"settlementRateOption=" + this.settlementRateOption +
		'}';
	}
}
