package org.isda.cdm;

import org.isda.cdm.BusinessCenterTime;
import org.isda.cdm.InformationSource;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.FxSpotRateSourceMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class defining the rate source and fixing time for an FX rate.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="FxSpotRateSource", source="FpML_5_10")
public class FxSpotRateSource extends RosettaModelObject {
	private final BusinessCenterTime fixingTime;
	private final InformationSource primaryRateSource;
	private final InformationSource secondaryRateSource;
	private static FxSpotRateSourceMeta metaData = new FxSpotRateSourceMeta();
	
	FxSpotRateSource(FxSpotRateSourceBuilder builder) {
		this.fixingTime = ofNullable(builder.getFixingTime()).map(f->f.build()).orElse(null);
		this.primaryRateSource = ofNullable(builder.getPrimaryRateSource()).map(f->f.build()).orElse(null);
		this.secondaryRateSource = ofNullable(builder.getSecondaryRateSource()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The time at which the spot currency exchange rate will be observed. It is specified as a time in a business day calendar location, e.g. 11:00am London time.
	 */
	@RosettaSynonym(value="fixingTime", source="FpML_5_10")
	@RosettaSynonym(value="fixingTime", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="fixingTime", source="DTCC_11_0")
	@RosettaSynonym(value="fixingTime", source="DTCC_9_0")
	@RosettaSynonym(value="fixingTime", source="CME_ClearedConfirm_1_17")
	public final BusinessCenterTime getFixingTime() {
		return fixingTime;
	}
	
	/**
	 * The primary source for where the rate observation will occur. Will typically be either a page or a reference bank published rate.
	 */
	@RosettaSynonym(value="primaryRateSource", source="FpML_5_10")
	@RosettaSynonym(value="primaryRateSource", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="primaryRateSource", source="DTCC_11_0")
	@RosettaSynonym(value="primaryRateSource", source="DTCC_9_0")
	@RosettaSynonym(value="primaryRateSource", source="CME_ClearedConfirm_1_17")
	public final InformationSource getPrimaryRateSource() {
		return primaryRateSource;
	}
	
	/**
	 * An alternative, or secondary, source for where the rate observation will occur. Will typically be either a page or a reference bank published rate.
	 */
	@RosettaSynonym(value="secondaryRateSource", source="FpML_5_10")
	@RosettaSynonym(value="secondaryRateSource", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="secondaryRateSource", source="DTCC_11_0")
	@RosettaSynonym(value="secondaryRateSource", source="DTCC_9_0")
	@RosettaSynonym(value="secondaryRateSource", source="CME_ClearedConfirm_1_17")
	public final InformationSource getSecondaryRateSource() {
		return secondaryRateSource;
	}
	
	
	@Override
	public RosettaMetaData<? extends FxSpotRateSource> metaData() {
		return metaData;
	} 
	
	public FxSpotRateSourceBuilder toBuilder() {
		FxSpotRateSourceBuilder builder = new FxSpotRateSourceBuilder();
		ofNullable(getFixingTime()).ifPresent(builder::setFixingTime);
		ofNullable(getPrimaryRateSource()).ifPresent(builder::setPrimaryRateSource);
		ofNullable(getSecondaryRateSource()).ifPresent(builder::setSecondaryRateSource);
		return builder;
	}

	public static FxSpotRateSourceBuilder builder() {
		return new FxSpotRateSourceBuilder();
	}

	public static  class FxSpotRateSourceBuilder extends RosettaModelObjectBuilder {
	
		protected BusinessCenterTime.BusinessCenterTimeBuilder fixingTime;
		protected InformationSource.InformationSourceBuilder primaryRateSource;
		protected InformationSource.InformationSourceBuilder secondaryRateSource;
	
		public FxSpotRateSourceBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends FxSpotRateSource> metaData() {
			return metaData;
		} 
	
		
		public BusinessCenterTime.BusinessCenterTimeBuilder getFixingTime() {
			return fixingTime;
		}
		
		public BusinessCenterTime.BusinessCenterTimeBuilder getOrCreateFixingTime() {
			if (fixingTime!=null) {
				return fixingTime;
			}
			else return fixingTime = new BusinessCenterTime.BusinessCenterTimeBuilder();
		}
		
		
		public InformationSource.InformationSourceBuilder getPrimaryRateSource() {
			return primaryRateSource;
		}
		
		public InformationSource.InformationSourceBuilder getOrCreatePrimaryRateSource() {
			if (primaryRateSource!=null) {
				return primaryRateSource;
			}
			else return primaryRateSource = new InformationSource.InformationSourceBuilder();
		}
		
		
		public InformationSource.InformationSourceBuilder getSecondaryRateSource() {
			return secondaryRateSource;
		}
		
		public InformationSource.InformationSourceBuilder getOrCreateSecondaryRateSource() {
			if (secondaryRateSource!=null) {
				return secondaryRateSource;
			}
			else return secondaryRateSource = new InformationSource.InformationSourceBuilder();
		}
		
	
		public FxSpotRateSourceBuilder setFixingTime(BusinessCenterTime fixingTime) {
			this.fixingTime = fixingTime.toBuilder();
			return this;
		}
		public FxSpotRateSourceBuilder setFixingTimeBuilder(BusinessCenterTime.BusinessCenterTimeBuilder fixingTime) {
			this.fixingTime = fixingTime;
			return this;
		}
		public FxSpotRateSourceBuilder setPrimaryRateSource(InformationSource primaryRateSource) {
			this.primaryRateSource = primaryRateSource.toBuilder();
			return this;
		}
		public FxSpotRateSourceBuilder setPrimaryRateSourceBuilder(InformationSource.InformationSourceBuilder primaryRateSource) {
			this.primaryRateSource = primaryRateSource;
			return this;
		}
		public FxSpotRateSourceBuilder setSecondaryRateSource(InformationSource secondaryRateSource) {
			this.secondaryRateSource = secondaryRateSource.toBuilder();
			return this;
		}
		public FxSpotRateSourceBuilder setSecondaryRateSourceBuilder(InformationSource.InformationSourceBuilder secondaryRateSource) {
			this.secondaryRateSource = secondaryRateSource;
			return this;
		}
		
		public FxSpotRateSource build() {
			return new FxSpotRateSource(this);
		}
	
		@Override
		public FxSpotRateSourceBuilder prune() {
			if (fixingTime!=null && !fixingTime.prune().hasData()) fixingTime = null;
			if (primaryRateSource!=null && !primaryRateSource.prune().hasData()) primaryRateSource = null;
			if (secondaryRateSource!=null && !secondaryRateSource.prune().hasData()) secondaryRateSource = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getFixingTime()!=null && getFixingTime().hasData()) return true;
			if (getPrimaryRateSource()!=null && getPrimaryRateSource().hasData()) return true;
			if (getSecondaryRateSource()!=null && getSecondaryRateSource().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("fixingTime"), processor, BusinessCenterTime.class, fixingTime);
			processRosetta(path.newSubPath("primaryRateSource"), processor, InformationSource.class, primaryRateSource);
			processRosetta(path.newSubPath("secondaryRateSource"), processor, InformationSource.class, secondaryRateSource);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			FxSpotRateSourceBuilder _that = (FxSpotRateSourceBuilder) o;
		
			if (fixingTime != null ? !fixingTime.equals(_that.fixingTime) : _that.fixingTime != null) return false;
			if (primaryRateSource != null ? !primaryRateSource.equals(_that.primaryRateSource) : _that.primaryRateSource != null) return false;
			if (secondaryRateSource != null ? !secondaryRateSource.equals(_that.secondaryRateSource) : _that.secondaryRateSource != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (fixingTime != null ? fixingTime.hashCode() : 0);
			_result = 31 * _result + (primaryRateSource != null ? primaryRateSource.hashCode() : 0);
			_result = 31 * _result + (secondaryRateSource != null ? secondaryRateSource.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "FxSpotRateSourceBuilder {" +
				"fixingTime=" + this.fixingTime + ", " +
				"primaryRateSource=" + this.primaryRateSource + ", " +
				"secondaryRateSource=" + this.secondaryRateSource +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("fixingTime"), processor, BusinessCenterTime.class, fixingTime);
		processRosetta(path.newSubPath("primaryRateSource"), processor, InformationSource.class, primaryRateSource);
		processRosetta(path.newSubPath("secondaryRateSource"), processor, InformationSource.class, secondaryRateSource);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		FxSpotRateSource _that = (FxSpotRateSource) o;
	
		if (fixingTime != null ? !fixingTime.equals(_that.fixingTime) : _that.fixingTime != null) return false;
		if (primaryRateSource != null ? !primaryRateSource.equals(_that.primaryRateSource) : _that.primaryRateSource != null) return false;
		if (secondaryRateSource != null ? !secondaryRateSource.equals(_that.secondaryRateSource) : _that.secondaryRateSource != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (fixingTime != null ? fixingTime.hashCode() : 0);
		_result = 31 * _result + (primaryRateSource != null ? primaryRateSource.hashCode() : 0);
		_result = 31 * _result + (secondaryRateSource != null ? secondaryRateSource.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "FxSpotRateSource {" +
			"fixingTime=" + this.fixingTime + ", " +
			"primaryRateSource=" + this.primaryRateSource + ", " +
			"secondaryRateSource=" + this.secondaryRateSource +
		'}';
	}
}
