package org.isda.cdm;

import java.lang.Boolean;
import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.BasketReferenceInformation;
import org.isda.cdm.BuyerSeller;
import org.isda.cdm.IndexReferenceInformation;
import org.isda.cdm.ReferenceInformation;
import org.isda.cdm.metafields.FieldWithMetaString;
import org.isda.cdm.metafields.FieldWithMetaString.FieldWithMetaStringBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.GeneralTermsMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;
		import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;

/**
 * A class specifying a set of non-monetary terms for the Credit Derivative Transaction, including the buyer and seller and selected items from the ISDA 2014 Credit Definition article II, such as the reference obligation and related terms. The CDM GeneralTerms class corresponds to the FpML GeneralTerms complex type, except that the effectiveDate and scheduledTerminationDate have been positioned as part of the InterestRatePayout class in the CDM instead of in GeneralTerms.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class GeneralTerms extends RosettaModelObject {
	private final List<FieldWithMetaString> additionalTerm;
	private final BasketReferenceInformation basketReferenceInformation;
	private final BuyerSeller buyerSeller;
	private final IndexReferenceInformation indexReferenceInformation;
	private final Boolean modifiedEquityDelivery;
	private final ReferenceInformation referenceInformation;
	private final Boolean substitution;
	private static GeneralTermsMeta metaData = new GeneralTermsMeta();
	
	GeneralTerms(GeneralTermsBuilder builder) {
		this.additionalTerm = ofNullable(builder.getAdditionalTerm()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.basketReferenceInformation = ofNullable(builder.getBasketReferenceInformation()).map(f->f.build()).orElse(null);
		this.buyerSeller = ofNullable(builder.getBuyerSeller()).map(f->f.build()).orElse(null);
		this.indexReferenceInformation = ofNullable(builder.getIndexReferenceInformation()).map(f->f.build()).orElse(null);
		this.modifiedEquityDelivery = builder.getModifiedEquityDelivery();
		this.referenceInformation = ofNullable(builder.getReferenceInformation()).map(f->f.build()).orElse(null);
		this.substitution = builder.getSubstitution();
	}
	
	/**
	 * This attribute is used for representing information contained in the Additional Terms field of the 2003 Master Credit Derivatives confirm.
	 */
	@RosettaSynonym(value="additionalTerm", source="FpML_5_10")
	@RosettaSynonym(value="additionalTerm", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="additionalTerm", source="DTCC_11_0")
	@RosettaSynonym(value="additionalTerm", source="DTCC_9_0")
	@RosettaSynonym(value="additionalTerm", source="CME_ClearedConfirm_1_17")
	public final List<FieldWithMetaString> getAdditionalTerm() {
		return additionalTerm;
	}
	
	/**
	 * This attribute contains all the terms relevant to defining the Credit Default Swap Basket.
	 */
	@RosettaSynonym(value="basketReferenceInformation", source="FpML_5_10")
	@RosettaSynonym(value="basketReferenceInformation", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="basketReferenceInformation", source="DTCC_11_0")
	@RosettaSynonym(value="basketReferenceInformation", source="DTCC_9_0")
	@RosettaSynonym(value="basketReferenceInformation", source="CME_ClearedConfirm_1_17")
	public final BasketReferenceInformation getBasketReferenceInformation() {
		return basketReferenceInformation;
	}
	
	/**
	 * The credit default payout buyer/seller parties.
	 */
	public final BuyerSeller getBuyerSeller() {
		return buyerSeller;
	}
	
	/**
	 * This attribute contains all the terms relevant to defining the Credit DefaultSwap Index.
	 */
	@RosettaSynonym(value="indexReferenceInformation", source="FpML_5_10")
	@RosettaSynonym(value="indexReferenceInformation", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="indexReferenceInformation", source="DTCC_11_0")
	@RosettaSynonym(value="indexReferenceInformation", source="DTCC_9_0")
	@RosettaSynonym(value="indexReferenceInformation", source="CME_ClearedConfirm_1_17")
	public final IndexReferenceInformation getIndexReferenceInformation() {
		return indexReferenceInformation;
	}
	
	/**
	 * Value of this attribute set to &#39;true&#39; indicates that modified equity delivery is applicable.
	 */
	@RosettaSynonym(value="modifiedEquityDelivery", source="FpML_5_10")
	@RosettaSynonym(value="modifiedEquityDelivery", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="modifiedEquityDelivery", source="DTCC_11_0")
	@RosettaSynonym(value="modifiedEquityDelivery", source="DTCC_9_0")
	@RosettaSynonym(value="modifiedEquityDelivery", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="modifiedEquityDelivery", source="Rosetta_Workbench")
	public final Boolean getModifiedEquityDelivery() {
		return modifiedEquityDelivery;
	}
	
	/**
	 * This attribute contains all the terms relevant to defining the reference entity and reference obligation(s).
	 */
	@RosettaSynonym(value="referenceInformation", source="FpML_5_10")
	@RosettaSynonym(value="referenceInformation", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="referenceInformation", source="DTCC_11_0")
	@RosettaSynonym(value="referenceInformation", source="DTCC_9_0")
	@RosettaSynonym(value="referenceInformation", source="CME_ClearedConfirm_1_17")
	public final ReferenceInformation getReferenceInformation() {
		return referenceInformation;
	}
	
	/**
	 * Value of this attribute set to &#39;true&#39; indicates that substitution is applicable.
	 */
	@RosettaSynonym(value="substitution", source="FpML_5_10")
	@RosettaSynonym(value="substitution", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="substitution", source="DTCC_11_0")
	@RosettaSynonym(value="substitution", source="DTCC_9_0")
	@RosettaSynonym(value="substitution", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="substitution", source="Rosetta_Workbench")
	public final Boolean getSubstitution() {
		return substitution;
	}
	
	
	@Override
	public RosettaMetaData<? extends GeneralTerms> metaData() {
		return metaData;
	} 
	
	public GeneralTermsBuilder toBuilder() {
		GeneralTermsBuilder builder = new GeneralTermsBuilder();
		ofNullable(getAdditionalTerm()).ifPresent(additionalTerm -> additionalTerm.forEach(builder::addAdditionalTerm));
		ofNullable(getBasketReferenceInformation()).ifPresent(builder::setBasketReferenceInformation);
		ofNullable(getBuyerSeller()).ifPresent(builder::setBuyerSeller);
		ofNullable(getIndexReferenceInformation()).ifPresent(builder::setIndexReferenceInformation);
		ofNullable(getModifiedEquityDelivery()).ifPresent(builder::setModifiedEquityDelivery);
		ofNullable(getReferenceInformation()).ifPresent(builder::setReferenceInformation);
		ofNullable(getSubstitution()).ifPresent(builder::setSubstitution);
		return builder;
	}

	public static GeneralTermsBuilder builder() {
		return new GeneralTermsBuilder();
	}

	public static  class GeneralTermsBuilder extends RosettaModelObjectBuilder {
	
		protected List<FieldWithMetaStringBuilder> additionalTerm;
		protected BasketReferenceInformation.BasketReferenceInformationBuilder basketReferenceInformation;
		protected BuyerSeller.BuyerSellerBuilder buyerSeller;
		protected IndexReferenceInformation.IndexReferenceInformationBuilder indexReferenceInformation;
		protected Boolean modifiedEquityDelivery;
		protected ReferenceInformation.ReferenceInformationBuilder referenceInformation;
		protected Boolean substitution;
	
		public GeneralTermsBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends GeneralTerms> metaData() {
			return metaData;
		} 
	
		
		public List<FieldWithMetaStringBuilder> getAdditionalTerm() {
			return additionalTerm;
		}
		
		public FieldWithMetaStringBuilder getOrCreateAdditionalTerm(int index) {
			if (additionalTerm==null) {
				this.additionalTerm = new ArrayList<>();
			}
			return getIndex(additionalTerm, index, ()->new FieldWithMetaStringBuilder());
		}
		
		
		public BasketReferenceInformation.BasketReferenceInformationBuilder getBasketReferenceInformation() {
			return basketReferenceInformation;
		}
		
		public BasketReferenceInformation.BasketReferenceInformationBuilder getOrCreateBasketReferenceInformation() {
			if (basketReferenceInformation!=null) {
				return basketReferenceInformation;
			}
			else return basketReferenceInformation = new BasketReferenceInformation.BasketReferenceInformationBuilder();
		}
		
		
		public BuyerSeller.BuyerSellerBuilder getBuyerSeller() {
			return buyerSeller;
		}
		
		public BuyerSeller.BuyerSellerBuilder getOrCreateBuyerSeller() {
			if (buyerSeller!=null) {
				return buyerSeller;
			}
			else return buyerSeller = new BuyerSeller.BuyerSellerBuilder();
		}
		
		
		public IndexReferenceInformation.IndexReferenceInformationBuilder getIndexReferenceInformation() {
			return indexReferenceInformation;
		}
		
		public IndexReferenceInformation.IndexReferenceInformationBuilder getOrCreateIndexReferenceInformation() {
			if (indexReferenceInformation!=null) {
				return indexReferenceInformation;
			}
			else return indexReferenceInformation = new IndexReferenceInformation.IndexReferenceInformationBuilder();
		}
		
		
		public Boolean getModifiedEquityDelivery() {
			return modifiedEquityDelivery;
		}
		
		
		public ReferenceInformation.ReferenceInformationBuilder getReferenceInformation() {
			return referenceInformation;
		}
		
		public ReferenceInformation.ReferenceInformationBuilder getOrCreateReferenceInformation() {
			if (referenceInformation!=null) {
				return referenceInformation;
			}
			else return referenceInformation = new ReferenceInformation.ReferenceInformationBuilder();
		}
		
		
		public Boolean getSubstitution() {
			return substitution;
		}
		
	
		public GeneralTermsBuilder addAdditionalTerm(FieldWithMetaString additionalTerm) {
			if(this.additionalTerm == null){
				this.additionalTerm = new ArrayList<>();
			}
			this.additionalTerm.add(additionalTerm.toBuilder());
			return this;
		}
		public GeneralTermsBuilder addAdditionalTerm(FieldWithMetaString additionalTerm, int _idx) {
			if(this.additionalTerm == null){
				this.additionalTerm = new ArrayList<>();
			}
			getIndex(this.additionalTerm, _idx, () -> additionalTerm.toBuilder());
			this.additionalTerm.set(_idx, additionalTerm.toBuilder());
			return this;
		}
		public GeneralTermsBuilder addAdditionalTerm(List<FieldWithMetaString> additionalTerms) {
			if(this.additionalTerm == null){
				this.additionalTerm = new ArrayList<>();
			}
			for (FieldWithMetaString toAdd : additionalTerms) {
				this.additionalTerm.add(toAdd.toBuilder());
			}
			return this;
		}
		
		public GeneralTermsBuilder clearAdditionalTerm() {
			this.additionalTerm = null;
			return this;
		}
		public GeneralTermsBuilder setBasketReferenceInformation(BasketReferenceInformation basketReferenceInformation) {
			this.basketReferenceInformation = basketReferenceInformation.toBuilder();
			return this;
		}
		public GeneralTermsBuilder setBasketReferenceInformationBuilder(BasketReferenceInformation.BasketReferenceInformationBuilder basketReferenceInformation) {
			this.basketReferenceInformation = basketReferenceInformation;
			return this;
		}
		public GeneralTermsBuilder setBuyerSeller(BuyerSeller buyerSeller) {
			this.buyerSeller = buyerSeller.toBuilder();
			return this;
		}
		public GeneralTermsBuilder setBuyerSellerBuilder(BuyerSeller.BuyerSellerBuilder buyerSeller) {
			this.buyerSeller = buyerSeller;
			return this;
		}
		public GeneralTermsBuilder setIndexReferenceInformation(IndexReferenceInformation indexReferenceInformation) {
			this.indexReferenceInformation = indexReferenceInformation.toBuilder();
			return this;
		}
		public GeneralTermsBuilder setIndexReferenceInformationBuilder(IndexReferenceInformation.IndexReferenceInformationBuilder indexReferenceInformation) {
			this.indexReferenceInformation = indexReferenceInformation;
			return this;
		}
		public GeneralTermsBuilder setModifiedEquityDelivery(Boolean modifiedEquityDelivery) {
			this.modifiedEquityDelivery = modifiedEquityDelivery;
			return this;
		}
		public GeneralTermsBuilder setReferenceInformation(ReferenceInformation referenceInformation) {
			this.referenceInformation = referenceInformation.toBuilder();
			return this;
		}
		public GeneralTermsBuilder setReferenceInformationBuilder(ReferenceInformation.ReferenceInformationBuilder referenceInformation) {
			this.referenceInformation = referenceInformation;
			return this;
		}
		public GeneralTermsBuilder setSubstitution(Boolean substitution) {
			this.substitution = substitution;
			return this;
		}
		
		public GeneralTerms build() {
			return new GeneralTerms(this);
		}
	
		@Override
		public GeneralTermsBuilder prune() {
			if (additionalTerm!=null) additionalTerm = additionalTerm.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (basketReferenceInformation!=null && !basketReferenceInformation.prune().hasData()) basketReferenceInformation = null;
			if (buyerSeller!=null && !buyerSeller.prune().hasData()) buyerSeller = null;
			if (indexReferenceInformation!=null && !indexReferenceInformation.prune().hasData()) indexReferenceInformation = null;
			if (referenceInformation!=null && !referenceInformation.prune().hasData()) referenceInformation = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getAdditionalTerm()!=null && !getAdditionalTerm().isEmpty()) return true;
			if (getBasketReferenceInformation()!=null && getBasketReferenceInformation().hasData()) return true;
			if (getBuyerSeller()!=null && getBuyerSeller().hasData()) return true;
			if (getIndexReferenceInformation()!=null && getIndexReferenceInformation().hasData()) return true;
			if (getModifiedEquityDelivery()!=null) return true;
			if (getReferenceInformation()!=null && getReferenceInformation().hasData()) return true;
			if (getSubstitution()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("modifiedEquityDelivery"), Boolean.class, modifiedEquityDelivery, this);
			processor.processBasic(path.newSubPath("substitution"), Boolean.class, substitution, this);
			
			processRosetta(path.newSubPath("additionalTerm"), processor, FieldWithMetaString.class, additionalTerm);
			processRosetta(path.newSubPath("basketReferenceInformation"), processor, BasketReferenceInformation.class, basketReferenceInformation);
			processRosetta(path.newSubPath("buyerSeller"), processor, BuyerSeller.class, buyerSeller);
			processRosetta(path.newSubPath("indexReferenceInformation"), processor, IndexReferenceInformation.class, indexReferenceInformation);
			processRosetta(path.newSubPath("referenceInformation"), processor, ReferenceInformation.class, referenceInformation);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			GeneralTermsBuilder _that = (GeneralTermsBuilder) o;
		
			if (!ListEquals.listEquals(additionalTerm, _that.additionalTerm)) return false;
			if (basketReferenceInformation != null ? !basketReferenceInformation.equals(_that.basketReferenceInformation) : _that.basketReferenceInformation != null) return false;
			if (buyerSeller != null ? !buyerSeller.equals(_that.buyerSeller) : _that.buyerSeller != null) return false;
			if (indexReferenceInformation != null ? !indexReferenceInformation.equals(_that.indexReferenceInformation) : _that.indexReferenceInformation != null) return false;
			if (modifiedEquityDelivery != null ? !modifiedEquityDelivery.equals(_that.modifiedEquityDelivery) : _that.modifiedEquityDelivery != null) return false;
			if (referenceInformation != null ? !referenceInformation.equals(_that.referenceInformation) : _that.referenceInformation != null) return false;
			if (substitution != null ? !substitution.equals(_that.substitution) : _that.substitution != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (additionalTerm != null ? additionalTerm.hashCode() : 0);
			_result = 31 * _result + (basketReferenceInformation != null ? basketReferenceInformation.hashCode() : 0);
			_result = 31 * _result + (buyerSeller != null ? buyerSeller.hashCode() : 0);
			_result = 31 * _result + (indexReferenceInformation != null ? indexReferenceInformation.hashCode() : 0);
			_result = 31 * _result + (modifiedEquityDelivery != null ? modifiedEquityDelivery.hashCode() : 0);
			_result = 31 * _result + (referenceInformation != null ? referenceInformation.hashCode() : 0);
			_result = 31 * _result + (substitution != null ? substitution.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "GeneralTermsBuilder {" +
				"additionalTerm=" + this.additionalTerm + ", " +
				"basketReferenceInformation=" + this.basketReferenceInformation + ", " +
				"buyerSeller=" + this.buyerSeller + ", " +
				"indexReferenceInformation=" + this.indexReferenceInformation + ", " +
				"modifiedEquityDelivery=" + this.modifiedEquityDelivery + ", " +
				"referenceInformation=" + this.referenceInformation + ", " +
				"substitution=" + this.substitution +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("modifiedEquityDelivery"), Boolean.class, modifiedEquityDelivery, this);
		processor.processBasic(path.newSubPath("substitution"), Boolean.class, substitution, this);
		
		processRosetta(path.newSubPath("additionalTerm"), processor, FieldWithMetaString.class, additionalTerm);
		processRosetta(path.newSubPath("basketReferenceInformation"), processor, BasketReferenceInformation.class, basketReferenceInformation);
		processRosetta(path.newSubPath("buyerSeller"), processor, BuyerSeller.class, buyerSeller);
		processRosetta(path.newSubPath("indexReferenceInformation"), processor, IndexReferenceInformation.class, indexReferenceInformation);
		processRosetta(path.newSubPath("referenceInformation"), processor, ReferenceInformation.class, referenceInformation);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		GeneralTerms _that = (GeneralTerms) o;
	
		if (!ListEquals.listEquals(additionalTerm, _that.additionalTerm)) return false;
		if (basketReferenceInformation != null ? !basketReferenceInformation.equals(_that.basketReferenceInformation) : _that.basketReferenceInformation != null) return false;
		if (buyerSeller != null ? !buyerSeller.equals(_that.buyerSeller) : _that.buyerSeller != null) return false;
		if (indexReferenceInformation != null ? !indexReferenceInformation.equals(_that.indexReferenceInformation) : _that.indexReferenceInformation != null) return false;
		if (modifiedEquityDelivery != null ? !modifiedEquityDelivery.equals(_that.modifiedEquityDelivery) : _that.modifiedEquityDelivery != null) return false;
		if (referenceInformation != null ? !referenceInformation.equals(_that.referenceInformation) : _that.referenceInformation != null) return false;
		if (substitution != null ? !substitution.equals(_that.substitution) : _that.substitution != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (additionalTerm != null ? additionalTerm.hashCode() : 0);
		_result = 31 * _result + (basketReferenceInformation != null ? basketReferenceInformation.hashCode() : 0);
		_result = 31 * _result + (buyerSeller != null ? buyerSeller.hashCode() : 0);
		_result = 31 * _result + (indexReferenceInformation != null ? indexReferenceInformation.hashCode() : 0);
		_result = 31 * _result + (modifiedEquityDelivery != null ? modifiedEquityDelivery.hashCode() : 0);
		_result = 31 * _result + (referenceInformation != null ? referenceInformation.hashCode() : 0);
		_result = 31 * _result + (substitution != null ? substitution.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "GeneralTerms {" +
			"additionalTerm=" + this.additionalTerm + ", " +
			"basketReferenceInformation=" + this.basketReferenceInformation + ", " +
			"buyerSeller=" + this.buyerSeller + ", " +
			"indexReferenceInformation=" + this.indexReferenceInformation + ", " +
			"modifiedEquityDelivery=" + this.modifiedEquityDelivery + ", " +
			"referenceInformation=" + this.referenceInformation + ", " +
			"substitution=" + this.substitution +
		'}';
	}
}
