package org.isda.cdm;

import java.lang.Boolean;
import org.isda.cdm.EligibilityToHoldCollateral;
import org.isda.cdm.metafields.ReferenceWithMetaParty;
import org.isda.cdm.metafields.ReferenceWithMetaParty.ReferenceWithMetaPartyBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.HoldingAndUsingPostedCollateralElectionMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.stream.Collectors;
		import static java.util.Optional.ofNullable;

/**
 * A class to specify the parties&#39; elections related to the holding and using of posted collateral. ISDA 2016 Credit Support Annex for Variation Margin, paragraph 13, (h): Holding and Using Posted Collateral (VM).
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="HoldingAndUsingPostedCollateral", source="FpML_5_10")
public class HoldingAndUsingPostedCollateralElection extends RosettaModelObject {
	private final EligibilityToHoldCollateral eligibilityToHoldCollateral;
	private final ReferenceWithMetaParty party;
	private final Boolean useOfPostedCollateral;
	private static HoldingAndUsingPostedCollateralElectionMeta metaData = new HoldingAndUsingPostedCollateralElectionMeta();
	
	HoldingAndUsingPostedCollateralElection(HoldingAndUsingPostedCollateralElectionBuilder builder) {
		this.eligibilityToHoldCollateral = ofNullable(builder.getEligibilityToHoldCollateral()).map(f->f.build()).orElse(null);
		this.party = ofNullable(builder.getParty()).map(f->f.build()).orElse(null);
		this.useOfPostedCollateral = builder.getUseOfPostedCollateral();
	}
	
	/**
	 * The specification of the conditions under which a party and its custodian(s) are entitled to hold posted collateral.
	 */
	@RosettaSynonym(value="eligibilityToHoldCollateral", source="FpML_5_10")
	public final EligibilityToHoldCollateral getEligibilityToHoldCollateral() {
		return eligibilityToHoldCollateral;
	}
	
	/**
	 * The elective party.
	 */
	@RosettaSynonym(value="partyReference", source="FpML_5_10")
	public final ReferenceWithMetaParty getParty() {
		return party;
	}
	
	/**
	 * Specifies whether the party to the agreement has the right to rehypothecate the collateral held (True), i.e. whether the condition specified in Paragraph 6, (c) of the ISDA 2016 Credit Support Annex for Variation Margin apply. ISDA 2016 Credit Support Annex for Variation Margin, paragraph 13, (h)(ii): Use of Posted Collateral (VM).
	 */
	@RosettaSynonym(value="useOfPostedCollateral", source="FpML_5_10")
	public final Boolean getUseOfPostedCollateral() {
		return useOfPostedCollateral;
	}
	
	
	@Override
	public RosettaMetaData<? extends HoldingAndUsingPostedCollateralElection> metaData() {
		return metaData;
	} 
	
	public HoldingAndUsingPostedCollateralElectionBuilder toBuilder() {
		HoldingAndUsingPostedCollateralElectionBuilder builder = new HoldingAndUsingPostedCollateralElectionBuilder();
		ofNullable(getEligibilityToHoldCollateral()).ifPresent(builder::setEligibilityToHoldCollateral);
		ofNullable(getParty()).ifPresent(builder::setParty);
		ofNullable(getUseOfPostedCollateral()).ifPresent(builder::setUseOfPostedCollateral);
		return builder;
	}

	public static HoldingAndUsingPostedCollateralElectionBuilder builder() {
		return new HoldingAndUsingPostedCollateralElectionBuilder();
	}

	public static  class HoldingAndUsingPostedCollateralElectionBuilder extends RosettaModelObjectBuilder {
	
		protected EligibilityToHoldCollateral.EligibilityToHoldCollateralBuilder eligibilityToHoldCollateral;
		protected ReferenceWithMetaPartyBuilder party;
		protected Boolean useOfPostedCollateral;
	
		public HoldingAndUsingPostedCollateralElectionBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends HoldingAndUsingPostedCollateralElection> metaData() {
			return metaData;
		} 
	
		
		public EligibilityToHoldCollateral.EligibilityToHoldCollateralBuilder getEligibilityToHoldCollateral() {
			return eligibilityToHoldCollateral;
		}
		
		public EligibilityToHoldCollateral.EligibilityToHoldCollateralBuilder getOrCreateEligibilityToHoldCollateral() {
			if (eligibilityToHoldCollateral!=null) {
				return eligibilityToHoldCollateral;
			}
			else return eligibilityToHoldCollateral = new EligibilityToHoldCollateral.EligibilityToHoldCollateralBuilder();
		}
		
		
		public ReferenceWithMetaPartyBuilder getParty() {
			return party;
		}
		
		public ReferenceWithMetaPartyBuilder getOrCreateParty() {
			if (party!=null) {
				return party;
			}
			else return party = new ReferenceWithMetaPartyBuilder();
		}
		
		
		public Boolean getUseOfPostedCollateral() {
			return useOfPostedCollateral;
		}
		
	
		public HoldingAndUsingPostedCollateralElectionBuilder setEligibilityToHoldCollateral(EligibilityToHoldCollateral eligibilityToHoldCollateral) {
			this.eligibilityToHoldCollateral = eligibilityToHoldCollateral.toBuilder();
			return this;
		}
		public HoldingAndUsingPostedCollateralElectionBuilder setEligibilityToHoldCollateralBuilder(EligibilityToHoldCollateral.EligibilityToHoldCollateralBuilder eligibilityToHoldCollateral) {
			this.eligibilityToHoldCollateral = eligibilityToHoldCollateral;
			return this;
		}
		public HoldingAndUsingPostedCollateralElectionBuilder setParty(ReferenceWithMetaParty party) {
			this.party = party.toBuilder();
			return this;
		}
		public HoldingAndUsingPostedCollateralElectionBuilder setPartyBuilder(ReferenceWithMetaPartyBuilder party) {
			this.party = party;
			return this;
		}
		public HoldingAndUsingPostedCollateralElectionBuilder setPartyRef(Party.PartyBuilder party) {
			return setParty(ReferenceWithMetaParty.builder().setValueBuilder(party).build());
		}
		public HoldingAndUsingPostedCollateralElectionBuilder setPartyRef(Party party) {
			return setPartyRef(Party.builder());
		}
		public HoldingAndUsingPostedCollateralElectionBuilder setUseOfPostedCollateral(Boolean useOfPostedCollateral) {
			this.useOfPostedCollateral = useOfPostedCollateral;
			return this;
		}
		
		public HoldingAndUsingPostedCollateralElection build() {
			return new HoldingAndUsingPostedCollateralElection(this);
		}
	
		@Override
		public HoldingAndUsingPostedCollateralElectionBuilder prune() {
			if (eligibilityToHoldCollateral!=null && !eligibilityToHoldCollateral.prune().hasData()) eligibilityToHoldCollateral = null;
			if (party!=null && !party.prune().hasData()) party = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getEligibilityToHoldCollateral()!=null && getEligibilityToHoldCollateral().hasData()) return true;
			if (getParty()!=null && getParty().hasData()) return true;
			if (getUseOfPostedCollateral()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("useOfPostedCollateral"), Boolean.class, useOfPostedCollateral, this);
			
			processRosetta(path.newSubPath("eligibilityToHoldCollateral"), processor, EligibilityToHoldCollateral.class, eligibilityToHoldCollateral);
			processRosetta(path.newSubPath("party"), processor, ReferenceWithMetaParty.class, party);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			HoldingAndUsingPostedCollateralElectionBuilder _that = (HoldingAndUsingPostedCollateralElectionBuilder) o;
		
			if (eligibilityToHoldCollateral != null ? !eligibilityToHoldCollateral.equals(_that.eligibilityToHoldCollateral) : _that.eligibilityToHoldCollateral != null) return false;
			if (party != null ? !party.equals(_that.party) : _that.party != null) return false;
			if (useOfPostedCollateral != null ? !useOfPostedCollateral.equals(_that.useOfPostedCollateral) : _that.useOfPostedCollateral != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (eligibilityToHoldCollateral != null ? eligibilityToHoldCollateral.hashCode() : 0);
			_result = 31 * _result + (party != null ? party.hashCode() : 0);
			_result = 31 * _result + (useOfPostedCollateral != null ? useOfPostedCollateral.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "HoldingAndUsingPostedCollateralElectionBuilder {" +
				"eligibilityToHoldCollateral=" + this.eligibilityToHoldCollateral + ", " +
				"party=" + this.party + ", " +
				"useOfPostedCollateral=" + this.useOfPostedCollateral +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("useOfPostedCollateral"), Boolean.class, useOfPostedCollateral, this);
		
		processRosetta(path.newSubPath("eligibilityToHoldCollateral"), processor, EligibilityToHoldCollateral.class, eligibilityToHoldCollateral);
		processRosetta(path.newSubPath("party"), processor, ReferenceWithMetaParty.class, party);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		HoldingAndUsingPostedCollateralElection _that = (HoldingAndUsingPostedCollateralElection) o;
	
		if (eligibilityToHoldCollateral != null ? !eligibilityToHoldCollateral.equals(_that.eligibilityToHoldCollateral) : _that.eligibilityToHoldCollateral != null) return false;
		if (party != null ? !party.equals(_that.party) : _that.party != null) return false;
		if (useOfPostedCollateral != null ? !useOfPostedCollateral.equals(_that.useOfPostedCollateral) : _that.useOfPostedCollateral != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (eligibilityToHoldCollateral != null ? eligibilityToHoldCollateral.hashCode() : 0);
		_result = 31 * _result + (party != null ? party.hashCode() : 0);
		_result = 31 * _result + (useOfPostedCollateral != null ? useOfPostedCollateral.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "HoldingAndUsingPostedCollateralElection {" +
			"eligibilityToHoldCollateral=" + this.eligibilityToHoldCollateral + ", " +
			"party=" + this.party + ", " +
			"useOfPostedCollateral=" + this.useOfPostedCollateral +
		'}';
	}
}
