package org.isda.cdm;

import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.AssignedIdentifier;
import org.isda.cdm.metafields.FieldWithMetaString;
import org.isda.cdm.metafields.FieldWithMetaString.FieldWithMetaStringBuilder;
import org.isda.cdm.metafields.MetaFields;
import org.isda.cdm.metafields.ReferenceWithMetaParty;
import org.isda.cdm.metafields.ReferenceWithMetaParty.ReferenceWithMetaPartyBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.IdentifierMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.stream.Collectors;
		import static java.util.Optional.ofNullable;
		import java.util.List;
		import java.util.ArrayList;
		import java.util.Objects;

/**
 * A class to specify a generic identifier, applicable to CDM artefacts such as executions, contracts, lifecycle events and legal documents. An issuer can be associated with the actual identifier value as a way to properly qualify it.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class Identifier extends RosettaModelObject implements GlobalKey {
	private final List<AssignedIdentifier> assignedIdentifier;
	private final FieldWithMetaString issuer;
	private final ReferenceWithMetaParty issuerReference;
	private final MetaFields meta;
	private static IdentifierMeta metaData = new IdentifierMeta();
	
	Identifier(IdentifierBuilder builder) {
		this.assignedIdentifier = ofNullable(builder.getAssignedIdentifier()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.issuer = ofNullable(builder.getIssuer()).map(f->f.build()).orElse(null);
		this.issuerReference = ofNullable(builder.getIssuerReference()).map(f->f.build()).orElse(null);
		this.meta = ofNullable(builder.getMeta()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The identifier value. This level of indirection between the issuer and the identifier and its version provides the ability to associate multiple identifiers to one issuer, consistently with the FpML PartyTradeIdentifier.
	 */
	public final List<AssignedIdentifier> getAssignedIdentifier() {
		return assignedIdentifier;
	}
	
	/**
	 * The identifier issuer, when specified explicitly alongside the identifier value (instead of being specified by reference to a party).
	 */
	@RosettaSynonym(value="issuer", source="FpML_5_10")
	@RosettaSynonym(value="issuer", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="USIIssuer", source="DTCC_11_0")
	@RosettaSynonym(value="USIIssuer", source="DTCC_9_0")
	@RosettaSynonym(value="UTIIssuer", source="DTCC_11_0")
	@RosettaSynonym(value="UTIIssuer", source="DTCC_9_0")
	@RosettaSynonym(value="issuer", source="CME_ClearedConfirm_1_17")
	public final FieldWithMetaString getIssuer() {
		return issuer;
	}
	
	/**
	 * The identifier issuer, when specified by reference to a party specified as part of the transaction.
	 */
	@RosettaSynonym(value="issuer", source="FpML_5_10")
	@RosettaSynonym(value="issuer", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="partyReference", source="FpML_5_10")
	@RosettaSynonym(value="partyReference", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="partyReference", source="DTCC_11_0", path="partyTradeIdentifier", maps=2)
	@RosettaSynonym(value="partyReference", source="DTCC_9_0", path="partyTradeIdentifier", maps=2)
	@RosettaSynonym(value="partyReference", source="CME_ClearedConfirm_1_17")
	public final ReferenceWithMetaParty getIssuerReference() {
		return issuerReference;
	}
	
	public final MetaFields getMeta() {
		return meta;
	}
	
	
	@Override
	public RosettaMetaData<? extends Identifier> metaData() {
		return metaData;
	} 
	
	public IdentifierBuilder toBuilder() {
		IdentifierBuilder builder = new IdentifierBuilder();
		ofNullable(getAssignedIdentifier()).ifPresent(assignedIdentifier -> assignedIdentifier.forEach(builder::addAssignedIdentifier));
		ofNullable(getIssuer()).ifPresent(builder::setIssuer);
		ofNullable(getIssuerReference()).ifPresent(builder::setIssuerReference);
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		return builder;
	}

	public static IdentifierBuilder builder() {
		return new IdentifierBuilder();
	}

	public static  class IdentifierBuilder extends RosettaModelObjectBuilder implements GlobalKeyBuilder<IdentifierBuilder> {
	
		protected List<AssignedIdentifier.AssignedIdentifierBuilder> assignedIdentifier;
		protected FieldWithMetaStringBuilder issuer;
		protected ReferenceWithMetaPartyBuilder issuerReference;
		protected MetaFields.MetaFieldsBuilder meta;
	
		public IdentifierBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends Identifier> metaData() {
			return metaData;
		} 
	
		
		public List<AssignedIdentifier.AssignedIdentifierBuilder> getAssignedIdentifier() {
			return assignedIdentifier;
		}
		
		public AssignedIdentifier.AssignedIdentifierBuilder getOrCreateAssignedIdentifier(int index) {
			if (assignedIdentifier==null) {
				this.assignedIdentifier = new ArrayList<>();
			}
			return getIndex(assignedIdentifier, index, ()->new AssignedIdentifier.AssignedIdentifierBuilder());
		}
		
		
		public FieldWithMetaStringBuilder getIssuer() {
			return issuer;
		}
		
		public FieldWithMetaStringBuilder getOrCreateIssuer() {
			if (issuer!=null) {
				return issuer;
			}
			else return issuer = new FieldWithMetaStringBuilder();
		}
		
		
		public ReferenceWithMetaPartyBuilder getIssuerReference() {
			return issuerReference;
		}
		
		public ReferenceWithMetaPartyBuilder getOrCreateIssuerReference() {
			if (issuerReference!=null) {
				return issuerReference;
			}
			else return issuerReference = new ReferenceWithMetaPartyBuilder();
		}
		
		
		public MetaFields.MetaFieldsBuilder getMeta() {
			return meta;
		}
		
		public MetaFields.MetaFieldsBuilder getOrCreateMeta() {
			if (meta!=null) {
				return meta;
			}
			else return meta = new MetaFields.MetaFieldsBuilder();
		}
		
	
		public IdentifierBuilder addAssignedIdentifier(AssignedIdentifier assignedIdentifier) {
			if(this.assignedIdentifier == null){
				this.assignedIdentifier = new ArrayList<>();
			}
			this.assignedIdentifier.add(assignedIdentifier.toBuilder());
			return this;
		}
		public IdentifierBuilder addAssignedIdentifier(AssignedIdentifier assignedIdentifier, int _idx) {
			if(this.assignedIdentifier == null){
				this.assignedIdentifier = new ArrayList<>();
			}
			getIndex(this.assignedIdentifier, _idx, () -> assignedIdentifier.toBuilder());
			this.assignedIdentifier.set(_idx, assignedIdentifier.toBuilder());
			return this;
		}
		public IdentifierBuilder addAssignedIdentifier(List<AssignedIdentifier> assignedIdentifiers) {
			if(this.assignedIdentifier == null){
				this.assignedIdentifier = new ArrayList<>();
			}
			for (AssignedIdentifier toAdd : assignedIdentifiers) {
				this.assignedIdentifier.add(toAdd.toBuilder());
			}
			return this;
		}
		public IdentifierBuilder addAssignedIdentifierBuilder(AssignedIdentifier.AssignedIdentifierBuilder assignedIdentifier) {
			if(this.assignedIdentifier == null){
				this.assignedIdentifier = new ArrayList<>();
			}
			this.assignedIdentifier.add(assignedIdentifier);
			return this;
		}
		
		public IdentifierBuilder clearAssignedIdentifier() {
			this.assignedIdentifier = null;
			return this;
		}
		public IdentifierBuilder setIssuer(FieldWithMetaString issuer) {
			this.issuer = issuer.toBuilder();
			return this;
		}
		public IdentifierBuilder setIssuerRef(String issuer) {
			return setIssuer(FieldWithMetaString.builder().setValue(issuer).build());
		}
		public IdentifierBuilder setIssuerReference(ReferenceWithMetaParty issuerReference) {
			this.issuerReference = issuerReference.toBuilder();
			return this;
		}
		public IdentifierBuilder setIssuerReferenceBuilder(ReferenceWithMetaPartyBuilder issuerReference) {
			this.issuerReference = issuerReference;
			return this;
		}
		public IdentifierBuilder setIssuerReferenceRef(Party.PartyBuilder issuerReference) {
			return setIssuerReference(ReferenceWithMetaParty.builder().setValueBuilder(issuerReference).build());
		}
		public IdentifierBuilder setIssuerReferenceRef(Party issuerReference) {
			return setIssuerReferenceRef(Party.builder());
		}
		public IdentifierBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		public IdentifierBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		
		public Identifier build() {
			return new Identifier(this);
		}
	
		@Override
		public IdentifierBuilder prune() {
			if (assignedIdentifier!=null) assignedIdentifier = assignedIdentifier.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (issuer!=null && !issuer.prune().hasData()) issuer = null;
			if (issuerReference!=null && !issuerReference.prune().hasData()) issuerReference = null;
			if (meta!=null && !meta.prune().hasData()) meta = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getAssignedIdentifier()!=null && getAssignedIdentifier().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getIssuer()!=null) return true;
			if (getIssuerReference()!=null && getIssuerReference().hasData()) return true;
			if (getMeta()!=null && getMeta().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("assignedIdentifier"), processor, AssignedIdentifier.class, assignedIdentifier);
			processRosetta(path.newSubPath("issuer"), processor, FieldWithMetaString.class, issuer);
			processRosetta(path.newSubPath("issuerReference"), processor, ReferenceWithMetaParty.class, issuerReference);
			processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			IdentifierBuilder _that = (IdentifierBuilder) o;
		
			if (!ListEquals.listEquals(assignedIdentifier, _that.assignedIdentifier)) return false;
			if (issuer != null ? !issuer.equals(_that.issuer) : _that.issuer != null) return false;
			if (issuerReference != null ? !issuerReference.equals(_that.issuerReference) : _that.issuerReference != null) return false;
			if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (assignedIdentifier != null ? assignedIdentifier.hashCode() : 0);
			_result = 31 * _result + (issuer != null ? issuer.hashCode() : 0);
			_result = 31 * _result + (issuerReference != null ? issuerReference.hashCode() : 0);
			_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "IdentifierBuilder {" +
				"assignedIdentifier=" + this.assignedIdentifier + ", " +
				"issuer=" + this.issuer + ", " +
				"issuerReference=" + this.issuerReference + ", " +
				"meta=" + this.meta +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("assignedIdentifier"), processor, AssignedIdentifier.class, assignedIdentifier);
		processRosetta(path.newSubPath("issuer"), processor, FieldWithMetaString.class, issuer);
		processRosetta(path.newSubPath("issuerReference"), processor, ReferenceWithMetaParty.class, issuerReference);
		processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		Identifier _that = (Identifier) o;
	
		if (!ListEquals.listEquals(assignedIdentifier, _that.assignedIdentifier)) return false;
		if (issuer != null ? !issuer.equals(_that.issuer) : _that.issuer != null) return false;
		if (issuerReference != null ? !issuerReference.equals(_that.issuerReference) : _that.issuerReference != null) return false;
		if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (assignedIdentifier != null ? assignedIdentifier.hashCode() : 0);
		_result = 31 * _result + (issuer != null ? issuer.hashCode() : 0);
		_result = 31 * _result + (issuerReference != null ? issuerReference.hashCode() : 0);
		_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "Identifier {" +
			"assignedIdentifier=" + this.assignedIdentifier + ", " +
			"issuer=" + this.issuer + ", " +
			"issuerReference=" + this.issuerReference + ", " +
			"meta=" + this.meta +
		'}';
	}
}
