package org.isda.cdm;

import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.ContractState;
import org.isda.cdm.PostInceptionState;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.InceptionMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;

/**
 * The primitive event for the inception of a new contract between parties. It is expected that this primitive will be adjusted or deprecated once the CDM scope is extended to the pre-execution space.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class Inception extends RosettaModelObject {
	private final PostInceptionState after;
	private final List<ContractState> before;
	private static InceptionMeta metaData = new InceptionMeta();
	
	Inception(InceptionBuilder builder) {
		this.after = ofNullable(builder.getAfter()).map(f->f.build()).orElse(null);
		this.before = ofNullable(builder.getBefore()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
	}
	
	/**
	 * The after state corresponds to the new contract between the parties.
	 */
	@RosettaSynonym(value="after", source="Rosetta_Workbench")
	@RosettaSynonym(value="OTC_RM", source="DTCC_11_0", path="Header")
	@RosettaSynonym(value="OTC_Matching", source="DTCC_11_0", path="Body")
	@RosettaSynonym(value="OTC_Matching", source="DTCC_9_0", path="Body")
	@RosettaSynonym(value="clearingConfirmed", source="CME_ClearedConfirm_1_17")
	public final PostInceptionState getAfter() {
		return after;
	}
	
	/**
	 * The (0..0) cardinality reflects the fact that there is no contract in the before state of an inception primitive. As noted in the definition associated with the class, this is expected to change once the CDM scope is extended to the pre-execution space.
	 */
	@RosettaSynonym(value="before", source="Rosetta_Workbench")
	public final List<ContractState> getBefore() {
		return before;
	}
	
	
	@Override
	public RosettaMetaData<? extends Inception> metaData() {
		return metaData;
	} 
	
	public InceptionBuilder toBuilder() {
		InceptionBuilder builder = new InceptionBuilder();
		ofNullable(getAfter()).ifPresent(builder::setAfter);
		ofNullable(getBefore()).ifPresent(before -> before.forEach(builder::addBefore));
		return builder;
	}

	public static InceptionBuilder builder() {
		return new InceptionBuilder();
	}

	public static  class InceptionBuilder extends RosettaModelObjectBuilder {
	
		protected PostInceptionState.PostInceptionStateBuilder after;
		protected List<ContractState.ContractStateBuilder> before;
	
		public InceptionBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends Inception> metaData() {
			return metaData;
		} 
	
		
		public PostInceptionState.PostInceptionStateBuilder getAfter() {
			return after;
		}
		
		public PostInceptionState.PostInceptionStateBuilder getOrCreateAfter() {
			if (after!=null) {
				return after;
			}
			else return after = new PostInceptionState.PostInceptionStateBuilder();
		}
		
		
		public List<ContractState.ContractStateBuilder> getBefore() {
			return before;
		}
		
		public ContractState.ContractStateBuilder getOrCreateBefore(int index) {
			if (before==null) {
				this.before = new ArrayList<>();
			}
			return getIndex(before, index, ()->new ContractState.ContractStateBuilder());
		}
		
	
		public InceptionBuilder setAfter(PostInceptionState after) {
			this.after = after.toBuilder();
			return this;
		}
		public InceptionBuilder setAfterBuilder(PostInceptionState.PostInceptionStateBuilder after) {
			this.after = after;
			return this;
		}
		public InceptionBuilder addBefore(ContractState before) {
			if(this.before == null){
				this.before = new ArrayList<>();
			}
			this.before.add(before.toBuilder());
			return this;
		}
		public InceptionBuilder addBefore(ContractState before, int _idx) {
			if(this.before == null){
				this.before = new ArrayList<>();
			}
			getIndex(this.before, _idx, () -> before.toBuilder());
			this.before.set(_idx, before.toBuilder());
			return this;
		}
		public InceptionBuilder addBefore(List<ContractState> befores) {
			if(this.before == null){
				this.before = new ArrayList<>();
			}
			for (ContractState toAdd : befores) {
				this.before.add(toAdd.toBuilder());
			}
			return this;
		}
		public InceptionBuilder addBeforeBuilder(ContractState.ContractStateBuilder before) {
			if(this.before == null){
				this.before = new ArrayList<>();
			}
			this.before.add(before);
			return this;
		}
		
		public InceptionBuilder clearBefore() {
			this.before = null;
			return this;
		}
		
		public Inception build() {
			return new Inception(this);
		}
	
		@Override
		public InceptionBuilder prune() {
			if (after!=null && !after.prune().hasData()) after = null;
			if (before!=null) before = before.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getAfter()!=null && getAfter().hasData()) return true;
			if (getBefore()!=null && getBefore().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("after"), processor, PostInceptionState.class, after);
			processRosetta(path.newSubPath("before"), processor, ContractState.class, before);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			InceptionBuilder _that = (InceptionBuilder) o;
		
			if (after != null ? !after.equals(_that.after) : _that.after != null) return false;
			if (!ListEquals.listEquals(before, _that.before)) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (after != null ? after.hashCode() : 0);
			_result = 31 * _result + (before != null ? before.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "InceptionBuilder {" +
				"after=" + this.after + ", " +
				"before=" + this.before +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("after"), processor, PostInceptionState.class, after);
		processRosetta(path.newSubPath("before"), processor, ContractState.class, before);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		Inception _that = (Inception) o;
	
		if (after != null ? !after.equals(_that.after) : _that.after != null) return false;
		if (!ListEquals.listEquals(before, _that.before)) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (after != null ? after.hashCode() : 0);
		_result = 31 * _result + (before != null ? before.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "Inception {" +
			"after=" + this.after + ", " +
			"before=" + this.before +
		'}';
	}
}
