package org.isda.cdm;

import org.isda.cdm.IndexEventConsequenceEnum;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.IndexAdjustmentEventsMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * Defines the specification of the consequences of Index Events as defined by the 2002 ISDA Equity Derivatives Definitions.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="IndexAdjustmentEvents", source="FpML_5_10")
public class IndexAdjustmentEvents extends RosettaModelObject {
	private final IndexEventConsequenceEnum indexCancellation;
	private final IndexEventConsequenceEnum indexDisruption;
	private final IndexEventConsequenceEnum indexModification;
	private static IndexAdjustmentEventsMeta metaData = new IndexAdjustmentEventsMeta();
	
	IndexAdjustmentEvents(IndexAdjustmentEventsBuilder builder) {
		this.indexCancellation = builder.getIndexCancellation();
		this.indexDisruption = builder.getIndexDisruption();
		this.indexModification = builder.getIndexModification();
	}
	
	/**
	 * Consequence of index cancellation.
	 */
	@RosettaSynonym(value="indexCancellation", source="FpML_5_10")
	public final IndexEventConsequenceEnum getIndexCancellation() {
		return indexCancellation;
	}
	
	/**
	 * Consequence of index disruption.
	 */
	@RosettaSynonym(value="indexDisruption", source="FpML_5_10")
	public final IndexEventConsequenceEnum getIndexDisruption() {
		return indexDisruption;
	}
	
	/**
	 * Consequence of index modification.
	 */
	@RosettaSynonym(value="indexModification", source="FpML_5_10")
	public final IndexEventConsequenceEnum getIndexModification() {
		return indexModification;
	}
	
	
	@Override
	public RosettaMetaData<? extends IndexAdjustmentEvents> metaData() {
		return metaData;
	} 
	
	public IndexAdjustmentEventsBuilder toBuilder() {
		IndexAdjustmentEventsBuilder builder = new IndexAdjustmentEventsBuilder();
		ofNullable(getIndexCancellation()).ifPresent(builder::setIndexCancellation);
		ofNullable(getIndexDisruption()).ifPresent(builder::setIndexDisruption);
		ofNullable(getIndexModification()).ifPresent(builder::setIndexModification);
		return builder;
	}

	public static IndexAdjustmentEventsBuilder builder() {
		return new IndexAdjustmentEventsBuilder();
	}

	public static  class IndexAdjustmentEventsBuilder extends RosettaModelObjectBuilder {
	
		protected IndexEventConsequenceEnum indexCancellation;
		protected IndexEventConsequenceEnum indexDisruption;
		protected IndexEventConsequenceEnum indexModification;
	
		public IndexAdjustmentEventsBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends IndexAdjustmentEvents> metaData() {
			return metaData;
		} 
	
		
		public IndexEventConsequenceEnum getIndexCancellation() {
			return indexCancellation;
		}
		
		
		public IndexEventConsequenceEnum getIndexDisruption() {
			return indexDisruption;
		}
		
		
		public IndexEventConsequenceEnum getIndexModification() {
			return indexModification;
		}
		
	
		public IndexAdjustmentEventsBuilder setIndexCancellation(IndexEventConsequenceEnum indexCancellation) {
			this.indexCancellation = indexCancellation;
			return this;
		}
		public IndexAdjustmentEventsBuilder setIndexDisruption(IndexEventConsequenceEnum indexDisruption) {
			this.indexDisruption = indexDisruption;
			return this;
		}
		public IndexAdjustmentEventsBuilder setIndexModification(IndexEventConsequenceEnum indexModification) {
			this.indexModification = indexModification;
			return this;
		}
		
		public IndexAdjustmentEvents build() {
			return new IndexAdjustmentEvents(this);
		}
	
		@Override
		public IndexAdjustmentEventsBuilder prune() {
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getIndexCancellation()!=null) return true;
			if (getIndexDisruption()!=null) return true;
			if (getIndexModification()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("indexCancellation"), IndexEventConsequenceEnum.class, indexCancellation, this);
			processor.processBasic(path.newSubPath("indexDisruption"), IndexEventConsequenceEnum.class, indexDisruption, this);
			processor.processBasic(path.newSubPath("indexModification"), IndexEventConsequenceEnum.class, indexModification, this);
			
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			IndexAdjustmentEventsBuilder _that = (IndexAdjustmentEventsBuilder) o;
		
			if (indexCancellation != null ? !indexCancellation.equals(_that.indexCancellation) : _that.indexCancellation != null) return false;
			if (indexDisruption != null ? !indexDisruption.equals(_that.indexDisruption) : _that.indexDisruption != null) return false;
			if (indexModification != null ? !indexModification.equals(_that.indexModification) : _that.indexModification != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (indexCancellation != null ? indexCancellation.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (indexDisruption != null ? indexDisruption.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (indexModification != null ? indexModification.getClass().getName().hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "IndexAdjustmentEventsBuilder {" +
				"indexCancellation=" + this.indexCancellation + ", " +
				"indexDisruption=" + this.indexDisruption + ", " +
				"indexModification=" + this.indexModification +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("indexCancellation"), IndexEventConsequenceEnum.class, indexCancellation, this);
		processor.processBasic(path.newSubPath("indexDisruption"), IndexEventConsequenceEnum.class, indexDisruption, this);
		processor.processBasic(path.newSubPath("indexModification"), IndexEventConsequenceEnum.class, indexModification, this);
		
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		IndexAdjustmentEvents _that = (IndexAdjustmentEvents) o;
	
		if (indexCancellation != null ? !indexCancellation.equals(_that.indexCancellation) : _that.indexCancellation != null) return false;
		if (indexDisruption != null ? !indexDisruption.equals(_that.indexDisruption) : _that.indexDisruption != null) return false;
		if (indexModification != null ? !indexModification.equals(_that.indexModification) : _that.indexModification != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (indexCancellation != null ? indexCancellation.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (indexDisruption != null ? indexDisruption.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (indexModification != null ? indexModification.getClass().getName().hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "IndexAdjustmentEvents {" +
			"indexCancellation=" + this.indexCancellation + ", " +
			"indexDisruption=" + this.indexDisruption + ", " +
			"indexModification=" + this.indexModification +
		'}';
	}
}
