package org.isda.cdm;

import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.meta.RosettaMetaData;
import java.lang.String;
import org.isda.cdm.meta.InformationSourceMeta;
import org.isda.cdm.metafields.FieldWithMetaInformationProviderEnum;
import org.isda.cdm.metafields.FieldWithMetaInformationProviderEnum.FieldWithMetaInformationProviderEnumBuilder;
import org.isda.cdm.metafields.FieldWithMetaString;
import org.isda.cdm.metafields.FieldWithMetaString.FieldWithMetaStringBuilder;

import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.BuilderProcessor;
import com.rosetta.model.lib.process.Processor;

import static java.util.Optional.ofNullable;

/**
 * A class defining the source for a piece of information (e.g. a rate fix or an FX fixing). The attribute names have been adjusted from FpML to address the fact that the information is not limited to rates.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="InformationSource", source="FpML_5_10")
public class InformationSource extends RosettaModelObject {
	private final FieldWithMetaString sourcePage;
	private final String sourcePageHeading;
	private final FieldWithMetaInformationProviderEnum sourceProvider;
	private static InformationSourceMeta metaData = new InformationSourceMeta();
	
	InformationSource(InformationSourceBuilder builder) {
		this.sourcePage = ofNullable(builder.getSourcePage()).map(f->f.build()).orElse(null);
		this.sourcePageHeading = builder.getSourcePageHeading();
		this.sourceProvider = ofNullable(builder.getSourceProvider()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * A specific page for the source for obtaining a market data point. In FpML, this is specified as a scheme, rateSourcePageScheme, for which no coding Scheme or URI is specified.
	 */
	@RosettaSynonym(value="rateSourcePage", source="FpML_5_10")
	@RosettaSynonym(value="rateSourcePage", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="rateSourcePage", source="DTCC_11_0")
	@RosettaSynonym(value="rateSourcePage", source="DTCC_9_0")
	@RosettaSynonym(value="rateSourcePage", source="CME_ClearedConfirm_1_17")
	public final FieldWithMetaString getSourcePage() {
		return sourcePage;
	}
	
	/**
	 * The heading for the source on a given source page.
	 */
	@RosettaSynonym(value="rateSourcePageHeading", source="FpML_5_10")
	@RosettaSynonym(value="rateSourcePageHeading", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="rateSourcePageHeading", source="DTCC_11_0")
	@RosettaSynonym(value="rateSourcePageHeading", source="DTCC_9_0")
	@RosettaSynonym(value="rateSourcePageHeading", source="CME_ClearedConfirm_1_17")
	public final String getSourcePageHeading() {
		return sourcePageHeading;
	}
	
	/**
	 * An information source for obtaining a market data point. For example Bloomberg, Reuters, Telerate, etc.
	 */
	@RosettaSynonym(value="rateSource", source="FpML_5_10")
	@RosettaSynonym(value="rateSource", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="rateSource", source="DTCC_11_0")
	@RosettaSynonym(value="rateSource", source="DTCC_9_0")
	@RosettaSynonym(value="rateSource", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="RateSource", source="FIX_5_0_SP2")
	public final FieldWithMetaInformationProviderEnum getSourceProvider() {
		return sourceProvider;
	}
	
	
	@Override
	public RosettaMetaData<? extends InformationSource> metaData() {
		return metaData;
	} 
	
	public InformationSourceBuilder toBuilder() {
		InformationSourceBuilder builder = new InformationSourceBuilder();
		ofNullable(getSourcePage()).ifPresent(builder::setSourcePage);
		ofNullable(getSourcePageHeading()).ifPresent(builder::setSourcePageHeading);
		ofNullable(getSourceProvider()).ifPresent(builder::setSourceProvider);
		return builder;
	}

	public static InformationSourceBuilder builder() {
		return new InformationSourceBuilder();
	}

	public static class InformationSourceBuilder extends RosettaModelObjectBuilder{
	
		protected FieldWithMetaStringBuilder sourcePage;
		protected String sourcePageHeading;
		protected FieldWithMetaInformationProviderEnumBuilder sourceProvider;
	
		public InformationSourceBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends InformationSource> metaData() {
			return metaData;
		} 
	
		
		public FieldWithMetaStringBuilder getSourcePage() {
			return sourcePage;
		}
		
		public FieldWithMetaStringBuilder getOrCreateSourcePage() {
			if (sourcePage!=null) {
				return sourcePage;
			}
			else return sourcePage = new FieldWithMetaStringBuilder();
		}
		
		
		public String getSourcePageHeading() {
			return sourcePageHeading;
		}
		
		
		public FieldWithMetaInformationProviderEnumBuilder getSourceProvider() {
			return sourceProvider;
		}
		
		public FieldWithMetaInformationProviderEnumBuilder getOrCreateSourceProvider() {
			if (sourceProvider!=null) {
				return sourceProvider;
			}
			else return sourceProvider = new FieldWithMetaInformationProviderEnumBuilder();
		}
		
	
		public InformationSourceBuilder setSourcePage(FieldWithMetaString sourcePage) {
			this.sourcePage = sourcePage.toBuilder();
			return this;
		}
		public InformationSourceBuilder setSourcePageRef(String sourcePage) {
			return setSourcePage(FieldWithMetaString.builder().setValue(sourcePage).build());
		}
		public InformationSourceBuilder setSourcePageHeading(String sourcePageHeading) {
			this.sourcePageHeading = sourcePageHeading;
			return this;
		}
		public InformationSourceBuilder setSourceProvider(FieldWithMetaInformationProviderEnum sourceProvider) {
			this.sourceProvider = sourceProvider.toBuilder();
			return this;
		}
		public InformationSourceBuilder setSourceProviderRef(InformationProviderEnum sourceProvider) {
			return setSourceProvider(FieldWithMetaInformationProviderEnum.builder().setValue(sourceProvider).build());
		}
	
		public InformationSource build() {
			return new InformationSource(this);
		}
	
		@Override
		public InformationSourceBuilder prune() {
			if (sourcePage!=null && !sourcePage.prune().hasData()) sourcePage = null;
			if (sourceProvider!=null && !sourceProvider.prune().hasData()) sourceProvider = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getSourcePage()!=null) return true;
			if (getSourcePageHeading()!=null) return true;
			if (getSourceProvider()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("sourcePageHeading"), String.class, sourcePageHeading, this);
			
			processRosetta(path.newSubPath("sourcePage"), processor, FieldWithMetaString.class, sourcePage);
			processRosetta(path.newSubPath("sourceProvider"), processor, FieldWithMetaInformationProviderEnum.class, sourceProvider);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			InformationSourceBuilder _that = (InformationSourceBuilder) o;
		
			if (sourcePage != null ? !sourcePage.equals(_that.sourcePage) : _that.sourcePage != null) return false;
			if (sourcePageHeading != null ? !sourcePageHeading.equals(_that.sourcePageHeading) : _that.sourcePageHeading != null) return false;
			if (sourceProvider != null ? !sourceProvider.equals(_that.sourceProvider) : _that.sourceProvider != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (sourcePage != null ? sourcePage.hashCode() : 0);
			_result = 31 * _result + (sourcePageHeading != null ? sourcePageHeading.hashCode() : 0);
			_result = 31 * _result + (sourceProvider != null ? sourceProvider.getClass().getName().hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "InformationSourceBuilder {" +
				"sourcePage=" + this.sourcePage + ", " +
				"sourcePageHeading=" + this.sourcePageHeading + ", " +
				"sourceProvider=" + this.sourceProvider +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("sourcePageHeading"), String.class, sourcePageHeading, this);
		
		processRosetta(path.newSubPath("sourcePage"), processor, FieldWithMetaString.class, sourcePage);
		processRosetta(path.newSubPath("sourceProvider"), processor, FieldWithMetaInformationProviderEnum.class, sourceProvider);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		InformationSource _that = (InformationSource) o;
	
		if (sourcePage != null ? !sourcePage.equals(_that.sourcePage) : _that.sourcePage != null) return false;
		if (sourcePageHeading != null ? !sourcePageHeading.equals(_that.sourcePageHeading) : _that.sourcePageHeading != null) return false;
		if (sourceProvider != null ? !sourceProvider.equals(_that.sourceProvider) : _that.sourceProvider != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (sourcePage != null ? sourcePage.hashCode() : 0);
		_result = 31 * _result + (sourcePageHeading != null ? sourcePageHeading.hashCode() : 0);
		_result = 31 * _result + (sourceProvider != null ? sourceProvider.getClass().getName().hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "InformationSource {" +
			"sourcePage=" + this.sourcePage + ", " +
			"sourcePageHeading=" + this.sourcePageHeading + ", " +
			"sourceProvider=" + this.sourceProvider +
		'}';
	}
}
