package org.isda.cdm;

import com.rosetta.model.lib.records.Date;
import org.isda.cdm.RelativeDateOffset;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.InitialFixingDateMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;
		import com.rosetta.model.lib.records.Date;

/**
 * A CDM class which purpose is to specify the initial fixing date either alongside the FpML interest rate specification as an offset of another date, or alongside the credit derivative specification as an unadjusted date.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class InitialFixingDate extends RosettaModelObject {
	private final Date initialFixingDate;
	private final RelativeDateOffset relativeDateOffset;
	private static InitialFixingDateMeta metaData = new InitialFixingDateMeta();
	
	InitialFixingDate(InitialFixingDateBuilder builder) {
		this.initialFixingDate = builder.getInitialFixingDate();
		this.relativeDateOffset = ofNullable(builder.getRelativeDateOffset()).map(f->f.build()).orElse(null);
	}
	
	@RosettaSynonym(value="initialFixingDate", source="FpML_5_10", path="periodicPayment.floatingAmountCalculation")
	@RosettaSynonym(value="initialFixingDate", source="CME_SubmissionIRS_1_0", path="periodicPayment.floatingAmountCalculation")
	@RosettaSynonym(value="initialFixingDate", source="DTCC_11_0", path="periodicPayment.floatingAmountCalculation")
	@RosettaSynonym(value="initialFixingDate", source="DTCC_9_0", path="periodicPayment.floatingAmountCalculation")
	@RosettaSynonym(value="initialFixingDate", source="CME_ClearedConfirm_1_17", path="periodicPayment.floatingAmountCalculation")
	public final Date getInitialFixingDate() {
		return initialFixingDate;
	}
	
	@RosettaSynonym(value="initialFixingDate", source="FpML_5_10", path="resetDates")
	@RosettaSynonym(value="initialFixingDate", source="CME_SubmissionIRS_1_0", path="resetDates")
	@RosettaSynonym(value="initialFixingDate", source="DTCC_11_0", path="resetDates")
	@RosettaSynonym(value="initialFixingDate", source="DTCC_9_0", path="resetDates")
	@RosettaSynonym(value="initialFixingDate", source="CME_ClearedConfirm_1_17", path="resetDates")
	public final RelativeDateOffset getRelativeDateOffset() {
		return relativeDateOffset;
	}
	
	
	@Override
	public RosettaMetaData<? extends InitialFixingDate> metaData() {
		return metaData;
	} 
	
	public InitialFixingDateBuilder toBuilder() {
		InitialFixingDateBuilder builder = new InitialFixingDateBuilder();
		ofNullable(getInitialFixingDate()).ifPresent(builder::setInitialFixingDate);
		ofNullable(getRelativeDateOffset()).ifPresent(builder::setRelativeDateOffset);
		return builder;
	}

	public static InitialFixingDateBuilder builder() {
		return new InitialFixingDateBuilder();
	}

	public static  class InitialFixingDateBuilder extends RosettaModelObjectBuilder {
	
		protected Date initialFixingDate;
		protected RelativeDateOffset.RelativeDateOffsetBuilder relativeDateOffset;
	
		public InitialFixingDateBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends InitialFixingDate> metaData() {
			return metaData;
		} 
	
		
		public Date getInitialFixingDate() {
			return initialFixingDate;
		}
		
		
		public RelativeDateOffset.RelativeDateOffsetBuilder getRelativeDateOffset() {
			return relativeDateOffset;
		}
		
		public RelativeDateOffset.RelativeDateOffsetBuilder getOrCreateRelativeDateOffset() {
			if (relativeDateOffset!=null) {
				return relativeDateOffset;
			}
			else return relativeDateOffset = new RelativeDateOffset.RelativeDateOffsetBuilder();
		}
		
	
		public InitialFixingDateBuilder setInitialFixingDate(Date initialFixingDate) {
			this.initialFixingDate = initialFixingDate;
			return this;
		}
		public InitialFixingDateBuilder setRelativeDateOffset(RelativeDateOffset relativeDateOffset) {
			this.relativeDateOffset = relativeDateOffset.toBuilder();
			return this;
		}
		public InitialFixingDateBuilder setRelativeDateOffsetBuilder(RelativeDateOffset.RelativeDateOffsetBuilder relativeDateOffset) {
			this.relativeDateOffset = relativeDateOffset;
			return this;
		}
		
		public InitialFixingDate build() {
			return new InitialFixingDate(this);
		}
	
		@Override
		public InitialFixingDateBuilder prune() {
			if (relativeDateOffset!=null && !relativeDateOffset.prune().hasData()) relativeDateOffset = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getInitialFixingDate()!=null) return true;
			if (getRelativeDateOffset()!=null && getRelativeDateOffset().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("initialFixingDate"), Date.class, initialFixingDate, this);
			
			processRosetta(path.newSubPath("relativeDateOffset"), processor, RelativeDateOffset.class, relativeDateOffset);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			InitialFixingDateBuilder _that = (InitialFixingDateBuilder) o;
		
			if (initialFixingDate != null ? !initialFixingDate.equals(_that.initialFixingDate) : _that.initialFixingDate != null) return false;
			if (relativeDateOffset != null ? !relativeDateOffset.equals(_that.relativeDateOffset) : _that.relativeDateOffset != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (initialFixingDate != null ? initialFixingDate.hashCode() : 0);
			_result = 31 * _result + (relativeDateOffset != null ? relativeDateOffset.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "InitialFixingDateBuilder {" +
				"initialFixingDate=" + this.initialFixingDate + ", " +
				"relativeDateOffset=" + this.relativeDateOffset +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("initialFixingDate"), Date.class, initialFixingDate, this);
		
		processRosetta(path.newSubPath("relativeDateOffset"), processor, RelativeDateOffset.class, relativeDateOffset);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		InitialFixingDate _that = (InitialFixingDate) o;
	
		if (initialFixingDate != null ? !initialFixingDate.equals(_that.initialFixingDate) : _that.initialFixingDate != null) return false;
		if (relativeDateOffset != null ? !relativeDateOffset.equals(_that.relativeDateOffset) : _that.relativeDateOffset != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (initialFixingDate != null ? initialFixingDate.hashCode() : 0);
		_result = 31 * _result + (relativeDateOffset != null ? relativeDateOffset.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "InitialFixingDate {" +
			"initialFixingDate=" + this.initialFixingDate + ", " +
			"relativeDateOffset=" + this.relativeDateOffset +
		'}';
	}
}
