package org.isda.cdm;

import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.InitialMarginCalculation;
import org.isda.cdm.MarginTypeEnum;
import org.isda.cdm.Money;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.InitialMarginMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;

/**
 * Defines initial margin applied to a repo transaction. Initial margin is an agreed premium to the Purchase Price of a repo to determine the required Market Value of the collateral to be delivered on the Purchase Date. It reflects quality of the collateral. Its aim is to calculate the risk-adjusted or liquidation value of collateral.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class InitialMargin extends RosettaModelObject {
	private final List<InitialMarginCalculation> margin;
	private final Money marginThreshold;
	private final MarginTypeEnum marginType;
	private final Money minimumTransferAmount;
	private static InitialMarginMeta metaData = new InitialMarginMeta();
	
	InitialMargin(InitialMarginBuilder builder) {
		this.margin = ofNullable(builder.getMargin()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.marginThreshold = ofNullable(builder.getMarginThreshold()).map(f->f.build()).orElse(null);
		this.marginType = builder.getMarginType();
		this.minimumTransferAmount = ofNullable(builder.getMinimumTransferAmount()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * Initial margin calculation for a collateral asset. Initial margin requirements may be specified for multiple pieces of collateral.
	 */
	@RosettaSynonym(value="margin", source="FpML_5_10")
	public final List<InitialMarginCalculation> getMargin() {
		return margin;
	}
	
	/**
	 * An element defining a margin threshold which is the Net Exposure of a trade below which parties agree they will not call a margin from each other.
	 */
	@RosettaSynonym(value="marginThreshold", source="FpML_5_10")
	public final Money getMarginThreshold() {
		return marginThreshold;
	}
	
	/**
	 * An element defining the type of assets (cash or securities) specified to apply as margin to the repo transaction. See GMRA 2011 paragraph 2(h) for &#39;Cash Margin&#39; and GMRA 2011 paragraph 2(cc) for &#39;Margin Securities&#39;.
	 */
	@RosettaSynonym(value="marginType", source="FpML_5_10")
	public final MarginTypeEnum getMarginType() {
		return marginType;
	}
	
	/**
	 * An element defining a minimum transfer amount which is the minimum margin call parties will make once the margin threshold (or margin ratio threshold / haircut threshold) has been exceeded.
	 */
	@RosettaSynonym(value="minimumTransferAmount", source="FpML_5_10")
	public final Money getMinimumTransferAmount() {
		return minimumTransferAmount;
	}
	
	
	@Override
	public RosettaMetaData<? extends InitialMargin> metaData() {
		return metaData;
	} 
	
	public InitialMarginBuilder toBuilder() {
		InitialMarginBuilder builder = new InitialMarginBuilder();
		ofNullable(getMargin()).ifPresent(margin -> margin.forEach(builder::addMargin));
		ofNullable(getMarginThreshold()).ifPresent(builder::setMarginThreshold);
		ofNullable(getMarginType()).ifPresent(builder::setMarginType);
		ofNullable(getMinimumTransferAmount()).ifPresent(builder::setMinimumTransferAmount);
		return builder;
	}

	public static InitialMarginBuilder builder() {
		return new InitialMarginBuilder();
	}

	public static  class InitialMarginBuilder extends RosettaModelObjectBuilder {
	
		protected List<InitialMarginCalculation.InitialMarginCalculationBuilder> margin;
		protected Money.MoneyBuilder marginThreshold;
		protected MarginTypeEnum marginType;
		protected Money.MoneyBuilder minimumTransferAmount;
	
		public InitialMarginBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends InitialMargin> metaData() {
			return metaData;
		} 
	
		
		public List<InitialMarginCalculation.InitialMarginCalculationBuilder> getMargin() {
			return margin;
		}
		
		public InitialMarginCalculation.InitialMarginCalculationBuilder getOrCreateMargin(int index) {
			if (margin==null) {
				this.margin = new ArrayList<>();
			}
			return getIndex(margin, index, ()->new InitialMarginCalculation.InitialMarginCalculationBuilder());
		}
		
		
		public Money.MoneyBuilder getMarginThreshold() {
			return marginThreshold;
		}
		
		public Money.MoneyBuilder getOrCreateMarginThreshold() {
			if (marginThreshold!=null) {
				return marginThreshold;
			}
			else return marginThreshold = new Money.MoneyBuilder();
		}
		
		
		public MarginTypeEnum getMarginType() {
			return marginType;
		}
		
		
		public Money.MoneyBuilder getMinimumTransferAmount() {
			return minimumTransferAmount;
		}
		
		public Money.MoneyBuilder getOrCreateMinimumTransferAmount() {
			if (minimumTransferAmount!=null) {
				return minimumTransferAmount;
			}
			else return minimumTransferAmount = new Money.MoneyBuilder();
		}
		
	
		public InitialMarginBuilder addMargin(InitialMarginCalculation margin) {
			if(this.margin == null){
				this.margin = new ArrayList<>();
			}
			this.margin.add(margin.toBuilder());
			return this;
		}
		public InitialMarginBuilder addMargin(InitialMarginCalculation margin, int _idx) {
			if(this.margin == null){
				this.margin = new ArrayList<>();
			}
			getIndex(this.margin, _idx, () -> margin.toBuilder());
			this.margin.set(_idx, margin.toBuilder());
			return this;
		}
		public InitialMarginBuilder addMargin(List<InitialMarginCalculation> margins) {
			if(this.margin == null){
				this.margin = new ArrayList<>();
			}
			for (InitialMarginCalculation toAdd : margins) {
				this.margin.add(toAdd.toBuilder());
			}
			return this;
		}
		public InitialMarginBuilder addMarginBuilder(InitialMarginCalculation.InitialMarginCalculationBuilder margin) {
			if(this.margin == null){
				this.margin = new ArrayList<>();
			}
			this.margin.add(margin);
			return this;
		}
		
		public InitialMarginBuilder clearMargin() {
			this.margin = null;
			return this;
		}
		public InitialMarginBuilder setMarginThreshold(Money marginThreshold) {
			this.marginThreshold = marginThreshold.toBuilder();
			return this;
		}
		public InitialMarginBuilder setMarginThresholdBuilder(Money.MoneyBuilder marginThreshold) {
			this.marginThreshold = marginThreshold;
			return this;
		}
		public InitialMarginBuilder setMarginType(MarginTypeEnum marginType) {
			this.marginType = marginType;
			return this;
		}
		public InitialMarginBuilder setMinimumTransferAmount(Money minimumTransferAmount) {
			this.minimumTransferAmount = minimumTransferAmount.toBuilder();
			return this;
		}
		public InitialMarginBuilder setMinimumTransferAmountBuilder(Money.MoneyBuilder minimumTransferAmount) {
			this.minimumTransferAmount = minimumTransferAmount;
			return this;
		}
		
		public InitialMargin build() {
			return new InitialMargin(this);
		}
	
		@Override
		public InitialMarginBuilder prune() {
			if (margin!=null) margin = margin.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (marginThreshold!=null && !marginThreshold.prune().hasData()) marginThreshold = null;
			if (minimumTransferAmount!=null && !minimumTransferAmount.prune().hasData()) minimumTransferAmount = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getMargin()!=null && getMargin().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getMarginThreshold()!=null && getMarginThreshold().hasData()) return true;
			if (getMarginType()!=null) return true;
			if (getMinimumTransferAmount()!=null && getMinimumTransferAmount().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("marginType"), MarginTypeEnum.class, marginType, this);
			
			processRosetta(path.newSubPath("margin"), processor, InitialMarginCalculation.class, margin);
			processRosetta(path.newSubPath("marginThreshold"), processor, Money.class, marginThreshold);
			processRosetta(path.newSubPath("minimumTransferAmount"), processor, Money.class, minimumTransferAmount);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			InitialMarginBuilder _that = (InitialMarginBuilder) o;
		
			if (!ListEquals.listEquals(margin, _that.margin)) return false;
			if (marginThreshold != null ? !marginThreshold.equals(_that.marginThreshold) : _that.marginThreshold != null) return false;
			if (marginType != null ? !marginType.equals(_that.marginType) : _that.marginType != null) return false;
			if (minimumTransferAmount != null ? !minimumTransferAmount.equals(_that.minimumTransferAmount) : _that.minimumTransferAmount != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (margin != null ? margin.hashCode() : 0);
			_result = 31 * _result + (marginThreshold != null ? marginThreshold.hashCode() : 0);
			_result = 31 * _result + (marginType != null ? marginType.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (minimumTransferAmount != null ? minimumTransferAmount.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "InitialMarginBuilder {" +
				"margin=" + this.margin + ", " +
				"marginThreshold=" + this.marginThreshold + ", " +
				"marginType=" + this.marginType + ", " +
				"minimumTransferAmount=" + this.minimumTransferAmount +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("marginType"), MarginTypeEnum.class, marginType, this);
		
		processRosetta(path.newSubPath("margin"), processor, InitialMarginCalculation.class, margin);
		processRosetta(path.newSubPath("marginThreshold"), processor, Money.class, marginThreshold);
		processRosetta(path.newSubPath("minimumTransferAmount"), processor, Money.class, minimumTransferAmount);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		InitialMargin _that = (InitialMargin) o;
	
		if (!ListEquals.listEquals(margin, _that.margin)) return false;
		if (marginThreshold != null ? !marginThreshold.equals(_that.marginThreshold) : _that.marginThreshold != null) return false;
		if (marginType != null ? !marginType.equals(_that.marginType) : _that.marginType != null) return false;
		if (minimumTransferAmount != null ? !minimumTransferAmount.equals(_that.minimumTransferAmount) : _that.minimumTransferAmount != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (margin != null ? margin.hashCode() : 0);
		_result = 31 * _result + (marginThreshold != null ? marginThreshold.hashCode() : 0);
		_result = 31 * _result + (marginType != null ? marginType.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (minimumTransferAmount != null ? minimumTransferAmount.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "InitialMargin {" +
			"margin=" + this.margin + ", " +
			"marginThreshold=" + this.marginThreshold + ", " +
			"marginType=" + this.marginType + ", " +
			"minimumTransferAmount=" + this.minimumTransferAmount +
		'}';
	}
}
