package org.isda.cdm;

import java.math.BigDecimal;
import java.util.List;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.InitialMarginCalculationMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.math.BigDecimal;
		import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;

/**
 * Defines the initial margin calculation applicable to a single piece of collateral.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class InitialMarginCalculation extends RosettaModelObject {
	private final BigDecimal haircut;
	private final List<BigDecimal> haircutThreshold;
	private final BigDecimal marginRatio;
	private final List<BigDecimal> marginRatioThreshold;
	private static InitialMarginCalculationMeta metaData = new InitialMarginCalculationMeta();
	
	InitialMarginCalculation(InitialMarginCalculationBuilder builder) {
		this.haircut = builder.getHaircut();
		this.haircutThreshold = builder.getHaircutThreshold();
		this.marginRatio = builder.getMarginRatio();
		this.marginRatioThreshold = builder.getMarginRatioThreshold();
	}
	
	/**
	 * An element defining a haircut expressed as the percentage difference between the Market Value of the collateral and the Purchase Price of the repo and calculated as 100 multiplied by a ratio of the difference between the Market Value of the collateral and the Purchase Price of the repo to the Market Value of the collateral. Haircut is alternative way to adjust the value of collateral sold in a repurchase agreement to initial margin ratio. Because an initial margin is a percentage of the Purchase Price, while a haircut is a percentage of the Market Value of collateral, the arithmetic of initial margins and haircuts is slightly different. For example, an initial margin of 102% is not equivalent to a haircut of 2%, but to 1.961% (ie 100/102%). See GMRA 2011 paragraph 2(aa).
	 */
	@RosettaSynonym(value="haircut", source="FpML_5_10")
	public final BigDecimal getHaircut() {
		return haircut;
	}
	
	/**
	 * An element defining a haircut percentage threshold which is the value above (when it&#39;s lower than initial haircut) or below (when it&#39;s higher than initial haircut) which parties agree they will not call a margin from each other.
	 */
	@RosettaSynonym(value="haircutThreshold", source="FpML_5_10")
	public final List<BigDecimal> getHaircutThreshold() {
		return haircutThreshold;
	}
	
	/**
	 * An element defining an initial margin expressed as a ratio of the Market Value of the collateral to the Purchase Price. A default value of initial margin ratio of 1.00 means there is no margin and thus no risk related with the collateral. See GMRA 2000 paragraph 2(z) and GMRA 2011 paragraph 2(bb).
	 */
	@RosettaSynonym(value="marginRatio", source="FpML_5_10")
	public final BigDecimal getMarginRatio() {
		return marginRatio;
	}
	
	/**
	 * An element defining a margin ratio threshold which is the value above (when it&#39;s lower than initial margin ratio) or below (when it&#39;s higher than initial margin ratio) which parties agree they will not call a margin from each other.
	 */
	@RosettaSynonym(value="marginRatioThreshold", source="FpML_5_10")
	public final List<BigDecimal> getMarginRatioThreshold() {
		return marginRatioThreshold;
	}
	
	
	@Override
	public RosettaMetaData<? extends InitialMarginCalculation> metaData() {
		return metaData;
	} 
	
	public InitialMarginCalculationBuilder toBuilder() {
		InitialMarginCalculationBuilder builder = new InitialMarginCalculationBuilder();
		ofNullable(getHaircut()).ifPresent(builder::setHaircut);
		ofNullable(getHaircutThreshold()).ifPresent(haircutThreshold -> haircutThreshold.forEach(builder::addHaircutThreshold));
		ofNullable(getMarginRatio()).ifPresent(builder::setMarginRatio);
		ofNullable(getMarginRatioThreshold()).ifPresent(marginRatioThreshold -> marginRatioThreshold.forEach(builder::addMarginRatioThreshold));
		return builder;
	}

	public static InitialMarginCalculationBuilder builder() {
		return new InitialMarginCalculationBuilder();
	}

	public static  class InitialMarginCalculationBuilder extends RosettaModelObjectBuilder {
	
		protected BigDecimal haircut;
		protected List<BigDecimal> haircutThreshold;
		protected BigDecimal marginRatio;
		protected List<BigDecimal> marginRatioThreshold;
	
		public InitialMarginCalculationBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends InitialMarginCalculation> metaData() {
			return metaData;
		} 
	
		
		public BigDecimal getHaircut() {
			return haircut;
		}
		
		
		public List<BigDecimal> getHaircutThreshold() {
			return haircutThreshold;
		}
		
		
		public BigDecimal getMarginRatio() {
			return marginRatio;
		}
		
		
		public List<BigDecimal> getMarginRatioThreshold() {
			return marginRatioThreshold;
		}
		
	
		public InitialMarginCalculationBuilder setHaircut(BigDecimal haircut) {
			this.haircut = haircut;
			return this;
		}
		public InitialMarginCalculationBuilder addHaircutThreshold(BigDecimal haircutThreshold) {
			if(this.haircutThreshold == null){
				this.haircutThreshold = new ArrayList<>();
			}
			this.haircutThreshold.add(haircutThreshold);
			return this;
		}
		public InitialMarginCalculationBuilder addHaircutThreshold(BigDecimal haircutThreshold, int _idx) {
			if(this.haircutThreshold == null){
				this.haircutThreshold = new ArrayList<>();
			}
			getIndex(this.haircutThreshold, _idx, () -> haircutThreshold);
			this.haircutThreshold.set(_idx, haircutThreshold);
			return this;
		}
		public InitialMarginCalculationBuilder addHaircutThreshold(List<BigDecimal> haircutThresholds) {
			if(this.haircutThreshold == null){
				this.haircutThreshold = new ArrayList<>();
			}
			for (BigDecimal toAdd : haircutThresholds) {
				this.haircutThreshold.add(toAdd);
			}
			return this;
		}
		
		public InitialMarginCalculationBuilder clearHaircutThreshold() {
			this.haircutThreshold = null;
			return this;
		}
		public InitialMarginCalculationBuilder setMarginRatio(BigDecimal marginRatio) {
			this.marginRatio = marginRatio;
			return this;
		}
		public InitialMarginCalculationBuilder addMarginRatioThreshold(BigDecimal marginRatioThreshold) {
			if(this.marginRatioThreshold == null){
				this.marginRatioThreshold = new ArrayList<>();
			}
			this.marginRatioThreshold.add(marginRatioThreshold);
			return this;
		}
		public InitialMarginCalculationBuilder addMarginRatioThreshold(BigDecimal marginRatioThreshold, int _idx) {
			if(this.marginRatioThreshold == null){
				this.marginRatioThreshold = new ArrayList<>();
			}
			getIndex(this.marginRatioThreshold, _idx, () -> marginRatioThreshold);
			this.marginRatioThreshold.set(_idx, marginRatioThreshold);
			return this;
		}
		public InitialMarginCalculationBuilder addMarginRatioThreshold(List<BigDecimal> marginRatioThresholds) {
			if(this.marginRatioThreshold == null){
				this.marginRatioThreshold = new ArrayList<>();
			}
			for (BigDecimal toAdd : marginRatioThresholds) {
				this.marginRatioThreshold.add(toAdd);
			}
			return this;
		}
		
		public InitialMarginCalculationBuilder clearMarginRatioThreshold() {
			this.marginRatioThreshold = null;
			return this;
		}
		
		public InitialMarginCalculation build() {
			return new InitialMarginCalculation(this);
		}
	
		@Override
		public InitialMarginCalculationBuilder prune() {
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getHaircut()!=null) return true;
			if (getHaircutThreshold()!=null && !getHaircutThreshold().isEmpty()) return true;
			if (getMarginRatio()!=null) return true;
			if (getMarginRatioThreshold()!=null && !getMarginRatioThreshold().isEmpty()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("haircut"), BigDecimal.class, haircut, this);
			processor.processBasic(path.newSubPath("haircutThreshold"), BigDecimal.class, haircutThreshold, this);
			processor.processBasic(path.newSubPath("marginRatio"), BigDecimal.class, marginRatio, this);
			processor.processBasic(path.newSubPath("marginRatioThreshold"), BigDecimal.class, marginRatioThreshold, this);
			
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			InitialMarginCalculationBuilder _that = (InitialMarginCalculationBuilder) o;
		
			if (haircut != null ? !haircut.equals(_that.haircut) : _that.haircut != null) return false;
			if (!ListEquals.listEquals(haircutThreshold, _that.haircutThreshold)) return false;
			if (marginRatio != null ? !marginRatio.equals(_that.marginRatio) : _that.marginRatio != null) return false;
			if (!ListEquals.listEquals(marginRatioThreshold, _that.marginRatioThreshold)) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (haircut != null ? haircut.hashCode() : 0);
			_result = 31 * _result + (haircutThreshold != null ? haircutThreshold.hashCode() : 0);
			_result = 31 * _result + (marginRatio != null ? marginRatio.hashCode() : 0);
			_result = 31 * _result + (marginRatioThreshold != null ? marginRatioThreshold.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "InitialMarginCalculationBuilder {" +
				"haircut=" + this.haircut + ", " +
				"haircutThreshold=" + this.haircutThreshold + ", " +
				"marginRatio=" + this.marginRatio + ", " +
				"marginRatioThreshold=" + this.marginRatioThreshold +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("haircut"), BigDecimal.class, haircut, this);
		haircutThreshold.stream().forEach(a->processor.processBasic(path.newSubPath("haircutThreshold"), BigDecimal.class, a, this));
		processor.processBasic(path.newSubPath("marginRatio"), BigDecimal.class, marginRatio, this);
		marginRatioThreshold.stream().forEach(a->processor.processBasic(path.newSubPath("marginRatioThreshold"), BigDecimal.class, a, this));
		
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		InitialMarginCalculation _that = (InitialMarginCalculation) o;
	
		if (haircut != null ? !haircut.equals(_that.haircut) : _that.haircut != null) return false;
		if (!ListEquals.listEquals(haircutThreshold, _that.haircutThreshold)) return false;
		if (marginRatio != null ? !marginRatio.equals(_that.marginRatio) : _that.marginRatio != null) return false;
		if (!ListEquals.listEquals(marginRatioThreshold, _that.marginRatioThreshold)) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (haircut != null ? haircut.hashCode() : 0);
		_result = 31 * _result + (haircutThreshold != null ? haircutThreshold.hashCode() : 0);
		_result = 31 * _result + (marginRatio != null ? marginRatio.hashCode() : 0);
		_result = 31 * _result + (marginRatioThreshold != null ? marginRatioThreshold.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "InitialMarginCalculation {" +
			"haircut=" + this.haircut + ", " +
			"haircutThreshold=" + this.haircutThreshold + ", " +
			"marginRatio=" + this.marginRatio + ", " +
			"marginRatioThreshold=" + this.marginRatioThreshold +
		'}';
	}
}
