package org.isda.cdm;

import java.lang.Boolean;
import org.isda.cdm.InterestAdjustmentPeriodicity;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.InterestAdjustmentMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class to specify whether the Interest Adjustment is applicable and what its periodicity is. ISDA 2016 Japanese Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (n)(ii).
 * 
 * @version 2.5.4
 */
@RosettaClass
public class InterestAdjustment extends RosettaModelObject {
	private final Boolean isApplicable;
	private final InterestAdjustmentPeriodicity periodicity;
	private static InterestAdjustmentMeta metaData = new InterestAdjustmentMeta();
	
	InterestAdjustment(InterestAdjustmentBuilder builder) {
		this.isApplicable = builder.getIsApplicable();
		this.periodicity = ofNullable(builder.getPeriodicity()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The Interest Adjustment is applicable when True and not applicable when False
	 */
	@RosettaSynonym(value="", source="ISDA_Create_1_0")
	public final Boolean getIsApplicable() {
		return isApplicable;
	}
	
	/**
	 * The qualification of the Interest Adjustment periodicity.
	 */
	public final InterestAdjustmentPeriodicity getPeriodicity() {
		return periodicity;
	}
	
	
	@Override
	public RosettaMetaData<? extends InterestAdjustment> metaData() {
		return metaData;
	} 
	
	public InterestAdjustmentBuilder toBuilder() {
		InterestAdjustmentBuilder builder = new InterestAdjustmentBuilder();
		ofNullable(getIsApplicable()).ifPresent(builder::setIsApplicable);
		ofNullable(getPeriodicity()).ifPresent(builder::setPeriodicity);
		return builder;
	}

	public static InterestAdjustmentBuilder builder() {
		return new InterestAdjustmentBuilder();
	}

	public static  class InterestAdjustmentBuilder extends RosettaModelObjectBuilder {
	
		protected Boolean isApplicable;
		protected InterestAdjustmentPeriodicity.InterestAdjustmentPeriodicityBuilder periodicity;
	
		public InterestAdjustmentBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends InterestAdjustment> metaData() {
			return metaData;
		} 
	
		
		public Boolean getIsApplicable() {
			return isApplicable;
		}
		
		
		public InterestAdjustmentPeriodicity.InterestAdjustmentPeriodicityBuilder getPeriodicity() {
			return periodicity;
		}
		
		public InterestAdjustmentPeriodicity.InterestAdjustmentPeriodicityBuilder getOrCreatePeriodicity() {
			if (periodicity!=null) {
				return periodicity;
			}
			else return periodicity = new InterestAdjustmentPeriodicity.InterestAdjustmentPeriodicityBuilder();
		}
		
	
		public InterestAdjustmentBuilder setIsApplicable(Boolean isApplicable) {
			this.isApplicable = isApplicable;
			return this;
		}
		public InterestAdjustmentBuilder setPeriodicity(InterestAdjustmentPeriodicity periodicity) {
			this.periodicity = periodicity.toBuilder();
			return this;
		}
		public InterestAdjustmentBuilder setPeriodicityBuilder(InterestAdjustmentPeriodicity.InterestAdjustmentPeriodicityBuilder periodicity) {
			this.periodicity = periodicity;
			return this;
		}
		
		public InterestAdjustment build() {
			return new InterestAdjustment(this);
		}
	
		@Override
		public InterestAdjustmentBuilder prune() {
			if (periodicity!=null && !periodicity.prune().hasData()) periodicity = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getIsApplicable()!=null) return true;
			if (getPeriodicity()!=null && getPeriodicity().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("isApplicable"), Boolean.class, isApplicable, this);
			
			processRosetta(path.newSubPath("periodicity"), processor, InterestAdjustmentPeriodicity.class, periodicity);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			InterestAdjustmentBuilder _that = (InterestAdjustmentBuilder) o;
		
			if (isApplicable != null ? !isApplicable.equals(_that.isApplicable) : _that.isApplicable != null) return false;
			if (periodicity != null ? !periodicity.equals(_that.periodicity) : _that.periodicity != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (isApplicable != null ? isApplicable.hashCode() : 0);
			_result = 31 * _result + (periodicity != null ? periodicity.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "InterestAdjustmentBuilder {" +
				"isApplicable=" + this.isApplicable + ", " +
				"periodicity=" + this.periodicity +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("isApplicable"), Boolean.class, isApplicable, this);
		
		processRosetta(path.newSubPath("periodicity"), processor, InterestAdjustmentPeriodicity.class, periodicity);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		InterestAdjustment _that = (InterestAdjustment) o;
	
		if (isApplicable != null ? !isApplicable.equals(_that.isApplicable) : _that.isApplicable != null) return false;
		if (periodicity != null ? !periodicity.equals(_that.periodicity) : _that.periodicity != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (isApplicable != null ? isApplicable.hashCode() : 0);
		_result = 31 * _result + (periodicity != null ? periodicity.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "InterestAdjustment {" +
			"isApplicable=" + this.isApplicable + ", " +
			"periodicity=" + this.periodicity +
		'}';
	}
}
