package org.isda.cdm;

import java.lang.String;
import org.isda.cdm.InterestAdjustmentPeriodicityEnum;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.InterestAdjustmentPeriodicityMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class to specify the Interest Adjustment periodicity either through a standardized election or a custom one. ISDA 2016 Japanese Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (n)(ii).
 * 
 * @version 2.5.4
 */
@RosettaClass
public class InterestAdjustmentPeriodicity extends RosettaModelObject {
	private final String customElection;
	private final InterestAdjustmentPeriodicityEnum standardElection;
	private static InterestAdjustmentPeriodicityMeta metaData = new InterestAdjustmentPeriodicityMeta();
	
	InterestAdjustmentPeriodicity(InterestAdjustmentPeriodicityBuilder builder) {
		this.customElection = builder.getCustomElection();
		this.standardElection = builder.getStandardElection();
	}
	
	/**
	 * The Interest Adjustment periodicity when specified through a custom election.
	 */
	public final String getCustomElection() {
		return customElection;
	}
	
	/**
	 * The Interest Adjustment periodicity when specified through a standardized election.
	 */
	public final InterestAdjustmentPeriodicityEnum getStandardElection() {
		return standardElection;
	}
	
	
	@Override
	public RosettaMetaData<? extends InterestAdjustmentPeriodicity> metaData() {
		return metaData;
	} 
	
	public InterestAdjustmentPeriodicityBuilder toBuilder() {
		InterestAdjustmentPeriodicityBuilder builder = new InterestAdjustmentPeriodicityBuilder();
		ofNullable(getCustomElection()).ifPresent(builder::setCustomElection);
		ofNullable(getStandardElection()).ifPresent(builder::setStandardElection);
		return builder;
	}

	public static InterestAdjustmentPeriodicityBuilder builder() {
		return new InterestAdjustmentPeriodicityBuilder();
	}

	public static  class InterestAdjustmentPeriodicityBuilder extends RosettaModelObjectBuilder {
	
		protected String customElection;
		protected InterestAdjustmentPeriodicityEnum standardElection;
	
		public InterestAdjustmentPeriodicityBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends InterestAdjustmentPeriodicity> metaData() {
			return metaData;
		} 
	
		
		public String getCustomElection() {
			return customElection;
		}
		
		
		public InterestAdjustmentPeriodicityEnum getStandardElection() {
			return standardElection;
		}
		
	
		public InterestAdjustmentPeriodicityBuilder setCustomElection(String customElection) {
			this.customElection = customElection;
			return this;
		}
		public InterestAdjustmentPeriodicityBuilder setStandardElection(InterestAdjustmentPeriodicityEnum standardElection) {
			this.standardElection = standardElection;
			return this;
		}
		
		public InterestAdjustmentPeriodicity build() {
			return new InterestAdjustmentPeriodicity(this);
		}
	
		@Override
		public InterestAdjustmentPeriodicityBuilder prune() {
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getCustomElection()!=null) return true;
			if (getStandardElection()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("customElection"), String.class, customElection, this);
			processor.processBasic(path.newSubPath("standardElection"), InterestAdjustmentPeriodicityEnum.class, standardElection, this);
			
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			InterestAdjustmentPeriodicityBuilder _that = (InterestAdjustmentPeriodicityBuilder) o;
		
			if (customElection != null ? !customElection.equals(_that.customElection) : _that.customElection != null) return false;
			if (standardElection != null ? !standardElection.equals(_that.standardElection) : _that.standardElection != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (customElection != null ? customElection.hashCode() : 0);
			_result = 31 * _result + (standardElection != null ? standardElection.getClass().getName().hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "InterestAdjustmentPeriodicityBuilder {" +
				"customElection=" + this.customElection + ", " +
				"standardElection=" + this.standardElection +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("customElection"), String.class, customElection, this);
		processor.processBasic(path.newSubPath("standardElection"), InterestAdjustmentPeriodicityEnum.class, standardElection, this);
		
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		InterestAdjustmentPeriodicity _that = (InterestAdjustmentPeriodicity) o;
	
		if (customElection != null ? !customElection.equals(_that.customElection) : _that.customElection != null) return false;
		if (standardElection != null ? !standardElection.equals(_that.standardElection) : _that.standardElection != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (customElection != null ? customElection.hashCode() : 0);
		_result = 31 * _result + (standardElection != null ? standardElection.getClass().getName().hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "InterestAdjustmentPeriodicity {" +
			"customElection=" + this.customElection + ", " +
			"standardElection=" + this.standardElection +
		'}';
	}
}
