package org.isda.cdm;

import org.isda.cdm.DeliveryAmount;
import org.isda.cdm.ReturnAmount;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.InterestAmountMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class to specify the application of Interest Amount with respect to the Delivery Amount and the Return Amount. ISDA 2016 Japanese Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (n)(ii).
 * 
 * @version 2.5.4
 */
@RosettaClass
public class InterestAmount extends RosettaModelObject {
	private final DeliveryAmount deliveryAmount;
	private final ReturnAmount returnAmount;
	private static InterestAmountMeta metaData = new InterestAmountMeta();
	
	InterestAmount(InterestAmountBuilder builder) {
		this.deliveryAmount = ofNullable(builder.getDeliveryAmount()).map(f->f.build()).orElse(null);
		this.returnAmount = ofNullable(builder.getReturnAmount()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The application of Interest Amount with respect the Delivery Amount. ISDA 2016 Japanese Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (n)(ii).
	 */
	public final DeliveryAmount getDeliveryAmount() {
		return deliveryAmount;
	}
	
	/**
	 * The application of Interest Amount with respect the Return Amount. ISDA 2016 Japanese Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (n)(ii).
	 */
	public final ReturnAmount getReturnAmount() {
		return returnAmount;
	}
	
	
	@Override
	public RosettaMetaData<? extends InterestAmount> metaData() {
		return metaData;
	} 
	
	public InterestAmountBuilder toBuilder() {
		InterestAmountBuilder builder = new InterestAmountBuilder();
		ofNullable(getDeliveryAmount()).ifPresent(builder::setDeliveryAmount);
		ofNullable(getReturnAmount()).ifPresent(builder::setReturnAmount);
		return builder;
	}

	public static InterestAmountBuilder builder() {
		return new InterestAmountBuilder();
	}

	public static  class InterestAmountBuilder extends RosettaModelObjectBuilder {
	
		protected DeliveryAmount.DeliveryAmountBuilder deliveryAmount;
		protected ReturnAmount.ReturnAmountBuilder returnAmount;
	
		public InterestAmountBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends InterestAmount> metaData() {
			return metaData;
		} 
	
		
		public DeliveryAmount.DeliveryAmountBuilder getDeliveryAmount() {
			return deliveryAmount;
		}
		
		public DeliveryAmount.DeliveryAmountBuilder getOrCreateDeliveryAmount() {
			if (deliveryAmount!=null) {
				return deliveryAmount;
			}
			else return deliveryAmount = new DeliveryAmount.DeliveryAmountBuilder();
		}
		
		
		public ReturnAmount.ReturnAmountBuilder getReturnAmount() {
			return returnAmount;
		}
		
		public ReturnAmount.ReturnAmountBuilder getOrCreateReturnAmount() {
			if (returnAmount!=null) {
				return returnAmount;
			}
			else return returnAmount = new ReturnAmount.ReturnAmountBuilder();
		}
		
	
		public InterestAmountBuilder setDeliveryAmount(DeliveryAmount deliveryAmount) {
			this.deliveryAmount = deliveryAmount.toBuilder();
			return this;
		}
		public InterestAmountBuilder setDeliveryAmountBuilder(DeliveryAmount.DeliveryAmountBuilder deliveryAmount) {
			this.deliveryAmount = deliveryAmount;
			return this;
		}
		public InterestAmountBuilder setReturnAmount(ReturnAmount returnAmount) {
			this.returnAmount = returnAmount.toBuilder();
			return this;
		}
		public InterestAmountBuilder setReturnAmountBuilder(ReturnAmount.ReturnAmountBuilder returnAmount) {
			this.returnAmount = returnAmount;
			return this;
		}
		
		public InterestAmount build() {
			return new InterestAmount(this);
		}
	
		@Override
		public InterestAmountBuilder prune() {
			if (deliveryAmount!=null && !deliveryAmount.prune().hasData()) deliveryAmount = null;
			if (returnAmount!=null && !returnAmount.prune().hasData()) returnAmount = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getDeliveryAmount()!=null && getDeliveryAmount().hasData()) return true;
			if (getReturnAmount()!=null && getReturnAmount().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("deliveryAmount"), processor, DeliveryAmount.class, deliveryAmount);
			processRosetta(path.newSubPath("returnAmount"), processor, ReturnAmount.class, returnAmount);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			InterestAmountBuilder _that = (InterestAmountBuilder) o;
		
			if (deliveryAmount != null ? !deliveryAmount.equals(_that.deliveryAmount) : _that.deliveryAmount != null) return false;
			if (returnAmount != null ? !returnAmount.equals(_that.returnAmount) : _that.returnAmount != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (deliveryAmount != null ? deliveryAmount.hashCode() : 0);
			_result = 31 * _result + (returnAmount != null ? returnAmount.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "InterestAmountBuilder {" +
				"deliveryAmount=" + this.deliveryAmount + ", " +
				"returnAmount=" + this.returnAmount +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("deliveryAmount"), processor, DeliveryAmount.class, deliveryAmount);
		processRosetta(path.newSubPath("returnAmount"), processor, ReturnAmount.class, returnAmount);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		InterestAmount _that = (InterestAmount) o;
	
		if (deliveryAmount != null ? !deliveryAmount.equals(_that.deliveryAmount) : _that.deliveryAmount != null) return false;
		if (returnAmount != null ? !returnAmount.equals(_that.returnAmount) : _that.returnAmount != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (deliveryAmount != null ? deliveryAmount.hashCode() : 0);
		_result = 31 * _result + (returnAmount != null ? returnAmount.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "InterestAmount {" +
			"deliveryAmount=" + this.deliveryAmount + ", " +
			"returnAmount=" + this.returnAmount +
		'}';
	}
}
