package org.isda.cdm;

import java.lang.Boolean;
import java.lang.String;
import org.isda.cdm.AdjustableDate;
import org.isda.cdm.BondReference;
import org.isda.cdm.CalculationPeriodDates;
import org.isda.cdm.CashflowRepresentation;
import org.isda.cdm.CompoundingMethodEnum;
import org.isda.cdm.ContractualQuantity;
import org.isda.cdm.CrossCurrencyTerms;
import org.isda.cdm.DiscountingMethod;
import org.isda.cdm.PayerReceiver;
import org.isda.cdm.PaymentDates;
import org.isda.cdm.RateSpecification;
import org.isda.cdm.ResetDates;
import org.isda.cdm.StubPeriod;
import org.isda.cdm.metafields.FieldWithMetaDayCountFractionEnum;
import org.isda.cdm.metafields.FieldWithMetaDayCountFractionEnum.FieldWithMetaDayCountFractionEnumBuilder;
import org.isda.cdm.metafields.MetaFields;


import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.InterestRatePayoutMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class to specify all of the terms necessary to define and calculate a cash flow based on a fixed, a floating or an inflation index rate. The interest rate payout can be applied to interest rate swaps and FRA (which both have two associated interest rate payouts), credit default swaps (to represent the fee leg when subject to periodic payments) and equity swaps (to represent the funding leg). The associated rosettaKey denotes the ability to associate a hash value to the InterestRatePayout instantiations for the purpose of model cross-referencing, in support of functionality such as the event effect and the lineage.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class InterestRatePayout extends PayoutBase implements GlobalKey {
	private final BondReference bondReference;
	private final CalculationPeriodDates calculationPeriodDates;
	private final CashflowRepresentation cashflowRepresentation;
	private final CompoundingMethodEnum compoundingMethod;
	private final CrossCurrencyTerms crossCurrencyTerms;
	private final FieldWithMetaDayCountFractionEnum dayCountFraction;
	private final DiscountingMethod discountingMethod;
	private final String fixedAmount;
	private final String floatingAmount;
	private final MetaFields meta;
	private final PayerReceiver payerReceiver;
	private final AdjustableDate paymentDate;
	private final PaymentDates paymentDates;
	private final Boolean paymentDelay;
	private final ContractualQuantity quantity;
	private final RateSpecification rateSpecification;
	private final ResetDates resetDates;
	private final StubPeriod stubPeriod;
	private static InterestRatePayoutMeta metaData = new InterestRatePayoutMeta();
	
	InterestRatePayout(InterestRatePayoutBuilder builder) {
		super(builder);
		this.bondReference = ofNullable(builder.getBondReference()).map(f->f.build()).orElse(null);
		this.calculationPeriodDates = ofNullable(builder.getCalculationPeriodDates()).map(f->f.build()).orElse(null);
		this.cashflowRepresentation = ofNullable(builder.getCashflowRepresentation()).map(f->f.build()).orElse(null);
		this.compoundingMethod = builder.getCompoundingMethod();
		this.crossCurrencyTerms = ofNullable(builder.getCrossCurrencyTerms()).map(f->f.build()).orElse(null);
		this.dayCountFraction = ofNullable(builder.getDayCountFraction()).map(f->f.build()).orElse(null);
		this.discountingMethod = ofNullable(builder.getDiscountingMethod()).map(f->f.build()).orElse(null);
		this.fixedAmount = builder.getFixedAmount();
		this.floatingAmount = builder.getFloatingAmount();
		this.meta = ofNullable(builder.getMeta()).map(f->f.build()).orElse(null);
		this.payerReceiver = ofNullable(builder.getPayerReceiver()).map(f->f.build()).orElse(null);
		this.paymentDate = ofNullable(builder.getPaymentDate()).map(f->f.build()).orElse(null);
		this.paymentDates = ofNullable(builder.getPaymentDates()).map(f->f.build()).orElse(null);
		this.paymentDelay = builder.getPaymentDelay();
		this.quantity = ofNullable(builder.getQuantity()).map(f->f.build()).orElse(null);
		this.rateSpecification = ofNullable(builder.getRateSpecification()).map(f->f.build()).orElse(null);
		this.resetDates = ofNullable(builder.getResetDates()).map(f->f.build()).orElse(null);
		this.stubPeriod = ofNullable(builder.getStubPeriod()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * Reference to a bond underlier to represent an asset swap or Condition Precedent Bond.
	 */
	@RosettaSynonym(value="bondReference", source="FpML_5_10")
	@RosettaSynonym(value="bondReference", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="bondReference", source="DTCC_11_0")
	@RosettaSynonym(value="bondReference", source="DTCC_9_0")
	@RosettaSynonym(value="bondReference", source="CME_ClearedConfirm_1_17")
	public final BondReference getBondReference() {
		return bondReference;
	}
	
	/**
	 * The parameters used to generate the calculation period dates schedule, including the specification of any initial or final stub calculation periods.
	 */
	public final CalculationPeriodDates getCalculationPeriodDates() {
		return calculationPeriodDates;
	}
	
	/**
	 * The cashflow representation of the swap stream.
	 */
	@RosettaSynonym(value="cashflows", source="FpML_5_10")
	@RosettaSynonym(value="cashflows", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="cashflows", source="DTCC_11_0")
	@RosettaSynonym(value="cashflows", source="DTCC_9_0")
	@RosettaSynonym(value="cashflows", source="CME_ClearedConfirm_1_17")
	public final CashflowRepresentation getCashflowRepresentation() {
		return cashflowRepresentation;
	}
	
	/**
	 * If one or more calculation period contributes to a single payment amount this element specifies whether compounding is applicable and, if so, what compounding method is to be used. This element must only be included when more than one calculation period contributes to a single payment amount.
	 */
	@RosettaSynonym(value="compoundingMethod", source="FpML_5_10", path="calculationPeriodAmount.calculation")
	@RosettaSynonym(value="compoundingMethod", source="CME_SubmissionIRS_1_0", path="calculationPeriodAmount.calculation")
	@RosettaSynonym(value="compoundingMethod", source="DTCC_11_0", path="calculationPeriodAmount.calculation")
	@RosettaSynonym(value="compoundingMethod", source="DTCC_9_0", path="calculationPeriodAmount.calculation")
	@RosettaSynonym(value="compoundingMethod", source="CME_ClearedConfirm_1_17", path="calculationPeriodAmount.calculation")
	public final CompoundingMethodEnum getCompoundingMethod() {
		return compoundingMethod;
	}
	
	/**
	 * The specification of the principle exchange and settlement provision terms.
	 */
	public final CrossCurrencyTerms getCrossCurrencyTerms() {
		return crossCurrencyTerms;
	}
	
	/**
	 * The day count fraction. The cardinality has been relaxed when compared with the FpML interest rate swap for the purpose of accommodating standardized credit default swaps which DCF is not explicitly stated as part of the economic terms. The data rule InterestRatePayout_dayCountFraction requires that the DCF be stated for interest rate products.
	 */
	@RosettaSynonym(value="dayCountFraction", source="FpML_5_10", path="calculationPeriodAmount.calculation")
	@RosettaSynonym(value="dayCountFraction", source="CME_SubmissionIRS_1_0", path="calculationPeriodAmount.calculation")
	@RosettaSynonym(value="dayCountFraction", source="DTCC_11_0", path="calculationPeriodAmount.calculation")
	@RosettaSynonym(value="dayCountFraction", source="DTCC_9_0", path="calculationPeriodAmount.calculation")
	@RosettaSynonym(value="dayCountFraction", source="CME_ClearedConfirm_1_17", path="calculationPeriodAmount.calculation")
	@RosettaSynonym(value="dayCountFraction", source="FpML_5_10", path="periodicPayment.fixedAmountCalculation")
	@RosettaSynonym(value="dayCountFraction", source="CME_SubmissionIRS_1_0", path="periodicPayment.fixedAmountCalculation")
	@RosettaSynonym(value="dayCountFraction", source="DTCC_11_0", path="periodicPayment.fixedAmountCalculation")
	@RosettaSynonym(value="dayCountFraction", source="DTCC_9_0", path="periodicPayment.fixedAmountCalculation")
	@RosettaSynonym(value="dayCountFraction", source="CME_ClearedConfirm_1_17", path="periodicPayment.fixedAmountCalculation")
	@RosettaSynonym(value="dayCountFraction", source="FpML_5_10", path="periodicPayment.floatingAmountCalculation")
	@RosettaSynonym(value="dayCountFraction", source="CME_SubmissionIRS_1_0", path="periodicPayment.floatingAmountCalculation")
	@RosettaSynonym(value="dayCountFraction", source="DTCC_11_0", path="periodicPayment.floatingAmountCalculation")
	@RosettaSynonym(value="dayCountFraction", source="DTCC_9_0", path="periodicPayment.floatingAmountCalculation")
	@RosettaSynonym(value="dayCountFraction", source="CME_ClearedConfirm_1_17", path="periodicPayment.floatingAmountCalculation")
	@RosettaSynonym(value="dayCountFraction", source="FpML_5_10")
	@RosettaSynonym(value="dayCountFraction", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="dayCountFraction", source="DTCC_11_0")
	@RosettaSynonym(value="dayCountFraction", source="DTCC_9_0")
	@RosettaSynonym(value="dayCountFraction", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="dayCountFraction", source="FpML_5_10", path="interestCalculation")
	@RosettaSynonym(value="dayCountFraction", source="CME_SubmissionIRS_1_0", path="interestCalculation")
	@RosettaSynonym(value="dayCountFraction", source="DTCC_11_0", path="interestCalculation")
	@RosettaSynonym(value="dayCountFraction", source="DTCC_9_0", path="interestCalculation")
	@RosettaSynonym(value="dayCountFraction", source="CME_ClearedConfirm_1_17", path="interestCalculation")
	public final FieldWithMetaDayCountFractionEnum getDayCountFraction() {
		return dayCountFraction;
	}
	
	/**
	 * The parameters specifying any discounting conventions that may apply. This element must only be included if discounting applies.
	 */
	@RosettaSynonym(value="discounting", source="FpML_5_10", path="calculationPeriodAmount.calculation")
	@RosettaSynonym(value="discounting", source="CME_SubmissionIRS_1_0", path="calculationPeriodAmount.calculation")
	@RosettaSynonym(value="discounting", source="DTCC_11_0", path="calculationPeriodAmount.calculation")
	@RosettaSynonym(value="discounting", source="DTCC_9_0", path="calculationPeriodAmount.calculation")
	@RosettaSynonym(value="discounting", source="CME_ClearedConfirm_1_17", path="calculationPeriodAmount.calculation")
	@RosettaSynonym(value="", source="FpML_5_10")
	@RosettaSynonym(value="", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="", source="DTCC_11_0")
	@RosettaSynonym(value="", source="DTCC_9_0")
	@RosettaSynonym(value="", source="CME_ClearedConfirm_1_17")
	public final DiscountingMethod getDiscountingMethod() {
		return discountingMethod;
	}
	
	/**
	 * Fixed Amount Calculation
	 */
	public final String getFixedAmount() {
		return fixedAmount;
	}
	
	/**
	 * Floating Amount Calculation
	 */
	public final String getFloatingAmount() {
		return floatingAmount;
	}
	
	public final MetaFields getMeta() {
		return meta;
	}
	
	/**
	 * The payer/seller paradigm applies to swap products, whether interest rate swaps or the fee leg of credit default swaps.
	 */
	public final PayerReceiver getPayerReceiver() {
		return payerReceiver;
	}
	
	/**
	 * The payment date, where only one date is specified, as for the FRA product.
	 */
	@RosettaSynonym(value="paymentDate", source="FpML_5_10")
	@RosettaSynonym(value="paymentDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="paymentDate", source="DTCC_11_0")
	@RosettaSynonym(value="paymentDate", source="DTCC_9_0")
	@RosettaSynonym(value="paymentDate", source="CME_ClearedConfirm_1_17")
	public final AdjustableDate getPaymentDate() {
		return paymentDate;
	}
	
	/**
	 * The payment date schedule, as defined by the parameters that are needed to specify it, either in a parametric way or by reference to another schedule of dates (e.g. the reset dates).
	 */
	@RosettaSynonym(value="paymentDates", source="FpML_5_10")
	@RosettaSynonym(value="paymentDates", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="paymentDates", source="DTCC_11_0")
	@RosettaSynonym(value="paymentDates", source="DTCC_9_0")
	@RosettaSynonym(value="paymentDates", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="periodicPayment", source="FpML_5_10")
	@RosettaSynonym(value="periodicPayment", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="periodicPayment", source="DTCC_11_0")
	@RosettaSynonym(value="periodicPayment", source="DTCC_9_0")
	@RosettaSynonym(value="periodicPayment", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="interestLegPaymentDates", source="FpML_5_10", path="interestLegCalculationPeriodDates")
	public final PaymentDates getPaymentDates() {
		return paymentDates;
	}
	
	/**
	 * Applicable to CDS on MBS to specify whether payment delays are applicable to the fixed Amount. RMBS typically have a payment delay of 5 days between the coupon date of the reference obligation and the payment date of the synthetic swap. CMBS do not, on the other hand, with both payment dates being on the 25th of each month.
	 */
	@RosettaSynonym(value="paymentDelay", source="FpML_5_10")
	@RosettaSynonym(value="paymentDelay", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="paymentDelay", source="DTCC_11_0")
	@RosettaSynonym(value="paymentDelay", source="DTCC_9_0")
	@RosettaSynonym(value="paymentDelay", source="CME_ClearedConfirm_1_17")
	public final Boolean getPaymentDelay() {
		return paymentDelay;
	}
	
	/**
	 * The specification of the notional amount that is associated with a contractual product and that is the base for the payout calculation, through the ContractualQuantity class which provides a generic approach that is applicable across all asset classes. The quantity attribute applies to products relating to securities or tangible assets (such as equities or commodities), while the notional amount applies to products pertaining to interest rate, FX or credit products.
	 */
	public final ContractualQuantity getQuantity() {
		return quantity;
	}
	
	/**
	 * The specification of the rate value(s) applicable to the contract using either a floating rate calculation, a single fixed rate, a fixed rate schedule, or an inflation rate calculation.
	 */
	public final RateSpecification getRateSpecification() {
		return rateSpecification;
	}
	
	/**
	 * The reset dates schedule, i.e. the dates on which the new observed index value is applied for each period and the interest rate hence begins to accrue.
	 */
	public final ResetDates getResetDates() {
		return resetDates;
	}
	
	/**
	 * The stub calculation period amount parameters. This element must only be included if there is an initial or final stub calculation period. Even then, it must only be included if either the stub references a different floating rate tenor to the regular calculation periods, or if the stub is calculated as a linear interpolation of two different floating rate tenors, or if a specific stub rate or stub amount has been negotiated.
	 */
	@RosettaSynonym(value="stubCalculationPeriodAmount", source="FpML_5_10")
	@RosettaSynonym(value="stubCalculationPeriodAmount", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="stubCalculationPeriodAmount", source="DTCC_11_0")
	@RosettaSynonym(value="stubCalculationPeriodAmount", source="DTCC_9_0")
	@RosettaSynonym(value="stubCalculationPeriodAmount", source="CME_ClearedConfirm_1_17")
	public final StubPeriod getStubPeriod() {
		return stubPeriod;
	}
	
	
	@Override
	public RosettaMetaData<? extends InterestRatePayout> metaData() {
		return metaData;
	} 
	
	public InterestRatePayoutBuilder toBuilder() {
		InterestRatePayoutBuilder builder = new InterestRatePayoutBuilder();
		ofNullable(getBondReference()).ifPresent(builder::setBondReference);
		ofNullable(getCalculationPeriodDates()).ifPresent(builder::setCalculationPeriodDates);
		ofNullable(getCashflowRepresentation()).ifPresent(builder::setCashflowRepresentation);
		ofNullable(getCompoundingMethod()).ifPresent(builder::setCompoundingMethod);
		ofNullable(getCrossCurrencyTerms()).ifPresent(builder::setCrossCurrencyTerms);
		ofNullable(getDayCountFraction()).ifPresent(builder::setDayCountFraction);
		ofNullable(getDiscountingMethod()).ifPresent(builder::setDiscountingMethod);
		ofNullable(getFixedAmount()).ifPresent(builder::setFixedAmount);
		ofNullable(getFloatingAmount()).ifPresent(builder::setFloatingAmount);
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		ofNullable(getPayerReceiver()).ifPresent(builder::setPayerReceiver);
		ofNullable(getPaymentDate()).ifPresent(builder::setPaymentDate);
		ofNullable(getPaymentDates()).ifPresent(builder::setPaymentDates);
		ofNullable(getPaymentDelay()).ifPresent(builder::setPaymentDelay);
		ofNullable(getQuantity()).ifPresent(builder::setQuantity);
		ofNullable(getRateSpecification()).ifPresent(builder::setRateSpecification);
		ofNullable(getResetDates()).ifPresent(builder::setResetDates);
		ofNullable(getStubPeriod()).ifPresent(builder::setStubPeriod);
		ofNullable(getPayoutQuantity()).ifPresent(builder::setPayoutQuantity);
		return builder;
	}

	public static InterestRatePayoutBuilder builder() {
		return new InterestRatePayoutBuilder();
	}

	public static  class InterestRatePayoutBuilder extends PayoutBaseBuilder implements GlobalKeyBuilder<InterestRatePayoutBuilder> {
	
		protected BondReference.BondReferenceBuilder bondReference;
		protected CalculationPeriodDates.CalculationPeriodDatesBuilder calculationPeriodDates;
		protected CashflowRepresentation.CashflowRepresentationBuilder cashflowRepresentation;
		protected CompoundingMethodEnum compoundingMethod;
		protected CrossCurrencyTerms.CrossCurrencyTermsBuilder crossCurrencyTerms;
		protected FieldWithMetaDayCountFractionEnumBuilder dayCountFraction;
		protected DiscountingMethod.DiscountingMethodBuilder discountingMethod;
		protected String fixedAmount;
		protected String floatingAmount;
		protected MetaFields.MetaFieldsBuilder meta;
		protected PayerReceiver.PayerReceiverBuilder payerReceiver;
		protected AdjustableDate.AdjustableDateBuilder paymentDate;
		protected PaymentDates.PaymentDatesBuilder paymentDates;
		protected Boolean paymentDelay;
		protected ContractualQuantity.ContractualQuantityBuilder quantity;
		protected RateSpecification.RateSpecificationBuilder rateSpecification;
		protected ResetDates.ResetDatesBuilder resetDates;
		protected StubPeriod.StubPeriodBuilder stubPeriod;
	
		public InterestRatePayoutBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends InterestRatePayout> metaData() {
			return metaData;
		} 
	
		
		public BondReference.BondReferenceBuilder getBondReference() {
			return bondReference;
		}
		
		public BondReference.BondReferenceBuilder getOrCreateBondReference() {
			if (bondReference!=null) {
				return bondReference;
			}
			else return bondReference = new BondReference.BondReferenceBuilder();
		}
		
		
		public CalculationPeriodDates.CalculationPeriodDatesBuilder getCalculationPeriodDates() {
			return calculationPeriodDates;
		}
		
		public CalculationPeriodDates.CalculationPeriodDatesBuilder getOrCreateCalculationPeriodDates() {
			if (calculationPeriodDates!=null) {
				return calculationPeriodDates;
			}
			else return calculationPeriodDates = new CalculationPeriodDates.CalculationPeriodDatesBuilder();
		}
		
		
		public CashflowRepresentation.CashflowRepresentationBuilder getCashflowRepresentation() {
			return cashflowRepresentation;
		}
		
		public CashflowRepresentation.CashflowRepresentationBuilder getOrCreateCashflowRepresentation() {
			if (cashflowRepresentation!=null) {
				return cashflowRepresentation;
			}
			else return cashflowRepresentation = new CashflowRepresentation.CashflowRepresentationBuilder();
		}
		
		
		public CompoundingMethodEnum getCompoundingMethod() {
			return compoundingMethod;
		}
		
		
		public CrossCurrencyTerms.CrossCurrencyTermsBuilder getCrossCurrencyTerms() {
			return crossCurrencyTerms;
		}
		
		public CrossCurrencyTerms.CrossCurrencyTermsBuilder getOrCreateCrossCurrencyTerms() {
			if (crossCurrencyTerms!=null) {
				return crossCurrencyTerms;
			}
			else return crossCurrencyTerms = new CrossCurrencyTerms.CrossCurrencyTermsBuilder();
		}
		
		
		public FieldWithMetaDayCountFractionEnumBuilder getDayCountFraction() {
			return dayCountFraction;
		}
		
		public FieldWithMetaDayCountFractionEnumBuilder getOrCreateDayCountFraction() {
			if (dayCountFraction!=null) {
				return dayCountFraction;
			}
			else return dayCountFraction = new FieldWithMetaDayCountFractionEnumBuilder();
		}
		
		
		public DiscountingMethod.DiscountingMethodBuilder getDiscountingMethod() {
			return discountingMethod;
		}
		
		public DiscountingMethod.DiscountingMethodBuilder getOrCreateDiscountingMethod() {
			if (discountingMethod!=null) {
				return discountingMethod;
			}
			else return discountingMethod = new DiscountingMethod.DiscountingMethodBuilder();
		}
		
		
		public String getFixedAmount() {
			return fixedAmount;
		}
		
		
		public String getFloatingAmount() {
			return floatingAmount;
		}
		
		
		public MetaFields.MetaFieldsBuilder getMeta() {
			return meta;
		}
		
		public MetaFields.MetaFieldsBuilder getOrCreateMeta() {
			if (meta!=null) {
				return meta;
			}
			else return meta = new MetaFields.MetaFieldsBuilder();
		}
		
		
		public PayerReceiver.PayerReceiverBuilder getPayerReceiver() {
			return payerReceiver;
		}
		
		public PayerReceiver.PayerReceiverBuilder getOrCreatePayerReceiver() {
			if (payerReceiver!=null) {
				return payerReceiver;
			}
			else return payerReceiver = new PayerReceiver.PayerReceiverBuilder();
		}
		
		
		public AdjustableDate.AdjustableDateBuilder getPaymentDate() {
			return paymentDate;
		}
		
		public AdjustableDate.AdjustableDateBuilder getOrCreatePaymentDate() {
			if (paymentDate!=null) {
				return paymentDate;
			}
			else return paymentDate = new AdjustableDate.AdjustableDateBuilder();
		}
		
		
		public PaymentDates.PaymentDatesBuilder getPaymentDates() {
			return paymentDates;
		}
		
		public PaymentDates.PaymentDatesBuilder getOrCreatePaymentDates() {
			if (paymentDates!=null) {
				return paymentDates;
			}
			else return paymentDates = new PaymentDates.PaymentDatesBuilder();
		}
		
		
		public Boolean getPaymentDelay() {
			return paymentDelay;
		}
		
		
		public ContractualQuantity.ContractualQuantityBuilder getQuantity() {
			return quantity;
		}
		
		public ContractualQuantity.ContractualQuantityBuilder getOrCreateQuantity() {
			if (quantity!=null) {
				return quantity;
			}
			else return quantity = new ContractualQuantity.ContractualQuantityBuilder();
		}
		
		
		public RateSpecification.RateSpecificationBuilder getRateSpecification() {
			return rateSpecification;
		}
		
		public RateSpecification.RateSpecificationBuilder getOrCreateRateSpecification() {
			if (rateSpecification!=null) {
				return rateSpecification;
			}
			else return rateSpecification = new RateSpecification.RateSpecificationBuilder();
		}
		
		
		public ResetDates.ResetDatesBuilder getResetDates() {
			return resetDates;
		}
		
		public ResetDates.ResetDatesBuilder getOrCreateResetDates() {
			if (resetDates!=null) {
				return resetDates;
			}
			else return resetDates = new ResetDates.ResetDatesBuilder();
		}
		
		
		public StubPeriod.StubPeriodBuilder getStubPeriod() {
			return stubPeriod;
		}
		
		public StubPeriod.StubPeriodBuilder getOrCreateStubPeriod() {
			if (stubPeriod!=null) {
				return stubPeriod;
			}
			else return stubPeriod = new StubPeriod.StubPeriodBuilder();
		}
		
	
		public InterestRatePayoutBuilder setBondReference(BondReference bondReference) {
			this.bondReference = bondReference.toBuilder();
			return this;
		}
		public InterestRatePayoutBuilder setBondReferenceBuilder(BondReference.BondReferenceBuilder bondReference) {
			this.bondReference = bondReference;
			return this;
		}
		public InterestRatePayoutBuilder setCalculationPeriodDates(CalculationPeriodDates calculationPeriodDates) {
			this.calculationPeriodDates = calculationPeriodDates.toBuilder();
			return this;
		}
		public InterestRatePayoutBuilder setCalculationPeriodDatesBuilder(CalculationPeriodDates.CalculationPeriodDatesBuilder calculationPeriodDates) {
			this.calculationPeriodDates = calculationPeriodDates;
			return this;
		}
		public InterestRatePayoutBuilder setCashflowRepresentation(CashflowRepresentation cashflowRepresentation) {
			this.cashflowRepresentation = cashflowRepresentation.toBuilder();
			return this;
		}
		public InterestRatePayoutBuilder setCashflowRepresentationBuilder(CashflowRepresentation.CashflowRepresentationBuilder cashflowRepresentation) {
			this.cashflowRepresentation = cashflowRepresentation;
			return this;
		}
		public InterestRatePayoutBuilder setCompoundingMethod(CompoundingMethodEnum compoundingMethod) {
			this.compoundingMethod = compoundingMethod;
			return this;
		}
		public InterestRatePayoutBuilder setCrossCurrencyTerms(CrossCurrencyTerms crossCurrencyTerms) {
			this.crossCurrencyTerms = crossCurrencyTerms.toBuilder();
			return this;
		}
		public InterestRatePayoutBuilder setCrossCurrencyTermsBuilder(CrossCurrencyTerms.CrossCurrencyTermsBuilder crossCurrencyTerms) {
			this.crossCurrencyTerms = crossCurrencyTerms;
			return this;
		}
		public InterestRatePayoutBuilder setDayCountFraction(FieldWithMetaDayCountFractionEnum dayCountFraction) {
			this.dayCountFraction = dayCountFraction.toBuilder();
			return this;
		}
		public InterestRatePayoutBuilder setDayCountFractionRef(DayCountFractionEnum dayCountFraction) {
			return setDayCountFraction(FieldWithMetaDayCountFractionEnum.builder().setValue(dayCountFraction).build());
		}
		public InterestRatePayoutBuilder setDiscountingMethod(DiscountingMethod discountingMethod) {
			this.discountingMethod = discountingMethod.toBuilder();
			return this;
		}
		public InterestRatePayoutBuilder setDiscountingMethodBuilder(DiscountingMethod.DiscountingMethodBuilder discountingMethod) {
			this.discountingMethod = discountingMethod;
			return this;
		}
		public InterestRatePayoutBuilder setFixedAmount(String fixedAmount) {
			this.fixedAmount = fixedAmount;
			return this;
		}
		public InterestRatePayoutBuilder setFloatingAmount(String floatingAmount) {
			this.floatingAmount = floatingAmount;
			return this;
		}
		public InterestRatePayoutBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		public InterestRatePayoutBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		public InterestRatePayoutBuilder setPayerReceiver(PayerReceiver payerReceiver) {
			this.payerReceiver = payerReceiver.toBuilder();
			return this;
		}
		public InterestRatePayoutBuilder setPayerReceiverBuilder(PayerReceiver.PayerReceiverBuilder payerReceiver) {
			this.payerReceiver = payerReceiver;
			return this;
		}
		public InterestRatePayoutBuilder setPaymentDate(AdjustableDate paymentDate) {
			this.paymentDate = paymentDate.toBuilder();
			return this;
		}
		public InterestRatePayoutBuilder setPaymentDateBuilder(AdjustableDate.AdjustableDateBuilder paymentDate) {
			this.paymentDate = paymentDate;
			return this;
		}
		public InterestRatePayoutBuilder setPaymentDates(PaymentDates paymentDates) {
			this.paymentDates = paymentDates.toBuilder();
			return this;
		}
		public InterestRatePayoutBuilder setPaymentDatesBuilder(PaymentDates.PaymentDatesBuilder paymentDates) {
			this.paymentDates = paymentDates;
			return this;
		}
		public InterestRatePayoutBuilder setPaymentDelay(Boolean paymentDelay) {
			this.paymentDelay = paymentDelay;
			return this;
		}
		public InterestRatePayoutBuilder setQuantity(ContractualQuantity quantity) {
			this.quantity = quantity.toBuilder();
			return this;
		}
		public InterestRatePayoutBuilder setQuantityBuilder(ContractualQuantity.ContractualQuantityBuilder quantity) {
			this.quantity = quantity;
			return this;
		}
		public InterestRatePayoutBuilder setRateSpecification(RateSpecification rateSpecification) {
			this.rateSpecification = rateSpecification.toBuilder();
			return this;
		}
		public InterestRatePayoutBuilder setRateSpecificationBuilder(RateSpecification.RateSpecificationBuilder rateSpecification) {
			this.rateSpecification = rateSpecification;
			return this;
		}
		public InterestRatePayoutBuilder setResetDates(ResetDates resetDates) {
			this.resetDates = resetDates.toBuilder();
			return this;
		}
		public InterestRatePayoutBuilder setResetDatesBuilder(ResetDates.ResetDatesBuilder resetDates) {
			this.resetDates = resetDates;
			return this;
		}
		public InterestRatePayoutBuilder setStubPeriod(StubPeriod stubPeriod) {
			this.stubPeriod = stubPeriod.toBuilder();
			return this;
		}
		public InterestRatePayoutBuilder setStubPeriodBuilder(StubPeriod.StubPeriodBuilder stubPeriod) {
			this.stubPeriod = stubPeriod;
			return this;
		}
		@Override public InterestRatePayoutBuilder setPayoutQuantity(ResolvablePayoutQuantity payoutQuantity) {
			this.payoutQuantity = payoutQuantity.toBuilder();
			return this;
		}
		@Override public InterestRatePayoutBuilder setPayoutQuantityBuilder(ResolvablePayoutQuantity.ResolvablePayoutQuantityBuilder payoutQuantity) {
			this.payoutQuantity = payoutQuantity;
			return this;
		}
		
		public InterestRatePayout build() {
			return new InterestRatePayout(this);
		}
	
		@Override
		public InterestRatePayoutBuilder prune() {
			super.prune();
			if (bondReference!=null && !bondReference.prune().hasData()) bondReference = null;
			if (calculationPeriodDates!=null && !calculationPeriodDates.prune().hasData()) calculationPeriodDates = null;
			if (cashflowRepresentation!=null && !cashflowRepresentation.prune().hasData()) cashflowRepresentation = null;
			if (crossCurrencyTerms!=null && !crossCurrencyTerms.prune().hasData()) crossCurrencyTerms = null;
			if (dayCountFraction!=null && !dayCountFraction.prune().hasData()) dayCountFraction = null;
			if (discountingMethod!=null && !discountingMethod.prune().hasData()) discountingMethod = null;
			if (meta!=null && !meta.prune().hasData()) meta = null;
			if (payerReceiver!=null && !payerReceiver.prune().hasData()) payerReceiver = null;
			if (paymentDate!=null && !paymentDate.prune().hasData()) paymentDate = null;
			if (paymentDates!=null && !paymentDates.prune().hasData()) paymentDates = null;
			if (quantity!=null && !quantity.prune().hasData()) quantity = null;
			if (rateSpecification!=null && !rateSpecification.prune().hasData()) rateSpecification = null;
			if (resetDates!=null && !resetDates.prune().hasData()) resetDates = null;
			if (stubPeriod!=null && !stubPeriod.prune().hasData()) stubPeriod = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (super.hasData()) return true;
			if (getBondReference()!=null && getBondReference().hasData()) return true;
			if (getCalculationPeriodDates()!=null && getCalculationPeriodDates().hasData()) return true;
			if (getCashflowRepresentation()!=null && getCashflowRepresentation().hasData()) return true;
			if (getCompoundingMethod()!=null) return true;
			if (getCrossCurrencyTerms()!=null && getCrossCurrencyTerms().hasData()) return true;
			if (getDayCountFraction()!=null) return true;
			if (getDiscountingMethod()!=null && getDiscountingMethod().hasData()) return true;
			if (getFixedAmount()!=null) return true;
			if (getFloatingAmount()!=null) return true;
			if (getMeta()!=null && getMeta().hasData()) return true;
			if (getPayerReceiver()!=null && getPayerReceiver().hasData()) return true;
			if (getPaymentDate()!=null && getPaymentDate().hasData()) return true;
			if (getPaymentDates()!=null && getPaymentDates().hasData()) return true;
			if (getPaymentDelay()!=null) return true;
			if (getQuantity()!=null && getQuantity().hasData()) return true;
			if (getRateSpecification()!=null && getRateSpecification().hasData()) return true;
			if (getResetDates()!=null && getResetDates().hasData()) return true;
			if (getStubPeriod()!=null && getStubPeriod().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
			super.process(path, processor);
		
			processor.processBasic(path.newSubPath("compoundingMethod"), CompoundingMethodEnum.class, compoundingMethod, this);
			processor.processBasic(path.newSubPath("fixedAmount"), String.class, fixedAmount, this);
			processor.processBasic(path.newSubPath("floatingAmount"), String.class, floatingAmount, this);
			processor.processBasic(path.newSubPath("paymentDelay"), Boolean.class, paymentDelay, this);
			
			processRosetta(path.newSubPath("bondReference"), processor, BondReference.class, bondReference);
			processRosetta(path.newSubPath("calculationPeriodDates"), processor, CalculationPeriodDates.class, calculationPeriodDates);
			processRosetta(path.newSubPath("cashflowRepresentation"), processor, CashflowRepresentation.class, cashflowRepresentation);
			processRosetta(path.newSubPath("crossCurrencyTerms"), processor, CrossCurrencyTerms.class, crossCurrencyTerms);
			processRosetta(path.newSubPath("dayCountFraction"), processor, FieldWithMetaDayCountFractionEnum.class, dayCountFraction);
			processRosetta(path.newSubPath("discountingMethod"), processor, DiscountingMethod.class, discountingMethod);
			processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
			processRosetta(path.newSubPath("payerReceiver"), processor, PayerReceiver.class, payerReceiver);
			processRosetta(path.newSubPath("paymentDate"), processor, AdjustableDate.class, paymentDate);
			processRosetta(path.newSubPath("paymentDates"), processor, PaymentDates.class, paymentDates);
			processRosetta(path.newSubPath("quantity"), processor, ContractualQuantity.class, quantity);
			processRosetta(path.newSubPath("rateSpecification"), processor, RateSpecification.class, rateSpecification);
			processRosetta(path.newSubPath("resetDates"), processor, ResetDates.class, resetDates);
			processRosetta(path.newSubPath("stubPeriod"), processor, StubPeriod.class, stubPeriod);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
			if (!super.equals(o)) return false;
		
			InterestRatePayoutBuilder _that = (InterestRatePayoutBuilder) o;
		
			if (bondReference != null ? !bondReference.equals(_that.bondReference) : _that.bondReference != null) return false;
			if (calculationPeriodDates != null ? !calculationPeriodDates.equals(_that.calculationPeriodDates) : _that.calculationPeriodDates != null) return false;
			if (cashflowRepresentation != null ? !cashflowRepresentation.equals(_that.cashflowRepresentation) : _that.cashflowRepresentation != null) return false;
			if (compoundingMethod != null ? !compoundingMethod.equals(_that.compoundingMethod) : _that.compoundingMethod != null) return false;
			if (crossCurrencyTerms != null ? !crossCurrencyTerms.equals(_that.crossCurrencyTerms) : _that.crossCurrencyTerms != null) return false;
			if (dayCountFraction != null ? !dayCountFraction.equals(_that.dayCountFraction) : _that.dayCountFraction != null) return false;
			if (discountingMethod != null ? !discountingMethod.equals(_that.discountingMethod) : _that.discountingMethod != null) return false;
			if (fixedAmount != null ? !fixedAmount.equals(_that.fixedAmount) : _that.fixedAmount != null) return false;
			if (floatingAmount != null ? !floatingAmount.equals(_that.floatingAmount) : _that.floatingAmount != null) return false;
			if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
			if (payerReceiver != null ? !payerReceiver.equals(_that.payerReceiver) : _that.payerReceiver != null) return false;
			if (paymentDate != null ? !paymentDate.equals(_that.paymentDate) : _that.paymentDate != null) return false;
			if (paymentDates != null ? !paymentDates.equals(_that.paymentDates) : _that.paymentDates != null) return false;
			if (paymentDelay != null ? !paymentDelay.equals(_that.paymentDelay) : _that.paymentDelay != null) return false;
			if (quantity != null ? !quantity.equals(_that.quantity) : _that.quantity != null) return false;
			if (rateSpecification != null ? !rateSpecification.equals(_that.rateSpecification) : _that.rateSpecification != null) return false;
			if (resetDates != null ? !resetDates.equals(_that.resetDates) : _that.resetDates != null) return false;
			if (stubPeriod != null ? !stubPeriod.equals(_that.stubPeriod) : _that.stubPeriod != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = super.hashCode();
			_result = 31 * _result + (bondReference != null ? bondReference.hashCode() : 0);
			_result = 31 * _result + (calculationPeriodDates != null ? calculationPeriodDates.hashCode() : 0);
			_result = 31 * _result + (cashflowRepresentation != null ? cashflowRepresentation.hashCode() : 0);
			_result = 31 * _result + (compoundingMethod != null ? compoundingMethod.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (crossCurrencyTerms != null ? crossCurrencyTerms.hashCode() : 0);
			_result = 31 * _result + (dayCountFraction != null ? dayCountFraction.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (discountingMethod != null ? discountingMethod.hashCode() : 0);
			_result = 31 * _result + (fixedAmount != null ? fixedAmount.hashCode() : 0);
			_result = 31 * _result + (floatingAmount != null ? floatingAmount.hashCode() : 0);
			_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
			_result = 31 * _result + (payerReceiver != null ? payerReceiver.hashCode() : 0);
			_result = 31 * _result + (paymentDate != null ? paymentDate.hashCode() : 0);
			_result = 31 * _result + (paymentDates != null ? paymentDates.hashCode() : 0);
			_result = 31 * _result + (paymentDelay != null ? paymentDelay.hashCode() : 0);
			_result = 31 * _result + (quantity != null ? quantity.hashCode() : 0);
			_result = 31 * _result + (rateSpecification != null ? rateSpecification.hashCode() : 0);
			_result = 31 * _result + (resetDates != null ? resetDates.hashCode() : 0);
			_result = 31 * _result + (stubPeriod != null ? stubPeriod.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "InterestRatePayoutBuilder {" +
				"bondReference=" + this.bondReference + ", " +
				"calculationPeriodDates=" + this.calculationPeriodDates + ", " +
				"cashflowRepresentation=" + this.cashflowRepresentation + ", " +
				"compoundingMethod=" + this.compoundingMethod + ", " +
				"crossCurrencyTerms=" + this.crossCurrencyTerms + ", " +
				"dayCountFraction=" + this.dayCountFraction + ", " +
				"discountingMethod=" + this.discountingMethod + ", " +
				"fixedAmount=" + this.fixedAmount + ", " +
				"floatingAmount=" + this.floatingAmount + ", " +
				"meta=" + this.meta + ", " +
				"payerReceiver=" + this.payerReceiver + ", " +
				"paymentDate=" + this.paymentDate + ", " +
				"paymentDates=" + this.paymentDates + ", " +
				"paymentDelay=" + this.paymentDelay + ", " +
				"quantity=" + this.quantity + ", " +
				"rateSpecification=" + this.rateSpecification + ", " +
				"resetDates=" + this.resetDates + ", " +
				"stubPeriod=" + this.stubPeriod +
			'}' + " " + super.toString();
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		super.process(path, processor);
		
		processor.processBasic(path.newSubPath("compoundingMethod"), CompoundingMethodEnum.class, compoundingMethod, this);
		processor.processBasic(path.newSubPath("fixedAmount"), String.class, fixedAmount, this);
		processor.processBasic(path.newSubPath("floatingAmount"), String.class, floatingAmount, this);
		processor.processBasic(path.newSubPath("paymentDelay"), Boolean.class, paymentDelay, this);
		
		processRosetta(path.newSubPath("bondReference"), processor, BondReference.class, bondReference);
		processRosetta(path.newSubPath("calculationPeriodDates"), processor, CalculationPeriodDates.class, calculationPeriodDates);
		processRosetta(path.newSubPath("cashflowRepresentation"), processor, CashflowRepresentation.class, cashflowRepresentation);
		processRosetta(path.newSubPath("crossCurrencyTerms"), processor, CrossCurrencyTerms.class, crossCurrencyTerms);
		processRosetta(path.newSubPath("dayCountFraction"), processor, FieldWithMetaDayCountFractionEnum.class, dayCountFraction);
		processRosetta(path.newSubPath("discountingMethod"), processor, DiscountingMethod.class, discountingMethod);
		processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
		processRosetta(path.newSubPath("payerReceiver"), processor, PayerReceiver.class, payerReceiver);
		processRosetta(path.newSubPath("paymentDate"), processor, AdjustableDate.class, paymentDate);
		processRosetta(path.newSubPath("paymentDates"), processor, PaymentDates.class, paymentDates);
		processRosetta(path.newSubPath("quantity"), processor, ContractualQuantity.class, quantity);
		processRosetta(path.newSubPath("rateSpecification"), processor, RateSpecification.class, rateSpecification);
		processRosetta(path.newSubPath("resetDates"), processor, ResetDates.class, resetDates);
		processRosetta(path.newSubPath("stubPeriod"), processor, StubPeriod.class, stubPeriod);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
		if (!super.equals(o)) return false;
	
		InterestRatePayout _that = (InterestRatePayout) o;
	
		if (bondReference != null ? !bondReference.equals(_that.bondReference) : _that.bondReference != null) return false;
		if (calculationPeriodDates != null ? !calculationPeriodDates.equals(_that.calculationPeriodDates) : _that.calculationPeriodDates != null) return false;
		if (cashflowRepresentation != null ? !cashflowRepresentation.equals(_that.cashflowRepresentation) : _that.cashflowRepresentation != null) return false;
		if (compoundingMethod != null ? !compoundingMethod.equals(_that.compoundingMethod) : _that.compoundingMethod != null) return false;
		if (crossCurrencyTerms != null ? !crossCurrencyTerms.equals(_that.crossCurrencyTerms) : _that.crossCurrencyTerms != null) return false;
		if (dayCountFraction != null ? !dayCountFraction.equals(_that.dayCountFraction) : _that.dayCountFraction != null) return false;
		if (discountingMethod != null ? !discountingMethod.equals(_that.discountingMethod) : _that.discountingMethod != null) return false;
		if (fixedAmount != null ? !fixedAmount.equals(_that.fixedAmount) : _that.fixedAmount != null) return false;
		if (floatingAmount != null ? !floatingAmount.equals(_that.floatingAmount) : _that.floatingAmount != null) return false;
		if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
		if (payerReceiver != null ? !payerReceiver.equals(_that.payerReceiver) : _that.payerReceiver != null) return false;
		if (paymentDate != null ? !paymentDate.equals(_that.paymentDate) : _that.paymentDate != null) return false;
		if (paymentDates != null ? !paymentDates.equals(_that.paymentDates) : _that.paymentDates != null) return false;
		if (paymentDelay != null ? !paymentDelay.equals(_that.paymentDelay) : _that.paymentDelay != null) return false;
		if (quantity != null ? !quantity.equals(_that.quantity) : _that.quantity != null) return false;
		if (rateSpecification != null ? !rateSpecification.equals(_that.rateSpecification) : _that.rateSpecification != null) return false;
		if (resetDates != null ? !resetDates.equals(_that.resetDates) : _that.resetDates != null) return false;
		if (stubPeriod != null ? !stubPeriod.equals(_that.stubPeriod) : _that.stubPeriod != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = super.hashCode();
		_result = 31 * _result + (bondReference != null ? bondReference.hashCode() : 0);
		_result = 31 * _result + (calculationPeriodDates != null ? calculationPeriodDates.hashCode() : 0);
		_result = 31 * _result + (cashflowRepresentation != null ? cashflowRepresentation.hashCode() : 0);
		_result = 31 * _result + (compoundingMethod != null ? compoundingMethod.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (crossCurrencyTerms != null ? crossCurrencyTerms.hashCode() : 0);
		_result = 31 * _result + (dayCountFraction != null ? dayCountFraction.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (discountingMethod != null ? discountingMethod.hashCode() : 0);
		_result = 31 * _result + (fixedAmount != null ? fixedAmount.hashCode() : 0);
		_result = 31 * _result + (floatingAmount != null ? floatingAmount.hashCode() : 0);
		_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
		_result = 31 * _result + (payerReceiver != null ? payerReceiver.hashCode() : 0);
		_result = 31 * _result + (paymentDate != null ? paymentDate.hashCode() : 0);
		_result = 31 * _result + (paymentDates != null ? paymentDates.hashCode() : 0);
		_result = 31 * _result + (paymentDelay != null ? paymentDelay.hashCode() : 0);
		_result = 31 * _result + (quantity != null ? quantity.hashCode() : 0);
		_result = 31 * _result + (rateSpecification != null ? rateSpecification.hashCode() : 0);
		_result = 31 * _result + (resetDates != null ? resetDates.hashCode() : 0);
		_result = 31 * _result + (stubPeriod != null ? stubPeriod.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "InterestRatePayout {" +
			"bondReference=" + this.bondReference + ", " +
			"calculationPeriodDates=" + this.calculationPeriodDates + ", " +
			"cashflowRepresentation=" + this.cashflowRepresentation + ", " +
			"compoundingMethod=" + this.compoundingMethod + ", " +
			"crossCurrencyTerms=" + this.crossCurrencyTerms + ", " +
			"dayCountFraction=" + this.dayCountFraction + ", " +
			"discountingMethod=" + this.discountingMethod + ", " +
			"fixedAmount=" + this.fixedAmount + ", " +
			"floatingAmount=" + this.floatingAmount + ", " +
			"meta=" + this.meta + ", " +
			"payerReceiver=" + this.payerReceiver + ", " +
			"paymentDate=" + this.paymentDate + ", " +
			"paymentDates=" + this.paymentDates + ", " +
			"paymentDelay=" + this.paymentDelay + ", " +
			"quantity=" + this.quantity + ", " +
			"rateSpecification=" + this.rateSpecification + ", " +
			"resetDates=" + this.resetDates + ", " +
			"stubPeriod=" + this.stubPeriod +
		'}' + " " + super.toString();
	}
}
