package org.isda.cdm;

import com.rosetta.model.lib.records.Date;
import org.isda.cdm.AdjustableOrRelativeDate;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.LastRegularPaymentDateMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import com.rosetta.model.lib.records.Date;
		import static java.util.Optional.ofNullable;

/**
 * A class which purpose is to provide the ability into the seemingly incompatible representations of the interest rate and equity last payment date.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class LastRegularPaymentDate extends RosettaModelObject {
	private final AdjustableOrRelativeDate finalPaymentDate;
	private final Date lastRegularPaymentDate;
	private static LastRegularPaymentDateMeta metaData = new LastRegularPaymentDateMeta();
	
	LastRegularPaymentDate(LastRegularPaymentDateBuilder builder) {
		this.finalPaymentDate = ofNullable(builder.getFinalPaymentDate()).map(f->f.build()).orElse(null);
		this.lastRegularPaymentDate = builder.getLastRegularPaymentDate();
	}
	
	/**
	 * The last payment when specified as an adjustable or relative date, as in the FpML total return construct.
	 */
	@RosettaSynonym(value="paymentDateFinal", source="FpML_5_10")
	@RosettaSynonym(value="paymentDateFinal", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="paymentDateFinal", source="DTCC_11_0")
	@RosettaSynonym(value="paymentDateFinal", source="DTCC_9_0")
	@RosettaSynonym(value="paymentDateFinal", source="CME_ClearedConfirm_1_17")
	public final AdjustableOrRelativeDate getFinalPaymentDate() {
		return finalPaymentDate;
	}
	
	/**
	 * The last regular payment date when specified as a date, as in the FpML interest rate construct. FpML specifies that this date may be subject to adjustment in accordance with any business day convention specified in the paymentDatesAdjustments attribute.
	 */
	@RosettaSynonym(value="lastRegularPaymentDate", source="FpML_5_10")
	@RosettaSynonym(value="lastRegularPaymentDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="lastRegularPaymentDate", source="DTCC_11_0")
	@RosettaSynonym(value="lastRegularPaymentDate", source="DTCC_9_0")
	@RosettaSynonym(value="lastRegularPaymentDate", source="CME_ClearedConfirm_1_17")
	public final Date getLastRegularPaymentDate() {
		return lastRegularPaymentDate;
	}
	
	
	@Override
	public RosettaMetaData<? extends LastRegularPaymentDate> metaData() {
		return metaData;
	} 
	
	public LastRegularPaymentDateBuilder toBuilder() {
		LastRegularPaymentDateBuilder builder = new LastRegularPaymentDateBuilder();
		ofNullable(getFinalPaymentDate()).ifPresent(builder::setFinalPaymentDate);
		ofNullable(getLastRegularPaymentDate()).ifPresent(builder::setLastRegularPaymentDate);
		return builder;
	}

	public static LastRegularPaymentDateBuilder builder() {
		return new LastRegularPaymentDateBuilder();
	}

	public static  class LastRegularPaymentDateBuilder extends RosettaModelObjectBuilder {
	
		protected AdjustableOrRelativeDate.AdjustableOrRelativeDateBuilder finalPaymentDate;
		protected Date lastRegularPaymentDate;
	
		public LastRegularPaymentDateBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends LastRegularPaymentDate> metaData() {
			return metaData;
		} 
	
		
		public AdjustableOrRelativeDate.AdjustableOrRelativeDateBuilder getFinalPaymentDate() {
			return finalPaymentDate;
		}
		
		public AdjustableOrRelativeDate.AdjustableOrRelativeDateBuilder getOrCreateFinalPaymentDate() {
			if (finalPaymentDate!=null) {
				return finalPaymentDate;
			}
			else return finalPaymentDate = new AdjustableOrRelativeDate.AdjustableOrRelativeDateBuilder();
		}
		
		
		public Date getLastRegularPaymentDate() {
			return lastRegularPaymentDate;
		}
		
	
		public LastRegularPaymentDateBuilder setFinalPaymentDate(AdjustableOrRelativeDate finalPaymentDate) {
			this.finalPaymentDate = finalPaymentDate.toBuilder();
			return this;
		}
		public LastRegularPaymentDateBuilder setFinalPaymentDateBuilder(AdjustableOrRelativeDate.AdjustableOrRelativeDateBuilder finalPaymentDate) {
			this.finalPaymentDate = finalPaymentDate;
			return this;
		}
		public LastRegularPaymentDateBuilder setLastRegularPaymentDate(Date lastRegularPaymentDate) {
			this.lastRegularPaymentDate = lastRegularPaymentDate;
			return this;
		}
		
		public LastRegularPaymentDate build() {
			return new LastRegularPaymentDate(this);
		}
	
		@Override
		public LastRegularPaymentDateBuilder prune() {
			if (finalPaymentDate!=null && !finalPaymentDate.prune().hasData()) finalPaymentDate = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getFinalPaymentDate()!=null && getFinalPaymentDate().hasData()) return true;
			if (getLastRegularPaymentDate()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("lastRegularPaymentDate"), Date.class, lastRegularPaymentDate, this);
			
			processRosetta(path.newSubPath("finalPaymentDate"), processor, AdjustableOrRelativeDate.class, finalPaymentDate);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			LastRegularPaymentDateBuilder _that = (LastRegularPaymentDateBuilder) o;
		
			if (finalPaymentDate != null ? !finalPaymentDate.equals(_that.finalPaymentDate) : _that.finalPaymentDate != null) return false;
			if (lastRegularPaymentDate != null ? !lastRegularPaymentDate.equals(_that.lastRegularPaymentDate) : _that.lastRegularPaymentDate != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (finalPaymentDate != null ? finalPaymentDate.hashCode() : 0);
			_result = 31 * _result + (lastRegularPaymentDate != null ? lastRegularPaymentDate.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "LastRegularPaymentDateBuilder {" +
				"finalPaymentDate=" + this.finalPaymentDate + ", " +
				"lastRegularPaymentDate=" + this.lastRegularPaymentDate +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("lastRegularPaymentDate"), Date.class, lastRegularPaymentDate, this);
		
		processRosetta(path.newSubPath("finalPaymentDate"), processor, AdjustableOrRelativeDate.class, finalPaymentDate);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		LastRegularPaymentDate _that = (LastRegularPaymentDate) o;
	
		if (finalPaymentDate != null ? !finalPaymentDate.equals(_that.finalPaymentDate) : _that.finalPaymentDate != null) return false;
		if (lastRegularPaymentDate != null ? !lastRegularPaymentDate.equals(_that.lastRegularPaymentDate) : _that.lastRegularPaymentDate != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (finalPaymentDate != null ? finalPaymentDate.hashCode() : 0);
		_result = 31 * _result + (lastRegularPaymentDate != null ? lastRegularPaymentDate.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "LastRegularPaymentDate {" +
			"finalPaymentDate=" + this.finalPaymentDate + ", " +
			"lastRegularPaymentDate=" + this.lastRegularPaymentDate +
		'}';
	}
}
