package org.isda.cdm;

import com.rosetta.model.lib.records.Date;
import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.Identifier;
import org.isda.cdm.LegalAgreementType;
import org.isda.cdm.Lineage;
import org.isda.cdm.Party;
import org.isda.cdm.PartyRole;
import org.isda.cdm.UmbrellaAgreement;
import org.isda.cdm.metafields.ReferenceWithMetaParty;
import org.isda.cdm.metafields.ReferenceWithMetaParty.ReferenceWithMetaPartyBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.LegalAgreementBaseMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import com.rosetta.model.lib.records.Date;
		import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;

/**
 * A class describing the legal agreement baseline information, other than the specialized elections: type of legal agreement, agreement date and effective date, parties to the agreement, ... The purpose of this abstract class is to provide the ability to associate a &#39;one of&#39; feature to the LegalAgreement class as a way to enforce a systematic choice between its attributes.
 * 
 * @version 2.5.4
 */
@RosettaClass
public abstract class LegalAgreementBase extends RosettaModelObject {
	private final Date agreementDate;
	private final LegalAgreementType agreementType;
	private final List<ReferenceWithMetaParty> contractualParty;
	private final Date effectiveDate;
	private final List<Identifier> identifier;
	private final Lineage lineage;
	private final List<PartyRole> otherParty;
	private final List<Party> partyInformation;
	private final UmbrellaAgreement umbrellaAgreement;
	private static LegalAgreementBaseMeta metaData = new LegalAgreementBaseMeta();
	
	LegalAgreementBase(LegalAgreementBaseBuilder builder) {
		this.agreementDate = builder.getAgreementDate();
		this.agreementType = ofNullable(builder.getAgreementType()).map(f->f.build()).orElse(null);
		this.contractualParty = ofNullable(builder.getContractualParty()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.effectiveDate = builder.getEffectiveDate();
		this.identifier = ofNullable(builder.getIdentifier()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.lineage = ofNullable(builder.getLineage()).map(f->f.build()).orElse(null);
		this.otherParty = ofNullable(builder.getOtherParty()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.partyInformation = ofNullable(builder.getPartyInformation()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.umbrellaAgreement = ofNullable(builder.getUmbrellaAgreement()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The date on which the legal agreement has been agreed between the parties. This corresponds to the Date of Deed in an English Law document.
	 */
	@RosettaSynonym(value="agreementDate", source="FpML_5_10", path="legalDocumentHeader")
	@RosettaSynonym(value="executionDate", source="ISDA_Create_1_0", path="answers.execution_date")
	public final Date getAgreementDate() {
		return agreementDate;
	}
	
	/**
	 * The type of legal agreement, identified via a set of distinct attributes: name, publisher, governing law and version, e.g. ISDA 2013 Standard Credit Support Annex English Law.
	 */
	@RosettaSynonym(value="documentType", source="FpML_5_10")
	public final LegalAgreementType getAgreementType() {
		return agreementType;
	}
	
	/**
	 * The two contractual parties to the legal agreement, which reference information is positioned as part of the partyInformation attribute.
	 */
	@RosettaSynonym(value="party", source="FpML_5_10")
	public final List<ReferenceWithMetaParty> getContractualParty() {
		return contractualParty;
	}
	
	/**
	 * The date on which the agreement is effective, if different from the agreement date. It is expected that it will most often correspond to the agreement date, although there could be situations where the parties will explicitly agree on a distinct effective date.
	 */
	@RosettaSynonym(value="effectiveDate", source="FpML_5_10", path="legalDocumentHeader")
	public final Date getEffectiveDate() {
		return effectiveDate;
	}
	
	/**
	 * The legal agreement identifier. Several identifiers can be specified.
	 */
	@RosettaSynonym(value="partyDocumentIdentifier", source="FpML_5_10", path="legalDocumentHeader")
	@RosettaSynonym(value="document", source="ISDA_Create_1_0")
	public final List<Identifier> getIdentifier() {
		return identifier;
	}
	
	/**
	 * The lineage into prior versions of this legal agreement or into other legal agreements that might be referenced by it.
	 */
	public final Lineage getLineage() {
		return lineage;
	}
	
	/**
	 * The role(s) that other party(ies) may have in relation to the legal agreement, further to the contractual parties.
	 */
	public final List<PartyRole> getOtherParty() {
		return otherParty;
	}
	
	/**
	 * The information about the parties involved with the legal agreements, which consists of both the contractual parties and the parties that might be involved in relation to some other roles (e.g. custody agent).
	 */
	public final List<Party> getPartyInformation() {
		return partyInformation;
	}
	
	/**
	 * The parties to the agreement that might be associated to it as part of what is termed an umbrella agreement.
	 */
	@RosettaSynonym(value="umbrella_agreement_and_principal_identification", source="ISDA_Create_1_0")
	public final UmbrellaAgreement getUmbrellaAgreement() {
		return umbrellaAgreement;
	}
	
	
	@Override
	public RosettaMetaData<? extends LegalAgreementBase> metaData() {
		return metaData;
	} 
	
	public abstract LegalAgreementBaseBuilder toBuilder();


	public static abstract
	 class LegalAgreementBaseBuilder extends RosettaModelObjectBuilder {
	
		protected Date agreementDate;
		protected LegalAgreementType.LegalAgreementTypeBuilder agreementType;
		protected List<ReferenceWithMetaPartyBuilder> contractualParty;
		protected Date effectiveDate;
		protected List<Identifier.IdentifierBuilder> identifier;
		protected Lineage.LineageBuilder lineage;
		protected List<PartyRole.PartyRoleBuilder> otherParty;
		protected List<Party.PartyBuilder> partyInformation;
		protected UmbrellaAgreement.UmbrellaAgreementBuilder umbrellaAgreement;
	
		public LegalAgreementBaseBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends LegalAgreementBase> metaData() {
			return metaData;
		} 
	
		
		public Date getAgreementDate() {
			return agreementDate;
		}
		
		
		public LegalAgreementType.LegalAgreementTypeBuilder getAgreementType() {
			return agreementType;
		}
		
		public LegalAgreementType.LegalAgreementTypeBuilder getOrCreateAgreementType() {
			if (agreementType!=null) {
				return agreementType;
			}
			else return agreementType = new LegalAgreementType.LegalAgreementTypeBuilder();
		}
		
		
		public List<ReferenceWithMetaPartyBuilder> getContractualParty() {
			return contractualParty;
		}
		
		public ReferenceWithMetaPartyBuilder getOrCreateContractualParty(int index) {
			if (contractualParty==null) {
				this.contractualParty = new ArrayList<>();
			}
			return getIndex(contractualParty, index, ()->new ReferenceWithMetaPartyBuilder());
		}
		
		
		public Date getEffectiveDate() {
			return effectiveDate;
		}
		
		
		public List<Identifier.IdentifierBuilder> getIdentifier() {
			return identifier;
		}
		
		public Identifier.IdentifierBuilder getOrCreateIdentifier(int index) {
			if (identifier==null) {
				this.identifier = new ArrayList<>();
			}
			return getIndex(identifier, index, ()->new Identifier.IdentifierBuilder());
		}
		
		
		public Lineage.LineageBuilder getLineage() {
			return lineage;
		}
		
		public Lineage.LineageBuilder getOrCreateLineage() {
			if (lineage!=null) {
				return lineage;
			}
			else return lineage = new Lineage.LineageBuilder();
		}
		
		
		public List<PartyRole.PartyRoleBuilder> getOtherParty() {
			return otherParty;
		}
		
		public PartyRole.PartyRoleBuilder getOrCreateOtherParty(int index) {
			if (otherParty==null) {
				this.otherParty = new ArrayList<>();
			}
			return getIndex(otherParty, index, ()->new PartyRole.PartyRoleBuilder());
		}
		
		
		public List<Party.PartyBuilder> getPartyInformation() {
			return partyInformation;
		}
		
		public Party.PartyBuilder getOrCreatePartyInformation(int index) {
			if (partyInformation==null) {
				this.partyInformation = new ArrayList<>();
			}
			return getIndex(partyInformation, index, ()->new Party.PartyBuilder());
		}
		
		
		public UmbrellaAgreement.UmbrellaAgreementBuilder getUmbrellaAgreement() {
			return umbrellaAgreement;
		}
		
		public UmbrellaAgreement.UmbrellaAgreementBuilder getOrCreateUmbrellaAgreement() {
			if (umbrellaAgreement!=null) {
				return umbrellaAgreement;
			}
			else return umbrellaAgreement = new UmbrellaAgreement.UmbrellaAgreementBuilder();
		}
		
	
		public LegalAgreementBaseBuilder setAgreementDate(Date agreementDate) {
			this.agreementDate = agreementDate;
			return this;
		}
		public LegalAgreementBaseBuilder setAgreementType(LegalAgreementType agreementType) {
			this.agreementType = agreementType.toBuilder();
			return this;
		}
		public LegalAgreementBaseBuilder setAgreementTypeBuilder(LegalAgreementType.LegalAgreementTypeBuilder agreementType) {
			this.agreementType = agreementType;
			return this;
		}
		public LegalAgreementBaseBuilder addContractualParty(ReferenceWithMetaParty contractualParty) {
			if(this.contractualParty == null){
				this.contractualParty = new ArrayList<>();
			}
			this.contractualParty.add(contractualParty.toBuilder());
			return this;
		}
		public LegalAgreementBaseBuilder addContractualParty(ReferenceWithMetaParty contractualParty, int _idx) {
			if(this.contractualParty == null){
				this.contractualParty = new ArrayList<>();
			}
			getIndex(this.contractualParty, _idx, () -> contractualParty.toBuilder());
			this.contractualParty.set(_idx, contractualParty.toBuilder());
			return this;
		}
		public LegalAgreementBaseBuilder addContractualParty(List<ReferenceWithMetaParty> contractualPartys) {
			if(this.contractualParty == null){
				this.contractualParty = new ArrayList<>();
			}
			for (ReferenceWithMetaParty toAdd : contractualPartys) {
				this.contractualParty.add(toAdd.toBuilder());
			}
			return this;
		}
		public LegalAgreementBaseBuilder addContractualPartyBuilder(ReferenceWithMetaPartyBuilder contractualParty) {
			if(this.contractualParty == null){
				this.contractualParty = new ArrayList<>();
			}
			this.contractualParty.add(contractualParty);
			return this;
		}
		
		public LegalAgreementBaseBuilder addContractualPartyRef(Party.PartyBuilder contractualParty) {
			return addContractualParty(ReferenceWithMetaParty.builder().setValueBuilder(contractualParty).build());
		}
		
		public LegalAgreementBaseBuilder addContractualPartyRef(Party.PartyBuilder contractualParty, int _idx) {
			return addContractualParty(ReferenceWithMetaParty.builder().setValueBuilder(contractualParty).build(), _idx);
		}
		
		public LegalAgreementBaseBuilder addContractualPartyRef(List<Party> contractualPartys) {
			for (Party toAdd : contractualPartys) {
				addContractualPartyRef(toAdd);
			}
			return this;
		}
		
		public LegalAgreementBaseBuilder addContractualPartyRef(Party contractualParty) {
			return addContractualPartyRef(contractualParty.toBuilder());
		}
		
		public LegalAgreementBaseBuilder clearContractualParty() {
			this.contractualParty = null;
			return this;
		}
		public LegalAgreementBaseBuilder setEffectiveDate(Date effectiveDate) {
			this.effectiveDate = effectiveDate;
			return this;
		}
		public LegalAgreementBaseBuilder addIdentifier(Identifier identifier) {
			if(this.identifier == null){
				this.identifier = new ArrayList<>();
			}
			this.identifier.add(identifier.toBuilder());
			return this;
		}
		public LegalAgreementBaseBuilder addIdentifier(Identifier identifier, int _idx) {
			if(this.identifier == null){
				this.identifier = new ArrayList<>();
			}
			getIndex(this.identifier, _idx, () -> identifier.toBuilder());
			this.identifier.set(_idx, identifier.toBuilder());
			return this;
		}
		public LegalAgreementBaseBuilder addIdentifier(List<Identifier> identifiers) {
			if(this.identifier == null){
				this.identifier = new ArrayList<>();
			}
			for (Identifier toAdd : identifiers) {
				this.identifier.add(toAdd.toBuilder());
			}
			return this;
		}
		public LegalAgreementBaseBuilder addIdentifierBuilder(Identifier.IdentifierBuilder identifier) {
			if(this.identifier == null){
				this.identifier = new ArrayList<>();
			}
			this.identifier.add(identifier);
			return this;
		}
		
		public LegalAgreementBaseBuilder clearIdentifier() {
			this.identifier = null;
			return this;
		}
		public LegalAgreementBaseBuilder setLineage(Lineage lineage) {
			this.lineage = lineage.toBuilder();
			return this;
		}
		public LegalAgreementBaseBuilder setLineageBuilder(Lineage.LineageBuilder lineage) {
			this.lineage = lineage;
			return this;
		}
		public LegalAgreementBaseBuilder addOtherParty(PartyRole otherParty) {
			if(this.otherParty == null){
				this.otherParty = new ArrayList<>();
			}
			this.otherParty.add(otherParty.toBuilder());
			return this;
		}
		public LegalAgreementBaseBuilder addOtherParty(PartyRole otherParty, int _idx) {
			if(this.otherParty == null){
				this.otherParty = new ArrayList<>();
			}
			getIndex(this.otherParty, _idx, () -> otherParty.toBuilder());
			this.otherParty.set(_idx, otherParty.toBuilder());
			return this;
		}
		public LegalAgreementBaseBuilder addOtherParty(List<PartyRole> otherPartys) {
			if(this.otherParty == null){
				this.otherParty = new ArrayList<>();
			}
			for (PartyRole toAdd : otherPartys) {
				this.otherParty.add(toAdd.toBuilder());
			}
			return this;
		}
		public LegalAgreementBaseBuilder addOtherPartyBuilder(PartyRole.PartyRoleBuilder otherParty) {
			if(this.otherParty == null){
				this.otherParty = new ArrayList<>();
			}
			this.otherParty.add(otherParty);
			return this;
		}
		
		public LegalAgreementBaseBuilder clearOtherParty() {
			this.otherParty = null;
			return this;
		}
		public LegalAgreementBaseBuilder addPartyInformation(Party partyInformation) {
			if(this.partyInformation == null){
				this.partyInformation = new ArrayList<>();
			}
			this.partyInformation.add(partyInformation.toBuilder());
			return this;
		}
		public LegalAgreementBaseBuilder addPartyInformation(Party partyInformation, int _idx) {
			if(this.partyInformation == null){
				this.partyInformation = new ArrayList<>();
			}
			getIndex(this.partyInformation, _idx, () -> partyInformation.toBuilder());
			this.partyInformation.set(_idx, partyInformation.toBuilder());
			return this;
		}
		public LegalAgreementBaseBuilder addPartyInformation(List<Party> partyInformations) {
			if(this.partyInformation == null){
				this.partyInformation = new ArrayList<>();
			}
			for (Party toAdd : partyInformations) {
				this.partyInformation.add(toAdd.toBuilder());
			}
			return this;
		}
		public LegalAgreementBaseBuilder addPartyInformationBuilder(Party.PartyBuilder partyInformation) {
			if(this.partyInformation == null){
				this.partyInformation = new ArrayList<>();
			}
			this.partyInformation.add(partyInformation);
			return this;
		}
		
		public LegalAgreementBaseBuilder clearPartyInformation() {
			this.partyInformation = null;
			return this;
		}
		public LegalAgreementBaseBuilder setUmbrellaAgreement(UmbrellaAgreement umbrellaAgreement) {
			this.umbrellaAgreement = umbrellaAgreement.toBuilder();
			return this;
		}
		public LegalAgreementBaseBuilder setUmbrellaAgreementBuilder(UmbrellaAgreement.UmbrellaAgreementBuilder umbrellaAgreement) {
			this.umbrellaAgreement = umbrellaAgreement;
			return this;
		}
		
		public abstract LegalAgreementBase build();
	
		@Override
		public LegalAgreementBaseBuilder prune() {
			if (agreementType!=null && !agreementType.prune().hasData()) agreementType = null;
			if (contractualParty!=null) contractualParty = contractualParty.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (identifier!=null) identifier = identifier.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (lineage!=null && !lineage.prune().hasData()) lineage = null;
			if (otherParty!=null) otherParty = otherParty.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (partyInformation!=null) partyInformation = partyInformation.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (umbrellaAgreement!=null && !umbrellaAgreement.prune().hasData()) umbrellaAgreement = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getAgreementDate()!=null) return true;
			if (getAgreementType()!=null && getAgreementType().hasData()) return true;
			if (getContractualParty()!=null && getContractualParty().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getEffectiveDate()!=null) return true;
			if (getIdentifier()!=null && getIdentifier().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getLineage()!=null && getLineage().hasData()) return true;
			if (getOtherParty()!=null && getOtherParty().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getPartyInformation()!=null && getPartyInformation().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getUmbrellaAgreement()!=null && getUmbrellaAgreement().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("agreementDate"), Date.class, agreementDate, this);
			processor.processBasic(path.newSubPath("effectiveDate"), Date.class, effectiveDate, this);
			
			processRosetta(path.newSubPath("agreementType"), processor, LegalAgreementType.class, agreementType);
			processRosetta(path.newSubPath("contractualParty"), processor, ReferenceWithMetaParty.class, contractualParty);
			processRosetta(path.newSubPath("identifier"), processor, Identifier.class, identifier);
			processRosetta(path.newSubPath("lineage"), processor, Lineage.class, lineage);
			processRosetta(path.newSubPath("otherParty"), processor, PartyRole.class, otherParty);
			processRosetta(path.newSubPath("partyInformation"), processor, Party.class, partyInformation);
			processRosetta(path.newSubPath("umbrellaAgreement"), processor, UmbrellaAgreement.class, umbrellaAgreement);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			LegalAgreementBaseBuilder _that = (LegalAgreementBaseBuilder) o;
		
			if (agreementDate != null ? !agreementDate.equals(_that.agreementDate) : _that.agreementDate != null) return false;
			if (agreementType != null ? !agreementType.equals(_that.agreementType) : _that.agreementType != null) return false;
			if (!ListEquals.listEquals(contractualParty, _that.contractualParty)) return false;
			if (effectiveDate != null ? !effectiveDate.equals(_that.effectiveDate) : _that.effectiveDate != null) return false;
			if (!ListEquals.listEquals(identifier, _that.identifier)) return false;
			if (lineage != null ? !lineage.equals(_that.lineage) : _that.lineage != null) return false;
			if (!ListEquals.listEquals(otherParty, _that.otherParty)) return false;
			if (!ListEquals.listEquals(partyInformation, _that.partyInformation)) return false;
			if (umbrellaAgreement != null ? !umbrellaAgreement.equals(_that.umbrellaAgreement) : _that.umbrellaAgreement != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (agreementDate != null ? agreementDate.hashCode() : 0);
			_result = 31 * _result + (agreementType != null ? agreementType.hashCode() : 0);
			_result = 31 * _result + (contractualParty != null ? contractualParty.hashCode() : 0);
			_result = 31 * _result + (effectiveDate != null ? effectiveDate.hashCode() : 0);
			_result = 31 * _result + (identifier != null ? identifier.hashCode() : 0);
			_result = 31 * _result + (lineage != null ? lineage.hashCode() : 0);
			_result = 31 * _result + (otherParty != null ? otherParty.hashCode() : 0);
			_result = 31 * _result + (partyInformation != null ? partyInformation.hashCode() : 0);
			_result = 31 * _result + (umbrellaAgreement != null ? umbrellaAgreement.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "LegalAgreementBaseBuilder {" +
				"agreementDate=" + this.agreementDate + ", " +
				"agreementType=" + this.agreementType + ", " +
				"contractualParty=" + this.contractualParty + ", " +
				"effectiveDate=" + this.effectiveDate + ", " +
				"identifier=" + this.identifier + ", " +
				"lineage=" + this.lineage + ", " +
				"otherParty=" + this.otherParty + ", " +
				"partyInformation=" + this.partyInformation + ", " +
				"umbrellaAgreement=" + this.umbrellaAgreement +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("agreementDate"), Date.class, agreementDate, this);
		processor.processBasic(path.newSubPath("effectiveDate"), Date.class, effectiveDate, this);
		
		processRosetta(path.newSubPath("agreementType"), processor, LegalAgreementType.class, agreementType);
		processRosetta(path.newSubPath("contractualParty"), processor, ReferenceWithMetaParty.class, contractualParty);
		processRosetta(path.newSubPath("identifier"), processor, Identifier.class, identifier);
		processRosetta(path.newSubPath("lineage"), processor, Lineage.class, lineage);
		processRosetta(path.newSubPath("otherParty"), processor, PartyRole.class, otherParty);
		processRosetta(path.newSubPath("partyInformation"), processor, Party.class, partyInformation);
		processRosetta(path.newSubPath("umbrellaAgreement"), processor, UmbrellaAgreement.class, umbrellaAgreement);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		LegalAgreementBase _that = (LegalAgreementBase) o;
	
		if (agreementDate != null ? !agreementDate.equals(_that.agreementDate) : _that.agreementDate != null) return false;
		if (agreementType != null ? !agreementType.equals(_that.agreementType) : _that.agreementType != null) return false;
		if (!ListEquals.listEquals(contractualParty, _that.contractualParty)) return false;
		if (effectiveDate != null ? !effectiveDate.equals(_that.effectiveDate) : _that.effectiveDate != null) return false;
		if (!ListEquals.listEquals(identifier, _that.identifier)) return false;
		if (lineage != null ? !lineage.equals(_that.lineage) : _that.lineage != null) return false;
		if (!ListEquals.listEquals(otherParty, _that.otherParty)) return false;
		if (!ListEquals.listEquals(partyInformation, _that.partyInformation)) return false;
		if (umbrellaAgreement != null ? !umbrellaAgreement.equals(_that.umbrellaAgreement) : _that.umbrellaAgreement != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (agreementDate != null ? agreementDate.hashCode() : 0);
		_result = 31 * _result + (agreementType != null ? agreementType.hashCode() : 0);
		_result = 31 * _result + (contractualParty != null ? contractualParty.hashCode() : 0);
		_result = 31 * _result + (effectiveDate != null ? effectiveDate.hashCode() : 0);
		_result = 31 * _result + (identifier != null ? identifier.hashCode() : 0);
		_result = 31 * _result + (lineage != null ? lineage.hashCode() : 0);
		_result = 31 * _result + (otherParty != null ? otherParty.hashCode() : 0);
		_result = 31 * _result + (partyInformation != null ? partyInformation.hashCode() : 0);
		_result = 31 * _result + (umbrellaAgreement != null ? umbrellaAgreement.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "LegalAgreementBase {" +
			"agreementDate=" + this.agreementDate + ", " +
			"agreementType=" + this.agreementType + ", " +
			"contractualParty=" + this.contractualParty + ", " +
			"effectiveDate=" + this.effectiveDate + ", " +
			"identifier=" + this.identifier + ", " +
			"lineage=" + this.lineage + ", " +
			"otherParty=" + this.otherParty + ", " +
			"partyInformation=" + this.partyInformation + ", " +
			"umbrellaAgreement=" + this.umbrellaAgreement +
		'}';
	}
}
