package org.isda.cdm;

import java.lang.String;
import org.isda.cdm.GoverningLawEnum;
import org.isda.cdm.LegalAgreementNameEnum;
import org.isda.cdm.LegalAgreementPublisherEnum;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.LegalAgreementTypeMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class to specify the type of legal agreement, which is extended by each legal agreement instance, such as the ISDA 2016 CSA for Initial Margin.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="LegalDocumentType", source="FpML_5_10")
public class LegalAgreementType extends RosettaModelObject {
	private final GoverningLawEnum governingLaw;
	private final LegalAgreementNameEnum name;
	private final LegalAgreementPublisherEnum publisher;
	private final String vintage;
	private static LegalAgreementTypeMeta metaData = new LegalAgreementTypeMeta();
	
	LegalAgreementType(LegalAgreementTypeBuilder builder) {
		this.governingLaw = builder.getGoverningLaw();
		this.name = builder.getName();
		this.publisher = builder.getPublisher();
		this.vintage = builder.getVintage();
	}
	
	/**
	 * The law governing the legal agreement, e.g. English Law, New York Law or Japanese Law.
	 */
	@RosettaSynonym(value="style", source="FpML_5_10")
	@RosettaSynonym(value="governingLaw", source="AcadiaSoft_AM_1_0")
	@RosettaSynonym(value="governingLaw", source="ISDA_Create_1_0", path="document")
	public final GoverningLawEnum getGoverningLaw() {
		return governingLaw;
	}
	
	/**
	 * The legal agreement name, e.g. Credit Support Annex for Variation Margin.
	 */
	@RosettaSynonym(value="name", source="FpML_5_10")
	@RosettaSynonym(value="documentName", source="AcadiaSoft_AM_1_0")
	@RosettaSynonym(value="documentType", source="ISDA_Create_1_0", path="document")
	public final LegalAgreementNameEnum getName() {
		return name;
	}
	
	/**
	 * The legal agreement publisher, e.g. ISDA.
	 */
	@RosettaSynonym(value="publisher", source="FpML_5_10")
	@RosettaSynonym(value="publisher", source="AcadiaSoft_AM_1_0")
	@RosettaSynonym(value="publisher", source="ISDA_Create_1_0", path="document")
	public final LegalAgreementPublisherEnum getPublisher() {
		return publisher;
	}
	
	/**
	 * In the case where successive definitions of the legal agreement have been developed, the vintage identification. This is typically (but not necessarily) done by referencing the year, e.g. 2013 in the case of the ISDA 2013 Standard Credit Support Annex.
	 */
	@RosettaSynonym(value="version", source="FpML_5_10")
	@RosettaSynonym(value="version", source="FpML_5_10")
	@RosettaSynonym(value="csaVersion", source="AcadiaSoft_AM_1_0")
	@RosettaSynonym(value="year", source="ISDA_Create_1_0", path="document")
	public final String getVintage() {
		return vintage;
	}
	
	
	@Override
	public RosettaMetaData<? extends LegalAgreementType> metaData() {
		return metaData;
	} 
	
	public LegalAgreementTypeBuilder toBuilder() {
		LegalAgreementTypeBuilder builder = new LegalAgreementTypeBuilder();
		ofNullable(getGoverningLaw()).ifPresent(builder::setGoverningLaw);
		ofNullable(getName()).ifPresent(builder::setName);
		ofNullable(getPublisher()).ifPresent(builder::setPublisher);
		ofNullable(getVintage()).ifPresent(builder::setVintage);
		return builder;
	}

	public static LegalAgreementTypeBuilder builder() {
		return new LegalAgreementTypeBuilder();
	}

	public static  class LegalAgreementTypeBuilder extends RosettaModelObjectBuilder {
	
		protected GoverningLawEnum governingLaw;
		protected LegalAgreementNameEnum name;
		protected LegalAgreementPublisherEnum publisher;
		protected String vintage;
	
		public LegalAgreementTypeBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends LegalAgreementType> metaData() {
			return metaData;
		} 
	
		
		public GoverningLawEnum getGoverningLaw() {
			return governingLaw;
		}
		
		
		public LegalAgreementNameEnum getName() {
			return name;
		}
		
		
		public LegalAgreementPublisherEnum getPublisher() {
			return publisher;
		}
		
		
		public String getVintage() {
			return vintage;
		}
		
	
		public LegalAgreementTypeBuilder setGoverningLaw(GoverningLawEnum governingLaw) {
			this.governingLaw = governingLaw;
			return this;
		}
		public LegalAgreementTypeBuilder setName(LegalAgreementNameEnum name) {
			this.name = name;
			return this;
		}
		public LegalAgreementTypeBuilder setPublisher(LegalAgreementPublisherEnum publisher) {
			this.publisher = publisher;
			return this;
		}
		public LegalAgreementTypeBuilder setVintage(String vintage) {
			this.vintage = vintage;
			return this;
		}
		
		public LegalAgreementType build() {
			return new LegalAgreementType(this);
		}
	
		@Override
		public LegalAgreementTypeBuilder prune() {
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getGoverningLaw()!=null) return true;
			if (getName()!=null) return true;
			if (getPublisher()!=null) return true;
			if (getVintage()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("governingLaw"), GoverningLawEnum.class, governingLaw, this);
			processor.processBasic(path.newSubPath("name"), LegalAgreementNameEnum.class, name, this);
			processor.processBasic(path.newSubPath("publisher"), LegalAgreementPublisherEnum.class, publisher, this);
			processor.processBasic(path.newSubPath("vintage"), String.class, vintage, this);
			
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			LegalAgreementTypeBuilder _that = (LegalAgreementTypeBuilder) o;
		
			if (governingLaw != null ? !governingLaw.equals(_that.governingLaw) : _that.governingLaw != null) return false;
			if (name != null ? !name.equals(_that.name) : _that.name != null) return false;
			if (publisher != null ? !publisher.equals(_that.publisher) : _that.publisher != null) return false;
			if (vintage != null ? !vintage.equals(_that.vintage) : _that.vintage != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (governingLaw != null ? governingLaw.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (name != null ? name.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (publisher != null ? publisher.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (vintage != null ? vintage.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "LegalAgreementTypeBuilder {" +
				"governingLaw=" + this.governingLaw + ", " +
				"name=" + this.name + ", " +
				"publisher=" + this.publisher + ", " +
				"vintage=" + this.vintage +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("governingLaw"), GoverningLawEnum.class, governingLaw, this);
		processor.processBasic(path.newSubPath("name"), LegalAgreementNameEnum.class, name, this);
		processor.processBasic(path.newSubPath("publisher"), LegalAgreementPublisherEnum.class, publisher, this);
		processor.processBasic(path.newSubPath("vintage"), String.class, vintage, this);
		
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		LegalAgreementType _that = (LegalAgreementType) o;
	
		if (governingLaw != null ? !governingLaw.equals(_that.governingLaw) : _that.governingLaw != null) return false;
		if (name != null ? !name.equals(_that.name) : _that.name != null) return false;
		if (publisher != null ? !publisher.equals(_that.publisher) : _that.publisher != null) return false;
		if (vintage != null ? !vintage.equals(_that.vintage) : _that.vintage != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (governingLaw != null ? governingLaw.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (name != null ? name.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (publisher != null ? publisher.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (vintage != null ? vintage.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "LegalAgreementType {" +
			"governingLaw=" + this.governingLaw + ", " +
			"name=" + this.name + ", " +
			"publisher=" + this.publisher + ", " +
			"vintage=" + this.vintage +
		'}';
	}
}
