package org.isda.cdm;

import java.math.BigDecimal;
import org.isda.cdm.metafields.FieldWithMetaCreditLimitTypeEnum;
import org.isda.cdm.metafields.FieldWithMetaLimitLevelEnum;
import org.isda.cdm.metafields.FieldWithMetaLimitLevelEnum.FieldWithMetaLimitLevelEnumBuilder;
import org.isda.cdm.metafields.FieldWithMetaString;


import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.LimitApplicableExtendedMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.math.BigDecimal;
		import static java.util.Optional.ofNullable;

/**
 * A class to represent the CDM attributes that are not part of the FpML standard. Once broader usage is confirmed, it is expected that those two classes can be collapsed.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="CreditLimitInformation", source="CME_ClearedConfirm_1_17")
public class LimitApplicableExtended extends LimitApplicable {
	private final BigDecimal limitAmount;
	private final BigDecimal limitImpactDueToTrade;
	private final FieldWithMetaLimitLevelEnum limitLevel;
	private static LimitApplicableExtendedMeta metaData = new LimitApplicableExtendedMeta();
	
	LimitApplicableExtended(LimitApplicableExtendedBuilder builder) {
		super(builder);
		this.limitAmount = builder.getLimitAmount();
		this.limitImpactDueToTrade = builder.getLimitImpactDueToTrade();
		this.limitLevel = ofNullable(builder.getLimitLevel()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The total limit available for the limit level and limit type. While the attribute is of type integer in the CME schema, it has been specified to be of type number in the CDM to take into consideration java size limits as well as for consistency purposes with the way most monetary amounts are expressed.
	 */
	@RosettaSynonym(value="limitAmount", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="limitAmount", source="Rosetta_Workbench")
	public final BigDecimal getLimitAmount() {
		return limitAmount;
	}
	
	/**
	 * The limit utilized by this specific trade. While the attribute is of type integer in the CME schema, it has been specified to be of type number in the CDM to take into consideration java size limits as well as for consistency purposes with the way most monetary amounts are expressed.
	 */
	@RosettaSynonym(value="limitImpactDueToTrade", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="limitImpactDueToTrade", source="Rosetta_Workbench")
	public final BigDecimal getLimitImpactDueToTrade() {
		return limitImpactDueToTrade;
	}
	
	/**
	 * The level at which the limit is set: customer business, proprietary business or account level. This attribute is specified as a string as part of the CME clearing confirmation specification.
	 */
	@RosettaSynonym(value="level", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="level", source="Rosetta_Workbench")
	public final FieldWithMetaLimitLevelEnum getLimitLevel() {
		return limitLevel;
	}
	
	
	@Override
	public RosettaMetaData<? extends LimitApplicableExtended> metaData() {
		return metaData;
	} 
	
	public LimitApplicableExtendedBuilder toBuilder() {
		LimitApplicableExtendedBuilder builder = new LimitApplicableExtendedBuilder();
		ofNullable(getLimitAmount()).ifPresent(builder::setLimitAmount);
		ofNullable(getLimitImpactDueToTrade()).ifPresent(builder::setLimitImpactDueToTrade);
		ofNullable(getLimitLevel()).ifPresent(builder::setLimitLevel);
		ofNullable(getAmountRemaining()).ifPresent(builder::setAmountRemaining);
		ofNullable(getAmountUtilized()).ifPresent(builder::setAmountUtilized);
		ofNullable(getClipSize()).ifPresent(builder::setClipSize);
		ofNullable(getCurrency()).ifPresent(builder::setCurrency);
		ofNullable(getLimitType()).ifPresent(builder::setLimitType);
		ofNullable(getUtilization()).ifPresent(builder::setUtilization);
		ofNullable(getVelocity()).ifPresent(builder::setVelocity);
		return builder;
	}

	public static LimitApplicableExtendedBuilder builder() {
		return new LimitApplicableExtendedBuilder();
	}

	public static  class LimitApplicableExtendedBuilder extends LimitApplicableBuilder {
	
		protected BigDecimal limitAmount;
		protected BigDecimal limitImpactDueToTrade;
		protected FieldWithMetaLimitLevelEnumBuilder limitLevel;
	
		public LimitApplicableExtendedBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends LimitApplicableExtended> metaData() {
			return metaData;
		} 
	
		
		public BigDecimal getLimitAmount() {
			return limitAmount;
		}
		
		
		public BigDecimal getLimitImpactDueToTrade() {
			return limitImpactDueToTrade;
		}
		
		
		public FieldWithMetaLimitLevelEnumBuilder getLimitLevel() {
			return limitLevel;
		}
		
		public FieldWithMetaLimitLevelEnumBuilder getOrCreateLimitLevel() {
			if (limitLevel!=null) {
				return limitLevel;
			}
			else return limitLevel = new FieldWithMetaLimitLevelEnumBuilder();
		}
		
	
		public LimitApplicableExtendedBuilder setLimitAmount(BigDecimal limitAmount) {
			this.limitAmount = limitAmount;
			return this;
		}
		public LimitApplicableExtendedBuilder setLimitImpactDueToTrade(BigDecimal limitImpactDueToTrade) {
			this.limitImpactDueToTrade = limitImpactDueToTrade;
			return this;
		}
		public LimitApplicableExtendedBuilder setLimitLevel(FieldWithMetaLimitLevelEnum limitLevel) {
			this.limitLevel = limitLevel.toBuilder();
			return this;
		}
		public LimitApplicableExtendedBuilder setLimitLevelRef(LimitLevelEnum limitLevel) {
			return setLimitLevel(FieldWithMetaLimitLevelEnum.builder().setValue(limitLevel).build());
		}
		@Override public LimitApplicableExtendedBuilder setAmountRemaining(BigDecimal amountRemaining) {
			this.amountRemaining = amountRemaining;
			return this;
		}
		@Override public LimitApplicableExtendedBuilder setAmountUtilized(BigDecimal amountUtilized) {
			this.amountUtilized = amountUtilized;
			return this;
		}
		@Override public LimitApplicableExtendedBuilder setClipSize(Integer clipSize) {
			this.clipSize = clipSize;
			return this;
		}
		@Override public LimitApplicableExtendedBuilder setCurrency(FieldWithMetaString currency) {
			this.currency = currency.toBuilder();
			return this;
		}
		@Override public LimitApplicableExtendedBuilder setCurrencyRef(String currency) {
			return setCurrency(FieldWithMetaString.builder().setValue(currency).build());
		}
		@Override public LimitApplicableExtendedBuilder setLimitType(FieldWithMetaCreditLimitTypeEnum limitType) {
			this.limitType = limitType.toBuilder();
			return this;
		}
		@Override public LimitApplicableExtendedBuilder setLimitTypeRef(CreditLimitTypeEnum limitType) {
			return setLimitType(FieldWithMetaCreditLimitTypeEnum.builder().setValue(limitType).build());
		}
		@Override public LimitApplicableExtendedBuilder setUtilization(CreditLimitUtilisation utilization) {
			this.utilization = utilization.toBuilder();
			return this;
		}
		@Override public LimitApplicableExtendedBuilder setUtilizationBuilder(CreditLimitUtilisation.CreditLimitUtilisationBuilder utilization) {
			this.utilization = utilization;
			return this;
		}
		@Override public LimitApplicableExtendedBuilder setVelocity(Velocity velocity) {
			this.velocity = velocity.toBuilder();
			return this;
		}
		@Override public LimitApplicableExtendedBuilder setVelocityBuilder(Velocity.VelocityBuilder velocity) {
			this.velocity = velocity;
			return this;
		}
		
		public LimitApplicableExtended build() {
			return new LimitApplicableExtended(this);
		}
	
		@Override
		public LimitApplicableExtendedBuilder prune() {
			super.prune();
			if (limitLevel!=null && !limitLevel.prune().hasData()) limitLevel = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (super.hasData()) return true;
			if (getLimitAmount()!=null) return true;
			if (getLimitImpactDueToTrade()!=null) return true;
			if (getLimitLevel()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
			super.process(path, processor);
		
			processor.processBasic(path.newSubPath("limitAmount"), BigDecimal.class, limitAmount, this);
			processor.processBasic(path.newSubPath("limitImpactDueToTrade"), BigDecimal.class, limitImpactDueToTrade, this);
			
			processRosetta(path.newSubPath("limitLevel"), processor, FieldWithMetaLimitLevelEnum.class, limitLevel);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
			if (!super.equals(o)) return false;
		
			LimitApplicableExtendedBuilder _that = (LimitApplicableExtendedBuilder) o;
		
			if (limitAmount != null ? !limitAmount.equals(_that.limitAmount) : _that.limitAmount != null) return false;
			if (limitImpactDueToTrade != null ? !limitImpactDueToTrade.equals(_that.limitImpactDueToTrade) : _that.limitImpactDueToTrade != null) return false;
			if (limitLevel != null ? !limitLevel.equals(_that.limitLevel) : _that.limitLevel != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = super.hashCode();
			_result = 31 * _result + (limitAmount != null ? limitAmount.hashCode() : 0);
			_result = 31 * _result + (limitImpactDueToTrade != null ? limitImpactDueToTrade.hashCode() : 0);
			_result = 31 * _result + (limitLevel != null ? limitLevel.getClass().getName().hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "LimitApplicableExtendedBuilder {" +
				"limitAmount=" + this.limitAmount + ", " +
				"limitImpactDueToTrade=" + this.limitImpactDueToTrade + ", " +
				"limitLevel=" + this.limitLevel +
			'}' + " " + super.toString();
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		super.process(path, processor);
		
		processor.processBasic(path.newSubPath("limitAmount"), BigDecimal.class, limitAmount, this);
		processor.processBasic(path.newSubPath("limitImpactDueToTrade"), BigDecimal.class, limitImpactDueToTrade, this);
		
		processRosetta(path.newSubPath("limitLevel"), processor, FieldWithMetaLimitLevelEnum.class, limitLevel);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
		if (!super.equals(o)) return false;
	
		LimitApplicableExtended _that = (LimitApplicableExtended) o;
	
		if (limitAmount != null ? !limitAmount.equals(_that.limitAmount) : _that.limitAmount != null) return false;
		if (limitImpactDueToTrade != null ? !limitImpactDueToTrade.equals(_that.limitImpactDueToTrade) : _that.limitImpactDueToTrade != null) return false;
		if (limitLevel != null ? !limitLevel.equals(_that.limitLevel) : _that.limitLevel != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = super.hashCode();
		_result = 31 * _result + (limitAmount != null ? limitAmount.hashCode() : 0);
		_result = 31 * _result + (limitImpactDueToTrade != null ? limitImpactDueToTrade.hashCode() : 0);
		_result = 31 * _result + (limitLevel != null ? limitLevel.getClass().getName().hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "LimitApplicableExtended {" +
			"limitAmount=" + this.limitAmount + ", " +
			"limitImpactDueToTrade=" + this.limitImpactDueToTrade + ", " +
			"limitLevel=" + this.limitLevel +
		'}' + " " + super.toString();
	}
}
