package org.isda.cdm;

import java.lang.String;


import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.LoanParticipationMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class to specify loan with a participation agreement whereby the buyer is capable of creating, or procuring the creation of, a contractual right in favour of the seller that provides the seller with recourse to the participation seller for a specified share in any payments due under the relevant loan which are received by the participation seller. ISDA 2003 Term: Direct Loan Participation.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="LoanParticipation", source="FpML_5_10")
public class LoanParticipation extends PCDeliverableObligationCharac {
	private final String qualifyingParticipationSeller;
	private static LoanParticipationMeta metaData = new LoanParticipationMeta();
	
	LoanParticipation(LoanParticipationBuilder builder) {
		super(builder);
		this.qualifyingParticipationSeller = builder.getQualifyingParticipationSeller();
	}
	
	/**
	 * If Direct Loan Participation is specified as a deliverable obligation characteristic, this specifies any requirements for the Qualifying Participation Seller. The requirements may be listed free-form. ISDA 2003 Term: Qualifying Participation Seller.
	 */
	@RosettaSynonym(value="qualifyingParticipationSeller", source="FpML_5_10")
	@RosettaSynonym(value="qualifyingParticipationSeller", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="qualifyingParticipationSeller", source="DTCC_11_0")
	@RosettaSynonym(value="qualifyingParticipationSeller", source="DTCC_9_0")
	@RosettaSynonym(value="qualifyingParticipationSeller", source="CME_ClearedConfirm_1_17")
	public final String getQualifyingParticipationSeller() {
		return qualifyingParticipationSeller;
	}
	
	
	@Override
	public RosettaMetaData<? extends LoanParticipation> metaData() {
		return metaData;
	} 
	
	public LoanParticipationBuilder toBuilder() {
		LoanParticipationBuilder builder = new LoanParticipationBuilder();
		ofNullable(getQualifyingParticipationSeller()).ifPresent(builder::setQualifyingParticipationSeller);
		ofNullable(getApplicable()).ifPresent(builder::setApplicable);
		ofNullable(getPartialCashSettlement()).ifPresent(builder::setPartialCashSettlement);
		return builder;
	}

	public static LoanParticipationBuilder builder() {
		return new LoanParticipationBuilder();
	}

	public static  class LoanParticipationBuilder extends PCDeliverableObligationCharacBuilder {
	
		protected String qualifyingParticipationSeller;
	
		public LoanParticipationBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends LoanParticipation> metaData() {
			return metaData;
		} 
	
		
		public String getQualifyingParticipationSeller() {
			return qualifyingParticipationSeller;
		}
		
	
		public LoanParticipationBuilder setQualifyingParticipationSeller(String qualifyingParticipationSeller) {
			this.qualifyingParticipationSeller = qualifyingParticipationSeller;
			return this;
		}
		@Override public LoanParticipationBuilder setApplicable(Boolean applicable) {
			this.applicable = applicable;
			return this;
		}
		@Override public LoanParticipationBuilder setPartialCashSettlement(Boolean partialCashSettlement) {
			this.partialCashSettlement = partialCashSettlement;
			return this;
		}
		
		public LoanParticipation build() {
			return new LoanParticipation(this);
		}
	
		@Override
		public LoanParticipationBuilder prune() {
			super.prune();
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (super.hasData()) return true;
			if (getQualifyingParticipationSeller()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
			super.process(path, processor);
		
			processor.processBasic(path.newSubPath("qualifyingParticipationSeller"), String.class, qualifyingParticipationSeller, this);
			
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
			if (!super.equals(o)) return false;
		
			LoanParticipationBuilder _that = (LoanParticipationBuilder) o;
		
			if (qualifyingParticipationSeller != null ? !qualifyingParticipationSeller.equals(_that.qualifyingParticipationSeller) : _that.qualifyingParticipationSeller != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = super.hashCode();
			_result = 31 * _result + (qualifyingParticipationSeller != null ? qualifyingParticipationSeller.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "LoanParticipationBuilder {" +
				"qualifyingParticipationSeller=" + this.qualifyingParticipationSeller +
			'}' + " " + super.toString();
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		super.process(path, processor);
		
		processor.processBasic(path.newSubPath("qualifyingParticipationSeller"), String.class, qualifyingParticipationSeller, this);
		
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
		if (!super.equals(o)) return false;
	
		LoanParticipation _that = (LoanParticipation) o;
	
		if (qualifyingParticipationSeller != null ? !qualifyingParticipationSeller.equals(_that.qualifyingParticipationSeller) : _that.qualifyingParticipationSeller != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = super.hashCode();
		_result = 31 * _result + (qualifyingParticipationSeller != null ? qualifyingParticipationSeller.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "LoanParticipation {" +
			"qualifyingParticipationSeller=" + this.qualifyingParticipationSeller +
		'}' + " " + super.toString();
	}
}
