package org.isda.cdm;

import org.isda.cdm.InterpolationMethodEnum;
import org.isda.cdm.metafields.FieldWithMetaDate;
import org.isda.cdm.metafields.FieldWithMetaDate.FieldWithMetaDateBuilder;


import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.MakeWholeAmountMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import com.rosetta.model.lib.records.Date;
		import static java.util.Optional.ofNullable;
		import java.math.BigDecimal;

/**
 * A class to specify the amount to be paid by the buyer of the option if the option is exercised prior to the Early Call Date (typically applicable to the convertible bond options).
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="MakeWholeAmount", source="FpML_5_10")
public class MakeWholeAmount extends SwapCurveValuation {
	private final FieldWithMetaDate earlyCallDate;
	private final InterpolationMethodEnum interpolationMethod;
	private static MakeWholeAmountMeta metaData = new MakeWholeAmountMeta();
	
	MakeWholeAmount(MakeWholeAmountBuilder builder) {
		super(builder);
		this.earlyCallDate = ofNullable(builder.getEarlyCallDate()).map(f->f.build()).orElse(null);
		this.interpolationMethod = builder.getInterpolationMethod();
	}
	
	/**
	 * Date prior to which the option buyer will have to pay a Make Whole Amount to the option seller if he/she exercises the option.
	 */
	@RosettaSynonym(value="earlyCallDate", source="FpML_5_10")
	@RosettaSynonym(value="earlyCallDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="earlyCallDate", source="DTCC_11_0")
	@RosettaSynonym(value="earlyCallDate", source="DTCC_9_0")
	@RosettaSynonym(value="earlyCallDate", source="CME_ClearedConfirm_1_17")
	public final FieldWithMetaDate getEarlyCallDate() {
		return earlyCallDate;
	}
	
	/**
	 * The type of interpolation method that the calculation agent reserves the right to use.
	 */
	@RosettaSynonym(value="interpolationMethod", source="FpML_5_10")
	@RosettaSynonym(value="interpolationMethod", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="interpolationMethod", source="DTCC_11_0")
	@RosettaSynonym(value="interpolationMethod", source="DTCC_9_0")
	@RosettaSynonym(value="interpolationMethod", source="CME_ClearedConfirm_1_17")
	public final InterpolationMethodEnum getInterpolationMethod() {
		return interpolationMethod;
	}
	
	
	@Override
	public RosettaMetaData<? extends MakeWholeAmount> metaData() {
		return metaData;
	} 
	
	public MakeWholeAmountBuilder toBuilder() {
		MakeWholeAmountBuilder builder = new MakeWholeAmountBuilder();
		ofNullable(getEarlyCallDate()).ifPresent(builder::setEarlyCallDate);
		ofNullable(getInterpolationMethod()).ifPresent(builder::setInterpolationMethod);
		ofNullable(getFloatingRateIndex()).ifPresent(builder::setFloatingRateIndex);
		ofNullable(getIndexTenor()).ifPresent(builder::setIndexTenor);
		ofNullable(getSide()).ifPresent(builder::setSide);
		ofNullable(getSpread()).ifPresent(builder::setSpread);
		return builder;
	}

	public static MakeWholeAmountBuilder builder() {
		return new MakeWholeAmountBuilder();
	}

	public static  class MakeWholeAmountBuilder extends SwapCurveValuationBuilder {
	
		protected FieldWithMetaDateBuilder earlyCallDate;
		protected InterpolationMethodEnum interpolationMethod;
	
		public MakeWholeAmountBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends MakeWholeAmount> metaData() {
			return metaData;
		} 
	
		
		public FieldWithMetaDateBuilder getEarlyCallDate() {
			return earlyCallDate;
		}
		
		public FieldWithMetaDateBuilder getOrCreateEarlyCallDate() {
			if (earlyCallDate!=null) {
				return earlyCallDate;
			}
			else return earlyCallDate = new FieldWithMetaDateBuilder();
		}
		
		
		public InterpolationMethodEnum getInterpolationMethod() {
			return interpolationMethod;
		}
		
	
		public MakeWholeAmountBuilder setEarlyCallDate(FieldWithMetaDate earlyCallDate) {
			this.earlyCallDate = earlyCallDate.toBuilder();
			return this;
		}
		public MakeWholeAmountBuilder setEarlyCallDateRef(Date earlyCallDate) {
			return setEarlyCallDate(FieldWithMetaDate.builder().setValue(earlyCallDate).build());
		}
		public MakeWholeAmountBuilder setInterpolationMethod(InterpolationMethodEnum interpolationMethod) {
			this.interpolationMethod = interpolationMethod;
			return this;
		}
		@Override public MakeWholeAmountBuilder setFloatingRateIndex(FloatingRateIndexEnum floatingRateIndex) {
			this.floatingRateIndex = floatingRateIndex;
			return this;
		}
		@Override public MakeWholeAmountBuilder setIndexTenor(Period indexTenor) {
			this.indexTenor = indexTenor.toBuilder();
			return this;
		}
		@Override public MakeWholeAmountBuilder setIndexTenorBuilder(Period.PeriodBuilder indexTenor) {
			this.indexTenor = indexTenor;
			return this;
		}
		@Override public MakeWholeAmountBuilder setSide(QuotationSideEnum side) {
			this.side = side;
			return this;
		}
		@Override public MakeWholeAmountBuilder setSpread(BigDecimal spread) {
			this.spread = spread;
			return this;
		}
		
		public MakeWholeAmount build() {
			return new MakeWholeAmount(this);
		}
	
		@Override
		public MakeWholeAmountBuilder prune() {
			super.prune();
			if (earlyCallDate!=null && !earlyCallDate.prune().hasData()) earlyCallDate = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (super.hasData()) return true;
			if (getEarlyCallDate()!=null) return true;
			if (getInterpolationMethod()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
			super.process(path, processor);
		
			processor.processBasic(path.newSubPath("interpolationMethod"), InterpolationMethodEnum.class, interpolationMethod, this);
			
			processRosetta(path.newSubPath("earlyCallDate"), processor, FieldWithMetaDate.class, earlyCallDate);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
			if (!super.equals(o)) return false;
		
			MakeWholeAmountBuilder _that = (MakeWholeAmountBuilder) o;
		
			if (earlyCallDate != null ? !earlyCallDate.equals(_that.earlyCallDate) : _that.earlyCallDate != null) return false;
			if (interpolationMethod != null ? !interpolationMethod.equals(_that.interpolationMethod) : _that.interpolationMethod != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = super.hashCode();
			_result = 31 * _result + (earlyCallDate != null ? earlyCallDate.hashCode() : 0);
			_result = 31 * _result + (interpolationMethod != null ? interpolationMethod.getClass().getName().hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "MakeWholeAmountBuilder {" +
				"earlyCallDate=" + this.earlyCallDate + ", " +
				"interpolationMethod=" + this.interpolationMethod +
			'}' + " " + super.toString();
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		super.process(path, processor);
		
		processor.processBasic(path.newSubPath("interpolationMethod"), InterpolationMethodEnum.class, interpolationMethod, this);
		
		processRosetta(path.newSubPath("earlyCallDate"), processor, FieldWithMetaDate.class, earlyCallDate);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
		if (!super.equals(o)) return false;
	
		MakeWholeAmount _that = (MakeWholeAmount) o;
	
		if (earlyCallDate != null ? !earlyCallDate.equals(_that.earlyCallDate) : _that.earlyCallDate != null) return false;
		if (interpolationMethod != null ? !interpolationMethod.equals(_that.interpolationMethod) : _that.interpolationMethod != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = super.hashCode();
		_result = 31 * _result + (earlyCallDate != null ? earlyCallDate.hashCode() : 0);
		_result = 31 * _result + (interpolationMethod != null ? interpolationMethod.getClass().getName().hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "MakeWholeAmount {" +
			"earlyCallDate=" + this.earlyCallDate + ", " +
			"interpolationMethod=" + this.interpolationMethod +
		'}' + " " + super.toString();
	}
}
