package org.isda.cdm;

import org.isda.cdm.AdjustableDate;
import org.isda.cdm.CalculationAgent;
import org.isda.cdm.MandatoryEarlyTerminationAdjustedDates;
import org.isda.cdm.OptionCashSettlement;
import org.isda.cdm.metafields.MetaFields;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.MandatoryEarlyTerminationMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class to define an early termination provision for which exercise is mandatory.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="MandatoryEarlyTermination", source="FpML_5_10")
@RosettaSynonym(value="MandatoryEarlyTermination", source="CME_SubmissionIRS_1_0")
@RosettaSynonym(value="MandatoryEarlyTermination", source="DTCC_11_0")
@RosettaSynonym(value="MandatoryEarlyTermination", source="DTCC_9_0")
@RosettaSynonym(value="MandatoryEarlyTermination", source="CME_ClearedConfirm_1_17")
public class MandatoryEarlyTermination extends RosettaModelObject implements GlobalKey {
	private final CalculationAgent calculationAgent;
	private final OptionCashSettlement cashSettlement;
	private final MandatoryEarlyTerminationAdjustedDates mandatoryEarlyTerminationAdjustedDates;
	private final AdjustableDate mandatoryEarlyTerminationDate;
	private final MetaFields meta;
	private static MandatoryEarlyTerminationMeta metaData = new MandatoryEarlyTerminationMeta();
	
	MandatoryEarlyTermination(MandatoryEarlyTerminationBuilder builder) {
		this.calculationAgent = ofNullable(builder.getCalculationAgent()).map(f->f.build()).orElse(null);
		this.cashSettlement = ofNullable(builder.getCashSettlement()).map(f->f.build()).orElse(null);
		this.mandatoryEarlyTerminationAdjustedDates = ofNullable(builder.getMandatoryEarlyTerminationAdjustedDates()).map(f->f.build()).orElse(null);
		this.mandatoryEarlyTerminationDate = ofNullable(builder.getMandatoryEarlyTerminationDate()).map(f->f.build()).orElse(null);
		this.meta = ofNullable(builder.getMeta()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The ISDA Calculation Agent responsible for performing duties associated with an optional early termination.
	 */
	@RosettaSynonym(value="calculationAgent", source="FpML_5_10")
	@RosettaSynonym(value="calculationAgent", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="calculationAgent", source="DTCC_11_0")
	@RosettaSynonym(value="calculationAgent", source="DTCC_9_0")
	@RosettaSynonym(value="calculationAgent", source="CME_ClearedConfirm_1_17")
	public final CalculationAgent getCalculationAgent() {
		return calculationAgent;
	}
	
	/**
	 * If specified, this means that cash settlement is applicable to the transaction and defines the parameters associated with the cash settlement procedure. If not specified, then physical settlement is applicable.
	 */
	@RosettaSynonym(value="cashSettlement", source="FpML_5_10")
	@RosettaSynonym(value="cashSettlement", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="cashSettlement", source="DTCC_11_0")
	@RosettaSynonym(value="cashSettlement", source="DTCC_9_0")
	@RosettaSynonym(value="cashSettlement", source="CME_ClearedConfirm_1_17")
	public final OptionCashSettlement getCashSettlement() {
		return cashSettlement;
	}
	
	/**
	 * The adjusted dates associated with a mandatory early termination provision. These dates have been adjusted for any applicable business day convention.
	 */
	@RosettaSynonym(value="mandatoryEarlyTerminationAdjustedDates", source="FpML_5_10")
	@RosettaSynonym(value="mandatoryEarlyTerminationAdjustedDates", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="mandatoryEarlyTerminationAdjustedDates", source="DTCC_11_0")
	@RosettaSynonym(value="mandatoryEarlyTerminationAdjustedDates", source="DTCC_9_0")
	@RosettaSynonym(value="mandatoryEarlyTerminationAdjustedDates", source="CME_ClearedConfirm_1_17")
	public final MandatoryEarlyTerminationAdjustedDates getMandatoryEarlyTerminationAdjustedDates() {
		return mandatoryEarlyTerminationAdjustedDates;
	}
	
	/**
	 * The early termination date associated with a mandatory early termination of a swap.
	 */
	@RosettaSynonym(value="mandatoryEarlyTerminationDate", source="FpML_5_10")
	@RosettaSynonym(value="mandatoryEarlyTerminationDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="mandatoryEarlyTerminationDate", source="DTCC_11_0")
	@RosettaSynonym(value="mandatoryEarlyTerminationDate", source="DTCC_9_0")
	@RosettaSynonym(value="mandatoryEarlyTerminationDate", source="CME_ClearedConfirm_1_17")
	public final AdjustableDate getMandatoryEarlyTerminationDate() {
		return mandatoryEarlyTerminationDate;
	}
	
	public final MetaFields getMeta() {
		return meta;
	}
	
	
	@Override
	public RosettaMetaData<? extends MandatoryEarlyTermination> metaData() {
		return metaData;
	} 
	
	public MandatoryEarlyTerminationBuilder toBuilder() {
		MandatoryEarlyTerminationBuilder builder = new MandatoryEarlyTerminationBuilder();
		ofNullable(getCalculationAgent()).ifPresent(builder::setCalculationAgent);
		ofNullable(getCashSettlement()).ifPresent(builder::setCashSettlement);
		ofNullable(getMandatoryEarlyTerminationAdjustedDates()).ifPresent(builder::setMandatoryEarlyTerminationAdjustedDates);
		ofNullable(getMandatoryEarlyTerminationDate()).ifPresent(builder::setMandatoryEarlyTerminationDate);
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		return builder;
	}

	public static MandatoryEarlyTerminationBuilder builder() {
		return new MandatoryEarlyTerminationBuilder();
	}

	public static  class MandatoryEarlyTerminationBuilder extends RosettaModelObjectBuilder implements GlobalKeyBuilder<MandatoryEarlyTerminationBuilder> {
	
		protected CalculationAgent.CalculationAgentBuilder calculationAgent;
		protected OptionCashSettlement.OptionCashSettlementBuilder cashSettlement;
		protected MandatoryEarlyTerminationAdjustedDates.MandatoryEarlyTerminationAdjustedDatesBuilder mandatoryEarlyTerminationAdjustedDates;
		protected AdjustableDate.AdjustableDateBuilder mandatoryEarlyTerminationDate;
		protected MetaFields.MetaFieldsBuilder meta;
	
		public MandatoryEarlyTerminationBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends MandatoryEarlyTermination> metaData() {
			return metaData;
		} 
	
		
		public CalculationAgent.CalculationAgentBuilder getCalculationAgent() {
			return calculationAgent;
		}
		
		public CalculationAgent.CalculationAgentBuilder getOrCreateCalculationAgent() {
			if (calculationAgent!=null) {
				return calculationAgent;
			}
			else return calculationAgent = new CalculationAgent.CalculationAgentBuilder();
		}
		
		
		public OptionCashSettlement.OptionCashSettlementBuilder getCashSettlement() {
			return cashSettlement;
		}
		
		public OptionCashSettlement.OptionCashSettlementBuilder getOrCreateCashSettlement() {
			if (cashSettlement!=null) {
				return cashSettlement;
			}
			else return cashSettlement = new OptionCashSettlement.OptionCashSettlementBuilder();
		}
		
		
		public MandatoryEarlyTerminationAdjustedDates.MandatoryEarlyTerminationAdjustedDatesBuilder getMandatoryEarlyTerminationAdjustedDates() {
			return mandatoryEarlyTerminationAdjustedDates;
		}
		
		public MandatoryEarlyTerminationAdjustedDates.MandatoryEarlyTerminationAdjustedDatesBuilder getOrCreateMandatoryEarlyTerminationAdjustedDates() {
			if (mandatoryEarlyTerminationAdjustedDates!=null) {
				return mandatoryEarlyTerminationAdjustedDates;
			}
			else return mandatoryEarlyTerminationAdjustedDates = new MandatoryEarlyTerminationAdjustedDates.MandatoryEarlyTerminationAdjustedDatesBuilder();
		}
		
		
		public AdjustableDate.AdjustableDateBuilder getMandatoryEarlyTerminationDate() {
			return mandatoryEarlyTerminationDate;
		}
		
		public AdjustableDate.AdjustableDateBuilder getOrCreateMandatoryEarlyTerminationDate() {
			if (mandatoryEarlyTerminationDate!=null) {
				return mandatoryEarlyTerminationDate;
			}
			else return mandatoryEarlyTerminationDate = new AdjustableDate.AdjustableDateBuilder();
		}
		
		
		public MetaFields.MetaFieldsBuilder getMeta() {
			return meta;
		}
		
		public MetaFields.MetaFieldsBuilder getOrCreateMeta() {
			if (meta!=null) {
				return meta;
			}
			else return meta = new MetaFields.MetaFieldsBuilder();
		}
		
	
		public MandatoryEarlyTerminationBuilder setCalculationAgent(CalculationAgent calculationAgent) {
			this.calculationAgent = calculationAgent.toBuilder();
			return this;
		}
		public MandatoryEarlyTerminationBuilder setCalculationAgentBuilder(CalculationAgent.CalculationAgentBuilder calculationAgent) {
			this.calculationAgent = calculationAgent;
			return this;
		}
		public MandatoryEarlyTerminationBuilder setCashSettlement(OptionCashSettlement cashSettlement) {
			this.cashSettlement = cashSettlement.toBuilder();
			return this;
		}
		public MandatoryEarlyTerminationBuilder setCashSettlementBuilder(OptionCashSettlement.OptionCashSettlementBuilder cashSettlement) {
			this.cashSettlement = cashSettlement;
			return this;
		}
		public MandatoryEarlyTerminationBuilder setMandatoryEarlyTerminationAdjustedDates(MandatoryEarlyTerminationAdjustedDates mandatoryEarlyTerminationAdjustedDates) {
			this.mandatoryEarlyTerminationAdjustedDates = mandatoryEarlyTerminationAdjustedDates.toBuilder();
			return this;
		}
		public MandatoryEarlyTerminationBuilder setMandatoryEarlyTerminationAdjustedDatesBuilder(MandatoryEarlyTerminationAdjustedDates.MandatoryEarlyTerminationAdjustedDatesBuilder mandatoryEarlyTerminationAdjustedDates) {
			this.mandatoryEarlyTerminationAdjustedDates = mandatoryEarlyTerminationAdjustedDates;
			return this;
		}
		public MandatoryEarlyTerminationBuilder setMandatoryEarlyTerminationDate(AdjustableDate mandatoryEarlyTerminationDate) {
			this.mandatoryEarlyTerminationDate = mandatoryEarlyTerminationDate.toBuilder();
			return this;
		}
		public MandatoryEarlyTerminationBuilder setMandatoryEarlyTerminationDateBuilder(AdjustableDate.AdjustableDateBuilder mandatoryEarlyTerminationDate) {
			this.mandatoryEarlyTerminationDate = mandatoryEarlyTerminationDate;
			return this;
		}
		public MandatoryEarlyTerminationBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		public MandatoryEarlyTerminationBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		
		public MandatoryEarlyTermination build() {
			return new MandatoryEarlyTermination(this);
		}
	
		@Override
		public MandatoryEarlyTerminationBuilder prune() {
			if (calculationAgent!=null && !calculationAgent.prune().hasData()) calculationAgent = null;
			if (cashSettlement!=null && !cashSettlement.prune().hasData()) cashSettlement = null;
			if (mandatoryEarlyTerminationAdjustedDates!=null && !mandatoryEarlyTerminationAdjustedDates.prune().hasData()) mandatoryEarlyTerminationAdjustedDates = null;
			if (mandatoryEarlyTerminationDate!=null && !mandatoryEarlyTerminationDate.prune().hasData()) mandatoryEarlyTerminationDate = null;
			if (meta!=null && !meta.prune().hasData()) meta = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getCalculationAgent()!=null && getCalculationAgent().hasData()) return true;
			if (getCashSettlement()!=null && getCashSettlement().hasData()) return true;
			if (getMandatoryEarlyTerminationAdjustedDates()!=null && getMandatoryEarlyTerminationAdjustedDates().hasData()) return true;
			if (getMandatoryEarlyTerminationDate()!=null && getMandatoryEarlyTerminationDate().hasData()) return true;
			if (getMeta()!=null && getMeta().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("calculationAgent"), processor, CalculationAgent.class, calculationAgent);
			processRosetta(path.newSubPath("cashSettlement"), processor, OptionCashSettlement.class, cashSettlement);
			processRosetta(path.newSubPath("mandatoryEarlyTerminationAdjustedDates"), processor, MandatoryEarlyTerminationAdjustedDates.class, mandatoryEarlyTerminationAdjustedDates);
			processRosetta(path.newSubPath("mandatoryEarlyTerminationDate"), processor, AdjustableDate.class, mandatoryEarlyTerminationDate);
			processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			MandatoryEarlyTerminationBuilder _that = (MandatoryEarlyTerminationBuilder) o;
		
			if (calculationAgent != null ? !calculationAgent.equals(_that.calculationAgent) : _that.calculationAgent != null) return false;
			if (cashSettlement != null ? !cashSettlement.equals(_that.cashSettlement) : _that.cashSettlement != null) return false;
			if (mandatoryEarlyTerminationAdjustedDates != null ? !mandatoryEarlyTerminationAdjustedDates.equals(_that.mandatoryEarlyTerminationAdjustedDates) : _that.mandatoryEarlyTerminationAdjustedDates != null) return false;
			if (mandatoryEarlyTerminationDate != null ? !mandatoryEarlyTerminationDate.equals(_that.mandatoryEarlyTerminationDate) : _that.mandatoryEarlyTerminationDate != null) return false;
			if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (calculationAgent != null ? calculationAgent.hashCode() : 0);
			_result = 31 * _result + (cashSettlement != null ? cashSettlement.hashCode() : 0);
			_result = 31 * _result + (mandatoryEarlyTerminationAdjustedDates != null ? mandatoryEarlyTerminationAdjustedDates.hashCode() : 0);
			_result = 31 * _result + (mandatoryEarlyTerminationDate != null ? mandatoryEarlyTerminationDate.hashCode() : 0);
			_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "MandatoryEarlyTerminationBuilder {" +
				"calculationAgent=" + this.calculationAgent + ", " +
				"cashSettlement=" + this.cashSettlement + ", " +
				"mandatoryEarlyTerminationAdjustedDates=" + this.mandatoryEarlyTerminationAdjustedDates + ", " +
				"mandatoryEarlyTerminationDate=" + this.mandatoryEarlyTerminationDate + ", " +
				"meta=" + this.meta +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("calculationAgent"), processor, CalculationAgent.class, calculationAgent);
		processRosetta(path.newSubPath("cashSettlement"), processor, OptionCashSettlement.class, cashSettlement);
		processRosetta(path.newSubPath("mandatoryEarlyTerminationAdjustedDates"), processor, MandatoryEarlyTerminationAdjustedDates.class, mandatoryEarlyTerminationAdjustedDates);
		processRosetta(path.newSubPath("mandatoryEarlyTerminationDate"), processor, AdjustableDate.class, mandatoryEarlyTerminationDate);
		processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		MandatoryEarlyTermination _that = (MandatoryEarlyTermination) o;
	
		if (calculationAgent != null ? !calculationAgent.equals(_that.calculationAgent) : _that.calculationAgent != null) return false;
		if (cashSettlement != null ? !cashSettlement.equals(_that.cashSettlement) : _that.cashSettlement != null) return false;
		if (mandatoryEarlyTerminationAdjustedDates != null ? !mandatoryEarlyTerminationAdjustedDates.equals(_that.mandatoryEarlyTerminationAdjustedDates) : _that.mandatoryEarlyTerminationAdjustedDates != null) return false;
		if (mandatoryEarlyTerminationDate != null ? !mandatoryEarlyTerminationDate.equals(_that.mandatoryEarlyTerminationDate) : _that.mandatoryEarlyTerminationDate != null) return false;
		if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (calculationAgent != null ? calculationAgent.hashCode() : 0);
		_result = 31 * _result + (cashSettlement != null ? cashSettlement.hashCode() : 0);
		_result = 31 * _result + (mandatoryEarlyTerminationAdjustedDates != null ? mandatoryEarlyTerminationAdjustedDates.hashCode() : 0);
		_result = 31 * _result + (mandatoryEarlyTerminationDate != null ? mandatoryEarlyTerminationDate.hashCode() : 0);
		_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "MandatoryEarlyTermination {" +
			"calculationAgent=" + this.calculationAgent + ", " +
			"cashSettlement=" + this.cashSettlement + ", " +
			"mandatoryEarlyTerminationAdjustedDates=" + this.mandatoryEarlyTerminationAdjustedDates + ", " +
			"mandatoryEarlyTerminationDate=" + this.mandatoryEarlyTerminationDate + ", " +
			"meta=" + this.meta +
		'}';
	}
}
