package org.isda.cdm;

import java.lang.Boolean;
import org.isda.cdm.ExerciseNotice;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.ManualExerciseMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class defining manual exercise, i.e. that the option buyer counterparty must give notice to the option seller of exercise.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="ManualExercise", source="FpML_5_10")
public class ManualExercise extends RosettaModelObject {
	private final ExerciseNotice exerciseNotice;
	private final Boolean fallbackExercise;
	private static ManualExerciseMeta metaData = new ManualExerciseMeta();
	
	ManualExercise(ManualExerciseBuilder builder) {
		this.exerciseNotice = ofNullable(builder.getExerciseNotice()).map(f->f.build()).orElse(null);
		this.fallbackExercise = builder.getFallbackExercise();
	}
	
	/**
	 * Definition of the party to whom notice of exercise should be given.
	 */
	@RosettaSynonym(value="exerciseNotice", source="FpML_5_10")
	@RosettaSynonym(value="exerciseNotice", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="exerciseNotice", source="DTCC_11_0")
	@RosettaSynonym(value="exerciseNotice", source="DTCC_9_0")
	@RosettaSynonym(value="exerciseNotice", source="CME_ClearedConfirm_1_17")
	public final ExerciseNotice getExerciseNotice() {
		return exerciseNotice;
	}
	
	/**
	 * If fallback exercise is specified then the notional amount of the underlying swap, not previously exercised under the swaption, will be automatically exercised at the expiration time on the expiration date if at such time the buyer is in-the-money, provided that the difference between the settlement rate and the fixed rate under the relevant underlying swap is not less than one tenth of a percentage point (0.10% or 0.001). The term in-the-money is assumed to have the meaning defined in the 2000 ISDA Definitions, Section 17.4. In-the-money.
	 */
	@RosettaSynonym(value="fallbackExercise", source="FpML_5_10")
	@RosettaSynonym(value="fallbackExercise", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="fallbackExercise", source="DTCC_11_0")
	@RosettaSynonym(value="fallbackExercise", source="DTCC_9_0")
	@RosettaSynonym(value="fallbackExercise", source="CME_ClearedConfirm_1_17")
	public final Boolean getFallbackExercise() {
		return fallbackExercise;
	}
	
	
	@Override
	public RosettaMetaData<? extends ManualExercise> metaData() {
		return metaData;
	} 
	
	public ManualExerciseBuilder toBuilder() {
		ManualExerciseBuilder builder = new ManualExerciseBuilder();
		ofNullable(getExerciseNotice()).ifPresent(builder::setExerciseNotice);
		ofNullable(getFallbackExercise()).ifPresent(builder::setFallbackExercise);
		return builder;
	}

	public static ManualExerciseBuilder builder() {
		return new ManualExerciseBuilder();
	}

	public static  class ManualExerciseBuilder extends RosettaModelObjectBuilder {
	
		protected ExerciseNotice.ExerciseNoticeBuilder exerciseNotice;
		protected Boolean fallbackExercise;
	
		public ManualExerciseBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends ManualExercise> metaData() {
			return metaData;
		} 
	
		
		public ExerciseNotice.ExerciseNoticeBuilder getExerciseNotice() {
			return exerciseNotice;
		}
		
		public ExerciseNotice.ExerciseNoticeBuilder getOrCreateExerciseNotice() {
			if (exerciseNotice!=null) {
				return exerciseNotice;
			}
			else return exerciseNotice = new ExerciseNotice.ExerciseNoticeBuilder();
		}
		
		
		public Boolean getFallbackExercise() {
			return fallbackExercise;
		}
		
	
		public ManualExerciseBuilder setExerciseNotice(ExerciseNotice exerciseNotice) {
			this.exerciseNotice = exerciseNotice.toBuilder();
			return this;
		}
		public ManualExerciseBuilder setExerciseNoticeBuilder(ExerciseNotice.ExerciseNoticeBuilder exerciseNotice) {
			this.exerciseNotice = exerciseNotice;
			return this;
		}
		public ManualExerciseBuilder setFallbackExercise(Boolean fallbackExercise) {
			this.fallbackExercise = fallbackExercise;
			return this;
		}
		
		public ManualExercise build() {
			return new ManualExercise(this);
		}
	
		@Override
		public ManualExerciseBuilder prune() {
			if (exerciseNotice!=null && !exerciseNotice.prune().hasData()) exerciseNotice = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getExerciseNotice()!=null && getExerciseNotice().hasData()) return true;
			if (getFallbackExercise()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("fallbackExercise"), Boolean.class, fallbackExercise, this);
			
			processRosetta(path.newSubPath("exerciseNotice"), processor, ExerciseNotice.class, exerciseNotice);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			ManualExerciseBuilder _that = (ManualExerciseBuilder) o;
		
			if (exerciseNotice != null ? !exerciseNotice.equals(_that.exerciseNotice) : _that.exerciseNotice != null) return false;
			if (fallbackExercise != null ? !fallbackExercise.equals(_that.fallbackExercise) : _that.fallbackExercise != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (exerciseNotice != null ? exerciseNotice.hashCode() : 0);
			_result = 31 * _result + (fallbackExercise != null ? fallbackExercise.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "ManualExerciseBuilder {" +
				"exerciseNotice=" + this.exerciseNotice + ", " +
				"fallbackExercise=" + this.fallbackExercise +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("fallbackExercise"), Boolean.class, fallbackExercise, this);
		
		processRosetta(path.newSubPath("exerciseNotice"), processor, ExerciseNotice.class, exerciseNotice);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		ManualExercise _that = (ManualExercise) o;
	
		if (exerciseNotice != null ? !exerciseNotice.equals(_that.exerciseNotice) : _that.exerciseNotice != null) return false;
		if (fallbackExercise != null ? !fallbackExercise.equals(_that.fallbackExercise) : _that.fallbackExercise != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (exerciseNotice != null ? exerciseNotice.hashCode() : 0);
		_result = 31 * _result + (fallbackExercise != null ? fallbackExercise.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "ManualExercise {" +
			"exerciseNotice=" + this.exerciseNotice + ", " +
			"fallbackExercise=" + this.fallbackExercise +
		'}';
	}
}
