package org.isda.cdm;

import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.SimmCalculationCurrency;
import org.isda.cdm.SimmVersion;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.MethodMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;
		import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;

/**
 * A class to specify the ISDA SIMM as the Method for all Covered Transactions with respect to all Regimes. ISDA 2016 Credit Support Annex for Initial Margin, paragraph 13, General Principles, (ee).
 * 
 * @version 2.5.4
 */
@RosettaClass
public class Method extends RosettaModelObject {
	private final List<SimmCalculationCurrency> calculationCurrency;
	private final SimmVersion version;
	private static MethodMeta metaData = new MethodMeta();
	
	Method(MethodBuilder builder) {
		this.calculationCurrency = ofNullable(builder.getCalculationCurrency()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.version = ofNullable(builder.getVersion()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The SIMM Calculation Currency, as specified for each of the parties to the CSA Initial Margin. ISDA 2016 Credit Support Annex for Initial Margin, paragraph 13, General Principles, (ee)(3).
	 */
	@RosettaSynonym(value="simm_calculation_currency", source="ISDA_Create_1_0")
	public final List<SimmCalculationCurrency> getCalculationCurrency() {
		return calculationCurrency;
	}
	
	/**
	 * The qualification of the ISDA SIMM version that is specified for all Covered Transactions as specified by ISDA 2018 CSA for Initial Margin, Paragraph 13, General Principles, (ee)(1).
	 */
	public final SimmVersion getVersion() {
		return version;
	}
	
	
	@Override
	public RosettaMetaData<? extends Method> metaData() {
		return metaData;
	} 
	
	public MethodBuilder toBuilder() {
		MethodBuilder builder = new MethodBuilder();
		ofNullable(getCalculationCurrency()).ifPresent(calculationCurrency -> calculationCurrency.forEach(builder::addCalculationCurrency));
		ofNullable(getVersion()).ifPresent(builder::setVersion);
		return builder;
	}

	public static MethodBuilder builder() {
		return new MethodBuilder();
	}

	public static  class MethodBuilder extends RosettaModelObjectBuilder {
	
		protected List<SimmCalculationCurrency.SimmCalculationCurrencyBuilder> calculationCurrency;
		protected SimmVersion.SimmVersionBuilder version;
	
		public MethodBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends Method> metaData() {
			return metaData;
		} 
	
		
		public List<SimmCalculationCurrency.SimmCalculationCurrencyBuilder> getCalculationCurrency() {
			return calculationCurrency;
		}
		
		public SimmCalculationCurrency.SimmCalculationCurrencyBuilder getOrCreateCalculationCurrency(int index) {
			if (calculationCurrency==null) {
				this.calculationCurrency = new ArrayList<>();
			}
			return getIndex(calculationCurrency, index, ()->new SimmCalculationCurrency.SimmCalculationCurrencyBuilder());
		}
		
		
		public SimmVersion.SimmVersionBuilder getVersion() {
			return version;
		}
		
		public SimmVersion.SimmVersionBuilder getOrCreateVersion() {
			if (version!=null) {
				return version;
			}
			else return version = new SimmVersion.SimmVersionBuilder();
		}
		
	
		public MethodBuilder addCalculationCurrency(SimmCalculationCurrency calculationCurrency) {
			if(this.calculationCurrency == null){
				this.calculationCurrency = new ArrayList<>();
			}
			this.calculationCurrency.add(calculationCurrency.toBuilder());
			return this;
		}
		public MethodBuilder addCalculationCurrency(SimmCalculationCurrency calculationCurrency, int _idx) {
			if(this.calculationCurrency == null){
				this.calculationCurrency = new ArrayList<>();
			}
			getIndex(this.calculationCurrency, _idx, () -> calculationCurrency.toBuilder());
			this.calculationCurrency.set(_idx, calculationCurrency.toBuilder());
			return this;
		}
		public MethodBuilder addCalculationCurrency(List<SimmCalculationCurrency> calculationCurrencys) {
			if(this.calculationCurrency == null){
				this.calculationCurrency = new ArrayList<>();
			}
			for (SimmCalculationCurrency toAdd : calculationCurrencys) {
				this.calculationCurrency.add(toAdd.toBuilder());
			}
			return this;
		}
		public MethodBuilder addCalculationCurrencyBuilder(SimmCalculationCurrency.SimmCalculationCurrencyBuilder calculationCurrency) {
			if(this.calculationCurrency == null){
				this.calculationCurrency = new ArrayList<>();
			}
			this.calculationCurrency.add(calculationCurrency);
			return this;
		}
		
		public MethodBuilder clearCalculationCurrency() {
			this.calculationCurrency = null;
			return this;
		}
		public MethodBuilder setVersion(SimmVersion version) {
			this.version = version.toBuilder();
			return this;
		}
		public MethodBuilder setVersionBuilder(SimmVersion.SimmVersionBuilder version) {
			this.version = version;
			return this;
		}
		
		public Method build() {
			return new Method(this);
		}
	
		@Override
		public MethodBuilder prune() {
			if (calculationCurrency!=null) calculationCurrency = calculationCurrency.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (version!=null && !version.prune().hasData()) version = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getCalculationCurrency()!=null && getCalculationCurrency().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getVersion()!=null && getVersion().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("calculationCurrency"), processor, SimmCalculationCurrency.class, calculationCurrency);
			processRosetta(path.newSubPath("version"), processor, SimmVersion.class, version);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			MethodBuilder _that = (MethodBuilder) o;
		
			if (!ListEquals.listEquals(calculationCurrency, _that.calculationCurrency)) return false;
			if (version != null ? !version.equals(_that.version) : _that.version != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (calculationCurrency != null ? calculationCurrency.hashCode() : 0);
			_result = 31 * _result + (version != null ? version.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "MethodBuilder {" +
				"calculationCurrency=" + this.calculationCurrency + ", " +
				"version=" + this.version +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("calculationCurrency"), processor, SimmCalculationCurrency.class, calculationCurrency);
		processRosetta(path.newSubPath("version"), processor, SimmVersion.class, version);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		Method _that = (Method) o;
	
		if (!ListEquals.listEquals(calculationCurrency, _that.calculationCurrency)) return false;
		if (version != null ? !version.equals(_that.version) : _that.version != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (calculationCurrency != null ? calculationCurrency.hashCode() : 0);
		_result = 31 * _result + (version != null ? version.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "Method {" +
			"calculationCurrency=" + this.calculationCurrency + ", " +
			"version=" + this.version +
		'}';
	}
}
