package org.isda.cdm;

import java.math.BigDecimal;
import org.isda.cdm.metafields.FieldWithMetaString;
import org.isda.cdm.metafields.FieldWithMetaString.FieldWithMetaStringBuilder;
import org.isda.cdm.metafields.MetaFields;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.MoneyMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.math.BigDecimal;
		import static java.util.Optional.ofNullable;

/**
 * A class defining a currency amount.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="Money", source="FpML_5_10")
@RosettaSynonym(value="Money", source="CME_SubmissionIRS_1_0")
@RosettaSynonym(value="Money", source="DTCC_11_0")
@RosettaSynonym(value="Money", source="DTCC_9_0")
@RosettaSynonym(value="Money", source="CME_ClearedConfirm_1_17")
public class Money extends RosettaModelObject implements GlobalKey {
	private final BigDecimal amount;
	private final FieldWithMetaString currency;
	private final MetaFields meta;
	private static MoneyMeta metaData = new MoneyMeta();
	
	Money(MoneyBuilder builder) {
		this.amount = builder.getAmount();
		this.currency = ofNullable(builder.getCurrency()).map(f->f.build()).orElse(null);
		this.meta = ofNullable(builder.getMeta()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The monetary quantity in currency units.
	 */
	@RosettaSynonym(value="amount", source="FpML_5_10", maps=2)
	@RosettaSynonym(value="amount", source="CME_SubmissionIRS_1_0", maps=2)
	@RosettaSynonym(value="amount", source="DTCC_11_0", maps=2)
	@RosettaSynonym(value="amount", source="DTCC_9_0", maps=2)
	@RosettaSynonym(value="amount", source="CME_ClearedConfirm_1_17", maps=2)
	@RosettaSynonym(value="partyA_amount", source="ISDA_Create_1_0")
	@RosettaSynonym(value="partyB_amount", source="ISDA_Create_1_0")
	public final BigDecimal getAmount() {
		return amount;
	}
	
	/**
	 * The currency in which the associated amount is denominated. The list of valid currencies is not presently positioned as an enumeration as part of the CDM because that scope is limited to the values specified by ISDA and FpML. As a result, implementers have to make reference to the relevant standard, such as the ISO 4217 standard for currency codes.
	 */
	@RosettaSynonym(value="currency", source="FpML_5_10", maps=2)
	@RosettaSynonym(value="currency", source="CME_SubmissionIRS_1_0", maps=2)
	@RosettaSynonym(value="currency", source="DTCC_11_0", maps=2)
	@RosettaSynonym(value="currency", source="DTCC_9_0", maps=2)
	@RosettaSynonym(value="currency", source="CME_ClearedConfirm_1_17", maps=2)
	@RosettaSynonym(value="partyA_currency", source="ISDA_Create_1_0")
	@RosettaSynonym(value="partyB_currency", source="ISDA_Create_1_0")
	public final FieldWithMetaString getCurrency() {
		return currency;
	}
	
	public final MetaFields getMeta() {
		return meta;
	}
	
	
	@Override
	public RosettaMetaData<? extends Money> metaData() {
		return metaData;
	} 
	
	public MoneyBuilder toBuilder() {
		MoneyBuilder builder = new MoneyBuilder();
		ofNullable(getAmount()).ifPresent(builder::setAmount);
		ofNullable(getCurrency()).ifPresent(builder::setCurrency);
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		return builder;
	}

	public static MoneyBuilder builder() {
		return new MoneyBuilder();
	}

	public static  class MoneyBuilder extends RosettaModelObjectBuilder implements GlobalKeyBuilder<MoneyBuilder> {
	
		protected BigDecimal amount;
		protected FieldWithMetaStringBuilder currency;
		protected MetaFields.MetaFieldsBuilder meta;
	
		public MoneyBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends Money> metaData() {
			return metaData;
		} 
	
		
		public BigDecimal getAmount() {
			return amount;
		}
		
		
		public FieldWithMetaStringBuilder getCurrency() {
			return currency;
		}
		
		public FieldWithMetaStringBuilder getOrCreateCurrency() {
			if (currency!=null) {
				return currency;
			}
			else return currency = new FieldWithMetaStringBuilder();
		}
		
		
		public MetaFields.MetaFieldsBuilder getMeta() {
			return meta;
		}
		
		public MetaFields.MetaFieldsBuilder getOrCreateMeta() {
			if (meta!=null) {
				return meta;
			}
			else return meta = new MetaFields.MetaFieldsBuilder();
		}
		
	
		public MoneyBuilder setAmount(BigDecimal amount) {
			this.amount = amount;
			return this;
		}
		public MoneyBuilder setCurrency(FieldWithMetaString currency) {
			this.currency = currency.toBuilder();
			return this;
		}
		public MoneyBuilder setCurrencyRef(String currency) {
			return setCurrency(FieldWithMetaString.builder().setValue(currency).build());
		}
		public MoneyBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		public MoneyBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		
		public Money build() {
			return new Money(this);
		}
	
		@Override
		public MoneyBuilder prune() {
			if (currency!=null && !currency.prune().hasData()) currency = null;
			if (meta!=null && !meta.prune().hasData()) meta = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getAmount()!=null) return true;
			if (getCurrency()!=null) return true;
			if (getMeta()!=null && getMeta().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("amount"), BigDecimal.class, amount, this);
			
			processRosetta(path.newSubPath("currency"), processor, FieldWithMetaString.class, currency);
			processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			MoneyBuilder _that = (MoneyBuilder) o;
		
			if (amount != null ? !amount.equals(_that.amount) : _that.amount != null) return false;
			if (currency != null ? !currency.equals(_that.currency) : _that.currency != null) return false;
			if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (amount != null ? amount.hashCode() : 0);
			_result = 31 * _result + (currency != null ? currency.hashCode() : 0);
			_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "MoneyBuilder {" +
				"amount=" + this.amount + ", " +
				"currency=" + this.currency + ", " +
				"meta=" + this.meta +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("amount"), BigDecimal.class, amount, this);
		
		processRosetta(path.newSubPath("currency"), processor, FieldWithMetaString.class, currency);
		processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		Money _that = (Money) o;
	
		if (amount != null ? !amount.equals(_that.amount) : _that.amount != null) return false;
		if (currency != null ? !currency.equals(_that.currency) : _that.currency != null) return false;
		if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (amount != null ? amount.hashCode() : 0);
		_result = 31 * _result + (currency != null ? currency.hashCode() : 0);
		_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "Money {" +
			"amount=" + this.amount + ", " +
			"currency=" + this.currency + ", " +
			"meta=" + this.meta +
		'}';
	}
}
