package org.isda.cdm;

import java.math.BigDecimal;
import org.isda.cdm.metafields.ReferenceWithMetaMoney;
import org.isda.cdm.metafields.ReferenceWithMetaMoney.ReferenceWithMetaMoneyBuilder;


import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.MultipleExerciseMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.math.BigDecimal;
		import java.util.stream.Collectors;
		import static java.util.Optional.ofNullable;

/**
 * A class defining multiple exercises. As defined in the 2000 ISDA Definitions, Section 12.4. Multiple Exercise, the buyer of the option has the right to exercise all or less than all the unexercised notional amount of the underlying swap on one or more days in the exercise period, but on any such day may not exercise less than the minimum notional amount or more than the maximum notional amount, and if an integral multiple amount is specified, the notional exercised must be equal to or, be an integral multiple of, the integral multiple amount. In FpML, MultipleExercise is built upon the PartialExercise.model.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="MultipleExercise", source="FpML_5_10")
public class MultipleExercise extends PartialExercise {
	private final BigDecimal maximumNotionalAmount;
	private final BigDecimal maximumNumberOfOptions;
	private static MultipleExerciseMeta metaData = new MultipleExerciseMeta();
	
	MultipleExercise(MultipleExerciseBuilder builder) {
		super(builder);
		this.maximumNotionalAmount = builder.getMaximumNotionalAmount();
		this.maximumNumberOfOptions = builder.getMaximumNumberOfOptions();
	}
	
	/**
	 * The maximum notional amount that can be exercised on a given exercise date.
	 */
	@RosettaSynonym(value="maximumNotionalAmount", source="FpML_5_10")
	@RosettaSynonym(value="maximumNotionalAmount", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="maximumNotionalAmount", source="DTCC_11_0")
	@RosettaSynonym(value="maximumNotionalAmount", source="DTCC_9_0")
	@RosettaSynonym(value="maximumNotionalAmount", source="CME_ClearedConfirm_1_17")
	public final BigDecimal getMaximumNotionalAmount() {
		return maximumNotionalAmount;
	}
	
	/**
	 * The maximum number of options that can be exercised on a given exercise date. If the number is not specified, it means that the maximum number of options corresponds to the remaining unexercised options.
	 */
	@RosettaSynonym(value="maximumNumberOfOptions", source="FpML_5_10")
	@RosettaSynonym(value="maximumNumberOfOptions", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="maximumNumberOfOptions", source="DTCC_11_0")
	@RosettaSynonym(value="maximumNumberOfOptions", source="DTCC_9_0")
	@RosettaSynonym(value="maximumNumberOfOptions", source="CME_ClearedConfirm_1_17")
	public final BigDecimal getMaximumNumberOfOptions() {
		return maximumNumberOfOptions;
	}
	
	
	@Override
	public RosettaMetaData<? extends MultipleExercise> metaData() {
		return metaData;
	} 
	
	public MultipleExerciseBuilder toBuilder() {
		MultipleExerciseBuilder builder = new MultipleExerciseBuilder();
		ofNullable(getMaximumNotionalAmount()).ifPresent(builder::setMaximumNotionalAmount);
		ofNullable(getMaximumNumberOfOptions()).ifPresent(builder::setMaximumNumberOfOptions);
		ofNullable(getIntegralMultipleAmount()).ifPresent(builder::setIntegralMultipleAmount);
		ofNullable(getMinimumNotionalAmount()).ifPresent(builder::setMinimumNotionalAmount);
		ofNullable(getMinimumNumberOfOptions()).ifPresent(builder::setMinimumNumberOfOptions);
		ofNullable(getNotionaReference()).ifPresent(builder::setNotionaReference);
		return builder;
	}

	public static MultipleExerciseBuilder builder() {
		return new MultipleExerciseBuilder();
	}

	public static  class MultipleExerciseBuilder extends PartialExerciseBuilder {
	
		protected BigDecimal maximumNotionalAmount;
		protected BigDecimal maximumNumberOfOptions;
	
		public MultipleExerciseBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends MultipleExercise> metaData() {
			return metaData;
		} 
	
		
		public BigDecimal getMaximumNotionalAmount() {
			return maximumNotionalAmount;
		}
		
		
		public BigDecimal getMaximumNumberOfOptions() {
			return maximumNumberOfOptions;
		}
		
	
		public MultipleExerciseBuilder setMaximumNotionalAmount(BigDecimal maximumNotionalAmount) {
			this.maximumNotionalAmount = maximumNotionalAmount;
			return this;
		}
		public MultipleExerciseBuilder setMaximumNumberOfOptions(BigDecimal maximumNumberOfOptions) {
			this.maximumNumberOfOptions = maximumNumberOfOptions;
			return this;
		}
		@Override public MultipleExerciseBuilder setIntegralMultipleAmount(BigDecimal integralMultipleAmount) {
			this.integralMultipleAmount = integralMultipleAmount;
			return this;
		}
		@Override public MultipleExerciseBuilder setMinimumNotionalAmount(BigDecimal minimumNotionalAmount) {
			this.minimumNotionalAmount = minimumNotionalAmount;
			return this;
		}
		@Override public MultipleExerciseBuilder setMinimumNumberOfOptions(Integer minimumNumberOfOptions) {
			this.minimumNumberOfOptions = minimumNumberOfOptions;
			return this;
		}
		@Override public MultipleExerciseBuilder setNotionaReference(ReferenceWithMetaMoney notionaReference) {
			this.notionaReference = notionaReference.toBuilder();
			return this;
		}
		@Override public MultipleExerciseBuilder setNotionaReferenceBuilder(ReferenceWithMetaMoneyBuilder notionaReference) {
			this.notionaReference = notionaReference;
			return this;
		}
		@Override public MultipleExerciseBuilder setNotionaReferenceRef(Money.MoneyBuilder notionaReference) {
			return setNotionaReference(ReferenceWithMetaMoney.builder().setValueBuilder(notionaReference).build());
		}
		@Override public MultipleExerciseBuilder setNotionaReferenceRef(Money notionaReference) {
			return setNotionaReferenceRef(Money.builder());
		}
		
		public MultipleExercise build() {
			return new MultipleExercise(this);
		}
	
		@Override
		public MultipleExerciseBuilder prune() {
			super.prune();
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (super.hasData()) return true;
			if (getMaximumNotionalAmount()!=null) return true;
			if (getMaximumNumberOfOptions()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
			super.process(path, processor);
		
			processor.processBasic(path.newSubPath("maximumNotionalAmount"), BigDecimal.class, maximumNotionalAmount, this);
			processor.processBasic(path.newSubPath("maximumNumberOfOptions"), BigDecimal.class, maximumNumberOfOptions, this);
			
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
			if (!super.equals(o)) return false;
		
			MultipleExerciseBuilder _that = (MultipleExerciseBuilder) o;
		
			if (maximumNotionalAmount != null ? !maximumNotionalAmount.equals(_that.maximumNotionalAmount) : _that.maximumNotionalAmount != null) return false;
			if (maximumNumberOfOptions != null ? !maximumNumberOfOptions.equals(_that.maximumNumberOfOptions) : _that.maximumNumberOfOptions != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = super.hashCode();
			_result = 31 * _result + (maximumNotionalAmount != null ? maximumNotionalAmount.hashCode() : 0);
			_result = 31 * _result + (maximumNumberOfOptions != null ? maximumNumberOfOptions.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "MultipleExerciseBuilder {" +
				"maximumNotionalAmount=" + this.maximumNotionalAmount + ", " +
				"maximumNumberOfOptions=" + this.maximumNumberOfOptions +
			'}' + " " + super.toString();
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		super.process(path, processor);
		
		processor.processBasic(path.newSubPath("maximumNotionalAmount"), BigDecimal.class, maximumNotionalAmount, this);
		processor.processBasic(path.newSubPath("maximumNumberOfOptions"), BigDecimal.class, maximumNumberOfOptions, this);
		
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
		if (!super.equals(o)) return false;
	
		MultipleExercise _that = (MultipleExercise) o;
	
		if (maximumNotionalAmount != null ? !maximumNotionalAmount.equals(_that.maximumNotionalAmount) : _that.maximumNotionalAmount != null) return false;
		if (maximumNumberOfOptions != null ? !maximumNumberOfOptions.equals(_that.maximumNumberOfOptions) : _that.maximumNumberOfOptions != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = super.hashCode();
		_result = 31 * _result + (maximumNotionalAmount != null ? maximumNotionalAmount.hashCode() : 0);
		_result = 31 * _result + (maximumNumberOfOptions != null ? maximumNumberOfOptions.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "MultipleExercise {" +
			"maximumNotionalAmount=" + this.maximumNotionalAmount + ", " +
			"maximumNumberOfOptions=" + this.maximumNumberOfOptions +
		'}' + " " + super.toString();
	}
}
