package org.isda.cdm;

import org.isda.cdm.metafields.FieldWithMetaNaturalPersonRoleEnum;
import org.isda.cdm.metafields.FieldWithMetaNaturalPersonRoleEnum.FieldWithMetaNaturalPersonRoleEnumBuilder;
import org.isda.cdm.metafields.ReferenceWithMetaNaturalPerson;
import org.isda.cdm.metafields.ReferenceWithMetaNaturalPerson.ReferenceWithMetaNaturalPersonBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.NaturalPersonRoleMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.stream.Collectors;
		import static java.util.Optional.ofNullable;

/**
 * A class to specify the role(s) that natural person(s) may have in relation to the contract.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class NaturalPersonRole extends RosettaModelObject {
	private final ReferenceWithMetaNaturalPerson personReference;
	private final FieldWithMetaNaturalPersonRoleEnum role;
	private static NaturalPersonRoleMeta metaData = new NaturalPersonRoleMeta();
	
	NaturalPersonRole(NaturalPersonRoleBuilder builder) {
		this.personReference = ofNullable(builder.getPersonReference()).map(f->f.build()).orElse(null);
		this.role = ofNullable(builder.getRole()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * A reference to the natural person to whom the role refers to.
	 */
	@RosettaSynonym(value="personReference", source="FpML_5_10")
	@RosettaSynonym(value="personReference", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="personReference", source="DTCC_11_0")
	@RosettaSynonym(value="personReference", source="DTCC_9_0")
	@RosettaSynonym(value="personReference", source="CME_ClearedConfirm_1_17")
	public final ReferenceWithMetaNaturalPerson getPersonReference() {
		return personReference;
	}
	
	/**
	 * FpML specifies a person role that is distinct from the party role.
	 */
	@RosettaSynonym(value="role", source="FpML_5_10")
	@RosettaSynonym(value="role", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="role", source="DTCC_11_0")
	@RosettaSynonym(value="role", source="DTCC_9_0")
	@RosettaSynonym(value="role", source="CME_ClearedConfirm_1_17")
	public final FieldWithMetaNaturalPersonRoleEnum getRole() {
		return role;
	}
	
	
	@Override
	public RosettaMetaData<? extends NaturalPersonRole> metaData() {
		return metaData;
	} 
	
	public NaturalPersonRoleBuilder toBuilder() {
		NaturalPersonRoleBuilder builder = new NaturalPersonRoleBuilder();
		ofNullable(getPersonReference()).ifPresent(builder::setPersonReference);
		ofNullable(getRole()).ifPresent(builder::setRole);
		return builder;
	}

	public static NaturalPersonRoleBuilder builder() {
		return new NaturalPersonRoleBuilder();
	}

	public static  class NaturalPersonRoleBuilder extends RosettaModelObjectBuilder {
	
		protected ReferenceWithMetaNaturalPersonBuilder personReference;
		protected FieldWithMetaNaturalPersonRoleEnumBuilder role;
	
		public NaturalPersonRoleBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends NaturalPersonRole> metaData() {
			return metaData;
		} 
	
		
		public ReferenceWithMetaNaturalPersonBuilder getPersonReference() {
			return personReference;
		}
		
		public ReferenceWithMetaNaturalPersonBuilder getOrCreatePersonReference() {
			if (personReference!=null) {
				return personReference;
			}
			else return personReference = new ReferenceWithMetaNaturalPersonBuilder();
		}
		
		
		public FieldWithMetaNaturalPersonRoleEnumBuilder getRole() {
			return role;
		}
		
		public FieldWithMetaNaturalPersonRoleEnumBuilder getOrCreateRole() {
			if (role!=null) {
				return role;
			}
			else return role = new FieldWithMetaNaturalPersonRoleEnumBuilder();
		}
		
	
		public NaturalPersonRoleBuilder setPersonReference(ReferenceWithMetaNaturalPerson personReference) {
			this.personReference = personReference.toBuilder();
			return this;
		}
		public NaturalPersonRoleBuilder setPersonReferenceBuilder(ReferenceWithMetaNaturalPersonBuilder personReference) {
			this.personReference = personReference;
			return this;
		}
		public NaturalPersonRoleBuilder setPersonReferenceRef(NaturalPerson.NaturalPersonBuilder personReference) {
			return setPersonReference(ReferenceWithMetaNaturalPerson.builder().setValueBuilder(personReference).build());
		}
		public NaturalPersonRoleBuilder setPersonReferenceRef(NaturalPerson personReference) {
			return setPersonReferenceRef(NaturalPerson.builder());
		}
		public NaturalPersonRoleBuilder setRole(FieldWithMetaNaturalPersonRoleEnum role) {
			this.role = role.toBuilder();
			return this;
		}
		public NaturalPersonRoleBuilder setRoleRef(NaturalPersonRoleEnum role) {
			return setRole(FieldWithMetaNaturalPersonRoleEnum.builder().setValue(role).build());
		}
		
		public NaturalPersonRole build() {
			return new NaturalPersonRole(this);
		}
	
		@Override
		public NaturalPersonRoleBuilder prune() {
			if (personReference!=null && !personReference.prune().hasData()) personReference = null;
			if (role!=null && !role.prune().hasData()) role = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getPersonReference()!=null && getPersonReference().hasData()) return true;
			if (getRole()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("personReference"), processor, ReferenceWithMetaNaturalPerson.class, personReference);
			processRosetta(path.newSubPath("role"), processor, FieldWithMetaNaturalPersonRoleEnum.class, role);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			NaturalPersonRoleBuilder _that = (NaturalPersonRoleBuilder) o;
		
			if (personReference != null ? !personReference.equals(_that.personReference) : _that.personReference != null) return false;
			if (role != null ? !role.equals(_that.role) : _that.role != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (personReference != null ? personReference.hashCode() : 0);
			_result = 31 * _result + (role != null ? role.getClass().getName().hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "NaturalPersonRoleBuilder {" +
				"personReference=" + this.personReference + ", " +
				"role=" + this.role +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("personReference"), processor, ReferenceWithMetaNaturalPerson.class, personReference);
		processRosetta(path.newSubPath("role"), processor, FieldWithMetaNaturalPersonRoleEnum.class, role);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		NaturalPersonRole _that = (NaturalPersonRole) o;
	
		if (personReference != null ? !personReference.equals(_that.personReference) : _that.personReference != null) return false;
		if (role != null ? !role.equals(_that.role) : _that.role != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (personReference != null ? personReference.hashCode() : 0);
		_result = 31 * _result + (role != null ? role.getClass().getName().hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "NaturalPersonRole {" +
			"personReference=" + this.personReference + ", " +
			"role=" + this.role +
		'}';
	}
}
