package org.isda.cdm;

import org.isda.cdm.AdjustableDates;
import org.isda.cdm.FxFixingDate;
import org.isda.cdm.PriceSourceDisruption;
import org.isda.cdm.metafields.FieldWithMetaSettlementRateOptionEnum;
import org.isda.cdm.metafields.FieldWithMetaSettlementRateOptionEnum.FieldWithMetaSettlementRateOptionEnumBuilder;
import org.isda.cdm.metafields.FieldWithMetaString;
import org.isda.cdm.metafields.FieldWithMetaString.FieldWithMetaStringBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.NonDeliverableSettlementMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class defining the parameters used when the reference currency of the payout is non-deliverable.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="NonDeliverableSettlement", source="FpML_5_10")
public class NonDeliverableSettlement extends RosettaModelObject {
	private final FxFixingDate fxFixingDate;
	private final AdjustableDates fxFixingSchedule;
	private final PriceSourceDisruption priceSourceDisruption;
	private final FieldWithMetaString referenceCurrency;
	private final FieldWithMetaSettlementRateOptionEnum settlementRateOption;
	private static NonDeliverableSettlementMeta metaData = new NonDeliverableSettlementMeta();
	
	NonDeliverableSettlement(NonDeliverableSettlementBuilder builder) {
		this.fxFixingDate = ofNullable(builder.getFxFixingDate()).map(f->f.build()).orElse(null);
		this.fxFixingSchedule = ofNullable(builder.getFxFixingSchedule()).map(f->f.build()).orElse(null);
		this.priceSourceDisruption = ofNullable(builder.getPriceSourceDisruption()).map(f->f.build()).orElse(null);
		this.referenceCurrency = ofNullable(builder.getReferenceCurrency()).map(f->f.build()).orElse(null);
		this.settlementRateOption = ofNullable(builder.getSettlementRateOption()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The date, when expressed as a relative date, on which the currency rate will be determined for the purpose of specifying the amount in deliverable currency.
	 */
	@RosettaSynonym(value="fxFixingDate", source="FpML_5_10")
	@RosettaSynonym(value="fxFixingDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="fxFixingDate", source="DTCC_11_0")
	@RosettaSynonym(value="fxFixingDate", source="DTCC_9_0")
	@RosettaSynonym(value="fxFixingDate", source="CME_ClearedConfirm_1_17")
	public final FxFixingDate getFxFixingDate() {
		return fxFixingDate;
	}
	
	/**
	 * The date, when expressed as a schedule of date(s), on which the currency rate will be determined for the purpose of specifying the amount in deliverable currency.
	 */
	@RosettaSynonym(value="fxFixingSchedule", source="FpML_5_10")
	@RosettaSynonym(value="fxFixingSchedule", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="fxFixingSchedule", source="DTCC_11_0")
	@RosettaSynonym(value="fxFixingSchedule", source="DTCC_9_0")
	@RosettaSynonym(value="fxFixingSchedule", source="CME_ClearedConfirm_1_17")
	public final AdjustableDates getFxFixingSchedule() {
		return fxFixingSchedule;
	}
	
	/**
	 * An attribute defining the parameters to get a new quote when a settlement rate option is disrupted.
	 */
	@RosettaSynonym(value="priceSourceDisruption", source="FpML_5_10")
	@RosettaSynonym(value="priceSourceDisruption", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="priceSourceDisruption", source="DTCC_11_0")
	@RosettaSynonym(value="priceSourceDisruption", source="DTCC_9_0")
	@RosettaSynonym(value="priceSourceDisruption", source="CME_ClearedConfirm_1_17")
	public final PriceSourceDisruption getPriceSourceDisruption() {
		return priceSourceDisruption;
	}
	
	/**
	 * The currency in which the swap stream is denominated, which is distinct from the currency in which the cashflows will be settled. The list of valid currencies is not presently positioned as an enumeration as part of the CDM because that scope is limited to the values specified by ISDA and FpML. As a result, implementers have to make reference to the relevant standard, such as the ISO 4217 standard for currency codes.
	 */
	@RosettaSynonym(value="referenceCurrency", source="FpML_5_10")
	@RosettaSynonym(value="referenceCurrency", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="referenceCurrency", source="DTCC_11_0")
	@RosettaSynonym(value="referenceCurrency", source="DTCC_9_0")
	@RosettaSynonym(value="referenceCurrency", source="CME_ClearedConfirm_1_17")
	public final FieldWithMetaString getReferenceCurrency() {
		return referenceCurrency;
	}
	
	/**
	 * The rate source for the conversion to the settlement currency. This source is specified through a scheme that reflects the terms of the Annex A to the 1998 FX and Currency Option Definitions.
	 */
	@RosettaSynonym(value="settlementRateOption", source="FpML_5_10")
	@RosettaSynonym(value="settlementRateOption", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="settlementRateOption", source="DTCC_11_0")
	@RosettaSynonym(value="settlementRateOption", source="DTCC_9_0")
	@RosettaSynonym(value="settlementRateOption", source="CME_ClearedConfirm_1_17")
	public final FieldWithMetaSettlementRateOptionEnum getSettlementRateOption() {
		return settlementRateOption;
	}
	
	
	@Override
	public RosettaMetaData<? extends NonDeliverableSettlement> metaData() {
		return metaData;
	} 
	
	public NonDeliverableSettlementBuilder toBuilder() {
		NonDeliverableSettlementBuilder builder = new NonDeliverableSettlementBuilder();
		ofNullable(getFxFixingDate()).ifPresent(builder::setFxFixingDate);
		ofNullable(getFxFixingSchedule()).ifPresent(builder::setFxFixingSchedule);
		ofNullable(getPriceSourceDisruption()).ifPresent(builder::setPriceSourceDisruption);
		ofNullable(getReferenceCurrency()).ifPresent(builder::setReferenceCurrency);
		ofNullable(getSettlementRateOption()).ifPresent(builder::setSettlementRateOption);
		return builder;
	}

	public static NonDeliverableSettlementBuilder builder() {
		return new NonDeliverableSettlementBuilder();
	}

	public static  class NonDeliverableSettlementBuilder extends RosettaModelObjectBuilder {
	
		protected FxFixingDate.FxFixingDateBuilder fxFixingDate;
		protected AdjustableDates.AdjustableDatesBuilder fxFixingSchedule;
		protected PriceSourceDisruption.PriceSourceDisruptionBuilder priceSourceDisruption;
		protected FieldWithMetaStringBuilder referenceCurrency;
		protected FieldWithMetaSettlementRateOptionEnumBuilder settlementRateOption;
	
		public NonDeliverableSettlementBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends NonDeliverableSettlement> metaData() {
			return metaData;
		} 
	
		
		public FxFixingDate.FxFixingDateBuilder getFxFixingDate() {
			return fxFixingDate;
		}
		
		public FxFixingDate.FxFixingDateBuilder getOrCreateFxFixingDate() {
			if (fxFixingDate!=null) {
				return fxFixingDate;
			}
			else return fxFixingDate = new FxFixingDate.FxFixingDateBuilder();
		}
		
		
		public AdjustableDates.AdjustableDatesBuilder getFxFixingSchedule() {
			return fxFixingSchedule;
		}
		
		public AdjustableDates.AdjustableDatesBuilder getOrCreateFxFixingSchedule() {
			if (fxFixingSchedule!=null) {
				return fxFixingSchedule;
			}
			else return fxFixingSchedule = new AdjustableDates.AdjustableDatesBuilder();
		}
		
		
		public PriceSourceDisruption.PriceSourceDisruptionBuilder getPriceSourceDisruption() {
			return priceSourceDisruption;
		}
		
		public PriceSourceDisruption.PriceSourceDisruptionBuilder getOrCreatePriceSourceDisruption() {
			if (priceSourceDisruption!=null) {
				return priceSourceDisruption;
			}
			else return priceSourceDisruption = new PriceSourceDisruption.PriceSourceDisruptionBuilder();
		}
		
		
		public FieldWithMetaStringBuilder getReferenceCurrency() {
			return referenceCurrency;
		}
		
		public FieldWithMetaStringBuilder getOrCreateReferenceCurrency() {
			if (referenceCurrency!=null) {
				return referenceCurrency;
			}
			else return referenceCurrency = new FieldWithMetaStringBuilder();
		}
		
		
		public FieldWithMetaSettlementRateOptionEnumBuilder getSettlementRateOption() {
			return settlementRateOption;
		}
		
		public FieldWithMetaSettlementRateOptionEnumBuilder getOrCreateSettlementRateOption() {
			if (settlementRateOption!=null) {
				return settlementRateOption;
			}
			else return settlementRateOption = new FieldWithMetaSettlementRateOptionEnumBuilder();
		}
		
	
		public NonDeliverableSettlementBuilder setFxFixingDate(FxFixingDate fxFixingDate) {
			this.fxFixingDate = fxFixingDate.toBuilder();
			return this;
		}
		public NonDeliverableSettlementBuilder setFxFixingDateBuilder(FxFixingDate.FxFixingDateBuilder fxFixingDate) {
			this.fxFixingDate = fxFixingDate;
			return this;
		}
		public NonDeliverableSettlementBuilder setFxFixingSchedule(AdjustableDates fxFixingSchedule) {
			this.fxFixingSchedule = fxFixingSchedule.toBuilder();
			return this;
		}
		public NonDeliverableSettlementBuilder setFxFixingScheduleBuilder(AdjustableDates.AdjustableDatesBuilder fxFixingSchedule) {
			this.fxFixingSchedule = fxFixingSchedule;
			return this;
		}
		public NonDeliverableSettlementBuilder setPriceSourceDisruption(PriceSourceDisruption priceSourceDisruption) {
			this.priceSourceDisruption = priceSourceDisruption.toBuilder();
			return this;
		}
		public NonDeliverableSettlementBuilder setPriceSourceDisruptionBuilder(PriceSourceDisruption.PriceSourceDisruptionBuilder priceSourceDisruption) {
			this.priceSourceDisruption = priceSourceDisruption;
			return this;
		}
		public NonDeliverableSettlementBuilder setReferenceCurrency(FieldWithMetaString referenceCurrency) {
			this.referenceCurrency = referenceCurrency.toBuilder();
			return this;
		}
		public NonDeliverableSettlementBuilder setReferenceCurrencyRef(String referenceCurrency) {
			return setReferenceCurrency(FieldWithMetaString.builder().setValue(referenceCurrency).build());
		}
		public NonDeliverableSettlementBuilder setSettlementRateOption(FieldWithMetaSettlementRateOptionEnum settlementRateOption) {
			this.settlementRateOption = settlementRateOption.toBuilder();
			return this;
		}
		public NonDeliverableSettlementBuilder setSettlementRateOptionRef(SettlementRateOptionEnum settlementRateOption) {
			return setSettlementRateOption(FieldWithMetaSettlementRateOptionEnum.builder().setValue(settlementRateOption).build());
		}
		
		public NonDeliverableSettlement build() {
			return new NonDeliverableSettlement(this);
		}
	
		@Override
		public NonDeliverableSettlementBuilder prune() {
			if (fxFixingDate!=null && !fxFixingDate.prune().hasData()) fxFixingDate = null;
			if (fxFixingSchedule!=null && !fxFixingSchedule.prune().hasData()) fxFixingSchedule = null;
			if (priceSourceDisruption!=null && !priceSourceDisruption.prune().hasData()) priceSourceDisruption = null;
			if (referenceCurrency!=null && !referenceCurrency.prune().hasData()) referenceCurrency = null;
			if (settlementRateOption!=null && !settlementRateOption.prune().hasData()) settlementRateOption = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getFxFixingDate()!=null && getFxFixingDate().hasData()) return true;
			if (getFxFixingSchedule()!=null && getFxFixingSchedule().hasData()) return true;
			if (getPriceSourceDisruption()!=null && getPriceSourceDisruption().hasData()) return true;
			if (getReferenceCurrency()!=null) return true;
			if (getSettlementRateOption()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("fxFixingDate"), processor, FxFixingDate.class, fxFixingDate);
			processRosetta(path.newSubPath("fxFixingSchedule"), processor, AdjustableDates.class, fxFixingSchedule);
			processRosetta(path.newSubPath("priceSourceDisruption"), processor, PriceSourceDisruption.class, priceSourceDisruption);
			processRosetta(path.newSubPath("referenceCurrency"), processor, FieldWithMetaString.class, referenceCurrency);
			processRosetta(path.newSubPath("settlementRateOption"), processor, FieldWithMetaSettlementRateOptionEnum.class, settlementRateOption);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			NonDeliverableSettlementBuilder _that = (NonDeliverableSettlementBuilder) o;
		
			if (fxFixingDate != null ? !fxFixingDate.equals(_that.fxFixingDate) : _that.fxFixingDate != null) return false;
			if (fxFixingSchedule != null ? !fxFixingSchedule.equals(_that.fxFixingSchedule) : _that.fxFixingSchedule != null) return false;
			if (priceSourceDisruption != null ? !priceSourceDisruption.equals(_that.priceSourceDisruption) : _that.priceSourceDisruption != null) return false;
			if (referenceCurrency != null ? !referenceCurrency.equals(_that.referenceCurrency) : _that.referenceCurrency != null) return false;
			if (settlementRateOption != null ? !settlementRateOption.equals(_that.settlementRateOption) : _that.settlementRateOption != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (fxFixingDate != null ? fxFixingDate.hashCode() : 0);
			_result = 31 * _result + (fxFixingSchedule != null ? fxFixingSchedule.hashCode() : 0);
			_result = 31 * _result + (priceSourceDisruption != null ? priceSourceDisruption.hashCode() : 0);
			_result = 31 * _result + (referenceCurrency != null ? referenceCurrency.hashCode() : 0);
			_result = 31 * _result + (settlementRateOption != null ? settlementRateOption.getClass().getName().hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "NonDeliverableSettlementBuilder {" +
				"fxFixingDate=" + this.fxFixingDate + ", " +
				"fxFixingSchedule=" + this.fxFixingSchedule + ", " +
				"priceSourceDisruption=" + this.priceSourceDisruption + ", " +
				"referenceCurrency=" + this.referenceCurrency + ", " +
				"settlementRateOption=" + this.settlementRateOption +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("fxFixingDate"), processor, FxFixingDate.class, fxFixingDate);
		processRosetta(path.newSubPath("fxFixingSchedule"), processor, AdjustableDates.class, fxFixingSchedule);
		processRosetta(path.newSubPath("priceSourceDisruption"), processor, PriceSourceDisruption.class, priceSourceDisruption);
		processRosetta(path.newSubPath("referenceCurrency"), processor, FieldWithMetaString.class, referenceCurrency);
		processRosetta(path.newSubPath("settlementRateOption"), processor, FieldWithMetaSettlementRateOptionEnum.class, settlementRateOption);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		NonDeliverableSettlement _that = (NonDeliverableSettlement) o;
	
		if (fxFixingDate != null ? !fxFixingDate.equals(_that.fxFixingDate) : _that.fxFixingDate != null) return false;
		if (fxFixingSchedule != null ? !fxFixingSchedule.equals(_that.fxFixingSchedule) : _that.fxFixingSchedule != null) return false;
		if (priceSourceDisruption != null ? !priceSourceDisruption.equals(_that.priceSourceDisruption) : _that.priceSourceDisruption != null) return false;
		if (referenceCurrency != null ? !referenceCurrency.equals(_that.referenceCurrency) : _that.referenceCurrency != null) return false;
		if (settlementRateOption != null ? !settlementRateOption.equals(_that.settlementRateOption) : _that.settlementRateOption != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (fxFixingDate != null ? fxFixingDate.hashCode() : 0);
		_result = 31 * _result + (fxFixingSchedule != null ? fxFixingSchedule.hashCode() : 0);
		_result = 31 * _result + (priceSourceDisruption != null ? priceSourceDisruption.hashCode() : 0);
		_result = 31 * _result + (referenceCurrency != null ? referenceCurrency.hashCode() : 0);
		_result = 31 * _result + (settlementRateOption != null ? settlementRateOption.getClass().getName().hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "NonDeliverableSettlement {" +
			"fxFixingDate=" + this.fxFixingDate + ", " +
			"fxFixingSchedule=" + this.fxFixingSchedule + ", " +
			"priceSourceDisruption=" + this.priceSourceDisruption + ", " +
			"referenceCurrency=" + this.referenceCurrency + ", " +
			"settlementRateOption=" + this.settlementRateOption +
		'}';
	}
}
