package org.isda.cdm;

import org.isda.cdm.metafields.FieldWithMetaString;
import org.isda.cdm.metafields.FieldWithMetaString.FieldWithMetaStringBuilder;


import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.NonNegativeAmountScheduleMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.math.BigDecimal;
		import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;

/**
 * A class defining a currency amount or a currency amount schedule.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="NonNegativeAmountSchedule", source="FpML_5_10")
public class NonNegativeAmountSchedule extends NonNegativeSchedule {
	private final FieldWithMetaString currency;
	private static NonNegativeAmountScheduleMeta metaData = new NonNegativeAmountScheduleMeta();
	
	NonNegativeAmountSchedule(NonNegativeAmountScheduleBuilder builder) {
		super(builder);
		this.currency = ofNullable(builder.getCurrency()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The currency in which the notional amount schedule is denominated. The currency is specified outside of the actual schedule in order to be applied uniformly to it. The list of valid currencies is not presently positioned as an enumeration as part of the CDM because that scope is limited to the values specified by ISDA and FpML. As a result, implementers have to make reference to the relevant standard, such as the ISO 4217 standard for currency codes.
	 */
	@RosettaSynonym(value="currency", source="FpML_5_10")
	@RosettaSynonym(value="currency", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="currency", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="currency", source="DTCC_11_0", maps=2)
	@RosettaSynonym(value="currency", source="DTCC_9_0", maps=2)
	public final FieldWithMetaString getCurrency() {
		return currency;
	}
	
	
	@Override
	public RosettaMetaData<? extends NonNegativeAmountSchedule> metaData() {
		return metaData;
	} 
	
	public NonNegativeAmountScheduleBuilder toBuilder() {
		NonNegativeAmountScheduleBuilder builder = new NonNegativeAmountScheduleBuilder();
		ofNullable(getCurrency()).ifPresent(builder::setCurrency);
		ofNullable(getInitialValue()).ifPresent(builder::setInitialValue);
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		ofNullable(getStep()).ifPresent(step -> step.forEach(builder::addStep));
		return builder;
	}

	public static NonNegativeAmountScheduleBuilder builder() {
		return new NonNegativeAmountScheduleBuilder();
	}

	public static  class NonNegativeAmountScheduleBuilder extends NonNegativeScheduleBuilder {
	
		protected FieldWithMetaStringBuilder currency;
	
		public NonNegativeAmountScheduleBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends NonNegativeAmountSchedule> metaData() {
			return metaData;
		} 
	
		
		public FieldWithMetaStringBuilder getCurrency() {
			return currency;
		}
		
		public FieldWithMetaStringBuilder getOrCreateCurrency() {
			if (currency!=null) {
				return currency;
			}
			else return currency = new FieldWithMetaStringBuilder();
		}
		
	
		public NonNegativeAmountScheduleBuilder setCurrency(FieldWithMetaString currency) {
			this.currency = currency.toBuilder();
			return this;
		}
		public NonNegativeAmountScheduleBuilder setCurrencyRef(String currency) {
			return setCurrency(FieldWithMetaString.builder().setValue(currency).build());
		}
		@Override public NonNegativeAmountScheduleBuilder setInitialValue(BigDecimal initialValue) {
			this.initialValue = initialValue;
			return this;
		}
		@Override public NonNegativeAmountScheduleBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		@Override public NonNegativeAmountScheduleBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		@Override public NonNegativeAmountScheduleBuilder addStep(NonNegativeStep step) {
			if(this.step == null){
				this.step = new ArrayList<>();
			}
			this.step.add(step.toBuilder());
			return this;
		}
		@Override public NonNegativeAmountScheduleBuilder addStep(NonNegativeStep step, int _idx) {
			if(this.step == null){
				this.step = new ArrayList<>();
			}
			getIndex(this.step, _idx, () -> step.toBuilder());
			this.step.set(_idx, step.toBuilder());
			return this;
		}
		@Override public NonNegativeAmountScheduleBuilder addStep(List<NonNegativeStep> steps) {
			if(this.step == null){
				this.step = new ArrayList<>();
			}
			for (NonNegativeStep toAdd : steps) {
				this.step.add(toAdd.toBuilder());
			}
			return this;
		}
		@Override public NonNegativeAmountScheduleBuilder addStepBuilder(NonNegativeStep.NonNegativeStepBuilder step) {
			if(this.step == null){
				this.step = new ArrayList<>();
			}
			this.step.add(step);
			return this;
		}
		
		@Override public NonNegativeAmountScheduleBuilder clearStep() {
			this.step = null;
			return this;
		}
		
		public NonNegativeAmountSchedule build() {
			return new NonNegativeAmountSchedule(this);
		}
	
		@Override
		public NonNegativeAmountScheduleBuilder prune() {
			super.prune();
			if (currency!=null && !currency.prune().hasData()) currency = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (super.hasData()) return true;
			if (getCurrency()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
			super.process(path, processor);
		
			
			processRosetta(path.newSubPath("currency"), processor, FieldWithMetaString.class, currency);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
			if (!super.equals(o)) return false;
		
			NonNegativeAmountScheduleBuilder _that = (NonNegativeAmountScheduleBuilder) o;
		
			if (currency != null ? !currency.equals(_that.currency) : _that.currency != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = super.hashCode();
			_result = 31 * _result + (currency != null ? currency.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "NonNegativeAmountScheduleBuilder {" +
				"currency=" + this.currency +
			'}' + " " + super.toString();
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		super.process(path, processor);
		
		
		processRosetta(path.newSubPath("currency"), processor, FieldWithMetaString.class, currency);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
		if (!super.equals(o)) return false;
	
		NonNegativeAmountSchedule _that = (NonNegativeAmountSchedule) o;
	
		if (currency != null ? !currency.equals(_that.currency) : _that.currency != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = super.hashCode();
		_result = 31 * _result + (currency != null ? currency.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "NonNegativeAmountSchedule {" +
			"currency=" + this.currency +
		'}' + " " + super.toString();
	}
}
