package org.isda.cdm;

import org.isda.cdm.NonNegativeStepSchedule;
import org.isda.cdm.metafields.FieldWithMetaString;


import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.NonNegativeQuantityScheduleMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;
		import java.math.BigDecimal;

/**
 * Class to specify a non-negative quantity schedule, which is used to define the quantity of a payout leg. This quantity cannot be negative, while direction is specified through a BuyerSeller or PayerReceiver attribute. The only available schedule implementation is a step schedule, specified as a set of date-value pairs. The non-negativity is enforced by extending the non-negative quantity and step schedule classes.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class NonNegativeQuantitySchedule extends NonNegativeQuantity {
	private final NonNegativeStepSchedule stepSchedule;
	private static NonNegativeQuantityScheduleMeta metaData = new NonNegativeQuantityScheduleMeta();
	
	NonNegativeQuantitySchedule(NonNegativeQuantityScheduleBuilder builder) {
		super(builder);
		this.stepSchedule = ofNullable(builder.getStepSchedule()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The schedule specified as a set of date-value pairs. Attribute is optional, to represent the case where no schedule is specified so quantity is just constant over time.
	 */
	@RosettaSynonym(value="notionalStepSchedule", source="FpML_5_10")
	@RosettaSynonym(value="notionalStepSchedule", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="notionalStepSchedule", source="DTCC_11_0")
	@RosettaSynonym(value="notionalStepSchedule", source="DTCC_9_0")
	@RosettaSynonym(value="notionalStepSchedule", source="CME_ClearedConfirm_1_17")
	public final NonNegativeStepSchedule getStepSchedule() {
		return stepSchedule;
	}
	
	
	@Override
	public RosettaMetaData<? extends NonNegativeQuantitySchedule> metaData() {
		return metaData;
	} 
	
	public NonNegativeQuantityScheduleBuilder toBuilder() {
		NonNegativeQuantityScheduleBuilder builder = new NonNegativeQuantityScheduleBuilder();
		ofNullable(getStepSchedule()).ifPresent(builder::setStepSchedule);
		ofNullable(getAmount()).ifPresent(builder::setAmount);
		ofNullable(getCurrency()).ifPresent(builder::setCurrency);
		ofNullable(getUnit()).ifPresent(builder::setUnit);
		return builder;
	}

	public static NonNegativeQuantityScheduleBuilder builder() {
		return new NonNegativeQuantityScheduleBuilder();
	}

	public static  class NonNegativeQuantityScheduleBuilder extends NonNegativeQuantityBuilder {
	
		protected NonNegativeStepSchedule.NonNegativeStepScheduleBuilder stepSchedule;
	
		public NonNegativeQuantityScheduleBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends NonNegativeQuantitySchedule> metaData() {
			return metaData;
		} 
	
		
		public NonNegativeStepSchedule.NonNegativeStepScheduleBuilder getStepSchedule() {
			return stepSchedule;
		}
		
		public NonNegativeStepSchedule.NonNegativeStepScheduleBuilder getOrCreateStepSchedule() {
			if (stepSchedule!=null) {
				return stepSchedule;
			}
			else return stepSchedule = new NonNegativeStepSchedule.NonNegativeStepScheduleBuilder();
		}
		
	
		public NonNegativeQuantityScheduleBuilder setStepSchedule(NonNegativeStepSchedule stepSchedule) {
			this.stepSchedule = stepSchedule.toBuilder();
			return this;
		}
		public NonNegativeQuantityScheduleBuilder setStepScheduleBuilder(NonNegativeStepSchedule.NonNegativeStepScheduleBuilder stepSchedule) {
			this.stepSchedule = stepSchedule;
			return this;
		}
		@Override public NonNegativeQuantityScheduleBuilder setAmount(BigDecimal amount) {
			this.amount = amount;
			return this;
		}
		@Override public NonNegativeQuantityScheduleBuilder setCurrency(FieldWithMetaString currency) {
			this.currency = currency.toBuilder();
			return this;
		}
		@Override public NonNegativeQuantityScheduleBuilder setCurrencyRef(String currency) {
			return setCurrency(FieldWithMetaString.builder().setValue(currency).build());
		}
		@Override public NonNegativeQuantityScheduleBuilder setUnit(UnitEnum unit) {
			this.unit = unit;
			return this;
		}
		
		public NonNegativeQuantitySchedule build() {
			return new NonNegativeQuantitySchedule(this);
		}
	
		@Override
		public NonNegativeQuantityScheduleBuilder prune() {
			super.prune();
			if (stepSchedule!=null && !stepSchedule.prune().hasData()) stepSchedule = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (super.hasData()) return true;
			if (getStepSchedule()!=null && getStepSchedule().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
			super.process(path, processor);
		
			
			processRosetta(path.newSubPath("stepSchedule"), processor, NonNegativeStepSchedule.class, stepSchedule);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
			if (!super.equals(o)) return false;
		
			NonNegativeQuantityScheduleBuilder _that = (NonNegativeQuantityScheduleBuilder) o;
		
			if (stepSchedule != null ? !stepSchedule.equals(_that.stepSchedule) : _that.stepSchedule != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = super.hashCode();
			_result = 31 * _result + (stepSchedule != null ? stepSchedule.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "NonNegativeQuantityScheduleBuilder {" +
				"stepSchedule=" + this.stepSchedule +
			'}' + " " + super.toString();
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		super.process(path, processor);
		
		
		processRosetta(path.newSubPath("stepSchedule"), processor, NonNegativeStepSchedule.class, stepSchedule);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
		if (!super.equals(o)) return false;
	
		NonNegativeQuantitySchedule _that = (NonNegativeQuantitySchedule) o;
	
		if (stepSchedule != null ? !stepSchedule.equals(_that.stepSchedule) : _that.stepSchedule != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = super.hashCode();
		_result = 31 * _result + (stepSchedule != null ? stepSchedule.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "NonNegativeQuantitySchedule {" +
			"stepSchedule=" + this.stepSchedule +
		'}' + " " + super.toString();
	}
}
