package org.isda.cdm;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.NonNegativeStep;
import org.isda.cdm.metafields.MetaFields;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.NonNegativeScheduleMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.math.BigDecimal;
		import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;

/**
 * A class defining a schedule of non-negative rates or amounts in terms of an initial value and then a series of step date and value pairs. On each step date the rate or amount changes to the new step value. The series of step date and value pairs are optional. If not specified, this implies that the initial value remains unchanged over time.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="NonNegativeSchedule", source="FpML_5_10", path="notionalStepSchedule" )
@RosettaSynonym(value="NonNegativeSchedule", source="CME_SubmissionIRS_1_0", path="notionalStepSchedule" )
@RosettaSynonym(value="NonNegativeSchedule", source="DTCC_11_0", path="notionalStepSchedule" )
@RosettaSynonym(value="NonNegativeSchedule", source="DTCC_9_0", path="notionalStepSchedule" )
@RosettaSynonym(value="NonNegativeSchedule", source="CME_ClearedConfirm_1_17", path="notionalStepSchedule" )
public class NonNegativeSchedule extends RosettaModelObject implements GlobalKey {
	private final BigDecimal initialValue;
	private final MetaFields meta;
	private final List<NonNegativeStep> step;
	private static NonNegativeScheduleMeta metaData = new NonNegativeScheduleMeta();
	
	NonNegativeSchedule(NonNegativeScheduleBuilder builder) {
		this.initialValue = builder.getInitialValue();
		this.meta = ofNullable(builder.getMeta()).map(f->f.build()).orElse(null);
		this.step = ofNullable(builder.getStep()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
	}
	
	/**
	 * The non-negative initial rate or amount, as the case may be. An initial rate of 5% would be represented as 0.05.
	 */
	@RosettaSynonym(value="initialValue", source="FpML_5_10")
	@RosettaSynonym(value="initialValue", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="initialValue", source="DTCC_11_0")
	@RosettaSynonym(value="initialValue", source="DTCC_9_0")
	@RosettaSynonym(value="initialValue", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="amount", source="DTCC_11_0", maps=2)
	@RosettaSynonym(value="amount", source="DTCC_9_0", maps=2)
	public final BigDecimal getInitialValue() {
		return initialValue;
	}
	
	public final MetaFields getMeta() {
		return meta;
	}
	
	/**
	 * The schedule of step date and non-negative value pairs. On each step date the associated step value becomes effective. A list of steps may be ordered in the document by ascending step date. An FpML document containing an unordered list of steps is still regarded as a conformant document.
	 */
	@RosettaSynonym(value="step", source="FpML_5_10")
	@RosettaSynonym(value="step", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="step", source="DTCC_11_0")
	@RosettaSynonym(value="step", source="DTCC_9_0")
	@RosettaSynonym(value="step", source="CME_ClearedConfirm_1_17")
	public final List<NonNegativeStep> getStep() {
		return step;
	}
	
	
	@Override
	public RosettaMetaData<? extends NonNegativeSchedule> metaData() {
		return metaData;
	} 
	
	public NonNegativeScheduleBuilder toBuilder() {
		NonNegativeScheduleBuilder builder = new NonNegativeScheduleBuilder();
		ofNullable(getInitialValue()).ifPresent(builder::setInitialValue);
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		ofNullable(getStep()).ifPresent(step -> step.forEach(builder::addStep));
		return builder;
	}

	public static NonNegativeScheduleBuilder builder() {
		return new NonNegativeScheduleBuilder();
	}

	public static  class NonNegativeScheduleBuilder extends RosettaModelObjectBuilder implements GlobalKeyBuilder<NonNegativeScheduleBuilder> {
	
		protected BigDecimal initialValue;
		protected MetaFields.MetaFieldsBuilder meta;
		protected List<NonNegativeStep.NonNegativeStepBuilder> step;
	
		public NonNegativeScheduleBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends NonNegativeSchedule> metaData() {
			return metaData;
		} 
	
		
		public BigDecimal getInitialValue() {
			return initialValue;
		}
		
		
		public MetaFields.MetaFieldsBuilder getMeta() {
			return meta;
		}
		
		public MetaFields.MetaFieldsBuilder getOrCreateMeta() {
			if (meta!=null) {
				return meta;
			}
			else return meta = new MetaFields.MetaFieldsBuilder();
		}
		
		
		public List<NonNegativeStep.NonNegativeStepBuilder> getStep() {
			return step;
		}
		
		public NonNegativeStep.NonNegativeStepBuilder getOrCreateStep(int index) {
			if (step==null) {
				this.step = new ArrayList<>();
			}
			return getIndex(step, index, ()->new NonNegativeStep.NonNegativeStepBuilder());
		}
		
	
		public NonNegativeScheduleBuilder setInitialValue(BigDecimal initialValue) {
			this.initialValue = initialValue;
			return this;
		}
		public NonNegativeScheduleBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		public NonNegativeScheduleBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		public NonNegativeScheduleBuilder addStep(NonNegativeStep step) {
			if(this.step == null){
				this.step = new ArrayList<>();
			}
			this.step.add(step.toBuilder());
			return this;
		}
		public NonNegativeScheduleBuilder addStep(NonNegativeStep step, int _idx) {
			if(this.step == null){
				this.step = new ArrayList<>();
			}
			getIndex(this.step, _idx, () -> step.toBuilder());
			this.step.set(_idx, step.toBuilder());
			return this;
		}
		public NonNegativeScheduleBuilder addStep(List<NonNegativeStep> steps) {
			if(this.step == null){
				this.step = new ArrayList<>();
			}
			for (NonNegativeStep toAdd : steps) {
				this.step.add(toAdd.toBuilder());
			}
			return this;
		}
		public NonNegativeScheduleBuilder addStepBuilder(NonNegativeStep.NonNegativeStepBuilder step) {
			if(this.step == null){
				this.step = new ArrayList<>();
			}
			this.step.add(step);
			return this;
		}
		
		public NonNegativeScheduleBuilder clearStep() {
			this.step = null;
			return this;
		}
		
		public NonNegativeSchedule build() {
			return new NonNegativeSchedule(this);
		}
	
		@Override
		public NonNegativeScheduleBuilder prune() {
			if (meta!=null && !meta.prune().hasData()) meta = null;
			if (step!=null) step = step.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getInitialValue()!=null) return true;
			if (getMeta()!=null && getMeta().hasData()) return true;
			if (getStep()!=null && getStep().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("initialValue"), BigDecimal.class, initialValue, this);
			
			processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
			processRosetta(path.newSubPath("step"), processor, NonNegativeStep.class, step);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			NonNegativeScheduleBuilder _that = (NonNegativeScheduleBuilder) o;
		
			if (initialValue != null ? !initialValue.equals(_that.initialValue) : _that.initialValue != null) return false;
			if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
			if (!ListEquals.listEquals(step, _that.step)) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (initialValue != null ? initialValue.hashCode() : 0);
			_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
			_result = 31 * _result + (step != null ? step.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "NonNegativeScheduleBuilder {" +
				"initialValue=" + this.initialValue + ", " +
				"meta=" + this.meta + ", " +
				"step=" + this.step +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("initialValue"), BigDecimal.class, initialValue, this);
		
		processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
		processRosetta(path.newSubPath("step"), processor, NonNegativeStep.class, step);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		NonNegativeSchedule _that = (NonNegativeSchedule) o;
	
		if (initialValue != null ? !initialValue.equals(_that.initialValue) : _that.initialValue != null) return false;
		if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
		if (!ListEquals.listEquals(step, _that.step)) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (initialValue != null ? initialValue.hashCode() : 0);
		_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
		_result = 31 * _result + (step != null ? step.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "NonNegativeSchedule {" +
			"initialValue=" + this.initialValue + ", " +
			"meta=" + this.meta + ", " +
			"step=" + this.step +
		'}';
	}
}
