package org.isda.cdm;

import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.NonNegativeStep;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.NonNegativeStepScheduleMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;

/**
 * Class to specify a non-negative schedule as a schedule of steps, typically used to define a payout leg with variable quantity. This quantity cannot be negative, while direction is specified through a BuyerSeller or PayerReceiver attribute. The non-negativity is enforced by making use of the non-negative step class.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class NonNegativeStepSchedule extends RosettaModelObject {
	private final List<NonNegativeStep> step;
	private static NonNegativeStepScheduleMeta metaData = new NonNegativeStepScheduleMeta();
	
	NonNegativeStepSchedule(NonNegativeStepScheduleBuilder builder) {
		this.step = ofNullable(builder.getStep()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
	}
	
	/**
	 * The schedule of step date and non-negative value pairs. On each step date the associated step value becomes effective. A list of steps may be ordered in the document by ascending step date. An FpML document containing an unordered list of steps is still regarded as a conformant document.
	 */
	@RosettaSynonym(value="step", source="FpML_5_10")
	@RosettaSynonym(value="step", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="step", source="DTCC_11_0")
	@RosettaSynonym(value="step", source="DTCC_9_0")
	@RosettaSynonym(value="step", source="CME_ClearedConfirm_1_17")
	public final List<NonNegativeStep> getStep() {
		return step;
	}
	
	
	@Override
	public RosettaMetaData<? extends NonNegativeStepSchedule> metaData() {
		return metaData;
	} 
	
	public NonNegativeStepScheduleBuilder toBuilder() {
		NonNegativeStepScheduleBuilder builder = new NonNegativeStepScheduleBuilder();
		ofNullable(getStep()).ifPresent(step -> step.forEach(builder::addStep));
		return builder;
	}

	public static NonNegativeStepScheduleBuilder builder() {
		return new NonNegativeStepScheduleBuilder();
	}

	public static  class NonNegativeStepScheduleBuilder extends RosettaModelObjectBuilder {
	
		protected List<NonNegativeStep.NonNegativeStepBuilder> step;
	
		public NonNegativeStepScheduleBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends NonNegativeStepSchedule> metaData() {
			return metaData;
		} 
	
		
		public List<NonNegativeStep.NonNegativeStepBuilder> getStep() {
			return step;
		}
		
		public NonNegativeStep.NonNegativeStepBuilder getOrCreateStep(int index) {
			if (step==null) {
				this.step = new ArrayList<>();
			}
			return getIndex(step, index, ()->new NonNegativeStep.NonNegativeStepBuilder());
		}
		
	
		public NonNegativeStepScheduleBuilder addStep(NonNegativeStep step) {
			if(this.step == null){
				this.step = new ArrayList<>();
			}
			this.step.add(step.toBuilder());
			return this;
		}
		public NonNegativeStepScheduleBuilder addStep(NonNegativeStep step, int _idx) {
			if(this.step == null){
				this.step = new ArrayList<>();
			}
			getIndex(this.step, _idx, () -> step.toBuilder());
			this.step.set(_idx, step.toBuilder());
			return this;
		}
		public NonNegativeStepScheduleBuilder addStep(List<NonNegativeStep> steps) {
			if(this.step == null){
				this.step = new ArrayList<>();
			}
			for (NonNegativeStep toAdd : steps) {
				this.step.add(toAdd.toBuilder());
			}
			return this;
		}
		public NonNegativeStepScheduleBuilder addStepBuilder(NonNegativeStep.NonNegativeStepBuilder step) {
			if(this.step == null){
				this.step = new ArrayList<>();
			}
			this.step.add(step);
			return this;
		}
		
		public NonNegativeStepScheduleBuilder clearStep() {
			this.step = null;
			return this;
		}
		
		public NonNegativeStepSchedule build() {
			return new NonNegativeStepSchedule(this);
		}
	
		@Override
		public NonNegativeStepScheduleBuilder prune() {
			if (step!=null) step = step.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getStep()!=null && getStep().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("step"), processor, NonNegativeStep.class, step);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			NonNegativeStepScheduleBuilder _that = (NonNegativeStepScheduleBuilder) o;
		
			if (!ListEquals.listEquals(step, _that.step)) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (step != null ? step.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "NonNegativeStepScheduleBuilder {" +
				"step=" + this.step +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("step"), processor, NonNegativeStep.class, step);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		NonNegativeStepSchedule _that = (NonNegativeStepSchedule) o;
	
		if (!ListEquals.listEquals(step, _that.step)) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (step != null ? step.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "NonNegativeStepSchedule {" +
			"step=" + this.step +
		'}';
	}
}
