package org.isda.cdm;

import java.lang.String;
import org.isda.cdm.BusinessCenterTime;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.NotificationTimeElectionMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class to specify the notification time election by the respective parties to the agreement. ISDA 2016 English Law Credit Support Deed for Initial Margin, paragraph 13, General Principles, (d)(iii): Notification Time. | ISDA 2016 Japanese Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (e)(iii): Notification Time. | ISDA 2016 New York Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (d)(iii): Notification Time.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class NotificationTimeElection extends RosettaModelObject {
	private final String customNotification;
	private final BusinessCenterTime notificationTime;
	private final String party;
	private static NotificationTimeElectionMeta metaData = new NotificationTimeElectionMeta();
	
	NotificationTimeElection(NotificationTimeElectionBuilder builder) {
		this.customNotification = builder.getCustomNotification();
		this.notificationTime = ofNullable(builder.getNotificationTime()).map(f->f.build()).orElse(null);
		this.party = builder.getParty();
	}
	
	/**
	 * The Notification Time as a custom election.
	 */
	@RosettaSynonym(value="partyA_specify", source="ISDA_Create_1_0")
	@RosettaSynonym(value="partyB_specify", source="ISDA_Create_1_0")
	public final String getCustomNotification() {
		return customNotification;
	}
	
	/**
	 * The Notification Time as a time that is qualified as a standard business center.
	 */
	public final BusinessCenterTime getNotificationTime() {
		return notificationTime;
	}
	
	/**
	 * The elective party.
	 */
	@RosettaSynonym(value="", source="ISDA_Create_1_0")
	@RosettaSynonym(value="", source="ISDA_Create_1_0")
	public final String getParty() {
		return party;
	}
	
	
	@Override
	public RosettaMetaData<? extends NotificationTimeElection> metaData() {
		return metaData;
	} 
	
	public NotificationTimeElectionBuilder toBuilder() {
		NotificationTimeElectionBuilder builder = new NotificationTimeElectionBuilder();
		ofNullable(getCustomNotification()).ifPresent(builder::setCustomNotification);
		ofNullable(getNotificationTime()).ifPresent(builder::setNotificationTime);
		ofNullable(getParty()).ifPresent(builder::setParty);
		return builder;
	}

	public static NotificationTimeElectionBuilder builder() {
		return new NotificationTimeElectionBuilder();
	}

	public static  class NotificationTimeElectionBuilder extends RosettaModelObjectBuilder {
	
		protected String customNotification;
		protected BusinessCenterTime.BusinessCenterTimeBuilder notificationTime;
		protected String party;
	
		public NotificationTimeElectionBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends NotificationTimeElection> metaData() {
			return metaData;
		} 
	
		
		public String getCustomNotification() {
			return customNotification;
		}
		
		
		public BusinessCenterTime.BusinessCenterTimeBuilder getNotificationTime() {
			return notificationTime;
		}
		
		public BusinessCenterTime.BusinessCenterTimeBuilder getOrCreateNotificationTime() {
			if (notificationTime!=null) {
				return notificationTime;
			}
			else return notificationTime = new BusinessCenterTime.BusinessCenterTimeBuilder();
		}
		
		
		public String getParty() {
			return party;
		}
		
	
		public NotificationTimeElectionBuilder setCustomNotification(String customNotification) {
			this.customNotification = customNotification;
			return this;
		}
		public NotificationTimeElectionBuilder setNotificationTime(BusinessCenterTime notificationTime) {
			this.notificationTime = notificationTime.toBuilder();
			return this;
		}
		public NotificationTimeElectionBuilder setNotificationTimeBuilder(BusinessCenterTime.BusinessCenterTimeBuilder notificationTime) {
			this.notificationTime = notificationTime;
			return this;
		}
		public NotificationTimeElectionBuilder setParty(String party) {
			this.party = party;
			return this;
		}
		
		public NotificationTimeElection build() {
			return new NotificationTimeElection(this);
		}
	
		@Override
		public NotificationTimeElectionBuilder prune() {
			if (notificationTime!=null && !notificationTime.prune().hasData()) notificationTime = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getCustomNotification()!=null) return true;
			if (getNotificationTime()!=null && getNotificationTime().hasData()) return true;
			if (getParty()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("customNotification"), String.class, customNotification, this);
			processor.processBasic(path.newSubPath("party"), String.class, party, this);
			
			processRosetta(path.newSubPath("notificationTime"), processor, BusinessCenterTime.class, notificationTime);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			NotificationTimeElectionBuilder _that = (NotificationTimeElectionBuilder) o;
		
			if (customNotification != null ? !customNotification.equals(_that.customNotification) : _that.customNotification != null) return false;
			if (notificationTime != null ? !notificationTime.equals(_that.notificationTime) : _that.notificationTime != null) return false;
			if (party != null ? !party.equals(_that.party) : _that.party != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (customNotification != null ? customNotification.hashCode() : 0);
			_result = 31 * _result + (notificationTime != null ? notificationTime.hashCode() : 0);
			_result = 31 * _result + (party != null ? party.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "NotificationTimeElectionBuilder {" +
				"customNotification=" + this.customNotification + ", " +
				"notificationTime=" + this.notificationTime + ", " +
				"party=" + this.party +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("customNotification"), String.class, customNotification, this);
		processor.processBasic(path.newSubPath("party"), String.class, party, this);
		
		processRosetta(path.newSubPath("notificationTime"), processor, BusinessCenterTime.class, notificationTime);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		NotificationTimeElection _that = (NotificationTimeElection) o;
	
		if (customNotification != null ? !customNotification.equals(_that.customNotification) : _that.customNotification != null) return false;
		if (notificationTime != null ? !notificationTime.equals(_that.notificationTime) : _that.notificationTime != null) return false;
		if (party != null ? !party.equals(_that.party) : _that.party != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (customNotification != null ? customNotification.hashCode() : 0);
		_result = 31 * _result + (notificationTime != null ? notificationTime.hashCode() : 0);
		_result = 31 * _result + (party != null ? party.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "NotificationTimeElection {" +
			"customNotification=" + this.customNotification + ", " +
			"notificationTime=" + this.notificationTime + ", " +
			"party=" + this.party +
		'}';
	}
}
