package org.isda.cdm;

import org.isda.cdm.NonNegativeAmountSchedule;
import org.isda.cdm.NotionalStepRule;
import org.isda.cdm.metafields.MetaFields;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.NotionalScheduleMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class specifying the notional amount or notional amount schedule associated with a contractual product. The notional schedule will be captured explicitly, specifying the dates that the notional changes and the outstanding notional amount that applies from that date. A parametric representation of the rules defining the notional step schedule can optionally be included.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class NotionalSchedule extends RosettaModelObject implements GlobalKey {
	private final MetaFields meta;
	private final NotionalStepRule notionalStepParameters;
	private final NonNegativeAmountSchedule notionalStepSchedule;
	private static NotionalScheduleMeta metaData = new NotionalScheduleMeta();
	
	NotionalSchedule(NotionalScheduleBuilder builder) {
		this.meta = ofNullable(builder.getMeta()).map(f->f.build()).orElse(null);
		this.notionalStepParameters = ofNullable(builder.getNotionalStepParameters()).map(f->f.build()).orElse(null);
		this.notionalStepSchedule = ofNullable(builder.getNotionalStepSchedule()).map(f->f.build()).orElse(null);
	}
	
	public final MetaFields getMeta() {
		return meta;
	}
	
	/**
	 * A parametric representation of the notional step schedule, to be used when the notional schedule is not just specified as a set of dates and notional amount. Those parameters include the step frequency, the first and last notional step dates, the step rate, ...
	 */
	@RosettaSynonym(value="notionalStepParameters", source="FpML_5_10")
	@RosettaSynonym(value="notionalStepParameters", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="notionalStepParameters", source="DTCC_11_0")
	@RosettaSynonym(value="notionalStepParameters", source="DTCC_9_0")
	@RosettaSynonym(value="notionalStepParameters", source="CME_ClearedConfirm_1_17")
	public final NotionalStepRule getNotionalStepParameters() {
		return notionalStepParameters;
	}
	
	/**
	 * The notional amount when expressed as a varying notional alongside a date schedule. The step dates may be subject to adjustments in accordance with any adjustments specified in calculationPeriodDatesAdjustments.
	 */
	@RosettaSynonym(value="notionalStepSchedule", source="FpML_5_10")
	@RosettaSynonym(value="notionalStepSchedule", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="notionalStepSchedule", source="DTCC_11_0")
	@RosettaSynonym(value="notionalStepSchedule", source="DTCC_9_0")
	@RosettaSynonym(value="notionalStepSchedule", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="notional", source="DTCC_11_0")
	@RosettaSynonym(value="notional", source="DTCC_9_0")
	public final NonNegativeAmountSchedule getNotionalStepSchedule() {
		return notionalStepSchedule;
	}
	
	
	@Override
	public RosettaMetaData<? extends NotionalSchedule> metaData() {
		return metaData;
	} 
	
	public NotionalScheduleBuilder toBuilder() {
		NotionalScheduleBuilder builder = new NotionalScheduleBuilder();
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		ofNullable(getNotionalStepParameters()).ifPresent(builder::setNotionalStepParameters);
		ofNullable(getNotionalStepSchedule()).ifPresent(builder::setNotionalStepSchedule);
		return builder;
	}

	public static NotionalScheduleBuilder builder() {
		return new NotionalScheduleBuilder();
	}

	public static  class NotionalScheduleBuilder extends RosettaModelObjectBuilder implements GlobalKeyBuilder<NotionalScheduleBuilder> {
	
		protected MetaFields.MetaFieldsBuilder meta;
		protected NotionalStepRule.NotionalStepRuleBuilder notionalStepParameters;
		protected NonNegativeAmountSchedule.NonNegativeAmountScheduleBuilder notionalStepSchedule;
	
		public NotionalScheduleBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends NotionalSchedule> metaData() {
			return metaData;
		} 
	
		
		public MetaFields.MetaFieldsBuilder getMeta() {
			return meta;
		}
		
		public MetaFields.MetaFieldsBuilder getOrCreateMeta() {
			if (meta!=null) {
				return meta;
			}
			else return meta = new MetaFields.MetaFieldsBuilder();
		}
		
		
		public NotionalStepRule.NotionalStepRuleBuilder getNotionalStepParameters() {
			return notionalStepParameters;
		}
		
		public NotionalStepRule.NotionalStepRuleBuilder getOrCreateNotionalStepParameters() {
			if (notionalStepParameters!=null) {
				return notionalStepParameters;
			}
			else return notionalStepParameters = new NotionalStepRule.NotionalStepRuleBuilder();
		}
		
		
		public NonNegativeAmountSchedule.NonNegativeAmountScheduleBuilder getNotionalStepSchedule() {
			return notionalStepSchedule;
		}
		
		public NonNegativeAmountSchedule.NonNegativeAmountScheduleBuilder getOrCreateNotionalStepSchedule() {
			if (notionalStepSchedule!=null) {
				return notionalStepSchedule;
			}
			else return notionalStepSchedule = new NonNegativeAmountSchedule.NonNegativeAmountScheduleBuilder();
		}
		
	
		public NotionalScheduleBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		public NotionalScheduleBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		public NotionalScheduleBuilder setNotionalStepParameters(NotionalStepRule notionalStepParameters) {
			this.notionalStepParameters = notionalStepParameters.toBuilder();
			return this;
		}
		public NotionalScheduleBuilder setNotionalStepParametersBuilder(NotionalStepRule.NotionalStepRuleBuilder notionalStepParameters) {
			this.notionalStepParameters = notionalStepParameters;
			return this;
		}
		public NotionalScheduleBuilder setNotionalStepSchedule(NonNegativeAmountSchedule notionalStepSchedule) {
			this.notionalStepSchedule = notionalStepSchedule.toBuilder();
			return this;
		}
		public NotionalScheduleBuilder setNotionalStepScheduleBuilder(NonNegativeAmountSchedule.NonNegativeAmountScheduleBuilder notionalStepSchedule) {
			this.notionalStepSchedule = notionalStepSchedule;
			return this;
		}
		
		public NotionalSchedule build() {
			return new NotionalSchedule(this);
		}
	
		@Override
		public NotionalScheduleBuilder prune() {
			if (meta!=null && !meta.prune().hasData()) meta = null;
			if (notionalStepParameters!=null && !notionalStepParameters.prune().hasData()) notionalStepParameters = null;
			if (notionalStepSchedule!=null && !notionalStepSchedule.prune().hasData()) notionalStepSchedule = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getMeta()!=null && getMeta().hasData()) return true;
			if (getNotionalStepParameters()!=null && getNotionalStepParameters().hasData()) return true;
			if (getNotionalStepSchedule()!=null && getNotionalStepSchedule().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
			processRosetta(path.newSubPath("notionalStepParameters"), processor, NotionalStepRule.class, notionalStepParameters);
			processRosetta(path.newSubPath("notionalStepSchedule"), processor, NonNegativeAmountSchedule.class, notionalStepSchedule);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			NotionalScheduleBuilder _that = (NotionalScheduleBuilder) o;
		
			if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
			if (notionalStepParameters != null ? !notionalStepParameters.equals(_that.notionalStepParameters) : _that.notionalStepParameters != null) return false;
			if (notionalStepSchedule != null ? !notionalStepSchedule.equals(_that.notionalStepSchedule) : _that.notionalStepSchedule != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
			_result = 31 * _result + (notionalStepParameters != null ? notionalStepParameters.hashCode() : 0);
			_result = 31 * _result + (notionalStepSchedule != null ? notionalStepSchedule.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "NotionalScheduleBuilder {" +
				"meta=" + this.meta + ", " +
				"notionalStepParameters=" + this.notionalStepParameters + ", " +
				"notionalStepSchedule=" + this.notionalStepSchedule +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
		processRosetta(path.newSubPath("notionalStepParameters"), processor, NotionalStepRule.class, notionalStepParameters);
		processRosetta(path.newSubPath("notionalStepSchedule"), processor, NonNegativeAmountSchedule.class, notionalStepSchedule);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		NotionalSchedule _that = (NotionalSchedule) o;
	
		if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
		if (notionalStepParameters != null ? !notionalStepParameters.equals(_that.notionalStepParameters) : _that.notionalStepParameters != null) return false;
		if (notionalStepSchedule != null ? !notionalStepSchedule.equals(_that.notionalStepSchedule) : _that.notionalStepSchedule != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
		_result = 31 * _result + (notionalStepParameters != null ? notionalStepParameters.hashCode() : 0);
		_result = 31 * _result + (notionalStepSchedule != null ? notionalStepSchedule.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "NotionalSchedule {" +
			"meta=" + this.meta + ", " +
			"notionalStepParameters=" + this.notionalStepParameters + ", " +
			"notionalStepSchedule=" + this.notionalStepSchedule +
		'}';
	}
}
