package org.isda.cdm;

import com.rosetta.model.lib.records.Date;
import java.math.BigDecimal;
import org.isda.cdm.Frequency;
import org.isda.cdm.StepRelativeToEnum;
import org.isda.cdm.metafields.ReferenceWithMetaCalculationPeriodDates;
import org.isda.cdm.metafields.ReferenceWithMetaCalculationPeriodDates.ReferenceWithMetaCalculationPeriodDatesBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.NotionalStepRuleMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.stream.Collectors;
		import static java.util.Optional.ofNullable;
		import com.rosetta.model.lib.records.Date;
		import java.math.BigDecimal;

/**
 * A class defining a parametric representation for the notional step schedule, i.e. parameters used to generate the notional amount on each step date. This parametric representation is intended to cover the more common amortizing/accreting variations of swaps.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="NotionalStepRule", source="FpML_5_10")
public class NotionalStepRule extends RosettaModelObject {
	private final ReferenceWithMetaCalculationPeriodDates calculationPeriodDatesReference;
	private final Date firstNotionalStepDate;
	private final Date lastNotionalStepDate;
	private final BigDecimal notionalStepAmount;
	private final BigDecimal notionalStepRate;
	private final Frequency stepFrequency;
	private final StepRelativeToEnum stepRelativeTo;
	private static NotionalStepRuleMeta metaData = new NotionalStepRuleMeta();
	
	NotionalStepRule(NotionalStepRuleBuilder builder) {
		this.calculationPeriodDatesReference = ofNullable(builder.getCalculationPeriodDatesReference()).map(f->f.build()).orElse(null);
		this.firstNotionalStepDate = builder.getFirstNotionalStepDate();
		this.lastNotionalStepDate = builder.getLastNotionalStepDate();
		this.notionalStepAmount = builder.getNotionalStepAmount();
		this.notionalStepRate = builder.getNotionalStepRate();
		this.stepFrequency = ofNullable(builder.getStepFrequency()).map(f->f.build()).orElse(null);
		this.stepRelativeTo = builder.getStepRelativeTo();
	}
	
	/**
	 * A pointer style reference to the associated calculation period dates component defined elsewhere in the document.
	 */
	@RosettaSynonym(value="calculationPeriodDatesReference", source="FpML_5_10")
	@RosettaSynonym(value="calculationPeriodDatesReference", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="calculationPeriodDatesReference", source="DTCC_11_0")
	@RosettaSynonym(value="calculationPeriodDatesReference", source="DTCC_9_0")
	@RosettaSynonym(value="calculationPeriodDatesReference", source="CME_ClearedConfirm_1_17")
	public final ReferenceWithMetaCalculationPeriodDates getCalculationPeriodDatesReference() {
		return calculationPeriodDatesReference;
	}
	
	/**
	 * Effective date of the first change in notional (i.e. a calculation period start date).
	 */
	@RosettaSynonym(value="firstNotionalStepDate", source="FpML_5_10")
	@RosettaSynonym(value="firstNotionalStepDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="firstNotionalStepDate", source="DTCC_11_0")
	@RosettaSynonym(value="firstNotionalStepDate", source="DTCC_9_0")
	@RosettaSynonym(value="firstNotionalStepDate", source="CME_ClearedConfirm_1_17")
	public final Date getFirstNotionalStepDate() {
		return firstNotionalStepDate;
	}
	
	/**
	 * Effective date of the last change in notional (i.e. a calculation period start date).
	 */
	@RosettaSynonym(value="lastNotionalStepDate", source="FpML_5_10")
	@RosettaSynonym(value="lastNotionalStepDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="lastNotionalStepDate", source="DTCC_11_0")
	@RosettaSynonym(value="lastNotionalStepDate", source="DTCC_9_0")
	@RosettaSynonym(value="lastNotionalStepDate", source="CME_ClearedConfirm_1_17")
	public final Date getLastNotionalStepDate() {
		return lastNotionalStepDate;
	}
	
	/**
	 * The explicit amount that the notional changes on each step date. This can be a positive or negative amount.
	 */
	@RosettaSynonym(value="notionalStepAmount", source="FpML_5_10")
	@RosettaSynonym(value="notionalStepAmount", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="notionalStepAmount", source="DTCC_11_0")
	@RosettaSynonym(value="notionalStepAmount", source="DTCC_9_0")
	@RosettaSynonym(value="notionalStepAmount", source="CME_ClearedConfirm_1_17")
	public final BigDecimal getNotionalStepAmount() {
		return notionalStepAmount;
	}
	
	/**
	 * The percentage amount by which the notional changes on each step date. The percentage is either a percentage applied to the initial notional amount or the previous outstanding notional, depending of the value of the attribute stepRelativeTo. The percentage can be either positive or negative. A percentage of 5% would be represented as 0.05.
	 */
	@RosettaSynonym(value="notionalStepRate", source="FpML_5_10")
	@RosettaSynonym(value="notionalStepRate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="notionalStepRate", source="DTCC_11_0")
	@RosettaSynonym(value="notionalStepRate", source="DTCC_9_0")
	@RosettaSynonym(value="notionalStepRate", source="CME_ClearedConfirm_1_17")
	public final BigDecimal getNotionalStepRate() {
		return notionalStepRate;
	}
	
	/**
	 * The frequency at which the notional step changes occur. This frequency must be an integer multiple of the stream calculation period frequency.
	 */
	@RosettaSynonym(value="stepFrequency", source="FpML_5_10")
	@RosettaSynonym(value="stepFrequency", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="stepFrequency", source="DTCC_11_0")
	@RosettaSynonym(value="stepFrequency", source="DTCC_9_0")
	@RosettaSynonym(value="stepFrequency", source="CME_ClearedConfirm_1_17")
	public final Frequency getStepFrequency() {
		return stepFrequency;
	}
	
	/**
	 * Specifies whether the notionalStepRate should be applied to the initial notional or the previous notional in order to calculate the notional step change amount.
	 */
	@RosettaSynonym(value="stepRelativeTo", source="FpML_5_10")
	@RosettaSynonym(value="stepRelativeTo", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="stepRelativeTo", source="DTCC_11_0")
	@RosettaSynonym(value="stepRelativeTo", source="DTCC_9_0")
	@RosettaSynonym(value="stepRelativeTo", source="CME_ClearedConfirm_1_17")
	public final StepRelativeToEnum getStepRelativeTo() {
		return stepRelativeTo;
	}
	
	
	@Override
	public RosettaMetaData<? extends NotionalStepRule> metaData() {
		return metaData;
	} 
	
	public NotionalStepRuleBuilder toBuilder() {
		NotionalStepRuleBuilder builder = new NotionalStepRuleBuilder();
		ofNullable(getCalculationPeriodDatesReference()).ifPresent(builder::setCalculationPeriodDatesReference);
		ofNullable(getFirstNotionalStepDate()).ifPresent(builder::setFirstNotionalStepDate);
		ofNullable(getLastNotionalStepDate()).ifPresent(builder::setLastNotionalStepDate);
		ofNullable(getNotionalStepAmount()).ifPresent(builder::setNotionalStepAmount);
		ofNullable(getNotionalStepRate()).ifPresent(builder::setNotionalStepRate);
		ofNullable(getStepFrequency()).ifPresent(builder::setStepFrequency);
		ofNullable(getStepRelativeTo()).ifPresent(builder::setStepRelativeTo);
		return builder;
	}

	public static NotionalStepRuleBuilder builder() {
		return new NotionalStepRuleBuilder();
	}

	public static  class NotionalStepRuleBuilder extends RosettaModelObjectBuilder {
	
		protected ReferenceWithMetaCalculationPeriodDatesBuilder calculationPeriodDatesReference;
		protected Date firstNotionalStepDate;
		protected Date lastNotionalStepDate;
		protected BigDecimal notionalStepAmount;
		protected BigDecimal notionalStepRate;
		protected Frequency.FrequencyBuilder stepFrequency;
		protected StepRelativeToEnum stepRelativeTo;
	
		public NotionalStepRuleBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends NotionalStepRule> metaData() {
			return metaData;
		} 
	
		
		public ReferenceWithMetaCalculationPeriodDatesBuilder getCalculationPeriodDatesReference() {
			return calculationPeriodDatesReference;
		}
		
		public ReferenceWithMetaCalculationPeriodDatesBuilder getOrCreateCalculationPeriodDatesReference() {
			if (calculationPeriodDatesReference!=null) {
				return calculationPeriodDatesReference;
			}
			else return calculationPeriodDatesReference = new ReferenceWithMetaCalculationPeriodDatesBuilder();
		}
		
		
		public Date getFirstNotionalStepDate() {
			return firstNotionalStepDate;
		}
		
		
		public Date getLastNotionalStepDate() {
			return lastNotionalStepDate;
		}
		
		
		public BigDecimal getNotionalStepAmount() {
			return notionalStepAmount;
		}
		
		
		public BigDecimal getNotionalStepRate() {
			return notionalStepRate;
		}
		
		
		public Frequency.FrequencyBuilder getStepFrequency() {
			return stepFrequency;
		}
		
		public Frequency.FrequencyBuilder getOrCreateStepFrequency() {
			if (stepFrequency!=null) {
				return stepFrequency;
			}
			else return stepFrequency = new Frequency.FrequencyBuilder();
		}
		
		
		public StepRelativeToEnum getStepRelativeTo() {
			return stepRelativeTo;
		}
		
	
		public NotionalStepRuleBuilder setCalculationPeriodDatesReference(ReferenceWithMetaCalculationPeriodDates calculationPeriodDatesReference) {
			this.calculationPeriodDatesReference = calculationPeriodDatesReference.toBuilder();
			return this;
		}
		public NotionalStepRuleBuilder setCalculationPeriodDatesReferenceBuilder(ReferenceWithMetaCalculationPeriodDatesBuilder calculationPeriodDatesReference) {
			this.calculationPeriodDatesReference = calculationPeriodDatesReference;
			return this;
		}
		public NotionalStepRuleBuilder setCalculationPeriodDatesReferenceRef(CalculationPeriodDates.CalculationPeriodDatesBuilder calculationPeriodDatesReference) {
			return setCalculationPeriodDatesReference(ReferenceWithMetaCalculationPeriodDates.builder().setValueBuilder(calculationPeriodDatesReference).build());
		}
		public NotionalStepRuleBuilder setCalculationPeriodDatesReferenceRef(CalculationPeriodDates calculationPeriodDatesReference) {
			return setCalculationPeriodDatesReferenceRef(CalculationPeriodDates.builder());
		}
		public NotionalStepRuleBuilder setFirstNotionalStepDate(Date firstNotionalStepDate) {
			this.firstNotionalStepDate = firstNotionalStepDate;
			return this;
		}
		public NotionalStepRuleBuilder setLastNotionalStepDate(Date lastNotionalStepDate) {
			this.lastNotionalStepDate = lastNotionalStepDate;
			return this;
		}
		public NotionalStepRuleBuilder setNotionalStepAmount(BigDecimal notionalStepAmount) {
			this.notionalStepAmount = notionalStepAmount;
			return this;
		}
		public NotionalStepRuleBuilder setNotionalStepRate(BigDecimal notionalStepRate) {
			this.notionalStepRate = notionalStepRate;
			return this;
		}
		public NotionalStepRuleBuilder setStepFrequency(Frequency stepFrequency) {
			this.stepFrequency = stepFrequency.toBuilder();
			return this;
		}
		public NotionalStepRuleBuilder setStepFrequencyBuilder(Frequency.FrequencyBuilder stepFrequency) {
			this.stepFrequency = stepFrequency;
			return this;
		}
		public NotionalStepRuleBuilder setStepRelativeTo(StepRelativeToEnum stepRelativeTo) {
			this.stepRelativeTo = stepRelativeTo;
			return this;
		}
		
		public NotionalStepRule build() {
			return new NotionalStepRule(this);
		}
	
		@Override
		public NotionalStepRuleBuilder prune() {
			if (calculationPeriodDatesReference!=null && !calculationPeriodDatesReference.prune().hasData()) calculationPeriodDatesReference = null;
			if (stepFrequency!=null && !stepFrequency.prune().hasData()) stepFrequency = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getCalculationPeriodDatesReference()!=null && getCalculationPeriodDatesReference().hasData()) return true;
			if (getFirstNotionalStepDate()!=null) return true;
			if (getLastNotionalStepDate()!=null) return true;
			if (getNotionalStepAmount()!=null) return true;
			if (getNotionalStepRate()!=null) return true;
			if (getStepFrequency()!=null && getStepFrequency().hasData()) return true;
			if (getStepRelativeTo()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("firstNotionalStepDate"), Date.class, firstNotionalStepDate, this);
			processor.processBasic(path.newSubPath("lastNotionalStepDate"), Date.class, lastNotionalStepDate, this);
			processor.processBasic(path.newSubPath("notionalStepAmount"), BigDecimal.class, notionalStepAmount, this);
			processor.processBasic(path.newSubPath("notionalStepRate"), BigDecimal.class, notionalStepRate, this);
			processor.processBasic(path.newSubPath("stepRelativeTo"), StepRelativeToEnum.class, stepRelativeTo, this);
			
			processRosetta(path.newSubPath("calculationPeriodDatesReference"), processor, ReferenceWithMetaCalculationPeriodDates.class, calculationPeriodDatesReference);
			processRosetta(path.newSubPath("stepFrequency"), processor, Frequency.class, stepFrequency);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			NotionalStepRuleBuilder _that = (NotionalStepRuleBuilder) o;
		
			if (calculationPeriodDatesReference != null ? !calculationPeriodDatesReference.equals(_that.calculationPeriodDatesReference) : _that.calculationPeriodDatesReference != null) return false;
			if (firstNotionalStepDate != null ? !firstNotionalStepDate.equals(_that.firstNotionalStepDate) : _that.firstNotionalStepDate != null) return false;
			if (lastNotionalStepDate != null ? !lastNotionalStepDate.equals(_that.lastNotionalStepDate) : _that.lastNotionalStepDate != null) return false;
			if (notionalStepAmount != null ? !notionalStepAmount.equals(_that.notionalStepAmount) : _that.notionalStepAmount != null) return false;
			if (notionalStepRate != null ? !notionalStepRate.equals(_that.notionalStepRate) : _that.notionalStepRate != null) return false;
			if (stepFrequency != null ? !stepFrequency.equals(_that.stepFrequency) : _that.stepFrequency != null) return false;
			if (stepRelativeTo != null ? !stepRelativeTo.equals(_that.stepRelativeTo) : _that.stepRelativeTo != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (calculationPeriodDatesReference != null ? calculationPeriodDatesReference.hashCode() : 0);
			_result = 31 * _result + (firstNotionalStepDate != null ? firstNotionalStepDate.hashCode() : 0);
			_result = 31 * _result + (lastNotionalStepDate != null ? lastNotionalStepDate.hashCode() : 0);
			_result = 31 * _result + (notionalStepAmount != null ? notionalStepAmount.hashCode() : 0);
			_result = 31 * _result + (notionalStepRate != null ? notionalStepRate.hashCode() : 0);
			_result = 31 * _result + (stepFrequency != null ? stepFrequency.hashCode() : 0);
			_result = 31 * _result + (stepRelativeTo != null ? stepRelativeTo.getClass().getName().hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "NotionalStepRuleBuilder {" +
				"calculationPeriodDatesReference=" + this.calculationPeriodDatesReference + ", " +
				"firstNotionalStepDate=" + this.firstNotionalStepDate + ", " +
				"lastNotionalStepDate=" + this.lastNotionalStepDate + ", " +
				"notionalStepAmount=" + this.notionalStepAmount + ", " +
				"notionalStepRate=" + this.notionalStepRate + ", " +
				"stepFrequency=" + this.stepFrequency + ", " +
				"stepRelativeTo=" + this.stepRelativeTo +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("firstNotionalStepDate"), Date.class, firstNotionalStepDate, this);
		processor.processBasic(path.newSubPath("lastNotionalStepDate"), Date.class, lastNotionalStepDate, this);
		processor.processBasic(path.newSubPath("notionalStepAmount"), BigDecimal.class, notionalStepAmount, this);
		processor.processBasic(path.newSubPath("notionalStepRate"), BigDecimal.class, notionalStepRate, this);
		processor.processBasic(path.newSubPath("stepRelativeTo"), StepRelativeToEnum.class, stepRelativeTo, this);
		
		processRosetta(path.newSubPath("calculationPeriodDatesReference"), processor, ReferenceWithMetaCalculationPeriodDates.class, calculationPeriodDatesReference);
		processRosetta(path.newSubPath("stepFrequency"), processor, Frequency.class, stepFrequency);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		NotionalStepRule _that = (NotionalStepRule) o;
	
		if (calculationPeriodDatesReference != null ? !calculationPeriodDatesReference.equals(_that.calculationPeriodDatesReference) : _that.calculationPeriodDatesReference != null) return false;
		if (firstNotionalStepDate != null ? !firstNotionalStepDate.equals(_that.firstNotionalStepDate) : _that.firstNotionalStepDate != null) return false;
		if (lastNotionalStepDate != null ? !lastNotionalStepDate.equals(_that.lastNotionalStepDate) : _that.lastNotionalStepDate != null) return false;
		if (notionalStepAmount != null ? !notionalStepAmount.equals(_that.notionalStepAmount) : _that.notionalStepAmount != null) return false;
		if (notionalStepRate != null ? !notionalStepRate.equals(_that.notionalStepRate) : _that.notionalStepRate != null) return false;
		if (stepFrequency != null ? !stepFrequency.equals(_that.stepFrequency) : _that.stepFrequency != null) return false;
		if (stepRelativeTo != null ? !stepRelativeTo.equals(_that.stepRelativeTo) : _that.stepRelativeTo != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (calculationPeriodDatesReference != null ? calculationPeriodDatesReference.hashCode() : 0);
		_result = 31 * _result + (firstNotionalStepDate != null ? firstNotionalStepDate.hashCode() : 0);
		_result = 31 * _result + (lastNotionalStepDate != null ? lastNotionalStepDate.hashCode() : 0);
		_result = 31 * _result + (notionalStepAmount != null ? notionalStepAmount.hashCode() : 0);
		_result = 31 * _result + (notionalStepRate != null ? notionalStepRate.hashCode() : 0);
		_result = 31 * _result + (stepFrequency != null ? stepFrequency.hashCode() : 0);
		_result = 31 * _result + (stepRelativeTo != null ? stepRelativeTo.getClass().getName().hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "NotionalStepRule {" +
			"calculationPeriodDatesReference=" + this.calculationPeriodDatesReference + ", " +
			"firstNotionalStepDate=" + this.firstNotionalStepDate + ", " +
			"lastNotionalStepDate=" + this.lastNotionalStepDate + ", " +
			"notionalStepAmount=" + this.notionalStepAmount + ", " +
			"notionalStepRate=" + this.notionalStepRate + ", " +
			"stepFrequency=" + this.stepFrequency + ", " +
			"stepRelativeTo=" + this.stepRelativeTo +
		'}';
	}
}
