package org.isda.cdm;

import java.lang.Boolean;
import java.lang.String;
import org.isda.cdm.NotDomesticCurrency;
import org.isda.cdm.ObligationCategoryEnum;
import org.isda.cdm.SpecifiedCurrency;
import org.isda.cdm.metafields.FieldWithMetaString;
import org.isda.cdm.metafields.FieldWithMetaString.FieldWithMetaStringBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.ObligationsMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class to specify the underlying obligations of the reference entity on which protection is purchased or sold through the Credit Default Swap.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="Obligations", source="FpML_5_10")
public class Obligations extends RosettaModelObject {
	private final Boolean cashSettlementOnly;
	private final ObligationCategoryEnum category;
	private final Boolean continuity;
	private final Boolean deliveryOfCommitments;
	private final FieldWithMetaString designatedPriority;
	private final String excluded;
	private final Boolean fullFaithAndCreditObLiability;
	private final Boolean generalFundObligationLiability;
	private final Boolean listed;
	private final Boolean notContingent;
	private final NotDomesticCurrency notDomesticCurrency;
	private final Boolean notDomesticIssuance;
	private final Boolean notDomesticLaw;
	private final Boolean notSovereignLender;
	private final Boolean notSubordinated;
	private final String othReferenceEntityObligations;
	private final Boolean revenueObligationLiability;
	private final SpecifiedCurrency specifiedCurrency;
	private static ObligationsMeta metaData = new ObligationsMeta();
	
	Obligations(ObligationsBuilder builder) {
		this.cashSettlementOnly = builder.getCashSettlementOnly();
		this.category = builder.getCategory();
		this.continuity = builder.getContinuity();
		this.deliveryOfCommitments = builder.getDeliveryOfCommitments();
		this.designatedPriority = ofNullable(builder.getDesignatedPriority()).map(f->f.build()).orElse(null);
		this.excluded = builder.getExcluded();
		this.fullFaithAndCreditObLiability = builder.getFullFaithAndCreditObLiability();
		this.generalFundObligationLiability = builder.getGeneralFundObligationLiability();
		this.listed = builder.getListed();
		this.notContingent = builder.getNotContingent();
		this.notDomesticCurrency = ofNullable(builder.getNotDomesticCurrency()).map(f->f.build()).orElse(null);
		this.notDomesticIssuance = builder.getNotDomesticIssuance();
		this.notDomesticLaw = builder.getNotDomesticLaw();
		this.notSovereignLender = builder.getNotSovereignLender();
		this.notSubordinated = builder.getNotSubordinated();
		this.othReferenceEntityObligations = builder.getOthReferenceEntityObligations();
		this.revenueObligationLiability = builder.getRevenueObligationLiability();
		this.specifiedCurrency = ofNullable(builder.getSpecifiedCurrency()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * An obligation and deliverable obligation characteristic. Defined in the ISDA published Standard Terms Supplement for use with CDS Transactions on Leveraged Loans. ISDA 2003 Term: Cash Settlement Only.
	 */
	@RosettaSynonym(value="cashSettlementOnly", source="FpML_5_10")
	@RosettaSynonym(value="cashSettlementOnly", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="cashSettlementOnly", source="DTCC_11_0")
	@RosettaSynonym(value="cashSettlementOnly", source="DTCC_9_0")
	@RosettaSynonym(value="cashSettlementOnly", source="CME_ClearedConfirm_1_17")
	public final Boolean getCashSettlementOnly() {
		return cashSettlementOnly;
	}
	
	/**
	 * Used in both obligations and deliverable obligations to represent a class or type of securities which apply. ISDA 2003 Term: Obligation Category/Deliverable Obligation Category.
	 */
	@RosettaSynonym(value="category", source="FpML_5_10")
	@RosettaSynonym(value="category", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="category", source="DTCC_11_0")
	@RosettaSynonym(value="category", source="DTCC_9_0")
	@RosettaSynonym(value="category", source="CME_ClearedConfirm_1_17")
	public final ObligationCategoryEnum getCategory() {
		return category;
	}
	
	/**
	 * An obligation and deliverable obligation characteristic. Defined in the ISDA published Standard Terms Supplement for use with CDS Transactions on Leveraged Loans. ISDA 2003 Term: Continuity.
	 */
	@RosettaSynonym(value="continuity", source="FpML_5_10")
	@RosettaSynonym(value="continuity", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="continuity", source="DTCC_11_0")
	@RosettaSynonym(value="continuity", source="DTCC_9_0")
	@RosettaSynonym(value="continuity", source="CME_ClearedConfirm_1_17")
	public final Boolean getContinuity() {
		return continuity;
	}
	
	/**
	 * An obligation and deliverable obligation characteristic. Defined in the ISDA published Standard Terms Supplement for use with CDS Transactions on Leveraged Loans. ISDA 2003 Term: Delivery of Commitments.
	 */
	@RosettaSynonym(value="deliveryOfCommitments", source="FpML_5_10")
	@RosettaSynonym(value="deliveryOfCommitments", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="deliveryOfCommitments", source="DTCC_11_0")
	@RosettaSynonym(value="deliveryOfCommitments", source="DTCC_9_0")
	@RosettaSynonym(value="deliveryOfCommitments", source="CME_ClearedConfirm_1_17")
	public final Boolean getDeliveryOfCommitments() {
		return deliveryOfCommitments;
	}
	
	/**
	 * Applies to Loan CDS, to indicate what lien level is appropriate for a deliverable obligation. Applies to European Loan CDS, to indicate the Ranking of the obligation. Example: a 2nd lien Loan CDS would imply that the deliverable obligations are 1st or 2nd lien loans.
	 */
	@RosettaSynonym(value="designatedPriority", source="FpML_5_10")
	@RosettaSynonym(value="designatedPriority", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="designatedPriority", source="DTCC_11_0")
	@RosettaSynonym(value="designatedPriority", source="DTCC_9_0")
	@RosettaSynonym(value="designatedPriority", source="CME_ClearedConfirm_1_17")
	public final FieldWithMetaString getDesignatedPriority() {
		return designatedPriority;
	}
	
	/**
	 * A free format string to specify any excluded obligations or deliverable obligations, as the case may be, of the reference entity or excluded types of obligations or deliverable obligations. ISDA 2003 Term: Excluded Obligations/Excluded Deliverable Obligations.
	 */
	@RosettaSynonym(value="excluded", source="FpML_5_10")
	@RosettaSynonym(value="excluded", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="excluded", source="DTCC_11_0")
	@RosettaSynonym(value="excluded", source="DTCC_9_0")
	@RosettaSynonym(value="excluded", source="CME_ClearedConfirm_1_17")
	public final String getExcluded() {
		return excluded;
	}
	
	/**
	 * An obligation and deliverable obligation characteristic. Defined in the ISDA published additional provisions for U.S. Municipal as Reference Entity. ISDA 2003 Term: Full Faith and Credit Obligation Liability.
	 */
	@RosettaSynonym(value="fullFaithAndCreditObLiability", source="FpML_5_10")
	@RosettaSynonym(value="fullFaithAndCreditObLiability", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="fullFaithAndCreditObLiability", source="DTCC_11_0")
	@RosettaSynonym(value="fullFaithAndCreditObLiability", source="DTCC_9_0")
	@RosettaSynonym(value="fullFaithAndCreditObLiability", source="CME_ClearedConfirm_1_17")
	public final Boolean getFullFaithAndCreditObLiability() {
		return fullFaithAndCreditObLiability;
	}
	
	/**
	 * An obligation and deliverable obligation characteristic. Defined in the ISDA published additional provisions for U.S. Municipal as Reference Entity. ISDA 2003 Term: General Fund Obligation Liability.
	 */
	@RosettaSynonym(value="generalFundObligationLiability", source="FpML_5_10")
	@RosettaSynonym(value="generalFundObligationLiability", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="generalFundObligationLiability", source="DTCC_11_0")
	@RosettaSynonym(value="generalFundObligationLiability", source="DTCC_9_0")
	@RosettaSynonym(value="generalFundObligationLiability", source="CME_ClearedConfirm_1_17")
	public final Boolean getGeneralFundObligationLiability() {
		return generalFundObligationLiability;
	}
	
	/**
	 * An obligation and deliverable obligation characteristic. Indicates whether or not the obligation is quoted, listed or ordinarily purchased and sold on an exchange. ISDA 2003 Term: Listed.
	 */
	@RosettaSynonym(value="listed", source="FpML_5_10")
	@RosettaSynonym(value="listed", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="listed", source="DTCC_11_0")
	@RosettaSynonym(value="listed", source="DTCC_9_0")
	@RosettaSynonym(value="listed", source="CME_ClearedConfirm_1_17")
	public final Boolean getListed() {
		return listed;
	}
	
	/**
	 * OTE: Only allowed as an obligation characteristic under ISDA Credit 1999. In essence Not Contingent means the repayment of principal cannot be dependent on a formula/index, i.e. to prevent the risk of being delivered an instrument that may never pay any element of principal, and to ensure that the obligation is interest bearing (on a regular schedule). ISDA 2003 Term: Not Contingent.
	 */
	@RosettaSynonym(value="notContingent", source="FpML_5_10")
	@RosettaSynonym(value="notContingent", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="notContingent", source="DTCC_11_0")
	@RosettaSynonym(value="notContingent", source="DTCC_9_0")
	@RosettaSynonym(value="notContingent", source="CME_ClearedConfirm_1_17")
	public final Boolean getNotContingent() {
		return notContingent;
	}
	
	/**
	 * An obligation and deliverable obligation characteristic. Any obligation that is payable in any currency other than the domestic currency. Domestic currency is either the currency so specified or, if no currency is specified, the currency of (a) the reference entity, if the reference entity is a sovereign, or (b) the jurisdiction in which the relevant reference entity is organised, if the reference entity is not a sovereign. ISDA 2003 Term: Not Domestic Currency.
	 */
	@RosettaSynonym(value="notDomesticCurrency", source="FpML_5_10")
	@RosettaSynonym(value="notDomesticCurrency", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="notDomesticCurrency", source="DTCC_11_0")
	@RosettaSynonym(value="notDomesticCurrency", source="DTCC_9_0")
	@RosettaSynonym(value="notDomesticCurrency", source="CME_ClearedConfirm_1_17")
	public final NotDomesticCurrency getNotDomesticCurrency() {
		return notDomesticCurrency;
	}
	
	/**
	 * An obligation and deliverable obligation characteristic. Any obligation other than an obligation that was intended to be offered for sale primarily in the domestic market of the relevant Reference Entity. This specifies that the obligation must be an internationally recognised bond. ISDA 2003 Term: Not Domestic Issuance.
	 */
	@RosettaSynonym(value="notDomesticIssuance", source="FpML_5_10")
	@RosettaSynonym(value="notDomesticIssuance", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="notDomesticIssuance", source="DTCC_11_0")
	@RosettaSynonym(value="notDomesticIssuance", source="DTCC_9_0")
	@RosettaSynonym(value="notDomesticIssuance", source="CME_ClearedConfirm_1_17")
	public final Boolean getNotDomesticIssuance() {
		return notDomesticIssuance;
	}
	
	/**
	 * An obligation and deliverable obligation characteristic. If the reference entity is a Sovereign, this means any obligation that is not subject to the laws of the reference entity. If the reference entity is not a sovereign, this means any obligation that is not subject to the laws of the jurisdiction of the reference entity. ISDA 2003 Term: Not Domestic Law.
	 */
	@RosettaSynonym(value="notDomesticLaw", source="FpML_5_10")
	@RosettaSynonym(value="notDomesticLaw", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="notDomesticLaw", source="DTCC_11_0")
	@RosettaSynonym(value="notDomesticLaw", source="DTCC_9_0")
	@RosettaSynonym(value="notDomesticLaw", source="CME_ClearedConfirm_1_17")
	public final Boolean getNotDomesticLaw() {
		return notDomesticLaw;
	}
	
	/**
	 * An obligation and deliverable obligation characteristic. Any obligation that is not primarily (majority) owed to a Sovereign or Supranational Organisation. ISDA 2003 Term: Not Sovereign Lender.
	 */
	@RosettaSynonym(value="notSovereignLender", source="FpML_5_10")
	@RosettaSynonym(value="notSovereignLender", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="notSovereignLender", source="DTCC_11_0")
	@RosettaSynonym(value="notSovereignLender", source="DTCC_9_0")
	@RosettaSynonym(value="notSovereignLender", source="CME_ClearedConfirm_1_17")
	public final Boolean getNotSovereignLender() {
		return notSovereignLender;
	}
	
	/**
	 * An obligation and deliverable obligation characteristic. An obligation that ranks at least equal with the most senior Reference Obligation in priority of payment or, if no Reference Obligation is specified in the related Confirmation, the obligations of the Reference Entity that are senior. ISDA 2003 Term: Not Subordinated.
	 */
	@RosettaSynonym(value="notSubordinated", source="FpML_5_10")
	@RosettaSynonym(value="notSubordinated", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="notSubordinated", source="DTCC_11_0")
	@RosettaSynonym(value="notSubordinated", source="DTCC_9_0")
	@RosettaSynonym(value="notSubordinated", source="CME_ClearedConfirm_1_17")
	public final Boolean getNotSubordinated() {
		return notSubordinated;
	}
	
	/**
	 * This element is used to specify any other obligations of a reference entity in both obligations and deliverable obligations. The obligations can be specified free-form. ISDA 2003 Term: Other Obligations of a Reference Entity.
	 */
	@RosettaSynonym(value="othReferenceEntityObligations", source="FpML_5_10")
	@RosettaSynonym(value="othReferenceEntityObligations", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="othReferenceEntityObligations", source="DTCC_11_0")
	@RosettaSynonym(value="othReferenceEntityObligations", source="DTCC_9_0")
	@RosettaSynonym(value="othReferenceEntityObligations", source="CME_ClearedConfirm_1_17")
	public final String getOthReferenceEntityObligations() {
		return othReferenceEntityObligations;
	}
	
	/**
	 * An obligation and deliverable obligation characteristic. Defined in the ISDA published additional provisions for U.S. Municipal as Reference Entity. ISDA 2003 Term: Revenue Obligation Liability.
	 */
	@RosettaSynonym(value="revenueObligationLiability", source="FpML_5_10")
	@RosettaSynonym(value="revenueObligationLiability", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="revenueObligationLiability", source="DTCC_11_0")
	@RosettaSynonym(value="revenueObligationLiability", source="DTCC_9_0")
	@RosettaSynonym(value="revenueObligationLiability", source="CME_ClearedConfirm_1_17")
	public final Boolean getRevenueObligationLiability() {
		return revenueObligationLiability;
	}
	
	/**
	 * An obligation and deliverable obligation characteristic. The currency or currencies in which an obligation or deliverable obligation must be payable. ISDA 2003 Term: Specified Currency.
	 */
	@RosettaSynonym(value="specifiedCurrency", source="FpML_5_10")
	@RosettaSynonym(value="specifiedCurrency", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="specifiedCurrency", source="DTCC_11_0")
	@RosettaSynonym(value="specifiedCurrency", source="DTCC_9_0")
	@RosettaSynonym(value="specifiedCurrency", source="CME_ClearedConfirm_1_17")
	public final SpecifiedCurrency getSpecifiedCurrency() {
		return specifiedCurrency;
	}
	
	
	@Override
	public RosettaMetaData<? extends Obligations> metaData() {
		return metaData;
	} 
	
	public ObligationsBuilder toBuilder() {
		ObligationsBuilder builder = new ObligationsBuilder();
		ofNullable(getCashSettlementOnly()).ifPresent(builder::setCashSettlementOnly);
		ofNullable(getCategory()).ifPresent(builder::setCategory);
		ofNullable(getContinuity()).ifPresent(builder::setContinuity);
		ofNullable(getDeliveryOfCommitments()).ifPresent(builder::setDeliveryOfCommitments);
		ofNullable(getDesignatedPriority()).ifPresent(builder::setDesignatedPriority);
		ofNullable(getExcluded()).ifPresent(builder::setExcluded);
		ofNullable(getFullFaithAndCreditObLiability()).ifPresent(builder::setFullFaithAndCreditObLiability);
		ofNullable(getGeneralFundObligationLiability()).ifPresent(builder::setGeneralFundObligationLiability);
		ofNullable(getListed()).ifPresent(builder::setListed);
		ofNullable(getNotContingent()).ifPresent(builder::setNotContingent);
		ofNullable(getNotDomesticCurrency()).ifPresent(builder::setNotDomesticCurrency);
		ofNullable(getNotDomesticIssuance()).ifPresent(builder::setNotDomesticIssuance);
		ofNullable(getNotDomesticLaw()).ifPresent(builder::setNotDomesticLaw);
		ofNullable(getNotSovereignLender()).ifPresent(builder::setNotSovereignLender);
		ofNullable(getNotSubordinated()).ifPresent(builder::setNotSubordinated);
		ofNullable(getOthReferenceEntityObligations()).ifPresent(builder::setOthReferenceEntityObligations);
		ofNullable(getRevenueObligationLiability()).ifPresent(builder::setRevenueObligationLiability);
		ofNullable(getSpecifiedCurrency()).ifPresent(builder::setSpecifiedCurrency);
		return builder;
	}

	public static ObligationsBuilder builder() {
		return new ObligationsBuilder();
	}

	public static  class ObligationsBuilder extends RosettaModelObjectBuilder {
	
		protected Boolean cashSettlementOnly;
		protected ObligationCategoryEnum category;
		protected Boolean continuity;
		protected Boolean deliveryOfCommitments;
		protected FieldWithMetaStringBuilder designatedPriority;
		protected String excluded;
		protected Boolean fullFaithAndCreditObLiability;
		protected Boolean generalFundObligationLiability;
		protected Boolean listed;
		protected Boolean notContingent;
		protected NotDomesticCurrency.NotDomesticCurrencyBuilder notDomesticCurrency;
		protected Boolean notDomesticIssuance;
		protected Boolean notDomesticLaw;
		protected Boolean notSovereignLender;
		protected Boolean notSubordinated;
		protected String othReferenceEntityObligations;
		protected Boolean revenueObligationLiability;
		protected SpecifiedCurrency.SpecifiedCurrencyBuilder specifiedCurrency;
	
		public ObligationsBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends Obligations> metaData() {
			return metaData;
		} 
	
		
		public Boolean getCashSettlementOnly() {
			return cashSettlementOnly;
		}
		
		
		public ObligationCategoryEnum getCategory() {
			return category;
		}
		
		
		public Boolean getContinuity() {
			return continuity;
		}
		
		
		public Boolean getDeliveryOfCommitments() {
			return deliveryOfCommitments;
		}
		
		
		public FieldWithMetaStringBuilder getDesignatedPriority() {
			return designatedPriority;
		}
		
		public FieldWithMetaStringBuilder getOrCreateDesignatedPriority() {
			if (designatedPriority!=null) {
				return designatedPriority;
			}
			else return designatedPriority = new FieldWithMetaStringBuilder();
		}
		
		
		public String getExcluded() {
			return excluded;
		}
		
		
		public Boolean getFullFaithAndCreditObLiability() {
			return fullFaithAndCreditObLiability;
		}
		
		
		public Boolean getGeneralFundObligationLiability() {
			return generalFundObligationLiability;
		}
		
		
		public Boolean getListed() {
			return listed;
		}
		
		
		public Boolean getNotContingent() {
			return notContingent;
		}
		
		
		public NotDomesticCurrency.NotDomesticCurrencyBuilder getNotDomesticCurrency() {
			return notDomesticCurrency;
		}
		
		public NotDomesticCurrency.NotDomesticCurrencyBuilder getOrCreateNotDomesticCurrency() {
			if (notDomesticCurrency!=null) {
				return notDomesticCurrency;
			}
			else return notDomesticCurrency = new NotDomesticCurrency.NotDomesticCurrencyBuilder();
		}
		
		
		public Boolean getNotDomesticIssuance() {
			return notDomesticIssuance;
		}
		
		
		public Boolean getNotDomesticLaw() {
			return notDomesticLaw;
		}
		
		
		public Boolean getNotSovereignLender() {
			return notSovereignLender;
		}
		
		
		public Boolean getNotSubordinated() {
			return notSubordinated;
		}
		
		
		public String getOthReferenceEntityObligations() {
			return othReferenceEntityObligations;
		}
		
		
		public Boolean getRevenueObligationLiability() {
			return revenueObligationLiability;
		}
		
		
		public SpecifiedCurrency.SpecifiedCurrencyBuilder getSpecifiedCurrency() {
			return specifiedCurrency;
		}
		
		public SpecifiedCurrency.SpecifiedCurrencyBuilder getOrCreateSpecifiedCurrency() {
			if (specifiedCurrency!=null) {
				return specifiedCurrency;
			}
			else return specifiedCurrency = new SpecifiedCurrency.SpecifiedCurrencyBuilder();
		}
		
	
		public ObligationsBuilder setCashSettlementOnly(Boolean cashSettlementOnly) {
			this.cashSettlementOnly = cashSettlementOnly;
			return this;
		}
		public ObligationsBuilder setCategory(ObligationCategoryEnum category) {
			this.category = category;
			return this;
		}
		public ObligationsBuilder setContinuity(Boolean continuity) {
			this.continuity = continuity;
			return this;
		}
		public ObligationsBuilder setDeliveryOfCommitments(Boolean deliveryOfCommitments) {
			this.deliveryOfCommitments = deliveryOfCommitments;
			return this;
		}
		public ObligationsBuilder setDesignatedPriority(FieldWithMetaString designatedPriority) {
			this.designatedPriority = designatedPriority.toBuilder();
			return this;
		}
		public ObligationsBuilder setDesignatedPriorityRef(String designatedPriority) {
			return setDesignatedPriority(FieldWithMetaString.builder().setValue(designatedPriority).build());
		}
		public ObligationsBuilder setExcluded(String excluded) {
			this.excluded = excluded;
			return this;
		}
		public ObligationsBuilder setFullFaithAndCreditObLiability(Boolean fullFaithAndCreditObLiability) {
			this.fullFaithAndCreditObLiability = fullFaithAndCreditObLiability;
			return this;
		}
		public ObligationsBuilder setGeneralFundObligationLiability(Boolean generalFundObligationLiability) {
			this.generalFundObligationLiability = generalFundObligationLiability;
			return this;
		}
		public ObligationsBuilder setListed(Boolean listed) {
			this.listed = listed;
			return this;
		}
		public ObligationsBuilder setNotContingent(Boolean notContingent) {
			this.notContingent = notContingent;
			return this;
		}
		public ObligationsBuilder setNotDomesticCurrency(NotDomesticCurrency notDomesticCurrency) {
			this.notDomesticCurrency = notDomesticCurrency.toBuilder();
			return this;
		}
		public ObligationsBuilder setNotDomesticCurrencyBuilder(NotDomesticCurrency.NotDomesticCurrencyBuilder notDomesticCurrency) {
			this.notDomesticCurrency = notDomesticCurrency;
			return this;
		}
		public ObligationsBuilder setNotDomesticIssuance(Boolean notDomesticIssuance) {
			this.notDomesticIssuance = notDomesticIssuance;
			return this;
		}
		public ObligationsBuilder setNotDomesticLaw(Boolean notDomesticLaw) {
			this.notDomesticLaw = notDomesticLaw;
			return this;
		}
		public ObligationsBuilder setNotSovereignLender(Boolean notSovereignLender) {
			this.notSovereignLender = notSovereignLender;
			return this;
		}
		public ObligationsBuilder setNotSubordinated(Boolean notSubordinated) {
			this.notSubordinated = notSubordinated;
			return this;
		}
		public ObligationsBuilder setOthReferenceEntityObligations(String othReferenceEntityObligations) {
			this.othReferenceEntityObligations = othReferenceEntityObligations;
			return this;
		}
		public ObligationsBuilder setRevenueObligationLiability(Boolean revenueObligationLiability) {
			this.revenueObligationLiability = revenueObligationLiability;
			return this;
		}
		public ObligationsBuilder setSpecifiedCurrency(SpecifiedCurrency specifiedCurrency) {
			this.specifiedCurrency = specifiedCurrency.toBuilder();
			return this;
		}
		public ObligationsBuilder setSpecifiedCurrencyBuilder(SpecifiedCurrency.SpecifiedCurrencyBuilder specifiedCurrency) {
			this.specifiedCurrency = specifiedCurrency;
			return this;
		}
		
		public Obligations build() {
			return new Obligations(this);
		}
	
		@Override
		public ObligationsBuilder prune() {
			if (designatedPriority!=null && !designatedPriority.prune().hasData()) designatedPriority = null;
			if (notDomesticCurrency!=null && !notDomesticCurrency.prune().hasData()) notDomesticCurrency = null;
			if (specifiedCurrency!=null && !specifiedCurrency.prune().hasData()) specifiedCurrency = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getCashSettlementOnly()!=null) return true;
			if (getCategory()!=null) return true;
			if (getContinuity()!=null) return true;
			if (getDeliveryOfCommitments()!=null) return true;
			if (getDesignatedPriority()!=null) return true;
			if (getExcluded()!=null) return true;
			if (getFullFaithAndCreditObLiability()!=null) return true;
			if (getGeneralFundObligationLiability()!=null) return true;
			if (getListed()!=null) return true;
			if (getNotContingent()!=null) return true;
			if (getNotDomesticCurrency()!=null && getNotDomesticCurrency().hasData()) return true;
			if (getNotDomesticIssuance()!=null) return true;
			if (getNotDomesticLaw()!=null) return true;
			if (getNotSovereignLender()!=null) return true;
			if (getNotSubordinated()!=null) return true;
			if (getOthReferenceEntityObligations()!=null) return true;
			if (getRevenueObligationLiability()!=null) return true;
			if (getSpecifiedCurrency()!=null && getSpecifiedCurrency().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("cashSettlementOnly"), Boolean.class, cashSettlementOnly, this);
			processor.processBasic(path.newSubPath("category"), ObligationCategoryEnum.class, category, this);
			processor.processBasic(path.newSubPath("continuity"), Boolean.class, continuity, this);
			processor.processBasic(path.newSubPath("deliveryOfCommitments"), Boolean.class, deliveryOfCommitments, this);
			processor.processBasic(path.newSubPath("excluded"), String.class, excluded, this);
			processor.processBasic(path.newSubPath("fullFaithAndCreditObLiability"), Boolean.class, fullFaithAndCreditObLiability, this);
			processor.processBasic(path.newSubPath("generalFundObligationLiability"), Boolean.class, generalFundObligationLiability, this);
			processor.processBasic(path.newSubPath("listed"), Boolean.class, listed, this);
			processor.processBasic(path.newSubPath("notContingent"), Boolean.class, notContingent, this);
			processor.processBasic(path.newSubPath("notDomesticIssuance"), Boolean.class, notDomesticIssuance, this);
			processor.processBasic(path.newSubPath("notDomesticLaw"), Boolean.class, notDomesticLaw, this);
			processor.processBasic(path.newSubPath("notSovereignLender"), Boolean.class, notSovereignLender, this);
			processor.processBasic(path.newSubPath("notSubordinated"), Boolean.class, notSubordinated, this);
			processor.processBasic(path.newSubPath("othReferenceEntityObligations"), String.class, othReferenceEntityObligations, this);
			processor.processBasic(path.newSubPath("revenueObligationLiability"), Boolean.class, revenueObligationLiability, this);
			
			processRosetta(path.newSubPath("designatedPriority"), processor, FieldWithMetaString.class, designatedPriority);
			processRosetta(path.newSubPath("notDomesticCurrency"), processor, NotDomesticCurrency.class, notDomesticCurrency);
			processRosetta(path.newSubPath("specifiedCurrency"), processor, SpecifiedCurrency.class, specifiedCurrency);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			ObligationsBuilder _that = (ObligationsBuilder) o;
		
			if (cashSettlementOnly != null ? !cashSettlementOnly.equals(_that.cashSettlementOnly) : _that.cashSettlementOnly != null) return false;
			if (category != null ? !category.equals(_that.category) : _that.category != null) return false;
			if (continuity != null ? !continuity.equals(_that.continuity) : _that.continuity != null) return false;
			if (deliveryOfCommitments != null ? !deliveryOfCommitments.equals(_that.deliveryOfCommitments) : _that.deliveryOfCommitments != null) return false;
			if (designatedPriority != null ? !designatedPriority.equals(_that.designatedPriority) : _that.designatedPriority != null) return false;
			if (excluded != null ? !excluded.equals(_that.excluded) : _that.excluded != null) return false;
			if (fullFaithAndCreditObLiability != null ? !fullFaithAndCreditObLiability.equals(_that.fullFaithAndCreditObLiability) : _that.fullFaithAndCreditObLiability != null) return false;
			if (generalFundObligationLiability != null ? !generalFundObligationLiability.equals(_that.generalFundObligationLiability) : _that.generalFundObligationLiability != null) return false;
			if (listed != null ? !listed.equals(_that.listed) : _that.listed != null) return false;
			if (notContingent != null ? !notContingent.equals(_that.notContingent) : _that.notContingent != null) return false;
			if (notDomesticCurrency != null ? !notDomesticCurrency.equals(_that.notDomesticCurrency) : _that.notDomesticCurrency != null) return false;
			if (notDomesticIssuance != null ? !notDomesticIssuance.equals(_that.notDomesticIssuance) : _that.notDomesticIssuance != null) return false;
			if (notDomesticLaw != null ? !notDomesticLaw.equals(_that.notDomesticLaw) : _that.notDomesticLaw != null) return false;
			if (notSovereignLender != null ? !notSovereignLender.equals(_that.notSovereignLender) : _that.notSovereignLender != null) return false;
			if (notSubordinated != null ? !notSubordinated.equals(_that.notSubordinated) : _that.notSubordinated != null) return false;
			if (othReferenceEntityObligations != null ? !othReferenceEntityObligations.equals(_that.othReferenceEntityObligations) : _that.othReferenceEntityObligations != null) return false;
			if (revenueObligationLiability != null ? !revenueObligationLiability.equals(_that.revenueObligationLiability) : _that.revenueObligationLiability != null) return false;
			if (specifiedCurrency != null ? !specifiedCurrency.equals(_that.specifiedCurrency) : _that.specifiedCurrency != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (cashSettlementOnly != null ? cashSettlementOnly.hashCode() : 0);
			_result = 31 * _result + (category != null ? category.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (continuity != null ? continuity.hashCode() : 0);
			_result = 31 * _result + (deliveryOfCommitments != null ? deliveryOfCommitments.hashCode() : 0);
			_result = 31 * _result + (designatedPriority != null ? designatedPriority.hashCode() : 0);
			_result = 31 * _result + (excluded != null ? excluded.hashCode() : 0);
			_result = 31 * _result + (fullFaithAndCreditObLiability != null ? fullFaithAndCreditObLiability.hashCode() : 0);
			_result = 31 * _result + (generalFundObligationLiability != null ? generalFundObligationLiability.hashCode() : 0);
			_result = 31 * _result + (listed != null ? listed.hashCode() : 0);
			_result = 31 * _result + (notContingent != null ? notContingent.hashCode() : 0);
			_result = 31 * _result + (notDomesticCurrency != null ? notDomesticCurrency.hashCode() : 0);
			_result = 31 * _result + (notDomesticIssuance != null ? notDomesticIssuance.hashCode() : 0);
			_result = 31 * _result + (notDomesticLaw != null ? notDomesticLaw.hashCode() : 0);
			_result = 31 * _result + (notSovereignLender != null ? notSovereignLender.hashCode() : 0);
			_result = 31 * _result + (notSubordinated != null ? notSubordinated.hashCode() : 0);
			_result = 31 * _result + (othReferenceEntityObligations != null ? othReferenceEntityObligations.hashCode() : 0);
			_result = 31 * _result + (revenueObligationLiability != null ? revenueObligationLiability.hashCode() : 0);
			_result = 31 * _result + (specifiedCurrency != null ? specifiedCurrency.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "ObligationsBuilder {" +
				"cashSettlementOnly=" + this.cashSettlementOnly + ", " +
				"category=" + this.category + ", " +
				"continuity=" + this.continuity + ", " +
				"deliveryOfCommitments=" + this.deliveryOfCommitments + ", " +
				"designatedPriority=" + this.designatedPriority + ", " +
				"excluded=" + this.excluded + ", " +
				"fullFaithAndCreditObLiability=" + this.fullFaithAndCreditObLiability + ", " +
				"generalFundObligationLiability=" + this.generalFundObligationLiability + ", " +
				"listed=" + this.listed + ", " +
				"notContingent=" + this.notContingent + ", " +
				"notDomesticCurrency=" + this.notDomesticCurrency + ", " +
				"notDomesticIssuance=" + this.notDomesticIssuance + ", " +
				"notDomesticLaw=" + this.notDomesticLaw + ", " +
				"notSovereignLender=" + this.notSovereignLender + ", " +
				"notSubordinated=" + this.notSubordinated + ", " +
				"othReferenceEntityObligations=" + this.othReferenceEntityObligations + ", " +
				"revenueObligationLiability=" + this.revenueObligationLiability + ", " +
				"specifiedCurrency=" + this.specifiedCurrency +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("cashSettlementOnly"), Boolean.class, cashSettlementOnly, this);
		processor.processBasic(path.newSubPath("category"), ObligationCategoryEnum.class, category, this);
		processor.processBasic(path.newSubPath("continuity"), Boolean.class, continuity, this);
		processor.processBasic(path.newSubPath("deliveryOfCommitments"), Boolean.class, deliveryOfCommitments, this);
		processor.processBasic(path.newSubPath("excluded"), String.class, excluded, this);
		processor.processBasic(path.newSubPath("fullFaithAndCreditObLiability"), Boolean.class, fullFaithAndCreditObLiability, this);
		processor.processBasic(path.newSubPath("generalFundObligationLiability"), Boolean.class, generalFundObligationLiability, this);
		processor.processBasic(path.newSubPath("listed"), Boolean.class, listed, this);
		processor.processBasic(path.newSubPath("notContingent"), Boolean.class, notContingent, this);
		processor.processBasic(path.newSubPath("notDomesticIssuance"), Boolean.class, notDomesticIssuance, this);
		processor.processBasic(path.newSubPath("notDomesticLaw"), Boolean.class, notDomesticLaw, this);
		processor.processBasic(path.newSubPath("notSovereignLender"), Boolean.class, notSovereignLender, this);
		processor.processBasic(path.newSubPath("notSubordinated"), Boolean.class, notSubordinated, this);
		processor.processBasic(path.newSubPath("othReferenceEntityObligations"), String.class, othReferenceEntityObligations, this);
		processor.processBasic(path.newSubPath("revenueObligationLiability"), Boolean.class, revenueObligationLiability, this);
		
		processRosetta(path.newSubPath("designatedPriority"), processor, FieldWithMetaString.class, designatedPriority);
		processRosetta(path.newSubPath("notDomesticCurrency"), processor, NotDomesticCurrency.class, notDomesticCurrency);
		processRosetta(path.newSubPath("specifiedCurrency"), processor, SpecifiedCurrency.class, specifiedCurrency);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		Obligations _that = (Obligations) o;
	
		if (cashSettlementOnly != null ? !cashSettlementOnly.equals(_that.cashSettlementOnly) : _that.cashSettlementOnly != null) return false;
		if (category != null ? !category.equals(_that.category) : _that.category != null) return false;
		if (continuity != null ? !continuity.equals(_that.continuity) : _that.continuity != null) return false;
		if (deliveryOfCommitments != null ? !deliveryOfCommitments.equals(_that.deliveryOfCommitments) : _that.deliveryOfCommitments != null) return false;
		if (designatedPriority != null ? !designatedPriority.equals(_that.designatedPriority) : _that.designatedPriority != null) return false;
		if (excluded != null ? !excluded.equals(_that.excluded) : _that.excluded != null) return false;
		if (fullFaithAndCreditObLiability != null ? !fullFaithAndCreditObLiability.equals(_that.fullFaithAndCreditObLiability) : _that.fullFaithAndCreditObLiability != null) return false;
		if (generalFundObligationLiability != null ? !generalFundObligationLiability.equals(_that.generalFundObligationLiability) : _that.generalFundObligationLiability != null) return false;
		if (listed != null ? !listed.equals(_that.listed) : _that.listed != null) return false;
		if (notContingent != null ? !notContingent.equals(_that.notContingent) : _that.notContingent != null) return false;
		if (notDomesticCurrency != null ? !notDomesticCurrency.equals(_that.notDomesticCurrency) : _that.notDomesticCurrency != null) return false;
		if (notDomesticIssuance != null ? !notDomesticIssuance.equals(_that.notDomesticIssuance) : _that.notDomesticIssuance != null) return false;
		if (notDomesticLaw != null ? !notDomesticLaw.equals(_that.notDomesticLaw) : _that.notDomesticLaw != null) return false;
		if (notSovereignLender != null ? !notSovereignLender.equals(_that.notSovereignLender) : _that.notSovereignLender != null) return false;
		if (notSubordinated != null ? !notSubordinated.equals(_that.notSubordinated) : _that.notSubordinated != null) return false;
		if (othReferenceEntityObligations != null ? !othReferenceEntityObligations.equals(_that.othReferenceEntityObligations) : _that.othReferenceEntityObligations != null) return false;
		if (revenueObligationLiability != null ? !revenueObligationLiability.equals(_that.revenueObligationLiability) : _that.revenueObligationLiability != null) return false;
		if (specifiedCurrency != null ? !specifiedCurrency.equals(_that.specifiedCurrency) : _that.specifiedCurrency != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (cashSettlementOnly != null ? cashSettlementOnly.hashCode() : 0);
		_result = 31 * _result + (category != null ? category.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (continuity != null ? continuity.hashCode() : 0);
		_result = 31 * _result + (deliveryOfCommitments != null ? deliveryOfCommitments.hashCode() : 0);
		_result = 31 * _result + (designatedPriority != null ? designatedPriority.hashCode() : 0);
		_result = 31 * _result + (excluded != null ? excluded.hashCode() : 0);
		_result = 31 * _result + (fullFaithAndCreditObLiability != null ? fullFaithAndCreditObLiability.hashCode() : 0);
		_result = 31 * _result + (generalFundObligationLiability != null ? generalFundObligationLiability.hashCode() : 0);
		_result = 31 * _result + (listed != null ? listed.hashCode() : 0);
		_result = 31 * _result + (notContingent != null ? notContingent.hashCode() : 0);
		_result = 31 * _result + (notDomesticCurrency != null ? notDomesticCurrency.hashCode() : 0);
		_result = 31 * _result + (notDomesticIssuance != null ? notDomesticIssuance.hashCode() : 0);
		_result = 31 * _result + (notDomesticLaw != null ? notDomesticLaw.hashCode() : 0);
		_result = 31 * _result + (notSovereignLender != null ? notSovereignLender.hashCode() : 0);
		_result = 31 * _result + (notSubordinated != null ? notSubordinated.hashCode() : 0);
		_result = 31 * _result + (othReferenceEntityObligations != null ? othReferenceEntityObligations.hashCode() : 0);
		_result = 31 * _result + (revenueObligationLiability != null ? revenueObligationLiability.hashCode() : 0);
		_result = 31 * _result + (specifiedCurrency != null ? specifiedCurrency.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "Obligations {" +
			"cashSettlementOnly=" + this.cashSettlementOnly + ", " +
			"category=" + this.category + ", " +
			"continuity=" + this.continuity + ", " +
			"deliveryOfCommitments=" + this.deliveryOfCommitments + ", " +
			"designatedPriority=" + this.designatedPriority + ", " +
			"excluded=" + this.excluded + ", " +
			"fullFaithAndCreditObLiability=" + this.fullFaithAndCreditObLiability + ", " +
			"generalFundObligationLiability=" + this.generalFundObligationLiability + ", " +
			"listed=" + this.listed + ", " +
			"notContingent=" + this.notContingent + ", " +
			"notDomesticCurrency=" + this.notDomesticCurrency + ", " +
			"notDomesticIssuance=" + this.notDomesticIssuance + ", " +
			"notDomesticLaw=" + this.notDomesticLaw + ", " +
			"notSovereignLender=" + this.notSovereignLender + ", " +
			"notSubordinated=" + this.notSubordinated + ", " +
			"othReferenceEntityObligations=" + this.othReferenceEntityObligations + ", " +
			"revenueObligationLiability=" + this.revenueObligationLiability + ", " +
			"specifiedCurrency=" + this.specifiedCurrency +
		'}';
	}
}
