package org.isda.cdm;

import com.rosetta.model.lib.records.Date;
import java.math.BigDecimal;
import org.isda.cdm.ObservationSource;
import org.isda.cdm.QuotationSideEnum;
import org.isda.cdm.TimeZone;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.ObservationPrimitiveMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;
		import java.math.BigDecimal;
		import com.rosetta.model.lib.records.Date;

/**
 * A class to specify the primitive object to specify market observation events, which is applicable across all asset classes.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class ObservationPrimitive extends RosettaModelObject {
	private final Date date;
	private final BigDecimal observation;
	private final QuotationSideEnum side;
	private final ObservationSource source;
	private final TimeZone time;
	private static ObservationPrimitiveMeta metaData = new ObservationPrimitiveMeta();
	
	ObservationPrimitive(ObservationPrimitiveBuilder builder) {
		this.date = builder.getDate();
		this.observation = builder.getObservation();
		this.side = builder.getSide();
		this.source = ofNullable(builder.getSource()).map(f->f.build()).orElse(null);
		this.time = ofNullable(builder.getTime()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The observation date.
	 */
	@RosettaSynonym(value="date", source="Rosetta_Workbench")
	@RosettaSynonym(value="observationDate", source="DTCC_11_0")
	@RosettaSynonym(value="observationDate", source="DTCC_9_0")
	public final Date getDate() {
		return date;
	}
	
	/**
	 * The observed value.
	 */
	@RosettaSynonym(value="observation", source="Rosetta_Workbench")
	public final BigDecimal getObservation() {
		return observation;
	}
	
	/**
	 * The side (bid/mid/ask) of the observation, when applicable.
	 */
	@RosettaSynonym(value="side", source="Rosetta_Workbench")
	public final QuotationSideEnum getSide() {
		return side;
	}
	
	/**
	 * The observation source, such as an interest rate curve or an information provider.
	 */
	@RosettaSynonym(value="source", source="Rosetta_Workbench")
	public final ObservationSource getSource() {
		return source;
	}
	
	/**
	 * The observation time.
	 */
	@RosettaSynonym(value="time", source="Rosetta_Workbench")
	public final TimeZone getTime() {
		return time;
	}
	
	
	@Override
	public RosettaMetaData<? extends ObservationPrimitive> metaData() {
		return metaData;
	} 
	
	public ObservationPrimitiveBuilder toBuilder() {
		ObservationPrimitiveBuilder builder = new ObservationPrimitiveBuilder();
		ofNullable(getDate()).ifPresent(builder::setDate);
		ofNullable(getObservation()).ifPresent(builder::setObservation);
		ofNullable(getSide()).ifPresent(builder::setSide);
		ofNullable(getSource()).ifPresent(builder::setSource);
		ofNullable(getTime()).ifPresent(builder::setTime);
		return builder;
	}

	public static ObservationPrimitiveBuilder builder() {
		return new ObservationPrimitiveBuilder();
	}

	public static  class ObservationPrimitiveBuilder extends RosettaModelObjectBuilder {
	
		protected Date date;
		protected BigDecimal observation;
		protected QuotationSideEnum side;
		protected ObservationSource.ObservationSourceBuilder source;
		protected TimeZone.TimeZoneBuilder time;
	
		public ObservationPrimitiveBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends ObservationPrimitive> metaData() {
			return metaData;
		} 
	
		
		public Date getDate() {
			return date;
		}
		
		
		public BigDecimal getObservation() {
			return observation;
		}
		
		
		public QuotationSideEnum getSide() {
			return side;
		}
		
		
		public ObservationSource.ObservationSourceBuilder getSource() {
			return source;
		}
		
		public ObservationSource.ObservationSourceBuilder getOrCreateSource() {
			if (source!=null) {
				return source;
			}
			else return source = new ObservationSource.ObservationSourceBuilder();
		}
		
		
		public TimeZone.TimeZoneBuilder getTime() {
			return time;
		}
		
		public TimeZone.TimeZoneBuilder getOrCreateTime() {
			if (time!=null) {
				return time;
			}
			else return time = new TimeZone.TimeZoneBuilder();
		}
		
	
		public ObservationPrimitiveBuilder setDate(Date date) {
			this.date = date;
			return this;
		}
		public ObservationPrimitiveBuilder setObservation(BigDecimal observation) {
			this.observation = observation;
			return this;
		}
		public ObservationPrimitiveBuilder setSide(QuotationSideEnum side) {
			this.side = side;
			return this;
		}
		public ObservationPrimitiveBuilder setSource(ObservationSource source) {
			this.source = source.toBuilder();
			return this;
		}
		public ObservationPrimitiveBuilder setSourceBuilder(ObservationSource.ObservationSourceBuilder source) {
			this.source = source;
			return this;
		}
		public ObservationPrimitiveBuilder setTime(TimeZone time) {
			this.time = time.toBuilder();
			return this;
		}
		public ObservationPrimitiveBuilder setTimeBuilder(TimeZone.TimeZoneBuilder time) {
			this.time = time;
			return this;
		}
		
		public ObservationPrimitive build() {
			return new ObservationPrimitive(this);
		}
	
		@Override
		public ObservationPrimitiveBuilder prune() {
			if (source!=null && !source.prune().hasData()) source = null;
			if (time!=null && !time.prune().hasData()) time = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getDate()!=null) return true;
			if (getObservation()!=null) return true;
			if (getSide()!=null) return true;
			if (getSource()!=null && getSource().hasData()) return true;
			if (getTime()!=null && getTime().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("date"), Date.class, date, this);
			processor.processBasic(path.newSubPath("observation"), BigDecimal.class, observation, this);
			processor.processBasic(path.newSubPath("side"), QuotationSideEnum.class, side, this);
			
			processRosetta(path.newSubPath("source"), processor, ObservationSource.class, source);
			processRosetta(path.newSubPath("time"), processor, TimeZone.class, time);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			ObservationPrimitiveBuilder _that = (ObservationPrimitiveBuilder) o;
		
			if (date != null ? !date.equals(_that.date) : _that.date != null) return false;
			if (observation != null ? !observation.equals(_that.observation) : _that.observation != null) return false;
			if (side != null ? !side.equals(_that.side) : _that.side != null) return false;
			if (source != null ? !source.equals(_that.source) : _that.source != null) return false;
			if (time != null ? !time.equals(_that.time) : _that.time != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (date != null ? date.hashCode() : 0);
			_result = 31 * _result + (observation != null ? observation.hashCode() : 0);
			_result = 31 * _result + (side != null ? side.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (source != null ? source.hashCode() : 0);
			_result = 31 * _result + (time != null ? time.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "ObservationPrimitiveBuilder {" +
				"date=" + this.date + ", " +
				"observation=" + this.observation + ", " +
				"side=" + this.side + ", " +
				"source=" + this.source + ", " +
				"time=" + this.time +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("date"), Date.class, date, this);
		processor.processBasic(path.newSubPath("observation"), BigDecimal.class, observation, this);
		processor.processBasic(path.newSubPath("side"), QuotationSideEnum.class, side, this);
		
		processRosetta(path.newSubPath("source"), processor, ObservationSource.class, source);
		processRosetta(path.newSubPath("time"), processor, TimeZone.class, time);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		ObservationPrimitive _that = (ObservationPrimitive) o;
	
		if (date != null ? !date.equals(_that.date) : _that.date != null) return false;
		if (observation != null ? !observation.equals(_that.observation) : _that.observation != null) return false;
		if (side != null ? !side.equals(_that.side) : _that.side != null) return false;
		if (source != null ? !source.equals(_that.source) : _that.source != null) return false;
		if (time != null ? !time.equals(_that.time) : _that.time != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (date != null ? date.hashCode() : 0);
		_result = 31 * _result + (observation != null ? observation.hashCode() : 0);
		_result = 31 * _result + (side != null ? side.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (source != null ? source.hashCode() : 0);
		_result = 31 * _result + (time != null ? time.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "ObservationPrimitive {" +
			"date=" + this.date + ", " +
			"observation=" + this.observation + ", " +
			"side=" + this.side + ", " +
			"source=" + this.source + ", " +
			"time=" + this.time +
		'}';
	}
}
