package org.isda.cdm;

import org.isda.cdm.Curve;
import org.isda.cdm.InformationSource;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.ObservationSourceMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * The observation source can be composed of an curve and/or and information source.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class ObservationSource extends RosettaModelObject {
	private final Curve curve;
	private final InformationSource informationSource;
	private static ObservationSourceMeta metaData = new ObservationSourceMeta();
	
	ObservationSource(ObservationSourceBuilder builder) {
		this.curve = ofNullable(builder.getCurve()).map(f->f.build()).orElse(null);
		this.informationSource = ofNullable(builder.getInformationSource()).map(f->f.build()).orElse(null);
	}
	
	@RosettaSynonym(value="curve", source="Rosetta_Workbench")
	public final Curve getCurve() {
		return curve;
	}
	
	@RosettaSynonym(value="informationSource", source="Rosetta_Workbench")
	public final InformationSource getInformationSource() {
		return informationSource;
	}
	
	
	@Override
	public RosettaMetaData<? extends ObservationSource> metaData() {
		return metaData;
	} 
	
	public ObservationSourceBuilder toBuilder() {
		ObservationSourceBuilder builder = new ObservationSourceBuilder();
		ofNullable(getCurve()).ifPresent(builder::setCurve);
		ofNullable(getInformationSource()).ifPresent(builder::setInformationSource);
		return builder;
	}

	public static ObservationSourceBuilder builder() {
		return new ObservationSourceBuilder();
	}

	public static  class ObservationSourceBuilder extends RosettaModelObjectBuilder {
	
		protected Curve.CurveBuilder curve;
		protected InformationSource.InformationSourceBuilder informationSource;
	
		public ObservationSourceBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends ObservationSource> metaData() {
			return metaData;
		} 
	
		
		public Curve.CurveBuilder getCurve() {
			return curve;
		}
		
		public Curve.CurveBuilder getOrCreateCurve() {
			if (curve!=null) {
				return curve;
			}
			else return curve = new Curve.CurveBuilder();
		}
		
		
		public InformationSource.InformationSourceBuilder getInformationSource() {
			return informationSource;
		}
		
		public InformationSource.InformationSourceBuilder getOrCreateInformationSource() {
			if (informationSource!=null) {
				return informationSource;
			}
			else return informationSource = new InformationSource.InformationSourceBuilder();
		}
		
	
		public ObservationSourceBuilder setCurve(Curve curve) {
			this.curve = curve.toBuilder();
			return this;
		}
		public ObservationSourceBuilder setCurveBuilder(Curve.CurveBuilder curve) {
			this.curve = curve;
			return this;
		}
		public ObservationSourceBuilder setInformationSource(InformationSource informationSource) {
			this.informationSource = informationSource.toBuilder();
			return this;
		}
		public ObservationSourceBuilder setInformationSourceBuilder(InformationSource.InformationSourceBuilder informationSource) {
			this.informationSource = informationSource;
			return this;
		}
		
		public ObservationSource build() {
			return new ObservationSource(this);
		}
	
		@Override
		public ObservationSourceBuilder prune() {
			if (curve!=null && !curve.prune().hasData()) curve = null;
			if (informationSource!=null && !informationSource.prune().hasData()) informationSource = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getCurve()!=null && getCurve().hasData()) return true;
			if (getInformationSource()!=null && getInformationSource().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("curve"), processor, Curve.class, curve);
			processRosetta(path.newSubPath("informationSource"), processor, InformationSource.class, informationSource);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			ObservationSourceBuilder _that = (ObservationSourceBuilder) o;
		
			if (curve != null ? !curve.equals(_that.curve) : _that.curve != null) return false;
			if (informationSource != null ? !informationSource.equals(_that.informationSource) : _that.informationSource != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (curve != null ? curve.hashCode() : 0);
			_result = 31 * _result + (informationSource != null ? informationSource.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "ObservationSourceBuilder {" +
				"curve=" + this.curve + ", " +
				"informationSource=" + this.informationSource +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("curve"), processor, Curve.class, curve);
		processRosetta(path.newSubPath("informationSource"), processor, InformationSource.class, informationSource);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		ObservationSource _that = (ObservationSource) o;
	
		if (curve != null ? !curve.equals(_that.curve) : _that.curve != null) return false;
		if (informationSource != null ? !informationSource.equals(_that.informationSource) : _that.informationSource != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (curve != null ? curve.hashCode() : 0);
		_result = 31 * _result + (informationSource != null ? informationSource.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "ObservationSource {" +
			"curve=" + this.curve + ", " +
			"informationSource=" + this.informationSource +
		'}';
	}
}
