package org.isda.cdm;

import org.isda.cdm.DayTypeEnum;


import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.OffsetMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class defining an offset used in calculating a new date relative to a reference date, e.g. calendar days, business days, commodity Business days, etc.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="Offset", source="FpML_5_10")
public class Offset extends Period {
	private final DayTypeEnum dayType;
	private static OffsetMeta metaData = new OffsetMeta();
	
	Offset(OffsetBuilder builder) {
		super(builder);
		this.dayType = builder.getDayType();
	}
	
	/**
	 * In the case of an offset specified as a number of days, this element defines whether consideration is given as to whether a day is a good business day or not. If a day type of business days is specified then non-business days are ignored when calculating the offset. The financial business centers to use for determination of business days are implied by the context in which this element is used. This element must only be included when the offset is specified as a number of days. If the offset is zero days then the dayType element should not be included.
	 */
	@RosettaSynonym(value="dayType", source="FpML_5_10")
	@RosettaSynonym(value="dayType", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="dayType", source="DTCC_11_0")
	@RosettaSynonym(value="dayType", source="DTCC_9_0")
	@RosettaSynonym(value="dayType", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="", source="ISDA_Create_1_0")
	public final DayTypeEnum getDayType() {
		return dayType;
	}
	
	
	@Override
	public RosettaMetaData<? extends Offset> metaData() {
		return metaData;
	} 
	
	public OffsetBuilder toBuilder() {
		OffsetBuilder builder = new OffsetBuilder();
		ofNullable(getDayType()).ifPresent(builder::setDayType);
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		ofNullable(getPeriod()).ifPresent(builder::setPeriod);
		ofNullable(getPeriodMultiplier()).ifPresent(builder::setPeriodMultiplier);
		return builder;
	}

	public static OffsetBuilder builder() {
		return new OffsetBuilder();
	}

	public static  class OffsetBuilder extends PeriodBuilder {
	
		protected DayTypeEnum dayType;
	
		public OffsetBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends Offset> metaData() {
			return metaData;
		} 
	
		
		public DayTypeEnum getDayType() {
			return dayType;
		}
		
	
		public OffsetBuilder setDayType(DayTypeEnum dayType) {
			this.dayType = dayType;
			return this;
		}
		@Override public OffsetBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		@Override public OffsetBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		@Override public OffsetBuilder setPeriod(PeriodEnum period) {
			this.period = period;
			return this;
		}
		@Override public OffsetBuilder setPeriodMultiplier(Integer periodMultiplier) {
			this.periodMultiplier = periodMultiplier;
			return this;
		}
		
		public Offset build() {
			return new Offset(this);
		}
	
		@Override
		public OffsetBuilder prune() {
			super.prune();
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (super.hasData()) return true;
			if (getDayType()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
			super.process(path, processor);
		
			processor.processBasic(path.newSubPath("dayType"), DayTypeEnum.class, dayType, this);
			
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
			if (!super.equals(o)) return false;
		
			OffsetBuilder _that = (OffsetBuilder) o;
		
			if (dayType != null ? !dayType.equals(_that.dayType) : _that.dayType != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = super.hashCode();
			_result = 31 * _result + (dayType != null ? dayType.getClass().getName().hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "OffsetBuilder {" +
				"dayType=" + this.dayType +
			'}' + " " + super.toString();
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		super.process(path, processor);
		
		processor.processBasic(path.newSubPath("dayType"), DayTypeEnum.class, dayType, this);
		
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
		if (!super.equals(o)) return false;
	
		Offset _that = (Offset) o;
	
		if (dayType != null ? !dayType.equals(_that.dayType) : _that.dayType != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = super.hashCode();
		_result = 31 * _result + (dayType != null ? dayType.getClass().getName().hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "Offset {" +
			"dayType=" + this.dayType +
		'}' + " " + super.toString();
	}
}
