package org.isda.cdm;

import java.lang.Boolean;
import java.lang.String;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.OneWayProvisionsMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class to specify whether One Way Provisions apply in relation to the ISDA CSA for Initial Margin and, if yes, to specify the Posting Party. ISDA 2016 Credit Support Annex for Initial Margin, paragraph 13, General Principles (aa): One Way Provisions.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class OneWayProvisions extends RosettaModelObject {
	private final Boolean isApplicable;
	private final String postingParty;
	private static OneWayProvisionsMeta metaData = new OneWayProvisionsMeta();
	
	OneWayProvisions(OneWayProvisionsBuilder builder) {
		this.isApplicable = builder.getIsApplicable();
		this.postingParty = builder.getPostingParty();
	}
	
	/**
	 * The determination of whether the One Way Provisions are applicable (true) or not applicable (false).
	 */
	@RosettaSynonym(value="", source="ISDA_Create_1_0")
	public final Boolean getIsApplicable() {
		return isApplicable;
	}
	
	/**
	 * The Posting Party for the purposes of One Way Provisions. It is specified in the case where the One Way Provision is deemed applicable.
	 */
	@RosettaSynonym(value="posting_party", source="ISDA_Create_1_0")
	public final String getPostingParty() {
		return postingParty;
	}
	
	
	@Override
	public RosettaMetaData<? extends OneWayProvisions> metaData() {
		return metaData;
	} 
	
	public OneWayProvisionsBuilder toBuilder() {
		OneWayProvisionsBuilder builder = new OneWayProvisionsBuilder();
		ofNullable(getIsApplicable()).ifPresent(builder::setIsApplicable);
		ofNullable(getPostingParty()).ifPresent(builder::setPostingParty);
		return builder;
	}

	public static OneWayProvisionsBuilder builder() {
		return new OneWayProvisionsBuilder();
	}

	public static  class OneWayProvisionsBuilder extends RosettaModelObjectBuilder {
	
		protected Boolean isApplicable;
		protected String postingParty;
	
		public OneWayProvisionsBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends OneWayProvisions> metaData() {
			return metaData;
		} 
	
		
		public Boolean getIsApplicable() {
			return isApplicable;
		}
		
		
		public String getPostingParty() {
			return postingParty;
		}
		
	
		public OneWayProvisionsBuilder setIsApplicable(Boolean isApplicable) {
			this.isApplicable = isApplicable;
			return this;
		}
		public OneWayProvisionsBuilder setPostingParty(String postingParty) {
			this.postingParty = postingParty;
			return this;
		}
		
		public OneWayProvisions build() {
			return new OneWayProvisions(this);
		}
	
		@Override
		public OneWayProvisionsBuilder prune() {
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getIsApplicable()!=null) return true;
			if (getPostingParty()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("isApplicable"), Boolean.class, isApplicable, this);
			processor.processBasic(path.newSubPath("postingParty"), String.class, postingParty, this);
			
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			OneWayProvisionsBuilder _that = (OneWayProvisionsBuilder) o;
		
			if (isApplicable != null ? !isApplicable.equals(_that.isApplicable) : _that.isApplicable != null) return false;
			if (postingParty != null ? !postingParty.equals(_that.postingParty) : _that.postingParty != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (isApplicable != null ? isApplicable.hashCode() : 0);
			_result = 31 * _result + (postingParty != null ? postingParty.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "OneWayProvisionsBuilder {" +
				"isApplicable=" + this.isApplicable + ", " +
				"postingParty=" + this.postingParty +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("isApplicable"), Boolean.class, isApplicable, this);
		processor.processBasic(path.newSubPath("postingParty"), String.class, postingParty, this);
		
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		OneWayProvisions _that = (OneWayProvisions) o;
	
		if (isApplicable != null ? !isApplicable.equals(_that.isApplicable) : _that.isApplicable != null) return false;
		if (postingParty != null ? !postingParty.equals(_that.postingParty) : _that.postingParty != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (isApplicable != null ? isApplicable.hashCode() : 0);
		_result = 31 * _result + (postingParty != null ? postingParty.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "OneWayProvisions {" +
			"isApplicable=" + this.isApplicable + ", " +
			"postingParty=" + this.postingParty +
		'}';
	}
}
