package org.isda.cdm;

import org.isda.cdm.BusinessCenterTime;
import org.isda.cdm.CashPriceMethod;
import org.isda.cdm.CashSettlementPaymentDate;
import org.isda.cdm.CrossCurrencyMethod;
import org.isda.cdm.RelativeDateOffset;
import org.isda.cdm.YieldCurveMethod;
import org.isda.cdm.metafields.MetaFields;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.OptionCashSettlementMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class to define the cash settlement terms for a product where cash settlement is applicable.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="CashSettlement", source="FpML_5_10")
@RosettaSynonym(value="CashSettlement", source="CME_SubmissionIRS_1_0")
@RosettaSynonym(value="CashSettlement", source="DTCC_11_0")
@RosettaSynonym(value="CashSettlement", source="DTCC_9_0")
@RosettaSynonym(value="CashSettlement", source="CME_ClearedConfirm_1_17")
public class OptionCashSettlement extends RosettaModelObject implements GlobalKey {
	private final CashPriceMethod cashPriceAlternateMethod;
	private final CashPriceMethod cashPriceMethod;
	private final CashSettlementPaymentDate cashSettlementPaymentDate;
	private final RelativeDateOffset cashSettlementValuationDate;
	private final BusinessCenterTime cashSettlementValuationTime;
	private final YieldCurveMethod collateralizedCashPriceMethod;
	private final CrossCurrencyMethod crossCurrencyMethod;
	private final MetaFields meta;
	private final YieldCurveMethod parYieldCurveAdjustedMethod;
	private final YieldCurveMethod parYieldCurveUnadjustedMethod;
	private final YieldCurveMethod zeroCouponYieldAdjustedMethod;
	private static OptionCashSettlementMeta metaData = new OptionCashSettlementMeta();
	
	OptionCashSettlement(OptionCashSettlementBuilder builder) {
		this.cashPriceAlternateMethod = ofNullable(builder.getCashPriceAlternateMethod()).map(f->f.build()).orElse(null);
		this.cashPriceMethod = ofNullable(builder.getCashPriceMethod()).map(f->f.build()).orElse(null);
		this.cashSettlementPaymentDate = ofNullable(builder.getCashSettlementPaymentDate()).map(f->f.build()).orElse(null);
		this.cashSettlementValuationDate = ofNullable(builder.getCashSettlementValuationDate()).map(f->f.build()).orElse(null);
		this.cashSettlementValuationTime = ofNullable(builder.getCashSettlementValuationTime()).map(f->f.build()).orElse(null);
		this.collateralizedCashPriceMethod = ofNullable(builder.getCollateralizedCashPriceMethod()).map(f->f.build()).orElse(null);
		this.crossCurrencyMethod = ofNullable(builder.getCrossCurrencyMethod()).map(f->f.build()).orElse(null);
		this.meta = ofNullable(builder.getMeta()).map(f->f.build()).orElse(null);
		this.parYieldCurveAdjustedMethod = ofNullable(builder.getParYieldCurveAdjustedMethod()).map(f->f.build()).orElse(null);
		this.parYieldCurveUnadjustedMethod = ofNullable(builder.getParYieldCurveUnadjustedMethod()).map(f->f.build()).orElse(null);
		this.zeroCouponYieldAdjustedMethod = ofNullable(builder.getZeroCouponYieldAdjustedMethod()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * An ISDA defined cash settlement method used for the determination of the applicable cash settlement amount. The method is defined in the 2006 ISDA Definitions, Section 18.3. Cash Settlement Methods, paragraph (b).
	 */
	@RosettaSynonym(value="cashPriceAlternateMethod", source="FpML_5_10")
	@RosettaSynonym(value="cashPriceAlternateMethod", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="cashPriceAlternateMethod", source="DTCC_11_0")
	@RosettaSynonym(value="cashPriceAlternateMethod", source="DTCC_9_0")
	@RosettaSynonym(value="cashPriceAlternateMethod", source="CME_ClearedConfirm_1_17")
	public final CashPriceMethod getCashPriceAlternateMethod() {
		return cashPriceAlternateMethod;
	}
	
	/**
	 * An ISDA defined cash settlement method used for the determination of the applicable cash settlement amount. The method is defined in the 2006 ISDA Definitions, Section 18.3. Cash Settlement Methods, paragraph (a).
	 */
	@RosettaSynonym(value="cashPriceMethod", source="FpML_5_10")
	@RosettaSynonym(value="cashPriceMethod", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="cashPriceMethod", source="DTCC_11_0")
	@RosettaSynonym(value="cashPriceMethod", source="DTCC_9_0")
	@RosettaSynonym(value="cashPriceMethod", source="CME_ClearedConfirm_1_17")
	public final CashPriceMethod getCashPriceMethod() {
		return cashPriceMethod;
	}
	
	/**
	 * The date on which the cash settlement amount will be paid, subject to adjustment in accordance with any applicable business day convention. This component would not be present for a mandatory early termination provision where the cash settlement payment date is the mandatory early termination date.
	 */
	@RosettaSynonym(value="cashSettlementPaymentDate", source="FpML_5_10")
	@RosettaSynonym(value="cashSettlementPaymentDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="cashSettlementPaymentDate", source="DTCC_11_0")
	@RosettaSynonym(value="cashSettlementPaymentDate", source="DTCC_9_0")
	@RosettaSynonym(value="cashSettlementPaymentDate", source="CME_ClearedConfirm_1_17")
	public final CashSettlementPaymentDate getCashSettlementPaymentDate() {
		return cashSettlementPaymentDate;
	}
	
	/**
	 * The date on which the cash settlement amount will be determined according to the cash settlement method if the parties have not otherwise been able to agree the cash settlement amount.
	 */
	@RosettaSynonym(value="cashSettlementValuationDate", source="FpML_5_10")
	@RosettaSynonym(value="cashSettlementValuationDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="cashSettlementValuationDate", source="DTCC_11_0")
	@RosettaSynonym(value="cashSettlementValuationDate", source="DTCC_9_0")
	@RosettaSynonym(value="cashSettlementValuationDate", source="CME_ClearedConfirm_1_17")
	public final RelativeDateOffset getCashSettlementValuationDate() {
		return cashSettlementValuationDate;
	}
	
	/**
	 * The time of the cash settlement valuation date when the cash settlement amount will be determined according to the cash settlement method if the parties have not otherwise been able to agree the cash settlement amount.
	 */
	@RosettaSynonym(value="cashSettlementValuationTime", source="FpML_5_10")
	@RosettaSynonym(value="cashSettlementValuationTime", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="cashSettlementValuationTime", source="DTCC_11_0")
	@RosettaSynonym(value="cashSettlementValuationTime", source="DTCC_9_0")
	@RosettaSynonym(value="cashSettlementValuationTime", source="CME_ClearedConfirm_1_17")
	public final BusinessCenterTime getCashSettlementValuationTime() {
		return cashSettlementValuationTime;
	}
	
	/**
	 * An ISDA defined cash settlement method used for the determination of the applicable cash settlement amount. The method is defined in the 2006 ISDA Definitions, Section 18.3. Cash Settlement Methods, paragraph (g) (published in Supplement number 28).
	 */
	@RosettaSynonym(value="collateralizedCashPriceMethod", source="FpML_5_10")
	@RosettaSynonym(value="collateralizedCashPriceMethod", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="collateralizedCashPriceMethod", source="DTCC_11_0")
	@RosettaSynonym(value="collateralizedCashPriceMethod", source="DTCC_9_0")
	@RosettaSynonym(value="collateralizedCashPriceMethod", source="CME_ClearedConfirm_1_17")
	public final YieldCurveMethod getCollateralizedCashPriceMethod() {
		return collateralizedCashPriceMethod;
	}
	
	/**
	 * An ISDA defined cash settlement method used for the determination of the applicable cash settlement amount. The method is defined in the 2006 ISDA Definitions, Section 18.3. Cash Settlement Methods, paragraph (f) (published in Supplement number 23).
	 */
	@RosettaSynonym(value="crossCurrencyMethod", source="FpML_5_10")
	@RosettaSynonym(value="crossCurrencyMethod", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="crossCurrencyMethod", source="DTCC_11_0")
	@RosettaSynonym(value="crossCurrencyMethod", source="DTCC_9_0")
	@RosettaSynonym(value="crossCurrencyMethod", source="CME_ClearedConfirm_1_17")
	public final CrossCurrencyMethod getCrossCurrencyMethod() {
		return crossCurrencyMethod;
	}
	
	public final MetaFields getMeta() {
		return meta;
	}
	
	/**
	 * An ISDA defined cash settlement method used for the determination of the applicable cash settlement amount. The method is defined in the 2006 ISDA Definitions, Section 18.3. Cash Settlement Methods, paragraph (c).
	 */
	@RosettaSynonym(value="parYieldCurveAdjustedMethod", source="FpML_5_10")
	@RosettaSynonym(value="parYieldCurveAdjustedMethod", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="parYieldCurveAdjustedMethod", source="DTCC_11_0")
	@RosettaSynonym(value="parYieldCurveAdjustedMethod", source="DTCC_9_0")
	@RosettaSynonym(value="parYieldCurveAdjustedMethod", source="CME_ClearedConfirm_1_17")
	public final YieldCurveMethod getParYieldCurveAdjustedMethod() {
		return parYieldCurveAdjustedMethod;
	}
	
	/**
	 * An ISDA defined cash settlement method used for the determination of the applicable cash settlement amount. The method is defined in the 2006 ISDA Definitions, Section 18.3. Cash Settlement Methods, paragraph (e).
	 */
	@RosettaSynonym(value="parYieldCurveUnadjustedMethod", source="FpML_5_10")
	@RosettaSynonym(value="parYieldCurveUnadjustedMethod", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="parYieldCurveUnadjustedMethod", source="DTCC_11_0")
	@RosettaSynonym(value="parYieldCurveUnadjustedMethod", source="DTCC_9_0")
	@RosettaSynonym(value="parYieldCurveUnadjustedMethod", source="CME_ClearedConfirm_1_17")
	public final YieldCurveMethod getParYieldCurveUnadjustedMethod() {
		return parYieldCurveUnadjustedMethod;
	}
	
	/**
	 * An ISDA defined cash settlement method used for the determination of the applicable cash settlement amount. The method is defined in the 2006 ISDA Definitions, Section 18.3. Cash Settlement Methods, paragraph (d).
	 */
	@RosettaSynonym(value="zeroCouponYieldAdjustedMethod", source="FpML_5_10")
	@RosettaSynonym(value="zeroCouponYieldAdjustedMethod", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="zeroCouponYieldAdjustedMethod", source="DTCC_11_0")
	@RosettaSynonym(value="zeroCouponYieldAdjustedMethod", source="DTCC_9_0")
	@RosettaSynonym(value="zeroCouponYieldAdjustedMethod", source="CME_ClearedConfirm_1_17")
	public final YieldCurveMethod getZeroCouponYieldAdjustedMethod() {
		return zeroCouponYieldAdjustedMethod;
	}
	
	
	@Override
	public RosettaMetaData<? extends OptionCashSettlement> metaData() {
		return metaData;
	} 
	
	public OptionCashSettlementBuilder toBuilder() {
		OptionCashSettlementBuilder builder = new OptionCashSettlementBuilder();
		ofNullable(getCashPriceAlternateMethod()).ifPresent(builder::setCashPriceAlternateMethod);
		ofNullable(getCashPriceMethod()).ifPresent(builder::setCashPriceMethod);
		ofNullable(getCashSettlementPaymentDate()).ifPresent(builder::setCashSettlementPaymentDate);
		ofNullable(getCashSettlementValuationDate()).ifPresent(builder::setCashSettlementValuationDate);
		ofNullable(getCashSettlementValuationTime()).ifPresent(builder::setCashSettlementValuationTime);
		ofNullable(getCollateralizedCashPriceMethod()).ifPresent(builder::setCollateralizedCashPriceMethod);
		ofNullable(getCrossCurrencyMethod()).ifPresent(builder::setCrossCurrencyMethod);
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		ofNullable(getParYieldCurveAdjustedMethod()).ifPresent(builder::setParYieldCurveAdjustedMethod);
		ofNullable(getParYieldCurveUnadjustedMethod()).ifPresent(builder::setParYieldCurveUnadjustedMethod);
		ofNullable(getZeroCouponYieldAdjustedMethod()).ifPresent(builder::setZeroCouponYieldAdjustedMethod);
		return builder;
	}

	public static OptionCashSettlementBuilder builder() {
		return new OptionCashSettlementBuilder();
	}

	public static  class OptionCashSettlementBuilder extends RosettaModelObjectBuilder implements GlobalKeyBuilder<OptionCashSettlementBuilder> {
	
		protected CashPriceMethod.CashPriceMethodBuilder cashPriceAlternateMethod;
		protected CashPriceMethod.CashPriceMethodBuilder cashPriceMethod;
		protected CashSettlementPaymentDate.CashSettlementPaymentDateBuilder cashSettlementPaymentDate;
		protected RelativeDateOffset.RelativeDateOffsetBuilder cashSettlementValuationDate;
		protected BusinessCenterTime.BusinessCenterTimeBuilder cashSettlementValuationTime;
		protected YieldCurveMethod.YieldCurveMethodBuilder collateralizedCashPriceMethod;
		protected CrossCurrencyMethod.CrossCurrencyMethodBuilder crossCurrencyMethod;
		protected MetaFields.MetaFieldsBuilder meta;
		protected YieldCurveMethod.YieldCurveMethodBuilder parYieldCurveAdjustedMethod;
		protected YieldCurveMethod.YieldCurveMethodBuilder parYieldCurveUnadjustedMethod;
		protected YieldCurveMethod.YieldCurveMethodBuilder zeroCouponYieldAdjustedMethod;
	
		public OptionCashSettlementBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends OptionCashSettlement> metaData() {
			return metaData;
		} 
	
		
		public CashPriceMethod.CashPriceMethodBuilder getCashPriceAlternateMethod() {
			return cashPriceAlternateMethod;
		}
		
		public CashPriceMethod.CashPriceMethodBuilder getOrCreateCashPriceAlternateMethod() {
			if (cashPriceAlternateMethod!=null) {
				return cashPriceAlternateMethod;
			}
			else return cashPriceAlternateMethod = new CashPriceMethod.CashPriceMethodBuilder();
		}
		
		
		public CashPriceMethod.CashPriceMethodBuilder getCashPriceMethod() {
			return cashPriceMethod;
		}
		
		public CashPriceMethod.CashPriceMethodBuilder getOrCreateCashPriceMethod() {
			if (cashPriceMethod!=null) {
				return cashPriceMethod;
			}
			else return cashPriceMethod = new CashPriceMethod.CashPriceMethodBuilder();
		}
		
		
		public CashSettlementPaymentDate.CashSettlementPaymentDateBuilder getCashSettlementPaymentDate() {
			return cashSettlementPaymentDate;
		}
		
		public CashSettlementPaymentDate.CashSettlementPaymentDateBuilder getOrCreateCashSettlementPaymentDate() {
			if (cashSettlementPaymentDate!=null) {
				return cashSettlementPaymentDate;
			}
			else return cashSettlementPaymentDate = new CashSettlementPaymentDate.CashSettlementPaymentDateBuilder();
		}
		
		
		public RelativeDateOffset.RelativeDateOffsetBuilder getCashSettlementValuationDate() {
			return cashSettlementValuationDate;
		}
		
		public RelativeDateOffset.RelativeDateOffsetBuilder getOrCreateCashSettlementValuationDate() {
			if (cashSettlementValuationDate!=null) {
				return cashSettlementValuationDate;
			}
			else return cashSettlementValuationDate = new RelativeDateOffset.RelativeDateOffsetBuilder();
		}
		
		
		public BusinessCenterTime.BusinessCenterTimeBuilder getCashSettlementValuationTime() {
			return cashSettlementValuationTime;
		}
		
		public BusinessCenterTime.BusinessCenterTimeBuilder getOrCreateCashSettlementValuationTime() {
			if (cashSettlementValuationTime!=null) {
				return cashSettlementValuationTime;
			}
			else return cashSettlementValuationTime = new BusinessCenterTime.BusinessCenterTimeBuilder();
		}
		
		
		public YieldCurveMethod.YieldCurveMethodBuilder getCollateralizedCashPriceMethod() {
			return collateralizedCashPriceMethod;
		}
		
		public YieldCurveMethod.YieldCurveMethodBuilder getOrCreateCollateralizedCashPriceMethod() {
			if (collateralizedCashPriceMethod!=null) {
				return collateralizedCashPriceMethod;
			}
			else return collateralizedCashPriceMethod = new YieldCurveMethod.YieldCurveMethodBuilder();
		}
		
		
		public CrossCurrencyMethod.CrossCurrencyMethodBuilder getCrossCurrencyMethod() {
			return crossCurrencyMethod;
		}
		
		public CrossCurrencyMethod.CrossCurrencyMethodBuilder getOrCreateCrossCurrencyMethod() {
			if (crossCurrencyMethod!=null) {
				return crossCurrencyMethod;
			}
			else return crossCurrencyMethod = new CrossCurrencyMethod.CrossCurrencyMethodBuilder();
		}
		
		
		public MetaFields.MetaFieldsBuilder getMeta() {
			return meta;
		}
		
		public MetaFields.MetaFieldsBuilder getOrCreateMeta() {
			if (meta!=null) {
				return meta;
			}
			else return meta = new MetaFields.MetaFieldsBuilder();
		}
		
		
		public YieldCurveMethod.YieldCurveMethodBuilder getParYieldCurveAdjustedMethod() {
			return parYieldCurveAdjustedMethod;
		}
		
		public YieldCurveMethod.YieldCurveMethodBuilder getOrCreateParYieldCurveAdjustedMethod() {
			if (parYieldCurveAdjustedMethod!=null) {
				return parYieldCurveAdjustedMethod;
			}
			else return parYieldCurveAdjustedMethod = new YieldCurveMethod.YieldCurveMethodBuilder();
		}
		
		
		public YieldCurveMethod.YieldCurveMethodBuilder getParYieldCurveUnadjustedMethod() {
			return parYieldCurveUnadjustedMethod;
		}
		
		public YieldCurveMethod.YieldCurveMethodBuilder getOrCreateParYieldCurveUnadjustedMethod() {
			if (parYieldCurveUnadjustedMethod!=null) {
				return parYieldCurveUnadjustedMethod;
			}
			else return parYieldCurveUnadjustedMethod = new YieldCurveMethod.YieldCurveMethodBuilder();
		}
		
		
		public YieldCurveMethod.YieldCurveMethodBuilder getZeroCouponYieldAdjustedMethod() {
			return zeroCouponYieldAdjustedMethod;
		}
		
		public YieldCurveMethod.YieldCurveMethodBuilder getOrCreateZeroCouponYieldAdjustedMethod() {
			if (zeroCouponYieldAdjustedMethod!=null) {
				return zeroCouponYieldAdjustedMethod;
			}
			else return zeroCouponYieldAdjustedMethod = new YieldCurveMethod.YieldCurveMethodBuilder();
		}
		
	
		public OptionCashSettlementBuilder setCashPriceAlternateMethod(CashPriceMethod cashPriceAlternateMethod) {
			this.cashPriceAlternateMethod = cashPriceAlternateMethod.toBuilder();
			return this;
		}
		public OptionCashSettlementBuilder setCashPriceAlternateMethodBuilder(CashPriceMethod.CashPriceMethodBuilder cashPriceAlternateMethod) {
			this.cashPriceAlternateMethod = cashPriceAlternateMethod;
			return this;
		}
		public OptionCashSettlementBuilder setCashPriceMethod(CashPriceMethod cashPriceMethod) {
			this.cashPriceMethod = cashPriceMethod.toBuilder();
			return this;
		}
		public OptionCashSettlementBuilder setCashPriceMethodBuilder(CashPriceMethod.CashPriceMethodBuilder cashPriceMethod) {
			this.cashPriceMethod = cashPriceMethod;
			return this;
		}
		public OptionCashSettlementBuilder setCashSettlementPaymentDate(CashSettlementPaymentDate cashSettlementPaymentDate) {
			this.cashSettlementPaymentDate = cashSettlementPaymentDate.toBuilder();
			return this;
		}
		public OptionCashSettlementBuilder setCashSettlementPaymentDateBuilder(CashSettlementPaymentDate.CashSettlementPaymentDateBuilder cashSettlementPaymentDate) {
			this.cashSettlementPaymentDate = cashSettlementPaymentDate;
			return this;
		}
		public OptionCashSettlementBuilder setCashSettlementValuationDate(RelativeDateOffset cashSettlementValuationDate) {
			this.cashSettlementValuationDate = cashSettlementValuationDate.toBuilder();
			return this;
		}
		public OptionCashSettlementBuilder setCashSettlementValuationDateBuilder(RelativeDateOffset.RelativeDateOffsetBuilder cashSettlementValuationDate) {
			this.cashSettlementValuationDate = cashSettlementValuationDate;
			return this;
		}
		public OptionCashSettlementBuilder setCashSettlementValuationTime(BusinessCenterTime cashSettlementValuationTime) {
			this.cashSettlementValuationTime = cashSettlementValuationTime.toBuilder();
			return this;
		}
		public OptionCashSettlementBuilder setCashSettlementValuationTimeBuilder(BusinessCenterTime.BusinessCenterTimeBuilder cashSettlementValuationTime) {
			this.cashSettlementValuationTime = cashSettlementValuationTime;
			return this;
		}
		public OptionCashSettlementBuilder setCollateralizedCashPriceMethod(YieldCurveMethod collateralizedCashPriceMethod) {
			this.collateralizedCashPriceMethod = collateralizedCashPriceMethod.toBuilder();
			return this;
		}
		public OptionCashSettlementBuilder setCollateralizedCashPriceMethodBuilder(YieldCurveMethod.YieldCurveMethodBuilder collateralizedCashPriceMethod) {
			this.collateralizedCashPriceMethod = collateralizedCashPriceMethod;
			return this;
		}
		public OptionCashSettlementBuilder setCrossCurrencyMethod(CrossCurrencyMethod crossCurrencyMethod) {
			this.crossCurrencyMethod = crossCurrencyMethod.toBuilder();
			return this;
		}
		public OptionCashSettlementBuilder setCrossCurrencyMethodBuilder(CrossCurrencyMethod.CrossCurrencyMethodBuilder crossCurrencyMethod) {
			this.crossCurrencyMethod = crossCurrencyMethod;
			return this;
		}
		public OptionCashSettlementBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		public OptionCashSettlementBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		public OptionCashSettlementBuilder setParYieldCurveAdjustedMethod(YieldCurveMethod parYieldCurveAdjustedMethod) {
			this.parYieldCurveAdjustedMethod = parYieldCurveAdjustedMethod.toBuilder();
			return this;
		}
		public OptionCashSettlementBuilder setParYieldCurveAdjustedMethodBuilder(YieldCurveMethod.YieldCurveMethodBuilder parYieldCurveAdjustedMethod) {
			this.parYieldCurveAdjustedMethod = parYieldCurveAdjustedMethod;
			return this;
		}
		public OptionCashSettlementBuilder setParYieldCurveUnadjustedMethod(YieldCurveMethod parYieldCurveUnadjustedMethod) {
			this.parYieldCurveUnadjustedMethod = parYieldCurveUnadjustedMethod.toBuilder();
			return this;
		}
		public OptionCashSettlementBuilder setParYieldCurveUnadjustedMethodBuilder(YieldCurveMethod.YieldCurveMethodBuilder parYieldCurveUnadjustedMethod) {
			this.parYieldCurveUnadjustedMethod = parYieldCurveUnadjustedMethod;
			return this;
		}
		public OptionCashSettlementBuilder setZeroCouponYieldAdjustedMethod(YieldCurveMethod zeroCouponYieldAdjustedMethod) {
			this.zeroCouponYieldAdjustedMethod = zeroCouponYieldAdjustedMethod.toBuilder();
			return this;
		}
		public OptionCashSettlementBuilder setZeroCouponYieldAdjustedMethodBuilder(YieldCurveMethod.YieldCurveMethodBuilder zeroCouponYieldAdjustedMethod) {
			this.zeroCouponYieldAdjustedMethod = zeroCouponYieldAdjustedMethod;
			return this;
		}
		
		public OptionCashSettlement build() {
			return new OptionCashSettlement(this);
		}
	
		@Override
		public OptionCashSettlementBuilder prune() {
			if (cashPriceAlternateMethod!=null && !cashPriceAlternateMethod.prune().hasData()) cashPriceAlternateMethod = null;
			if (cashPriceMethod!=null && !cashPriceMethod.prune().hasData()) cashPriceMethod = null;
			if (cashSettlementPaymentDate!=null && !cashSettlementPaymentDate.prune().hasData()) cashSettlementPaymentDate = null;
			if (cashSettlementValuationDate!=null && !cashSettlementValuationDate.prune().hasData()) cashSettlementValuationDate = null;
			if (cashSettlementValuationTime!=null && !cashSettlementValuationTime.prune().hasData()) cashSettlementValuationTime = null;
			if (collateralizedCashPriceMethod!=null && !collateralizedCashPriceMethod.prune().hasData()) collateralizedCashPriceMethod = null;
			if (crossCurrencyMethod!=null && !crossCurrencyMethod.prune().hasData()) crossCurrencyMethod = null;
			if (meta!=null && !meta.prune().hasData()) meta = null;
			if (parYieldCurveAdjustedMethod!=null && !parYieldCurveAdjustedMethod.prune().hasData()) parYieldCurveAdjustedMethod = null;
			if (parYieldCurveUnadjustedMethod!=null && !parYieldCurveUnadjustedMethod.prune().hasData()) parYieldCurveUnadjustedMethod = null;
			if (zeroCouponYieldAdjustedMethod!=null && !zeroCouponYieldAdjustedMethod.prune().hasData()) zeroCouponYieldAdjustedMethod = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getCashPriceAlternateMethod()!=null && getCashPriceAlternateMethod().hasData()) return true;
			if (getCashPriceMethod()!=null && getCashPriceMethod().hasData()) return true;
			if (getCashSettlementPaymentDate()!=null && getCashSettlementPaymentDate().hasData()) return true;
			if (getCashSettlementValuationDate()!=null && getCashSettlementValuationDate().hasData()) return true;
			if (getCashSettlementValuationTime()!=null && getCashSettlementValuationTime().hasData()) return true;
			if (getCollateralizedCashPriceMethod()!=null && getCollateralizedCashPriceMethod().hasData()) return true;
			if (getCrossCurrencyMethod()!=null && getCrossCurrencyMethod().hasData()) return true;
			if (getMeta()!=null && getMeta().hasData()) return true;
			if (getParYieldCurveAdjustedMethod()!=null && getParYieldCurveAdjustedMethod().hasData()) return true;
			if (getParYieldCurveUnadjustedMethod()!=null && getParYieldCurveUnadjustedMethod().hasData()) return true;
			if (getZeroCouponYieldAdjustedMethod()!=null && getZeroCouponYieldAdjustedMethod().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("cashPriceAlternateMethod"), processor, CashPriceMethod.class, cashPriceAlternateMethod);
			processRosetta(path.newSubPath("cashPriceMethod"), processor, CashPriceMethod.class, cashPriceMethod);
			processRosetta(path.newSubPath("cashSettlementPaymentDate"), processor, CashSettlementPaymentDate.class, cashSettlementPaymentDate);
			processRosetta(path.newSubPath("cashSettlementValuationDate"), processor, RelativeDateOffset.class, cashSettlementValuationDate);
			processRosetta(path.newSubPath("cashSettlementValuationTime"), processor, BusinessCenterTime.class, cashSettlementValuationTime);
			processRosetta(path.newSubPath("collateralizedCashPriceMethod"), processor, YieldCurveMethod.class, collateralizedCashPriceMethod);
			processRosetta(path.newSubPath("crossCurrencyMethod"), processor, CrossCurrencyMethod.class, crossCurrencyMethod);
			processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
			processRosetta(path.newSubPath("parYieldCurveAdjustedMethod"), processor, YieldCurveMethod.class, parYieldCurveAdjustedMethod);
			processRosetta(path.newSubPath("parYieldCurveUnadjustedMethod"), processor, YieldCurveMethod.class, parYieldCurveUnadjustedMethod);
			processRosetta(path.newSubPath("zeroCouponYieldAdjustedMethod"), processor, YieldCurveMethod.class, zeroCouponYieldAdjustedMethod);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			OptionCashSettlementBuilder _that = (OptionCashSettlementBuilder) o;
		
			if (cashPriceAlternateMethod != null ? !cashPriceAlternateMethod.equals(_that.cashPriceAlternateMethod) : _that.cashPriceAlternateMethod != null) return false;
			if (cashPriceMethod != null ? !cashPriceMethod.equals(_that.cashPriceMethod) : _that.cashPriceMethod != null) return false;
			if (cashSettlementPaymentDate != null ? !cashSettlementPaymentDate.equals(_that.cashSettlementPaymentDate) : _that.cashSettlementPaymentDate != null) return false;
			if (cashSettlementValuationDate != null ? !cashSettlementValuationDate.equals(_that.cashSettlementValuationDate) : _that.cashSettlementValuationDate != null) return false;
			if (cashSettlementValuationTime != null ? !cashSettlementValuationTime.equals(_that.cashSettlementValuationTime) : _that.cashSettlementValuationTime != null) return false;
			if (collateralizedCashPriceMethod != null ? !collateralizedCashPriceMethod.equals(_that.collateralizedCashPriceMethod) : _that.collateralizedCashPriceMethod != null) return false;
			if (crossCurrencyMethod != null ? !crossCurrencyMethod.equals(_that.crossCurrencyMethod) : _that.crossCurrencyMethod != null) return false;
			if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
			if (parYieldCurveAdjustedMethod != null ? !parYieldCurveAdjustedMethod.equals(_that.parYieldCurveAdjustedMethod) : _that.parYieldCurveAdjustedMethod != null) return false;
			if (parYieldCurveUnadjustedMethod != null ? !parYieldCurveUnadjustedMethod.equals(_that.parYieldCurveUnadjustedMethod) : _that.parYieldCurveUnadjustedMethod != null) return false;
			if (zeroCouponYieldAdjustedMethod != null ? !zeroCouponYieldAdjustedMethod.equals(_that.zeroCouponYieldAdjustedMethod) : _that.zeroCouponYieldAdjustedMethod != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (cashPriceAlternateMethod != null ? cashPriceAlternateMethod.hashCode() : 0);
			_result = 31 * _result + (cashPriceMethod != null ? cashPriceMethod.hashCode() : 0);
			_result = 31 * _result + (cashSettlementPaymentDate != null ? cashSettlementPaymentDate.hashCode() : 0);
			_result = 31 * _result + (cashSettlementValuationDate != null ? cashSettlementValuationDate.hashCode() : 0);
			_result = 31 * _result + (cashSettlementValuationTime != null ? cashSettlementValuationTime.hashCode() : 0);
			_result = 31 * _result + (collateralizedCashPriceMethod != null ? collateralizedCashPriceMethod.hashCode() : 0);
			_result = 31 * _result + (crossCurrencyMethod != null ? crossCurrencyMethod.hashCode() : 0);
			_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
			_result = 31 * _result + (parYieldCurveAdjustedMethod != null ? parYieldCurveAdjustedMethod.hashCode() : 0);
			_result = 31 * _result + (parYieldCurveUnadjustedMethod != null ? parYieldCurveUnadjustedMethod.hashCode() : 0);
			_result = 31 * _result + (zeroCouponYieldAdjustedMethod != null ? zeroCouponYieldAdjustedMethod.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "OptionCashSettlementBuilder {" +
				"cashPriceAlternateMethod=" + this.cashPriceAlternateMethod + ", " +
				"cashPriceMethod=" + this.cashPriceMethod + ", " +
				"cashSettlementPaymentDate=" + this.cashSettlementPaymentDate + ", " +
				"cashSettlementValuationDate=" + this.cashSettlementValuationDate + ", " +
				"cashSettlementValuationTime=" + this.cashSettlementValuationTime + ", " +
				"collateralizedCashPriceMethod=" + this.collateralizedCashPriceMethod + ", " +
				"crossCurrencyMethod=" + this.crossCurrencyMethod + ", " +
				"meta=" + this.meta + ", " +
				"parYieldCurveAdjustedMethod=" + this.parYieldCurveAdjustedMethod + ", " +
				"parYieldCurveUnadjustedMethod=" + this.parYieldCurveUnadjustedMethod + ", " +
				"zeroCouponYieldAdjustedMethod=" + this.zeroCouponYieldAdjustedMethod +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("cashPriceAlternateMethod"), processor, CashPriceMethod.class, cashPriceAlternateMethod);
		processRosetta(path.newSubPath("cashPriceMethod"), processor, CashPriceMethod.class, cashPriceMethod);
		processRosetta(path.newSubPath("cashSettlementPaymentDate"), processor, CashSettlementPaymentDate.class, cashSettlementPaymentDate);
		processRosetta(path.newSubPath("cashSettlementValuationDate"), processor, RelativeDateOffset.class, cashSettlementValuationDate);
		processRosetta(path.newSubPath("cashSettlementValuationTime"), processor, BusinessCenterTime.class, cashSettlementValuationTime);
		processRosetta(path.newSubPath("collateralizedCashPriceMethod"), processor, YieldCurveMethod.class, collateralizedCashPriceMethod);
		processRosetta(path.newSubPath("crossCurrencyMethod"), processor, CrossCurrencyMethod.class, crossCurrencyMethod);
		processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
		processRosetta(path.newSubPath("parYieldCurveAdjustedMethod"), processor, YieldCurveMethod.class, parYieldCurveAdjustedMethod);
		processRosetta(path.newSubPath("parYieldCurveUnadjustedMethod"), processor, YieldCurveMethod.class, parYieldCurveUnadjustedMethod);
		processRosetta(path.newSubPath("zeroCouponYieldAdjustedMethod"), processor, YieldCurveMethod.class, zeroCouponYieldAdjustedMethod);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		OptionCashSettlement _that = (OptionCashSettlement) o;
	
		if (cashPriceAlternateMethod != null ? !cashPriceAlternateMethod.equals(_that.cashPriceAlternateMethod) : _that.cashPriceAlternateMethod != null) return false;
		if (cashPriceMethod != null ? !cashPriceMethod.equals(_that.cashPriceMethod) : _that.cashPriceMethod != null) return false;
		if (cashSettlementPaymentDate != null ? !cashSettlementPaymentDate.equals(_that.cashSettlementPaymentDate) : _that.cashSettlementPaymentDate != null) return false;
		if (cashSettlementValuationDate != null ? !cashSettlementValuationDate.equals(_that.cashSettlementValuationDate) : _that.cashSettlementValuationDate != null) return false;
		if (cashSettlementValuationTime != null ? !cashSettlementValuationTime.equals(_that.cashSettlementValuationTime) : _that.cashSettlementValuationTime != null) return false;
		if (collateralizedCashPriceMethod != null ? !collateralizedCashPriceMethod.equals(_that.collateralizedCashPriceMethod) : _that.collateralizedCashPriceMethod != null) return false;
		if (crossCurrencyMethod != null ? !crossCurrencyMethod.equals(_that.crossCurrencyMethod) : _that.crossCurrencyMethod != null) return false;
		if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
		if (parYieldCurveAdjustedMethod != null ? !parYieldCurveAdjustedMethod.equals(_that.parYieldCurveAdjustedMethod) : _that.parYieldCurveAdjustedMethod != null) return false;
		if (parYieldCurveUnadjustedMethod != null ? !parYieldCurveUnadjustedMethod.equals(_that.parYieldCurveUnadjustedMethod) : _that.parYieldCurveUnadjustedMethod != null) return false;
		if (zeroCouponYieldAdjustedMethod != null ? !zeroCouponYieldAdjustedMethod.equals(_that.zeroCouponYieldAdjustedMethod) : _that.zeroCouponYieldAdjustedMethod != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (cashPriceAlternateMethod != null ? cashPriceAlternateMethod.hashCode() : 0);
		_result = 31 * _result + (cashPriceMethod != null ? cashPriceMethod.hashCode() : 0);
		_result = 31 * _result + (cashSettlementPaymentDate != null ? cashSettlementPaymentDate.hashCode() : 0);
		_result = 31 * _result + (cashSettlementValuationDate != null ? cashSettlementValuationDate.hashCode() : 0);
		_result = 31 * _result + (cashSettlementValuationTime != null ? cashSettlementValuationTime.hashCode() : 0);
		_result = 31 * _result + (collateralizedCashPriceMethod != null ? collateralizedCashPriceMethod.hashCode() : 0);
		_result = 31 * _result + (crossCurrencyMethod != null ? crossCurrencyMethod.hashCode() : 0);
		_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
		_result = 31 * _result + (parYieldCurveAdjustedMethod != null ? parYieldCurveAdjustedMethod.hashCode() : 0);
		_result = 31 * _result + (parYieldCurveUnadjustedMethod != null ? parYieldCurveUnadjustedMethod.hashCode() : 0);
		_result = 31 * _result + (zeroCouponYieldAdjustedMethod != null ? zeroCouponYieldAdjustedMethod.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "OptionCashSettlement {" +
			"cashPriceAlternateMethod=" + this.cashPriceAlternateMethod + ", " +
			"cashPriceMethod=" + this.cashPriceMethod + ", " +
			"cashSettlementPaymentDate=" + this.cashSettlementPaymentDate + ", " +
			"cashSettlementValuationDate=" + this.cashSettlementValuationDate + ", " +
			"cashSettlementValuationTime=" + this.cashSettlementValuationTime + ", " +
			"collateralizedCashPriceMethod=" + this.collateralizedCashPriceMethod + ", " +
			"crossCurrencyMethod=" + this.crossCurrencyMethod + ", " +
			"meta=" + this.meta + ", " +
			"parYieldCurveAdjustedMethod=" + this.parYieldCurveAdjustedMethod + ", " +
			"parYieldCurveUnadjustedMethod=" + this.parYieldCurveUnadjustedMethod + ", " +
			"zeroCouponYieldAdjustedMethod=" + this.zeroCouponYieldAdjustedMethod +
		'}';
	}
}
