package org.isda.cdm;

import org.isda.cdm.ExerciseProcedure;
import org.isda.cdm.OptionSettlement;
import org.isda.cdm.OptionStrike;
import org.isda.cdm.OptionStyle;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.OptionExerciseMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class to represent the applicable terms to qualify an option exercise: the option style (e.g. American style option), the exercise procedure (e.g. manual exercise) and the settlement terms (e.g. physical vs. cash).
 * 
 * @version 2.5.4
 */
@RosettaClass
public class OptionExercise extends RosettaModelObject {
	private final ExerciseProcedure exerciseProcedure;
	private final OptionStyle optionStyle;
	private final OptionSettlement settlement;
	private final OptionStrike strike;
	private static OptionExerciseMeta metaData = new OptionExerciseMeta();
	
	OptionExercise(OptionExerciseBuilder builder) {
		this.exerciseProcedure = ofNullable(builder.getExerciseProcedure()).map(f->f.build()).orElse(null);
		this.optionStyle = ofNullable(builder.getOptionStyle()).map(f->f.build()).orElse(null);
		this.settlement = ofNullable(builder.getSettlement()).map(f->f.build()).orElse(null);
		this.strike = ofNullable(builder.getStrike()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The set of parameters defining the procedure associated with the exercise, e.g. manual exercise.
	 */
	@RosettaSynonym(value="exerciseProcedure", source="FpML_5_10")
	@RosettaSynonym(value="exerciseProcedure", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="exerciseProcedure", source="DTCC_11_0")
	@RosettaSynonym(value="exerciseProcedure", source="DTCC_9_0")
	@RosettaSynonym(value="exerciseProcedure", source="CME_ClearedConfirm_1_17")
	public final ExerciseProcedure getExerciseProcedure() {
		return exerciseProcedure;
	}
	
	/**
	 * The option exercise can be of American style, Bermuda style or European style. The FpML implementation makes use of a substitution group.
	 */
	public final OptionStyle getOptionStyle() {
		return optionStyle;
	}
	
	/**
	 * The option settlement terms, such as cash vs. physical, the settlement date, the settlement currency and/or amount. FpML represents this information as part of the OptionSettlement.model.
	 */
	public final OptionSettlement getSettlement() {
		return settlement;
	}
	
	/**
	 * Specifies the strike of the option on credit default swap.
	 */
	@RosettaSynonym(value="strike", source="FpML_5_10")
	@RosettaSynonym(value="strike", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="strike", source="DTCC_11_0")
	@RosettaSynonym(value="strike", source="DTCC_9_0")
	@RosettaSynonym(value="strike", source="CME_ClearedConfirm_1_17")
	public final OptionStrike getStrike() {
		return strike;
	}
	
	
	@Override
	public RosettaMetaData<? extends OptionExercise> metaData() {
		return metaData;
	} 
	
	public OptionExerciseBuilder toBuilder() {
		OptionExerciseBuilder builder = new OptionExerciseBuilder();
		ofNullable(getExerciseProcedure()).ifPresent(builder::setExerciseProcedure);
		ofNullable(getOptionStyle()).ifPresent(builder::setOptionStyle);
		ofNullable(getSettlement()).ifPresent(builder::setSettlement);
		ofNullable(getStrike()).ifPresent(builder::setStrike);
		return builder;
	}

	public static OptionExerciseBuilder builder() {
		return new OptionExerciseBuilder();
	}

	public static  class OptionExerciseBuilder extends RosettaModelObjectBuilder {
	
		protected ExerciseProcedure.ExerciseProcedureBuilder exerciseProcedure;
		protected OptionStyle.OptionStyleBuilder optionStyle;
		protected OptionSettlement.OptionSettlementBuilder settlement;
		protected OptionStrike.OptionStrikeBuilder strike;
	
		public OptionExerciseBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends OptionExercise> metaData() {
			return metaData;
		} 
	
		
		public ExerciseProcedure.ExerciseProcedureBuilder getExerciseProcedure() {
			return exerciseProcedure;
		}
		
		public ExerciseProcedure.ExerciseProcedureBuilder getOrCreateExerciseProcedure() {
			if (exerciseProcedure!=null) {
				return exerciseProcedure;
			}
			else return exerciseProcedure = new ExerciseProcedure.ExerciseProcedureBuilder();
		}
		
		
		public OptionStyle.OptionStyleBuilder getOptionStyle() {
			return optionStyle;
		}
		
		public OptionStyle.OptionStyleBuilder getOrCreateOptionStyle() {
			if (optionStyle!=null) {
				return optionStyle;
			}
			else return optionStyle = new OptionStyle.OptionStyleBuilder();
		}
		
		
		public OptionSettlement.OptionSettlementBuilder getSettlement() {
			return settlement;
		}
		
		public OptionSettlement.OptionSettlementBuilder getOrCreateSettlement() {
			if (settlement!=null) {
				return settlement;
			}
			else return settlement = new OptionSettlement.OptionSettlementBuilder();
		}
		
		
		public OptionStrike.OptionStrikeBuilder getStrike() {
			return strike;
		}
		
		public OptionStrike.OptionStrikeBuilder getOrCreateStrike() {
			if (strike!=null) {
				return strike;
			}
			else return strike = new OptionStrike.OptionStrikeBuilder();
		}
		
	
		public OptionExerciseBuilder setExerciseProcedure(ExerciseProcedure exerciseProcedure) {
			this.exerciseProcedure = exerciseProcedure.toBuilder();
			return this;
		}
		public OptionExerciseBuilder setExerciseProcedureBuilder(ExerciseProcedure.ExerciseProcedureBuilder exerciseProcedure) {
			this.exerciseProcedure = exerciseProcedure;
			return this;
		}
		public OptionExerciseBuilder setOptionStyle(OptionStyle optionStyle) {
			this.optionStyle = optionStyle.toBuilder();
			return this;
		}
		public OptionExerciseBuilder setOptionStyleBuilder(OptionStyle.OptionStyleBuilder optionStyle) {
			this.optionStyle = optionStyle;
			return this;
		}
		public OptionExerciseBuilder setSettlement(OptionSettlement settlement) {
			this.settlement = settlement.toBuilder();
			return this;
		}
		public OptionExerciseBuilder setSettlementBuilder(OptionSettlement.OptionSettlementBuilder settlement) {
			this.settlement = settlement;
			return this;
		}
		public OptionExerciseBuilder setStrike(OptionStrike strike) {
			this.strike = strike.toBuilder();
			return this;
		}
		public OptionExerciseBuilder setStrikeBuilder(OptionStrike.OptionStrikeBuilder strike) {
			this.strike = strike;
			return this;
		}
		
		public OptionExercise build() {
			return new OptionExercise(this);
		}
	
		@Override
		public OptionExerciseBuilder prune() {
			if (exerciseProcedure!=null && !exerciseProcedure.prune().hasData()) exerciseProcedure = null;
			if (optionStyle!=null && !optionStyle.prune().hasData()) optionStyle = null;
			if (settlement!=null && !settlement.prune().hasData()) settlement = null;
			if (strike!=null && !strike.prune().hasData()) strike = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getExerciseProcedure()!=null && getExerciseProcedure().hasData()) return true;
			if (getOptionStyle()!=null && getOptionStyle().hasData()) return true;
			if (getSettlement()!=null && getSettlement().hasData()) return true;
			if (getStrike()!=null && getStrike().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("exerciseProcedure"), processor, ExerciseProcedure.class, exerciseProcedure);
			processRosetta(path.newSubPath("optionStyle"), processor, OptionStyle.class, optionStyle);
			processRosetta(path.newSubPath("settlement"), processor, OptionSettlement.class, settlement);
			processRosetta(path.newSubPath("strike"), processor, OptionStrike.class, strike);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			OptionExerciseBuilder _that = (OptionExerciseBuilder) o;
		
			if (exerciseProcedure != null ? !exerciseProcedure.equals(_that.exerciseProcedure) : _that.exerciseProcedure != null) return false;
			if (optionStyle != null ? !optionStyle.equals(_that.optionStyle) : _that.optionStyle != null) return false;
			if (settlement != null ? !settlement.equals(_that.settlement) : _that.settlement != null) return false;
			if (strike != null ? !strike.equals(_that.strike) : _that.strike != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (exerciseProcedure != null ? exerciseProcedure.hashCode() : 0);
			_result = 31 * _result + (optionStyle != null ? optionStyle.hashCode() : 0);
			_result = 31 * _result + (settlement != null ? settlement.hashCode() : 0);
			_result = 31 * _result + (strike != null ? strike.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "OptionExerciseBuilder {" +
				"exerciseProcedure=" + this.exerciseProcedure + ", " +
				"optionStyle=" + this.optionStyle + ", " +
				"settlement=" + this.settlement + ", " +
				"strike=" + this.strike +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("exerciseProcedure"), processor, ExerciseProcedure.class, exerciseProcedure);
		processRosetta(path.newSubPath("optionStyle"), processor, OptionStyle.class, optionStyle);
		processRosetta(path.newSubPath("settlement"), processor, OptionSettlement.class, settlement);
		processRosetta(path.newSubPath("strike"), processor, OptionStrike.class, strike);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		OptionExercise _that = (OptionExercise) o;
	
		if (exerciseProcedure != null ? !exerciseProcedure.equals(_that.exerciseProcedure) : _that.exerciseProcedure != null) return false;
		if (optionStyle != null ? !optionStyle.equals(_that.optionStyle) : _that.optionStyle != null) return false;
		if (settlement != null ? !settlement.equals(_that.settlement) : _that.settlement != null) return false;
		if (strike != null ? !strike.equals(_that.strike) : _that.strike != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (exerciseProcedure != null ? exerciseProcedure.hashCode() : 0);
		_result = 31 * _result + (optionStyle != null ? optionStyle.hashCode() : 0);
		_result = 31 * _result + (settlement != null ? settlement.hashCode() : 0);
		_result = 31 * _result + (strike != null ? strike.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "OptionExercise {" +
			"exerciseProcedure=" + this.exerciseProcedure + ", " +
			"optionStyle=" + this.optionStyle + ", " +
			"settlement=" + this.settlement + ", " +
			"strike=" + this.strike +
		'}';
	}
}
