package org.isda.cdm;

import org.isda.cdm.Asian;
import org.isda.cdm.Barrier;
import org.isda.cdm.FxFeature;
import org.isda.cdm.Knock;
import org.isda.cdm.PassThrough;
import org.isda.cdm.StrategyFeature;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.OptionFeatureMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class for defining option features.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="OptionFeature", source="FpML_5_10")
public class OptionFeature extends RosettaModelObject {
	private final Asian asian;
	private final Barrier barrier;
	private final FxFeature fxFeature;
	private final Knock knock;
	private final PassThrough passThrough;
	private final StrategyFeature strategyFeature;
	private static OptionFeatureMeta metaData = new OptionFeatureMeta();
	
	OptionFeature(OptionFeatureBuilder builder) {
		this.asian = ofNullable(builder.getAsian()).map(f->f.build()).orElse(null);
		this.barrier = ofNullable(builder.getBarrier()).map(f->f.build()).orElse(null);
		this.fxFeature = ofNullable(builder.getFxFeature()).map(f->f.build()).orElse(null);
		this.knock = ofNullable(builder.getKnock()).map(f->f.build()).orElse(null);
		this.passThrough = ofNullable(builder.getPassThrough()).map(f->f.build()).orElse(null);
		this.strategyFeature = ofNullable(builder.getStrategyFeature()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * An option where and average price is taken on valuation.
	 */
	@RosettaSynonym(value="asian", source="FpML_5_10")
	@RosettaSynonym(value="asian", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="asian", source="DTCC_11_0")
	@RosettaSynonym(value="asian", source="DTCC_9_0")
	@RosettaSynonym(value="asian", source="CME_ClearedConfirm_1_17")
	public final Asian getAsian() {
		return asian;
	}
	
	/**
	 * An option with a barrier feature.
	 */
	@RosettaSynonym(value="barrier", source="FpML_5_10")
	@RosettaSynonym(value="barrier", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="barrier", source="DTCC_11_0")
	@RosettaSynonym(value="barrier", source="DTCC_9_0")
	@RosettaSynonym(value="barrier", source="CME_ClearedConfirm_1_17")
	public final Barrier getBarrier() {
		return barrier;
	}
	
	/**
	 * A quanto or composite FX feature.
	 */
	@RosettaSynonym(value="fxFeature", source="FpML_5_10")
	@RosettaSynonym(value="fxFeature", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="fxFeature", source="DTCC_11_0")
	@RosettaSynonym(value="fxFeature", source="DTCC_9_0")
	@RosettaSynonym(value="fxFeature", source="CME_ClearedConfirm_1_17")
	public final FxFeature getFxFeature() {
		return fxFeature;
	}
	
	/**
	 * A knock feature.
	 */
	@RosettaSynonym(value="knock", source="FpML_5_10")
	@RosettaSynonym(value="knock", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="knock", source="DTCC_11_0")
	@RosettaSynonym(value="knock", source="DTCC_9_0")
	@RosettaSynonym(value="knock", source="CME_ClearedConfirm_1_17")
	public final Knock getKnock() {
		return knock;
	}
	
	/**
	 * Pass-through payments from the underlier, such as dividends.
	 */
	@RosettaSynonym(value="passThrough", source="FpML_5_10")
	@RosettaSynonym(value="passThrough", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="passThrough", source="DTCC_11_0")
	@RosettaSynonym(value="passThrough", source="DTCC_9_0")
	@RosettaSynonym(value="passThrough", source="CME_ClearedConfirm_1_17")
	public final PassThrough getPassThrough() {
		return passThrough;
	}
	
	/**
	 * A simple strategy feature.
	 */
	public final StrategyFeature getStrategyFeature() {
		return strategyFeature;
	}
	
	
	@Override
	public RosettaMetaData<? extends OptionFeature> metaData() {
		return metaData;
	} 
	
	public OptionFeatureBuilder toBuilder() {
		OptionFeatureBuilder builder = new OptionFeatureBuilder();
		ofNullable(getAsian()).ifPresent(builder::setAsian);
		ofNullable(getBarrier()).ifPresent(builder::setBarrier);
		ofNullable(getFxFeature()).ifPresent(builder::setFxFeature);
		ofNullable(getKnock()).ifPresent(builder::setKnock);
		ofNullable(getPassThrough()).ifPresent(builder::setPassThrough);
		ofNullable(getStrategyFeature()).ifPresent(builder::setStrategyFeature);
		return builder;
	}

	public static OptionFeatureBuilder builder() {
		return new OptionFeatureBuilder();
	}

	public static  class OptionFeatureBuilder extends RosettaModelObjectBuilder {
	
		protected Asian.AsianBuilder asian;
		protected Barrier.BarrierBuilder barrier;
		protected FxFeature.FxFeatureBuilder fxFeature;
		protected Knock.KnockBuilder knock;
		protected PassThrough.PassThroughBuilder passThrough;
		protected StrategyFeature.StrategyFeatureBuilder strategyFeature;
	
		public OptionFeatureBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends OptionFeature> metaData() {
			return metaData;
		} 
	
		
		public Asian.AsianBuilder getAsian() {
			return asian;
		}
		
		public Asian.AsianBuilder getOrCreateAsian() {
			if (asian!=null) {
				return asian;
			}
			else return asian = new Asian.AsianBuilder();
		}
		
		
		public Barrier.BarrierBuilder getBarrier() {
			return barrier;
		}
		
		public Barrier.BarrierBuilder getOrCreateBarrier() {
			if (barrier!=null) {
				return barrier;
			}
			else return barrier = new Barrier.BarrierBuilder();
		}
		
		
		public FxFeature.FxFeatureBuilder getFxFeature() {
			return fxFeature;
		}
		
		public FxFeature.FxFeatureBuilder getOrCreateFxFeature() {
			if (fxFeature!=null) {
				return fxFeature;
			}
			else return fxFeature = new FxFeature.FxFeatureBuilder();
		}
		
		
		public Knock.KnockBuilder getKnock() {
			return knock;
		}
		
		public Knock.KnockBuilder getOrCreateKnock() {
			if (knock!=null) {
				return knock;
			}
			else return knock = new Knock.KnockBuilder();
		}
		
		
		public PassThrough.PassThroughBuilder getPassThrough() {
			return passThrough;
		}
		
		public PassThrough.PassThroughBuilder getOrCreatePassThrough() {
			if (passThrough!=null) {
				return passThrough;
			}
			else return passThrough = new PassThrough.PassThroughBuilder();
		}
		
		
		public StrategyFeature.StrategyFeatureBuilder getStrategyFeature() {
			return strategyFeature;
		}
		
		public StrategyFeature.StrategyFeatureBuilder getOrCreateStrategyFeature() {
			if (strategyFeature!=null) {
				return strategyFeature;
			}
			else return strategyFeature = new StrategyFeature.StrategyFeatureBuilder();
		}
		
	
		public OptionFeatureBuilder setAsian(Asian asian) {
			this.asian = asian.toBuilder();
			return this;
		}
		public OptionFeatureBuilder setAsianBuilder(Asian.AsianBuilder asian) {
			this.asian = asian;
			return this;
		}
		public OptionFeatureBuilder setBarrier(Barrier barrier) {
			this.barrier = barrier.toBuilder();
			return this;
		}
		public OptionFeatureBuilder setBarrierBuilder(Barrier.BarrierBuilder barrier) {
			this.barrier = barrier;
			return this;
		}
		public OptionFeatureBuilder setFxFeature(FxFeature fxFeature) {
			this.fxFeature = fxFeature.toBuilder();
			return this;
		}
		public OptionFeatureBuilder setFxFeatureBuilder(FxFeature.FxFeatureBuilder fxFeature) {
			this.fxFeature = fxFeature;
			return this;
		}
		public OptionFeatureBuilder setKnock(Knock knock) {
			this.knock = knock.toBuilder();
			return this;
		}
		public OptionFeatureBuilder setKnockBuilder(Knock.KnockBuilder knock) {
			this.knock = knock;
			return this;
		}
		public OptionFeatureBuilder setPassThrough(PassThrough passThrough) {
			this.passThrough = passThrough.toBuilder();
			return this;
		}
		public OptionFeatureBuilder setPassThroughBuilder(PassThrough.PassThroughBuilder passThrough) {
			this.passThrough = passThrough;
			return this;
		}
		public OptionFeatureBuilder setStrategyFeature(StrategyFeature strategyFeature) {
			this.strategyFeature = strategyFeature.toBuilder();
			return this;
		}
		public OptionFeatureBuilder setStrategyFeatureBuilder(StrategyFeature.StrategyFeatureBuilder strategyFeature) {
			this.strategyFeature = strategyFeature;
			return this;
		}
		
		public OptionFeature build() {
			return new OptionFeature(this);
		}
	
		@Override
		public OptionFeatureBuilder prune() {
			if (asian!=null && !asian.prune().hasData()) asian = null;
			if (barrier!=null && !barrier.prune().hasData()) barrier = null;
			if (fxFeature!=null && !fxFeature.prune().hasData()) fxFeature = null;
			if (knock!=null && !knock.prune().hasData()) knock = null;
			if (passThrough!=null && !passThrough.prune().hasData()) passThrough = null;
			if (strategyFeature!=null && !strategyFeature.prune().hasData()) strategyFeature = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getAsian()!=null && getAsian().hasData()) return true;
			if (getBarrier()!=null && getBarrier().hasData()) return true;
			if (getFxFeature()!=null && getFxFeature().hasData()) return true;
			if (getKnock()!=null && getKnock().hasData()) return true;
			if (getPassThrough()!=null && getPassThrough().hasData()) return true;
			if (getStrategyFeature()!=null && getStrategyFeature().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("asian"), processor, Asian.class, asian);
			processRosetta(path.newSubPath("barrier"), processor, Barrier.class, barrier);
			processRosetta(path.newSubPath("fxFeature"), processor, FxFeature.class, fxFeature);
			processRosetta(path.newSubPath("knock"), processor, Knock.class, knock);
			processRosetta(path.newSubPath("passThrough"), processor, PassThrough.class, passThrough);
			processRosetta(path.newSubPath("strategyFeature"), processor, StrategyFeature.class, strategyFeature);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			OptionFeatureBuilder _that = (OptionFeatureBuilder) o;
		
			if (asian != null ? !asian.equals(_that.asian) : _that.asian != null) return false;
			if (barrier != null ? !barrier.equals(_that.barrier) : _that.barrier != null) return false;
			if (fxFeature != null ? !fxFeature.equals(_that.fxFeature) : _that.fxFeature != null) return false;
			if (knock != null ? !knock.equals(_that.knock) : _that.knock != null) return false;
			if (passThrough != null ? !passThrough.equals(_that.passThrough) : _that.passThrough != null) return false;
			if (strategyFeature != null ? !strategyFeature.equals(_that.strategyFeature) : _that.strategyFeature != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (asian != null ? asian.hashCode() : 0);
			_result = 31 * _result + (barrier != null ? barrier.hashCode() : 0);
			_result = 31 * _result + (fxFeature != null ? fxFeature.hashCode() : 0);
			_result = 31 * _result + (knock != null ? knock.hashCode() : 0);
			_result = 31 * _result + (passThrough != null ? passThrough.hashCode() : 0);
			_result = 31 * _result + (strategyFeature != null ? strategyFeature.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "OptionFeatureBuilder {" +
				"asian=" + this.asian + ", " +
				"barrier=" + this.barrier + ", " +
				"fxFeature=" + this.fxFeature + ", " +
				"knock=" + this.knock + ", " +
				"passThrough=" + this.passThrough + ", " +
				"strategyFeature=" + this.strategyFeature +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("asian"), processor, Asian.class, asian);
		processRosetta(path.newSubPath("barrier"), processor, Barrier.class, barrier);
		processRosetta(path.newSubPath("fxFeature"), processor, FxFeature.class, fxFeature);
		processRosetta(path.newSubPath("knock"), processor, Knock.class, knock);
		processRosetta(path.newSubPath("passThrough"), processor, PassThrough.class, passThrough);
		processRosetta(path.newSubPath("strategyFeature"), processor, StrategyFeature.class, strategyFeature);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		OptionFeature _that = (OptionFeature) o;
	
		if (asian != null ? !asian.equals(_that.asian) : _that.asian != null) return false;
		if (barrier != null ? !barrier.equals(_that.barrier) : _that.barrier != null) return false;
		if (fxFeature != null ? !fxFeature.equals(_that.fxFeature) : _that.fxFeature != null) return false;
		if (knock != null ? !knock.equals(_that.knock) : _that.knock != null) return false;
		if (passThrough != null ? !passThrough.equals(_that.passThrough) : _that.passThrough != null) return false;
		if (strategyFeature != null ? !strategyFeature.equals(_that.strategyFeature) : _that.strategyFeature != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (asian != null ? asian.hashCode() : 0);
		_result = 31 * _result + (barrier != null ? barrier.hashCode() : 0);
		_result = 31 * _result + (fxFeature != null ? fxFeature.hashCode() : 0);
		_result = 31 * _result + (knock != null ? knock.hashCode() : 0);
		_result = 31 * _result + (passThrough != null ? passThrough.hashCode() : 0);
		_result = 31 * _result + (strategyFeature != null ? strategyFeature.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "OptionFeature {" +
			"asian=" + this.asian + ", " +
			"barrier=" + this.barrier + ", " +
			"fxFeature=" + this.fxFeature + ", " +
			"knock=" + this.knock + ", " +
			"passThrough=" + this.passThrough + ", " +
			"strategyFeature=" + this.strategyFeature +
		'}';
	}
}
