package org.isda.cdm;

import org.isda.cdm.BuyerSeller;
import org.isda.cdm.ContractualQuantity;
import org.isda.cdm.OptionDenomination;
import org.isda.cdm.OptionExercise;
import org.isda.cdm.OptionFeature;
import org.isda.cdm.OptionTypeEnum;
import org.isda.cdm.Underlier;
import org.isda.cdm.metafields.MetaFields;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.OptionPayoutMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * The option payout specification terms. The associated rosettaKey denotes the ability to associate a hash value to the respective OptionPayout instantiation for the purpose of model cross-referencing, in support of functionality such as the event effect and the lineage.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class OptionPayout extends RosettaModelObject implements GlobalKey {
	private final BuyerSeller buyerSeller;
	private final OptionDenomination denomination;
	private final OptionExercise exerciseTerms;
	private final OptionFeature feature;
	private final MetaFields meta;
	private final OptionTypeEnum optionType;
	private final ContractualQuantity quantity;
	private final Underlier underlier;
	private static OptionPayoutMeta metaData = new OptionPayoutMeta();
	
	OptionPayout(OptionPayoutBuilder builder) {
		this.buyerSeller = ofNullable(builder.getBuyerSeller()).map(f->f.build()).orElse(null);
		this.denomination = ofNullable(builder.getDenomination()).map(f->f.build()).orElse(null);
		this.exerciseTerms = ofNullable(builder.getExerciseTerms()).map(f->f.build()).orElse(null);
		this.feature = ofNullable(builder.getFeature()).map(f->f.build()).orElse(null);
		this.meta = ofNullable(builder.getMeta()).map(f->f.build()).orElse(null);
		this.optionType = builder.getOptionType();
		this.quantity = ofNullable(builder.getQuantity()).map(f->f.build()).orElse(null);
		this.underlier = ofNullable(builder.getUnderlier()).map(f->f.build()).orElse(null);
	}
	
	public final BuyerSeller getBuyerSeller() {
		return buyerSeller;
	}
	
	/**
	 * The denomination qualifies the number of units of underlier per option and the number of options comprised in the option transaction. FpML represents this information as part of the OptionDenomination.model.
	 */
	public final OptionDenomination getDenomination() {
		return denomination;
	}
	
	/**
	 * The terms for exercising the option, which include the option style (e.g. American style option), the exercise procedure (e.g. manual exercise) and the settlement terms (e.g. physical vs. cash).
	 */
	public final OptionExercise getExerciseTerms() {
		return exerciseTerms;
	}
	
	/**
	 * The option feature, such as quanto, Asian, barrier, knock.
	 */
	@RosettaSynonym(value="feature", source="FpML_5_10")
	@RosettaSynonym(value="feature", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="feature", source="DTCC_11_0")
	@RosettaSynonym(value="feature", source="DTCC_9_0")
	@RosettaSynonym(value="feature", source="CME_ClearedConfirm_1_17")
	public final OptionFeature getFeature() {
		return feature;
	}
	
	public final MetaFields getMeta() {
		return meta;
	}
	
	/**
	 * The type of option transaction. From a usage standpoint, put/call is the default option type, while payer/receiver indicator is used for options on index credit default swaps, consistently with the industry practice. Straddle is used for the case of straddle strategy, that combine a call and a put with the same strike.
	 */
	@RosettaSynonym(value="optionType", source="FpML_5_10")
	@RosettaSynonym(value="optionType", source="DTCC_11_0")
	@RosettaSynonym(value="optionType", source="DTCC_9_0")
	@RosettaSynonym(value="optionType", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="optionType", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="", source="FpML_5_10")
	@RosettaSynonym(value="", source="DTCC_11_0")
	@RosettaSynonym(value="", source="DTCC_9_0")
	@RosettaSynonym(value="", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="", source="CME_ClearedConfirm_1_17")
	public final OptionTypeEnum getOptionType() {
		return optionType;
	}
	
	/**
	 * The option notional amount. The CDM uses the generic term quantity rather than notional amount. The number of options is specified as part of the denomination attribute.
	 */
	public final ContractualQuantity getQuantity() {
		return quantity;
	}
	
	/**
	 * The product underlying the option, which can be of any type including ContractualProduct or Security.
	 */
	public final Underlier getUnderlier() {
		return underlier;
	}
	
	
	@Override
	public RosettaMetaData<? extends OptionPayout> metaData() {
		return metaData;
	} 
	
	public OptionPayoutBuilder toBuilder() {
		OptionPayoutBuilder builder = new OptionPayoutBuilder();
		ofNullable(getBuyerSeller()).ifPresent(builder::setBuyerSeller);
		ofNullable(getDenomination()).ifPresent(builder::setDenomination);
		ofNullable(getExerciseTerms()).ifPresent(builder::setExerciseTerms);
		ofNullable(getFeature()).ifPresent(builder::setFeature);
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		ofNullable(getOptionType()).ifPresent(builder::setOptionType);
		ofNullable(getQuantity()).ifPresent(builder::setQuantity);
		ofNullable(getUnderlier()).ifPresent(builder::setUnderlier);
		return builder;
	}

	public static OptionPayoutBuilder builder() {
		return new OptionPayoutBuilder();
	}

	public static  class OptionPayoutBuilder extends RosettaModelObjectBuilder implements GlobalKeyBuilder<OptionPayoutBuilder> {
	
		protected BuyerSeller.BuyerSellerBuilder buyerSeller;
		protected OptionDenomination.OptionDenominationBuilder denomination;
		protected OptionExercise.OptionExerciseBuilder exerciseTerms;
		protected OptionFeature.OptionFeatureBuilder feature;
		protected MetaFields.MetaFieldsBuilder meta;
		protected OptionTypeEnum optionType;
		protected ContractualQuantity.ContractualQuantityBuilder quantity;
		protected Underlier.UnderlierBuilder underlier;
	
		public OptionPayoutBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends OptionPayout> metaData() {
			return metaData;
		} 
	
		
		public BuyerSeller.BuyerSellerBuilder getBuyerSeller() {
			return buyerSeller;
		}
		
		public BuyerSeller.BuyerSellerBuilder getOrCreateBuyerSeller() {
			if (buyerSeller!=null) {
				return buyerSeller;
			}
			else return buyerSeller = new BuyerSeller.BuyerSellerBuilder();
		}
		
		
		public OptionDenomination.OptionDenominationBuilder getDenomination() {
			return denomination;
		}
		
		public OptionDenomination.OptionDenominationBuilder getOrCreateDenomination() {
			if (denomination!=null) {
				return denomination;
			}
			else return denomination = new OptionDenomination.OptionDenominationBuilder();
		}
		
		
		public OptionExercise.OptionExerciseBuilder getExerciseTerms() {
			return exerciseTerms;
		}
		
		public OptionExercise.OptionExerciseBuilder getOrCreateExerciseTerms() {
			if (exerciseTerms!=null) {
				return exerciseTerms;
			}
			else return exerciseTerms = new OptionExercise.OptionExerciseBuilder();
		}
		
		
		public OptionFeature.OptionFeatureBuilder getFeature() {
			return feature;
		}
		
		public OptionFeature.OptionFeatureBuilder getOrCreateFeature() {
			if (feature!=null) {
				return feature;
			}
			else return feature = new OptionFeature.OptionFeatureBuilder();
		}
		
		
		public MetaFields.MetaFieldsBuilder getMeta() {
			return meta;
		}
		
		public MetaFields.MetaFieldsBuilder getOrCreateMeta() {
			if (meta!=null) {
				return meta;
			}
			else return meta = new MetaFields.MetaFieldsBuilder();
		}
		
		
		public OptionTypeEnum getOptionType() {
			return optionType;
		}
		
		
		public ContractualQuantity.ContractualQuantityBuilder getQuantity() {
			return quantity;
		}
		
		public ContractualQuantity.ContractualQuantityBuilder getOrCreateQuantity() {
			if (quantity!=null) {
				return quantity;
			}
			else return quantity = new ContractualQuantity.ContractualQuantityBuilder();
		}
		
		
		public Underlier.UnderlierBuilder getUnderlier() {
			return underlier;
		}
		
		public Underlier.UnderlierBuilder getOrCreateUnderlier() {
			if (underlier!=null) {
				return underlier;
			}
			else return underlier = new Underlier.UnderlierBuilder();
		}
		
	
		public OptionPayoutBuilder setBuyerSeller(BuyerSeller buyerSeller) {
			this.buyerSeller = buyerSeller.toBuilder();
			return this;
		}
		public OptionPayoutBuilder setBuyerSellerBuilder(BuyerSeller.BuyerSellerBuilder buyerSeller) {
			this.buyerSeller = buyerSeller;
			return this;
		}
		public OptionPayoutBuilder setDenomination(OptionDenomination denomination) {
			this.denomination = denomination.toBuilder();
			return this;
		}
		public OptionPayoutBuilder setDenominationBuilder(OptionDenomination.OptionDenominationBuilder denomination) {
			this.denomination = denomination;
			return this;
		}
		public OptionPayoutBuilder setExerciseTerms(OptionExercise exerciseTerms) {
			this.exerciseTerms = exerciseTerms.toBuilder();
			return this;
		}
		public OptionPayoutBuilder setExerciseTermsBuilder(OptionExercise.OptionExerciseBuilder exerciseTerms) {
			this.exerciseTerms = exerciseTerms;
			return this;
		}
		public OptionPayoutBuilder setFeature(OptionFeature feature) {
			this.feature = feature.toBuilder();
			return this;
		}
		public OptionPayoutBuilder setFeatureBuilder(OptionFeature.OptionFeatureBuilder feature) {
			this.feature = feature;
			return this;
		}
		public OptionPayoutBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		public OptionPayoutBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		public OptionPayoutBuilder setOptionType(OptionTypeEnum optionType) {
			this.optionType = optionType;
			return this;
		}
		public OptionPayoutBuilder setQuantity(ContractualQuantity quantity) {
			this.quantity = quantity.toBuilder();
			return this;
		}
		public OptionPayoutBuilder setQuantityBuilder(ContractualQuantity.ContractualQuantityBuilder quantity) {
			this.quantity = quantity;
			return this;
		}
		public OptionPayoutBuilder setUnderlier(Underlier underlier) {
			this.underlier = underlier.toBuilder();
			return this;
		}
		public OptionPayoutBuilder setUnderlierBuilder(Underlier.UnderlierBuilder underlier) {
			this.underlier = underlier;
			return this;
		}
		
		public OptionPayout build() {
			return new OptionPayout(this);
		}
	
		@Override
		public OptionPayoutBuilder prune() {
			if (buyerSeller!=null && !buyerSeller.prune().hasData()) buyerSeller = null;
			if (denomination!=null && !denomination.prune().hasData()) denomination = null;
			if (exerciseTerms!=null && !exerciseTerms.prune().hasData()) exerciseTerms = null;
			if (feature!=null && !feature.prune().hasData()) feature = null;
			if (meta!=null && !meta.prune().hasData()) meta = null;
			if (quantity!=null && !quantity.prune().hasData()) quantity = null;
			if (underlier!=null && !underlier.prune().hasData()) underlier = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getBuyerSeller()!=null && getBuyerSeller().hasData()) return true;
			if (getDenomination()!=null && getDenomination().hasData()) return true;
			if (getExerciseTerms()!=null && getExerciseTerms().hasData()) return true;
			if (getFeature()!=null && getFeature().hasData()) return true;
			if (getMeta()!=null && getMeta().hasData()) return true;
			if (getOptionType()!=null) return true;
			if (getQuantity()!=null && getQuantity().hasData()) return true;
			if (getUnderlier()!=null && getUnderlier().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("optionType"), OptionTypeEnum.class, optionType, this);
			
			processRosetta(path.newSubPath("buyerSeller"), processor, BuyerSeller.class, buyerSeller);
			processRosetta(path.newSubPath("denomination"), processor, OptionDenomination.class, denomination);
			processRosetta(path.newSubPath("exerciseTerms"), processor, OptionExercise.class, exerciseTerms);
			processRosetta(path.newSubPath("feature"), processor, OptionFeature.class, feature);
			processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
			processRosetta(path.newSubPath("quantity"), processor, ContractualQuantity.class, quantity);
			processRosetta(path.newSubPath("underlier"), processor, Underlier.class, underlier);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			OptionPayoutBuilder _that = (OptionPayoutBuilder) o;
		
			if (buyerSeller != null ? !buyerSeller.equals(_that.buyerSeller) : _that.buyerSeller != null) return false;
			if (denomination != null ? !denomination.equals(_that.denomination) : _that.denomination != null) return false;
			if (exerciseTerms != null ? !exerciseTerms.equals(_that.exerciseTerms) : _that.exerciseTerms != null) return false;
			if (feature != null ? !feature.equals(_that.feature) : _that.feature != null) return false;
			if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
			if (optionType != null ? !optionType.equals(_that.optionType) : _that.optionType != null) return false;
			if (quantity != null ? !quantity.equals(_that.quantity) : _that.quantity != null) return false;
			if (underlier != null ? !underlier.equals(_that.underlier) : _that.underlier != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (buyerSeller != null ? buyerSeller.hashCode() : 0);
			_result = 31 * _result + (denomination != null ? denomination.hashCode() : 0);
			_result = 31 * _result + (exerciseTerms != null ? exerciseTerms.hashCode() : 0);
			_result = 31 * _result + (feature != null ? feature.hashCode() : 0);
			_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
			_result = 31 * _result + (optionType != null ? optionType.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (quantity != null ? quantity.hashCode() : 0);
			_result = 31 * _result + (underlier != null ? underlier.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "OptionPayoutBuilder {" +
				"buyerSeller=" + this.buyerSeller + ", " +
				"denomination=" + this.denomination + ", " +
				"exerciseTerms=" + this.exerciseTerms + ", " +
				"feature=" + this.feature + ", " +
				"meta=" + this.meta + ", " +
				"optionType=" + this.optionType + ", " +
				"quantity=" + this.quantity + ", " +
				"underlier=" + this.underlier +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("optionType"), OptionTypeEnum.class, optionType, this);
		
		processRosetta(path.newSubPath("buyerSeller"), processor, BuyerSeller.class, buyerSeller);
		processRosetta(path.newSubPath("denomination"), processor, OptionDenomination.class, denomination);
		processRosetta(path.newSubPath("exerciseTerms"), processor, OptionExercise.class, exerciseTerms);
		processRosetta(path.newSubPath("feature"), processor, OptionFeature.class, feature);
		processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
		processRosetta(path.newSubPath("quantity"), processor, ContractualQuantity.class, quantity);
		processRosetta(path.newSubPath("underlier"), processor, Underlier.class, underlier);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		OptionPayout _that = (OptionPayout) o;
	
		if (buyerSeller != null ? !buyerSeller.equals(_that.buyerSeller) : _that.buyerSeller != null) return false;
		if (denomination != null ? !denomination.equals(_that.denomination) : _that.denomination != null) return false;
		if (exerciseTerms != null ? !exerciseTerms.equals(_that.exerciseTerms) : _that.exerciseTerms != null) return false;
		if (feature != null ? !feature.equals(_that.feature) : _that.feature != null) return false;
		if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
		if (optionType != null ? !optionType.equals(_that.optionType) : _that.optionType != null) return false;
		if (quantity != null ? !quantity.equals(_that.quantity) : _that.quantity != null) return false;
		if (underlier != null ? !underlier.equals(_that.underlier) : _that.underlier != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (buyerSeller != null ? buyerSeller.hashCode() : 0);
		_result = 31 * _result + (denomination != null ? denomination.hashCode() : 0);
		_result = 31 * _result + (exerciseTerms != null ? exerciseTerms.hashCode() : 0);
		_result = 31 * _result + (feature != null ? feature.hashCode() : 0);
		_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
		_result = 31 * _result + (optionType != null ? optionType.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (quantity != null ? quantity.hashCode() : 0);
		_result = 31 * _result + (underlier != null ? underlier.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "OptionPayout {" +
			"buyerSeller=" + this.buyerSeller + ", " +
			"denomination=" + this.denomination + ", " +
			"exerciseTerms=" + this.exerciseTerms + ", " +
			"feature=" + this.feature + ", " +
			"meta=" + this.meta + ", " +
			"optionType=" + this.optionType + ", " +
			"quantity=" + this.quantity + ", " +
			"underlier=" + this.underlier +
		'}';
	}
}
