package org.isda.cdm;

import org.isda.cdm.FxCashSettlement;
import org.isda.cdm.OptionCashSettlement;
import org.isda.cdm.OptionPhysicalSettlement;
import org.isda.cdm.metafields.FieldWithMetaString;


import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.OptionSettlementMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;
		import com.rosetta.model.lib.records.Date;

/**
 * The option settlement terms, which can either be cash, physical, or fx-based cash-settlement. This class can be used for both the settlement of options or forwards.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class OptionSettlement extends SettlementTerms {
	private final OptionCashSettlement cashSettlementTerms;
	private final FxCashSettlement fxSettlementTerms;
	private final OptionPhysicalSettlement physicalSettlementTerms;
	private static OptionSettlementMeta metaData = new OptionSettlementMeta();
	
	OptionSettlement(OptionSettlementBuilder builder) {
		super(builder);
		this.cashSettlementTerms = ofNullable(builder.getCashSettlementTerms()).map(f->f.build()).orElse(null);
		this.fxSettlementTerms = ofNullable(builder.getFxSettlementTerms()).map(f->f.build()).orElse(null);
		this.physicalSettlementTerms = ofNullable(builder.getPhysicalSettlementTerms()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * Specifies the parameters associated with the cash settlement procedure.
	 */
	@RosettaSynonym(value="cashSettlement", source="FpML_5_10")
	@RosettaSynonym(value="cashSettlement", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="cashSettlement", source="DTCC_11_0")
	@RosettaSynonym(value="cashSettlement", source="DTCC_9_0")
	@RosettaSynonym(value="cashSettlement", source="CME_ClearedConfirm_1_17")
	public final OptionCashSettlement getCashSettlementTerms() {
		return cashSettlementTerms;
	}
	
	/**
	 * Specifies the cash settlement of a non deliverable fx option / forward. It includes the currency to settle into together with the fixings required to calculate the currency amount
	 */
	@RosettaSynonym(value="nonDeliverableSettlement", source="FpML_5_10")
	public final FxCashSettlement getFxSettlementTerms() {
		return fxSettlementTerms;
	}
	
	/**
	 * Specifies the physical settlement terms which apply to the transaction.
	 */
	@RosettaSynonym(value="physicalSettlement", source="FpML_5_10")
	@RosettaSynonym(value="physicalSettlement", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="physicalSettlement", source="DTCC_11_0")
	@RosettaSynonym(value="physicalSettlement", source="DTCC_9_0")
	@RosettaSynonym(value="physicalSettlement", source="CME_ClearedConfirm_1_17")
	public final OptionPhysicalSettlement getPhysicalSettlementTerms() {
		return physicalSettlementTerms;
	}
	
	
	@Override
	public RosettaMetaData<? extends OptionSettlement> metaData() {
		return metaData;
	} 
	
	public OptionSettlementBuilder toBuilder() {
		OptionSettlementBuilder builder = new OptionSettlementBuilder();
		ofNullable(getCashSettlementTerms()).ifPresent(builder::setCashSettlementTerms);
		ofNullable(getFxSettlementTerms()).ifPresent(builder::setFxSettlementTerms);
		ofNullable(getPhysicalSettlementTerms()).ifPresent(builder::setPhysicalSettlementTerms);
		ofNullable(getSettlementAmount()).ifPresent(builder::setSettlementAmount);
		ofNullable(getSettlementDate()).ifPresent(builder::setSettlementDate);
		ofNullable(getSettlementType()).ifPresent(builder::setSettlementType);
		ofNullable(getTransferSettlementType()).ifPresent(builder::setTransferSettlementType);
		ofNullable(getValueDate()).ifPresent(builder::setValueDate);
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		ofNullable(getSettlementCurrency()).ifPresent(builder::setSettlementCurrency);
		return builder;
	}

	public static OptionSettlementBuilder builder() {
		return new OptionSettlementBuilder();
	}

	public static  class OptionSettlementBuilder extends SettlementTermsBuilder {
	
		protected OptionCashSettlement.OptionCashSettlementBuilder cashSettlementTerms;
		protected FxCashSettlement.FxCashSettlementBuilder fxSettlementTerms;
		protected OptionPhysicalSettlement.OptionPhysicalSettlementBuilder physicalSettlementTerms;
	
		public OptionSettlementBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends OptionSettlement> metaData() {
			return metaData;
		} 
	
		
		public OptionCashSettlement.OptionCashSettlementBuilder getCashSettlementTerms() {
			return cashSettlementTerms;
		}
		
		public OptionCashSettlement.OptionCashSettlementBuilder getOrCreateCashSettlementTerms() {
			if (cashSettlementTerms!=null) {
				return cashSettlementTerms;
			}
			else return cashSettlementTerms = new OptionCashSettlement.OptionCashSettlementBuilder();
		}
		
		
		public FxCashSettlement.FxCashSettlementBuilder getFxSettlementTerms() {
			return fxSettlementTerms;
		}
		
		public FxCashSettlement.FxCashSettlementBuilder getOrCreateFxSettlementTerms() {
			if (fxSettlementTerms!=null) {
				return fxSettlementTerms;
			}
			else return fxSettlementTerms = new FxCashSettlement.FxCashSettlementBuilder();
		}
		
		
		public OptionPhysicalSettlement.OptionPhysicalSettlementBuilder getPhysicalSettlementTerms() {
			return physicalSettlementTerms;
		}
		
		public OptionPhysicalSettlement.OptionPhysicalSettlementBuilder getOrCreatePhysicalSettlementTerms() {
			if (physicalSettlementTerms!=null) {
				return physicalSettlementTerms;
			}
			else return physicalSettlementTerms = new OptionPhysicalSettlement.OptionPhysicalSettlementBuilder();
		}
		
	
		public OptionSettlementBuilder setCashSettlementTerms(OptionCashSettlement cashSettlementTerms) {
			this.cashSettlementTerms = cashSettlementTerms.toBuilder();
			return this;
		}
		public OptionSettlementBuilder setCashSettlementTermsBuilder(OptionCashSettlement.OptionCashSettlementBuilder cashSettlementTerms) {
			this.cashSettlementTerms = cashSettlementTerms;
			return this;
		}
		public OptionSettlementBuilder setFxSettlementTerms(FxCashSettlement fxSettlementTerms) {
			this.fxSettlementTerms = fxSettlementTerms.toBuilder();
			return this;
		}
		public OptionSettlementBuilder setFxSettlementTermsBuilder(FxCashSettlement.FxCashSettlementBuilder fxSettlementTerms) {
			this.fxSettlementTerms = fxSettlementTerms;
			return this;
		}
		public OptionSettlementBuilder setPhysicalSettlementTerms(OptionPhysicalSettlement physicalSettlementTerms) {
			this.physicalSettlementTerms = physicalSettlementTerms.toBuilder();
			return this;
		}
		public OptionSettlementBuilder setPhysicalSettlementTermsBuilder(OptionPhysicalSettlement.OptionPhysicalSettlementBuilder physicalSettlementTerms) {
			this.physicalSettlementTerms = physicalSettlementTerms;
			return this;
		}
		@Override public OptionSettlementBuilder setSettlementAmount(Money settlementAmount) {
			this.settlementAmount = settlementAmount.toBuilder();
			return this;
		}
		@Override public OptionSettlementBuilder setSettlementAmountBuilder(Money.MoneyBuilder settlementAmount) {
			this.settlementAmount = settlementAmount;
			return this;
		}
		@Override public OptionSettlementBuilder setSettlementDate(AdjustableOrRelativeDate settlementDate) {
			this.settlementDate = settlementDate.toBuilder();
			return this;
		}
		@Override public OptionSettlementBuilder setSettlementDateBuilder(AdjustableOrRelativeDate.AdjustableOrRelativeDateBuilder settlementDate) {
			this.settlementDate = settlementDate;
			return this;
		}
		@Override public OptionSettlementBuilder setSettlementType(SettlementTypeEnum settlementType) {
			this.settlementType = settlementType;
			return this;
		}
		@Override public OptionSettlementBuilder setTransferSettlementType(TransferSettlementEnum transferSettlementType) {
			this.transferSettlementType = transferSettlementType;
			return this;
		}
		@Override public OptionSettlementBuilder setValueDate(Date valueDate) {
			this.valueDate = valueDate;
			return this;
		}
		@Override public OptionSettlementBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		@Override public OptionSettlementBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		@Override public OptionSettlementBuilder setSettlementCurrency(FieldWithMetaString settlementCurrency) {
			this.settlementCurrency = settlementCurrency.toBuilder();
			return this;
		}
		@Override public OptionSettlementBuilder setSettlementCurrencyRef(String settlementCurrency) {
			return setSettlementCurrency(FieldWithMetaString.builder().setValue(settlementCurrency).build());
		}
		
		public OptionSettlement build() {
			return new OptionSettlement(this);
		}
	
		@Override
		public OptionSettlementBuilder prune() {
			super.prune();
			if (cashSettlementTerms!=null && !cashSettlementTerms.prune().hasData()) cashSettlementTerms = null;
			if (fxSettlementTerms!=null && !fxSettlementTerms.prune().hasData()) fxSettlementTerms = null;
			if (physicalSettlementTerms!=null && !physicalSettlementTerms.prune().hasData()) physicalSettlementTerms = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (super.hasData()) return true;
			if (getCashSettlementTerms()!=null && getCashSettlementTerms().hasData()) return true;
			if (getFxSettlementTerms()!=null && getFxSettlementTerms().hasData()) return true;
			if (getPhysicalSettlementTerms()!=null && getPhysicalSettlementTerms().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
			super.process(path, processor);
		
			
			processRosetta(path.newSubPath("cashSettlementTerms"), processor, OptionCashSettlement.class, cashSettlementTerms);
			processRosetta(path.newSubPath("fxSettlementTerms"), processor, FxCashSettlement.class, fxSettlementTerms);
			processRosetta(path.newSubPath("physicalSettlementTerms"), processor, OptionPhysicalSettlement.class, physicalSettlementTerms);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
			if (!super.equals(o)) return false;
		
			OptionSettlementBuilder _that = (OptionSettlementBuilder) o;
		
			if (cashSettlementTerms != null ? !cashSettlementTerms.equals(_that.cashSettlementTerms) : _that.cashSettlementTerms != null) return false;
			if (fxSettlementTerms != null ? !fxSettlementTerms.equals(_that.fxSettlementTerms) : _that.fxSettlementTerms != null) return false;
			if (physicalSettlementTerms != null ? !physicalSettlementTerms.equals(_that.physicalSettlementTerms) : _that.physicalSettlementTerms != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = super.hashCode();
			_result = 31 * _result + (cashSettlementTerms != null ? cashSettlementTerms.hashCode() : 0);
			_result = 31 * _result + (fxSettlementTerms != null ? fxSettlementTerms.hashCode() : 0);
			_result = 31 * _result + (physicalSettlementTerms != null ? physicalSettlementTerms.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "OptionSettlementBuilder {" +
				"cashSettlementTerms=" + this.cashSettlementTerms + ", " +
				"fxSettlementTerms=" + this.fxSettlementTerms + ", " +
				"physicalSettlementTerms=" + this.physicalSettlementTerms +
			'}' + " " + super.toString();
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		super.process(path, processor);
		
		
		processRosetta(path.newSubPath("cashSettlementTerms"), processor, OptionCashSettlement.class, cashSettlementTerms);
		processRosetta(path.newSubPath("fxSettlementTerms"), processor, FxCashSettlement.class, fxSettlementTerms);
		processRosetta(path.newSubPath("physicalSettlementTerms"), processor, OptionPhysicalSettlement.class, physicalSettlementTerms);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
		if (!super.equals(o)) return false;
	
		OptionSettlement _that = (OptionSettlement) o;
	
		if (cashSettlementTerms != null ? !cashSettlementTerms.equals(_that.cashSettlementTerms) : _that.cashSettlementTerms != null) return false;
		if (fxSettlementTerms != null ? !fxSettlementTerms.equals(_that.fxSettlementTerms) : _that.fxSettlementTerms != null) return false;
		if (physicalSettlementTerms != null ? !physicalSettlementTerms.equals(_that.physicalSettlementTerms) : _that.physicalSettlementTerms != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = super.hashCode();
		_result = 31 * _result + (cashSettlementTerms != null ? cashSettlementTerms.hashCode() : 0);
		_result = 31 * _result + (fxSettlementTerms != null ? fxSettlementTerms.hashCode() : 0);
		_result = 31 * _result + (physicalSettlementTerms != null ? physicalSettlementTerms.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "OptionSettlement {" +
			"cashSettlementTerms=" + this.cashSettlementTerms + ", " +
			"fxSettlementTerms=" + this.fxSettlementTerms + ", " +
			"physicalSettlementTerms=" + this.physicalSettlementTerms +
		'}' + " " + super.toString();
	}
}
