package org.isda.cdm;

import java.math.BigDecimal;
import org.isda.cdm.ReferenceSwapCurve;
import org.isda.cdm.metafields.FieldWithMetaString;
import org.isda.cdm.metafields.FieldWithMetaString.FieldWithMetaStringBuilder;
import org.isda.cdm.metafields.ReferenceWithMetaSchedule;
import org.isda.cdm.metafields.ReferenceWithMetaSchedule.ReferenceWithMetaScheduleBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.OptionStrikeMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.math.BigDecimal;
		import java.util.stream.Collectors;
		import static java.util.Optional.ofNullable;

/**
 * A class to specify the option strike.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="CreditOptionStrike", source="FpML_5_10")
public class OptionStrike extends RosettaModelObject {
	private final FieldWithMetaString currency;
	private final BigDecimal percentage;
	private final BigDecimal price;
	private final ReferenceSwapCurve referenceSwapCurve;
	private final BigDecimal spread;
	private final ReferenceWithMetaSchedule strikeReference;
	private static OptionStrikeMeta metaData = new OptionStrikeMeta();
	
	OptionStrike(OptionStrikeBuilder builder) {
		this.currency = ofNullable(builder.getCurrency()).map(f->f.build()).orElse(null);
		this.percentage = builder.getPercentage();
		this.price = builder.getPrice();
		this.referenceSwapCurve = ofNullable(builder.getReferenceSwapCurve()).map(f->f.build()).orElse(null);
		this.spread = builder.getSpread();
		this.strikeReference = ofNullable(builder.getStrikeReference()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The currency in which the option strike is denominated. The list of valid currencies is not presently positioned as an enumeration as part of the CDM because that scope is limited to the values specified by ISDA and FpML. As a result, implementers have to make reference to the relevant standard, such as the ISO 4217 standard for currency codes.
	 */
	@RosettaSynonym(value="currency", source="FpML_5_10", path="price")
	@RosettaSynonym(value="currency", source="CME_SubmissionIRS_1_0", path="price")
	@RosettaSynonym(value="currency", source="DTCC_11_0", path="price")
	@RosettaSynonym(value="currency", source="DTCC_9_0", path="price")
	@RosettaSynonym(value="currency", source="CME_ClearedConfirm_1_17", path="price")
	public final FieldWithMetaString getCurrency() {
		return currency;
	}
	
	/**
	 * The price or level expressed as a percentage of the forward starting spot price.
	 */
	@RosettaSynonym(value="strikePercentage", source="FpML_5_10", path="price")
	@RosettaSynonym(value="strikePercentage", source="CME_SubmissionIRS_1_0", path="price")
	@RosettaSynonym(value="strikePercentage", source="DTCC_11_0", path="price")
	@RosettaSynonym(value="strikePercentage", source="DTCC_9_0", path="price")
	@RosettaSynonym(value="strikePercentage", source="CME_ClearedConfirm_1_17", path="price")
	public final BigDecimal getPercentage() {
		return percentage;
	}
	
	/**
	 * The strike of a credit default swap option or credit swaption when expressed as in reference to the price of the underlying obligation(s) or index.
	 */
	@RosettaSynonym(value="strikePrice", source="FpML_5_10", path="price")
	@RosettaSynonym(value="strikePrice", source="CME_SubmissionIRS_1_0", path="price")
	@RosettaSynonym(value="strikePrice", source="DTCC_11_0", path="price")
	@RosettaSynonym(value="strikePrice", source="DTCC_9_0", path="price")
	@RosettaSynonym(value="strikePrice", source="CME_ClearedConfirm_1_17", path="price")
	public final BigDecimal getPrice() {
		return price;
	}
	
	/**
	 * The strike of an option when expressed by reference to a swap curve. (Typically the case for a convertible bond option.)
	 */
	@RosettaSynonym(value="referenceSwapCurve", source="FpML_5_10")
	@RosettaSynonym(value="referenceSwapCurve", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="referenceSwapCurve", source="DTCC_11_0")
	@RosettaSynonym(value="referenceSwapCurve", source="DTCC_9_0")
	@RosettaSynonym(value="referenceSwapCurve", source="CME_ClearedConfirm_1_17")
	public final ReferenceSwapCurve getReferenceSwapCurve() {
		return referenceSwapCurve;
	}
	
	/**
	 * The strike of a credit default swap option or credit swaption when expressed as a spread per annum.
	 */
	@RosettaSynonym(value="spread", source="FpML_5_10")
	@RosettaSynonym(value="spread", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="spread", source="DTCC_11_0")
	@RosettaSynonym(value="spread", source="DTCC_9_0")
	@RosettaSynonym(value="spread", source="CME_ClearedConfirm_1_17")
	public final BigDecimal getSpread() {
		return spread;
	}
	
	/**
	 * The strike of an option on a credit default swap when expressed in reference to the spread of the underlying swap (typical practice in the case of credit single name swaps).
	 */
	@RosettaSynonym(value="strikeReference", source="FpML_5_10")
	@RosettaSynonym(value="strikeReference", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="strikeReference", source="DTCC_11_0")
	@RosettaSynonym(value="strikeReference", source="DTCC_9_0")
	@RosettaSynonym(value="strikeReference", source="CME_ClearedConfirm_1_17")
	public final ReferenceWithMetaSchedule getStrikeReference() {
		return strikeReference;
	}
	
	
	@Override
	public RosettaMetaData<? extends OptionStrike> metaData() {
		return metaData;
	} 
	
	public OptionStrikeBuilder toBuilder() {
		OptionStrikeBuilder builder = new OptionStrikeBuilder();
		ofNullable(getCurrency()).ifPresent(builder::setCurrency);
		ofNullable(getPercentage()).ifPresent(builder::setPercentage);
		ofNullable(getPrice()).ifPresent(builder::setPrice);
		ofNullable(getReferenceSwapCurve()).ifPresent(builder::setReferenceSwapCurve);
		ofNullable(getSpread()).ifPresent(builder::setSpread);
		ofNullable(getStrikeReference()).ifPresent(builder::setStrikeReference);
		return builder;
	}

	public static OptionStrikeBuilder builder() {
		return new OptionStrikeBuilder();
	}

	public static  class OptionStrikeBuilder extends RosettaModelObjectBuilder {
	
		protected FieldWithMetaStringBuilder currency;
		protected BigDecimal percentage;
		protected BigDecimal price;
		protected ReferenceSwapCurve.ReferenceSwapCurveBuilder referenceSwapCurve;
		protected BigDecimal spread;
		protected ReferenceWithMetaScheduleBuilder strikeReference;
	
		public OptionStrikeBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends OptionStrike> metaData() {
			return metaData;
		} 
	
		
		public FieldWithMetaStringBuilder getCurrency() {
			return currency;
		}
		
		public FieldWithMetaStringBuilder getOrCreateCurrency() {
			if (currency!=null) {
				return currency;
			}
			else return currency = new FieldWithMetaStringBuilder();
		}
		
		
		public BigDecimal getPercentage() {
			return percentage;
		}
		
		
		public BigDecimal getPrice() {
			return price;
		}
		
		
		public ReferenceSwapCurve.ReferenceSwapCurveBuilder getReferenceSwapCurve() {
			return referenceSwapCurve;
		}
		
		public ReferenceSwapCurve.ReferenceSwapCurveBuilder getOrCreateReferenceSwapCurve() {
			if (referenceSwapCurve!=null) {
				return referenceSwapCurve;
			}
			else return referenceSwapCurve = new ReferenceSwapCurve.ReferenceSwapCurveBuilder();
		}
		
		
		public BigDecimal getSpread() {
			return spread;
		}
		
		
		public ReferenceWithMetaScheduleBuilder getStrikeReference() {
			return strikeReference;
		}
		
		public ReferenceWithMetaScheduleBuilder getOrCreateStrikeReference() {
			if (strikeReference!=null) {
				return strikeReference;
			}
			else return strikeReference = new ReferenceWithMetaScheduleBuilder();
		}
		
	
		public OptionStrikeBuilder setCurrency(FieldWithMetaString currency) {
			this.currency = currency.toBuilder();
			return this;
		}
		public OptionStrikeBuilder setCurrencyRef(String currency) {
			return setCurrency(FieldWithMetaString.builder().setValue(currency).build());
		}
		public OptionStrikeBuilder setPercentage(BigDecimal percentage) {
			this.percentage = percentage;
			return this;
		}
		public OptionStrikeBuilder setPrice(BigDecimal price) {
			this.price = price;
			return this;
		}
		public OptionStrikeBuilder setReferenceSwapCurve(ReferenceSwapCurve referenceSwapCurve) {
			this.referenceSwapCurve = referenceSwapCurve.toBuilder();
			return this;
		}
		public OptionStrikeBuilder setReferenceSwapCurveBuilder(ReferenceSwapCurve.ReferenceSwapCurveBuilder referenceSwapCurve) {
			this.referenceSwapCurve = referenceSwapCurve;
			return this;
		}
		public OptionStrikeBuilder setSpread(BigDecimal spread) {
			this.spread = spread;
			return this;
		}
		public OptionStrikeBuilder setStrikeReference(ReferenceWithMetaSchedule strikeReference) {
			this.strikeReference = strikeReference.toBuilder();
			return this;
		}
		public OptionStrikeBuilder setStrikeReferenceBuilder(ReferenceWithMetaScheduleBuilder strikeReference) {
			this.strikeReference = strikeReference;
			return this;
		}
		public OptionStrikeBuilder setStrikeReferenceRef(Schedule.ScheduleBuilder strikeReference) {
			return setStrikeReference(ReferenceWithMetaSchedule.builder().setValueBuilder(strikeReference).build());
		}
		public OptionStrikeBuilder setStrikeReferenceRef(Schedule strikeReference) {
			return setStrikeReferenceRef(Schedule.builder());
		}
		
		public OptionStrike build() {
			return new OptionStrike(this);
		}
	
		@Override
		public OptionStrikeBuilder prune() {
			if (currency!=null && !currency.prune().hasData()) currency = null;
			if (referenceSwapCurve!=null && !referenceSwapCurve.prune().hasData()) referenceSwapCurve = null;
			if (strikeReference!=null && !strikeReference.prune().hasData()) strikeReference = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getCurrency()!=null) return true;
			if (getPercentage()!=null) return true;
			if (getPrice()!=null) return true;
			if (getReferenceSwapCurve()!=null && getReferenceSwapCurve().hasData()) return true;
			if (getSpread()!=null) return true;
			if (getStrikeReference()!=null && getStrikeReference().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("percentage"), BigDecimal.class, percentage, this);
			processor.processBasic(path.newSubPath("price"), BigDecimal.class, price, this);
			processor.processBasic(path.newSubPath("spread"), BigDecimal.class, spread, this);
			
			processRosetta(path.newSubPath("currency"), processor, FieldWithMetaString.class, currency);
			processRosetta(path.newSubPath("referenceSwapCurve"), processor, ReferenceSwapCurve.class, referenceSwapCurve);
			processRosetta(path.newSubPath("strikeReference"), processor, ReferenceWithMetaSchedule.class, strikeReference);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			OptionStrikeBuilder _that = (OptionStrikeBuilder) o;
		
			if (currency != null ? !currency.equals(_that.currency) : _that.currency != null) return false;
			if (percentage != null ? !percentage.equals(_that.percentage) : _that.percentage != null) return false;
			if (price != null ? !price.equals(_that.price) : _that.price != null) return false;
			if (referenceSwapCurve != null ? !referenceSwapCurve.equals(_that.referenceSwapCurve) : _that.referenceSwapCurve != null) return false;
			if (spread != null ? !spread.equals(_that.spread) : _that.spread != null) return false;
			if (strikeReference != null ? !strikeReference.equals(_that.strikeReference) : _that.strikeReference != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (currency != null ? currency.hashCode() : 0);
			_result = 31 * _result + (percentage != null ? percentage.hashCode() : 0);
			_result = 31 * _result + (price != null ? price.hashCode() : 0);
			_result = 31 * _result + (referenceSwapCurve != null ? referenceSwapCurve.hashCode() : 0);
			_result = 31 * _result + (spread != null ? spread.hashCode() : 0);
			_result = 31 * _result + (strikeReference != null ? strikeReference.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "OptionStrikeBuilder {" +
				"currency=" + this.currency + ", " +
				"percentage=" + this.percentage + ", " +
				"price=" + this.price + ", " +
				"referenceSwapCurve=" + this.referenceSwapCurve + ", " +
				"spread=" + this.spread + ", " +
				"strikeReference=" + this.strikeReference +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("percentage"), BigDecimal.class, percentage, this);
		processor.processBasic(path.newSubPath("price"), BigDecimal.class, price, this);
		processor.processBasic(path.newSubPath("spread"), BigDecimal.class, spread, this);
		
		processRosetta(path.newSubPath("currency"), processor, FieldWithMetaString.class, currency);
		processRosetta(path.newSubPath("referenceSwapCurve"), processor, ReferenceSwapCurve.class, referenceSwapCurve);
		processRosetta(path.newSubPath("strikeReference"), processor, ReferenceWithMetaSchedule.class, strikeReference);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		OptionStrike _that = (OptionStrike) o;
	
		if (currency != null ? !currency.equals(_that.currency) : _that.currency != null) return false;
		if (percentage != null ? !percentage.equals(_that.percentage) : _that.percentage != null) return false;
		if (price != null ? !price.equals(_that.price) : _that.price != null) return false;
		if (referenceSwapCurve != null ? !referenceSwapCurve.equals(_that.referenceSwapCurve) : _that.referenceSwapCurve != null) return false;
		if (spread != null ? !spread.equals(_that.spread) : _that.spread != null) return false;
		if (strikeReference != null ? !strikeReference.equals(_that.strikeReference) : _that.strikeReference != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (currency != null ? currency.hashCode() : 0);
		_result = 31 * _result + (percentage != null ? percentage.hashCode() : 0);
		_result = 31 * _result + (price != null ? price.hashCode() : 0);
		_result = 31 * _result + (referenceSwapCurve != null ? referenceSwapCurve.hashCode() : 0);
		_result = 31 * _result + (spread != null ? spread.hashCode() : 0);
		_result = 31 * _result + (strikeReference != null ? strikeReference.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "OptionStrike {" +
			"currency=" + this.currency + ", " +
			"percentage=" + this.percentage + ", " +
			"price=" + this.price + ", " +
			"referenceSwapCurve=" + this.referenceSwapCurve + ", " +
			"spread=" + this.spread + ", " +
			"strikeReference=" + this.strikeReference +
		'}';
	}
}
