package org.isda.cdm;

import java.lang.Boolean;
import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.AmericanExercise;
import org.isda.cdm.BermudaExercise;
import org.isda.cdm.BuyerSeller;
import org.isda.cdm.CalculationAgent;
import org.isda.cdm.EuropeanExercise;
import org.isda.cdm.ExerciseNotice;
import org.isda.cdm.OptionCashSettlement;
import org.isda.cdm.OptionalEarlyTerminationAdjustedDates;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.OptionalEarlyTerminationMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;
		import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;

/**
 * A class defining an early termination provision where either or both parties have the right to exercise.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="OptionalEarlyTermination", source="FpML_5_10")
public class OptionalEarlyTermination extends RosettaModelObject {
	private final AmericanExercise americanExercise;
	private final BermudaExercise bermudaExercise;
	private final CalculationAgent calculationAgent;
	private final OptionCashSettlement cashSettlement;
	private final EuropeanExercise europeanExercise;
	private final List<ExerciseNotice> exerciseNotice;
	private final Boolean followUpConfirmation;
	private final OptionalEarlyTerminationAdjustedDates optionalEarlyTerminationAdjustedDates;
	private final BuyerSeller singlePartyOption;
	private static OptionalEarlyTerminationMeta metaData = new OptionalEarlyTerminationMeta();
	
	OptionalEarlyTermination(OptionalEarlyTerminationBuilder builder) {
		this.americanExercise = ofNullable(builder.getAmericanExercise()).map(f->f.build()).orElse(null);
		this.bermudaExercise = ofNullable(builder.getBermudaExercise()).map(f->f.build()).orElse(null);
		this.calculationAgent = ofNullable(builder.getCalculationAgent()).map(f->f.build()).orElse(null);
		this.cashSettlement = ofNullable(builder.getCashSettlement()).map(f->f.build()).orElse(null);
		this.europeanExercise = ofNullable(builder.getEuropeanExercise()).map(f->f.build()).orElse(null);
		this.exerciseNotice = ofNullable(builder.getExerciseNotice()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.followUpConfirmation = builder.getFollowUpConfirmation();
		this.optionalEarlyTerminationAdjustedDates = ofNullable(builder.getOptionalEarlyTerminationAdjustedDates()).map(f->f.build()).orElse(null);
		this.singlePartyOption = ofNullable(builder.getSinglePartyOption()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * American exercise. FpML implementations consists in an exercise substitution group.
	 */
	@RosettaSynonym(value="americanExercise", source="FpML_5_10")
	@RosettaSynonym(value="americanExercise", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="americanExercise", source="DTCC_11_0")
	@RosettaSynonym(value="americanExercise", source="DTCC_9_0")
	@RosettaSynonym(value="americanExercise", source="CME_ClearedConfirm_1_17")
	public final AmericanExercise getAmericanExercise() {
		return americanExercise;
	}
	
	/**
	 * Bermuda exercise. FpML implementations consists in an exercise substitution group.
	 */
	@RosettaSynonym(value="bermudaExercise", source="FpML_5_10")
	@RosettaSynonym(value="bermudaExercise", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="bermudaExercise", source="DTCC_11_0")
	@RosettaSynonym(value="bermudaExercise", source="DTCC_9_0")
	@RosettaSynonym(value="bermudaExercise", source="CME_ClearedConfirm_1_17")
	public final BermudaExercise getBermudaExercise() {
		return bermudaExercise;
	}
	
	/**
	 * The ISDA Calculation Agent responsible for performing duties associated with an optional early termination.
	 */
	@RosettaSynonym(value="calculationAgent", source="FpML_5_10")
	@RosettaSynonym(value="calculationAgent", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="calculationAgent", source="DTCC_11_0")
	@RosettaSynonym(value="calculationAgent", source="DTCC_9_0")
	@RosettaSynonym(value="calculationAgent", source="CME_ClearedConfirm_1_17")
	public final CalculationAgent getCalculationAgent() {
		return calculationAgent;
	}
	
	/**
	 * If specified, this means that cash settlement is applicable to the transaction and defines the parameters associated with the cash settlement procedure. If not specified, then physical settlement is applicable.
	 */
	@RosettaSynonym(value="cashSettlement", source="FpML_5_10")
	@RosettaSynonym(value="cashSettlement", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="cashSettlement", source="DTCC_11_0")
	@RosettaSynonym(value="cashSettlement", source="DTCC_9_0")
	@RosettaSynonym(value="cashSettlement", source="CME_ClearedConfirm_1_17")
	public final OptionCashSettlement getCashSettlement() {
		return cashSettlement;
	}
	
	/**
	 * European exercise. FpML implementations consists in an exercise substitution group.
	 */
	@RosettaSynonym(value="europeanExercise", source="FpML_5_10")
	@RosettaSynonym(value="europeanExercise", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="europeanExercise", source="DTCC_11_0")
	@RosettaSynonym(value="europeanExercise", source="DTCC_9_0")
	@RosettaSynonym(value="europeanExercise", source="CME_ClearedConfirm_1_17")
	public final EuropeanExercise getEuropeanExercise() {
		return europeanExercise;
	}
	
	/**
	 * Definition of the party to whom notice of exercise should be given.
	 */
	@RosettaSynonym(value="exerciseNotice", source="FpML_5_10")
	@RosettaSynonym(value="exerciseNotice", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="exerciseNotice", source="DTCC_11_0")
	@RosettaSynonym(value="exerciseNotice", source="DTCC_9_0")
	@RosettaSynonym(value="exerciseNotice", source="CME_ClearedConfirm_1_17")
	public final List<ExerciseNotice> getExerciseNotice() {
		return exerciseNotice;
	}
	
	/**
	 * A flag to indicate whether follow-up confirmation of exercise (written or electronic) is required following telephonic notice by the buyer to the seller or seller&#39;s agent.
	 */
	@RosettaSynonym(value="followUpConfirmation", source="FpML_5_10")
	@RosettaSynonym(value="followUpConfirmation", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="followUpConfirmation", source="DTCC_11_0")
	@RosettaSynonym(value="followUpConfirmation", source="DTCC_9_0")
	@RosettaSynonym(value="followUpConfirmation", source="CME_ClearedConfirm_1_17")
	public final Boolean getFollowUpConfirmation() {
		return followUpConfirmation;
	}
	
	/**
	 * An early termination provision to terminate the trade at fair value where one or both parties have the right to decide on termination.
	 */
	@RosettaSynonym(value="optionalEarlyTerminationAdjustedDates", source="FpML_5_10")
	@RosettaSynonym(value="optionalEarlyTerminationAdjustedDates", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="optionalEarlyTerminationAdjustedDates", source="DTCC_11_0")
	@RosettaSynonym(value="optionalEarlyTerminationAdjustedDates", source="DTCC_9_0")
	@RosettaSynonym(value="optionalEarlyTerminationAdjustedDates", source="CME_ClearedConfirm_1_17")
	public final OptionalEarlyTerminationAdjustedDates getOptionalEarlyTerminationAdjustedDates() {
		return optionalEarlyTerminationAdjustedDates;
	}
	
	/**
	 * If optional early termination is not available to both parties then this component specifies the buyer and seller of the option. In FpML, this attribute is of type SinglePsrtyOption, which actually consists of the BuyerSeller.model.
	 */
	@RosettaSynonym(value="singlePartyOption", source="FpML_5_10")
	@RosettaSynonym(value="singlePartyOption", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="singlePartyOption", source="DTCC_11_0")
	@RosettaSynonym(value="singlePartyOption", source="DTCC_9_0")
	@RosettaSynonym(value="singlePartyOption", source="CME_ClearedConfirm_1_17")
	public final BuyerSeller getSinglePartyOption() {
		return singlePartyOption;
	}
	
	
	@Override
	public RosettaMetaData<? extends OptionalEarlyTermination> metaData() {
		return metaData;
	} 
	
	public OptionalEarlyTerminationBuilder toBuilder() {
		OptionalEarlyTerminationBuilder builder = new OptionalEarlyTerminationBuilder();
		ofNullable(getAmericanExercise()).ifPresent(builder::setAmericanExercise);
		ofNullable(getBermudaExercise()).ifPresent(builder::setBermudaExercise);
		ofNullable(getCalculationAgent()).ifPresent(builder::setCalculationAgent);
		ofNullable(getCashSettlement()).ifPresent(builder::setCashSettlement);
		ofNullable(getEuropeanExercise()).ifPresent(builder::setEuropeanExercise);
		ofNullable(getExerciseNotice()).ifPresent(exerciseNotice -> exerciseNotice.forEach(builder::addExerciseNotice));
		ofNullable(getFollowUpConfirmation()).ifPresent(builder::setFollowUpConfirmation);
		ofNullable(getOptionalEarlyTerminationAdjustedDates()).ifPresent(builder::setOptionalEarlyTerminationAdjustedDates);
		ofNullable(getSinglePartyOption()).ifPresent(builder::setSinglePartyOption);
		return builder;
	}

	public static OptionalEarlyTerminationBuilder builder() {
		return new OptionalEarlyTerminationBuilder();
	}

	public static  class OptionalEarlyTerminationBuilder extends RosettaModelObjectBuilder {
	
		protected AmericanExercise.AmericanExerciseBuilder americanExercise;
		protected BermudaExercise.BermudaExerciseBuilder bermudaExercise;
		protected CalculationAgent.CalculationAgentBuilder calculationAgent;
		protected OptionCashSettlement.OptionCashSettlementBuilder cashSettlement;
		protected EuropeanExercise.EuropeanExerciseBuilder europeanExercise;
		protected List<ExerciseNotice.ExerciseNoticeBuilder> exerciseNotice;
		protected Boolean followUpConfirmation;
		protected OptionalEarlyTerminationAdjustedDates.OptionalEarlyTerminationAdjustedDatesBuilder optionalEarlyTerminationAdjustedDates;
		protected BuyerSeller.BuyerSellerBuilder singlePartyOption;
	
		public OptionalEarlyTerminationBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends OptionalEarlyTermination> metaData() {
			return metaData;
		} 
	
		
		public AmericanExercise.AmericanExerciseBuilder getAmericanExercise() {
			return americanExercise;
		}
		
		public AmericanExercise.AmericanExerciseBuilder getOrCreateAmericanExercise() {
			if (americanExercise!=null) {
				return americanExercise;
			}
			else return americanExercise = new AmericanExercise.AmericanExerciseBuilder();
		}
		
		
		public BermudaExercise.BermudaExerciseBuilder getBermudaExercise() {
			return bermudaExercise;
		}
		
		public BermudaExercise.BermudaExerciseBuilder getOrCreateBermudaExercise() {
			if (bermudaExercise!=null) {
				return bermudaExercise;
			}
			else return bermudaExercise = new BermudaExercise.BermudaExerciseBuilder();
		}
		
		
		public CalculationAgent.CalculationAgentBuilder getCalculationAgent() {
			return calculationAgent;
		}
		
		public CalculationAgent.CalculationAgentBuilder getOrCreateCalculationAgent() {
			if (calculationAgent!=null) {
				return calculationAgent;
			}
			else return calculationAgent = new CalculationAgent.CalculationAgentBuilder();
		}
		
		
		public OptionCashSettlement.OptionCashSettlementBuilder getCashSettlement() {
			return cashSettlement;
		}
		
		public OptionCashSettlement.OptionCashSettlementBuilder getOrCreateCashSettlement() {
			if (cashSettlement!=null) {
				return cashSettlement;
			}
			else return cashSettlement = new OptionCashSettlement.OptionCashSettlementBuilder();
		}
		
		
		public EuropeanExercise.EuropeanExerciseBuilder getEuropeanExercise() {
			return europeanExercise;
		}
		
		public EuropeanExercise.EuropeanExerciseBuilder getOrCreateEuropeanExercise() {
			if (europeanExercise!=null) {
				return europeanExercise;
			}
			else return europeanExercise = new EuropeanExercise.EuropeanExerciseBuilder();
		}
		
		
		public List<ExerciseNotice.ExerciseNoticeBuilder> getExerciseNotice() {
			return exerciseNotice;
		}
		
		public ExerciseNotice.ExerciseNoticeBuilder getOrCreateExerciseNotice(int index) {
			if (exerciseNotice==null) {
				this.exerciseNotice = new ArrayList<>();
			}
			return getIndex(exerciseNotice, index, ()->new ExerciseNotice.ExerciseNoticeBuilder());
		}
		
		
		public Boolean getFollowUpConfirmation() {
			return followUpConfirmation;
		}
		
		
		public OptionalEarlyTerminationAdjustedDates.OptionalEarlyTerminationAdjustedDatesBuilder getOptionalEarlyTerminationAdjustedDates() {
			return optionalEarlyTerminationAdjustedDates;
		}
		
		public OptionalEarlyTerminationAdjustedDates.OptionalEarlyTerminationAdjustedDatesBuilder getOrCreateOptionalEarlyTerminationAdjustedDates() {
			if (optionalEarlyTerminationAdjustedDates!=null) {
				return optionalEarlyTerminationAdjustedDates;
			}
			else return optionalEarlyTerminationAdjustedDates = new OptionalEarlyTerminationAdjustedDates.OptionalEarlyTerminationAdjustedDatesBuilder();
		}
		
		
		public BuyerSeller.BuyerSellerBuilder getSinglePartyOption() {
			return singlePartyOption;
		}
		
		public BuyerSeller.BuyerSellerBuilder getOrCreateSinglePartyOption() {
			if (singlePartyOption!=null) {
				return singlePartyOption;
			}
			else return singlePartyOption = new BuyerSeller.BuyerSellerBuilder();
		}
		
	
		public OptionalEarlyTerminationBuilder setAmericanExercise(AmericanExercise americanExercise) {
			this.americanExercise = americanExercise.toBuilder();
			return this;
		}
		public OptionalEarlyTerminationBuilder setAmericanExerciseBuilder(AmericanExercise.AmericanExerciseBuilder americanExercise) {
			this.americanExercise = americanExercise;
			return this;
		}
		public OptionalEarlyTerminationBuilder setBermudaExercise(BermudaExercise bermudaExercise) {
			this.bermudaExercise = bermudaExercise.toBuilder();
			return this;
		}
		public OptionalEarlyTerminationBuilder setBermudaExerciseBuilder(BermudaExercise.BermudaExerciseBuilder bermudaExercise) {
			this.bermudaExercise = bermudaExercise;
			return this;
		}
		public OptionalEarlyTerminationBuilder setCalculationAgent(CalculationAgent calculationAgent) {
			this.calculationAgent = calculationAgent.toBuilder();
			return this;
		}
		public OptionalEarlyTerminationBuilder setCalculationAgentBuilder(CalculationAgent.CalculationAgentBuilder calculationAgent) {
			this.calculationAgent = calculationAgent;
			return this;
		}
		public OptionalEarlyTerminationBuilder setCashSettlement(OptionCashSettlement cashSettlement) {
			this.cashSettlement = cashSettlement.toBuilder();
			return this;
		}
		public OptionalEarlyTerminationBuilder setCashSettlementBuilder(OptionCashSettlement.OptionCashSettlementBuilder cashSettlement) {
			this.cashSettlement = cashSettlement;
			return this;
		}
		public OptionalEarlyTerminationBuilder setEuropeanExercise(EuropeanExercise europeanExercise) {
			this.europeanExercise = europeanExercise.toBuilder();
			return this;
		}
		public OptionalEarlyTerminationBuilder setEuropeanExerciseBuilder(EuropeanExercise.EuropeanExerciseBuilder europeanExercise) {
			this.europeanExercise = europeanExercise;
			return this;
		}
		public OptionalEarlyTerminationBuilder addExerciseNotice(ExerciseNotice exerciseNotice) {
			if(this.exerciseNotice == null){
				this.exerciseNotice = new ArrayList<>();
			}
			this.exerciseNotice.add(exerciseNotice.toBuilder());
			return this;
		}
		public OptionalEarlyTerminationBuilder addExerciseNotice(ExerciseNotice exerciseNotice, int _idx) {
			if(this.exerciseNotice == null){
				this.exerciseNotice = new ArrayList<>();
			}
			getIndex(this.exerciseNotice, _idx, () -> exerciseNotice.toBuilder());
			this.exerciseNotice.set(_idx, exerciseNotice.toBuilder());
			return this;
		}
		public OptionalEarlyTerminationBuilder addExerciseNotice(List<ExerciseNotice> exerciseNotices) {
			if(this.exerciseNotice == null){
				this.exerciseNotice = new ArrayList<>();
			}
			for (ExerciseNotice toAdd : exerciseNotices) {
				this.exerciseNotice.add(toAdd.toBuilder());
			}
			return this;
		}
		public OptionalEarlyTerminationBuilder addExerciseNoticeBuilder(ExerciseNotice.ExerciseNoticeBuilder exerciseNotice) {
			if(this.exerciseNotice == null){
				this.exerciseNotice = new ArrayList<>();
			}
			this.exerciseNotice.add(exerciseNotice);
			return this;
		}
		
		public OptionalEarlyTerminationBuilder clearExerciseNotice() {
			this.exerciseNotice = null;
			return this;
		}
		public OptionalEarlyTerminationBuilder setFollowUpConfirmation(Boolean followUpConfirmation) {
			this.followUpConfirmation = followUpConfirmation;
			return this;
		}
		public OptionalEarlyTerminationBuilder setOptionalEarlyTerminationAdjustedDates(OptionalEarlyTerminationAdjustedDates optionalEarlyTerminationAdjustedDates) {
			this.optionalEarlyTerminationAdjustedDates = optionalEarlyTerminationAdjustedDates.toBuilder();
			return this;
		}
		public OptionalEarlyTerminationBuilder setOptionalEarlyTerminationAdjustedDatesBuilder(OptionalEarlyTerminationAdjustedDates.OptionalEarlyTerminationAdjustedDatesBuilder optionalEarlyTerminationAdjustedDates) {
			this.optionalEarlyTerminationAdjustedDates = optionalEarlyTerminationAdjustedDates;
			return this;
		}
		public OptionalEarlyTerminationBuilder setSinglePartyOption(BuyerSeller singlePartyOption) {
			this.singlePartyOption = singlePartyOption.toBuilder();
			return this;
		}
		public OptionalEarlyTerminationBuilder setSinglePartyOptionBuilder(BuyerSeller.BuyerSellerBuilder singlePartyOption) {
			this.singlePartyOption = singlePartyOption;
			return this;
		}
		
		public OptionalEarlyTermination build() {
			return new OptionalEarlyTermination(this);
		}
	
		@Override
		public OptionalEarlyTerminationBuilder prune() {
			if (americanExercise!=null && !americanExercise.prune().hasData()) americanExercise = null;
			if (bermudaExercise!=null && !bermudaExercise.prune().hasData()) bermudaExercise = null;
			if (calculationAgent!=null && !calculationAgent.prune().hasData()) calculationAgent = null;
			if (cashSettlement!=null && !cashSettlement.prune().hasData()) cashSettlement = null;
			if (europeanExercise!=null && !europeanExercise.prune().hasData()) europeanExercise = null;
			if (exerciseNotice!=null) exerciseNotice = exerciseNotice.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (optionalEarlyTerminationAdjustedDates!=null && !optionalEarlyTerminationAdjustedDates.prune().hasData()) optionalEarlyTerminationAdjustedDates = null;
			if (singlePartyOption!=null && !singlePartyOption.prune().hasData()) singlePartyOption = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getAmericanExercise()!=null && getAmericanExercise().hasData()) return true;
			if (getBermudaExercise()!=null && getBermudaExercise().hasData()) return true;
			if (getCalculationAgent()!=null && getCalculationAgent().hasData()) return true;
			if (getCashSettlement()!=null && getCashSettlement().hasData()) return true;
			if (getEuropeanExercise()!=null && getEuropeanExercise().hasData()) return true;
			if (getExerciseNotice()!=null && getExerciseNotice().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getFollowUpConfirmation()!=null) return true;
			if (getOptionalEarlyTerminationAdjustedDates()!=null && getOptionalEarlyTerminationAdjustedDates().hasData()) return true;
			if (getSinglePartyOption()!=null && getSinglePartyOption().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("followUpConfirmation"), Boolean.class, followUpConfirmation, this);
			
			processRosetta(path.newSubPath("americanExercise"), processor, AmericanExercise.class, americanExercise);
			processRosetta(path.newSubPath("bermudaExercise"), processor, BermudaExercise.class, bermudaExercise);
			processRosetta(path.newSubPath("calculationAgent"), processor, CalculationAgent.class, calculationAgent);
			processRosetta(path.newSubPath("cashSettlement"), processor, OptionCashSettlement.class, cashSettlement);
			processRosetta(path.newSubPath("europeanExercise"), processor, EuropeanExercise.class, europeanExercise);
			processRosetta(path.newSubPath("exerciseNotice"), processor, ExerciseNotice.class, exerciseNotice);
			processRosetta(path.newSubPath("optionalEarlyTerminationAdjustedDates"), processor, OptionalEarlyTerminationAdjustedDates.class, optionalEarlyTerminationAdjustedDates);
			processRosetta(path.newSubPath("singlePartyOption"), processor, BuyerSeller.class, singlePartyOption);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			OptionalEarlyTerminationBuilder _that = (OptionalEarlyTerminationBuilder) o;
		
			if (americanExercise != null ? !americanExercise.equals(_that.americanExercise) : _that.americanExercise != null) return false;
			if (bermudaExercise != null ? !bermudaExercise.equals(_that.bermudaExercise) : _that.bermudaExercise != null) return false;
			if (calculationAgent != null ? !calculationAgent.equals(_that.calculationAgent) : _that.calculationAgent != null) return false;
			if (cashSettlement != null ? !cashSettlement.equals(_that.cashSettlement) : _that.cashSettlement != null) return false;
			if (europeanExercise != null ? !europeanExercise.equals(_that.europeanExercise) : _that.europeanExercise != null) return false;
			if (!ListEquals.listEquals(exerciseNotice, _that.exerciseNotice)) return false;
			if (followUpConfirmation != null ? !followUpConfirmation.equals(_that.followUpConfirmation) : _that.followUpConfirmation != null) return false;
			if (optionalEarlyTerminationAdjustedDates != null ? !optionalEarlyTerminationAdjustedDates.equals(_that.optionalEarlyTerminationAdjustedDates) : _that.optionalEarlyTerminationAdjustedDates != null) return false;
			if (singlePartyOption != null ? !singlePartyOption.equals(_that.singlePartyOption) : _that.singlePartyOption != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (americanExercise != null ? americanExercise.hashCode() : 0);
			_result = 31 * _result + (bermudaExercise != null ? bermudaExercise.hashCode() : 0);
			_result = 31 * _result + (calculationAgent != null ? calculationAgent.hashCode() : 0);
			_result = 31 * _result + (cashSettlement != null ? cashSettlement.hashCode() : 0);
			_result = 31 * _result + (europeanExercise != null ? europeanExercise.hashCode() : 0);
			_result = 31 * _result + (exerciseNotice != null ? exerciseNotice.hashCode() : 0);
			_result = 31 * _result + (followUpConfirmation != null ? followUpConfirmation.hashCode() : 0);
			_result = 31 * _result + (optionalEarlyTerminationAdjustedDates != null ? optionalEarlyTerminationAdjustedDates.hashCode() : 0);
			_result = 31 * _result + (singlePartyOption != null ? singlePartyOption.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "OptionalEarlyTerminationBuilder {" +
				"americanExercise=" + this.americanExercise + ", " +
				"bermudaExercise=" + this.bermudaExercise + ", " +
				"calculationAgent=" + this.calculationAgent + ", " +
				"cashSettlement=" + this.cashSettlement + ", " +
				"europeanExercise=" + this.europeanExercise + ", " +
				"exerciseNotice=" + this.exerciseNotice + ", " +
				"followUpConfirmation=" + this.followUpConfirmation + ", " +
				"optionalEarlyTerminationAdjustedDates=" + this.optionalEarlyTerminationAdjustedDates + ", " +
				"singlePartyOption=" + this.singlePartyOption +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("followUpConfirmation"), Boolean.class, followUpConfirmation, this);
		
		processRosetta(path.newSubPath("americanExercise"), processor, AmericanExercise.class, americanExercise);
		processRosetta(path.newSubPath("bermudaExercise"), processor, BermudaExercise.class, bermudaExercise);
		processRosetta(path.newSubPath("calculationAgent"), processor, CalculationAgent.class, calculationAgent);
		processRosetta(path.newSubPath("cashSettlement"), processor, OptionCashSettlement.class, cashSettlement);
		processRosetta(path.newSubPath("europeanExercise"), processor, EuropeanExercise.class, europeanExercise);
		processRosetta(path.newSubPath("exerciseNotice"), processor, ExerciseNotice.class, exerciseNotice);
		processRosetta(path.newSubPath("optionalEarlyTerminationAdjustedDates"), processor, OptionalEarlyTerminationAdjustedDates.class, optionalEarlyTerminationAdjustedDates);
		processRosetta(path.newSubPath("singlePartyOption"), processor, BuyerSeller.class, singlePartyOption);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		OptionalEarlyTermination _that = (OptionalEarlyTermination) o;
	
		if (americanExercise != null ? !americanExercise.equals(_that.americanExercise) : _that.americanExercise != null) return false;
		if (bermudaExercise != null ? !bermudaExercise.equals(_that.bermudaExercise) : _that.bermudaExercise != null) return false;
		if (calculationAgent != null ? !calculationAgent.equals(_that.calculationAgent) : _that.calculationAgent != null) return false;
		if (cashSettlement != null ? !cashSettlement.equals(_that.cashSettlement) : _that.cashSettlement != null) return false;
		if (europeanExercise != null ? !europeanExercise.equals(_that.europeanExercise) : _that.europeanExercise != null) return false;
		if (!ListEquals.listEquals(exerciseNotice, _that.exerciseNotice)) return false;
		if (followUpConfirmation != null ? !followUpConfirmation.equals(_that.followUpConfirmation) : _that.followUpConfirmation != null) return false;
		if (optionalEarlyTerminationAdjustedDates != null ? !optionalEarlyTerminationAdjustedDates.equals(_that.optionalEarlyTerminationAdjustedDates) : _that.optionalEarlyTerminationAdjustedDates != null) return false;
		if (singlePartyOption != null ? !singlePartyOption.equals(_that.singlePartyOption) : _that.singlePartyOption != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (americanExercise != null ? americanExercise.hashCode() : 0);
		_result = 31 * _result + (bermudaExercise != null ? bermudaExercise.hashCode() : 0);
		_result = 31 * _result + (calculationAgent != null ? calculationAgent.hashCode() : 0);
		_result = 31 * _result + (cashSettlement != null ? cashSettlement.hashCode() : 0);
		_result = 31 * _result + (europeanExercise != null ? europeanExercise.hashCode() : 0);
		_result = 31 * _result + (exerciseNotice != null ? exerciseNotice.hashCode() : 0);
		_result = 31 * _result + (followUpConfirmation != null ? followUpConfirmation.hashCode() : 0);
		_result = 31 * _result + (optionalEarlyTerminationAdjustedDates != null ? optionalEarlyTerminationAdjustedDates.hashCode() : 0);
		_result = 31 * _result + (singlePartyOption != null ? singlePartyOption.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "OptionalEarlyTermination {" +
			"americanExercise=" + this.americanExercise + ", " +
			"bermudaExercise=" + this.bermudaExercise + ", " +
			"calculationAgent=" + this.calculationAgent + ", " +
			"cashSettlement=" + this.cashSettlement + ", " +
			"europeanExercise=" + this.europeanExercise + ", " +
			"exerciseNotice=" + this.exerciseNotice + ", " +
			"followUpConfirmation=" + this.followUpConfirmation + ", " +
			"optionalEarlyTerminationAdjustedDates=" + this.optionalEarlyTerminationAdjustedDates + ", " +
			"singlePartyOption=" + this.singlePartyOption +
		'}';
	}
}
