package org.isda.cdm;

import java.lang.Boolean;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.OtherEligibleAndPostedSupportMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class to specify the Other Eligible Support elections associated with Japanese and New York Law Initial and Variation margin agreements. ISDA 2016 Japanese Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (p): Other Eligible Support (IM) and Other Posted Support (IM). | ISDA 2016 New York Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (p): Other Eligible Support (IM) and Other Posted Support (IM). | ISDA 2016 Credit Support Annex for Variation Margin, paragraph 13, (l): Other Eligible Support (VM) and Other Posted Support (VM).
 * 
 * @version 2.5.4
 */
@RosettaClass
public class OtherEligibleAndPostedSupport extends RosettaModelObject {
	private final Boolean applicableTransfer;
	private final Boolean applicableValue;
	private static OtherEligibleAndPostedSupportMeta metaData = new OtherEligibleAndPostedSupportMeta();
	
	OtherEligibleAndPostedSupport(OtherEligibleAndPostedSupportBuilder builder) {
		this.applicableTransfer = builder.getApplicableTransfer();
		this.applicableValue = builder.getApplicableValue();
	}
	
	/**
	 * ISDA 2016 Japanese Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (p)(ii): Transfer. | ISDA 2016 New York Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (p)(ii): Transfer.
	 */
	@RosettaSynonym(value="", source="ISDA_Create_1_0")
	public final Boolean getApplicableTransfer() {
		return applicableTransfer;
	}
	
	/**
	 * ISDA 2016 Japanese Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (p)(i): Value. | ISDA 2016 New York Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (p)(i): Value.
	 */
	@RosettaSynonym(value="", source="ISDA_Create_1_0")
	public final Boolean getApplicableValue() {
		return applicableValue;
	}
	
	
	@Override
	public RosettaMetaData<? extends OtherEligibleAndPostedSupport> metaData() {
		return metaData;
	} 
	
	public OtherEligibleAndPostedSupportBuilder toBuilder() {
		OtherEligibleAndPostedSupportBuilder builder = new OtherEligibleAndPostedSupportBuilder();
		ofNullable(getApplicableTransfer()).ifPresent(builder::setApplicableTransfer);
		ofNullable(getApplicableValue()).ifPresent(builder::setApplicableValue);
		return builder;
	}

	public static OtherEligibleAndPostedSupportBuilder builder() {
		return new OtherEligibleAndPostedSupportBuilder();
	}

	public static  class OtherEligibleAndPostedSupportBuilder extends RosettaModelObjectBuilder {
	
		protected Boolean applicableTransfer;
		protected Boolean applicableValue;
	
		public OtherEligibleAndPostedSupportBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends OtherEligibleAndPostedSupport> metaData() {
			return metaData;
		} 
	
		
		public Boolean getApplicableTransfer() {
			return applicableTransfer;
		}
		
		
		public Boolean getApplicableValue() {
			return applicableValue;
		}
		
	
		public OtherEligibleAndPostedSupportBuilder setApplicableTransfer(Boolean applicableTransfer) {
			this.applicableTransfer = applicableTransfer;
			return this;
		}
		public OtherEligibleAndPostedSupportBuilder setApplicableValue(Boolean applicableValue) {
			this.applicableValue = applicableValue;
			return this;
		}
		
		public OtherEligibleAndPostedSupport build() {
			return new OtherEligibleAndPostedSupport(this);
		}
	
		@Override
		public OtherEligibleAndPostedSupportBuilder prune() {
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getApplicableTransfer()!=null) return true;
			if (getApplicableValue()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("applicableTransfer"), Boolean.class, applicableTransfer, this);
			processor.processBasic(path.newSubPath("applicableValue"), Boolean.class, applicableValue, this);
			
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			OtherEligibleAndPostedSupportBuilder _that = (OtherEligibleAndPostedSupportBuilder) o;
		
			if (applicableTransfer != null ? !applicableTransfer.equals(_that.applicableTransfer) : _that.applicableTransfer != null) return false;
			if (applicableValue != null ? !applicableValue.equals(_that.applicableValue) : _that.applicableValue != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (applicableTransfer != null ? applicableTransfer.hashCode() : 0);
			_result = 31 * _result + (applicableValue != null ? applicableValue.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "OtherEligibleAndPostedSupportBuilder {" +
				"applicableTransfer=" + this.applicableTransfer + ", " +
				"applicableValue=" + this.applicableValue +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("applicableTransfer"), Boolean.class, applicableTransfer, this);
		processor.processBasic(path.newSubPath("applicableValue"), Boolean.class, applicableValue, this);
		
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		OtherEligibleAndPostedSupport _that = (OtherEligibleAndPostedSupport) o;
	
		if (applicableTransfer != null ? !applicableTransfer.equals(_that.applicableTransfer) : _that.applicableTransfer != null) return false;
		if (applicableValue != null ? !applicableValue.equals(_that.applicableValue) : _that.applicableValue != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (applicableTransfer != null ? applicableTransfer.hashCode() : 0);
		_result = 31 * _result + (applicableValue != null ? applicableValue.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "OtherEligibleAndPostedSupport {" +
			"applicableTransfer=" + this.applicableTransfer + ", " +
			"applicableValue=" + this.applicableValue +
		'}';
	}
}
