package org.isda.cdm;

import java.lang.Boolean;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.PCDeliverableObligationCharacMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class to specify the Partial Cash Deliverable Obligation Characteristic.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="PCDeliverableObligationCharac", source="FpML_5_10")
public class PCDeliverableObligationCharac extends RosettaModelObject {
	private final Boolean applicable;
	private final Boolean partialCashSettlement;
	private static PCDeliverableObligationCharacMeta metaData = new PCDeliverableObligationCharacMeta();
	
	PCDeliverableObligationCharac(PCDeliverableObligationCharacBuilder builder) {
		this.applicable = builder.getApplicable();
		this.partialCashSettlement = builder.getPartialCashSettlement();
	}
	
	/**
	 * Indicates whether the provision is applicable.
	 */
	@RosettaSynonym(value="applicable", source="FpML_5_10")
	@RosettaSynonym(value="applicable", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="applicable", source="DTCC_11_0")
	@RosettaSynonym(value="applicable", source="DTCC_9_0")
	@RosettaSynonym(value="applicable", source="CME_ClearedConfirm_1_17")
	public final Boolean getApplicable() {
		return applicable;
	}
	
	/**
	 * Specifies whether either &#39;Partial Cash Settlement of Assignable Loans&#39;, &#39;Partial Cash Settlement of Consent Required Loans&#39; or &#39;Partial Cash Settlement of Participations&#39; is applicable. If this element is specified and Assignable Loan is a Deliverable Obligation Characteristic, any Assignable Loan that is deliverable, but where a non-receipt of Consent by the Physical Settlement Date has occurred, the Loan can be cash settled rather than physically delivered. If this element is specified and Consent Required Loan is a Deliverable Obligation Characteristic, any Consent Required Loan that is deliverable, but where a non-receipt of Consent by the Physical Settlement Date has occurred, the Loan can be cash settled rather than physically delivered. If this element is specified and Direct Loan Participation is a Deliverable Obligation Characteristic, any Participation that is deliverable, but where this participation has not been effected (has not come into effect) by the Physical Settlement Date, the participation can be cash settled rather than physically delivered.
	 */
	@RosettaSynonym(value="partialCashSettlement", source="FpML_5_10")
	@RosettaSynonym(value="partialCashSettlement", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="partialCashSettlement", source="DTCC_11_0")
	@RosettaSynonym(value="partialCashSettlement", source="DTCC_9_0")
	@RosettaSynonym(value="partialCashSettlement", source="CME_ClearedConfirm_1_17")
	public final Boolean getPartialCashSettlement() {
		return partialCashSettlement;
	}
	
	
	@Override
	public RosettaMetaData<? extends PCDeliverableObligationCharac> metaData() {
		return metaData;
	} 
	
	public PCDeliverableObligationCharacBuilder toBuilder() {
		PCDeliverableObligationCharacBuilder builder = new PCDeliverableObligationCharacBuilder();
		ofNullable(getApplicable()).ifPresent(builder::setApplicable);
		ofNullable(getPartialCashSettlement()).ifPresent(builder::setPartialCashSettlement);
		return builder;
	}

	public static PCDeliverableObligationCharacBuilder builder() {
		return new PCDeliverableObligationCharacBuilder();
	}

	public static  class PCDeliverableObligationCharacBuilder extends RosettaModelObjectBuilder {
	
		protected Boolean applicable;
		protected Boolean partialCashSettlement;
	
		public PCDeliverableObligationCharacBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends PCDeliverableObligationCharac> metaData() {
			return metaData;
		} 
	
		
		public Boolean getApplicable() {
			return applicable;
		}
		
		
		public Boolean getPartialCashSettlement() {
			return partialCashSettlement;
		}
		
	
		public PCDeliverableObligationCharacBuilder setApplicable(Boolean applicable) {
			this.applicable = applicable;
			return this;
		}
		public PCDeliverableObligationCharacBuilder setPartialCashSettlement(Boolean partialCashSettlement) {
			this.partialCashSettlement = partialCashSettlement;
			return this;
		}
		
		public PCDeliverableObligationCharac build() {
			return new PCDeliverableObligationCharac(this);
		}
	
		@Override
		public PCDeliverableObligationCharacBuilder prune() {
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getApplicable()!=null) return true;
			if (getPartialCashSettlement()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("applicable"), Boolean.class, applicable, this);
			processor.processBasic(path.newSubPath("partialCashSettlement"), Boolean.class, partialCashSettlement, this);
			
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			PCDeliverableObligationCharacBuilder _that = (PCDeliverableObligationCharacBuilder) o;
		
			if (applicable != null ? !applicable.equals(_that.applicable) : _that.applicable != null) return false;
			if (partialCashSettlement != null ? !partialCashSettlement.equals(_that.partialCashSettlement) : _that.partialCashSettlement != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (applicable != null ? applicable.hashCode() : 0);
			_result = 31 * _result + (partialCashSettlement != null ? partialCashSettlement.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "PCDeliverableObligationCharacBuilder {" +
				"applicable=" + this.applicable + ", " +
				"partialCashSettlement=" + this.partialCashSettlement +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("applicable"), Boolean.class, applicable, this);
		processor.processBasic(path.newSubPath("partialCashSettlement"), Boolean.class, partialCashSettlement, this);
		
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		PCDeliverableObligationCharac _that = (PCDeliverableObligationCharac) o;
	
		if (applicable != null ? !applicable.equals(_that.applicable) : _that.applicable != null) return false;
		if (partialCashSettlement != null ? !partialCashSettlement.equals(_that.partialCashSettlement) : _that.partialCashSettlement != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (applicable != null ? applicable.hashCode() : 0);
		_result = 31 * _result + (partialCashSettlement != null ? partialCashSettlement.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "PCDeliverableObligationCharac {" +
			"applicable=" + this.applicable + ", " +
			"partialCashSettlement=" + this.partialCashSettlement +
		'}';
	}
}
