package org.isda.cdm;

import java.lang.Integer;
import java.math.BigDecimal;
import org.isda.cdm.metafields.ReferenceWithMetaMoney;
import org.isda.cdm.metafields.ReferenceWithMetaMoney.ReferenceWithMetaMoneyBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.PartialExerciseMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.stream.Collectors;
		import static java.util.Optional.ofNullable;
		import java.math.BigDecimal;

/**
 * A class defining partial exercise. As defined in the 2000 ISDA Definitions, Section 12.3 Partial Exercise, the buyer of the option may exercise all or less than all the notional amount of the underlying swap but may not be less than the minimum notional amount (if specified) and must be an integral multiple of the integral multiple amount if specified.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="PartialExercise", source="FpML_5_10")
public class PartialExercise extends RosettaModelObject {
	private final BigDecimal integralMultipleAmount;
	private final BigDecimal minimumNotionalAmount;
	private final Integer minimumNumberOfOptions;
	private final ReferenceWithMetaMoney notionaReference;
	private static PartialExerciseMeta metaData = new PartialExerciseMeta();
	
	PartialExercise(PartialExerciseBuilder builder) {
		this.integralMultipleAmount = builder.getIntegralMultipleAmount();
		this.minimumNotionalAmount = builder.getMinimumNotionalAmount();
		this.minimumNumberOfOptions = builder.getMinimumNumberOfOptions();
		this.notionaReference = ofNullable(builder.getNotionaReference()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * A notional amount which restricts the amount of notional that can be exercised when partial exercise or multiple exercise is applicable. The integral multiple amount defines a lower limit of notional that can be exercised and also defines a unit multiple of notional that can be exercised, i.e. only integer multiples of this amount can be exercised.
	 */
	@RosettaSynonym(value="integralMultipleAmount", source="FpML_5_10")
	@RosettaSynonym(value="integralMultipleAmount", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="integralMultipleAmount", source="DTCC_11_0")
	@RosettaSynonym(value="integralMultipleAmount", source="DTCC_9_0")
	@RosettaSynonym(value="integralMultipleAmount", source="CME_ClearedConfirm_1_17")
	public final BigDecimal getIntegralMultipleAmount() {
		return integralMultipleAmount;
	}
	
	/**
	 * The minimum notional amount that can be exercised on a given exercise date. See multipleExercise.
	 */
	@RosettaSynonym(value="minimumNotionalAmount", source="FpML_5_10")
	@RosettaSynonym(value="minimumNotionalAmount", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="minimumNotionalAmount", source="DTCC_11_0")
	@RosettaSynonym(value="minimumNotionalAmount", source="DTCC_9_0")
	@RosettaSynonym(value="minimumNotionalAmount", source="CME_ClearedConfirm_1_17")
	public final BigDecimal getMinimumNotionalAmount() {
		return minimumNotionalAmount;
	}
	
	/**
	 * The minimum number of options that can be exercised on a given exercise date.
	 */
	@RosettaSynonym(value="minimumNumberOfOptions", source="FpML_5_10")
	@RosettaSynonym(value="minimumNumberOfOptions", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="minimumNumberOfOptions", source="DTCC_11_0")
	@RosettaSynonym(value="minimumNumberOfOptions", source="DTCC_9_0")
	@RosettaSynonym(value="minimumNumberOfOptions", source="CME_ClearedConfirm_1_17")
	public final Integer getMinimumNumberOfOptions() {
		return minimumNumberOfOptions;
	}
	
	/**
	 * A pointer style reference to the associated notional schedule defined elsewhere in the document. This element has been made optional as part of its integration in the OptionBaseExtended, because not required for the options on securities.
	 */
	@RosettaSynonym(value="notionalReference", source="FpML_5_10")
	@RosettaSynonym(value="notionalReference", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="notionalReference", source="DTCC_11_0")
	@RosettaSynonym(value="notionalReference", source="DTCC_9_0")
	@RosettaSynonym(value="notionalReference", source="CME_ClearedConfirm_1_17")
	public final ReferenceWithMetaMoney getNotionaReference() {
		return notionaReference;
	}
	
	
	@Override
	public RosettaMetaData<? extends PartialExercise> metaData() {
		return metaData;
	} 
	
	public PartialExerciseBuilder toBuilder() {
		PartialExerciseBuilder builder = new PartialExerciseBuilder();
		ofNullable(getIntegralMultipleAmount()).ifPresent(builder::setIntegralMultipleAmount);
		ofNullable(getMinimumNotionalAmount()).ifPresent(builder::setMinimumNotionalAmount);
		ofNullable(getMinimumNumberOfOptions()).ifPresent(builder::setMinimumNumberOfOptions);
		ofNullable(getNotionaReference()).ifPresent(builder::setNotionaReference);
		return builder;
	}

	public static PartialExerciseBuilder builder() {
		return new PartialExerciseBuilder();
	}

	public static  class PartialExerciseBuilder extends RosettaModelObjectBuilder {
	
		protected BigDecimal integralMultipleAmount;
		protected BigDecimal minimumNotionalAmount;
		protected Integer minimumNumberOfOptions;
		protected ReferenceWithMetaMoneyBuilder notionaReference;
	
		public PartialExerciseBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends PartialExercise> metaData() {
			return metaData;
		} 
	
		
		public BigDecimal getIntegralMultipleAmount() {
			return integralMultipleAmount;
		}
		
		
		public BigDecimal getMinimumNotionalAmount() {
			return minimumNotionalAmount;
		}
		
		
		public Integer getMinimumNumberOfOptions() {
			return minimumNumberOfOptions;
		}
		
		
		public ReferenceWithMetaMoneyBuilder getNotionaReference() {
			return notionaReference;
		}
		
		public ReferenceWithMetaMoneyBuilder getOrCreateNotionaReference() {
			if (notionaReference!=null) {
				return notionaReference;
			}
			else return notionaReference = new ReferenceWithMetaMoneyBuilder();
		}
		
	
		public PartialExerciseBuilder setIntegralMultipleAmount(BigDecimal integralMultipleAmount) {
			this.integralMultipleAmount = integralMultipleAmount;
			return this;
		}
		public PartialExerciseBuilder setMinimumNotionalAmount(BigDecimal minimumNotionalAmount) {
			this.minimumNotionalAmount = minimumNotionalAmount;
			return this;
		}
		public PartialExerciseBuilder setMinimumNumberOfOptions(Integer minimumNumberOfOptions) {
			this.minimumNumberOfOptions = minimumNumberOfOptions;
			return this;
		}
		public PartialExerciseBuilder setNotionaReference(ReferenceWithMetaMoney notionaReference) {
			this.notionaReference = notionaReference.toBuilder();
			return this;
		}
		public PartialExerciseBuilder setNotionaReferenceBuilder(ReferenceWithMetaMoneyBuilder notionaReference) {
			this.notionaReference = notionaReference;
			return this;
		}
		public PartialExerciseBuilder setNotionaReferenceRef(Money.MoneyBuilder notionaReference) {
			return setNotionaReference(ReferenceWithMetaMoney.builder().setValueBuilder(notionaReference).build());
		}
		public PartialExerciseBuilder setNotionaReferenceRef(Money notionaReference) {
			return setNotionaReferenceRef(Money.builder());
		}
		
		public PartialExercise build() {
			return new PartialExercise(this);
		}
	
		@Override
		public PartialExerciseBuilder prune() {
			if (notionaReference!=null && !notionaReference.prune().hasData()) notionaReference = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getIntegralMultipleAmount()!=null) return true;
			if (getMinimumNotionalAmount()!=null) return true;
			if (getMinimumNumberOfOptions()!=null) return true;
			if (getNotionaReference()!=null && getNotionaReference().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("integralMultipleAmount"), BigDecimal.class, integralMultipleAmount, this);
			processor.processBasic(path.newSubPath("minimumNotionalAmount"), BigDecimal.class, minimumNotionalAmount, this);
			processor.processBasic(path.newSubPath("minimumNumberOfOptions"), Integer.class, minimumNumberOfOptions, this);
			
			processRosetta(path.newSubPath("notionaReference"), processor, ReferenceWithMetaMoney.class, notionaReference);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			PartialExerciseBuilder _that = (PartialExerciseBuilder) o;
		
			if (integralMultipleAmount != null ? !integralMultipleAmount.equals(_that.integralMultipleAmount) : _that.integralMultipleAmount != null) return false;
			if (minimumNotionalAmount != null ? !minimumNotionalAmount.equals(_that.minimumNotionalAmount) : _that.minimumNotionalAmount != null) return false;
			if (minimumNumberOfOptions != null ? !minimumNumberOfOptions.equals(_that.minimumNumberOfOptions) : _that.minimumNumberOfOptions != null) return false;
			if (notionaReference != null ? !notionaReference.equals(_that.notionaReference) : _that.notionaReference != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (integralMultipleAmount != null ? integralMultipleAmount.hashCode() : 0);
			_result = 31 * _result + (minimumNotionalAmount != null ? minimumNotionalAmount.hashCode() : 0);
			_result = 31 * _result + (minimumNumberOfOptions != null ? minimumNumberOfOptions.hashCode() : 0);
			_result = 31 * _result + (notionaReference != null ? notionaReference.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "PartialExerciseBuilder {" +
				"integralMultipleAmount=" + this.integralMultipleAmount + ", " +
				"minimumNotionalAmount=" + this.minimumNotionalAmount + ", " +
				"minimumNumberOfOptions=" + this.minimumNumberOfOptions + ", " +
				"notionaReference=" + this.notionaReference +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("integralMultipleAmount"), BigDecimal.class, integralMultipleAmount, this);
		processor.processBasic(path.newSubPath("minimumNotionalAmount"), BigDecimal.class, minimumNotionalAmount, this);
		processor.processBasic(path.newSubPath("minimumNumberOfOptions"), Integer.class, minimumNumberOfOptions, this);
		
		processRosetta(path.newSubPath("notionaReference"), processor, ReferenceWithMetaMoney.class, notionaReference);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		PartialExercise _that = (PartialExercise) o;
	
		if (integralMultipleAmount != null ? !integralMultipleAmount.equals(_that.integralMultipleAmount) : _that.integralMultipleAmount != null) return false;
		if (minimumNotionalAmount != null ? !minimumNotionalAmount.equals(_that.minimumNotionalAmount) : _that.minimumNotionalAmount != null) return false;
		if (minimumNumberOfOptions != null ? !minimumNumberOfOptions.equals(_that.minimumNumberOfOptions) : _that.minimumNumberOfOptions != null) return false;
		if (notionaReference != null ? !notionaReference.equals(_that.notionaReference) : _that.notionaReference != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (integralMultipleAmount != null ? integralMultipleAmount.hashCode() : 0);
		_result = 31 * _result + (minimumNotionalAmount != null ? minimumNotionalAmount.hashCode() : 0);
		_result = 31 * _result + (minimumNumberOfOptions != null ? minimumNumberOfOptions.hashCode() : 0);
		_result = 31 * _result + (notionaReference != null ? notionaReference.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "PartialExercise {" +
			"integralMultipleAmount=" + this.integralMultipleAmount + ", " +
			"minimumNotionalAmount=" + this.minimumNotionalAmount + ", " +
			"minimumNumberOfOptions=" + this.minimumNumberOfOptions + ", " +
			"notionaReference=" + this.notionaReference +
		'}';
	}
}
