package org.isda.cdm;

import java.lang.String;
import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.BusinessUnit;
import org.isda.cdm.ContactInformation;
import org.isda.cdm.NaturalPerson;
import org.isda.cdm.metafields.ReferenceWithMetaParty;
import org.isda.cdm.metafields.ReferenceWithMetaParty.ReferenceWithMetaPartyBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.PartyContactInformationMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.stream.Collectors;
		import static java.util.Optional.ofNullable;
		import java.util.List;
		import java.util.ArrayList;
		import java.util.Objects;

/**
 * A class to specify contact information within a party: address and, optionally, associated business unit and person. This class also supports the ISDA CSA representation as a single string, through the address attribute.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="PartyContactInformation", source="FpML_5_10")
public class PartyContactInformation extends RosettaModelObject {
	private final String address;
	private final List<BusinessUnit> businessUnit;
	private final ContactInformation contactInformation;
	private final ReferenceWithMetaParty partyReference;
	private final List<NaturalPerson> person;
	private static PartyContactInformationMeta metaData = new PartyContactInformationMeta();
	
	PartyContactInformation(PartyContactInformationBuilder builder) {
		this.address = builder.getAddress();
		this.businessUnit = ofNullable(builder.getBusinessUnit()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.contactInformation = ofNullable(builder.getContactInformation()).map(f->f.build()).orElse(null);
		this.partyReference = ofNullable(builder.getPartyReference()).map(f->f.build()).orElse(null);
		this.person = ofNullable(builder.getPerson()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
	}
	
	/**
	 * The address specified as a string to support non-normalized contact information, such as in the case of ISDA Create.
	 */
	@RosettaSynonym(value="partyA_specify", source="ISDA_Create_1_0")
	@RosettaSynonym(value="partyB_specify", source="ISDA_Create_1_0")
	public final String getAddress() {
		return address;
	}
	
	/**
	 * Optional organization unit information used to describe the organization units (e.g. trading desks) involved in a transaction or business process, incl. the contact information (when relevant).
	 */
	@RosettaSynonym(value="businessUnit", source="FpML_5_10")
	public final List<BusinessUnit> getBusinessUnit() {
		return businessUnit;
	}
	
	/**
	 * The postal/street address, telephone number, email address and/or web page. If the contact information is specific to the associated business unit(s), it should be associated with those.
	 */
	@RosettaSynonym(value="contactInfo", source="FpML_5_10")
	public final ContactInformation getContactInformation() {
		return contactInformation;
	}
	
	/**
	 * The reference to the party to which the contact information refers to.
	 */
	@RosettaSynonym(value="partyReference", source="FpML_5_10")
	public final ReferenceWithMetaParty getPartyReference() {
		return partyReference;
	}
	
	/**
	 * Optional information about people involved in a transaction or business process. (These are employees of the party.)
	 */
	@RosettaSynonym(value="person", source="FpML_5_10")
	public final List<NaturalPerson> getPerson() {
		return person;
	}
	
	
	@Override
	public RosettaMetaData<? extends PartyContactInformation> metaData() {
		return metaData;
	} 
	
	public PartyContactInformationBuilder toBuilder() {
		PartyContactInformationBuilder builder = new PartyContactInformationBuilder();
		ofNullable(getAddress()).ifPresent(builder::setAddress);
		ofNullable(getBusinessUnit()).ifPresent(businessUnit -> businessUnit.forEach(builder::addBusinessUnit));
		ofNullable(getContactInformation()).ifPresent(builder::setContactInformation);
		ofNullable(getPartyReference()).ifPresent(builder::setPartyReference);
		ofNullable(getPerson()).ifPresent(person -> person.forEach(builder::addPerson));
		return builder;
	}

	public static PartyContactInformationBuilder builder() {
		return new PartyContactInformationBuilder();
	}

	public static  class PartyContactInformationBuilder extends RosettaModelObjectBuilder {
	
		protected String address;
		protected List<BusinessUnit.BusinessUnitBuilder> businessUnit;
		protected ContactInformation.ContactInformationBuilder contactInformation;
		protected ReferenceWithMetaPartyBuilder partyReference;
		protected List<NaturalPerson.NaturalPersonBuilder> person;
	
		public PartyContactInformationBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends PartyContactInformation> metaData() {
			return metaData;
		} 
	
		
		public String getAddress() {
			return address;
		}
		
		
		public List<BusinessUnit.BusinessUnitBuilder> getBusinessUnit() {
			return businessUnit;
		}
		
		public BusinessUnit.BusinessUnitBuilder getOrCreateBusinessUnit(int index) {
			if (businessUnit==null) {
				this.businessUnit = new ArrayList<>();
			}
			return getIndex(businessUnit, index, ()->new BusinessUnit.BusinessUnitBuilder());
		}
		
		
		public ContactInformation.ContactInformationBuilder getContactInformation() {
			return contactInformation;
		}
		
		public ContactInformation.ContactInformationBuilder getOrCreateContactInformation() {
			if (contactInformation!=null) {
				return contactInformation;
			}
			else return contactInformation = new ContactInformation.ContactInformationBuilder();
		}
		
		
		public ReferenceWithMetaPartyBuilder getPartyReference() {
			return partyReference;
		}
		
		public ReferenceWithMetaPartyBuilder getOrCreatePartyReference() {
			if (partyReference!=null) {
				return partyReference;
			}
			else return partyReference = new ReferenceWithMetaPartyBuilder();
		}
		
		
		public List<NaturalPerson.NaturalPersonBuilder> getPerson() {
			return person;
		}
		
		public NaturalPerson.NaturalPersonBuilder getOrCreatePerson(int index) {
			if (person==null) {
				this.person = new ArrayList<>();
			}
			return getIndex(person, index, ()->new NaturalPerson.NaturalPersonBuilder());
		}
		
	
		public PartyContactInformationBuilder setAddress(String address) {
			this.address = address;
			return this;
		}
		public PartyContactInformationBuilder addBusinessUnit(BusinessUnit businessUnit) {
			if(this.businessUnit == null){
				this.businessUnit = new ArrayList<>();
			}
			this.businessUnit.add(businessUnit.toBuilder());
			return this;
		}
		public PartyContactInformationBuilder addBusinessUnit(BusinessUnit businessUnit, int _idx) {
			if(this.businessUnit == null){
				this.businessUnit = new ArrayList<>();
			}
			getIndex(this.businessUnit, _idx, () -> businessUnit.toBuilder());
			this.businessUnit.set(_idx, businessUnit.toBuilder());
			return this;
		}
		public PartyContactInformationBuilder addBusinessUnit(List<BusinessUnit> businessUnits) {
			if(this.businessUnit == null){
				this.businessUnit = new ArrayList<>();
			}
			for (BusinessUnit toAdd : businessUnits) {
				this.businessUnit.add(toAdd.toBuilder());
			}
			return this;
		}
		public PartyContactInformationBuilder addBusinessUnitBuilder(BusinessUnit.BusinessUnitBuilder businessUnit) {
			if(this.businessUnit == null){
				this.businessUnit = new ArrayList<>();
			}
			this.businessUnit.add(businessUnit);
			return this;
		}
		
		public PartyContactInformationBuilder clearBusinessUnit() {
			this.businessUnit = null;
			return this;
		}
		public PartyContactInformationBuilder setContactInformation(ContactInformation contactInformation) {
			this.contactInformation = contactInformation.toBuilder();
			return this;
		}
		public PartyContactInformationBuilder setContactInformationBuilder(ContactInformation.ContactInformationBuilder contactInformation) {
			this.contactInformation = contactInformation;
			return this;
		}
		public PartyContactInformationBuilder setPartyReference(ReferenceWithMetaParty partyReference) {
			this.partyReference = partyReference.toBuilder();
			return this;
		}
		public PartyContactInformationBuilder setPartyReferenceBuilder(ReferenceWithMetaPartyBuilder partyReference) {
			this.partyReference = partyReference;
			return this;
		}
		public PartyContactInformationBuilder setPartyReferenceRef(Party.PartyBuilder partyReference) {
			return setPartyReference(ReferenceWithMetaParty.builder().setValueBuilder(partyReference).build());
		}
		public PartyContactInformationBuilder setPartyReferenceRef(Party partyReference) {
			return setPartyReferenceRef(Party.builder());
		}
		public PartyContactInformationBuilder addPerson(NaturalPerson person) {
			if(this.person == null){
				this.person = new ArrayList<>();
			}
			this.person.add(person.toBuilder());
			return this;
		}
		public PartyContactInformationBuilder addPerson(NaturalPerson person, int _idx) {
			if(this.person == null){
				this.person = new ArrayList<>();
			}
			getIndex(this.person, _idx, () -> person.toBuilder());
			this.person.set(_idx, person.toBuilder());
			return this;
		}
		public PartyContactInformationBuilder addPerson(List<NaturalPerson> persons) {
			if(this.person == null){
				this.person = new ArrayList<>();
			}
			for (NaturalPerson toAdd : persons) {
				this.person.add(toAdd.toBuilder());
			}
			return this;
		}
		public PartyContactInformationBuilder addPersonBuilder(NaturalPerson.NaturalPersonBuilder person) {
			if(this.person == null){
				this.person = new ArrayList<>();
			}
			this.person.add(person);
			return this;
		}
		
		public PartyContactInformationBuilder clearPerson() {
			this.person = null;
			return this;
		}
		
		public PartyContactInformation build() {
			return new PartyContactInformation(this);
		}
	
		@Override
		public PartyContactInformationBuilder prune() {
			if (businessUnit!=null) businessUnit = businessUnit.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (contactInformation!=null && !contactInformation.prune().hasData()) contactInformation = null;
			if (partyReference!=null && !partyReference.prune().hasData()) partyReference = null;
			if (person!=null) person = person.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getAddress()!=null) return true;
			if (getBusinessUnit()!=null && getBusinessUnit().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getContactInformation()!=null && getContactInformation().hasData()) return true;
			if (getPartyReference()!=null && getPartyReference().hasData()) return true;
			if (getPerson()!=null && getPerson().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("address"), String.class, address, this);
			
			processRosetta(path.newSubPath("businessUnit"), processor, BusinessUnit.class, businessUnit);
			processRosetta(path.newSubPath("contactInformation"), processor, ContactInformation.class, contactInformation);
			processRosetta(path.newSubPath("partyReference"), processor, ReferenceWithMetaParty.class, partyReference);
			processRosetta(path.newSubPath("person"), processor, NaturalPerson.class, person);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			PartyContactInformationBuilder _that = (PartyContactInformationBuilder) o;
		
			if (address != null ? !address.equals(_that.address) : _that.address != null) return false;
			if (!ListEquals.listEquals(businessUnit, _that.businessUnit)) return false;
			if (contactInformation != null ? !contactInformation.equals(_that.contactInformation) : _that.contactInformation != null) return false;
			if (partyReference != null ? !partyReference.equals(_that.partyReference) : _that.partyReference != null) return false;
			if (!ListEquals.listEquals(person, _that.person)) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (address != null ? address.hashCode() : 0);
			_result = 31 * _result + (businessUnit != null ? businessUnit.hashCode() : 0);
			_result = 31 * _result + (contactInformation != null ? contactInformation.hashCode() : 0);
			_result = 31 * _result + (partyReference != null ? partyReference.hashCode() : 0);
			_result = 31 * _result + (person != null ? person.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "PartyContactInformationBuilder {" +
				"address=" + this.address + ", " +
				"businessUnit=" + this.businessUnit + ", " +
				"contactInformation=" + this.contactInformation + ", " +
				"partyReference=" + this.partyReference + ", " +
				"person=" + this.person +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("address"), String.class, address, this);
		
		processRosetta(path.newSubPath("businessUnit"), processor, BusinessUnit.class, businessUnit);
		processRosetta(path.newSubPath("contactInformation"), processor, ContactInformation.class, contactInformation);
		processRosetta(path.newSubPath("partyReference"), processor, ReferenceWithMetaParty.class, partyReference);
		processRosetta(path.newSubPath("person"), processor, NaturalPerson.class, person);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		PartyContactInformation _that = (PartyContactInformation) o;
	
		if (address != null ? !address.equals(_that.address) : _that.address != null) return false;
		if (!ListEquals.listEquals(businessUnit, _that.businessUnit)) return false;
		if (contactInformation != null ? !contactInformation.equals(_that.contactInformation) : _that.contactInformation != null) return false;
		if (partyReference != null ? !partyReference.equals(_that.partyReference) : _that.partyReference != null) return false;
		if (!ListEquals.listEquals(person, _that.person)) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (address != null ? address.hashCode() : 0);
		_result = 31 * _result + (businessUnit != null ? businessUnit.hashCode() : 0);
		_result = 31 * _result + (contactInformation != null ? contactInformation.hashCode() : 0);
		_result = 31 * _result + (partyReference != null ? partyReference.hashCode() : 0);
		_result = 31 * _result + (person != null ? person.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "PartyContactInformation {" +
			"address=" + this.address + ", " +
			"businessUnit=" + this.businessUnit + ", " +
			"contactInformation=" + this.contactInformation + ", " +
			"partyReference=" + this.partyReference + ", " +
			"person=" + this.person +
		'}';
	}
}
