package org.isda.cdm;

import org.isda.cdm.PartyRoleEnum;
import org.isda.cdm.metafields.ReferenceWithMetaParty;
import org.isda.cdm.metafields.ReferenceWithMetaParty.ReferenceWithMetaPartyBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.PartyRoleMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.stream.Collectors;
		import static java.util.Optional.ofNullable;

/**
 * A class to specify the role(s) that party(ies) may have in relation to the execution, contract or other legal agreement.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class PartyRole extends RosettaModelObject {
	private final ReferenceWithMetaParty ownershipPartyReference;
	private final ReferenceWithMetaParty partyReference;
	private final PartyRoleEnum role;
	private static PartyRoleMeta metaData = new PartyRoleMeta();
	
	PartyRole(PartyRoleBuilder builder) {
		this.ownershipPartyReference = ofNullable(builder.getOwnershipPartyReference()).map(f->f.build()).orElse(null);
		this.partyReference = ofNullable(builder.getPartyReference()).map(f->f.build()).orElse(null);
		this.role = builder.getRole();
	}
	
	/**
	 * A reference to the party that has ownership of this party role information. FpML specifies that For shared trade information, this attribute will reference the originator of the data (for example, an execution facility or clearing house).
	 */
	@RosettaSynonym(value="partyReference", source="FpML_5_10", path="tradeHeader.partyTradeInformation")
	@RosettaSynonym(value="partyReference", source="CME_SubmissionIRS_1_0", path="tradeHeader.partyTradeInformation")
	@RosettaSynonym(value="partyReference", source="DTCC_11_0", path="tradeHeader.partyTradeInformation")
	@RosettaSynonym(value="partyReference", source="DTCC_9_0", path="tradeHeader.partyTradeInformation")
	public final ReferenceWithMetaParty getOwnershipPartyReference() {
		return ownershipPartyReference;
	}
	
	/**
	 * A reference to the party to which the role refers to.
	 */
	@RosettaSynonym(value="determiningParty", source="FpML_5_10", path="trade")
	@RosettaSynonym(value="determiningParty", source="CME_SubmissionIRS_1_0", path="trade")
	@RosettaSynonym(value="determiningParty", source="DTCC_11_0", path="trade")
	@RosettaSynonym(value="determiningParty", source="DTCC_9_0", path="trade")
	@RosettaSynonym(value="determiningParty", source="CME_ClearedConfirm_1_17", path="trade")
	@RosettaSynonym(value="barrierDeterminationAgent", source="FpML_5_10", path="trade")
	@RosettaSynonym(value="barrierDeterminationAgent", source="CME_SubmissionIRS_1_0", path="trade")
	@RosettaSynonym(value="barrierDeterminationAgent", source="DTCC_11_0", path="trade")
	@RosettaSynonym(value="barrierDeterminationAgent", source="DTCC_9_0", path="trade")
	@RosettaSynonym(value="barrierDeterminationAgent", source="CME_ClearedConfirm_1_17", path="trade")
	@RosettaSynonym(value="hedgingParty", source="FpML_5_10", path="trade")
	@RosettaSynonym(value="hedgingParty", source="CME_SubmissionIRS_1_0", path="trade")
	@RosettaSynonym(value="hedgingParty", source="DTCC_11_0", path="trade")
	@RosettaSynonym(value="hedgingParty", source="DTCC_9_0", path="trade")
	@RosettaSynonym(value="hedgingParty", source="CME_ClearedConfirm_1_17", path="trade")
	@RosettaSynonym(value="brokerPartyReference", source="FpML_5_10", path="trade")
	@RosettaSynonym(value="brokerPartyReference", source="CME_SubmissionIRS_1_0", path="trade")
	@RosettaSynonym(value="brokerPartyReference", source="DTCC_11_0", path="trade")
	@RosettaSynonym(value="brokerPartyReference", source="DTCC_9_0", path="trade")
	@RosettaSynonym(value="brokerPartyReference", source="CME_ClearedConfirm_1_17", path="trade")
	@RosettaSynonym(value="partyReference", source="FpML_5_10", path="trade.tradeHeader.partyTradeIdentifier.relatedParty")
	@RosettaSynonym(value="partyReference", source="CME_SubmissionIRS_1_0", path="trade.tradeHeader.partyTradeIdentifier.relatedParty")
	@RosettaSynonym(value="partyReference", source="DTCC_11_0", path="trade.tradeHeader.partyTradeIdentifier.relatedParty")
	@RosettaSynonym(value="partyReference", source="DTCC_9_0", path="trade.tradeHeader.partyTradeIdentifier.relatedParty")
	@RosettaSynonym(value="partyReference", source="CME_ClearedConfirm_1_17", path="trade.tradeHeader.partyTradeIdentifier.relatedParty")
	@RosettaSynonym(value="partyReference", source="FpML_5_10", path="tradeHeader.partyTradeInformation.relatedParty")
	@RosettaSynonym(value="partyReference", source="CME_SubmissionIRS_1_0", path="tradeHeader.partyTradeInformation.relatedParty")
	@RosettaSynonym(value="partyReference", source="DTCC_11_0", path="tradeHeader.partyTradeInformation.relatedParty")
	@RosettaSynonym(value="partyReference", source="DTCC_9_0", path="tradeHeader.partyTradeInformation.relatedParty")
	@RosettaSynonym(value="partyA", source="ISDA_Create_1_0")
	public final ReferenceWithMetaParty getPartyReference() {
		return partyReference;
	}
	
	/**
	 * The party role.
	 */
	@RosettaSynonym(value="", source="FpML_5_10")
	@RosettaSynonym(value="", source="DTCC_11_0")
	@RosettaSynonym(value="", source="DTCC_9_0")
	@RosettaSynonym(value="", source="FpML_5_10")
	@RosettaSynonym(value="", source="DTCC_11_0")
	@RosettaSynonym(value="", source="DTCC_9_0")
	@RosettaSynonym(value="", source="FpML_5_10")
	@RosettaSynonym(value="", source="DTCC_11_0")
	@RosettaSynonym(value="", source="DTCC_9_0")
	@RosettaSynonym(value="", source="FpML_5_10")
	@RosettaSynonym(value="", source="DTCC_11_0")
	@RosettaSynonym(value="", source="DTCC_9_0")
	@RosettaSynonym(value="role", source="CME_ClearedConfirm_1_17", path="trade.tradeHeader.partyTradeIdentifier.relatedParty")
	@RosettaSynonym(value="role", source="FpML_5_10", path="tradeHeader.partyTradeInformation.relatedParty")
	@RosettaSynonym(value="role", source="DTCC_11_0", path="tradeHeader.partyTradeInformation.relatedParty")
	@RosettaSynonym(value="role", source="DTCC_9_0", path="tradeHeader.partyTradeInformation.relatedParty")
	public final PartyRoleEnum getRole() {
		return role;
	}
	
	
	@Override
	public RosettaMetaData<? extends PartyRole> metaData() {
		return metaData;
	} 
	
	public PartyRoleBuilder toBuilder() {
		PartyRoleBuilder builder = new PartyRoleBuilder();
		ofNullable(getOwnershipPartyReference()).ifPresent(builder::setOwnershipPartyReference);
		ofNullable(getPartyReference()).ifPresent(builder::setPartyReference);
		ofNullable(getRole()).ifPresent(builder::setRole);
		return builder;
	}

	public static PartyRoleBuilder builder() {
		return new PartyRoleBuilder();
	}

	public static  class PartyRoleBuilder extends RosettaModelObjectBuilder {
	
		protected ReferenceWithMetaPartyBuilder ownershipPartyReference;
		protected ReferenceWithMetaPartyBuilder partyReference;
		protected PartyRoleEnum role;
	
		public PartyRoleBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends PartyRole> metaData() {
			return metaData;
		} 
	
		
		public ReferenceWithMetaPartyBuilder getOwnershipPartyReference() {
			return ownershipPartyReference;
		}
		
		public ReferenceWithMetaPartyBuilder getOrCreateOwnershipPartyReference() {
			if (ownershipPartyReference!=null) {
				return ownershipPartyReference;
			}
			else return ownershipPartyReference = new ReferenceWithMetaPartyBuilder();
		}
		
		
		public ReferenceWithMetaPartyBuilder getPartyReference() {
			return partyReference;
		}
		
		public ReferenceWithMetaPartyBuilder getOrCreatePartyReference() {
			if (partyReference!=null) {
				return partyReference;
			}
			else return partyReference = new ReferenceWithMetaPartyBuilder();
		}
		
		
		public PartyRoleEnum getRole() {
			return role;
		}
		
	
		public PartyRoleBuilder setOwnershipPartyReference(ReferenceWithMetaParty ownershipPartyReference) {
			this.ownershipPartyReference = ownershipPartyReference.toBuilder();
			return this;
		}
		public PartyRoleBuilder setOwnershipPartyReferenceBuilder(ReferenceWithMetaPartyBuilder ownershipPartyReference) {
			this.ownershipPartyReference = ownershipPartyReference;
			return this;
		}
		public PartyRoleBuilder setOwnershipPartyReferenceRef(Party.PartyBuilder ownershipPartyReference) {
			return setOwnershipPartyReference(ReferenceWithMetaParty.builder().setValueBuilder(ownershipPartyReference).build());
		}
		public PartyRoleBuilder setOwnershipPartyReferenceRef(Party ownershipPartyReference) {
			return setOwnershipPartyReferenceRef(Party.builder());
		}
		public PartyRoleBuilder setPartyReference(ReferenceWithMetaParty partyReference) {
			this.partyReference = partyReference.toBuilder();
			return this;
		}
		public PartyRoleBuilder setPartyReferenceBuilder(ReferenceWithMetaPartyBuilder partyReference) {
			this.partyReference = partyReference;
			return this;
		}
		public PartyRoleBuilder setPartyReferenceRef(Party.PartyBuilder partyReference) {
			return setPartyReference(ReferenceWithMetaParty.builder().setValueBuilder(partyReference).build());
		}
		public PartyRoleBuilder setPartyReferenceRef(Party partyReference) {
			return setPartyReferenceRef(Party.builder());
		}
		public PartyRoleBuilder setRole(PartyRoleEnum role) {
			this.role = role;
			return this;
		}
		
		public PartyRole build() {
			return new PartyRole(this);
		}
	
		@Override
		public PartyRoleBuilder prune() {
			if (ownershipPartyReference!=null && !ownershipPartyReference.prune().hasData()) ownershipPartyReference = null;
			if (partyReference!=null && !partyReference.prune().hasData()) partyReference = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getOwnershipPartyReference()!=null && getOwnershipPartyReference().hasData()) return true;
			if (getPartyReference()!=null && getPartyReference().hasData()) return true;
			if (getRole()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("role"), PartyRoleEnum.class, role, this);
			
			processRosetta(path.newSubPath("ownershipPartyReference"), processor, ReferenceWithMetaParty.class, ownershipPartyReference);
			processRosetta(path.newSubPath("partyReference"), processor, ReferenceWithMetaParty.class, partyReference);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			PartyRoleBuilder _that = (PartyRoleBuilder) o;
		
			if (ownershipPartyReference != null ? !ownershipPartyReference.equals(_that.ownershipPartyReference) : _that.ownershipPartyReference != null) return false;
			if (partyReference != null ? !partyReference.equals(_that.partyReference) : _that.partyReference != null) return false;
			if (role != null ? !role.equals(_that.role) : _that.role != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (ownershipPartyReference != null ? ownershipPartyReference.hashCode() : 0);
			_result = 31 * _result + (partyReference != null ? partyReference.hashCode() : 0);
			_result = 31 * _result + (role != null ? role.getClass().getName().hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "PartyRoleBuilder {" +
				"ownershipPartyReference=" + this.ownershipPartyReference + ", " +
				"partyReference=" + this.partyReference + ", " +
				"role=" + this.role +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("role"), PartyRoleEnum.class, role, this);
		
		processRosetta(path.newSubPath("ownershipPartyReference"), processor, ReferenceWithMetaParty.class, ownershipPartyReference);
		processRosetta(path.newSubPath("partyReference"), processor, ReferenceWithMetaParty.class, partyReference);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		PartyRole _that = (PartyRole) o;
	
		if (ownershipPartyReference != null ? !ownershipPartyReference.equals(_that.ownershipPartyReference) : _that.ownershipPartyReference != null) return false;
		if (partyReference != null ? !partyReference.equals(_that.partyReference) : _that.partyReference != null) return false;
		if (role != null ? !role.equals(_that.role) : _that.role != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (ownershipPartyReference != null ? ownershipPartyReference.hashCode() : 0);
		_result = 31 * _result + (partyReference != null ? partyReference.hashCode() : 0);
		_result = 31 * _result + (role != null ? role.getClass().getName().hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "PartyRole {" +
			"ownershipPartyReference=" + this.ownershipPartyReference + ", " +
			"partyReference=" + this.partyReference + ", " +
			"role=" + this.role +
		'}';
	}
}
