package org.isda.cdm;

import java.math.BigDecimal;
import org.isda.cdm.PayerReceiver;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.PassThroughItemMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;
		import java.math.BigDecimal;

/**
 * Class to represent a single pass through payment.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="PassThroughItem", source="FpML_5_10")
public class PassThroughItem extends RosettaModelObject {
	private final BigDecimal passThroughPercentage;
	private final PayerReceiver payerReceiver;
	private static PassThroughItemMeta metaData = new PassThroughItemMeta();
	
	PassThroughItem(PassThroughItemBuilder builder) {
		this.passThroughPercentage = builder.getPassThroughPercentage();
		this.payerReceiver = ofNullable(builder.getPayerReceiver()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * Percentage of payments from the underlier which are passed through.
	 */
	@RosettaSynonym(value="passThroughPercentage", source="FpML_5_10")
	@RosettaSynonym(value="passThroughPercentage", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="passThroughPercentage", source="DTCC_11_0")
	@RosettaSynonym(value="passThroughPercentage", source="DTCC_9_0")
	@RosettaSynonym(value="passThroughPercentage", source="CME_ClearedConfirm_1_17")
	public final BigDecimal getPassThroughPercentage() {
		return passThroughPercentage;
	}
	
	/**
	 * This attribute doesn&#39;t exists in the FpML construct, which makes use of the PayerReceiver.model group.
	 */
	public final PayerReceiver getPayerReceiver() {
		return payerReceiver;
	}
	
	
	@Override
	public RosettaMetaData<? extends PassThroughItem> metaData() {
		return metaData;
	} 
	
	public PassThroughItemBuilder toBuilder() {
		PassThroughItemBuilder builder = new PassThroughItemBuilder();
		ofNullable(getPassThroughPercentage()).ifPresent(builder::setPassThroughPercentage);
		ofNullable(getPayerReceiver()).ifPresent(builder::setPayerReceiver);
		return builder;
	}

	public static PassThroughItemBuilder builder() {
		return new PassThroughItemBuilder();
	}

	public static  class PassThroughItemBuilder extends RosettaModelObjectBuilder {
	
		protected BigDecimal passThroughPercentage;
		protected PayerReceiver.PayerReceiverBuilder payerReceiver;
	
		public PassThroughItemBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends PassThroughItem> metaData() {
			return metaData;
		} 
	
		
		public BigDecimal getPassThroughPercentage() {
			return passThroughPercentage;
		}
		
		
		public PayerReceiver.PayerReceiverBuilder getPayerReceiver() {
			return payerReceiver;
		}
		
		public PayerReceiver.PayerReceiverBuilder getOrCreatePayerReceiver() {
			if (payerReceiver!=null) {
				return payerReceiver;
			}
			else return payerReceiver = new PayerReceiver.PayerReceiverBuilder();
		}
		
	
		public PassThroughItemBuilder setPassThroughPercentage(BigDecimal passThroughPercentage) {
			this.passThroughPercentage = passThroughPercentage;
			return this;
		}
		public PassThroughItemBuilder setPayerReceiver(PayerReceiver payerReceiver) {
			this.payerReceiver = payerReceiver.toBuilder();
			return this;
		}
		public PassThroughItemBuilder setPayerReceiverBuilder(PayerReceiver.PayerReceiverBuilder payerReceiver) {
			this.payerReceiver = payerReceiver;
			return this;
		}
		
		public PassThroughItem build() {
			return new PassThroughItem(this);
		}
	
		@Override
		public PassThroughItemBuilder prune() {
			if (payerReceiver!=null && !payerReceiver.prune().hasData()) payerReceiver = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getPassThroughPercentage()!=null) return true;
			if (getPayerReceiver()!=null && getPayerReceiver().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("passThroughPercentage"), BigDecimal.class, passThroughPercentage, this);
			
			processRosetta(path.newSubPath("payerReceiver"), processor, PayerReceiver.class, payerReceiver);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			PassThroughItemBuilder _that = (PassThroughItemBuilder) o;
		
			if (passThroughPercentage != null ? !passThroughPercentage.equals(_that.passThroughPercentage) : _that.passThroughPercentage != null) return false;
			if (payerReceiver != null ? !payerReceiver.equals(_that.payerReceiver) : _that.payerReceiver != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (passThroughPercentage != null ? passThroughPercentage.hashCode() : 0);
			_result = 31 * _result + (payerReceiver != null ? payerReceiver.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "PassThroughItemBuilder {" +
				"passThroughPercentage=" + this.passThroughPercentage + ", " +
				"payerReceiver=" + this.payerReceiver +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("passThroughPercentage"), BigDecimal.class, passThroughPercentage, this);
		
		processRosetta(path.newSubPath("payerReceiver"), processor, PayerReceiver.class, payerReceiver);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		PassThroughItem _that = (PassThroughItem) o;
	
		if (passThroughPercentage != null ? !passThroughPercentage.equals(_that.passThroughPercentage) : _that.passThroughPercentage != null) return false;
		if (payerReceiver != null ? !payerReceiver.equals(_that.payerReceiver) : _that.payerReceiver != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (passThroughPercentage != null ? passThroughPercentage.hashCode() : 0);
		_result = 31 * _result + (payerReceiver != null ? payerReceiver.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "PassThroughItem {" +
			"passThroughPercentage=" + this.passThroughPercentage + ", " +
			"payerReceiver=" + this.payerReceiver +
		'}';
	}
}
