package org.isda.cdm;

import com.rosetta.model.lib.records.Date;
import org.isda.cdm.BusinessDayAdjustments;
import org.isda.cdm.Frequency;
import org.isda.cdm.LastRegularPaymentDate;
import org.isda.cdm.Offset;
import org.isda.cdm.PayRelativeToEnum;
import org.isda.cdm.metafields.BasicReferenceWithMetaDate;
import org.isda.cdm.metafields.BasicReferenceWithMetaDate.BasicReferenceWithMetaDateBuilder;
import org.isda.cdm.metafields.MetaFields;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.PaymentDatesMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import com.rosetta.model.lib.records.Date;
		import static java.util.Optional.ofNullable;

/**
 * A class to specify the parameters to generate the payment date schedule, either through a parametric representation or by reference to other dates specified in the instance document (e.g. the reset dates or valuation dates). The CDM representation combines the FpML payment dates representation for interest rate and total return swap products by providing the ability to specify the payment dates as a function of some other dates specified in the instance document.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="PaymentDates", source="FpML_5_10")
@RosettaSynonym(value="PaymentDates", source="CME_SubmissionIRS_1_0")
@RosettaSynonym(value="PaymentDates", source="DTCC_11_0")
@RosettaSynonym(value="PaymentDates", source="DTCC_9_0")
@RosettaSynonym(value="PaymentDates", source="CME_ClearedConfirm_1_17")

@RosettaSynonym(value="PaymentDates", source="FpML_5_10")
public class PaymentDates extends RosettaModelObject implements GlobalKey {
	private final BasicReferenceWithMetaDate dateRelativeTo;
	private final Date firstPaymentDate;
	private final LastRegularPaymentDate lastPaymentDate;
	private final MetaFields meta;
	private final PayRelativeToEnum payRelativeTo;
	private final BusinessDayAdjustments paymentDatesAdjustments;
	private final Offset paymentDaysOffset;
	private final Frequency paymentFrequency;
	private static PaymentDatesMeta metaData = new PaymentDatesMeta();
	
	PaymentDates(PaymentDatesBuilder builder) {
		this.dateRelativeTo = ofNullable(builder.getDateRelativeTo()).map(f->f.build()).orElse(null);
		this.firstPaymentDate = builder.getFirstPaymentDate();
		this.lastPaymentDate = ofNullable(builder.getLastPaymentDate()).map(f->f.build()).orElse(null);
		this.meta = ofNullable(builder.getMeta()).map(f->f.build()).orElse(null);
		this.payRelativeTo = builder.getPayRelativeTo();
		this.paymentDatesAdjustments = ofNullable(builder.getPaymentDatesAdjustments()).map(f->f.build()).orElse(null);
		this.paymentDaysOffset = ofNullable(builder.getPaymentDaysOffset()).map(f->f.build()).orElse(null);
		this.paymentFrequency = ofNullable(builder.getPaymentFrequency()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The anchor date when the payment dates are specified by reference to a set of dates specified somewhere else in the instance document/transaction, e.g. the valuation dates as typically the case for equity swaps.  When this attribute is specified, the paymentDaysOffset and the paymentDatesAdjustments also need to specified.
	 */
	@RosettaSynonym(value="dateRelativeTo", source="FpML_5_10", path="paymentDatesInterim.relativeDates")
	@RosettaSynonym(value="dateRelativeTo", source="FpML_5_10", path="relativeDates")
	public final BasicReferenceWithMetaDate getDateRelativeTo() {
		return dateRelativeTo;
	}
	
	/**
	 * The first unadjusted payment date. This day may be subject to adjustment in accordance with any business day convention specified in paymentDatesAdjustments. This element must only be included if there is an initial stub. This date will normally correspond to an unadjusted calculation period start or end date. This is true even if early or delayed payment is specified to be applicable since the actual first payment date will be the specified number of days before or after the applicable adjusted calculation period start or end date with the resulting payment date then being adjusted in accordance with any business day convention specified in paymentDatesAdjustments.
	 */
	@RosettaSynonym(value="firstPaymentDate", source="FpML_5_10")
	@RosettaSynonym(value="firstPaymentDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="firstPaymentDate", source="DTCC_11_0")
	@RosettaSynonym(value="firstPaymentDate", source="DTCC_9_0")
	@RosettaSynonym(value="firstPaymentDate", source="CME_ClearedConfirm_1_17")
	public final Date getFirstPaymentDate() {
		return firstPaymentDate;
	}
	
	/**
	 * The last payment date. In the case of interest rate swaps, FpML specifies that this date must only be included if there is a final stub, with all calculation periods after this date contributing to the final payment.
	 */
	public final LastRegularPaymentDate getLastPaymentDate() {
		return lastPaymentDate;
	}
	
	public final MetaFields getMeta() {
		return meta;
	}
	
	/**
	 * Specifies whether the payments occur relative to each adjusted calculation period start date or end date, each reset date, valuation date or the last pricing date. Calculation period start date means relative to the start of the first calculation period contributing to a given payment. Similarly, calculation period end date means the end of the last calculation period contributing to a given payment. The valuation date is applicable for Brazilian-CDI and equity swaps.
	 */
	@RosettaSynonym(value="payRelativeTo", source="FpML_5_10")
	@RosettaSynonym(value="payRelativeTo", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="payRelativeTo", source="DTCC_11_0")
	@RosettaSynonym(value="payRelativeTo", source="DTCC_9_0")
	@RosettaSynonym(value="payRelativeTo", source="CME_ClearedConfirm_1_17")
	public final PayRelativeToEnum getPayRelativeTo() {
		return payRelativeTo;
	}
	
	/**
	 * The definition of the business day convention and financial business centers used for adjusting the payment date if it would otherwise fall on a day that is not a business day in the specified business center.
	 */
	@RosettaSynonym(value="paymentDatesAdjustments", source="FpML_5_10")
	@RosettaSynonym(value="paymentDatesAdjustments", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="paymentDatesAdjustments", source="DTCC_11_0")
	@RosettaSynonym(value="paymentDatesAdjustments", source="DTCC_9_0")
	@RosettaSynonym(value="paymentDatesAdjustments", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="relativeDates", source="FpML_5_10", path="paymentDatesInterim")
	public final BusinessDayAdjustments getPaymentDatesAdjustments() {
		return paymentDatesAdjustments;
	}
	
	/**
	 * If early payment or delayed payment is required, specifies the number of days offset that the payment occurs relative to what would otherwise be the unadjusted payment date. The offset can be specified in terms of either calendar or business days. Even in the case of a calendar days offset, the resulting payment date, adjusted for the specified calendar days offset, will still be adjusted in accordance with the specified payment dates adjustments. This element should only be included if early or delayed payment is applicable, i.e. if the periodMultiplier element value is not equal to zero. An early payment would be indicated by a negative periodMultiplier element value and a delayed payment (or payment lag) would be indicated by a positive periodMultiplier element value.
	 */
	@RosettaSynonym(value="paymentDaysOffset", source="FpML_5_10")
	@RosettaSynonym(value="paymentDaysOffset", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="paymentDaysOffset", source="DTCC_11_0")
	@RosettaSynonym(value="paymentDaysOffset", source="DTCC_9_0")
	@RosettaSynonym(value="paymentDaysOffset", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="relativeDates", source="FpML_5_10", path="paymentDatesInterim")
	@RosettaSynonym(value="relativeDates", source="FpML_5_10", path="")
	public final Offset getPaymentDaysOffset() {
		return paymentDaysOffset;
	}
	
	/**
	 * The frequency at which regular payment dates occur. If the payment frequency is equal to the frequency defined in the calculation period dates component then one calculation period contributes to each payment amount. If the payment frequency is less frequent than the frequency defined in the calculation period dates component then more than one calculation period will contribute to the payment amount. A payment frequency more frequent than the calculation period frequency or one that is not a multiple of the calculation period frequency is invalid. If the payment frequency is of value T (term), the period is defined by the effectiveDate and the terminationDate.
	 */
	@RosettaSynonym(value="paymentFrequency", source="FpML_5_10")
	@RosettaSynonym(value="paymentFrequency", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="paymentFrequency", source="DTCC_11_0")
	@RosettaSynonym(value="paymentFrequency", source="DTCC_9_0")
	@RosettaSynonym(value="paymentFrequency", source="CME_ClearedConfirm_1_17")
	public final Frequency getPaymentFrequency() {
		return paymentFrequency;
	}
	
	
	@Override
	public RosettaMetaData<? extends PaymentDates> metaData() {
		return metaData;
	} 
	
	public PaymentDatesBuilder toBuilder() {
		PaymentDatesBuilder builder = new PaymentDatesBuilder();
		ofNullable(getDateRelativeTo()).ifPresent(builder::setDateRelativeTo);
		ofNullable(getFirstPaymentDate()).ifPresent(builder::setFirstPaymentDate);
		ofNullable(getLastPaymentDate()).ifPresent(builder::setLastPaymentDate);
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		ofNullable(getPayRelativeTo()).ifPresent(builder::setPayRelativeTo);
		ofNullable(getPaymentDatesAdjustments()).ifPresent(builder::setPaymentDatesAdjustments);
		ofNullable(getPaymentDaysOffset()).ifPresent(builder::setPaymentDaysOffset);
		ofNullable(getPaymentFrequency()).ifPresent(builder::setPaymentFrequency);
		return builder;
	}

	public static PaymentDatesBuilder builder() {
		return new PaymentDatesBuilder();
	}

	public static  class PaymentDatesBuilder extends RosettaModelObjectBuilder implements GlobalKeyBuilder<PaymentDatesBuilder> {
	
		protected BasicReferenceWithMetaDateBuilder dateRelativeTo;
		protected Date firstPaymentDate;
		protected LastRegularPaymentDate.LastRegularPaymentDateBuilder lastPaymentDate;
		protected MetaFields.MetaFieldsBuilder meta;
		protected PayRelativeToEnum payRelativeTo;
		protected BusinessDayAdjustments.BusinessDayAdjustmentsBuilder paymentDatesAdjustments;
		protected Offset.OffsetBuilder paymentDaysOffset;
		protected Frequency.FrequencyBuilder paymentFrequency;
	
		public PaymentDatesBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends PaymentDates> metaData() {
			return metaData;
		} 
	
		
		public BasicReferenceWithMetaDateBuilder getDateRelativeTo() {
			return dateRelativeTo;
		}
		
		public BasicReferenceWithMetaDateBuilder getOrCreateDateRelativeTo() {
			if (dateRelativeTo!=null) {
				return dateRelativeTo;
			}
			else return dateRelativeTo = new BasicReferenceWithMetaDateBuilder();
		}
		
		
		public Date getFirstPaymentDate() {
			return firstPaymentDate;
		}
		
		
		public LastRegularPaymentDate.LastRegularPaymentDateBuilder getLastPaymentDate() {
			return lastPaymentDate;
		}
		
		public LastRegularPaymentDate.LastRegularPaymentDateBuilder getOrCreateLastPaymentDate() {
			if (lastPaymentDate!=null) {
				return lastPaymentDate;
			}
			else return lastPaymentDate = new LastRegularPaymentDate.LastRegularPaymentDateBuilder();
		}
		
		
		public MetaFields.MetaFieldsBuilder getMeta() {
			return meta;
		}
		
		public MetaFields.MetaFieldsBuilder getOrCreateMeta() {
			if (meta!=null) {
				return meta;
			}
			else return meta = new MetaFields.MetaFieldsBuilder();
		}
		
		
		public PayRelativeToEnum getPayRelativeTo() {
			return payRelativeTo;
		}
		
		
		public BusinessDayAdjustments.BusinessDayAdjustmentsBuilder getPaymentDatesAdjustments() {
			return paymentDatesAdjustments;
		}
		
		public BusinessDayAdjustments.BusinessDayAdjustmentsBuilder getOrCreatePaymentDatesAdjustments() {
			if (paymentDatesAdjustments!=null) {
				return paymentDatesAdjustments;
			}
			else return paymentDatesAdjustments = new BusinessDayAdjustments.BusinessDayAdjustmentsBuilder();
		}
		
		
		public Offset.OffsetBuilder getPaymentDaysOffset() {
			return paymentDaysOffset;
		}
		
		public Offset.OffsetBuilder getOrCreatePaymentDaysOffset() {
			if (paymentDaysOffset!=null) {
				return paymentDaysOffset;
			}
			else return paymentDaysOffset = new Offset.OffsetBuilder();
		}
		
		
		public Frequency.FrequencyBuilder getPaymentFrequency() {
			return paymentFrequency;
		}
		
		public Frequency.FrequencyBuilder getOrCreatePaymentFrequency() {
			if (paymentFrequency!=null) {
				return paymentFrequency;
			}
			else return paymentFrequency = new Frequency.FrequencyBuilder();
		}
		
	
		public PaymentDatesBuilder setDateRelativeTo(BasicReferenceWithMetaDate dateRelativeTo) {
			this.dateRelativeTo = dateRelativeTo.toBuilder();
			return this;
		}
		public PaymentDatesBuilder setDateRelativeToRef(Date dateRelativeTo) {
			return setDateRelativeTo(BasicReferenceWithMetaDate.builder().setValue(dateRelativeTo).build());
		}
		public PaymentDatesBuilder setFirstPaymentDate(Date firstPaymentDate) {
			this.firstPaymentDate = firstPaymentDate;
			return this;
		}
		public PaymentDatesBuilder setLastPaymentDate(LastRegularPaymentDate lastPaymentDate) {
			this.lastPaymentDate = lastPaymentDate.toBuilder();
			return this;
		}
		public PaymentDatesBuilder setLastPaymentDateBuilder(LastRegularPaymentDate.LastRegularPaymentDateBuilder lastPaymentDate) {
			this.lastPaymentDate = lastPaymentDate;
			return this;
		}
		public PaymentDatesBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		public PaymentDatesBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		public PaymentDatesBuilder setPayRelativeTo(PayRelativeToEnum payRelativeTo) {
			this.payRelativeTo = payRelativeTo;
			return this;
		}
		public PaymentDatesBuilder setPaymentDatesAdjustments(BusinessDayAdjustments paymentDatesAdjustments) {
			this.paymentDatesAdjustments = paymentDatesAdjustments.toBuilder();
			return this;
		}
		public PaymentDatesBuilder setPaymentDatesAdjustmentsBuilder(BusinessDayAdjustments.BusinessDayAdjustmentsBuilder paymentDatesAdjustments) {
			this.paymentDatesAdjustments = paymentDatesAdjustments;
			return this;
		}
		public PaymentDatesBuilder setPaymentDaysOffset(Offset paymentDaysOffset) {
			this.paymentDaysOffset = paymentDaysOffset.toBuilder();
			return this;
		}
		public PaymentDatesBuilder setPaymentDaysOffsetBuilder(Offset.OffsetBuilder paymentDaysOffset) {
			this.paymentDaysOffset = paymentDaysOffset;
			return this;
		}
		public PaymentDatesBuilder setPaymentFrequency(Frequency paymentFrequency) {
			this.paymentFrequency = paymentFrequency.toBuilder();
			return this;
		}
		public PaymentDatesBuilder setPaymentFrequencyBuilder(Frequency.FrequencyBuilder paymentFrequency) {
			this.paymentFrequency = paymentFrequency;
			return this;
		}
		
		public PaymentDates build() {
			return new PaymentDates(this);
		}
	
		@Override
		public PaymentDatesBuilder prune() {
			if (dateRelativeTo!=null && !dateRelativeTo.prune().hasData()) dateRelativeTo = null;
			if (lastPaymentDate!=null && !lastPaymentDate.prune().hasData()) lastPaymentDate = null;
			if (meta!=null && !meta.prune().hasData()) meta = null;
			if (paymentDatesAdjustments!=null && !paymentDatesAdjustments.prune().hasData()) paymentDatesAdjustments = null;
			if (paymentDaysOffset!=null && !paymentDaysOffset.prune().hasData()) paymentDaysOffset = null;
			if (paymentFrequency!=null && !paymentFrequency.prune().hasData()) paymentFrequency = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getDateRelativeTo()!=null) return true;
			if (getFirstPaymentDate()!=null) return true;
			if (getLastPaymentDate()!=null && getLastPaymentDate().hasData()) return true;
			if (getMeta()!=null && getMeta().hasData()) return true;
			if (getPayRelativeTo()!=null) return true;
			if (getPaymentDatesAdjustments()!=null && getPaymentDatesAdjustments().hasData()) return true;
			if (getPaymentDaysOffset()!=null && getPaymentDaysOffset().hasData()) return true;
			if (getPaymentFrequency()!=null && getPaymentFrequency().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("firstPaymentDate"), Date.class, firstPaymentDate, this);
			processor.processBasic(path.newSubPath("payRelativeTo"), PayRelativeToEnum.class, payRelativeTo, this);
			
			processRosetta(path.newSubPath("dateRelativeTo"), processor, BasicReferenceWithMetaDate.class, dateRelativeTo);
			processRosetta(path.newSubPath("lastPaymentDate"), processor, LastRegularPaymentDate.class, lastPaymentDate);
			processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
			processRosetta(path.newSubPath("paymentDatesAdjustments"), processor, BusinessDayAdjustments.class, paymentDatesAdjustments);
			processRosetta(path.newSubPath("paymentDaysOffset"), processor, Offset.class, paymentDaysOffset);
			processRosetta(path.newSubPath("paymentFrequency"), processor, Frequency.class, paymentFrequency);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			PaymentDatesBuilder _that = (PaymentDatesBuilder) o;
		
			if (dateRelativeTo != null ? !dateRelativeTo.equals(_that.dateRelativeTo) : _that.dateRelativeTo != null) return false;
			if (firstPaymentDate != null ? !firstPaymentDate.equals(_that.firstPaymentDate) : _that.firstPaymentDate != null) return false;
			if (lastPaymentDate != null ? !lastPaymentDate.equals(_that.lastPaymentDate) : _that.lastPaymentDate != null) return false;
			if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
			if (payRelativeTo != null ? !payRelativeTo.equals(_that.payRelativeTo) : _that.payRelativeTo != null) return false;
			if (paymentDatesAdjustments != null ? !paymentDatesAdjustments.equals(_that.paymentDatesAdjustments) : _that.paymentDatesAdjustments != null) return false;
			if (paymentDaysOffset != null ? !paymentDaysOffset.equals(_that.paymentDaysOffset) : _that.paymentDaysOffset != null) return false;
			if (paymentFrequency != null ? !paymentFrequency.equals(_that.paymentFrequency) : _that.paymentFrequency != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (dateRelativeTo != null ? dateRelativeTo.hashCode() : 0);
			_result = 31 * _result + (firstPaymentDate != null ? firstPaymentDate.hashCode() : 0);
			_result = 31 * _result + (lastPaymentDate != null ? lastPaymentDate.hashCode() : 0);
			_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
			_result = 31 * _result + (payRelativeTo != null ? payRelativeTo.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (paymentDatesAdjustments != null ? paymentDatesAdjustments.hashCode() : 0);
			_result = 31 * _result + (paymentDaysOffset != null ? paymentDaysOffset.hashCode() : 0);
			_result = 31 * _result + (paymentFrequency != null ? paymentFrequency.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "PaymentDatesBuilder {" +
				"dateRelativeTo=" + this.dateRelativeTo + ", " +
				"firstPaymentDate=" + this.firstPaymentDate + ", " +
				"lastPaymentDate=" + this.lastPaymentDate + ", " +
				"meta=" + this.meta + ", " +
				"payRelativeTo=" + this.payRelativeTo + ", " +
				"paymentDatesAdjustments=" + this.paymentDatesAdjustments + ", " +
				"paymentDaysOffset=" + this.paymentDaysOffset + ", " +
				"paymentFrequency=" + this.paymentFrequency +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("firstPaymentDate"), Date.class, firstPaymentDate, this);
		processor.processBasic(path.newSubPath("payRelativeTo"), PayRelativeToEnum.class, payRelativeTo, this);
		
		processRosetta(path.newSubPath("dateRelativeTo"), processor, BasicReferenceWithMetaDate.class, dateRelativeTo);
		processRosetta(path.newSubPath("lastPaymentDate"), processor, LastRegularPaymentDate.class, lastPaymentDate);
		processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
		processRosetta(path.newSubPath("paymentDatesAdjustments"), processor, BusinessDayAdjustments.class, paymentDatesAdjustments);
		processRosetta(path.newSubPath("paymentDaysOffset"), processor, Offset.class, paymentDaysOffset);
		processRosetta(path.newSubPath("paymentFrequency"), processor, Frequency.class, paymentFrequency);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		PaymentDates _that = (PaymentDates) o;
	
		if (dateRelativeTo != null ? !dateRelativeTo.equals(_that.dateRelativeTo) : _that.dateRelativeTo != null) return false;
		if (firstPaymentDate != null ? !firstPaymentDate.equals(_that.firstPaymentDate) : _that.firstPaymentDate != null) return false;
		if (lastPaymentDate != null ? !lastPaymentDate.equals(_that.lastPaymentDate) : _that.lastPaymentDate != null) return false;
		if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
		if (payRelativeTo != null ? !payRelativeTo.equals(_that.payRelativeTo) : _that.payRelativeTo != null) return false;
		if (paymentDatesAdjustments != null ? !paymentDatesAdjustments.equals(_that.paymentDatesAdjustments) : _that.paymentDatesAdjustments != null) return false;
		if (paymentDaysOffset != null ? !paymentDaysOffset.equals(_that.paymentDaysOffset) : _that.paymentDaysOffset != null) return false;
		if (paymentFrequency != null ? !paymentFrequency.equals(_that.paymentFrequency) : _that.paymentFrequency != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (dateRelativeTo != null ? dateRelativeTo.hashCode() : 0);
		_result = 31 * _result + (firstPaymentDate != null ? firstPaymentDate.hashCode() : 0);
		_result = 31 * _result + (lastPaymentDate != null ? lastPaymentDate.hashCode() : 0);
		_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
		_result = 31 * _result + (payRelativeTo != null ? payRelativeTo.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (paymentDatesAdjustments != null ? paymentDatesAdjustments.hashCode() : 0);
		_result = 31 * _result + (paymentDaysOffset != null ? paymentDaysOffset.hashCode() : 0);
		_result = 31 * _result + (paymentFrequency != null ? paymentFrequency.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "PaymentDates {" +
			"dateRelativeTo=" + this.dateRelativeTo + ", " +
			"firstPaymentDate=" + this.firstPaymentDate + ", " +
			"lastPaymentDate=" + this.lastPaymentDate + ", " +
			"meta=" + this.meta + ", " +
			"payRelativeTo=" + this.payRelativeTo + ", " +
			"paymentDatesAdjustments=" + this.paymentDatesAdjustments + ", " +
			"paymentDaysOffset=" + this.paymentDaysOffset + ", " +
			"paymentFrequency=" + this.paymentFrequency +
		'}';
	}
}
