package org.isda.cdm;

import java.math.BigDecimal;
import org.isda.cdm.Money;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.PaymentDiscountingMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.math.BigDecimal;
		import static java.util.Optional.ofNullable;

/**
 * This class corresponds to the FpML PaymentDiscounting.model group for representing the discounting elements that can be associated with a payment.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class PaymentDiscounting extends RosettaModelObject {
	private final BigDecimal discountFactor;
	private final Money presentValueAmount;
	private static PaymentDiscountingMeta metaData = new PaymentDiscountingMeta();
	
	PaymentDiscounting(PaymentDiscountingBuilder builder) {
		this.discountFactor = builder.getDiscountFactor();
		this.presentValueAmount = ofNullable(builder.getPresentValueAmount()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The value representing the discount factor used to calculate the present value of the cash flow.
	 */
	@RosettaSynonym(value="discountFactor", source="FpML_5_10")
	@RosettaSynonym(value="discountFactor", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="discountFactor", source="DTCC_11_0")
	@RosettaSynonym(value="discountFactor", source="DTCC_9_0")
	@RosettaSynonym(value="discountFactor", source="CME_ClearedConfirm_1_17")
	public final BigDecimal getDiscountFactor() {
		return discountFactor;
	}
	
	/**
	 * The amount representing the present value of the forecast payment.
	 */
	@RosettaSynonym(value="presentValueAmount", source="FpML_5_10")
	@RosettaSynonym(value="presentValueAmount", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="presentValueAmount", source="DTCC_11_0")
	@RosettaSynonym(value="presentValueAmount", source="DTCC_9_0")
	@RosettaSynonym(value="presentValueAmount", source="CME_ClearedConfirm_1_17")
	public final Money getPresentValueAmount() {
		return presentValueAmount;
	}
	
	
	@Override
	public RosettaMetaData<? extends PaymentDiscounting> metaData() {
		return metaData;
	} 
	
	public PaymentDiscountingBuilder toBuilder() {
		PaymentDiscountingBuilder builder = new PaymentDiscountingBuilder();
		ofNullable(getDiscountFactor()).ifPresent(builder::setDiscountFactor);
		ofNullable(getPresentValueAmount()).ifPresent(builder::setPresentValueAmount);
		return builder;
	}

	public static PaymentDiscountingBuilder builder() {
		return new PaymentDiscountingBuilder();
	}

	public static  class PaymentDiscountingBuilder extends RosettaModelObjectBuilder {
	
		protected BigDecimal discountFactor;
		protected Money.MoneyBuilder presentValueAmount;
	
		public PaymentDiscountingBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends PaymentDiscounting> metaData() {
			return metaData;
		} 
	
		
		public BigDecimal getDiscountFactor() {
			return discountFactor;
		}
		
		
		public Money.MoneyBuilder getPresentValueAmount() {
			return presentValueAmount;
		}
		
		public Money.MoneyBuilder getOrCreatePresentValueAmount() {
			if (presentValueAmount!=null) {
				return presentValueAmount;
			}
			else return presentValueAmount = new Money.MoneyBuilder();
		}
		
	
		public PaymentDiscountingBuilder setDiscountFactor(BigDecimal discountFactor) {
			this.discountFactor = discountFactor;
			return this;
		}
		public PaymentDiscountingBuilder setPresentValueAmount(Money presentValueAmount) {
			this.presentValueAmount = presentValueAmount.toBuilder();
			return this;
		}
		public PaymentDiscountingBuilder setPresentValueAmountBuilder(Money.MoneyBuilder presentValueAmount) {
			this.presentValueAmount = presentValueAmount;
			return this;
		}
		
		public PaymentDiscounting build() {
			return new PaymentDiscounting(this);
		}
	
		@Override
		public PaymentDiscountingBuilder prune() {
			if (presentValueAmount!=null && !presentValueAmount.prune().hasData()) presentValueAmount = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getDiscountFactor()!=null) return true;
			if (getPresentValueAmount()!=null && getPresentValueAmount().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("discountFactor"), BigDecimal.class, discountFactor, this);
			
			processRosetta(path.newSubPath("presentValueAmount"), processor, Money.class, presentValueAmount);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			PaymentDiscountingBuilder _that = (PaymentDiscountingBuilder) o;
		
			if (discountFactor != null ? !discountFactor.equals(_that.discountFactor) : _that.discountFactor != null) return false;
			if (presentValueAmount != null ? !presentValueAmount.equals(_that.presentValueAmount) : _that.presentValueAmount != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (discountFactor != null ? discountFactor.hashCode() : 0);
			_result = 31 * _result + (presentValueAmount != null ? presentValueAmount.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "PaymentDiscountingBuilder {" +
				"discountFactor=" + this.discountFactor + ", " +
				"presentValueAmount=" + this.presentValueAmount +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("discountFactor"), BigDecimal.class, discountFactor, this);
		
		processRosetta(path.newSubPath("presentValueAmount"), processor, Money.class, presentValueAmount);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		PaymentDiscounting _that = (PaymentDiscounting) o;
	
		if (discountFactor != null ? !discountFactor.equals(_that.discountFactor) : _that.discountFactor != null) return false;
		if (presentValueAmount != null ? !presentValueAmount.equals(_that.presentValueAmount) : _that.presentValueAmount != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (discountFactor != null ? discountFactor.hashCode() : 0);
		_result = 31 * _result + (presentValueAmount != null ? presentValueAmount.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "PaymentDiscounting {" +
			"discountFactor=" + this.discountFactor + ", " +
			"presentValueAmount=" + this.presentValueAmount +
		'}';
	}
}
