package org.isda.cdm;

import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.Cashflow;
import org.isda.cdm.CreditDefaultPayout;
import org.isda.cdm.EquityPayout;
import org.isda.cdm.ForwardPayout;
import org.isda.cdm.InterestRatePayout;
import org.isda.cdm.OptionPayout;
import org.isda.cdm.SecurityPayout;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.PayoutMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;

/**
 * A class to represent the set of future cashflow methodologies in the form of specific payout class(es) that can be associated for the purpose of specifying a financial product. For example, two interest rate payouts can be combined to specify an interest rate swap, or one interest rate payout can be combined with a credit default payout to specify a credit default swap.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class Payout extends RosettaModelObject {
	private final List<Cashflow> cashflow;
	private final CreditDefaultPayout creditDefaultPayout;
	private final List<EquityPayout> equityPayout;
	private final List<ForwardPayout> forwardPayout;
	private final List<InterestRatePayout> interestRatePayout;
	private final List<OptionPayout> optionPayout;
	private final List<SecurityPayout> securityPayout;
	private static PayoutMeta metaData = new PayoutMeta();
	
	Payout(PayoutBuilder builder) {
		this.cashflow = ofNullable(builder.getCashflow()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.creditDefaultPayout = ofNullable(builder.getCreditDefaultPayout()).map(f->f.build()).orElse(null);
		this.equityPayout = ofNullable(builder.getEquityPayout()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.forwardPayout = ofNullable(builder.getForwardPayout()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.interestRatePayout = ofNullable(builder.getInterestRatePayout()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.optionPayout = ofNullable(builder.getOptionPayout()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.securityPayout = ofNullable(builder.getSecurityPayout()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
	}
	
	/**
	 * A cashflow between the parties to the trade. For interest rate and equity products, this corresponds to the FpML additionalPayment element. For credit default swaps, this corresponds to the FpML initialPayment element and the singlePayment element of the fee leg. For option products, it represents the FpML premium element.
	 */
	@RosettaSynonym(value="additionalPayment", source="FpML_5_10", path="trade.swap")
	@RosettaSynonym(value="additionalPayment", source="CME_SubmissionIRS_1_0", path="trade.swap")
	@RosettaSynonym(value="additionalPayment", source="DTCC_11_0", path="trade.swap")
	@RosettaSynonym(value="additionalPayment", source="DTCC_9_0", path="trade.swap")
	@RosettaSynonym(value="additionalPayment", source="CME_ClearedConfirm_1_17", path="trade.swap")
	@RosettaSynonym(value="additionalPayment", source="FpML_5_10", path="swap")
	@RosettaSynonym(value="additionalPayment", source="CME_SubmissionIRS_1_0", path="swap")
	@RosettaSynonym(value="additionalPayment", source="DTCC_11_0", path="swap")
	@RosettaSynonym(value="additionalPayment", source="DTCC_9_0", path="swap")
	@RosettaSynonym(value="additionalPayment", source="CME_ClearedConfirm_1_17", path="swap")
	@RosettaSynonym(value="additionalPayment", source="FpML_5_10")
	@RosettaSynonym(value="additionalPayment", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="additionalPayment", source="DTCC_11_0")
	@RosettaSynonym(value="additionalPayment", source="DTCC_9_0")
	@RosettaSynonym(value="additionalPayment", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="initialPayment", source="FpML_5_10", path="trade.creditDefaultSwap.feeLeg")
	@RosettaSynonym(value="initialPayment", source="CME_SubmissionIRS_1_0", path="trade.creditDefaultSwap.feeLeg")
	@RosettaSynonym(value="initialPayment", source="DTCC_11_0", path="trade.creditDefaultSwap.feeLeg")
	@RosettaSynonym(value="initialPayment", source="DTCC_9_0", path="trade.creditDefaultSwap.feeLeg")
	@RosettaSynonym(value="initialPayment", source="CME_ClearedConfirm_1_17", path="trade.creditDefaultSwap.feeLeg")
	@RosettaSynonym(value="initialPayment", source="FpML_5_10", path="creditDefaultSwap.feeLeg")
	@RosettaSynonym(value="initialPayment", source="CME_SubmissionIRS_1_0", path="creditDefaultSwap.feeLeg")
	@RosettaSynonym(value="initialPayment", source="DTCC_11_0", path="creditDefaultSwap.feeLeg")
	@RosettaSynonym(value="initialPayment", source="DTCC_9_0", path="creditDefaultSwap.feeLeg")
	@RosettaSynonym(value="initialPayment", source="CME_ClearedConfirm_1_17", path="creditDefaultSwap.feeLeg")
	@RosettaSynonym(value="singlePayment", source="FpML_5_10", path="trade.creditDefaultSwap.feeLeg")
	@RosettaSynonym(value="singlePayment", source="CME_SubmissionIRS_1_0", path="trade.creditDefaultSwap.feeLeg")
	@RosettaSynonym(value="singlePayment", source="DTCC_11_0", path="trade.creditDefaultSwap.feeLeg")
	@RosettaSynonym(value="singlePayment", source="DTCC_9_0", path="trade.creditDefaultSwap.feeLeg")
	@RosettaSynonym(value="singlePayment", source="CME_ClearedConfirm_1_17", path="trade.creditDefaultSwap.feeLeg")
	@RosettaSynonym(value="singlePayment", source="FpML_5_10", path="creditDefaultSwap.feeLeg")
	@RosettaSynonym(value="singlePayment", source="CME_SubmissionIRS_1_0", path="creditDefaultSwap.feeLeg")
	@RosettaSynonym(value="singlePayment", source="DTCC_11_0", path="creditDefaultSwap.feeLeg")
	@RosettaSynonym(value="singlePayment", source="DTCC_9_0", path="creditDefaultSwap.feeLeg")
	@RosettaSynonym(value="singlePayment", source="CME_ClearedConfirm_1_17", path="creditDefaultSwap.feeLeg")
	@RosettaSynonym(value="singlePayment", source="FpML_5_10")
	@RosettaSynonym(value="singlePayment", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="singlePayment", source="DTCC_11_0")
	@RosettaSynonym(value="singlePayment", source="DTCC_9_0")
	@RosettaSynonym(value="singlePayment", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="", source="FpML_5_10")
	@RosettaSynonym(value="", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="", source="DTCC_11_0")
	@RosettaSynonym(value="", source="DTCC_9_0")
	@RosettaSynonym(value="", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="", source="FpML_5_10")
	@RosettaSynonym(value="", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="", source="DTCC_11_0")
	@RosettaSynonym(value="", source="DTCC_9_0")
	@RosettaSynonym(value="", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="premium", source="FpML_5_10", path="trade.swaption")
	@RosettaSynonym(value="premium", source="CME_SubmissionIRS_1_0", path="trade.swaption")
	@RosettaSynonym(value="premium", source="DTCC_11_0", path="trade.swaption")
	@RosettaSynonym(value="premium", source="DTCC_9_0", path="trade.swaption")
	@RosettaSynonym(value="premium", source="CME_ClearedConfirm_1_17", path="trade.swaption")
	@RosettaSynonym(value="premium", source="FpML_5_10", path="swaption")
	@RosettaSynonym(value="premium", source="CME_SubmissionIRS_1_0", path="swaption")
	@RosettaSynonym(value="premium", source="DTCC_11_0", path="swaption")
	@RosettaSynonym(value="premium", source="DTCC_9_0", path="swaption")
	@RosettaSynonym(value="premium", source="CME_ClearedConfirm_1_17", path="swaption")
	@RosettaSynonym(value="premium", source="FpML_5_10", path="trade.creditDefaultSwapOption")
	@RosettaSynonym(value="premium", source="CME_SubmissionIRS_1_0", path="trade.creditDefaultSwapOption")
	@RosettaSynonym(value="premium", source="DTCC_11_0", path="trade.creditDefaultSwapOption")
	@RosettaSynonym(value="premium", source="DTCC_9_0", path="trade.creditDefaultSwapOption")
	@RosettaSynonym(value="premium", source="CME_ClearedConfirm_1_17", path="trade.creditDefaultSwapOption")
	@RosettaSynonym(value="premium", source="FpML_5_10", path="creditDefaultSwapOption")
	@RosettaSynonym(value="premium", source="CME_SubmissionIRS_1_0", path="creditDefaultSwapOption")
	@RosettaSynonym(value="premium", source="DTCC_11_0", path="creditDefaultSwapOption")
	@RosettaSynonym(value="premium", source="DTCC_9_0", path="creditDefaultSwapOption")
	@RosettaSynonym(value="premium", source="CME_ClearedConfirm_1_17", path="creditDefaultSwapOption")
	@RosettaSynonym(value="premium", source="FpML_5_10", path="trade.bondOption")
	@RosettaSynonym(value="premium", source="CME_SubmissionIRS_1_0", path="trade.bondOption")
	@RosettaSynonym(value="premium", source="DTCC_11_0", path="trade.bondOption")
	@RosettaSynonym(value="premium", source="DTCC_9_0", path="trade.bondOption")
	@RosettaSynonym(value="premium", source="CME_ClearedConfirm_1_17", path="trade.bondOption")
	@RosettaSynonym(value="premium", source="FpML_5_10", path="bondOption")
	@RosettaSynonym(value="premium", source="CME_SubmissionIRS_1_0", path="bondOption")
	@RosettaSynonym(value="premium", source="DTCC_11_0", path="bondOption")
	@RosettaSynonym(value="premium", source="DTCC_9_0", path="bondOption")
	@RosettaSynonym(value="premium", source="CME_ClearedConfirm_1_17", path="bondOption")
	@RosettaSynonym(value="premium", source="FpML_5_10", path="trade.capFloor")
	@RosettaSynonym(value="additionalPayment", source="FpML_5_10", path="trade.capFloor")
	@RosettaSynonym(value="premium", source="CME_SubmissionIRS_1_0", path="trade.capFloor")
	@RosettaSynonym(value="additionalPayment", source="CME_SubmissionIRS_1_0", path="trade.capFloor")
	@RosettaSynonym(value="premium", source="DTCC_11_0", path="trade.capFloor")
	@RosettaSynonym(value="additionalPayment", source="DTCC_11_0", path="trade.capFloor")
	@RosettaSynonym(value="premium", source="DTCC_9_0", path="trade.capFloor")
	@RosettaSynonym(value="additionalPayment", source="DTCC_9_0", path="trade.capFloor")
	@RosettaSynonym(value="premium", source="CME_ClearedConfirm_1_17", path="trade.capFloor")
	@RosettaSynonym(value="additionalPayment", source="CME_ClearedConfirm_1_17", path="trade.capFloor")
	@RosettaSynonym(value="otherPartyPayment", source="FpML_5_10", path="trade")
	@RosettaSynonym(value="otherPartyPayment", source="CME_SubmissionIRS_1_0", path="trade")
	@RosettaSynonym(value="otherPartyPayment", source="DTCC_11_0", path="trade")
	@RosettaSynonym(value="otherPartyPayment", source="DTCC_9_0", path="trade")
	@RosettaSynonym(value="otherPartyPayment", source="CME_ClearedConfirm_1_17", path="trade")
	@RosettaSynonym(value="otherPartyPayment", source="FpML_5_10")
	@RosettaSynonym(value="otherPartyPayment", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="otherPartyPayment", source="DTCC_11_0")
	@RosettaSynonym(value="otherPartyPayment", source="DTCC_9_0")
	@RosettaSynonym(value="otherPartyPayment", source="CME_ClearedConfirm_1_17")
	public final List<Cashflow> getCashflow() {
		return cashflow;
	}
	
	/**
	 * The credit default payout, which provides the details necessary for determining when a credit payout will be triggered as well as the parameters for calculating the payout and the settlement terms.
	 */
	public final CreditDefaultPayout getCreditDefaultPayout() {
		return creditDefaultPayout;
	}
	
	/**
	 * The equity payout, which encompasses the equity price returns, dividend returns, volatility and variance return provisions.
	 */
	@RosettaSynonym(value="returnLeg", source="FpML_5_10", path="trade.returnSwap")
	@RosettaSynonym(value="returnLeg", source="FpML_5_10", path="trade.equitySwapTransactionSupplement")
	@RosettaSynonym(value="ignore", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="ignore", source="DTCC_11_0")
	@RosettaSynonym(value="ignore", source="DTCC_9_0")
	public final List<EquityPayout> getEquityPayout() {
		return equityPayout;
	}
	
	@RosettaSynonym(value="fxSingleLeg", source="FpML_5_10", path="trade")
	@RosettaSynonym(value="fxSingleLeg", source="FpML_5_10")
	@RosettaSynonym(value="ignore", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="ignore", source="DTCC_11_0")
	@RosettaSynonym(value="ignore", source="DTCC_9_0")
	public final List<ForwardPayout> getForwardPayout() {
		return forwardPayout;
	}
	
	/**
	 * All of the terms necessary to define and calculate a cash flow based on a fixed, a floating or an inflation index rate. The interest rate payout can be applied to interest rate swaps and FRA (which both have two associated interest rate payouts), credit default swaps (to represent the fee leg when subject to periodic payments) and equity swaps (to represent the funding leg).
	 */
	@RosettaSynonym(value="swapStream", source="FpML_5_10", path="trade.swap")
	@RosettaSynonym(value="swapStream", source="CME_SubmissionIRS_1_0", path="trade.swap")
	@RosettaSynonym(value="swapStream", source="DTCC_11_0", path="trade.swap")
	@RosettaSynonym(value="swapStream", source="DTCC_9_0", path="trade.swap")
	@RosettaSynonym(value="swapStream", source="CME_ClearedConfirm_1_17", path="trade.swap")
	@RosettaSynonym(value="swapStream", source="FpML_5_10", path="swap")
	@RosettaSynonym(value="swapStream", source="CME_SubmissionIRS_1_0", path="swap")
	@RosettaSynonym(value="swapStream", source="DTCC_11_0", path="swap")
	@RosettaSynonym(value="swapStream", source="DTCC_9_0", path="swap")
	@RosettaSynonym(value="swapStream", source="CME_ClearedConfirm_1_17", path="swap")
	@RosettaSynonym(value="swapStream", source="FpML_5_10")
	@RosettaSynonym(value="swapStream", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="swapStream", source="DTCC_11_0")
	@RosettaSynonym(value="swapStream", source="DTCC_9_0")
	@RosettaSynonym(value="swapStream", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="generalTerms", source="FpML_5_10", path="trade.creditDefaultSwap")
	@RosettaSynonym(value="feeLeg", source="FpML_5_10", path="trade.creditDefaultSwap")
	@RosettaSynonym(value="generalTerms", source="CME_SubmissionIRS_1_0", path="trade.creditDefaultSwap")
	@RosettaSynonym(value="feeLeg", source="CME_SubmissionIRS_1_0", path="trade.creditDefaultSwap")
	@RosettaSynonym(value="generalTerms", source="DTCC_11_0", path="trade.creditDefaultSwap")
	@RosettaSynonym(value="feeLeg", source="DTCC_11_0", path="trade.creditDefaultSwap")
	@RosettaSynonym(value="generalTerms", source="DTCC_9_0", path="trade.creditDefaultSwap")
	@RosettaSynonym(value="feeLeg", source="DTCC_9_0", path="trade.creditDefaultSwap")
	@RosettaSynonym(value="generalTerms", source="CME_ClearedConfirm_1_17", path="trade.creditDefaultSwap")
	@RosettaSynonym(value="feeLeg", source="CME_ClearedConfirm_1_17", path="trade.creditDefaultSwap")
	@RosettaSynonym(value="generalTerms", source="FpML_5_10", path="creditDefaultSwap")
	@RosettaSynonym(value="feeLeg", source="FpML_5_10", path="creditDefaultSwap")
	@RosettaSynonym(value="generalTerms", source="CME_SubmissionIRS_1_0", path="creditDefaultSwap")
	@RosettaSynonym(value="feeLeg", source="CME_SubmissionIRS_1_0", path="creditDefaultSwap")
	@RosettaSynonym(value="generalTerms", source="DTCC_11_0", path="creditDefaultSwap")
	@RosettaSynonym(value="feeLeg", source="DTCC_11_0", path="creditDefaultSwap")
	@RosettaSynonym(value="generalTerms", source="DTCC_9_0", path="creditDefaultSwap")
	@RosettaSynonym(value="feeLeg", source="DTCC_9_0", path="creditDefaultSwap")
	@RosettaSynonym(value="generalTerms", source="CME_ClearedConfirm_1_17", path="creditDefaultSwap")
	@RosettaSynonym(value="feeLeg", source="CME_ClearedConfirm_1_17", path="creditDefaultSwap")
	@RosettaSynonym(value="feeLeg,generalTerms", source="FpML_5_10")
	@RosettaSynonym(value="feeLeg,generalTerms", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="feeLeg,generalTerms", source="DTCC_11_0")
	@RosettaSynonym(value="feeLeg,generalTerms", source="DTCC_9_0")
	@RosettaSynonym(value="feeLeg,generalTerms", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="capFloorStream", source="FpML_5_10", path="trade.capFloor")
	@RosettaSynonym(value="capFloorStream", source="CME_SubmissionIRS_1_0", path="trade.capFloor")
	@RosettaSynonym(value="capFloorStream", source="DTCC_11_0", path="trade.capFloor")
	@RosettaSynonym(value="capFloorStream", source="DTCC_9_0", path="trade.capFloor")
	@RosettaSynonym(value="capFloorStream", source="CME_ClearedConfirm_1_17", path="trade.capFloor")
	@RosettaSynonym(value="fra", source="FpML_5_10", path="trade")
	@RosettaSynonym(value="fra", source="DTCC_11_0", path="trade")
	@RosettaSynonym(value="fra", source="DTCC_9_0", path="trade")
	@RosettaSynonym(value="fra", source="CME_ClearedConfirm_1_17", path="trade")
	@RosettaSynonym(value="fra", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="interestLeg", source="FpML_5_10", path="trade.returnSwap")
	@RosettaSynonym(value="interestLeg", source="FpML_5_10", path="trade.equitySwapTransactionSupplement")
	@RosettaSynonym(value="interestLeg", source="CME_SubmissionIRS_1_0", path="trade.returnSwap")
	@RosettaSynonym(value="interestLeg", source="CME_SubmissionIRS_1_0", path="trade.equitySwapTransactionSupplement")
	@RosettaSynonym(value="interestLeg", source="DTCC_11_0", path="trade.returnSwap")
	@RosettaSynonym(value="interestLeg", source="DTCC_11_0", path="trade.equitySwapTransactionSupplement")
	@RosettaSynonym(value="interestLeg", source="DTCC_9_0", path="trade.returnSwap")
	@RosettaSynonym(value="interestLeg", source="DTCC_9_0", path="trade.equitySwapTransactionSupplement")
	@RosettaSynonym(value="interestLeg", source="CME_ClearedConfirm_1_17", path="trade.returnSwap")
	@RosettaSynonym(value="interestLeg", source="CME_ClearedConfirm_1_17", path="trade.equitySwapTransactionSupplement")
	@RosettaSynonym(value="repo", source="FpML_5_10", path="trade")
	@RosettaSynonym(value="repo", source="FpML_5_10")
	public final List<InterestRatePayout> getInterestRatePayout() {
		return interestRatePayout;
	}
	
	/**
	 * The option payout.
	 */
	@RosettaSynonym(value="swaption", source="FpML_5_10", path="trade")
	@RosettaSynonym(value="swaption", source="CME_SubmissionIRS_1_0", path="trade")
	@RosettaSynonym(value="swaption", source="DTCC_11_0", path="trade")
	@RosettaSynonym(value="swaption", source="DTCC_9_0", path="trade")
	@RosettaSynonym(value="swaption", source="CME_ClearedConfirm_1_17", path="trade")
	@RosettaSynonym(value="swaption", source="FpML_5_10")
	@RosettaSynonym(value="swaption", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="swaption", source="DTCC_11_0")
	@RosettaSynonym(value="swaption", source="DTCC_9_0")
	@RosettaSynonym(value="swaption", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="creditDefaultSwapOption", source="FpML_5_10", path="trade")
	@RosettaSynonym(value="creditDefaultSwapOption", source="CME_SubmissionIRS_1_0", path="trade")
	@RosettaSynonym(value="creditDefaultSwapOption", source="DTCC_11_0", path="trade")
	@RosettaSynonym(value="creditDefaultSwapOption", source="DTCC_9_0", path="trade")
	@RosettaSynonym(value="creditDefaultSwapOption", source="CME_ClearedConfirm_1_17", path="trade")
	@RosettaSynonym(value="creditDefaultSwapOption", source="FpML_5_10")
	@RosettaSynonym(value="creditDefaultSwapOption", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="creditDefaultSwapOption", source="DTCC_11_0")
	@RosettaSynonym(value="creditDefaultSwapOption", source="DTCC_9_0")
	@RosettaSynonym(value="creditDefaultSwapOption", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="bondOption", source="FpML_5_10", path="trade")
	@RosettaSynonym(value="bondOption", source="CME_SubmissionIRS_1_0", path="trade")
	@RosettaSynonym(value="bondOption", source="DTCC_11_0", path="trade")
	@RosettaSynonym(value="bondOption", source="DTCC_9_0", path="trade")
	@RosettaSynonym(value="bondOption", source="CME_ClearedConfirm_1_17", path="trade")
	@RosettaSynonym(value="bondOption", source="FpML_5_10")
	@RosettaSynonym(value="bondOption", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="bondOption", source="DTCC_11_0")
	@RosettaSynonym(value="bondOption", source="DTCC_9_0")
	@RosettaSynonym(value="bondOption", source="CME_ClearedConfirm_1_17")
	public final List<OptionPayout> getOptionPayout() {
		return optionPayout;
	}
	
	/**
	 * The security payout when the product involves some form of securities, such as collateral in a securities financing transaction
	 */
	@RosettaSynonym(value="repo", source="FpML_5_10", path="trade")
	@RosettaSynonym(value="repo", source="FpML_5_10")
	public final List<SecurityPayout> getSecurityPayout() {
		return securityPayout;
	}
	
	
	@Override
	public RosettaMetaData<? extends Payout> metaData() {
		return metaData;
	} 
	
	public PayoutBuilder toBuilder() {
		PayoutBuilder builder = new PayoutBuilder();
		ofNullable(getCashflow()).ifPresent(cashflow -> cashflow.forEach(builder::addCashflow));
		ofNullable(getCreditDefaultPayout()).ifPresent(builder::setCreditDefaultPayout);
		ofNullable(getEquityPayout()).ifPresent(equityPayout -> equityPayout.forEach(builder::addEquityPayout));
		ofNullable(getForwardPayout()).ifPresent(forwardPayout -> forwardPayout.forEach(builder::addForwardPayout));
		ofNullable(getInterestRatePayout()).ifPresent(interestRatePayout -> interestRatePayout.forEach(builder::addInterestRatePayout));
		ofNullable(getOptionPayout()).ifPresent(optionPayout -> optionPayout.forEach(builder::addOptionPayout));
		ofNullable(getSecurityPayout()).ifPresent(securityPayout -> securityPayout.forEach(builder::addSecurityPayout));
		return builder;
	}

	public static PayoutBuilder builder() {
		return new PayoutBuilder();
	}

	public static  class PayoutBuilder extends RosettaModelObjectBuilder {
	
		protected List<Cashflow.CashflowBuilder> cashflow;
		protected CreditDefaultPayout.CreditDefaultPayoutBuilder creditDefaultPayout;
		protected List<EquityPayout.EquityPayoutBuilder> equityPayout;
		protected List<ForwardPayout.ForwardPayoutBuilder> forwardPayout;
		protected List<InterestRatePayout.InterestRatePayoutBuilder> interestRatePayout;
		protected List<OptionPayout.OptionPayoutBuilder> optionPayout;
		protected List<SecurityPayout.SecurityPayoutBuilder> securityPayout;
	
		public PayoutBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends Payout> metaData() {
			return metaData;
		} 
	
		
		public List<Cashflow.CashflowBuilder> getCashflow() {
			return cashflow;
		}
		
		public Cashflow.CashflowBuilder getOrCreateCashflow(int index) {
			if (cashflow==null) {
				this.cashflow = new ArrayList<>();
			}
			return getIndex(cashflow, index, ()->new Cashflow.CashflowBuilder());
		}
		
		
		public CreditDefaultPayout.CreditDefaultPayoutBuilder getCreditDefaultPayout() {
			return creditDefaultPayout;
		}
		
		public CreditDefaultPayout.CreditDefaultPayoutBuilder getOrCreateCreditDefaultPayout() {
			if (creditDefaultPayout!=null) {
				return creditDefaultPayout;
			}
			else return creditDefaultPayout = new CreditDefaultPayout.CreditDefaultPayoutBuilder();
		}
		
		
		public List<EquityPayout.EquityPayoutBuilder> getEquityPayout() {
			return equityPayout;
		}
		
		public EquityPayout.EquityPayoutBuilder getOrCreateEquityPayout(int index) {
			if (equityPayout==null) {
				this.equityPayout = new ArrayList<>();
			}
			return getIndex(equityPayout, index, ()->new EquityPayout.EquityPayoutBuilder());
		}
		
		
		public List<ForwardPayout.ForwardPayoutBuilder> getForwardPayout() {
			return forwardPayout;
		}
		
		public ForwardPayout.ForwardPayoutBuilder getOrCreateForwardPayout(int index) {
			if (forwardPayout==null) {
				this.forwardPayout = new ArrayList<>();
			}
			return getIndex(forwardPayout, index, ()->new ForwardPayout.ForwardPayoutBuilder());
		}
		
		
		public List<InterestRatePayout.InterestRatePayoutBuilder> getInterestRatePayout() {
			return interestRatePayout;
		}
		
		public InterestRatePayout.InterestRatePayoutBuilder getOrCreateInterestRatePayout(int index) {
			if (interestRatePayout==null) {
				this.interestRatePayout = new ArrayList<>();
			}
			return getIndex(interestRatePayout, index, ()->new InterestRatePayout.InterestRatePayoutBuilder());
		}
		
		
		public List<OptionPayout.OptionPayoutBuilder> getOptionPayout() {
			return optionPayout;
		}
		
		public OptionPayout.OptionPayoutBuilder getOrCreateOptionPayout(int index) {
			if (optionPayout==null) {
				this.optionPayout = new ArrayList<>();
			}
			return getIndex(optionPayout, index, ()->new OptionPayout.OptionPayoutBuilder());
		}
		
		
		public List<SecurityPayout.SecurityPayoutBuilder> getSecurityPayout() {
			return securityPayout;
		}
		
		public SecurityPayout.SecurityPayoutBuilder getOrCreateSecurityPayout(int index) {
			if (securityPayout==null) {
				this.securityPayout = new ArrayList<>();
			}
			return getIndex(securityPayout, index, ()->new SecurityPayout.SecurityPayoutBuilder());
		}
		
	
		public PayoutBuilder addCashflow(Cashflow cashflow) {
			if(this.cashflow == null){
				this.cashflow = new ArrayList<>();
			}
			this.cashflow.add(cashflow.toBuilder());
			return this;
		}
		public PayoutBuilder addCashflow(Cashflow cashflow, int _idx) {
			if(this.cashflow == null){
				this.cashflow = new ArrayList<>();
			}
			getIndex(this.cashflow, _idx, () -> cashflow.toBuilder());
			this.cashflow.set(_idx, cashflow.toBuilder());
			return this;
		}
		public PayoutBuilder addCashflow(List<Cashflow> cashflows) {
			if(this.cashflow == null){
				this.cashflow = new ArrayList<>();
			}
			for (Cashflow toAdd : cashflows) {
				this.cashflow.add(toAdd.toBuilder());
			}
			return this;
		}
		public PayoutBuilder addCashflowBuilder(Cashflow.CashflowBuilder cashflow) {
			if(this.cashflow == null){
				this.cashflow = new ArrayList<>();
			}
			this.cashflow.add(cashflow);
			return this;
		}
		
		public PayoutBuilder clearCashflow() {
			this.cashflow = null;
			return this;
		}
		public PayoutBuilder setCreditDefaultPayout(CreditDefaultPayout creditDefaultPayout) {
			this.creditDefaultPayout = creditDefaultPayout.toBuilder();
			return this;
		}
		public PayoutBuilder setCreditDefaultPayoutBuilder(CreditDefaultPayout.CreditDefaultPayoutBuilder creditDefaultPayout) {
			this.creditDefaultPayout = creditDefaultPayout;
			return this;
		}
		public PayoutBuilder addEquityPayout(EquityPayout equityPayout) {
			if(this.equityPayout == null){
				this.equityPayout = new ArrayList<>();
			}
			this.equityPayout.add(equityPayout.toBuilder());
			return this;
		}
		public PayoutBuilder addEquityPayout(EquityPayout equityPayout, int _idx) {
			if(this.equityPayout == null){
				this.equityPayout = new ArrayList<>();
			}
			getIndex(this.equityPayout, _idx, () -> equityPayout.toBuilder());
			this.equityPayout.set(_idx, equityPayout.toBuilder());
			return this;
		}
		public PayoutBuilder addEquityPayout(List<EquityPayout> equityPayouts) {
			if(this.equityPayout == null){
				this.equityPayout = new ArrayList<>();
			}
			for (EquityPayout toAdd : equityPayouts) {
				this.equityPayout.add(toAdd.toBuilder());
			}
			return this;
		}
		public PayoutBuilder addEquityPayoutBuilder(EquityPayout.EquityPayoutBuilder equityPayout) {
			if(this.equityPayout == null){
				this.equityPayout = new ArrayList<>();
			}
			this.equityPayout.add(equityPayout);
			return this;
		}
		
		public PayoutBuilder clearEquityPayout() {
			this.equityPayout = null;
			return this;
		}
		public PayoutBuilder addForwardPayout(ForwardPayout forwardPayout) {
			if(this.forwardPayout == null){
				this.forwardPayout = new ArrayList<>();
			}
			this.forwardPayout.add(forwardPayout.toBuilder());
			return this;
		}
		public PayoutBuilder addForwardPayout(ForwardPayout forwardPayout, int _idx) {
			if(this.forwardPayout == null){
				this.forwardPayout = new ArrayList<>();
			}
			getIndex(this.forwardPayout, _idx, () -> forwardPayout.toBuilder());
			this.forwardPayout.set(_idx, forwardPayout.toBuilder());
			return this;
		}
		public PayoutBuilder addForwardPayout(List<ForwardPayout> forwardPayouts) {
			if(this.forwardPayout == null){
				this.forwardPayout = new ArrayList<>();
			}
			for (ForwardPayout toAdd : forwardPayouts) {
				this.forwardPayout.add(toAdd.toBuilder());
			}
			return this;
		}
		public PayoutBuilder addForwardPayoutBuilder(ForwardPayout.ForwardPayoutBuilder forwardPayout) {
			if(this.forwardPayout == null){
				this.forwardPayout = new ArrayList<>();
			}
			this.forwardPayout.add(forwardPayout);
			return this;
		}
		
		public PayoutBuilder clearForwardPayout() {
			this.forwardPayout = null;
			return this;
		}
		public PayoutBuilder addInterestRatePayout(InterestRatePayout interestRatePayout) {
			if(this.interestRatePayout == null){
				this.interestRatePayout = new ArrayList<>();
			}
			this.interestRatePayout.add(interestRatePayout.toBuilder());
			return this;
		}
		public PayoutBuilder addInterestRatePayout(InterestRatePayout interestRatePayout, int _idx) {
			if(this.interestRatePayout == null){
				this.interestRatePayout = new ArrayList<>();
			}
			getIndex(this.interestRatePayout, _idx, () -> interestRatePayout.toBuilder());
			this.interestRatePayout.set(_idx, interestRatePayout.toBuilder());
			return this;
		}
		public PayoutBuilder addInterestRatePayout(List<InterestRatePayout> interestRatePayouts) {
			if(this.interestRatePayout == null){
				this.interestRatePayout = new ArrayList<>();
			}
			for (InterestRatePayout toAdd : interestRatePayouts) {
				this.interestRatePayout.add(toAdd.toBuilder());
			}
			return this;
		}
		public PayoutBuilder addInterestRatePayoutBuilder(InterestRatePayout.InterestRatePayoutBuilder interestRatePayout) {
			if(this.interestRatePayout == null){
				this.interestRatePayout = new ArrayList<>();
			}
			this.interestRatePayout.add(interestRatePayout);
			return this;
		}
		
		public PayoutBuilder clearInterestRatePayout() {
			this.interestRatePayout = null;
			return this;
		}
		public PayoutBuilder addOptionPayout(OptionPayout optionPayout) {
			if(this.optionPayout == null){
				this.optionPayout = new ArrayList<>();
			}
			this.optionPayout.add(optionPayout.toBuilder());
			return this;
		}
		public PayoutBuilder addOptionPayout(OptionPayout optionPayout, int _idx) {
			if(this.optionPayout == null){
				this.optionPayout = new ArrayList<>();
			}
			getIndex(this.optionPayout, _idx, () -> optionPayout.toBuilder());
			this.optionPayout.set(_idx, optionPayout.toBuilder());
			return this;
		}
		public PayoutBuilder addOptionPayout(List<OptionPayout> optionPayouts) {
			if(this.optionPayout == null){
				this.optionPayout = new ArrayList<>();
			}
			for (OptionPayout toAdd : optionPayouts) {
				this.optionPayout.add(toAdd.toBuilder());
			}
			return this;
		}
		public PayoutBuilder addOptionPayoutBuilder(OptionPayout.OptionPayoutBuilder optionPayout) {
			if(this.optionPayout == null){
				this.optionPayout = new ArrayList<>();
			}
			this.optionPayout.add(optionPayout);
			return this;
		}
		
		public PayoutBuilder clearOptionPayout() {
			this.optionPayout = null;
			return this;
		}
		public PayoutBuilder addSecurityPayout(SecurityPayout securityPayout) {
			if(this.securityPayout == null){
				this.securityPayout = new ArrayList<>();
			}
			this.securityPayout.add(securityPayout.toBuilder());
			return this;
		}
		public PayoutBuilder addSecurityPayout(SecurityPayout securityPayout, int _idx) {
			if(this.securityPayout == null){
				this.securityPayout = new ArrayList<>();
			}
			getIndex(this.securityPayout, _idx, () -> securityPayout.toBuilder());
			this.securityPayout.set(_idx, securityPayout.toBuilder());
			return this;
		}
		public PayoutBuilder addSecurityPayout(List<SecurityPayout> securityPayouts) {
			if(this.securityPayout == null){
				this.securityPayout = new ArrayList<>();
			}
			for (SecurityPayout toAdd : securityPayouts) {
				this.securityPayout.add(toAdd.toBuilder());
			}
			return this;
		}
		public PayoutBuilder addSecurityPayoutBuilder(SecurityPayout.SecurityPayoutBuilder securityPayout) {
			if(this.securityPayout == null){
				this.securityPayout = new ArrayList<>();
			}
			this.securityPayout.add(securityPayout);
			return this;
		}
		
		public PayoutBuilder clearSecurityPayout() {
			this.securityPayout = null;
			return this;
		}
		
		public Payout build() {
			return new Payout(this);
		}
	
		@Override
		public PayoutBuilder prune() {
			if (cashflow!=null) cashflow = cashflow.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (creditDefaultPayout!=null && !creditDefaultPayout.prune().hasData()) creditDefaultPayout = null;
			if (equityPayout!=null) equityPayout = equityPayout.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (forwardPayout!=null) forwardPayout = forwardPayout.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (interestRatePayout!=null) interestRatePayout = interestRatePayout.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (optionPayout!=null) optionPayout = optionPayout.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (securityPayout!=null) securityPayout = securityPayout.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getCashflow()!=null && getCashflow().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getCreditDefaultPayout()!=null && getCreditDefaultPayout().hasData()) return true;
			if (getEquityPayout()!=null && getEquityPayout().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getForwardPayout()!=null && getForwardPayout().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getInterestRatePayout()!=null && getInterestRatePayout().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getOptionPayout()!=null && getOptionPayout().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getSecurityPayout()!=null && getSecurityPayout().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("cashflow"), processor, Cashflow.class, cashflow);
			processRosetta(path.newSubPath("creditDefaultPayout"), processor, CreditDefaultPayout.class, creditDefaultPayout);
			processRosetta(path.newSubPath("equityPayout"), processor, EquityPayout.class, equityPayout);
			processRosetta(path.newSubPath("forwardPayout"), processor, ForwardPayout.class, forwardPayout);
			processRosetta(path.newSubPath("interestRatePayout"), processor, InterestRatePayout.class, interestRatePayout);
			processRosetta(path.newSubPath("optionPayout"), processor, OptionPayout.class, optionPayout);
			processRosetta(path.newSubPath("securityPayout"), processor, SecurityPayout.class, securityPayout);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			PayoutBuilder _that = (PayoutBuilder) o;
		
			if (!ListEquals.listEquals(cashflow, _that.cashflow)) return false;
			if (creditDefaultPayout != null ? !creditDefaultPayout.equals(_that.creditDefaultPayout) : _that.creditDefaultPayout != null) return false;
			if (!ListEquals.listEquals(equityPayout, _that.equityPayout)) return false;
			if (!ListEquals.listEquals(forwardPayout, _that.forwardPayout)) return false;
			if (!ListEquals.listEquals(interestRatePayout, _that.interestRatePayout)) return false;
			if (!ListEquals.listEquals(optionPayout, _that.optionPayout)) return false;
			if (!ListEquals.listEquals(securityPayout, _that.securityPayout)) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (cashflow != null ? cashflow.hashCode() : 0);
			_result = 31 * _result + (creditDefaultPayout != null ? creditDefaultPayout.hashCode() : 0);
			_result = 31 * _result + (equityPayout != null ? equityPayout.hashCode() : 0);
			_result = 31 * _result + (forwardPayout != null ? forwardPayout.hashCode() : 0);
			_result = 31 * _result + (interestRatePayout != null ? interestRatePayout.hashCode() : 0);
			_result = 31 * _result + (optionPayout != null ? optionPayout.hashCode() : 0);
			_result = 31 * _result + (securityPayout != null ? securityPayout.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "PayoutBuilder {" +
				"cashflow=" + this.cashflow + ", " +
				"creditDefaultPayout=" + this.creditDefaultPayout + ", " +
				"equityPayout=" + this.equityPayout + ", " +
				"forwardPayout=" + this.forwardPayout + ", " +
				"interestRatePayout=" + this.interestRatePayout + ", " +
				"optionPayout=" + this.optionPayout + ", " +
				"securityPayout=" + this.securityPayout +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("cashflow"), processor, Cashflow.class, cashflow);
		processRosetta(path.newSubPath("creditDefaultPayout"), processor, CreditDefaultPayout.class, creditDefaultPayout);
		processRosetta(path.newSubPath("equityPayout"), processor, EquityPayout.class, equityPayout);
		processRosetta(path.newSubPath("forwardPayout"), processor, ForwardPayout.class, forwardPayout);
		processRosetta(path.newSubPath("interestRatePayout"), processor, InterestRatePayout.class, interestRatePayout);
		processRosetta(path.newSubPath("optionPayout"), processor, OptionPayout.class, optionPayout);
		processRosetta(path.newSubPath("securityPayout"), processor, SecurityPayout.class, securityPayout);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		Payout _that = (Payout) o;
	
		if (!ListEquals.listEquals(cashflow, _that.cashflow)) return false;
		if (creditDefaultPayout != null ? !creditDefaultPayout.equals(_that.creditDefaultPayout) : _that.creditDefaultPayout != null) return false;
		if (!ListEquals.listEquals(equityPayout, _that.equityPayout)) return false;
		if (!ListEquals.listEquals(forwardPayout, _that.forwardPayout)) return false;
		if (!ListEquals.listEquals(interestRatePayout, _that.interestRatePayout)) return false;
		if (!ListEquals.listEquals(optionPayout, _that.optionPayout)) return false;
		if (!ListEquals.listEquals(securityPayout, _that.securityPayout)) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (cashflow != null ? cashflow.hashCode() : 0);
		_result = 31 * _result + (creditDefaultPayout != null ? creditDefaultPayout.hashCode() : 0);
		_result = 31 * _result + (equityPayout != null ? equityPayout.hashCode() : 0);
		_result = 31 * _result + (forwardPayout != null ? forwardPayout.hashCode() : 0);
		_result = 31 * _result + (interestRatePayout != null ? interestRatePayout.hashCode() : 0);
		_result = 31 * _result + (optionPayout != null ? optionPayout.hashCode() : 0);
		_result = 31 * _result + (securityPayout != null ? securityPayout.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "Payout {" +
			"cashflow=" + this.cashflow + ", " +
			"creditDefaultPayout=" + this.creditDefaultPayout + ", " +
			"equityPayout=" + this.equityPayout + ", " +
			"forwardPayout=" + this.forwardPayout + ", " +
			"interestRatePayout=" + this.interestRatePayout + ", " +
			"optionPayout=" + this.optionPayout + ", " +
			"securityPayout=" + this.securityPayout +
		'}';
	}
}
