package org.isda.cdm;

import org.isda.cdm.ResolvablePayoutQuantity;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.PayoutBaseMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * Base class that all payout types should extend. Use case is that some validation rules may need to apply across all payout types, for which the data rule can be written at the base class level
 * 
 * @version 2.5.4
 */
@RosettaClass
public abstract class PayoutBase extends RosettaModelObject {
	private final ResolvablePayoutQuantity payoutQuantity;
	private static PayoutBaseMeta metaData = new PayoutBaseMeta();
	
	PayoutBase(PayoutBaseBuilder builder) {
		this.payoutQuantity = ofNullable(builder.getPayoutQuantity()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * Each payout leg must implement the quantity concept as a &#39;resolvable&#39; type, which allows for different payout legs to be linked to each other (e.g. in the case of cross-curreny products).
	 */
	@RosettaSynonym(value="notional,rateOfReturn", source="FpML_5_10")
	@RosettaSynonym(value="notionalSchedule", source="FpML_5_10", path="calculationPeriodAmount.calculation")
	@RosettaSynonym(value="notionalSchedule", source="CME_SubmissionIRS_1_0", path="calculationPeriodAmount.calculation")
	@RosettaSynonym(value="notionalSchedule", source="DTCC_11_0", path="calculationPeriodAmount.calculation")
	@RosettaSynonym(value="notionalSchedule", source="DTCC_9_0", path="calculationPeriodAmount.calculation")
	@RosettaSynonym(value="notionalSchedule", source="CME_ClearedConfirm_1_17", path="calculationPeriodAmount.calculation")
	public final ResolvablePayoutQuantity getPayoutQuantity() {
		return payoutQuantity;
	}
	
	
	@Override
	public RosettaMetaData<? extends PayoutBase> metaData() {
		return metaData;
	} 
	
	public abstract PayoutBaseBuilder toBuilder();


	public static abstract
	 class PayoutBaseBuilder extends RosettaModelObjectBuilder {
	
		protected ResolvablePayoutQuantity.ResolvablePayoutQuantityBuilder payoutQuantity;
	
		public PayoutBaseBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends PayoutBase> metaData() {
			return metaData;
		} 
	
		
		public ResolvablePayoutQuantity.ResolvablePayoutQuantityBuilder getPayoutQuantity() {
			return payoutQuantity;
		}
		
		public ResolvablePayoutQuantity.ResolvablePayoutQuantityBuilder getOrCreatePayoutQuantity() {
			if (payoutQuantity!=null) {
				return payoutQuantity;
			}
			else return payoutQuantity = new ResolvablePayoutQuantity.ResolvablePayoutQuantityBuilder();
		}
		
	
		public PayoutBaseBuilder setPayoutQuantity(ResolvablePayoutQuantity payoutQuantity) {
			this.payoutQuantity = payoutQuantity.toBuilder();
			return this;
		}
		public PayoutBaseBuilder setPayoutQuantityBuilder(ResolvablePayoutQuantity.ResolvablePayoutQuantityBuilder payoutQuantity) {
			this.payoutQuantity = payoutQuantity;
			return this;
		}
		
		public abstract PayoutBase build();
	
		@Override
		public PayoutBaseBuilder prune() {
			if (payoutQuantity!=null && !payoutQuantity.prune().hasData()) payoutQuantity = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getPayoutQuantity()!=null && getPayoutQuantity().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("payoutQuantity"), processor, ResolvablePayoutQuantity.class, payoutQuantity);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			PayoutBaseBuilder _that = (PayoutBaseBuilder) o;
		
			if (payoutQuantity != null ? !payoutQuantity.equals(_that.payoutQuantity) : _that.payoutQuantity != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (payoutQuantity != null ? payoutQuantity.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "PayoutBaseBuilder {" +
				"payoutQuantity=" + this.payoutQuantity +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("payoutQuantity"), processor, ResolvablePayoutQuantity.class, payoutQuantity);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		PayoutBase _that = (PayoutBase) o;
	
		if (payoutQuantity != null ? !payoutQuantity.equals(_that.payoutQuantity) : _that.payoutQuantity != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (payoutQuantity != null ? payoutQuantity.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "PayoutBase {" +
			"payoutQuantity=" + this.payoutQuantity +
		'}';
	}
}
