package org.isda.cdm;

import java.lang.Integer;
import org.isda.cdm.PeriodEnum;
import org.isda.cdm.metafields.MetaFields;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.PeriodMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class to define recurring periods or time offsets.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="Period", source="FpML_5_10")
@RosettaSynonym(value="Period", source="CME_SubmissionIRS_1_0")
@RosettaSynonym(value="Period", source="DTCC_11_0")
@RosettaSynonym(value="Period", source="DTCC_9_0")
@RosettaSynonym(value="Period", source="CME_ClearedConfirm_1_17")
public class Period extends RosettaModelObject implements GlobalKey {
	private final MetaFields meta;
	private final PeriodEnum period;
	private final Integer periodMultiplier;
	private static PeriodMeta metaData = new PeriodMeta();
	
	Period(PeriodBuilder builder) {
		this.meta = ofNullable(builder.getMeta()).map(f->f.build()).orElse(null);
		this.period = builder.getPeriod();
		this.periodMultiplier = builder.getPeriodMultiplier();
	}
	
	public final MetaFields getMeta() {
		return meta;
	}
	
	/**
	 * A time period, e.g. a day, week, month or year of the stream. If the periodMultiplier value is 0 (zero) then period must contain the value D (day).
	 */
	@RosettaSynonym(value="period", source="FpML_5_10")
	@RosettaSynonym(value="period", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="period", source="DTCC_11_0")
	@RosettaSynonym(value="period", source="DTCC_9_0")
	@RosettaSynonym(value="period", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="period", source="FpML_5_10", path="dateOffset")
	public final PeriodEnum getPeriod() {
		return period;
	}
	
	/**
	 * A time period multiplier, e.g. 1, 2 or 3 etc. A negative value can be used when specifying an offset relative to another date, e.g. -2 days.
	 */
	@RosettaSynonym(value="periodMultiplier", source="FpML_5_10")
	@RosettaSynonym(value="periodMultiplier", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="periodMultiplier", source="DTCC_11_0")
	@RosettaSynonym(value="periodMultiplier", source="DTCC_9_0")
	@RosettaSynonym(value="periodMultiplier", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="periodMultiplier", source="FpML_5_10", path="dateOffset")
	@RosettaSynonym(value="periodMultiplier", source="FpML_5_10", path="calculationPeriodDates.relativeEffectiveDate")
	@RosettaSynonym(value="timely_days,release_days,after_days", source="ISDA_Create_1_0")
	public final Integer getPeriodMultiplier() {
		return periodMultiplier;
	}
	
	
	@Override
	public RosettaMetaData<? extends Period> metaData() {
		return metaData;
	} 
	
	public PeriodBuilder toBuilder() {
		PeriodBuilder builder = new PeriodBuilder();
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		ofNullable(getPeriod()).ifPresent(builder::setPeriod);
		ofNullable(getPeriodMultiplier()).ifPresent(builder::setPeriodMultiplier);
		return builder;
	}

	public static PeriodBuilder builder() {
		return new PeriodBuilder();
	}

	public static  class PeriodBuilder extends RosettaModelObjectBuilder implements GlobalKeyBuilder<PeriodBuilder> {
	
		protected MetaFields.MetaFieldsBuilder meta;
		protected PeriodEnum period;
		protected Integer periodMultiplier;
	
		public PeriodBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends Period> metaData() {
			return metaData;
		} 
	
		
		public MetaFields.MetaFieldsBuilder getMeta() {
			return meta;
		}
		
		public MetaFields.MetaFieldsBuilder getOrCreateMeta() {
			if (meta!=null) {
				return meta;
			}
			else return meta = new MetaFields.MetaFieldsBuilder();
		}
		
		
		public PeriodEnum getPeriod() {
			return period;
		}
		
		
		public Integer getPeriodMultiplier() {
			return periodMultiplier;
		}
		
	
		public PeriodBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		public PeriodBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		public PeriodBuilder setPeriod(PeriodEnum period) {
			this.period = period;
			return this;
		}
		public PeriodBuilder setPeriodMultiplier(Integer periodMultiplier) {
			this.periodMultiplier = periodMultiplier;
			return this;
		}
		
		public Period build() {
			return new Period(this);
		}
	
		@Override
		public PeriodBuilder prune() {
			if (meta!=null && !meta.prune().hasData()) meta = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getMeta()!=null && getMeta().hasData()) return true;
			if (getPeriod()!=null) return true;
			if (getPeriodMultiplier()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("period"), PeriodEnum.class, period, this);
			processor.processBasic(path.newSubPath("periodMultiplier"), Integer.class, periodMultiplier, this);
			
			processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			PeriodBuilder _that = (PeriodBuilder) o;
		
			if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
			if (period != null ? !period.equals(_that.period) : _that.period != null) return false;
			if (periodMultiplier != null ? !periodMultiplier.equals(_that.periodMultiplier) : _that.periodMultiplier != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
			_result = 31 * _result + (period != null ? period.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (periodMultiplier != null ? periodMultiplier.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "PeriodBuilder {" +
				"meta=" + this.meta + ", " +
				"period=" + this.period + ", " +
				"periodMultiplier=" + this.periodMultiplier +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("period"), PeriodEnum.class, period, this);
		processor.processBasic(path.newSubPath("periodMultiplier"), Integer.class, periodMultiplier, this);
		
		processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		Period _that = (Period) o;
	
		if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
		if (period != null ? !period.equals(_that.period) : _that.period != null) return false;
		if (periodMultiplier != null ? !periodMultiplier.equals(_that.periodMultiplier) : _that.periodMultiplier != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
		_result = 31 * _result + (period != null ? period.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (periodMultiplier != null ? periodMultiplier.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "Period {" +
			"meta=" + this.meta + ", " +
			"period=" + this.period + ", " +
			"periodMultiplier=" + this.periodMultiplier +
		'}';
	}
}
