package org.isda.cdm;

import org.isda.cdm.Cashflow;
import org.isda.cdm.ExecutionQuantity;
import org.isda.cdm.Product;
import org.isda.cdm.Quantity;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.PhysicalExerciseMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * The physical exercise results into a financial product which is represented through the Product class, with an associated quantity and cashflow (e.g. a physical exercise of a bond option will result into a bond with a cash proceed which will be a function of the option strike price).
 * 
 * @version 2.5.4
 */
@RosettaClass
public class PhysicalExercise extends RosettaModelObject {
	private final Cashflow cashflow;
	private final ExecutionQuantity contractualQuantity;
	private final Product product;
	private final Quantity quantity;
	private static PhysicalExerciseMeta metaData = new PhysicalExerciseMeta();
	
	PhysicalExercise(PhysicalExerciseBuilder builder) {
		this.cashflow = ofNullable(builder.getCashflow()).map(f->f.build()).orElse(null);
		this.contractualQuantity = ofNullable(builder.getContractualQuantity()).map(f->f.build()).orElse(null);
		this.product = ofNullable(builder.getProduct()).map(f->f.build()).orElse(null);
		this.quantity = ofNullable(builder.getQuantity()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The cashflow component of the physical exercise.
	 */
	@RosettaSynonym(value="cashflow", source="Rosetta_Workbench")
	public final Cashflow getCashflow() {
		return cashflow;
	}
	
	/**
	 * Complex attribute to specify the quantity of the contractual product that is being exercised. This temporarily makes uses of the &#39;ExecutionQuantity&#39; class, until such time when the latter can be renamed as, and supersede, the existing &#39;ContractualQuantity&#39; class. To be later merged with the &#39;quantity&#39; attribute to have consistent mechanism between contractual and non-contractual products.
	 */
	@RosettaSynonym(value="swap", source="Rosetta_Workbench", path="product")
	public final ExecutionQuantity getContractualQuantity() {
		return contractualQuantity;
	}
	
	@RosettaSynonym(value="product", source="Rosetta_Workbench")
	public final Product getProduct() {
		return product;
	}
	
	/**
	 * The quantity associated the asset that is physically settled. This quantity should only be associated with non-contractual products, as it is then expressed as part of the contractual terms.
	 */
	@RosettaSynonym(value="quantity", source="Rosetta_Workbench")
	public final Quantity getQuantity() {
		return quantity;
	}
	
	
	@Override
	public RosettaMetaData<? extends PhysicalExercise> metaData() {
		return metaData;
	} 
	
	public PhysicalExerciseBuilder toBuilder() {
		PhysicalExerciseBuilder builder = new PhysicalExerciseBuilder();
		ofNullable(getCashflow()).ifPresent(builder::setCashflow);
		ofNullable(getContractualQuantity()).ifPresent(builder::setContractualQuantity);
		ofNullable(getProduct()).ifPresent(builder::setProduct);
		ofNullable(getQuantity()).ifPresent(builder::setQuantity);
		return builder;
	}

	public static PhysicalExerciseBuilder builder() {
		return new PhysicalExerciseBuilder();
	}

	public static  class PhysicalExerciseBuilder extends RosettaModelObjectBuilder {
	
		protected Cashflow.CashflowBuilder cashflow;
		protected ExecutionQuantity.ExecutionQuantityBuilder contractualQuantity;
		protected Product.ProductBuilder product;
		protected Quantity.QuantityBuilder quantity;
	
		public PhysicalExerciseBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends PhysicalExercise> metaData() {
			return metaData;
		} 
	
		
		public Cashflow.CashflowBuilder getCashflow() {
			return cashflow;
		}
		
		public Cashflow.CashflowBuilder getOrCreateCashflow() {
			if (cashflow!=null) {
				return cashflow;
			}
			else return cashflow = new Cashflow.CashflowBuilder();
		}
		
		
		public ExecutionQuantity.ExecutionQuantityBuilder getContractualQuantity() {
			return contractualQuantity;
		}
		
		public ExecutionQuantity.ExecutionQuantityBuilder getOrCreateContractualQuantity() {
			if (contractualQuantity!=null) {
				return contractualQuantity;
			}
			else return contractualQuantity = new ExecutionQuantity.ExecutionQuantityBuilder();
		}
		
		
		public Product.ProductBuilder getProduct() {
			return product;
		}
		
		public Product.ProductBuilder getOrCreateProduct() {
			if (product!=null) {
				return product;
			}
			else return product = new Product.ProductBuilder();
		}
		
		
		public Quantity.QuantityBuilder getQuantity() {
			return quantity;
		}
		
		public Quantity.QuantityBuilder getOrCreateQuantity() {
			if (quantity!=null) {
				return quantity;
			}
			else return quantity = new Quantity.QuantityBuilder();
		}
		
	
		public PhysicalExerciseBuilder setCashflow(Cashflow cashflow) {
			this.cashflow = cashflow.toBuilder();
			return this;
		}
		public PhysicalExerciseBuilder setCashflowBuilder(Cashflow.CashflowBuilder cashflow) {
			this.cashflow = cashflow;
			return this;
		}
		public PhysicalExerciseBuilder setContractualQuantity(ExecutionQuantity contractualQuantity) {
			this.contractualQuantity = contractualQuantity.toBuilder();
			return this;
		}
		public PhysicalExerciseBuilder setContractualQuantityBuilder(ExecutionQuantity.ExecutionQuantityBuilder contractualQuantity) {
			this.contractualQuantity = contractualQuantity;
			return this;
		}
		public PhysicalExerciseBuilder setProduct(Product product) {
			this.product = product.toBuilder();
			return this;
		}
		public PhysicalExerciseBuilder setProductBuilder(Product.ProductBuilder product) {
			this.product = product;
			return this;
		}
		public PhysicalExerciseBuilder setQuantity(Quantity quantity) {
			this.quantity = quantity.toBuilder();
			return this;
		}
		public PhysicalExerciseBuilder setQuantityBuilder(Quantity.QuantityBuilder quantity) {
			this.quantity = quantity;
			return this;
		}
		
		public PhysicalExercise build() {
			return new PhysicalExercise(this);
		}
	
		@Override
		public PhysicalExerciseBuilder prune() {
			if (cashflow!=null && !cashflow.prune().hasData()) cashflow = null;
			if (contractualQuantity!=null && !contractualQuantity.prune().hasData()) contractualQuantity = null;
			if (product!=null && !product.prune().hasData()) product = null;
			if (quantity!=null && !quantity.prune().hasData()) quantity = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getCashflow()!=null && getCashflow().hasData()) return true;
			if (getContractualQuantity()!=null && getContractualQuantity().hasData()) return true;
			if (getProduct()!=null && getProduct().hasData()) return true;
			if (getQuantity()!=null && getQuantity().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("cashflow"), processor, Cashflow.class, cashflow);
			processRosetta(path.newSubPath("contractualQuantity"), processor, ExecutionQuantity.class, contractualQuantity);
			processRosetta(path.newSubPath("product"), processor, Product.class, product);
			processRosetta(path.newSubPath("quantity"), processor, Quantity.class, quantity);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			PhysicalExerciseBuilder _that = (PhysicalExerciseBuilder) o;
		
			if (cashflow != null ? !cashflow.equals(_that.cashflow) : _that.cashflow != null) return false;
			if (contractualQuantity != null ? !contractualQuantity.equals(_that.contractualQuantity) : _that.contractualQuantity != null) return false;
			if (product != null ? !product.equals(_that.product) : _that.product != null) return false;
			if (quantity != null ? !quantity.equals(_that.quantity) : _that.quantity != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (cashflow != null ? cashflow.hashCode() : 0);
			_result = 31 * _result + (contractualQuantity != null ? contractualQuantity.hashCode() : 0);
			_result = 31 * _result + (product != null ? product.hashCode() : 0);
			_result = 31 * _result + (quantity != null ? quantity.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "PhysicalExerciseBuilder {" +
				"cashflow=" + this.cashflow + ", " +
				"contractualQuantity=" + this.contractualQuantity + ", " +
				"product=" + this.product + ", " +
				"quantity=" + this.quantity +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("cashflow"), processor, Cashflow.class, cashflow);
		processRosetta(path.newSubPath("contractualQuantity"), processor, ExecutionQuantity.class, contractualQuantity);
		processRosetta(path.newSubPath("product"), processor, Product.class, product);
		processRosetta(path.newSubPath("quantity"), processor, Quantity.class, quantity);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		PhysicalExercise _that = (PhysicalExercise) o;
	
		if (cashflow != null ? !cashflow.equals(_that.cashflow) : _that.cashflow != null) return false;
		if (contractualQuantity != null ? !contractualQuantity.equals(_that.contractualQuantity) : _that.contractualQuantity != null) return false;
		if (product != null ? !product.equals(_that.product) : _that.product != null) return false;
		if (quantity != null ? !quantity.equals(_that.quantity) : _that.quantity != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (cashflow != null ? cashflow.hashCode() : 0);
		_result = 31 * _result + (contractualQuantity != null ? contractualQuantity.hashCode() : 0);
		_result = 31 * _result + (product != null ? product.hashCode() : 0);
		_result = 31 * _result + (quantity != null ? quantity.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "PhysicalExercise {" +
			"cashflow=" + this.cashflow + ", " +
			"contractualQuantity=" + this.contractualQuantity + ", " +
			"product=" + this.product + ", " +
			"quantity=" + this.quantity +
		'}';
	}
}
