package org.isda.cdm;

import java.lang.Boolean;
import java.lang.Integer;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.PhysicalSettlementPeriodMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="PhysicalSettlementPeriod", source="FpML_5_10")
public class PhysicalSettlementPeriod extends RosettaModelObject {
	private final Integer businessDays;
	private final Boolean businessDaysNotSpecified;
	private final Integer maximumBusinessDays;
	private static PhysicalSettlementPeriodMeta metaData = new PhysicalSettlementPeriodMeta();
	
	PhysicalSettlementPeriod(PhysicalSettlementPeriodBuilder builder) {
		this.businessDays = builder.getBusinessDays();
		this.businessDaysNotSpecified = builder.getBusinessDaysNotSpecified();
		this.maximumBusinessDays = builder.getMaximumBusinessDays();
	}
	
	/**
	 * A number of business days. Its precise meaning is dependant on the context in which this element is used. ISDA 2003 Term: Business Day.
	 */
	@RosettaSynonym(value="businessDays", source="FpML_5_10")
	@RosettaSynonym(value="businessDays", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="businessDays", source="DTCC_11_0")
	@RosettaSynonym(value="businessDays", source="DTCC_9_0")
	@RosettaSynonym(value="businessDays", source="CME_ClearedConfirm_1_17")
	public final Integer getBusinessDays() {
		return businessDays;
	}
	
	/**
	 * An explicit indication that a number of business days are not specified and therefore ISDA fallback provisions should apply.
	 */
	@RosettaSynonym(value="businessDaysNotSpecified", source="FpML_5_10")
	@RosettaSynonym(value="businessDaysNotSpecified", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="businessDaysNotSpecified", source="DTCC_11_0")
	@RosettaSynonym(value="businessDaysNotSpecified", source="DTCC_9_0")
	@RosettaSynonym(value="businessDaysNotSpecified", source="CME_ClearedConfirm_1_17")
	public final Boolean getBusinessDaysNotSpecified() {
		return businessDaysNotSpecified;
	}
	
	/**
	 * A maximum number of business days. Its precise meaning is dependant on the context in which this element is used. Intended to be used to limit a particular ISDA fallback provision.
	 */
	@RosettaSynonym(value="maximumBusinessDays", source="FpML_5_10")
	@RosettaSynonym(value="maximumBusinessDays", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="maximumBusinessDays", source="DTCC_11_0")
	@RosettaSynonym(value="maximumBusinessDays", source="DTCC_9_0")
	@RosettaSynonym(value="maximumBusinessDays", source="CME_ClearedConfirm_1_17")
	public final Integer getMaximumBusinessDays() {
		return maximumBusinessDays;
	}
	
	
	@Override
	public RosettaMetaData<? extends PhysicalSettlementPeriod> metaData() {
		return metaData;
	} 
	
	public PhysicalSettlementPeriodBuilder toBuilder() {
		PhysicalSettlementPeriodBuilder builder = new PhysicalSettlementPeriodBuilder();
		ofNullable(getBusinessDays()).ifPresent(builder::setBusinessDays);
		ofNullable(getBusinessDaysNotSpecified()).ifPresent(builder::setBusinessDaysNotSpecified);
		ofNullable(getMaximumBusinessDays()).ifPresent(builder::setMaximumBusinessDays);
		return builder;
	}

	public static PhysicalSettlementPeriodBuilder builder() {
		return new PhysicalSettlementPeriodBuilder();
	}

	public static  class PhysicalSettlementPeriodBuilder extends RosettaModelObjectBuilder {
	
		protected Integer businessDays;
		protected Boolean businessDaysNotSpecified;
		protected Integer maximumBusinessDays;
	
		public PhysicalSettlementPeriodBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends PhysicalSettlementPeriod> metaData() {
			return metaData;
		} 
	
		
		public Integer getBusinessDays() {
			return businessDays;
		}
		
		
		public Boolean getBusinessDaysNotSpecified() {
			return businessDaysNotSpecified;
		}
		
		
		public Integer getMaximumBusinessDays() {
			return maximumBusinessDays;
		}
		
	
		public PhysicalSettlementPeriodBuilder setBusinessDays(Integer businessDays) {
			this.businessDays = businessDays;
			return this;
		}
		public PhysicalSettlementPeriodBuilder setBusinessDaysNotSpecified(Boolean businessDaysNotSpecified) {
			this.businessDaysNotSpecified = businessDaysNotSpecified;
			return this;
		}
		public PhysicalSettlementPeriodBuilder setMaximumBusinessDays(Integer maximumBusinessDays) {
			this.maximumBusinessDays = maximumBusinessDays;
			return this;
		}
		
		public PhysicalSettlementPeriod build() {
			return new PhysicalSettlementPeriod(this);
		}
	
		@Override
		public PhysicalSettlementPeriodBuilder prune() {
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getBusinessDays()!=null) return true;
			if (getBusinessDaysNotSpecified()!=null) return true;
			if (getMaximumBusinessDays()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("businessDays"), Integer.class, businessDays, this);
			processor.processBasic(path.newSubPath("businessDaysNotSpecified"), Boolean.class, businessDaysNotSpecified, this);
			processor.processBasic(path.newSubPath("maximumBusinessDays"), Integer.class, maximumBusinessDays, this);
			
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			PhysicalSettlementPeriodBuilder _that = (PhysicalSettlementPeriodBuilder) o;
		
			if (businessDays != null ? !businessDays.equals(_that.businessDays) : _that.businessDays != null) return false;
			if (businessDaysNotSpecified != null ? !businessDaysNotSpecified.equals(_that.businessDaysNotSpecified) : _that.businessDaysNotSpecified != null) return false;
			if (maximumBusinessDays != null ? !maximumBusinessDays.equals(_that.maximumBusinessDays) : _that.maximumBusinessDays != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (businessDays != null ? businessDays.hashCode() : 0);
			_result = 31 * _result + (businessDaysNotSpecified != null ? businessDaysNotSpecified.hashCode() : 0);
			_result = 31 * _result + (maximumBusinessDays != null ? maximumBusinessDays.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "PhysicalSettlementPeriodBuilder {" +
				"businessDays=" + this.businessDays + ", " +
				"businessDaysNotSpecified=" + this.businessDaysNotSpecified + ", " +
				"maximumBusinessDays=" + this.maximumBusinessDays +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("businessDays"), Integer.class, businessDays, this);
		processor.processBasic(path.newSubPath("businessDaysNotSpecified"), Boolean.class, businessDaysNotSpecified, this);
		processor.processBasic(path.newSubPath("maximumBusinessDays"), Integer.class, maximumBusinessDays, this);
		
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		PhysicalSettlementPeriod _that = (PhysicalSettlementPeriod) o;
	
		if (businessDays != null ? !businessDays.equals(_that.businessDays) : _that.businessDays != null) return false;
		if (businessDaysNotSpecified != null ? !businessDaysNotSpecified.equals(_that.businessDaysNotSpecified) : _that.businessDaysNotSpecified != null) return false;
		if (maximumBusinessDays != null ? !maximumBusinessDays.equals(_that.maximumBusinessDays) : _that.maximumBusinessDays != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (businessDays != null ? businessDays.hashCode() : 0);
		_result = 31 * _result + (businessDaysNotSpecified != null ? businessDaysNotSpecified.hashCode() : 0);
		_result = 31 * _result + (maximumBusinessDays != null ? maximumBusinessDays.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "PhysicalSettlementPeriod {" +
			"businessDays=" + this.businessDays + ", " +
			"businessDaysNotSpecified=" + this.businessDaysNotSpecified + ", " +
			"maximumBusinessDays=" + this.maximumBusinessDays +
		'}';
	}
}
