package org.isda.cdm;

import java.lang.Boolean;
import org.isda.cdm.DeliverableObligations;
import org.isda.cdm.PhysicalSettlementPeriod;
import org.isda.cdm.metafields.FieldWithMetaString;


import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.PhysicalSettlementTermsMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * In FpML, PhysicalSettlementTerms and CashSettlementTerms extend SettlementTerms.  In the CDM, this extension paradigm has not been used because SettlementTerms class has been used for purposes related to securities transactions, while it is not used as such in the FpML standard (i.e. only as an abstract construct.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="PhysicalSettlementTerms", source="FpML_5_10")
public class PhysicalSettlementTerms extends SettlementBase {
	private final DeliverableObligations deliverableObligations;
	private final Boolean escrow;
	private final PhysicalSettlementPeriod physicalSettlementPeriod;
	private final Boolean sixtyBusinessDaySettlementCap;
	private static PhysicalSettlementTermsMeta metaData = new PhysicalSettlementTermsMeta();
	
	PhysicalSettlementTerms(PhysicalSettlementTermsBuilder builder) {
		super(builder);
		this.deliverableObligations = ofNullable(builder.getDeliverableObligations()).map(f->f.build()).orElse(null);
		this.escrow = builder.getEscrow();
		this.physicalSettlementPeriod = ofNullable(builder.getPhysicalSettlementPeriod()).map(f->f.build()).orElse(null);
		this.sixtyBusinessDaySettlementCap = builder.getSixtyBusinessDaySettlementCap();
	}
	
	/**
	 * This element contains all the ISDA terms relevant to defining the deliverable obligations.
	 */
	@RosettaSynonym(value="deliverableObligations", source="FpML_5_10")
	@RosettaSynonym(value="deliverableObligations", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="deliverableObligations", source="DTCC_11_0")
	@RosettaSynonym(value="deliverableObligations", source="DTCC_9_0")
	@RosettaSynonym(value="deliverableObligations", source="CME_ClearedConfirm_1_17")
	public final DeliverableObligations getDeliverableObligations() {
		return deliverableObligations;
	}
	
	/**
	 * If this element is specified and set to &#39;true&#39;, indicates that physical settlement must take place through the use of an escrow agent. (For Canadian counterparties this is always &#39;Not Applicable&#39;. ISDA 2003 Term: Escrow.
	 */
	@RosettaSynonym(value="escrow", source="FpML_5_10")
	@RosettaSynonym(value="escrow", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="escrow", source="DTCC_11_0")
	@RosettaSynonym(value="escrow", source="DTCC_9_0")
	@RosettaSynonym(value="escrow", source="CME_ClearedConfirm_1_17")
	public final Boolean getEscrow() {
		return escrow;
	}
	
	/**
	 * The number of business days used in the determination of the physical settlement date. The physical settlement date is this number of business days after all applicable conditions to settlement are satisfied. If a number of business days is not specified fallback provisions apply for determining the number of business days. If Section 8.5/8.6 of the 1999/2003 ISDA Definitions are to apply the businessDaysNotSpecified element should be included. If a specified number of business days are to apply these should be specified in the businessDays element. If Section 8.5/8.6 of the 1999/2003 ISDA Definitions are to apply but capped at a maximum number of business days then the maximum number should be specified in the maximumBusinessDays element. ISDA 2003 Term: Physical Settlement Period.
	 */
	@RosettaSynonym(value="physicalSettlementPeriod", source="FpML_5_10")
	@RosettaSynonym(value="physicalSettlementPeriod", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="physicalSettlementPeriod", source="DTCC_11_0")
	@RosettaSynonym(value="physicalSettlementPeriod", source="DTCC_9_0")
	@RosettaSynonym(value="physicalSettlementPeriod", source="CME_ClearedConfirm_1_17")
	public final PhysicalSettlementPeriod getPhysicalSettlementPeriod() {
		return physicalSettlementPeriod;
	}
	
	/**
	 * If this element is specified and set to &#39;true&#39;, for a transaction documented under the 2003 ISDA Credit Derivatives Definitions, has the effect of incorporating the language set forth below into the confirmation. The section references are to the 2003 ISDA Credit Derivatives Definitions. Notwithstanding Section 1.7 or any provisions of Sections 9.9 or 9.10 to the contrary, but without prejudice to Section 9.3 and (where applicable) Sections 9.4, 9.5 and 9.6, if the Termination Date has not occurred on or prior to the date that is 60 Business Days following the Physical Settlement Date, such 60th Business Day shall be deemed to be the Termination Date with respect to this Transaction except in relation to any portion of the Transaction (an &#39;Affected Portion&#39;) in respect of which: (1) a valid notice of Buy-in Price has been delivered that is effective fewer than three Business Days prior to such 60th Business Day, in which case the Termination Date for that Affected Portion shall be the third Business Day following the date on which such notice is effective; or (2) Buyer has purchased but not Delivered Deliverable Obligations validly specified by Seller pursuant to Section 9.10(b), in which case the Termination Date for that Affected Portion shall be the tenth Business Day following the date on which Seller validly specified such Deliverable Obligations to Buyer.
	 */
	@RosettaSynonym(value="sixtyBusinessDaySettlementCap", source="FpML_5_10")
	@RosettaSynonym(value="sixtyBusinessDaySettlementCap", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="sixtyBusinessDaySettlementCap", source="DTCC_11_0")
	@RosettaSynonym(value="sixtyBusinessDaySettlementCap", source="DTCC_9_0")
	@RosettaSynonym(value="sixtyBusinessDaySettlementCap", source="CME_ClearedConfirm_1_17")
	public final Boolean getSixtyBusinessDaySettlementCap() {
		return sixtyBusinessDaySettlementCap;
	}
	
	
	@Override
	public RosettaMetaData<? extends PhysicalSettlementTerms> metaData() {
		return metaData;
	} 
	
	public PhysicalSettlementTermsBuilder toBuilder() {
		PhysicalSettlementTermsBuilder builder = new PhysicalSettlementTermsBuilder();
		ofNullable(getDeliverableObligations()).ifPresent(builder::setDeliverableObligations);
		ofNullable(getEscrow()).ifPresent(builder::setEscrow);
		ofNullable(getPhysicalSettlementPeriod()).ifPresent(builder::setPhysicalSettlementPeriod);
		ofNullable(getSixtyBusinessDaySettlementCap()).ifPresent(builder::setSixtyBusinessDaySettlementCap);
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		ofNullable(getSettlementCurrency()).ifPresent(builder::setSettlementCurrency);
		return builder;
	}

	public static PhysicalSettlementTermsBuilder builder() {
		return new PhysicalSettlementTermsBuilder();
	}

	public static  class PhysicalSettlementTermsBuilder extends SettlementBaseBuilder {
	
		protected DeliverableObligations.DeliverableObligationsBuilder deliverableObligations;
		protected Boolean escrow;
		protected PhysicalSettlementPeriod.PhysicalSettlementPeriodBuilder physicalSettlementPeriod;
		protected Boolean sixtyBusinessDaySettlementCap;
	
		public PhysicalSettlementTermsBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends PhysicalSettlementTerms> metaData() {
			return metaData;
		} 
	
		
		public DeliverableObligations.DeliverableObligationsBuilder getDeliverableObligations() {
			return deliverableObligations;
		}
		
		public DeliverableObligations.DeliverableObligationsBuilder getOrCreateDeliverableObligations() {
			if (deliverableObligations!=null) {
				return deliverableObligations;
			}
			else return deliverableObligations = new DeliverableObligations.DeliverableObligationsBuilder();
		}
		
		
		public Boolean getEscrow() {
			return escrow;
		}
		
		
		public PhysicalSettlementPeriod.PhysicalSettlementPeriodBuilder getPhysicalSettlementPeriod() {
			return physicalSettlementPeriod;
		}
		
		public PhysicalSettlementPeriod.PhysicalSettlementPeriodBuilder getOrCreatePhysicalSettlementPeriod() {
			if (physicalSettlementPeriod!=null) {
				return physicalSettlementPeriod;
			}
			else return physicalSettlementPeriod = new PhysicalSettlementPeriod.PhysicalSettlementPeriodBuilder();
		}
		
		
		public Boolean getSixtyBusinessDaySettlementCap() {
			return sixtyBusinessDaySettlementCap;
		}
		
	
		public PhysicalSettlementTermsBuilder setDeliverableObligations(DeliverableObligations deliverableObligations) {
			this.deliverableObligations = deliverableObligations.toBuilder();
			return this;
		}
		public PhysicalSettlementTermsBuilder setDeliverableObligationsBuilder(DeliverableObligations.DeliverableObligationsBuilder deliverableObligations) {
			this.deliverableObligations = deliverableObligations;
			return this;
		}
		public PhysicalSettlementTermsBuilder setEscrow(Boolean escrow) {
			this.escrow = escrow;
			return this;
		}
		public PhysicalSettlementTermsBuilder setPhysicalSettlementPeriod(PhysicalSettlementPeriod physicalSettlementPeriod) {
			this.physicalSettlementPeriod = physicalSettlementPeriod.toBuilder();
			return this;
		}
		public PhysicalSettlementTermsBuilder setPhysicalSettlementPeriodBuilder(PhysicalSettlementPeriod.PhysicalSettlementPeriodBuilder physicalSettlementPeriod) {
			this.physicalSettlementPeriod = physicalSettlementPeriod;
			return this;
		}
		public PhysicalSettlementTermsBuilder setSixtyBusinessDaySettlementCap(Boolean sixtyBusinessDaySettlementCap) {
			this.sixtyBusinessDaySettlementCap = sixtyBusinessDaySettlementCap;
			return this;
		}
		@Override public PhysicalSettlementTermsBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		@Override public PhysicalSettlementTermsBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		@Override public PhysicalSettlementTermsBuilder setSettlementCurrency(FieldWithMetaString settlementCurrency) {
			this.settlementCurrency = settlementCurrency.toBuilder();
			return this;
		}
		@Override public PhysicalSettlementTermsBuilder setSettlementCurrencyRef(String settlementCurrency) {
			return setSettlementCurrency(FieldWithMetaString.builder().setValue(settlementCurrency).build());
		}
		
		public PhysicalSettlementTerms build() {
			return new PhysicalSettlementTerms(this);
		}
	
		@Override
		public PhysicalSettlementTermsBuilder prune() {
			super.prune();
			if (deliverableObligations!=null && !deliverableObligations.prune().hasData()) deliverableObligations = null;
			if (physicalSettlementPeriod!=null && !physicalSettlementPeriod.prune().hasData()) physicalSettlementPeriod = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (super.hasData()) return true;
			if (getDeliverableObligations()!=null && getDeliverableObligations().hasData()) return true;
			if (getEscrow()!=null) return true;
			if (getPhysicalSettlementPeriod()!=null && getPhysicalSettlementPeriod().hasData()) return true;
			if (getSixtyBusinessDaySettlementCap()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
			super.process(path, processor);
		
			processor.processBasic(path.newSubPath("escrow"), Boolean.class, escrow, this);
			processor.processBasic(path.newSubPath("sixtyBusinessDaySettlementCap"), Boolean.class, sixtyBusinessDaySettlementCap, this);
			
			processRosetta(path.newSubPath("deliverableObligations"), processor, DeliverableObligations.class, deliverableObligations);
			processRosetta(path.newSubPath("physicalSettlementPeriod"), processor, PhysicalSettlementPeriod.class, physicalSettlementPeriod);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
			if (!super.equals(o)) return false;
		
			PhysicalSettlementTermsBuilder _that = (PhysicalSettlementTermsBuilder) o;
		
			if (deliverableObligations != null ? !deliverableObligations.equals(_that.deliverableObligations) : _that.deliverableObligations != null) return false;
			if (escrow != null ? !escrow.equals(_that.escrow) : _that.escrow != null) return false;
			if (physicalSettlementPeriod != null ? !physicalSettlementPeriod.equals(_that.physicalSettlementPeriod) : _that.physicalSettlementPeriod != null) return false;
			if (sixtyBusinessDaySettlementCap != null ? !sixtyBusinessDaySettlementCap.equals(_that.sixtyBusinessDaySettlementCap) : _that.sixtyBusinessDaySettlementCap != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = super.hashCode();
			_result = 31 * _result + (deliverableObligations != null ? deliverableObligations.hashCode() : 0);
			_result = 31 * _result + (escrow != null ? escrow.hashCode() : 0);
			_result = 31 * _result + (physicalSettlementPeriod != null ? physicalSettlementPeriod.hashCode() : 0);
			_result = 31 * _result + (sixtyBusinessDaySettlementCap != null ? sixtyBusinessDaySettlementCap.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "PhysicalSettlementTermsBuilder {" +
				"deliverableObligations=" + this.deliverableObligations + ", " +
				"escrow=" + this.escrow + ", " +
				"physicalSettlementPeriod=" + this.physicalSettlementPeriod + ", " +
				"sixtyBusinessDaySettlementCap=" + this.sixtyBusinessDaySettlementCap +
			'}' + " " + super.toString();
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		super.process(path, processor);
		
		processor.processBasic(path.newSubPath("escrow"), Boolean.class, escrow, this);
		processor.processBasic(path.newSubPath("sixtyBusinessDaySettlementCap"), Boolean.class, sixtyBusinessDaySettlementCap, this);
		
		processRosetta(path.newSubPath("deliverableObligations"), processor, DeliverableObligations.class, deliverableObligations);
		processRosetta(path.newSubPath("physicalSettlementPeriod"), processor, PhysicalSettlementPeriod.class, physicalSettlementPeriod);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
		if (!super.equals(o)) return false;
	
		PhysicalSettlementTerms _that = (PhysicalSettlementTerms) o;
	
		if (deliverableObligations != null ? !deliverableObligations.equals(_that.deliverableObligations) : _that.deliverableObligations != null) return false;
		if (escrow != null ? !escrow.equals(_that.escrow) : _that.escrow != null) return false;
		if (physicalSettlementPeriod != null ? !physicalSettlementPeriod.equals(_that.physicalSettlementPeriod) : _that.physicalSettlementPeriod != null) return false;
		if (sixtyBusinessDaySettlementCap != null ? !sixtyBusinessDaySettlementCap.equals(_that.sixtyBusinessDaySettlementCap) : _that.sixtyBusinessDaySettlementCap != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = super.hashCode();
		_result = 31 * _result + (deliverableObligations != null ? deliverableObligations.hashCode() : 0);
		_result = 31 * _result + (escrow != null ? escrow.hashCode() : 0);
		_result = 31 * _result + (physicalSettlementPeriod != null ? physicalSettlementPeriod.hashCode() : 0);
		_result = 31 * _result + (sixtyBusinessDaySettlementCap != null ? sixtyBusinessDaySettlementCap.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "PhysicalSettlementTerms {" +
			"deliverableObligations=" + this.deliverableObligations + ", " +
			"escrow=" + this.escrow + ", " +
			"physicalSettlementPeriod=" + this.physicalSettlementPeriod + ", " +
			"sixtyBusinessDaySettlementCap=" + this.sixtyBusinessDaySettlementCap +
		'}' + " " + super.toString();
	}
}
