package org.isda.cdm;

import org.isda.cdm.AggregationParameters;
import org.isda.cdm.PortfolioState;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.PortfolioMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A Portfolio represents an aggregation of multiple Positions, by describing the parameters that this Portfolio should be aggregated based on. The resulting PortfolioState is calculated using these aggregation parameters as inputs, by aggregating all the Events that are relevant to this Portfolio. The concept of Portfolio works at all levels in the model: from the highest for a given LegalEntity for instance, to the lowest to account for security substitutions in a secutity financing transaction. As such, Portfolio can be used either above or below the Contract level.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class Portfolio extends RosettaModelObject {
	private final AggregationParameters aggregationParameters;
	private final PortfolioState portfolioState;
	private static PortfolioMeta metaData = new PortfolioMeta();
	
	Portfolio(PortfolioBuilder builder) {
		this.aggregationParameters = ofNullable(builder.getAggregationParameters()).map(f->f.build()).orElse(null);
		this.portfolioState = ofNullable(builder.getPortfolioState()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * Describes the portfolio by describing how to aggregate all its relevant Events.
	 */
	public final AggregationParameters getAggregationParameters() {
		return aggregationParameters;
	}
	
	/**
	 * Describes the state of the Portfolio as a list of Positions resulting from the aggregation.
	 */
	public final PortfolioState getPortfolioState() {
		return portfolioState;
	}
	
	
	@Override
	public RosettaMetaData<? extends Portfolio> metaData() {
		return metaData;
	} 
	
	public PortfolioBuilder toBuilder() {
		PortfolioBuilder builder = new PortfolioBuilder();
		ofNullable(getAggregationParameters()).ifPresent(builder::setAggregationParameters);
		ofNullable(getPortfolioState()).ifPresent(builder::setPortfolioState);
		return builder;
	}

	public static PortfolioBuilder builder() {
		return new PortfolioBuilder();
	}

	public static  class PortfolioBuilder extends RosettaModelObjectBuilder {
	
		protected AggregationParameters.AggregationParametersBuilder aggregationParameters;
		protected PortfolioState.PortfolioStateBuilder portfolioState;
	
		public PortfolioBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends Portfolio> metaData() {
			return metaData;
		} 
	
		
		public AggregationParameters.AggregationParametersBuilder getAggregationParameters() {
			return aggregationParameters;
		}
		
		public AggregationParameters.AggregationParametersBuilder getOrCreateAggregationParameters() {
			if (aggregationParameters!=null) {
				return aggregationParameters;
			}
			else return aggregationParameters = new AggregationParameters.AggregationParametersBuilder();
		}
		
		
		public PortfolioState.PortfolioStateBuilder getPortfolioState() {
			return portfolioState;
		}
		
		public PortfolioState.PortfolioStateBuilder getOrCreatePortfolioState() {
			if (portfolioState!=null) {
				return portfolioState;
			}
			else return portfolioState = new PortfolioState.PortfolioStateBuilder();
		}
		
	
		public PortfolioBuilder setAggregationParameters(AggregationParameters aggregationParameters) {
			this.aggregationParameters = aggregationParameters.toBuilder();
			return this;
		}
		public PortfolioBuilder setAggregationParametersBuilder(AggregationParameters.AggregationParametersBuilder aggregationParameters) {
			this.aggregationParameters = aggregationParameters;
			return this;
		}
		public PortfolioBuilder setPortfolioState(PortfolioState portfolioState) {
			this.portfolioState = portfolioState.toBuilder();
			return this;
		}
		public PortfolioBuilder setPortfolioStateBuilder(PortfolioState.PortfolioStateBuilder portfolioState) {
			this.portfolioState = portfolioState;
			return this;
		}
		
		public Portfolio build() {
			return new Portfolio(this);
		}
	
		@Override
		public PortfolioBuilder prune() {
			if (aggregationParameters!=null && !aggregationParameters.prune().hasData()) aggregationParameters = null;
			if (portfolioState!=null && !portfolioState.prune().hasData()) portfolioState = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getAggregationParameters()!=null && getAggregationParameters().hasData()) return true;
			if (getPortfolioState()!=null && getPortfolioState().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("aggregationParameters"), processor, AggregationParameters.class, aggregationParameters);
			processRosetta(path.newSubPath("portfolioState"), processor, PortfolioState.class, portfolioState);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			PortfolioBuilder _that = (PortfolioBuilder) o;
		
			if (aggregationParameters != null ? !aggregationParameters.equals(_that.aggregationParameters) : _that.aggregationParameters != null) return false;
			if (portfolioState != null ? !portfolioState.equals(_that.portfolioState) : _that.portfolioState != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (aggregationParameters != null ? aggregationParameters.hashCode() : 0);
			_result = 31 * _result + (portfolioState != null ? portfolioState.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "PortfolioBuilder {" +
				"aggregationParameters=" + this.aggregationParameters + ", " +
				"portfolioState=" + this.portfolioState +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("aggregationParameters"), processor, AggregationParameters.class, aggregationParameters);
		processRosetta(path.newSubPath("portfolioState"), processor, PortfolioState.class, portfolioState);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		Portfolio _that = (Portfolio) o;
	
		if (aggregationParameters != null ? !aggregationParameters.equals(_that.aggregationParameters) : _that.aggregationParameters != null) return false;
		if (portfolioState != null ? !portfolioState.equals(_that.portfolioState) : _that.portfolioState != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (aggregationParameters != null ? aggregationParameters.hashCode() : 0);
		_result = 31 * _result + (portfolioState != null ? portfolioState.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "Portfolio {" +
			"aggregationParameters=" + this.aggregationParameters + ", " +
			"portfolioState=" + this.portfolioState +
		'}';
	}
}
