package org.isda.cdm;

import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.Lineage;
import org.isda.cdm.Position;
import org.isda.cdm.metafields.MetaFields;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.PortfolioStateMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;

/**
 * State-full representation of a Portfolio that describes all the positions held at a given time, in various states which can be either traded, settled, etc., with lineage information to the previous state
 * 
 * @version 2.5.4
 */
@RosettaClass
public class PortfolioState extends RosettaModelObject implements GlobalKey {
	private final Lineage lineage;
	private final MetaFields meta;
	private final List<Position> positions;
	private static PortfolioStateMeta metaData = new PortfolioStateMeta();
	
	PortfolioState(PortfolioStateBuilder builder) {
		this.lineage = ofNullable(builder.getLineage()).map(f->f.build()).orElse(null);
		this.meta = ofNullable(builder.getMeta()).map(f->f.build()).orElse(null);
		this.positions = ofNullable(builder.getPositions()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
	}
	
	/**
	 * Pointer to the previous PortfolioState and new Event(s) leading to the current (new) state. Previous PortfolioState in the Lineage can be Null in case this is the start of the chain of Events.
	 */
	public final Lineage getLineage() {
		return lineage;
	}
	
	public final MetaFields getMeta() {
		return meta;
	}
	
	/**
	 * The list of positions, each containing a Quantity and a Product.
	 */
	public final List<Position> getPositions() {
		return positions;
	}
	
	
	@Override
	public RosettaMetaData<? extends PortfolioState> metaData() {
		return metaData;
	} 
	
	public PortfolioStateBuilder toBuilder() {
		PortfolioStateBuilder builder = new PortfolioStateBuilder();
		ofNullable(getLineage()).ifPresent(builder::setLineage);
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		ofNullable(getPositions()).ifPresent(positions -> positions.forEach(builder::addPositions));
		return builder;
	}

	public static PortfolioStateBuilder builder() {
		return new PortfolioStateBuilder();
	}

	public static  class PortfolioStateBuilder extends RosettaModelObjectBuilder implements GlobalKeyBuilder<PortfolioStateBuilder> {
	
		protected Lineage.LineageBuilder lineage;
		protected MetaFields.MetaFieldsBuilder meta;
		protected List<Position.PositionBuilder> positions;
	
		public PortfolioStateBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends PortfolioState> metaData() {
			return metaData;
		} 
	
		
		public Lineage.LineageBuilder getLineage() {
			return lineage;
		}
		
		public Lineage.LineageBuilder getOrCreateLineage() {
			if (lineage!=null) {
				return lineage;
			}
			else return lineage = new Lineage.LineageBuilder();
		}
		
		
		public MetaFields.MetaFieldsBuilder getMeta() {
			return meta;
		}
		
		public MetaFields.MetaFieldsBuilder getOrCreateMeta() {
			if (meta!=null) {
				return meta;
			}
			else return meta = new MetaFields.MetaFieldsBuilder();
		}
		
		
		public List<Position.PositionBuilder> getPositions() {
			return positions;
		}
		
		public Position.PositionBuilder getOrCreatePositions(int index) {
			if (positions==null) {
				this.positions = new ArrayList<>();
			}
			return getIndex(positions, index, ()->new Position.PositionBuilder());
		}
		
	
		public PortfolioStateBuilder setLineage(Lineage lineage) {
			this.lineage = lineage.toBuilder();
			return this;
		}
		public PortfolioStateBuilder setLineageBuilder(Lineage.LineageBuilder lineage) {
			this.lineage = lineage;
			return this;
		}
		public PortfolioStateBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		public PortfolioStateBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		public PortfolioStateBuilder addPositions(Position positions) {
			if(this.positions == null){
				this.positions = new ArrayList<>();
			}
			this.positions.add(positions.toBuilder());
			return this;
		}
		public PortfolioStateBuilder addPositions(Position positions, int _idx) {
			if(this.positions == null){
				this.positions = new ArrayList<>();
			}
			getIndex(this.positions, _idx, () -> positions.toBuilder());
			this.positions.set(_idx, positions.toBuilder());
			return this;
		}
		public PortfolioStateBuilder addPositions(List<Position> positionss) {
			if(this.positions == null){
				this.positions = new ArrayList<>();
			}
			for (Position toAdd : positionss) {
				this.positions.add(toAdd.toBuilder());
			}
			return this;
		}
		public PortfolioStateBuilder addPositionsBuilder(Position.PositionBuilder positions) {
			if(this.positions == null){
				this.positions = new ArrayList<>();
			}
			this.positions.add(positions);
			return this;
		}
		
		public PortfolioStateBuilder clearPositions() {
			this.positions = null;
			return this;
		}
		
		public PortfolioState build() {
			return new PortfolioState(this);
		}
	
		@Override
		public PortfolioStateBuilder prune() {
			if (lineage!=null && !lineage.prune().hasData()) lineage = null;
			if (meta!=null && !meta.prune().hasData()) meta = null;
			if (positions!=null) positions = positions.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getLineage()!=null && getLineage().hasData()) return true;
			if (getMeta()!=null && getMeta().hasData()) return true;
			if (getPositions()!=null && getPositions().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("lineage"), processor, Lineage.class, lineage);
			processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
			processRosetta(path.newSubPath("positions"), processor, Position.class, positions);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			PortfolioStateBuilder _that = (PortfolioStateBuilder) o;
		
			if (lineage != null ? !lineage.equals(_that.lineage) : _that.lineage != null) return false;
			if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
			if (!ListEquals.listEquals(positions, _that.positions)) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (lineage != null ? lineage.hashCode() : 0);
			_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
			_result = 31 * _result + (positions != null ? positions.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "PortfolioStateBuilder {" +
				"lineage=" + this.lineage + ", " +
				"meta=" + this.meta + ", " +
				"positions=" + this.positions +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("lineage"), processor, Lineage.class, lineage);
		processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
		processRosetta(path.newSubPath("positions"), processor, Position.class, positions);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		PortfolioState _that = (PortfolioState) o;
	
		if (lineage != null ? !lineage.equals(_that.lineage) : _that.lineage != null) return false;
		if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
		if (!ListEquals.listEquals(positions, _that.positions)) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (lineage != null ? lineage.hashCode() : 0);
		_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
		_result = 31 * _result + (positions != null ? positions.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "PortfolioState {" +
			"lineage=" + this.lineage + ", " +
			"meta=" + this.meta + ", " +
			"positions=" + this.positions +
		'}';
	}
}
