package org.isda.cdm;

import org.isda.cdm.Money;
import org.isda.cdm.PositionStatusEnum;
import org.isda.cdm.Product;
import org.isda.cdm.Quantity;
import org.isda.cdm.metafields.ReferenceWithMetaContract;
import org.isda.cdm.metafields.ReferenceWithMetaContract.ReferenceWithMetaContractBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.PositionMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;
		import java.util.stream.Collectors;

/**
 * A Position describes how much of a given Product is being held and constitutes the atomic element of a Portfolio.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class Position extends RosettaModelObject {
	private final Money cashBalance;
	private final ReferenceWithMetaContract contractReference;
	private final PositionStatusEnum positionStatus;
	private final Product product;
	private final Quantity quantity;
	private static PositionMeta metaData = new PositionMeta();
	
	Position(PositionBuilder builder) {
		this.cashBalance = ofNullable(builder.getCashBalance()).map(f->f.build()).orElse(null);
		this.contractReference = ofNullable(builder.getContractReference()).map(f->f.build()).orElse(null);
		this.positionStatus = builder.getPositionStatus();
		this.product = ofNullable(builder.getProduct()).map(f->f.build()).orElse(null);
		this.quantity = ofNullable(builder.getQuantity()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The aggregate cost of proceeds
	 */
	public final Money getCashBalance() {
		return cashBalance;
	}
	
	/**
	 * Reference to the Contract, in case product is contractual and the contract has been formed
	 */
	public final ReferenceWithMetaContract getContractReference() {
		return contractReference;
	}
	
	/**
	 * Qualifier for the state of the Position, to distinguish if just executed, formed, already settled, closed etc.
	 */
	public final PositionStatusEnum getPositionStatus() {
		return positionStatus;
	}
	
	/**
	 * The product underlying the position, which can either be a contractual product or securities.
	 */
	public final Product getProduct() {
		return product;
	}
	
	/**
	 * The quantity of the product, which can be a negative number in case of a short position.
	 */
	public final Quantity getQuantity() {
		return quantity;
	}
	
	
	@Override
	public RosettaMetaData<? extends Position> metaData() {
		return metaData;
	} 
	
	public PositionBuilder toBuilder() {
		PositionBuilder builder = new PositionBuilder();
		ofNullable(getCashBalance()).ifPresent(builder::setCashBalance);
		ofNullable(getContractReference()).ifPresent(builder::setContractReference);
		ofNullable(getPositionStatus()).ifPresent(builder::setPositionStatus);
		ofNullable(getProduct()).ifPresent(builder::setProduct);
		ofNullable(getQuantity()).ifPresent(builder::setQuantity);
		return builder;
	}

	public static PositionBuilder builder() {
		return new PositionBuilder();
	}

	public static  class PositionBuilder extends RosettaModelObjectBuilder {
	
		protected Money.MoneyBuilder cashBalance;
		protected ReferenceWithMetaContractBuilder contractReference;
		protected PositionStatusEnum positionStatus;
		protected Product.ProductBuilder product;
		protected Quantity.QuantityBuilder quantity;
	
		public PositionBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends Position> metaData() {
			return metaData;
		} 
	
		
		public Money.MoneyBuilder getCashBalance() {
			return cashBalance;
		}
		
		public Money.MoneyBuilder getOrCreateCashBalance() {
			if (cashBalance!=null) {
				return cashBalance;
			}
			else return cashBalance = new Money.MoneyBuilder();
		}
		
		
		public ReferenceWithMetaContractBuilder getContractReference() {
			return contractReference;
		}
		
		public ReferenceWithMetaContractBuilder getOrCreateContractReference() {
			if (contractReference!=null) {
				return contractReference;
			}
			else return contractReference = new ReferenceWithMetaContractBuilder();
		}
		
		
		public PositionStatusEnum getPositionStatus() {
			return positionStatus;
		}
		
		
		public Product.ProductBuilder getProduct() {
			return product;
		}
		
		public Product.ProductBuilder getOrCreateProduct() {
			if (product!=null) {
				return product;
			}
			else return product = new Product.ProductBuilder();
		}
		
		
		public Quantity.QuantityBuilder getQuantity() {
			return quantity;
		}
		
		public Quantity.QuantityBuilder getOrCreateQuantity() {
			if (quantity!=null) {
				return quantity;
			}
			else return quantity = new Quantity.QuantityBuilder();
		}
		
	
		public PositionBuilder setCashBalance(Money cashBalance) {
			this.cashBalance = cashBalance.toBuilder();
			return this;
		}
		public PositionBuilder setCashBalanceBuilder(Money.MoneyBuilder cashBalance) {
			this.cashBalance = cashBalance;
			return this;
		}
		public PositionBuilder setContractReference(ReferenceWithMetaContract contractReference) {
			this.contractReference = contractReference.toBuilder();
			return this;
		}
		public PositionBuilder setContractReferenceBuilder(ReferenceWithMetaContractBuilder contractReference) {
			this.contractReference = contractReference;
			return this;
		}
		public PositionBuilder setContractReferenceRef(Contract.ContractBuilder contractReference) {
			return setContractReference(ReferenceWithMetaContract.builder().setValueBuilder(contractReference).build());
		}
		public PositionBuilder setContractReferenceRef(Contract contractReference) {
			return setContractReferenceRef(Contract.builder());
		}
		public PositionBuilder setPositionStatus(PositionStatusEnum positionStatus) {
			this.positionStatus = positionStatus;
			return this;
		}
		public PositionBuilder setProduct(Product product) {
			this.product = product.toBuilder();
			return this;
		}
		public PositionBuilder setProductBuilder(Product.ProductBuilder product) {
			this.product = product;
			return this;
		}
		public PositionBuilder setQuantity(Quantity quantity) {
			this.quantity = quantity.toBuilder();
			return this;
		}
		public PositionBuilder setQuantityBuilder(Quantity.QuantityBuilder quantity) {
			this.quantity = quantity;
			return this;
		}
		
		public Position build() {
			return new Position(this);
		}
	
		@Override
		public PositionBuilder prune() {
			if (cashBalance!=null && !cashBalance.prune().hasData()) cashBalance = null;
			if (contractReference!=null && !contractReference.prune().hasData()) contractReference = null;
			if (product!=null && !product.prune().hasData()) product = null;
			if (quantity!=null && !quantity.prune().hasData()) quantity = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getCashBalance()!=null && getCashBalance().hasData()) return true;
			if (getContractReference()!=null && getContractReference().hasData()) return true;
			if (getPositionStatus()!=null) return true;
			if (getProduct()!=null && getProduct().hasData()) return true;
			if (getQuantity()!=null && getQuantity().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("positionStatus"), PositionStatusEnum.class, positionStatus, this);
			
			processRosetta(path.newSubPath("cashBalance"), processor, Money.class, cashBalance);
			processRosetta(path.newSubPath("contractReference"), processor, ReferenceWithMetaContract.class, contractReference);
			processRosetta(path.newSubPath("product"), processor, Product.class, product);
			processRosetta(path.newSubPath("quantity"), processor, Quantity.class, quantity);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			PositionBuilder _that = (PositionBuilder) o;
		
			if (cashBalance != null ? !cashBalance.equals(_that.cashBalance) : _that.cashBalance != null) return false;
			if (contractReference != null ? !contractReference.equals(_that.contractReference) : _that.contractReference != null) return false;
			if (positionStatus != null ? !positionStatus.equals(_that.positionStatus) : _that.positionStatus != null) return false;
			if (product != null ? !product.equals(_that.product) : _that.product != null) return false;
			if (quantity != null ? !quantity.equals(_that.quantity) : _that.quantity != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (cashBalance != null ? cashBalance.hashCode() : 0);
			_result = 31 * _result + (contractReference != null ? contractReference.hashCode() : 0);
			_result = 31 * _result + (positionStatus != null ? positionStatus.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (product != null ? product.hashCode() : 0);
			_result = 31 * _result + (quantity != null ? quantity.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "PositionBuilder {" +
				"cashBalance=" + this.cashBalance + ", " +
				"contractReference=" + this.contractReference + ", " +
				"positionStatus=" + this.positionStatus + ", " +
				"product=" + this.product + ", " +
				"quantity=" + this.quantity +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("positionStatus"), PositionStatusEnum.class, positionStatus, this);
		
		processRosetta(path.newSubPath("cashBalance"), processor, Money.class, cashBalance);
		processRosetta(path.newSubPath("contractReference"), processor, ReferenceWithMetaContract.class, contractReference);
		processRosetta(path.newSubPath("product"), processor, Product.class, product);
		processRosetta(path.newSubPath("quantity"), processor, Quantity.class, quantity);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		Position _that = (Position) o;
	
		if (cashBalance != null ? !cashBalance.equals(_that.cashBalance) : _that.cashBalance != null) return false;
		if (contractReference != null ? !contractReference.equals(_that.contractReference) : _that.contractReference != null) return false;
		if (positionStatus != null ? !positionStatus.equals(_that.positionStatus) : _that.positionStatus != null) return false;
		if (product != null ? !product.equals(_that.product) : _that.product != null) return false;
		if (quantity != null ? !quantity.equals(_that.quantity) : _that.quantity != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (cashBalance != null ? cashBalance.hashCode() : 0);
		_result = 31 * _result + (contractReference != null ? contractReference.hashCode() : 0);
		_result = 31 * _result + (positionStatus != null ? positionStatus.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (product != null ? product.hashCode() : 0);
		_result = 31 * _result + (quantity != null ? quantity.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "Position {" +
			"cashBalance=" + this.cashBalance + ", " +
			"contractReference=" + this.contractReference + ", " +
			"positionStatus=" + this.positionStatus + ", " +
			"product=" + this.product + ", " +
			"quantity=" + this.quantity +
		'}';
	}
}
