package org.isda.cdm;

import org.isda.cdm.CreditLimitInformation;
import org.isda.cdm.TradeWarehouseWorkflow;


import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.PostInceptionStateMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * @version 2.5.4
 */
@RosettaClass
public class PostInceptionState extends ContractState {
	private final CreditLimitInformation creditLimitInformation;
	private final TradeWarehouseWorkflow tradeWarehouseWorkflow;
	private static PostInceptionStateMeta metaData = new PostInceptionStateMeta();
	
	PostInceptionState(PostInceptionStateBuilder builder) {
		super(builder);
		this.creditLimitInformation = ofNullable(builder.getCreditLimitInformation()).map(f->f.build()).orElse(null);
		this.tradeWarehouseWorkflow = ofNullable(builder.getTradeWarehouseWorkflow()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * Credit limit utilization information.
	 */
	@RosettaSynonym(value="creditLimitInformation", source="CME_ClearedConfirm_1_17", path="trade.tradeHeader")
	@RosettaSynonym(value="creditLimitInformation", source="Rosetta_Workbench")
	public final CreditLimitInformation getCreditLimitInformation() {
		return creditLimitInformation;
	}
	
	/**
	 * Information related to trade warehouse workflow.
	 */
	public final TradeWarehouseWorkflow getTradeWarehouseWorkflow() {
		return tradeWarehouseWorkflow;
	}
	
	
	@Override
	public RosettaMetaData<? extends PostInceptionState> metaData() {
		return metaData;
	} 
	
	public PostInceptionStateBuilder toBuilder() {
		PostInceptionStateBuilder builder = new PostInceptionStateBuilder();
		ofNullable(getCreditLimitInformation()).ifPresent(builder::setCreditLimitInformation);
		ofNullable(getTradeWarehouseWorkflow()).ifPresent(builder::setTradeWarehouseWorkflow);
		ofNullable(getContract()).ifPresent(builder::setContract);
		ofNullable(getUpdatedContract()).ifPresent(builder::setUpdatedContract);
		return builder;
	}

	public static PostInceptionStateBuilder builder() {
		return new PostInceptionStateBuilder();
	}

	public static  class PostInceptionStateBuilder extends ContractStateBuilder {
	
		protected CreditLimitInformation.CreditLimitInformationBuilder creditLimitInformation;
		protected TradeWarehouseWorkflow.TradeWarehouseWorkflowBuilder tradeWarehouseWorkflow;
	
		public PostInceptionStateBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends PostInceptionState> metaData() {
			return metaData;
		} 
	
		
		public CreditLimitInformation.CreditLimitInformationBuilder getCreditLimitInformation() {
			return creditLimitInformation;
		}
		
		public CreditLimitInformation.CreditLimitInformationBuilder getOrCreateCreditLimitInformation() {
			if (creditLimitInformation!=null) {
				return creditLimitInformation;
			}
			else return creditLimitInformation = new CreditLimitInformation.CreditLimitInformationBuilder();
		}
		
		
		public TradeWarehouseWorkflow.TradeWarehouseWorkflowBuilder getTradeWarehouseWorkflow() {
			return tradeWarehouseWorkflow;
		}
		
		public TradeWarehouseWorkflow.TradeWarehouseWorkflowBuilder getOrCreateTradeWarehouseWorkflow() {
			if (tradeWarehouseWorkflow!=null) {
				return tradeWarehouseWorkflow;
			}
			else return tradeWarehouseWorkflow = new TradeWarehouseWorkflow.TradeWarehouseWorkflowBuilder();
		}
		
	
		public PostInceptionStateBuilder setCreditLimitInformation(CreditLimitInformation creditLimitInformation) {
			this.creditLimitInformation = creditLimitInformation.toBuilder();
			return this;
		}
		public PostInceptionStateBuilder setCreditLimitInformationBuilder(CreditLimitInformation.CreditLimitInformationBuilder creditLimitInformation) {
			this.creditLimitInformation = creditLimitInformation;
			return this;
		}
		public PostInceptionStateBuilder setTradeWarehouseWorkflow(TradeWarehouseWorkflow tradeWarehouseWorkflow) {
			this.tradeWarehouseWorkflow = tradeWarehouseWorkflow.toBuilder();
			return this;
		}
		public PostInceptionStateBuilder setTradeWarehouseWorkflowBuilder(TradeWarehouseWorkflow.TradeWarehouseWorkflowBuilder tradeWarehouseWorkflow) {
			this.tradeWarehouseWorkflow = tradeWarehouseWorkflow;
			return this;
		}
		@Override public PostInceptionStateBuilder setContract(Contract contract) {
			this.contract = contract.toBuilder();
			return this;
		}
		@Override public PostInceptionStateBuilder setContractBuilder(Contract.ContractBuilder contract) {
			this.contract = contract;
			return this;
		}
		@Override public PostInceptionStateBuilder setUpdatedContract(Contract updatedContract) {
			this.updatedContract = updatedContract.toBuilder();
			return this;
		}
		@Override public PostInceptionStateBuilder setUpdatedContractBuilder(Contract.ContractBuilder updatedContract) {
			this.updatedContract = updatedContract;
			return this;
		}
		
		public PostInceptionState build() {
			return new PostInceptionState(this);
		}
	
		@Override
		public PostInceptionStateBuilder prune() {
			super.prune();
			if (creditLimitInformation!=null && !creditLimitInformation.prune().hasData()) creditLimitInformation = null;
			if (tradeWarehouseWorkflow!=null && !tradeWarehouseWorkflow.prune().hasData()) tradeWarehouseWorkflow = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (super.hasData()) return true;
			if (getCreditLimitInformation()!=null && getCreditLimitInformation().hasData()) return true;
			if (getTradeWarehouseWorkflow()!=null && getTradeWarehouseWorkflow().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
			super.process(path, processor);
		
			
			processRosetta(path.newSubPath("creditLimitInformation"), processor, CreditLimitInformation.class, creditLimitInformation);
			processRosetta(path.newSubPath("tradeWarehouseWorkflow"), processor, TradeWarehouseWorkflow.class, tradeWarehouseWorkflow);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
			if (!super.equals(o)) return false;
		
			PostInceptionStateBuilder _that = (PostInceptionStateBuilder) o;
		
			if (creditLimitInformation != null ? !creditLimitInformation.equals(_that.creditLimitInformation) : _that.creditLimitInformation != null) return false;
			if (tradeWarehouseWorkflow != null ? !tradeWarehouseWorkflow.equals(_that.tradeWarehouseWorkflow) : _that.tradeWarehouseWorkflow != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = super.hashCode();
			_result = 31 * _result + (creditLimitInformation != null ? creditLimitInformation.hashCode() : 0);
			_result = 31 * _result + (tradeWarehouseWorkflow != null ? tradeWarehouseWorkflow.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "PostInceptionStateBuilder {" +
				"creditLimitInformation=" + this.creditLimitInformation + ", " +
				"tradeWarehouseWorkflow=" + this.tradeWarehouseWorkflow +
			'}' + " " + super.toString();
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		super.process(path, processor);
		
		
		processRosetta(path.newSubPath("creditLimitInformation"), processor, CreditLimitInformation.class, creditLimitInformation);
		processRosetta(path.newSubPath("tradeWarehouseWorkflow"), processor, TradeWarehouseWorkflow.class, tradeWarehouseWorkflow);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
		if (!super.equals(o)) return false;
	
		PostInceptionState _that = (PostInceptionState) o;
	
		if (creditLimitInformation != null ? !creditLimitInformation.equals(_that.creditLimitInformation) : _that.creditLimitInformation != null) return false;
		if (tradeWarehouseWorkflow != null ? !tradeWarehouseWorkflow.equals(_that.tradeWarehouseWorkflow) : _that.tradeWarehouseWorkflow != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = super.hashCode();
		_result = 31 * _result + (creditLimitInformation != null ? creditLimitInformation.hashCode() : 0);
		_result = 31 * _result + (tradeWarehouseWorkflow != null ? tradeWarehouseWorkflow.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "PostInceptionState {" +
			"creditLimitInformation=" + this.creditLimitInformation + ", " +
			"tradeWarehouseWorkflow=" + this.tradeWarehouseWorkflow +
		'}' + " " + super.toString();
	}
}
