package org.isda.cdm;

import java.lang.Boolean;
import java.lang.String;
import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.EligibleCollateral;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.PostingObligationsElectionMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;

/**
 * A class to specify the collateral posting obligations as specified under the terms of the ISDA 2016 Credit Support Annex for Initial Margin, paragraph 13, General Principles, (ii).
 * 
 * @version 2.5.4
 */
@RosettaClass
public class PostingObligationsElection extends RosettaModelObject {
	private final String additionalLanguage;
	private final Boolean asPermitted;
	private final List<EligibleCollateral> eligibleCollateral;
	private final String excludedCollateral;
	private final String party;
	private static PostingObligationsElectionMeta metaData = new PostingObligationsElectionMeta();
	
	PostingObligationsElection(PostingObligationsElectionBuilder builder) {
		this.additionalLanguage = builder.getAdditionalLanguage();
		this.asPermitted = builder.getAsPermitted();
		this.eligibleCollateral = ofNullable(builder.getEligibleCollateral()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.excludedCollateral = builder.getExcludedCollateral();
		this.party = builder.getParty();
	}
	
	/**
	 * The additional language that might be specified by the parties to the legal agreement.
	 */
	@RosettaSynonym(value="partyA_additional_language", source="ISDA_Create_1_0")
	@RosettaSynonym(value="partyB_additional_language", source="ISDA_Create_1_0")
	public final String getAdditionalLanguage() {
		return additionalLanguage;
	}
	
	/**
	 * If set to True, the Control Agreement is a Credit Support Document with respect to the party(ies). ISDA 2016 Credit Support Annex for Initial Margin, paragraph 6, (e).
	 */
	@RosettaSynonym(value="", source="ISDA_Create_1_0")
	@RosettaSynonym(value="", source="ISDA_Create_1_0")
	public final Boolean getAsPermitted() {
		return asPermitted;
	}
	
	/**
	 * The eligible collateral as specified in relation to the pledgor/chargor/obligor(s) posting obligation. ISDA 2016 Credit Support Annex for Initial Margin, Eligible Credit Support (IM) Schedule.
	 */
	public final List<EligibleCollateral> getEligibleCollateral() {
		return eligibleCollateral;
	}
	
	/**
	 * The excluded collateral as specified in relation to the pledgor/chargor/obligor(s) posting obligation. ISDA 2016 Credit Support Annex for Initial Margin, paragraph 13, General Principles, (ii)(B)(i).
	 */
	@RosettaSynonym(value="partyA_control_agreement_specify", source="ISDA_Create_1_0")
	@RosettaSynonym(value="partyB_control_agreement_specify", source="ISDA_Create_1_0")
	public final String getExcludedCollateral() {
		return excludedCollateral;
	}
	
	/**
	 * The elective party.
	 */
	@RosettaSynonym(value="", source="ISDA_Create_1_0")
	@RosettaSynonym(value="", source="ISDA_Create_1_0")
	public final String getParty() {
		return party;
	}
	
	
	@Override
	public RosettaMetaData<? extends PostingObligationsElection> metaData() {
		return metaData;
	} 
	
	public PostingObligationsElectionBuilder toBuilder() {
		PostingObligationsElectionBuilder builder = new PostingObligationsElectionBuilder();
		ofNullable(getAdditionalLanguage()).ifPresent(builder::setAdditionalLanguage);
		ofNullable(getAsPermitted()).ifPresent(builder::setAsPermitted);
		ofNullable(getEligibleCollateral()).ifPresent(eligibleCollateral -> eligibleCollateral.forEach(builder::addEligibleCollateral));
		ofNullable(getExcludedCollateral()).ifPresent(builder::setExcludedCollateral);
		ofNullable(getParty()).ifPresent(builder::setParty);
		return builder;
	}

	public static PostingObligationsElectionBuilder builder() {
		return new PostingObligationsElectionBuilder();
	}

	public static  class PostingObligationsElectionBuilder extends RosettaModelObjectBuilder {
	
		protected String additionalLanguage;
		protected Boolean asPermitted;
		protected List<EligibleCollateral.EligibleCollateralBuilder> eligibleCollateral;
		protected String excludedCollateral;
		protected String party;
	
		public PostingObligationsElectionBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends PostingObligationsElection> metaData() {
			return metaData;
		} 
	
		
		public String getAdditionalLanguage() {
			return additionalLanguage;
		}
		
		
		public Boolean getAsPermitted() {
			return asPermitted;
		}
		
		
		public List<EligibleCollateral.EligibleCollateralBuilder> getEligibleCollateral() {
			return eligibleCollateral;
		}
		
		public EligibleCollateral.EligibleCollateralBuilder getOrCreateEligibleCollateral(int index) {
			if (eligibleCollateral==null) {
				this.eligibleCollateral = new ArrayList<>();
			}
			return getIndex(eligibleCollateral, index, ()->new EligibleCollateral.EligibleCollateralBuilder());
		}
		
		
		public String getExcludedCollateral() {
			return excludedCollateral;
		}
		
		
		public String getParty() {
			return party;
		}
		
	
		public PostingObligationsElectionBuilder setAdditionalLanguage(String additionalLanguage) {
			this.additionalLanguage = additionalLanguage;
			return this;
		}
		public PostingObligationsElectionBuilder setAsPermitted(Boolean asPermitted) {
			this.asPermitted = asPermitted;
			return this;
		}
		public PostingObligationsElectionBuilder addEligibleCollateral(EligibleCollateral eligibleCollateral) {
			if(this.eligibleCollateral == null){
				this.eligibleCollateral = new ArrayList<>();
			}
			this.eligibleCollateral.add(eligibleCollateral.toBuilder());
			return this;
		}
		public PostingObligationsElectionBuilder addEligibleCollateral(EligibleCollateral eligibleCollateral, int _idx) {
			if(this.eligibleCollateral == null){
				this.eligibleCollateral = new ArrayList<>();
			}
			getIndex(this.eligibleCollateral, _idx, () -> eligibleCollateral.toBuilder());
			this.eligibleCollateral.set(_idx, eligibleCollateral.toBuilder());
			return this;
		}
		public PostingObligationsElectionBuilder addEligibleCollateral(List<EligibleCollateral> eligibleCollaterals) {
			if(this.eligibleCollateral == null){
				this.eligibleCollateral = new ArrayList<>();
			}
			for (EligibleCollateral toAdd : eligibleCollaterals) {
				this.eligibleCollateral.add(toAdd.toBuilder());
			}
			return this;
		}
		public PostingObligationsElectionBuilder addEligibleCollateralBuilder(EligibleCollateral.EligibleCollateralBuilder eligibleCollateral) {
			if(this.eligibleCollateral == null){
				this.eligibleCollateral = new ArrayList<>();
			}
			this.eligibleCollateral.add(eligibleCollateral);
			return this;
		}
		
		public PostingObligationsElectionBuilder clearEligibleCollateral() {
			this.eligibleCollateral = null;
			return this;
		}
		public PostingObligationsElectionBuilder setExcludedCollateral(String excludedCollateral) {
			this.excludedCollateral = excludedCollateral;
			return this;
		}
		public PostingObligationsElectionBuilder setParty(String party) {
			this.party = party;
			return this;
		}
		
		public PostingObligationsElection build() {
			return new PostingObligationsElection(this);
		}
	
		@Override
		public PostingObligationsElectionBuilder prune() {
			if (eligibleCollateral!=null) eligibleCollateral = eligibleCollateral.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getAdditionalLanguage()!=null) return true;
			if (getAsPermitted()!=null) return true;
			if (getEligibleCollateral()!=null && getEligibleCollateral().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getExcludedCollateral()!=null) return true;
			if (getParty()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("additionalLanguage"), String.class, additionalLanguage, this);
			processor.processBasic(path.newSubPath("asPermitted"), Boolean.class, asPermitted, this);
			processor.processBasic(path.newSubPath("excludedCollateral"), String.class, excludedCollateral, this);
			processor.processBasic(path.newSubPath("party"), String.class, party, this);
			
			processRosetta(path.newSubPath("eligibleCollateral"), processor, EligibleCollateral.class, eligibleCollateral);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			PostingObligationsElectionBuilder _that = (PostingObligationsElectionBuilder) o;
		
			if (additionalLanguage != null ? !additionalLanguage.equals(_that.additionalLanguage) : _that.additionalLanguage != null) return false;
			if (asPermitted != null ? !asPermitted.equals(_that.asPermitted) : _that.asPermitted != null) return false;
			if (!ListEquals.listEquals(eligibleCollateral, _that.eligibleCollateral)) return false;
			if (excludedCollateral != null ? !excludedCollateral.equals(_that.excludedCollateral) : _that.excludedCollateral != null) return false;
			if (party != null ? !party.equals(_that.party) : _that.party != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (additionalLanguage != null ? additionalLanguage.hashCode() : 0);
			_result = 31 * _result + (asPermitted != null ? asPermitted.hashCode() : 0);
			_result = 31 * _result + (eligibleCollateral != null ? eligibleCollateral.hashCode() : 0);
			_result = 31 * _result + (excludedCollateral != null ? excludedCollateral.hashCode() : 0);
			_result = 31 * _result + (party != null ? party.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "PostingObligationsElectionBuilder {" +
				"additionalLanguage=" + this.additionalLanguage + ", " +
				"asPermitted=" + this.asPermitted + ", " +
				"eligibleCollateral=" + this.eligibleCollateral + ", " +
				"excludedCollateral=" + this.excludedCollateral + ", " +
				"party=" + this.party +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("additionalLanguage"), String.class, additionalLanguage, this);
		processor.processBasic(path.newSubPath("asPermitted"), Boolean.class, asPermitted, this);
		processor.processBasic(path.newSubPath("excludedCollateral"), String.class, excludedCollateral, this);
		processor.processBasic(path.newSubPath("party"), String.class, party, this);
		
		processRosetta(path.newSubPath("eligibleCollateral"), processor, EligibleCollateral.class, eligibleCollateral);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		PostingObligationsElection _that = (PostingObligationsElection) o;
	
		if (additionalLanguage != null ? !additionalLanguage.equals(_that.additionalLanguage) : _that.additionalLanguage != null) return false;
		if (asPermitted != null ? !asPermitted.equals(_that.asPermitted) : _that.asPermitted != null) return false;
		if (!ListEquals.listEquals(eligibleCollateral, _that.eligibleCollateral)) return false;
		if (excludedCollateral != null ? !excludedCollateral.equals(_that.excludedCollateral) : _that.excludedCollateral != null) return false;
		if (party != null ? !party.equals(_that.party) : _that.party != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (additionalLanguage != null ? additionalLanguage.hashCode() : 0);
		_result = 31 * _result + (asPermitted != null ? asPermitted.hashCode() : 0);
		_result = 31 * _result + (eligibleCollateral != null ? eligibleCollateral.hashCode() : 0);
		_result = 31 * _result + (excludedCollateral != null ? excludedCollateral.hashCode() : 0);
		_result = 31 * _result + (party != null ? party.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "PostingObligationsElection {" +
			"additionalLanguage=" + this.additionalLanguage + ", " +
			"asPermitted=" + this.asPermitted + ", " +
			"eligibleCollateral=" + this.eligibleCollateral + ", " +
			"excludedCollateral=" + this.excludedCollateral + ", " +
			"party=" + this.party +
		'}';
	}
}
