package org.isda.cdm;

import java.math.BigDecimal;
import org.isda.cdm.Money;
import org.isda.cdm.PremiumTypeEnum;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.PremiumExpressionMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;
		import java.math.BigDecimal;

/**
 * This class corresponds to the FpML Premium.model group for representing the option premium when expressed in a way other than an amount.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class PremiumExpression extends RosettaModelObject {
	private final BigDecimal percentageOfNotional;
	private final PremiumTypeEnum premiumType;
	private final Money pricePerOption;
	private static PremiumExpressionMeta metaData = new PremiumExpressionMeta();
	
	PremiumExpression(PremiumExpressionBuilder builder) {
		this.percentageOfNotional = builder.getPercentageOfNotional();
		this.premiumType = builder.getPremiumType();
		this.pricePerOption = ofNullable(builder.getPricePerOption()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The amount of premium to be paid expressed as a percentage of the notional value of the transaction. A percentage of 5% would be expressed as 0.05.
	 */
	@RosettaSynonym(value="percentageOfNotional", source="FpML_5_10")
	@RosettaSynonym(value="percentageOfNotional", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="percentageOfNotional", source="DTCC_11_0")
	@RosettaSynonym(value="percentageOfNotional", source="DTCC_9_0")
	@RosettaSynonym(value="percentageOfNotional", source="CME_ClearedConfirm_1_17")
	public final BigDecimal getPercentageOfNotional() {
		return percentageOfNotional;
	}
	
	/**
	 * Forward start premium type
	 */
	@RosettaSynonym(value="premiumType", source="FpML_5_10")
	@RosettaSynonym(value="premiumType", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="premiumType", source="DTCC_11_0")
	@RosettaSynonym(value="premiumType", source="DTCC_9_0")
	@RosettaSynonym(value="premiumType", source="CME_ClearedConfirm_1_17")
	public final PremiumTypeEnum getPremiumType() {
		return premiumType;
	}
	
	/**
	 * The amount of premium to be paid expressed as a function of the number of options.
	 */
	@RosettaSynonym(value="pricePerOption", source="FpML_5_10")
	@RosettaSynonym(value="pricePerOption", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="pricePerOption", source="DTCC_11_0")
	@RosettaSynonym(value="pricePerOption", source="DTCC_9_0")
	@RosettaSynonym(value="pricePerOption", source="CME_ClearedConfirm_1_17")
	public final Money getPricePerOption() {
		return pricePerOption;
	}
	
	
	@Override
	public RosettaMetaData<? extends PremiumExpression> metaData() {
		return metaData;
	} 
	
	public PremiumExpressionBuilder toBuilder() {
		PremiumExpressionBuilder builder = new PremiumExpressionBuilder();
		ofNullable(getPercentageOfNotional()).ifPresent(builder::setPercentageOfNotional);
		ofNullable(getPremiumType()).ifPresent(builder::setPremiumType);
		ofNullable(getPricePerOption()).ifPresent(builder::setPricePerOption);
		return builder;
	}

	public static PremiumExpressionBuilder builder() {
		return new PremiumExpressionBuilder();
	}

	public static  class PremiumExpressionBuilder extends RosettaModelObjectBuilder {
	
		protected BigDecimal percentageOfNotional;
		protected PremiumTypeEnum premiumType;
		protected Money.MoneyBuilder pricePerOption;
	
		public PremiumExpressionBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends PremiumExpression> metaData() {
			return metaData;
		} 
	
		
		public BigDecimal getPercentageOfNotional() {
			return percentageOfNotional;
		}
		
		
		public PremiumTypeEnum getPremiumType() {
			return premiumType;
		}
		
		
		public Money.MoneyBuilder getPricePerOption() {
			return pricePerOption;
		}
		
		public Money.MoneyBuilder getOrCreatePricePerOption() {
			if (pricePerOption!=null) {
				return pricePerOption;
			}
			else return pricePerOption = new Money.MoneyBuilder();
		}
		
	
		public PremiumExpressionBuilder setPercentageOfNotional(BigDecimal percentageOfNotional) {
			this.percentageOfNotional = percentageOfNotional;
			return this;
		}
		public PremiumExpressionBuilder setPremiumType(PremiumTypeEnum premiumType) {
			this.premiumType = premiumType;
			return this;
		}
		public PremiumExpressionBuilder setPricePerOption(Money pricePerOption) {
			this.pricePerOption = pricePerOption.toBuilder();
			return this;
		}
		public PremiumExpressionBuilder setPricePerOptionBuilder(Money.MoneyBuilder pricePerOption) {
			this.pricePerOption = pricePerOption;
			return this;
		}
		
		public PremiumExpression build() {
			return new PremiumExpression(this);
		}
	
		@Override
		public PremiumExpressionBuilder prune() {
			if (pricePerOption!=null && !pricePerOption.prune().hasData()) pricePerOption = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getPercentageOfNotional()!=null) return true;
			if (getPremiumType()!=null) return true;
			if (getPricePerOption()!=null && getPricePerOption().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("percentageOfNotional"), BigDecimal.class, percentageOfNotional, this);
			processor.processBasic(path.newSubPath("premiumType"), PremiumTypeEnum.class, premiumType, this);
			
			processRosetta(path.newSubPath("pricePerOption"), processor, Money.class, pricePerOption);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			PremiumExpressionBuilder _that = (PremiumExpressionBuilder) o;
		
			if (percentageOfNotional != null ? !percentageOfNotional.equals(_that.percentageOfNotional) : _that.percentageOfNotional != null) return false;
			if (premiumType != null ? !premiumType.equals(_that.premiumType) : _that.premiumType != null) return false;
			if (pricePerOption != null ? !pricePerOption.equals(_that.pricePerOption) : _that.pricePerOption != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (percentageOfNotional != null ? percentageOfNotional.hashCode() : 0);
			_result = 31 * _result + (premiumType != null ? premiumType.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (pricePerOption != null ? pricePerOption.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "PremiumExpressionBuilder {" +
				"percentageOfNotional=" + this.percentageOfNotional + ", " +
				"premiumType=" + this.premiumType + ", " +
				"pricePerOption=" + this.pricePerOption +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("percentageOfNotional"), BigDecimal.class, percentageOfNotional, this);
		processor.processBasic(path.newSubPath("premiumType"), PremiumTypeEnum.class, premiumType, this);
		
		processRosetta(path.newSubPath("pricePerOption"), processor, Money.class, pricePerOption);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		PremiumExpression _that = (PremiumExpression) o;
	
		if (percentageOfNotional != null ? !percentageOfNotional.equals(_that.percentageOfNotional) : _that.percentageOfNotional != null) return false;
		if (premiumType != null ? !premiumType.equals(_that.premiumType) : _that.premiumType != null) return false;
		if (pricePerOption != null ? !pricePerOption.equals(_that.pricePerOption) : _that.pricePerOption != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (percentageOfNotional != null ? percentageOfNotional.hashCode() : 0);
		_result = 31 * _result + (premiumType != null ? premiumType.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (pricePerOption != null ? pricePerOption.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "PremiumExpression {" +
			"percentageOfNotional=" + this.percentageOfNotional + ", " +
			"premiumType=" + this.premiumType + ", " +
			"pricePerOption=" + this.pricePerOption +
		'}';
	}
}
