package org.isda.cdm;

import java.math.BigDecimal;
import org.isda.cdm.ActualPrice;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.PriceMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;
		import java.math.BigDecimal;

/**
 * A generic representation of price applicable to both derivatives and securities.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="Price", source="FpML_5_10")
public class Price extends RosettaModelObject {
	private final BigDecimal accruedInterest;
	private final ActualPrice cleanNetPrice;
	private final ActualPrice grossPrice;
	private final ActualPrice netPrice;
	private static PriceMeta metaData = new PriceMeta();
	
	Price(PriceBuilder builder) {
		this.accruedInterest = builder.getAccruedInterest();
		this.cleanNetPrice = ofNullable(builder.getCleanNetPrice()).map(f->f.build()).orElse(null);
		this.grossPrice = ofNullable(builder.getGrossPrice()).map(f->f.build()).orElse(null);
		this.netPrice = ofNullable(builder.getNetPrice()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * Specifies the accrued interest that are part of the dirty price in the case of a fixed income security or a convertible bond. Expressed in percentage of the notional.
	 */
	@RosettaSynonym(value="accruedInterestPrice", source="FpML_5_10")
	public final BigDecimal getAccruedInterest() {
		return accruedInterest;
	}
	
	/**
	 * Specifies the clean price of the underlier (e.g. excluding accrued interest), net of commissions.
	 */
	@RosettaSynonym(value="cleanNetPrice", source="FpML_5_10")
	public final ActualPrice getCleanNetPrice() {
		return cleanNetPrice;
	}
	
	/**
	 * Specifies the price of the underlier, before commissions.
	 */
	@RosettaSynonym(value="grossPrice", source="FpML_5_10")
	public final ActualPrice getGrossPrice() {
		return grossPrice;
	}
	
	/**
	 * Specifies the dirty price of the underlier (e.g. including accrued interest), net of commissions.
	 */
	@RosettaSynonym(value="netPrice", source="FpML_5_10")
	public final ActualPrice getNetPrice() {
		return netPrice;
	}
	
	
	@Override
	public RosettaMetaData<? extends Price> metaData() {
		return metaData;
	} 
	
	public PriceBuilder toBuilder() {
		PriceBuilder builder = new PriceBuilder();
		ofNullable(getAccruedInterest()).ifPresent(builder::setAccruedInterest);
		ofNullable(getCleanNetPrice()).ifPresent(builder::setCleanNetPrice);
		ofNullable(getGrossPrice()).ifPresent(builder::setGrossPrice);
		ofNullable(getNetPrice()).ifPresent(builder::setNetPrice);
		return builder;
	}

	public static PriceBuilder builder() {
		return new PriceBuilder();
	}

	public static  class PriceBuilder extends RosettaModelObjectBuilder {
	
		protected BigDecimal accruedInterest;
		protected ActualPrice.ActualPriceBuilder cleanNetPrice;
		protected ActualPrice.ActualPriceBuilder grossPrice;
		protected ActualPrice.ActualPriceBuilder netPrice;
	
		public PriceBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends Price> metaData() {
			return metaData;
		} 
	
		
		public BigDecimal getAccruedInterest() {
			return accruedInterest;
		}
		
		
		public ActualPrice.ActualPriceBuilder getCleanNetPrice() {
			return cleanNetPrice;
		}
		
		public ActualPrice.ActualPriceBuilder getOrCreateCleanNetPrice() {
			if (cleanNetPrice!=null) {
				return cleanNetPrice;
			}
			else return cleanNetPrice = new ActualPrice.ActualPriceBuilder();
		}
		
		
		public ActualPrice.ActualPriceBuilder getGrossPrice() {
			return grossPrice;
		}
		
		public ActualPrice.ActualPriceBuilder getOrCreateGrossPrice() {
			if (grossPrice!=null) {
				return grossPrice;
			}
			else return grossPrice = new ActualPrice.ActualPriceBuilder();
		}
		
		
		public ActualPrice.ActualPriceBuilder getNetPrice() {
			return netPrice;
		}
		
		public ActualPrice.ActualPriceBuilder getOrCreateNetPrice() {
			if (netPrice!=null) {
				return netPrice;
			}
			else return netPrice = new ActualPrice.ActualPriceBuilder();
		}
		
	
		public PriceBuilder setAccruedInterest(BigDecimal accruedInterest) {
			this.accruedInterest = accruedInterest;
			return this;
		}
		public PriceBuilder setCleanNetPrice(ActualPrice cleanNetPrice) {
			this.cleanNetPrice = cleanNetPrice.toBuilder();
			return this;
		}
		public PriceBuilder setCleanNetPriceBuilder(ActualPrice.ActualPriceBuilder cleanNetPrice) {
			this.cleanNetPrice = cleanNetPrice;
			return this;
		}
		public PriceBuilder setGrossPrice(ActualPrice grossPrice) {
			this.grossPrice = grossPrice.toBuilder();
			return this;
		}
		public PriceBuilder setGrossPriceBuilder(ActualPrice.ActualPriceBuilder grossPrice) {
			this.grossPrice = grossPrice;
			return this;
		}
		public PriceBuilder setNetPrice(ActualPrice netPrice) {
			this.netPrice = netPrice.toBuilder();
			return this;
		}
		public PriceBuilder setNetPriceBuilder(ActualPrice.ActualPriceBuilder netPrice) {
			this.netPrice = netPrice;
			return this;
		}
		
		public Price build() {
			return new Price(this);
		}
	
		@Override
		public PriceBuilder prune() {
			if (cleanNetPrice!=null && !cleanNetPrice.prune().hasData()) cleanNetPrice = null;
			if (grossPrice!=null && !grossPrice.prune().hasData()) grossPrice = null;
			if (netPrice!=null && !netPrice.prune().hasData()) netPrice = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getAccruedInterest()!=null) return true;
			if (getCleanNetPrice()!=null && getCleanNetPrice().hasData()) return true;
			if (getGrossPrice()!=null && getGrossPrice().hasData()) return true;
			if (getNetPrice()!=null && getNetPrice().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("accruedInterest"), BigDecimal.class, accruedInterest, this);
			
			processRosetta(path.newSubPath("cleanNetPrice"), processor, ActualPrice.class, cleanNetPrice);
			processRosetta(path.newSubPath("grossPrice"), processor, ActualPrice.class, grossPrice);
			processRosetta(path.newSubPath("netPrice"), processor, ActualPrice.class, netPrice);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			PriceBuilder _that = (PriceBuilder) o;
		
			if (accruedInterest != null ? !accruedInterest.equals(_that.accruedInterest) : _that.accruedInterest != null) return false;
			if (cleanNetPrice != null ? !cleanNetPrice.equals(_that.cleanNetPrice) : _that.cleanNetPrice != null) return false;
			if (grossPrice != null ? !grossPrice.equals(_that.grossPrice) : _that.grossPrice != null) return false;
			if (netPrice != null ? !netPrice.equals(_that.netPrice) : _that.netPrice != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (accruedInterest != null ? accruedInterest.hashCode() : 0);
			_result = 31 * _result + (cleanNetPrice != null ? cleanNetPrice.hashCode() : 0);
			_result = 31 * _result + (grossPrice != null ? grossPrice.hashCode() : 0);
			_result = 31 * _result + (netPrice != null ? netPrice.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "PriceBuilder {" +
				"accruedInterest=" + this.accruedInterest + ", " +
				"cleanNetPrice=" + this.cleanNetPrice + ", " +
				"grossPrice=" + this.grossPrice + ", " +
				"netPrice=" + this.netPrice +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("accruedInterest"), BigDecimal.class, accruedInterest, this);
		
		processRosetta(path.newSubPath("cleanNetPrice"), processor, ActualPrice.class, cleanNetPrice);
		processRosetta(path.newSubPath("grossPrice"), processor, ActualPrice.class, grossPrice);
		processRosetta(path.newSubPath("netPrice"), processor, ActualPrice.class, netPrice);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		Price _that = (Price) o;
	
		if (accruedInterest != null ? !accruedInterest.equals(_that.accruedInterest) : _that.accruedInterest != null) return false;
		if (cleanNetPrice != null ? !cleanNetPrice.equals(_that.cleanNetPrice) : _that.cleanNetPrice != null) return false;
		if (grossPrice != null ? !grossPrice.equals(_that.grossPrice) : _that.grossPrice != null) return false;
		if (netPrice != null ? !netPrice.equals(_that.netPrice) : _that.netPrice != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (accruedInterest != null ? accruedInterest.hashCode() : 0);
		_result = 31 * _result + (cleanNetPrice != null ? cleanNetPrice.hashCode() : 0);
		_result = 31 * _result + (grossPrice != null ? grossPrice.hashCode() : 0);
		_result = 31 * _result + (netPrice != null ? netPrice.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "Price {" +
			"accruedInterest=" + this.accruedInterest + ", " +
			"cleanNetPrice=" + this.cleanNetPrice + ", " +
			"grossPrice=" + this.grossPrice + ", " +
			"netPrice=" + this.netPrice +
		'}';
	}
}
