package org.isda.cdm;

import org.isda.cdm.EquityValuation;
import org.isda.cdm.Price;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.PriceReturnTermsMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="ReturnLegValuationPrice", source="FpML_5_10")
public class PriceReturnTerms extends RosettaModelObject {
	private final Price initialPrice;
	private final EquityValuation valuationPriceFinal;
	private final EquityValuation valuationPriceInterim;
	private static PriceReturnTermsMeta metaData = new PriceReturnTermsMeta();
	
	PriceReturnTerms(PriceReturnTermsBuilder builder) {
		this.initialPrice = ofNullable(builder.getInitialPrice()).map(f->f.build()).orElse(null);
		this.valuationPriceFinal = ofNullable(builder.getValuationPriceFinal()).map(f->f.build()).orElse(null);
		this.valuationPriceInterim = ofNullable(builder.getValuationPriceInterim()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * Specifies the initial reference price of the underlier. This price can be expressed either as an actual amount/currency, as a determination method, or by reference to another value specified in the swap document.
	 */
	@RosettaSynonym(value="initialPrice", source="FpML_5_10", path="rateOfReturn")
	public final Price getInitialPrice() {
		return initialPrice;
	}
	
	/**
	 * 2018 ISDA CDM Equity Confirmation for Security Equity Swap: Final Price | Specifies the final valuation price of the underlier. This price can be expressed either as an actual amount/currency, as a determination method, or by reference to another value specified in the swap document.
	 */
	@RosettaSynonym(value="valuationPriceFinal", source="FpML_5_10", path="rateOfReturn")
	public final EquityValuation getValuationPriceFinal() {
		return valuationPriceFinal;
	}
	
	/**
	 * Specifies the interim valuation price(s) of the underlier. This price can be expressed either as an actual amount/currency, as a determination method, or by reference to another value specified in the swap document.
	 */
	@RosettaSynonym(value="valuationPriceInterim", source="FpML_5_10", path="rateOfReturn")
	public final EquityValuation getValuationPriceInterim() {
		return valuationPriceInterim;
	}
	
	
	@Override
	public RosettaMetaData<? extends PriceReturnTerms> metaData() {
		return metaData;
	} 
	
	public PriceReturnTermsBuilder toBuilder() {
		PriceReturnTermsBuilder builder = new PriceReturnTermsBuilder();
		ofNullable(getInitialPrice()).ifPresent(builder::setInitialPrice);
		ofNullable(getValuationPriceFinal()).ifPresent(builder::setValuationPriceFinal);
		ofNullable(getValuationPriceInterim()).ifPresent(builder::setValuationPriceInterim);
		return builder;
	}

	public static PriceReturnTermsBuilder builder() {
		return new PriceReturnTermsBuilder();
	}

	public static  class PriceReturnTermsBuilder extends RosettaModelObjectBuilder {
	
		protected Price.PriceBuilder initialPrice;
		protected EquityValuation.EquityValuationBuilder valuationPriceFinal;
		protected EquityValuation.EquityValuationBuilder valuationPriceInterim;
	
		public PriceReturnTermsBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends PriceReturnTerms> metaData() {
			return metaData;
		} 
	
		
		public Price.PriceBuilder getInitialPrice() {
			return initialPrice;
		}
		
		public Price.PriceBuilder getOrCreateInitialPrice() {
			if (initialPrice!=null) {
				return initialPrice;
			}
			else return initialPrice = new Price.PriceBuilder();
		}
		
		
		public EquityValuation.EquityValuationBuilder getValuationPriceFinal() {
			return valuationPriceFinal;
		}
		
		public EquityValuation.EquityValuationBuilder getOrCreateValuationPriceFinal() {
			if (valuationPriceFinal!=null) {
				return valuationPriceFinal;
			}
			else return valuationPriceFinal = new EquityValuation.EquityValuationBuilder();
		}
		
		
		public EquityValuation.EquityValuationBuilder getValuationPriceInterim() {
			return valuationPriceInterim;
		}
		
		public EquityValuation.EquityValuationBuilder getOrCreateValuationPriceInterim() {
			if (valuationPriceInterim!=null) {
				return valuationPriceInterim;
			}
			else return valuationPriceInterim = new EquityValuation.EquityValuationBuilder();
		}
		
	
		public PriceReturnTermsBuilder setInitialPrice(Price initialPrice) {
			this.initialPrice = initialPrice.toBuilder();
			return this;
		}
		public PriceReturnTermsBuilder setInitialPriceBuilder(Price.PriceBuilder initialPrice) {
			this.initialPrice = initialPrice;
			return this;
		}
		public PriceReturnTermsBuilder setValuationPriceFinal(EquityValuation valuationPriceFinal) {
			this.valuationPriceFinal = valuationPriceFinal.toBuilder();
			return this;
		}
		public PriceReturnTermsBuilder setValuationPriceFinalBuilder(EquityValuation.EquityValuationBuilder valuationPriceFinal) {
			this.valuationPriceFinal = valuationPriceFinal;
			return this;
		}
		public PriceReturnTermsBuilder setValuationPriceInterim(EquityValuation valuationPriceInterim) {
			this.valuationPriceInterim = valuationPriceInterim.toBuilder();
			return this;
		}
		public PriceReturnTermsBuilder setValuationPriceInterimBuilder(EquityValuation.EquityValuationBuilder valuationPriceInterim) {
			this.valuationPriceInterim = valuationPriceInterim;
			return this;
		}
		
		public PriceReturnTerms build() {
			return new PriceReturnTerms(this);
		}
	
		@Override
		public PriceReturnTermsBuilder prune() {
			if (initialPrice!=null && !initialPrice.prune().hasData()) initialPrice = null;
			if (valuationPriceFinal!=null && !valuationPriceFinal.prune().hasData()) valuationPriceFinal = null;
			if (valuationPriceInterim!=null && !valuationPriceInterim.prune().hasData()) valuationPriceInterim = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getInitialPrice()!=null && getInitialPrice().hasData()) return true;
			if (getValuationPriceFinal()!=null && getValuationPriceFinal().hasData()) return true;
			if (getValuationPriceInterim()!=null && getValuationPriceInterim().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("initialPrice"), processor, Price.class, initialPrice);
			processRosetta(path.newSubPath("valuationPriceFinal"), processor, EquityValuation.class, valuationPriceFinal);
			processRosetta(path.newSubPath("valuationPriceInterim"), processor, EquityValuation.class, valuationPriceInterim);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			PriceReturnTermsBuilder _that = (PriceReturnTermsBuilder) o;
		
			if (initialPrice != null ? !initialPrice.equals(_that.initialPrice) : _that.initialPrice != null) return false;
			if (valuationPriceFinal != null ? !valuationPriceFinal.equals(_that.valuationPriceFinal) : _that.valuationPriceFinal != null) return false;
			if (valuationPriceInterim != null ? !valuationPriceInterim.equals(_that.valuationPriceInterim) : _that.valuationPriceInterim != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (initialPrice != null ? initialPrice.hashCode() : 0);
			_result = 31 * _result + (valuationPriceFinal != null ? valuationPriceFinal.hashCode() : 0);
			_result = 31 * _result + (valuationPriceInterim != null ? valuationPriceInterim.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "PriceReturnTermsBuilder {" +
				"initialPrice=" + this.initialPrice + ", " +
				"valuationPriceFinal=" + this.valuationPriceFinal + ", " +
				"valuationPriceInterim=" + this.valuationPriceInterim +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("initialPrice"), processor, Price.class, initialPrice);
		processRosetta(path.newSubPath("valuationPriceFinal"), processor, EquityValuation.class, valuationPriceFinal);
		processRosetta(path.newSubPath("valuationPriceInterim"), processor, EquityValuation.class, valuationPriceInterim);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		PriceReturnTerms _that = (PriceReturnTerms) o;
	
		if (initialPrice != null ? !initialPrice.equals(_that.initialPrice) : _that.initialPrice != null) return false;
		if (valuationPriceFinal != null ? !valuationPriceFinal.equals(_that.valuationPriceFinal) : _that.valuationPriceFinal != null) return false;
		if (valuationPriceInterim != null ? !valuationPriceInterim.equals(_that.valuationPriceInterim) : _that.valuationPriceInterim != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (initialPrice != null ? initialPrice.hashCode() : 0);
		_result = 31 * _result + (valuationPriceFinal != null ? valuationPriceFinal.hashCode() : 0);
		_result = 31 * _result + (valuationPriceInterim != null ? valuationPriceInterim.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "PriceReturnTerms {" +
			"initialPrice=" + this.initialPrice + ", " +
			"valuationPriceFinal=" + this.valuationPriceFinal + ", " +
			"valuationPriceInterim=" + this.valuationPriceInterim +
		'}';
	}
}
