package org.isda.cdm;

import java.lang.Boolean;
import java.lang.String;
import org.isda.cdm.metafields.ReferenceWithMetaParty;
import org.isda.cdm.metafields.ReferenceWithMetaParty.ReferenceWithMetaPartyBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.ProcessAgentElectionMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.stream.Collectors;
		import static java.util.Optional.ofNullable;

/**
 * A class to specify the parties&#39; respective elections with respect to the Process Agent as part of the English Law ISDA CSA.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class ProcessAgentElection extends RosettaModelObject {
	private final Boolean isApplicable;
	private final ReferenceWithMetaParty party;
	private final String partyName;
	private final String specification;
	private static ProcessAgentElectionMeta metaData = new ProcessAgentElectionMeta();
	
	ProcessAgentElection(ProcessAgentElectionBuilder builder) {
		this.isApplicable = builder.getIsApplicable();
		this.party = ofNullable(builder.getParty()).map(f->f.build()).orElse(null);
		this.partyName = builder.getPartyName();
		this.specification = builder.getSpecification();
	}
	
	/**
	 * The qualification of whether the Process Agent is applicable (True) or not applicable (False).
	 */
	@RosettaSynonym(value="", source="ISDA_Create_1_0")
	@RosettaSynonym(value="", source="ISDA_Create_1_0")
	public final Boolean getIsApplicable() {
		return isApplicable;
	}
	
	/**
	 * The elective party.
	 */
	@RosettaSynonym(value="party", source="AcadiaSoft_AM_1_0")
	public final ReferenceWithMetaParty getParty() {
		return party;
	}
	
	/**
	 * The elective party as string.
	 */
	@RosettaSynonym(value="", source="ISDA_Create_1_0")
	@RosettaSynonym(value="", source="ISDA_Create_1_0")
	public final String getPartyName() {
		return partyName;
	}
	
	/**
	 * The Process Agent specification, when applicable.
	 */
	@RosettaSynonym(value="partyA_specify", source="ISDA_Create_1_0")
	@RosettaSynonym(value="partyB_specify", source="ISDA_Create_1_0")
	public final String getSpecification() {
		return specification;
	}
	
	
	@Override
	public RosettaMetaData<? extends ProcessAgentElection> metaData() {
		return metaData;
	} 
	
	public ProcessAgentElectionBuilder toBuilder() {
		ProcessAgentElectionBuilder builder = new ProcessAgentElectionBuilder();
		ofNullable(getIsApplicable()).ifPresent(builder::setIsApplicable);
		ofNullable(getParty()).ifPresent(builder::setParty);
		ofNullable(getPartyName()).ifPresent(builder::setPartyName);
		ofNullable(getSpecification()).ifPresent(builder::setSpecification);
		return builder;
	}

	public static ProcessAgentElectionBuilder builder() {
		return new ProcessAgentElectionBuilder();
	}

	public static  class ProcessAgentElectionBuilder extends RosettaModelObjectBuilder {
	
		protected Boolean isApplicable;
		protected ReferenceWithMetaPartyBuilder party;
		protected String partyName;
		protected String specification;
	
		public ProcessAgentElectionBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends ProcessAgentElection> metaData() {
			return metaData;
		} 
	
		
		public Boolean getIsApplicable() {
			return isApplicable;
		}
		
		
		public ReferenceWithMetaPartyBuilder getParty() {
			return party;
		}
		
		public ReferenceWithMetaPartyBuilder getOrCreateParty() {
			if (party!=null) {
				return party;
			}
			else return party = new ReferenceWithMetaPartyBuilder();
		}
		
		
		public String getPartyName() {
			return partyName;
		}
		
		
		public String getSpecification() {
			return specification;
		}
		
	
		public ProcessAgentElectionBuilder setIsApplicable(Boolean isApplicable) {
			this.isApplicable = isApplicable;
			return this;
		}
		public ProcessAgentElectionBuilder setParty(ReferenceWithMetaParty party) {
			this.party = party.toBuilder();
			return this;
		}
		public ProcessAgentElectionBuilder setPartyBuilder(ReferenceWithMetaPartyBuilder party) {
			this.party = party;
			return this;
		}
		public ProcessAgentElectionBuilder setPartyRef(Party.PartyBuilder party) {
			return setParty(ReferenceWithMetaParty.builder().setValueBuilder(party).build());
		}
		public ProcessAgentElectionBuilder setPartyRef(Party party) {
			return setPartyRef(Party.builder());
		}
		public ProcessAgentElectionBuilder setPartyName(String partyName) {
			this.partyName = partyName;
			return this;
		}
		public ProcessAgentElectionBuilder setSpecification(String specification) {
			this.specification = specification;
			return this;
		}
		
		public ProcessAgentElection build() {
			return new ProcessAgentElection(this);
		}
	
		@Override
		public ProcessAgentElectionBuilder prune() {
			if (party!=null && !party.prune().hasData()) party = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getIsApplicable()!=null) return true;
			if (getParty()!=null && getParty().hasData()) return true;
			if (getPartyName()!=null) return true;
			if (getSpecification()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("isApplicable"), Boolean.class, isApplicable, this);
			processor.processBasic(path.newSubPath("partyName"), String.class, partyName, this);
			processor.processBasic(path.newSubPath("specification"), String.class, specification, this);
			
			processRosetta(path.newSubPath("party"), processor, ReferenceWithMetaParty.class, party);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			ProcessAgentElectionBuilder _that = (ProcessAgentElectionBuilder) o;
		
			if (isApplicable != null ? !isApplicable.equals(_that.isApplicable) : _that.isApplicable != null) return false;
			if (party != null ? !party.equals(_that.party) : _that.party != null) return false;
			if (partyName != null ? !partyName.equals(_that.partyName) : _that.partyName != null) return false;
			if (specification != null ? !specification.equals(_that.specification) : _that.specification != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (isApplicable != null ? isApplicable.hashCode() : 0);
			_result = 31 * _result + (party != null ? party.hashCode() : 0);
			_result = 31 * _result + (partyName != null ? partyName.hashCode() : 0);
			_result = 31 * _result + (specification != null ? specification.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "ProcessAgentElectionBuilder {" +
				"isApplicable=" + this.isApplicable + ", " +
				"party=" + this.party + ", " +
				"partyName=" + this.partyName + ", " +
				"specification=" + this.specification +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("isApplicable"), Boolean.class, isApplicable, this);
		processor.processBasic(path.newSubPath("partyName"), String.class, partyName, this);
		processor.processBasic(path.newSubPath("specification"), String.class, specification, this);
		
		processRosetta(path.newSubPath("party"), processor, ReferenceWithMetaParty.class, party);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		ProcessAgentElection _that = (ProcessAgentElection) o;
	
		if (isApplicable != null ? !isApplicable.equals(_that.isApplicable) : _that.isApplicable != null) return false;
		if (party != null ? !party.equals(_that.party) : _that.party != null) return false;
		if (partyName != null ? !partyName.equals(_that.partyName) : _that.partyName != null) return false;
		if (specification != null ? !specification.equals(_that.specification) : _that.specification != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (isApplicable != null ? isApplicable.hashCode() : 0);
		_result = 31 * _result + (party != null ? party.hashCode() : 0);
		_result = 31 * _result + (partyName != null ? partyName.hashCode() : 0);
		_result = 31 * _result + (specification != null ? specification.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "ProcessAgentElection {" +
			"isApplicable=" + this.isApplicable + ", " +
			"party=" + this.party + ", " +
			"partyName=" + this.partyName + ", " +
			"specification=" + this.specification +
		'}';
	}
}
