package org.isda.cdm;

import org.isda.cdm.ContractualProduct;
import org.isda.cdm.ForeignExchange;
import org.isda.cdm.Index;
import org.isda.cdm.Loan;
import org.isda.cdm.Security;
import org.isda.cdm.metafields.MetaFields;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.ProductMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class to represent a financial product. With respect to contractual products, this class specifies the pre-execution product characteristics (the ContractualProduct class). This class is used as underlying for the option exercise representation, which makes use of the contractualProduct attribute to support the swaption use case, with the exercise into a swap. In a complete workflow, the swaption contract itself then needs to be superseded by a swap contract underpinned by the exercised swap as a contractualProduct.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class Product extends RosettaModelObject implements GlobalKey {
	private final ContractualProduct contractualProduct;
	private final ForeignExchange foreignExchange;
	private final Index index;
	private final Loan loan;
	private final MetaFields meta;
	private final Security security;
	private static ProductMeta metaData = new ProductMeta();
	
	Product(ProductBuilder builder) {
		this.contractualProduct = ofNullable(builder.getContractualProduct()).map(f->f.build()).orElse(null);
		this.foreignExchange = ofNullable(builder.getForeignExchange()).map(f->f.build()).orElse(null);
		this.index = ofNullable(builder.getIndex()).map(f->f.build()).orElse(null);
		this.loan = ofNullable(builder.getLoan()).map(f->f.build()).orElse(null);
		this.meta = ofNullable(builder.getMeta()).map(f->f.build()).orElse(null);
		this.security = ofNullable(builder.getSecurity()).map(f->f.build()).orElse(null);
	}
	
	@RosettaSynonym(value="creditDefaultSwap", source="FpML_5_10")
	@RosettaSynonym(value="creditDefaultSwap", source="DTCC_11_0")
	@RosettaSynonym(value="creditDefaultSwap", source="DTCC_9_0")
	@RosettaSynonym(value="creditDefaultSwap", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="creditDefaultSwap", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="swap", source="FpML_5_10")
	@RosettaSynonym(value="swap", source="DTCC_11_0")
	@RosettaSynonym(value="swap", source="DTCC_9_0")
	@RosettaSynonym(value="swap", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="swap", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="trade", source="CME_SubmissionIRS_1_0")
	public final ContractualProduct getContractualProduct() {
		return contractualProduct;
	}
	
	public final ForeignExchange getForeignExchange() {
		return foreignExchange;
	}
	
	@RosettaSynonym(value="index", source="FpML_5_10")
	public final Index getIndex() {
		return index;
	}
	
	@RosettaSynonym(value="loan", source="FpML_5_10")
	@RosettaSynonym(value="loan", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="loan", source="DTCC_11_0")
	@RosettaSynonym(value="loan", source="DTCC_9_0")
	@RosettaSynonym(value="loan", source="CME_ClearedConfirm_1_17")
	public final Loan getLoan() {
		return loan;
	}
	
	public final MetaFields getMeta() {
		return meta;
	}
	
	public final Security getSecurity() {
		return security;
	}
	
	
	@Override
	public RosettaMetaData<? extends Product> metaData() {
		return metaData;
	} 
	
	public ProductBuilder toBuilder() {
		ProductBuilder builder = new ProductBuilder();
		ofNullable(getContractualProduct()).ifPresent(builder::setContractualProduct);
		ofNullable(getForeignExchange()).ifPresent(builder::setForeignExchange);
		ofNullable(getIndex()).ifPresent(builder::setIndex);
		ofNullable(getLoan()).ifPresent(builder::setLoan);
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		ofNullable(getSecurity()).ifPresent(builder::setSecurity);
		return builder;
	}

	public static ProductBuilder builder() {
		return new ProductBuilder();
	}

	public static  class ProductBuilder extends RosettaModelObjectBuilder implements GlobalKeyBuilder<ProductBuilder> {
	
		protected ContractualProduct.ContractualProductBuilder contractualProduct;
		protected ForeignExchange.ForeignExchangeBuilder foreignExchange;
		protected Index.IndexBuilder index;
		protected Loan.LoanBuilder loan;
		protected MetaFields.MetaFieldsBuilder meta;
		protected Security.SecurityBuilder security;
	
		public ProductBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends Product> metaData() {
			return metaData;
		} 
	
		
		public ContractualProduct.ContractualProductBuilder getContractualProduct() {
			return contractualProduct;
		}
		
		public ContractualProduct.ContractualProductBuilder getOrCreateContractualProduct() {
			if (contractualProduct!=null) {
				return contractualProduct;
			}
			else return contractualProduct = new ContractualProduct.ContractualProductBuilder();
		}
		
		
		public ForeignExchange.ForeignExchangeBuilder getForeignExchange() {
			return foreignExchange;
		}
		
		public ForeignExchange.ForeignExchangeBuilder getOrCreateForeignExchange() {
			if (foreignExchange!=null) {
				return foreignExchange;
			}
			else return foreignExchange = new ForeignExchange.ForeignExchangeBuilder();
		}
		
		
		public Index.IndexBuilder getIndex() {
			return index;
		}
		
		public Index.IndexBuilder getOrCreateIndex() {
			if (index!=null) {
				return index;
			}
			else return index = new Index.IndexBuilder();
		}
		
		
		public Loan.LoanBuilder getLoan() {
			return loan;
		}
		
		public Loan.LoanBuilder getOrCreateLoan() {
			if (loan!=null) {
				return loan;
			}
			else return loan = new Loan.LoanBuilder();
		}
		
		
		public MetaFields.MetaFieldsBuilder getMeta() {
			return meta;
		}
		
		public MetaFields.MetaFieldsBuilder getOrCreateMeta() {
			if (meta!=null) {
				return meta;
			}
			else return meta = new MetaFields.MetaFieldsBuilder();
		}
		
		
		public Security.SecurityBuilder getSecurity() {
			return security;
		}
		
		public Security.SecurityBuilder getOrCreateSecurity() {
			if (security!=null) {
				return security;
			}
			else return security = new Security.SecurityBuilder();
		}
		
	
		public ProductBuilder setContractualProduct(ContractualProduct contractualProduct) {
			this.contractualProduct = contractualProduct.toBuilder();
			return this;
		}
		public ProductBuilder setContractualProductBuilder(ContractualProduct.ContractualProductBuilder contractualProduct) {
			this.contractualProduct = contractualProduct;
			return this;
		}
		public ProductBuilder setForeignExchange(ForeignExchange foreignExchange) {
			this.foreignExchange = foreignExchange.toBuilder();
			return this;
		}
		public ProductBuilder setForeignExchangeBuilder(ForeignExchange.ForeignExchangeBuilder foreignExchange) {
			this.foreignExchange = foreignExchange;
			return this;
		}
		public ProductBuilder setIndex(Index index) {
			this.index = index.toBuilder();
			return this;
		}
		public ProductBuilder setIndexBuilder(Index.IndexBuilder index) {
			this.index = index;
			return this;
		}
		public ProductBuilder setLoan(Loan loan) {
			this.loan = loan.toBuilder();
			return this;
		}
		public ProductBuilder setLoanBuilder(Loan.LoanBuilder loan) {
			this.loan = loan;
			return this;
		}
		public ProductBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		public ProductBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		public ProductBuilder setSecurity(Security security) {
			this.security = security.toBuilder();
			return this;
		}
		public ProductBuilder setSecurityBuilder(Security.SecurityBuilder security) {
			this.security = security;
			return this;
		}
		
		public Product build() {
			return new Product(this);
		}
	
		@Override
		public ProductBuilder prune() {
			if (contractualProduct!=null && !contractualProduct.prune().hasData()) contractualProduct = null;
			if (foreignExchange!=null && !foreignExchange.prune().hasData()) foreignExchange = null;
			if (index!=null && !index.prune().hasData()) index = null;
			if (loan!=null && !loan.prune().hasData()) loan = null;
			if (meta!=null && !meta.prune().hasData()) meta = null;
			if (security!=null && !security.prune().hasData()) security = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getContractualProduct()!=null && getContractualProduct().hasData()) return true;
			if (getForeignExchange()!=null && getForeignExchange().hasData()) return true;
			if (getIndex()!=null && getIndex().hasData()) return true;
			if (getLoan()!=null && getLoan().hasData()) return true;
			if (getMeta()!=null && getMeta().hasData()) return true;
			if (getSecurity()!=null && getSecurity().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("contractualProduct"), processor, ContractualProduct.class, contractualProduct);
			processRosetta(path.newSubPath("foreignExchange"), processor, ForeignExchange.class, foreignExchange);
			processRosetta(path.newSubPath("index"), processor, Index.class, index);
			processRosetta(path.newSubPath("loan"), processor, Loan.class, loan);
			processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
			processRosetta(path.newSubPath("security"), processor, Security.class, security);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			ProductBuilder _that = (ProductBuilder) o;
		
			if (contractualProduct != null ? !contractualProduct.equals(_that.contractualProduct) : _that.contractualProduct != null) return false;
			if (foreignExchange != null ? !foreignExchange.equals(_that.foreignExchange) : _that.foreignExchange != null) return false;
			if (index != null ? !index.equals(_that.index) : _that.index != null) return false;
			if (loan != null ? !loan.equals(_that.loan) : _that.loan != null) return false;
			if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
			if (security != null ? !security.equals(_that.security) : _that.security != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (contractualProduct != null ? contractualProduct.hashCode() : 0);
			_result = 31 * _result + (foreignExchange != null ? foreignExchange.hashCode() : 0);
			_result = 31 * _result + (index != null ? index.hashCode() : 0);
			_result = 31 * _result + (loan != null ? loan.hashCode() : 0);
			_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
			_result = 31 * _result + (security != null ? security.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "ProductBuilder {" +
				"contractualProduct=" + this.contractualProduct + ", " +
				"foreignExchange=" + this.foreignExchange + ", " +
				"index=" + this.index + ", " +
				"loan=" + this.loan + ", " +
				"meta=" + this.meta + ", " +
				"security=" + this.security +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("contractualProduct"), processor, ContractualProduct.class, contractualProduct);
		processRosetta(path.newSubPath("foreignExchange"), processor, ForeignExchange.class, foreignExchange);
		processRosetta(path.newSubPath("index"), processor, Index.class, index);
		processRosetta(path.newSubPath("loan"), processor, Loan.class, loan);
		processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
		processRosetta(path.newSubPath("security"), processor, Security.class, security);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		Product _that = (Product) o;
	
		if (contractualProduct != null ? !contractualProduct.equals(_that.contractualProduct) : _that.contractualProduct != null) return false;
		if (foreignExchange != null ? !foreignExchange.equals(_that.foreignExchange) : _that.foreignExchange != null) return false;
		if (index != null ? !index.equals(_that.index) : _that.index != null) return false;
		if (loan != null ? !loan.equals(_that.loan) : _that.loan != null) return false;
		if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
		if (security != null ? !security.equals(_that.security) : _that.security != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (contractualProduct != null ? contractualProduct.hashCode() : 0);
		_result = 31 * _result + (foreignExchange != null ? foreignExchange.hashCode() : 0);
		_result = 31 * _result + (index != null ? index.hashCode() : 0);
		_result = 31 * _result + (loan != null ? loan.hashCode() : 0);
		_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
		_result = 31 * _result + (security != null ? security.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "Product {" +
			"contractualProduct=" + this.contractualProduct + ", " +
			"foreignExchange=" + this.foreignExchange + ", " +
			"index=" + this.index + ", " +
			"loan=" + this.loan + ", " +
			"meta=" + this.meta + ", " +
			"security=" + this.security +
		'}';
	}
}
