package org.isda.cdm;

import org.isda.cdm.CreditEvents;
import org.isda.cdm.FloatingAmountEvents;
import org.isda.cdm.Money;
import org.isda.cdm.Obligations;
import org.isda.cdm.metafields.MetaFields;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.ProtectionTermsMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class to specify the terms for calculating a payout to protect the buyer of the swap in the case of a qualified credit event. These terms include the notional amount, the applicable credit events, the reference obligation, and in the case of a CDS on mortgage-backed securities, the floatingAmountEvents.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="ProtectionTerms", source="FpML_5_10")
@RosettaSynonym(value="ProtectionTerms", source="CME_SubmissionIRS_1_0")
@RosettaSynonym(value="ProtectionTerms", source="DTCC_11_0")
@RosettaSynonym(value="ProtectionTerms", source="DTCC_9_0")
@RosettaSynonym(value="ProtectionTerms", source="CME_ClearedConfirm_1_17")
public class ProtectionTerms extends RosettaModelObject implements GlobalKey {
	private final CreditEvents creditEvents;
	private final FloatingAmountEvents floatingAmountEvents;
	private final MetaFields meta;
	private final Money notionalAmount;
	private final Obligations obligations;
	private static ProtectionTermsMeta metaData = new ProtectionTermsMeta();
	
	ProtectionTerms(ProtectionTermsBuilder builder) {
		this.creditEvents = ofNullable(builder.getCreditEvents()).map(f->f.build()).orElse(null);
		this.floatingAmountEvents = ofNullable(builder.getFloatingAmountEvents()).map(f->f.build()).orElse(null);
		this.meta = ofNullable(builder.getMeta()).map(f->f.build()).orElse(null);
		this.notionalAmount = ofNullable(builder.getNotionalAmount()).map(f->f.build()).orElse(null);
		this.obligations = ofNullable(builder.getObligations()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * Specifies the applicable Credit Events that would trigger a settlement, as specified in the related Confirmation and defined in the ISDA 2014 Credit Definition article IV section 4.1.
	 */
	@RosettaSynonym(value="creditEvents", source="FpML_5_10")
	@RosettaSynonym(value="creditEvents", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="creditEvents", source="DTCC_11_0")
	@RosettaSynonym(value="creditEvents", source="DTCC_9_0")
	@RosettaSynonym(value="creditEvents", source="CME_ClearedConfirm_1_17")
	public final CreditEvents getCreditEvents() {
		return creditEvents;
	}
	
	/**
	 * This element contains the ISDA terms relating to the floating rate payment events and the implied additional fixed payments, applicable to the credit derivatives transactions on mortgage-backed securities with pay-as-you-go or physical settlement.
	 */
	@RosettaSynonym(value="floatingAmountEvents", source="FpML_5_10")
	@RosettaSynonym(value="floatingAmountEvents", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="floatingAmountEvents", source="DTCC_11_0")
	@RosettaSynonym(value="floatingAmountEvents", source="DTCC_9_0")
	@RosettaSynonym(value="floatingAmountEvents", source="CME_ClearedConfirm_1_17")
	public final FloatingAmountEvents getFloatingAmountEvents() {
		return floatingAmountEvents;
	}
	
	public final MetaFields getMeta() {
		return meta;
	}
	
	/**
	 * The notional amount of protection coverage. ISDA 2003 Term: Floating Rate Payer Calculation Amount.
	 */
	@RosettaSynonym(value="calculationAmount", source="FpML_5_10")
	@RosettaSynonym(value="calculationAmount", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="calculationAmount", source="DTCC_11_0")
	@RosettaSynonym(value="calculationAmount", source="DTCC_9_0")
	@RosettaSynonym(value="calculationAmount", source="CME_ClearedConfirm_1_17")
	public final Money getNotionalAmount() {
		return notionalAmount;
	}
	
	/**
	 * The underlying obligations of the reference entity on which you are buying or selling protection. The credit events Failure to Pay, Obligation Acceleration, Obligation Default, Restructuring, Repudiation/Moratorium are defined with respect to these obligations.
	 */
	@RosettaSynonym(value="obligations", source="FpML_5_10")
	@RosettaSynonym(value="obligations", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="obligations", source="DTCC_11_0")
	@RosettaSynonym(value="obligations", source="DTCC_9_0")
	@RosettaSynonym(value="obligations", source="CME_ClearedConfirm_1_17")
	public final Obligations getObligations() {
		return obligations;
	}
	
	
	@Override
	public RosettaMetaData<? extends ProtectionTerms> metaData() {
		return metaData;
	} 
	
	public ProtectionTermsBuilder toBuilder() {
		ProtectionTermsBuilder builder = new ProtectionTermsBuilder();
		ofNullable(getCreditEvents()).ifPresent(builder::setCreditEvents);
		ofNullable(getFloatingAmountEvents()).ifPresent(builder::setFloatingAmountEvents);
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		ofNullable(getNotionalAmount()).ifPresent(builder::setNotionalAmount);
		ofNullable(getObligations()).ifPresent(builder::setObligations);
		return builder;
	}

	public static ProtectionTermsBuilder builder() {
		return new ProtectionTermsBuilder();
	}

	public static  class ProtectionTermsBuilder extends RosettaModelObjectBuilder implements GlobalKeyBuilder<ProtectionTermsBuilder> {
	
		protected CreditEvents.CreditEventsBuilder creditEvents;
		protected FloatingAmountEvents.FloatingAmountEventsBuilder floatingAmountEvents;
		protected MetaFields.MetaFieldsBuilder meta;
		protected Money.MoneyBuilder notionalAmount;
		protected Obligations.ObligationsBuilder obligations;
	
		public ProtectionTermsBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends ProtectionTerms> metaData() {
			return metaData;
		} 
	
		
		public CreditEvents.CreditEventsBuilder getCreditEvents() {
			return creditEvents;
		}
		
		public CreditEvents.CreditEventsBuilder getOrCreateCreditEvents() {
			if (creditEvents!=null) {
				return creditEvents;
			}
			else return creditEvents = new CreditEvents.CreditEventsBuilder();
		}
		
		
		public FloatingAmountEvents.FloatingAmountEventsBuilder getFloatingAmountEvents() {
			return floatingAmountEvents;
		}
		
		public FloatingAmountEvents.FloatingAmountEventsBuilder getOrCreateFloatingAmountEvents() {
			if (floatingAmountEvents!=null) {
				return floatingAmountEvents;
			}
			else return floatingAmountEvents = new FloatingAmountEvents.FloatingAmountEventsBuilder();
		}
		
		
		public MetaFields.MetaFieldsBuilder getMeta() {
			return meta;
		}
		
		public MetaFields.MetaFieldsBuilder getOrCreateMeta() {
			if (meta!=null) {
				return meta;
			}
			else return meta = new MetaFields.MetaFieldsBuilder();
		}
		
		
		public Money.MoneyBuilder getNotionalAmount() {
			return notionalAmount;
		}
		
		public Money.MoneyBuilder getOrCreateNotionalAmount() {
			if (notionalAmount!=null) {
				return notionalAmount;
			}
			else return notionalAmount = new Money.MoneyBuilder();
		}
		
		
		public Obligations.ObligationsBuilder getObligations() {
			return obligations;
		}
		
		public Obligations.ObligationsBuilder getOrCreateObligations() {
			if (obligations!=null) {
				return obligations;
			}
			else return obligations = new Obligations.ObligationsBuilder();
		}
		
	
		public ProtectionTermsBuilder setCreditEvents(CreditEvents creditEvents) {
			this.creditEvents = creditEvents.toBuilder();
			return this;
		}
		public ProtectionTermsBuilder setCreditEventsBuilder(CreditEvents.CreditEventsBuilder creditEvents) {
			this.creditEvents = creditEvents;
			return this;
		}
		public ProtectionTermsBuilder setFloatingAmountEvents(FloatingAmountEvents floatingAmountEvents) {
			this.floatingAmountEvents = floatingAmountEvents.toBuilder();
			return this;
		}
		public ProtectionTermsBuilder setFloatingAmountEventsBuilder(FloatingAmountEvents.FloatingAmountEventsBuilder floatingAmountEvents) {
			this.floatingAmountEvents = floatingAmountEvents;
			return this;
		}
		public ProtectionTermsBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		public ProtectionTermsBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		public ProtectionTermsBuilder setNotionalAmount(Money notionalAmount) {
			this.notionalAmount = notionalAmount.toBuilder();
			return this;
		}
		public ProtectionTermsBuilder setNotionalAmountBuilder(Money.MoneyBuilder notionalAmount) {
			this.notionalAmount = notionalAmount;
			return this;
		}
		public ProtectionTermsBuilder setObligations(Obligations obligations) {
			this.obligations = obligations.toBuilder();
			return this;
		}
		public ProtectionTermsBuilder setObligationsBuilder(Obligations.ObligationsBuilder obligations) {
			this.obligations = obligations;
			return this;
		}
		
		public ProtectionTerms build() {
			return new ProtectionTerms(this);
		}
	
		@Override
		public ProtectionTermsBuilder prune() {
			if (creditEvents!=null && !creditEvents.prune().hasData()) creditEvents = null;
			if (floatingAmountEvents!=null && !floatingAmountEvents.prune().hasData()) floatingAmountEvents = null;
			if (meta!=null && !meta.prune().hasData()) meta = null;
			if (notionalAmount!=null && !notionalAmount.prune().hasData()) notionalAmount = null;
			if (obligations!=null && !obligations.prune().hasData()) obligations = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getCreditEvents()!=null && getCreditEvents().hasData()) return true;
			if (getFloatingAmountEvents()!=null && getFloatingAmountEvents().hasData()) return true;
			if (getMeta()!=null && getMeta().hasData()) return true;
			if (getNotionalAmount()!=null && getNotionalAmount().hasData()) return true;
			if (getObligations()!=null && getObligations().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("creditEvents"), processor, CreditEvents.class, creditEvents);
			processRosetta(path.newSubPath("floatingAmountEvents"), processor, FloatingAmountEvents.class, floatingAmountEvents);
			processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
			processRosetta(path.newSubPath("notionalAmount"), processor, Money.class, notionalAmount);
			processRosetta(path.newSubPath("obligations"), processor, Obligations.class, obligations);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			ProtectionTermsBuilder _that = (ProtectionTermsBuilder) o;
		
			if (creditEvents != null ? !creditEvents.equals(_that.creditEvents) : _that.creditEvents != null) return false;
			if (floatingAmountEvents != null ? !floatingAmountEvents.equals(_that.floatingAmountEvents) : _that.floatingAmountEvents != null) return false;
			if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
			if (notionalAmount != null ? !notionalAmount.equals(_that.notionalAmount) : _that.notionalAmount != null) return false;
			if (obligations != null ? !obligations.equals(_that.obligations) : _that.obligations != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (creditEvents != null ? creditEvents.hashCode() : 0);
			_result = 31 * _result + (floatingAmountEvents != null ? floatingAmountEvents.hashCode() : 0);
			_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
			_result = 31 * _result + (notionalAmount != null ? notionalAmount.hashCode() : 0);
			_result = 31 * _result + (obligations != null ? obligations.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "ProtectionTermsBuilder {" +
				"creditEvents=" + this.creditEvents + ", " +
				"floatingAmountEvents=" + this.floatingAmountEvents + ", " +
				"meta=" + this.meta + ", " +
				"notionalAmount=" + this.notionalAmount + ", " +
				"obligations=" + this.obligations +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("creditEvents"), processor, CreditEvents.class, creditEvents);
		processRosetta(path.newSubPath("floatingAmountEvents"), processor, FloatingAmountEvents.class, floatingAmountEvents);
		processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
		processRosetta(path.newSubPath("notionalAmount"), processor, Money.class, notionalAmount);
		processRosetta(path.newSubPath("obligations"), processor, Obligations.class, obligations);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		ProtectionTerms _that = (ProtectionTerms) o;
	
		if (creditEvents != null ? !creditEvents.equals(_that.creditEvents) : _that.creditEvents != null) return false;
		if (floatingAmountEvents != null ? !floatingAmountEvents.equals(_that.floatingAmountEvents) : _that.floatingAmountEvents != null) return false;
		if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
		if (notionalAmount != null ? !notionalAmount.equals(_that.notionalAmount) : _that.notionalAmount != null) return false;
		if (obligations != null ? !obligations.equals(_that.obligations) : _that.obligations != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (creditEvents != null ? creditEvents.hashCode() : 0);
		_result = 31 * _result + (floatingAmountEvents != null ? floatingAmountEvents.hashCode() : 0);
		_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
		_result = 31 * _result + (notionalAmount != null ? notionalAmount.hashCode() : 0);
		_result = 31 * _result + (obligations != null ? obligations.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "ProtectionTerms {" +
			"creditEvents=" + this.creditEvents + ", " +
			"floatingAmountEvents=" + this.floatingAmountEvents + ", " +
			"meta=" + this.meta + ", " +
			"notionalAmount=" + this.notionalAmount + ", " +
			"obligations=" + this.obligations +
		'}';
	}
}
