package org.isda.cdm;

import java.math.BigDecimal;
import org.isda.cdm.UnitEnum;
import org.isda.cdm.metafields.FieldWithMetaString;
import org.isda.cdm.metafields.FieldWithMetaString.FieldWithMetaStringBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.QuantityMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.math.BigDecimal;
		import static java.util.Optional.ofNullable;

/**
 * A class to specify an amount/number of securities or tangible assets such as a commodity product. The units qualifier is not used if the Quantity class is applied to securities.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class Quantity extends RosettaModelObject {
	private final BigDecimal amount;
	private final FieldWithMetaString currency;
	private final UnitEnum unit;
	private static QuantityMeta metaData = new QuantityMeta();
	
	Quantity(QuantityBuilder builder) {
		this.amount = builder.getAmount();
		this.currency = ofNullable(builder.getCurrency()).map(f->f.build()).orElse(null);
		this.unit = builder.getUnit();
	}
	
	/**
	 * The amount to quantify 
	 */
	@RosettaSynonym(value="amount", source="FpML_5_10")
	@RosettaSynonym(value="amount", source="Rosetta_Workbench")
	@RosettaSynonym(value="openUnits", source="FpML_5_10")
	@RosettaSynonym(value="initialValue", source="FpML_5_10")
	@RosettaSynonym(value="initialValue", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="initialValue", source="DTCC_11_0")
	@RosettaSynonym(value="initialValue", source="DTCC_9_0")
	@RosettaSynonym(value="initialValue", source="CME_ClearedConfirm_1_17")
	public final BigDecimal getAmount() {
		return amount;
	}
	
	/**
	 * The currency in which the associated amount is denominated, if applicable. The list of valid currencies is not presently positioned as an enumeration as part of the CDM because that scope is limited to the values specified by ISDA and FpML. As a result, implementers have to make reference to the relevant standard, such as the ISO 4217 standard for currency codes.
	 */
	@RosettaSynonym(value="currency", source="FpML_5_10")
	@RosettaSynonym(value="currency", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="currency", source="DTCC_11_0")
	@RosettaSynonym(value="currency", source="DTCC_9_0")
	@RosettaSynonym(value="currency", source="CME_ClearedConfirm_1_17")
	public final FieldWithMetaString getCurrency() {
		return currency;
	}
	
	/**
	 * The unit of measure, applicable to physical assets, e.g. MWh or MMBTU.
	 */
	@RosettaSynonym(value="unit", source="Rosetta_Workbench")
	public final UnitEnum getUnit() {
		return unit;
	}
	
	
	@Override
	public RosettaMetaData<? extends Quantity> metaData() {
		return metaData;
	} 
	
	public QuantityBuilder toBuilder() {
		QuantityBuilder builder = new QuantityBuilder();
		ofNullable(getAmount()).ifPresent(builder::setAmount);
		ofNullable(getCurrency()).ifPresent(builder::setCurrency);
		ofNullable(getUnit()).ifPresent(builder::setUnit);
		return builder;
	}

	public static QuantityBuilder builder() {
		return new QuantityBuilder();
	}

	public static  class QuantityBuilder extends RosettaModelObjectBuilder {
	
		protected BigDecimal amount;
		protected FieldWithMetaStringBuilder currency;
		protected UnitEnum unit;
	
		public QuantityBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends Quantity> metaData() {
			return metaData;
		} 
	
		
		public BigDecimal getAmount() {
			return amount;
		}
		
		
		public FieldWithMetaStringBuilder getCurrency() {
			return currency;
		}
		
		public FieldWithMetaStringBuilder getOrCreateCurrency() {
			if (currency!=null) {
				return currency;
			}
			else return currency = new FieldWithMetaStringBuilder();
		}
		
		
		public UnitEnum getUnit() {
			return unit;
		}
		
	
		public QuantityBuilder setAmount(BigDecimal amount) {
			this.amount = amount;
			return this;
		}
		public QuantityBuilder setCurrency(FieldWithMetaString currency) {
			this.currency = currency.toBuilder();
			return this;
		}
		public QuantityBuilder setCurrencyRef(String currency) {
			return setCurrency(FieldWithMetaString.builder().setValue(currency).build());
		}
		public QuantityBuilder setUnit(UnitEnum unit) {
			this.unit = unit;
			return this;
		}
		
		public Quantity build() {
			return new Quantity(this);
		}
	
		@Override
		public QuantityBuilder prune() {
			if (currency!=null && !currency.prune().hasData()) currency = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getAmount()!=null) return true;
			if (getCurrency()!=null) return true;
			if (getUnit()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("amount"), BigDecimal.class, amount, this);
			processor.processBasic(path.newSubPath("unit"), UnitEnum.class, unit, this);
			
			processRosetta(path.newSubPath("currency"), processor, FieldWithMetaString.class, currency);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			QuantityBuilder _that = (QuantityBuilder) o;
		
			if (amount != null ? !amount.equals(_that.amount) : _that.amount != null) return false;
			if (currency != null ? !currency.equals(_that.currency) : _that.currency != null) return false;
			if (unit != null ? !unit.equals(_that.unit) : _that.unit != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (amount != null ? amount.hashCode() : 0);
			_result = 31 * _result + (currency != null ? currency.hashCode() : 0);
			_result = 31 * _result + (unit != null ? unit.getClass().getName().hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "QuantityBuilder {" +
				"amount=" + this.amount + ", " +
				"currency=" + this.currency + ", " +
				"unit=" + this.unit +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("amount"), BigDecimal.class, amount, this);
		processor.processBasic(path.newSubPath("unit"), UnitEnum.class, unit, this);
		
		processRosetta(path.newSubPath("currency"), processor, FieldWithMetaString.class, currency);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		Quantity _that = (Quantity) o;
	
		if (amount != null ? !amount.equals(_that.amount) : _that.amount != null) return false;
		if (currency != null ? !currency.equals(_that.currency) : _that.currency != null) return false;
		if (unit != null ? !unit.equals(_that.unit) : _that.unit != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (amount != null ? amount.hashCode() : 0);
		_result = 31 * _result + (currency != null ? currency.hashCode() : 0);
		_result = 31 * _result + (unit != null ? unit.getClass().getName().hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "Quantity {" +
			"amount=" + this.amount + ", " +
			"currency=" + this.currency + ", " +
			"unit=" + this.unit +
		'}';
	}
}
