package org.isda.cdm;

import org.isda.cdm.Trade;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.QuantityChangePrimitiveMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * The primitive event to represent a change in quantity or notional.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class QuantityChangePrimitive extends RosettaModelObject {
	private final Trade after;
	private final Trade before;
	private static QuantityChangePrimitiveMeta metaData = new QuantityChangePrimitiveMeta();
	
	QuantityChangePrimitive(QuantityChangePrimitiveBuilder builder) {
		this.after = ofNullable(builder.getAfter()).map(f->f.build()).orElse(null);
		this.before = ofNullable(builder.getBefore()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The state of the trade (either an execution or a contract, as a follow-up from the event.
	 */
	@RosettaSynonym(value="after", source="Rosetta_Workbench")
	public final Trade getAfter() {
		return after;
	}
	
	/**
	 * The state of the trade (either an execution or a contract, before the event.
	 */
	@RosettaSynonym(value="before", source="Rosetta_Workbench")
	public final Trade getBefore() {
		return before;
	}
	
	
	@Override
	public RosettaMetaData<? extends QuantityChangePrimitive> metaData() {
		return metaData;
	} 
	
	public QuantityChangePrimitiveBuilder toBuilder() {
		QuantityChangePrimitiveBuilder builder = new QuantityChangePrimitiveBuilder();
		ofNullable(getAfter()).ifPresent(builder::setAfter);
		ofNullable(getBefore()).ifPresent(builder::setBefore);
		return builder;
	}

	public static QuantityChangePrimitiveBuilder builder() {
		return new QuantityChangePrimitiveBuilder();
	}

	public static  class QuantityChangePrimitiveBuilder extends RosettaModelObjectBuilder {
	
		protected Trade.TradeBuilder after;
		protected Trade.TradeBuilder before;
	
		public QuantityChangePrimitiveBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends QuantityChangePrimitive> metaData() {
			return metaData;
		} 
	
		
		public Trade.TradeBuilder getAfter() {
			return after;
		}
		
		public Trade.TradeBuilder getOrCreateAfter() {
			if (after!=null) {
				return after;
			}
			else return after = new Trade.TradeBuilder();
		}
		
		
		public Trade.TradeBuilder getBefore() {
			return before;
		}
		
		public Trade.TradeBuilder getOrCreateBefore() {
			if (before!=null) {
				return before;
			}
			else return before = new Trade.TradeBuilder();
		}
		
	
		public QuantityChangePrimitiveBuilder setAfter(Trade after) {
			this.after = after.toBuilder();
			return this;
		}
		public QuantityChangePrimitiveBuilder setAfterBuilder(Trade.TradeBuilder after) {
			this.after = after;
			return this;
		}
		public QuantityChangePrimitiveBuilder setBefore(Trade before) {
			this.before = before.toBuilder();
			return this;
		}
		public QuantityChangePrimitiveBuilder setBeforeBuilder(Trade.TradeBuilder before) {
			this.before = before;
			return this;
		}
		
		public QuantityChangePrimitive build() {
			return new QuantityChangePrimitive(this);
		}
	
		@Override
		public QuantityChangePrimitiveBuilder prune() {
			if (after!=null && !after.prune().hasData()) after = null;
			if (before!=null && !before.prune().hasData()) before = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getAfter()!=null && getAfter().hasData()) return true;
			if (getBefore()!=null && getBefore().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("after"), processor, Trade.class, after);
			processRosetta(path.newSubPath("before"), processor, Trade.class, before);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			QuantityChangePrimitiveBuilder _that = (QuantityChangePrimitiveBuilder) o;
		
			if (after != null ? !after.equals(_that.after) : _that.after != null) return false;
			if (before != null ? !before.equals(_that.before) : _that.before != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (after != null ? after.hashCode() : 0);
			_result = 31 * _result + (before != null ? before.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "QuantityChangePrimitiveBuilder {" +
				"after=" + this.after + ", " +
				"before=" + this.before +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("after"), processor, Trade.class, after);
		processRosetta(path.newSubPath("before"), processor, Trade.class, before);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		QuantityChangePrimitive _that = (QuantityChangePrimitive) o;
	
		if (after != null ? !after.equals(_that.after) : _that.after != null) return false;
		if (before != null ? !before.equals(_that.before) : _that.before != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (after != null ? after.hashCode() : 0);
		_result = 31 * _result + (before != null ? before.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "QuantityChangePrimitive {" +
			"after=" + this.after + ", " +
			"before=" + this.before +
		'}';
	}
}
