package org.isda.cdm;

import java.math.BigDecimal;
import org.isda.cdm.EquityValuation;
import org.isda.cdm.FxLinkedNotionalSchedule;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.QuantityMultiplierMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;
		import java.math.BigDecimal;

/**
 * Class to specify a mechanism for a quantity to be set as a multiplier to another (reference) quantity, based on a price observation. At the moment this class only supports FX or Equity-linked notional and re-uses existing building blocks for those 2 cases, until such time when component can be made more generic. This captures the case of resetting cross-currency swaps and resetting equity swaps.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class QuantityMultiplier extends RosettaModelObject {
	private final EquityValuation equityValuation;
	private final FxLinkedNotionalSchedule fxLinkedNotionalSchedule;
	private final BigDecimal multiplierValue;
	private static QuantityMultiplierMeta metaData = new QuantityMultiplierMeta();
	
	QuantityMultiplier(QuantityMultiplierBuilder builder) {
		this.equityValuation = ofNullable(builder.getEquityValuation()).map(f->f.build()).orElse(null);
		this.fxLinkedNotionalSchedule = ofNullable(builder.getFxLinkedNotionalSchedule()).map(f->f.build()).orElse(null);
		this.multiplierValue = builder.getMultiplierValue();
	}
	
	/**
	 * Multiplier specified as an equity valuation schedule, e.g. for a resetting equity swap.
	 */
	public final EquityValuation getEquityValuation() {
		return equityValuation;
	}
	
	/**
	 * Multiplier specified as an FX-linked schedule, e.g. for a resetting cross-currency swap..
	 */
	public final FxLinkedNotionalSchedule getFxLinkedNotionalSchedule() {
		return fxLinkedNotionalSchedule;
	}
	
	public final BigDecimal getMultiplierValue() {
		return multiplierValue;
	}
	
	
	@Override
	public RosettaMetaData<? extends QuantityMultiplier> metaData() {
		return metaData;
	} 
	
	public QuantityMultiplierBuilder toBuilder() {
		QuantityMultiplierBuilder builder = new QuantityMultiplierBuilder();
		ofNullable(getEquityValuation()).ifPresent(builder::setEquityValuation);
		ofNullable(getFxLinkedNotionalSchedule()).ifPresent(builder::setFxLinkedNotionalSchedule);
		ofNullable(getMultiplierValue()).ifPresent(builder::setMultiplierValue);
		return builder;
	}

	public static QuantityMultiplierBuilder builder() {
		return new QuantityMultiplierBuilder();
	}

	public static  class QuantityMultiplierBuilder extends RosettaModelObjectBuilder {
	
		protected EquityValuation.EquityValuationBuilder equityValuation;
		protected FxLinkedNotionalSchedule.FxLinkedNotionalScheduleBuilder fxLinkedNotionalSchedule;
		protected BigDecimal multiplierValue;
	
		public QuantityMultiplierBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends QuantityMultiplier> metaData() {
			return metaData;
		} 
	
		
		public EquityValuation.EquityValuationBuilder getEquityValuation() {
			return equityValuation;
		}
		
		public EquityValuation.EquityValuationBuilder getOrCreateEquityValuation() {
			if (equityValuation!=null) {
				return equityValuation;
			}
			else return equityValuation = new EquityValuation.EquityValuationBuilder();
		}
		
		
		public FxLinkedNotionalSchedule.FxLinkedNotionalScheduleBuilder getFxLinkedNotionalSchedule() {
			return fxLinkedNotionalSchedule;
		}
		
		public FxLinkedNotionalSchedule.FxLinkedNotionalScheduleBuilder getOrCreateFxLinkedNotionalSchedule() {
			if (fxLinkedNotionalSchedule!=null) {
				return fxLinkedNotionalSchedule;
			}
			else return fxLinkedNotionalSchedule = new FxLinkedNotionalSchedule.FxLinkedNotionalScheduleBuilder();
		}
		
		
		public BigDecimal getMultiplierValue() {
			return multiplierValue;
		}
		
	
		public QuantityMultiplierBuilder setEquityValuation(EquityValuation equityValuation) {
			this.equityValuation = equityValuation.toBuilder();
			return this;
		}
		public QuantityMultiplierBuilder setEquityValuationBuilder(EquityValuation.EquityValuationBuilder equityValuation) {
			this.equityValuation = equityValuation;
			return this;
		}
		public QuantityMultiplierBuilder setFxLinkedNotionalSchedule(FxLinkedNotionalSchedule fxLinkedNotionalSchedule) {
			this.fxLinkedNotionalSchedule = fxLinkedNotionalSchedule.toBuilder();
			return this;
		}
		public QuantityMultiplierBuilder setFxLinkedNotionalScheduleBuilder(FxLinkedNotionalSchedule.FxLinkedNotionalScheduleBuilder fxLinkedNotionalSchedule) {
			this.fxLinkedNotionalSchedule = fxLinkedNotionalSchedule;
			return this;
		}
		public QuantityMultiplierBuilder setMultiplierValue(BigDecimal multiplierValue) {
			this.multiplierValue = multiplierValue;
			return this;
		}
		
		public QuantityMultiplier build() {
			return new QuantityMultiplier(this);
		}
	
		@Override
		public QuantityMultiplierBuilder prune() {
			if (equityValuation!=null && !equityValuation.prune().hasData()) equityValuation = null;
			if (fxLinkedNotionalSchedule!=null && !fxLinkedNotionalSchedule.prune().hasData()) fxLinkedNotionalSchedule = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getEquityValuation()!=null && getEquityValuation().hasData()) return true;
			if (getFxLinkedNotionalSchedule()!=null && getFxLinkedNotionalSchedule().hasData()) return true;
			if (getMultiplierValue()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("multiplierValue"), BigDecimal.class, multiplierValue, this);
			
			processRosetta(path.newSubPath("equityValuation"), processor, EquityValuation.class, equityValuation);
			processRosetta(path.newSubPath("fxLinkedNotionalSchedule"), processor, FxLinkedNotionalSchedule.class, fxLinkedNotionalSchedule);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			QuantityMultiplierBuilder _that = (QuantityMultiplierBuilder) o;
		
			if (equityValuation != null ? !equityValuation.equals(_that.equityValuation) : _that.equityValuation != null) return false;
			if (fxLinkedNotionalSchedule != null ? !fxLinkedNotionalSchedule.equals(_that.fxLinkedNotionalSchedule) : _that.fxLinkedNotionalSchedule != null) return false;
			if (multiplierValue != null ? !multiplierValue.equals(_that.multiplierValue) : _that.multiplierValue != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (equityValuation != null ? equityValuation.hashCode() : 0);
			_result = 31 * _result + (fxLinkedNotionalSchedule != null ? fxLinkedNotionalSchedule.hashCode() : 0);
			_result = 31 * _result + (multiplierValue != null ? multiplierValue.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "QuantityMultiplierBuilder {" +
				"equityValuation=" + this.equityValuation + ", " +
				"fxLinkedNotionalSchedule=" + this.fxLinkedNotionalSchedule + ", " +
				"multiplierValue=" + this.multiplierValue +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("multiplierValue"), BigDecimal.class, multiplierValue, this);
		
		processRosetta(path.newSubPath("equityValuation"), processor, EquityValuation.class, equityValuation);
		processRosetta(path.newSubPath("fxLinkedNotionalSchedule"), processor, FxLinkedNotionalSchedule.class, fxLinkedNotionalSchedule);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		QuantityMultiplier _that = (QuantityMultiplier) o;
	
		if (equityValuation != null ? !equityValuation.equals(_that.equityValuation) : _that.equityValuation != null) return false;
		if (fxLinkedNotionalSchedule != null ? !fxLinkedNotionalSchedule.equals(_that.fxLinkedNotionalSchedule) : _that.fxLinkedNotionalSchedule != null) return false;
		if (multiplierValue != null ? !multiplierValue.equals(_that.multiplierValue) : _that.multiplierValue != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (equityValuation != null ? equityValuation.hashCode() : 0);
		_result = 31 * _result + (fxLinkedNotionalSchedule != null ? fxLinkedNotionalSchedule.hashCode() : 0);
		_result = 31 * _result + (multiplierValue != null ? multiplierValue.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "QuantityMultiplier {" +
			"equityValuation=" + this.equityValuation + ", " +
			"fxLinkedNotionalSchedule=" + this.fxLinkedNotionalSchedule + ", " +
			"multiplierValue=" + this.multiplierValue +
		'}';
	}
}
