package org.isda.cdm;

import org.isda.cdm.NonNegativeQuantity;
import org.isda.cdm.QuantityNotationEnum;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.QuantityNotationMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * Class to specify the quantity of a product as a single, non-negative amount, characterised with a notation enumeration to indicate the type of quantity being specified. For some product cases, several quantity notations can be associated with a single product: e.g. for an Equity Swap, both the notional and (optionally) the number of securities can be specified, with a validation to check that they are consistent in relation to the equity price.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class QuantityNotation extends RosettaModelObject {
	private final QuantityNotationEnum notationTag;
	private final NonNegativeQuantity quantity;
	private static QuantityNotationMeta metaData = new QuantityNotationMeta();
	
	QuantityNotation(QuantityNotationBuilder builder) {
		this.notationTag = builder.getNotationTag();
		this.quantity = ofNullable(builder.getQuantity()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The notation type associated with this quantity, specified from an enumeration.
	 */
	@RosettaSynonym(value="", source="FpML_5_10")
	public final QuantityNotationEnum getNotationTag() {
		return notationTag;
	}
	
	/**
	 * The quantity as a non-negative amount.
	 */
	@RosettaSynonym(value="notionalStepSchedule", source="FpML_5_10")
	@RosettaSynonym(value="notionalStepSchedule", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="notionalStepSchedule", source="DTCC_11_0")
	@RosettaSynonym(value="notionalStepSchedule", source="DTCC_9_0")
	@RosettaSynonym(value="notionalStepSchedule", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="notionalAmount,singleUnderlyer", source="FpML_5_10")
	public final NonNegativeQuantity getQuantity() {
		return quantity;
	}
	
	
	@Override
	public RosettaMetaData<? extends QuantityNotation> metaData() {
		return metaData;
	} 
	
	public QuantityNotationBuilder toBuilder() {
		QuantityNotationBuilder builder = new QuantityNotationBuilder();
		ofNullable(getNotationTag()).ifPresent(builder::setNotationTag);
		ofNullable(getQuantity()).ifPresent(builder::setQuantity);
		return builder;
	}

	public static QuantityNotationBuilder builder() {
		return new QuantityNotationBuilder();
	}

	public static  class QuantityNotationBuilder extends RosettaModelObjectBuilder {
	
		protected QuantityNotationEnum notationTag;
		protected NonNegativeQuantity.NonNegativeQuantityBuilder quantity;
	
		public QuantityNotationBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends QuantityNotation> metaData() {
			return metaData;
		} 
	
		
		public QuantityNotationEnum getNotationTag() {
			return notationTag;
		}
		
		
		public NonNegativeQuantity.NonNegativeQuantityBuilder getQuantity() {
			return quantity;
		}
		
		public NonNegativeQuantity.NonNegativeQuantityBuilder getOrCreateQuantity() {
			if (quantity!=null) {
				return quantity;
			}
			else return quantity = new NonNegativeQuantity.NonNegativeQuantityBuilder();
		}
		
	
		public QuantityNotationBuilder setNotationTag(QuantityNotationEnum notationTag) {
			this.notationTag = notationTag;
			return this;
		}
		public QuantityNotationBuilder setQuantity(NonNegativeQuantity quantity) {
			this.quantity = quantity.toBuilder();
			return this;
		}
		public QuantityNotationBuilder setQuantityBuilder(NonNegativeQuantity.NonNegativeQuantityBuilder quantity) {
			this.quantity = quantity;
			return this;
		}
		
		public QuantityNotation build() {
			return new QuantityNotation(this);
		}
	
		@Override
		public QuantityNotationBuilder prune() {
			if (quantity!=null && !quantity.prune().hasData()) quantity = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getNotationTag()!=null) return true;
			if (getQuantity()!=null && getQuantity().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("notationTag"), QuantityNotationEnum.class, notationTag, this);
			
			processRosetta(path.newSubPath("quantity"), processor, NonNegativeQuantity.class, quantity);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			QuantityNotationBuilder _that = (QuantityNotationBuilder) o;
		
			if (notationTag != null ? !notationTag.equals(_that.notationTag) : _that.notationTag != null) return false;
			if (quantity != null ? !quantity.equals(_that.quantity) : _that.quantity != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (notationTag != null ? notationTag.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (quantity != null ? quantity.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "QuantityNotationBuilder {" +
				"notationTag=" + this.notationTag + ", " +
				"quantity=" + this.quantity +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("notationTag"), QuantityNotationEnum.class, notationTag, this);
		
		processRosetta(path.newSubPath("quantity"), processor, NonNegativeQuantity.class, quantity);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		QuantityNotation _that = (QuantityNotation) o;
	
		if (notationTag != null ? !notationTag.equals(_that.notationTag) : _that.notationTag != null) return false;
		if (quantity != null ? !quantity.equals(_that.quantity) : _that.quantity != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (notationTag != null ? notationTag.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (quantity != null ? quantity.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "QuantityNotation {" +
			"notationTag=" + this.notationTag + ", " +
			"quantity=" + this.quantity +
		'}';
	}
}
