package org.isda.cdm;

import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.QuoteBasisEnum;
import org.isda.cdm.meta.QuotedCurrencyPairMeta;
import org.isda.cdm.metafields.FieldWithMetaString;
import org.isda.cdm.metafields.FieldWithMetaString.FieldWithMetaStringBuilder;

import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.BuilderProcessor;
import com.rosetta.model.lib.process.Processor;

import static java.util.Optional.ofNullable;

/**
 * A class that describes the composition of a rate that has been quoted or is to be quoted. This includes the two currencies and the quotation relationship between the two currencies and is used as a building block throughout the FX specification.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="QuotedCurrencyPair", source="FpML_5_10")
public class QuotedCurrencyPair extends RosettaModelObject {
	private final FieldWithMetaString currency1;
	private final FieldWithMetaString currency2;
	private final QuoteBasisEnum quoteBasis;
	private static QuotedCurrencyPairMeta metaData = new QuotedCurrencyPairMeta();
	
	QuotedCurrencyPair(QuotedCurrencyPairBuilder builder) {
		this.currency1 = ofNullable(builder.getCurrency1()).map(f->f.build()).orElse(null);
		this.currency2 = ofNullable(builder.getCurrency2()).map(f->f.build()).orElse(null);
		this.quoteBasis = builder.getQuoteBasis();
	}
	
	/**
	 * The first currency specified when a pair of currencies is to be evaluated.
	 */
	@RosettaSynonym(value="currency1", source="FpML_5_10")
	@RosettaSynonym(value="currency1", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="currency1", source="DTCC_11_0")
	@RosettaSynonym(value="currency1", source="DTCC_9_0")
	@RosettaSynonym(value="currency1", source="CME_ClearedConfirm_1_17")
	public final FieldWithMetaString getCurrency1() {
		return currency1;
	}
	
	/**
	 * The second currency specified when a pair of currencies is to be evaluated.
	 */
	@RosettaSynonym(value="currency2", source="FpML_5_10")
	@RosettaSynonym(value="currency2", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="currency2", source="DTCC_11_0")
	@RosettaSynonym(value="currency2", source="DTCC_9_0")
	@RosettaSynonym(value="currency2", source="CME_ClearedConfirm_1_17")
	public final FieldWithMetaString getCurrency2() {
		return currency2;
	}
	
	/**
	 * The method by which the exchange rate is quoted.
	 */
	@RosettaSynonym(value="quoteBasis", source="FpML_5_10")
	@RosettaSynonym(value="quoteBasis", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="quoteBasis", source="DTCC_11_0")
	@RosettaSynonym(value="quoteBasis", source="DTCC_9_0")
	@RosettaSynonym(value="quoteBasis", source="CME_ClearedConfirm_1_17")
	public final QuoteBasisEnum getQuoteBasis() {
		return quoteBasis;
	}
	
	
	@Override
	public RosettaMetaData<? extends QuotedCurrencyPair> metaData() {
		return metaData;
	} 
	
	public QuotedCurrencyPairBuilder toBuilder() {
		QuotedCurrencyPairBuilder builder = new QuotedCurrencyPairBuilder();
		ofNullable(getCurrency1()).ifPresent(builder::setCurrency1);
		ofNullable(getCurrency2()).ifPresent(builder::setCurrency2);
		ofNullable(getQuoteBasis()).ifPresent(builder::setQuoteBasis);
		return builder;
	}

	public static QuotedCurrencyPairBuilder builder() {
		return new QuotedCurrencyPairBuilder();
	}

	public static class QuotedCurrencyPairBuilder extends RosettaModelObjectBuilder{
	
		protected FieldWithMetaStringBuilder currency1;
		protected FieldWithMetaStringBuilder currency2;
		protected QuoteBasisEnum quoteBasis;
	
		public QuotedCurrencyPairBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends QuotedCurrencyPair> metaData() {
			return metaData;
		} 
	
		
		public FieldWithMetaStringBuilder getCurrency1() {
			return currency1;
		}
		
		public FieldWithMetaStringBuilder getOrCreateCurrency1() {
			if (currency1!=null) {
				return currency1;
			}
			else return currency1 = new FieldWithMetaStringBuilder();
		}
		
		
		public FieldWithMetaStringBuilder getCurrency2() {
			return currency2;
		}
		
		public FieldWithMetaStringBuilder getOrCreateCurrency2() {
			if (currency2!=null) {
				return currency2;
			}
			else return currency2 = new FieldWithMetaStringBuilder();
		}
		
		
		public QuoteBasisEnum getQuoteBasis() {
			return quoteBasis;
		}
		
	
		public QuotedCurrencyPairBuilder setCurrency1(FieldWithMetaString currency1) {
			this.currency1 = currency1.toBuilder();
			return this;
		}
		public QuotedCurrencyPairBuilder setCurrency1Ref(String currency1) {
			return setCurrency1(FieldWithMetaString.builder().setValue(currency1).build());
		}
		public QuotedCurrencyPairBuilder setCurrency2(FieldWithMetaString currency2) {
			this.currency2 = currency2.toBuilder();
			return this;
		}
		public QuotedCurrencyPairBuilder setCurrency2Ref(String currency2) {
			return setCurrency2(FieldWithMetaString.builder().setValue(currency2).build());
		}
		public QuotedCurrencyPairBuilder setQuoteBasis(QuoteBasisEnum quoteBasis) {
			this.quoteBasis = quoteBasis;
			return this;
		}
	
		public QuotedCurrencyPair build() {
			return new QuotedCurrencyPair(this);
		}
	
		@Override
		public QuotedCurrencyPairBuilder prune() {
			if (currency1!=null && !currency1.prune().hasData()) currency1 = null;
			if (currency2!=null && !currency2.prune().hasData()) currency2 = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getCurrency1()!=null) return true;
			if (getCurrency2()!=null) return true;
			if (getQuoteBasis()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("quoteBasis"), QuoteBasisEnum.class, quoteBasis, this);
			
			processRosetta(path.newSubPath("currency1"), processor, FieldWithMetaString.class, currency1);
			processRosetta(path.newSubPath("currency2"), processor, FieldWithMetaString.class, currency2);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			QuotedCurrencyPairBuilder _that = (QuotedCurrencyPairBuilder) o;
		
			if (currency1 != null ? !currency1.equals(_that.currency1) : _that.currency1 != null) return false;
			if (currency2 != null ? !currency2.equals(_that.currency2) : _that.currency2 != null) return false;
			if (quoteBasis != null ? !quoteBasis.equals(_that.quoteBasis) : _that.quoteBasis != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (currency1 != null ? currency1.hashCode() : 0);
			_result = 31 * _result + (currency2 != null ? currency2.hashCode() : 0);
			_result = 31 * _result + (quoteBasis != null ? quoteBasis.getClass().getName().hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "QuotedCurrencyPairBuilder {" +
				"currency1=" + this.currency1 + ", " +
				"currency2=" + this.currency2 + ", " +
				"quoteBasis=" + this.quoteBasis +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("quoteBasis"), QuoteBasisEnum.class, quoteBasis, this);
		
		processRosetta(path.newSubPath("currency1"), processor, FieldWithMetaString.class, currency1);
		processRosetta(path.newSubPath("currency2"), processor, FieldWithMetaString.class, currency2);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		QuotedCurrencyPair _that = (QuotedCurrencyPair) o;
	
		if (currency1 != null ? !currency1.equals(_that.currency1) : _that.currency1 != null) return false;
		if (currency2 != null ? !currency2.equals(_that.currency2) : _that.currency2 != null) return false;
		if (quoteBasis != null ? !quoteBasis.equals(_that.quoteBasis) : _that.quoteBasis != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (currency1 != null ? currency1.hashCode() : 0);
		_result = 31 * _result + (currency2 != null ? currency2.hashCode() : 0);
		_result = 31 * _result + (quoteBasis != null ? quoteBasis.getClass().getName().hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "QuotedCurrencyPair {" +
			"currency1=" + this.currency1 + ", " +
			"currency2=" + this.currency2 + ", " +
			"quoteBasis=" + this.quoteBasis +
		'}';
	}
}
