package org.isda.cdm;

import org.isda.cdm.FloatingRateSpecification;
import org.isda.cdm.InflationRateSpecification;
import org.isda.cdm.Schedule;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.RateSpecificationMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class to specify the fixed interest rate, floating interest rate or inflation rate.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class RateSpecification extends RosettaModelObject {
	private final Schedule fixedRate;
	private final FloatingRateSpecification floatingRate;
	private final InflationRateSpecification inflationRate;
	private static RateSpecificationMeta metaData = new RateSpecificationMeta();
	
	RateSpecification(RateSpecificationBuilder builder) {
		this.fixedRate = ofNullable(builder.getFixedRate()).map(f->f.build()).orElse(null);
		this.floatingRate = ofNullable(builder.getFloatingRate()).map(f->f.build()).orElse(null);
		this.inflationRate = ofNullable(builder.getInflationRate()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The fixed rate or fixed rate schedule expressed as explicit fixed rates and dates. In the case of a schedule, the step dates may be subject to adjustment in accordance with any adjustments specified in calculationPeriodDatesAdjustments.
	 */
	public final Schedule getFixedRate() {
		return fixedRate;
	}
	
	/**
	 * The floating interest rate specification, which includes the definition of the floating rate index. the tenor, the initial value, and, when applicable, the spread, the rounding convention, the averaging method and the negative interest rate treatment.
	 */
	public final FloatingRateSpecification getFloatingRate() {
		return floatingRate;
	}
	
	/**
	 * An inflation rate calculation definition.
	 */
	@RosettaSynonym(value="inflationRateCalculation", source="FpML_5_10", path="calculationPeriodAmount.calculation")
	@RosettaSynonym(value="inflationRateCalculation", source="CME_SubmissionIRS_1_0", path="calculationPeriodAmount.calculation")
	@RosettaSynonym(value="inflationRateCalculation", source="DTCC_11_0", path="calculationPeriodAmount.calculation")
	@RosettaSynonym(value="inflationRateCalculation", source="DTCC_9_0", path="calculationPeriodAmount.calculation")
	@RosettaSynonym(value="inflationRateCalculation", source="CME_ClearedConfirm_1_17", path="calculationPeriodAmount.calculation")
	public final InflationRateSpecification getInflationRate() {
		return inflationRate;
	}
	
	
	@Override
	public RosettaMetaData<? extends RateSpecification> metaData() {
		return metaData;
	} 
	
	public RateSpecificationBuilder toBuilder() {
		RateSpecificationBuilder builder = new RateSpecificationBuilder();
		ofNullable(getFixedRate()).ifPresent(builder::setFixedRate);
		ofNullable(getFloatingRate()).ifPresent(builder::setFloatingRate);
		ofNullable(getInflationRate()).ifPresent(builder::setInflationRate);
		return builder;
	}

	public static RateSpecificationBuilder builder() {
		return new RateSpecificationBuilder();
	}

	public static  class RateSpecificationBuilder extends RosettaModelObjectBuilder {
	
		protected Schedule.ScheduleBuilder fixedRate;
		protected FloatingRateSpecification.FloatingRateSpecificationBuilder floatingRate;
		protected InflationRateSpecification.InflationRateSpecificationBuilder inflationRate;
	
		public RateSpecificationBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends RateSpecification> metaData() {
			return metaData;
		} 
	
		
		public Schedule.ScheduleBuilder getFixedRate() {
			return fixedRate;
		}
		
		public Schedule.ScheduleBuilder getOrCreateFixedRate() {
			if (fixedRate!=null) {
				return fixedRate;
			}
			else return fixedRate = new Schedule.ScheduleBuilder();
		}
		
		
		public FloatingRateSpecification.FloatingRateSpecificationBuilder getFloatingRate() {
			return floatingRate;
		}
		
		public FloatingRateSpecification.FloatingRateSpecificationBuilder getOrCreateFloatingRate() {
			if (floatingRate!=null) {
				return floatingRate;
			}
			else return floatingRate = new FloatingRateSpecification.FloatingRateSpecificationBuilder();
		}
		
		
		public InflationRateSpecification.InflationRateSpecificationBuilder getInflationRate() {
			return inflationRate;
		}
		
		public InflationRateSpecification.InflationRateSpecificationBuilder getOrCreateInflationRate() {
			if (inflationRate!=null) {
				return inflationRate;
			}
			else return inflationRate = new InflationRateSpecification.InflationRateSpecificationBuilder();
		}
		
	
		public RateSpecificationBuilder setFixedRate(Schedule fixedRate) {
			this.fixedRate = fixedRate.toBuilder();
			return this;
		}
		public RateSpecificationBuilder setFixedRateBuilder(Schedule.ScheduleBuilder fixedRate) {
			this.fixedRate = fixedRate;
			return this;
		}
		public RateSpecificationBuilder setFloatingRate(FloatingRateSpecification floatingRate) {
			this.floatingRate = floatingRate.toBuilder();
			return this;
		}
		public RateSpecificationBuilder setFloatingRateBuilder(FloatingRateSpecification.FloatingRateSpecificationBuilder floatingRate) {
			this.floatingRate = floatingRate;
			return this;
		}
		public RateSpecificationBuilder setInflationRate(InflationRateSpecification inflationRate) {
			this.inflationRate = inflationRate.toBuilder();
			return this;
		}
		public RateSpecificationBuilder setInflationRateBuilder(InflationRateSpecification.InflationRateSpecificationBuilder inflationRate) {
			this.inflationRate = inflationRate;
			return this;
		}
		
		public RateSpecification build() {
			return new RateSpecification(this);
		}
	
		@Override
		public RateSpecificationBuilder prune() {
			if (fixedRate!=null && !fixedRate.prune().hasData()) fixedRate = null;
			if (floatingRate!=null && !floatingRate.prune().hasData()) floatingRate = null;
			if (inflationRate!=null && !inflationRate.prune().hasData()) inflationRate = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getFixedRate()!=null && getFixedRate().hasData()) return true;
			if (getFloatingRate()!=null && getFloatingRate().hasData()) return true;
			if (getInflationRate()!=null && getInflationRate().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("fixedRate"), processor, Schedule.class, fixedRate);
			processRosetta(path.newSubPath("floatingRate"), processor, FloatingRateSpecification.class, floatingRate);
			processRosetta(path.newSubPath("inflationRate"), processor, InflationRateSpecification.class, inflationRate);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			RateSpecificationBuilder _that = (RateSpecificationBuilder) o;
		
			if (fixedRate != null ? !fixedRate.equals(_that.fixedRate) : _that.fixedRate != null) return false;
			if (floatingRate != null ? !floatingRate.equals(_that.floatingRate) : _that.floatingRate != null) return false;
			if (inflationRate != null ? !inflationRate.equals(_that.inflationRate) : _that.inflationRate != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (fixedRate != null ? fixedRate.hashCode() : 0);
			_result = 31 * _result + (floatingRate != null ? floatingRate.hashCode() : 0);
			_result = 31 * _result + (inflationRate != null ? inflationRate.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "RateSpecificationBuilder {" +
				"fixedRate=" + this.fixedRate + ", " +
				"floatingRate=" + this.floatingRate + ", " +
				"inflationRate=" + this.inflationRate +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("fixedRate"), processor, Schedule.class, fixedRate);
		processRosetta(path.newSubPath("floatingRate"), processor, FloatingRateSpecification.class, floatingRate);
		processRosetta(path.newSubPath("inflationRate"), processor, InflationRateSpecification.class, inflationRate);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		RateSpecification _that = (RateSpecification) o;
	
		if (fixedRate != null ? !fixedRate.equals(_that.fixedRate) : _that.fixedRate != null) return false;
		if (floatingRate != null ? !floatingRate.equals(_that.floatingRate) : _that.floatingRate != null) return false;
		if (inflationRate != null ? !inflationRate.equals(_that.inflationRate) : _that.inflationRate != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (fixedRate != null ? fixedRate.hashCode() : 0);
		_result = 31 * _result + (floatingRate != null ? floatingRate.hashCode() : 0);
		_result = 31 * _result + (inflationRate != null ? inflationRate.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "RateSpecification {" +
			"fixedRate=" + this.fixedRate + ", " +
			"floatingRate=" + this.floatingRate + ", " +
			"inflationRate=" + this.inflationRate +
		'}';
	}
}
