package org.isda.cdm;

import java.lang.Boolean;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.LegalEntity;
import org.isda.cdm.ReferenceObligation;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.ReferenceInformationMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;
		import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import java.math.BigDecimal;

/**
 * A class specifying the Credit Default Swap Reference Information.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="ReferenceInformation", source="FpML_5_10")
public class ReferenceInformation extends RosettaModelObject {
	private final Boolean allGuarantees;
	private final Boolean noReferenceObligation;
	private final LegalEntity referenceEntity;
	private final List<ReferenceObligation> referenceObligation;
	private final Boolean referencePolicy;
	private final BigDecimal referencePrice;
	private final Boolean securedList;
	private final Boolean unknownReferenceObligation;
	private static ReferenceInformationMeta metaData = new ReferenceInformationMeta();
	
	ReferenceInformation(ReferenceInformationBuilder builder) {
		this.allGuarantees = builder.getAllGuarantees();
		this.noReferenceObligation = builder.getNoReferenceObligation();
		this.referenceEntity = ofNullable(builder.getReferenceEntity()).map(f->f.build()).orElse(null);
		this.referenceObligation = ofNullable(builder.getReferenceObligation()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.referencePolicy = builder.getReferencePolicy();
		this.referencePrice = builder.getReferencePrice();
		this.securedList = builder.getSecuredList();
		this.unknownReferenceObligation = builder.getUnknownReferenceObligation();
	}
	
	/**
	 * Indicates whether an obligation of the Reference Entity, guaranteed by the Reference Entity on behalf of a non-Affiliate, is to be considered an Obligation for the purpose of the transaction. It will be considered an obligation if allGuarantees is applicable (true) and not if allGuarantees is inapplicable (false). ISDA 2003 Term: All Guarantees.
	 */
	@RosettaSynonym(value="allGuarantees", source="FpML_5_10")
	@RosettaSynonym(value="allGuarantees", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="allGuarantees", source="DTCC_11_0")
	@RosettaSynonym(value="allGuarantees", source="DTCC_9_0")
	@RosettaSynonym(value="allGuarantees", source="CME_ClearedConfirm_1_17")
	public final Boolean getAllGuarantees() {
		return allGuarantees;
	}
	
	/**
	 * Used to indicate that there is no Reference Obligation associated with this Credit Default Swap and that there will never be one.
	 */
	@RosettaSynonym(value="noReferenceObligation", source="FpML_5_10")
	@RosettaSynonym(value="noReferenceObligation", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="noReferenceObligation", source="DTCC_11_0")
	@RosettaSynonym(value="noReferenceObligation", source="DTCC_9_0")
	@RosettaSynonym(value="noReferenceObligation", source="CME_ClearedConfirm_1_17")
	public final Boolean getNoReferenceObligation() {
		return noReferenceObligation;
	}
	
	/**
	 * The corporate or sovereign entity which is subject to the swap transaction and any successor that assumes all or substantially all of its contractual and other obligations. Reference Entities cannot be senior or subordinated. It is the obligations of the Reference Entities that can be senior or subordinated. ISDA 2014 Credit definitions article II section 2.1: `Reference Entity` means the entity specified as such in the related Confirmation.
	 */
	public final LegalEntity getReferenceEntity() {
		return referenceEntity;
	}
	
	/**
	 * The Reference Obligation is a financial instrument that is either issued or guaranteed by the reference entity. It serves to clarify the precise reference entity protection is being offered upon, and its legal position with regard to other related firms (parents/subsidiaries). Furthermore the Reference Obligation is ALWAYS deliverable and establishes the Pari Passu ranking (as the deliverable bonds must rank equal to the reference obligation). ISDA 2003 Term: Reference Obligation.
	 */
	@RosettaSynonym(value="referenceObligation", source="FpML_5_10")
	@RosettaSynonym(value="referenceObligation", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="referenceObligation", source="DTCC_11_0")
	@RosettaSynonym(value="referenceObligation", source="DTCC_9_0")
	@RosettaSynonym(value="referenceObligation", source="CME_ClearedConfirm_1_17")
	public final List<ReferenceObligation> getReferenceObligation() {
		return referenceObligation;
	}
	
	/**
	 * Applicable to the transactions on mortgage-backed security, which can make use of a reference policy. Presence of the element with value set to &#39;true&#39; indicates that the reference policy is applicable; absence implies that it is not.
	 */
	@RosettaSynonym(value="referencePolicy", source="FpML_5_10")
	@RosettaSynonym(value="referencePolicy", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="referencePolicy", source="DTCC_11_0")
	@RosettaSynonym(value="referencePolicy", source="DTCC_9_0")
	@RosettaSynonym(value="referencePolicy", source="CME_ClearedConfirm_1_17")
	public final Boolean getReferencePolicy() {
		return referencePolicy;
	}
	
	/**
	 * Used to determine (a) for physically settled trades, the Physical Settlement Amount, which equals the Floating Rate Payer Calculation Amount times the Reference Price and (b) for cash settled trades, the Cash Settlement Amount, which equals the greater of (i) the difference between the Reference Price and the Final Price and (ii) zero. ISDA 2003 Term: Reference Price.
	 */
	@RosettaSynonym(value="referencePrice", source="FpML_5_10")
	@RosettaSynonym(value="referencePrice", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="referencePrice", source="DTCC_11_0")
	@RosettaSynonym(value="referencePrice", source="DTCC_9_0")
	@RosettaSynonym(value="referencePrice", source="CME_ClearedConfirm_1_17")
	public final BigDecimal getReferencePrice() {
		return referencePrice;
	}
	
	/**
	 * With respect to any day, the list of Syndicated Secured Obligations of the Designated Priority of the Reference Entity published by Markit Group Limited or any successor thereto appointed by the Specified Dealers (the &#39;Secured List Publisher&#39;) on or most recently before such day, which list is currently available at [http://www.markit.com]. ISDA 2003 Term: Relevant Secured List.
	 */
	@RosettaSynonym(value="securedList", source="FpML_5_10")
	@RosettaSynonym(value="securedList", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="securedList", source="DTCC_11_0")
	@RosettaSynonym(value="securedList", source="DTCC_9_0")
	@RosettaSynonym(value="securedList", source="CME_ClearedConfirm_1_17")
	public final Boolean getSecuredList() {
		return securedList;
	}
	
	/**
	 * Used to indicate that the Reference obligation associated with the Credit Default Swap is currently not known. This is not valid for Legal Confirmation purposes, but is valid for earlier stages in the trade life cycle (e.g. Broker Confirmation).
	 */
	@RosettaSynonym(value="unknownReferenceObligation", source="FpML_5_10")
	@RosettaSynonym(value="unknownReferenceObligation", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="unknownReferenceObligation", source="DTCC_11_0")
	@RosettaSynonym(value="unknownReferenceObligation", source="DTCC_9_0")
	@RosettaSynonym(value="unknownReferenceObligation", source="CME_ClearedConfirm_1_17")
	public final Boolean getUnknownReferenceObligation() {
		return unknownReferenceObligation;
	}
	
	
	@Override
	public RosettaMetaData<? extends ReferenceInformation> metaData() {
		return metaData;
	} 
	
	public ReferenceInformationBuilder toBuilder() {
		ReferenceInformationBuilder builder = new ReferenceInformationBuilder();
		ofNullable(getAllGuarantees()).ifPresent(builder::setAllGuarantees);
		ofNullable(getNoReferenceObligation()).ifPresent(builder::setNoReferenceObligation);
		ofNullable(getReferenceEntity()).ifPresent(builder::setReferenceEntity);
		ofNullable(getReferenceObligation()).ifPresent(referenceObligation -> referenceObligation.forEach(builder::addReferenceObligation));
		ofNullable(getReferencePolicy()).ifPresent(builder::setReferencePolicy);
		ofNullable(getReferencePrice()).ifPresent(builder::setReferencePrice);
		ofNullable(getSecuredList()).ifPresent(builder::setSecuredList);
		ofNullable(getUnknownReferenceObligation()).ifPresent(builder::setUnknownReferenceObligation);
		return builder;
	}

	public static ReferenceInformationBuilder builder() {
		return new ReferenceInformationBuilder();
	}

	public static  class ReferenceInformationBuilder extends RosettaModelObjectBuilder {
	
		protected Boolean allGuarantees;
		protected Boolean noReferenceObligation;
		protected LegalEntity.LegalEntityBuilder referenceEntity;
		protected List<ReferenceObligation.ReferenceObligationBuilder> referenceObligation;
		protected Boolean referencePolicy;
		protected BigDecimal referencePrice;
		protected Boolean securedList;
		protected Boolean unknownReferenceObligation;
	
		public ReferenceInformationBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends ReferenceInformation> metaData() {
			return metaData;
		} 
	
		
		public Boolean getAllGuarantees() {
			return allGuarantees;
		}
		
		
		public Boolean getNoReferenceObligation() {
			return noReferenceObligation;
		}
		
		
		public LegalEntity.LegalEntityBuilder getReferenceEntity() {
			return referenceEntity;
		}
		
		public LegalEntity.LegalEntityBuilder getOrCreateReferenceEntity() {
			if (referenceEntity!=null) {
				return referenceEntity;
			}
			else return referenceEntity = new LegalEntity.LegalEntityBuilder();
		}
		
		
		public List<ReferenceObligation.ReferenceObligationBuilder> getReferenceObligation() {
			return referenceObligation;
		}
		
		public ReferenceObligation.ReferenceObligationBuilder getOrCreateReferenceObligation(int index) {
			if (referenceObligation==null) {
				this.referenceObligation = new ArrayList<>();
			}
			return getIndex(referenceObligation, index, ()->new ReferenceObligation.ReferenceObligationBuilder());
		}
		
		
		public Boolean getReferencePolicy() {
			return referencePolicy;
		}
		
		
		public BigDecimal getReferencePrice() {
			return referencePrice;
		}
		
		
		public Boolean getSecuredList() {
			return securedList;
		}
		
		
		public Boolean getUnknownReferenceObligation() {
			return unknownReferenceObligation;
		}
		
	
		public ReferenceInformationBuilder setAllGuarantees(Boolean allGuarantees) {
			this.allGuarantees = allGuarantees;
			return this;
		}
		public ReferenceInformationBuilder setNoReferenceObligation(Boolean noReferenceObligation) {
			this.noReferenceObligation = noReferenceObligation;
			return this;
		}
		public ReferenceInformationBuilder setReferenceEntity(LegalEntity referenceEntity) {
			this.referenceEntity = referenceEntity.toBuilder();
			return this;
		}
		public ReferenceInformationBuilder setReferenceEntityBuilder(LegalEntity.LegalEntityBuilder referenceEntity) {
			this.referenceEntity = referenceEntity;
			return this;
		}
		public ReferenceInformationBuilder addReferenceObligation(ReferenceObligation referenceObligation) {
			if(this.referenceObligation == null){
				this.referenceObligation = new ArrayList<>();
			}
			this.referenceObligation.add(referenceObligation.toBuilder());
			return this;
		}
		public ReferenceInformationBuilder addReferenceObligation(ReferenceObligation referenceObligation, int _idx) {
			if(this.referenceObligation == null){
				this.referenceObligation = new ArrayList<>();
			}
			getIndex(this.referenceObligation, _idx, () -> referenceObligation.toBuilder());
			this.referenceObligation.set(_idx, referenceObligation.toBuilder());
			return this;
		}
		public ReferenceInformationBuilder addReferenceObligation(List<ReferenceObligation> referenceObligations) {
			if(this.referenceObligation == null){
				this.referenceObligation = new ArrayList<>();
			}
			for (ReferenceObligation toAdd : referenceObligations) {
				this.referenceObligation.add(toAdd.toBuilder());
			}
			return this;
		}
		public ReferenceInformationBuilder addReferenceObligationBuilder(ReferenceObligation.ReferenceObligationBuilder referenceObligation) {
			if(this.referenceObligation == null){
				this.referenceObligation = new ArrayList<>();
			}
			this.referenceObligation.add(referenceObligation);
			return this;
		}
		
		public ReferenceInformationBuilder clearReferenceObligation() {
			this.referenceObligation = null;
			return this;
		}
		public ReferenceInformationBuilder setReferencePolicy(Boolean referencePolicy) {
			this.referencePolicy = referencePolicy;
			return this;
		}
		public ReferenceInformationBuilder setReferencePrice(BigDecimal referencePrice) {
			this.referencePrice = referencePrice;
			return this;
		}
		public ReferenceInformationBuilder setSecuredList(Boolean securedList) {
			this.securedList = securedList;
			return this;
		}
		public ReferenceInformationBuilder setUnknownReferenceObligation(Boolean unknownReferenceObligation) {
			this.unknownReferenceObligation = unknownReferenceObligation;
			return this;
		}
		
		public ReferenceInformation build() {
			return new ReferenceInformation(this);
		}
	
		@Override
		public ReferenceInformationBuilder prune() {
			if (referenceEntity!=null && !referenceEntity.prune().hasData()) referenceEntity = null;
			if (referenceObligation!=null) referenceObligation = referenceObligation.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getAllGuarantees()!=null) return true;
			if (getNoReferenceObligation()!=null) return true;
			if (getReferenceEntity()!=null && getReferenceEntity().hasData()) return true;
			if (getReferenceObligation()!=null && getReferenceObligation().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getReferencePolicy()!=null) return true;
			if (getReferencePrice()!=null) return true;
			if (getSecuredList()!=null) return true;
			if (getUnknownReferenceObligation()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("allGuarantees"), Boolean.class, allGuarantees, this);
			processor.processBasic(path.newSubPath("noReferenceObligation"), Boolean.class, noReferenceObligation, this);
			processor.processBasic(path.newSubPath("referencePolicy"), Boolean.class, referencePolicy, this);
			processor.processBasic(path.newSubPath("referencePrice"), BigDecimal.class, referencePrice, this);
			processor.processBasic(path.newSubPath("securedList"), Boolean.class, securedList, this);
			processor.processBasic(path.newSubPath("unknownReferenceObligation"), Boolean.class, unknownReferenceObligation, this);
			
			processRosetta(path.newSubPath("referenceEntity"), processor, LegalEntity.class, referenceEntity);
			processRosetta(path.newSubPath("referenceObligation"), processor, ReferenceObligation.class, referenceObligation);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			ReferenceInformationBuilder _that = (ReferenceInformationBuilder) o;
		
			if (allGuarantees != null ? !allGuarantees.equals(_that.allGuarantees) : _that.allGuarantees != null) return false;
			if (noReferenceObligation != null ? !noReferenceObligation.equals(_that.noReferenceObligation) : _that.noReferenceObligation != null) return false;
			if (referenceEntity != null ? !referenceEntity.equals(_that.referenceEntity) : _that.referenceEntity != null) return false;
			if (!ListEquals.listEquals(referenceObligation, _that.referenceObligation)) return false;
			if (referencePolicy != null ? !referencePolicy.equals(_that.referencePolicy) : _that.referencePolicy != null) return false;
			if (referencePrice != null ? !referencePrice.equals(_that.referencePrice) : _that.referencePrice != null) return false;
			if (securedList != null ? !securedList.equals(_that.securedList) : _that.securedList != null) return false;
			if (unknownReferenceObligation != null ? !unknownReferenceObligation.equals(_that.unknownReferenceObligation) : _that.unknownReferenceObligation != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (allGuarantees != null ? allGuarantees.hashCode() : 0);
			_result = 31 * _result + (noReferenceObligation != null ? noReferenceObligation.hashCode() : 0);
			_result = 31 * _result + (referenceEntity != null ? referenceEntity.hashCode() : 0);
			_result = 31 * _result + (referenceObligation != null ? referenceObligation.hashCode() : 0);
			_result = 31 * _result + (referencePolicy != null ? referencePolicy.hashCode() : 0);
			_result = 31 * _result + (referencePrice != null ? referencePrice.hashCode() : 0);
			_result = 31 * _result + (securedList != null ? securedList.hashCode() : 0);
			_result = 31 * _result + (unknownReferenceObligation != null ? unknownReferenceObligation.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "ReferenceInformationBuilder {" +
				"allGuarantees=" + this.allGuarantees + ", " +
				"noReferenceObligation=" + this.noReferenceObligation + ", " +
				"referenceEntity=" + this.referenceEntity + ", " +
				"referenceObligation=" + this.referenceObligation + ", " +
				"referencePolicy=" + this.referencePolicy + ", " +
				"referencePrice=" + this.referencePrice + ", " +
				"securedList=" + this.securedList + ", " +
				"unknownReferenceObligation=" + this.unknownReferenceObligation +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("allGuarantees"), Boolean.class, allGuarantees, this);
		processor.processBasic(path.newSubPath("noReferenceObligation"), Boolean.class, noReferenceObligation, this);
		processor.processBasic(path.newSubPath("referencePolicy"), Boolean.class, referencePolicy, this);
		processor.processBasic(path.newSubPath("referencePrice"), BigDecimal.class, referencePrice, this);
		processor.processBasic(path.newSubPath("securedList"), Boolean.class, securedList, this);
		processor.processBasic(path.newSubPath("unknownReferenceObligation"), Boolean.class, unknownReferenceObligation, this);
		
		processRosetta(path.newSubPath("referenceEntity"), processor, LegalEntity.class, referenceEntity);
		processRosetta(path.newSubPath("referenceObligation"), processor, ReferenceObligation.class, referenceObligation);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		ReferenceInformation _that = (ReferenceInformation) o;
	
		if (allGuarantees != null ? !allGuarantees.equals(_that.allGuarantees) : _that.allGuarantees != null) return false;
		if (noReferenceObligation != null ? !noReferenceObligation.equals(_that.noReferenceObligation) : _that.noReferenceObligation != null) return false;
		if (referenceEntity != null ? !referenceEntity.equals(_that.referenceEntity) : _that.referenceEntity != null) return false;
		if (!ListEquals.listEquals(referenceObligation, _that.referenceObligation)) return false;
		if (referencePolicy != null ? !referencePolicy.equals(_that.referencePolicy) : _that.referencePolicy != null) return false;
		if (referencePrice != null ? !referencePrice.equals(_that.referencePrice) : _that.referencePrice != null) return false;
		if (securedList != null ? !securedList.equals(_that.securedList) : _that.securedList != null) return false;
		if (unknownReferenceObligation != null ? !unknownReferenceObligation.equals(_that.unknownReferenceObligation) : _that.unknownReferenceObligation != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (allGuarantees != null ? allGuarantees.hashCode() : 0);
		_result = 31 * _result + (noReferenceObligation != null ? noReferenceObligation.hashCode() : 0);
		_result = 31 * _result + (referenceEntity != null ? referenceEntity.hashCode() : 0);
		_result = 31 * _result + (referenceObligation != null ? referenceObligation.hashCode() : 0);
		_result = 31 * _result + (referencePolicy != null ? referencePolicy.hashCode() : 0);
		_result = 31 * _result + (referencePrice != null ? referencePrice.hashCode() : 0);
		_result = 31 * _result + (securedList != null ? securedList.hashCode() : 0);
		_result = 31 * _result + (unknownReferenceObligation != null ? unknownReferenceObligation.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "ReferenceInformation {" +
			"allGuarantees=" + this.allGuarantees + ", " +
			"noReferenceObligation=" + this.noReferenceObligation + ", " +
			"referenceEntity=" + this.referenceEntity + ", " +
			"referenceObligation=" + this.referenceObligation + ", " +
			"referencePolicy=" + this.referencePolicy + ", " +
			"referencePrice=" + this.referencePrice + ", " +
			"securedList=" + this.securedList + ", " +
			"unknownReferenceObligation=" + this.unknownReferenceObligation +
		'}';
	}
}
