package org.isda.cdm;

import java.lang.Boolean;
import java.lang.String;
import org.isda.cdm.Bond;
import org.isda.cdm.ConvertibleBond;
import org.isda.cdm.LegalEntity;
import org.isda.cdm.Loan;
import org.isda.cdm.MortgageBackedSecurity;
import org.isda.cdm.metafields.ReferenceWithMetaLegalEntity;
import org.isda.cdm.metafields.ReferenceWithMetaLegalEntity.ReferenceWithMetaLegalEntityBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.ReferenceObligationMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;
		import java.util.stream.Collectors;

/**
 * A class to specify the reference obligation that is associated with a credit derivative instrument.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="ReferenceObligation", source="FpML_5_10")
public class ReferenceObligation extends RosettaModelObject {
	private final Bond bond;
	private final ConvertibleBond convertibleBond;
	private final LegalEntity guarantor;
	private final String guarantorReference;
	private final Loan loan;
	private final MortgageBackedSecurity mortgageBackedSecurity;
	private final LegalEntity primaryObligor;
	private final ReferenceWithMetaLegalEntity primaryObligorReference;
	private final Boolean standardReferenceObligation;
	private static ReferenceObligationMeta metaData = new ReferenceObligationMeta();
	
	ReferenceObligation(ReferenceObligationBuilder builder) {
		this.bond = ofNullable(builder.getBond()).map(f->f.build()).orElse(null);
		this.convertibleBond = ofNullable(builder.getConvertibleBond()).map(f->f.build()).orElse(null);
		this.guarantor = ofNullable(builder.getGuarantor()).map(f->f.build()).orElse(null);
		this.guarantorReference = builder.getGuarantorReference();
		this.loan = ofNullable(builder.getLoan()).map(f->f.build()).orElse(null);
		this.mortgageBackedSecurity = ofNullable(builder.getMortgageBackedSecurity()).map(f->f.build()).orElse(null);
		this.primaryObligor = ofNullable(builder.getPrimaryObligor()).map(f->f.build()).orElse(null);
		this.primaryObligorReference = ofNullable(builder.getPrimaryObligorReference()).map(f->f.build()).orElse(null);
		this.standardReferenceObligation = builder.getStandardReferenceObligation();
	}
	
	/**
	 * Identifies the underlying asset when it is a series or a class of bonds. As a difference with FpML, the CDM specifies the bond only with this product identifier attribute. The reason for this approach is to avoid the potential for conflicting information between the information associated with the contractual product and the reference information maintained by the relevant service provider.
	 */
	@RosettaSynonym(value="bond", source="FpML_5_10")
	@RosettaSynonym(value="bond", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="bond", source="DTCC_11_0")
	@RosettaSynonym(value="bond", source="DTCC_9_0")
	@RosettaSynonym(value="bond", source="CME_ClearedConfirm_1_17")
	public final Bond getBond() {
		return bond;
	}
	
	/**
	 * Identifies the underlying asset when it is a convertible bond. As a difference with FpML, the CDM specifies the convertible bond only with this product identifier attribute. The reason for this approach is to avoid the potential for conflicting information between the information associated with the contractual product and the reference information maintained by the relevant service provider.
	 */
	@RosettaSynonym(value="convertibleBond", source="FpML_5_10")
	@RosettaSynonym(value="convertibleBond", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="convertibleBond", source="DTCC_11_0")
	@RosettaSynonym(value="convertibleBond", source="DTCC_9_0")
	@RosettaSynonym(value="convertibleBond", source="CME_ClearedConfirm_1_17")
	public final ConvertibleBond getConvertibleBond() {
		return convertibleBond;
	}
	
	/**
	 * The party that guarantees by way of a contractual arrangement to pay the debts of an obligor if the obligor is unable to make the required payments itself. ISDA 2003 Term: Guarantor.
	 */
	@RosettaSynonym(value="guarantor", source="FpML_5_10")
	@RosettaSynonym(value="guarantor", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="guarantor", source="DTCC_11_0")
	@RosettaSynonym(value="guarantor", source="DTCC_9_0")
	@RosettaSynonym(value="guarantor", source="CME_ClearedConfirm_1_17")
	public final LegalEntity getGuarantor() {
		return guarantor;
	}
	
	/**
	 * A pointer style reference to a reference entity defined elsewhere in the document. Used when the reference entity is the guarantor.
	 */
	@RosettaSynonym(value="guarantorReference", source="FpML_5_10")
	@RosettaSynonym(value="guarantorReference", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="guarantorReference", source="DTCC_11_0")
	@RosettaSynonym(value="guarantorReference", source="DTCC_9_0")
	@RosettaSynonym(value="guarantorReference", source="CME_ClearedConfirm_1_17")
	public final String getGuarantorReference() {
		return guarantorReference;
	}
	
	/**
	 * Identifies the underlying asset when it is a loan.
	 */
	@RosettaSynonym(value="loan", source="FpML_5_10")
	@RosettaSynonym(value="loan", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="loan", source="DTCC_11_0")
	@RosettaSynonym(value="loan", source="DTCC_9_0")
	@RosettaSynonym(value="loan", source="CME_ClearedConfirm_1_17")
	public final Loan getLoan() {
		return loan;
	}
	
	/**
	 * Identifies the underlying asset when it is a mortgage backed security.
	 */
	@RosettaSynonym(value="mortgage", source="FpML_5_10")
	@RosettaSynonym(value="mortgage", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="mortgage", source="DTCC_11_0")
	@RosettaSynonym(value="mortgage", source="DTCC_9_0")
	@RosettaSynonym(value="mortgage", source="CME_ClearedConfirm_1_17")
	public final MortgageBackedSecurity getMortgageBackedSecurity() {
		return mortgageBackedSecurity;
	}
	
	/**
	 * The entity primarily responsible for repaying debt to a creditor as a result of borrowing or issuing bonds. ISDA 2003 Term: Primary Obligor.
	 */
	@RosettaSynonym(value="primaryObligor", source="FpML_5_10")
	@RosettaSynonym(value="primaryObligor", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="primaryObligor", source="DTCC_11_0")
	@RosettaSynonym(value="primaryObligor", source="DTCC_9_0")
	@RosettaSynonym(value="primaryObligor", source="CME_ClearedConfirm_1_17")
	public final LegalEntity getPrimaryObligor() {
		return primaryObligor;
	}
	
	/**
	 * A pointer style reference to a reference entity defined elsewhere in the document. Used when the reference entity is the primary obligor.
	 */
	@RosettaSynonym(value="primaryObligorReference", source="FpML_5_10")
	@RosettaSynonym(value="primaryObligorReference", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="primaryObligorReference", source="DTCC_11_0")
	@RosettaSynonym(value="primaryObligorReference", source="DTCC_9_0")
	@RosettaSynonym(value="primaryObligorReference", source="CME_ClearedConfirm_1_17")
	public final ReferenceWithMetaLegalEntity getPrimaryObligorReference() {
		return primaryObligorReference;
	}
	
	/**
	 * Indicates if the reference obligation is a Standard Reference Obligation. ISDA 2014 Term: Standard Reference Obligation.
	 */
	@RosettaSynonym(value="standardReferenceObligation", source="FpML_5_10")
	@RosettaSynonym(value="standardReferenceObligation", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="standardReferenceObligation", source="DTCC_11_0")
	@RosettaSynonym(value="standardReferenceObligation", source="DTCC_9_0")
	@RosettaSynonym(value="standardReferenceObligation", source="CME_ClearedConfirm_1_17")
	public final Boolean getStandardReferenceObligation() {
		return standardReferenceObligation;
	}
	
	
	@Override
	public RosettaMetaData<? extends ReferenceObligation> metaData() {
		return metaData;
	} 
	
	public ReferenceObligationBuilder toBuilder() {
		ReferenceObligationBuilder builder = new ReferenceObligationBuilder();
		ofNullable(getBond()).ifPresent(builder::setBond);
		ofNullable(getConvertibleBond()).ifPresent(builder::setConvertibleBond);
		ofNullable(getGuarantor()).ifPresent(builder::setGuarantor);
		ofNullable(getGuarantorReference()).ifPresent(builder::setGuarantorReference);
		ofNullable(getLoan()).ifPresent(builder::setLoan);
		ofNullable(getMortgageBackedSecurity()).ifPresent(builder::setMortgageBackedSecurity);
		ofNullable(getPrimaryObligor()).ifPresent(builder::setPrimaryObligor);
		ofNullable(getPrimaryObligorReference()).ifPresent(builder::setPrimaryObligorReference);
		ofNullable(getStandardReferenceObligation()).ifPresent(builder::setStandardReferenceObligation);
		return builder;
	}

	public static ReferenceObligationBuilder builder() {
		return new ReferenceObligationBuilder();
	}

	public static  class ReferenceObligationBuilder extends RosettaModelObjectBuilder {
	
		protected Bond.BondBuilder bond;
		protected ConvertibleBond.ConvertibleBondBuilder convertibleBond;
		protected LegalEntity.LegalEntityBuilder guarantor;
		protected String guarantorReference;
		protected Loan.LoanBuilder loan;
		protected MortgageBackedSecurity.MortgageBackedSecurityBuilder mortgageBackedSecurity;
		protected LegalEntity.LegalEntityBuilder primaryObligor;
		protected ReferenceWithMetaLegalEntityBuilder primaryObligorReference;
		protected Boolean standardReferenceObligation;
	
		public ReferenceObligationBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends ReferenceObligation> metaData() {
			return metaData;
		} 
	
		
		public Bond.BondBuilder getBond() {
			return bond;
		}
		
		public Bond.BondBuilder getOrCreateBond() {
			if (bond!=null) {
				return bond;
			}
			else return bond = new Bond.BondBuilder();
		}
		
		
		public ConvertibleBond.ConvertibleBondBuilder getConvertibleBond() {
			return convertibleBond;
		}
		
		public ConvertibleBond.ConvertibleBondBuilder getOrCreateConvertibleBond() {
			if (convertibleBond!=null) {
				return convertibleBond;
			}
			else return convertibleBond = new ConvertibleBond.ConvertibleBondBuilder();
		}
		
		
		public LegalEntity.LegalEntityBuilder getGuarantor() {
			return guarantor;
		}
		
		public LegalEntity.LegalEntityBuilder getOrCreateGuarantor() {
			if (guarantor!=null) {
				return guarantor;
			}
			else return guarantor = new LegalEntity.LegalEntityBuilder();
		}
		
		
		public String getGuarantorReference() {
			return guarantorReference;
		}
		
		
		public Loan.LoanBuilder getLoan() {
			return loan;
		}
		
		public Loan.LoanBuilder getOrCreateLoan() {
			if (loan!=null) {
				return loan;
			}
			else return loan = new Loan.LoanBuilder();
		}
		
		
		public MortgageBackedSecurity.MortgageBackedSecurityBuilder getMortgageBackedSecurity() {
			return mortgageBackedSecurity;
		}
		
		public MortgageBackedSecurity.MortgageBackedSecurityBuilder getOrCreateMortgageBackedSecurity() {
			if (mortgageBackedSecurity!=null) {
				return mortgageBackedSecurity;
			}
			else return mortgageBackedSecurity = new MortgageBackedSecurity.MortgageBackedSecurityBuilder();
		}
		
		
		public LegalEntity.LegalEntityBuilder getPrimaryObligor() {
			return primaryObligor;
		}
		
		public LegalEntity.LegalEntityBuilder getOrCreatePrimaryObligor() {
			if (primaryObligor!=null) {
				return primaryObligor;
			}
			else return primaryObligor = new LegalEntity.LegalEntityBuilder();
		}
		
		
		public ReferenceWithMetaLegalEntityBuilder getPrimaryObligorReference() {
			return primaryObligorReference;
		}
		
		public ReferenceWithMetaLegalEntityBuilder getOrCreatePrimaryObligorReference() {
			if (primaryObligorReference!=null) {
				return primaryObligorReference;
			}
			else return primaryObligorReference = new ReferenceWithMetaLegalEntityBuilder();
		}
		
		
		public Boolean getStandardReferenceObligation() {
			return standardReferenceObligation;
		}
		
	
		public ReferenceObligationBuilder setBond(Bond bond) {
			this.bond = bond.toBuilder();
			return this;
		}
		public ReferenceObligationBuilder setBondBuilder(Bond.BondBuilder bond) {
			this.bond = bond;
			return this;
		}
		public ReferenceObligationBuilder setConvertibleBond(ConvertibleBond convertibleBond) {
			this.convertibleBond = convertibleBond.toBuilder();
			return this;
		}
		public ReferenceObligationBuilder setConvertibleBondBuilder(ConvertibleBond.ConvertibleBondBuilder convertibleBond) {
			this.convertibleBond = convertibleBond;
			return this;
		}
		public ReferenceObligationBuilder setGuarantor(LegalEntity guarantor) {
			this.guarantor = guarantor.toBuilder();
			return this;
		}
		public ReferenceObligationBuilder setGuarantorBuilder(LegalEntity.LegalEntityBuilder guarantor) {
			this.guarantor = guarantor;
			return this;
		}
		public ReferenceObligationBuilder setGuarantorReference(String guarantorReference) {
			this.guarantorReference = guarantorReference;
			return this;
		}
		public ReferenceObligationBuilder setLoan(Loan loan) {
			this.loan = loan.toBuilder();
			return this;
		}
		public ReferenceObligationBuilder setLoanBuilder(Loan.LoanBuilder loan) {
			this.loan = loan;
			return this;
		}
		public ReferenceObligationBuilder setMortgageBackedSecurity(MortgageBackedSecurity mortgageBackedSecurity) {
			this.mortgageBackedSecurity = mortgageBackedSecurity.toBuilder();
			return this;
		}
		public ReferenceObligationBuilder setMortgageBackedSecurityBuilder(MortgageBackedSecurity.MortgageBackedSecurityBuilder mortgageBackedSecurity) {
			this.mortgageBackedSecurity = mortgageBackedSecurity;
			return this;
		}
		public ReferenceObligationBuilder setPrimaryObligor(LegalEntity primaryObligor) {
			this.primaryObligor = primaryObligor.toBuilder();
			return this;
		}
		public ReferenceObligationBuilder setPrimaryObligorBuilder(LegalEntity.LegalEntityBuilder primaryObligor) {
			this.primaryObligor = primaryObligor;
			return this;
		}
		public ReferenceObligationBuilder setPrimaryObligorReference(ReferenceWithMetaLegalEntity primaryObligorReference) {
			this.primaryObligorReference = primaryObligorReference.toBuilder();
			return this;
		}
		public ReferenceObligationBuilder setPrimaryObligorReferenceBuilder(ReferenceWithMetaLegalEntityBuilder primaryObligorReference) {
			this.primaryObligorReference = primaryObligorReference;
			return this;
		}
		public ReferenceObligationBuilder setPrimaryObligorReferenceRef(LegalEntity.LegalEntityBuilder primaryObligorReference) {
			return setPrimaryObligorReference(ReferenceWithMetaLegalEntity.builder().setValueBuilder(primaryObligorReference).build());
		}
		public ReferenceObligationBuilder setPrimaryObligorReferenceRef(LegalEntity primaryObligorReference) {
			return setPrimaryObligorReferenceRef(LegalEntity.builder());
		}
		public ReferenceObligationBuilder setStandardReferenceObligation(Boolean standardReferenceObligation) {
			this.standardReferenceObligation = standardReferenceObligation;
			return this;
		}
		
		public ReferenceObligation build() {
			return new ReferenceObligation(this);
		}
	
		@Override
		public ReferenceObligationBuilder prune() {
			if (bond!=null && !bond.prune().hasData()) bond = null;
			if (convertibleBond!=null && !convertibleBond.prune().hasData()) convertibleBond = null;
			if (guarantor!=null && !guarantor.prune().hasData()) guarantor = null;
			if (loan!=null && !loan.prune().hasData()) loan = null;
			if (mortgageBackedSecurity!=null && !mortgageBackedSecurity.prune().hasData()) mortgageBackedSecurity = null;
			if (primaryObligor!=null && !primaryObligor.prune().hasData()) primaryObligor = null;
			if (primaryObligorReference!=null && !primaryObligorReference.prune().hasData()) primaryObligorReference = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getBond()!=null && getBond().hasData()) return true;
			if (getConvertibleBond()!=null && getConvertibleBond().hasData()) return true;
			if (getGuarantor()!=null && getGuarantor().hasData()) return true;
			if (getGuarantorReference()!=null) return true;
			if (getLoan()!=null && getLoan().hasData()) return true;
			if (getMortgageBackedSecurity()!=null && getMortgageBackedSecurity().hasData()) return true;
			if (getPrimaryObligor()!=null && getPrimaryObligor().hasData()) return true;
			if (getPrimaryObligorReference()!=null && getPrimaryObligorReference().hasData()) return true;
			if (getStandardReferenceObligation()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("guarantorReference"), String.class, guarantorReference, this);
			processor.processBasic(path.newSubPath("standardReferenceObligation"), Boolean.class, standardReferenceObligation, this);
			
			processRosetta(path.newSubPath("bond"), processor, Bond.class, bond);
			processRosetta(path.newSubPath("convertibleBond"), processor, ConvertibleBond.class, convertibleBond);
			processRosetta(path.newSubPath("guarantor"), processor, LegalEntity.class, guarantor);
			processRosetta(path.newSubPath("loan"), processor, Loan.class, loan);
			processRosetta(path.newSubPath("mortgageBackedSecurity"), processor, MortgageBackedSecurity.class, mortgageBackedSecurity);
			processRosetta(path.newSubPath("primaryObligor"), processor, LegalEntity.class, primaryObligor);
			processRosetta(path.newSubPath("primaryObligorReference"), processor, ReferenceWithMetaLegalEntity.class, primaryObligorReference);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			ReferenceObligationBuilder _that = (ReferenceObligationBuilder) o;
		
			if (bond != null ? !bond.equals(_that.bond) : _that.bond != null) return false;
			if (convertibleBond != null ? !convertibleBond.equals(_that.convertibleBond) : _that.convertibleBond != null) return false;
			if (guarantor != null ? !guarantor.equals(_that.guarantor) : _that.guarantor != null) return false;
			if (guarantorReference != null ? !guarantorReference.equals(_that.guarantorReference) : _that.guarantorReference != null) return false;
			if (loan != null ? !loan.equals(_that.loan) : _that.loan != null) return false;
			if (mortgageBackedSecurity != null ? !mortgageBackedSecurity.equals(_that.mortgageBackedSecurity) : _that.mortgageBackedSecurity != null) return false;
			if (primaryObligor != null ? !primaryObligor.equals(_that.primaryObligor) : _that.primaryObligor != null) return false;
			if (primaryObligorReference != null ? !primaryObligorReference.equals(_that.primaryObligorReference) : _that.primaryObligorReference != null) return false;
			if (standardReferenceObligation != null ? !standardReferenceObligation.equals(_that.standardReferenceObligation) : _that.standardReferenceObligation != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (bond != null ? bond.hashCode() : 0);
			_result = 31 * _result + (convertibleBond != null ? convertibleBond.hashCode() : 0);
			_result = 31 * _result + (guarantor != null ? guarantor.hashCode() : 0);
			_result = 31 * _result + (guarantorReference != null ? guarantorReference.hashCode() : 0);
			_result = 31 * _result + (loan != null ? loan.hashCode() : 0);
			_result = 31 * _result + (mortgageBackedSecurity != null ? mortgageBackedSecurity.hashCode() : 0);
			_result = 31 * _result + (primaryObligor != null ? primaryObligor.hashCode() : 0);
			_result = 31 * _result + (primaryObligorReference != null ? primaryObligorReference.hashCode() : 0);
			_result = 31 * _result + (standardReferenceObligation != null ? standardReferenceObligation.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "ReferenceObligationBuilder {" +
				"bond=" + this.bond + ", " +
				"convertibleBond=" + this.convertibleBond + ", " +
				"guarantor=" + this.guarantor + ", " +
				"guarantorReference=" + this.guarantorReference + ", " +
				"loan=" + this.loan + ", " +
				"mortgageBackedSecurity=" + this.mortgageBackedSecurity + ", " +
				"primaryObligor=" + this.primaryObligor + ", " +
				"primaryObligorReference=" + this.primaryObligorReference + ", " +
				"standardReferenceObligation=" + this.standardReferenceObligation +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("guarantorReference"), String.class, guarantorReference, this);
		processor.processBasic(path.newSubPath("standardReferenceObligation"), Boolean.class, standardReferenceObligation, this);
		
		processRosetta(path.newSubPath("bond"), processor, Bond.class, bond);
		processRosetta(path.newSubPath("convertibleBond"), processor, ConvertibleBond.class, convertibleBond);
		processRosetta(path.newSubPath("guarantor"), processor, LegalEntity.class, guarantor);
		processRosetta(path.newSubPath("loan"), processor, Loan.class, loan);
		processRosetta(path.newSubPath("mortgageBackedSecurity"), processor, MortgageBackedSecurity.class, mortgageBackedSecurity);
		processRosetta(path.newSubPath("primaryObligor"), processor, LegalEntity.class, primaryObligor);
		processRosetta(path.newSubPath("primaryObligorReference"), processor, ReferenceWithMetaLegalEntity.class, primaryObligorReference);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		ReferenceObligation _that = (ReferenceObligation) o;
	
		if (bond != null ? !bond.equals(_that.bond) : _that.bond != null) return false;
		if (convertibleBond != null ? !convertibleBond.equals(_that.convertibleBond) : _that.convertibleBond != null) return false;
		if (guarantor != null ? !guarantor.equals(_that.guarantor) : _that.guarantor != null) return false;
		if (guarantorReference != null ? !guarantorReference.equals(_that.guarantorReference) : _that.guarantorReference != null) return false;
		if (loan != null ? !loan.equals(_that.loan) : _that.loan != null) return false;
		if (mortgageBackedSecurity != null ? !mortgageBackedSecurity.equals(_that.mortgageBackedSecurity) : _that.mortgageBackedSecurity != null) return false;
		if (primaryObligor != null ? !primaryObligor.equals(_that.primaryObligor) : _that.primaryObligor != null) return false;
		if (primaryObligorReference != null ? !primaryObligorReference.equals(_that.primaryObligorReference) : _that.primaryObligorReference != null) return false;
		if (standardReferenceObligation != null ? !standardReferenceObligation.equals(_that.standardReferenceObligation) : _that.standardReferenceObligation != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (bond != null ? bond.hashCode() : 0);
		_result = 31 * _result + (convertibleBond != null ? convertibleBond.hashCode() : 0);
		_result = 31 * _result + (guarantor != null ? guarantor.hashCode() : 0);
		_result = 31 * _result + (guarantorReference != null ? guarantorReference.hashCode() : 0);
		_result = 31 * _result + (loan != null ? loan.hashCode() : 0);
		_result = 31 * _result + (mortgageBackedSecurity != null ? mortgageBackedSecurity.hashCode() : 0);
		_result = 31 * _result + (primaryObligor != null ? primaryObligor.hashCode() : 0);
		_result = 31 * _result + (primaryObligorReference != null ? primaryObligorReference.hashCode() : 0);
		_result = 31 * _result + (standardReferenceObligation != null ? standardReferenceObligation.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "ReferenceObligation {" +
			"bond=" + this.bond + ", " +
			"convertibleBond=" + this.convertibleBond + ", " +
			"guarantor=" + this.guarantor + ", " +
			"guarantorReference=" + this.guarantorReference + ", " +
			"loan=" + this.loan + ", " +
			"mortgageBackedSecurity=" + this.mortgageBackedSecurity + ", " +
			"primaryObligor=" + this.primaryObligor + ", " +
			"primaryObligorReference=" + this.primaryObligorReference + ", " +
			"standardReferenceObligation=" + this.standardReferenceObligation +
		'}';
	}
}
