package org.isda.cdm;

import org.isda.cdm.ConstituentWeight;
import org.isda.cdm.ReferencePair;
import org.isda.cdm.metafields.ReferenceWithMetaCashSettlementTerms;
import org.isda.cdm.metafields.ReferenceWithMetaCashSettlementTerms.ReferenceWithMetaCashSettlementTermsBuilder;
import org.isda.cdm.metafields.ReferenceWithMetaPhysicalSettlementTerms;
import org.isda.cdm.metafields.ReferenceWithMetaPhysicalSettlementTerms.ReferenceWithMetaPhysicalSettlementTermsBuilder;
import org.isda.cdm.metafields.ReferenceWithMetaProtectionTerms;
import org.isda.cdm.metafields.ReferenceWithMetaProtectionTerms.ReferenceWithMetaProtectionTermsBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.ReferencePoolItemMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;
		import java.util.stream.Collectors;

/**
 * This type contains all the constituent weight and reference information.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="ReferencePoolItem", source="FpML_5_10")
public class ReferencePoolItem extends RosettaModelObject {
	private final ReferenceWithMetaCashSettlementTerms cashSettlementTermsReference;
	private final ConstituentWeight constituentWeight;
	private final ReferenceWithMetaPhysicalSettlementTerms physicalSettlementTermsReference;
	private final ReferenceWithMetaProtectionTerms protectionTermsReference;
	private final ReferencePair referencePair;
	private static ReferencePoolItemMeta metaData = new ReferencePoolItemMeta();
	
	ReferencePoolItem(ReferencePoolItemBuilder builder) {
		this.cashSettlementTermsReference = ofNullable(builder.getCashSettlementTermsReference()).map(f->f.build()).orElse(null);
		this.constituentWeight = ofNullable(builder.getConstituentWeight()).map(f->f.build()).orElse(null);
		this.physicalSettlementTermsReference = ofNullable(builder.getPhysicalSettlementTermsReference()).map(f->f.build()).orElse(null);
		this.protectionTermsReference = ofNullable(builder.getProtectionTermsReference()).map(f->f.build()).orElse(null);
		this.referencePair = ofNullable(builder.getReferencePair()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * Reference to the cash settlement terms applicable to this item.
	 */
	@RosettaSynonym(value="settlementTermsReference", source="FpML_5_10", maps=2)
	@RosettaSynonym(value="settlementTermsReference", source="CME_SubmissionIRS_1_0", maps=2)
	@RosettaSynonym(value="settlementTermsReference", source="DTCC_11_0", maps=2)
	@RosettaSynonym(value="settlementTermsReference", source="DTCC_9_0", maps=2)
	@RosettaSynonym(value="settlementTermsReference", source="CME_ClearedConfirm_1_17", maps=2)
	public final ReferenceWithMetaCashSettlementTerms getCashSettlementTermsReference() {
		return cashSettlementTermsReference;
	}
	
	/**
	 * Describes the weight of each of the constituents within the basket. If not provided, it is assumed to be equal weighted.
	 */
	@RosettaSynonym(value="constituentWeight", source="FpML_5_10")
	@RosettaSynonym(value="constituentWeight", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="constituentWeight", source="DTCC_11_0")
	@RosettaSynonym(value="constituentWeight", source="DTCC_9_0")
	@RosettaSynonym(value="constituentWeight", source="CME_ClearedConfirm_1_17")
	public final ConstituentWeight getConstituentWeight() {
		return constituentWeight;
	}
	
	/**
	 * Reference to the physical settlement terms applicable to this item.
	 */
	@RosettaSynonym(value="settlementTermsReference", source="FpML_5_10", maps=2)
	@RosettaSynonym(value="settlementTermsReference", source="CME_SubmissionIRS_1_0", maps=2)
	@RosettaSynonym(value="settlementTermsReference", source="DTCC_11_0", maps=2)
	@RosettaSynonym(value="settlementTermsReference", source="DTCC_9_0", maps=2)
	@RosettaSynonym(value="settlementTermsReference", source="CME_ClearedConfirm_1_17", maps=2)
	public final ReferenceWithMetaPhysicalSettlementTerms getPhysicalSettlementTermsReference() {
		return physicalSettlementTermsReference;
	}
	
	/**
	 * Reference to the documentation terms applicable to this item.
	 */
	@RosettaSynonym(value="protectionTermsReference", source="FpML_5_10")
	@RosettaSynonym(value="protectionTermsReference", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="protectionTermsReference", source="DTCC_11_0")
	@RosettaSynonym(value="protectionTermsReference", source="DTCC_9_0")
	@RosettaSynonym(value="protectionTermsReference", source="CME_ClearedConfirm_1_17")
	public final ReferenceWithMetaProtectionTerms getProtectionTermsReference() {
		return protectionTermsReference;
	}
	
	@RosettaSynonym(value="referencePair", source="FpML_5_10")
	@RosettaSynonym(value="referencePair", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="referencePair", source="DTCC_11_0")
	@RosettaSynonym(value="referencePair", source="DTCC_9_0")
	@RosettaSynonym(value="referencePair", source="CME_ClearedConfirm_1_17")
	public final ReferencePair getReferencePair() {
		return referencePair;
	}
	
	
	@Override
	public RosettaMetaData<? extends ReferencePoolItem> metaData() {
		return metaData;
	} 
	
	public ReferencePoolItemBuilder toBuilder() {
		ReferencePoolItemBuilder builder = new ReferencePoolItemBuilder();
		ofNullable(getCashSettlementTermsReference()).ifPresent(builder::setCashSettlementTermsReference);
		ofNullable(getConstituentWeight()).ifPresent(builder::setConstituentWeight);
		ofNullable(getPhysicalSettlementTermsReference()).ifPresent(builder::setPhysicalSettlementTermsReference);
		ofNullable(getProtectionTermsReference()).ifPresent(builder::setProtectionTermsReference);
		ofNullable(getReferencePair()).ifPresent(builder::setReferencePair);
		return builder;
	}

	public static ReferencePoolItemBuilder builder() {
		return new ReferencePoolItemBuilder();
	}

	public static  class ReferencePoolItemBuilder extends RosettaModelObjectBuilder {
	
		protected ReferenceWithMetaCashSettlementTermsBuilder cashSettlementTermsReference;
		protected ConstituentWeight.ConstituentWeightBuilder constituentWeight;
		protected ReferenceWithMetaPhysicalSettlementTermsBuilder physicalSettlementTermsReference;
		protected ReferenceWithMetaProtectionTermsBuilder protectionTermsReference;
		protected ReferencePair.ReferencePairBuilder referencePair;
	
		public ReferencePoolItemBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends ReferencePoolItem> metaData() {
			return metaData;
		} 
	
		
		public ReferenceWithMetaCashSettlementTermsBuilder getCashSettlementTermsReference() {
			return cashSettlementTermsReference;
		}
		
		public ReferenceWithMetaCashSettlementTermsBuilder getOrCreateCashSettlementTermsReference() {
			if (cashSettlementTermsReference!=null) {
				return cashSettlementTermsReference;
			}
			else return cashSettlementTermsReference = new ReferenceWithMetaCashSettlementTermsBuilder();
		}
		
		
		public ConstituentWeight.ConstituentWeightBuilder getConstituentWeight() {
			return constituentWeight;
		}
		
		public ConstituentWeight.ConstituentWeightBuilder getOrCreateConstituentWeight() {
			if (constituentWeight!=null) {
				return constituentWeight;
			}
			else return constituentWeight = new ConstituentWeight.ConstituentWeightBuilder();
		}
		
		
		public ReferenceWithMetaPhysicalSettlementTermsBuilder getPhysicalSettlementTermsReference() {
			return physicalSettlementTermsReference;
		}
		
		public ReferenceWithMetaPhysicalSettlementTermsBuilder getOrCreatePhysicalSettlementTermsReference() {
			if (physicalSettlementTermsReference!=null) {
				return physicalSettlementTermsReference;
			}
			else return physicalSettlementTermsReference = new ReferenceWithMetaPhysicalSettlementTermsBuilder();
		}
		
		
		public ReferenceWithMetaProtectionTermsBuilder getProtectionTermsReference() {
			return protectionTermsReference;
		}
		
		public ReferenceWithMetaProtectionTermsBuilder getOrCreateProtectionTermsReference() {
			if (protectionTermsReference!=null) {
				return protectionTermsReference;
			}
			else return protectionTermsReference = new ReferenceWithMetaProtectionTermsBuilder();
		}
		
		
		public ReferencePair.ReferencePairBuilder getReferencePair() {
			return referencePair;
		}
		
		public ReferencePair.ReferencePairBuilder getOrCreateReferencePair() {
			if (referencePair!=null) {
				return referencePair;
			}
			else return referencePair = new ReferencePair.ReferencePairBuilder();
		}
		
	
		public ReferencePoolItemBuilder setCashSettlementTermsReference(ReferenceWithMetaCashSettlementTerms cashSettlementTermsReference) {
			this.cashSettlementTermsReference = cashSettlementTermsReference.toBuilder();
			return this;
		}
		public ReferencePoolItemBuilder setCashSettlementTermsReferenceBuilder(ReferenceWithMetaCashSettlementTermsBuilder cashSettlementTermsReference) {
			this.cashSettlementTermsReference = cashSettlementTermsReference;
			return this;
		}
		public ReferencePoolItemBuilder setCashSettlementTermsReferenceRef(CashSettlementTerms.CashSettlementTermsBuilder cashSettlementTermsReference) {
			return setCashSettlementTermsReference(ReferenceWithMetaCashSettlementTerms.builder().setValueBuilder(cashSettlementTermsReference).build());
		}
		public ReferencePoolItemBuilder setCashSettlementTermsReferenceRef(CashSettlementTerms cashSettlementTermsReference) {
			return setCashSettlementTermsReferenceRef(CashSettlementTerms.builder());
		}
		public ReferencePoolItemBuilder setConstituentWeight(ConstituentWeight constituentWeight) {
			this.constituentWeight = constituentWeight.toBuilder();
			return this;
		}
		public ReferencePoolItemBuilder setConstituentWeightBuilder(ConstituentWeight.ConstituentWeightBuilder constituentWeight) {
			this.constituentWeight = constituentWeight;
			return this;
		}
		public ReferencePoolItemBuilder setPhysicalSettlementTermsReference(ReferenceWithMetaPhysicalSettlementTerms physicalSettlementTermsReference) {
			this.physicalSettlementTermsReference = physicalSettlementTermsReference.toBuilder();
			return this;
		}
		public ReferencePoolItemBuilder setPhysicalSettlementTermsReferenceBuilder(ReferenceWithMetaPhysicalSettlementTermsBuilder physicalSettlementTermsReference) {
			this.physicalSettlementTermsReference = physicalSettlementTermsReference;
			return this;
		}
		public ReferencePoolItemBuilder setPhysicalSettlementTermsReferenceRef(PhysicalSettlementTerms.PhysicalSettlementTermsBuilder physicalSettlementTermsReference) {
			return setPhysicalSettlementTermsReference(ReferenceWithMetaPhysicalSettlementTerms.builder().setValueBuilder(physicalSettlementTermsReference).build());
		}
		public ReferencePoolItemBuilder setPhysicalSettlementTermsReferenceRef(PhysicalSettlementTerms physicalSettlementTermsReference) {
			return setPhysicalSettlementTermsReferenceRef(PhysicalSettlementTerms.builder());
		}
		public ReferencePoolItemBuilder setProtectionTermsReference(ReferenceWithMetaProtectionTerms protectionTermsReference) {
			this.protectionTermsReference = protectionTermsReference.toBuilder();
			return this;
		}
		public ReferencePoolItemBuilder setProtectionTermsReferenceBuilder(ReferenceWithMetaProtectionTermsBuilder protectionTermsReference) {
			this.protectionTermsReference = protectionTermsReference;
			return this;
		}
		public ReferencePoolItemBuilder setProtectionTermsReferenceRef(ProtectionTerms.ProtectionTermsBuilder protectionTermsReference) {
			return setProtectionTermsReference(ReferenceWithMetaProtectionTerms.builder().setValueBuilder(protectionTermsReference).build());
		}
		public ReferencePoolItemBuilder setProtectionTermsReferenceRef(ProtectionTerms protectionTermsReference) {
			return setProtectionTermsReferenceRef(ProtectionTerms.builder());
		}
		public ReferencePoolItemBuilder setReferencePair(ReferencePair referencePair) {
			this.referencePair = referencePair.toBuilder();
			return this;
		}
		public ReferencePoolItemBuilder setReferencePairBuilder(ReferencePair.ReferencePairBuilder referencePair) {
			this.referencePair = referencePair;
			return this;
		}
		
		public ReferencePoolItem build() {
			return new ReferencePoolItem(this);
		}
	
		@Override
		public ReferencePoolItemBuilder prune() {
			if (cashSettlementTermsReference!=null && !cashSettlementTermsReference.prune().hasData()) cashSettlementTermsReference = null;
			if (constituentWeight!=null && !constituentWeight.prune().hasData()) constituentWeight = null;
			if (physicalSettlementTermsReference!=null && !physicalSettlementTermsReference.prune().hasData()) physicalSettlementTermsReference = null;
			if (protectionTermsReference!=null && !protectionTermsReference.prune().hasData()) protectionTermsReference = null;
			if (referencePair!=null && !referencePair.prune().hasData()) referencePair = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getCashSettlementTermsReference()!=null && getCashSettlementTermsReference().hasData()) return true;
			if (getConstituentWeight()!=null && getConstituentWeight().hasData()) return true;
			if (getPhysicalSettlementTermsReference()!=null && getPhysicalSettlementTermsReference().hasData()) return true;
			if (getProtectionTermsReference()!=null && getProtectionTermsReference().hasData()) return true;
			if (getReferencePair()!=null && getReferencePair().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("cashSettlementTermsReference"), processor, ReferenceWithMetaCashSettlementTerms.class, cashSettlementTermsReference);
			processRosetta(path.newSubPath("constituentWeight"), processor, ConstituentWeight.class, constituentWeight);
			processRosetta(path.newSubPath("physicalSettlementTermsReference"), processor, ReferenceWithMetaPhysicalSettlementTerms.class, physicalSettlementTermsReference);
			processRosetta(path.newSubPath("protectionTermsReference"), processor, ReferenceWithMetaProtectionTerms.class, protectionTermsReference);
			processRosetta(path.newSubPath("referencePair"), processor, ReferencePair.class, referencePair);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			ReferencePoolItemBuilder _that = (ReferencePoolItemBuilder) o;
		
			if (cashSettlementTermsReference != null ? !cashSettlementTermsReference.equals(_that.cashSettlementTermsReference) : _that.cashSettlementTermsReference != null) return false;
			if (constituentWeight != null ? !constituentWeight.equals(_that.constituentWeight) : _that.constituentWeight != null) return false;
			if (physicalSettlementTermsReference != null ? !physicalSettlementTermsReference.equals(_that.physicalSettlementTermsReference) : _that.physicalSettlementTermsReference != null) return false;
			if (protectionTermsReference != null ? !protectionTermsReference.equals(_that.protectionTermsReference) : _that.protectionTermsReference != null) return false;
			if (referencePair != null ? !referencePair.equals(_that.referencePair) : _that.referencePair != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (cashSettlementTermsReference != null ? cashSettlementTermsReference.hashCode() : 0);
			_result = 31 * _result + (constituentWeight != null ? constituentWeight.hashCode() : 0);
			_result = 31 * _result + (physicalSettlementTermsReference != null ? physicalSettlementTermsReference.hashCode() : 0);
			_result = 31 * _result + (protectionTermsReference != null ? protectionTermsReference.hashCode() : 0);
			_result = 31 * _result + (referencePair != null ? referencePair.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "ReferencePoolItemBuilder {" +
				"cashSettlementTermsReference=" + this.cashSettlementTermsReference + ", " +
				"constituentWeight=" + this.constituentWeight + ", " +
				"physicalSettlementTermsReference=" + this.physicalSettlementTermsReference + ", " +
				"protectionTermsReference=" + this.protectionTermsReference + ", " +
				"referencePair=" + this.referencePair +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("cashSettlementTermsReference"), processor, ReferenceWithMetaCashSettlementTerms.class, cashSettlementTermsReference);
		processRosetta(path.newSubPath("constituentWeight"), processor, ConstituentWeight.class, constituentWeight);
		processRosetta(path.newSubPath("physicalSettlementTermsReference"), processor, ReferenceWithMetaPhysicalSettlementTerms.class, physicalSettlementTermsReference);
		processRosetta(path.newSubPath("protectionTermsReference"), processor, ReferenceWithMetaProtectionTerms.class, protectionTermsReference);
		processRosetta(path.newSubPath("referencePair"), processor, ReferencePair.class, referencePair);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		ReferencePoolItem _that = (ReferencePoolItem) o;
	
		if (cashSettlementTermsReference != null ? !cashSettlementTermsReference.equals(_that.cashSettlementTermsReference) : _that.cashSettlementTermsReference != null) return false;
		if (constituentWeight != null ? !constituentWeight.equals(_that.constituentWeight) : _that.constituentWeight != null) return false;
		if (physicalSettlementTermsReference != null ? !physicalSettlementTermsReference.equals(_that.physicalSettlementTermsReference) : _that.physicalSettlementTermsReference != null) return false;
		if (protectionTermsReference != null ? !protectionTermsReference.equals(_that.protectionTermsReference) : _that.protectionTermsReference != null) return false;
		if (referencePair != null ? !referencePair.equals(_that.referencePair) : _that.referencePair != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (cashSettlementTermsReference != null ? cashSettlementTermsReference.hashCode() : 0);
		_result = 31 * _result + (constituentWeight != null ? constituentWeight.hashCode() : 0);
		_result = 31 * _result + (physicalSettlementTermsReference != null ? physicalSettlementTermsReference.hashCode() : 0);
		_result = 31 * _result + (protectionTermsReference != null ? protectionTermsReference.hashCode() : 0);
		_result = 31 * _result + (referencePair != null ? referencePair.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "ReferencePoolItem {" +
			"cashSettlementTermsReference=" + this.cashSettlementTermsReference + ", " +
			"constituentWeight=" + this.constituentWeight + ", " +
			"physicalSettlementTermsReference=" + this.physicalSettlementTermsReference + ", " +
			"protectionTermsReference=" + this.protectionTermsReference + ", " +
			"referencePair=" + this.referencePair +
		'}';
	}
}
