package org.isda.cdm;

import org.isda.cdm.MakeWholeAmount;
import org.isda.cdm.SwapCurveValuation;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.ReferenceSwapCurveMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A complex type used to specify the option and convertible bond option strike when expressed in reference to a swap curve.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="ReferenceSwapCurve", source="FpML_5_10")
public class ReferenceSwapCurve extends RosettaModelObject {
	private final MakeWholeAmount makeWholeAmount;
	private final SwapCurveValuation swapUnwindValue;
	private static ReferenceSwapCurveMeta metaData = new ReferenceSwapCurveMeta();
	
	ReferenceSwapCurve(ReferenceSwapCurveBuilder builder) {
		this.makeWholeAmount = ofNullable(builder.getMakeWholeAmount()).map(f->f.build()).orElse(null);
		this.swapUnwindValue = ofNullable(builder.getSwapUnwindValue()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * Amount to be paid by the buyer of the option if the option is exercised prior to the Early Call Date. (The market practice in the convertible bond option space being that the buyer should be penalised if he/she exercises the option early on.)
	 */
	@RosettaSynonym(value="makeWholeAmount", source="FpML_5_10")
	@RosettaSynonym(value="makeWholeAmount", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="makeWholeAmount", source="DTCC_11_0")
	@RosettaSynonym(value="makeWholeAmount", source="DTCC_9_0")
	@RosettaSynonym(value="makeWholeAmount", source="CME_ClearedConfirm_1_17")
	public final MakeWholeAmount getMakeWholeAmount() {
		return makeWholeAmount;
	}
	
	@RosettaSynonym(value="swapUnwindValue", source="FpML_5_10")
	@RosettaSynonym(value="swapUnwindValue", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="swapUnwindValue", source="DTCC_11_0")
	@RosettaSynonym(value="swapUnwindValue", source="DTCC_9_0")
	@RosettaSynonym(value="swapUnwindValue", source="CME_ClearedConfirm_1_17")
	public final SwapCurveValuation getSwapUnwindValue() {
		return swapUnwindValue;
	}
	
	
	@Override
	public RosettaMetaData<? extends ReferenceSwapCurve> metaData() {
		return metaData;
	} 
	
	public ReferenceSwapCurveBuilder toBuilder() {
		ReferenceSwapCurveBuilder builder = new ReferenceSwapCurveBuilder();
		ofNullable(getMakeWholeAmount()).ifPresent(builder::setMakeWholeAmount);
		ofNullable(getSwapUnwindValue()).ifPresent(builder::setSwapUnwindValue);
		return builder;
	}

	public static ReferenceSwapCurveBuilder builder() {
		return new ReferenceSwapCurveBuilder();
	}

	public static  class ReferenceSwapCurveBuilder extends RosettaModelObjectBuilder {
	
		protected MakeWholeAmount.MakeWholeAmountBuilder makeWholeAmount;
		protected SwapCurveValuation.SwapCurveValuationBuilder swapUnwindValue;
	
		public ReferenceSwapCurveBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends ReferenceSwapCurve> metaData() {
			return metaData;
		} 
	
		
		public MakeWholeAmount.MakeWholeAmountBuilder getMakeWholeAmount() {
			return makeWholeAmount;
		}
		
		public MakeWholeAmount.MakeWholeAmountBuilder getOrCreateMakeWholeAmount() {
			if (makeWholeAmount!=null) {
				return makeWholeAmount;
			}
			else return makeWholeAmount = new MakeWholeAmount.MakeWholeAmountBuilder();
		}
		
		
		public SwapCurveValuation.SwapCurveValuationBuilder getSwapUnwindValue() {
			return swapUnwindValue;
		}
		
		public SwapCurveValuation.SwapCurveValuationBuilder getOrCreateSwapUnwindValue() {
			if (swapUnwindValue!=null) {
				return swapUnwindValue;
			}
			else return swapUnwindValue = new SwapCurveValuation.SwapCurveValuationBuilder();
		}
		
	
		public ReferenceSwapCurveBuilder setMakeWholeAmount(MakeWholeAmount makeWholeAmount) {
			this.makeWholeAmount = makeWholeAmount.toBuilder();
			return this;
		}
		public ReferenceSwapCurveBuilder setMakeWholeAmountBuilder(MakeWholeAmount.MakeWholeAmountBuilder makeWholeAmount) {
			this.makeWholeAmount = makeWholeAmount;
			return this;
		}
		public ReferenceSwapCurveBuilder setSwapUnwindValue(SwapCurveValuation swapUnwindValue) {
			this.swapUnwindValue = swapUnwindValue.toBuilder();
			return this;
		}
		public ReferenceSwapCurveBuilder setSwapUnwindValueBuilder(SwapCurveValuation.SwapCurveValuationBuilder swapUnwindValue) {
			this.swapUnwindValue = swapUnwindValue;
			return this;
		}
		
		public ReferenceSwapCurve build() {
			return new ReferenceSwapCurve(this);
		}
	
		@Override
		public ReferenceSwapCurveBuilder prune() {
			if (makeWholeAmount!=null && !makeWholeAmount.prune().hasData()) makeWholeAmount = null;
			if (swapUnwindValue!=null && !swapUnwindValue.prune().hasData()) swapUnwindValue = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getMakeWholeAmount()!=null && getMakeWholeAmount().hasData()) return true;
			if (getSwapUnwindValue()!=null && getSwapUnwindValue().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("makeWholeAmount"), processor, MakeWholeAmount.class, makeWholeAmount);
			processRosetta(path.newSubPath("swapUnwindValue"), processor, SwapCurveValuation.class, swapUnwindValue);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			ReferenceSwapCurveBuilder _that = (ReferenceSwapCurveBuilder) o;
		
			if (makeWholeAmount != null ? !makeWholeAmount.equals(_that.makeWholeAmount) : _that.makeWholeAmount != null) return false;
			if (swapUnwindValue != null ? !swapUnwindValue.equals(_that.swapUnwindValue) : _that.swapUnwindValue != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (makeWholeAmount != null ? makeWholeAmount.hashCode() : 0);
			_result = 31 * _result + (swapUnwindValue != null ? swapUnwindValue.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "ReferenceSwapCurveBuilder {" +
				"makeWholeAmount=" + this.makeWholeAmount + ", " +
				"swapUnwindValue=" + this.swapUnwindValue +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("makeWholeAmount"), processor, MakeWholeAmount.class, makeWholeAmount);
		processRosetta(path.newSubPath("swapUnwindValue"), processor, SwapCurveValuation.class, swapUnwindValue);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		ReferenceSwapCurve _that = (ReferenceSwapCurve) o;
	
		if (makeWholeAmount != null ? !makeWholeAmount.equals(_that.makeWholeAmount) : _that.makeWholeAmount != null) return false;
		if (swapUnwindValue != null ? !swapUnwindValue.equals(_that.swapUnwindValue) : _that.swapUnwindValue != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (makeWholeAmount != null ? makeWholeAmount.hashCode() : 0);
		_result = 31 * _result + (swapUnwindValue != null ? swapUnwindValue.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "ReferenceSwapCurve {" +
			"makeWholeAmount=" + this.makeWholeAmount + ", " +
			"swapUnwindValue=" + this.swapUnwindValue +
		'}';
	}
}
