package org.isda.cdm;

import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.AdditionalRegime;
import org.isda.cdm.ApplicableRegime;
import org.isda.cdm.metafields.ReferenceWithMetaParty;
import org.isda.cdm.metafields.ReferenceWithMetaParty.ReferenceWithMetaPartyBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.RegimeElectionMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.stream.Collectors;
		import static java.util.Optional.ofNullable;
		import java.util.List;
		import java.util.ArrayList;
		import java.util.Objects;

/**
 * A class to specify the parties&#39; respective elections with respect to the applicable regulatory regime(s) in their capacity as Secured Party (English Law &amp; New York Law) or Obligee (Japanese Law).
 * 
 * @version 2.5.4
 */
@RosettaClass
public class RegimeElection extends RosettaModelObject {
	private final List<AdditionalRegime> additionalRegime;
	private final List<ApplicableRegime> applicableRegime;
	private final ReferenceWithMetaParty party;
	private static RegimeElectionMeta metaData = new RegimeElectionMeta();
	
	RegimeElection(RegimeElectionBuilder builder) {
		this.additionalRegime = ofNullable(builder.getAdditionalRegime()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.applicableRegime = ofNullable(builder.getApplicableRegime()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.party = ofNullable(builder.getParty()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The additional regulatory regime(s) that might be specified by the parties to a legal agreement. ISDA 2016 Credit Support Annex for Initial Margin paragraph 13, General Principles, (ll): The parties may from time to time agree in writing that other regimes also comprise `Regimes` and that the General Principles be adopted and/or amended to accommodate such additional Regimes.
	 */
	@RosettaSynonym(value="additional_regimes", source="ISDA_Create_1_0")
	public final List<AdditionalRegime> getAdditionalRegime() {
		return additionalRegime;
	}
	
	/**
	 * The set of regulatory regimes and associated provisions applicable to the respective parties to the legal agreement.
	 */
	@RosettaSynonym(value="canada", source="ISDA_Create_1_0")
	@RosettaSynonym(value="sec", source="ISDA_Create_1_0")
	@RosettaSynonym(value="cftc", source="ISDA_Create_1_0")
	@RosettaSynonym(value="emir", source="ISDA_Create_1_0")
	@RosettaSynonym(value="japan", source="ISDA_Create_1_0")
	@RosettaSynonym(value="switzerland", source="ISDA_Create_1_0")
	@RosettaSynonym(value="prudential", source="ISDA_Create_1_0")
	public final List<ApplicableRegime> getApplicableRegime() {
		return applicableRegime;
	}
	
	/**
	 * The elective party.
	 */
	@RosettaSynonym(value="party", source="AcadiaSoft_AM_1_0")
	public final ReferenceWithMetaParty getParty() {
		return party;
	}
	
	
	@Override
	public RosettaMetaData<? extends RegimeElection> metaData() {
		return metaData;
	} 
	
	public RegimeElectionBuilder toBuilder() {
		RegimeElectionBuilder builder = new RegimeElectionBuilder();
		ofNullable(getAdditionalRegime()).ifPresent(additionalRegime -> additionalRegime.forEach(builder::addAdditionalRegime));
		ofNullable(getApplicableRegime()).ifPresent(applicableRegime -> applicableRegime.forEach(builder::addApplicableRegime));
		ofNullable(getParty()).ifPresent(builder::setParty);
		return builder;
	}

	public static RegimeElectionBuilder builder() {
		return new RegimeElectionBuilder();
	}

	public static  class RegimeElectionBuilder extends RosettaModelObjectBuilder {
	
		protected List<AdditionalRegime.AdditionalRegimeBuilder> additionalRegime;
		protected List<ApplicableRegime.ApplicableRegimeBuilder> applicableRegime;
		protected ReferenceWithMetaPartyBuilder party;
	
		public RegimeElectionBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends RegimeElection> metaData() {
			return metaData;
		} 
	
		
		public List<AdditionalRegime.AdditionalRegimeBuilder> getAdditionalRegime() {
			return additionalRegime;
		}
		
		public AdditionalRegime.AdditionalRegimeBuilder getOrCreateAdditionalRegime(int index) {
			if (additionalRegime==null) {
				this.additionalRegime = new ArrayList<>();
			}
			return getIndex(additionalRegime, index, ()->new AdditionalRegime.AdditionalRegimeBuilder());
		}
		
		
		public List<ApplicableRegime.ApplicableRegimeBuilder> getApplicableRegime() {
			return applicableRegime;
		}
		
		public ApplicableRegime.ApplicableRegimeBuilder getOrCreateApplicableRegime(int index) {
			if (applicableRegime==null) {
				this.applicableRegime = new ArrayList<>();
			}
			return getIndex(applicableRegime, index, ()->new ApplicableRegime.ApplicableRegimeBuilder());
		}
		
		
		public ReferenceWithMetaPartyBuilder getParty() {
			return party;
		}
		
		public ReferenceWithMetaPartyBuilder getOrCreateParty() {
			if (party!=null) {
				return party;
			}
			else return party = new ReferenceWithMetaPartyBuilder();
		}
		
	
		public RegimeElectionBuilder addAdditionalRegime(AdditionalRegime additionalRegime) {
			if(this.additionalRegime == null){
				this.additionalRegime = new ArrayList<>();
			}
			this.additionalRegime.add(additionalRegime.toBuilder());
			return this;
		}
		public RegimeElectionBuilder addAdditionalRegime(AdditionalRegime additionalRegime, int _idx) {
			if(this.additionalRegime == null){
				this.additionalRegime = new ArrayList<>();
			}
			getIndex(this.additionalRegime, _idx, () -> additionalRegime.toBuilder());
			this.additionalRegime.set(_idx, additionalRegime.toBuilder());
			return this;
		}
		public RegimeElectionBuilder addAdditionalRegime(List<AdditionalRegime> additionalRegimes) {
			if(this.additionalRegime == null){
				this.additionalRegime = new ArrayList<>();
			}
			for (AdditionalRegime toAdd : additionalRegimes) {
				this.additionalRegime.add(toAdd.toBuilder());
			}
			return this;
		}
		public RegimeElectionBuilder addAdditionalRegimeBuilder(AdditionalRegime.AdditionalRegimeBuilder additionalRegime) {
			if(this.additionalRegime == null){
				this.additionalRegime = new ArrayList<>();
			}
			this.additionalRegime.add(additionalRegime);
			return this;
		}
		
		public RegimeElectionBuilder clearAdditionalRegime() {
			this.additionalRegime = null;
			return this;
		}
		public RegimeElectionBuilder addApplicableRegime(ApplicableRegime applicableRegime) {
			if(this.applicableRegime == null){
				this.applicableRegime = new ArrayList<>();
			}
			this.applicableRegime.add(applicableRegime.toBuilder());
			return this;
		}
		public RegimeElectionBuilder addApplicableRegime(ApplicableRegime applicableRegime, int _idx) {
			if(this.applicableRegime == null){
				this.applicableRegime = new ArrayList<>();
			}
			getIndex(this.applicableRegime, _idx, () -> applicableRegime.toBuilder());
			this.applicableRegime.set(_idx, applicableRegime.toBuilder());
			return this;
		}
		public RegimeElectionBuilder addApplicableRegime(List<ApplicableRegime> applicableRegimes) {
			if(this.applicableRegime == null){
				this.applicableRegime = new ArrayList<>();
			}
			for (ApplicableRegime toAdd : applicableRegimes) {
				this.applicableRegime.add(toAdd.toBuilder());
			}
			return this;
		}
		public RegimeElectionBuilder addApplicableRegimeBuilder(ApplicableRegime.ApplicableRegimeBuilder applicableRegime) {
			if(this.applicableRegime == null){
				this.applicableRegime = new ArrayList<>();
			}
			this.applicableRegime.add(applicableRegime);
			return this;
		}
		
		public RegimeElectionBuilder clearApplicableRegime() {
			this.applicableRegime = null;
			return this;
		}
		public RegimeElectionBuilder setParty(ReferenceWithMetaParty party) {
			this.party = party.toBuilder();
			return this;
		}
		public RegimeElectionBuilder setPartyBuilder(ReferenceWithMetaPartyBuilder party) {
			this.party = party;
			return this;
		}
		public RegimeElectionBuilder setPartyRef(Party.PartyBuilder party) {
			return setParty(ReferenceWithMetaParty.builder().setValueBuilder(party).build());
		}
		public RegimeElectionBuilder setPartyRef(Party party) {
			return setPartyRef(Party.builder());
		}
		
		public RegimeElection build() {
			return new RegimeElection(this);
		}
	
		@Override
		public RegimeElectionBuilder prune() {
			if (additionalRegime!=null) additionalRegime = additionalRegime.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (applicableRegime!=null) applicableRegime = applicableRegime.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (party!=null && !party.prune().hasData()) party = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getAdditionalRegime()!=null && getAdditionalRegime().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getApplicableRegime()!=null && getApplicableRegime().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getParty()!=null && getParty().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("additionalRegime"), processor, AdditionalRegime.class, additionalRegime);
			processRosetta(path.newSubPath("applicableRegime"), processor, ApplicableRegime.class, applicableRegime);
			processRosetta(path.newSubPath("party"), processor, ReferenceWithMetaParty.class, party);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			RegimeElectionBuilder _that = (RegimeElectionBuilder) o;
		
			if (!ListEquals.listEquals(additionalRegime, _that.additionalRegime)) return false;
			if (!ListEquals.listEquals(applicableRegime, _that.applicableRegime)) return false;
			if (party != null ? !party.equals(_that.party) : _that.party != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (additionalRegime != null ? additionalRegime.hashCode() : 0);
			_result = 31 * _result + (applicableRegime != null ? applicableRegime.hashCode() : 0);
			_result = 31 * _result + (party != null ? party.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "RegimeElectionBuilder {" +
				"additionalRegime=" + this.additionalRegime + ", " +
				"applicableRegime=" + this.applicableRegime + ", " +
				"party=" + this.party +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("additionalRegime"), processor, AdditionalRegime.class, additionalRegime);
		processRosetta(path.newSubPath("applicableRegime"), processor, ApplicableRegime.class, applicableRegime);
		processRosetta(path.newSubPath("party"), processor, ReferenceWithMetaParty.class, party);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		RegimeElection _that = (RegimeElection) o;
	
		if (!ListEquals.listEquals(additionalRegime, _that.additionalRegime)) return false;
		if (!ListEquals.listEquals(applicableRegime, _that.applicableRegime)) return false;
		if (party != null ? !party.equals(_that.party) : _that.party != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (additionalRegime != null ? additionalRegime.hashCode() : 0);
		_result = 31 * _result + (applicableRegime != null ? applicableRegime.hashCode() : 0);
		_result = 31 * _result + (party != null ? party.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "RegimeElection {" +
			"additionalRegime=" + this.additionalRegime + ", " +
			"applicableRegime=" + this.applicableRegime + ", " +
			"party=" + this.party +
		'}';
	}
}
