package org.isda.cdm;

import java.lang.Boolean;
import java.lang.String;
import org.isda.cdm.AdditionalTypeEnum;
import org.isda.cdm.SimmException;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.RegimeTermsMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class that is used by the ApplicableRegime and the AdditionalRegime classes to specify the regulatory regime terms which are referred to as part of certain legal agreements, such as such as the ISDA 2016 and 2018 CSA for Initial Margin.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class RegimeTerms extends RosettaModelObject {
	private final AdditionalTypeEnum additionalType;
	private final Boolean isApplicable;
	private final String party;
	private final Boolean retrospectiveEffect;
	private final SimmException simmException;
	private static RegimeTermsMeta metaData = new RegimeTermsMeta();
	
	RegimeTerms(RegimeTermsBuilder builder) {
		this.additionalType = builder.getAdditionalType();
		this.isApplicable = builder.getIsApplicable();
		this.party = builder.getParty();
		this.retrospectiveEffect = builder.getRetrospectiveEffect();
		this.simmException = ofNullable(builder.getSimmException()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The Additional Type of transaction that can require the collection or delivery of initial margin under the specified regulatory regime for the purposes of Covered Transactions, as specified in ISDA 2016 Credit Support Annex for Initial Margin, paragraph 13, General Principles, (b)(B).
	 */
	public final AdditionalTypeEnum getAdditionalType() {
		return additionalType;
	}
	
	/**
	 * A boolean flag to specify whether the regulatory regime is applicable.
	 */
	@RosettaSynonym(value="", source="ISDA_Create_1_0")
	@RosettaSynonym(value="", source="ISDA_Create_1_0")
	public final Boolean getIsApplicable() {
		return isApplicable;
	}
	
	/**
	 * The party to which the regime terms apply to.
	 */
	@RosettaSynonym(value="", source="ISDA_Create_1_0")
	@RosettaSynonym(value="", source="ISDA_Create_1_0")
	public final String getParty() {
		return party;
	}
	
	/**
	 * ISDA 2016 CSA for Initial Margin, paragraph 13 (b)(i): if `Retrospective Effect` is specified as applicable to a Regime (a `Retrospective Regime`) then all Covered Transactions (IM) under all other Regimes with an earlier Regime Effective Time will, to the extent that they would have been Covered Transactions (IM) under such Retrospective Regime had such Transactions been entered into at or after the Regime Effective Time of the Retrospective Regime, be deemed to be Covered Transactions (IM) for such Retrospective Regime.
	 */
	@RosettaSynonym(value="retrospectiveEffect", source="AcadiaSoft_AM_1_0")
	@RosettaSynonym(value="", source="ISDA_Create_1_0")
	@RosettaSynonym(value="", source="ISDA_Create_1_0")
	public final Boolean getRetrospectiveEffect() {
		return retrospectiveEffect;
	}
	
	/**
	 * The SIMM exception to the regulatory regime clause of the ISDA 2016 and 2018 CSA for Initial Margin as either a normalized value specified as part of an enumeration or a customized value specified of type string. ISDA 2016 Credit Support Annex for Initial Margin paragraph 13, Regime: SIMM Exception.
	 */
	public final SimmException getSimmException() {
		return simmException;
	}
	
	
	@Override
	public RosettaMetaData<? extends RegimeTerms> metaData() {
		return metaData;
	} 
	
	public RegimeTermsBuilder toBuilder() {
		RegimeTermsBuilder builder = new RegimeTermsBuilder();
		ofNullable(getAdditionalType()).ifPresent(builder::setAdditionalType);
		ofNullable(getIsApplicable()).ifPresent(builder::setIsApplicable);
		ofNullable(getParty()).ifPresent(builder::setParty);
		ofNullable(getRetrospectiveEffect()).ifPresent(builder::setRetrospectiveEffect);
		ofNullable(getSimmException()).ifPresent(builder::setSimmException);
		return builder;
	}

	public static RegimeTermsBuilder builder() {
		return new RegimeTermsBuilder();
	}

	public static  class RegimeTermsBuilder extends RosettaModelObjectBuilder {
	
		protected AdditionalTypeEnum additionalType;
		protected Boolean isApplicable;
		protected String party;
		protected Boolean retrospectiveEffect;
		protected SimmException.SimmExceptionBuilder simmException;
	
		public RegimeTermsBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends RegimeTerms> metaData() {
			return metaData;
		} 
	
		
		public AdditionalTypeEnum getAdditionalType() {
			return additionalType;
		}
		
		
		public Boolean getIsApplicable() {
			return isApplicable;
		}
		
		
		public String getParty() {
			return party;
		}
		
		
		public Boolean getRetrospectiveEffect() {
			return retrospectiveEffect;
		}
		
		
		public SimmException.SimmExceptionBuilder getSimmException() {
			return simmException;
		}
		
		public SimmException.SimmExceptionBuilder getOrCreateSimmException() {
			if (simmException!=null) {
				return simmException;
			}
			else return simmException = new SimmException.SimmExceptionBuilder();
		}
		
	
		public RegimeTermsBuilder setAdditionalType(AdditionalTypeEnum additionalType) {
			this.additionalType = additionalType;
			return this;
		}
		public RegimeTermsBuilder setIsApplicable(Boolean isApplicable) {
			this.isApplicable = isApplicable;
			return this;
		}
		public RegimeTermsBuilder setParty(String party) {
			this.party = party;
			return this;
		}
		public RegimeTermsBuilder setRetrospectiveEffect(Boolean retrospectiveEffect) {
			this.retrospectiveEffect = retrospectiveEffect;
			return this;
		}
		public RegimeTermsBuilder setSimmException(SimmException simmException) {
			this.simmException = simmException.toBuilder();
			return this;
		}
		public RegimeTermsBuilder setSimmExceptionBuilder(SimmException.SimmExceptionBuilder simmException) {
			this.simmException = simmException;
			return this;
		}
		
		public RegimeTerms build() {
			return new RegimeTerms(this);
		}
	
		@Override
		public RegimeTermsBuilder prune() {
			if (simmException!=null && !simmException.prune().hasData()) simmException = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getAdditionalType()!=null) return true;
			if (getIsApplicable()!=null) return true;
			if (getParty()!=null) return true;
			if (getRetrospectiveEffect()!=null) return true;
			if (getSimmException()!=null && getSimmException().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("additionalType"), AdditionalTypeEnum.class, additionalType, this);
			processor.processBasic(path.newSubPath("isApplicable"), Boolean.class, isApplicable, this);
			processor.processBasic(path.newSubPath("party"), String.class, party, this);
			processor.processBasic(path.newSubPath("retrospectiveEffect"), Boolean.class, retrospectiveEffect, this);
			
			processRosetta(path.newSubPath("simmException"), processor, SimmException.class, simmException);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			RegimeTermsBuilder _that = (RegimeTermsBuilder) o;
		
			if (additionalType != null ? !additionalType.equals(_that.additionalType) : _that.additionalType != null) return false;
			if (isApplicable != null ? !isApplicable.equals(_that.isApplicable) : _that.isApplicable != null) return false;
			if (party != null ? !party.equals(_that.party) : _that.party != null) return false;
			if (retrospectiveEffect != null ? !retrospectiveEffect.equals(_that.retrospectiveEffect) : _that.retrospectiveEffect != null) return false;
			if (simmException != null ? !simmException.equals(_that.simmException) : _that.simmException != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (additionalType != null ? additionalType.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (isApplicable != null ? isApplicable.hashCode() : 0);
			_result = 31 * _result + (party != null ? party.hashCode() : 0);
			_result = 31 * _result + (retrospectiveEffect != null ? retrospectiveEffect.hashCode() : 0);
			_result = 31 * _result + (simmException != null ? simmException.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "RegimeTermsBuilder {" +
				"additionalType=" + this.additionalType + ", " +
				"isApplicable=" + this.isApplicable + ", " +
				"party=" + this.party + ", " +
				"retrospectiveEffect=" + this.retrospectiveEffect + ", " +
				"simmException=" + this.simmException +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("additionalType"), AdditionalTypeEnum.class, additionalType, this);
		processor.processBasic(path.newSubPath("isApplicable"), Boolean.class, isApplicable, this);
		processor.processBasic(path.newSubPath("party"), String.class, party, this);
		processor.processBasic(path.newSubPath("retrospectiveEffect"), Boolean.class, retrospectiveEffect, this);
		
		processRosetta(path.newSubPath("simmException"), processor, SimmException.class, simmException);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		RegimeTerms _that = (RegimeTerms) o;
	
		if (additionalType != null ? !additionalType.equals(_that.additionalType) : _that.additionalType != null) return false;
		if (isApplicable != null ? !isApplicable.equals(_that.isApplicable) : _that.isApplicable != null) return false;
		if (party != null ? !party.equals(_that.party) : _that.party != null) return false;
		if (retrospectiveEffect != null ? !retrospectiveEffect.equals(_that.retrospectiveEffect) : _that.retrospectiveEffect != null) return false;
		if (simmException != null ? !simmException.equals(_that.simmException) : _that.simmException != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (additionalType != null ? additionalType.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (isApplicable != null ? isApplicable.hashCode() : 0);
		_result = 31 * _result + (party != null ? party.hashCode() : 0);
		_result = 31 * _result + (retrospectiveEffect != null ? retrospectiveEffect.hashCode() : 0);
		_result = 31 * _result + (simmException != null ? simmException.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "RegimeTerms {" +
			"additionalType=" + this.additionalType + ", " +
			"isApplicable=" + this.isApplicable + ", " +
			"party=" + this.party + ", " +
			"retrospectiveEffect=" + this.retrospectiveEffect + ", " +
			"simmException=" + this.simmException +
		'}';
	}
}
