package org.isda.cdm;

import java.lang.Boolean;
import java.lang.String;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.RelatedAgreementMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class to specify a related legal agreement. ISDA 2016 Credit Support Annex for Initial Margin, paragraph 13, General Principles, (s): Other CSA and Japanese Law CSA (VM). | ISDA 2016 Credit Support Annex for Variation Margin, paragraph 13, (o): Other CSA.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class RelatedAgreement extends RosettaModelObject {
	private final Boolean isSpecified;
	private final String legalDocument;
	private static RelatedAgreementMeta metaData = new RelatedAgreementMeta();
	
	RelatedAgreement(RelatedAgreementBuilder builder) {
		this.isSpecified = builder.getIsSpecified();
		this.legalDocument = builder.getLegalDocument();
	}
	
	/**
	 * The qualification of whether some other related agreement is specified (True) or not (False).
	 */
	public final Boolean getIsSpecified() {
		return isSpecified;
	}
	
	/**
	 * The specification of this other agreement, when the qualification is True.
	 */
	@RosettaSynonym(value="specify", source="ISDA_Create_1_0")
	public final String getLegalDocument() {
		return legalDocument;
	}
	
	
	@Override
	public RosettaMetaData<? extends RelatedAgreement> metaData() {
		return metaData;
	} 
	
	public RelatedAgreementBuilder toBuilder() {
		RelatedAgreementBuilder builder = new RelatedAgreementBuilder();
		ofNullable(getIsSpecified()).ifPresent(builder::setIsSpecified);
		ofNullable(getLegalDocument()).ifPresent(builder::setLegalDocument);
		return builder;
	}

	public static RelatedAgreementBuilder builder() {
		return new RelatedAgreementBuilder();
	}

	public static  class RelatedAgreementBuilder extends RosettaModelObjectBuilder {
	
		protected Boolean isSpecified;
		protected String legalDocument;
	
		public RelatedAgreementBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends RelatedAgreement> metaData() {
			return metaData;
		} 
	
		
		public Boolean getIsSpecified() {
			return isSpecified;
		}
		
		
		public String getLegalDocument() {
			return legalDocument;
		}
		
	
		public RelatedAgreementBuilder setIsSpecified(Boolean isSpecified) {
			this.isSpecified = isSpecified;
			return this;
		}
		public RelatedAgreementBuilder setLegalDocument(String legalDocument) {
			this.legalDocument = legalDocument;
			return this;
		}
		
		public RelatedAgreement build() {
			return new RelatedAgreement(this);
		}
	
		@Override
		public RelatedAgreementBuilder prune() {
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getIsSpecified()!=null) return true;
			if (getLegalDocument()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("isSpecified"), Boolean.class, isSpecified, this);
			processor.processBasic(path.newSubPath("legalDocument"), String.class, legalDocument, this);
			
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			RelatedAgreementBuilder _that = (RelatedAgreementBuilder) o;
		
			if (isSpecified != null ? !isSpecified.equals(_that.isSpecified) : _that.isSpecified != null) return false;
			if (legalDocument != null ? !legalDocument.equals(_that.legalDocument) : _that.legalDocument != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (isSpecified != null ? isSpecified.hashCode() : 0);
			_result = 31 * _result + (legalDocument != null ? legalDocument.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "RelatedAgreementBuilder {" +
				"isSpecified=" + this.isSpecified + ", " +
				"legalDocument=" + this.legalDocument +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("isSpecified"), Boolean.class, isSpecified, this);
		processor.processBasic(path.newSubPath("legalDocument"), String.class, legalDocument, this);
		
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		RelatedAgreement _that = (RelatedAgreement) o;
	
		if (isSpecified != null ? !isSpecified.equals(_that.isSpecified) : _that.isSpecified != null) return false;
		if (legalDocument != null ? !legalDocument.equals(_that.legalDocument) : _that.legalDocument != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (isSpecified != null ? isSpecified.hashCode() : 0);
		_result = 31 * _result + (legalDocument != null ? legalDocument.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "RelatedAgreement {" +
			"isSpecified=" + this.isSpecified + ", " +
			"legalDocument=" + this.legalDocument +
		'}';
	}
}
