package org.isda.cdm;

import com.rosetta.model.lib.records.Date;
import org.isda.cdm.BusinessCenters;
import org.isda.cdm.BusinessDayConventionEnum;
import org.isda.cdm.metafields.BasicReferenceWithMetaDate;
import org.isda.cdm.metafields.BasicReferenceWithMetaDate.BasicReferenceWithMetaDateBuilder;
import org.isda.cdm.metafields.ReferenceWithMetaBusinessCenters;
import org.isda.cdm.metafields.ReferenceWithMetaBusinessCenters.ReferenceWithMetaBusinessCentersBuilder;


import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.RelativeDateOffsetMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;
		import java.util.stream.Collectors;
		import com.rosetta.model.lib.records.Date;

/**
 * A class defining a date (referred to as the derived date) as a relative offset from another date (referred to as the anchor date). If the anchor date is itself an adjustable date then the offset is assumed to be calculated from the adjusted anchor date. A number of different scenarios can be supported, namely; 1) the derived date may simply be a number of calendar periods (days, weeks, months or years) preceding or following the anchor date; 2) the unadjusted derived date may be a number of calendar periods (days, weeks, months or years) preceding or following the anchor date with the resulting unadjusted derived date subject to adjustment in accordance with a specified business day convention, i.e. the derived date must fall on a good business day; 3) the derived date may be a number of business days preceding or following the anchor date. Note that the businessDayConvention specifies any required adjustment to the unadjusted derived date. A negative or positive value in the periodMultiplier indicates whether the unadjusted derived precedes or follows the anchor date. The businessDayConvention should contain a value NONE if the day type element contains a value of Business (since specifying a negative or positive business days offset would already guarantee that the derived date would fall on a good business day in the specified business centers).
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="RelativeDateOffset", source="FpML_5_10")
public class RelativeDateOffset extends Offset {
	private final Date adjustedDate;
	private final BusinessCenters businessCenters;
	private final ReferenceWithMetaBusinessCenters businessCentersReference;
	private final BusinessDayConventionEnum businessDayConvention;
	private final BasicReferenceWithMetaDate dateRelativeTo;
	private static RelativeDateOffsetMeta metaData = new RelativeDateOffsetMeta();
	
	RelativeDateOffset(RelativeDateOffsetBuilder builder) {
		super(builder);
		this.adjustedDate = builder.getAdjustedDate();
		this.businessCenters = ofNullable(builder.getBusinessCenters()).map(f->f.build()).orElse(null);
		this.businessCentersReference = ofNullable(builder.getBusinessCentersReference()).map(f->f.build()).orElse(null);
		this.businessDayConvention = builder.getBusinessDayConvention();
		this.dateRelativeTo = ofNullable(builder.getDateRelativeTo()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The date once the adjustment has been performed. (Note that this date may change if the business center holidays change).
	 */
	@RosettaSynonym(value="adjustedDate", source="FpML_5_10")
	@RosettaSynonym(value="adjustedDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="adjustedDate", source="DTCC_11_0")
	@RosettaSynonym(value="adjustedDate", source="DTCC_9_0")
	@RosettaSynonym(value="adjustedDate", source="CME_ClearedConfirm_1_17")
	public final Date getAdjustedDate() {
		return adjustedDate;
	}
	
	@RosettaSynonym(value="businessCenters", source="FpML_5_10")
	@RosettaSynonym(value="businessCenters", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="businessCenters", source="DTCC_11_0")
	@RosettaSynonym(value="businessCenters", source="DTCC_9_0")
	@RosettaSynonym(value="businessCenters", source="CME_ClearedConfirm_1_17")
	public final BusinessCenters getBusinessCenters() {
		return businessCenters;
	}
	
	/**
	 * A pointer style reference to a set of financial business centers defined elsewhere in the document. This set of business centers is used to determine whether a particular day is a business day or not.
	 */
	@RosettaSynonym(value="businessCentersReference", source="FpML_5_10")
	@RosettaSynonym(value="businessCentersReference", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="businessCentersReference", source="DTCC_11_0")
	@RosettaSynonym(value="businessCentersReference", source="DTCC_9_0")
	@RosettaSynonym(value="businessCentersReference", source="CME_ClearedConfirm_1_17")
	public final ReferenceWithMetaBusinessCenters getBusinessCentersReference() {
		return businessCentersReference;
	}
	
	/**
	 * The convention for adjusting a date if it would otherwise fall on a day that is not a business day, as specified by an ISDA convention (e.g. Following, Precedent).
	 */
	@RosettaSynonym(value="businessDayConvention", source="FpML_5_10")
	@RosettaSynonym(value="businessDayConvention", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="businessDayConvention", source="DTCC_11_0")
	@RosettaSynonym(value="businessDayConvention", source="DTCC_9_0")
	@RosettaSynonym(value="businessDayConvention", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="businessDayConvention", source="FpML_5_10", path="dateOffset")
	public final BusinessDayConventionEnum getBusinessDayConvention() {
		return businessDayConvention;
	}
	
	/**
	 * Specifies the anchor as an href attribute. The href attribute value is a pointer style reference to the element or component elsewhere in the document where the anchor date is defined.
	 */
	@RosettaSynonym(value="dateRelativeTo", source="FpML_5_10")
	@RosettaSynonym(value="dateRelativeTo", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="dateRelativeTo", source="DTCC_11_0")
	@RosettaSynonym(value="dateRelativeTo", source="DTCC_9_0")
	@RosettaSynonym(value="dateRelativeTo", source="CME_ClearedConfirm_1_17")
	public final BasicReferenceWithMetaDate getDateRelativeTo() {
		return dateRelativeTo;
	}
	
	
	@Override
	public RosettaMetaData<? extends RelativeDateOffset> metaData() {
		return metaData;
	} 
	
	public RelativeDateOffsetBuilder toBuilder() {
		RelativeDateOffsetBuilder builder = new RelativeDateOffsetBuilder();
		ofNullable(getAdjustedDate()).ifPresent(builder::setAdjustedDate);
		ofNullable(getBusinessCenters()).ifPresent(builder::setBusinessCenters);
		ofNullable(getBusinessCentersReference()).ifPresent(builder::setBusinessCentersReference);
		ofNullable(getBusinessDayConvention()).ifPresent(builder::setBusinessDayConvention);
		ofNullable(getDateRelativeTo()).ifPresent(builder::setDateRelativeTo);
		ofNullable(getDayType()).ifPresent(builder::setDayType);
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		ofNullable(getPeriod()).ifPresent(builder::setPeriod);
		ofNullable(getPeriodMultiplier()).ifPresent(builder::setPeriodMultiplier);
		return builder;
	}

	public static RelativeDateOffsetBuilder builder() {
		return new RelativeDateOffsetBuilder();
	}

	public static  class RelativeDateOffsetBuilder extends OffsetBuilder {
	
		protected Date adjustedDate;
		protected BusinessCenters.BusinessCentersBuilder businessCenters;
		protected ReferenceWithMetaBusinessCentersBuilder businessCentersReference;
		protected BusinessDayConventionEnum businessDayConvention;
		protected BasicReferenceWithMetaDateBuilder dateRelativeTo;
	
		public RelativeDateOffsetBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends RelativeDateOffset> metaData() {
			return metaData;
		} 
	
		
		public Date getAdjustedDate() {
			return adjustedDate;
		}
		
		
		public BusinessCenters.BusinessCentersBuilder getBusinessCenters() {
			return businessCenters;
		}
		
		public BusinessCenters.BusinessCentersBuilder getOrCreateBusinessCenters() {
			if (businessCenters!=null) {
				return businessCenters;
			}
			else return businessCenters = new BusinessCenters.BusinessCentersBuilder();
		}
		
		
		public ReferenceWithMetaBusinessCentersBuilder getBusinessCentersReference() {
			return businessCentersReference;
		}
		
		public ReferenceWithMetaBusinessCentersBuilder getOrCreateBusinessCentersReference() {
			if (businessCentersReference!=null) {
				return businessCentersReference;
			}
			else return businessCentersReference = new ReferenceWithMetaBusinessCentersBuilder();
		}
		
		
		public BusinessDayConventionEnum getBusinessDayConvention() {
			return businessDayConvention;
		}
		
		
		public BasicReferenceWithMetaDateBuilder getDateRelativeTo() {
			return dateRelativeTo;
		}
		
		public BasicReferenceWithMetaDateBuilder getOrCreateDateRelativeTo() {
			if (dateRelativeTo!=null) {
				return dateRelativeTo;
			}
			else return dateRelativeTo = new BasicReferenceWithMetaDateBuilder();
		}
		
	
		public RelativeDateOffsetBuilder setAdjustedDate(Date adjustedDate) {
			this.adjustedDate = adjustedDate;
			return this;
		}
		public RelativeDateOffsetBuilder setBusinessCenters(BusinessCenters businessCenters) {
			this.businessCenters = businessCenters.toBuilder();
			return this;
		}
		public RelativeDateOffsetBuilder setBusinessCentersBuilder(BusinessCenters.BusinessCentersBuilder businessCenters) {
			this.businessCenters = businessCenters;
			return this;
		}
		public RelativeDateOffsetBuilder setBusinessCentersReference(ReferenceWithMetaBusinessCenters businessCentersReference) {
			this.businessCentersReference = businessCentersReference.toBuilder();
			return this;
		}
		public RelativeDateOffsetBuilder setBusinessCentersReferenceBuilder(ReferenceWithMetaBusinessCentersBuilder businessCentersReference) {
			this.businessCentersReference = businessCentersReference;
			return this;
		}
		public RelativeDateOffsetBuilder setBusinessCentersReferenceRef(BusinessCenters.BusinessCentersBuilder businessCentersReference) {
			return setBusinessCentersReference(ReferenceWithMetaBusinessCenters.builder().setValueBuilder(businessCentersReference).build());
		}
		public RelativeDateOffsetBuilder setBusinessCentersReferenceRef(BusinessCenters businessCentersReference) {
			return setBusinessCentersReferenceRef(BusinessCenters.builder());
		}
		public RelativeDateOffsetBuilder setBusinessDayConvention(BusinessDayConventionEnum businessDayConvention) {
			this.businessDayConvention = businessDayConvention;
			return this;
		}
		public RelativeDateOffsetBuilder setDateRelativeTo(BasicReferenceWithMetaDate dateRelativeTo) {
			this.dateRelativeTo = dateRelativeTo.toBuilder();
			return this;
		}
		public RelativeDateOffsetBuilder setDateRelativeToRef(Date dateRelativeTo) {
			return setDateRelativeTo(BasicReferenceWithMetaDate.builder().setValue(dateRelativeTo).build());
		}
		@Override public RelativeDateOffsetBuilder setDayType(DayTypeEnum dayType) {
			this.dayType = dayType;
			return this;
		}
		@Override public RelativeDateOffsetBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		@Override public RelativeDateOffsetBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		@Override public RelativeDateOffsetBuilder setPeriod(PeriodEnum period) {
			this.period = period;
			return this;
		}
		@Override public RelativeDateOffsetBuilder setPeriodMultiplier(Integer periodMultiplier) {
			this.periodMultiplier = periodMultiplier;
			return this;
		}
		
		public RelativeDateOffset build() {
			return new RelativeDateOffset(this);
		}
	
		@Override
		public RelativeDateOffsetBuilder prune() {
			super.prune();
			if (businessCenters!=null && !businessCenters.prune().hasData()) businessCenters = null;
			if (businessCentersReference!=null && !businessCentersReference.prune().hasData()) businessCentersReference = null;
			if (dateRelativeTo!=null && !dateRelativeTo.prune().hasData()) dateRelativeTo = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (super.hasData()) return true;
			if (getAdjustedDate()!=null) return true;
			if (getBusinessCenters()!=null && getBusinessCenters().hasData()) return true;
			if (getBusinessCentersReference()!=null && getBusinessCentersReference().hasData()) return true;
			if (getBusinessDayConvention()!=null) return true;
			if (getDateRelativeTo()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
			super.process(path, processor);
		
			processor.processBasic(path.newSubPath("adjustedDate"), Date.class, adjustedDate, this);
			processor.processBasic(path.newSubPath("businessDayConvention"), BusinessDayConventionEnum.class, businessDayConvention, this);
			
			processRosetta(path.newSubPath("businessCenters"), processor, BusinessCenters.class, businessCenters);
			processRosetta(path.newSubPath("businessCentersReference"), processor, ReferenceWithMetaBusinessCenters.class, businessCentersReference);
			processRosetta(path.newSubPath("dateRelativeTo"), processor, BasicReferenceWithMetaDate.class, dateRelativeTo);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
			if (!super.equals(o)) return false;
		
			RelativeDateOffsetBuilder _that = (RelativeDateOffsetBuilder) o;
		
			if (adjustedDate != null ? !adjustedDate.equals(_that.adjustedDate) : _that.adjustedDate != null) return false;
			if (businessCenters != null ? !businessCenters.equals(_that.businessCenters) : _that.businessCenters != null) return false;
			if (businessCentersReference != null ? !businessCentersReference.equals(_that.businessCentersReference) : _that.businessCentersReference != null) return false;
			if (businessDayConvention != null ? !businessDayConvention.equals(_that.businessDayConvention) : _that.businessDayConvention != null) return false;
			if (dateRelativeTo != null ? !dateRelativeTo.equals(_that.dateRelativeTo) : _that.dateRelativeTo != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = super.hashCode();
			_result = 31 * _result + (adjustedDate != null ? adjustedDate.hashCode() : 0);
			_result = 31 * _result + (businessCenters != null ? businessCenters.hashCode() : 0);
			_result = 31 * _result + (businessCentersReference != null ? businessCentersReference.hashCode() : 0);
			_result = 31 * _result + (businessDayConvention != null ? businessDayConvention.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (dateRelativeTo != null ? dateRelativeTo.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "RelativeDateOffsetBuilder {" +
				"adjustedDate=" + this.adjustedDate + ", " +
				"businessCenters=" + this.businessCenters + ", " +
				"businessCentersReference=" + this.businessCentersReference + ", " +
				"businessDayConvention=" + this.businessDayConvention + ", " +
				"dateRelativeTo=" + this.dateRelativeTo +
			'}' + " " + super.toString();
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		super.process(path, processor);
		
		processor.processBasic(path.newSubPath("adjustedDate"), Date.class, adjustedDate, this);
		processor.processBasic(path.newSubPath("businessDayConvention"), BusinessDayConventionEnum.class, businessDayConvention, this);
		
		processRosetta(path.newSubPath("businessCenters"), processor, BusinessCenters.class, businessCenters);
		processRosetta(path.newSubPath("businessCentersReference"), processor, ReferenceWithMetaBusinessCenters.class, businessCentersReference);
		processRosetta(path.newSubPath("dateRelativeTo"), processor, BasicReferenceWithMetaDate.class, dateRelativeTo);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
		if (!super.equals(o)) return false;
	
		RelativeDateOffset _that = (RelativeDateOffset) o;
	
		if (adjustedDate != null ? !adjustedDate.equals(_that.adjustedDate) : _that.adjustedDate != null) return false;
		if (businessCenters != null ? !businessCenters.equals(_that.businessCenters) : _that.businessCenters != null) return false;
		if (businessCentersReference != null ? !businessCentersReference.equals(_that.businessCentersReference) : _that.businessCentersReference != null) return false;
		if (businessDayConvention != null ? !businessDayConvention.equals(_that.businessDayConvention) : _that.businessDayConvention != null) return false;
		if (dateRelativeTo != null ? !dateRelativeTo.equals(_that.dateRelativeTo) : _that.dateRelativeTo != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = super.hashCode();
		_result = 31 * _result + (adjustedDate != null ? adjustedDate.hashCode() : 0);
		_result = 31 * _result + (businessCenters != null ? businessCenters.hashCode() : 0);
		_result = 31 * _result + (businessCentersReference != null ? businessCentersReference.hashCode() : 0);
		_result = 31 * _result + (businessDayConvention != null ? businessDayConvention.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (dateRelativeTo != null ? dateRelativeTo.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "RelativeDateOffset {" +
			"adjustedDate=" + this.adjustedDate + ", " +
			"businessCenters=" + this.businessCenters + ", " +
			"businessCentersReference=" + this.businessCentersReference + ", " +
			"businessDayConvention=" + this.businessDayConvention + ", " +
			"dateRelativeTo=" + this.dateRelativeTo +
		'}' + " " + super.toString();
	}
}
