package org.isda.cdm;

import java.lang.Integer;
import org.isda.cdm.DateRange;
import org.isda.cdm.metafields.BasicReferenceWithMetaDate;
import org.isda.cdm.metafields.ReferenceWithMetaBusinessCenters;
import org.isda.cdm.metafields.ReferenceWithMetaBusinessCenters.ReferenceWithMetaBusinessCentersBuilder;


import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.RelativeDatesMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;
		import java.util.stream.Collectors;
		import com.rosetta.model.lib.records.Date;

/**
 * A class describing a set of dates defined as relative to another set of dates.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="RelativeDates", source="FpML_5_10")
public class RelativeDates extends RelativeDateOffset {
	private final Integer periodSkip;
	private final DateRange scheduleBounds;
	private static RelativeDatesMeta metaData = new RelativeDatesMeta();
	
	RelativeDates(RelativeDatesBuilder builder) {
		super(builder);
		this.periodSkip = builder.getPeriodSkip();
		this.scheduleBounds = ofNullable(builder.getScheduleBounds()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The number of periods in the referenced date schedule that are between each date in the relative date schedule. Thus a skip of 2 would mean that dates are relative to every second date in the referenced schedule. If present this should have a value greater than 1.
	 */
	@RosettaSynonym(value="periodSkip", source="FpML_5_10")
	@RosettaSynonym(value="periodSkip", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="periodSkip", source="DTCC_11_0")
	@RosettaSynonym(value="periodSkip", source="DTCC_9_0")
	@RosettaSynonym(value="periodSkip", source="CME_ClearedConfirm_1_17")
	public final Integer getPeriodSkip() {
		return periodSkip;
	}
	
	/**
	 * The first and last dates of a schedule. This can be used to restrict the range of values in a reference series of dates.
	 */
	@RosettaSynonym(value="scheduleBounds", source="FpML_5_10")
	@RosettaSynonym(value="scheduleBounds", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="scheduleBounds", source="DTCC_11_0")
	@RosettaSynonym(value="scheduleBounds", source="DTCC_9_0")
	@RosettaSynonym(value="scheduleBounds", source="CME_ClearedConfirm_1_17")
	public final DateRange getScheduleBounds() {
		return scheduleBounds;
	}
	
	
	@Override
	public RosettaMetaData<? extends RelativeDates> metaData() {
		return metaData;
	} 
	
	public RelativeDatesBuilder toBuilder() {
		RelativeDatesBuilder builder = new RelativeDatesBuilder();
		ofNullable(getPeriodSkip()).ifPresent(builder::setPeriodSkip);
		ofNullable(getScheduleBounds()).ifPresent(builder::setScheduleBounds);
		ofNullable(getAdjustedDate()).ifPresent(builder::setAdjustedDate);
		ofNullable(getBusinessCenters()).ifPresent(builder::setBusinessCenters);
		ofNullable(getBusinessCentersReference()).ifPresent(builder::setBusinessCentersReference);
		ofNullable(getBusinessDayConvention()).ifPresent(builder::setBusinessDayConvention);
		ofNullable(getDateRelativeTo()).ifPresent(builder::setDateRelativeTo);
		ofNullable(getDayType()).ifPresent(builder::setDayType);
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		ofNullable(getPeriod()).ifPresent(builder::setPeriod);
		ofNullable(getPeriodMultiplier()).ifPresent(builder::setPeriodMultiplier);
		return builder;
	}

	public static RelativeDatesBuilder builder() {
		return new RelativeDatesBuilder();
	}

	public static  class RelativeDatesBuilder extends RelativeDateOffsetBuilder {
	
		protected Integer periodSkip;
		protected DateRange.DateRangeBuilder scheduleBounds;
	
		public RelativeDatesBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends RelativeDates> metaData() {
			return metaData;
		} 
	
		
		public Integer getPeriodSkip() {
			return periodSkip;
		}
		
		
		public DateRange.DateRangeBuilder getScheduleBounds() {
			return scheduleBounds;
		}
		
		public DateRange.DateRangeBuilder getOrCreateScheduleBounds() {
			if (scheduleBounds!=null) {
				return scheduleBounds;
			}
			else return scheduleBounds = new DateRange.DateRangeBuilder();
		}
		
	
		public RelativeDatesBuilder setPeriodSkip(Integer periodSkip) {
			this.periodSkip = periodSkip;
			return this;
		}
		public RelativeDatesBuilder setScheduleBounds(DateRange scheduleBounds) {
			this.scheduleBounds = scheduleBounds.toBuilder();
			return this;
		}
		public RelativeDatesBuilder setScheduleBoundsBuilder(DateRange.DateRangeBuilder scheduleBounds) {
			this.scheduleBounds = scheduleBounds;
			return this;
		}
		@Override public RelativeDatesBuilder setAdjustedDate(Date adjustedDate) {
			this.adjustedDate = adjustedDate;
			return this;
		}
		@Override public RelativeDatesBuilder setBusinessCenters(BusinessCenters businessCenters) {
			this.businessCenters = businessCenters.toBuilder();
			return this;
		}
		@Override public RelativeDatesBuilder setBusinessCentersBuilder(BusinessCenters.BusinessCentersBuilder businessCenters) {
			this.businessCenters = businessCenters;
			return this;
		}
		@Override public RelativeDatesBuilder setBusinessCentersReference(ReferenceWithMetaBusinessCenters businessCentersReference) {
			this.businessCentersReference = businessCentersReference.toBuilder();
			return this;
		}
		@Override public RelativeDatesBuilder setBusinessCentersReferenceBuilder(ReferenceWithMetaBusinessCentersBuilder businessCentersReference) {
			this.businessCentersReference = businessCentersReference;
			return this;
		}
		@Override public RelativeDatesBuilder setBusinessCentersReferenceRef(BusinessCenters.BusinessCentersBuilder businessCentersReference) {
			return setBusinessCentersReference(ReferenceWithMetaBusinessCenters.builder().setValueBuilder(businessCentersReference).build());
		}
		@Override public RelativeDatesBuilder setBusinessCentersReferenceRef(BusinessCenters businessCentersReference) {
			return setBusinessCentersReferenceRef(BusinessCenters.builder());
		}
		@Override public RelativeDatesBuilder setBusinessDayConvention(BusinessDayConventionEnum businessDayConvention) {
			this.businessDayConvention = businessDayConvention;
			return this;
		}
		@Override public RelativeDatesBuilder setDateRelativeTo(BasicReferenceWithMetaDate dateRelativeTo) {
			this.dateRelativeTo = dateRelativeTo.toBuilder();
			return this;
		}
		@Override public RelativeDatesBuilder setDateRelativeToRef(Date dateRelativeTo) {
			return setDateRelativeTo(BasicReferenceWithMetaDate.builder().setValue(dateRelativeTo).build());
		}
		@Override public RelativeDatesBuilder setDayType(DayTypeEnum dayType) {
			this.dayType = dayType;
			return this;
		}
		@Override public RelativeDatesBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		@Override public RelativeDatesBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		@Override public RelativeDatesBuilder setPeriod(PeriodEnum period) {
			this.period = period;
			return this;
		}
		@Override public RelativeDatesBuilder setPeriodMultiplier(Integer periodMultiplier) {
			this.periodMultiplier = periodMultiplier;
			return this;
		}
		
		public RelativeDates build() {
			return new RelativeDates(this);
		}
	
		@Override
		public RelativeDatesBuilder prune() {
			super.prune();
			if (scheduleBounds!=null && !scheduleBounds.prune().hasData()) scheduleBounds = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (super.hasData()) return true;
			if (getPeriodSkip()!=null) return true;
			if (getScheduleBounds()!=null && getScheduleBounds().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
			super.process(path, processor);
		
			processor.processBasic(path.newSubPath("periodSkip"), Integer.class, periodSkip, this);
			
			processRosetta(path.newSubPath("scheduleBounds"), processor, DateRange.class, scheduleBounds);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
			if (!super.equals(o)) return false;
		
			RelativeDatesBuilder _that = (RelativeDatesBuilder) o;
		
			if (periodSkip != null ? !periodSkip.equals(_that.periodSkip) : _that.periodSkip != null) return false;
			if (scheduleBounds != null ? !scheduleBounds.equals(_that.scheduleBounds) : _that.scheduleBounds != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = super.hashCode();
			_result = 31 * _result + (periodSkip != null ? periodSkip.hashCode() : 0);
			_result = 31 * _result + (scheduleBounds != null ? scheduleBounds.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "RelativeDatesBuilder {" +
				"periodSkip=" + this.periodSkip + ", " +
				"scheduleBounds=" + this.scheduleBounds +
			'}' + " " + super.toString();
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		super.process(path, processor);
		
		processor.processBasic(path.newSubPath("periodSkip"), Integer.class, periodSkip, this);
		
		processRosetta(path.newSubPath("scheduleBounds"), processor, DateRange.class, scheduleBounds);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
		if (!super.equals(o)) return false;
	
		RelativeDates _that = (RelativeDates) o;
	
		if (periodSkip != null ? !periodSkip.equals(_that.periodSkip) : _that.periodSkip != null) return false;
		if (scheduleBounds != null ? !scheduleBounds.equals(_that.scheduleBounds) : _that.scheduleBounds != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = super.hashCode();
		_result = 31 * _result + (periodSkip != null ? periodSkip.hashCode() : 0);
		_result = 31 * _result + (scheduleBounds != null ? scheduleBounds.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "RelativeDates {" +
			"periodSkip=" + this.periodSkip + ", " +
			"scheduleBounds=" + this.scheduleBounds +
		'}' + " " + super.toString();
	}
}
