package org.isda.cdm;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.BondEquityModel;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.RelativePriceMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.math.BigDecimal;
		import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;

/**
 * Bond price relative to a benchmark, as in a convertible bond.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class RelativePrice extends RosettaModelObject {
	private final List<BondEquityModel> bondEquityModel;
	private final BigDecimal spread;
	private static RelativePriceMeta metaData = new RelativePriceMeta();
	
	RelativePrice(RelativePriceBuilder builder) {
		this.bondEquityModel = ofNullable(builder.getBondEquityModel()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.spread = builder.getSpread();
	}
	
	/**
	 * Bond equity model for convertible bonds.
	 */
	public final List<BondEquityModel> getBondEquityModel() {
		return bondEquityModel;
	}
	
	/**
	 * The spread to a benchmark.
	 */
	public final BigDecimal getSpread() {
		return spread;
	}
	
	
	@Override
	public RosettaMetaData<? extends RelativePrice> metaData() {
		return metaData;
	} 
	
	public RelativePriceBuilder toBuilder() {
		RelativePriceBuilder builder = new RelativePriceBuilder();
		ofNullable(getBondEquityModel()).ifPresent(bondEquityModel -> bondEquityModel.forEach(builder::addBondEquityModel));
		ofNullable(getSpread()).ifPresent(builder::setSpread);
		return builder;
	}

	public static RelativePriceBuilder builder() {
		return new RelativePriceBuilder();
	}

	public static  class RelativePriceBuilder extends RosettaModelObjectBuilder {
	
		protected List<BondEquityModel.BondEquityModelBuilder> bondEquityModel;
		protected BigDecimal spread;
	
		public RelativePriceBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends RelativePrice> metaData() {
			return metaData;
		} 
	
		
		public List<BondEquityModel.BondEquityModelBuilder> getBondEquityModel() {
			return bondEquityModel;
		}
		
		public BondEquityModel.BondEquityModelBuilder getOrCreateBondEquityModel(int index) {
			if (bondEquityModel==null) {
				this.bondEquityModel = new ArrayList<>();
			}
			return getIndex(bondEquityModel, index, ()->new BondEquityModel.BondEquityModelBuilder());
		}
		
		
		public BigDecimal getSpread() {
			return spread;
		}
		
	
		public RelativePriceBuilder addBondEquityModel(BondEquityModel bondEquityModel) {
			if(this.bondEquityModel == null){
				this.bondEquityModel = new ArrayList<>();
			}
			this.bondEquityModel.add(bondEquityModel.toBuilder());
			return this;
		}
		public RelativePriceBuilder addBondEquityModel(BondEquityModel bondEquityModel, int _idx) {
			if(this.bondEquityModel == null){
				this.bondEquityModel = new ArrayList<>();
			}
			getIndex(this.bondEquityModel, _idx, () -> bondEquityModel.toBuilder());
			this.bondEquityModel.set(_idx, bondEquityModel.toBuilder());
			return this;
		}
		public RelativePriceBuilder addBondEquityModel(List<BondEquityModel> bondEquityModels) {
			if(this.bondEquityModel == null){
				this.bondEquityModel = new ArrayList<>();
			}
			for (BondEquityModel toAdd : bondEquityModels) {
				this.bondEquityModel.add(toAdd.toBuilder());
			}
			return this;
		}
		public RelativePriceBuilder addBondEquityModelBuilder(BondEquityModel.BondEquityModelBuilder bondEquityModel) {
			if(this.bondEquityModel == null){
				this.bondEquityModel = new ArrayList<>();
			}
			this.bondEquityModel.add(bondEquityModel);
			return this;
		}
		
		public RelativePriceBuilder clearBondEquityModel() {
			this.bondEquityModel = null;
			return this;
		}
		public RelativePriceBuilder setSpread(BigDecimal spread) {
			this.spread = spread;
			return this;
		}
		
		public RelativePrice build() {
			return new RelativePrice(this);
		}
	
		@Override
		public RelativePriceBuilder prune() {
			if (bondEquityModel!=null) bondEquityModel = bondEquityModel.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getBondEquityModel()!=null && getBondEquityModel().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getSpread()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("spread"), BigDecimal.class, spread, this);
			
			processRosetta(path.newSubPath("bondEquityModel"), processor, BondEquityModel.class, bondEquityModel);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			RelativePriceBuilder _that = (RelativePriceBuilder) o;
		
			if (!ListEquals.listEquals(bondEquityModel, _that.bondEquityModel)) return false;
			if (spread != null ? !spread.equals(_that.spread) : _that.spread != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (bondEquityModel != null ? bondEquityModel.hashCode() : 0);
			_result = 31 * _result + (spread != null ? spread.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "RelativePriceBuilder {" +
				"bondEquityModel=" + this.bondEquityModel + ", " +
				"spread=" + this.spread +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("spread"), BigDecimal.class, spread, this);
		
		processRosetta(path.newSubPath("bondEquityModel"), processor, BondEquityModel.class, bondEquityModel);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		RelativePrice _that = (RelativePrice) o;
	
		if (!ListEquals.listEquals(bondEquityModel, _that.bondEquityModel)) return false;
		if (spread != null ? !spread.equals(_that.spread) : _that.spread != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (bondEquityModel != null ? bondEquityModel.hashCode() : 0);
		_result = 31 * _result + (spread != null ? spread.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "RelativePrice {" +
			"bondEquityModel=" + this.bondEquityModel + ", " +
			"spread=" + this.spread +
		'}';
	}
}
