package org.isda.cdm;

import org.isda.cdm.AdjustableDate;
import org.isda.cdm.BusinessDayAdjustments;
import org.isda.cdm.InitialFixingDate;
import org.isda.cdm.Offset;
import org.isda.cdm.RelativeDateOffset;
import org.isda.cdm.ResetFrequency;
import org.isda.cdm.ResetRelativeToEnum;
import org.isda.cdm.metafields.MetaFields;
import org.isda.cdm.metafields.ReferenceWithMetaCalculationPeriodDates;
import org.isda.cdm.metafields.ReferenceWithMetaCalculationPeriodDates.ReferenceWithMetaCalculationPeriodDatesBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.ResetDatesMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.stream.Collectors;
		import static java.util.Optional.ofNullable;

/**
 * A class defining the parameters used to generate the reset dates schedule and associated fixing dates. The reset dates are the dates on which the new index value (which is observed on the fixing date) is applied for each period and on which the interest rate hence begins to accrue.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="ResetDates", source="FpML_5_10")
@RosettaSynonym(value="ResetDates", source="CME_SubmissionIRS_1_0")
@RosettaSynonym(value="ResetDates", source="DTCC_11_0")
@RosettaSynonym(value="ResetDates", source="DTCC_9_0")
@RosettaSynonym(value="ResetDates", source="CME_ClearedConfirm_1_17")
public class ResetDates extends RosettaModelObject implements GlobalKey {
	private final ReferenceWithMetaCalculationPeriodDates calculationPeriodDatesReference;
	private final AdjustableDate finalFixingDate;
	private final RelativeDateOffset fixingDates;
	private final InitialFixingDate initialFixingDate;
	private final MetaFields meta;
	private final Offset rateCutOffDaysOffset;
	private final BusinessDayAdjustments resetDatesAdjustments;
	private final ResetFrequency resetFrequency;
	private final ResetRelativeToEnum resetRelativeTo;
	private static ResetDatesMeta metaData = new ResetDatesMeta();
	
	ResetDates(ResetDatesBuilder builder) {
		this.calculationPeriodDatesReference = ofNullable(builder.getCalculationPeriodDatesReference()).map(f->f.build()).orElse(null);
		this.finalFixingDate = ofNullable(builder.getFinalFixingDate()).map(f->f.build()).orElse(null);
		this.fixingDates = ofNullable(builder.getFixingDates()).map(f->f.build()).orElse(null);
		this.initialFixingDate = ofNullable(builder.getInitialFixingDate()).map(f->f.build()).orElse(null);
		this.meta = ofNullable(builder.getMeta()).map(f->f.build()).orElse(null);
		this.rateCutOffDaysOffset = ofNullable(builder.getRateCutOffDaysOffset()).map(f->f.build()).orElse(null);
		this.resetDatesAdjustments = ofNullable(builder.getResetDatesAdjustments()).map(f->f.build()).orElse(null);
		this.resetFrequency = ofNullable(builder.getResetFrequency()).map(f->f.build()).orElse(null);
		this.resetRelativeTo = builder.getResetRelativeTo();
	}
	
	/**
	 * A pointer style reference to the associated calculation period dates component defined elsewhere in the document.
	 */
	@RosettaSynonym(value="calculationPeriodDatesReference", source="FpML_5_10", path="resetDates")
	@RosettaSynonym(value="calculationPeriodDatesReference", source="CME_SubmissionIRS_1_0", path="resetDates")
	@RosettaSynonym(value="calculationPeriodDatesReference", source="DTCC_11_0", path="resetDates")
	@RosettaSynonym(value="calculationPeriodDatesReference", source="DTCC_9_0", path="resetDates")
	@RosettaSynonym(value="calculationPeriodDatesReference", source="CME_ClearedConfirm_1_17", path="resetDates")
	@RosettaSynonym(value="calculationPeriodDatesReference", source="FpML_5_10", path="interestLegCalculationPeriodDates.interestLegResetDates")
	public final ReferenceWithMetaCalculationPeriodDates getCalculationPeriodDatesReference() {
		return calculationPeriodDatesReference;
	}
	
	/**
	 * This attribute is not part of the FpML ResetDate, and has been added as part of the CDM to support the credit derivatives final fixing date.
	 */
	@RosettaSynonym(value="finalFixingDate", source="FpML_5_10", path="periodicPayment.floatingAmountCalculation")
	@RosettaSynonym(value="finalFixingDate", source="CME_SubmissionIRS_1_0", path="periodicPayment.floatingAmountCalculation")
	@RosettaSynonym(value="finalFixingDate", source="DTCC_11_0", path="periodicPayment.floatingAmountCalculation")
	@RosettaSynonym(value="finalFixingDate", source="DTCC_9_0", path="periodicPayment.floatingAmountCalculation")
	@RosettaSynonym(value="finalFixingDate", source="CME_ClearedConfirm_1_17", path="periodicPayment.floatingAmountCalculation")
	public final AdjustableDate getFinalFixingDate() {
		return finalFixingDate;
	}
	
	/**
	 * The fixing dates are the dates on which the index values are observed. The fixing dates are specified by reference to the reset date through business days offset and an associated set of financial business centers. Normally these offset calculation rules will be those specified in the ISDA definition for the relevant floating rate index (ISDA&#39;s Floating Rate Option). However, non-standard offset calculation rules may apply for a trade if mutually agreed by the principal parties to the transaction.
	 */
	@RosettaSynonym(value="fixingDates", source="FpML_5_10", path="resetDates")
	@RosettaSynonym(value="fixingDates", source="CME_SubmissionIRS_1_0", path="resetDates")
	@RosettaSynonym(value="fixingDates", source="DTCC_11_0", path="resetDates")
	@RosettaSynonym(value="fixingDates", source="DTCC_9_0", path="resetDates")
	@RosettaSynonym(value="fixingDates", source="CME_ClearedConfirm_1_17", path="resetDates")
	@RosettaSynonym(value="fixingDateOffset", source="FpML_5_10")
	@RosettaSynonym(value="fixingDateOffset", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="fixingDateOffset", source="DTCC_11_0")
	@RosettaSynonym(value="fixingDateOffset", source="DTCC_9_0")
	@RosettaSynonym(value="fixingDateOffset", source="CME_ClearedConfirm_1_17")
	public final RelativeDateOffset getFixingDates() {
		return fixingDates;
	}
	
	/**
	 * The initial fixing date.
	 */
	public final InitialFixingDate getInitialFixingDate() {
		return initialFixingDate;
	}
	
	public final MetaFields getMeta() {
		return meta;
	}
	
	/**
	 * Specifies the number of business days before the period end date when the rate cut-off date is assumed to apply. The financial business centers associated with determining the rate cut-off date are those specified in the reset dates adjustments. The rate cut-off number of days must be a negative integer (a value of zero would imply no rate cut off applies in which case the rateCutOffDaysOffset element should not be included). The relevant rate for each reset date in the period from, and including, a rate cut-off date to, but excluding, the next applicable period end date (or, in the case of the last calculation period, the termination date) will (solely for purposes of calculating the floating amount payable on the next applicable payment date) be deemed to be the relevant rate in effect on that rate cut-off date. For example, if rate cut-off days for a daily averaging deal is -2 business days, then the refix rate applied on (period end date - 2 days) will also be applied as the reset on (period end date - 1 day), i.e. the actual number of reset dates remains the same but from the rate cut-off date until the period end date, the same refix rate is applied. Note that in the case of several calculation periods contributing to a single payment, the rate cut-off is assumed only to apply to the final calculation period contributing to that payment. The day type associated with the offset must imply a business days offset.
	 */
	@RosettaSynonym(value="resetCutOffDaysOffset", source="FpML_5_10", path="resetDates")
	@RosettaSynonym(value="resetCutOffDaysOffset", source="CME_SubmissionIRS_1_0", path="resetDates")
	@RosettaSynonym(value="resetCutOffDaysOffset", source="DTCC_11_0", path="resetDates")
	@RosettaSynonym(value="resetCutOffDaysOffset", source="DTCC_9_0", path="resetDates")
	@RosettaSynonym(value="resetCutOffDaysOffset", source="CME_ClearedConfirm_1_17", path="resetDates")
	public final Offset getRateCutOffDaysOffset() {
		return rateCutOffDaysOffset;
	}
	
	/**
	 * The definition of the business day convention and financial business centers used for adjusting the reset date if it would otherwise fall on a day that is not a business day in the specified business center.
	 */
	@RosettaSynonym(value="resetDatesAdjustments", source="FpML_5_10", path="resetDates")
	@RosettaSynonym(value="resetDatesAdjustments", source="CME_SubmissionIRS_1_0", path="resetDates")
	@RosettaSynonym(value="resetDatesAdjustments", source="DTCC_11_0", path="resetDates")
	@RosettaSynonym(value="resetDatesAdjustments", source="DTCC_9_0", path="resetDates")
	@RosettaSynonym(value="resetDatesAdjustments", source="CME_ClearedConfirm_1_17", path="resetDates")
	public final BusinessDayAdjustments getResetDatesAdjustments() {
		return resetDatesAdjustments;
	}
	
	/**
	 * The frequency at which the reset dates occur. In the case of a weekly reset frequency, also specifies the day of the week that the reset occurs. If the reset frequency is greater than the calculation period frequency then this implies that more than one reset is established for each calculation period and some form of rate averaging is applicable.
	 */
	@RosettaSynonym(value="resetFrequency", source="FpML_5_10", path="resetDates")
	@RosettaSynonym(value="resetFrequency", source="FpML_5_10", path="interestLegCalculationPeriodDates.interestLegResetDates")
	@RosettaSynonym(value="resetFrequency", source="CME_SubmissionIRS_1_0", path="resetDates")
	@RosettaSynonym(value="resetFrequency", source="CME_SubmissionIRS_1_0", path="interestLegCalculationPeriodDates.interestLegResetDates")
	@RosettaSynonym(value="resetFrequency", source="DTCC_11_0", path="resetDates")
	@RosettaSynonym(value="resetFrequency", source="DTCC_11_0", path="interestLegCalculationPeriodDates.interestLegResetDates")
	@RosettaSynonym(value="resetFrequency", source="DTCC_9_0", path="resetDates")
	@RosettaSynonym(value="resetFrequency", source="DTCC_9_0", path="interestLegCalculationPeriodDates.interestLegResetDates")
	@RosettaSynonym(value="resetFrequency", source="CME_ClearedConfirm_1_17", path="resetDates")
	@RosettaSynonym(value="resetFrequency", source="CME_ClearedConfirm_1_17", path="interestLegCalculationPeriodDates.interestLegResetDates")
	public final ResetFrequency getResetFrequency() {
		return resetFrequency;
	}
	
	/**
	 * Specifies whether the reset dates are determined with respect to each adjusted calculation period start date or adjusted calculation period end date. If the reset frequency is specified as daily this element must not be included.
	 */
	@RosettaSynonym(value="resetRelativeTo", source="FpML_5_10", path="resetDates")
	@RosettaSynonym(value="resetRelativeTo", source="CME_SubmissionIRS_1_0", path="resetDates")
	@RosettaSynonym(value="resetRelativeTo", source="DTCC_11_0", path="resetDates")
	@RosettaSynonym(value="resetRelativeTo", source="DTCC_9_0", path="resetDates")
	@RosettaSynonym(value="resetRelativeTo", source="CME_ClearedConfirm_1_17", path="resetDates")
	@RosettaSynonym(value="resetRelativeTo", source="FpML_5_10", path="interestLegCalculationPeriodDates.interestLegResetDates")
	public final ResetRelativeToEnum getResetRelativeTo() {
		return resetRelativeTo;
	}
	
	
	@Override
	public RosettaMetaData<? extends ResetDates> metaData() {
		return metaData;
	} 
	
	public ResetDatesBuilder toBuilder() {
		ResetDatesBuilder builder = new ResetDatesBuilder();
		ofNullable(getCalculationPeriodDatesReference()).ifPresent(builder::setCalculationPeriodDatesReference);
		ofNullable(getFinalFixingDate()).ifPresent(builder::setFinalFixingDate);
		ofNullable(getFixingDates()).ifPresent(builder::setFixingDates);
		ofNullable(getInitialFixingDate()).ifPresent(builder::setInitialFixingDate);
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		ofNullable(getRateCutOffDaysOffset()).ifPresent(builder::setRateCutOffDaysOffset);
		ofNullable(getResetDatesAdjustments()).ifPresent(builder::setResetDatesAdjustments);
		ofNullable(getResetFrequency()).ifPresent(builder::setResetFrequency);
		ofNullable(getResetRelativeTo()).ifPresent(builder::setResetRelativeTo);
		return builder;
	}

	public static ResetDatesBuilder builder() {
		return new ResetDatesBuilder();
	}

	public static  class ResetDatesBuilder extends RosettaModelObjectBuilder implements GlobalKeyBuilder<ResetDatesBuilder> {
	
		protected ReferenceWithMetaCalculationPeriodDatesBuilder calculationPeriodDatesReference;
		protected AdjustableDate.AdjustableDateBuilder finalFixingDate;
		protected RelativeDateOffset.RelativeDateOffsetBuilder fixingDates;
		protected InitialFixingDate.InitialFixingDateBuilder initialFixingDate;
		protected MetaFields.MetaFieldsBuilder meta;
		protected Offset.OffsetBuilder rateCutOffDaysOffset;
		protected BusinessDayAdjustments.BusinessDayAdjustmentsBuilder resetDatesAdjustments;
		protected ResetFrequency.ResetFrequencyBuilder resetFrequency;
		protected ResetRelativeToEnum resetRelativeTo;
	
		public ResetDatesBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends ResetDates> metaData() {
			return metaData;
		} 
	
		
		public ReferenceWithMetaCalculationPeriodDatesBuilder getCalculationPeriodDatesReference() {
			return calculationPeriodDatesReference;
		}
		
		public ReferenceWithMetaCalculationPeriodDatesBuilder getOrCreateCalculationPeriodDatesReference() {
			if (calculationPeriodDatesReference!=null) {
				return calculationPeriodDatesReference;
			}
			else return calculationPeriodDatesReference = new ReferenceWithMetaCalculationPeriodDatesBuilder();
		}
		
		
		public AdjustableDate.AdjustableDateBuilder getFinalFixingDate() {
			return finalFixingDate;
		}
		
		public AdjustableDate.AdjustableDateBuilder getOrCreateFinalFixingDate() {
			if (finalFixingDate!=null) {
				return finalFixingDate;
			}
			else return finalFixingDate = new AdjustableDate.AdjustableDateBuilder();
		}
		
		
		public RelativeDateOffset.RelativeDateOffsetBuilder getFixingDates() {
			return fixingDates;
		}
		
		public RelativeDateOffset.RelativeDateOffsetBuilder getOrCreateFixingDates() {
			if (fixingDates!=null) {
				return fixingDates;
			}
			else return fixingDates = new RelativeDateOffset.RelativeDateOffsetBuilder();
		}
		
		
		public InitialFixingDate.InitialFixingDateBuilder getInitialFixingDate() {
			return initialFixingDate;
		}
		
		public InitialFixingDate.InitialFixingDateBuilder getOrCreateInitialFixingDate() {
			if (initialFixingDate!=null) {
				return initialFixingDate;
			}
			else return initialFixingDate = new InitialFixingDate.InitialFixingDateBuilder();
		}
		
		
		public MetaFields.MetaFieldsBuilder getMeta() {
			return meta;
		}
		
		public MetaFields.MetaFieldsBuilder getOrCreateMeta() {
			if (meta!=null) {
				return meta;
			}
			else return meta = new MetaFields.MetaFieldsBuilder();
		}
		
		
		public Offset.OffsetBuilder getRateCutOffDaysOffset() {
			return rateCutOffDaysOffset;
		}
		
		public Offset.OffsetBuilder getOrCreateRateCutOffDaysOffset() {
			if (rateCutOffDaysOffset!=null) {
				return rateCutOffDaysOffset;
			}
			else return rateCutOffDaysOffset = new Offset.OffsetBuilder();
		}
		
		
		public BusinessDayAdjustments.BusinessDayAdjustmentsBuilder getResetDatesAdjustments() {
			return resetDatesAdjustments;
		}
		
		public BusinessDayAdjustments.BusinessDayAdjustmentsBuilder getOrCreateResetDatesAdjustments() {
			if (resetDatesAdjustments!=null) {
				return resetDatesAdjustments;
			}
			else return resetDatesAdjustments = new BusinessDayAdjustments.BusinessDayAdjustmentsBuilder();
		}
		
		
		public ResetFrequency.ResetFrequencyBuilder getResetFrequency() {
			return resetFrequency;
		}
		
		public ResetFrequency.ResetFrequencyBuilder getOrCreateResetFrequency() {
			if (resetFrequency!=null) {
				return resetFrequency;
			}
			else return resetFrequency = new ResetFrequency.ResetFrequencyBuilder();
		}
		
		
		public ResetRelativeToEnum getResetRelativeTo() {
			return resetRelativeTo;
		}
		
	
		public ResetDatesBuilder setCalculationPeriodDatesReference(ReferenceWithMetaCalculationPeriodDates calculationPeriodDatesReference) {
			this.calculationPeriodDatesReference = calculationPeriodDatesReference.toBuilder();
			return this;
		}
		public ResetDatesBuilder setCalculationPeriodDatesReferenceBuilder(ReferenceWithMetaCalculationPeriodDatesBuilder calculationPeriodDatesReference) {
			this.calculationPeriodDatesReference = calculationPeriodDatesReference;
			return this;
		}
		public ResetDatesBuilder setCalculationPeriodDatesReferenceRef(CalculationPeriodDates.CalculationPeriodDatesBuilder calculationPeriodDatesReference) {
			return setCalculationPeriodDatesReference(ReferenceWithMetaCalculationPeriodDates.builder().setValueBuilder(calculationPeriodDatesReference).build());
		}
		public ResetDatesBuilder setCalculationPeriodDatesReferenceRef(CalculationPeriodDates calculationPeriodDatesReference) {
			return setCalculationPeriodDatesReferenceRef(CalculationPeriodDates.builder());
		}
		public ResetDatesBuilder setFinalFixingDate(AdjustableDate finalFixingDate) {
			this.finalFixingDate = finalFixingDate.toBuilder();
			return this;
		}
		public ResetDatesBuilder setFinalFixingDateBuilder(AdjustableDate.AdjustableDateBuilder finalFixingDate) {
			this.finalFixingDate = finalFixingDate;
			return this;
		}
		public ResetDatesBuilder setFixingDates(RelativeDateOffset fixingDates) {
			this.fixingDates = fixingDates.toBuilder();
			return this;
		}
		public ResetDatesBuilder setFixingDatesBuilder(RelativeDateOffset.RelativeDateOffsetBuilder fixingDates) {
			this.fixingDates = fixingDates;
			return this;
		}
		public ResetDatesBuilder setInitialFixingDate(InitialFixingDate initialFixingDate) {
			this.initialFixingDate = initialFixingDate.toBuilder();
			return this;
		}
		public ResetDatesBuilder setInitialFixingDateBuilder(InitialFixingDate.InitialFixingDateBuilder initialFixingDate) {
			this.initialFixingDate = initialFixingDate;
			return this;
		}
		public ResetDatesBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		public ResetDatesBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		public ResetDatesBuilder setRateCutOffDaysOffset(Offset rateCutOffDaysOffset) {
			this.rateCutOffDaysOffset = rateCutOffDaysOffset.toBuilder();
			return this;
		}
		public ResetDatesBuilder setRateCutOffDaysOffsetBuilder(Offset.OffsetBuilder rateCutOffDaysOffset) {
			this.rateCutOffDaysOffset = rateCutOffDaysOffset;
			return this;
		}
		public ResetDatesBuilder setResetDatesAdjustments(BusinessDayAdjustments resetDatesAdjustments) {
			this.resetDatesAdjustments = resetDatesAdjustments.toBuilder();
			return this;
		}
		public ResetDatesBuilder setResetDatesAdjustmentsBuilder(BusinessDayAdjustments.BusinessDayAdjustmentsBuilder resetDatesAdjustments) {
			this.resetDatesAdjustments = resetDatesAdjustments;
			return this;
		}
		public ResetDatesBuilder setResetFrequency(ResetFrequency resetFrequency) {
			this.resetFrequency = resetFrequency.toBuilder();
			return this;
		}
		public ResetDatesBuilder setResetFrequencyBuilder(ResetFrequency.ResetFrequencyBuilder resetFrequency) {
			this.resetFrequency = resetFrequency;
			return this;
		}
		public ResetDatesBuilder setResetRelativeTo(ResetRelativeToEnum resetRelativeTo) {
			this.resetRelativeTo = resetRelativeTo;
			return this;
		}
		
		public ResetDates build() {
			return new ResetDates(this);
		}
	
		@Override
		public ResetDatesBuilder prune() {
			if (calculationPeriodDatesReference!=null && !calculationPeriodDatesReference.prune().hasData()) calculationPeriodDatesReference = null;
			if (finalFixingDate!=null && !finalFixingDate.prune().hasData()) finalFixingDate = null;
			if (fixingDates!=null && !fixingDates.prune().hasData()) fixingDates = null;
			if (initialFixingDate!=null && !initialFixingDate.prune().hasData()) initialFixingDate = null;
			if (meta!=null && !meta.prune().hasData()) meta = null;
			if (rateCutOffDaysOffset!=null && !rateCutOffDaysOffset.prune().hasData()) rateCutOffDaysOffset = null;
			if (resetDatesAdjustments!=null && !resetDatesAdjustments.prune().hasData()) resetDatesAdjustments = null;
			if (resetFrequency!=null && !resetFrequency.prune().hasData()) resetFrequency = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getCalculationPeriodDatesReference()!=null && getCalculationPeriodDatesReference().hasData()) return true;
			if (getFinalFixingDate()!=null && getFinalFixingDate().hasData()) return true;
			if (getFixingDates()!=null && getFixingDates().hasData()) return true;
			if (getInitialFixingDate()!=null && getInitialFixingDate().hasData()) return true;
			if (getMeta()!=null && getMeta().hasData()) return true;
			if (getRateCutOffDaysOffset()!=null && getRateCutOffDaysOffset().hasData()) return true;
			if (getResetDatesAdjustments()!=null && getResetDatesAdjustments().hasData()) return true;
			if (getResetFrequency()!=null && getResetFrequency().hasData()) return true;
			if (getResetRelativeTo()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("resetRelativeTo"), ResetRelativeToEnum.class, resetRelativeTo, this);
			
			processRosetta(path.newSubPath("calculationPeriodDatesReference"), processor, ReferenceWithMetaCalculationPeriodDates.class, calculationPeriodDatesReference);
			processRosetta(path.newSubPath("finalFixingDate"), processor, AdjustableDate.class, finalFixingDate);
			processRosetta(path.newSubPath("fixingDates"), processor, RelativeDateOffset.class, fixingDates);
			processRosetta(path.newSubPath("initialFixingDate"), processor, InitialFixingDate.class, initialFixingDate);
			processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
			processRosetta(path.newSubPath("rateCutOffDaysOffset"), processor, Offset.class, rateCutOffDaysOffset);
			processRosetta(path.newSubPath("resetDatesAdjustments"), processor, BusinessDayAdjustments.class, resetDatesAdjustments);
			processRosetta(path.newSubPath("resetFrequency"), processor, ResetFrequency.class, resetFrequency);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			ResetDatesBuilder _that = (ResetDatesBuilder) o;
		
			if (calculationPeriodDatesReference != null ? !calculationPeriodDatesReference.equals(_that.calculationPeriodDatesReference) : _that.calculationPeriodDatesReference != null) return false;
			if (finalFixingDate != null ? !finalFixingDate.equals(_that.finalFixingDate) : _that.finalFixingDate != null) return false;
			if (fixingDates != null ? !fixingDates.equals(_that.fixingDates) : _that.fixingDates != null) return false;
			if (initialFixingDate != null ? !initialFixingDate.equals(_that.initialFixingDate) : _that.initialFixingDate != null) return false;
			if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
			if (rateCutOffDaysOffset != null ? !rateCutOffDaysOffset.equals(_that.rateCutOffDaysOffset) : _that.rateCutOffDaysOffset != null) return false;
			if (resetDatesAdjustments != null ? !resetDatesAdjustments.equals(_that.resetDatesAdjustments) : _that.resetDatesAdjustments != null) return false;
			if (resetFrequency != null ? !resetFrequency.equals(_that.resetFrequency) : _that.resetFrequency != null) return false;
			if (resetRelativeTo != null ? !resetRelativeTo.equals(_that.resetRelativeTo) : _that.resetRelativeTo != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (calculationPeriodDatesReference != null ? calculationPeriodDatesReference.hashCode() : 0);
			_result = 31 * _result + (finalFixingDate != null ? finalFixingDate.hashCode() : 0);
			_result = 31 * _result + (fixingDates != null ? fixingDates.hashCode() : 0);
			_result = 31 * _result + (initialFixingDate != null ? initialFixingDate.hashCode() : 0);
			_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
			_result = 31 * _result + (rateCutOffDaysOffset != null ? rateCutOffDaysOffset.hashCode() : 0);
			_result = 31 * _result + (resetDatesAdjustments != null ? resetDatesAdjustments.hashCode() : 0);
			_result = 31 * _result + (resetFrequency != null ? resetFrequency.hashCode() : 0);
			_result = 31 * _result + (resetRelativeTo != null ? resetRelativeTo.getClass().getName().hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "ResetDatesBuilder {" +
				"calculationPeriodDatesReference=" + this.calculationPeriodDatesReference + ", " +
				"finalFixingDate=" + this.finalFixingDate + ", " +
				"fixingDates=" + this.fixingDates + ", " +
				"initialFixingDate=" + this.initialFixingDate + ", " +
				"meta=" + this.meta + ", " +
				"rateCutOffDaysOffset=" + this.rateCutOffDaysOffset + ", " +
				"resetDatesAdjustments=" + this.resetDatesAdjustments + ", " +
				"resetFrequency=" + this.resetFrequency + ", " +
				"resetRelativeTo=" + this.resetRelativeTo +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("resetRelativeTo"), ResetRelativeToEnum.class, resetRelativeTo, this);
		
		processRosetta(path.newSubPath("calculationPeriodDatesReference"), processor, ReferenceWithMetaCalculationPeriodDates.class, calculationPeriodDatesReference);
		processRosetta(path.newSubPath("finalFixingDate"), processor, AdjustableDate.class, finalFixingDate);
		processRosetta(path.newSubPath("fixingDates"), processor, RelativeDateOffset.class, fixingDates);
		processRosetta(path.newSubPath("initialFixingDate"), processor, InitialFixingDate.class, initialFixingDate);
		processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
		processRosetta(path.newSubPath("rateCutOffDaysOffset"), processor, Offset.class, rateCutOffDaysOffset);
		processRosetta(path.newSubPath("resetDatesAdjustments"), processor, BusinessDayAdjustments.class, resetDatesAdjustments);
		processRosetta(path.newSubPath("resetFrequency"), processor, ResetFrequency.class, resetFrequency);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		ResetDates _that = (ResetDates) o;
	
		if (calculationPeriodDatesReference != null ? !calculationPeriodDatesReference.equals(_that.calculationPeriodDatesReference) : _that.calculationPeriodDatesReference != null) return false;
		if (finalFixingDate != null ? !finalFixingDate.equals(_that.finalFixingDate) : _that.finalFixingDate != null) return false;
		if (fixingDates != null ? !fixingDates.equals(_that.fixingDates) : _that.fixingDates != null) return false;
		if (initialFixingDate != null ? !initialFixingDate.equals(_that.initialFixingDate) : _that.initialFixingDate != null) return false;
		if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
		if (rateCutOffDaysOffset != null ? !rateCutOffDaysOffset.equals(_that.rateCutOffDaysOffset) : _that.rateCutOffDaysOffset != null) return false;
		if (resetDatesAdjustments != null ? !resetDatesAdjustments.equals(_that.resetDatesAdjustments) : _that.resetDatesAdjustments != null) return false;
		if (resetFrequency != null ? !resetFrequency.equals(_that.resetFrequency) : _that.resetFrequency != null) return false;
		if (resetRelativeTo != null ? !resetRelativeTo.equals(_that.resetRelativeTo) : _that.resetRelativeTo != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (calculationPeriodDatesReference != null ? calculationPeriodDatesReference.hashCode() : 0);
		_result = 31 * _result + (finalFixingDate != null ? finalFixingDate.hashCode() : 0);
		_result = 31 * _result + (fixingDates != null ? fixingDates.hashCode() : 0);
		_result = 31 * _result + (initialFixingDate != null ? initialFixingDate.hashCode() : 0);
		_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
		_result = 31 * _result + (rateCutOffDaysOffset != null ? rateCutOffDaysOffset.hashCode() : 0);
		_result = 31 * _result + (resetDatesAdjustments != null ? resetDatesAdjustments.hashCode() : 0);
		_result = 31 * _result + (resetFrequency != null ? resetFrequency.hashCode() : 0);
		_result = 31 * _result + (resetRelativeTo != null ? resetRelativeTo.getClass().getName().hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "ResetDates {" +
			"calculationPeriodDatesReference=" + this.calculationPeriodDatesReference + ", " +
			"finalFixingDate=" + this.finalFixingDate + ", " +
			"fixingDates=" + this.fixingDates + ", " +
			"initialFixingDate=" + this.initialFixingDate + ", " +
			"meta=" + this.meta + ", " +
			"rateCutOffDaysOffset=" + this.rateCutOffDaysOffset + ", " +
			"resetDatesAdjustments=" + this.resetDatesAdjustments + ", " +
			"resetFrequency=" + this.resetFrequency + ", " +
			"resetRelativeTo=" + this.resetRelativeTo +
		'}';
	}
}
