package org.isda.cdm;

import org.isda.cdm.WeeklyRollConventionEnum;


import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.ResetFrequencyMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class defining the reset frequency. In the case of a weekly reset, also specifies the day of the week that the reset occurs. If the reset frequency is greater than the calculation period frequency the this implies that more or more reset dates is established for each calculation period and some form of rate averaging is applicable. The specific averaging method of calculation is specified in FloatingRateCalculation. In case the reset frequency is of value T (term), the period is defined by the swap/swapStream/calculationPerioDates/effectiveDate and the swap/swapStream/calculationPerioDates/terminationDate.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="ResetFrequency", source="FpML_5_10")
public class ResetFrequency extends Frequency {
	private final WeeklyRollConventionEnum weeklyRollConvention;
	private static ResetFrequencyMeta metaData = new ResetFrequencyMeta();
	
	ResetFrequency(ResetFrequencyBuilder builder) {
		super(builder);
		this.weeklyRollConvention = builder.getWeeklyRollConvention();
	}
	
	/**
	 * The day of the week on which a weekly reset date occurs. This element must be included if the reset frequency is defined as weekly and not otherwise.
	 */
	@RosettaSynonym(value="weeklyRollConvention", source="FpML_5_10")
	@RosettaSynonym(value="weeklyRollConvention", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="weeklyRollConvention", source="DTCC_11_0")
	@RosettaSynonym(value="weeklyRollConvention", source="DTCC_9_0")
	@RosettaSynonym(value="weeklyRollConvention", source="CME_ClearedConfirm_1_17")
	public final WeeklyRollConventionEnum getWeeklyRollConvention() {
		return weeklyRollConvention;
	}
	
	
	@Override
	public RosettaMetaData<? extends ResetFrequency> metaData() {
		return metaData;
	} 
	
	public ResetFrequencyBuilder toBuilder() {
		ResetFrequencyBuilder builder = new ResetFrequencyBuilder();
		ofNullable(getWeeklyRollConvention()).ifPresent(builder::setWeeklyRollConvention);
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		ofNullable(getPeriod()).ifPresent(builder::setPeriod);
		ofNullable(getPeriodMultiplier()).ifPresent(builder::setPeriodMultiplier);
		return builder;
	}

	public static ResetFrequencyBuilder builder() {
		return new ResetFrequencyBuilder();
	}

	public static  class ResetFrequencyBuilder extends FrequencyBuilder {
	
		protected WeeklyRollConventionEnum weeklyRollConvention;
	
		public ResetFrequencyBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends ResetFrequency> metaData() {
			return metaData;
		} 
	
		
		public WeeklyRollConventionEnum getWeeklyRollConvention() {
			return weeklyRollConvention;
		}
		
	
		public ResetFrequencyBuilder setWeeklyRollConvention(WeeklyRollConventionEnum weeklyRollConvention) {
			this.weeklyRollConvention = weeklyRollConvention;
			return this;
		}
		@Override public ResetFrequencyBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		@Override public ResetFrequencyBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		@Override public ResetFrequencyBuilder setPeriod(PeriodExtendedEnum period) {
			this.period = period;
			return this;
		}
		@Override public ResetFrequencyBuilder setPeriodMultiplier(Integer periodMultiplier) {
			this.periodMultiplier = periodMultiplier;
			return this;
		}
		
		public ResetFrequency build() {
			return new ResetFrequency(this);
		}
	
		@Override
		public ResetFrequencyBuilder prune() {
			super.prune();
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (super.hasData()) return true;
			if (getWeeklyRollConvention()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
			super.process(path, processor);
		
			processor.processBasic(path.newSubPath("weeklyRollConvention"), WeeklyRollConventionEnum.class, weeklyRollConvention, this);
			
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
			if (!super.equals(o)) return false;
		
			ResetFrequencyBuilder _that = (ResetFrequencyBuilder) o;
		
			if (weeklyRollConvention != null ? !weeklyRollConvention.equals(_that.weeklyRollConvention) : _that.weeklyRollConvention != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = super.hashCode();
			_result = 31 * _result + (weeklyRollConvention != null ? weeklyRollConvention.getClass().getName().hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "ResetFrequencyBuilder {" +
				"weeklyRollConvention=" + this.weeklyRollConvention +
			'}' + " " + super.toString();
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		super.process(path, processor);
		
		processor.processBasic(path.newSubPath("weeklyRollConvention"), WeeklyRollConventionEnum.class, weeklyRollConvention, this);
		
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
		if (!super.equals(o)) return false;
	
		ResetFrequency _that = (ResetFrequency) o;
	
		if (weeklyRollConvention != null ? !weeklyRollConvention.equals(_that.weeklyRollConvention) : _that.weeklyRollConvention != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = super.hashCode();
		_result = 31 * _result + (weeklyRollConvention != null ? weeklyRollConvention.getClass().getName().hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "ResetFrequency {" +
			"weeklyRollConvention=" + this.weeklyRollConvention +
		'}' + " " + super.toString();
	}
}
