package org.isda.cdm;

import org.isda.cdm.ContractState;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.ResetPrimitiveMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * The primitive event to represent a reset.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class ResetPrimitive extends RosettaModelObject {
	private final ContractState after;
	private final ContractState before;
	private static ResetPrimitiveMeta metaData = new ResetPrimitiveMeta();
	
	ResetPrimitive(ResetPrimitiveBuilder builder) {
		this.after = ofNullable(builder.getAfter()).map(f->f.build()).orElse(null);
		this.before = ofNullable(builder.getBefore()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * Contract state after the reset, that embeds the reset value as an updated field on the contract state.
	 */
	@RosettaSynonym(value="ignore", source="Rosetta_Workbench")
	public final ContractState getAfter() {
		return after;
	}
	
	/**
	 * Contract state before the reset, as per previous events processed on the contract.
	 */
	@RosettaSynonym(value="ignore", source="Rosetta_Workbench")
	public final ContractState getBefore() {
		return before;
	}
	
	
	@Override
	public RosettaMetaData<? extends ResetPrimitive> metaData() {
		return metaData;
	} 
	
	public ResetPrimitiveBuilder toBuilder() {
		ResetPrimitiveBuilder builder = new ResetPrimitiveBuilder();
		ofNullable(getAfter()).ifPresent(builder::setAfter);
		ofNullable(getBefore()).ifPresent(builder::setBefore);
		return builder;
	}

	public static ResetPrimitiveBuilder builder() {
		return new ResetPrimitiveBuilder();
	}

	public static  class ResetPrimitiveBuilder extends RosettaModelObjectBuilder {
	
		protected ContractState.ContractStateBuilder after;
		protected ContractState.ContractStateBuilder before;
	
		public ResetPrimitiveBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends ResetPrimitive> metaData() {
			return metaData;
		} 
	
		
		public ContractState.ContractStateBuilder getAfter() {
			return after;
		}
		
		public ContractState.ContractStateBuilder getOrCreateAfter() {
			if (after!=null) {
				return after;
			}
			else return after = new ContractState.ContractStateBuilder();
		}
		
		
		public ContractState.ContractStateBuilder getBefore() {
			return before;
		}
		
		public ContractState.ContractStateBuilder getOrCreateBefore() {
			if (before!=null) {
				return before;
			}
			else return before = new ContractState.ContractStateBuilder();
		}
		
	
		public ResetPrimitiveBuilder setAfter(ContractState after) {
			this.after = after.toBuilder();
			return this;
		}
		public ResetPrimitiveBuilder setAfterBuilder(ContractState.ContractStateBuilder after) {
			this.after = after;
			return this;
		}
		public ResetPrimitiveBuilder setBefore(ContractState before) {
			this.before = before.toBuilder();
			return this;
		}
		public ResetPrimitiveBuilder setBeforeBuilder(ContractState.ContractStateBuilder before) {
			this.before = before;
			return this;
		}
		
		public ResetPrimitive build() {
			return new ResetPrimitive(this);
		}
	
		@Override
		public ResetPrimitiveBuilder prune() {
			if (after!=null && !after.prune().hasData()) after = null;
			if (before!=null && !before.prune().hasData()) before = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getAfter()!=null && getAfter().hasData()) return true;
			if (getBefore()!=null && getBefore().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("after"), processor, ContractState.class, after);
			processRosetta(path.newSubPath("before"), processor, ContractState.class, before);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			ResetPrimitiveBuilder _that = (ResetPrimitiveBuilder) o;
		
			if (after != null ? !after.equals(_that.after) : _that.after != null) return false;
			if (before != null ? !before.equals(_that.before) : _that.before != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (after != null ? after.hashCode() : 0);
			_result = 31 * _result + (before != null ? before.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "ResetPrimitiveBuilder {" +
				"after=" + this.after + ", " +
				"before=" + this.before +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("after"), processor, ContractState.class, after);
		processRosetta(path.newSubPath("before"), processor, ContractState.class, before);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		ResetPrimitive _that = (ResetPrimitive) o;
	
		if (after != null ? !after.equals(_that.after) : _that.after != null) return false;
		if (before != null ? !before.equals(_that.before) : _that.before != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (after != null ? after.hashCode() : 0);
		_result = 31 * _result + (before != null ? before.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "ResetPrimitive {" +
			"after=" + this.after + ", " +
			"before=" + this.before +
		'}';
	}
}
