package org.isda.cdm;

import java.lang.Boolean;
import org.isda.cdm.NonNegativeQuantitySchedule;
import org.isda.cdm.QuantityMultiplier;
import org.isda.cdm.QuantityNotationEnum;
import org.isda.cdm.metafields.MetaFields;
import org.isda.cdm.metafields.ReferenceWithMetaResolvablePayoutQuantity;
import org.isda.cdm.metafields.ReferenceWithMetaResolvablePayoutQuantity.ReferenceWithMetaResolvablePayoutQuantityBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.ResolvablePayoutQuantityMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;
		import java.util.stream.Collectors;

/**
 * Generic class to specify the quantity for different payout legs in a contractual product, when that quantity can vary across payout legs or across time. A resolvable quantity can always be resolved into one number based on: (i) the quantity notation that must be specified alongside the contractual product description and (ii) a given date, where applicable. In addition to the base case where quantity is directly specified as a number as part of the quantity notation, the use cases are: (i) quantity based on some pre-defined schedule (eg amortising notional), (ii) quantity based on some pre-defined events (eg resetting cross-currency notional), or quantity set as reference to another quantity (eg equity notional as no. securities x price).
 * 
 * @version 2.5.4
 */
@RosettaClass
public class ResolvablePayoutQuantity extends RosettaModelObject implements GlobalKey {
	private final MetaFields meta;
	private final QuantityMultiplier quantityMultiplier;
	private final QuantityNotationEnum quantityNotationTag;
	private final ReferenceWithMetaResolvablePayoutQuantity quantityReference;
	private final NonNegativeQuantitySchedule quantitySchedule;
	private final Boolean reset;
	private static ResolvablePayoutQuantityMeta metaData = new ResolvablePayoutQuantityMeta();
	
	ResolvablePayoutQuantity(ResolvablePayoutQuantityBuilder builder) {
		this.meta = ofNullable(builder.getMeta()).map(f->f.build()).orElse(null);
		this.quantityMultiplier = ofNullable(builder.getQuantityMultiplier()).map(f->f.build()).orElse(null);
		this.quantityNotationTag = builder.getQuantityNotationTag();
		this.quantityReference = ofNullable(builder.getQuantityReference()).map(f->f.build()).orElse(null);
		this.quantitySchedule = ofNullable(builder.getQuantitySchedule()).map(f->f.build()).orElse(null);
		this.reset = builder.getReset();
	}
	
	public final MetaFields getMeta() {
		return meta;
	}
	
	/**
	 * Quantity multiplier is specified on top of a reference quantity and is used as a multiplying factor when resolving the quantity. A quantity multiplier can only exist when the resolvable quantity specifies a reference quantity.
	 */
	public final QuantityMultiplier getQuantityMultiplier() {
		return quantityMultiplier;
	}
	
	/**
	 * Notation tag to be used as a key to fetch the payout quantity from the appropriate quantity notation, that is provided alongside the contractual product. For instance: if this quantity notation tag is &#39;Notional&#39;, the actual quantity will have to be fetched from the quantity notation labelled as &#39;Notional&#39;.
	 */
	@RosettaSynonym(value="", source="FpML_5_10")
	public final QuantityNotationEnum getQuantityNotationTag() {
		return quantityNotationTag;
	}
	
	/**
	 * Reference quantity when resolvable quantity is defined as relative to another (resolvable) quantity. A resolvable quantity needs to contain either an absolute quantity or a reference to another (resolvable) quantity. This requirement is captured by a choice rule on the class.
	 */
	@RosettaSynonym(value="relativeNotionalAmount", source="FpML_5_10")
	public final ReferenceWithMetaResolvablePayoutQuantity getQuantityReference() {
		return quantityReference;
	}
	
	/**
	 * Quantity specified as an absolute number, (possibly) with some step schedule, (possibly) with some unit, such as the currency in the case of a notional. There needs to be at least 1 such absolutely defined quantity across payout legs of a product, to define an anchor that other payout quantities can refer to. This is enforced by a data rule on the Payout class
	 */
	public final NonNegativeQuantitySchedule getQuantitySchedule() {
		return quantitySchedule;
	}
	
	/**
	 * Whether the quantity is resettable
	 */
	@RosettaSynonym(value="notionalReset", source="FpML_5_10")
	public final Boolean getReset() {
		return reset;
	}
	
	
	@Override
	public RosettaMetaData<? extends ResolvablePayoutQuantity> metaData() {
		return metaData;
	} 
	
	public ResolvablePayoutQuantityBuilder toBuilder() {
		ResolvablePayoutQuantityBuilder builder = new ResolvablePayoutQuantityBuilder();
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		ofNullable(getQuantityMultiplier()).ifPresent(builder::setQuantityMultiplier);
		ofNullable(getQuantityNotationTag()).ifPresent(builder::setQuantityNotationTag);
		ofNullable(getQuantityReference()).ifPresent(builder::setQuantityReference);
		ofNullable(getQuantitySchedule()).ifPresent(builder::setQuantitySchedule);
		ofNullable(getReset()).ifPresent(builder::setReset);
		return builder;
	}

	public static ResolvablePayoutQuantityBuilder builder() {
		return new ResolvablePayoutQuantityBuilder();
	}

	public static  class ResolvablePayoutQuantityBuilder extends RosettaModelObjectBuilder implements GlobalKeyBuilder<ResolvablePayoutQuantityBuilder> {
	
		protected MetaFields.MetaFieldsBuilder meta;
		protected QuantityMultiplier.QuantityMultiplierBuilder quantityMultiplier;
		protected QuantityNotationEnum quantityNotationTag;
		protected ReferenceWithMetaResolvablePayoutQuantityBuilder quantityReference;
		protected NonNegativeQuantitySchedule.NonNegativeQuantityScheduleBuilder quantitySchedule;
		protected Boolean reset;
	
		public ResolvablePayoutQuantityBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends ResolvablePayoutQuantity> metaData() {
			return metaData;
		} 
	
		
		public MetaFields.MetaFieldsBuilder getMeta() {
			return meta;
		}
		
		public MetaFields.MetaFieldsBuilder getOrCreateMeta() {
			if (meta!=null) {
				return meta;
			}
			else return meta = new MetaFields.MetaFieldsBuilder();
		}
		
		
		public QuantityMultiplier.QuantityMultiplierBuilder getQuantityMultiplier() {
			return quantityMultiplier;
		}
		
		public QuantityMultiplier.QuantityMultiplierBuilder getOrCreateQuantityMultiplier() {
			if (quantityMultiplier!=null) {
				return quantityMultiplier;
			}
			else return quantityMultiplier = new QuantityMultiplier.QuantityMultiplierBuilder();
		}
		
		
		public QuantityNotationEnum getQuantityNotationTag() {
			return quantityNotationTag;
		}
		
		
		public ReferenceWithMetaResolvablePayoutQuantityBuilder getQuantityReference() {
			return quantityReference;
		}
		
		public ReferenceWithMetaResolvablePayoutQuantityBuilder getOrCreateQuantityReference() {
			if (quantityReference!=null) {
				return quantityReference;
			}
			else return quantityReference = new ReferenceWithMetaResolvablePayoutQuantityBuilder();
		}
		
		
		public NonNegativeQuantitySchedule.NonNegativeQuantityScheduleBuilder getQuantitySchedule() {
			return quantitySchedule;
		}
		
		public NonNegativeQuantitySchedule.NonNegativeQuantityScheduleBuilder getOrCreateQuantitySchedule() {
			if (quantitySchedule!=null) {
				return quantitySchedule;
			}
			else return quantitySchedule = new NonNegativeQuantitySchedule.NonNegativeQuantityScheduleBuilder();
		}
		
		
		public Boolean getReset() {
			return reset;
		}
		
	
		public ResolvablePayoutQuantityBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		public ResolvablePayoutQuantityBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		public ResolvablePayoutQuantityBuilder setQuantityMultiplier(QuantityMultiplier quantityMultiplier) {
			this.quantityMultiplier = quantityMultiplier.toBuilder();
			return this;
		}
		public ResolvablePayoutQuantityBuilder setQuantityMultiplierBuilder(QuantityMultiplier.QuantityMultiplierBuilder quantityMultiplier) {
			this.quantityMultiplier = quantityMultiplier;
			return this;
		}
		public ResolvablePayoutQuantityBuilder setQuantityNotationTag(QuantityNotationEnum quantityNotationTag) {
			this.quantityNotationTag = quantityNotationTag;
			return this;
		}
		public ResolvablePayoutQuantityBuilder setQuantityReference(ReferenceWithMetaResolvablePayoutQuantity quantityReference) {
			this.quantityReference = quantityReference.toBuilder();
			return this;
		}
		public ResolvablePayoutQuantityBuilder setQuantityReferenceBuilder(ReferenceWithMetaResolvablePayoutQuantityBuilder quantityReference) {
			this.quantityReference = quantityReference;
			return this;
		}
		public ResolvablePayoutQuantityBuilder setQuantityReferenceRef(ResolvablePayoutQuantity.ResolvablePayoutQuantityBuilder quantityReference) {
			return setQuantityReference(ReferenceWithMetaResolvablePayoutQuantity.builder().setValueBuilder(quantityReference).build());
		}
		public ResolvablePayoutQuantityBuilder setQuantityReferenceRef(ResolvablePayoutQuantity quantityReference) {
			return setQuantityReferenceRef(ResolvablePayoutQuantity.builder());
		}
		public ResolvablePayoutQuantityBuilder setQuantitySchedule(NonNegativeQuantitySchedule quantitySchedule) {
			this.quantitySchedule = quantitySchedule.toBuilder();
			return this;
		}
		public ResolvablePayoutQuantityBuilder setQuantityScheduleBuilder(NonNegativeQuantitySchedule.NonNegativeQuantityScheduleBuilder quantitySchedule) {
			this.quantitySchedule = quantitySchedule;
			return this;
		}
		public ResolvablePayoutQuantityBuilder setReset(Boolean reset) {
			this.reset = reset;
			return this;
		}
		
		public ResolvablePayoutQuantity build() {
			return new ResolvablePayoutQuantity(this);
		}
	
		@Override
		public ResolvablePayoutQuantityBuilder prune() {
			if (meta!=null && !meta.prune().hasData()) meta = null;
			if (quantityMultiplier!=null && !quantityMultiplier.prune().hasData()) quantityMultiplier = null;
			if (quantityReference!=null && !quantityReference.prune().hasData()) quantityReference = null;
			if (quantitySchedule!=null && !quantitySchedule.prune().hasData()) quantitySchedule = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getMeta()!=null && getMeta().hasData()) return true;
			if (getQuantityMultiplier()!=null && getQuantityMultiplier().hasData()) return true;
			if (getQuantityNotationTag()!=null) return true;
			if (getQuantityReference()!=null && getQuantityReference().hasData()) return true;
			if (getQuantitySchedule()!=null && getQuantitySchedule().hasData()) return true;
			if (getReset()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("quantityNotationTag"), QuantityNotationEnum.class, quantityNotationTag, this);
			processor.processBasic(path.newSubPath("reset"), Boolean.class, reset, this);
			
			processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
			processRosetta(path.newSubPath("quantityMultiplier"), processor, QuantityMultiplier.class, quantityMultiplier);
			processRosetta(path.newSubPath("quantityReference"), processor, ReferenceWithMetaResolvablePayoutQuantity.class, quantityReference);
			processRosetta(path.newSubPath("quantitySchedule"), processor, NonNegativeQuantitySchedule.class, quantitySchedule);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			ResolvablePayoutQuantityBuilder _that = (ResolvablePayoutQuantityBuilder) o;
		
			if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
			if (quantityMultiplier != null ? !quantityMultiplier.equals(_that.quantityMultiplier) : _that.quantityMultiplier != null) return false;
			if (quantityNotationTag != null ? !quantityNotationTag.equals(_that.quantityNotationTag) : _that.quantityNotationTag != null) return false;
			if (quantityReference != null ? !quantityReference.equals(_that.quantityReference) : _that.quantityReference != null) return false;
			if (quantitySchedule != null ? !quantitySchedule.equals(_that.quantitySchedule) : _that.quantitySchedule != null) return false;
			if (reset != null ? !reset.equals(_that.reset) : _that.reset != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
			_result = 31 * _result + (quantityMultiplier != null ? quantityMultiplier.hashCode() : 0);
			_result = 31 * _result + (quantityNotationTag != null ? quantityNotationTag.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (quantityReference != null ? quantityReference.hashCode() : 0);
			_result = 31 * _result + (quantitySchedule != null ? quantitySchedule.hashCode() : 0);
			_result = 31 * _result + (reset != null ? reset.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "ResolvablePayoutQuantityBuilder {" +
				"meta=" + this.meta + ", " +
				"quantityMultiplier=" + this.quantityMultiplier + ", " +
				"quantityNotationTag=" + this.quantityNotationTag + ", " +
				"quantityReference=" + this.quantityReference + ", " +
				"quantitySchedule=" + this.quantitySchedule + ", " +
				"reset=" + this.reset +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("quantityNotationTag"), QuantityNotationEnum.class, quantityNotationTag, this);
		processor.processBasic(path.newSubPath("reset"), Boolean.class, reset, this);
		
		processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
		processRosetta(path.newSubPath("quantityMultiplier"), processor, QuantityMultiplier.class, quantityMultiplier);
		processRosetta(path.newSubPath("quantityReference"), processor, ReferenceWithMetaResolvablePayoutQuantity.class, quantityReference);
		processRosetta(path.newSubPath("quantitySchedule"), processor, NonNegativeQuantitySchedule.class, quantitySchedule);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		ResolvablePayoutQuantity _that = (ResolvablePayoutQuantity) o;
	
		if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
		if (quantityMultiplier != null ? !quantityMultiplier.equals(_that.quantityMultiplier) : _that.quantityMultiplier != null) return false;
		if (quantityNotationTag != null ? !quantityNotationTag.equals(_that.quantityNotationTag) : _that.quantityNotationTag != null) return false;
		if (quantityReference != null ? !quantityReference.equals(_that.quantityReference) : _that.quantityReference != null) return false;
		if (quantitySchedule != null ? !quantitySchedule.equals(_that.quantitySchedule) : _that.quantitySchedule != null) return false;
		if (reset != null ? !reset.equals(_that.reset) : _that.reset != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
		_result = 31 * _result + (quantityMultiplier != null ? quantityMultiplier.hashCode() : 0);
		_result = 31 * _result + (quantityNotationTag != null ? quantityNotationTag.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (quantityReference != null ? quantityReference.hashCode() : 0);
		_result = 31 * _result + (quantitySchedule != null ? quantitySchedule.hashCode() : 0);
		_result = 31 * _result + (reset != null ? reset.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "ResolvablePayoutQuantity {" +
			"meta=" + this.meta + ", " +
			"quantityMultiplier=" + this.quantityMultiplier + ", " +
			"quantityNotationTag=" + this.quantityNotationTag + ", " +
			"quantityReference=" + this.quantityReference + ", " +
			"quantitySchedule=" + this.quantitySchedule + ", " +
			"reset=" + this.reset +
		'}';
	}
}
