package org.isda.cdm;

import java.lang.Boolean;
import java.lang.String;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.ReturnAmountMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class to specify the application of Interest Amount with respect the Return Amount. ISDA 2016 Japanese Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (n)(ii).
 * 
 * @version 2.5.4
 */
@RosettaClass
public class ReturnAmount extends RosettaModelObject {
	private final String customElection;
	private final Boolean includesDefaultLanguage;
	private static ReturnAmountMeta metaData = new ReturnAmountMeta();
	
	ReturnAmount(ReturnAmountBuilder builder) {
		this.customElection = builder.getCustomElection();
		this.includesDefaultLanguage = builder.getIncludesDefaultLanguage();
	}
	
	/**
	 * Custom election that might be specified by the parties to the agreement.
	 */
	@RosettaSynonym(value="specify_return_amount", source="ISDA_Create_1_0")
	public final String getCustomElection() {
		return customElection;
	}
	
	/**
	 * Default language is included when True, and excluded when False.
	 */
	@RosettaSynonym(value="", source="ISDA_Create_1_0")
	public final Boolean getIncludesDefaultLanguage() {
		return includesDefaultLanguage;
	}
	
	
	@Override
	public RosettaMetaData<? extends ReturnAmount> metaData() {
		return metaData;
	} 
	
	public ReturnAmountBuilder toBuilder() {
		ReturnAmountBuilder builder = new ReturnAmountBuilder();
		ofNullable(getCustomElection()).ifPresent(builder::setCustomElection);
		ofNullable(getIncludesDefaultLanguage()).ifPresent(builder::setIncludesDefaultLanguage);
		return builder;
	}

	public static ReturnAmountBuilder builder() {
		return new ReturnAmountBuilder();
	}

	public static  class ReturnAmountBuilder extends RosettaModelObjectBuilder {
	
		protected String customElection;
		protected Boolean includesDefaultLanguage;
	
		public ReturnAmountBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends ReturnAmount> metaData() {
			return metaData;
		} 
	
		
		public String getCustomElection() {
			return customElection;
		}
		
		
		public Boolean getIncludesDefaultLanguage() {
			return includesDefaultLanguage;
		}
		
	
		public ReturnAmountBuilder setCustomElection(String customElection) {
			this.customElection = customElection;
			return this;
		}
		public ReturnAmountBuilder setIncludesDefaultLanguage(Boolean includesDefaultLanguage) {
			this.includesDefaultLanguage = includesDefaultLanguage;
			return this;
		}
		
		public ReturnAmount build() {
			return new ReturnAmount(this);
		}
	
		@Override
		public ReturnAmountBuilder prune() {
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getCustomElection()!=null) return true;
			if (getIncludesDefaultLanguage()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("customElection"), String.class, customElection, this);
			processor.processBasic(path.newSubPath("includesDefaultLanguage"), Boolean.class, includesDefaultLanguage, this);
			
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			ReturnAmountBuilder _that = (ReturnAmountBuilder) o;
		
			if (customElection != null ? !customElection.equals(_that.customElection) : _that.customElection != null) return false;
			if (includesDefaultLanguage != null ? !includesDefaultLanguage.equals(_that.includesDefaultLanguage) : _that.includesDefaultLanguage != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (customElection != null ? customElection.hashCode() : 0);
			_result = 31 * _result + (includesDefaultLanguage != null ? includesDefaultLanguage.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "ReturnAmountBuilder {" +
				"customElection=" + this.customElection + ", " +
				"includesDefaultLanguage=" + this.includesDefaultLanguage +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("customElection"), String.class, customElection, this);
		processor.processBasic(path.newSubPath("includesDefaultLanguage"), Boolean.class, includesDefaultLanguage, this);
		
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		ReturnAmount _that = (ReturnAmount) o;
	
		if (customElection != null ? !customElection.equals(_that.customElection) : _that.customElection != null) return false;
		if (includesDefaultLanguage != null ? !includesDefaultLanguage.equals(_that.includesDefaultLanguage) : _that.includesDefaultLanguage != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (customElection != null ? customElection.hashCode() : 0);
		_result = 31 * _result + (includesDefaultLanguage != null ? includesDefaultLanguage.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "ReturnAmount {" +
			"customElection=" + this.customElection + ", " +
			"includesDefaultLanguage=" + this.includesDefaultLanguage +
		'}';
	}
}
