package org.isda.cdm;

import java.lang.Boolean;
import java.lang.String;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.RightsEventMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class to specify the Pledgor/Obligor/Chargor Rights Event election. ISDA 2016 English Law Credit Support Deed for Initial Margin, paragraph 13, General Principles, (j): Chargor Rights Event. | ISDA 2016 Japanese Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (j): Obligor Rights Event. | ISDA 2016 New York Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (j): Pledgor Rights Event.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class RightsEvent extends RosettaModelObject {
	private final String customElection;
	private final Boolean includeCoolingOffLanguage;
	private static RightsEventMeta metaData = new RightsEventMeta();
	
	RightsEvent(RightsEventBuilder builder) {
		this.customElection = builder.getCustomElection();
		this.includeCoolingOffLanguage = builder.getIncludeCoolingOffLanguage();
	}
	
	/**
	 * A custom Pledgor/Obligor/Chargor Rights Event election might be specified by the parties.
	 */
	@RosettaSynonym(value="specify", source="ISDA_Create_1_0")
	public final String getCustomElection() {
		return customElection;
	}
	
	/**
	 * The Pledgor/Obligor/Chargor Rights Event election includes cooling off language when the attribute is set of True.
	 */
	@RosettaSynonym(value="", source="ISDA_Create_1_0")
	public final Boolean getIncludeCoolingOffLanguage() {
		return includeCoolingOffLanguage;
	}
	
	
	@Override
	public RosettaMetaData<? extends RightsEvent> metaData() {
		return metaData;
	} 
	
	public RightsEventBuilder toBuilder() {
		RightsEventBuilder builder = new RightsEventBuilder();
		ofNullable(getCustomElection()).ifPresent(builder::setCustomElection);
		ofNullable(getIncludeCoolingOffLanguage()).ifPresent(builder::setIncludeCoolingOffLanguage);
		return builder;
	}

	public static RightsEventBuilder builder() {
		return new RightsEventBuilder();
	}

	public static  class RightsEventBuilder extends RosettaModelObjectBuilder {
	
		protected String customElection;
		protected Boolean includeCoolingOffLanguage;
	
		public RightsEventBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends RightsEvent> metaData() {
			return metaData;
		} 
	
		
		public String getCustomElection() {
			return customElection;
		}
		
		
		public Boolean getIncludeCoolingOffLanguage() {
			return includeCoolingOffLanguage;
		}
		
	
		public RightsEventBuilder setCustomElection(String customElection) {
			this.customElection = customElection;
			return this;
		}
		public RightsEventBuilder setIncludeCoolingOffLanguage(Boolean includeCoolingOffLanguage) {
			this.includeCoolingOffLanguage = includeCoolingOffLanguage;
			return this;
		}
		
		public RightsEvent build() {
			return new RightsEvent(this);
		}
	
		@Override
		public RightsEventBuilder prune() {
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getCustomElection()!=null) return true;
			if (getIncludeCoolingOffLanguage()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("customElection"), String.class, customElection, this);
			processor.processBasic(path.newSubPath("includeCoolingOffLanguage"), Boolean.class, includeCoolingOffLanguage, this);
			
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			RightsEventBuilder _that = (RightsEventBuilder) o;
		
			if (customElection != null ? !customElection.equals(_that.customElection) : _that.customElection != null) return false;
			if (includeCoolingOffLanguage != null ? !includeCoolingOffLanguage.equals(_that.includeCoolingOffLanguage) : _that.includeCoolingOffLanguage != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (customElection != null ? customElection.hashCode() : 0);
			_result = 31 * _result + (includeCoolingOffLanguage != null ? includeCoolingOffLanguage.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "RightsEventBuilder {" +
				"customElection=" + this.customElection + ", " +
				"includeCoolingOffLanguage=" + this.includeCoolingOffLanguage +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("customElection"), String.class, customElection, this);
		processor.processBasic(path.newSubPath("includeCoolingOffLanguage"), Boolean.class, includeCoolingOffLanguage, this);
		
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		RightsEvent _that = (RightsEvent) o;
	
		if (customElection != null ? !customElection.equals(_that.customElection) : _that.customElection != null) return false;
		if (includeCoolingOffLanguage != null ? !includeCoolingOffLanguage.equals(_that.includeCoolingOffLanguage) : _that.includeCoolingOffLanguage != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (customElection != null ? customElection.hashCode() : 0);
		_result = 31 * _result + (includeCoolingOffLanguage != null ? includeCoolingOffLanguage.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "RightsEvent {" +
			"customElection=" + this.customElection + ", " +
			"includeCoolingOffLanguage=" + this.includeCoolingOffLanguage +
		'}';
	}
}
