package org.isda.cdm;

import java.lang.Integer;
import org.isda.cdm.RoundingDirectionEnum;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.RoundingMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class defining a rounding direction and precision to be used in the rounding of a rate.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="Rounding", source="FpML_5_10")
public class Rounding extends RosettaModelObject {
	private final Integer precision;
	private final RoundingDirectionEnum roundingDirection;
	private static RoundingMeta metaData = new RoundingMeta();
	
	Rounding(RoundingBuilder builder) {
		this.precision = builder.getPrecision();
		this.roundingDirection = builder.getRoundingDirection();
	}
	
	/**
	 * Specifies the rounding precision in terms of a number of decimal places. Note how a percentage rate rounding of 5 decimal places is expressed as a rounding precision of 7 in the FpML document since the percentage is expressed as a decimal, e.g. 9.876543% (or 0.09876543) being rounded to the nearest 5 decimal places is 9.87654% (or 0.0987654).
	 */
	@RosettaSynonym(value="precision", source="FpML_5_10")
	@RosettaSynonym(value="precision", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="precision", source="DTCC_11_0")
	@RosettaSynonym(value="precision", source="DTCC_9_0")
	@RosettaSynonym(value="precision", source="CME_ClearedConfirm_1_17")
	public final Integer getPrecision() {
		return precision;
	}
	
	/**
	 * Specifies the rounding direction.
	 */
	@RosettaSynonym(value="roundingDirection", source="FpML_5_10")
	@RosettaSynonym(value="roundingDirection", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="roundingDirection", source="DTCC_11_0")
	@RosettaSynonym(value="roundingDirection", source="DTCC_9_0")
	@RosettaSynonym(value="roundingDirection", source="CME_ClearedConfirm_1_17")
	public final RoundingDirectionEnum getRoundingDirection() {
		return roundingDirection;
	}
	
	
	@Override
	public RosettaMetaData<? extends Rounding> metaData() {
		return metaData;
	} 
	
	public RoundingBuilder toBuilder() {
		RoundingBuilder builder = new RoundingBuilder();
		ofNullable(getPrecision()).ifPresent(builder::setPrecision);
		ofNullable(getRoundingDirection()).ifPresent(builder::setRoundingDirection);
		return builder;
	}

	public static RoundingBuilder builder() {
		return new RoundingBuilder();
	}

	public static  class RoundingBuilder extends RosettaModelObjectBuilder {
	
		protected Integer precision;
		protected RoundingDirectionEnum roundingDirection;
	
		public RoundingBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends Rounding> metaData() {
			return metaData;
		} 
	
		
		public Integer getPrecision() {
			return precision;
		}
		
		
		public RoundingDirectionEnum getRoundingDirection() {
			return roundingDirection;
		}
		
	
		public RoundingBuilder setPrecision(Integer precision) {
			this.precision = precision;
			return this;
		}
		public RoundingBuilder setRoundingDirection(RoundingDirectionEnum roundingDirection) {
			this.roundingDirection = roundingDirection;
			return this;
		}
		
		public Rounding build() {
			return new Rounding(this);
		}
	
		@Override
		public RoundingBuilder prune() {
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getPrecision()!=null) return true;
			if (getRoundingDirection()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("precision"), Integer.class, precision, this);
			processor.processBasic(path.newSubPath("roundingDirection"), RoundingDirectionEnum.class, roundingDirection, this);
			
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			RoundingBuilder _that = (RoundingBuilder) o;
		
			if (precision != null ? !precision.equals(_that.precision) : _that.precision != null) return false;
			if (roundingDirection != null ? !roundingDirection.equals(_that.roundingDirection) : _that.roundingDirection != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (precision != null ? precision.hashCode() : 0);
			_result = 31 * _result + (roundingDirection != null ? roundingDirection.getClass().getName().hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "RoundingBuilder {" +
				"precision=" + this.precision + ", " +
				"roundingDirection=" + this.roundingDirection +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("precision"), Integer.class, precision, this);
		processor.processBasic(path.newSubPath("roundingDirection"), RoundingDirectionEnum.class, roundingDirection, this);
		
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		Rounding _that = (Rounding) o;
	
		if (precision != null ? !precision.equals(_that.precision) : _that.precision != null) return false;
		if (roundingDirection != null ? !roundingDirection.equals(_that.roundingDirection) : _that.roundingDirection != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (precision != null ? precision.hashCode() : 0);
		_result = 31 * _result + (roundingDirection != null ? roundingDirection.getClass().getName().hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "Rounding {" +
			"precision=" + this.precision + ", " +
			"roundingDirection=" + this.roundingDirection +
		'}';
	}
}
