package org.isda.cdm;

import java.lang.String;
import org.isda.cdm.AdjustableOrRelativeDate;
import org.isda.cdm.BuyerSeller;
import org.isda.cdm.DeliveryMethodEnum;
import org.isda.cdm.ExchangeRate;
import org.isda.cdm.Money;
import org.isda.cdm.metafields.MetaFields;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.SecurityLegMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * Terms defining a security leg in a securities financing transaction, which can either be the near leg or the far leg and is closely modelled onto the nearLeg and farLeg types in FpML
 * 
 * @version 2.5.4
 */
@RosettaClass
public class SecurityLeg extends RosettaModelObject implements GlobalKey {
	private final BuyerSeller buyerSeller;
	private final AdjustableOrRelativeDate deliveryDate;
	private final DeliveryMethodEnum deliveryMethod;
	private final ExchangeRate fxRate;
	private final MetaFields meta;
	private final Money settlementAmount;
	private final String settlementCurrency;
	private final AdjustableOrRelativeDate settlementDate;
	private static SecurityLegMeta metaData = new SecurityLegMeta();
	
	SecurityLeg(SecurityLegBuilder builder) {
		this.buyerSeller = ofNullable(builder.getBuyerSeller()).map(f->f.build()).orElse(null);
		this.deliveryDate = ofNullable(builder.getDeliveryDate()).map(f->f.build()).orElse(null);
		this.deliveryMethod = builder.getDeliveryMethod();
		this.fxRate = ofNullable(builder.getFxRate()).map(f->f.build()).orElse(null);
		this.meta = ofNullable(builder.getMeta()).map(f->f.build()).orElse(null);
		this.settlementAmount = ofNullable(builder.getSettlementAmount()).map(f->f.build()).orElse(null);
		this.settlementCurrency = builder.getSettlementCurrency();
		this.settlementDate = ofNullable(builder.getSettlementDate()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * Whether the leg is a buyer or seller of security
	 */
	public final BuyerSeller getBuyerSeller() {
		return buyerSeller;
	}
	
	/**
	 * Delivery Date for the transaction. Delivery Date can be populated when it is not equal to the Settlement Date.
	 */
	@RosettaSynonym(value="deliveryDate", source="FpML_5_10")
	public final AdjustableOrRelativeDate getDeliveryDate() {
		return deliveryDate;
	}
	
	/**
	 * Specifies a delivery method for the security transaction.
	 */
	@RosettaSynonym(value="deliveryMethod", source="FpML_5_10")
	public final DeliveryMethodEnum getDeliveryMethod() {
		return deliveryMethod;
	}
	
	/**
	 * FX rate in case when cash settlement amount is in a different currency to the security.
	 */
	public final ExchangeRate getFxRate() {
		return fxRate;
	}
	
	public final MetaFields getMeta() {
		return meta;
	}
	
	/**
	 * Settlement amount for the security leg
	 */
	@RosettaSynonym(value="settlementAmount", source="FpML_5_10")
	public final Money getSettlementAmount() {
		return settlementAmount;
	}
	
	/**
	 * Settlement Currency for use where the Settlement Amount cannot be known in advance.
	 */
	@RosettaSynonym(value="settlementCurrency", source="FpML_5_10")
	public final String getSettlementCurrency() {
		return settlementCurrency;
	}
	
	/**
	 * Settlement or Payment Date for the security leg
	 */
	@RosettaSynonym(value="settlementDate", source="FpML_5_10")
	public final AdjustableOrRelativeDate getSettlementDate() {
		return settlementDate;
	}
	
	
	@Override
	public RosettaMetaData<? extends SecurityLeg> metaData() {
		return metaData;
	} 
	
	public SecurityLegBuilder toBuilder() {
		SecurityLegBuilder builder = new SecurityLegBuilder();
		ofNullable(getBuyerSeller()).ifPresent(builder::setBuyerSeller);
		ofNullable(getDeliveryDate()).ifPresent(builder::setDeliveryDate);
		ofNullable(getDeliveryMethod()).ifPresent(builder::setDeliveryMethod);
		ofNullable(getFxRate()).ifPresent(builder::setFxRate);
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		ofNullable(getSettlementAmount()).ifPresent(builder::setSettlementAmount);
		ofNullable(getSettlementCurrency()).ifPresent(builder::setSettlementCurrency);
		ofNullable(getSettlementDate()).ifPresent(builder::setSettlementDate);
		return builder;
	}

	public static SecurityLegBuilder builder() {
		return new SecurityLegBuilder();
	}

	public static  class SecurityLegBuilder extends RosettaModelObjectBuilder implements GlobalKeyBuilder<SecurityLegBuilder> {
	
		protected BuyerSeller.BuyerSellerBuilder buyerSeller;
		protected AdjustableOrRelativeDate.AdjustableOrRelativeDateBuilder deliveryDate;
		protected DeliveryMethodEnum deliveryMethod;
		protected ExchangeRate.ExchangeRateBuilder fxRate;
		protected MetaFields.MetaFieldsBuilder meta;
		protected Money.MoneyBuilder settlementAmount;
		protected String settlementCurrency;
		protected AdjustableOrRelativeDate.AdjustableOrRelativeDateBuilder settlementDate;
	
		public SecurityLegBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends SecurityLeg> metaData() {
			return metaData;
		} 
	
		
		public BuyerSeller.BuyerSellerBuilder getBuyerSeller() {
			return buyerSeller;
		}
		
		public BuyerSeller.BuyerSellerBuilder getOrCreateBuyerSeller() {
			if (buyerSeller!=null) {
				return buyerSeller;
			}
			else return buyerSeller = new BuyerSeller.BuyerSellerBuilder();
		}
		
		
		public AdjustableOrRelativeDate.AdjustableOrRelativeDateBuilder getDeliveryDate() {
			return deliveryDate;
		}
		
		public AdjustableOrRelativeDate.AdjustableOrRelativeDateBuilder getOrCreateDeliveryDate() {
			if (deliveryDate!=null) {
				return deliveryDate;
			}
			else return deliveryDate = new AdjustableOrRelativeDate.AdjustableOrRelativeDateBuilder();
		}
		
		
		public DeliveryMethodEnum getDeliveryMethod() {
			return deliveryMethod;
		}
		
		
		public ExchangeRate.ExchangeRateBuilder getFxRate() {
			return fxRate;
		}
		
		public ExchangeRate.ExchangeRateBuilder getOrCreateFxRate() {
			if (fxRate!=null) {
				return fxRate;
			}
			else return fxRate = new ExchangeRate.ExchangeRateBuilder();
		}
		
		
		public MetaFields.MetaFieldsBuilder getMeta() {
			return meta;
		}
		
		public MetaFields.MetaFieldsBuilder getOrCreateMeta() {
			if (meta!=null) {
				return meta;
			}
			else return meta = new MetaFields.MetaFieldsBuilder();
		}
		
		
		public Money.MoneyBuilder getSettlementAmount() {
			return settlementAmount;
		}
		
		public Money.MoneyBuilder getOrCreateSettlementAmount() {
			if (settlementAmount!=null) {
				return settlementAmount;
			}
			else return settlementAmount = new Money.MoneyBuilder();
		}
		
		
		public String getSettlementCurrency() {
			return settlementCurrency;
		}
		
		
		public AdjustableOrRelativeDate.AdjustableOrRelativeDateBuilder getSettlementDate() {
			return settlementDate;
		}
		
		public AdjustableOrRelativeDate.AdjustableOrRelativeDateBuilder getOrCreateSettlementDate() {
			if (settlementDate!=null) {
				return settlementDate;
			}
			else return settlementDate = new AdjustableOrRelativeDate.AdjustableOrRelativeDateBuilder();
		}
		
	
		public SecurityLegBuilder setBuyerSeller(BuyerSeller buyerSeller) {
			this.buyerSeller = buyerSeller.toBuilder();
			return this;
		}
		public SecurityLegBuilder setBuyerSellerBuilder(BuyerSeller.BuyerSellerBuilder buyerSeller) {
			this.buyerSeller = buyerSeller;
			return this;
		}
		public SecurityLegBuilder setDeliveryDate(AdjustableOrRelativeDate deliveryDate) {
			this.deliveryDate = deliveryDate.toBuilder();
			return this;
		}
		public SecurityLegBuilder setDeliveryDateBuilder(AdjustableOrRelativeDate.AdjustableOrRelativeDateBuilder deliveryDate) {
			this.deliveryDate = deliveryDate;
			return this;
		}
		public SecurityLegBuilder setDeliveryMethod(DeliveryMethodEnum deliveryMethod) {
			this.deliveryMethod = deliveryMethod;
			return this;
		}
		public SecurityLegBuilder setFxRate(ExchangeRate fxRate) {
			this.fxRate = fxRate.toBuilder();
			return this;
		}
		public SecurityLegBuilder setFxRateBuilder(ExchangeRate.ExchangeRateBuilder fxRate) {
			this.fxRate = fxRate;
			return this;
		}
		public SecurityLegBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		public SecurityLegBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		public SecurityLegBuilder setSettlementAmount(Money settlementAmount) {
			this.settlementAmount = settlementAmount.toBuilder();
			return this;
		}
		public SecurityLegBuilder setSettlementAmountBuilder(Money.MoneyBuilder settlementAmount) {
			this.settlementAmount = settlementAmount;
			return this;
		}
		public SecurityLegBuilder setSettlementCurrency(String settlementCurrency) {
			this.settlementCurrency = settlementCurrency;
			return this;
		}
		public SecurityLegBuilder setSettlementDate(AdjustableOrRelativeDate settlementDate) {
			this.settlementDate = settlementDate.toBuilder();
			return this;
		}
		public SecurityLegBuilder setSettlementDateBuilder(AdjustableOrRelativeDate.AdjustableOrRelativeDateBuilder settlementDate) {
			this.settlementDate = settlementDate;
			return this;
		}
		
		public SecurityLeg build() {
			return new SecurityLeg(this);
		}
	
		@Override
		public SecurityLegBuilder prune() {
			if (buyerSeller!=null && !buyerSeller.prune().hasData()) buyerSeller = null;
			if (deliveryDate!=null && !deliveryDate.prune().hasData()) deliveryDate = null;
			if (fxRate!=null && !fxRate.prune().hasData()) fxRate = null;
			if (meta!=null && !meta.prune().hasData()) meta = null;
			if (settlementAmount!=null && !settlementAmount.prune().hasData()) settlementAmount = null;
			if (settlementDate!=null && !settlementDate.prune().hasData()) settlementDate = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getBuyerSeller()!=null && getBuyerSeller().hasData()) return true;
			if (getDeliveryDate()!=null && getDeliveryDate().hasData()) return true;
			if (getDeliveryMethod()!=null) return true;
			if (getFxRate()!=null && getFxRate().hasData()) return true;
			if (getMeta()!=null && getMeta().hasData()) return true;
			if (getSettlementAmount()!=null && getSettlementAmount().hasData()) return true;
			if (getSettlementCurrency()!=null) return true;
			if (getSettlementDate()!=null && getSettlementDate().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("deliveryMethod"), DeliveryMethodEnum.class, deliveryMethod, this);
			processor.processBasic(path.newSubPath("settlementCurrency"), String.class, settlementCurrency, this);
			
			processRosetta(path.newSubPath("buyerSeller"), processor, BuyerSeller.class, buyerSeller);
			processRosetta(path.newSubPath("deliveryDate"), processor, AdjustableOrRelativeDate.class, deliveryDate);
			processRosetta(path.newSubPath("fxRate"), processor, ExchangeRate.class, fxRate);
			processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
			processRosetta(path.newSubPath("settlementAmount"), processor, Money.class, settlementAmount);
			processRosetta(path.newSubPath("settlementDate"), processor, AdjustableOrRelativeDate.class, settlementDate);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			SecurityLegBuilder _that = (SecurityLegBuilder) o;
		
			if (buyerSeller != null ? !buyerSeller.equals(_that.buyerSeller) : _that.buyerSeller != null) return false;
			if (deliveryDate != null ? !deliveryDate.equals(_that.deliveryDate) : _that.deliveryDate != null) return false;
			if (deliveryMethod != null ? !deliveryMethod.equals(_that.deliveryMethod) : _that.deliveryMethod != null) return false;
			if (fxRate != null ? !fxRate.equals(_that.fxRate) : _that.fxRate != null) return false;
			if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
			if (settlementAmount != null ? !settlementAmount.equals(_that.settlementAmount) : _that.settlementAmount != null) return false;
			if (settlementCurrency != null ? !settlementCurrency.equals(_that.settlementCurrency) : _that.settlementCurrency != null) return false;
			if (settlementDate != null ? !settlementDate.equals(_that.settlementDate) : _that.settlementDate != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (buyerSeller != null ? buyerSeller.hashCode() : 0);
			_result = 31 * _result + (deliveryDate != null ? deliveryDate.hashCode() : 0);
			_result = 31 * _result + (deliveryMethod != null ? deliveryMethod.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (fxRate != null ? fxRate.hashCode() : 0);
			_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
			_result = 31 * _result + (settlementAmount != null ? settlementAmount.hashCode() : 0);
			_result = 31 * _result + (settlementCurrency != null ? settlementCurrency.hashCode() : 0);
			_result = 31 * _result + (settlementDate != null ? settlementDate.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "SecurityLegBuilder {" +
				"buyerSeller=" + this.buyerSeller + ", " +
				"deliveryDate=" + this.deliveryDate + ", " +
				"deliveryMethod=" + this.deliveryMethod + ", " +
				"fxRate=" + this.fxRate + ", " +
				"meta=" + this.meta + ", " +
				"settlementAmount=" + this.settlementAmount + ", " +
				"settlementCurrency=" + this.settlementCurrency + ", " +
				"settlementDate=" + this.settlementDate +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("deliveryMethod"), DeliveryMethodEnum.class, deliveryMethod, this);
		processor.processBasic(path.newSubPath("settlementCurrency"), String.class, settlementCurrency, this);
		
		processRosetta(path.newSubPath("buyerSeller"), processor, BuyerSeller.class, buyerSeller);
		processRosetta(path.newSubPath("deliveryDate"), processor, AdjustableOrRelativeDate.class, deliveryDate);
		processRosetta(path.newSubPath("fxRate"), processor, ExchangeRate.class, fxRate);
		processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
		processRosetta(path.newSubPath("settlementAmount"), processor, Money.class, settlementAmount);
		processRosetta(path.newSubPath("settlementDate"), processor, AdjustableOrRelativeDate.class, settlementDate);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		SecurityLeg _that = (SecurityLeg) o;
	
		if (buyerSeller != null ? !buyerSeller.equals(_that.buyerSeller) : _that.buyerSeller != null) return false;
		if (deliveryDate != null ? !deliveryDate.equals(_that.deliveryDate) : _that.deliveryDate != null) return false;
		if (deliveryMethod != null ? !deliveryMethod.equals(_that.deliveryMethod) : _that.deliveryMethod != null) return false;
		if (fxRate != null ? !fxRate.equals(_that.fxRate) : _that.fxRate != null) return false;
		if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
		if (settlementAmount != null ? !settlementAmount.equals(_that.settlementAmount) : _that.settlementAmount != null) return false;
		if (settlementCurrency != null ? !settlementCurrency.equals(_that.settlementCurrency) : _that.settlementCurrency != null) return false;
		if (settlementDate != null ? !settlementDate.equals(_that.settlementDate) : _that.settlementDate != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (buyerSeller != null ? buyerSeller.hashCode() : 0);
		_result = 31 * _result + (deliveryDate != null ? deliveryDate.hashCode() : 0);
		_result = 31 * _result + (deliveryMethod != null ? deliveryMethod.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (fxRate != null ? fxRate.hashCode() : 0);
		_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
		_result = 31 * _result + (settlementAmount != null ? settlementAmount.hashCode() : 0);
		_result = 31 * _result + (settlementCurrency != null ? settlementCurrency.hashCode() : 0);
		_result = 31 * _result + (settlementDate != null ? settlementDate.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "SecurityLeg {" +
			"buyerSeller=" + this.buyerSeller + ", " +
			"deliveryDate=" + this.deliveryDate + ", " +
			"deliveryMethod=" + this.deliveryMethod + ", " +
			"fxRate=" + this.fxRate + ", " +
			"meta=" + this.meta + ", " +
			"settlementAmount=" + this.settlementAmount + ", " +
			"settlementCurrency=" + this.settlementCurrency + ", " +
			"settlementDate=" + this.settlementDate +
		'}';
	}
}
