package org.isda.cdm;

import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.InitialMargin;
import org.isda.cdm.RepoDurationEnum;
import org.isda.cdm.SecurityLeg;
import org.isda.cdm.SecurityValuation;
import org.isda.cdm.metafields.MetaFields;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.SecurityPayoutMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;

/**
 * Security payout specification in case the product payout involves some form of security collateral, as in a securities financing transaction.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class SecurityPayout extends RosettaModelObject implements GlobalKey {
	private final InitialMargin initialMargin;
	private final MetaFields meta;
	private final RepoDurationEnum repoDuration;
	private final List<SecurityLeg> securityLeg;
	private final List<SecurityValuation> securityValuation;
	private static SecurityPayoutMeta metaData = new SecurityPayoutMeta();
	
	SecurityPayout(SecurityPayoutBuilder builder) {
		this.initialMargin = ofNullable(builder.getInitialMargin()).map(f->f.build()).orElse(null);
		this.meta = ofNullable(builder.getMeta()).map(f->f.build()).orElse(null);
		this.repoDuration = builder.getRepoDuration();
		this.securityLeg = ofNullable(builder.getSecurityLeg()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.securityValuation = ofNullable(builder.getSecurityValuation()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
	}
	
	/**
	 * RepoDurationEnum.
	 */
	@RosettaSynonym(value="initialMargin", source="FpML_5_10")
	public final InitialMargin getInitialMargin() {
		return initialMargin;
	}
	
	public final MetaFields getMeta() {
		return meta;
	}
	
	/**
	 * A duration code for the repo transaction. This defines a type of a repo transaction with fixed duration.
	 */
	@RosettaSynonym(value="duration", source="FpML_5_10")
	public final RepoDurationEnum getRepoDuration() {
		return repoDuration;
	}
	
	/**
	 * Each SecurityLeg represent a buy/sell at different dates, typically 1 near leg and 1 far leg in a securities financing transaction.
	 */
	@RosettaSynonym(value="nearLeg", source="FpML_5_10")
	@RosettaSynonym(value="farLeg", source="FpML_5_10")
	public final List<SecurityLeg> getSecurityLeg() {
		return securityLeg;
	}
	
	/**
	 * The underlying securities and their valuation for the security leg.
	 */
	@RosettaSynonym(value="nearLeg,bond", source="FpML_5_10")
	public final List<SecurityValuation> getSecurityValuation() {
		return securityValuation;
	}
	
	
	@Override
	public RosettaMetaData<? extends SecurityPayout> metaData() {
		return metaData;
	} 
	
	public SecurityPayoutBuilder toBuilder() {
		SecurityPayoutBuilder builder = new SecurityPayoutBuilder();
		ofNullable(getInitialMargin()).ifPresent(builder::setInitialMargin);
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		ofNullable(getRepoDuration()).ifPresent(builder::setRepoDuration);
		ofNullable(getSecurityLeg()).ifPresent(securityLeg -> securityLeg.forEach(builder::addSecurityLeg));
		ofNullable(getSecurityValuation()).ifPresent(securityValuation -> securityValuation.forEach(builder::addSecurityValuation));
		return builder;
	}

	public static SecurityPayoutBuilder builder() {
		return new SecurityPayoutBuilder();
	}

	public static  class SecurityPayoutBuilder extends RosettaModelObjectBuilder implements GlobalKeyBuilder<SecurityPayoutBuilder> {
	
		protected InitialMargin.InitialMarginBuilder initialMargin;
		protected MetaFields.MetaFieldsBuilder meta;
		protected RepoDurationEnum repoDuration;
		protected List<SecurityLeg.SecurityLegBuilder> securityLeg;
		protected List<SecurityValuation.SecurityValuationBuilder> securityValuation;
	
		public SecurityPayoutBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends SecurityPayout> metaData() {
			return metaData;
		} 
	
		
		public InitialMargin.InitialMarginBuilder getInitialMargin() {
			return initialMargin;
		}
		
		public InitialMargin.InitialMarginBuilder getOrCreateInitialMargin() {
			if (initialMargin!=null) {
				return initialMargin;
			}
			else return initialMargin = new InitialMargin.InitialMarginBuilder();
		}
		
		
		public MetaFields.MetaFieldsBuilder getMeta() {
			return meta;
		}
		
		public MetaFields.MetaFieldsBuilder getOrCreateMeta() {
			if (meta!=null) {
				return meta;
			}
			else return meta = new MetaFields.MetaFieldsBuilder();
		}
		
		
		public RepoDurationEnum getRepoDuration() {
			return repoDuration;
		}
		
		
		public List<SecurityLeg.SecurityLegBuilder> getSecurityLeg() {
			return securityLeg;
		}
		
		public SecurityLeg.SecurityLegBuilder getOrCreateSecurityLeg(int index) {
			if (securityLeg==null) {
				this.securityLeg = new ArrayList<>();
			}
			return getIndex(securityLeg, index, ()->new SecurityLeg.SecurityLegBuilder());
		}
		
		
		public List<SecurityValuation.SecurityValuationBuilder> getSecurityValuation() {
			return securityValuation;
		}
		
		public SecurityValuation.SecurityValuationBuilder getOrCreateSecurityValuation(int index) {
			if (securityValuation==null) {
				this.securityValuation = new ArrayList<>();
			}
			return getIndex(securityValuation, index, ()->new SecurityValuation.SecurityValuationBuilder());
		}
		
	
		public SecurityPayoutBuilder setInitialMargin(InitialMargin initialMargin) {
			this.initialMargin = initialMargin.toBuilder();
			return this;
		}
		public SecurityPayoutBuilder setInitialMarginBuilder(InitialMargin.InitialMarginBuilder initialMargin) {
			this.initialMargin = initialMargin;
			return this;
		}
		public SecurityPayoutBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		public SecurityPayoutBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		public SecurityPayoutBuilder setRepoDuration(RepoDurationEnum repoDuration) {
			this.repoDuration = repoDuration;
			return this;
		}
		public SecurityPayoutBuilder addSecurityLeg(SecurityLeg securityLeg) {
			if(this.securityLeg == null){
				this.securityLeg = new ArrayList<>();
			}
			this.securityLeg.add(securityLeg.toBuilder());
			return this;
		}
		public SecurityPayoutBuilder addSecurityLeg(SecurityLeg securityLeg, int _idx) {
			if(this.securityLeg == null){
				this.securityLeg = new ArrayList<>();
			}
			getIndex(this.securityLeg, _idx, () -> securityLeg.toBuilder());
			this.securityLeg.set(_idx, securityLeg.toBuilder());
			return this;
		}
		public SecurityPayoutBuilder addSecurityLeg(List<SecurityLeg> securityLegs) {
			if(this.securityLeg == null){
				this.securityLeg = new ArrayList<>();
			}
			for (SecurityLeg toAdd : securityLegs) {
				this.securityLeg.add(toAdd.toBuilder());
			}
			return this;
		}
		public SecurityPayoutBuilder addSecurityLegBuilder(SecurityLeg.SecurityLegBuilder securityLeg) {
			if(this.securityLeg == null){
				this.securityLeg = new ArrayList<>();
			}
			this.securityLeg.add(securityLeg);
			return this;
		}
		
		public SecurityPayoutBuilder clearSecurityLeg() {
			this.securityLeg = null;
			return this;
		}
		public SecurityPayoutBuilder addSecurityValuation(SecurityValuation securityValuation) {
			if(this.securityValuation == null){
				this.securityValuation = new ArrayList<>();
			}
			this.securityValuation.add(securityValuation.toBuilder());
			return this;
		}
		public SecurityPayoutBuilder addSecurityValuation(SecurityValuation securityValuation, int _idx) {
			if(this.securityValuation == null){
				this.securityValuation = new ArrayList<>();
			}
			getIndex(this.securityValuation, _idx, () -> securityValuation.toBuilder());
			this.securityValuation.set(_idx, securityValuation.toBuilder());
			return this;
		}
		public SecurityPayoutBuilder addSecurityValuation(List<SecurityValuation> securityValuations) {
			if(this.securityValuation == null){
				this.securityValuation = new ArrayList<>();
			}
			for (SecurityValuation toAdd : securityValuations) {
				this.securityValuation.add(toAdd.toBuilder());
			}
			return this;
		}
		public SecurityPayoutBuilder addSecurityValuationBuilder(SecurityValuation.SecurityValuationBuilder securityValuation) {
			if(this.securityValuation == null){
				this.securityValuation = new ArrayList<>();
			}
			this.securityValuation.add(securityValuation);
			return this;
		}
		
		public SecurityPayoutBuilder clearSecurityValuation() {
			this.securityValuation = null;
			return this;
		}
		
		public SecurityPayout build() {
			return new SecurityPayout(this);
		}
	
		@Override
		public SecurityPayoutBuilder prune() {
			if (initialMargin!=null && !initialMargin.prune().hasData()) initialMargin = null;
			if (meta!=null && !meta.prune().hasData()) meta = null;
			if (securityLeg!=null) securityLeg = securityLeg.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (securityValuation!=null) securityValuation = securityValuation.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getInitialMargin()!=null && getInitialMargin().hasData()) return true;
			if (getMeta()!=null && getMeta().hasData()) return true;
			if (getRepoDuration()!=null) return true;
			if (getSecurityLeg()!=null && getSecurityLeg().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getSecurityValuation()!=null && getSecurityValuation().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("repoDuration"), RepoDurationEnum.class, repoDuration, this);
			
			processRosetta(path.newSubPath("initialMargin"), processor, InitialMargin.class, initialMargin);
			processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
			processRosetta(path.newSubPath("securityLeg"), processor, SecurityLeg.class, securityLeg);
			processRosetta(path.newSubPath("securityValuation"), processor, SecurityValuation.class, securityValuation);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			SecurityPayoutBuilder _that = (SecurityPayoutBuilder) o;
		
			if (initialMargin != null ? !initialMargin.equals(_that.initialMargin) : _that.initialMargin != null) return false;
			if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
			if (repoDuration != null ? !repoDuration.equals(_that.repoDuration) : _that.repoDuration != null) return false;
			if (!ListEquals.listEquals(securityLeg, _that.securityLeg)) return false;
			if (!ListEquals.listEquals(securityValuation, _that.securityValuation)) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (initialMargin != null ? initialMargin.hashCode() : 0);
			_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
			_result = 31 * _result + (repoDuration != null ? repoDuration.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (securityLeg != null ? securityLeg.hashCode() : 0);
			_result = 31 * _result + (securityValuation != null ? securityValuation.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "SecurityPayoutBuilder {" +
				"initialMargin=" + this.initialMargin + ", " +
				"meta=" + this.meta + ", " +
				"repoDuration=" + this.repoDuration + ", " +
				"securityLeg=" + this.securityLeg + ", " +
				"securityValuation=" + this.securityValuation +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("repoDuration"), RepoDurationEnum.class, repoDuration, this);
		
		processRosetta(path.newSubPath("initialMargin"), processor, InitialMargin.class, initialMargin);
		processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
		processRosetta(path.newSubPath("securityLeg"), processor, SecurityLeg.class, securityLeg);
		processRosetta(path.newSubPath("securityValuation"), processor, SecurityValuation.class, securityValuation);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		SecurityPayout _that = (SecurityPayout) o;
	
		if (initialMargin != null ? !initialMargin.equals(_that.initialMargin) : _that.initialMargin != null) return false;
		if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
		if (repoDuration != null ? !repoDuration.equals(_that.repoDuration) : _that.repoDuration != null) return false;
		if (!ListEquals.listEquals(securityLeg, _that.securityLeg)) return false;
		if (!ListEquals.listEquals(securityValuation, _that.securityValuation)) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (initialMargin != null ? initialMargin.hashCode() : 0);
		_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
		_result = 31 * _result + (repoDuration != null ? repoDuration.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (securityLeg != null ? securityLeg.hashCode() : 0);
		_result = 31 * _result + (securityValuation != null ? securityValuation.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "SecurityPayout {" +
			"initialMargin=" + this.initialMargin + ", " +
			"meta=" + this.meta + ", " +
			"repoDuration=" + this.repoDuration + ", " +
			"securityLeg=" + this.securityLeg + ", " +
			"securityValuation=" + this.securityValuation +
		'}';
	}
}
