package org.isda.cdm;

import org.isda.cdm.Security;
import org.isda.cdm.SecurityValuationModel;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.SecurityValuationMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * Terms defining the security valuation method as part of a security leg in a securities fianncing transaction and closely modelled onto the CollateralValuation type in FpML.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class SecurityValuation extends RosettaModelObject {
	private final SecurityValuationModel securityValuationModel;
	private final Security underlier;
	private static SecurityValuationMeta metaData = new SecurityValuationMeta();
	
	SecurityValuation(SecurityValuationBuilder builder) {
		this.securityValuationModel = ofNullable(builder.getSecurityValuationModel()).map(f->f.build()).orElse(null);
		this.underlier = ofNullable(builder.getUnderlier()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The security valuation model choice, based on either a nominal amount or a number of units.
	 */
	@RosettaSynonym(value="collateral", source="FpML_5_10")
	public final SecurityValuationModel getSecurityValuationModel() {
		return securityValuationModel;
	}
	
	/**
	 * The underlying security of the security leg.
	 */
	public final Security getUnderlier() {
		return underlier;
	}
	
	
	@Override
	public RosettaMetaData<? extends SecurityValuation> metaData() {
		return metaData;
	} 
	
	public SecurityValuationBuilder toBuilder() {
		SecurityValuationBuilder builder = new SecurityValuationBuilder();
		ofNullable(getSecurityValuationModel()).ifPresent(builder::setSecurityValuationModel);
		ofNullable(getUnderlier()).ifPresent(builder::setUnderlier);
		return builder;
	}

	public static SecurityValuationBuilder builder() {
		return new SecurityValuationBuilder();
	}

	public static  class SecurityValuationBuilder extends RosettaModelObjectBuilder {
	
		protected SecurityValuationModel.SecurityValuationModelBuilder securityValuationModel;
		protected Security.SecurityBuilder underlier;
	
		public SecurityValuationBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends SecurityValuation> metaData() {
			return metaData;
		} 
	
		
		public SecurityValuationModel.SecurityValuationModelBuilder getSecurityValuationModel() {
			return securityValuationModel;
		}
		
		public SecurityValuationModel.SecurityValuationModelBuilder getOrCreateSecurityValuationModel() {
			if (securityValuationModel!=null) {
				return securityValuationModel;
			}
			else return securityValuationModel = new SecurityValuationModel.SecurityValuationModelBuilder();
		}
		
		
		public Security.SecurityBuilder getUnderlier() {
			return underlier;
		}
		
		public Security.SecurityBuilder getOrCreateUnderlier() {
			if (underlier!=null) {
				return underlier;
			}
			else return underlier = new Security.SecurityBuilder();
		}
		
	
		public SecurityValuationBuilder setSecurityValuationModel(SecurityValuationModel securityValuationModel) {
			this.securityValuationModel = securityValuationModel.toBuilder();
			return this;
		}
		public SecurityValuationBuilder setSecurityValuationModelBuilder(SecurityValuationModel.SecurityValuationModelBuilder securityValuationModel) {
			this.securityValuationModel = securityValuationModel;
			return this;
		}
		public SecurityValuationBuilder setUnderlier(Security underlier) {
			this.underlier = underlier.toBuilder();
			return this;
		}
		public SecurityValuationBuilder setUnderlierBuilder(Security.SecurityBuilder underlier) {
			this.underlier = underlier;
			return this;
		}
		
		public SecurityValuation build() {
			return new SecurityValuation(this);
		}
	
		@Override
		public SecurityValuationBuilder prune() {
			if (securityValuationModel!=null && !securityValuationModel.prune().hasData()) securityValuationModel = null;
			if (underlier!=null && !underlier.prune().hasData()) underlier = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getSecurityValuationModel()!=null && getSecurityValuationModel().hasData()) return true;
			if (getUnderlier()!=null && getUnderlier().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("securityValuationModel"), processor, SecurityValuationModel.class, securityValuationModel);
			processRosetta(path.newSubPath("underlier"), processor, Security.class, underlier);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			SecurityValuationBuilder _that = (SecurityValuationBuilder) o;
		
			if (securityValuationModel != null ? !securityValuationModel.equals(_that.securityValuationModel) : _that.securityValuationModel != null) return false;
			if (underlier != null ? !underlier.equals(_that.underlier) : _that.underlier != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (securityValuationModel != null ? securityValuationModel.hashCode() : 0);
			_result = 31 * _result + (underlier != null ? underlier.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "SecurityValuationBuilder {" +
				"securityValuationModel=" + this.securityValuationModel + ", " +
				"underlier=" + this.underlier +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("securityValuationModel"), processor, SecurityValuationModel.class, securityValuationModel);
		processRosetta(path.newSubPath("underlier"), processor, Security.class, underlier);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		SecurityValuation _that = (SecurityValuation) o;
	
		if (securityValuationModel != null ? !securityValuationModel.equals(_that.securityValuationModel) : _that.securityValuationModel != null) return false;
		if (underlier != null ? !underlier.equals(_that.underlier) : _that.underlier != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (securityValuationModel != null ? securityValuationModel.hashCode() : 0);
		_result = 31 * _result + (underlier != null ? underlier.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "SecurityValuation {" +
			"securityValuationModel=" + this.securityValuationModel + ", " +
			"underlier=" + this.underlier +
		'}';
	}
}
