package org.isda.cdm;

import org.isda.cdm.BondValuationModel;
import org.isda.cdm.UnitContractValuationModel;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.SecurityValuationModelMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * The security valuation model choice, which can either be based on nominal amount as for a bond, or on the number of contract units as for equity.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class SecurityValuationModel extends RosettaModelObject {
	private final BondValuationModel bondValuationModel;
	private final UnitContractValuationModel unitContractValuationModel;
	private static SecurityValuationModelMeta metaData = new SecurityValuationModelMeta();
	
	SecurityValuationModel(SecurityValuationModelBuilder builder) {
		this.bondValuationModel = ofNullable(builder.getBondValuationModel()).map(f->f.build()).orElse(null);
		this.unitContractValuationModel = ofNullable(builder.getUnitContractValuationModel()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The valuation model when the security is a bond.
	 */
	public final BondValuationModel getBondValuationModel() {
		return bondValuationModel;
	}
	
	/**
	 * The valuation model when the security is a unit contract like equity.
	 */
	public final UnitContractValuationModel getUnitContractValuationModel() {
		return unitContractValuationModel;
	}
	
	
	@Override
	public RosettaMetaData<? extends SecurityValuationModel> metaData() {
		return metaData;
	} 
	
	public SecurityValuationModelBuilder toBuilder() {
		SecurityValuationModelBuilder builder = new SecurityValuationModelBuilder();
		ofNullable(getBondValuationModel()).ifPresent(builder::setBondValuationModel);
		ofNullable(getUnitContractValuationModel()).ifPresent(builder::setUnitContractValuationModel);
		return builder;
	}

	public static SecurityValuationModelBuilder builder() {
		return new SecurityValuationModelBuilder();
	}

	public static  class SecurityValuationModelBuilder extends RosettaModelObjectBuilder {
	
		protected BondValuationModel.BondValuationModelBuilder bondValuationModel;
		protected UnitContractValuationModel.UnitContractValuationModelBuilder unitContractValuationModel;
	
		public SecurityValuationModelBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends SecurityValuationModel> metaData() {
			return metaData;
		} 
	
		
		public BondValuationModel.BondValuationModelBuilder getBondValuationModel() {
			return bondValuationModel;
		}
		
		public BondValuationModel.BondValuationModelBuilder getOrCreateBondValuationModel() {
			if (bondValuationModel!=null) {
				return bondValuationModel;
			}
			else return bondValuationModel = new BondValuationModel.BondValuationModelBuilder();
		}
		
		
		public UnitContractValuationModel.UnitContractValuationModelBuilder getUnitContractValuationModel() {
			return unitContractValuationModel;
		}
		
		public UnitContractValuationModel.UnitContractValuationModelBuilder getOrCreateUnitContractValuationModel() {
			if (unitContractValuationModel!=null) {
				return unitContractValuationModel;
			}
			else return unitContractValuationModel = new UnitContractValuationModel.UnitContractValuationModelBuilder();
		}
		
	
		public SecurityValuationModelBuilder setBondValuationModel(BondValuationModel bondValuationModel) {
			this.bondValuationModel = bondValuationModel.toBuilder();
			return this;
		}
		public SecurityValuationModelBuilder setBondValuationModelBuilder(BondValuationModel.BondValuationModelBuilder bondValuationModel) {
			this.bondValuationModel = bondValuationModel;
			return this;
		}
		public SecurityValuationModelBuilder setUnitContractValuationModel(UnitContractValuationModel unitContractValuationModel) {
			this.unitContractValuationModel = unitContractValuationModel.toBuilder();
			return this;
		}
		public SecurityValuationModelBuilder setUnitContractValuationModelBuilder(UnitContractValuationModel.UnitContractValuationModelBuilder unitContractValuationModel) {
			this.unitContractValuationModel = unitContractValuationModel;
			return this;
		}
		
		public SecurityValuationModel build() {
			return new SecurityValuationModel(this);
		}
	
		@Override
		public SecurityValuationModelBuilder prune() {
			if (bondValuationModel!=null && !bondValuationModel.prune().hasData()) bondValuationModel = null;
			if (unitContractValuationModel!=null && !unitContractValuationModel.prune().hasData()) unitContractValuationModel = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getBondValuationModel()!=null && getBondValuationModel().hasData()) return true;
			if (getUnitContractValuationModel()!=null && getUnitContractValuationModel().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("bondValuationModel"), processor, BondValuationModel.class, bondValuationModel);
			processRosetta(path.newSubPath("unitContractValuationModel"), processor, UnitContractValuationModel.class, unitContractValuationModel);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			SecurityValuationModelBuilder _that = (SecurityValuationModelBuilder) o;
		
			if (bondValuationModel != null ? !bondValuationModel.equals(_that.bondValuationModel) : _that.bondValuationModel != null) return false;
			if (unitContractValuationModel != null ? !unitContractValuationModel.equals(_that.unitContractValuationModel) : _that.unitContractValuationModel != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (bondValuationModel != null ? bondValuationModel.hashCode() : 0);
			_result = 31 * _result + (unitContractValuationModel != null ? unitContractValuationModel.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "SecurityValuationModelBuilder {" +
				"bondValuationModel=" + this.bondValuationModel + ", " +
				"unitContractValuationModel=" + this.unitContractValuationModel +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("bondValuationModel"), processor, BondValuationModel.class, bondValuationModel);
		processRosetta(path.newSubPath("unitContractValuationModel"), processor, UnitContractValuationModel.class, unitContractValuationModel);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		SecurityValuationModel _that = (SecurityValuationModel) o;
	
		if (bondValuationModel != null ? !bondValuationModel.equals(_that.bondValuationModel) : _that.bondValuationModel != null) return false;
		if (unitContractValuationModel != null ? !unitContractValuationModel.equals(_that.unitContractValuationModel) : _that.unitContractValuationModel != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (bondValuationModel != null ? bondValuationModel.hashCode() : 0);
		_result = 31 * _result + (unitContractValuationModel != null ? unitContractValuationModel.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "SecurityValuationModel {" +
			"bondValuationModel=" + this.bondValuationModel + ", " +
			"unitContractValuationModel=" + this.unitContractValuationModel +
		'}';
	}
}
