package org.isda.cdm;

import java.lang.String;
import org.isda.cdm.SensitivitiesEnum;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.SensitivityMethodologyMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class to specify the methodology according to which sensitivities to (i) equity indices, funds and ETFs, and (ii) commodity indices are computed. This specification is done either through a normalized election as part of the specifiedMethodology, or through a custom election via the customMethodology attribute. ISDA 2016 Credit Support Annex for Initial Margin, paragraph 13, General Principles, (gg)(2).
 * 
 * @version 2.5.4
 */
@RosettaClass
public class SensitivityMethodology extends RosettaModelObject {
	private final String customMethodology;
	private final SensitivitiesEnum specifiedMethodology;
	private static SensitivityMethodologyMeta metaData = new SensitivityMethodologyMeta();
	
	SensitivityMethodology(SensitivityMethodologyBuilder builder) {
		this.customMethodology = builder.getCustomMethodology();
		this.specifiedMethodology = builder.getSpecifiedMethodology();
	}
	
	/**
	 * The methodology according to which sensitivities will be computed, when specified through a custom election.
	 */
	@RosettaSynonym(value="sensitivities_equity_indices_funds_etfs_specify,sensitivities_to_commodity_indices_specify", source="ISDA_Create_1_0")
	public final String getCustomMethodology() {
		return customMethodology;
	}
	
	/**
	 * The methodology according to which sensitivities will be computed, when specified through a normalized election.
	 */
	@RosettaSynonym(value="sensitivities_equity_indices_funds_etfs,sensitivities_commodity_indices", source="ISDA_Create_1_0")
	public final SensitivitiesEnum getSpecifiedMethodology() {
		return specifiedMethodology;
	}
	
	
	@Override
	public RosettaMetaData<? extends SensitivityMethodology> metaData() {
		return metaData;
	} 
	
	public SensitivityMethodologyBuilder toBuilder() {
		SensitivityMethodologyBuilder builder = new SensitivityMethodologyBuilder();
		ofNullable(getCustomMethodology()).ifPresent(builder::setCustomMethodology);
		ofNullable(getSpecifiedMethodology()).ifPresent(builder::setSpecifiedMethodology);
		return builder;
	}

	public static SensitivityMethodologyBuilder builder() {
		return new SensitivityMethodologyBuilder();
	}

	public static  class SensitivityMethodologyBuilder extends RosettaModelObjectBuilder {
	
		protected String customMethodology;
		protected SensitivitiesEnum specifiedMethodology;
	
		public SensitivityMethodologyBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends SensitivityMethodology> metaData() {
			return metaData;
		} 
	
		
		public String getCustomMethodology() {
			return customMethodology;
		}
		
		
		public SensitivitiesEnum getSpecifiedMethodology() {
			return specifiedMethodology;
		}
		
	
		public SensitivityMethodologyBuilder setCustomMethodology(String customMethodology) {
			this.customMethodology = customMethodology;
			return this;
		}
		public SensitivityMethodologyBuilder setSpecifiedMethodology(SensitivitiesEnum specifiedMethodology) {
			this.specifiedMethodology = specifiedMethodology;
			return this;
		}
		
		public SensitivityMethodology build() {
			return new SensitivityMethodology(this);
		}
	
		@Override
		public SensitivityMethodologyBuilder prune() {
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getCustomMethodology()!=null) return true;
			if (getSpecifiedMethodology()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("customMethodology"), String.class, customMethodology, this);
			processor.processBasic(path.newSubPath("specifiedMethodology"), SensitivitiesEnum.class, specifiedMethodology, this);
			
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			SensitivityMethodologyBuilder _that = (SensitivityMethodologyBuilder) o;
		
			if (customMethodology != null ? !customMethodology.equals(_that.customMethodology) : _that.customMethodology != null) return false;
			if (specifiedMethodology != null ? !specifiedMethodology.equals(_that.specifiedMethodology) : _that.specifiedMethodology != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (customMethodology != null ? customMethodology.hashCode() : 0);
			_result = 31 * _result + (specifiedMethodology != null ? specifiedMethodology.getClass().getName().hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "SensitivityMethodologyBuilder {" +
				"customMethodology=" + this.customMethodology + ", " +
				"specifiedMethodology=" + this.specifiedMethodology +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("customMethodology"), String.class, customMethodology, this);
		processor.processBasic(path.newSubPath("specifiedMethodology"), SensitivitiesEnum.class, specifiedMethodology, this);
		
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		SensitivityMethodology _that = (SensitivityMethodology) o;
	
		if (customMethodology != null ? !customMethodology.equals(_that.customMethodology) : _that.customMethodology != null) return false;
		if (specifiedMethodology != null ? !specifiedMethodology.equals(_that.specifiedMethodology) : _that.specifiedMethodology != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (customMethodology != null ? customMethodology.hashCode() : 0);
		_result = 31 * _result + (specifiedMethodology != null ? specifiedMethodology.getClass().getName().hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "SensitivityMethodology {" +
			"customMethodology=" + this.customMethodology + ", " +
			"specifiedMethodology=" + this.specifiedMethodology +
		'}';
	}
}
