package org.isda.cdm;

import org.isda.cdm.metafields.FieldWithMetaString;
import org.isda.cdm.metafields.FieldWithMetaString.FieldWithMetaStringBuilder;
import org.isda.cdm.metafields.MetaFields;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.SettlementBaseMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A base class to be extended by the SettlementTerms, CashSettlementTerms and PhysicalSettlementTerms classes.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class SettlementBase extends RosettaModelObject implements GlobalKey {
	private final MetaFields meta;
	private final FieldWithMetaString settlementCurrency;
	private static SettlementBaseMeta metaData = new SettlementBaseMeta();
	
	SettlementBase(SettlementBaseBuilder builder) {
		this.meta = ofNullable(builder.getMeta()).map(f->f.build()).orElse(null);
		this.settlementCurrency = ofNullable(builder.getSettlementCurrency()).map(f->f.build()).orElse(null);
	}
	
	public final MetaFields getMeta() {
		return meta;
	}
	
	/**
	 * The settlement currency is to be specified when the Settlement Amount cannot be known in advance. The list of valid currencies is not presently positioned as an enumeration as part of the CDM because that scope is limited to the values specified by ISDA and FpML. As a result, implementers have to make reference to the relevant standard, such as the ISO 4217 standard for currency codes.
	 */
	@RosettaSynonym(value="settlementCurrency", source="FpML_5_10")
	@RosettaSynonym(value="settlementCurrency", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="settlementCurrency", source="DTCC_11_0")
	@RosettaSynonym(value="settlementCurrency", source="DTCC_9_0")
	@RosettaSynonym(value="settlementCurrency", source="CME_ClearedConfirm_1_17")
	public final FieldWithMetaString getSettlementCurrency() {
		return settlementCurrency;
	}
	
	
	@Override
	public RosettaMetaData<? extends SettlementBase> metaData() {
		return metaData;
	} 
	
	public SettlementBaseBuilder toBuilder() {
		SettlementBaseBuilder builder = new SettlementBaseBuilder();
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		ofNullable(getSettlementCurrency()).ifPresent(builder::setSettlementCurrency);
		return builder;
	}

	public static SettlementBaseBuilder builder() {
		return new SettlementBaseBuilder();
	}

	public static  class SettlementBaseBuilder extends RosettaModelObjectBuilder implements GlobalKeyBuilder<SettlementBaseBuilder> {
	
		protected MetaFields.MetaFieldsBuilder meta;
		protected FieldWithMetaStringBuilder settlementCurrency;
	
		public SettlementBaseBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends SettlementBase> metaData() {
			return metaData;
		} 
	
		
		public MetaFields.MetaFieldsBuilder getMeta() {
			return meta;
		}
		
		public MetaFields.MetaFieldsBuilder getOrCreateMeta() {
			if (meta!=null) {
				return meta;
			}
			else return meta = new MetaFields.MetaFieldsBuilder();
		}
		
		
		public FieldWithMetaStringBuilder getSettlementCurrency() {
			return settlementCurrency;
		}
		
		public FieldWithMetaStringBuilder getOrCreateSettlementCurrency() {
			if (settlementCurrency!=null) {
				return settlementCurrency;
			}
			else return settlementCurrency = new FieldWithMetaStringBuilder();
		}
		
	
		public SettlementBaseBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		public SettlementBaseBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		public SettlementBaseBuilder setSettlementCurrency(FieldWithMetaString settlementCurrency) {
			this.settlementCurrency = settlementCurrency.toBuilder();
			return this;
		}
		public SettlementBaseBuilder setSettlementCurrencyRef(String settlementCurrency) {
			return setSettlementCurrency(FieldWithMetaString.builder().setValue(settlementCurrency).build());
		}
		
		public SettlementBase build() {
			return new SettlementBase(this);
		}
	
		@Override
		public SettlementBaseBuilder prune() {
			if (meta!=null && !meta.prune().hasData()) meta = null;
			if (settlementCurrency!=null && !settlementCurrency.prune().hasData()) settlementCurrency = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getMeta()!=null && getMeta().hasData()) return true;
			if (getSettlementCurrency()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
			processRosetta(path.newSubPath("settlementCurrency"), processor, FieldWithMetaString.class, settlementCurrency);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			SettlementBaseBuilder _that = (SettlementBaseBuilder) o;
		
			if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
			if (settlementCurrency != null ? !settlementCurrency.equals(_that.settlementCurrency) : _that.settlementCurrency != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
			_result = 31 * _result + (settlementCurrency != null ? settlementCurrency.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "SettlementBaseBuilder {" +
				"meta=" + this.meta + ", " +
				"settlementCurrency=" + this.settlementCurrency +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
		processRosetta(path.newSubPath("settlementCurrency"), processor, FieldWithMetaString.class, settlementCurrency);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		SettlementBase _that = (SettlementBase) o;
	
		if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
		if (settlementCurrency != null ? !settlementCurrency.equals(_that.settlementCurrency) : _that.settlementCurrency != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
		_result = 31 * _result + (settlementCurrency != null ? settlementCurrency.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "SettlementBase {" +
			"meta=" + this.meta + ", " +
			"settlementCurrency=" + this.settlementCurrency +
		'}';
	}
}
