package org.isda.cdm;

import org.isda.cdm.NonDeliverableSettlement;
import org.isda.cdm.metafields.FieldWithMetaString;
import org.isda.cdm.metafields.FieldWithMetaString.FieldWithMetaStringBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.SettlementProvisionMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class defining the specification of settlement terms, occurring when the settlement currency is different to the notional currency of the trade.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="SettlementProvision", source="FpML_5_10")
public class SettlementProvision extends RosettaModelObject {
	private final NonDeliverableSettlement nonDeliverableSettlement;
	private final FieldWithMetaString settlementCurrency;
	private static SettlementProvisionMeta metaData = new SettlementProvisionMeta();
	
	SettlementProvision(SettlementProvisionBuilder builder) {
		this.nonDeliverableSettlement = ofNullable(builder.getNonDeliverableSettlement()).map(f->f.build()).orElse(null);
		this.settlementCurrency = ofNullable(builder.getSettlementCurrency()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The specification of the non-deliverable settlement provision.
	 */
	@RosettaSynonym(value="nonDeliverableSettlement", source="FpML_5_10")
	@RosettaSynonym(value="nonDeliverableSettlement", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="nonDeliverableSettlement", source="DTCC_11_0")
	@RosettaSynonym(value="nonDeliverableSettlement", source="DTCC_9_0")
	@RosettaSynonym(value="nonDeliverableSettlement", source="CME_ClearedConfirm_1_17")
	public final NonDeliverableSettlement getNonDeliverableSettlement() {
		return nonDeliverableSettlement;
	}
	
	/**
	 * The currency that the payout are settled in when the currency in which the payout is specified is non-deliverable. The list of valid currencies is not presently positioned as an enumeration as part of the CDM because that scope is limited to the values specified by ISDA and FpML. As a result, implementers have to make reference to the relevant standard, such as the ISO 4217 standard for currency codes.
	 */
	@RosettaSynonym(value="settlementCurrency", source="FpML_5_10")
	@RosettaSynonym(value="settlementCurrency", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="settlementCurrency", source="DTCC_11_0")
	@RosettaSynonym(value="settlementCurrency", source="DTCC_9_0")
	@RosettaSynonym(value="settlementCurrency", source="CME_ClearedConfirm_1_17")
	public final FieldWithMetaString getSettlementCurrency() {
		return settlementCurrency;
	}
	
	
	@Override
	public RosettaMetaData<? extends SettlementProvision> metaData() {
		return metaData;
	} 
	
	public SettlementProvisionBuilder toBuilder() {
		SettlementProvisionBuilder builder = new SettlementProvisionBuilder();
		ofNullable(getNonDeliverableSettlement()).ifPresent(builder::setNonDeliverableSettlement);
		ofNullable(getSettlementCurrency()).ifPresent(builder::setSettlementCurrency);
		return builder;
	}

	public static SettlementProvisionBuilder builder() {
		return new SettlementProvisionBuilder();
	}

	public static  class SettlementProvisionBuilder extends RosettaModelObjectBuilder {
	
		protected NonDeliverableSettlement.NonDeliverableSettlementBuilder nonDeliverableSettlement;
		protected FieldWithMetaStringBuilder settlementCurrency;
	
		public SettlementProvisionBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends SettlementProvision> metaData() {
			return metaData;
		} 
	
		
		public NonDeliverableSettlement.NonDeliverableSettlementBuilder getNonDeliverableSettlement() {
			return nonDeliverableSettlement;
		}
		
		public NonDeliverableSettlement.NonDeliverableSettlementBuilder getOrCreateNonDeliverableSettlement() {
			if (nonDeliverableSettlement!=null) {
				return nonDeliverableSettlement;
			}
			else return nonDeliverableSettlement = new NonDeliverableSettlement.NonDeliverableSettlementBuilder();
		}
		
		
		public FieldWithMetaStringBuilder getSettlementCurrency() {
			return settlementCurrency;
		}
		
		public FieldWithMetaStringBuilder getOrCreateSettlementCurrency() {
			if (settlementCurrency!=null) {
				return settlementCurrency;
			}
			else return settlementCurrency = new FieldWithMetaStringBuilder();
		}
		
	
		public SettlementProvisionBuilder setNonDeliverableSettlement(NonDeliverableSettlement nonDeliverableSettlement) {
			this.nonDeliverableSettlement = nonDeliverableSettlement.toBuilder();
			return this;
		}
		public SettlementProvisionBuilder setNonDeliverableSettlementBuilder(NonDeliverableSettlement.NonDeliverableSettlementBuilder nonDeliverableSettlement) {
			this.nonDeliverableSettlement = nonDeliverableSettlement;
			return this;
		}
		public SettlementProvisionBuilder setSettlementCurrency(FieldWithMetaString settlementCurrency) {
			this.settlementCurrency = settlementCurrency.toBuilder();
			return this;
		}
		public SettlementProvisionBuilder setSettlementCurrencyRef(String settlementCurrency) {
			return setSettlementCurrency(FieldWithMetaString.builder().setValue(settlementCurrency).build());
		}
		
		public SettlementProvision build() {
			return new SettlementProvision(this);
		}
	
		@Override
		public SettlementProvisionBuilder prune() {
			if (nonDeliverableSettlement!=null && !nonDeliverableSettlement.prune().hasData()) nonDeliverableSettlement = null;
			if (settlementCurrency!=null && !settlementCurrency.prune().hasData()) settlementCurrency = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getNonDeliverableSettlement()!=null && getNonDeliverableSettlement().hasData()) return true;
			if (getSettlementCurrency()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("nonDeliverableSettlement"), processor, NonDeliverableSettlement.class, nonDeliverableSettlement);
			processRosetta(path.newSubPath("settlementCurrency"), processor, FieldWithMetaString.class, settlementCurrency);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			SettlementProvisionBuilder _that = (SettlementProvisionBuilder) o;
		
			if (nonDeliverableSettlement != null ? !nonDeliverableSettlement.equals(_that.nonDeliverableSettlement) : _that.nonDeliverableSettlement != null) return false;
			if (settlementCurrency != null ? !settlementCurrency.equals(_that.settlementCurrency) : _that.settlementCurrency != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (nonDeliverableSettlement != null ? nonDeliverableSettlement.hashCode() : 0);
			_result = 31 * _result + (settlementCurrency != null ? settlementCurrency.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "SettlementProvisionBuilder {" +
				"nonDeliverableSettlement=" + this.nonDeliverableSettlement + ", " +
				"settlementCurrency=" + this.settlementCurrency +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("nonDeliverableSettlement"), processor, NonDeliverableSettlement.class, nonDeliverableSettlement);
		processRosetta(path.newSubPath("settlementCurrency"), processor, FieldWithMetaString.class, settlementCurrency);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		SettlementProvision _that = (SettlementProvision) o;
	
		if (nonDeliverableSettlement != null ? !nonDeliverableSettlement.equals(_that.nonDeliverableSettlement) : _that.nonDeliverableSettlement != null) return false;
		if (settlementCurrency != null ? !settlementCurrency.equals(_that.settlementCurrency) : _that.settlementCurrency != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (nonDeliverableSettlement != null ? nonDeliverableSettlement.hashCode() : 0);
		_result = 31 * _result + (settlementCurrency != null ? settlementCurrency.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "SettlementProvision {" +
			"nonDeliverableSettlement=" + this.nonDeliverableSettlement + ", " +
			"settlementCurrency=" + this.settlementCurrency +
		'}';
	}
}
