package org.isda.cdm;

import org.isda.cdm.CashSettlementReferenceBanks;
import org.isda.cdm.InformationSource;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.SettlementRateSourceMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class describing the method for obtaining a settlement rate.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="SettlementRateSource", source="FpML_5_10")
public class SettlementRateSource extends RosettaModelObject {
	private final CashSettlementReferenceBanks cashSettlementReferenceBanks;
	private final InformationSource informationSource;
	private static SettlementRateSourceMeta metaData = new SettlementRateSourceMeta();
	
	SettlementRateSource(SettlementRateSourceBuilder builder) {
		this.cashSettlementReferenceBanks = ofNullable(builder.getCashSettlementReferenceBanks()).map(f->f.build()).orElse(null);
		this.informationSource = ofNullable(builder.getInformationSource()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * A container for a set of reference institutions that may be called upon to provide rate quotations as part of the method to determine the applicable cash settlement amount. If institutions are not specified, it is assumed that reference institutions will be agreed between the parties on the exercise date, or in the case of swap transaction to which mandatory early termination is applicable, the cash settlement valuation date.
	 */
	@RosettaSynonym(value="cashSettlementReferenceBanks", source="FpML_5_10")
	@RosettaSynonym(value="cashSettlementReferenceBanks", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="cashSettlementReferenceBanks", source="DTCC_11_0")
	@RosettaSynonym(value="cashSettlementReferenceBanks", source="DTCC_9_0")
	@RosettaSynonym(value="cashSettlementReferenceBanks", source="CME_ClearedConfirm_1_17")
	public final CashSettlementReferenceBanks getCashSettlementReferenceBanks() {
		return cashSettlementReferenceBanks;
	}
	
	/**
	 * The information source where a published or displayed market rate will be obtained, e.g. Telerate Page 3750.
	 */
	@RosettaSynonym(value="informationSource", source="FpML_5_10")
	@RosettaSynonym(value="informationSource", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="informationSource", source="DTCC_11_0")
	@RosettaSynonym(value="informationSource", source="DTCC_9_0")
	@RosettaSynonym(value="informationSource", source="CME_ClearedConfirm_1_17")
	public final InformationSource getInformationSource() {
		return informationSource;
	}
	
	
	@Override
	public RosettaMetaData<? extends SettlementRateSource> metaData() {
		return metaData;
	} 
	
	public SettlementRateSourceBuilder toBuilder() {
		SettlementRateSourceBuilder builder = new SettlementRateSourceBuilder();
		ofNullable(getCashSettlementReferenceBanks()).ifPresent(builder::setCashSettlementReferenceBanks);
		ofNullable(getInformationSource()).ifPresent(builder::setInformationSource);
		return builder;
	}

	public static SettlementRateSourceBuilder builder() {
		return new SettlementRateSourceBuilder();
	}

	public static  class SettlementRateSourceBuilder extends RosettaModelObjectBuilder {
	
		protected CashSettlementReferenceBanks.CashSettlementReferenceBanksBuilder cashSettlementReferenceBanks;
		protected InformationSource.InformationSourceBuilder informationSource;
	
		public SettlementRateSourceBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends SettlementRateSource> metaData() {
			return metaData;
		} 
	
		
		public CashSettlementReferenceBanks.CashSettlementReferenceBanksBuilder getCashSettlementReferenceBanks() {
			return cashSettlementReferenceBanks;
		}
		
		public CashSettlementReferenceBanks.CashSettlementReferenceBanksBuilder getOrCreateCashSettlementReferenceBanks() {
			if (cashSettlementReferenceBanks!=null) {
				return cashSettlementReferenceBanks;
			}
			else return cashSettlementReferenceBanks = new CashSettlementReferenceBanks.CashSettlementReferenceBanksBuilder();
		}
		
		
		public InformationSource.InformationSourceBuilder getInformationSource() {
			return informationSource;
		}
		
		public InformationSource.InformationSourceBuilder getOrCreateInformationSource() {
			if (informationSource!=null) {
				return informationSource;
			}
			else return informationSource = new InformationSource.InformationSourceBuilder();
		}
		
	
		public SettlementRateSourceBuilder setCashSettlementReferenceBanks(CashSettlementReferenceBanks cashSettlementReferenceBanks) {
			this.cashSettlementReferenceBanks = cashSettlementReferenceBanks.toBuilder();
			return this;
		}
		public SettlementRateSourceBuilder setCashSettlementReferenceBanksBuilder(CashSettlementReferenceBanks.CashSettlementReferenceBanksBuilder cashSettlementReferenceBanks) {
			this.cashSettlementReferenceBanks = cashSettlementReferenceBanks;
			return this;
		}
		public SettlementRateSourceBuilder setInformationSource(InformationSource informationSource) {
			this.informationSource = informationSource.toBuilder();
			return this;
		}
		public SettlementRateSourceBuilder setInformationSourceBuilder(InformationSource.InformationSourceBuilder informationSource) {
			this.informationSource = informationSource;
			return this;
		}
		
		public SettlementRateSource build() {
			return new SettlementRateSource(this);
		}
	
		@Override
		public SettlementRateSourceBuilder prune() {
			if (cashSettlementReferenceBanks!=null && !cashSettlementReferenceBanks.prune().hasData()) cashSettlementReferenceBanks = null;
			if (informationSource!=null && !informationSource.prune().hasData()) informationSource = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getCashSettlementReferenceBanks()!=null && getCashSettlementReferenceBanks().hasData()) return true;
			if (getInformationSource()!=null && getInformationSource().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("cashSettlementReferenceBanks"), processor, CashSettlementReferenceBanks.class, cashSettlementReferenceBanks);
			processRosetta(path.newSubPath("informationSource"), processor, InformationSource.class, informationSource);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			SettlementRateSourceBuilder _that = (SettlementRateSourceBuilder) o;
		
			if (cashSettlementReferenceBanks != null ? !cashSettlementReferenceBanks.equals(_that.cashSettlementReferenceBanks) : _that.cashSettlementReferenceBanks != null) return false;
			if (informationSource != null ? !informationSource.equals(_that.informationSource) : _that.informationSource != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (cashSettlementReferenceBanks != null ? cashSettlementReferenceBanks.hashCode() : 0);
			_result = 31 * _result + (informationSource != null ? informationSource.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "SettlementRateSourceBuilder {" +
				"cashSettlementReferenceBanks=" + this.cashSettlementReferenceBanks + ", " +
				"informationSource=" + this.informationSource +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("cashSettlementReferenceBanks"), processor, CashSettlementReferenceBanks.class, cashSettlementReferenceBanks);
		processRosetta(path.newSubPath("informationSource"), processor, InformationSource.class, informationSource);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		SettlementRateSource _that = (SettlementRateSource) o;
	
		if (cashSettlementReferenceBanks != null ? !cashSettlementReferenceBanks.equals(_that.cashSettlementReferenceBanks) : _that.cashSettlementReferenceBanks != null) return false;
		if (informationSource != null ? !informationSource.equals(_that.informationSource) : _that.informationSource != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (cashSettlementReferenceBanks != null ? cashSettlementReferenceBanks.hashCode() : 0);
		_result = 31 * _result + (informationSource != null ? informationSource.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "SettlementRateSource {" +
			"cashSettlementReferenceBanks=" + this.cashSettlementReferenceBanks + ", " +
			"informationSource=" + this.informationSource +
		'}';
	}
}
