package org.isda.cdm;

import com.rosetta.model.lib.records.Date;
import org.isda.cdm.AdjustableOrRelativeDate;
import org.isda.cdm.Money;
import org.isda.cdm.SettlementTypeEnum;
import org.isda.cdm.TransferSettlementEnum;
import org.isda.cdm.metafields.FieldWithMetaString;


import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.SettlementTermsMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;
		import com.rosetta.model.lib.records.Date;

/**
 * A class to specify the settlement terms. This class reflects the FpML OptionSettlement.model, although with no option reference.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class SettlementTerms extends SettlementBase {
	private final Money settlementAmount;
	private final AdjustableOrRelativeDate settlementDate;
	private final SettlementTypeEnum settlementType;
	private final TransferSettlementEnum transferSettlementType;
	private final Date valueDate;
	private static SettlementTermsMeta metaData = new SettlementTermsMeta();
	
	SettlementTerms(SettlementTermsBuilder builder) {
		super(builder);
		this.settlementAmount = ofNullable(builder.getSettlementAmount()).map(f->f.build()).orElse(null);
		this.settlementDate = ofNullable(builder.getSettlementDate()).map(f->f.build()).orElse(null);
		this.settlementType = builder.getSettlementType();
		this.transferSettlementType = builder.getTransferSettlementType();
		this.valueDate = builder.getValueDate();
	}
	
	/**
	 * The Settlement Amount, when known in advance.
	 */
	@RosettaSynonym(value="settlementAmount", source="FpML_5_10")
	@RosettaSynonym(value="settlementAmount", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="settlementAmount", source="DTCC_11_0")
	@RosettaSynonym(value="settlementAmount", source="DTCC_9_0")
	@RosettaSynonym(value="settlementAmount", source="CME_ClearedConfirm_1_17")
	public final Money getSettlementAmount() {
		return settlementAmount;
	}
	
	@RosettaSynonym(value="settlementDate", source="FpML_5_10")
	@RosettaSynonym(value="settlementDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="settlementDate", source="DTCC_11_0")
	@RosettaSynonym(value="settlementDate", source="DTCC_9_0")
	@RosettaSynonym(value="settlementDate", source="CME_ClearedConfirm_1_17")
	public final AdjustableOrRelativeDate getSettlementDate() {
		return settlementDate;
	}
	
	/**
	 * Whether the settlement will be cash, physical, by election, ...
	 */
	@RosettaSynonym(value="settlementType", source="FpML_5_10")
	@RosettaSynonym(value="settlementType", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="settlementType", source="DTCC_11_0")
	@RosettaSynonym(value="settlementType", source="DTCC_9_0")
	@RosettaSynonym(value="settlementType", source="CME_ClearedConfirm_1_17")
	public final SettlementTypeEnum getSettlementType() {
		return settlementType;
	}
	
	/**
	 * The qualification as to how the transfer will settle, e.g. a DvP settlement.
	 */
	public final TransferSettlementEnum getTransferSettlementType() {
		return transferSettlementType;
	}
	
	/**
	 * The settlement date for a forward settling product. For Foreign Exchange contracts, this represents a common settlement date between both currency legs. To specify different settlement dates for each currency leg, see the ForeignExchange class. This attribute is meant to be merged with the &#39;settlementDate&#39; at some future point noce we refactor &#39;Date&#39; to use a single complex type across the model.
	 */
	@RosettaSynonym(value="valueDate", source="FpML_5_10")
	public final Date getValueDate() {
		return valueDate;
	}
	
	
	@Override
	public RosettaMetaData<? extends SettlementTerms> metaData() {
		return metaData;
	} 
	
	public SettlementTermsBuilder toBuilder() {
		SettlementTermsBuilder builder = new SettlementTermsBuilder();
		ofNullable(getSettlementAmount()).ifPresent(builder::setSettlementAmount);
		ofNullable(getSettlementDate()).ifPresent(builder::setSettlementDate);
		ofNullable(getSettlementType()).ifPresent(builder::setSettlementType);
		ofNullable(getTransferSettlementType()).ifPresent(builder::setTransferSettlementType);
		ofNullable(getValueDate()).ifPresent(builder::setValueDate);
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		ofNullable(getSettlementCurrency()).ifPresent(builder::setSettlementCurrency);
		return builder;
	}

	public static SettlementTermsBuilder builder() {
		return new SettlementTermsBuilder();
	}

	public static  class SettlementTermsBuilder extends SettlementBaseBuilder {
	
		protected Money.MoneyBuilder settlementAmount;
		protected AdjustableOrRelativeDate.AdjustableOrRelativeDateBuilder settlementDate;
		protected SettlementTypeEnum settlementType;
		protected TransferSettlementEnum transferSettlementType;
		protected Date valueDate;
	
		public SettlementTermsBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends SettlementTerms> metaData() {
			return metaData;
		} 
	
		
		public Money.MoneyBuilder getSettlementAmount() {
			return settlementAmount;
		}
		
		public Money.MoneyBuilder getOrCreateSettlementAmount() {
			if (settlementAmount!=null) {
				return settlementAmount;
			}
			else return settlementAmount = new Money.MoneyBuilder();
		}
		
		
		public AdjustableOrRelativeDate.AdjustableOrRelativeDateBuilder getSettlementDate() {
			return settlementDate;
		}
		
		public AdjustableOrRelativeDate.AdjustableOrRelativeDateBuilder getOrCreateSettlementDate() {
			if (settlementDate!=null) {
				return settlementDate;
			}
			else return settlementDate = new AdjustableOrRelativeDate.AdjustableOrRelativeDateBuilder();
		}
		
		
		public SettlementTypeEnum getSettlementType() {
			return settlementType;
		}
		
		
		public TransferSettlementEnum getTransferSettlementType() {
			return transferSettlementType;
		}
		
		
		public Date getValueDate() {
			return valueDate;
		}
		
	
		public SettlementTermsBuilder setSettlementAmount(Money settlementAmount) {
			this.settlementAmount = settlementAmount.toBuilder();
			return this;
		}
		public SettlementTermsBuilder setSettlementAmountBuilder(Money.MoneyBuilder settlementAmount) {
			this.settlementAmount = settlementAmount;
			return this;
		}
		public SettlementTermsBuilder setSettlementDate(AdjustableOrRelativeDate settlementDate) {
			this.settlementDate = settlementDate.toBuilder();
			return this;
		}
		public SettlementTermsBuilder setSettlementDateBuilder(AdjustableOrRelativeDate.AdjustableOrRelativeDateBuilder settlementDate) {
			this.settlementDate = settlementDate;
			return this;
		}
		public SettlementTermsBuilder setSettlementType(SettlementTypeEnum settlementType) {
			this.settlementType = settlementType;
			return this;
		}
		public SettlementTermsBuilder setTransferSettlementType(TransferSettlementEnum transferSettlementType) {
			this.transferSettlementType = transferSettlementType;
			return this;
		}
		public SettlementTermsBuilder setValueDate(Date valueDate) {
			this.valueDate = valueDate;
			return this;
		}
		@Override public SettlementTermsBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		@Override public SettlementTermsBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		@Override public SettlementTermsBuilder setSettlementCurrency(FieldWithMetaString settlementCurrency) {
			this.settlementCurrency = settlementCurrency.toBuilder();
			return this;
		}
		@Override public SettlementTermsBuilder setSettlementCurrencyRef(String settlementCurrency) {
			return setSettlementCurrency(FieldWithMetaString.builder().setValue(settlementCurrency).build());
		}
		
		public SettlementTerms build() {
			return new SettlementTerms(this);
		}
	
		@Override
		public SettlementTermsBuilder prune() {
			super.prune();
			if (settlementAmount!=null && !settlementAmount.prune().hasData()) settlementAmount = null;
			if (settlementDate!=null && !settlementDate.prune().hasData()) settlementDate = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (super.hasData()) return true;
			if (getSettlementAmount()!=null && getSettlementAmount().hasData()) return true;
			if (getSettlementDate()!=null && getSettlementDate().hasData()) return true;
			if (getSettlementType()!=null) return true;
			if (getTransferSettlementType()!=null) return true;
			if (getValueDate()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
			super.process(path, processor);
		
			processor.processBasic(path.newSubPath("settlementType"), SettlementTypeEnum.class, settlementType, this);
			processor.processBasic(path.newSubPath("transferSettlementType"), TransferSettlementEnum.class, transferSettlementType, this);
			processor.processBasic(path.newSubPath("valueDate"), Date.class, valueDate, this);
			
			processRosetta(path.newSubPath("settlementAmount"), processor, Money.class, settlementAmount);
			processRosetta(path.newSubPath("settlementDate"), processor, AdjustableOrRelativeDate.class, settlementDate);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
			if (!super.equals(o)) return false;
		
			SettlementTermsBuilder _that = (SettlementTermsBuilder) o;
		
			if (settlementAmount != null ? !settlementAmount.equals(_that.settlementAmount) : _that.settlementAmount != null) return false;
			if (settlementDate != null ? !settlementDate.equals(_that.settlementDate) : _that.settlementDate != null) return false;
			if (settlementType != null ? !settlementType.equals(_that.settlementType) : _that.settlementType != null) return false;
			if (transferSettlementType != null ? !transferSettlementType.equals(_that.transferSettlementType) : _that.transferSettlementType != null) return false;
			if (valueDate != null ? !valueDate.equals(_that.valueDate) : _that.valueDate != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = super.hashCode();
			_result = 31 * _result + (settlementAmount != null ? settlementAmount.hashCode() : 0);
			_result = 31 * _result + (settlementDate != null ? settlementDate.hashCode() : 0);
			_result = 31 * _result + (settlementType != null ? settlementType.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (transferSettlementType != null ? transferSettlementType.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (valueDate != null ? valueDate.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "SettlementTermsBuilder {" +
				"settlementAmount=" + this.settlementAmount + ", " +
				"settlementDate=" + this.settlementDate + ", " +
				"settlementType=" + this.settlementType + ", " +
				"transferSettlementType=" + this.transferSettlementType + ", " +
				"valueDate=" + this.valueDate +
			'}' + " " + super.toString();
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		super.process(path, processor);
		
		processor.processBasic(path.newSubPath("settlementType"), SettlementTypeEnum.class, settlementType, this);
		processor.processBasic(path.newSubPath("transferSettlementType"), TransferSettlementEnum.class, transferSettlementType, this);
		processor.processBasic(path.newSubPath("valueDate"), Date.class, valueDate, this);
		
		processRosetta(path.newSubPath("settlementAmount"), processor, Money.class, settlementAmount);
		processRosetta(path.newSubPath("settlementDate"), processor, AdjustableOrRelativeDate.class, settlementDate);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
		if (!super.equals(o)) return false;
	
		SettlementTerms _that = (SettlementTerms) o;
	
		if (settlementAmount != null ? !settlementAmount.equals(_that.settlementAmount) : _that.settlementAmount != null) return false;
		if (settlementDate != null ? !settlementDate.equals(_that.settlementDate) : _that.settlementDate != null) return false;
		if (settlementType != null ? !settlementType.equals(_that.settlementType) : _that.settlementType != null) return false;
		if (transferSettlementType != null ? !transferSettlementType.equals(_that.transferSettlementType) : _that.transferSettlementType != null) return false;
		if (valueDate != null ? !valueDate.equals(_that.valueDate) : _that.valueDate != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = super.hashCode();
		_result = 31 * _result + (settlementAmount != null ? settlementAmount.hashCode() : 0);
		_result = 31 * _result + (settlementDate != null ? settlementDate.hashCode() : 0);
		_result = 31 * _result + (settlementType != null ? settlementType.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (transferSettlementType != null ? transferSettlementType.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (valueDate != null ? valueDate.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "SettlementTerms {" +
			"settlementAmount=" + this.settlementAmount + ", " +
			"settlementDate=" + this.settlementDate + ", " +
			"settlementType=" + this.settlementType + ", " +
			"transferSettlementType=" + this.transferSettlementType + ", " +
			"valueDate=" + this.valueDate +
		'}' + " " + super.toString();
	}
}
