package org.isda.cdm;

import java.lang.Boolean;
import java.lang.String;
import org.isda.cdm.metafields.FieldWithMetaString;
import org.isda.cdm.metafields.FieldWithMetaString.FieldWithMetaStringBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.SimmCalculationCurrencyMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class to specify the ISDA SIMM Calculation Currency as either the Base Currency or an alternative currency. ISDA 2016 CSA for Initial Margin, Paragraph 13, General Principles (ee)(3). | ISDA 2018 CSA for Initial Margin, Paragraph 13, General Principles (ee)(3).
 * 
 * @version 2.5.4
 */
@RosettaClass
public class SimmCalculationCurrency extends RosettaModelObject {
	private final FieldWithMetaString currency;
	private final Boolean isBaseCurrency;
	private final String party;
	private static SimmCalculationCurrencyMeta metaData = new SimmCalculationCurrencyMeta();
	
	SimmCalculationCurrency(SimmCalculationCurrencyBuilder builder) {
		this.currency = ofNullable(builder.getCurrency()).map(f->f.build()).orElse(null);
		this.isBaseCurrency = builder.getIsBaseCurrency();
		this.party = builder.getParty();
	}
	
	/**
	 * The currency in which the ISDA SIMM Calculation is denominated, when different from the Base Currency. The list of valid currencies is not presently positioned as an enumeration as part of the CDM because that scope is limited to the values specified by ISDA and FpML. As a result, implementers have to make reference to the relevant standard, such as the ISO 4217 standard for currency codes.
	 */
	@RosettaSynonym(value="currency", source="AcadiaSoft_AM_1_0")
	@RosettaSynonym(value="partyA_use_other_currency", source="ISDA_Create_1_0")
	@RosettaSynonym(value="partyB_use_other_currency", source="ISDA_Create_1_0")
	public final FieldWithMetaString getCurrency() {
		return currency;
	}
	
	/**
	 * The SIMM Calculation Currency (also known as SIMM Reporting Currency) means the Base Currency when True. It means a different currency when False. In that latter case, the SIMM Calculation Currency is specified as part of the currency attribute.
	 */
	@RosettaSynonym(value="partyA_use_base_currency", source="ISDA_Create_1_0")
	@RosettaSynonym(value="partyB_use_base_currency", source="ISDA_Create_1_0")
	public final Boolean getIsBaseCurrency() {
		return isBaseCurrency;
	}
	
	/**
	 * The party which the SIMM Calculation Currency qualification applies to.
	 */
	@RosettaSynonym(value="", source="ISDA_Create_1_0")
	@RosettaSynonym(value="", source="ISDA_Create_1_0")
	public final String getParty() {
		return party;
	}
	
	
	@Override
	public RosettaMetaData<? extends SimmCalculationCurrency> metaData() {
		return metaData;
	} 
	
	public SimmCalculationCurrencyBuilder toBuilder() {
		SimmCalculationCurrencyBuilder builder = new SimmCalculationCurrencyBuilder();
		ofNullable(getCurrency()).ifPresent(builder::setCurrency);
		ofNullable(getIsBaseCurrency()).ifPresent(builder::setIsBaseCurrency);
		ofNullable(getParty()).ifPresent(builder::setParty);
		return builder;
	}

	public static SimmCalculationCurrencyBuilder builder() {
		return new SimmCalculationCurrencyBuilder();
	}

	public static  class SimmCalculationCurrencyBuilder extends RosettaModelObjectBuilder {
	
		protected FieldWithMetaStringBuilder currency;
		protected Boolean isBaseCurrency;
		protected String party;
	
		public SimmCalculationCurrencyBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends SimmCalculationCurrency> metaData() {
			return metaData;
		} 
	
		
		public FieldWithMetaStringBuilder getCurrency() {
			return currency;
		}
		
		public FieldWithMetaStringBuilder getOrCreateCurrency() {
			if (currency!=null) {
				return currency;
			}
			else return currency = new FieldWithMetaStringBuilder();
		}
		
		
		public Boolean getIsBaseCurrency() {
			return isBaseCurrency;
		}
		
		
		public String getParty() {
			return party;
		}
		
	
		public SimmCalculationCurrencyBuilder setCurrency(FieldWithMetaString currency) {
			this.currency = currency.toBuilder();
			return this;
		}
		public SimmCalculationCurrencyBuilder setCurrencyRef(String currency) {
			return setCurrency(FieldWithMetaString.builder().setValue(currency).build());
		}
		public SimmCalculationCurrencyBuilder setIsBaseCurrency(Boolean isBaseCurrency) {
			this.isBaseCurrency = isBaseCurrency;
			return this;
		}
		public SimmCalculationCurrencyBuilder setParty(String party) {
			this.party = party;
			return this;
		}
		
		public SimmCalculationCurrency build() {
			return new SimmCalculationCurrency(this);
		}
	
		@Override
		public SimmCalculationCurrencyBuilder prune() {
			if (currency!=null && !currency.prune().hasData()) currency = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getCurrency()!=null) return true;
			if (getIsBaseCurrency()!=null) return true;
			if (getParty()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("isBaseCurrency"), Boolean.class, isBaseCurrency, this);
			processor.processBasic(path.newSubPath("party"), String.class, party, this);
			
			processRosetta(path.newSubPath("currency"), processor, FieldWithMetaString.class, currency);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			SimmCalculationCurrencyBuilder _that = (SimmCalculationCurrencyBuilder) o;
		
			if (currency != null ? !currency.equals(_that.currency) : _that.currency != null) return false;
			if (isBaseCurrency != null ? !isBaseCurrency.equals(_that.isBaseCurrency) : _that.isBaseCurrency != null) return false;
			if (party != null ? !party.equals(_that.party) : _that.party != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (currency != null ? currency.hashCode() : 0);
			_result = 31 * _result + (isBaseCurrency != null ? isBaseCurrency.hashCode() : 0);
			_result = 31 * _result + (party != null ? party.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "SimmCalculationCurrencyBuilder {" +
				"currency=" + this.currency + ", " +
				"isBaseCurrency=" + this.isBaseCurrency + ", " +
				"party=" + this.party +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("isBaseCurrency"), Boolean.class, isBaseCurrency, this);
		processor.processBasic(path.newSubPath("party"), String.class, party, this);
		
		processRosetta(path.newSubPath("currency"), processor, FieldWithMetaString.class, currency);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		SimmCalculationCurrency _that = (SimmCalculationCurrency) o;
	
		if (currency != null ? !currency.equals(_that.currency) : _that.currency != null) return false;
		if (isBaseCurrency != null ? !isBaseCurrency.equals(_that.isBaseCurrency) : _that.isBaseCurrency != null) return false;
		if (party != null ? !party.equals(_that.party) : _that.party != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (currency != null ? currency.hashCode() : 0);
		_result = 31 * _result + (isBaseCurrency != null ? isBaseCurrency.hashCode() : 0);
		_result = 31 * _result + (party != null ? party.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "SimmCalculationCurrency {" +
			"currency=" + this.currency + ", " +
			"isBaseCurrency=" + this.isBaseCurrency + ", " +
			"party=" + this.party +
		'}';
	}
}
