package org.isda.cdm;

import java.lang.String;
import org.isda.cdm.SimmExceptionEnum;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.SimmExceptionMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class to specify the SIMM exception to the regulatory regime clause of the ISDA 2016 and 2018 CSA for Initial Margin as either a normalized value specified as part of an enumeration or a customized value specified of type string. ISDA 2016 Credit Support Annex for Initial Margin paragraph 13, Regime: SIMM Exception.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class SimmException extends RosettaModelObject {
	private final String asSpecified;
	private final SimmExceptionEnum standardisedException;
	private static SimmExceptionMeta metaData = new SimmExceptionMeta();
	
	SimmException(SimmExceptionBuilder builder) {
		this.asSpecified = builder.getAsSpecified();
		this.standardisedException = builder.getStandardisedException();
	}
	
	/**
	 * The Standard Initial Margin Model exception when specified as a customized approach by the party.
	 */
	@RosettaSynonym(value="partyA_SIMM_applicable_specify,partyB_SIMM_applicable_specify", source="ISDA_Create_1_0")
	public final String getAsSpecified() {
		return asSpecified;
	}
	
	/**
	 * The Standard Initial Margin Model exception when specified by the party according to one of the enumerated values.
	 */
	@RosettaSynonym(value="", source="AcadiaSoft_AM_1_0")
	public final SimmExceptionEnum getStandardisedException() {
		return standardisedException;
	}
	
	
	@Override
	public RosettaMetaData<? extends SimmException> metaData() {
		return metaData;
	} 
	
	public SimmExceptionBuilder toBuilder() {
		SimmExceptionBuilder builder = new SimmExceptionBuilder();
		ofNullable(getAsSpecified()).ifPresent(builder::setAsSpecified);
		ofNullable(getStandardisedException()).ifPresent(builder::setStandardisedException);
		return builder;
	}

	public static SimmExceptionBuilder builder() {
		return new SimmExceptionBuilder();
	}

	public static  class SimmExceptionBuilder extends RosettaModelObjectBuilder {
	
		protected String asSpecified;
		protected SimmExceptionEnum standardisedException;
	
		public SimmExceptionBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends SimmException> metaData() {
			return metaData;
		} 
	
		
		public String getAsSpecified() {
			return asSpecified;
		}
		
		
		public SimmExceptionEnum getStandardisedException() {
			return standardisedException;
		}
		
	
		public SimmExceptionBuilder setAsSpecified(String asSpecified) {
			this.asSpecified = asSpecified;
			return this;
		}
		public SimmExceptionBuilder setStandardisedException(SimmExceptionEnum standardisedException) {
			this.standardisedException = standardisedException;
			return this;
		}
		
		public SimmException build() {
			return new SimmException(this);
		}
	
		@Override
		public SimmExceptionBuilder prune() {
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getAsSpecified()!=null) return true;
			if (getStandardisedException()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("asSpecified"), String.class, asSpecified, this);
			processor.processBasic(path.newSubPath("standardisedException"), SimmExceptionEnum.class, standardisedException, this);
			
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			SimmExceptionBuilder _that = (SimmExceptionBuilder) o;
		
			if (asSpecified != null ? !asSpecified.equals(_that.asSpecified) : _that.asSpecified != null) return false;
			if (standardisedException != null ? !standardisedException.equals(_that.standardisedException) : _that.standardisedException != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (asSpecified != null ? asSpecified.hashCode() : 0);
			_result = 31 * _result + (standardisedException != null ? standardisedException.getClass().getName().hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "SimmExceptionBuilder {" +
				"asSpecified=" + this.asSpecified + ", " +
				"standardisedException=" + this.standardisedException +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("asSpecified"), String.class, asSpecified, this);
		processor.processBasic(path.newSubPath("standardisedException"), SimmExceptionEnum.class, standardisedException, this);
		
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		SimmException _that = (SimmException) o;
	
		if (asSpecified != null ? !asSpecified.equals(_that.asSpecified) : _that.asSpecified != null) return false;
		if (standardisedException != null ? !standardisedException.equals(_that.standardisedException) : _that.standardisedException != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (asSpecified != null ? asSpecified.hashCode() : 0);
		_result = 31 * _result + (standardisedException != null ? standardisedException.getClass().getName().hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "SimmException {" +
			"asSpecified=" + this.asSpecified + ", " +
			"standardisedException=" + this.standardisedException +
		'}';
	}
}
