package org.isda.cdm;

import java.lang.Boolean;
import org.isda.cdm.SpecifiedSimmVersion;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.SimmVersionMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class to specify the ISDA SIMM version that applies to the ISDA 2018 CSA for Initial Margin. According to the ISDA 2018 CSA for Initial Margin, Paragraph 13, General Principles (ee) (1) provisions, the SIMM version is either not specified, or references a version used by one of the parties to the agreement.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class SimmVersion extends RosettaModelObject {
	private final Boolean isSpecified;
	private final SpecifiedSimmVersion version;
	private static SimmVersionMeta metaData = new SimmVersionMeta();
	
	SimmVersion(SimmVersionBuilder builder) {
		this.isSpecified = builder.getIsSpecified();
		this.version = ofNullable(builder.getVersion()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * A boolean attribute to determine whether the SIMM version is specified for the purpose of the legal agreement.
	 */
	public final Boolean getIsSpecified() {
		return isSpecified;
	}
	
	/**
	 * When the SIMM version is specified, it will reference the party that uses such version.
	 */
	public final SpecifiedSimmVersion getVersion() {
		return version;
	}
	
	
	@Override
	public RosettaMetaData<? extends SimmVersion> metaData() {
		return metaData;
	} 
	
	public SimmVersionBuilder toBuilder() {
		SimmVersionBuilder builder = new SimmVersionBuilder();
		ofNullable(getIsSpecified()).ifPresent(builder::setIsSpecified);
		ofNullable(getVersion()).ifPresent(builder::setVersion);
		return builder;
	}

	public static SimmVersionBuilder builder() {
		return new SimmVersionBuilder();
	}

	public static  class SimmVersionBuilder extends RosettaModelObjectBuilder {
	
		protected Boolean isSpecified;
		protected SpecifiedSimmVersion.SpecifiedSimmVersionBuilder version;
	
		public SimmVersionBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends SimmVersion> metaData() {
			return metaData;
		} 
	
		
		public Boolean getIsSpecified() {
			return isSpecified;
		}
		
		
		public SpecifiedSimmVersion.SpecifiedSimmVersionBuilder getVersion() {
			return version;
		}
		
		public SpecifiedSimmVersion.SpecifiedSimmVersionBuilder getOrCreateVersion() {
			if (version!=null) {
				return version;
			}
			else return version = new SpecifiedSimmVersion.SpecifiedSimmVersionBuilder();
		}
		
	
		public SimmVersionBuilder setIsSpecified(Boolean isSpecified) {
			this.isSpecified = isSpecified;
			return this;
		}
		public SimmVersionBuilder setVersion(SpecifiedSimmVersion version) {
			this.version = version.toBuilder();
			return this;
		}
		public SimmVersionBuilder setVersionBuilder(SpecifiedSimmVersion.SpecifiedSimmVersionBuilder version) {
			this.version = version;
			return this;
		}
		
		public SimmVersion build() {
			return new SimmVersion(this);
		}
	
		@Override
		public SimmVersionBuilder prune() {
			if (version!=null && !version.prune().hasData()) version = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getIsSpecified()!=null) return true;
			if (getVersion()!=null && getVersion().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("isSpecified"), Boolean.class, isSpecified, this);
			
			processRosetta(path.newSubPath("version"), processor, SpecifiedSimmVersion.class, version);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			SimmVersionBuilder _that = (SimmVersionBuilder) o;
		
			if (isSpecified != null ? !isSpecified.equals(_that.isSpecified) : _that.isSpecified != null) return false;
			if (version != null ? !version.equals(_that.version) : _that.version != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (isSpecified != null ? isSpecified.hashCode() : 0);
			_result = 31 * _result + (version != null ? version.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "SimmVersionBuilder {" +
				"isSpecified=" + this.isSpecified + ", " +
				"version=" + this.version +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("isSpecified"), Boolean.class, isSpecified, this);
		
		processRosetta(path.newSubPath("version"), processor, SpecifiedSimmVersion.class, version);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		SimmVersion _that = (SimmVersion) o;
	
		if (isSpecified != null ? !isSpecified.equals(_that.isSpecified) : _that.isSpecified != null) return false;
		if (version != null ? !version.equals(_that.version) : _that.version != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (isSpecified != null ? isSpecified.hashCode() : 0);
		_result = 31 * _result + (version != null ? version.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "SimmVersion {" +
			"isSpecified=" + this.isSpecified + ", " +
			"version=" + this.version +
		'}';
	}
}
