package org.isda.cdm;

import org.isda.cdm.Product;
import org.isda.cdm.ResolvablePayoutQuantity;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.SingleUnderlierMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="SingleUnderlyer", source="FpML_5_10")
public class SingleUnderlier extends RosettaModelObject {
	private final ResolvablePayoutQuantity quantity;
	private final Product underlyingProduct;
	private static SingleUnderlierMeta metaData = new SingleUnderlierMeta();
	
	SingleUnderlier(SingleUnderlierBuilder builder) {
		this.quantity = ofNullable(builder.getQuantity()).map(f->f.build()).orElse(null);
		this.underlyingProduct = ofNullable(builder.getUnderlyingProduct()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * Quantity of the underlying product in the underlier, as a resolvable type so that it can be specified outside of the product description.
	 */
	public final ResolvablePayoutQuantity getQuantity() {
		return quantity;
	}
	
	/**
	 * Specifies the underlying product. As a difference with FpML, and for the purpose of also supporting swaption and options on CDS through a generic construct, its scope also includes the contract and contractualProduct.
	 */
	public final Product getUnderlyingProduct() {
		return underlyingProduct;
	}
	
	
	@Override
	public RosettaMetaData<? extends SingleUnderlier> metaData() {
		return metaData;
	} 
	
	public SingleUnderlierBuilder toBuilder() {
		SingleUnderlierBuilder builder = new SingleUnderlierBuilder();
		ofNullable(getQuantity()).ifPresent(builder::setQuantity);
		ofNullable(getUnderlyingProduct()).ifPresent(builder::setUnderlyingProduct);
		return builder;
	}

	public static SingleUnderlierBuilder builder() {
		return new SingleUnderlierBuilder();
	}

	public static  class SingleUnderlierBuilder extends RosettaModelObjectBuilder {
	
		protected ResolvablePayoutQuantity.ResolvablePayoutQuantityBuilder quantity;
		protected Product.ProductBuilder underlyingProduct;
	
		public SingleUnderlierBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends SingleUnderlier> metaData() {
			return metaData;
		} 
	
		
		public ResolvablePayoutQuantity.ResolvablePayoutQuantityBuilder getQuantity() {
			return quantity;
		}
		
		public ResolvablePayoutQuantity.ResolvablePayoutQuantityBuilder getOrCreateQuantity() {
			if (quantity!=null) {
				return quantity;
			}
			else return quantity = new ResolvablePayoutQuantity.ResolvablePayoutQuantityBuilder();
		}
		
		
		public Product.ProductBuilder getUnderlyingProduct() {
			return underlyingProduct;
		}
		
		public Product.ProductBuilder getOrCreateUnderlyingProduct() {
			if (underlyingProduct!=null) {
				return underlyingProduct;
			}
			else return underlyingProduct = new Product.ProductBuilder();
		}
		
	
		public SingleUnderlierBuilder setQuantity(ResolvablePayoutQuantity quantity) {
			this.quantity = quantity.toBuilder();
			return this;
		}
		public SingleUnderlierBuilder setQuantityBuilder(ResolvablePayoutQuantity.ResolvablePayoutQuantityBuilder quantity) {
			this.quantity = quantity;
			return this;
		}
		public SingleUnderlierBuilder setUnderlyingProduct(Product underlyingProduct) {
			this.underlyingProduct = underlyingProduct.toBuilder();
			return this;
		}
		public SingleUnderlierBuilder setUnderlyingProductBuilder(Product.ProductBuilder underlyingProduct) {
			this.underlyingProduct = underlyingProduct;
			return this;
		}
		
		public SingleUnderlier build() {
			return new SingleUnderlier(this);
		}
	
		@Override
		public SingleUnderlierBuilder prune() {
			if (quantity!=null && !quantity.prune().hasData()) quantity = null;
			if (underlyingProduct!=null && !underlyingProduct.prune().hasData()) underlyingProduct = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getQuantity()!=null && getQuantity().hasData()) return true;
			if (getUnderlyingProduct()!=null && getUnderlyingProduct().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("quantity"), processor, ResolvablePayoutQuantity.class, quantity);
			processRosetta(path.newSubPath("underlyingProduct"), processor, Product.class, underlyingProduct);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			SingleUnderlierBuilder _that = (SingleUnderlierBuilder) o;
		
			if (quantity != null ? !quantity.equals(_that.quantity) : _that.quantity != null) return false;
			if (underlyingProduct != null ? !underlyingProduct.equals(_that.underlyingProduct) : _that.underlyingProduct != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (quantity != null ? quantity.hashCode() : 0);
			_result = 31 * _result + (underlyingProduct != null ? underlyingProduct.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "SingleUnderlierBuilder {" +
				"quantity=" + this.quantity + ", " +
				"underlyingProduct=" + this.underlyingProduct +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("quantity"), processor, ResolvablePayoutQuantity.class, quantity);
		processRosetta(path.newSubPath("underlyingProduct"), processor, Product.class, underlyingProduct);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		SingleUnderlier _that = (SingleUnderlier) o;
	
		if (quantity != null ? !quantity.equals(_that.quantity) : _that.quantity != null) return false;
		if (underlyingProduct != null ? !underlyingProduct.equals(_that.underlyingProduct) : _that.underlyingProduct != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (quantity != null ? quantity.hashCode() : 0);
		_result = 31 * _result + (underlyingProduct != null ? underlyingProduct.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "SingleUnderlier {" +
			"quantity=" + this.quantity + ", " +
			"underlyingProduct=" + this.underlyingProduct +
		'}';
	}
}
