package org.isda.cdm;

import java.lang.String;
import org.isda.cdm.metafields.ReferenceWithMetaParty;
import org.isda.cdm.metafields.ReferenceWithMetaParty.ReferenceWithMetaPartyBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.SpecifiedSimmVersionMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.stream.Collectors;
		import static java.util.Optional.ofNullable;

/**
 * A class to specify the ISDA SIMM version applicable to one of the parties to the CSA agreement that will then be the relevant version for that CSA.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class SpecifiedSimmVersion extends RosettaModelObject {
	private final ReferenceWithMetaParty party;
	private final String simmVersion;
	private static SpecifiedSimmVersionMeta metaData = new SpecifiedSimmVersionMeta();
	
	SpecifiedSimmVersion(SpecifiedSimmVersionBuilder builder) {
		this.party = ofNullable(builder.getParty()).map(f->f.build()).orElse(null);
		this.simmVersion = builder.getSimmVersion();
	}
	
	/**
	 * The party which the specified SIMM version applies to.
	 */
	public final ReferenceWithMetaParty getParty() {
		return party;
	}
	
	/**
	 * The applicable ISDA SIMM version.
	 */
	public final String getSimmVersion() {
		return simmVersion;
	}
	
	
	@Override
	public RosettaMetaData<? extends SpecifiedSimmVersion> metaData() {
		return metaData;
	} 
	
	public SpecifiedSimmVersionBuilder toBuilder() {
		SpecifiedSimmVersionBuilder builder = new SpecifiedSimmVersionBuilder();
		ofNullable(getParty()).ifPresent(builder::setParty);
		ofNullable(getSimmVersion()).ifPresent(builder::setSimmVersion);
		return builder;
	}

	public static SpecifiedSimmVersionBuilder builder() {
		return new SpecifiedSimmVersionBuilder();
	}

	public static  class SpecifiedSimmVersionBuilder extends RosettaModelObjectBuilder {
	
		protected ReferenceWithMetaPartyBuilder party;
		protected String simmVersion;
	
		public SpecifiedSimmVersionBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends SpecifiedSimmVersion> metaData() {
			return metaData;
		} 
	
		
		public ReferenceWithMetaPartyBuilder getParty() {
			return party;
		}
		
		public ReferenceWithMetaPartyBuilder getOrCreateParty() {
			if (party!=null) {
				return party;
			}
			else return party = new ReferenceWithMetaPartyBuilder();
		}
		
		
		public String getSimmVersion() {
			return simmVersion;
		}
		
	
		public SpecifiedSimmVersionBuilder setParty(ReferenceWithMetaParty party) {
			this.party = party.toBuilder();
			return this;
		}
		public SpecifiedSimmVersionBuilder setPartyBuilder(ReferenceWithMetaPartyBuilder party) {
			this.party = party;
			return this;
		}
		public SpecifiedSimmVersionBuilder setPartyRef(Party.PartyBuilder party) {
			return setParty(ReferenceWithMetaParty.builder().setValueBuilder(party).build());
		}
		public SpecifiedSimmVersionBuilder setPartyRef(Party party) {
			return setPartyRef(Party.builder());
		}
		public SpecifiedSimmVersionBuilder setSimmVersion(String simmVersion) {
			this.simmVersion = simmVersion;
			return this;
		}
		
		public SpecifiedSimmVersion build() {
			return new SpecifiedSimmVersion(this);
		}
	
		@Override
		public SpecifiedSimmVersionBuilder prune() {
			if (party!=null && !party.prune().hasData()) party = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getParty()!=null && getParty().hasData()) return true;
			if (getSimmVersion()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("simmVersion"), String.class, simmVersion, this);
			
			processRosetta(path.newSubPath("party"), processor, ReferenceWithMetaParty.class, party);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			SpecifiedSimmVersionBuilder _that = (SpecifiedSimmVersionBuilder) o;
		
			if (party != null ? !party.equals(_that.party) : _that.party != null) return false;
			if (simmVersion != null ? !simmVersion.equals(_that.simmVersion) : _that.simmVersion != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (party != null ? party.hashCode() : 0);
			_result = 31 * _result + (simmVersion != null ? simmVersion.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "SpecifiedSimmVersionBuilder {" +
				"party=" + this.party + ", " +
				"simmVersion=" + this.simmVersion +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("simmVersion"), String.class, simmVersion, this);
		
		processRosetta(path.newSubPath("party"), processor, ReferenceWithMetaParty.class, party);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		SpecifiedSimmVersion _that = (SpecifiedSimmVersion) o;
	
		if (party != null ? !party.equals(_that.party) : _that.party != null) return false;
		if (simmVersion != null ? !simmVersion.equals(_that.simmVersion) : _that.simmVersion != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (party != null ? party.hashCode() : 0);
		_result = 31 * _result + (simmVersion != null ? simmVersion.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "SpecifiedSimmVersion {" +
			"party=" + this.party + ", " +
			"simmVersion=" + this.simmVersion +
		'}';
	}
}
