package org.isda.cdm;

import org.isda.cdm.CalendarSpread;
import org.isda.cdm.StrikeSpread;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.StrategyFeatureMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class for defining option strategy features.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class StrategyFeature extends RosettaModelObject {
	private final CalendarSpread calendarSpread;
	private final StrikeSpread strikeSpread;
	private static StrategyFeatureMeta metaData = new StrategyFeatureMeta();
	
	StrategyFeature(StrategyFeatureBuilder builder) {
		this.calendarSpread = ofNullable(builder.getCalendarSpread()).map(f->f.build()).orElse(null);
		this.strikeSpread = ofNullable(builder.getStrikeSpread()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * Definition of the later expiration date in a calendar spread.
	 */
	@RosettaSynonym(value="calendarSpread", source="FpML_5_10", path="strategyFeature")
	@RosettaSynonym(value="calendarSpread", source="CME_SubmissionIRS_1_0", path="strategyFeature")
	@RosettaSynonym(value="calendarSpread", source="DTCC_11_0", path="strategyFeature")
	@RosettaSynonym(value="calendarSpread", source="DTCC_9_0", path="strategyFeature")
	@RosettaSynonym(value="calendarSpread", source="CME_ClearedConfirm_1_17", path="strategyFeature")
	public final CalendarSpread getCalendarSpread() {
		return calendarSpread;
	}
	
	/**
	 * Definition of the upper strike in a strike spread.
	 */
	@RosettaSynonym(value="strikeSpread", source="FpML_5_10", path="strategyFeature")
	@RosettaSynonym(value="strikeSpread", source="CME_SubmissionIRS_1_0", path="strategyFeature")
	@RosettaSynonym(value="strikeSpread", source="DTCC_11_0", path="strategyFeature")
	@RosettaSynonym(value="strikeSpread", source="DTCC_9_0", path="strategyFeature")
	@RosettaSynonym(value="strikeSpread", source="CME_ClearedConfirm_1_17", path="strategyFeature")
	public final StrikeSpread getStrikeSpread() {
		return strikeSpread;
	}
	
	
	@Override
	public RosettaMetaData<? extends StrategyFeature> metaData() {
		return metaData;
	} 
	
	public StrategyFeatureBuilder toBuilder() {
		StrategyFeatureBuilder builder = new StrategyFeatureBuilder();
		ofNullable(getCalendarSpread()).ifPresent(builder::setCalendarSpread);
		ofNullable(getStrikeSpread()).ifPresent(builder::setStrikeSpread);
		return builder;
	}

	public static StrategyFeatureBuilder builder() {
		return new StrategyFeatureBuilder();
	}

	public static  class StrategyFeatureBuilder extends RosettaModelObjectBuilder {
	
		protected CalendarSpread.CalendarSpreadBuilder calendarSpread;
		protected StrikeSpread.StrikeSpreadBuilder strikeSpread;
	
		public StrategyFeatureBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends StrategyFeature> metaData() {
			return metaData;
		} 
	
		
		public CalendarSpread.CalendarSpreadBuilder getCalendarSpread() {
			return calendarSpread;
		}
		
		public CalendarSpread.CalendarSpreadBuilder getOrCreateCalendarSpread() {
			if (calendarSpread!=null) {
				return calendarSpread;
			}
			else return calendarSpread = new CalendarSpread.CalendarSpreadBuilder();
		}
		
		
		public StrikeSpread.StrikeSpreadBuilder getStrikeSpread() {
			return strikeSpread;
		}
		
		public StrikeSpread.StrikeSpreadBuilder getOrCreateStrikeSpread() {
			if (strikeSpread!=null) {
				return strikeSpread;
			}
			else return strikeSpread = new StrikeSpread.StrikeSpreadBuilder();
		}
		
	
		public StrategyFeatureBuilder setCalendarSpread(CalendarSpread calendarSpread) {
			this.calendarSpread = calendarSpread.toBuilder();
			return this;
		}
		public StrategyFeatureBuilder setCalendarSpreadBuilder(CalendarSpread.CalendarSpreadBuilder calendarSpread) {
			this.calendarSpread = calendarSpread;
			return this;
		}
		public StrategyFeatureBuilder setStrikeSpread(StrikeSpread strikeSpread) {
			this.strikeSpread = strikeSpread.toBuilder();
			return this;
		}
		public StrategyFeatureBuilder setStrikeSpreadBuilder(StrikeSpread.StrikeSpreadBuilder strikeSpread) {
			this.strikeSpread = strikeSpread;
			return this;
		}
		
		public StrategyFeature build() {
			return new StrategyFeature(this);
		}
	
		@Override
		public StrategyFeatureBuilder prune() {
			if (calendarSpread!=null && !calendarSpread.prune().hasData()) calendarSpread = null;
			if (strikeSpread!=null && !strikeSpread.prune().hasData()) strikeSpread = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getCalendarSpread()!=null && getCalendarSpread().hasData()) return true;
			if (getStrikeSpread()!=null && getStrikeSpread().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("calendarSpread"), processor, CalendarSpread.class, calendarSpread);
			processRosetta(path.newSubPath("strikeSpread"), processor, StrikeSpread.class, strikeSpread);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			StrategyFeatureBuilder _that = (StrategyFeatureBuilder) o;
		
			if (calendarSpread != null ? !calendarSpread.equals(_that.calendarSpread) : _that.calendarSpread != null) return false;
			if (strikeSpread != null ? !strikeSpread.equals(_that.strikeSpread) : _that.strikeSpread != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (calendarSpread != null ? calendarSpread.hashCode() : 0);
			_result = 31 * _result + (strikeSpread != null ? strikeSpread.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "StrategyFeatureBuilder {" +
				"calendarSpread=" + this.calendarSpread + ", " +
				"strikeSpread=" + this.strikeSpread +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("calendarSpread"), processor, CalendarSpread.class, calendarSpread);
		processRosetta(path.newSubPath("strikeSpread"), processor, StrikeSpread.class, strikeSpread);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		StrategyFeature _that = (StrategyFeature) o;
	
		if (calendarSpread != null ? !calendarSpread.equals(_that.calendarSpread) : _that.calendarSpread != null) return false;
		if (strikeSpread != null ? !strikeSpread.equals(_that.strikeSpread) : _that.strikeSpread != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (calendarSpread != null ? calendarSpread.hashCode() : 0);
		_result = 31 * _result + (strikeSpread != null ? strikeSpread.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "StrategyFeature {" +
			"calendarSpread=" + this.calendarSpread + ", " +
			"strikeSpread=" + this.strikeSpread +
		'}';
	}
}
