package org.isda.cdm;

import org.isda.cdm.PayerReceiverEnum;


import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.StrikeScheduleMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.math.BigDecimal;
		import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;

/**
 * A class describing a schedule of cap or floor rates.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="StrikeSchedule", source="FpML_5_10")
public class StrikeSchedule extends Schedule {
	private final PayerReceiverEnum buyer;
	private final PayerReceiverEnum seller;
	private static StrikeScheduleMeta metaData = new StrikeScheduleMeta();
	
	StrikeSchedule(StrikeScheduleBuilder builder) {
		super(builder);
		this.buyer = builder.getBuyer();
		this.seller = builder.getSeller();
	}
	
	/**
	 * The buyer of the option.
	 */
	@RosettaSynonym(value="buyer", source="FpML_5_10")
	@RosettaSynonym(value="buyer", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="buyer", source="DTCC_11_0")
	@RosettaSynonym(value="buyer", source="DTCC_9_0")
	@RosettaSynonym(value="buyer", source="CME_ClearedConfirm_1_17")
	public final PayerReceiverEnum getBuyer() {
		return buyer;
	}
	
	/**
	 * The party that has sold.
	 */
	@RosettaSynonym(value="seller", source="FpML_5_10")
	@RosettaSynonym(value="seller", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="seller", source="DTCC_11_0")
	@RosettaSynonym(value="seller", source="DTCC_9_0")
	@RosettaSynonym(value="seller", source="CME_ClearedConfirm_1_17")
	public final PayerReceiverEnum getSeller() {
		return seller;
	}
	
	
	@Override
	public RosettaMetaData<? extends StrikeSchedule> metaData() {
		return metaData;
	} 
	
	public StrikeScheduleBuilder toBuilder() {
		StrikeScheduleBuilder builder = new StrikeScheduleBuilder();
		ofNullable(getBuyer()).ifPresent(builder::setBuyer);
		ofNullable(getSeller()).ifPresent(builder::setSeller);
		ofNullable(getInitialValue()).ifPresent(builder::setInitialValue);
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		ofNullable(getStep()).ifPresent(step -> step.forEach(builder::addStep));
		return builder;
	}

	public static StrikeScheduleBuilder builder() {
		return new StrikeScheduleBuilder();
	}

	public static  class StrikeScheduleBuilder extends ScheduleBuilder {
	
		protected PayerReceiverEnum buyer;
		protected PayerReceiverEnum seller;
	
		public StrikeScheduleBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends StrikeSchedule> metaData() {
			return metaData;
		} 
	
		
		public PayerReceiverEnum getBuyer() {
			return buyer;
		}
		
		
		public PayerReceiverEnum getSeller() {
			return seller;
		}
		
	
		public StrikeScheduleBuilder setBuyer(PayerReceiverEnum buyer) {
			this.buyer = buyer;
			return this;
		}
		public StrikeScheduleBuilder setSeller(PayerReceiverEnum seller) {
			this.seller = seller;
			return this;
		}
		@Override public StrikeScheduleBuilder setInitialValue(BigDecimal initialValue) {
			this.initialValue = initialValue;
			return this;
		}
		@Override public StrikeScheduleBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		@Override public StrikeScheduleBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		@Override public StrikeScheduleBuilder addStep(Step step) {
			if(this.step == null){
				this.step = new ArrayList<>();
			}
			this.step.add(step.toBuilder());
			return this;
		}
		@Override public StrikeScheduleBuilder addStep(Step step, int _idx) {
			if(this.step == null){
				this.step = new ArrayList<>();
			}
			getIndex(this.step, _idx, () -> step.toBuilder());
			this.step.set(_idx, step.toBuilder());
			return this;
		}
		@Override public StrikeScheduleBuilder addStep(List<Step> steps) {
			if(this.step == null){
				this.step = new ArrayList<>();
			}
			for (Step toAdd : steps) {
				this.step.add(toAdd.toBuilder());
			}
			return this;
		}
		@Override public StrikeScheduleBuilder addStepBuilder(Step.StepBuilder step) {
			if(this.step == null){
				this.step = new ArrayList<>();
			}
			this.step.add(step);
			return this;
		}
		
		@Override public StrikeScheduleBuilder clearStep() {
			this.step = null;
			return this;
		}
		
		public StrikeSchedule build() {
			return new StrikeSchedule(this);
		}
	
		@Override
		public StrikeScheduleBuilder prune() {
			super.prune();
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (super.hasData()) return true;
			if (getBuyer()!=null) return true;
			if (getSeller()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
			super.process(path, processor);
		
			processor.processBasic(path.newSubPath("buyer"), PayerReceiverEnum.class, buyer, this);
			processor.processBasic(path.newSubPath("seller"), PayerReceiverEnum.class, seller, this);
			
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
			if (!super.equals(o)) return false;
		
			StrikeScheduleBuilder _that = (StrikeScheduleBuilder) o;
		
			if (buyer != null ? !buyer.equals(_that.buyer) : _that.buyer != null) return false;
			if (seller != null ? !seller.equals(_that.seller) : _that.seller != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = super.hashCode();
			_result = 31 * _result + (buyer != null ? buyer.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (seller != null ? seller.getClass().getName().hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "StrikeScheduleBuilder {" +
				"buyer=" + this.buyer + ", " +
				"seller=" + this.seller +
			'}' + " " + super.toString();
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		super.process(path, processor);
		
		processor.processBasic(path.newSubPath("buyer"), PayerReceiverEnum.class, buyer, this);
		processor.processBasic(path.newSubPath("seller"), PayerReceiverEnum.class, seller, this);
		
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
		if (!super.equals(o)) return false;
	
		StrikeSchedule _that = (StrikeSchedule) o;
	
		if (buyer != null ? !buyer.equals(_that.buyer) : _that.buyer != null) return false;
		if (seller != null ? !seller.equals(_that.seller) : _that.seller != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = super.hashCode();
		_result = 31 * _result + (buyer != null ? buyer.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (seller != null ? seller.getClass().getName().hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "StrikeSchedule {" +
			"buyer=" + this.buyer + ", " +
			"seller=" + this.seller +
		'}' + " " + super.toString();
	}
}
