package org.isda.cdm;

import java.math.BigDecimal;
import org.isda.cdm.OptionStrike;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.StrikeSpreadMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;
		import java.math.BigDecimal;

/**
 * A class for defining a strike spread feature.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="StrikeSpread", source="FpML_5_10")
public class StrikeSpread extends RosettaModelObject {
	private final OptionStrike upperStrike;
	private final BigDecimal upperStrikeNumberOfOptions;
	private static StrikeSpreadMeta metaData = new StrikeSpreadMeta();
	
	StrikeSpread(StrikeSpreadBuilder builder) {
		this.upperStrike = ofNullable(builder.getUpperStrike()).map(f->f.build()).orElse(null);
		this.upperStrikeNumberOfOptions = builder.getUpperStrikeNumberOfOptions();
	}
	
	/**
	 * Upper strike in a strike spread.
	 */
	@RosettaSynonym(value="upperStrike", source="FpML_5_10")
	@RosettaSynonym(value="upperStrike", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="upperStrike", source="DTCC_11_0")
	@RosettaSynonym(value="upperStrike", source="DTCC_9_0")
	@RosettaSynonym(value="upperStrike", source="CME_ClearedConfirm_1_17")
	public final OptionStrike getUpperStrike() {
		return upperStrike;
	}
	
	/**
	 * Number of options at the upper strike price in a strike spread.
	 */
	@RosettaSynonym(value="upperStrikeNumberOfOptions", source="FpML_5_10")
	@RosettaSynonym(value="upperStrikeNumberOfOptions", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="upperStrikeNumberOfOptions", source="DTCC_11_0")
	@RosettaSynonym(value="upperStrikeNumberOfOptions", source="DTCC_9_0")
	@RosettaSynonym(value="upperStrikeNumberOfOptions", source="CME_ClearedConfirm_1_17")
	public final BigDecimal getUpperStrikeNumberOfOptions() {
		return upperStrikeNumberOfOptions;
	}
	
	
	@Override
	public RosettaMetaData<? extends StrikeSpread> metaData() {
		return metaData;
	} 
	
	public StrikeSpreadBuilder toBuilder() {
		StrikeSpreadBuilder builder = new StrikeSpreadBuilder();
		ofNullable(getUpperStrike()).ifPresent(builder::setUpperStrike);
		ofNullable(getUpperStrikeNumberOfOptions()).ifPresent(builder::setUpperStrikeNumberOfOptions);
		return builder;
	}

	public static StrikeSpreadBuilder builder() {
		return new StrikeSpreadBuilder();
	}

	public static  class StrikeSpreadBuilder extends RosettaModelObjectBuilder {
	
		protected OptionStrike.OptionStrikeBuilder upperStrike;
		protected BigDecimal upperStrikeNumberOfOptions;
	
		public StrikeSpreadBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends StrikeSpread> metaData() {
			return metaData;
		} 
	
		
		public OptionStrike.OptionStrikeBuilder getUpperStrike() {
			return upperStrike;
		}
		
		public OptionStrike.OptionStrikeBuilder getOrCreateUpperStrike() {
			if (upperStrike!=null) {
				return upperStrike;
			}
			else return upperStrike = new OptionStrike.OptionStrikeBuilder();
		}
		
		
		public BigDecimal getUpperStrikeNumberOfOptions() {
			return upperStrikeNumberOfOptions;
		}
		
	
		public StrikeSpreadBuilder setUpperStrike(OptionStrike upperStrike) {
			this.upperStrike = upperStrike.toBuilder();
			return this;
		}
		public StrikeSpreadBuilder setUpperStrikeBuilder(OptionStrike.OptionStrikeBuilder upperStrike) {
			this.upperStrike = upperStrike;
			return this;
		}
		public StrikeSpreadBuilder setUpperStrikeNumberOfOptions(BigDecimal upperStrikeNumberOfOptions) {
			this.upperStrikeNumberOfOptions = upperStrikeNumberOfOptions;
			return this;
		}
		
		public StrikeSpread build() {
			return new StrikeSpread(this);
		}
	
		@Override
		public StrikeSpreadBuilder prune() {
			if (upperStrike!=null && !upperStrike.prune().hasData()) upperStrike = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getUpperStrike()!=null && getUpperStrike().hasData()) return true;
			if (getUpperStrikeNumberOfOptions()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("upperStrikeNumberOfOptions"), BigDecimal.class, upperStrikeNumberOfOptions, this);
			
			processRosetta(path.newSubPath("upperStrike"), processor, OptionStrike.class, upperStrike);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			StrikeSpreadBuilder _that = (StrikeSpreadBuilder) o;
		
			if (upperStrike != null ? !upperStrike.equals(_that.upperStrike) : _that.upperStrike != null) return false;
			if (upperStrikeNumberOfOptions != null ? !upperStrikeNumberOfOptions.equals(_that.upperStrikeNumberOfOptions) : _that.upperStrikeNumberOfOptions != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (upperStrike != null ? upperStrike.hashCode() : 0);
			_result = 31 * _result + (upperStrikeNumberOfOptions != null ? upperStrikeNumberOfOptions.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "StrikeSpreadBuilder {" +
				"upperStrike=" + this.upperStrike + ", " +
				"upperStrikeNumberOfOptions=" + this.upperStrikeNumberOfOptions +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("upperStrikeNumberOfOptions"), BigDecimal.class, upperStrikeNumberOfOptions, this);
		
		processRosetta(path.newSubPath("upperStrike"), processor, OptionStrike.class, upperStrike);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		StrikeSpread _that = (StrikeSpread) o;
	
		if (upperStrike != null ? !upperStrike.equals(_that.upperStrike) : _that.upperStrike != null) return false;
		if (upperStrikeNumberOfOptions != null ? !upperStrikeNumberOfOptions.equals(_that.upperStrikeNumberOfOptions) : _that.upperStrikeNumberOfOptions != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (upperStrike != null ? upperStrike.hashCode() : 0);
		_result = 31 * _result + (upperStrikeNumberOfOptions != null ? upperStrikeNumberOfOptions.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "StrikeSpread {" +
			"upperStrike=" + this.upperStrike + ", " +
			"upperStrikeNumberOfOptions=" + this.upperStrikeNumberOfOptions +
		'}';
	}
}
