package org.isda.cdm;

import org.isda.cdm.StubValue;
import org.isda.cdm.metafields.ReferenceWithMetaCalculationPeriodDates;
import org.isda.cdm.metafields.ReferenceWithMetaCalculationPeriodDates.ReferenceWithMetaCalculationPeriodDatesBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.StubCalculationPeriodAmountMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.stream.Collectors;
		import static java.util.Optional.ofNullable;

/**
 * A class defining how the initial or final stub calculation period amounts is calculated. For example, the rate to be applied to the initial or final stub calculation period may be the linear interpolation of two different tenors for the floating rate index specified in the calculation period amount component, e.g. A two month stub period may used the linear interpolation of a one month and three month floating rate. The different rate tenors would be specified in this component. Note that a maximum of two rate tenors can be specified. If a stub period uses a single index tenor and this is the same as that specified in the calculation period amount component then the initial stub or final stub component, as the case may be, must not be included.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="StubCalculationPeriodAmount", source="FpML_5_10")
public class StubCalculationPeriodAmount extends RosettaModelObject {
	private final ReferenceWithMetaCalculationPeriodDates calculationPeriodDatesReference;
	private final StubValue finalStub;
	private final StubValue initialStub;
	private static StubCalculationPeriodAmountMeta metaData = new StubCalculationPeriodAmountMeta();
	
	StubCalculationPeriodAmount(StubCalculationPeriodAmountBuilder builder) {
		this.calculationPeriodDatesReference = ofNullable(builder.getCalculationPeriodDatesReference()).map(f->f.build()).orElse(null);
		this.finalStub = ofNullable(builder.getFinalStub()).map(f->f.build()).orElse(null);
		this.initialStub = ofNullable(builder.getInitialStub()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * A pointer style reference to the associated calculation period dates component defined elsewhere in the document.
	 */
	@RosettaSynonym(value="calculationPeriodDatesReference", source="FpML_5_10")
	@RosettaSynonym(value="calculationPeriodDatesReference", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="calculationPeriodDatesReference", source="DTCC_11_0")
	@RosettaSynonym(value="calculationPeriodDatesReference", source="DTCC_9_0")
	@RosettaSynonym(value="calculationPeriodDatesReference", source="CME_ClearedConfirm_1_17")
	public final ReferenceWithMetaCalculationPeriodDates getCalculationPeriodDatesReference() {
		return calculationPeriodDatesReference;
	}
	
	/**
	 * Specifies how the final stub amount is calculated. A single floating rate tenor different to that used for the regular part of the calculation periods schedule may be specified, or two floating tenors may be specified. If two floating rate tenors are specified then Linear Interpolation (in accordance with the 2000 ISDA Definitions, Section 8.3. Interpolation) is assumed to apply. Alternatively, an actual known stub rate or stub amount may be specified.
	 */
	@RosettaSynonym(value="finalStub", source="FpML_5_10")
	@RosettaSynonym(value="finalStub", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="finalStub", source="DTCC_11_0")
	@RosettaSynonym(value="finalStub", source="DTCC_9_0")
	@RosettaSynonym(value="finalStub", source="CME_ClearedConfirm_1_17")
	public final StubValue getFinalStub() {
		return finalStub;
	}
	
	/**
	 * Specifies how the initial stub amount is calculated. A single floating rate tenor different to that used for the regular part of the calculation periods schedule may be specified, or two floating tenors may be specified. If two floating rate tenors are specified then Linear Interpolation (in accordance with the 2000 ISDA Definitions, Section 8.3. Interpolation) is assumed to apply. Alternatively, an actual known stub rate or stub amount may be specified.
	 */
	@RosettaSynonym(value="initialStub", source="FpML_5_10")
	@RosettaSynonym(value="initialStub", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="initialStub", source="DTCC_11_0")
	@RosettaSynonym(value="initialStub", source="DTCC_9_0")
	@RosettaSynonym(value="initialStub", source="CME_ClearedConfirm_1_17")
	public final StubValue getInitialStub() {
		return initialStub;
	}
	
	
	@Override
	public RosettaMetaData<? extends StubCalculationPeriodAmount> metaData() {
		return metaData;
	} 
	
	public StubCalculationPeriodAmountBuilder toBuilder() {
		StubCalculationPeriodAmountBuilder builder = new StubCalculationPeriodAmountBuilder();
		ofNullable(getCalculationPeriodDatesReference()).ifPresent(builder::setCalculationPeriodDatesReference);
		ofNullable(getFinalStub()).ifPresent(builder::setFinalStub);
		ofNullable(getInitialStub()).ifPresent(builder::setInitialStub);
		return builder;
	}

	public static StubCalculationPeriodAmountBuilder builder() {
		return new StubCalculationPeriodAmountBuilder();
	}

	public static  class StubCalculationPeriodAmountBuilder extends RosettaModelObjectBuilder {
	
		protected ReferenceWithMetaCalculationPeriodDatesBuilder calculationPeriodDatesReference;
		protected StubValue.StubValueBuilder finalStub;
		protected StubValue.StubValueBuilder initialStub;
	
		public StubCalculationPeriodAmountBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends StubCalculationPeriodAmount> metaData() {
			return metaData;
		} 
	
		
		public ReferenceWithMetaCalculationPeriodDatesBuilder getCalculationPeriodDatesReference() {
			return calculationPeriodDatesReference;
		}
		
		public ReferenceWithMetaCalculationPeriodDatesBuilder getOrCreateCalculationPeriodDatesReference() {
			if (calculationPeriodDatesReference!=null) {
				return calculationPeriodDatesReference;
			}
			else return calculationPeriodDatesReference = new ReferenceWithMetaCalculationPeriodDatesBuilder();
		}
		
		
		public StubValue.StubValueBuilder getFinalStub() {
			return finalStub;
		}
		
		public StubValue.StubValueBuilder getOrCreateFinalStub() {
			if (finalStub!=null) {
				return finalStub;
			}
			else return finalStub = new StubValue.StubValueBuilder();
		}
		
		
		public StubValue.StubValueBuilder getInitialStub() {
			return initialStub;
		}
		
		public StubValue.StubValueBuilder getOrCreateInitialStub() {
			if (initialStub!=null) {
				return initialStub;
			}
			else return initialStub = new StubValue.StubValueBuilder();
		}
		
	
		public StubCalculationPeriodAmountBuilder setCalculationPeriodDatesReference(ReferenceWithMetaCalculationPeriodDates calculationPeriodDatesReference) {
			this.calculationPeriodDatesReference = calculationPeriodDatesReference.toBuilder();
			return this;
		}
		public StubCalculationPeriodAmountBuilder setCalculationPeriodDatesReferenceBuilder(ReferenceWithMetaCalculationPeriodDatesBuilder calculationPeriodDatesReference) {
			this.calculationPeriodDatesReference = calculationPeriodDatesReference;
			return this;
		}
		public StubCalculationPeriodAmountBuilder setCalculationPeriodDatesReferenceRef(CalculationPeriodDates.CalculationPeriodDatesBuilder calculationPeriodDatesReference) {
			return setCalculationPeriodDatesReference(ReferenceWithMetaCalculationPeriodDates.builder().setValueBuilder(calculationPeriodDatesReference).build());
		}
		public StubCalculationPeriodAmountBuilder setCalculationPeriodDatesReferenceRef(CalculationPeriodDates calculationPeriodDatesReference) {
			return setCalculationPeriodDatesReferenceRef(CalculationPeriodDates.builder());
		}
		public StubCalculationPeriodAmountBuilder setFinalStub(StubValue finalStub) {
			this.finalStub = finalStub.toBuilder();
			return this;
		}
		public StubCalculationPeriodAmountBuilder setFinalStubBuilder(StubValue.StubValueBuilder finalStub) {
			this.finalStub = finalStub;
			return this;
		}
		public StubCalculationPeriodAmountBuilder setInitialStub(StubValue initialStub) {
			this.initialStub = initialStub.toBuilder();
			return this;
		}
		public StubCalculationPeriodAmountBuilder setInitialStubBuilder(StubValue.StubValueBuilder initialStub) {
			this.initialStub = initialStub;
			return this;
		}
		
		public StubCalculationPeriodAmount build() {
			return new StubCalculationPeriodAmount(this);
		}
	
		@Override
		public StubCalculationPeriodAmountBuilder prune() {
			if (calculationPeriodDatesReference!=null && !calculationPeriodDatesReference.prune().hasData()) calculationPeriodDatesReference = null;
			if (finalStub!=null && !finalStub.prune().hasData()) finalStub = null;
			if (initialStub!=null && !initialStub.prune().hasData()) initialStub = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getCalculationPeriodDatesReference()!=null && getCalculationPeriodDatesReference().hasData()) return true;
			if (getFinalStub()!=null && getFinalStub().hasData()) return true;
			if (getInitialStub()!=null && getInitialStub().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("calculationPeriodDatesReference"), processor, ReferenceWithMetaCalculationPeriodDates.class, calculationPeriodDatesReference);
			processRosetta(path.newSubPath("finalStub"), processor, StubValue.class, finalStub);
			processRosetta(path.newSubPath("initialStub"), processor, StubValue.class, initialStub);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			StubCalculationPeriodAmountBuilder _that = (StubCalculationPeriodAmountBuilder) o;
		
			if (calculationPeriodDatesReference != null ? !calculationPeriodDatesReference.equals(_that.calculationPeriodDatesReference) : _that.calculationPeriodDatesReference != null) return false;
			if (finalStub != null ? !finalStub.equals(_that.finalStub) : _that.finalStub != null) return false;
			if (initialStub != null ? !initialStub.equals(_that.initialStub) : _that.initialStub != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (calculationPeriodDatesReference != null ? calculationPeriodDatesReference.hashCode() : 0);
			_result = 31 * _result + (finalStub != null ? finalStub.hashCode() : 0);
			_result = 31 * _result + (initialStub != null ? initialStub.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "StubCalculationPeriodAmountBuilder {" +
				"calculationPeriodDatesReference=" + this.calculationPeriodDatesReference + ", " +
				"finalStub=" + this.finalStub + ", " +
				"initialStub=" + this.initialStub +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("calculationPeriodDatesReference"), processor, ReferenceWithMetaCalculationPeriodDates.class, calculationPeriodDatesReference);
		processRosetta(path.newSubPath("finalStub"), processor, StubValue.class, finalStub);
		processRosetta(path.newSubPath("initialStub"), processor, StubValue.class, initialStub);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		StubCalculationPeriodAmount _that = (StubCalculationPeriodAmount) o;
	
		if (calculationPeriodDatesReference != null ? !calculationPeriodDatesReference.equals(_that.calculationPeriodDatesReference) : _that.calculationPeriodDatesReference != null) return false;
		if (finalStub != null ? !finalStub.equals(_that.finalStub) : _that.finalStub != null) return false;
		if (initialStub != null ? !initialStub.equals(_that.initialStub) : _that.initialStub != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (calculationPeriodDatesReference != null ? calculationPeriodDatesReference.hashCode() : 0);
		_result = 31 * _result + (finalStub != null ? finalStub.hashCode() : 0);
		_result = 31 * _result + (initialStub != null ? initialStub.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "StubCalculationPeriodAmount {" +
			"calculationPeriodDatesReference=" + this.calculationPeriodDatesReference + ", " +
			"finalStub=" + this.finalStub + ", " +
			"initialStub=" + this.initialStub +
		'}';
	}
}
