package org.isda.cdm;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.Money;
import org.isda.cdm.StubFloatingRate;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.StubValueMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;
		import java.math.BigDecimal;

/**
 * A type defining how a stub calculation period amount is calculated. A single floating rate tenor different to that used for the regular part of the calculation periods schedule may be specified, or two floating rate tenors many be specified. If two floating rate tenors are specified then Linear Interpolation (in accordance with the 2000 ISDA Definitions, Section 8.3 Interpolation) is assumed to apply. Alternatively, an actual known stub rate or stub amount may be specified.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="StubValue", source="FpML_5_10")
public class StubValue extends RosettaModelObject {
	private final List<StubFloatingRate> floatingRate;
	private final Money stubAmount;
	private final BigDecimal stubRate;
	private static StubValueMeta metaData = new StubValueMeta();
	
	StubValue(StubValueBuilder builder) {
		this.floatingRate = ofNullable(builder.getFloatingRate()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.stubAmount = ofNullable(builder.getStubAmount()).map(f->f.build()).orElse(null);
		this.stubRate = builder.getStubRate();
	}
	
	/**
	 * The rates to be applied to the initial or final stub may be the linear interpolation of two different rates. While the majority of the time, the rate indices will be the same as that specified in the stream and only the tenor itself will be different, it is possible to specift two different rates. For example, a 2 month stub period may use the linear interpolation of a 1 month and 3 month rate. The different rates would be specified in this component. Note that a maximum of two rates can be specified. If a stub period uses the same floating rate index, including tenor, as the regular calculation periods then this should not be specified again within this component, i.e. the stub calculation period amount component may not need to be specified even if there is an initial or final stub period. If a stub period uses a different floating rate index compared to the regular calculation periods then this should be specified within this component. If specified here, they are likely to have id attributes, allowing them to be referenced from within the cashflows component.
	 */
	@RosettaSynonym(value="floatingRate", source="FpML_5_10")
	@RosettaSynonym(value="floatingRate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="floatingRate", source="DTCC_11_0")
	@RosettaSynonym(value="floatingRate", source="DTCC_9_0")
	@RosettaSynonym(value="floatingRate", source="CME_ClearedConfirm_1_17")
	public final List<StubFloatingRate> getFloatingRate() {
		return floatingRate;
	}
	
	/**
	 * An actual amount to apply for the initial or final stub period may have been agreed between the two parties. If an actual stub amount has been agreed then it would be included in this component.
	 */
	@RosettaSynonym(value="stubAmount", source="FpML_5_10")
	@RosettaSynonym(value="stubAmount", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="stubAmount", source="DTCC_11_0")
	@RosettaSynonym(value="stubAmount", source="DTCC_9_0")
	@RosettaSynonym(value="stubAmount", source="CME_ClearedConfirm_1_17")
	public final Money getStubAmount() {
		return stubAmount;
	}
	
	/**
	 * An actual rate to apply for the initial or final stub period may have been agreed between the principal parties (in a similar way to how an initial rate may have been agreed for the first regular period). If an actual stub rate has been agreed then it would be included in this component. It will be a per annum rate, expressed as a decimal. A stub rate of 5% would be represented as 0.05.
	 */
	@RosettaSynonym(value="stubRate", source="FpML_5_10")
	@RosettaSynonym(value="stubRate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="stubRate", source="DTCC_11_0")
	@RosettaSynonym(value="stubRate", source="DTCC_9_0")
	@RosettaSynonym(value="stubRate", source="CME_ClearedConfirm_1_17")
	public final BigDecimal getStubRate() {
		return stubRate;
	}
	
	
	@Override
	public RosettaMetaData<? extends StubValue> metaData() {
		return metaData;
	} 
	
	public StubValueBuilder toBuilder() {
		StubValueBuilder builder = new StubValueBuilder();
		ofNullable(getFloatingRate()).ifPresent(floatingRate -> floatingRate.forEach(builder::addFloatingRate));
		ofNullable(getStubAmount()).ifPresent(builder::setStubAmount);
		ofNullable(getStubRate()).ifPresent(builder::setStubRate);
		return builder;
	}

	public static StubValueBuilder builder() {
		return new StubValueBuilder();
	}

	public static  class StubValueBuilder extends RosettaModelObjectBuilder {
	
		protected List<StubFloatingRate.StubFloatingRateBuilder> floatingRate;
		protected Money.MoneyBuilder stubAmount;
		protected BigDecimal stubRate;
	
		public StubValueBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends StubValue> metaData() {
			return metaData;
		} 
	
		
		public List<StubFloatingRate.StubFloatingRateBuilder> getFloatingRate() {
			return floatingRate;
		}
		
		public StubFloatingRate.StubFloatingRateBuilder getOrCreateFloatingRate(int index) {
			if (floatingRate==null) {
				this.floatingRate = new ArrayList<>();
			}
			return getIndex(floatingRate, index, ()->new StubFloatingRate.StubFloatingRateBuilder());
		}
		
		
		public Money.MoneyBuilder getStubAmount() {
			return stubAmount;
		}
		
		public Money.MoneyBuilder getOrCreateStubAmount() {
			if (stubAmount!=null) {
				return stubAmount;
			}
			else return stubAmount = new Money.MoneyBuilder();
		}
		
		
		public BigDecimal getStubRate() {
			return stubRate;
		}
		
	
		public StubValueBuilder addFloatingRate(StubFloatingRate floatingRate) {
			if(this.floatingRate == null){
				this.floatingRate = new ArrayList<>();
			}
			this.floatingRate.add(floatingRate.toBuilder());
			return this;
		}
		public StubValueBuilder addFloatingRate(StubFloatingRate floatingRate, int _idx) {
			if(this.floatingRate == null){
				this.floatingRate = new ArrayList<>();
			}
			getIndex(this.floatingRate, _idx, () -> floatingRate.toBuilder());
			this.floatingRate.set(_idx, floatingRate.toBuilder());
			return this;
		}
		public StubValueBuilder addFloatingRate(List<StubFloatingRate> floatingRates) {
			if(this.floatingRate == null){
				this.floatingRate = new ArrayList<>();
			}
			for (StubFloatingRate toAdd : floatingRates) {
				this.floatingRate.add(toAdd.toBuilder());
			}
			return this;
		}
		public StubValueBuilder addFloatingRateBuilder(StubFloatingRate.StubFloatingRateBuilder floatingRate) {
			if(this.floatingRate == null){
				this.floatingRate = new ArrayList<>();
			}
			this.floatingRate.add(floatingRate);
			return this;
		}
		
		public StubValueBuilder clearFloatingRate() {
			this.floatingRate = null;
			return this;
		}
		public StubValueBuilder setStubAmount(Money stubAmount) {
			this.stubAmount = stubAmount.toBuilder();
			return this;
		}
		public StubValueBuilder setStubAmountBuilder(Money.MoneyBuilder stubAmount) {
			this.stubAmount = stubAmount;
			return this;
		}
		public StubValueBuilder setStubRate(BigDecimal stubRate) {
			this.stubRate = stubRate;
			return this;
		}
		
		public StubValue build() {
			return new StubValue(this);
		}
	
		@Override
		public StubValueBuilder prune() {
			if (floatingRate!=null) floatingRate = floatingRate.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (stubAmount!=null && !stubAmount.prune().hasData()) stubAmount = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getFloatingRate()!=null && getFloatingRate().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getStubAmount()!=null && getStubAmount().hasData()) return true;
			if (getStubRate()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("stubRate"), BigDecimal.class, stubRate, this);
			
			processRosetta(path.newSubPath("floatingRate"), processor, StubFloatingRate.class, floatingRate);
			processRosetta(path.newSubPath("stubAmount"), processor, Money.class, stubAmount);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			StubValueBuilder _that = (StubValueBuilder) o;
		
			if (!ListEquals.listEquals(floatingRate, _that.floatingRate)) return false;
			if (stubAmount != null ? !stubAmount.equals(_that.stubAmount) : _that.stubAmount != null) return false;
			if (stubRate != null ? !stubRate.equals(_that.stubRate) : _that.stubRate != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (floatingRate != null ? floatingRate.hashCode() : 0);
			_result = 31 * _result + (stubAmount != null ? stubAmount.hashCode() : 0);
			_result = 31 * _result + (stubRate != null ? stubRate.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "StubValueBuilder {" +
				"floatingRate=" + this.floatingRate + ", " +
				"stubAmount=" + this.stubAmount + ", " +
				"stubRate=" + this.stubRate +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("stubRate"), BigDecimal.class, stubRate, this);
		
		processRosetta(path.newSubPath("floatingRate"), processor, StubFloatingRate.class, floatingRate);
		processRosetta(path.newSubPath("stubAmount"), processor, Money.class, stubAmount);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		StubValue _that = (StubValue) o;
	
		if (!ListEquals.listEquals(floatingRate, _that.floatingRate)) return false;
		if (stubAmount != null ? !stubAmount.equals(_that.stubAmount) : _that.stubAmount != null) return false;
		if (stubRate != null ? !stubRate.equals(_that.stubRate) : _that.stubRate != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (floatingRate != null ? floatingRate.hashCode() : 0);
		_result = 31 * _result + (stubAmount != null ? stubAmount.hashCode() : 0);
		_result = 31 * _result + (stubRate != null ? stubRate.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "StubValue {" +
			"floatingRate=" + this.floatingRate + ", " +
			"stubAmount=" + this.stubAmount + ", " +
			"stubRate=" + this.stubRate +
		'}';
	}
}
