package org.isda.cdm;

import java.lang.Boolean;
import java.lang.String;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.SubstitutionMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class to specify the conditions under which the Pledgor can substitute posted collateral. ISDA 2016 Credit Support Annex for Initial Margin, paragraph 13, General Principles, (f): Substitution. | ISDA 2016 Credit Support Annex for Variation Margin, paragraph 13, (f): Substitutions.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class Substitution extends RosettaModelObject {
	private final Boolean needsConsent;
	private final String specificConsentLanguage;
	private static SubstitutionMeta metaData = new SubstitutionMeta();
	
	Substitution(SubstitutionBuilder builder) {
		this.needsConsent = builder.getNeedsConsent();
		this.specificConsentLanguage = builder.getSpecificConsentLanguage();
	}
	
	/**
	 * The election as to whether the Pledgor/Obligor/Chargor must obtain the Secured Party’s consent for any collateral substitution. ISDA 2016 Credit Support Annex for Initial Margin, paragraph 13, General Principles, (f)(ii): Consent. | ISDA 2016 Credit Support Annex for Variation Margin, paragraph 13, (f)(ii): Consent.
	 */
	@RosettaSynonym(value="", source="ISDA_Create_1_0")
	public final Boolean getNeedsConsent() {
		return needsConsent;
	}
	
	/**
	 * Specific consent language might be specified by the parties.
	 */
	@RosettaSynonym(value="specify", source="ISDA_Create_1_0")
	public final String getSpecificConsentLanguage() {
		return specificConsentLanguage;
	}
	
	
	@Override
	public RosettaMetaData<? extends Substitution> metaData() {
		return metaData;
	} 
	
	public SubstitutionBuilder toBuilder() {
		SubstitutionBuilder builder = new SubstitutionBuilder();
		ofNullable(getNeedsConsent()).ifPresent(builder::setNeedsConsent);
		ofNullable(getSpecificConsentLanguage()).ifPresent(builder::setSpecificConsentLanguage);
		return builder;
	}

	public static SubstitutionBuilder builder() {
		return new SubstitutionBuilder();
	}

	public static  class SubstitutionBuilder extends RosettaModelObjectBuilder {
	
		protected Boolean needsConsent;
		protected String specificConsentLanguage;
	
		public SubstitutionBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends Substitution> metaData() {
			return metaData;
		} 
	
		
		public Boolean getNeedsConsent() {
			return needsConsent;
		}
		
		
		public String getSpecificConsentLanguage() {
			return specificConsentLanguage;
		}
		
	
		public SubstitutionBuilder setNeedsConsent(Boolean needsConsent) {
			this.needsConsent = needsConsent;
			return this;
		}
		public SubstitutionBuilder setSpecificConsentLanguage(String specificConsentLanguage) {
			this.specificConsentLanguage = specificConsentLanguage;
			return this;
		}
		
		public Substitution build() {
			return new Substitution(this);
		}
	
		@Override
		public SubstitutionBuilder prune() {
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getNeedsConsent()!=null) return true;
			if (getSpecificConsentLanguage()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("needsConsent"), Boolean.class, needsConsent, this);
			processor.processBasic(path.newSubPath("specificConsentLanguage"), String.class, specificConsentLanguage, this);
			
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			SubstitutionBuilder _that = (SubstitutionBuilder) o;
		
			if (needsConsent != null ? !needsConsent.equals(_that.needsConsent) : _that.needsConsent != null) return false;
			if (specificConsentLanguage != null ? !specificConsentLanguage.equals(_that.specificConsentLanguage) : _that.specificConsentLanguage != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (needsConsent != null ? needsConsent.hashCode() : 0);
			_result = 31 * _result + (specificConsentLanguage != null ? specificConsentLanguage.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "SubstitutionBuilder {" +
				"needsConsent=" + this.needsConsent + ", " +
				"specificConsentLanguage=" + this.specificConsentLanguage +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("needsConsent"), Boolean.class, needsConsent, this);
		processor.processBasic(path.newSubPath("specificConsentLanguage"), String.class, specificConsentLanguage, this);
		
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		Substitution _that = (Substitution) o;
	
		if (needsConsent != null ? !needsConsent.equals(_that.needsConsent) : _that.needsConsent != null) return false;
		if (specificConsentLanguage != null ? !specificConsentLanguage.equals(_that.specificConsentLanguage) : _that.specificConsentLanguage != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (needsConsent != null ? needsConsent.hashCode() : 0);
		_result = 31 * _result + (specificConsentLanguage != null ? specificConsentLanguage.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "Substitution {" +
			"needsConsent=" + this.needsConsent + ", " +
			"specificConsentLanguage=" + this.specificConsentLanguage +
		'}';
	}
}
