package org.isda.cdm;

import java.math.BigDecimal;
import org.isda.cdm.FloatingRateIndexEnum;
import org.isda.cdm.Period;
import org.isda.cdm.QuotationSideEnum;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.SwapCurveValuationMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;
		import java.math.BigDecimal;

/**
 * A class to specify a valuation swap curve, which is used as part of the strike construct for the bond and convertible bond options.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="SwapCurveValuation", source="FpML_5_10")
public class SwapCurveValuation extends RosettaModelObject {
	private final FloatingRateIndexEnum floatingRateIndex;
	private final Period indexTenor;
	private final QuotationSideEnum side;
	private final BigDecimal spread;
	private static SwapCurveValuationMeta metaData = new SwapCurveValuationMeta();
	
	SwapCurveValuation(SwapCurveValuationBuilder builder) {
		this.floatingRateIndex = builder.getFloatingRateIndex();
		this.indexTenor = ofNullable(builder.getIndexTenor()).map(f->f.build()).orElse(null);
		this.side = builder.getSide();
		this.spread = builder.getSpread();
	}
	
	@RosettaSynonym(value="floatingRateIndex", source="FpML_5_10")
	@RosettaSynonym(value="floatingRateIndex", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="floatingRateIndex", source="DTCC_11_0")
	@RosettaSynonym(value="floatingRateIndex", source="DTCC_9_0")
	@RosettaSynonym(value="floatingRateIndex", source="CME_ClearedConfirm_1_17")
	public final FloatingRateIndexEnum getFloatingRateIndex() {
		return floatingRateIndex;
	}
	
	/**
	 * The ISDA Designated Maturity, i.e. the tenor of the floating rate.
	 */
	@RosettaSynonym(value="indexTenor", source="FpML_5_10")
	@RosettaSynonym(value="indexTenor", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="indexTenor", source="DTCC_11_0")
	@RosettaSynonym(value="indexTenor", source="DTCC_9_0")
	@RosettaSynonym(value="indexTenor", source="CME_ClearedConfirm_1_17")
	public final Period getIndexTenor() {
		return indexTenor;
	}
	
	/**
	 * The side (bid/mid/ask) of the measure.
	 */
	@RosettaSynonym(value="side", source="FpML_5_10")
	@RosettaSynonym(value="side", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="side", source="DTCC_11_0")
	@RosettaSynonym(value="side", source="DTCC_9_0")
	@RosettaSynonym(value="side", source="CME_ClearedConfirm_1_17")
	public final QuotationSideEnum getSide() {
		return side;
	}
	
	/**
	 * Spread in basis points over the floating rate index.
	 */
	@RosettaSynonym(value="spread", source="FpML_5_10")
	@RosettaSynonym(value="spread", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="spread", source="DTCC_11_0")
	@RosettaSynonym(value="spread", source="DTCC_9_0")
	@RosettaSynonym(value="spread", source="CME_ClearedConfirm_1_17")
	public final BigDecimal getSpread() {
		return spread;
	}
	
	
	@Override
	public RosettaMetaData<? extends SwapCurveValuation> metaData() {
		return metaData;
	} 
	
	public SwapCurveValuationBuilder toBuilder() {
		SwapCurveValuationBuilder builder = new SwapCurveValuationBuilder();
		ofNullable(getFloatingRateIndex()).ifPresent(builder::setFloatingRateIndex);
		ofNullable(getIndexTenor()).ifPresent(builder::setIndexTenor);
		ofNullable(getSide()).ifPresent(builder::setSide);
		ofNullable(getSpread()).ifPresent(builder::setSpread);
		return builder;
	}

	public static SwapCurveValuationBuilder builder() {
		return new SwapCurveValuationBuilder();
	}

	public static  class SwapCurveValuationBuilder extends RosettaModelObjectBuilder {
	
		protected FloatingRateIndexEnum floatingRateIndex;
		protected Period.PeriodBuilder indexTenor;
		protected QuotationSideEnum side;
		protected BigDecimal spread;
	
		public SwapCurveValuationBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends SwapCurveValuation> metaData() {
			return metaData;
		} 
	
		
		public FloatingRateIndexEnum getFloatingRateIndex() {
			return floatingRateIndex;
		}
		
		
		public Period.PeriodBuilder getIndexTenor() {
			return indexTenor;
		}
		
		public Period.PeriodBuilder getOrCreateIndexTenor() {
			if (indexTenor!=null) {
				return indexTenor;
			}
			else return indexTenor = new Period.PeriodBuilder();
		}
		
		
		public QuotationSideEnum getSide() {
			return side;
		}
		
		
		public BigDecimal getSpread() {
			return spread;
		}
		
	
		public SwapCurveValuationBuilder setFloatingRateIndex(FloatingRateIndexEnum floatingRateIndex) {
			this.floatingRateIndex = floatingRateIndex;
			return this;
		}
		public SwapCurveValuationBuilder setIndexTenor(Period indexTenor) {
			this.indexTenor = indexTenor.toBuilder();
			return this;
		}
		public SwapCurveValuationBuilder setIndexTenorBuilder(Period.PeriodBuilder indexTenor) {
			this.indexTenor = indexTenor;
			return this;
		}
		public SwapCurveValuationBuilder setSide(QuotationSideEnum side) {
			this.side = side;
			return this;
		}
		public SwapCurveValuationBuilder setSpread(BigDecimal spread) {
			this.spread = spread;
			return this;
		}
		
		public SwapCurveValuation build() {
			return new SwapCurveValuation(this);
		}
	
		@Override
		public SwapCurveValuationBuilder prune() {
			if (indexTenor!=null && !indexTenor.prune().hasData()) indexTenor = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getFloatingRateIndex()!=null) return true;
			if (getIndexTenor()!=null && getIndexTenor().hasData()) return true;
			if (getSide()!=null) return true;
			if (getSpread()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("floatingRateIndex"), FloatingRateIndexEnum.class, floatingRateIndex, this);
			processor.processBasic(path.newSubPath("side"), QuotationSideEnum.class, side, this);
			processor.processBasic(path.newSubPath("spread"), BigDecimal.class, spread, this);
			
			processRosetta(path.newSubPath("indexTenor"), processor, Period.class, indexTenor);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			SwapCurveValuationBuilder _that = (SwapCurveValuationBuilder) o;
		
			if (floatingRateIndex != null ? !floatingRateIndex.equals(_that.floatingRateIndex) : _that.floatingRateIndex != null) return false;
			if (indexTenor != null ? !indexTenor.equals(_that.indexTenor) : _that.indexTenor != null) return false;
			if (side != null ? !side.equals(_that.side) : _that.side != null) return false;
			if (spread != null ? !spread.equals(_that.spread) : _that.spread != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (floatingRateIndex != null ? floatingRateIndex.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (indexTenor != null ? indexTenor.hashCode() : 0);
			_result = 31 * _result + (side != null ? side.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (spread != null ? spread.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "SwapCurveValuationBuilder {" +
				"floatingRateIndex=" + this.floatingRateIndex + ", " +
				"indexTenor=" + this.indexTenor + ", " +
				"side=" + this.side + ", " +
				"spread=" + this.spread +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("floatingRateIndex"), FloatingRateIndexEnum.class, floatingRateIndex, this);
		processor.processBasic(path.newSubPath("side"), QuotationSideEnum.class, side, this);
		processor.processBasic(path.newSubPath("spread"), BigDecimal.class, spread, this);
		
		processRosetta(path.newSubPath("indexTenor"), processor, Period.class, indexTenor);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		SwapCurveValuation _that = (SwapCurveValuation) o;
	
		if (floatingRateIndex != null ? !floatingRateIndex.equals(_that.floatingRateIndex) : _that.floatingRateIndex != null) return false;
		if (indexTenor != null ? !indexTenor.equals(_that.indexTenor) : _that.indexTenor != null) return false;
		if (side != null ? !side.equals(_that.side) : _that.side != null) return false;
		if (spread != null ? !spread.equals(_that.spread) : _that.spread != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (floatingRateIndex != null ? floatingRateIndex.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (indexTenor != null ? indexTenor.hashCode() : 0);
		_result = 31 * _result + (side != null ? side.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (spread != null ? spread.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "SwapCurveValuation {" +
			"floatingRateIndex=" + this.floatingRateIndex + ", " +
			"indexTenor=" + this.indexTenor + ", " +
			"side=" + this.side + ", " +
			"spread=" + this.spread +
		'}';
	}
}
