package org.isda.cdm;

import java.lang.Boolean;
import org.isda.cdm.AmendmentEffectiveDate;
import org.isda.cdm.PartyTerminationCurrency;
import org.isda.cdm.metafields.FieldWithMetaString;
import org.isda.cdm.metafields.FieldWithMetaString.FieldWithMetaStringBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.TerminationCurrencyAmendmentMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class to specify the Amendment to Termination Currency elections by the parties to the agreement. ISDA 2016 Credit Support Annex for Initial Margin, paragraph 13, General Principles, (t): Amendment to Termination Currency.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class TerminationCurrencyAmendment extends RosettaModelObject {
	private final FieldWithMetaString bothPartiesCurrency;
	private final AmendmentEffectiveDate effectiveDate;
	private final Boolean isApplicable;
	private final PartyTerminationCurrency terminationCurrency;
	private static TerminationCurrencyAmendmentMeta metaData = new TerminationCurrencyAmendmentMeta();
	
	TerminationCurrencyAmendment(TerminationCurrencyAmendmentBuilder builder) {
		this.bothPartiesCurrency = ofNullable(builder.getBothPartiesCurrency()).map(f->f.build()).orElse(null);
		this.effectiveDate = ofNullable(builder.getEffectiveDate()).map(f->f.build()).orElse(null);
		this.isApplicable = builder.getIsApplicable();
		this.terminationCurrency = ofNullable(builder.getTerminationCurrency()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The currency election in the case of a Termination Event where there are two Affected Parties.
	 */
	@RosettaSynonym(value="both_parties_termination_currency", source="ISDA_Create_1_0")
	public final FieldWithMetaString getBothPartiesCurrency() {
		return bothPartiesCurrency;
	}
	
	/**
	 * The effective date of the Amendment to Termination Currency. This date can be specified as either an actual date, a specific date (e.g. the annex date) or as a custom provision.
	 */
	public final AmendmentEffectiveDate getEffectiveDate() {
		return effectiveDate;
	}
	
	/**
	 * The qualification of whether the Amendment to Termination Currency is deemed applicable by the parties (True) or not (False).
	 */
	@RosettaSynonym(value="", source="ISDA_Create_1_0")
	public final Boolean getIsApplicable() {
		return isApplicable;
	}
	
	/**
	 * The parties&#39; election with respect to the Termination Currency.
	 */
	public final PartyTerminationCurrency getTerminationCurrency() {
		return terminationCurrency;
	}
	
	
	@Override
	public RosettaMetaData<? extends TerminationCurrencyAmendment> metaData() {
		return metaData;
	} 
	
	public TerminationCurrencyAmendmentBuilder toBuilder() {
		TerminationCurrencyAmendmentBuilder builder = new TerminationCurrencyAmendmentBuilder();
		ofNullable(getBothPartiesCurrency()).ifPresent(builder::setBothPartiesCurrency);
		ofNullable(getEffectiveDate()).ifPresent(builder::setEffectiveDate);
		ofNullable(getIsApplicable()).ifPresent(builder::setIsApplicable);
		ofNullable(getTerminationCurrency()).ifPresent(builder::setTerminationCurrency);
		return builder;
	}

	public static TerminationCurrencyAmendmentBuilder builder() {
		return new TerminationCurrencyAmendmentBuilder();
	}

	public static  class TerminationCurrencyAmendmentBuilder extends RosettaModelObjectBuilder {
	
		protected FieldWithMetaStringBuilder bothPartiesCurrency;
		protected AmendmentEffectiveDate.AmendmentEffectiveDateBuilder effectiveDate;
		protected Boolean isApplicable;
		protected PartyTerminationCurrency.PartyTerminationCurrencyBuilder terminationCurrency;
	
		public TerminationCurrencyAmendmentBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends TerminationCurrencyAmendment> metaData() {
			return metaData;
		} 
	
		
		public FieldWithMetaStringBuilder getBothPartiesCurrency() {
			return bothPartiesCurrency;
		}
		
		public FieldWithMetaStringBuilder getOrCreateBothPartiesCurrency() {
			if (bothPartiesCurrency!=null) {
				return bothPartiesCurrency;
			}
			else return bothPartiesCurrency = new FieldWithMetaStringBuilder();
		}
		
		
		public AmendmentEffectiveDate.AmendmentEffectiveDateBuilder getEffectiveDate() {
			return effectiveDate;
		}
		
		public AmendmentEffectiveDate.AmendmentEffectiveDateBuilder getOrCreateEffectiveDate() {
			if (effectiveDate!=null) {
				return effectiveDate;
			}
			else return effectiveDate = new AmendmentEffectiveDate.AmendmentEffectiveDateBuilder();
		}
		
		
		public Boolean getIsApplicable() {
			return isApplicable;
		}
		
		
		public PartyTerminationCurrency.PartyTerminationCurrencyBuilder getTerminationCurrency() {
			return terminationCurrency;
		}
		
		public PartyTerminationCurrency.PartyTerminationCurrencyBuilder getOrCreateTerminationCurrency() {
			if (terminationCurrency!=null) {
				return terminationCurrency;
			}
			else return terminationCurrency = new PartyTerminationCurrency.PartyTerminationCurrencyBuilder();
		}
		
	
		public TerminationCurrencyAmendmentBuilder setBothPartiesCurrency(FieldWithMetaString bothPartiesCurrency) {
			this.bothPartiesCurrency = bothPartiesCurrency.toBuilder();
			return this;
		}
		public TerminationCurrencyAmendmentBuilder setBothPartiesCurrencyRef(String bothPartiesCurrency) {
			return setBothPartiesCurrency(FieldWithMetaString.builder().setValue(bothPartiesCurrency).build());
		}
		public TerminationCurrencyAmendmentBuilder setEffectiveDate(AmendmentEffectiveDate effectiveDate) {
			this.effectiveDate = effectiveDate.toBuilder();
			return this;
		}
		public TerminationCurrencyAmendmentBuilder setEffectiveDateBuilder(AmendmentEffectiveDate.AmendmentEffectiveDateBuilder effectiveDate) {
			this.effectiveDate = effectiveDate;
			return this;
		}
		public TerminationCurrencyAmendmentBuilder setIsApplicable(Boolean isApplicable) {
			this.isApplicable = isApplicable;
			return this;
		}
		public TerminationCurrencyAmendmentBuilder setTerminationCurrency(PartyTerminationCurrency terminationCurrency) {
			this.terminationCurrency = terminationCurrency.toBuilder();
			return this;
		}
		public TerminationCurrencyAmendmentBuilder setTerminationCurrencyBuilder(PartyTerminationCurrency.PartyTerminationCurrencyBuilder terminationCurrency) {
			this.terminationCurrency = terminationCurrency;
			return this;
		}
		
		public TerminationCurrencyAmendment build() {
			return new TerminationCurrencyAmendment(this);
		}
	
		@Override
		public TerminationCurrencyAmendmentBuilder prune() {
			if (bothPartiesCurrency!=null && !bothPartiesCurrency.prune().hasData()) bothPartiesCurrency = null;
			if (effectiveDate!=null && !effectiveDate.prune().hasData()) effectiveDate = null;
			if (terminationCurrency!=null && !terminationCurrency.prune().hasData()) terminationCurrency = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getBothPartiesCurrency()!=null) return true;
			if (getEffectiveDate()!=null && getEffectiveDate().hasData()) return true;
			if (getIsApplicable()!=null) return true;
			if (getTerminationCurrency()!=null && getTerminationCurrency().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("isApplicable"), Boolean.class, isApplicable, this);
			
			processRosetta(path.newSubPath("bothPartiesCurrency"), processor, FieldWithMetaString.class, bothPartiesCurrency);
			processRosetta(path.newSubPath("effectiveDate"), processor, AmendmentEffectiveDate.class, effectiveDate);
			processRosetta(path.newSubPath("terminationCurrency"), processor, PartyTerminationCurrency.class, terminationCurrency);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			TerminationCurrencyAmendmentBuilder _that = (TerminationCurrencyAmendmentBuilder) o;
		
			if (bothPartiesCurrency != null ? !bothPartiesCurrency.equals(_that.bothPartiesCurrency) : _that.bothPartiesCurrency != null) return false;
			if (effectiveDate != null ? !effectiveDate.equals(_that.effectiveDate) : _that.effectiveDate != null) return false;
			if (isApplicable != null ? !isApplicable.equals(_that.isApplicable) : _that.isApplicable != null) return false;
			if (terminationCurrency != null ? !terminationCurrency.equals(_that.terminationCurrency) : _that.terminationCurrency != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (bothPartiesCurrency != null ? bothPartiesCurrency.hashCode() : 0);
			_result = 31 * _result + (effectiveDate != null ? effectiveDate.hashCode() : 0);
			_result = 31 * _result + (isApplicable != null ? isApplicable.hashCode() : 0);
			_result = 31 * _result + (terminationCurrency != null ? terminationCurrency.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "TerminationCurrencyAmendmentBuilder {" +
				"bothPartiesCurrency=" + this.bothPartiesCurrency + ", " +
				"effectiveDate=" + this.effectiveDate + ", " +
				"isApplicable=" + this.isApplicable + ", " +
				"terminationCurrency=" + this.terminationCurrency +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("isApplicable"), Boolean.class, isApplicable, this);
		
		processRosetta(path.newSubPath("bothPartiesCurrency"), processor, FieldWithMetaString.class, bothPartiesCurrency);
		processRosetta(path.newSubPath("effectiveDate"), processor, AmendmentEffectiveDate.class, effectiveDate);
		processRosetta(path.newSubPath("terminationCurrency"), processor, PartyTerminationCurrency.class, terminationCurrency);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		TerminationCurrencyAmendment _that = (TerminationCurrencyAmendment) o;
	
		if (bothPartiesCurrency != null ? !bothPartiesCurrency.equals(_that.bothPartiesCurrency) : _that.bothPartiesCurrency != null) return false;
		if (effectiveDate != null ? !effectiveDate.equals(_that.effectiveDate) : _that.effectiveDate != null) return false;
		if (isApplicable != null ? !isApplicable.equals(_that.isApplicable) : _that.isApplicable != null) return false;
		if (terminationCurrency != null ? !terminationCurrency.equals(_that.terminationCurrency) : _that.terminationCurrency != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (bothPartiesCurrency != null ? bothPartiesCurrency.hashCode() : 0);
		_result = 31 * _result + (effectiveDate != null ? effectiveDate.hashCode() : 0);
		_result = 31 * _result + (isApplicable != null ? isApplicable.hashCode() : 0);
		_result = 31 * _result + (terminationCurrency != null ? terminationCurrency.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "TerminationCurrencyAmendment {" +
			"bothPartiesCurrency=" + this.bothPartiesCurrency + ", " +
			"effectiveDate=" + this.effectiveDate + ", " +
			"isApplicable=" + this.isApplicable + ", " +
			"terminationCurrency=" + this.terminationCurrency +
		'}';
	}
}
