package org.isda.cdm;

import java.lang.String;
import org.isda.cdm.metafields.FieldWithMetaString;
import org.isda.cdm.metafields.FieldWithMetaString.FieldWithMetaStringBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.TerminationCurrencyElectionMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class to specify the Amendment to Termination Currency election by the parties to the agreement. ISDA 2016 Credit Support Annex for Initial Margin, paragraph 13, General Principles, (t) (A) &amp; (B).
 * 
 * @version 2.5.4
 */
@RosettaClass
public class TerminationCurrencyElection extends RosettaModelObject {
	private final FieldWithMetaString currency;
	private final String party;
	private static TerminationCurrencyElectionMeta metaData = new TerminationCurrencyElectionMeta();
	
	TerminationCurrencyElection(TerminationCurrencyElectionBuilder builder) {
		this.currency = ofNullable(builder.getCurrency()).map(f->f.build()).orElse(null);
		this.party = builder.getParty();
	}
	
	/**
	 * The Termination Currency associated with the party that referenced as part of this class. The list of valid currencies is not presently positioned as an enumeration as part of the CDM because that scope is limited to the values specified by ISDA and FpML. As a result, implementers have to make reference to the relevant standard, such as the ISO 4217 standard for currency codes.
	 */
	@RosettaSynonym(value="partyA_termination_currency", source="ISDA_Create_1_0")
	@RosettaSynonym(value="partyB_termination_currency", source="ISDA_Create_1_0")
	public final FieldWithMetaString getCurrency() {
		return currency;
	}
	
	/**
	 * The elective party.
	 */
	@RosettaSynonym(value="", source="ISDA_Create_1_0")
	@RosettaSynonym(value="", source="ISDA_Create_1_0")
	public final String getParty() {
		return party;
	}
	
	
	@Override
	public RosettaMetaData<? extends TerminationCurrencyElection> metaData() {
		return metaData;
	} 
	
	public TerminationCurrencyElectionBuilder toBuilder() {
		TerminationCurrencyElectionBuilder builder = new TerminationCurrencyElectionBuilder();
		ofNullable(getCurrency()).ifPresent(builder::setCurrency);
		ofNullable(getParty()).ifPresent(builder::setParty);
		return builder;
	}

	public static TerminationCurrencyElectionBuilder builder() {
		return new TerminationCurrencyElectionBuilder();
	}

	public static  class TerminationCurrencyElectionBuilder extends RosettaModelObjectBuilder {
	
		protected FieldWithMetaStringBuilder currency;
		protected String party;
	
		public TerminationCurrencyElectionBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends TerminationCurrencyElection> metaData() {
			return metaData;
		} 
	
		
		public FieldWithMetaStringBuilder getCurrency() {
			return currency;
		}
		
		public FieldWithMetaStringBuilder getOrCreateCurrency() {
			if (currency!=null) {
				return currency;
			}
			else return currency = new FieldWithMetaStringBuilder();
		}
		
		
		public String getParty() {
			return party;
		}
		
	
		public TerminationCurrencyElectionBuilder setCurrency(FieldWithMetaString currency) {
			this.currency = currency.toBuilder();
			return this;
		}
		public TerminationCurrencyElectionBuilder setCurrencyRef(String currency) {
			return setCurrency(FieldWithMetaString.builder().setValue(currency).build());
		}
		public TerminationCurrencyElectionBuilder setParty(String party) {
			this.party = party;
			return this;
		}
		
		public TerminationCurrencyElection build() {
			return new TerminationCurrencyElection(this);
		}
	
		@Override
		public TerminationCurrencyElectionBuilder prune() {
			if (currency!=null && !currency.prune().hasData()) currency = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getCurrency()!=null) return true;
			if (getParty()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("party"), String.class, party, this);
			
			processRosetta(path.newSubPath("currency"), processor, FieldWithMetaString.class, currency);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			TerminationCurrencyElectionBuilder _that = (TerminationCurrencyElectionBuilder) o;
		
			if (currency != null ? !currency.equals(_that.currency) : _that.currency != null) return false;
			if (party != null ? !party.equals(_that.party) : _that.party != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (currency != null ? currency.hashCode() : 0);
			_result = 31 * _result + (party != null ? party.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "TerminationCurrencyElectionBuilder {" +
				"currency=" + this.currency + ", " +
				"party=" + this.party +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("party"), String.class, party, this);
		
		processRosetta(path.newSubPath("currency"), processor, FieldWithMetaString.class, currency);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		TerminationCurrencyElection _that = (TerminationCurrencyElection) o;
	
		if (currency != null ? !currency.equals(_that.currency) : _that.currency != null) return false;
		if (party != null ? !party.equals(_that.party) : _that.party != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (currency != null ? currency.hashCode() : 0);
		_result = 31 * _result + (party != null ? party.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "TerminationCurrencyElection {" +
			"currency=" + this.currency + ", " +
			"party=" + this.party +
		'}';
	}
}
