package org.isda.cdm;

import org.isda.cdm.Contract;
import org.isda.cdm.Execution;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.TradeMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class to represent the general trade concept, which can either be an execution or a contract. The execution consists essentially in the economic terms which are agreed between the parties. The contract will further qualify those with the legal entities (think of the allocation case, which execution state can involve the investment adviser rather not the actual funds) while not specify the master agreement or collateral terms which might be associated with the subsequent contract.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class Trade extends RosettaModelObject {
	private final Contract contract;
	private final Execution execution;
	private static TradeMeta metaData = new TradeMeta();
	
	Trade(TradeBuilder builder) {
		this.contract = ofNullable(builder.getContract()).map(f->f.build()).orElse(null);
		this.execution = ofNullable(builder.getExecution()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The contract differs from the execution by the fact that its legal terms are fully specified. This includes the legal entities that are associated to it as well as any associated legal agreement, e.g. master agreement, credit and collateral terms, ... 
	 */
	@RosettaSynonym(value="FpML", source="DTCC_11_0", path="Trade")
	@RosettaSynonym(value="FpML", source="DTCC_9_0", path="Trade")
	@RosettaSynonym(value="TradeMsg", source="DTCC_11_0", path="Manifest")
	@RosettaSynonym(value="TradeMsg", source="DTCC_9_0", path="Manifest")
	@RosettaSynonym(value="ReportingJurisdiction", source="DTCC_11_0", path="ReportingData.ReportingHeader")
	@RosettaSynonym(value="ReportingJurisdiction", source="DTCC_9_0", path="ReportingData.ReportingHeader")
	@RosettaSynonym(value="trade", source="Rosetta_Workbench")
	@RosettaSynonym(value="TrdCaptRpt", source="CME_SubmissionIRS_1_0")
	public final Contract getContract() {
		return contract;
	}
	
	/**
	 * The execution corresponds to economic terms that are agreed between parties, but which legal terms are not yet specified. The execution attribute applies to the post-execution scenario of a product that is subject to the clearing mandate and is then routed to the CCP as an execution.
	 */
	@RosettaSynonym(value="TrdCaptRpt", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="trade", source="Rosetta_Workbench")
	public final Execution getExecution() {
		return execution;
	}
	
	
	@Override
	public RosettaMetaData<? extends Trade> metaData() {
		return metaData;
	} 
	
	public TradeBuilder toBuilder() {
		TradeBuilder builder = new TradeBuilder();
		ofNullable(getContract()).ifPresent(builder::setContract);
		ofNullable(getExecution()).ifPresent(builder::setExecution);
		return builder;
	}

	public static TradeBuilder builder() {
		return new TradeBuilder();
	}

	public static  class TradeBuilder extends RosettaModelObjectBuilder {
	
		protected Contract.ContractBuilder contract;
		protected Execution.ExecutionBuilder execution;
	
		public TradeBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends Trade> metaData() {
			return metaData;
		} 
	
		
		public Contract.ContractBuilder getContract() {
			return contract;
		}
		
		public Contract.ContractBuilder getOrCreateContract() {
			if (contract!=null) {
				return contract;
			}
			else return contract = new Contract.ContractBuilder();
		}
		
		
		public Execution.ExecutionBuilder getExecution() {
			return execution;
		}
		
		public Execution.ExecutionBuilder getOrCreateExecution() {
			if (execution!=null) {
				return execution;
			}
			else return execution = new Execution.ExecutionBuilder();
		}
		
	
		public TradeBuilder setContract(Contract contract) {
			this.contract = contract.toBuilder();
			return this;
		}
		public TradeBuilder setContractBuilder(Contract.ContractBuilder contract) {
			this.contract = contract;
			return this;
		}
		public TradeBuilder setExecution(Execution execution) {
			this.execution = execution.toBuilder();
			return this;
		}
		public TradeBuilder setExecutionBuilder(Execution.ExecutionBuilder execution) {
			this.execution = execution;
			return this;
		}
		
		public Trade build() {
			return new Trade(this);
		}
	
		@Override
		public TradeBuilder prune() {
			if (contract!=null && !contract.prune().hasData()) contract = null;
			if (execution!=null && !execution.prune().hasData()) execution = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getContract()!=null && getContract().hasData()) return true;
			if (getExecution()!=null && getExecution().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("contract"), processor, Contract.class, contract);
			processRosetta(path.newSubPath("execution"), processor, Execution.class, execution);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			TradeBuilder _that = (TradeBuilder) o;
		
			if (contract != null ? !contract.equals(_that.contract) : _that.contract != null) return false;
			if (execution != null ? !execution.equals(_that.execution) : _that.execution != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (contract != null ? contract.hashCode() : 0);
			_result = 31 * _result + (execution != null ? execution.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "TradeBuilder {" +
				"contract=" + this.contract + ", " +
				"execution=" + this.execution +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("contract"), processor, Contract.class, contract);
		processRosetta(path.newSubPath("execution"), processor, Execution.class, execution);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		Trade _that = (Trade) o;
	
		if (contract != null ? !contract.equals(_that.contract) : _that.contract != null) return false;
		if (execution != null ? !execution.equals(_that.execution) : _that.execution != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (contract != null ? contract.hashCode() : 0);
		_result = 31 * _result + (execution != null ? execution.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "Trade {" +
			"contract=" + this.contract + ", " +
			"execution=" + this.execution +
		'}';
	}
}
