package org.isda.cdm;

import java.lang.Boolean;
import java.math.BigDecimal;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.TrancheMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.math.BigDecimal;
		import static java.util.Optional.ofNullable;

/**
 * The class to represent a CDS Tranche.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="Tranche", source="FpML_5_10")
public class Tranche extends RosettaModelObject {
	private final BigDecimal attachmentPoint;
	private final BigDecimal exhaustionPoint;
	private final Boolean incurredRecoveryApplicable;
	private static TrancheMeta metaData = new TrancheMeta();
	
	Tranche(TrancheBuilder builder) {
		this.attachmentPoint = builder.getAttachmentPoint();
		this.exhaustionPoint = builder.getExhaustionPoint();
		this.incurredRecoveryApplicable = builder.getIncurredRecoveryApplicable();
	}
	
	/**
	 * Lower bound percentage of the loss that the Tranche can endure, expressed as a decimal. An attachment point of 5% would be represented as 0.05. The difference between Attachment and Exhaustion points is called the width of the Tranche.
	 */
	@RosettaSynonym(value="attachmentPoint", source="FpML_5_10")
	@RosettaSynonym(value="attachmentPoint", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="attachmentPoint", source="DTCC_11_0")
	@RosettaSynonym(value="attachmentPoint", source="DTCC_9_0")
	@RosettaSynonym(value="attachmentPoint", source="CME_ClearedConfirm_1_17")
	public final BigDecimal getAttachmentPoint() {
		return attachmentPoint;
	}
	
	/**
	 * Upper bound percentage of the loss that the Tranche can endure, expressed as a decimal. An exhaustion point of 5% would be represented as 0.05. The difference between Attachment and Exhaustion points is call the width of the Tranche.
	 */
	@RosettaSynonym(value="exhaustionPoint", source="FpML_5_10")
	@RosettaSynonym(value="exhaustionPoint", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="exhaustionPoint", source="DTCC_11_0")
	@RosettaSynonym(value="exhaustionPoint", source="DTCC_9_0")
	@RosettaSynonym(value="exhaustionPoint", source="CME_ClearedConfirm_1_17")
	public final BigDecimal getExhaustionPoint() {
		return exhaustionPoint;
	}
	
	/**
	 * Outstanding Swap Notional Amount is defined at any time on any day, as the greater of: (a) Zero; If Incurred Recovery Amount Applicable: (b) The Original Swap Notional Amount minus the sum of all Incurred Loss Amounts and all Incurred Recovery Amounts (if any) determined under this Confirmation at or prior to such time.Incurred Recovery Amount not populated: (b) The Original Swap Notional Amount minus the sum of all Incurred Loss Amounts determined under this Confirmation at or prior to such time.
	 */
	@RosettaSynonym(value="incurredRecoveryApplicable", source="FpML_5_10")
	@RosettaSynonym(value="incurredRecoveryApplicable", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="incurredRecoveryApplicable", source="DTCC_11_0")
	@RosettaSynonym(value="incurredRecoveryApplicable", source="DTCC_9_0")
	@RosettaSynonym(value="incurredRecoveryApplicable", source="CME_ClearedConfirm_1_17")
	public final Boolean getIncurredRecoveryApplicable() {
		return incurredRecoveryApplicable;
	}
	
	
	@Override
	public RosettaMetaData<? extends Tranche> metaData() {
		return metaData;
	} 
	
	public TrancheBuilder toBuilder() {
		TrancheBuilder builder = new TrancheBuilder();
		ofNullable(getAttachmentPoint()).ifPresent(builder::setAttachmentPoint);
		ofNullable(getExhaustionPoint()).ifPresent(builder::setExhaustionPoint);
		ofNullable(getIncurredRecoveryApplicable()).ifPresent(builder::setIncurredRecoveryApplicable);
		return builder;
	}

	public static TrancheBuilder builder() {
		return new TrancheBuilder();
	}

	public static  class TrancheBuilder extends RosettaModelObjectBuilder {
	
		protected BigDecimal attachmentPoint;
		protected BigDecimal exhaustionPoint;
		protected Boolean incurredRecoveryApplicable;
	
		public TrancheBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends Tranche> metaData() {
			return metaData;
		} 
	
		
		public BigDecimal getAttachmentPoint() {
			return attachmentPoint;
		}
		
		
		public BigDecimal getExhaustionPoint() {
			return exhaustionPoint;
		}
		
		
		public Boolean getIncurredRecoveryApplicable() {
			return incurredRecoveryApplicable;
		}
		
	
		public TrancheBuilder setAttachmentPoint(BigDecimal attachmentPoint) {
			this.attachmentPoint = attachmentPoint;
			return this;
		}
		public TrancheBuilder setExhaustionPoint(BigDecimal exhaustionPoint) {
			this.exhaustionPoint = exhaustionPoint;
			return this;
		}
		public TrancheBuilder setIncurredRecoveryApplicable(Boolean incurredRecoveryApplicable) {
			this.incurredRecoveryApplicable = incurredRecoveryApplicable;
			return this;
		}
		
		public Tranche build() {
			return new Tranche(this);
		}
	
		@Override
		public TrancheBuilder prune() {
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getAttachmentPoint()!=null) return true;
			if (getExhaustionPoint()!=null) return true;
			if (getIncurredRecoveryApplicable()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("attachmentPoint"), BigDecimal.class, attachmentPoint, this);
			processor.processBasic(path.newSubPath("exhaustionPoint"), BigDecimal.class, exhaustionPoint, this);
			processor.processBasic(path.newSubPath("incurredRecoveryApplicable"), Boolean.class, incurredRecoveryApplicable, this);
			
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			TrancheBuilder _that = (TrancheBuilder) o;
		
			if (attachmentPoint != null ? !attachmentPoint.equals(_that.attachmentPoint) : _that.attachmentPoint != null) return false;
			if (exhaustionPoint != null ? !exhaustionPoint.equals(_that.exhaustionPoint) : _that.exhaustionPoint != null) return false;
			if (incurredRecoveryApplicable != null ? !incurredRecoveryApplicable.equals(_that.incurredRecoveryApplicable) : _that.incurredRecoveryApplicable != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (attachmentPoint != null ? attachmentPoint.hashCode() : 0);
			_result = 31 * _result + (exhaustionPoint != null ? exhaustionPoint.hashCode() : 0);
			_result = 31 * _result + (incurredRecoveryApplicable != null ? incurredRecoveryApplicable.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "TrancheBuilder {" +
				"attachmentPoint=" + this.attachmentPoint + ", " +
				"exhaustionPoint=" + this.exhaustionPoint + ", " +
				"incurredRecoveryApplicable=" + this.incurredRecoveryApplicable +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("attachmentPoint"), BigDecimal.class, attachmentPoint, this);
		processor.processBasic(path.newSubPath("exhaustionPoint"), BigDecimal.class, exhaustionPoint, this);
		processor.processBasic(path.newSubPath("incurredRecoveryApplicable"), Boolean.class, incurredRecoveryApplicable, this);
		
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		Tranche _that = (Tranche) o;
	
		if (attachmentPoint != null ? !attachmentPoint.equals(_that.attachmentPoint) : _that.attachmentPoint != null) return false;
		if (exhaustionPoint != null ? !exhaustionPoint.equals(_that.exhaustionPoint) : _that.exhaustionPoint != null) return false;
		if (incurredRecoveryApplicable != null ? !incurredRecoveryApplicable.equals(_that.incurredRecoveryApplicable) : _that.incurredRecoveryApplicable != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (attachmentPoint != null ? attachmentPoint.hashCode() : 0);
		_result = 31 * _result + (exhaustionPoint != null ? exhaustionPoint.hashCode() : 0);
		_result = 31 * _result + (incurredRecoveryApplicable != null ? incurredRecoveryApplicable.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "Tranche {" +
			"attachmentPoint=" + this.attachmentPoint + ", " +
			"exhaustionPoint=" + this.exhaustionPoint + ", " +
			"incurredRecoveryApplicable=" + this.incurredRecoveryApplicable +
		'}';
	}
}
