package org.isda.cdm;

import java.math.BigDecimal;
import org.isda.cdm.QuotationStyleEnum;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.TransactedPriceMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.math.BigDecimal;
		import static java.util.Optional.ofNullable;

/**
 * A class to represent the transacted price attributes that are positioned as part of the FpML FeeLeg.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class TransactedPrice extends RosettaModelObject {
	private final BigDecimal initialPoints;
	private final BigDecimal marketFixedRate;
	private final BigDecimal marketPrice;
	private final QuotationStyleEnum quotationStyle;
	private static TransactedPriceMeta metaData = new TransactedPriceMeta();
	
	TransactedPrice(TransactedPriceBuilder builder) {
		this.initialPoints = builder.getInitialPoints();
		this.marketFixedRate = builder.getMarketFixedRate();
		this.marketPrice = builder.getMarketPrice();
		this.quotationStyle = builder.getQuotationStyle();
	}
	
	/**
	 * An optional element that contains the up-front points expressed as a percentage of the notional. An initialPoints value of 5% would be represented as 0.05. The initialPoints element is an alternative to marketFixedRate in quoting the traded level of a trade. When initialPoints is used, the traded level is the sum of fixedRate and initialPoints. The initialPoints is one of the items that are factored into the initialPayment calculation and is payable by the Buyer to the Seller. Note that initialPoints and marketFixedRate may both be present in the same document when both implied values are desired.
	 */
	@RosettaSynonym(value="initialPoints", source="FpML_5_10", path="feeLeg")
	@RosettaSynonym(value="initialPoints", source="CME_SubmissionIRS_1_0", path="feeLeg")
	@RosettaSynonym(value="initialPoints", source="DTCC_11_0", path="feeLeg")
	@RosettaSynonym(value="initialPoints", source="DTCC_9_0", path="feeLeg")
	@RosettaSynonym(value="initialPoints", source="CME_ClearedConfirm_1_17", path="feeLeg")
	public final BigDecimal getInitialPoints() {
		return initialPoints;
	}
	
	/**
	 * An optional element that only has meaning in a credit index trade. This element contains the credit spread (&#39;fair value&#39;) at which the trade was executed. Unlike the fixedRate of an index, the marketFixedRate varies over the life of the index depending on market conditions. The marketFixedRate is the price of the index as quoted by trading desks.
	 */
	@RosettaSynonym(value="marketFixedRate", source="FpML_5_10", path="feeLeg")
	@RosettaSynonym(value="marketFixedRate", source="CME_SubmissionIRS_1_0", path="feeLeg")
	@RosettaSynonym(value="marketFixedRate", source="DTCC_11_0", path="feeLeg")
	@RosettaSynonym(value="marketFixedRate", source="DTCC_9_0", path="feeLeg")
	@RosettaSynonym(value="marketFixedRate", source="CME_ClearedConfirm_1_17", path="feeLeg")
	public final BigDecimal getMarketFixedRate() {
		return marketFixedRate;
	}
	
	/**
	 * An optional element that only has meaning in a credit index trade. This element contains the price at which the trade was executed and is used instead of marketFixedRate on credit trades on certain indicies which are quoted using a price rather than a spread.
	 */
	@RosettaSynonym(value="marketPrice", source="FpML_5_10", path="feeLeg")
	@RosettaSynonym(value="marketPrice", source="CME_SubmissionIRS_1_0", path="feeLeg")
	@RosettaSynonym(value="marketPrice", source="DTCC_11_0", path="feeLeg")
	@RosettaSynonym(value="marketPrice", source="DTCC_9_0", path="feeLeg")
	@RosettaSynonym(value="marketPrice", source="CME_ClearedConfirm_1_17", path="feeLeg")
	public final BigDecimal getMarketPrice() {
		return marketPrice;
	}
	
	/**
	 * An optional element that contains the up-front points expressed as a percentage of the notional. An initialPoints value of 5% would be represented as 0.05. The initialPoints element is an alternative to marketFixedRate in quoting the traded level of a trade. When initialPoints is used, the traded level is the sum of fixedRate and initialPoints. The initialPoints is one of the items that are factored into the initialPayment calculation and is payable by the Buyer to the Seller. Note that initialPoints and marketFixedRate may both be present in the same document when both implied values are desired.
	 */
	@RosettaSynonym(value="quotationStyle", source="FpML_5_10", path="feeLeg")
	@RosettaSynonym(value="quotationStyle", source="CME_SubmissionIRS_1_0", path="feeLeg")
	@RosettaSynonym(value="quotationStyle", source="DTCC_11_0", path="feeLeg")
	@RosettaSynonym(value="quotationStyle", source="DTCC_9_0", path="feeLeg")
	@RosettaSynonym(value="quotationStyle", source="CME_ClearedConfirm_1_17", path="feeLeg")
	public final QuotationStyleEnum getQuotationStyle() {
		return quotationStyle;
	}
	
	
	@Override
	public RosettaMetaData<? extends TransactedPrice> metaData() {
		return metaData;
	} 
	
	public TransactedPriceBuilder toBuilder() {
		TransactedPriceBuilder builder = new TransactedPriceBuilder();
		ofNullable(getInitialPoints()).ifPresent(builder::setInitialPoints);
		ofNullable(getMarketFixedRate()).ifPresent(builder::setMarketFixedRate);
		ofNullable(getMarketPrice()).ifPresent(builder::setMarketPrice);
		ofNullable(getQuotationStyle()).ifPresent(builder::setQuotationStyle);
		return builder;
	}

	public static TransactedPriceBuilder builder() {
		return new TransactedPriceBuilder();
	}

	public static  class TransactedPriceBuilder extends RosettaModelObjectBuilder {
	
		protected BigDecimal initialPoints;
		protected BigDecimal marketFixedRate;
		protected BigDecimal marketPrice;
		protected QuotationStyleEnum quotationStyle;
	
		public TransactedPriceBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends TransactedPrice> metaData() {
			return metaData;
		} 
	
		
		public BigDecimal getInitialPoints() {
			return initialPoints;
		}
		
		
		public BigDecimal getMarketFixedRate() {
			return marketFixedRate;
		}
		
		
		public BigDecimal getMarketPrice() {
			return marketPrice;
		}
		
		
		public QuotationStyleEnum getQuotationStyle() {
			return quotationStyle;
		}
		
	
		public TransactedPriceBuilder setInitialPoints(BigDecimal initialPoints) {
			this.initialPoints = initialPoints;
			return this;
		}
		public TransactedPriceBuilder setMarketFixedRate(BigDecimal marketFixedRate) {
			this.marketFixedRate = marketFixedRate;
			return this;
		}
		public TransactedPriceBuilder setMarketPrice(BigDecimal marketPrice) {
			this.marketPrice = marketPrice;
			return this;
		}
		public TransactedPriceBuilder setQuotationStyle(QuotationStyleEnum quotationStyle) {
			this.quotationStyle = quotationStyle;
			return this;
		}
		
		public TransactedPrice build() {
			return new TransactedPrice(this);
		}
	
		@Override
		public TransactedPriceBuilder prune() {
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getInitialPoints()!=null) return true;
			if (getMarketFixedRate()!=null) return true;
			if (getMarketPrice()!=null) return true;
			if (getQuotationStyle()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("initialPoints"), BigDecimal.class, initialPoints, this);
			processor.processBasic(path.newSubPath("marketFixedRate"), BigDecimal.class, marketFixedRate, this);
			processor.processBasic(path.newSubPath("marketPrice"), BigDecimal.class, marketPrice, this);
			processor.processBasic(path.newSubPath("quotationStyle"), QuotationStyleEnum.class, quotationStyle, this);
			
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			TransactedPriceBuilder _that = (TransactedPriceBuilder) o;
		
			if (initialPoints != null ? !initialPoints.equals(_that.initialPoints) : _that.initialPoints != null) return false;
			if (marketFixedRate != null ? !marketFixedRate.equals(_that.marketFixedRate) : _that.marketFixedRate != null) return false;
			if (marketPrice != null ? !marketPrice.equals(_that.marketPrice) : _that.marketPrice != null) return false;
			if (quotationStyle != null ? !quotationStyle.equals(_that.quotationStyle) : _that.quotationStyle != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (initialPoints != null ? initialPoints.hashCode() : 0);
			_result = 31 * _result + (marketFixedRate != null ? marketFixedRate.hashCode() : 0);
			_result = 31 * _result + (marketPrice != null ? marketPrice.hashCode() : 0);
			_result = 31 * _result + (quotationStyle != null ? quotationStyle.getClass().getName().hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "TransactedPriceBuilder {" +
				"initialPoints=" + this.initialPoints + ", " +
				"marketFixedRate=" + this.marketFixedRate + ", " +
				"marketPrice=" + this.marketPrice + ", " +
				"quotationStyle=" + this.quotationStyle +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("initialPoints"), BigDecimal.class, initialPoints, this);
		processor.processBasic(path.newSubPath("marketFixedRate"), BigDecimal.class, marketFixedRate, this);
		processor.processBasic(path.newSubPath("marketPrice"), BigDecimal.class, marketPrice, this);
		processor.processBasic(path.newSubPath("quotationStyle"), QuotationStyleEnum.class, quotationStyle, this);
		
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		TransactedPrice _that = (TransactedPrice) o;
	
		if (initialPoints != null ? !initialPoints.equals(_that.initialPoints) : _that.initialPoints != null) return false;
		if (marketFixedRate != null ? !marketFixedRate.equals(_that.marketFixedRate) : _that.marketFixedRate != null) return false;
		if (marketPrice != null ? !marketPrice.equals(_that.marketPrice) : _that.marketPrice != null) return false;
		if (quotationStyle != null ? !quotationStyle.equals(_that.quotationStyle) : _that.quotationStyle != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (initialPoints != null ? initialPoints.hashCode() : 0);
		_result = 31 * _result + (marketFixedRate != null ? marketFixedRate.hashCode() : 0);
		_result = 31 * _result + (marketPrice != null ? marketPrice.hashCode() : 0);
		_result = 31 * _result + (quotationStyle != null ? quotationStyle.getClass().getName().hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "TransactedPrice {" +
			"initialPoints=" + this.initialPoints + ", " +
			"marketFixedRate=" + this.marketFixedRate + ", " +
			"marketPrice=" + this.marketPrice + ", " +
			"quotationStyle=" + this.quotationStyle +
		'}';
	}
}
