package org.isda.cdm;

import java.lang.String;
import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.CalculationPeriodBase;
import org.isda.cdm.Lineage;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.TransferCalculationMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;
		import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;

/**
 * @version 2.5.4
 */
@RosettaClass
public class TransferCalculation extends RosettaModelObject {
	private final String calculationOutcome;
	private final List<Lineage> lineage;
	private final CalculationPeriodBase period;
	private static TransferCalculationMeta metaData = new TransferCalculationMeta();
	
	TransferCalculation(TransferCalculationBuilder builder) {
		this.calculationOutcome = builder.getCalculationOutcome();
		this.lineage = ofNullable(builder.getLineage()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.period = ofNullable(builder.getPeriod()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * This is a conceptual placeholder for providing the breakdown into the cashflow calculation components, leveraging the fact that the CDM provides calculation components, starting with the FixedAmount and the FloatingAmount. Further evaluation of expected usage needs to take place to confirm and prioritize such implementation.
	 */
	@RosettaSynonym(value="transferCalculation", source="Rosetta_Workbench")
	public final String getCalculationOutcome() {
		return calculationOutcome;
	}
	
	/**
	 * The lineage into the components used for the calculation.
	 */
	public final List<Lineage> getLineage() {
		return lineage;
	}
	
	/**
	 * The period adjusted start and end dates.
	 */
	@RosettaSynonym(value="calculationPeriod", source="DTCC_11_0", path="calculationElements")
	@RosettaSynonym(value="calculationPeriod", source="DTCC_9_0", path="calculationElements")
	public final CalculationPeriodBase getPeriod() {
		return period;
	}
	
	
	@Override
	public RosettaMetaData<? extends TransferCalculation> metaData() {
		return metaData;
	} 
	
	public TransferCalculationBuilder toBuilder() {
		TransferCalculationBuilder builder = new TransferCalculationBuilder();
		ofNullable(getCalculationOutcome()).ifPresent(builder::setCalculationOutcome);
		ofNullable(getLineage()).ifPresent(lineage -> lineage.forEach(builder::addLineage));
		ofNullable(getPeriod()).ifPresent(builder::setPeriod);
		return builder;
	}

	public static TransferCalculationBuilder builder() {
		return new TransferCalculationBuilder();
	}

	public static  class TransferCalculationBuilder extends RosettaModelObjectBuilder {
	
		protected String calculationOutcome;
		protected List<Lineage.LineageBuilder> lineage;
		protected CalculationPeriodBase.CalculationPeriodBaseBuilder period;
	
		public TransferCalculationBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends TransferCalculation> metaData() {
			return metaData;
		} 
	
		
		public String getCalculationOutcome() {
			return calculationOutcome;
		}
		
		
		public List<Lineage.LineageBuilder> getLineage() {
			return lineage;
		}
		
		public Lineage.LineageBuilder getOrCreateLineage(int index) {
			if (lineage==null) {
				this.lineage = new ArrayList<>();
			}
			return getIndex(lineage, index, ()->new Lineage.LineageBuilder());
		}
		
		
		public CalculationPeriodBase.CalculationPeriodBaseBuilder getPeriod() {
			return period;
		}
		
		public CalculationPeriodBase.CalculationPeriodBaseBuilder getOrCreatePeriod() {
			if (period!=null) {
				return period;
			}
			else return period = new CalculationPeriodBase.CalculationPeriodBaseBuilder();
		}
		
	
		public TransferCalculationBuilder setCalculationOutcome(String calculationOutcome) {
			this.calculationOutcome = calculationOutcome;
			return this;
		}
		public TransferCalculationBuilder addLineage(Lineage lineage) {
			if(this.lineage == null){
				this.lineage = new ArrayList<>();
			}
			this.lineage.add(lineage.toBuilder());
			return this;
		}
		public TransferCalculationBuilder addLineage(Lineage lineage, int _idx) {
			if(this.lineage == null){
				this.lineage = new ArrayList<>();
			}
			getIndex(this.lineage, _idx, () -> lineage.toBuilder());
			this.lineage.set(_idx, lineage.toBuilder());
			return this;
		}
		public TransferCalculationBuilder addLineage(List<Lineage> lineages) {
			if(this.lineage == null){
				this.lineage = new ArrayList<>();
			}
			for (Lineage toAdd : lineages) {
				this.lineage.add(toAdd.toBuilder());
			}
			return this;
		}
		public TransferCalculationBuilder addLineageBuilder(Lineage.LineageBuilder lineage) {
			if(this.lineage == null){
				this.lineage = new ArrayList<>();
			}
			this.lineage.add(lineage);
			return this;
		}
		
		public TransferCalculationBuilder clearLineage() {
			this.lineage = null;
			return this;
		}
		public TransferCalculationBuilder setPeriod(CalculationPeriodBase period) {
			this.period = period.toBuilder();
			return this;
		}
		public TransferCalculationBuilder setPeriodBuilder(CalculationPeriodBase.CalculationPeriodBaseBuilder period) {
			this.period = period;
			return this;
		}
		
		public TransferCalculation build() {
			return new TransferCalculation(this);
		}
	
		@Override
		public TransferCalculationBuilder prune() {
			if (lineage!=null) lineage = lineage.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (period!=null && !period.prune().hasData()) period = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getCalculationOutcome()!=null) return true;
			if (getLineage()!=null && getLineage().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getPeriod()!=null && getPeriod().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("calculationOutcome"), String.class, calculationOutcome, this);
			
			processRosetta(path.newSubPath("lineage"), processor, Lineage.class, lineage);
			processRosetta(path.newSubPath("period"), processor, CalculationPeriodBase.class, period);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			TransferCalculationBuilder _that = (TransferCalculationBuilder) o;
		
			if (calculationOutcome != null ? !calculationOutcome.equals(_that.calculationOutcome) : _that.calculationOutcome != null) return false;
			if (!ListEquals.listEquals(lineage, _that.lineage)) return false;
			if (period != null ? !period.equals(_that.period) : _that.period != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (calculationOutcome != null ? calculationOutcome.hashCode() : 0);
			_result = 31 * _result + (lineage != null ? lineage.hashCode() : 0);
			_result = 31 * _result + (period != null ? period.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "TransferCalculationBuilder {" +
				"calculationOutcome=" + this.calculationOutcome + ", " +
				"lineage=" + this.lineage + ", " +
				"period=" + this.period +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("calculationOutcome"), String.class, calculationOutcome, this);
		
		processRosetta(path.newSubPath("lineage"), processor, Lineage.class, lineage);
		processRosetta(path.newSubPath("period"), processor, CalculationPeriodBase.class, period);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		TransferCalculation _that = (TransferCalculation) o;
	
		if (calculationOutcome != null ? !calculationOutcome.equals(_that.calculationOutcome) : _that.calculationOutcome != null) return false;
		if (!ListEquals.listEquals(lineage, _that.lineage)) return false;
		if (period != null ? !period.equals(_that.period) : _that.period != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (calculationOutcome != null ? calculationOutcome.hashCode() : 0);
		_result = 31 * _result + (lineage != null ? lineage.hashCode() : 0);
		_result = 31 * _result + (period != null ? period.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "TransferCalculation {" +
			"calculationOutcome=" + this.calculationOutcome + ", " +
			"lineage=" + this.lineage + ", " +
			"period=" + this.period +
		'}';
	}
}
