package org.isda.cdm;

import java.lang.String;
import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.AdjustableOrAdjustedOrRelativeDate;
import org.isda.cdm.CashTransferComponent;
import org.isda.cdm.CommodityTransferComponent;
import org.isda.cdm.SecurityTransferComponent;
import org.isda.cdm.TransferSettlementEnum;
import org.isda.cdm.TransferStatusEnum;
import org.isda.cdm.metafields.FieldWithMetaString;
import org.isda.cdm.metafields.FieldWithMetaString.FieldWithMetaStringBuilder;
import org.isda.cdm.metafields.MetaFields;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.TransferPrimitiveMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;
		import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;

/**
 * A class to specify the transfer of assets between parties, those assets being either cash, securities or physical assets. This class combines components that are cross-assets (settlement date, settlement type, status, ...) and some other which are specialized by asset class (e.g. the payer/receiver amount and cashflow type for a cash transfer, the transferor/transferee, security indication, quantity, and asset transfer type qualification for the case of a security). The associated rosettaKey denotes the ability to associate a hash value to the respective Execution instantiations for the purpose of model cross-referencing, in support of functionality such as the event effect and the lineage.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class TransferPrimitive extends RosettaModelObject implements GlobalKey {
	private final List<CashTransferComponent> cashTransfer;
	private final List<CommodityTransferComponent> commodityTransfer;
	private final FieldWithMetaString identifier;
	private final MetaFields meta;
	private final List<SecurityTransferComponent> securityTransfer;
	private final AdjustableOrAdjustedOrRelativeDate settlementDate;
	private final String settlementReference;
	private final TransferSettlementEnum settlementType;
	private final TransferStatusEnum status;
	private static TransferPrimitiveMeta metaData = new TransferPrimitiveMeta();
	
	TransferPrimitive(TransferPrimitiveBuilder builder) {
		this.cashTransfer = ofNullable(builder.getCashTransfer()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.commodityTransfer = ofNullable(builder.getCommodityTransfer()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.identifier = ofNullable(builder.getIdentifier()).map(f->f.build()).orElse(null);
		this.meta = ofNullable(builder.getMeta()).map(f->f.build()).orElse(null);
		this.securityTransfer = ofNullable(builder.getSecurityTransfer()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.settlementDate = ofNullable(builder.getSettlementDate()).map(f->f.build()).orElse(null);
		this.settlementReference = builder.getSettlementReference();
		this.settlementType = builder.getSettlementType();
		this.status = builder.getStatus();
	}
	
	/**
	 * The cash transfer component of the transfer. In the case where several currencies are involved in the transfer, several components should be used, as the component supports one single currency amount.
	 */
	@RosettaSynonym(value="cashTransfer", source="Rosetta_Workbench")
	@RosettaSynonym(value="payment", source="DTCC_11_0", path="FpML")
	@RosettaSynonym(value="payment", source="DTCC_9_0", path="FpML")
	public final List<CashTransferComponent> getCashTransfer() {
		return cashTransfer;
	}
	
	public final List<CommodityTransferComponent> getCommodityTransfer() {
		return commodityTransfer;
	}
	
	/**
	 * The identifier which might be associated with the transfer.
	 */
	@RosettaSynonym(value="tradeCashflowsId", source="DTCC_11_0", path="FpML")
	@RosettaSynonym(value="tradeCashflowsId", source="DTCC_9_0", path="FpML")
	public final FieldWithMetaString getIdentifier() {
		return identifier;
	}
	
	public final MetaFields getMeta() {
		return meta;
	}
	
	/**
	 * The security transfer component of the transfer. In the case where several securities are involved in the transfer, several components should be used, as the component supports one single security.
	 */
	@RosettaSynonym(value="securityTransfer", source="Rosetta_Workbench")
	public final List<SecurityTransferComponent> getSecurityTransfer() {
		return securityTransfer;
	}
	
	@RosettaSynonym(value="settlementDate", source="Rosetta_Workbench")
	public final AdjustableOrAdjustedOrRelativeDate getSettlementDate() {
		return settlementDate;
	}
	
	/**
	 * The settlement reference, when applicable.
	 */
	@RosettaSynonym(value="settlementReference", source="Rosetta_Workbench")
	public final String getSettlementReference() {
		return settlementReference;
	}
	
	/**
	 * The qualification as to how the transfer will settle, e.g. a DvP settlement.
	 */
	@RosettaSynonym(value="transferType", source="Rosetta_Workbench")
	@RosettaSynonym(value="SettlementType", source="DTCC_11_0", path="PaymentDetails")
	@RosettaSynonym(value="SettlementType", source="DTCC_9_0", path="PaymentDetails")
	public final TransferSettlementEnum getSettlementType() {
		return settlementType;
	}
	
	/**
	 * The transfer status, e.g. Instructed, Settled, ...
	 */
	@RosettaSynonym(value="status", source="Rosetta_Workbench")
	public final TransferStatusEnum getStatus() {
		return status;
	}
	
	
	@Override
	public RosettaMetaData<? extends TransferPrimitive> metaData() {
		return metaData;
	} 
	
	public TransferPrimitiveBuilder toBuilder() {
		TransferPrimitiveBuilder builder = new TransferPrimitiveBuilder();
		ofNullable(getCashTransfer()).ifPresent(cashTransfer -> cashTransfer.forEach(builder::addCashTransfer));
		ofNullable(getCommodityTransfer()).ifPresent(commodityTransfer -> commodityTransfer.forEach(builder::addCommodityTransfer));
		ofNullable(getIdentifier()).ifPresent(builder::setIdentifier);
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		ofNullable(getSecurityTransfer()).ifPresent(securityTransfer -> securityTransfer.forEach(builder::addSecurityTransfer));
		ofNullable(getSettlementDate()).ifPresent(builder::setSettlementDate);
		ofNullable(getSettlementReference()).ifPresent(builder::setSettlementReference);
		ofNullable(getSettlementType()).ifPresent(builder::setSettlementType);
		ofNullable(getStatus()).ifPresent(builder::setStatus);
		return builder;
	}

	public static TransferPrimitiveBuilder builder() {
		return new TransferPrimitiveBuilder();
	}

	public static  class TransferPrimitiveBuilder extends RosettaModelObjectBuilder implements GlobalKeyBuilder<TransferPrimitiveBuilder> {
	
		protected List<CashTransferComponent.CashTransferComponentBuilder> cashTransfer;
		protected List<CommodityTransferComponent.CommodityTransferComponentBuilder> commodityTransfer;
		protected FieldWithMetaStringBuilder identifier;
		protected MetaFields.MetaFieldsBuilder meta;
		protected List<SecurityTransferComponent.SecurityTransferComponentBuilder> securityTransfer;
		protected AdjustableOrAdjustedOrRelativeDate.AdjustableOrAdjustedOrRelativeDateBuilder settlementDate;
		protected String settlementReference;
		protected TransferSettlementEnum settlementType;
		protected TransferStatusEnum status;
	
		public TransferPrimitiveBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends TransferPrimitive> metaData() {
			return metaData;
		} 
	
		
		public List<CashTransferComponent.CashTransferComponentBuilder> getCashTransfer() {
			return cashTransfer;
		}
		
		public CashTransferComponent.CashTransferComponentBuilder getOrCreateCashTransfer(int index) {
			if (cashTransfer==null) {
				this.cashTransfer = new ArrayList<>();
			}
			return getIndex(cashTransfer, index, ()->new CashTransferComponent.CashTransferComponentBuilder());
		}
		
		
		public List<CommodityTransferComponent.CommodityTransferComponentBuilder> getCommodityTransfer() {
			return commodityTransfer;
		}
		
		public CommodityTransferComponent.CommodityTransferComponentBuilder getOrCreateCommodityTransfer(int index) {
			if (commodityTransfer==null) {
				this.commodityTransfer = new ArrayList<>();
			}
			return getIndex(commodityTransfer, index, ()->new CommodityTransferComponent.CommodityTransferComponentBuilder());
		}
		
		
		public FieldWithMetaStringBuilder getIdentifier() {
			return identifier;
		}
		
		public FieldWithMetaStringBuilder getOrCreateIdentifier() {
			if (identifier!=null) {
				return identifier;
			}
			else return identifier = new FieldWithMetaStringBuilder();
		}
		
		
		public MetaFields.MetaFieldsBuilder getMeta() {
			return meta;
		}
		
		public MetaFields.MetaFieldsBuilder getOrCreateMeta() {
			if (meta!=null) {
				return meta;
			}
			else return meta = new MetaFields.MetaFieldsBuilder();
		}
		
		
		public List<SecurityTransferComponent.SecurityTransferComponentBuilder> getSecurityTransfer() {
			return securityTransfer;
		}
		
		public SecurityTransferComponent.SecurityTransferComponentBuilder getOrCreateSecurityTransfer(int index) {
			if (securityTransfer==null) {
				this.securityTransfer = new ArrayList<>();
			}
			return getIndex(securityTransfer, index, ()->new SecurityTransferComponent.SecurityTransferComponentBuilder());
		}
		
		
		public AdjustableOrAdjustedOrRelativeDate.AdjustableOrAdjustedOrRelativeDateBuilder getSettlementDate() {
			return settlementDate;
		}
		
		public AdjustableOrAdjustedOrRelativeDate.AdjustableOrAdjustedOrRelativeDateBuilder getOrCreateSettlementDate() {
			if (settlementDate!=null) {
				return settlementDate;
			}
			else return settlementDate = new AdjustableOrAdjustedOrRelativeDate.AdjustableOrAdjustedOrRelativeDateBuilder();
		}
		
		
		public String getSettlementReference() {
			return settlementReference;
		}
		
		
		public TransferSettlementEnum getSettlementType() {
			return settlementType;
		}
		
		
		public TransferStatusEnum getStatus() {
			return status;
		}
		
	
		public TransferPrimitiveBuilder addCashTransfer(CashTransferComponent cashTransfer) {
			if(this.cashTransfer == null){
				this.cashTransfer = new ArrayList<>();
			}
			this.cashTransfer.add(cashTransfer.toBuilder());
			return this;
		}
		public TransferPrimitiveBuilder addCashTransfer(CashTransferComponent cashTransfer, int _idx) {
			if(this.cashTransfer == null){
				this.cashTransfer = new ArrayList<>();
			}
			getIndex(this.cashTransfer, _idx, () -> cashTransfer.toBuilder());
			this.cashTransfer.set(_idx, cashTransfer.toBuilder());
			return this;
		}
		public TransferPrimitiveBuilder addCashTransfer(List<CashTransferComponent> cashTransfers) {
			if(this.cashTransfer == null){
				this.cashTransfer = new ArrayList<>();
			}
			for (CashTransferComponent toAdd : cashTransfers) {
				this.cashTransfer.add(toAdd.toBuilder());
			}
			return this;
		}
		public TransferPrimitiveBuilder addCashTransferBuilder(CashTransferComponent.CashTransferComponentBuilder cashTransfer) {
			if(this.cashTransfer == null){
				this.cashTransfer = new ArrayList<>();
			}
			this.cashTransfer.add(cashTransfer);
			return this;
		}
		
		public TransferPrimitiveBuilder clearCashTransfer() {
			this.cashTransfer = null;
			return this;
		}
		public TransferPrimitiveBuilder addCommodityTransfer(CommodityTransferComponent commodityTransfer) {
			if(this.commodityTransfer == null){
				this.commodityTransfer = new ArrayList<>();
			}
			this.commodityTransfer.add(commodityTransfer.toBuilder());
			return this;
		}
		public TransferPrimitiveBuilder addCommodityTransfer(CommodityTransferComponent commodityTransfer, int _idx) {
			if(this.commodityTransfer == null){
				this.commodityTransfer = new ArrayList<>();
			}
			getIndex(this.commodityTransfer, _idx, () -> commodityTransfer.toBuilder());
			this.commodityTransfer.set(_idx, commodityTransfer.toBuilder());
			return this;
		}
		public TransferPrimitiveBuilder addCommodityTransfer(List<CommodityTransferComponent> commodityTransfers) {
			if(this.commodityTransfer == null){
				this.commodityTransfer = new ArrayList<>();
			}
			for (CommodityTransferComponent toAdd : commodityTransfers) {
				this.commodityTransfer.add(toAdd.toBuilder());
			}
			return this;
		}
		public TransferPrimitiveBuilder addCommodityTransferBuilder(CommodityTransferComponent.CommodityTransferComponentBuilder commodityTransfer) {
			if(this.commodityTransfer == null){
				this.commodityTransfer = new ArrayList<>();
			}
			this.commodityTransfer.add(commodityTransfer);
			return this;
		}
		
		public TransferPrimitiveBuilder clearCommodityTransfer() {
			this.commodityTransfer = null;
			return this;
		}
		public TransferPrimitiveBuilder setIdentifier(FieldWithMetaString identifier) {
			this.identifier = identifier.toBuilder();
			return this;
		}
		public TransferPrimitiveBuilder setIdentifierRef(String identifier) {
			return setIdentifier(FieldWithMetaString.builder().setValue(identifier).build());
		}
		public TransferPrimitiveBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		public TransferPrimitiveBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		public TransferPrimitiveBuilder addSecurityTransfer(SecurityTransferComponent securityTransfer) {
			if(this.securityTransfer == null){
				this.securityTransfer = new ArrayList<>();
			}
			this.securityTransfer.add(securityTransfer.toBuilder());
			return this;
		}
		public TransferPrimitiveBuilder addSecurityTransfer(SecurityTransferComponent securityTransfer, int _idx) {
			if(this.securityTransfer == null){
				this.securityTransfer = new ArrayList<>();
			}
			getIndex(this.securityTransfer, _idx, () -> securityTransfer.toBuilder());
			this.securityTransfer.set(_idx, securityTransfer.toBuilder());
			return this;
		}
		public TransferPrimitiveBuilder addSecurityTransfer(List<SecurityTransferComponent> securityTransfers) {
			if(this.securityTransfer == null){
				this.securityTransfer = new ArrayList<>();
			}
			for (SecurityTransferComponent toAdd : securityTransfers) {
				this.securityTransfer.add(toAdd.toBuilder());
			}
			return this;
		}
		public TransferPrimitiveBuilder addSecurityTransferBuilder(SecurityTransferComponent.SecurityTransferComponentBuilder securityTransfer) {
			if(this.securityTransfer == null){
				this.securityTransfer = new ArrayList<>();
			}
			this.securityTransfer.add(securityTransfer);
			return this;
		}
		
		public TransferPrimitiveBuilder clearSecurityTransfer() {
			this.securityTransfer = null;
			return this;
		}
		public TransferPrimitiveBuilder setSettlementDate(AdjustableOrAdjustedOrRelativeDate settlementDate) {
			this.settlementDate = settlementDate.toBuilder();
			return this;
		}
		public TransferPrimitiveBuilder setSettlementDateBuilder(AdjustableOrAdjustedOrRelativeDate.AdjustableOrAdjustedOrRelativeDateBuilder settlementDate) {
			this.settlementDate = settlementDate;
			return this;
		}
		public TransferPrimitiveBuilder setSettlementReference(String settlementReference) {
			this.settlementReference = settlementReference;
			return this;
		}
		public TransferPrimitiveBuilder setSettlementType(TransferSettlementEnum settlementType) {
			this.settlementType = settlementType;
			return this;
		}
		public TransferPrimitiveBuilder setStatus(TransferStatusEnum status) {
			this.status = status;
			return this;
		}
		
		public TransferPrimitive build() {
			return new TransferPrimitive(this);
		}
	
		@Override
		public TransferPrimitiveBuilder prune() {
			if (cashTransfer!=null) cashTransfer = cashTransfer.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (commodityTransfer!=null) commodityTransfer = commodityTransfer.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (identifier!=null && !identifier.prune().hasData()) identifier = null;
			if (meta!=null && !meta.prune().hasData()) meta = null;
			if (securityTransfer!=null) securityTransfer = securityTransfer.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (settlementDate!=null && !settlementDate.prune().hasData()) settlementDate = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getCashTransfer()!=null && getCashTransfer().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getCommodityTransfer()!=null && getCommodityTransfer().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getIdentifier()!=null) return true;
			if (getMeta()!=null && getMeta().hasData()) return true;
			if (getSecurityTransfer()!=null && getSecurityTransfer().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getSettlementDate()!=null && getSettlementDate().hasData()) return true;
			if (getSettlementReference()!=null) return true;
			if (getSettlementType()!=null) return true;
			if (getStatus()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("settlementReference"), String.class, settlementReference, this);
			processor.processBasic(path.newSubPath("settlementType"), TransferSettlementEnum.class, settlementType, this);
			processor.processBasic(path.newSubPath("status"), TransferStatusEnum.class, status, this);
			
			processRosetta(path.newSubPath("cashTransfer"), processor, CashTransferComponent.class, cashTransfer);
			processRosetta(path.newSubPath("commodityTransfer"), processor, CommodityTransferComponent.class, commodityTransfer);
			processRosetta(path.newSubPath("identifier"), processor, FieldWithMetaString.class, identifier);
			processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
			processRosetta(path.newSubPath("securityTransfer"), processor, SecurityTransferComponent.class, securityTransfer);
			processRosetta(path.newSubPath("settlementDate"), processor, AdjustableOrAdjustedOrRelativeDate.class, settlementDate);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			TransferPrimitiveBuilder _that = (TransferPrimitiveBuilder) o;
		
			if (!ListEquals.listEquals(cashTransfer, _that.cashTransfer)) return false;
			if (!ListEquals.listEquals(commodityTransfer, _that.commodityTransfer)) return false;
			if (identifier != null ? !identifier.equals(_that.identifier) : _that.identifier != null) return false;
			if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
			if (!ListEquals.listEquals(securityTransfer, _that.securityTransfer)) return false;
			if (settlementDate != null ? !settlementDate.equals(_that.settlementDate) : _that.settlementDate != null) return false;
			if (settlementReference != null ? !settlementReference.equals(_that.settlementReference) : _that.settlementReference != null) return false;
			if (settlementType != null ? !settlementType.equals(_that.settlementType) : _that.settlementType != null) return false;
			if (status != null ? !status.equals(_that.status) : _that.status != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (cashTransfer != null ? cashTransfer.hashCode() : 0);
			_result = 31 * _result + (commodityTransfer != null ? commodityTransfer.hashCode() : 0);
			_result = 31 * _result + (identifier != null ? identifier.hashCode() : 0);
			_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
			_result = 31 * _result + (securityTransfer != null ? securityTransfer.hashCode() : 0);
			_result = 31 * _result + (settlementDate != null ? settlementDate.hashCode() : 0);
			_result = 31 * _result + (settlementReference != null ? settlementReference.hashCode() : 0);
			_result = 31 * _result + (settlementType != null ? settlementType.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (status != null ? status.getClass().getName().hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "TransferPrimitiveBuilder {" +
				"cashTransfer=" + this.cashTransfer + ", " +
				"commodityTransfer=" + this.commodityTransfer + ", " +
				"identifier=" + this.identifier + ", " +
				"meta=" + this.meta + ", " +
				"securityTransfer=" + this.securityTransfer + ", " +
				"settlementDate=" + this.settlementDate + ", " +
				"settlementReference=" + this.settlementReference + ", " +
				"settlementType=" + this.settlementType + ", " +
				"status=" + this.status +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("settlementReference"), String.class, settlementReference, this);
		processor.processBasic(path.newSubPath("settlementType"), TransferSettlementEnum.class, settlementType, this);
		processor.processBasic(path.newSubPath("status"), TransferStatusEnum.class, status, this);
		
		processRosetta(path.newSubPath("cashTransfer"), processor, CashTransferComponent.class, cashTransfer);
		processRosetta(path.newSubPath("commodityTransfer"), processor, CommodityTransferComponent.class, commodityTransfer);
		processRosetta(path.newSubPath("identifier"), processor, FieldWithMetaString.class, identifier);
		processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
		processRosetta(path.newSubPath("securityTransfer"), processor, SecurityTransferComponent.class, securityTransfer);
		processRosetta(path.newSubPath("settlementDate"), processor, AdjustableOrAdjustedOrRelativeDate.class, settlementDate);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		TransferPrimitive _that = (TransferPrimitive) o;
	
		if (!ListEquals.listEquals(cashTransfer, _that.cashTransfer)) return false;
		if (!ListEquals.listEquals(commodityTransfer, _that.commodityTransfer)) return false;
		if (identifier != null ? !identifier.equals(_that.identifier) : _that.identifier != null) return false;
		if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
		if (!ListEquals.listEquals(securityTransfer, _that.securityTransfer)) return false;
		if (settlementDate != null ? !settlementDate.equals(_that.settlementDate) : _that.settlementDate != null) return false;
		if (settlementReference != null ? !settlementReference.equals(_that.settlementReference) : _that.settlementReference != null) return false;
		if (settlementType != null ? !settlementType.equals(_that.settlementType) : _that.settlementType != null) return false;
		if (status != null ? !status.equals(_that.status) : _that.status != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (cashTransfer != null ? cashTransfer.hashCode() : 0);
		_result = 31 * _result + (commodityTransfer != null ? commodityTransfer.hashCode() : 0);
		_result = 31 * _result + (identifier != null ? identifier.hashCode() : 0);
		_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
		_result = 31 * _result + (securityTransfer != null ? securityTransfer.hashCode() : 0);
		_result = 31 * _result + (settlementDate != null ? settlementDate.hashCode() : 0);
		_result = 31 * _result + (settlementReference != null ? settlementReference.hashCode() : 0);
		_result = 31 * _result + (settlementType != null ? settlementType.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (status != null ? status.getClass().getName().hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "TransferPrimitive {" +
			"cashTransfer=" + this.cashTransfer + ", " +
			"commodityTransfer=" + this.commodityTransfer + ", " +
			"identifier=" + this.identifier + ", " +
			"meta=" + this.meta + ", " +
			"securityTransfer=" + this.securityTransfer + ", " +
			"settlementDate=" + this.settlementDate + ", " +
			"settlementReference=" + this.settlementReference + ", " +
			"settlementType=" + this.settlementType + ", " +
			"status=" + this.status +
		'}';
	}
}
