package org.isda.cdm;

import org.isda.cdm.metafields.ReferenceWithMetaAccount;
import org.isda.cdm.metafields.ReferenceWithMetaAccount.ReferenceWithMetaAccountBuilder;
import org.isda.cdm.metafields.ReferenceWithMetaParty;
import org.isda.cdm.metafields.ReferenceWithMetaParty.ReferenceWithMetaPartyBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.TransferorTransfereeMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.stream.Collectors;
		import static java.util.Optional.ofNullable;

/**
 * A class mimicking the PayerReceiver, which is itself derived from the FpML PayerReceiver.model, to represent the transferee and transferor party information in relation to the transfer of security or commodities.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class TransferorTransferee extends RosettaModelObject {
	private final ReferenceWithMetaAccount transfereeAccountReference;
	private final ReferenceWithMetaParty transfereePartyReference;
	private final ReferenceWithMetaAccount transferorAccountReference;
	private final ReferenceWithMetaParty transferorPartyReference;
	private static TransferorTransfereeMeta metaData = new TransferorTransfereeMeta();
	
	TransferorTransferee(TransferorTransfereeBuilder builder) {
		this.transfereeAccountReference = ofNullable(builder.getTransfereeAccountReference()).map(f->f.build()).orElse(null);
		this.transfereePartyReference = ofNullable(builder.getTransfereePartyReference()).map(f->f.build()).orElse(null);
		this.transferorAccountReference = ofNullable(builder.getTransferorAccountReference()).map(f->f.build()).orElse(null);
		this.transferorPartyReference = ofNullable(builder.getTransferorPartyReference()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * A reference to the account that receives the payments corresponding to this structure.
	 */
	@RosettaSynonym(value="receiverAccountReference", source="FpML_5_10")
	@RosettaSynonym(value="receiverAccountReference", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="receiverAccountReference", source="DTCC_11_0")
	@RosettaSynonym(value="receiverAccountReference", source="DTCC_9_0")
	@RosettaSynonym(value="receiverAccountReference", source="CME_ClearedConfirm_1_17")
	public final ReferenceWithMetaAccount getTransfereeAccountReference() {
		return transfereeAccountReference;
	}
	
	/**
	 * A reference to the party that receives the payments corresponding to this structure.
	 */
	@RosettaSynonym(value="receiverPartyReference", source="FpML_5_10")
	@RosettaSynonym(value="receiverPartyReference", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="receiverPartyReference", source="DTCC_11_0")
	@RosettaSynonym(value="receiverPartyReference", source="DTCC_9_0")
	@RosettaSynonym(value="receiverPartyReference", source="CME_ClearedConfirm_1_17")
	public final ReferenceWithMetaParty getTransfereePartyReference() {
		return transfereePartyReference;
	}
	
	/**
	 * A reference to the account responsible for making the payments defined by this structure.
	 */
	@RosettaSynonym(value="payerAccountReference", source="FpML_5_10")
	@RosettaSynonym(value="payerAccountReference", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="payerAccountReference", source="DTCC_11_0")
	@RosettaSynonym(value="payerAccountReference", source="DTCC_9_0")
	@RosettaSynonym(value="payerAccountReference", source="CME_ClearedConfirm_1_17")
	public final ReferenceWithMetaAccount getTransferorAccountReference() {
		return transferorAccountReference;
	}
	
	/**
	 * A reference to the party responsible for making the payments defined by this structure.
	 */
	@RosettaSynonym(value="payerPartyReference", source="FpML_5_10")
	@RosettaSynonym(value="payerPartyReference", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="payerPartyReference", source="DTCC_11_0")
	@RosettaSynonym(value="payerPartyReference", source="DTCC_9_0")
	@RosettaSynonym(value="payerPartyReference", source="CME_ClearedConfirm_1_17")
	public final ReferenceWithMetaParty getTransferorPartyReference() {
		return transferorPartyReference;
	}
	
	
	@Override
	public RosettaMetaData<? extends TransferorTransferee> metaData() {
		return metaData;
	} 
	
	public TransferorTransfereeBuilder toBuilder() {
		TransferorTransfereeBuilder builder = new TransferorTransfereeBuilder();
		ofNullable(getTransfereeAccountReference()).ifPresent(builder::setTransfereeAccountReference);
		ofNullable(getTransfereePartyReference()).ifPresent(builder::setTransfereePartyReference);
		ofNullable(getTransferorAccountReference()).ifPresent(builder::setTransferorAccountReference);
		ofNullable(getTransferorPartyReference()).ifPresent(builder::setTransferorPartyReference);
		return builder;
	}

	public static TransferorTransfereeBuilder builder() {
		return new TransferorTransfereeBuilder();
	}

	public static  class TransferorTransfereeBuilder extends RosettaModelObjectBuilder {
	
		protected ReferenceWithMetaAccountBuilder transfereeAccountReference;
		protected ReferenceWithMetaPartyBuilder transfereePartyReference;
		protected ReferenceWithMetaAccountBuilder transferorAccountReference;
		protected ReferenceWithMetaPartyBuilder transferorPartyReference;
	
		public TransferorTransfereeBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends TransferorTransferee> metaData() {
			return metaData;
		} 
	
		
		public ReferenceWithMetaAccountBuilder getTransfereeAccountReference() {
			return transfereeAccountReference;
		}
		
		public ReferenceWithMetaAccountBuilder getOrCreateTransfereeAccountReference() {
			if (transfereeAccountReference!=null) {
				return transfereeAccountReference;
			}
			else return transfereeAccountReference = new ReferenceWithMetaAccountBuilder();
		}
		
		
		public ReferenceWithMetaPartyBuilder getTransfereePartyReference() {
			return transfereePartyReference;
		}
		
		public ReferenceWithMetaPartyBuilder getOrCreateTransfereePartyReference() {
			if (transfereePartyReference!=null) {
				return transfereePartyReference;
			}
			else return transfereePartyReference = new ReferenceWithMetaPartyBuilder();
		}
		
		
		public ReferenceWithMetaAccountBuilder getTransferorAccountReference() {
			return transferorAccountReference;
		}
		
		public ReferenceWithMetaAccountBuilder getOrCreateTransferorAccountReference() {
			if (transferorAccountReference!=null) {
				return transferorAccountReference;
			}
			else return transferorAccountReference = new ReferenceWithMetaAccountBuilder();
		}
		
		
		public ReferenceWithMetaPartyBuilder getTransferorPartyReference() {
			return transferorPartyReference;
		}
		
		public ReferenceWithMetaPartyBuilder getOrCreateTransferorPartyReference() {
			if (transferorPartyReference!=null) {
				return transferorPartyReference;
			}
			else return transferorPartyReference = new ReferenceWithMetaPartyBuilder();
		}
		
	
		public TransferorTransfereeBuilder setTransfereeAccountReference(ReferenceWithMetaAccount transfereeAccountReference) {
			this.transfereeAccountReference = transfereeAccountReference.toBuilder();
			return this;
		}
		public TransferorTransfereeBuilder setTransfereeAccountReferenceBuilder(ReferenceWithMetaAccountBuilder transfereeAccountReference) {
			this.transfereeAccountReference = transfereeAccountReference;
			return this;
		}
		public TransferorTransfereeBuilder setTransfereeAccountReferenceRef(Account.AccountBuilder transfereeAccountReference) {
			return setTransfereeAccountReference(ReferenceWithMetaAccount.builder().setValueBuilder(transfereeAccountReference).build());
		}
		public TransferorTransfereeBuilder setTransfereeAccountReferenceRef(Account transfereeAccountReference) {
			return setTransfereeAccountReferenceRef(Account.builder());
		}
		public TransferorTransfereeBuilder setTransfereePartyReference(ReferenceWithMetaParty transfereePartyReference) {
			this.transfereePartyReference = transfereePartyReference.toBuilder();
			return this;
		}
		public TransferorTransfereeBuilder setTransfereePartyReferenceBuilder(ReferenceWithMetaPartyBuilder transfereePartyReference) {
			this.transfereePartyReference = transfereePartyReference;
			return this;
		}
		public TransferorTransfereeBuilder setTransfereePartyReferenceRef(Party.PartyBuilder transfereePartyReference) {
			return setTransfereePartyReference(ReferenceWithMetaParty.builder().setValueBuilder(transfereePartyReference).build());
		}
		public TransferorTransfereeBuilder setTransfereePartyReferenceRef(Party transfereePartyReference) {
			return setTransfereePartyReferenceRef(Party.builder());
		}
		public TransferorTransfereeBuilder setTransferorAccountReference(ReferenceWithMetaAccount transferorAccountReference) {
			this.transferorAccountReference = transferorAccountReference.toBuilder();
			return this;
		}
		public TransferorTransfereeBuilder setTransferorAccountReferenceBuilder(ReferenceWithMetaAccountBuilder transferorAccountReference) {
			this.transferorAccountReference = transferorAccountReference;
			return this;
		}
		public TransferorTransfereeBuilder setTransferorAccountReferenceRef(Account.AccountBuilder transferorAccountReference) {
			return setTransferorAccountReference(ReferenceWithMetaAccount.builder().setValueBuilder(transferorAccountReference).build());
		}
		public TransferorTransfereeBuilder setTransferorAccountReferenceRef(Account transferorAccountReference) {
			return setTransferorAccountReferenceRef(Account.builder());
		}
		public TransferorTransfereeBuilder setTransferorPartyReference(ReferenceWithMetaParty transferorPartyReference) {
			this.transferorPartyReference = transferorPartyReference.toBuilder();
			return this;
		}
		public TransferorTransfereeBuilder setTransferorPartyReferenceBuilder(ReferenceWithMetaPartyBuilder transferorPartyReference) {
			this.transferorPartyReference = transferorPartyReference;
			return this;
		}
		public TransferorTransfereeBuilder setTransferorPartyReferenceRef(Party.PartyBuilder transferorPartyReference) {
			return setTransferorPartyReference(ReferenceWithMetaParty.builder().setValueBuilder(transferorPartyReference).build());
		}
		public TransferorTransfereeBuilder setTransferorPartyReferenceRef(Party transferorPartyReference) {
			return setTransferorPartyReferenceRef(Party.builder());
		}
		
		public TransferorTransferee build() {
			return new TransferorTransferee(this);
		}
	
		@Override
		public TransferorTransfereeBuilder prune() {
			if (transfereeAccountReference!=null && !transfereeAccountReference.prune().hasData()) transfereeAccountReference = null;
			if (transfereePartyReference!=null && !transfereePartyReference.prune().hasData()) transfereePartyReference = null;
			if (transferorAccountReference!=null && !transferorAccountReference.prune().hasData()) transferorAccountReference = null;
			if (transferorPartyReference!=null && !transferorPartyReference.prune().hasData()) transferorPartyReference = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getTransfereeAccountReference()!=null && getTransfereeAccountReference().hasData()) return true;
			if (getTransfereePartyReference()!=null && getTransfereePartyReference().hasData()) return true;
			if (getTransferorAccountReference()!=null && getTransferorAccountReference().hasData()) return true;
			if (getTransferorPartyReference()!=null && getTransferorPartyReference().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("transfereeAccountReference"), processor, ReferenceWithMetaAccount.class, transfereeAccountReference);
			processRosetta(path.newSubPath("transfereePartyReference"), processor, ReferenceWithMetaParty.class, transfereePartyReference);
			processRosetta(path.newSubPath("transferorAccountReference"), processor, ReferenceWithMetaAccount.class, transferorAccountReference);
			processRosetta(path.newSubPath("transferorPartyReference"), processor, ReferenceWithMetaParty.class, transferorPartyReference);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			TransferorTransfereeBuilder _that = (TransferorTransfereeBuilder) o;
		
			if (transfereeAccountReference != null ? !transfereeAccountReference.equals(_that.transfereeAccountReference) : _that.transfereeAccountReference != null) return false;
			if (transfereePartyReference != null ? !transfereePartyReference.equals(_that.transfereePartyReference) : _that.transfereePartyReference != null) return false;
			if (transferorAccountReference != null ? !transferorAccountReference.equals(_that.transferorAccountReference) : _that.transferorAccountReference != null) return false;
			if (transferorPartyReference != null ? !transferorPartyReference.equals(_that.transferorPartyReference) : _that.transferorPartyReference != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (transfereeAccountReference != null ? transfereeAccountReference.hashCode() : 0);
			_result = 31 * _result + (transfereePartyReference != null ? transfereePartyReference.hashCode() : 0);
			_result = 31 * _result + (transferorAccountReference != null ? transferorAccountReference.hashCode() : 0);
			_result = 31 * _result + (transferorPartyReference != null ? transferorPartyReference.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "TransferorTransfereeBuilder {" +
				"transfereeAccountReference=" + this.transfereeAccountReference + ", " +
				"transfereePartyReference=" + this.transfereePartyReference + ", " +
				"transferorAccountReference=" + this.transferorAccountReference + ", " +
				"transferorPartyReference=" + this.transferorPartyReference +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("transfereeAccountReference"), processor, ReferenceWithMetaAccount.class, transfereeAccountReference);
		processRosetta(path.newSubPath("transfereePartyReference"), processor, ReferenceWithMetaParty.class, transfereePartyReference);
		processRosetta(path.newSubPath("transferorAccountReference"), processor, ReferenceWithMetaAccount.class, transferorAccountReference);
		processRosetta(path.newSubPath("transferorPartyReference"), processor, ReferenceWithMetaParty.class, transferorPartyReference);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		TransferorTransferee _that = (TransferorTransferee) o;
	
		if (transfereeAccountReference != null ? !transfereeAccountReference.equals(_that.transfereeAccountReference) : _that.transfereeAccountReference != null) return false;
		if (transfereePartyReference != null ? !transfereePartyReference.equals(_that.transfereePartyReference) : _that.transfereePartyReference != null) return false;
		if (transferorAccountReference != null ? !transferorAccountReference.equals(_that.transferorAccountReference) : _that.transferorAccountReference != null) return false;
		if (transferorPartyReference != null ? !transferorPartyReference.equals(_that.transferorPartyReference) : _that.transferorPartyReference != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (transfereeAccountReference != null ? transfereeAccountReference.hashCode() : 0);
		_result = 31 * _result + (transfereePartyReference != null ? transfereePartyReference.hashCode() : 0);
		_result = 31 * _result + (transferorAccountReference != null ? transferorAccountReference.hashCode() : 0);
		_result = 31 * _result + (transferorPartyReference != null ? transferorPartyReference.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "TransferorTransferee {" +
			"transfereeAccountReference=" + this.transfereeAccountReference + ", " +
			"transfereePartyReference=" + this.transfereePartyReference + ", " +
			"transferorAccountReference=" + this.transferorAccountReference + ", " +
			"transferorPartyReference=" + this.transferorPartyReference +
		'}';
	}
}
