package org.isda.cdm;

import java.lang.String;
import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.UmbrellaAgreementEntity;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.UmbrellaAgreementMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;

/**
 * A class to specify a set of legal entities which are part of a legal agreement beyond the two contracting parties to that agreement. This data representation reflects the ISDA Create representation.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class UmbrellaAgreement extends RosettaModelObject {
	private final String language;
	private final List<UmbrellaAgreementEntity> parties;
	private static UmbrellaAgreementMeta metaData = new UmbrellaAgreementMeta();
	
	UmbrellaAgreement(UmbrellaAgreementBuilder builder) {
		this.language = builder.getLanguage();
		this.parties = ofNullable(builder.getParties()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
	}
	
	/**
	 * The language associated with the umbrella agreement, and which applies to all the parties to the umbrella agreement.
	 */
	@RosettaSynonym(value="language", source="ISDA_Create_1_0")
	public final String getLanguage() {
		return language;
	}
	
	/**
	 * The parties to the umbrella agreement.
	 */
	@RosettaSynonym(value="principal_identification_schedule", source="ISDA_Create_1_0")
	public final List<UmbrellaAgreementEntity> getParties() {
		return parties;
	}
	
	
	@Override
	public RosettaMetaData<? extends UmbrellaAgreement> metaData() {
		return metaData;
	} 
	
	public UmbrellaAgreementBuilder toBuilder() {
		UmbrellaAgreementBuilder builder = new UmbrellaAgreementBuilder();
		ofNullable(getLanguage()).ifPresent(builder::setLanguage);
		ofNullable(getParties()).ifPresent(parties -> parties.forEach(builder::addParties));
		return builder;
	}

	public static UmbrellaAgreementBuilder builder() {
		return new UmbrellaAgreementBuilder();
	}

	public static  class UmbrellaAgreementBuilder extends RosettaModelObjectBuilder {
	
		protected String language;
		protected List<UmbrellaAgreementEntity.UmbrellaAgreementEntityBuilder> parties;
	
		public UmbrellaAgreementBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends UmbrellaAgreement> metaData() {
			return metaData;
		} 
	
		
		public String getLanguage() {
			return language;
		}
		
		
		public List<UmbrellaAgreementEntity.UmbrellaAgreementEntityBuilder> getParties() {
			return parties;
		}
		
		public UmbrellaAgreementEntity.UmbrellaAgreementEntityBuilder getOrCreateParties(int index) {
			if (parties==null) {
				this.parties = new ArrayList<>();
			}
			return getIndex(parties, index, ()->new UmbrellaAgreementEntity.UmbrellaAgreementEntityBuilder());
		}
		
	
		public UmbrellaAgreementBuilder setLanguage(String language) {
			this.language = language;
			return this;
		}
		public UmbrellaAgreementBuilder addParties(UmbrellaAgreementEntity parties) {
			if(this.parties == null){
				this.parties = new ArrayList<>();
			}
			this.parties.add(parties.toBuilder());
			return this;
		}
		public UmbrellaAgreementBuilder addParties(UmbrellaAgreementEntity parties, int _idx) {
			if(this.parties == null){
				this.parties = new ArrayList<>();
			}
			getIndex(this.parties, _idx, () -> parties.toBuilder());
			this.parties.set(_idx, parties.toBuilder());
			return this;
		}
		public UmbrellaAgreementBuilder addParties(List<UmbrellaAgreementEntity> partiess) {
			if(this.parties == null){
				this.parties = new ArrayList<>();
			}
			for (UmbrellaAgreementEntity toAdd : partiess) {
				this.parties.add(toAdd.toBuilder());
			}
			return this;
		}
		public UmbrellaAgreementBuilder addPartiesBuilder(UmbrellaAgreementEntity.UmbrellaAgreementEntityBuilder parties) {
			if(this.parties == null){
				this.parties = new ArrayList<>();
			}
			this.parties.add(parties);
			return this;
		}
		
		public UmbrellaAgreementBuilder clearParties() {
			this.parties = null;
			return this;
		}
		
		public UmbrellaAgreement build() {
			return new UmbrellaAgreement(this);
		}
	
		@Override
		public UmbrellaAgreementBuilder prune() {
			if (parties!=null) parties = parties.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getLanguage()!=null) return true;
			if (getParties()!=null && getParties().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("language"), String.class, language, this);
			
			processRosetta(path.newSubPath("parties"), processor, UmbrellaAgreementEntity.class, parties);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			UmbrellaAgreementBuilder _that = (UmbrellaAgreementBuilder) o;
		
			if (language != null ? !language.equals(_that.language) : _that.language != null) return false;
			if (!ListEquals.listEquals(parties, _that.parties)) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (language != null ? language.hashCode() : 0);
			_result = 31 * _result + (parties != null ? parties.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "UmbrellaAgreementBuilder {" +
				"language=" + this.language + ", " +
				"parties=" + this.parties +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("language"), String.class, language, this);
		
		processRosetta(path.newSubPath("parties"), processor, UmbrellaAgreementEntity.class, parties);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		UmbrellaAgreement _that = (UmbrellaAgreement) o;
	
		if (language != null ? !language.equals(_that.language) : _that.language != null) return false;
		if (!ListEquals.listEquals(parties, _that.parties)) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (language != null ? language.hashCode() : 0);
		_result = 31 * _result + (parties != null ? parties.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "UmbrellaAgreement {" +
			"language=" + this.language + ", " +
			"parties=" + this.parties +
		'}';
	}
}
