package org.isda.cdm;

import java.lang.String;
import org.isda.cdm.metafields.FieldWithMetaString;


import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.UmbrellaAgreementEntityMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;

/**
 * A class to specify the legal entities that are part of the umbrella agreement.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class UmbrellaAgreementEntity extends LegalEntity {
	private final String terms;
	private static UmbrellaAgreementEntityMeta metaData = new UmbrellaAgreementEntityMeta();
	
	UmbrellaAgreementEntity(UmbrellaAgreementEntityBuilder builder) {
		super(builder);
		this.terms = builder.getTerms();
	}
	
	/**
	 * The terms that might be associated with each party to the umbrella agreement.
	 */
	@RosettaSynonym(value="additional", source="ISDA_Create_1_0")
	public final String getTerms() {
		return terms;
	}
	
	
	@Override
	public RosettaMetaData<? extends UmbrellaAgreementEntity> metaData() {
		return metaData;
	} 
	
	public UmbrellaAgreementEntityBuilder toBuilder() {
		UmbrellaAgreementEntityBuilder builder = new UmbrellaAgreementEntityBuilder();
		ofNullable(getTerms()).ifPresent(builder::setTerms);
		ofNullable(getEntityId()).ifPresent(entityId -> entityId.forEach(builder::addEntityId));
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		ofNullable(getName()).ifPresent(builder::setName);
		return builder;
	}

	public static UmbrellaAgreementEntityBuilder builder() {
		return new UmbrellaAgreementEntityBuilder();
	}

	public static  class UmbrellaAgreementEntityBuilder extends LegalEntityBuilder {
	
		protected String terms;
	
		public UmbrellaAgreementEntityBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends UmbrellaAgreementEntity> metaData() {
			return metaData;
		} 
	
		
		public String getTerms() {
			return terms;
		}
		
	
		public UmbrellaAgreementEntityBuilder setTerms(String terms) {
			this.terms = terms;
			return this;
		}
		@Override public UmbrellaAgreementEntityBuilder addEntityId(FieldWithMetaString entityId) {
			if(this.entityId == null){
				this.entityId = new ArrayList<>();
			}
			this.entityId.add(entityId.toBuilder());
			return this;
		}
		@Override public UmbrellaAgreementEntityBuilder addEntityId(FieldWithMetaString entityId, int _idx) {
			if(this.entityId == null){
				this.entityId = new ArrayList<>();
			}
			getIndex(this.entityId, _idx, () -> entityId.toBuilder());
			this.entityId.set(_idx, entityId.toBuilder());
			return this;
		}
		@Override public UmbrellaAgreementEntityBuilder addEntityId(List<FieldWithMetaString> entityIds) {
			if(this.entityId == null){
				this.entityId = new ArrayList<>();
			}
			for (FieldWithMetaString toAdd : entityIds) {
				this.entityId.add(toAdd.toBuilder());
			}
			return this;
		}
		
		@Override public UmbrellaAgreementEntityBuilder clearEntityId() {
			this.entityId = null;
			return this;
		}
		@Override public UmbrellaAgreementEntityBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		@Override public UmbrellaAgreementEntityBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		@Override public UmbrellaAgreementEntityBuilder setName(FieldWithMetaString name) {
			this.name = name.toBuilder();
			return this;
		}
		@Override public UmbrellaAgreementEntityBuilder setNameRef(String name) {
			return setName(FieldWithMetaString.builder().setValue(name).build());
		}
		
		public UmbrellaAgreementEntity build() {
			return new UmbrellaAgreementEntity(this);
		}
	
		@Override
		public UmbrellaAgreementEntityBuilder prune() {
			super.prune();
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (super.hasData()) return true;
			if (getTerms()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
			super.process(path, processor);
		
			processor.processBasic(path.newSubPath("terms"), String.class, terms, this);
			
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
			if (!super.equals(o)) return false;
		
			UmbrellaAgreementEntityBuilder _that = (UmbrellaAgreementEntityBuilder) o;
		
			if (terms != null ? !terms.equals(_that.terms) : _that.terms != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = super.hashCode();
			_result = 31 * _result + (terms != null ? terms.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "UmbrellaAgreementEntityBuilder {" +
				"terms=" + this.terms +
			'}' + " " + super.toString();
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		super.process(path, processor);
		
		processor.processBasic(path.newSubPath("terms"), String.class, terms, this);
		
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
		if (!super.equals(o)) return false;
	
		UmbrellaAgreementEntity _that = (UmbrellaAgreementEntity) o;
	
		if (terms != null ? !terms.equals(_that.terms) : _that.terms != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = super.hashCode();
		_result = 31 * _result + (terms != null ? terms.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "UmbrellaAgreementEntity {" +
			"terms=" + this.terms +
		'}' + " " + super.toString();
	}
}
