package org.isda.cdm;

import org.isda.cdm.Basket;
import org.isda.cdm.SingleUnderlier;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.UnderlierMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class describing the whole set of possible underliers: single underliers or multiple underliers, each of these having either security or index components.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="Underlyer", source="FpML_5_10")
public class Underlier extends RosettaModelObject {
	private final Basket basket;
	private final SingleUnderlier singleUnderlier;
	private static UnderlierMeta metaData = new UnderlierMeta();
	
	Underlier(UnderlierBuilder builder) {
		this.basket = ofNullable(builder.getBasket()).map(f->f.build()).orElse(null);
		this.singleUnderlier = ofNullable(builder.getSingleUnderlier()).map(f->f.build()).orElse(null);
	}
	
	public final Basket getBasket() {
		return basket;
	}
	
	public final SingleUnderlier getSingleUnderlier() {
		return singleUnderlier;
	}
	
	
	@Override
	public RosettaMetaData<? extends Underlier> metaData() {
		return metaData;
	} 
	
	public UnderlierBuilder toBuilder() {
		UnderlierBuilder builder = new UnderlierBuilder();
		ofNullable(getBasket()).ifPresent(builder::setBasket);
		ofNullable(getSingleUnderlier()).ifPresent(builder::setSingleUnderlier);
		return builder;
	}

	public static UnderlierBuilder builder() {
		return new UnderlierBuilder();
	}

	public static  class UnderlierBuilder extends RosettaModelObjectBuilder {
	
		protected Basket.BasketBuilder basket;
		protected SingleUnderlier.SingleUnderlierBuilder singleUnderlier;
	
		public UnderlierBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends Underlier> metaData() {
			return metaData;
		} 
	
		
		public Basket.BasketBuilder getBasket() {
			return basket;
		}
		
		public Basket.BasketBuilder getOrCreateBasket() {
			if (basket!=null) {
				return basket;
			}
			else return basket = new Basket.BasketBuilder();
		}
		
		
		public SingleUnderlier.SingleUnderlierBuilder getSingleUnderlier() {
			return singleUnderlier;
		}
		
		public SingleUnderlier.SingleUnderlierBuilder getOrCreateSingleUnderlier() {
			if (singleUnderlier!=null) {
				return singleUnderlier;
			}
			else return singleUnderlier = new SingleUnderlier.SingleUnderlierBuilder();
		}
		
	
		public UnderlierBuilder setBasket(Basket basket) {
			this.basket = basket.toBuilder();
			return this;
		}
		public UnderlierBuilder setBasketBuilder(Basket.BasketBuilder basket) {
			this.basket = basket;
			return this;
		}
		public UnderlierBuilder setSingleUnderlier(SingleUnderlier singleUnderlier) {
			this.singleUnderlier = singleUnderlier.toBuilder();
			return this;
		}
		public UnderlierBuilder setSingleUnderlierBuilder(SingleUnderlier.SingleUnderlierBuilder singleUnderlier) {
			this.singleUnderlier = singleUnderlier;
			return this;
		}
		
		public Underlier build() {
			return new Underlier(this);
		}
	
		@Override
		public UnderlierBuilder prune() {
			if (basket!=null && !basket.prune().hasData()) basket = null;
			if (singleUnderlier!=null && !singleUnderlier.prune().hasData()) singleUnderlier = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getBasket()!=null && getBasket().hasData()) return true;
			if (getSingleUnderlier()!=null && getSingleUnderlier().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("basket"), processor, Basket.class, basket);
			processRosetta(path.newSubPath("singleUnderlier"), processor, SingleUnderlier.class, singleUnderlier);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			UnderlierBuilder _that = (UnderlierBuilder) o;
		
			if (basket != null ? !basket.equals(_that.basket) : _that.basket != null) return false;
			if (singleUnderlier != null ? !singleUnderlier.equals(_that.singleUnderlier) : _that.singleUnderlier != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (basket != null ? basket.hashCode() : 0);
			_result = 31 * _result + (singleUnderlier != null ? singleUnderlier.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "UnderlierBuilder {" +
				"basket=" + this.basket + ", " +
				"singleUnderlier=" + this.singleUnderlier +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("basket"), processor, Basket.class, basket);
		processRosetta(path.newSubPath("singleUnderlier"), processor, SingleUnderlier.class, singleUnderlier);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		Underlier _that = (Underlier) o;
	
		if (basket != null ? !basket.equals(_that.basket) : _that.basket != null) return false;
		if (singleUnderlier != null ? !singleUnderlier.equals(_that.singleUnderlier) : _that.singleUnderlier != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (basket != null ? basket.hashCode() : 0);
		_result = 31 * _result + (singleUnderlier != null ? singleUnderlier.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "Underlier {" +
			"basket=" + this.basket + ", " +
			"singleUnderlier=" + this.singleUnderlier +
		'}';
	}
}
