package org.isda.cdm;

import org.isda.cdm.Money;
import org.isda.cdm.Quantity;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.UnitContractValuationModelMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * Unit contract model for security valuation, e.g. for equity, modelled onto UnitContract.model in FpML.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class UnitContractValuationModel extends RosettaModelObject {
	private final Quantity numberOfUnits;
	private final Money unitPrice;
	private static UnitContractValuationModelMeta metaData = new UnitContractValuationModelMeta();
	
	UnitContractValuationModel(UnitContractValuationModelBuilder builder) {
		this.numberOfUnits = ofNullable(builder.getNumberOfUnits()).map(f->f.build()).orElse(null);
		this.unitPrice = ofNullable(builder.getUnitPrice()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The number of units (index or securities).
	 */
	public final Quantity getNumberOfUnits() {
		return numberOfUnits;
	}
	
	/**
	 * The price of each unit.
	 */
	public final Money getUnitPrice() {
		return unitPrice;
	}
	
	
	@Override
	public RosettaMetaData<? extends UnitContractValuationModel> metaData() {
		return metaData;
	} 
	
	public UnitContractValuationModelBuilder toBuilder() {
		UnitContractValuationModelBuilder builder = new UnitContractValuationModelBuilder();
		ofNullable(getNumberOfUnits()).ifPresent(builder::setNumberOfUnits);
		ofNullable(getUnitPrice()).ifPresent(builder::setUnitPrice);
		return builder;
	}

	public static UnitContractValuationModelBuilder builder() {
		return new UnitContractValuationModelBuilder();
	}

	public static  class UnitContractValuationModelBuilder extends RosettaModelObjectBuilder {
	
		protected Quantity.QuantityBuilder numberOfUnits;
		protected Money.MoneyBuilder unitPrice;
	
		public UnitContractValuationModelBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends UnitContractValuationModel> metaData() {
			return metaData;
		} 
	
		
		public Quantity.QuantityBuilder getNumberOfUnits() {
			return numberOfUnits;
		}
		
		public Quantity.QuantityBuilder getOrCreateNumberOfUnits() {
			if (numberOfUnits!=null) {
				return numberOfUnits;
			}
			else return numberOfUnits = new Quantity.QuantityBuilder();
		}
		
		
		public Money.MoneyBuilder getUnitPrice() {
			return unitPrice;
		}
		
		public Money.MoneyBuilder getOrCreateUnitPrice() {
			if (unitPrice!=null) {
				return unitPrice;
			}
			else return unitPrice = new Money.MoneyBuilder();
		}
		
	
		public UnitContractValuationModelBuilder setNumberOfUnits(Quantity numberOfUnits) {
			this.numberOfUnits = numberOfUnits.toBuilder();
			return this;
		}
		public UnitContractValuationModelBuilder setNumberOfUnitsBuilder(Quantity.QuantityBuilder numberOfUnits) {
			this.numberOfUnits = numberOfUnits;
			return this;
		}
		public UnitContractValuationModelBuilder setUnitPrice(Money unitPrice) {
			this.unitPrice = unitPrice.toBuilder();
			return this;
		}
		public UnitContractValuationModelBuilder setUnitPriceBuilder(Money.MoneyBuilder unitPrice) {
			this.unitPrice = unitPrice;
			return this;
		}
		
		public UnitContractValuationModel build() {
			return new UnitContractValuationModel(this);
		}
	
		@Override
		public UnitContractValuationModelBuilder prune() {
			if (numberOfUnits!=null && !numberOfUnits.prune().hasData()) numberOfUnits = null;
			if (unitPrice!=null && !unitPrice.prune().hasData()) unitPrice = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getNumberOfUnits()!=null && getNumberOfUnits().hasData()) return true;
			if (getUnitPrice()!=null && getUnitPrice().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("numberOfUnits"), processor, Quantity.class, numberOfUnits);
			processRosetta(path.newSubPath("unitPrice"), processor, Money.class, unitPrice);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			UnitContractValuationModelBuilder _that = (UnitContractValuationModelBuilder) o;
		
			if (numberOfUnits != null ? !numberOfUnits.equals(_that.numberOfUnits) : _that.numberOfUnits != null) return false;
			if (unitPrice != null ? !unitPrice.equals(_that.unitPrice) : _that.unitPrice != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (numberOfUnits != null ? numberOfUnits.hashCode() : 0);
			_result = 31 * _result + (unitPrice != null ? unitPrice.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "UnitContractValuationModelBuilder {" +
				"numberOfUnits=" + this.numberOfUnits + ", " +
				"unitPrice=" + this.unitPrice +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("numberOfUnits"), processor, Quantity.class, numberOfUnits);
		processRosetta(path.newSubPath("unitPrice"), processor, Money.class, unitPrice);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		UnitContractValuationModel _that = (UnitContractValuationModel) o;
	
		if (numberOfUnits != null ? !numberOfUnits.equals(_that.numberOfUnits) : _that.numberOfUnits != null) return false;
		if (unitPrice != null ? !unitPrice.equals(_that.unitPrice) : _that.unitPrice != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (numberOfUnits != null ? numberOfUnits.hashCode() : 0);
		_result = 31 * _result + (unitPrice != null ? unitPrice.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "UnitContractValuationModel {" +
			"numberOfUnits=" + this.numberOfUnits + ", " +
			"unitPrice=" + this.unitPrice +
		'}';
	}
}
