package org.isda.cdm;

import org.isda.cdm.MultipleValuationDates;
import org.isda.cdm.SingleValuationDate;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.ValuationDateMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="ValuationDate", source="FpML_5_10")
public class ValuationDate extends RosettaModelObject {
	private final MultipleValuationDates multipleValuationDates;
	private final SingleValuationDate singleValuationDate;
	private static ValuationDateMeta metaData = new ValuationDateMeta();
	
	ValuationDate(ValuationDateBuilder builder) {
		this.multipleValuationDates = ofNullable(builder.getMultipleValuationDates()).map(f->f.build()).orElse(null);
		this.singleValuationDate = ofNullable(builder.getSingleValuationDate()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * Where multiple valuation dates are specified as being applicable for cash settlement, this element specifies (a) the number of applicable valuation dates, and (b) the number of business days after satisfaction of all conditions to settlement when the first such valuation date occurs, and (c) the number of business days thereafter of each successive valuation date. ISDA 2003 Term: Multiple Valuation Dates.
	 */
	@RosettaSynonym(value="multipleValuationDates", source="FpML_5_10")
	@RosettaSynonym(value="multipleValuationDates", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="multipleValuationDates", source="DTCC_11_0")
	@RosettaSynonym(value="multipleValuationDates", source="DTCC_9_0")
	@RosettaSynonym(value="multipleValuationDates", source="CME_ClearedConfirm_1_17")
	public final MultipleValuationDates getMultipleValuationDates() {
		return multipleValuationDates;
	}
	
	/**
	 * Where single valuation date is specified as being applicable for cash settlement, this element specifies the number of business days after satisfaction of all conditions to settlement when such valuation date occurs. ISDA 2003 Term: Single Valuation Date.
	 */
	@RosettaSynonym(value="singleValuationDate", source="FpML_5_10")
	@RosettaSynonym(value="singleValuationDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="singleValuationDate", source="DTCC_11_0")
	@RosettaSynonym(value="singleValuationDate", source="DTCC_9_0")
	@RosettaSynonym(value="singleValuationDate", source="CME_ClearedConfirm_1_17")
	public final SingleValuationDate getSingleValuationDate() {
		return singleValuationDate;
	}
	
	
	@Override
	public RosettaMetaData<? extends ValuationDate> metaData() {
		return metaData;
	} 
	
	public ValuationDateBuilder toBuilder() {
		ValuationDateBuilder builder = new ValuationDateBuilder();
		ofNullable(getMultipleValuationDates()).ifPresent(builder::setMultipleValuationDates);
		ofNullable(getSingleValuationDate()).ifPresent(builder::setSingleValuationDate);
		return builder;
	}

	public static ValuationDateBuilder builder() {
		return new ValuationDateBuilder();
	}

	public static  class ValuationDateBuilder extends RosettaModelObjectBuilder {
	
		protected MultipleValuationDates.MultipleValuationDatesBuilder multipleValuationDates;
		protected SingleValuationDate.SingleValuationDateBuilder singleValuationDate;
	
		public ValuationDateBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends ValuationDate> metaData() {
			return metaData;
		} 
	
		
		public MultipleValuationDates.MultipleValuationDatesBuilder getMultipleValuationDates() {
			return multipleValuationDates;
		}
		
		public MultipleValuationDates.MultipleValuationDatesBuilder getOrCreateMultipleValuationDates() {
			if (multipleValuationDates!=null) {
				return multipleValuationDates;
			}
			else return multipleValuationDates = new MultipleValuationDates.MultipleValuationDatesBuilder();
		}
		
		
		public SingleValuationDate.SingleValuationDateBuilder getSingleValuationDate() {
			return singleValuationDate;
		}
		
		public SingleValuationDate.SingleValuationDateBuilder getOrCreateSingleValuationDate() {
			if (singleValuationDate!=null) {
				return singleValuationDate;
			}
			else return singleValuationDate = new SingleValuationDate.SingleValuationDateBuilder();
		}
		
	
		public ValuationDateBuilder setMultipleValuationDates(MultipleValuationDates multipleValuationDates) {
			this.multipleValuationDates = multipleValuationDates.toBuilder();
			return this;
		}
		public ValuationDateBuilder setMultipleValuationDatesBuilder(MultipleValuationDates.MultipleValuationDatesBuilder multipleValuationDates) {
			this.multipleValuationDates = multipleValuationDates;
			return this;
		}
		public ValuationDateBuilder setSingleValuationDate(SingleValuationDate singleValuationDate) {
			this.singleValuationDate = singleValuationDate.toBuilder();
			return this;
		}
		public ValuationDateBuilder setSingleValuationDateBuilder(SingleValuationDate.SingleValuationDateBuilder singleValuationDate) {
			this.singleValuationDate = singleValuationDate;
			return this;
		}
		
		public ValuationDate build() {
			return new ValuationDate(this);
		}
	
		@Override
		public ValuationDateBuilder prune() {
			if (multipleValuationDates!=null && !multipleValuationDates.prune().hasData()) multipleValuationDates = null;
			if (singleValuationDate!=null && !singleValuationDate.prune().hasData()) singleValuationDate = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getMultipleValuationDates()!=null && getMultipleValuationDates().hasData()) return true;
			if (getSingleValuationDate()!=null && getSingleValuationDate().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("multipleValuationDates"), processor, MultipleValuationDates.class, multipleValuationDates);
			processRosetta(path.newSubPath("singleValuationDate"), processor, SingleValuationDate.class, singleValuationDate);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			ValuationDateBuilder _that = (ValuationDateBuilder) o;
		
			if (multipleValuationDates != null ? !multipleValuationDates.equals(_that.multipleValuationDates) : _that.multipleValuationDates != null) return false;
			if (singleValuationDate != null ? !singleValuationDate.equals(_that.singleValuationDate) : _that.singleValuationDate != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (multipleValuationDates != null ? multipleValuationDates.hashCode() : 0);
			_result = 31 * _result + (singleValuationDate != null ? singleValuationDate.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "ValuationDateBuilder {" +
				"multipleValuationDates=" + this.multipleValuationDates + ", " +
				"singleValuationDate=" + this.singleValuationDate +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("multipleValuationDates"), processor, MultipleValuationDates.class, multipleValuationDates);
		processRosetta(path.newSubPath("singleValuationDate"), processor, SingleValuationDate.class, singleValuationDate);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		ValuationDate _that = (ValuationDate) o;
	
		if (multipleValuationDates != null ? !multipleValuationDates.equals(_that.multipleValuationDates) : _that.multipleValuationDates != null) return false;
		if (singleValuationDate != null ? !singleValuationDate.equals(_that.singleValuationDate) : _that.singleValuationDate != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (multipleValuationDates != null ? multipleValuationDates.hashCode() : 0);
		_result = 31 * _result + (singleValuationDate != null ? singleValuationDate.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "ValuationDate {" +
			"multipleValuationDates=" + this.multipleValuationDates + ", " +
			"singleValuationDate=" + this.singleValuationDate +
		'}';
	}
}
