package org.isda.cdm;

import java.lang.Integer;
import java.math.BigDecimal;
import java.time.ZonedDateTime;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.WeightedAveragingObservationMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.time.ZonedDateTime;
		import java.math.BigDecimal;
		import static java.util.Optional.ofNullable;

/**
 * A single weighted averaging observation.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="WeightedAveragingObservation", source="FpML_5_10")
public class WeightedAveragingObservation extends RosettaModelObject {
	private final ZonedDateTime dateTime;
	private final Integer observationNumber;
	private final BigDecimal weight;
	private static WeightedAveragingObservationMeta metaData = new WeightedAveragingObservationMeta();
	
	WeightedAveragingObservation(WeightedAveragingObservationBuilder builder) {
		this.dateTime = builder.getDateTime();
		this.observationNumber = builder.getObservationNumber();
		this.weight = builder.getWeight();
	}
	
	/**
	 * Observation date time, which should be used when literal observation dates are required. The CDM specifies that the zoned date time is to be expressed in accordance with ISO 8601, either as UTC as an offset to UTC.
	 */
	@RosettaSynonym(value="dateTime", source="FpML_5_10")
	@RosettaSynonym(value="dateTime", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="dateTime", source="DTCC_11_0")
	@RosettaSynonym(value="dateTime", source="DTCC_9_0")
	@RosettaSynonym(value="dateTime", source="CME_ClearedConfirm_1_17")
	public final ZonedDateTime getDateTime() {
		return dateTime;
	}
	
	/**
	 * Observation number, which should be unique, within a series generated by a date schedule.
	 */
	@RosettaSynonym(value="observationNumber", source="FpML_5_10")
	@RosettaSynonym(value="observationNumber", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="observationNumber", source="DTCC_11_0")
	@RosettaSynonym(value="observationNumber", source="DTCC_9_0")
	@RosettaSynonym(value="observationNumber", source="CME_ClearedConfirm_1_17")
	public final Integer getObservationNumber() {
		return observationNumber;
	}
	
	/**
	 * Observation weight, which is used as a multiplier for the observation value.
	 */
	@RosettaSynonym(value="weight", source="FpML_5_10")
	@RosettaSynonym(value="weight", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="weight", source="DTCC_11_0")
	@RosettaSynonym(value="weight", source="DTCC_9_0")
	@RosettaSynonym(value="weight", source="CME_ClearedConfirm_1_17")
	public final BigDecimal getWeight() {
		return weight;
	}
	
	
	@Override
	public RosettaMetaData<? extends WeightedAveragingObservation> metaData() {
		return metaData;
	} 
	
	public WeightedAveragingObservationBuilder toBuilder() {
		WeightedAveragingObservationBuilder builder = new WeightedAveragingObservationBuilder();
		ofNullable(getDateTime()).ifPresent(builder::setDateTime);
		ofNullable(getObservationNumber()).ifPresent(builder::setObservationNumber);
		ofNullable(getWeight()).ifPresent(builder::setWeight);
		return builder;
	}

	public static WeightedAveragingObservationBuilder builder() {
		return new WeightedAveragingObservationBuilder();
	}

	public static  class WeightedAveragingObservationBuilder extends RosettaModelObjectBuilder {
	
		protected ZonedDateTime dateTime;
		protected Integer observationNumber;
		protected BigDecimal weight;
	
		public WeightedAveragingObservationBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends WeightedAveragingObservation> metaData() {
			return metaData;
		} 
	
		
		public ZonedDateTime getDateTime() {
			return dateTime;
		}
		
		
		public Integer getObservationNumber() {
			return observationNumber;
		}
		
		
		public BigDecimal getWeight() {
			return weight;
		}
		
	
		public WeightedAveragingObservationBuilder setDateTime(ZonedDateTime dateTime) {
			this.dateTime = dateTime;
			return this;
		}
		public WeightedAveragingObservationBuilder setObservationNumber(Integer observationNumber) {
			this.observationNumber = observationNumber;
			return this;
		}
		public WeightedAveragingObservationBuilder setWeight(BigDecimal weight) {
			this.weight = weight;
			return this;
		}
		
		public WeightedAveragingObservation build() {
			return new WeightedAveragingObservation(this);
		}
	
		@Override
		public WeightedAveragingObservationBuilder prune() {
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getDateTime()!=null) return true;
			if (getObservationNumber()!=null) return true;
			if (getWeight()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("dateTime"), ZonedDateTime.class, dateTime, this);
			processor.processBasic(path.newSubPath("observationNumber"), Integer.class, observationNumber, this);
			processor.processBasic(path.newSubPath("weight"), BigDecimal.class, weight, this);
			
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			WeightedAveragingObservationBuilder _that = (WeightedAveragingObservationBuilder) o;
		
			if (dateTime != null ? !dateTime.equals(_that.dateTime) : _that.dateTime != null) return false;
			if (observationNumber != null ? !observationNumber.equals(_that.observationNumber) : _that.observationNumber != null) return false;
			if (weight != null ? !weight.equals(_that.weight) : _that.weight != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (dateTime != null ? dateTime.hashCode() : 0);
			_result = 31 * _result + (observationNumber != null ? observationNumber.hashCode() : 0);
			_result = 31 * _result + (weight != null ? weight.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "WeightedAveragingObservationBuilder {" +
				"dateTime=" + this.dateTime + ", " +
				"observationNumber=" + this.observationNumber + ", " +
				"weight=" + this.weight +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("dateTime"), ZonedDateTime.class, dateTime, this);
		processor.processBasic(path.newSubPath("observationNumber"), Integer.class, observationNumber, this);
		processor.processBasic(path.newSubPath("weight"), BigDecimal.class, weight, this);
		
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		WeightedAveragingObservation _that = (WeightedAveragingObservation) o;
	
		if (dateTime != null ? !dateTime.equals(_that.dateTime) : _that.dateTime != null) return false;
		if (observationNumber != null ? !observationNumber.equals(_that.observationNumber) : _that.observationNumber != null) return false;
		if (weight != null ? !weight.equals(_that.weight) : _that.weight != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (dateTime != null ? dateTime.hashCode() : 0);
		_result = 31 * _result + (observationNumber != null ? observationNumber.hashCode() : 0);
		_result = 31 * _result + (weight != null ? weight.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "WeightedAveragingObservation {" +
			"dateTime=" + this.dateTime + ", " +
			"observationNumber=" + this.observationNumber + ", " +
			"weight=" + this.weight +
		'}';
	}
}
