package org.isda.cdm;

import org.isda.cdm.QuotationRateTypeEnum;
import org.isda.cdm.SettlementRateSource;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.YieldCurveMethodMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class defining the parameters required for each of the ISDA defined yield curve methods for cash settlement.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="YieldCurveMethod", source="FpML_5_10")
public class YieldCurveMethod extends RosettaModelObject {
	private final QuotationRateTypeEnum quotationRateType;
	private final SettlementRateSource settlementRateSource;
	private static YieldCurveMethodMeta metaData = new YieldCurveMethodMeta();
	
	YieldCurveMethod(YieldCurveMethodBuilder builder) {
		this.quotationRateType = builder.getQuotationRateType();
		this.settlementRateSource = ofNullable(builder.getSettlementRateSource()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * Which rate quote is to be observed, either Bid, Mid, Offer or Exercising Party Pays. The meaning of Exercising Party Pays is defined in the 2000 ISDA Definitions, Section 17.2. Certain Definitions Relating to Cash Settlement, paragraph (j).
	 */
	@RosettaSynonym(value="quotationRateType", source="FpML_5_10")
	@RosettaSynonym(value="quotationRateType", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="quotationRateType", source="DTCC_11_0")
	@RosettaSynonym(value="quotationRateType", source="DTCC_9_0")
	@RosettaSynonym(value="quotationRateType", source="CME_ClearedConfirm_1_17")
	public final QuotationRateTypeEnum getQuotationRateType() {
		return quotationRateType;
	}
	
	/**
	 * The method for obtaining a settlement rate. This may be from some information source (e.g. Reuters) or from a set of reference banks.
	 */
	@RosettaSynonym(value="settlementRateSource", source="FpML_5_10")
	@RosettaSynonym(value="settlementRateSource", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="settlementRateSource", source="DTCC_11_0")
	@RosettaSynonym(value="settlementRateSource", source="DTCC_9_0")
	@RosettaSynonym(value="settlementRateSource", source="CME_ClearedConfirm_1_17")
	public final SettlementRateSource getSettlementRateSource() {
		return settlementRateSource;
	}
	
	
	@Override
	public RosettaMetaData<? extends YieldCurveMethod> metaData() {
		return metaData;
	} 
	
	public YieldCurveMethodBuilder toBuilder() {
		YieldCurveMethodBuilder builder = new YieldCurveMethodBuilder();
		ofNullable(getQuotationRateType()).ifPresent(builder::setQuotationRateType);
		ofNullable(getSettlementRateSource()).ifPresent(builder::setSettlementRateSource);
		return builder;
	}

	public static YieldCurveMethodBuilder builder() {
		return new YieldCurveMethodBuilder();
	}

	public static  class YieldCurveMethodBuilder extends RosettaModelObjectBuilder {
	
		protected QuotationRateTypeEnum quotationRateType;
		protected SettlementRateSource.SettlementRateSourceBuilder settlementRateSource;
	
		public YieldCurveMethodBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends YieldCurveMethod> metaData() {
			return metaData;
		} 
	
		
		public QuotationRateTypeEnum getQuotationRateType() {
			return quotationRateType;
		}
		
		
		public SettlementRateSource.SettlementRateSourceBuilder getSettlementRateSource() {
			return settlementRateSource;
		}
		
		public SettlementRateSource.SettlementRateSourceBuilder getOrCreateSettlementRateSource() {
			if (settlementRateSource!=null) {
				return settlementRateSource;
			}
			else return settlementRateSource = new SettlementRateSource.SettlementRateSourceBuilder();
		}
		
	
		public YieldCurveMethodBuilder setQuotationRateType(QuotationRateTypeEnum quotationRateType) {
			this.quotationRateType = quotationRateType;
			return this;
		}
		public YieldCurveMethodBuilder setSettlementRateSource(SettlementRateSource settlementRateSource) {
			this.settlementRateSource = settlementRateSource.toBuilder();
			return this;
		}
		public YieldCurveMethodBuilder setSettlementRateSourceBuilder(SettlementRateSource.SettlementRateSourceBuilder settlementRateSource) {
			this.settlementRateSource = settlementRateSource;
			return this;
		}
		
		public YieldCurveMethod build() {
			return new YieldCurveMethod(this);
		}
	
		@Override
		public YieldCurveMethodBuilder prune() {
			if (settlementRateSource!=null && !settlementRateSource.prune().hasData()) settlementRateSource = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getQuotationRateType()!=null) return true;
			if (getSettlementRateSource()!=null && getSettlementRateSource().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("quotationRateType"), QuotationRateTypeEnum.class, quotationRateType, this);
			
			processRosetta(path.newSubPath("settlementRateSource"), processor, SettlementRateSource.class, settlementRateSource);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			YieldCurveMethodBuilder _that = (YieldCurveMethodBuilder) o;
		
			if (quotationRateType != null ? !quotationRateType.equals(_that.quotationRateType) : _that.quotationRateType != null) return false;
			if (settlementRateSource != null ? !settlementRateSource.equals(_that.settlementRateSource) : _that.settlementRateSource != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (quotationRateType != null ? quotationRateType.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (settlementRateSource != null ? settlementRateSource.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "YieldCurveMethodBuilder {" +
				"quotationRateType=" + this.quotationRateType + ", " +
				"settlementRateSource=" + this.settlementRateSource +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("quotationRateType"), QuotationRateTypeEnum.class, quotationRateType, this);
		
		processRosetta(path.newSubPath("settlementRateSource"), processor, SettlementRateSource.class, settlementRateSource);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		YieldCurveMethod _that = (YieldCurveMethod) o;
	
		if (quotationRateType != null ? !quotationRateType.equals(_that.quotationRateType) : _that.quotationRateType != null) return false;
		if (settlementRateSource != null ? !settlementRateSource.equals(_that.settlementRateSource) : _that.settlementRateSource != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (quotationRateType != null ? quotationRateType.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (settlementRateSource != null ? settlementRateSource.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "YieldCurveMethod {" +
			"quotationRateType=" + this.quotationRateType + ", " +
			"settlementRateSource=" + this.settlementRateSource +
		'}';
	}
}
