package org.isda.cdm.functions;

import com.google.inject.Inject;
import com.rosetta.model.lib.functions.IsLeapYear;
import com.rosetta.model.lib.functions.Mapper;
import com.rosetta.model.lib.functions.MapperBuilder;
import com.rosetta.model.lib.functions.MapperMaths;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.Min;
import com.rosetta.model.lib.functions.RosettaFunction;
import com.rosetta.model.lib.math.BigDecimalExtensions;
import com.rosetta.model.lib.records.Date;
import java.lang.Boolean;
import java.lang.Integer;
import java.math.BigDecimal;
import org.isda.cdm.AdjustableOrRelativeDate;
import org.isda.cdm.CalculationPeriodData;
import org.isda.cdm.CalculationPeriodDates;
import org.isda.cdm.CalculationPeriodFrequency;
import org.isda.cdm.DayCountFractionEnum;
import org.isda.cdm.InterestRatePayout;
import org.isda.cdm.functions.CalculationPeriod;
import org.isda.cdm.functions.PeriodsInYear;
import org.isda.cdm.functions.ResolveAdjustableDate;

import static com.rosetta.model.lib.validation.ValidatorHelper.areEqual;
import static com.rosetta.model.lib.validation.ValidatorHelper.doIf;
import static com.rosetta.model.lib.validation.ValidatorHelper.greaterThan;

/**
 * @version 2.5.4
 */
public class DayCountFraction {
	
	@Inject protected DayCountFraction._1_1 _1_1;
	@Inject protected DayCountFraction.ACT_ACT_ISDA ACT_ACT_ISDA;
	@Inject protected DayCountFraction.ACT_ACT_ICMA ACT_ACT_ICMA;
	@Inject protected DayCountFraction.ACT_365_FIXED ACT_365_FIXED;
	@Inject protected DayCountFraction.ACT_360 ACT_360;
	@Inject protected DayCountFraction._30_360 _30_360;
	@Inject protected DayCountFraction._30E_360 _30E_360;
	@Inject protected DayCountFraction._30E_360_ISDA _30E_360_ISDA;
	@Inject protected DayCountFraction.ACT_365L ACT_365L;
	
	public BigDecimal evaluate(InterestRatePayout interestRatePayout, DayCountFractionEnum dayCountFractionEnum, Date date) {
		switch (dayCountFractionEnum) {
			case _1_1:
				return _1_1.evaluate(interestRatePayout, dayCountFractionEnum, date);
			case ACT_ACT_ISDA:
				return ACT_ACT_ISDA.evaluate(interestRatePayout, dayCountFractionEnum, date);
			case ACT_ACT_ICMA:
				return ACT_ACT_ICMA.evaluate(interestRatePayout, dayCountFractionEnum, date);
			case ACT_365_FIXED:
				return ACT_365_FIXED.evaluate(interestRatePayout, dayCountFractionEnum, date);
			case ACT_360:
				return ACT_360.evaluate(interestRatePayout, dayCountFractionEnum, date);
			case _30_360:
				return _30_360.evaluate(interestRatePayout, dayCountFractionEnum, date);
			case _30E_360:
				return _30E_360.evaluate(interestRatePayout, dayCountFractionEnum, date);
			case _30E_360_ISDA:
				return _30E_360_ISDA.evaluate(interestRatePayout, dayCountFractionEnum, date);
			case ACT_365L:
				return ACT_365L.evaluate(interestRatePayout, dayCountFractionEnum, date);
			default:
				throw new IllegalArgumentException("Enum value not implemented: " + dayCountFractionEnum);
		}
	}
	
	
	public static class _1_1 implements RosettaFunction {
	
		/**
		* @param interestRatePayout 
		* @param dayCountFractionEnum 
		* @param date 
		* @return result 
		*/
		public BigDecimal evaluate(InterestRatePayout interestRatePayout, DayCountFractionEnum dayCountFractionEnum, Date date) {
			
			BigDecimal result = doEvaluate(interestRatePayout, dayCountFractionEnum, date);
			
			return result;
		}
		
		protected BigDecimal doEvaluate(InterestRatePayout interestRatePayout, DayCountFractionEnum dayCountFractionEnum, Date date) {
			Mapper<BigDecimal> resultHolder = null;
			resultHolder = MapperS.of(BigDecimalExtensions.valueOf(1.0));
			return resultHolder.get();
		}
		
	}
	
	public static class ACT_ACT_ISDA implements RosettaFunction {
		
		// RosettaFunction dependencies
		//
		@Inject protected CalculationPeriod calculationPeriod;
	
		/**
		* @param interestRatePayout 
		* @param dayCountFractionEnum 
		* @param date 
		* @return result 
		*/
		public BigDecimal evaluate(InterestRatePayout interestRatePayout, DayCountFractionEnum dayCountFractionEnum, Date date) {
			
			BigDecimal result = doEvaluate(interestRatePayout, dayCountFractionEnum, date);
			
			return result;
		}
		
		protected BigDecimal doEvaluate(InterestRatePayout interestRatePayout, DayCountFractionEnum dayCountFractionEnum, Date date) {
			Mapper<BigDecimal> resultHolder = null;
			resultHolder = MapperS.of(BigDecimalExtensions.add((BigDecimalExtensions.divide(BigDecimalExtensions.valueOf(daysInNonLeapPeriod(interestRatePayout, dayCountFractionEnum, date).get()), BigDecimalExtensions.valueOf(365))), (BigDecimalExtensions.divide(BigDecimalExtensions.valueOf(daysInLeapYearPeriod(interestRatePayout, dayCountFractionEnum, date).get()), BigDecimalExtensions.valueOf(366)))));
			return resultHolder.get();
		}
		
		
		protected MapperBuilder<CalculationPeriodData> calculationPeriod(InterestRatePayout interestRatePayout, DayCountFractionEnum dayCountFractionEnum, Date date) {
			return MapperS.of(calculationPeriod.evaluate(MapperS.of(interestRatePayout).<CalculationPeriodDates>map("getCalculationPeriodDates", InterestRatePayout::getCalculationPeriodDates).get(), MapperS.of(date).get()));
		}
		
		protected Mapper<Integer> daysInLeapYearPeriod(InterestRatePayout interestRatePayout, DayCountFractionEnum dayCountFractionEnum, Date date) {
			return MapperS.of(calculationPeriod(interestRatePayout, dayCountFractionEnum, date).get()).<Integer>map("getDaysInLeapYearPeriod", CalculationPeriodData::getDaysInLeapYearPeriod);
		}
		
		protected Mapper<Integer> daysInNonLeapPeriod(InterestRatePayout interestRatePayout, DayCountFractionEnum dayCountFractionEnum, Date date) {
			return MapperMaths.<Integer, Integer, Integer>subtract(MapperS.of(calculationPeriod(interestRatePayout, dayCountFractionEnum, date).get()).<Integer>map("getDaysInPeriod", CalculationPeriodData::getDaysInPeriod), MapperS.of(calculationPeriod(interestRatePayout, dayCountFractionEnum, date).get()).<Integer>map("getDaysInLeapYearPeriod", CalculationPeriodData::getDaysInLeapYearPeriod));
		}
	}
	
	public static class ACT_ACT_ICMA implements RosettaFunction {
		
		// RosettaFunction dependencies
		//
		@Inject protected CalculationPeriod calculationPeriod;
		@Inject protected PeriodsInYear periodsInYear;
	
		/**
		* @param interestRatePayout 
		* @param dayCountFractionEnum 
		* @param date 
		* @return result 
		*/
		public BigDecimal evaluate(InterestRatePayout interestRatePayout, DayCountFractionEnum dayCountFractionEnum, Date date) {
			
			BigDecimal result = doEvaluate(interestRatePayout, dayCountFractionEnum, date);
			
			return result;
		}
		
		protected BigDecimal doEvaluate(InterestRatePayout interestRatePayout, DayCountFractionEnum dayCountFractionEnum, Date date) {
			Mapper<BigDecimal> resultHolder = null;
			resultHolder = MapperS.of(BigDecimalExtensions.divide(BigDecimalExtensions.valueOf(daysInPeriod(interestRatePayout, dayCountFractionEnum, date).get()), BigDecimalExtensions.valueOf(((daysInPeriod(interestRatePayout, dayCountFractionEnum, date).get() * periodsInYear(interestRatePayout, dayCountFractionEnum, date).get())))));
			return resultHolder.get();
		}
		
		
		protected MapperBuilder<CalculationPeriodData> calculationPeriod(InterestRatePayout interestRatePayout, DayCountFractionEnum dayCountFractionEnum, Date date) {
			return MapperS.of(calculationPeriod.evaluate(MapperS.of(interestRatePayout).<CalculationPeriodDates>map("getCalculationPeriodDates", InterestRatePayout::getCalculationPeriodDates).get(), MapperS.of(date).get()));
		}
		
		protected Mapper<Integer> daysInPeriod(InterestRatePayout interestRatePayout, DayCountFractionEnum dayCountFractionEnum, Date date) {
			return MapperS.of(calculationPeriod(interestRatePayout, dayCountFractionEnum, date).get()).<Integer>map("getDaysInPeriod", CalculationPeriodData::getDaysInPeriod);
		}
		
		protected Mapper<Integer> periodsInYear(InterestRatePayout interestRatePayout, DayCountFractionEnum dayCountFractionEnum, Date date) {
			return MapperS.of(periodsInYear.evaluate(MapperS.of(interestRatePayout).<CalculationPeriodDates>map("getCalculationPeriodDates", InterestRatePayout::getCalculationPeriodDates).<CalculationPeriodFrequency>map("getCalculationPeriodFrequency", CalculationPeriodDates::getCalculationPeriodFrequency).get()));
		}
	}
	
	public static class ACT_365_FIXED implements RosettaFunction {
		
		// RosettaFunction dependencies
		//
		@Inject protected CalculationPeriod calculationPeriod;
	
		/**
		* @param interestRatePayout 
		* @param dayCountFractionEnum 
		* @param date 
		* @return result 
		*/
		public BigDecimal evaluate(InterestRatePayout interestRatePayout, DayCountFractionEnum dayCountFractionEnum, Date date) {
			
			BigDecimal result = doEvaluate(interestRatePayout, dayCountFractionEnum, date);
			
			return result;
		}
		
		protected BigDecimal doEvaluate(InterestRatePayout interestRatePayout, DayCountFractionEnum dayCountFractionEnum, Date date) {
			Mapper<BigDecimal> resultHolder = null;
			resultHolder = MapperS.of(BigDecimalExtensions.divide(BigDecimalExtensions.valueOf(daysInPeriod(interestRatePayout, dayCountFractionEnum, date).get()), BigDecimalExtensions.valueOf(365)));
			return resultHolder.get();
		}
		
		
		protected Mapper<Integer> daysInPeriod(InterestRatePayout interestRatePayout, DayCountFractionEnum dayCountFractionEnum, Date date) {
			return MapperS.of(calculationPeriod.evaluate(MapperS.of(interestRatePayout).<CalculationPeriodDates>map("getCalculationPeriodDates", InterestRatePayout::getCalculationPeriodDates).get(), MapperS.of(date).get())).<Integer>map("getDaysInPeriod", CalculationPeriodData::getDaysInPeriod);
		}
	}
	
	public static class ACT_360 implements RosettaFunction {
		
		// RosettaFunction dependencies
		//
		@Inject protected CalculationPeriod calculationPeriod;
	
		/**
		* @param interestRatePayout 
		* @param dayCountFractionEnum 
		* @param date 
		* @return result 
		*/
		public BigDecimal evaluate(InterestRatePayout interestRatePayout, DayCountFractionEnum dayCountFractionEnum, Date date) {
			
			BigDecimal result = doEvaluate(interestRatePayout, dayCountFractionEnum, date);
			
			return result;
		}
		
		protected BigDecimal doEvaluate(InterestRatePayout interestRatePayout, DayCountFractionEnum dayCountFractionEnum, Date date) {
			Mapper<BigDecimal> resultHolder = null;
			resultHolder = MapperS.of(BigDecimalExtensions.divide(BigDecimalExtensions.valueOf(daysInPeriod(interestRatePayout, dayCountFractionEnum, date).get()), BigDecimalExtensions.valueOf(360)));
			return resultHolder.get();
		}
		
		
		protected Mapper<Integer> daysInPeriod(InterestRatePayout interestRatePayout, DayCountFractionEnum dayCountFractionEnum, Date date) {
			return MapperS.of(calculationPeriod.evaluate(MapperS.of(interestRatePayout).<CalculationPeriodDates>map("getCalculationPeriodDates", InterestRatePayout::getCalculationPeriodDates).get(), MapperS.of(date).get())).<Integer>map("getDaysInPeriod", CalculationPeriodData::getDaysInPeriod);
		}
	}
	
	public static class _30_360 implements RosettaFunction {
		
		// RosettaFunction dependencies
		//
		@Inject protected CalculationPeriod calculationPeriod;
	
		/**
		* @param interestRatePayout 
		* @param dayCountFractionEnum 
		* @param date 
		* @return result 
		*/
		public BigDecimal evaluate(InterestRatePayout interestRatePayout, DayCountFractionEnum dayCountFractionEnum, Date date) {
			
			BigDecimal result = doEvaluate(interestRatePayout, dayCountFractionEnum, date);
			
			return result;
		}
		
		protected BigDecimal doEvaluate(InterestRatePayout interestRatePayout, DayCountFractionEnum dayCountFractionEnum, Date date) {
			Mapper<BigDecimal> resultHolder = null;
			resultHolder = MapperS.of(BigDecimalExtensions.divide(BigDecimalExtensions.valueOf(((((360 * ((endYear(interestRatePayout, dayCountFractionEnum, date).get() - startYear(interestRatePayout, dayCountFractionEnum, date).get()))) + (30 * ((endMonth(interestRatePayout, dayCountFractionEnum, date).get() - startMonth(interestRatePayout, dayCountFractionEnum, date).get())))) + ((endDay(interestRatePayout, dayCountFractionEnum, date).get() - startDay(interestRatePayout, dayCountFractionEnum, date).get()))))), BigDecimalExtensions.valueOf(360)));
			return resultHolder.get();
		}
		
		
		protected MapperBuilder<CalculationPeriodData> calculationPeriod(InterestRatePayout interestRatePayout, DayCountFractionEnum dayCountFractionEnum, Date date) {
			return MapperS.of(calculationPeriod.evaluate(MapperS.of(interestRatePayout).<CalculationPeriodDates>map("getCalculationPeriodDates", InterestRatePayout::getCalculationPeriodDates).get(), MapperS.of(date).get()));
		}
		
		protected Mapper<Integer> startYear(InterestRatePayout interestRatePayout, DayCountFractionEnum dayCountFractionEnum, Date date) {
			return MapperS.of(calculationPeriod(interestRatePayout, dayCountFractionEnum, date).get()).<Date>map("getStartDate", CalculationPeriodData::getStartDate).map("getYear", Date::getYear);
		}
		
		protected Mapper<Integer> endYear(InterestRatePayout interestRatePayout, DayCountFractionEnum dayCountFractionEnum, Date date) {
			return MapperS.of(calculationPeriod(interestRatePayout, dayCountFractionEnum, date).get()).<Date>map("getEndDate", CalculationPeriodData::getEndDate).map("getYear", Date::getYear);
		}
		
		protected Mapper<Integer> startMonth(InterestRatePayout interestRatePayout, DayCountFractionEnum dayCountFractionEnum, Date date) {
			return MapperS.of(calculationPeriod(interestRatePayout, dayCountFractionEnum, date).get()).<Date>map("getStartDate", CalculationPeriodData::getStartDate).map("getMonth", Date::getMonth);
		}
		
		protected Mapper<Integer> endMonth(InterestRatePayout interestRatePayout, DayCountFractionEnum dayCountFractionEnum, Date date) {
			return MapperS.of(calculationPeriod(interestRatePayout, dayCountFractionEnum, date).get()).<Date>map("getEndDate", CalculationPeriodData::getEndDate).map("getMonth", Date::getMonth);
		}
		
		protected Mapper<Integer> endDay(InterestRatePayout interestRatePayout, DayCountFractionEnum dayCountFractionEnum, Date date) {
			return doIf(greaterThan(MapperS.of(calculationPeriod(interestRatePayout, dayCountFractionEnum, date).get()).<Date>map("getStartDate", CalculationPeriodData::getStartDate).map("getDay", Date::getDay), MapperS.of(Integer.valueOf(29))),MapperS.of(new Min().execute(MapperS.of(calculationPeriod(interestRatePayout, dayCountFractionEnum, date).get()).<Date>map("getEndDate", CalculationPeriodData::getEndDate).map("getDay", Date::getDay).get(), MapperS.of(Integer.valueOf(30)).get())),MapperS.of(calculationPeriod(interestRatePayout, dayCountFractionEnum, date).get()).<Date>map("getEndDate", CalculationPeriodData::getEndDate).map("getDay", Date::getDay));
		}
		
		protected Mapper<Integer> startDay(InterestRatePayout interestRatePayout, DayCountFractionEnum dayCountFractionEnum, Date date) {
			return MapperS.of(new Min().execute(MapperS.of(calculationPeriod(interestRatePayout, dayCountFractionEnum, date).get()).<Date>map("getStartDate", CalculationPeriodData::getStartDate).map("getDay", Date::getDay).get(), MapperS.of(Integer.valueOf(30)).get()));
		}
	}
	
	public static class _30E_360 implements RosettaFunction {
		
		// RosettaFunction dependencies
		//
		@Inject protected CalculationPeriod calculationPeriod;
	
		/**
		* @param interestRatePayout 
		* @param dayCountFractionEnum 
		* @param date 
		* @return result 
		*/
		public BigDecimal evaluate(InterestRatePayout interestRatePayout, DayCountFractionEnum dayCountFractionEnum, Date date) {
			
			BigDecimal result = doEvaluate(interestRatePayout, dayCountFractionEnum, date);
			
			return result;
		}
		
		protected BigDecimal doEvaluate(InterestRatePayout interestRatePayout, DayCountFractionEnum dayCountFractionEnum, Date date) {
			Mapper<BigDecimal> resultHolder = null;
			resultHolder = MapperS.of(BigDecimalExtensions.divide(BigDecimalExtensions.valueOf(((((360 * ((endYear(interestRatePayout, dayCountFractionEnum, date).get() - startYear(interestRatePayout, dayCountFractionEnum, date).get()))) + (30 * ((endMonth(interestRatePayout, dayCountFractionEnum, date).get() - startMonth(interestRatePayout, dayCountFractionEnum, date).get())))) + ((endDay(interestRatePayout, dayCountFractionEnum, date).get() - startDay(interestRatePayout, dayCountFractionEnum, date).get()))))), BigDecimalExtensions.valueOf(360)));
			return resultHolder.get();
		}
		
		
		protected MapperBuilder<CalculationPeriodData> calculationPeriod(InterestRatePayout interestRatePayout, DayCountFractionEnum dayCountFractionEnum, Date date) {
			return MapperS.of(calculationPeriod.evaluate(MapperS.of(interestRatePayout).<CalculationPeriodDates>map("getCalculationPeriodDates", InterestRatePayout::getCalculationPeriodDates).get(), MapperS.of(date).get()));
		}
		
		protected Mapper<Integer> startYear(InterestRatePayout interestRatePayout, DayCountFractionEnum dayCountFractionEnum, Date date) {
			return MapperS.of(calculationPeriod(interestRatePayout, dayCountFractionEnum, date).get()).<Date>map("getStartDate", CalculationPeriodData::getStartDate).map("getYear", Date::getYear);
		}
		
		protected Mapper<Integer> endYear(InterestRatePayout interestRatePayout, DayCountFractionEnum dayCountFractionEnum, Date date) {
			return MapperS.of(calculationPeriod(interestRatePayout, dayCountFractionEnum, date).get()).<Date>map("getEndDate", CalculationPeriodData::getEndDate).map("getYear", Date::getYear);
		}
		
		protected Mapper<Integer> startMonth(InterestRatePayout interestRatePayout, DayCountFractionEnum dayCountFractionEnum, Date date) {
			return MapperS.of(calculationPeriod(interestRatePayout, dayCountFractionEnum, date).get()).<Date>map("getStartDate", CalculationPeriodData::getStartDate).map("getMonth", Date::getMonth);
		}
		
		protected Mapper<Integer> endMonth(InterestRatePayout interestRatePayout, DayCountFractionEnum dayCountFractionEnum, Date date) {
			return MapperS.of(calculationPeriod(interestRatePayout, dayCountFractionEnum, date).get()).<Date>map("getEndDate", CalculationPeriodData::getEndDate).map("getMonth", Date::getMonth);
		}
		
		protected Mapper<Integer> endDay(InterestRatePayout interestRatePayout, DayCountFractionEnum dayCountFractionEnum, Date date) {
			return MapperS.of(new Min().execute(MapperS.of(calculationPeriod(interestRatePayout, dayCountFractionEnum, date).get()).<Date>map("getEndDate", CalculationPeriodData::getEndDate).map("getDay", Date::getDay).get(), MapperS.of(Integer.valueOf(30)).get()));
		}
		
		protected Mapper<Integer> startDay(InterestRatePayout interestRatePayout, DayCountFractionEnum dayCountFractionEnum, Date date) {
			return MapperS.of(new Min().execute(MapperS.of(calculationPeriod(interestRatePayout, dayCountFractionEnum, date).get()).<Date>map("getStartDate", CalculationPeriodData::getStartDate).map("getDay", Date::getDay).get(), MapperS.of(Integer.valueOf(30)).get()));
		}
	}
	
	public static class _30E_360_ISDA implements RosettaFunction {
		
		// RosettaFunction dependencies
		//
		@Inject protected CalculationPeriod calculationPeriod;
		@Inject protected ResolveAdjustableDate resolveAdjustableDate;
	
		/**
		* @param interestRatePayout 
		* @param dayCountFractionEnum 
		* @param date 
		* @return result 
		*/
		public BigDecimal evaluate(InterestRatePayout interestRatePayout, DayCountFractionEnum dayCountFractionEnum, Date date) {
			
			BigDecimal result = doEvaluate(interestRatePayout, dayCountFractionEnum, date);
			
			return result;
		}
		
		protected BigDecimal doEvaluate(InterestRatePayout interestRatePayout, DayCountFractionEnum dayCountFractionEnum, Date date) {
			Mapper<BigDecimal> resultHolder = null;
			resultHolder = MapperS.of(BigDecimalExtensions.divide(BigDecimalExtensions.valueOf(((((360 * ((endYear(interestRatePayout, dayCountFractionEnum, date).get() - startYear(interestRatePayout, dayCountFractionEnum, date).get()))) + (30 * ((endMonth(interestRatePayout, dayCountFractionEnum, date).get() - startMonth(interestRatePayout, dayCountFractionEnum, date).get())))) + ((endDay(interestRatePayout, dayCountFractionEnum, date).get() - startDay(interestRatePayout, dayCountFractionEnum, date).get()))))), BigDecimalExtensions.valueOf(360)));
			return resultHolder.get();
		}
		
		
		protected MapperBuilder<CalculationPeriodData> calculationPeriod(InterestRatePayout interestRatePayout, DayCountFractionEnum dayCountFractionEnum, Date date) {
			return MapperS.of(calculationPeriod.evaluate(MapperS.of(interestRatePayout).<CalculationPeriodDates>map("getCalculationPeriodDates", InterestRatePayout::getCalculationPeriodDates).get(), MapperS.of(date).get()));
		}
		
		protected Mapper<Date> interestRatePayoutTerminationDate(InterestRatePayout interestRatePayout, DayCountFractionEnum dayCountFractionEnum, Date date) {
			return MapperS.of(resolveAdjustableDate.evaluate(MapperS.of(interestRatePayout).<CalculationPeriodDates>map("getCalculationPeriodDates", InterestRatePayout::getCalculationPeriodDates).<AdjustableOrRelativeDate>map("getTerminationDate", CalculationPeriodDates::getTerminationDate).get()));
		}
		
		protected Mapper<Boolean> startDateIsInLeapYear(InterestRatePayout interestRatePayout, DayCountFractionEnum dayCountFractionEnum, Date date) {
			return MapperS.of(new IsLeapYear().execute(MapperS.of(calculationPeriod(interestRatePayout, dayCountFractionEnum, date).get()).<Date>map("getStartDate", CalculationPeriodData::getStartDate).map("getYear", Date::getYear).get()));
		}
		
		protected Mapper<Boolean> endDateIsInLeapYear(InterestRatePayout interestRatePayout, DayCountFractionEnum dayCountFractionEnum, Date date) {
			return MapperS.of(new IsLeapYear().execute(MapperS.of(calculationPeriod(interestRatePayout, dayCountFractionEnum, date).get()).<Date>map("getEndDate", CalculationPeriodData::getEndDate).map("getYear", Date::getYear).get()));
		}
		
		protected Mapper<Integer> startYear(InterestRatePayout interestRatePayout, DayCountFractionEnum dayCountFractionEnum, Date date) {
			return MapperS.of(calculationPeriod(interestRatePayout, dayCountFractionEnum, date).get()).<Date>map("getStartDate", CalculationPeriodData::getStartDate).map("getYear", Date::getYear);
		}
		
		protected Mapper<Integer> endYear(InterestRatePayout interestRatePayout, DayCountFractionEnum dayCountFractionEnum, Date date) {
			return MapperS.of(calculationPeriod(interestRatePayout, dayCountFractionEnum, date).get()).<Date>map("getEndDate", CalculationPeriodData::getEndDate).map("getYear", Date::getYear);
		}
		
		protected Mapper<Integer> startMonth(InterestRatePayout interestRatePayout, DayCountFractionEnum dayCountFractionEnum, Date date) {
			return MapperS.of(calculationPeriod(interestRatePayout, dayCountFractionEnum, date).get()).<Date>map("getStartDate", CalculationPeriodData::getStartDate).map("getMonth", Date::getMonth);
		}
		
		protected Mapper<Integer> endMonth(InterestRatePayout interestRatePayout, DayCountFractionEnum dayCountFractionEnum, Date date) {
			return MapperS.of(calculationPeriod(interestRatePayout, dayCountFractionEnum, date).get()).<Date>map("getEndDate", CalculationPeriodData::getEndDate).map("getMonth", Date::getMonth);
		}
		
		protected Mapper<Integer> startDay(InterestRatePayout interestRatePayout, DayCountFractionEnum dayCountFractionEnum, Date date) {
			return doIf(areEqual(MapperS.of(startDateIsInLeapYear(interestRatePayout, dayCountFractionEnum, date).get()), MapperS.of(Boolean.valueOf(false))).and(areEqual(MapperS.of(calculationPeriod(interestRatePayout, dayCountFractionEnum, date).get()).<Date>map("getStartDate", CalculationPeriodData::getStartDate).map("getMonth", Date::getMonth), MapperS.of(Integer.valueOf(2)))).and(areEqual(MapperS.of(calculationPeriod(interestRatePayout, dayCountFractionEnum, date).get()).<Date>map("getStartDate", CalculationPeriodData::getStartDate).map("getDay", Date::getDay), MapperS.of(Integer.valueOf(28)))).or(areEqual(MapperS.of(startDateIsInLeapYear(interestRatePayout, dayCountFractionEnum, date).get()), MapperS.of(Boolean.valueOf(true))).and(areEqual(MapperS.of(calculationPeriod(interestRatePayout, dayCountFractionEnum, date).get()).<Date>map("getStartDate", CalculationPeriodData::getStartDate).map("getMonth", Date::getMonth), MapperS.of(Integer.valueOf(2)))).and(areEqual(MapperS.of(calculationPeriod(interestRatePayout, dayCountFractionEnum, date).get()).<Date>map("getStartDate", CalculationPeriodData::getStartDate).map("getDay", Date::getDay), MapperS.of(Integer.valueOf(29))))).or(areEqual(MapperS.of(calculationPeriod(interestRatePayout, dayCountFractionEnum, date).get()).<Date>map("getStartDate", CalculationPeriodData::getStartDate).map("getDay", Date::getDay), MapperS.of(Integer.valueOf(31)))),MapperS.of(Integer.valueOf(30)),MapperS.of(calculationPeriod(interestRatePayout, dayCountFractionEnum, date).get()).<Date>map("getEndDate", CalculationPeriodData::getEndDate).map("getDay", Date::getDay));
		}
		
		protected Mapper<Integer> endDay(InterestRatePayout interestRatePayout, DayCountFractionEnum dayCountFractionEnum, Date date) {
			return doIf(areEqual(MapperS.of(endDateIsInLeapYear(interestRatePayout, dayCountFractionEnum, date).get()), MapperS.of(Boolean.valueOf(false))).and(areEqual(MapperS.of(calculationPeriod(interestRatePayout, dayCountFractionEnum, date).get()).<Date>map("getEndDate", CalculationPeriodData::getEndDate).map("getMonth", Date::getMonth), MapperS.of(Integer.valueOf(2)))).and(areEqual(MapperS.of(calculationPeriod(interestRatePayout, dayCountFractionEnum, date).get()).<Date>map("getEndDate", CalculationPeriodData::getEndDate).map("getDay", Date::getDay), MapperS.of(Integer.valueOf(28)))).or(areEqual(MapperS.of(endDateIsInLeapYear(interestRatePayout, dayCountFractionEnum, date).get()), MapperS.of(Boolean.valueOf(true))).and(areEqual(MapperS.of(calculationPeriod(interestRatePayout, dayCountFractionEnum, date).get()).<Date>map("getEndDate", CalculationPeriodData::getEndDate).map("getMonth", Date::getMonth), MapperS.of(Integer.valueOf(2)))).and(areEqual(MapperS.of(calculationPeriod(interestRatePayout, dayCountFractionEnum, date).get()).<Date>map("getEndDate", CalculationPeriodData::getEndDate).map("getDay", Date::getDay), MapperS.of(Integer.valueOf(29))))).or(areEqual(MapperS.of(calculationPeriod(interestRatePayout, dayCountFractionEnum, date).get()).<Date>map("getEndDate", CalculationPeriodData::getEndDate).map("getDay", Date::getDay), MapperS.of(Integer.valueOf(31)))).or(areEqual(MapperS.of(calculationPeriod(interestRatePayout, dayCountFractionEnum, date).get()).<Date>map("getEndDate", CalculationPeriodData::getEndDate), MapperS.of(interestRatePayoutTerminationDate(interestRatePayout, dayCountFractionEnum, date).get()))),MapperS.of(Integer.valueOf(30)),MapperS.of(calculationPeriod(interestRatePayout, dayCountFractionEnum, date).get()).<Date>map("getEndDate", CalculationPeriodData::getEndDate).map("getDay", Date::getDay));
		}
	}
	
	public static class ACT_365L implements RosettaFunction {
		
		// RosettaFunction dependencies
		//
		@Inject protected CalculationPeriod calculationPeriod;
	
		/**
		* @param interestRatePayout 
		* @param dayCountFractionEnum 
		* @param date 
		* @return result 
		*/
		public BigDecimal evaluate(InterestRatePayout interestRatePayout, DayCountFractionEnum dayCountFractionEnum, Date date) {
			
			BigDecimal result = doEvaluate(interestRatePayout, dayCountFractionEnum, date);
			
			return result;
		}
		
		protected BigDecimal doEvaluate(InterestRatePayout interestRatePayout, DayCountFractionEnum dayCountFractionEnum, Date date) {
			Mapper<BigDecimal> resultHolder = null;
			resultHolder = MapperS.of(BigDecimalExtensions.divide(BigDecimalExtensions.valueOf(daysInPeriod(interestRatePayout, dayCountFractionEnum, date).get()), BigDecimalExtensions.valueOf(daysInYear(interestRatePayout, dayCountFractionEnum, date).get())));
			return resultHolder.get();
		}
		
		
		protected MapperBuilder<CalculationPeriodData> calculationPeriod(InterestRatePayout interestRatePayout, DayCountFractionEnum dayCountFractionEnum, Date date) {
			return MapperS.of(calculationPeriod.evaluate(MapperS.of(interestRatePayout).<CalculationPeriodDates>map("getCalculationPeriodDates", InterestRatePayout::getCalculationPeriodDates).get(), MapperS.of(date).get()));
		}
		
		protected Mapper<Boolean> endDateIsInLeapYear(InterestRatePayout interestRatePayout, DayCountFractionEnum dayCountFractionEnum, Date date) {
			return MapperS.of(new IsLeapYear().execute(MapperS.of(calculationPeriod(interestRatePayout, dayCountFractionEnum, date).get()).<Date>map("getEndDate", CalculationPeriodData::getEndDate).map("getYear", Date::getYear).get()));
		}
		
		protected Mapper<Integer> daysInPeriod(InterestRatePayout interestRatePayout, DayCountFractionEnum dayCountFractionEnum, Date date) {
			return MapperS.of(calculationPeriod(interestRatePayout, dayCountFractionEnum, date).get()).<Integer>map("getDaysInPeriod", CalculationPeriodData::getDaysInPeriod);
		}
		
		protected Mapper<Integer> daysInYear(InterestRatePayout interestRatePayout, DayCountFractionEnum dayCountFractionEnum, Date date) {
			return doIf(areEqual(MapperS.of(endDateIsInLeapYear(interestRatePayout, dayCountFractionEnum, date).get()), MapperS.of(Boolean.valueOf(true))),MapperS.of(Integer.valueOf(366)),MapperS.of(Integer.valueOf(365)));
		}
	}
}
