package org.isda.cdm.functions;

import com.google.inject.ImplementedBy;
import com.google.inject.Inject;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.RosettaFunction;
import com.rosetta.model.lib.records.Date;
import com.rosetta.model.lib.validation.ModelObjectValidator;
import java.math.BigDecimal;
import org.isda.cdm.AdjustableOrRelativeDate;
import org.isda.cdm.BusinessCenterTime;
import org.isda.cdm.DeterminationMethodEnum;
import org.isda.cdm.Equity;
import org.isda.cdm.ObservationPrimitive;
import org.isda.cdm.TimeTypeEnum;
import org.isda.cdm.TimeZone;
import org.isda.cdm.functions.EquitySpot;
import org.isda.cdm.functions.ResolveAdjustableDate;
import org.isda.cdm.functions.ResolveTimeZoneFromTimeType;
import org.isda.cdm.functions.TimeZoneFromBusinessCenterTime;

import static com.rosetta.model.lib.validation.ValidatorHelper.areEqual;
import static com.rosetta.model.lib.validation.ValidatorHelper.doIf;
import static com.rosetta.model.lib.validation.ValidatorHelper.exists;
import static com.rosetta.model.lib.validation.ValidatorHelper.notExists;

@ImplementedBy(EquityPriceObservationImpl.class)
public abstract class EquityPriceObservation implements RosettaFunction {
	
	@Inject protected ModelObjectValidator objectValidator;
	
	// RosettaFunction dependencies
	//
	@Inject protected EquitySpot equitySpot;
	@Inject protected ResolveAdjustableDate resolveAdjustableDate;
	@Inject protected ResolveTimeZoneFromTimeType resolveTimeZoneFromTimeType;
	@Inject protected TimeZoneFromBusinessCenterTime timeZoneFromBusinessCenterTime;

	/**
	* @param equity 
	* @param valuationDate 
	* @param valuationTime 
	* @param timeType 
	* @param determinationMethod 
	* @return observation 
	*/
	public ObservationPrimitive evaluate(Equity equity, AdjustableOrRelativeDate valuationDate, BusinessCenterTime valuationTime, TimeTypeEnum timeType, DeterminationMethodEnum determinationMethod) {
		// pre-conditions
		
			assert
				doIf(exists(MapperS.of(valuationTime), false),notExists(MapperS.of(timeType)),doIf(exists(MapperS.of(timeType), false),notExists(MapperS.of(valuationTime)),MapperS.of(Boolean.valueOf(false)))).get()
				: "Optional choice between directly passing a time or a timeType, which has to be resolved into a time based on the determination method.";
		
		ObservationPrimitive observation = doEvaluate(equity, valuationDate, valuationTime, timeType, determinationMethod).build();
		
		// post-conditions

		assert
			areEqual(MapperS.of(observation).<Date>map("getDate", ObservationPrimitive::getDate), MapperS.of(resolveAdjustableDate.evaluate(MapperS.of(valuationDate).get()))).and(doIf(exists(MapperS.of(valuationTime), false),areEqual(MapperS.of(observation).<TimeZone>map("getTime", ObservationPrimitive::getTime), MapperS.of(timeZoneFromBusinessCenterTime.evaluate(MapperS.of(valuationTime).get()))),areEqual(MapperS.of(observation).<TimeZone>map("getTime", ObservationPrimitive::getTime), MapperS.of(resolveTimeZoneFromTimeType.evaluate(MapperS.of(timeType).get(), MapperS.of(determinationMethod).get()))))).get()
			: "The date and time must be properly resolved as attributes on the output.";

		assert
			areEqual(MapperS.of(observation).<BigDecimal>map("getObservation", ObservationPrimitive::getObservation), MapperS.of(equitySpot.evaluate(MapperS.of(equity).get(), MapperS.of(observation).<Date>map("getDate", ObservationPrimitive::getDate).get(), MapperS.of(observation).<TimeZone>map("getTime", ObservationPrimitive::getTime).get()))).get()
			: "The number recorded in the observation must match the number fetched from the source.";
		objectValidator.validateAndFailOnErorr(ObservationPrimitive.class, observation);
		return observation;
	}
	
	protected abstract ObservationPrimitive.ObservationPrimitiveBuilder doEvaluate(Equity equity, AdjustableOrRelativeDate valuationDate, BusinessCenterTime valuationTime, TimeTypeEnum timeType, DeterminationMethodEnum determinationMethod);
}
