package org.isda.cdm.functions;

import com.google.inject.Inject;
import com.rosetta.model.lib.functions.MapperC;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.RosettaFunction;
import com.rosetta.model.lib.validation.ModelObjectValidator;
import java.lang.SuppressWarnings;
import org.isda.cdm.ContractualProduct;
import org.isda.cdm.ExecutionPrimitive;
import org.isda.cdm.ExecutionQuantity;
import org.isda.cdm.ExecutionState;
import org.isda.cdm.Party;
import org.isda.cdm.Product;

import static com.rosetta.model.lib.validation.ValidatorHelper.notEqual;
import static com.rosetta.model.lib.validation.ValidatorHelper.notExists;

public class NewExecutionPrimitive implements RosettaFunction {
	
	@Inject protected ModelObjectValidator objectValidator;

	/**
	* @param product The financial product, either contractual or fungible, that is the subject of the execution.
	* @param partyA The parties to that execution, for now assumed Principal-Principal, i.e. no Agent.
	* @param partyB The parties to that execution, for now assumed Principal-Principal, i.e. no Agent.
	* @param quantity The quantity of the financial product being executed.
	* @return execution 
	*/
	public ExecutionPrimitive evaluate(Product product, Party partyA, Party partyB, ExecutionQuantity quantity) {
		// pre-conditions
		
			assert
				notEqual(MapperS.of(partyA), MapperS.of(partyB)).get()
				: "Parties must be different, and match the contractual parties to the legal agreement when the latter exists.";
		
		ExecutionPrimitive execution = doEvaluate(product, partyA, partyB, quantity).build();
		
		// post-conditions

		assert
			notExists(MapperS.of(execution).<ExecutionState>mapC("getBefore", ExecutionPrimitive::getBefore)).get()
			: "The execution will be the first event in any post-trade life cycle and therefore does not have a before state. Once pre-trade elements are brought into CDM, the 'before' will be used to refer to the last of the pre-trade event states.";
		objectValidator.validateAndFailOnErorr(ExecutionPrimitive.class, execution);
		return execution;
	}
	
	protected ExecutionPrimitive.ExecutionPrimitiveBuilder doEvaluate(Product product, Party partyA, Party partyB, ExecutionQuantity quantity) {
		ExecutionPrimitive.ExecutionPrimitiveBuilder executionHolder = ExecutionPrimitive.builder();
		@SuppressWarnings("unused") ExecutionPrimitive execution = executionHolder.build();
		executionHolder
			.getOrCreateAfter().getOrCreateExecution().getOrCreateProduct()
			.setContractualProduct(MapperS.of(product).<ContractualProduct>map("getContractualProduct", Product::getContractualProduct).get());
		;
		execution = executionHolder.build();
		executionHolder
			.getOrCreateAfter().getOrCreateExecution()
			.addPartyRef(MapperC.of(MapperS.of(partyA), MapperS.of(partyB)).getMulti());
		;
		return executionHolder;
	}
	
}
