package org.isda.cdm.functions;

import com.google.inject.Inject;
import com.rosetta.model.lib.functions.MapperBuilder;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.RosettaFunction;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.records.Date;
import com.rosetta.model.lib.validation.ModelObjectValidator;
import java.lang.SuppressWarnings;
import org.isda.cdm.Contract;
import org.isda.cdm.ContractState;
import org.isda.cdm.Event;
import org.isda.cdm.EventEffect;
import org.isda.cdm.Execution;
import org.isda.cdm.ObservationPrimitive;
import org.isda.cdm.PrimitiveEvent;
import org.isda.cdm.ProductIdentifier;
import org.isda.cdm.functions.ExtractContractState;
import org.isda.cdm.functions.NewResetPrimitive;
import org.isda.cdm.functions.ToDateTime;

import static com.rosetta.model.lib.validation.ValidatorHelper.exists;
import static com.rosetta.model.lib.validation.ValidatorHelper.notExists;

public class Reset implements RosettaFunction {
	
	@Inject protected ModelObjectValidator objectValidator;
	
	// RosettaFunction dependencies
	//
	@Inject protected ExtractContractState extractContractState;
	@Inject protected NewResetPrimitive newResetPrimitive;
	@Inject protected ToDateTime toDateTime;

	/**
	* @param previousEvent 
	* @param observationEvent 
	* @param date 
	* @return reset 
	*/
	public Event evaluate(Event previousEvent, Event observationEvent, Date date) {
		// pre-conditions
		
			assert
				exists(MapperS.of(contractState(previousEvent, observationEvent, date).get()), false).get()
				: "ExtractContractState would result in nothing (empty), check we have something.";
		
			assert
				exists(MapperS.of(observation(previousEvent, observationEvent, date).get()), false).get()
				: "The observation primitive must be part of the observation event";
		
		Event reset = doEvaluate(previousEvent, observationEvent, date).build();
		
		// post-conditions

		assert
			notExists(MapperS.of(reset).<EventEffect>map("getEventEffect", Event::getEventEffect).mapC("getEffectedExecution", EventEffect::getEffectedExecution).<Execution>map("getValue", FieldWithMeta::getValue)).and(notExists(MapperS.of(reset).<EventEffect>map("getEventEffect", Event::getEventEffect).mapC("getExecution", EventEffect::getExecution).<Execution>map("getValue", FieldWithMeta::getValue))).and(notExists(MapperS.of(reset).<EventEffect>map("getEventEffect", Event::getEventEffect).mapC("getProductIdentifier", EventEffect::getProductIdentifier).<ProductIdentifier>map("getValue", FieldWithMeta::getValue))).get()
			: "";
		objectValidator.validateAndFailOnErorr(Event.class, reset);
		return reset;
	}
	
	protected Event.EventBuilder doEvaluate(Event previousEvent, Event observationEvent, Date date) {
		Event.EventBuilder resetHolder = Event.builder();
		@SuppressWarnings("unused") Event reset = resetHolder.build();
		resetHolder
			.getOrCreatePrimitive()
			.addReset(MapperS.of(newResetPrimitive.evaluate(MapperS.of(contractState(previousEvent, observationEvent, date).get()).get(), MapperS.of(observation(previousEvent, observationEvent, date).get()).get(), MapperS.of(date).get())).get());
		;
		reset = resetHolder.build();
		resetHolder
			.getOrCreateLineage()
			.addEventReferenceRef(MapperS.of(previousEvent).get());
		;
		reset = resetHolder.build();
		resetHolder
			.getOrCreateLineage()
			.addContractReferenceRef(MapperS.of(contract(previousEvent, observationEvent, date).get()).get());
		;
		reset = resetHolder.build();
		resetHolder
			.getOrCreateEventEffect()
			.addEffectedContractRef(MapperS.of(contract(previousEvent, observationEvent, date).get()).get());
		;
		reset = resetHolder.build();
		resetHolder
			.getOrCreateEventEffect()
			.addContractRef(MapperS.of(contract(previousEvent, observationEvent, date).get()).get());
		;
		reset = resetHolder.build();
		resetHolder
			.getOrCreateTimestamp(0)
			.setDateTime(MapperS.of(toDateTime.evaluate(MapperS.of(date).get())).get());
		;
		return resetHolder;
	}
	
	
	protected MapperBuilder<ContractState> contractState(Event previousEvent, Event observationEvent, Date date) {
		return MapperS.of(extractContractState.evaluate(MapperS.of(previousEvent).get()));
	}
	
	protected MapperBuilder<Contract> contract(Event previousEvent, Event observationEvent, Date date) {
		return MapperS.of(contractState(previousEvent, observationEvent, date).get()).<Contract>map("getContract", ContractState::getContract);
	}
	
	protected MapperBuilder<ObservationPrimitive> observation(Event previousEvent, Event observationEvent, Date date) {
		return MapperS.of(observationEvent).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<ObservationPrimitive>mapC("getObservation", PrimitiveEvent::getObservation);
	}
}
