package org.isda.cdm.qualify.event;

import com.rosetta.model.lib.annotations.RosettaQualifiable;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.qualify.QualifyResult;
import com.rosetta.model.lib.validation.ComparisonResult;
import java.math.BigDecimal;
import java.util.function.Function;
import org.isda.cdm.Contract;
import org.isda.cdm.ContractualProduct;
import org.isda.cdm.ContractualQuantity;
import org.isda.cdm.CreditDefaultPayout;
import org.isda.cdm.EconomicTerms;
import org.isda.cdm.Event;
import org.isda.cdm.EventWorkflow;
import org.isda.cdm.Execution;
import org.isda.cdm.ExecutionQuantity;
import org.isda.cdm.FxLinkedNotionalSchedule;
import org.isda.cdm.InterestRatePayout;
import org.isda.cdm.Money;
import org.isda.cdm.NonNegativeAmountSchedule;
import org.isda.cdm.NonNegativeQuantity;
import org.isda.cdm.NonNegativeSchedule;
import org.isda.cdm.NonNegativeStep;
import org.isda.cdm.NotionalSchedule;
import org.isda.cdm.NotionalStepRule;
import org.isda.cdm.OptionPayout;
import org.isda.cdm.PartyRole;
import org.isda.cdm.PartyRoleEnum;
import org.isda.cdm.Payout;
import org.isda.cdm.PrimitiveEvent;
import org.isda.cdm.ProtectionTerms;
import org.isda.cdm.Quantity;
import org.isda.cdm.QuantityChangePrimitive;
import org.isda.cdm.QuantityNotation;
import org.isda.cdm.TermsChangePrimitive;
import org.isda.cdm.Trade;
import org.isda.cdm.WorkflowStatusEnum;
import org.isda.cdm.validation.datarule.*;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaQualifiable("ClearingRejection")
public class IsClearingRejection implements Function<Event, QualifyResult>{
	
	private static final String QUALIFY_DEFINITION = "isEvent ClearingRejection <The rejection of an alpha trade submission by a CCP, which is qualified by the fact that (i) termsChange is the only primitive, the workflow status is `rejected`, and (iii) the CCP is positioned as a partyRole to the execution or contract in the after state, while it didn't exist in the before state.>\n" + 
		"	Event -> primitive -> termsChange only exists\n" + 
		"	and Event -> eventWorkflow -> workflowStatus = WorkflowStatusEnum.Rejected\n" + 
		"	and ( Event -> primitive -> termsChange -> before -> execution -> partyRole is absent\n" + 
		"		or Event -> primitive -> termsChange -> before -> execution -> partyRole -> role <> PartyRoleEnum.ClearingOrganization\n" + 
		"		or Event -> primitive -> termsChange -> before -> contract -> partyRole is absent\n" + 
		"		or Event -> primitive -> termsChange -> before -> contract -> partyRole -> role <> PartyRoleEnum.ClearingOrganization )\n" + 
		"	and ( Event -> primitive -> termsChange -> after -> execution -> partyRole -> role = PartyRoleEnum.ClearingOrganization \n" + 
		"		or Event -> primitive -> termsChange -> after -> contract -> partyRole -> role = PartyRoleEnum.ClearingOrganization )\n" + 
		"	and quantityAfterQuantityChange = quantityBeforeQuantityChange";
	
	private static final String EXPR_DEFINITION = "Event -> primitive -> termsChange only exists\n" + 
		"	and Event -> eventWorkflow -> workflowStatus = WorkflowStatusEnum.Rejected\n" + 
		"	and ( Event -> primitive -> termsChange -> before -> execution -> partyRole is absent\n" + 
		"		or Event -> primitive -> termsChange -> before -> execution -> partyRole -> role <> PartyRoleEnum.ClearingOrganization\n" + 
		"		or Event -> primitive -> termsChange -> before -> contract -> partyRole is absent\n" + 
		"		or Event -> primitive -> termsChange -> before -> contract -> partyRole -> role <> PartyRoleEnum.ClearingOrganization )\n" + 
		"	and ( Event -> primitive -> termsChange -> after -> execution -> partyRole -> role = PartyRoleEnum.ClearingOrganization \n" + 
		"		or Event -> primitive -> termsChange -> after -> contract -> partyRole -> role = PartyRoleEnum.ClearingOrganization )\n" + 
		"	and quantityAfterQuantityChange = quantityBeforeQuantityChange";
	
	@Override
	public QualifyResult apply(Event event) {
		ComparisonResult exprResult = exists(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<TermsChangePrimitive>map("getTermsChange", PrimitiveEvent::getTermsChange), true).and(areEqual(MapperS.of(event).<EventWorkflow>map("getEventWorkflow", Event::getEventWorkflow).<WorkflowStatusEnum>mapC("getWorkflowStatus", EventWorkflow::getWorkflowStatus), MapperS.of(WorkflowStatusEnum.REJECTED))).and(notExists(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<TermsChangePrimitive>map("getTermsChange", PrimitiveEvent::getTermsChange).<Trade>map("getBefore", TermsChangePrimitive::getBefore).<Execution>map("getExecution", Trade::getExecution).<PartyRole>mapC("getPartyRole", Execution::getPartyRole)).or(notEqual(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<TermsChangePrimitive>map("getTermsChange", PrimitiveEvent::getTermsChange).<Trade>map("getBefore", TermsChangePrimitive::getBefore).<Execution>map("getExecution", Trade::getExecution).<PartyRole>mapC("getPartyRole", Execution::getPartyRole).<PartyRoleEnum>map("getRole", PartyRole::getRole), MapperS.of(PartyRoleEnum.CLEARING_ORGANIZATION))).or(notExists(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<TermsChangePrimitive>map("getTermsChange", PrimitiveEvent::getTermsChange).<Trade>map("getBefore", TermsChangePrimitive::getBefore).<Contract>map("getContract", Trade::getContract).<PartyRole>mapC("getPartyRole", Contract::getPartyRole))).or(notEqual(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<TermsChangePrimitive>map("getTermsChange", PrimitiveEvent::getTermsChange).<Trade>map("getBefore", TermsChangePrimitive::getBefore).<Contract>map("getContract", Trade::getContract).<PartyRole>mapC("getPartyRole", Contract::getPartyRole).<PartyRoleEnum>map("getRole", PartyRole::getRole), MapperS.of(PartyRoleEnum.CLEARING_ORGANIZATION)))).and(areEqual(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<TermsChangePrimitive>map("getTermsChange", PrimitiveEvent::getTermsChange).<Trade>map("getAfter", TermsChangePrimitive::getAfter).<Execution>map("getExecution", Trade::getExecution).<PartyRole>mapC("getPartyRole", Execution::getPartyRole).<PartyRoleEnum>map("getRole", PartyRole::getRole), MapperS.of(PartyRoleEnum.CLEARING_ORGANIZATION)).or(areEqual(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<TermsChangePrimitive>map("getTermsChange", PrimitiveEvent::getTermsChange).<Trade>map("getAfter", TermsChangePrimitive::getAfter).<Contract>map("getContract", Trade::getContract).<PartyRole>mapC("getPartyRole", Contract::getPartyRole).<PartyRoleEnum>map("getRole", PartyRole::getRole), MapperS.of(PartyRoleEnum.CLEARING_ORGANIZATION)))).and(areEqual(MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getAfter", QuantityChangePrimitive::getAfter).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<ContractualQuantity>map("getQuantity", InterestRatePayout::getQuantity).<Quantity>map("getQuantity", ContractualQuantity::getQuantity).<BigDecimal>map("getAmount", Quantity::getAmount)), MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getAfter", QuantityChangePrimitive::getAfter).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<ContractualQuantity>map("getQuantity", InterestRatePayout::getQuantity).map("getNotionalAmount", ContractualQuantity::getNotionalAmount).<Money>map("getValue", FieldWithMeta::getValue).<String>groupBy(g->new MapperS<>(g).map("getCurrency", Money::getCurrency).<String>map("getValue", FieldWithMeta::getValue)).<BigDecimal>map("getAmount", Money::getAmount)), MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getAfter", QuantityChangePrimitive::getAfter).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<ContractualQuantity>map("getQuantity", InterestRatePayout::getQuantity).<NotionalSchedule>map("getNotionalSchedule", ContractualQuantity::getNotionalSchedule).<NonNegativeAmountSchedule>map("getNotionalStepSchedule", NotionalSchedule::getNotionalStepSchedule).<String>groupBy(g->new MapperS<>(g).map("getCurrency", NonNegativeAmountSchedule::getCurrency).<String>map("getValue", FieldWithMeta::getValue)).<BigDecimal>map("getInitialValue", NonNegativeSchedule::getInitialValue)), MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getAfter", QuantityChangePrimitive::getAfter).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<ContractualQuantity>map("getQuantity", InterestRatePayout::getQuantity).<NotionalSchedule>map("getNotionalSchedule", ContractualQuantity::getNotionalSchedule).<NonNegativeAmountSchedule>map("getNotionalStepSchedule", NotionalSchedule::getNotionalStepSchedule).<String>groupBy(g->new MapperS<>(g).map("getCurrency", NonNegativeAmountSchedule::getCurrency).<String>map("getValue", FieldWithMeta::getValue)).<NonNegativeStep>mapC("getStep", NonNegativeSchedule::getStep).<BigDecimal>map("getStepValue", NonNegativeStep::getStepValue)), MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getAfter", QuantityChangePrimitive::getAfter).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<ContractualQuantity>map("getQuantity", InterestRatePayout::getQuantity).<NotionalSchedule>map("getNotionalSchedule", ContractualQuantity::getNotionalSchedule).<String>groupBy(g->new MapperS<>(g).<NonNegativeAmountSchedule>map("getNotionalStepSchedule", NotionalSchedule::getNotionalStepSchedule).map("getCurrency", NonNegativeAmountSchedule::getCurrency).<String>map("getValue", FieldWithMeta::getValue)).<NotionalStepRule>map("getNotionalStepParameters", NotionalSchedule::getNotionalStepParameters).<BigDecimal>map("getNotionalStepAmount", NotionalStepRule::getNotionalStepAmount)), MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getAfter", QuantityChangePrimitive::getAfter).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<ContractualQuantity>map("getQuantity", InterestRatePayout::getQuantity).<FxLinkedNotionalSchedule>map("getFxLinkedNotional", ContractualQuantity::getFxLinkedNotional).<String>groupBy(g->new MapperS<>(g).map("getVaryingNotionalCurrency", FxLinkedNotionalSchedule::getVaryingNotionalCurrency).<String>map("getValue", FieldWithMeta::getValue)).<BigDecimal>map("getInitialValue", FxLinkedNotionalSchedule::getInitialValue)), MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getAfter", QuantityChangePrimitive::getAfter).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<CreditDefaultPayout>map("getCreditDefaultPayout", Payout::getCreditDefaultPayout).<ProtectionTerms>mapC("getProtectionTerms", CreditDefaultPayout::getProtectionTerms).<Money>map("getNotionalAmount", ProtectionTerms::getNotionalAmount).<String>groupBy(g->new MapperS<>(g).map("getCurrency", Money::getCurrency).<String>map("getValue", FieldWithMeta::getValue)).<BigDecimal>map("getAmount", Money::getAmount)), MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getAfter", QuantityChangePrimitive::getAfter).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<OptionPayout>mapC("getOptionPayout", Payout::getOptionPayout).<ContractualQuantity>map("getQuantity", OptionPayout::getQuantity).map("getNotionalAmount", ContractualQuantity::getNotionalAmount).<Money>map("getValue", FieldWithMeta::getValue).<String>groupBy(g->new MapperS<>(g).map("getCurrency", Money::getCurrency).<String>map("getValue", FieldWithMeta::getValue)).<BigDecimal>map("getAmount", Money::getAmount)), MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getAfter", QuantityChangePrimitive::getAfter).<Contract>map("getContract", Trade::getContract).<ExecutionQuantity>map("getContractualQuantity", Contract::getContractualQuantity).<QuantityNotation>mapC("getQuantityNotation", ExecutionQuantity::getQuantityNotation).<NonNegativeQuantity>map("getQuantity", QuantityNotation::getQuantity).<String>groupBy(g->new MapperS<>(g).map("getCurrency", Quantity::getCurrency).<String>map("getValue", FieldWithMeta::getValue)).<BigDecimal>map("getAmount", Quantity::getAmount)))))))))), MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getBefore", QuantityChangePrimitive::getBefore).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<ContractualQuantity>map("getQuantity", InterestRatePayout::getQuantity).<Quantity>map("getQuantity", ContractualQuantity::getQuantity).<BigDecimal>map("getAmount", Quantity::getAmount)), MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getBefore", QuantityChangePrimitive::getBefore).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<ContractualQuantity>map("getQuantity", InterestRatePayout::getQuantity).map("getNotionalAmount", ContractualQuantity::getNotionalAmount).<Money>map("getValue", FieldWithMeta::getValue).<String>groupBy(g->new MapperS<>(g).map("getCurrency", Money::getCurrency).<String>map("getValue", FieldWithMeta::getValue)).<BigDecimal>map("getAmount", Money::getAmount)), MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getBefore", QuantityChangePrimitive::getBefore).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<ContractualQuantity>map("getQuantity", InterestRatePayout::getQuantity).<NotionalSchedule>map("getNotionalSchedule", ContractualQuantity::getNotionalSchedule).<NonNegativeAmountSchedule>map("getNotionalStepSchedule", NotionalSchedule::getNotionalStepSchedule).<String>groupBy(g->new MapperS<>(g).map("getCurrency", NonNegativeAmountSchedule::getCurrency).<String>map("getValue", FieldWithMeta::getValue)).<BigDecimal>map("getInitialValue", NonNegativeSchedule::getInitialValue)), MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getBefore", QuantityChangePrimitive::getBefore).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<ContractualQuantity>map("getQuantity", InterestRatePayout::getQuantity).<NotionalSchedule>map("getNotionalSchedule", ContractualQuantity::getNotionalSchedule).<NonNegativeAmountSchedule>map("getNotionalStepSchedule", NotionalSchedule::getNotionalStepSchedule).<String>groupBy(g->new MapperS<>(g).map("getCurrency", NonNegativeAmountSchedule::getCurrency).<String>map("getValue", FieldWithMeta::getValue)).<NonNegativeStep>mapC("getStep", NonNegativeSchedule::getStep).<BigDecimal>map("getStepValue", NonNegativeStep::getStepValue)), MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getBefore", QuantityChangePrimitive::getBefore).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<ContractualQuantity>map("getQuantity", InterestRatePayout::getQuantity).<NotionalSchedule>map("getNotionalSchedule", ContractualQuantity::getNotionalSchedule).<String>groupBy(g->new MapperS<>(g).<NonNegativeAmountSchedule>map("getNotionalStepSchedule", NotionalSchedule::getNotionalStepSchedule).map("getCurrency", NonNegativeAmountSchedule::getCurrency).<String>map("getValue", FieldWithMeta::getValue)).<NotionalStepRule>map("getNotionalStepParameters", NotionalSchedule::getNotionalStepParameters).<BigDecimal>map("getNotionalStepAmount", NotionalStepRule::getNotionalStepAmount)), MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getBefore", QuantityChangePrimitive::getBefore).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<ContractualQuantity>map("getQuantity", InterestRatePayout::getQuantity).<FxLinkedNotionalSchedule>map("getFxLinkedNotional", ContractualQuantity::getFxLinkedNotional).<String>groupBy(g->new MapperS<>(g).map("getVaryingNotionalCurrency", FxLinkedNotionalSchedule::getVaryingNotionalCurrency).<String>map("getValue", FieldWithMeta::getValue)).<BigDecimal>map("getInitialValue", FxLinkedNotionalSchedule::getInitialValue)), MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getBefore", QuantityChangePrimitive::getBefore).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<CreditDefaultPayout>map("getCreditDefaultPayout", Payout::getCreditDefaultPayout).<ProtectionTerms>mapC("getProtectionTerms", CreditDefaultPayout::getProtectionTerms).<Money>map("getNotionalAmount", ProtectionTerms::getNotionalAmount).<String>groupBy(g->new MapperS<>(g).map("getCurrency", Money::getCurrency).<String>map("getValue", FieldWithMeta::getValue)).<BigDecimal>map("getAmount", Money::getAmount)), MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getBefore", QuantityChangePrimitive::getBefore).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<OptionPayout>mapC("getOptionPayout", Payout::getOptionPayout).<ContractualQuantity>map("getQuantity", OptionPayout::getQuantity).map("getNotionalAmount", ContractualQuantity::getNotionalAmount).<Money>map("getValue", FieldWithMeta::getValue).<String>groupBy(g->new MapperS<>(g).map("getCurrency", Money::getCurrency).<String>map("getValue", FieldWithMeta::getValue)).<BigDecimal>map("getAmount", Money::getAmount)), MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getBefore", QuantityChangePrimitive::getBefore).<Contract>map("getContract", Trade::getContract).<ExecutionQuantity>map("getContractualQuantity", Contract::getContractualQuantity).<QuantityNotation>mapC("getQuantityNotation", ExecutionQuantity::getQuantityNotation).<NonNegativeQuantity>map("getQuantity", QuantityNotation::getQuantity).<String>groupBy(g->new MapperS<>(g).map("getCurrency", Quantity::getCurrency).<String>map("getValue", FieldWithMeta::getValue)).<BigDecimal>map("getAmount", Quantity::getAmount))))))))))));
		
		return QualifyResult.builder()
				.setName("ClearingRejection")
				.setDefinition(QUALIFY_DEFINITION)
				.setExpressionResult(EXPR_DEFINITION, exprResult)
				.build();
	}
}
