package org.isda.cdm.qualify.event;

import com.rosetta.model.lib.annotations.RosettaQualifiable;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.qualify.QualifyResult;
import com.rosetta.model.lib.validation.ComparisonResult;
import java.util.function.Function;
import org.isda.cdm.Contract;
import org.isda.cdm.Event;
import org.isda.cdm.EventWorkflow;
import org.isda.cdm.Execution;
import org.isda.cdm.PartyRole;
import org.isda.cdm.PartyRoleEnum;
import org.isda.cdm.PrimitiveEvent;
import org.isda.cdm.TermsChangePrimitive;
import org.isda.cdm.Trade;
import org.isda.cdm.WorkflowStatusEnum;
import org.isda.cdm.validation.datarule.*;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaQualifiable("ClearingSubmission")
public class IsClearingSubmission implements Function<Event, QualifyResult>{
	
	private static final String QUALIFY_DEFINITION = "isEvent ClearingSubmission <The submission of a contract to a CCP, which is qualified by the fact that (i) the termsChange is the only primitive, and (ii) the CCP is positioned as a partyRole to the execution or contract in the after state, while it didn't exist in the before state.>\n" + 
		"	Event -> primitive -> termsChange only exists\n" + 
		"	and Event -> eventWorkflow -> workflowStatus = WorkflowStatusEnum.Submitted\n" + 
		"	and ( Event -> primitive -> termsChange -> before -> execution -> partyRole is absent\n" + 
		"		or Event -> primitive -> termsChange -> before -> execution -> partyRole -> role <> PartyRoleEnum.ClearingOrganization\n" + 
		"		or Event -> primitive -> termsChange -> before -> contract -> partyRole is absent\n" + 
		"		or Event -> primitive -> termsChange -> before -> contract -> partyRole -> role <> PartyRoleEnum.ClearingOrganization )\n" + 
		"	and ( Event -> primitive -> termsChange -> after -> contract -> partyRole -> role = PartyRoleEnum.ClearingOrganization\n" + 
		"		or Event -> primitive -> termsChange -> after -> execution -> partyRole -> role = PartyRoleEnum.ClearingOrganization )";
	
	private static final String EXPR_DEFINITION = "Event -> primitive -> termsChange only exists\n" + 
		"	and Event -> eventWorkflow -> workflowStatus = WorkflowStatusEnum.Submitted\n" + 
		"	and ( Event -> primitive -> termsChange -> before -> execution -> partyRole is absent\n" + 
		"		or Event -> primitive -> termsChange -> before -> execution -> partyRole -> role <> PartyRoleEnum.ClearingOrganization\n" + 
		"		or Event -> primitive -> termsChange -> before -> contract -> partyRole is absent\n" + 
		"		or Event -> primitive -> termsChange -> before -> contract -> partyRole -> role <> PartyRoleEnum.ClearingOrganization )\n" + 
		"	and ( Event -> primitive -> termsChange -> after -> contract -> partyRole -> role = PartyRoleEnum.ClearingOrganization\n" + 
		"		or Event -> primitive -> termsChange -> after -> execution -> partyRole -> role = PartyRoleEnum.ClearingOrganization )";
	
	@Override
	public QualifyResult apply(Event event) {
		ComparisonResult exprResult = exists(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<TermsChangePrimitive>map("getTermsChange", PrimitiveEvent::getTermsChange), true).and(areEqual(MapperS.of(event).<EventWorkflow>map("getEventWorkflow", Event::getEventWorkflow).<WorkflowStatusEnum>mapC("getWorkflowStatus", EventWorkflow::getWorkflowStatus), MapperS.of(WorkflowStatusEnum.SUBMITTED))).and(notExists(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<TermsChangePrimitive>map("getTermsChange", PrimitiveEvent::getTermsChange).<Trade>map("getBefore", TermsChangePrimitive::getBefore).<Execution>map("getExecution", Trade::getExecution).<PartyRole>mapC("getPartyRole", Execution::getPartyRole)).or(notEqual(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<TermsChangePrimitive>map("getTermsChange", PrimitiveEvent::getTermsChange).<Trade>map("getBefore", TermsChangePrimitive::getBefore).<Execution>map("getExecution", Trade::getExecution).<PartyRole>mapC("getPartyRole", Execution::getPartyRole).<PartyRoleEnum>map("getRole", PartyRole::getRole), MapperS.of(PartyRoleEnum.CLEARING_ORGANIZATION))).or(notExists(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<TermsChangePrimitive>map("getTermsChange", PrimitiveEvent::getTermsChange).<Trade>map("getBefore", TermsChangePrimitive::getBefore).<Contract>map("getContract", Trade::getContract).<PartyRole>mapC("getPartyRole", Contract::getPartyRole))).or(notEqual(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<TermsChangePrimitive>map("getTermsChange", PrimitiveEvent::getTermsChange).<Trade>map("getBefore", TermsChangePrimitive::getBefore).<Contract>map("getContract", Trade::getContract).<PartyRole>mapC("getPartyRole", Contract::getPartyRole).<PartyRoleEnum>map("getRole", PartyRole::getRole), MapperS.of(PartyRoleEnum.CLEARING_ORGANIZATION)))).and(areEqual(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<TermsChangePrimitive>map("getTermsChange", PrimitiveEvent::getTermsChange).<Trade>map("getAfter", TermsChangePrimitive::getAfter).<Contract>map("getContract", Trade::getContract).<PartyRole>mapC("getPartyRole", Contract::getPartyRole).<PartyRoleEnum>map("getRole", PartyRole::getRole), MapperS.of(PartyRoleEnum.CLEARING_ORGANIZATION)).or(areEqual(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<TermsChangePrimitive>map("getTermsChange", PrimitiveEvent::getTermsChange).<Trade>map("getAfter", TermsChangePrimitive::getAfter).<Execution>map("getExecution", Trade::getExecution).<PartyRole>mapC("getPartyRole", Execution::getPartyRole).<PartyRoleEnum>map("getRole", PartyRole::getRole), MapperS.of(PartyRoleEnum.CLEARING_ORGANIZATION))));
		
		return QualifyResult.builder()
				.setName("ClearingSubmission")
				.setDefinition(QUALIFY_DEFINITION)
				.setExpressionResult(EXPR_DEFINITION, exprResult)
				.build();
	}
}
