package org.isda.cdm.qualify.event;

import com.rosetta.model.lib.annotations.RosettaQualifiable;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.qualify.QualifyResult;
import com.rosetta.model.lib.validation.ComparisonResult;
import java.math.BigDecimal;
import java.util.function.Function;
import org.isda.cdm.CashTransferComponent;
import org.isda.cdm.ClosedState;
import org.isda.cdm.Contract;
import org.isda.cdm.ContractualProduct;
import org.isda.cdm.ContractualQuantity;
import org.isda.cdm.CreditDefaultPayout;
import org.isda.cdm.EconomicTerms;
import org.isda.cdm.Event;
import org.isda.cdm.ExecutionQuantity;
import org.isda.cdm.FxLinkedNotionalSchedule;
import org.isda.cdm.IntentEnum;
import org.isda.cdm.InterestRatePayout;
import org.isda.cdm.Money;
import org.isda.cdm.NonNegativeAmountSchedule;
import org.isda.cdm.NonNegativeQuantity;
import org.isda.cdm.NonNegativeSchedule;
import org.isda.cdm.NonNegativeStep;
import org.isda.cdm.NotionalSchedule;
import org.isda.cdm.NotionalStepRule;
import org.isda.cdm.OptionPayout;
import org.isda.cdm.Payout;
import org.isda.cdm.PrimitiveEvent;
import org.isda.cdm.ProtectionTerms;
import org.isda.cdm.Quantity;
import org.isda.cdm.QuantityChangePrimitive;
import org.isda.cdm.QuantityNotation;
import org.isda.cdm.Trade;
import org.isda.cdm.TransferPrimitive;
import org.isda.cdm.validation.datarule.*;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaQualifiable("Increase")
public class IsIncrease implements Function<Event, QualifyResult>{
	
	private static final String QUALIFY_DEFINITION = "isEvent Increase <The qualification of a increase event from the fact that (i) the intent is Increase when specified, (ii) the associated primitives are the quantityChange and the cash transfer, the (iii) the quantity associated with the contract increases.>\n" + 
		"	Event -> intent when present = IntentEnum.Increase\n" + 
		"	and ( Event -> primitive -> quantityChange only exists\n" + 
		"		or ( Event -> primitive -> quantityChange and Event -> primitive -> transfer -> cashTransfer ) exists )\n" + 
		"	and quantityBeforeQuantityChange < quantityAfterQuantityChange\n" + 
		"	and Event -> primitive -> quantityChange -> after -> contract -> closedState is absent";
	
	private static final String EXPR_DEFINITION = "Event -> intent when present = IntentEnum.Increase\n" + 
		"	and ( Event -> primitive -> quantityChange only exists\n" + 
		"		or ( Event -> primitive -> quantityChange and Event -> primitive -> transfer -> cashTransfer ) exists )\n" + 
		"	and quantityBeforeQuantityChange < quantityAfterQuantityChange\n" + 
		"	and Event -> primitive -> quantityChange -> after -> contract -> closedState is absent";
	
	@Override
	public QualifyResult apply(Event event) {
		ComparisonResult exprResult = doWhenPresent(MapperS.of(event).<IntentEnum>map("getIntent", Event::getIntent), areEqual(MapperS.of(event).<IntentEnum>map("getIntent", Event::getIntent), MapperS.of(IntentEnum.INCREASE))).and(exists(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange), true).or(exists(MapperTree.andDifferent(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange)), MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<TransferPrimitive>mapC("getTransfer", PrimitiveEvent::getTransfer).<CashTransferComponent>mapC("getCashTransfer", TransferPrimitive::getCashTransfer))), false))).and(lessThan(MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getBefore", QuantityChangePrimitive::getBefore).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<ContractualQuantity>map("getQuantity", InterestRatePayout::getQuantity).<Quantity>map("getQuantity", ContractualQuantity::getQuantity).<BigDecimal>map("getAmount", Quantity::getAmount)), MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getBefore", QuantityChangePrimitive::getBefore).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<ContractualQuantity>map("getQuantity", InterestRatePayout::getQuantity).map("getNotionalAmount", ContractualQuantity::getNotionalAmount).<Money>map("getValue", FieldWithMeta::getValue).<String>groupBy(g->new MapperS<>(g).map("getCurrency", Money::getCurrency).<String>map("getValue", FieldWithMeta::getValue)).<BigDecimal>map("getAmount", Money::getAmount)), MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getBefore", QuantityChangePrimitive::getBefore).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<ContractualQuantity>map("getQuantity", InterestRatePayout::getQuantity).<NotionalSchedule>map("getNotionalSchedule", ContractualQuantity::getNotionalSchedule).<NonNegativeAmountSchedule>map("getNotionalStepSchedule", NotionalSchedule::getNotionalStepSchedule).<String>groupBy(g->new MapperS<>(g).map("getCurrency", NonNegativeAmountSchedule::getCurrency).<String>map("getValue", FieldWithMeta::getValue)).<BigDecimal>map("getInitialValue", NonNegativeSchedule::getInitialValue)), MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getBefore", QuantityChangePrimitive::getBefore).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<ContractualQuantity>map("getQuantity", InterestRatePayout::getQuantity).<NotionalSchedule>map("getNotionalSchedule", ContractualQuantity::getNotionalSchedule).<NonNegativeAmountSchedule>map("getNotionalStepSchedule", NotionalSchedule::getNotionalStepSchedule).<String>groupBy(g->new MapperS<>(g).map("getCurrency", NonNegativeAmountSchedule::getCurrency).<String>map("getValue", FieldWithMeta::getValue)).<NonNegativeStep>mapC("getStep", NonNegativeSchedule::getStep).<BigDecimal>map("getStepValue", NonNegativeStep::getStepValue)), MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getBefore", QuantityChangePrimitive::getBefore).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<ContractualQuantity>map("getQuantity", InterestRatePayout::getQuantity).<NotionalSchedule>map("getNotionalSchedule", ContractualQuantity::getNotionalSchedule).<String>groupBy(g->new MapperS<>(g).<NonNegativeAmountSchedule>map("getNotionalStepSchedule", NotionalSchedule::getNotionalStepSchedule).map("getCurrency", NonNegativeAmountSchedule::getCurrency).<String>map("getValue", FieldWithMeta::getValue)).<NotionalStepRule>map("getNotionalStepParameters", NotionalSchedule::getNotionalStepParameters).<BigDecimal>map("getNotionalStepAmount", NotionalStepRule::getNotionalStepAmount)), MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getBefore", QuantityChangePrimitive::getBefore).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<ContractualQuantity>map("getQuantity", InterestRatePayout::getQuantity).<FxLinkedNotionalSchedule>map("getFxLinkedNotional", ContractualQuantity::getFxLinkedNotional).<String>groupBy(g->new MapperS<>(g).map("getVaryingNotionalCurrency", FxLinkedNotionalSchedule::getVaryingNotionalCurrency).<String>map("getValue", FieldWithMeta::getValue)).<BigDecimal>map("getInitialValue", FxLinkedNotionalSchedule::getInitialValue)), MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getBefore", QuantityChangePrimitive::getBefore).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<CreditDefaultPayout>map("getCreditDefaultPayout", Payout::getCreditDefaultPayout).<ProtectionTerms>mapC("getProtectionTerms", CreditDefaultPayout::getProtectionTerms).<Money>map("getNotionalAmount", ProtectionTerms::getNotionalAmount).<String>groupBy(g->new MapperS<>(g).map("getCurrency", Money::getCurrency).<String>map("getValue", FieldWithMeta::getValue)).<BigDecimal>map("getAmount", Money::getAmount)), MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getBefore", QuantityChangePrimitive::getBefore).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<OptionPayout>mapC("getOptionPayout", Payout::getOptionPayout).<ContractualQuantity>map("getQuantity", OptionPayout::getQuantity).map("getNotionalAmount", ContractualQuantity::getNotionalAmount).<Money>map("getValue", FieldWithMeta::getValue).<String>groupBy(g->new MapperS<>(g).map("getCurrency", Money::getCurrency).<String>map("getValue", FieldWithMeta::getValue)).<BigDecimal>map("getAmount", Money::getAmount)), MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getBefore", QuantityChangePrimitive::getBefore).<Contract>map("getContract", Trade::getContract).<ExecutionQuantity>map("getContractualQuantity", Contract::getContractualQuantity).<QuantityNotation>mapC("getQuantityNotation", ExecutionQuantity::getQuantityNotation).<NonNegativeQuantity>map("getQuantity", QuantityNotation::getQuantity).<String>groupBy(g->new MapperS<>(g).map("getCurrency", Quantity::getCurrency).<String>map("getValue", FieldWithMeta::getValue)).<BigDecimal>map("getAmount", Quantity::getAmount)))))))))), MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getAfter", QuantityChangePrimitive::getAfter).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<ContractualQuantity>map("getQuantity", InterestRatePayout::getQuantity).<Quantity>map("getQuantity", ContractualQuantity::getQuantity).<BigDecimal>map("getAmount", Quantity::getAmount)), MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getAfter", QuantityChangePrimitive::getAfter).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<ContractualQuantity>map("getQuantity", InterestRatePayout::getQuantity).map("getNotionalAmount", ContractualQuantity::getNotionalAmount).<Money>map("getValue", FieldWithMeta::getValue).<String>groupBy(g->new MapperS<>(g).map("getCurrency", Money::getCurrency).<String>map("getValue", FieldWithMeta::getValue)).<BigDecimal>map("getAmount", Money::getAmount)), MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getAfter", QuantityChangePrimitive::getAfter).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<ContractualQuantity>map("getQuantity", InterestRatePayout::getQuantity).<NotionalSchedule>map("getNotionalSchedule", ContractualQuantity::getNotionalSchedule).<NonNegativeAmountSchedule>map("getNotionalStepSchedule", NotionalSchedule::getNotionalStepSchedule).<String>groupBy(g->new MapperS<>(g).map("getCurrency", NonNegativeAmountSchedule::getCurrency).<String>map("getValue", FieldWithMeta::getValue)).<BigDecimal>map("getInitialValue", NonNegativeSchedule::getInitialValue)), MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getAfter", QuantityChangePrimitive::getAfter).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<ContractualQuantity>map("getQuantity", InterestRatePayout::getQuantity).<NotionalSchedule>map("getNotionalSchedule", ContractualQuantity::getNotionalSchedule).<NonNegativeAmountSchedule>map("getNotionalStepSchedule", NotionalSchedule::getNotionalStepSchedule).<String>groupBy(g->new MapperS<>(g).map("getCurrency", NonNegativeAmountSchedule::getCurrency).<String>map("getValue", FieldWithMeta::getValue)).<NonNegativeStep>mapC("getStep", NonNegativeSchedule::getStep).<BigDecimal>map("getStepValue", NonNegativeStep::getStepValue)), MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getAfter", QuantityChangePrimitive::getAfter).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<ContractualQuantity>map("getQuantity", InterestRatePayout::getQuantity).<NotionalSchedule>map("getNotionalSchedule", ContractualQuantity::getNotionalSchedule).<String>groupBy(g->new MapperS<>(g).<NonNegativeAmountSchedule>map("getNotionalStepSchedule", NotionalSchedule::getNotionalStepSchedule).map("getCurrency", NonNegativeAmountSchedule::getCurrency).<String>map("getValue", FieldWithMeta::getValue)).<NotionalStepRule>map("getNotionalStepParameters", NotionalSchedule::getNotionalStepParameters).<BigDecimal>map("getNotionalStepAmount", NotionalStepRule::getNotionalStepAmount)), MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getAfter", QuantityChangePrimitive::getAfter).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<ContractualQuantity>map("getQuantity", InterestRatePayout::getQuantity).<FxLinkedNotionalSchedule>map("getFxLinkedNotional", ContractualQuantity::getFxLinkedNotional).<String>groupBy(g->new MapperS<>(g).map("getVaryingNotionalCurrency", FxLinkedNotionalSchedule::getVaryingNotionalCurrency).<String>map("getValue", FieldWithMeta::getValue)).<BigDecimal>map("getInitialValue", FxLinkedNotionalSchedule::getInitialValue)), MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getAfter", QuantityChangePrimitive::getAfter).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<CreditDefaultPayout>map("getCreditDefaultPayout", Payout::getCreditDefaultPayout).<ProtectionTerms>mapC("getProtectionTerms", CreditDefaultPayout::getProtectionTerms).<Money>map("getNotionalAmount", ProtectionTerms::getNotionalAmount).<String>groupBy(g->new MapperS<>(g).map("getCurrency", Money::getCurrency).<String>map("getValue", FieldWithMeta::getValue)).<BigDecimal>map("getAmount", Money::getAmount)), MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getAfter", QuantityChangePrimitive::getAfter).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<OptionPayout>mapC("getOptionPayout", Payout::getOptionPayout).<ContractualQuantity>map("getQuantity", OptionPayout::getQuantity).map("getNotionalAmount", ContractualQuantity::getNotionalAmount).<Money>map("getValue", FieldWithMeta::getValue).<String>groupBy(g->new MapperS<>(g).map("getCurrency", Money::getCurrency).<String>map("getValue", FieldWithMeta::getValue)).<BigDecimal>map("getAmount", Money::getAmount)), MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getAfter", QuantityChangePrimitive::getAfter).<Contract>map("getContract", Trade::getContract).<ExecutionQuantity>map("getContractualQuantity", Contract::getContractualQuantity).<QuantityNotation>mapC("getQuantityNotation", ExecutionQuantity::getQuantityNotation).<NonNegativeQuantity>map("getQuantity", QuantityNotation::getQuantity).<String>groupBy(g->new MapperS<>(g).map("getCurrency", Quantity::getCurrency).<String>map("getValue", FieldWithMeta::getValue)).<BigDecimal>map("getAmount", Quantity::getAmount)))))))))))).and(notExists(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getAfter", QuantityChangePrimitive::getAfter).<Contract>map("getContract", Trade::getContract).<ClosedState>map("getClosedState", Contract::getClosedState)));
		
		return QualifyResult.builder()
				.setName("Increase")
				.setDefinition(QUALIFY_DEFINITION)
				.setExpressionResult(EXPR_DEFINITION, exprResult)
				.build();
	}
}
