package org.isda.cdm.qualify.product;

import com.rosetta.model.lib.annotations.RosettaQualifiable;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.qualify.QualifyResult;
import com.rosetta.model.lib.validation.ComparisonResult;
import java.util.function.Function;
import org.isda.cdm.EconomicTerms;
import org.isda.cdm.Equity;
import org.isda.cdm.EquityPayout;
import org.isda.cdm.ExchangeTradedFund;
import org.isda.cdm.InterestRatePayout;
import org.isda.cdm.MutualFund;
import org.isda.cdm.Payout;
import org.isda.cdm.Product;
import org.isda.cdm.ReturnTypeEnum;
import org.isda.cdm.Security;
import org.isda.cdm.SingleUnderlier;
import org.isda.cdm.Underlier;
import org.isda.cdm.Warrant;
import org.isda.cdm.validation.datarule.*;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaQualifiable("EquitySwap_ParameterReturnDividend_SingleName")
public class IsEquitySwapParameterReturnDividendSingleName implements Function<EconomicTerms, QualifyResult>{
	
	private static final String QUALIFY_DEFINITION = "isProduct EquitySwap_ParameterReturnDividend_SingleName <The Parameter Return Dividend equity swap specified in the ISDA taxonomy V2.0 corresponds to the dividend return swap.>\n" + 
		"	EconomicTerms -> payout -> interestRatePayout exists\n" + 
		"	and EconomicTerms -> payout -> equityPayout -> returnType = ReturnTypeEnum.Dividend\n" + 
		"	and ( EconomicTerms -> payout -> equityPayout -> underlier -> singleUnderlier -> underlyingProduct -> security -> equity\n" + 
		"		or EconomicTerms -> payout -> equityPayout -> underlier -> singleUnderlier -> underlyingProduct -> security -> exchangeTradedFund\n" + 
		"		or EconomicTerms -> payout -> equityPayout -> underlier -> singleUnderlier -> underlyingProduct -> security -> mutualFund\n" + 
		"		or EconomicTerms -> payout -> equityPayout -> underlier -> singleUnderlier -> underlyingProduct -> security -> warrant\n" + 
		"		) exists";
	
	private static final String EXPR_DEFINITION = "EconomicTerms -> payout -> interestRatePayout exists\n" + 
		"	and EconomicTerms -> payout -> equityPayout -> returnType = ReturnTypeEnum.Dividend\n" + 
		"	and ( EconomicTerms -> payout -> equityPayout -> underlier -> singleUnderlier -> underlyingProduct -> security -> equity\n" + 
		"		or EconomicTerms -> payout -> equityPayout -> underlier -> singleUnderlier -> underlyingProduct -> security -> exchangeTradedFund\n" + 
		"		or EconomicTerms -> payout -> equityPayout -> underlier -> singleUnderlier -> underlyingProduct -> security -> mutualFund\n" + 
		"		or EconomicTerms -> payout -> equityPayout -> underlier -> singleUnderlier -> underlyingProduct -> security -> warrant\n" + 
		"		) exists";
	
	@Override
	public QualifyResult apply(EconomicTerms economicTerms) {
		ComparisonResult exprResult = exists(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout), false).and(areEqual(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<EquityPayout>mapC("getEquityPayout", Payout::getEquityPayout).<ReturnTypeEnum>map("getReturnType", EquityPayout::getReturnType), MapperS.of(ReturnTypeEnum.DIVIDEND))).and(exists(MapperTree.orDifferent(MapperTree.orDifferent(MapperTree.orDifferent(MapperTree.of(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<EquityPayout>mapC("getEquityPayout", Payout::getEquityPayout).<Underlier>map("getUnderlier", EquityPayout::getUnderlier).<SingleUnderlier>map("getSingleUnderlier", Underlier::getSingleUnderlier).<Product>map("getUnderlyingProduct", SingleUnderlier::getUnderlyingProduct).<Security>map("getSecurity", Product::getSecurity).<Equity>map("getEquity", Security::getEquity)), MapperTree.of(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<EquityPayout>mapC("getEquityPayout", Payout::getEquityPayout).<Underlier>map("getUnderlier", EquityPayout::getUnderlier).<SingleUnderlier>map("getSingleUnderlier", Underlier::getSingleUnderlier).<Product>map("getUnderlyingProduct", SingleUnderlier::getUnderlyingProduct).<Security>map("getSecurity", Product::getSecurity).<ExchangeTradedFund>map("getExchangeTradedFund", Security::getExchangeTradedFund))), MapperTree.of(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<EquityPayout>mapC("getEquityPayout", Payout::getEquityPayout).<Underlier>map("getUnderlier", EquityPayout::getUnderlier).<SingleUnderlier>map("getSingleUnderlier", Underlier::getSingleUnderlier).<Product>map("getUnderlyingProduct", SingleUnderlier::getUnderlyingProduct).<Security>map("getSecurity", Product::getSecurity).<MutualFund>map("getMutualFund", Security::getMutualFund))), MapperTree.of(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<EquityPayout>mapC("getEquityPayout", Payout::getEquityPayout).<Underlier>map("getUnderlier", EquityPayout::getUnderlier).<SingleUnderlier>map("getSingleUnderlier", Underlier::getSingleUnderlier).<Product>map("getUnderlyingProduct", SingleUnderlier::getUnderlyingProduct).<Security>map("getSecurity", Product::getSecurity).<Warrant>map("getWarrant", Security::getWarrant))), false));
		
		return QualifyResult.builder()
				.setName("EquitySwap_ParameterReturnDividend_SingleName")
				.setDefinition(QUALIFY_DEFINITION)
				.setExpressionResult(EXPR_DEFINITION, exprResult)
				.build();
	}
}
