package org.isda.cdm.qualify.product;

import com.rosetta.model.lib.annotations.RosettaQualifiable;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.qualify.QualifyResult;
import com.rosetta.model.lib.records.Date;
import com.rosetta.model.lib.validation.ComparisonResult;
import java.util.function.Function;
import org.isda.cdm.AdjustableDate;
import org.isda.cdm.AdjustableOrRelativeDate;
import org.isda.cdm.AdjustedRelativeDateOffset;
import org.isda.cdm.CalculationPeriodDates;
import org.isda.cdm.Cashflow;
import org.isda.cdm.CreditDefaultPayout;
import org.isda.cdm.EconomicTerms;
import org.isda.cdm.EquityPayout;
import org.isda.cdm.InterestRatePayout;
import org.isda.cdm.Payout;
import org.isda.cdm.RelativeDateOffset;
import org.isda.cdm.validation.datarule.*;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaQualifiable("InterestRate_Fra")
public class IsInterestRateFra implements Function<EconomicTerms, QualifyResult>{
	
	private static final String QUALIFY_DEFINITION = "isProduct InterestRate_Fra <null>\n" + 
		"	EconomicTerms -> payout -> interestRatePayout count = 2\n" + 
		"	and EconomicTerms -> payout -> creditDefaultPayout is absent\n" + 
		"	and EconomicTerms -> payout -> cashflow is absent\n" + 
		"	and effectiveDate = paymentDate";
	
	private static final String EXPR_DEFINITION = "EconomicTerms -> payout -> interestRatePayout count = 2\n" + 
		"	and EconomicTerms -> payout -> creditDefaultPayout is absent\n" + 
		"	and EconomicTerms -> payout -> cashflow is absent\n" + 
		"	and effectiveDate = paymentDate";
	
	@Override
	public QualifyResult apply(EconomicTerms economicTerms) {
		ComparisonResult exprResult = areEqual(MapperS.of(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).resultCount()), MapperS.of(Integer.valueOf(2))).and(notExists(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<CreditDefaultPayout>map("getCreditDefaultPayout", Payout::getCreditDefaultPayout))).and(notExists(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<Cashflow>mapC("getCashflow", Payout::getCashflow))).and(areEqual(MapperTree.or(MapperTree.of(MapperS.of(economicTerms).<AdjustableOrRelativeDate>map("getEffectiveDate", EconomicTerms::getEffectiveDate).<AdjustableDate>map("getAdjustableDate", AdjustableOrRelativeDate::getAdjustableDate).map("getAdjustedDate", AdjustableDate::getAdjustedDate).<Date>map("getValue", FieldWithMeta::getValue)), MapperTree.or(MapperTree.of(MapperS.of(economicTerms).<AdjustableOrRelativeDate>map("getEffectiveDate", EconomicTerms::getEffectiveDate).<AdjustableDate>map("getAdjustableDate", AdjustableOrRelativeDate::getAdjustableDate).<Date>map("getUnadjustedDate", AdjustableDate::getUnadjustedDate)), MapperTree.or(MapperTree.of(MapperS.of(economicTerms).<AdjustableOrRelativeDate>map("getEffectiveDate", EconomicTerms::getEffectiveDate).<AdjustedRelativeDateOffset>map("getRelativeDate", AdjustableOrRelativeDate::getRelativeDate).<Date>map("getAdjustedDate", RelativeDateOffset::getAdjustedDate)), MapperTree.or(MapperTree.of(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<CalculationPeriodDates>map("getCalculationPeriodDates", InterestRatePayout::getCalculationPeriodDates).<AdjustableOrRelativeDate>map("getEffectiveDate", CalculationPeriodDates::getEffectiveDate).<AdjustableDate>map("getAdjustableDate", AdjustableOrRelativeDate::getAdjustableDate).map("getAdjustedDate", AdjustableDate::getAdjustedDate).<Date>map("getValue", FieldWithMeta::getValue)), MapperTree.or(MapperTree.of(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<CalculationPeriodDates>map("getCalculationPeriodDates", InterestRatePayout::getCalculationPeriodDates).<AdjustableOrRelativeDate>map("getEffectiveDate", CalculationPeriodDates::getEffectiveDate).<AdjustableDate>map("getAdjustableDate", AdjustableOrRelativeDate::getAdjustableDate).<Date>map("getUnadjustedDate", AdjustableDate::getUnadjustedDate)), MapperTree.or(MapperTree.of(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<CalculationPeriodDates>map("getCalculationPeriodDates", InterestRatePayout::getCalculationPeriodDates).<AdjustableOrRelativeDate>map("getEffectiveDate", CalculationPeriodDates::getEffectiveDate).<AdjustedRelativeDateOffset>map("getRelativeDate", AdjustableOrRelativeDate::getRelativeDate).<Date>map("getAdjustedDate", RelativeDateOffset::getAdjustedDate)), MapperTree.or(MapperTree.of(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<EquityPayout>mapC("getEquityPayout", Payout::getEquityPayout).<CalculationPeriodDates>map("getCalculationPeriodDates", EquityPayout::getCalculationPeriodDates).<AdjustableOrRelativeDate>map("getEffectiveDate", CalculationPeriodDates::getEffectiveDate).<AdjustableDate>map("getAdjustableDate", AdjustableOrRelativeDate::getAdjustableDate).map("getAdjustedDate", AdjustableDate::getAdjustedDate).<Date>map("getValue", FieldWithMeta::getValue)), MapperTree.or(MapperTree.of(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<EquityPayout>mapC("getEquityPayout", Payout::getEquityPayout).<CalculationPeriodDates>map("getCalculationPeriodDates", EquityPayout::getCalculationPeriodDates).<AdjustableOrRelativeDate>map("getEffectiveDate", CalculationPeriodDates::getEffectiveDate).<AdjustableDate>map("getAdjustableDate", AdjustableOrRelativeDate::getAdjustableDate).<Date>map("getUnadjustedDate", AdjustableDate::getUnadjustedDate)), MapperTree.of(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<EquityPayout>mapC("getEquityPayout", Payout::getEquityPayout).<CalculationPeriodDates>map("getCalculationPeriodDates", EquityPayout::getCalculationPeriodDates).<AdjustableOrRelativeDate>map("getEffectiveDate", CalculationPeriodDates::getEffectiveDate).<AdjustedRelativeDateOffset>map("getRelativeDate", AdjustableOrRelativeDate::getRelativeDate).<Date>map("getAdjustedDate", RelativeDateOffset::getAdjustedDate)))))))))), MapperTree.of(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<AdjustableDate>map("getPaymentDate", InterestRatePayout::getPaymentDate).map("getAdjustedDate", AdjustableDate::getAdjustedDate).<Date>map("getValue", FieldWithMeta::getValue))));
		
		return QualifyResult.builder()
				.setName("InterestRate_Fra")
				.setDefinition(QUALIFY_DEFINITION)
				.setExpressionResult(EXPR_DEFINITION, exprResult)
				.build();
	}
}
