package org.isda.cdm.qualify.product;

import com.rosetta.model.lib.annotations.RosettaQualifiable;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.qualify.QualifyResult;
import com.rosetta.model.lib.validation.ComparisonResult;
import java.util.function.Function;
import org.isda.cdm.CalculationPeriodDates;
import org.isda.cdm.CompoundingMethodEnum;
import org.isda.cdm.ContractualQuantity;
import org.isda.cdm.CrossCurrencyTerms;
import org.isda.cdm.EconomicTerms;
import org.isda.cdm.FloatingRate;
import org.isda.cdm.FloatingRateSpecification;
import org.isda.cdm.Frequency;
import org.isda.cdm.InflationRateSpecification;
import org.isda.cdm.InterestRatePayout;
import org.isda.cdm.NonNegativeAmountSchedule;
import org.isda.cdm.NonNegativeQuantitySchedule;
import org.isda.cdm.NonNegativeSchedule;
import org.isda.cdm.NonNegativeStep;
import org.isda.cdm.NonNegativeStepSchedule;
import org.isda.cdm.NotionalSchedule;
import org.isda.cdm.NotionalStepRule;
import org.isda.cdm.OptionPayout;
import org.isda.cdm.PaymentDates;
import org.isda.cdm.Payout;
import org.isda.cdm.PayoutBase;
import org.isda.cdm.Period;
import org.isda.cdm.PeriodExtendedEnum;
import org.isda.cdm.PrincipalExchanges;
import org.isda.cdm.RateSpecification;
import org.isda.cdm.ResetDates;
import org.isda.cdm.ResetFrequency;
import org.isda.cdm.ResolvablePayoutQuantity;
import org.isda.cdm.Schedule;
import org.isda.cdm.SettlementProvision;
import org.isda.cdm.SpreadSchedule;
import org.isda.cdm.Step;
import org.isda.cdm.StubPeriod;
import org.isda.cdm.StubPeriodTypeEnum;
import org.isda.cdm.validation.datarule.*;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaQualifiable("InterestRate_IRSwap_FixedFloat_PlainVanilla")
public class IsInterestRateIRSwapFixedFloatPlainVanilla implements Function<EconomicTerms, QualifyResult>{
	
	private static final String QUALIFY_DEFINITION = "isProduct InterestRate_IRSwap_FixedFloat_PlainVanilla <This product qualification doesn't represent the exact terms of the ISDA Taxonomomy V2.0 for the plain vanilla swaps, as some of those cannot be represented as part of the CDM syntax (e.g. the qualification that there is no provision for early termination which uses an off-market valuation), while some other are deemed missing in the ISDA taxonomy and have been added as part of the CDM (absence of cross-currency settlement provision, absence of fixed rate and notional step schedule, absence of stub). >\n" + 
		"	EconomicTerms -> payout -> interestRatePayout -> rateSpecification -> fixedRate count = 1\n" + 
		"	and EconomicTerms -> payout -> interestRatePayout -> rateSpecification -> floatingRate count = 1\n" + 
		"	and EconomicTerms -> payout -> interestRatePayout -> rateSpecification -> inflationRate is absent\n" + 
		"	and EconomicTerms -> payout -> interestRatePayout -> crossCurrencyTerms -> principalExchanges is absent\n" + 
		"	and EconomicTerms -> payout -> interestRatePayout -> crossCurrencyTerms -> settlementProvision is absent\n" + 
		"	and EconomicTerms -> payout -> optionPayout is absent\n" + 
		"	and EconomicTerms -> payout -> interestRatePayout -> rateSpecification -> floatingRate -> spreadSchedule -> step is absent\n" + 
		"	and EconomicTerms -> payout -> interestRatePayout -> rateSpecification -> fixedRate -> step is absent\n" + 
		"	and EconomicTerms -> payout -> interestRatePayout -> rateSpecification -> floatingRate -> floatingRateMultiplierSchedule is absent\n" + 
		"	and EconomicTerms -> payout -> interestRatePayout -> quantity -> notionalSchedule -> notionalStepSchedule -> step is absent\n" + 
		"	and EconomicTerms -> payout -> interestRatePayout -> quantity -> notionalSchedule -> notionalStepParameters is absent\n" + 
		"	and EconomicTerms -> payout -> interestRatePayout -> payoutQuantity -> quantitySchedule -> stepSchedule -> step is absent\n" + 
		"	and EconomicTerms -> payout -> interestRatePayout -> stubPeriod is absent\n" + 
		"	and EconomicTerms -> payout -> interestRatePayout -> calculationPeriodDates -> stubPeriodType is absent\n" + 
		"	/*\n" + 
		" 	* TODO - Cannot compare the below because in FpML the period associated with the reset frequency is of type PeriodExtendedEnum, which includes 'T'\n" + 
		"	* Need to adjust the implementation once a proper implementation approach has been firmed out\n" + 
		"	*/\n" + 
		"//	and EconomicTerms -> payout -> interestRatePayout -> resetDates -> resetFrequency -> period =\n" + 
		"//		EconomicTerms -> payout -> interestRatePayout -> interestRate -> floatingRate -> indexTenor -> period\n" + 
		"	and EconomicTerms -> payout -> interestRatePayout -> resetDates -> resetFrequency -> periodMultiplier =\n" + 
		"		EconomicTerms -> payout -> interestRatePayout -> rateSpecification -> floatingRate -> indexTenor -> periodMultiplier\n" + 
		"	and EconomicTerms -> payout -> interestRatePayout -> paymentDates -> paymentFrequency -> period <> PeriodExtendedEnum.T\n" + 
		"	and EconomicTerms -> payout -> interestRatePayout -> compoundingMethod is absent";
	
	private static final String EXPR_DEFINITION = "EconomicTerms -> payout -> interestRatePayout -> rateSpecification -> fixedRate count = 1\n" + 
		"	and EconomicTerms -> payout -> interestRatePayout -> rateSpecification -> floatingRate count = 1\n" + 
		"	and EconomicTerms -> payout -> interestRatePayout -> rateSpecification -> inflationRate is absent\n" + 
		"	and EconomicTerms -> payout -> interestRatePayout -> crossCurrencyTerms -> principalExchanges is absent\n" + 
		"	and EconomicTerms -> payout -> interestRatePayout -> crossCurrencyTerms -> settlementProvision is absent\n" + 
		"	and EconomicTerms -> payout -> optionPayout is absent\n" + 
		"	and EconomicTerms -> payout -> interestRatePayout -> rateSpecification -> floatingRate -> spreadSchedule -> step is absent\n" + 
		"	and EconomicTerms -> payout -> interestRatePayout -> rateSpecification -> fixedRate -> step is absent\n" + 
		"	and EconomicTerms -> payout -> interestRatePayout -> rateSpecification -> floatingRate -> floatingRateMultiplierSchedule is absent\n" + 
		"	and EconomicTerms -> payout -> interestRatePayout -> quantity -> notionalSchedule -> notionalStepSchedule -> step is absent\n" + 
		"	and EconomicTerms -> payout -> interestRatePayout -> quantity -> notionalSchedule -> notionalStepParameters is absent\n" + 
		"	and EconomicTerms -> payout -> interestRatePayout -> payoutQuantity -> quantitySchedule -> stepSchedule -> step is absent\n" + 
		"	and EconomicTerms -> payout -> interestRatePayout -> stubPeriod is absent\n" + 
		"	and EconomicTerms -> payout -> interestRatePayout -> calculationPeriodDates -> stubPeriodType is absent\n" + 
		"	/*\n" + 
		" 	* TODO - Cannot compare the below because in FpML the period associated with the reset frequency is of type PeriodExtendedEnum, which includes 'T'\n" + 
		"	* Need to adjust the implementation once a proper implementation approach has been firmed out\n" + 
		"	*/\n" + 
		"//	and EconomicTerms -> payout -> interestRatePayout -> resetDates -> resetFrequency -> period =\n" + 
		"//		EconomicTerms -> payout -> interestRatePayout -> interestRate -> floatingRate -> indexTenor -> period\n" + 
		"	and EconomicTerms -> payout -> interestRatePayout -> resetDates -> resetFrequency -> periodMultiplier =\n" + 
		"		EconomicTerms -> payout -> interestRatePayout -> rateSpecification -> floatingRate -> indexTenor -> periodMultiplier\n" + 
		"	and EconomicTerms -> payout -> interestRatePayout -> paymentDates -> paymentFrequency -> period <> PeriodExtendedEnum.T\n" + 
		"	and EconomicTerms -> payout -> interestRatePayout -> compoundingMethod is absent";
	
	@Override
	public QualifyResult apply(EconomicTerms economicTerms) {
		ComparisonResult exprResult = areEqual(MapperS.of(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<RateSpecification>map("getRateSpecification", InterestRatePayout::getRateSpecification).<Schedule>map("getFixedRate", RateSpecification::getFixedRate).resultCount()), MapperS.of(Integer.valueOf(1))).and(areEqual(MapperS.of(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<RateSpecification>map("getRateSpecification", InterestRatePayout::getRateSpecification).<FloatingRateSpecification>map("getFloatingRate", RateSpecification::getFloatingRate).resultCount()), MapperS.of(Integer.valueOf(1)))).and(notExists(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<RateSpecification>map("getRateSpecification", InterestRatePayout::getRateSpecification).<InflationRateSpecification>map("getInflationRate", RateSpecification::getInflationRate))).and(notExists(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<CrossCurrencyTerms>map("getCrossCurrencyTerms", InterestRatePayout::getCrossCurrencyTerms).<PrincipalExchanges>map("getPrincipalExchanges", CrossCurrencyTerms::getPrincipalExchanges))).and(notExists(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<CrossCurrencyTerms>map("getCrossCurrencyTerms", InterestRatePayout::getCrossCurrencyTerms).<SettlementProvision>map("getSettlementProvision", CrossCurrencyTerms::getSettlementProvision))).and(notExists(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<OptionPayout>mapC("getOptionPayout", Payout::getOptionPayout))).and(notExists(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<RateSpecification>map("getRateSpecification", InterestRatePayout::getRateSpecification).<FloatingRateSpecification>map("getFloatingRate", RateSpecification::getFloatingRate).<SpreadSchedule>mapC("getSpreadSchedule", FloatingRate::getSpreadSchedule).<Step>mapC("getStep", Schedule::getStep))).and(notExists(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<RateSpecification>map("getRateSpecification", InterestRatePayout::getRateSpecification).<Schedule>map("getFixedRate", RateSpecification::getFixedRate).<Step>mapC("getStep", Schedule::getStep))).and(notExists(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<RateSpecification>map("getRateSpecification", InterestRatePayout::getRateSpecification).<FloatingRateSpecification>map("getFloatingRate", RateSpecification::getFloatingRate).<Schedule>map("getFloatingRateMultiplierSchedule", FloatingRate::getFloatingRateMultiplierSchedule))).and(notExists(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<ContractualQuantity>map("getQuantity", InterestRatePayout::getQuantity).<NotionalSchedule>map("getNotionalSchedule", ContractualQuantity::getNotionalSchedule).<NonNegativeAmountSchedule>map("getNotionalStepSchedule", NotionalSchedule::getNotionalStepSchedule).<NonNegativeStep>mapC("getStep", NonNegativeSchedule::getStep))).and(notExists(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<ContractualQuantity>map("getQuantity", InterestRatePayout::getQuantity).<NotionalSchedule>map("getNotionalSchedule", ContractualQuantity::getNotionalSchedule).<NotionalStepRule>map("getNotionalStepParameters", NotionalSchedule::getNotionalStepParameters))).and(notExists(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<ResolvablePayoutQuantity>map("getPayoutQuantity", PayoutBase::getPayoutQuantity).<NonNegativeQuantitySchedule>map("getQuantitySchedule", ResolvablePayoutQuantity::getQuantitySchedule).<NonNegativeStepSchedule>map("getStepSchedule", NonNegativeQuantitySchedule::getStepSchedule).<NonNegativeStep>mapC("getStep", NonNegativeStepSchedule::getStep))).and(notExists(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<StubPeriod>map("getStubPeriod", InterestRatePayout::getStubPeriod))).and(notExists(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<CalculationPeriodDates>map("getCalculationPeriodDates", InterestRatePayout::getCalculationPeriodDates).<StubPeriodTypeEnum>map("getStubPeriodType", CalculationPeriodDates::getStubPeriodType))).and(areEqual(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<ResetDates>map("getResetDates", InterestRatePayout::getResetDates).<ResetFrequency>map("getResetFrequency", ResetDates::getResetFrequency).<Integer>map("getPeriodMultiplier", Frequency::getPeriodMultiplier), MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<RateSpecification>map("getRateSpecification", InterestRatePayout::getRateSpecification).<FloatingRateSpecification>map("getFloatingRate", RateSpecification::getFloatingRate).<Period>map("getIndexTenor", FloatingRate::getIndexTenor).<Integer>map("getPeriodMultiplier", Period::getPeriodMultiplier))).and(notEqual(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<PaymentDates>map("getPaymentDates", InterestRatePayout::getPaymentDates).<Frequency>map("getPaymentFrequency", PaymentDates::getPaymentFrequency).<PeriodExtendedEnum>map("getPeriod", Frequency::getPeriod), MapperS.of(PeriodExtendedEnum.T))).and(notExists(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<CompoundingMethodEnum>map("getCompoundingMethod", InterestRatePayout::getCompoundingMethod)));
		
		return QualifyResult.builder()
				.setName("InterestRate_IRSwap_FixedFloat_PlainVanilla")
				.setDefinition(QUALIFY_DEFINITION)
				.setExpressionResult(EXPR_DEFINITION, exprResult)
				.build();
	}
}
