package org.isda.cdm.qualify.product;

import com.rosetta.model.lib.annotations.RosettaQualifiable;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.qualify.QualifyResult;
import com.rosetta.model.lib.validation.ComparisonResult;
import java.util.function.Function;
import org.isda.cdm.CrossCurrencyTerms;
import org.isda.cdm.EconomicTerms;
import org.isda.cdm.FloatingRateSpecification;
import org.isda.cdm.Frequency;
import org.isda.cdm.InflationRateSpecification;
import org.isda.cdm.InterestRatePayout;
import org.isda.cdm.OptionPayout;
import org.isda.cdm.PaymentDates;
import org.isda.cdm.Payout;
import org.isda.cdm.PeriodExtendedEnum;
import org.isda.cdm.PrincipalExchanges;
import org.isda.cdm.RateSpecification;
import org.isda.cdm.Schedule;
import org.isda.cdm.validation.datarule.*;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaQualifiable("InterestRate_InflationSwap_FixedFloat_YearOn_Year")
public class IsInterestRateInflationSwapFixedFloatYearOnYear implements Function<EconomicTerms, QualifyResult>{
	
	private static final String QUALIFY_DEFINITION = "isProduct InterestRate_InflationSwap_FixedFloat_YearOn_Year <null>\n" + 
		"	EconomicTerms -> payout -> interestRatePayout -> rateSpecification -> fixedRate count = 1\n" + 
		"	and EconomicTerms -> payout -> interestRatePayout -> rateSpecification -> inflationRate count = 1\n" + 
		"	and EconomicTerms -> payout -> interestRatePayout -> rateSpecification -> floatingRate is absent\n" + 
		"	and EconomicTerms -> payout -> interestRatePayout -> crossCurrencyTerms -> principalExchanges is absent\n" + 
		"	and EconomicTerms -> payout -> optionPayout is absent\n" + 
		"	and EconomicTerms -> payout -> interestRatePayout -> paymentDates -> paymentFrequency -> periodMultiplier = 1\n" + 
		"	and EconomicTerms -> payout -> interestRatePayout -> paymentDates -> paymentFrequency -> period = PeriodExtendedEnum.Y";
	
	private static final String EXPR_DEFINITION = "EconomicTerms -> payout -> interestRatePayout -> rateSpecification -> fixedRate count = 1\n" + 
		"	and EconomicTerms -> payout -> interestRatePayout -> rateSpecification -> inflationRate count = 1\n" + 
		"	and EconomicTerms -> payout -> interestRatePayout -> rateSpecification -> floatingRate is absent\n" + 
		"	and EconomicTerms -> payout -> interestRatePayout -> crossCurrencyTerms -> principalExchanges is absent\n" + 
		"	and EconomicTerms -> payout -> optionPayout is absent\n" + 
		"	and EconomicTerms -> payout -> interestRatePayout -> paymentDates -> paymentFrequency -> periodMultiplier = 1\n" + 
		"	and EconomicTerms -> payout -> interestRatePayout -> paymentDates -> paymentFrequency -> period = PeriodExtendedEnum.Y";
	
	@Override
	public QualifyResult apply(EconomicTerms economicTerms) {
		ComparisonResult exprResult = areEqual(MapperS.of(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<RateSpecification>map("getRateSpecification", InterestRatePayout::getRateSpecification).<Schedule>map("getFixedRate", RateSpecification::getFixedRate).resultCount()), MapperS.of(Integer.valueOf(1))).and(areEqual(MapperS.of(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<RateSpecification>map("getRateSpecification", InterestRatePayout::getRateSpecification).<InflationRateSpecification>map("getInflationRate", RateSpecification::getInflationRate).resultCount()), MapperS.of(Integer.valueOf(1)))).and(notExists(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<RateSpecification>map("getRateSpecification", InterestRatePayout::getRateSpecification).<FloatingRateSpecification>map("getFloatingRate", RateSpecification::getFloatingRate))).and(notExists(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<CrossCurrencyTerms>map("getCrossCurrencyTerms", InterestRatePayout::getCrossCurrencyTerms).<PrincipalExchanges>map("getPrincipalExchanges", CrossCurrencyTerms::getPrincipalExchanges))).and(notExists(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<OptionPayout>mapC("getOptionPayout", Payout::getOptionPayout))).and(areEqual(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<PaymentDates>map("getPaymentDates", InterestRatePayout::getPaymentDates).<Frequency>map("getPaymentFrequency", PaymentDates::getPaymentFrequency).<Integer>map("getPeriodMultiplier", Frequency::getPeriodMultiplier), MapperS.of(Integer.valueOf(1)))).and(areEqual(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<PaymentDates>map("getPaymentDates", InterestRatePayout::getPaymentDates).<Frequency>map("getPaymentFrequency", PaymentDates::getPaymentFrequency).<PeriodExtendedEnum>map("getPeriod", Frequency::getPeriod), MapperS.of(PeriodExtendedEnum.Y)));
		
		return QualifyResult.builder()
				.setName("InterestRate_InflationSwap_FixedFloat_YearOn_Year")
				.setDefinition(QUALIFY_DEFINITION)
				.setExpressionResult(EXPR_DEFINITION, exprResult)
				.build();
	}
}
