package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.Affirmation;
import org.isda.cdm.Execution;
import org.isda.cdm.Lineage;
import org.isda.cdm.PartyRole;
import org.isda.cdm.PartyRoleEnum;
import org.isda.cdm.Product;
import org.isda.cdm.Security;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("Affirmation_partyRole_buyerSeller")
public class AffirmationPartyRoleBuyerSellerDataRule implements Validator<Affirmation> {
	
	private static final String NAME = "Affirmation_partyRole_buyerSeller";
	private static final String DEFINITION = "when Affirmation -> lineage -> executionReference -> product -> security exists\n" + 
		"then Affirmation -> partyRole -> role contains PartyRoleEnum.Buyer or Affirmation -> partyRole -> role contains PartyRoleEnum.Seller";
	
	@Override
	public ValidationResult<Affirmation> validate(RosettaPath path, Affirmation affirmation) {
		ComparisonResult result = executeDataRule(affirmation);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "Affirmation", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "Affirmation", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<Affirmation> validate(RosettaPath path, RosettaModelObjectBuilder affirmation) {
		ComparisonResult result = executeDataRule((Affirmation)affirmation.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "Affirmation", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "Affirmation", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(Affirmation affirmation) {
		if (ruleIsApplicable(affirmation).get()) {
			return evaluateThenExpression(affirmation);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(Affirmation affirmation) {
		return exists(MapperS.of(affirmation).<Lineage>map("getLineage", Affirmation::getLineage).mapC("getExecutionReference", Lineage::getExecutionReference).<Execution>map("getValue", FieldWithMeta::getValue).<Product>map("getProduct", Execution::getProduct).<Security>map("getSecurity", Product::getSecurity), false);
	}
	
	private ComparisonResult evaluateThenExpression(Affirmation affirmation) {
		return contains(MapperS.of(affirmation).<PartyRole>mapC("getPartyRole", Affirmation::getPartyRole).<PartyRoleEnum>map("getRole", PartyRole::getRole), MapperS.of(PartyRoleEnum.BUYER)).or(contains(MapperS.of(affirmation).<PartyRole>mapC("getPartyRole", Affirmation::getPartyRole).<PartyRoleEnum>map("getRole", PartyRole::getRole), MapperS.of(PartyRoleEnum.SELLER)));
	}
}
