package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.AllocationOutcome;
import org.isda.cdm.ClosedState;
import org.isda.cdm.ClosedStateEnum;
import org.isda.cdm.Contract;
import org.isda.cdm.Trade;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("AllocationOutcome_contractClosed")
public class AllocationOutcomeContractClosedDataRule implements Validator<AllocationOutcome> {
	
	private static final String NAME = "AllocationOutcome_contractClosed";
	private static final String DEFINITION = "when AllocationOutcome exists and AllocationOutcome -> originalTrade -> contract exists\n" + 
		"then AllocationOutcome -> originalTrade -> contract -> closedState -> state = ClosedStateEnum.Allocated";
	
	@Override
	public ValidationResult<AllocationOutcome> validate(RosettaPath path, AllocationOutcome allocationOutcome) {
		ComparisonResult result = executeDataRule(allocationOutcome);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "AllocationOutcome", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "AllocationOutcome", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<AllocationOutcome> validate(RosettaPath path, RosettaModelObjectBuilder allocationOutcome) {
		ComparisonResult result = executeDataRule((AllocationOutcome)allocationOutcome.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "AllocationOutcome", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "AllocationOutcome", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(AllocationOutcome allocationOutcome) {
		if (ruleIsApplicable(allocationOutcome).get()) {
			return evaluateThenExpression(allocationOutcome);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(AllocationOutcome allocationOutcome) {
		return exists(MapperS.of(allocationOutcome), false).and(exists(MapperS.of(allocationOutcome).<Trade>map("getOriginalTrade", AllocationOutcome::getOriginalTrade).<Contract>map("getContract", Trade::getContract), false));
	}
	
	private ComparisonResult evaluateThenExpression(AllocationOutcome allocationOutcome) {
		return areEqual(MapperS.of(allocationOutcome).<Trade>map("getOriginalTrade", AllocationOutcome::getOriginalTrade).<Contract>map("getContract", Trade::getContract).<ClosedState>map("getClosedState", Contract::getClosedState).<ClosedStateEnum>map("getState", ClosedState::getState), MapperS.of(ClosedStateEnum.ALLOCATED));
	}
}
