package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.math.BigDecimal;
import org.isda.cdm.CashSettlementTerms;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("CashSettlementTerms_recoveryFactor")
public class CashSettlementTermsRecoveryFactorDataRule implements Validator<CashSettlementTerms> {
	
	private static final String NAME = "CashSettlementTerms_recoveryFactor";
	private static final String DEFINITION = "when CashSettlementTerms -> recoveryFactor exists\n" + 
		"then CashSettlementTerms -> recoveryFactor >= 0.0 and CashSettlementTerms -> recoveryFactor <= 1.0";
	
	@Override
	public ValidationResult<CashSettlementTerms> validate(RosettaPath path, CashSettlementTerms cashSettlementTerms) {
		ComparisonResult result = executeDataRule(cashSettlementTerms);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "CashSettlementTerms", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "CashSettlementTerms", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<CashSettlementTerms> validate(RosettaPath path, RosettaModelObjectBuilder cashSettlementTerms) {
		ComparisonResult result = executeDataRule((CashSettlementTerms)cashSettlementTerms.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "CashSettlementTerms", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "CashSettlementTerms", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(CashSettlementTerms cashSettlementTerms) {
		if (ruleIsApplicable(cashSettlementTerms).get()) {
			return evaluateThenExpression(cashSettlementTerms);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(CashSettlementTerms cashSettlementTerms) {
		return exists(MapperS.of(cashSettlementTerms).<BigDecimal>map("getRecoveryFactor", CashSettlementTerms::getRecoveryFactor), false);
	}
	
	private ComparisonResult evaluateThenExpression(CashSettlementTerms cashSettlementTerms) {
		return greaterThanEquals(MapperS.of(cashSettlementTerms).<BigDecimal>map("getRecoveryFactor", CashSettlementTerms::getRecoveryFactor), MapperS.of(new BigDecimal(0.0))).and(lessThanEquals(MapperS.of(cashSettlementTerms).<BigDecimal>map("getRecoveryFactor", CashSettlementTerms::getRecoveryFactor), MapperS.of(new BigDecimal(1.0))));
	}
}
