package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.math.BigDecimal;
import org.isda.cdm.Cashflow;
import org.isda.cdm.Money;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("Cashflow_cashflowAmount")
public class CashflowCashflowAmountDataRule implements Validator<Cashflow> {
	
	private static final String NAME = "Cashflow_cashflowAmount";
	private static final String DEFINITION = "when Cashflow -> cashflowAmount exists\n" + 
		"then Cashflow -> cashflowAmount -> amount >= 0.0";
	
	@Override
	public ValidationResult<Cashflow> validate(RosettaPath path, Cashflow cashflow) {
		ComparisonResult result = executeDataRule(cashflow);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "Cashflow", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "Cashflow", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<Cashflow> validate(RosettaPath path, RosettaModelObjectBuilder cashflow) {
		ComparisonResult result = executeDataRule((Cashflow)cashflow.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "Cashflow", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "Cashflow", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(Cashflow cashflow) {
		if (ruleIsApplicable(cashflow).get()) {
			return evaluateThenExpression(cashflow);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(Cashflow cashflow) {
		return exists(MapperS.of(cashflow).<Money>map("getCashflowAmount", Cashflow::getCashflowAmount), false);
	}
	
	private ComparisonResult evaluateThenExpression(Cashflow cashflow) {
		return greaterThanEquals(MapperS.of(cashflow).<Money>map("getCashflowAmount", Cashflow::getCashflowAmount).<BigDecimal>map("getAmount", Money::getAmount), MapperS.of(new BigDecimal(0.0)));
	}
}
