package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.Confirmation;
import org.isda.cdm.Execution;
import org.isda.cdm.Lineage;
import org.isda.cdm.PartyRole;
import org.isda.cdm.PartyRoleEnum;
import org.isda.cdm.Product;
import org.isda.cdm.Security;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("Confirmation_partyRole_buyerSeller")
public class ConfirmationPartyRoleBuyerSellerDataRule implements Validator<Confirmation> {
	
	private static final String NAME = "Confirmation_partyRole_buyerSeller";
	private static final String DEFINITION = "when Confirmation -> lineage -> executionReference -> product -> security exists\n" + 
		"then Confirmation -> partyRole -> role contains PartyRoleEnum.Buyer or Confirmation -> partyRole -> role contains PartyRoleEnum.Seller";
	
	@Override
	public ValidationResult<Confirmation> validate(RosettaPath path, Confirmation confirmation) {
		ComparisonResult result = executeDataRule(confirmation);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "Confirmation", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "Confirmation", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<Confirmation> validate(RosettaPath path, RosettaModelObjectBuilder confirmation) {
		ComparisonResult result = executeDataRule((Confirmation)confirmation.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "Confirmation", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "Confirmation", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(Confirmation confirmation) {
		if (ruleIsApplicable(confirmation).get()) {
			return evaluateThenExpression(confirmation);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(Confirmation confirmation) {
		return exists(MapperS.of(confirmation).<Lineage>map("getLineage", Confirmation::getLineage).mapC("getExecutionReference", Lineage::getExecutionReference).<Execution>map("getValue", FieldWithMeta::getValue).<Product>map("getProduct", Execution::getProduct).<Security>map("getSecurity", Product::getSecurity), false);
	}
	
	private ComparisonResult evaluateThenExpression(Confirmation confirmation) {
		return contains(MapperS.of(confirmation).<PartyRole>mapC("getPartyRole", Confirmation::getPartyRole).<PartyRoleEnum>map("getRole", PartyRole::getRole), MapperS.of(PartyRoleEnum.BUYER)).or(contains(MapperS.of(confirmation).<PartyRole>mapC("getPartyRole", Confirmation::getPartyRole).<PartyRoleEnum>map("getRole", PartyRole::getRole), MapperS.of(PartyRoleEnum.SELLER)));
	}
}
