package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.math.BigDecimal;
import org.isda.cdm.ConstituentWeight;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("ConstituentWeight_basketPercentage")
public class ConstituentWeightBasketPercentageDataRule implements Validator<ConstituentWeight> {
	
	private static final String NAME = "ConstituentWeight_basketPercentage";
	private static final String DEFINITION = "when ConstituentWeight -> basketPercentage exists\n" + 
		"then ConstituentWeight -> basketPercentage >= 0.0 and ConstituentWeight -> basketPercentage <= 1.0";
	
	@Override
	public ValidationResult<ConstituentWeight> validate(RosettaPath path, ConstituentWeight constituentWeight) {
		ComparisonResult result = executeDataRule(constituentWeight);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "ConstituentWeight", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "ConstituentWeight", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<ConstituentWeight> validate(RosettaPath path, RosettaModelObjectBuilder constituentWeight) {
		ComparisonResult result = executeDataRule((ConstituentWeight)constituentWeight.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "ConstituentWeight", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "ConstituentWeight", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(ConstituentWeight constituentWeight) {
		if (ruleIsApplicable(constituentWeight).get()) {
			return evaluateThenExpression(constituentWeight);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(ConstituentWeight constituentWeight) {
		return exists(MapperS.of(constituentWeight).<BigDecimal>map("getBasketPercentage", ConstituentWeight::getBasketPercentage), false);
	}
	
	private ComparisonResult evaluateThenExpression(ConstituentWeight constituentWeight) {
		return greaterThanEquals(MapperS.of(constituentWeight).<BigDecimal>map("getBasketPercentage", ConstituentWeight::getBasketPercentage), MapperS.of(new BigDecimal(0.0))).and(lessThanEquals(MapperS.of(constituentWeight).<BigDecimal>map("getBasketPercentage", ConstituentWeight::getBasketPercentage), MapperS.of(new BigDecimal(1.0))));
	}
}
