package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.Contract;
import org.isda.cdm.ContractualMatrix;
import org.isda.cdm.ContractualProduct;
import org.isda.cdm.CreditDefaultPayout;
import org.isda.cdm.DeliverableObligations;
import org.isda.cdm.Documentation;
import org.isda.cdm.DocumentationIdentification;
import org.isda.cdm.EconomicTerms;
import org.isda.cdm.MatrixTypeEnum;
import org.isda.cdm.NotDomesticCurrency;
import org.isda.cdm.PCDeliverableObligationCharac;
import org.isda.cdm.Payout;
import org.isda.cdm.Period;
import org.isda.cdm.PhysicalSettlementTerms;
import org.isda.cdm.SpecifiedCurrency;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("DeliverableObligations_physicalSettlementMatrix")
public class DeliverableObligationsPhysicalSettlementMatrixDataRule implements Validator<Contract> {
	
	private static final String NAME = "DeliverableObligations_physicalSettlementMatrix";
	private static final String DEFINITION = "when ( Contract -> documentation -> documentationIdentification -> contractualMatrix -> matrixType <> MatrixTypeEnum.CreditDerivativesPhysicalSettlementMatrix\n" + 
		"		or Contract -> documentation -> documentationIdentification -> contractualMatrix -> matrixType is absent )\n" + 
		"		and Contract -> contractualProduct -> economicTerms -> payout -> creditDefaultPayout -> physicalSettlementTerms -> deliverableObligations exists\n" + 
		"then ( Contract -> contractualProduct -> economicTerms -> payout -> creditDefaultPayout -> physicalSettlementTerms -> deliverableObligations -> notSubordinated\n" + 
		"		and Contract -> contractualProduct -> economicTerms -> payout -> creditDefaultPayout -> physicalSettlementTerms -> deliverableObligations -> specifiedCurrency\n" + 
		"		and Contract -> contractualProduct -> economicTerms -> payout -> creditDefaultPayout -> physicalSettlementTerms -> deliverableObligations -> notSovereignLender\n" + 
		"		and Contract -> contractualProduct -> economicTerms -> payout -> creditDefaultPayout -> physicalSettlementTerms -> deliverableObligations -> notDomesticCurrency\n" + 
		"		and Contract -> contractualProduct -> economicTerms -> payout -> creditDefaultPayout -> physicalSettlementTerms -> deliverableObligations -> notDomesticLaw\n" + 
		"		and Contract -> contractualProduct -> economicTerms -> payout -> creditDefaultPayout -> physicalSettlementTerms -> deliverableObligations -> notContingent\n" + 
		"		and Contract -> contractualProduct -> economicTerms -> payout -> creditDefaultPayout -> physicalSettlementTerms -> deliverableObligations -> notDomesticIssuance\n" + 
		"		and Contract -> contractualProduct -> economicTerms -> payout -> creditDefaultPayout -> physicalSettlementTerms -> deliverableObligations -> assignableLoan\n" + 
		"		and Contract -> contractualProduct -> economicTerms -> payout -> creditDefaultPayout -> physicalSettlementTerms -> deliverableObligations -> consentRequiredLoan\n" + 
		"		and Contract -> contractualProduct -> economicTerms -> payout -> creditDefaultPayout -> physicalSettlementTerms -> deliverableObligations -> transferable\n" + 
		"		and Contract -> contractualProduct -> economicTerms -> payout -> creditDefaultPayout -> physicalSettlementTerms -> deliverableObligations -> maximumMaturity\n" + 
		"		and Contract -> contractualProduct -> economicTerms -> payout -> creditDefaultPayout -> physicalSettlementTerms -> deliverableObligations -> notBearer\n" + 
		"	) exists\n" + 
		"	and ( Contract -> contractualProduct -> economicTerms -> payout -> creditDefaultPayout -> physicalSettlementTerms -> deliverableObligations -> fullFaithAndCreditObLiability\n" + 
		"		or Contract -> contractualProduct -> economicTerms -> payout -> creditDefaultPayout -> physicalSettlementTerms -> deliverableObligations -> generalFundObligationLiability\n" + 
		"		or Contract -> contractualProduct -> economicTerms -> payout -> creditDefaultPayout -> physicalSettlementTerms -> deliverableObligations -> revenueObligationLiability\n" + 
		"	) exists";
	
	@Override
	public ValidationResult<Contract> validate(RosettaPath path, Contract contract) {
		ComparisonResult result = executeDataRule(contract);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "Contract", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "Contract", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<Contract> validate(RosettaPath path, RosettaModelObjectBuilder contract) {
		ComparisonResult result = executeDataRule((Contract)contract.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "Contract", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "Contract", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(Contract contract) {
		if (ruleIsApplicable(contract).get()) {
			return evaluateThenExpression(contract);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(Contract contract) {
		return notEqual(MapperS.of(contract).<Documentation>map("getDocumentation", Contract::getDocumentation).<DocumentationIdentification>map("getDocumentationIdentification", Documentation::getDocumentationIdentification).<ContractualMatrix>mapC("getContractualMatrix", DocumentationIdentification::getContractualMatrix).map("getMatrixType", ContractualMatrix::getMatrixType).<MatrixTypeEnum>map("getValue", FieldWithMeta::getValue), MapperS.of(MatrixTypeEnum.CREDIT_DERIVATIVES_PHYSICAL_SETTLEMENT_MATRIX)).or(notExists(MapperS.of(contract).<Documentation>map("getDocumentation", Contract::getDocumentation).<DocumentationIdentification>map("getDocumentationIdentification", Documentation::getDocumentationIdentification).<ContractualMatrix>mapC("getContractualMatrix", DocumentationIdentification::getContractualMatrix).map("getMatrixType", ContractualMatrix::getMatrixType).<MatrixTypeEnum>map("getValue", FieldWithMeta::getValue))).and(exists(MapperS.of(contract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<CreditDefaultPayout>map("getCreditDefaultPayout", Payout::getCreditDefaultPayout).<PhysicalSettlementTerms>mapC("getPhysicalSettlementTerms", CreditDefaultPayout::getPhysicalSettlementTerms).<DeliverableObligations>map("getDeliverableObligations", PhysicalSettlementTerms::getDeliverableObligations), false));
	}
	
	private ComparisonResult evaluateThenExpression(Contract contract) {
		return exists(MapperTree.andDifferent(MapperTree.andDifferent(MapperTree.andDifferent(MapperTree.andDifferent(MapperTree.andDifferent(MapperTree.andDifferent(MapperTree.andDifferent(MapperTree.andDifferent(MapperTree.andDifferent(MapperTree.andDifferent(MapperTree.andDifferent(MapperTree.of(MapperS.of(contract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<CreditDefaultPayout>map("getCreditDefaultPayout", Payout::getCreditDefaultPayout).<PhysicalSettlementTerms>mapC("getPhysicalSettlementTerms", CreditDefaultPayout::getPhysicalSettlementTerms).<DeliverableObligations>map("getDeliverableObligations", PhysicalSettlementTerms::getDeliverableObligations).<Boolean>map("getNotSubordinated", DeliverableObligations::getNotSubordinated)), MapperTree.of(MapperS.of(contract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<CreditDefaultPayout>map("getCreditDefaultPayout", Payout::getCreditDefaultPayout).<PhysicalSettlementTerms>mapC("getPhysicalSettlementTerms", CreditDefaultPayout::getPhysicalSettlementTerms).<DeliverableObligations>map("getDeliverableObligations", PhysicalSettlementTerms::getDeliverableObligations).<SpecifiedCurrency>map("getSpecifiedCurrency", DeliverableObligations::getSpecifiedCurrency))), MapperTree.of(MapperS.of(contract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<CreditDefaultPayout>map("getCreditDefaultPayout", Payout::getCreditDefaultPayout).<PhysicalSettlementTerms>mapC("getPhysicalSettlementTerms", CreditDefaultPayout::getPhysicalSettlementTerms).<DeliverableObligations>map("getDeliverableObligations", PhysicalSettlementTerms::getDeliverableObligations).<Boolean>map("getNotSovereignLender", DeliverableObligations::getNotSovereignLender))), MapperTree.of(MapperS.of(contract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<CreditDefaultPayout>map("getCreditDefaultPayout", Payout::getCreditDefaultPayout).<PhysicalSettlementTerms>mapC("getPhysicalSettlementTerms", CreditDefaultPayout::getPhysicalSettlementTerms).<DeliverableObligations>map("getDeliverableObligations", PhysicalSettlementTerms::getDeliverableObligations).<NotDomesticCurrency>map("getNotDomesticCurrency", DeliverableObligations::getNotDomesticCurrency))), MapperTree.of(MapperS.of(contract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<CreditDefaultPayout>map("getCreditDefaultPayout", Payout::getCreditDefaultPayout).<PhysicalSettlementTerms>mapC("getPhysicalSettlementTerms", CreditDefaultPayout::getPhysicalSettlementTerms).<DeliverableObligations>map("getDeliverableObligations", PhysicalSettlementTerms::getDeliverableObligations).<Boolean>map("getNotDomesticLaw", DeliverableObligations::getNotDomesticLaw))), MapperTree.of(MapperS.of(contract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<CreditDefaultPayout>map("getCreditDefaultPayout", Payout::getCreditDefaultPayout).<PhysicalSettlementTerms>mapC("getPhysicalSettlementTerms", CreditDefaultPayout::getPhysicalSettlementTerms).<DeliverableObligations>map("getDeliverableObligations", PhysicalSettlementTerms::getDeliverableObligations).<Boolean>map("getNotContingent", DeliverableObligations::getNotContingent))), MapperTree.of(MapperS.of(contract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<CreditDefaultPayout>map("getCreditDefaultPayout", Payout::getCreditDefaultPayout).<PhysicalSettlementTerms>mapC("getPhysicalSettlementTerms", CreditDefaultPayout::getPhysicalSettlementTerms).<DeliverableObligations>map("getDeliverableObligations", PhysicalSettlementTerms::getDeliverableObligations).<Boolean>map("getNotDomesticIssuance", DeliverableObligations::getNotDomesticIssuance))), MapperTree.of(MapperS.of(contract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<CreditDefaultPayout>map("getCreditDefaultPayout", Payout::getCreditDefaultPayout).<PhysicalSettlementTerms>mapC("getPhysicalSettlementTerms", CreditDefaultPayout::getPhysicalSettlementTerms).<DeliverableObligations>map("getDeliverableObligations", PhysicalSettlementTerms::getDeliverableObligations).<PCDeliverableObligationCharac>map("getAssignableLoan", DeliverableObligations::getAssignableLoan))), MapperTree.of(MapperS.of(contract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<CreditDefaultPayout>map("getCreditDefaultPayout", Payout::getCreditDefaultPayout).<PhysicalSettlementTerms>mapC("getPhysicalSettlementTerms", CreditDefaultPayout::getPhysicalSettlementTerms).<DeliverableObligations>map("getDeliverableObligations", PhysicalSettlementTerms::getDeliverableObligations).<PCDeliverableObligationCharac>map("getConsentRequiredLoan", DeliverableObligations::getConsentRequiredLoan))), MapperTree.of(MapperS.of(contract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<CreditDefaultPayout>map("getCreditDefaultPayout", Payout::getCreditDefaultPayout).<PhysicalSettlementTerms>mapC("getPhysicalSettlementTerms", CreditDefaultPayout::getPhysicalSettlementTerms).<DeliverableObligations>map("getDeliverableObligations", PhysicalSettlementTerms::getDeliverableObligations).<Boolean>map("getTransferable", DeliverableObligations::getTransferable))), MapperTree.of(MapperS.of(contract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<CreditDefaultPayout>map("getCreditDefaultPayout", Payout::getCreditDefaultPayout).<PhysicalSettlementTerms>mapC("getPhysicalSettlementTerms", CreditDefaultPayout::getPhysicalSettlementTerms).<DeliverableObligations>map("getDeliverableObligations", PhysicalSettlementTerms::getDeliverableObligations).<Period>map("getMaximumMaturity", DeliverableObligations::getMaximumMaturity))), MapperTree.of(MapperS.of(contract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<CreditDefaultPayout>map("getCreditDefaultPayout", Payout::getCreditDefaultPayout).<PhysicalSettlementTerms>mapC("getPhysicalSettlementTerms", CreditDefaultPayout::getPhysicalSettlementTerms).<DeliverableObligations>map("getDeliverableObligations", PhysicalSettlementTerms::getDeliverableObligations).<Boolean>map("getNotBearer", DeliverableObligations::getNotBearer))), false).and(exists(MapperTree.or(MapperTree.or(MapperTree.of(MapperS.of(contract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<CreditDefaultPayout>map("getCreditDefaultPayout", Payout::getCreditDefaultPayout).<PhysicalSettlementTerms>mapC("getPhysicalSettlementTerms", CreditDefaultPayout::getPhysicalSettlementTerms).<DeliverableObligations>map("getDeliverableObligations", PhysicalSettlementTerms::getDeliverableObligations).<Boolean>map("getFullFaithAndCreditObLiability", DeliverableObligations::getFullFaithAndCreditObLiability)), MapperTree.of(MapperS.of(contract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<CreditDefaultPayout>map("getCreditDefaultPayout", Payout::getCreditDefaultPayout).<PhysicalSettlementTerms>mapC("getPhysicalSettlementTerms", CreditDefaultPayout::getPhysicalSettlementTerms).<DeliverableObligations>map("getDeliverableObligations", PhysicalSettlementTerms::getDeliverableObligations).<Boolean>map("getGeneralFundObligationLiability", DeliverableObligations::getGeneralFundObligationLiability))), MapperTree.of(MapperS.of(contract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<CreditDefaultPayout>map("getCreditDefaultPayout", Payout::getCreditDefaultPayout).<PhysicalSettlementTerms>mapC("getPhysicalSettlementTerms", CreditDefaultPayout::getPhysicalSettlementTerms).<DeliverableObligations>map("getDeliverableObligations", PhysicalSettlementTerms::getDeliverableObligations).<Boolean>map("getRevenueObligationLiability", DeliverableObligations::getRevenueObligationLiability))), false));
	}
}
