package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.math.BigDecimal;
import org.isda.cdm.EligibleCollateral;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("EligibleCollateral_valuationPercentage")
public class EligibleCollateralValuationPercentageDataRule implements Validator<EligibleCollateral> {
	
	private static final String NAME = "EligibleCollateral_valuationPercentage";
	private static final String DEFINITION = "when EligibleCollateral -> valuationPercentage exists\n" + 
		"then EligibleCollateral -> valuationPercentage >= 0\n" + 
		"	and EligibleCollateral -> valuationPercentage <= 1";
	
	@Override
	public ValidationResult<EligibleCollateral> validate(RosettaPath path, EligibleCollateral eligibleCollateral) {
		ComparisonResult result = executeDataRule(eligibleCollateral);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "EligibleCollateral", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "EligibleCollateral", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<EligibleCollateral> validate(RosettaPath path, RosettaModelObjectBuilder eligibleCollateral) {
		ComparisonResult result = executeDataRule((EligibleCollateral)eligibleCollateral.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "EligibleCollateral", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "EligibleCollateral", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(EligibleCollateral eligibleCollateral) {
		if (ruleIsApplicable(eligibleCollateral).get()) {
			return evaluateThenExpression(eligibleCollateral);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(EligibleCollateral eligibleCollateral) {
		return exists(MapperS.of(eligibleCollateral).<BigDecimal>map("getValuationPercentage", EligibleCollateral::getValuationPercentage), false);
	}
	
	private ComparisonResult evaluateThenExpression(EligibleCollateral eligibleCollateral) {
		return greaterThanEquals(MapperS.of(eligibleCollateral).<BigDecimal>map("getValuationPercentage", EligibleCollateral::getValuationPercentage), MapperS.of(Integer.valueOf(0))).and(lessThanEquals(MapperS.of(eligibleCollateral).<BigDecimal>map("getValuationPercentage", EligibleCollateral::getValuationPercentage), MapperS.of(Integer.valueOf(1))));
	}
}
