package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.DividendReturnTerms;
import org.isda.cdm.EquityPayout;
import org.isda.cdm.PriceReturnTerms;
import org.isda.cdm.ReturnTypeEnum;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("EquityPayout_priceReturn")
public class EquityPayoutPriceReturnDataRule implements Validator<EquityPayout> {
	
	private static final String NAME = "EquityPayout_priceReturn";
	private static final String DEFINITION = "when EquityPayout -> returnType = ReturnTypeEnum.Dividend\n" + 
		"then EquityPayout -> dividendReturnTerms exists\n" + 
		"	and EquityPayout -> priceReturnTerms is absent";
	
	@Override
	public ValidationResult<EquityPayout> validate(RosettaPath path, EquityPayout equityPayout) {
		ComparisonResult result = executeDataRule(equityPayout);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "EquityPayout", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "EquityPayout", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<EquityPayout> validate(RosettaPath path, RosettaModelObjectBuilder equityPayout) {
		ComparisonResult result = executeDataRule((EquityPayout)equityPayout.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "EquityPayout", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "EquityPayout", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(EquityPayout equityPayout) {
		if (ruleIsApplicable(equityPayout).get()) {
			return evaluateThenExpression(equityPayout);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(EquityPayout equityPayout) {
		return areEqual(MapperS.of(equityPayout).<ReturnTypeEnum>map("getReturnType", EquityPayout::getReturnType), MapperS.of(ReturnTypeEnum.DIVIDEND));
	}
	
	private ComparisonResult evaluateThenExpression(EquityPayout equityPayout) {
		return exists(MapperS.of(equityPayout).<DividendReturnTerms>map("getDividendReturnTerms", EquityPayout::getDividendReturnTerms), false).and(notExists(MapperS.of(equityPayout).<PriceReturnTerms>map("getPriceReturnTerms", EquityPayout::getPriceReturnTerms)));
	}
}
