package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.Equity;
import org.isda.cdm.EquityPayout;
import org.isda.cdm.ExchangeTradedFund;
import org.isda.cdm.MutualFund;
import org.isda.cdm.Product;
import org.isda.cdm.Security;
import org.isda.cdm.SingleUnderlier;
import org.isda.cdm.Underlier;
import org.isda.cdm.Warrant;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("EquityPayout_singleUnderlier")
public class EquityPayoutSingleUnderlierDataRule implements Validator<EquityPayout> {
	
	private static final String NAME = "EquityPayout_singleUnderlier";
	private static final String DEFINITION = "when EquityPayout -> underlier -> singleUnderlier exists\n" + 
		"then ( EquityPayout -> underlier -> singleUnderlier -> underlyingProduct -> security -> equity\n" + 
		"		or EquityPayout -> underlier -> singleUnderlier -> underlyingProduct -> security -> exchangeTradedFund\n" + 
		"		or EquityPayout -> underlier -> singleUnderlier -> underlyingProduct -> security -> mutualFund\n" + 
		"		or EquityPayout -> underlier -> singleUnderlier -> underlyingProduct -> security -> warrant ) exists";
	
	@Override
	public ValidationResult<EquityPayout> validate(RosettaPath path, EquityPayout equityPayout) {
		ComparisonResult result = executeDataRule(equityPayout);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "EquityPayout", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "EquityPayout", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<EquityPayout> validate(RosettaPath path, RosettaModelObjectBuilder equityPayout) {
		ComparisonResult result = executeDataRule((EquityPayout)equityPayout.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "EquityPayout", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "EquityPayout", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(EquityPayout equityPayout) {
		if (ruleIsApplicable(equityPayout).get()) {
			return evaluateThenExpression(equityPayout);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(EquityPayout equityPayout) {
		return exists(MapperS.of(equityPayout).<Underlier>map("getUnderlier", EquityPayout::getUnderlier).<SingleUnderlier>map("getSingleUnderlier", Underlier::getSingleUnderlier), false);
	}
	
	private ComparisonResult evaluateThenExpression(EquityPayout equityPayout) {
		return exists(MapperTree.orDifferent(MapperTree.orDifferent(MapperTree.orDifferent(MapperTree.of(MapperS.of(equityPayout).<Underlier>map("getUnderlier", EquityPayout::getUnderlier).<SingleUnderlier>map("getSingleUnderlier", Underlier::getSingleUnderlier).<Product>map("getUnderlyingProduct", SingleUnderlier::getUnderlyingProduct).<Security>map("getSecurity", Product::getSecurity).<Equity>map("getEquity", Security::getEquity)), MapperTree.of(MapperS.of(equityPayout).<Underlier>map("getUnderlier", EquityPayout::getUnderlier).<SingleUnderlier>map("getSingleUnderlier", Underlier::getSingleUnderlier).<Product>map("getUnderlyingProduct", SingleUnderlier::getUnderlyingProduct).<Security>map("getSecurity", Product::getSecurity).<ExchangeTradedFund>map("getExchangeTradedFund", Security::getExchangeTradedFund))), MapperTree.of(MapperS.of(equityPayout).<Underlier>map("getUnderlier", EquityPayout::getUnderlier).<SingleUnderlier>map("getSingleUnderlier", Underlier::getSingleUnderlier).<Product>map("getUnderlyingProduct", SingleUnderlier::getUnderlyingProduct).<Security>map("getSecurity", Product::getSecurity).<MutualFund>map("getMutualFund", Security::getMutualFund))), MapperTree.of(MapperS.of(equityPayout).<Underlier>map("getUnderlier", EquityPayout::getUnderlier).<SingleUnderlier>map("getSingleUnderlier", Underlier::getSingleUnderlier).<Product>map("getUnderlyingProduct", SingleUnderlier::getUnderlyingProduct).<Security>map("getSecurity", Product::getSecurity).<Warrant>map("getWarrant", Security::getWarrant))), false);
	}
}
