package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.records.Date;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.InterestRatePayout;
import org.isda.cdm.LastRegularPaymentDate;
import org.isda.cdm.PaymentDates;
import org.isda.cdm.Payout;
import org.isda.cdm.StubPeriod;
import org.isda.cdm.StubValue;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("FinalStub_lastRegularPaymentDate")
public class FinalStubLastRegularPaymentDateDataRule implements Validator<Payout> {
	
	private static final String NAME = "FinalStub_lastRegularPaymentDate";
	private static final String DEFINITION = "when Payout -> interestRatePayout -> paymentDates -> lastPaymentDate -> lastRegularPaymentDate exists\n" + 
		"		and Payout -> interestRatePayout -> stubPeriod -> finalStub exists\n" + 
		"then Payout -> interestRatePayout count = 2";
	
	@Override
	public ValidationResult<Payout> validate(RosettaPath path, Payout payout) {
		ComparisonResult result = executeDataRule(payout);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "Payout", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "Payout", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<Payout> validate(RosettaPath path, RosettaModelObjectBuilder payout) {
		ComparisonResult result = executeDataRule((Payout)payout.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "Payout", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "Payout", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(Payout payout) {
		if (ruleIsApplicable(payout).get()) {
			return evaluateThenExpression(payout);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(Payout payout) {
		return exists(MapperS.of(payout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<PaymentDates>map("getPaymentDates", InterestRatePayout::getPaymentDates).<LastRegularPaymentDate>map("getLastPaymentDate", PaymentDates::getLastPaymentDate).<Date>map("getLastRegularPaymentDate", LastRegularPaymentDate::getLastRegularPaymentDate), false).and(exists(MapperS.of(payout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<StubPeriod>map("getStubPeriod", InterestRatePayout::getStubPeriod).<StubValue>map("getFinalStub", StubPeriod::getFinalStub), false));
	}
	
	private ComparisonResult evaluateThenExpression(Payout payout) {
		return areEqual(MapperS.of(MapperS.of(payout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).resultCount()), MapperS.of(Integer.valueOf(2)));
	}
}
