package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.math.BigDecimal;
import org.isda.cdm.CreditDefaultPayout;
import org.isda.cdm.GeneralTerms;
import org.isda.cdm.ReferenceInformation;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("FpML_cd_12")
public class FpMLCd12DataRule implements Validator<CreditDefaultPayout> {
	
	private static final String NAME = "FpML_cd_12";
	private static final String DEFINITION = "when CreditDefaultPayout -> generalTerms -> referenceInformation -> referencePrice exists\n" + 
		"then CreditDefaultPayout -> generalTerms -> referenceInformation -> referencePrice >= 0";
	
	@Override
	public ValidationResult<CreditDefaultPayout> validate(RosettaPath path, CreditDefaultPayout creditDefaultPayout) {
		ComparisonResult result = executeDataRule(creditDefaultPayout);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "CreditDefaultPayout", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "CreditDefaultPayout", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<CreditDefaultPayout> validate(RosettaPath path, RosettaModelObjectBuilder creditDefaultPayout) {
		ComparisonResult result = executeDataRule((CreditDefaultPayout)creditDefaultPayout.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "CreditDefaultPayout", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "CreditDefaultPayout", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(CreditDefaultPayout creditDefaultPayout) {
		if (ruleIsApplicable(creditDefaultPayout).get()) {
			return evaluateThenExpression(creditDefaultPayout);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(CreditDefaultPayout creditDefaultPayout) {
		return exists(MapperS.of(creditDefaultPayout).<GeneralTerms>map("getGeneralTerms", CreditDefaultPayout::getGeneralTerms).<ReferenceInformation>map("getReferenceInformation", GeneralTerms::getReferenceInformation).<BigDecimal>map("getReferencePrice", ReferenceInformation::getReferencePrice), false);
	}
	
	private ComparisonResult evaluateThenExpression(CreditDefaultPayout creditDefaultPayout) {
		return greaterThanEquals(MapperS.of(creditDefaultPayout).<GeneralTerms>map("getGeneralTerms", CreditDefaultPayout::getGeneralTerms).<ReferenceInformation>map("getReferenceInformation", GeneralTerms::getReferenceInformation).<BigDecimal>map("getReferencePrice", ReferenceInformation::getReferencePrice), MapperS.of(Integer.valueOf(0)));
	}
}
