package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.BuyerSeller;
import org.isda.cdm.CreditDefaultPayout;
import org.isda.cdm.CreditEventNotice;
import org.isda.cdm.CreditEvents;
import org.isda.cdm.GeneralTerms;
import org.isda.cdm.NotifyingParty;
import org.isda.cdm.Party;
import org.isda.cdm.ProtectionTerms;
import org.isda.cdm.metafields.*;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("FpML_cd_14")
public class FpMLCd14DataRule implements Validator<CreditDefaultPayout> {
	
	private static final String NAME = "FpML_cd_14";
	private static final String DEFINITION = "when CreditDefaultPayout -> protectionTerms -> creditEvents -> creditEventNotice -> notifyingParty -> sellerPartyReference -> reference exists\n" + 
		"then CreditDefaultPayout -> protectionTerms -> creditEvents -> creditEventNotice -> notifyingParty -> sellerPartyReference -> reference = CreditDefaultPayout -> generalTerms -> buyerSeller -> sellerPartyReference -> reference";
	
	@Override
	public ValidationResult<CreditDefaultPayout> validate(RosettaPath path, CreditDefaultPayout creditDefaultPayout) {
		ComparisonResult result = executeDataRule(creditDefaultPayout);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "CreditDefaultPayout", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "CreditDefaultPayout", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<CreditDefaultPayout> validate(RosettaPath path, RosettaModelObjectBuilder creditDefaultPayout) {
		ComparisonResult result = executeDataRule((CreditDefaultPayout)creditDefaultPayout.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "CreditDefaultPayout", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "CreditDefaultPayout", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(CreditDefaultPayout creditDefaultPayout) {
		if (ruleIsApplicable(creditDefaultPayout).get()) {
			return evaluateThenExpression(creditDefaultPayout);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(CreditDefaultPayout creditDefaultPayout) {
		return exists(MapperS.of(creditDefaultPayout).<ProtectionTerms>mapC("getProtectionTerms", CreditDefaultPayout::getProtectionTerms).<CreditEvents>map("getCreditEvents", ProtectionTerms::getCreditEvents).<CreditEventNotice>map("getCreditEventNotice", CreditEvents::getCreditEventNotice).<NotifyingParty>map("getNotifyingParty", CreditEventNotice::getNotifyingParty).<ReferenceWithMetaParty>map("getSellerPartyReference", NotifyingParty::getSellerPartyReference).map("getReference", a->a.getGlobalReference()), false);
	}
	
	private ComparisonResult evaluateThenExpression(CreditDefaultPayout creditDefaultPayout) {
		return areEqual(MapperS.of(creditDefaultPayout).<ProtectionTerms>mapC("getProtectionTerms", CreditDefaultPayout::getProtectionTerms).<CreditEvents>map("getCreditEvents", ProtectionTerms::getCreditEvents).<CreditEventNotice>map("getCreditEventNotice", CreditEvents::getCreditEventNotice).<NotifyingParty>map("getNotifyingParty", CreditEventNotice::getNotifyingParty).<ReferenceWithMetaParty>map("getSellerPartyReference", NotifyingParty::getSellerPartyReference).map("getReference", a->a.getGlobalReference()), MapperS.of(creditDefaultPayout).<GeneralTerms>map("getGeneralTerms", CreditDefaultPayout::getGeneralTerms).<BuyerSeller>map("getBuyerSeller", GeneralTerms::getBuyerSeller).<ReferenceWithMetaParty>map("getSellerPartyReference", BuyerSeller::getSellerPartyReference).map("getReference", a->a.getGlobalReference()));
	}
}
