package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.records.Date;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.AdjustableDate;
import org.isda.cdm.AdjustableOrRelativeDate;
import org.isda.cdm.Contract;
import org.isda.cdm.ContractualProduct;
import org.isda.cdm.CreditDefaultPayout;
import org.isda.cdm.EconomicTerms;
import org.isda.cdm.GeneralTerms;
import org.isda.cdm.Payout;
import org.isda.cdm.ReferenceInformation;
import org.isda.cdm.TradeDate;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("FpML_cd_1")
public class FpMLCd1DataRule implements Validator<Contract> {
	
	private static final String NAME = "FpML_cd_1";
	private static final String DEFINITION = "when Contract -> contractualProduct -> economicTerms -> payout -> creditDefaultPayout -> generalTerms -> referenceInformation exists\n" + 
		"then Contract -> tradeDate -> date < Contract -> contractualProduct -> economicTerms -> effectiveDate -> adjustableDate -> unadjustedDate\n" + 
		"		or Contract -> tradeDate -> date < Contract -> contractualProduct -> economicTerms -> effectiveDate -> adjustableDate -> adjustedDate";
	
	@Override
	public ValidationResult<Contract> validate(RosettaPath path, Contract contract) {
		ComparisonResult result = executeDataRule(contract);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "Contract", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "Contract", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<Contract> validate(RosettaPath path, RosettaModelObjectBuilder contract) {
		ComparisonResult result = executeDataRule((Contract)contract.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "Contract", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "Contract", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(Contract contract) {
		if (ruleIsApplicable(contract).get()) {
			return evaluateThenExpression(contract);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(Contract contract) {
		return exists(MapperS.of(contract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<CreditDefaultPayout>map("getCreditDefaultPayout", Payout::getCreditDefaultPayout).<GeneralTerms>map("getGeneralTerms", CreditDefaultPayout::getGeneralTerms).<ReferenceInformation>map("getReferenceInformation", GeneralTerms::getReferenceInformation), false);
	}
	
	private ComparisonResult evaluateThenExpression(Contract contract) {
		return lessThan(MapperS.of(contract).<TradeDate>map("getTradeDate", Contract::getTradeDate).<Date>map("getDate", TradeDate::getDate), MapperS.of(contract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<AdjustableOrRelativeDate>map("getEffectiveDate", EconomicTerms::getEffectiveDate).<AdjustableDate>map("getAdjustableDate", AdjustableOrRelativeDate::getAdjustableDate).<Date>map("getUnadjustedDate", AdjustableDate::getUnadjustedDate)).or(lessThan(MapperS.of(contract).<TradeDate>map("getTradeDate", Contract::getTradeDate).<Date>map("getDate", TradeDate::getDate), MapperS.of(contract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<AdjustableOrRelativeDate>map("getEffectiveDate", EconomicTerms::getEffectiveDate).<AdjustableDate>map("getAdjustableDate", AdjustableOrRelativeDate::getAdjustableDate).map("getAdjustedDate", AdjustableDate::getAdjustedDate).<Date>map("getValue", FieldWithMeta::getValue)));
	}
}
