package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.Contract;
import org.isda.cdm.ContractualDefinitionsEnum;
import org.isda.cdm.ContractualProduct;
import org.isda.cdm.CreditDefaultPayout;
import org.isda.cdm.Documentation;
import org.isda.cdm.DocumentationIdentification;
import org.isda.cdm.EconomicTerms;
import org.isda.cdm.Obligations;
import org.isda.cdm.Payout;
import org.isda.cdm.ProtectionTerms;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("FpML_cd_20")
public class FpMLCd20DataRule implements Validator<Contract> {
	
	private static final String NAME = "FpML_cd_20";
	private static final String DEFINITION = "when Contract -> documentation -> documentationIdentification -> contractualDefinitions = ContractualDefinitionsEnum.ISDA2003Credit\n" + 
		"then Contract -> contractualProduct -> economicTerms -> payout -> creditDefaultPayout -> protectionTerms -> obligations -> notContingent is absent";
	
	@Override
	public ValidationResult<Contract> validate(RosettaPath path, Contract contract) {
		ComparisonResult result = executeDataRule(contract);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "Contract", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "Contract", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<Contract> validate(RosettaPath path, RosettaModelObjectBuilder contract) {
		ComparisonResult result = executeDataRule((Contract)contract.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "Contract", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "Contract", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(Contract contract) {
		if (ruleIsApplicable(contract).get()) {
			return evaluateThenExpression(contract);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(Contract contract) {
		return areEqual(MapperS.of(contract).<Documentation>map("getDocumentation", Contract::getDocumentation).<DocumentationIdentification>map("getDocumentationIdentification", Documentation::getDocumentationIdentification).mapC("getContractualDefinitions", DocumentationIdentification::getContractualDefinitions).<ContractualDefinitionsEnum>map("getValue", FieldWithMeta::getValue), MapperS.of(ContractualDefinitionsEnum.ISDA_2003_CREDIT));
	}
	
	private ComparisonResult evaluateThenExpression(Contract contract) {
		return notExists(MapperS.of(contract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<CreditDefaultPayout>map("getCreditDefaultPayout", Payout::getCreditDefaultPayout).<ProtectionTerms>mapC("getProtectionTerms", CreditDefaultPayout::getProtectionTerms).<Obligations>map("getObligations", ProtectionTerms::getObligations).<Boolean>map("getNotContingent", Obligations::getNotContingent));
	}
}
