package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.records.Date;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.AdjustableDate;
import org.isda.cdm.AdjustableOrAdjustedOrRelativeDate;
import org.isda.cdm.AdjustableOrRelativeDate;
import org.isda.cdm.Cashflow;
import org.isda.cdm.CreditDefaultPayout;
import org.isda.cdm.EconomicTerms;
import org.isda.cdm.Payout;
import org.isda.cdm.RelativeDateOffset;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("FpML_cd_27")
public class FpMLCd27DataRule implements Validator<EconomicTerms> {
	
	private static final String NAME = "FpML_cd_27";
	private static final String DEFINITION = "when EconomicTerms -> payout -> creditDefaultPayout exists\n" + 
		"		and EconomicTerms -> payout -> cashflow exists\n" + 
		"		and EconomicTerms -> terminationDate exists\n" + 
		"		and EconomicTerms -> payout -> cashflow -> cashflowDate exists\n" + 
		"then EconomicTerms -> payout -> cashflow -> cashflowDate -> unadjustedDate < EconomicTerms -> terminationDate -> adjustableDate -> unadjustedDate\n" + 
		"		or EconomicTerms -> payout -> cashflow -> cashflowDate -> adjustedDate < EconomicTerms -> terminationDate -> adjustableDate -> adjustedDate\n" + 
		"		or EconomicTerms -> payout -> cashflow -> cashflowDate -> relativeDate -> adjustedDate < EconomicTerms -> terminationDate -> adjustableDate -> adjustedDate";
	
	@Override
	public ValidationResult<EconomicTerms> validate(RosettaPath path, EconomicTerms economicTerms) {
		ComparisonResult result = executeDataRule(economicTerms);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "EconomicTerms", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "EconomicTerms", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<EconomicTerms> validate(RosettaPath path, RosettaModelObjectBuilder economicTerms) {
		ComparisonResult result = executeDataRule((EconomicTerms)economicTerms.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "EconomicTerms", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "EconomicTerms", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(EconomicTerms economicTerms) {
		if (ruleIsApplicable(economicTerms).get()) {
			return evaluateThenExpression(economicTerms);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(EconomicTerms economicTerms) {
		return exists(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<CreditDefaultPayout>map("getCreditDefaultPayout", Payout::getCreditDefaultPayout), false).and(exists(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<Cashflow>mapC("getCashflow", Payout::getCashflow), false)).and(exists(MapperS.of(economicTerms).<AdjustableOrRelativeDate>map("getTerminationDate", EconomicTerms::getTerminationDate), false)).and(exists(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<Cashflow>mapC("getCashflow", Payout::getCashflow).<AdjustableOrAdjustedOrRelativeDate>map("getCashflowDate", Cashflow::getCashflowDate), false));
	}
	
	private ComparisonResult evaluateThenExpression(EconomicTerms economicTerms) {
		return lessThan(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<Cashflow>mapC("getCashflow", Payout::getCashflow).<AdjustableOrAdjustedOrRelativeDate>map("getCashflowDate", Cashflow::getCashflowDate).<Date>map("getUnadjustedDate", AdjustableOrAdjustedOrRelativeDate::getUnadjustedDate), MapperS.of(economicTerms).<AdjustableOrRelativeDate>map("getTerminationDate", EconomicTerms::getTerminationDate).<AdjustableDate>map("getAdjustableDate", AdjustableOrRelativeDate::getAdjustableDate).<Date>map("getUnadjustedDate", AdjustableDate::getUnadjustedDate)).or(lessThan(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<Cashflow>mapC("getCashflow", Payout::getCashflow).<AdjustableOrAdjustedOrRelativeDate>map("getCashflowDate", Cashflow::getCashflowDate).map("getAdjustedDate", AdjustableOrAdjustedOrRelativeDate::getAdjustedDate).<Date>map("getValue", FieldWithMeta::getValue), MapperS.of(economicTerms).<AdjustableOrRelativeDate>map("getTerminationDate", EconomicTerms::getTerminationDate).<AdjustableDate>map("getAdjustableDate", AdjustableOrRelativeDate::getAdjustableDate).map("getAdjustedDate", AdjustableDate::getAdjustedDate).<Date>map("getValue", FieldWithMeta::getValue))).or(lessThan(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<Cashflow>mapC("getCashflow", Payout::getCashflow).<AdjustableOrAdjustedOrRelativeDate>map("getCashflowDate", Cashflow::getCashflowDate).<RelativeDateOffset>map("getRelativeDate", AdjustableOrAdjustedOrRelativeDate::getRelativeDate).<Date>map("getAdjustedDate", RelativeDateOffset::getAdjustedDate), MapperS.of(economicTerms).<AdjustableOrRelativeDate>map("getTerminationDate", EconomicTerms::getTerminationDate).<AdjustableDate>map("getAdjustableDate", AdjustableOrRelativeDate::getAdjustableDate).map("getAdjustedDate", AdjustableDate::getAdjustedDate).<Date>map("getValue", FieldWithMeta::getValue)));
	}
}
