package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.records.Date;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.AdjustableDate;
import org.isda.cdm.AdjustableOrRelativeDate;
import org.isda.cdm.EconomicTerms;
import org.isda.cdm.InterestRatePayout;
import org.isda.cdm.LastRegularPaymentDate;
import org.isda.cdm.PaymentDates;
import org.isda.cdm.Payout;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("FpML_cd_30")
public class FpMLCd30DataRule implements Validator<EconomicTerms> {
	
	private static final String NAME = "FpML_cd_30";
	private static final String DEFINITION = "when (EconomicTerms -> payout -> interestRatePayout -> paymentDates -> lastPaymentDate -> lastRegularPaymentDate\n" + 
		"		and EconomicTerms -> terminationDate) exists\n" + 
		"then EconomicTerms -> payout -> interestRatePayout -> paymentDates -> lastPaymentDate -> lastRegularPaymentDate < EconomicTerms -> terminationDate -> adjustableDate -> unadjustedDate";
	
	@Override
	public ValidationResult<EconomicTerms> validate(RosettaPath path, EconomicTerms economicTerms) {
		ComparisonResult result = executeDataRule(economicTerms);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "EconomicTerms", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "EconomicTerms", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<EconomicTerms> validate(RosettaPath path, RosettaModelObjectBuilder economicTerms) {
		ComparisonResult result = executeDataRule((EconomicTerms)economicTerms.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "EconomicTerms", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "EconomicTerms", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(EconomicTerms economicTerms) {
		if (ruleIsApplicable(economicTerms).get()) {
			return evaluateThenExpression(economicTerms);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(EconomicTerms economicTerms) {
		return exists(MapperTree.andDifferent(MapperTree.of(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<PaymentDates>map("getPaymentDates", InterestRatePayout::getPaymentDates).<LastRegularPaymentDate>map("getLastPaymentDate", PaymentDates::getLastPaymentDate).<Date>map("getLastRegularPaymentDate", LastRegularPaymentDate::getLastRegularPaymentDate)), MapperTree.of(MapperS.of(economicTerms).<AdjustableOrRelativeDate>map("getTerminationDate", EconomicTerms::getTerminationDate))), false);
	}
	
	private ComparisonResult evaluateThenExpression(EconomicTerms economicTerms) {
		return lessThan(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<PaymentDates>map("getPaymentDates", InterestRatePayout::getPaymentDates).<LastRegularPaymentDate>map("getLastPaymentDate", PaymentDates::getLastPaymentDate).<Date>map("getLastRegularPaymentDate", LastRegularPaymentDate::getLastRegularPaymentDate), MapperS.of(economicTerms).<AdjustableOrRelativeDate>map("getTerminationDate", EconomicTerms::getTerminationDate).<AdjustableDate>map("getAdjustableDate", AdjustableOrRelativeDate::getAdjustableDate).<Date>map("getUnadjustedDate", AdjustableDate::getUnadjustedDate));
	}
}
