package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.Contract;
import org.isda.cdm.ContractualMatrix;
import org.isda.cdm.ContractualProduct;
import org.isda.cdm.ContractualQuantity;
import org.isda.cdm.CreditDefaultPayout;
import org.isda.cdm.DayCountFractionEnum;
import org.isda.cdm.Documentation;
import org.isda.cdm.DocumentationIdentification;
import org.isda.cdm.EconomicTerms;
import org.isda.cdm.GeneralTerms;
import org.isda.cdm.InterestRatePayout;
import org.isda.cdm.MasterConfirmation;
import org.isda.cdm.Money;
import org.isda.cdm.Payout;
import org.isda.cdm.ReferenceInformation;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("FpML_cd_32")
public class FpMLCd32DataRule implements Validator<Contract> {
	
	private static final String NAME = "FpML_cd_32";
	private static final String DEFINITION = "when ( Contract -> documentation -> documentationIdentification -> masterConfirmation and Contract -> documentation -> documentationIdentification -> contractualMatrix ) is absent\n" + 
		"		and Contract -> contractualProduct -> economicTerms -> payout -> creditDefaultPayout -> generalTerms -> referenceInformation exists\n" + 
		"		and Contract -> contractualProduct -> economicTerms -> payout -> interestRatePayout -> quantity -> notionalAmount exists\n" + 
		"then Contract -> contractualProduct -> economicTerms -> payout -> interestRatePayout -> dayCountFraction exists";
	
	@Override
	public ValidationResult<Contract> validate(RosettaPath path, Contract contract) {
		ComparisonResult result = executeDataRule(contract);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "Contract", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "Contract", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<Contract> validate(RosettaPath path, RosettaModelObjectBuilder contract) {
		ComparisonResult result = executeDataRule((Contract)contract.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "Contract", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "Contract", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(Contract contract) {
		if (ruleIsApplicable(contract).get()) {
			return evaluateThenExpression(contract);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(Contract contract) {
		return notExists(MapperTree.andDifferent(MapperTree.of(MapperS.of(contract).<Documentation>map("getDocumentation", Contract::getDocumentation).<DocumentationIdentification>map("getDocumentationIdentification", Documentation::getDocumentationIdentification).<MasterConfirmation>map("getMasterConfirmation", DocumentationIdentification::getMasterConfirmation)), MapperTree.of(MapperS.of(contract).<Documentation>map("getDocumentation", Contract::getDocumentation).<DocumentationIdentification>map("getDocumentationIdentification", Documentation::getDocumentationIdentification).<ContractualMatrix>mapC("getContractualMatrix", DocumentationIdentification::getContractualMatrix)))).and(exists(MapperS.of(contract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<CreditDefaultPayout>map("getCreditDefaultPayout", Payout::getCreditDefaultPayout).<GeneralTerms>map("getGeneralTerms", CreditDefaultPayout::getGeneralTerms).<ReferenceInformation>map("getReferenceInformation", GeneralTerms::getReferenceInformation), false)).and(exists(MapperS.of(contract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<ContractualQuantity>map("getQuantity", InterestRatePayout::getQuantity).map("getNotionalAmount", ContractualQuantity::getNotionalAmount).<Money>map("getValue", FieldWithMeta::getValue), false));
	}
	
	private ComparisonResult evaluateThenExpression(Contract contract) {
		return exists(MapperS.of(contract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).map("getDayCountFraction", InterestRatePayout::getDayCountFraction).<DayCountFractionEnum>map("getValue", FieldWithMeta::getValue), false);
	}
}
