package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.DeliverableObligations;
import org.isda.cdm.LoanParticipation;
import org.isda.cdm.NotDomesticCurrency;
import org.isda.cdm.ObligationCategoryEnum;
import org.isda.cdm.PCDeliverableObligationCharac;
import org.isda.cdm.Period;
import org.isda.cdm.SpecifiedCurrency;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("FpML_cd_34")
public class FpMLCd34DataRule implements Validator<DeliverableObligations> {
	
	private static final String NAME = "FpML_cd_34";
	private static final String DEFINITION = "when DeliverableObligations -> category exists\n" + 
		"	and DeliverableObligations -> category = ObligationCategoryEnum.ReferenceObligationsOnly\n" + 
		"then (DeliverableObligations -> acceleratedOrMatured\n" + 
		"		and DeliverableObligations -> accruedInterest\n" + 
		"		and DeliverableObligations -> assignableLoan\n" + 
		"		and DeliverableObligations -> consentRequiredLoan\n" + 
		"		and DeliverableObligations -> directLoanParticipation\n" + 
		"		and DeliverableObligations -> excluded\n" + 
		"		and DeliverableObligations -> fullFaithAndCreditObLiability\n" + 
		"		and DeliverableObligations -> generalFundObligationLiability\n" + 
		"		and DeliverableObligations -> indirectLoanParticipation\n" + 
		"		and DeliverableObligations -> listed\n" + 
		"		and DeliverableObligations -> maximumMaturity\n" + 
		"		and DeliverableObligations -> notBearer\n" + 
		"		and DeliverableObligations -> notContingent\n" + 
		"		and DeliverableObligations -> notDomesticCurrency\n" + 
		"		and DeliverableObligations -> notDomesticIssuance\n" + 
		"		and DeliverableObligations -> notDomesticLaw\n" + 
		"		and DeliverableObligations -> notSovereignLender\n" + 
		"		and DeliverableObligations -> notSubordinated\n" + 
		"		and DeliverableObligations -> othReferenceEntityObligations\n" + 
		"		and DeliverableObligations -> revenueObligationLiability\n" + 
		"		and DeliverableObligations -> specifiedCurrency\n" + 
		"		and DeliverableObligations -> transferable)\n" + 
		"	is absent";
	
	@Override
	public ValidationResult<DeliverableObligations> validate(RosettaPath path, DeliverableObligations deliverableObligations) {
		ComparisonResult result = executeDataRule(deliverableObligations);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "DeliverableObligations", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "DeliverableObligations", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<DeliverableObligations> validate(RosettaPath path, RosettaModelObjectBuilder deliverableObligations) {
		ComparisonResult result = executeDataRule((DeliverableObligations)deliverableObligations.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "DeliverableObligations", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "DeliverableObligations", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(DeliverableObligations deliverableObligations) {
		if (ruleIsApplicable(deliverableObligations).get()) {
			return evaluateThenExpression(deliverableObligations);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(DeliverableObligations deliverableObligations) {
		return exists(MapperS.of(deliverableObligations).<ObligationCategoryEnum>map("getCategory", DeliverableObligations::getCategory), false).and(areEqual(MapperS.of(deliverableObligations).<ObligationCategoryEnum>map("getCategory", DeliverableObligations::getCategory), MapperS.of(ObligationCategoryEnum.REFERENCE_OBLIGATIONS_ONLY)));
	}
	
	private ComparisonResult evaluateThenExpression(DeliverableObligations deliverableObligations) {
		return notExists(MapperTree.andDifferent(MapperTree.andDifferent(MapperTree.andDifferent(MapperTree.andDifferent(MapperTree.andDifferent(MapperTree.andDifferent(MapperTree.andDifferent(MapperTree.andDifferent(MapperTree.andDifferent(MapperTree.andDifferent(MapperTree.andDifferent(MapperTree.andDifferent(MapperTree.andDifferent(MapperTree.andDifferent(MapperTree.andDifferent(MapperTree.andDifferent(MapperTree.andDifferent(MapperTree.andDifferent(MapperTree.andDifferent(MapperTree.andDifferent(MapperTree.and(MapperTree.of(MapperS.of(deliverableObligations).<Boolean>map("getAcceleratedOrMatured", DeliverableObligations::getAcceleratedOrMatured)), MapperTree.of(MapperS.of(deliverableObligations).<Boolean>map("getAccruedInterest", DeliverableObligations::getAccruedInterest))), MapperTree.of(MapperS.of(deliverableObligations).<PCDeliverableObligationCharac>map("getAssignableLoan", DeliverableObligations::getAssignableLoan))), MapperTree.of(MapperS.of(deliverableObligations).<PCDeliverableObligationCharac>map("getConsentRequiredLoan", DeliverableObligations::getConsentRequiredLoan))), MapperTree.of(MapperS.of(deliverableObligations).<LoanParticipation>map("getDirectLoanParticipation", DeliverableObligations::getDirectLoanParticipation))), MapperTree.of(MapperS.of(deliverableObligations).<String>map("getExcluded", DeliverableObligations::getExcluded))), MapperTree.of(MapperS.of(deliverableObligations).<Boolean>map("getFullFaithAndCreditObLiability", DeliverableObligations::getFullFaithAndCreditObLiability))), MapperTree.of(MapperS.of(deliverableObligations).<Boolean>map("getGeneralFundObligationLiability", DeliverableObligations::getGeneralFundObligationLiability))), MapperTree.of(MapperS.of(deliverableObligations).<LoanParticipation>map("getIndirectLoanParticipation", DeliverableObligations::getIndirectLoanParticipation))), MapperTree.of(MapperS.of(deliverableObligations).<Boolean>map("getListed", DeliverableObligations::getListed))), MapperTree.of(MapperS.of(deliverableObligations).<Period>map("getMaximumMaturity", DeliverableObligations::getMaximumMaturity))), MapperTree.of(MapperS.of(deliverableObligations).<Boolean>map("getNotBearer", DeliverableObligations::getNotBearer))), MapperTree.of(MapperS.of(deliverableObligations).<Boolean>map("getNotContingent", DeliverableObligations::getNotContingent))), MapperTree.of(MapperS.of(deliverableObligations).<NotDomesticCurrency>map("getNotDomesticCurrency", DeliverableObligations::getNotDomesticCurrency))), MapperTree.of(MapperS.of(deliverableObligations).<Boolean>map("getNotDomesticIssuance", DeliverableObligations::getNotDomesticIssuance))), MapperTree.of(MapperS.of(deliverableObligations).<Boolean>map("getNotDomesticLaw", DeliverableObligations::getNotDomesticLaw))), MapperTree.of(MapperS.of(deliverableObligations).<Boolean>map("getNotSovereignLender", DeliverableObligations::getNotSovereignLender))), MapperTree.of(MapperS.of(deliverableObligations).<Boolean>map("getNotSubordinated", DeliverableObligations::getNotSubordinated))), MapperTree.of(MapperS.of(deliverableObligations).<String>map("getOthReferenceEntityObligations", DeliverableObligations::getOthReferenceEntityObligations))), MapperTree.of(MapperS.of(deliverableObligations).<Boolean>map("getRevenueObligationLiability", DeliverableObligations::getRevenueObligationLiability))), MapperTree.of(MapperS.of(deliverableObligations).<SpecifiedCurrency>map("getSpecifiedCurrency", DeliverableObligations::getSpecifiedCurrency))), MapperTree.of(MapperS.of(deliverableObligations).<Boolean>map("getTransferable", DeliverableObligations::getTransferable))));
	}
}
