package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.math.BigDecimal;
import org.isda.cdm.CashSettlementTerms;
import org.isda.cdm.Money;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("FpML_cd_37")
public class FpMLCd37DataRule implements Validator<CashSettlementTerms> {
	
	private static final String NAME = "FpML_cd_37";
	private static final String DEFINITION = "when (CashSettlementTerms -> quotationAmount and CashSettlementTerms -> minimumQuotationAmout) exists\n" + 
		"		and CashSettlementTerms -> quotationAmount -> currency = CashSettlementTerms -> minimumQuotationAmout -> currency\n" + 
		"then CashSettlementTerms -> quotationAmount -> amount > CashSettlementTerms -> minimumQuotationAmout -> amount";
	
	@Override
	public ValidationResult<CashSettlementTerms> validate(RosettaPath path, CashSettlementTerms cashSettlementTerms) {
		ComparisonResult result = executeDataRule(cashSettlementTerms);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "CashSettlementTerms", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "CashSettlementTerms", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<CashSettlementTerms> validate(RosettaPath path, RosettaModelObjectBuilder cashSettlementTerms) {
		ComparisonResult result = executeDataRule((CashSettlementTerms)cashSettlementTerms.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "CashSettlementTerms", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "CashSettlementTerms", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(CashSettlementTerms cashSettlementTerms) {
		if (ruleIsApplicable(cashSettlementTerms).get()) {
			return evaluateThenExpression(cashSettlementTerms);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(CashSettlementTerms cashSettlementTerms) {
		return exists(MapperTree.and(MapperTree.of(MapperS.of(cashSettlementTerms).<Money>map("getQuotationAmount", CashSettlementTerms::getQuotationAmount)), MapperTree.of(MapperS.of(cashSettlementTerms).<Money>map("getMinimumQuotationAmout", CashSettlementTerms::getMinimumQuotationAmout))), false).and(areEqual(MapperS.of(cashSettlementTerms).<Money>map("getQuotationAmount", CashSettlementTerms::getQuotationAmount).map("getCurrency", Money::getCurrency).<String>map("getValue", FieldWithMeta::getValue), MapperS.of(cashSettlementTerms).<Money>map("getMinimumQuotationAmout", CashSettlementTerms::getMinimumQuotationAmout).map("getCurrency", Money::getCurrency).<String>map("getValue", FieldWithMeta::getValue)));
	}
	
	private ComparisonResult evaluateThenExpression(CashSettlementTerms cashSettlementTerms) {
		return greaterThan(MapperS.of(cashSettlementTerms).<Money>map("getQuotationAmount", CashSettlementTerms::getQuotationAmount).<BigDecimal>map("getAmount", Money::getAmount), MapperS.of(cashSettlementTerms).<Money>map("getMinimumQuotationAmout", CashSettlementTerms::getMinimumQuotationAmout).<BigDecimal>map("getAmount", Money::getAmount));
	}
}
