package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.records.Date;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.AdjustableDate;
import org.isda.cdm.AdjustableOrRelativeDate;
import org.isda.cdm.BusinessDayAdjustments;
import org.isda.cdm.CalculationPeriodDates;
import org.isda.cdm.Contract;
import org.isda.cdm.ContractualMatrix;
import org.isda.cdm.ContractualProduct;
import org.isda.cdm.CreditDefaultPayout;
import org.isda.cdm.Documentation;
import org.isda.cdm.DocumentationIdentification;
import org.isda.cdm.EconomicTerms;
import org.isda.cdm.GeneralTerms;
import org.isda.cdm.InterestRatePayout;
import org.isda.cdm.MasterConfirmation;
import org.isda.cdm.Payout;
import org.isda.cdm.ReferenceInformation;
import org.isda.cdm.TradeDate;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("FpML_cd_7")
public class FpMLCd7DataRule implements Validator<Contract> {
	
	private static final String NAME = "FpML_cd_7";
	private static final String DEFINITION = "when ( Contract -> documentation -> documentationIdentification -> masterConfirmation and Contract -> documentation -> documentationIdentification -> contractualMatrix ) is absent\n" + 
		"		and Contract -> contractualProduct -> economicTerms -> payout -> creditDefaultPayout -> generalTerms -> referenceInformation exists\n" + 
		"then Contract -> contractualProduct -> economicTerms -> payout -> interestRatePayout -> calculationPeriodDates -> effectiveDate -> adjustableDate -> dateAdjustments exists\n" + 
		"		or Contract -> tradeDate -> date < Contract -> contractualProduct -> economicTerms -> effectiveDate -> adjustableDate -> adjustedDate";
	
	@Override
	public ValidationResult<Contract> validate(RosettaPath path, Contract contract) {
		ComparisonResult result = executeDataRule(contract);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "Contract", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "Contract", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<Contract> validate(RosettaPath path, RosettaModelObjectBuilder contract) {
		ComparisonResult result = executeDataRule((Contract)contract.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "Contract", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "Contract", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(Contract contract) {
		if (ruleIsApplicable(contract).get()) {
			return evaluateThenExpression(contract);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(Contract contract) {
		return notExists(MapperTree.andDifferent(MapperTree.of(MapperS.of(contract).<Documentation>map("getDocumentation", Contract::getDocumentation).<DocumentationIdentification>map("getDocumentationIdentification", Documentation::getDocumentationIdentification).<MasterConfirmation>map("getMasterConfirmation", DocumentationIdentification::getMasterConfirmation)), MapperTree.of(MapperS.of(contract).<Documentation>map("getDocumentation", Contract::getDocumentation).<DocumentationIdentification>map("getDocumentationIdentification", Documentation::getDocumentationIdentification).<ContractualMatrix>mapC("getContractualMatrix", DocumentationIdentification::getContractualMatrix)))).and(exists(MapperS.of(contract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<CreditDefaultPayout>map("getCreditDefaultPayout", Payout::getCreditDefaultPayout).<GeneralTerms>map("getGeneralTerms", CreditDefaultPayout::getGeneralTerms).<ReferenceInformation>map("getReferenceInformation", GeneralTerms::getReferenceInformation), false));
	}
	
	private ComparisonResult evaluateThenExpression(Contract contract) {
		return exists(MapperS.of(contract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<CalculationPeriodDates>map("getCalculationPeriodDates", InterestRatePayout::getCalculationPeriodDates).<AdjustableOrRelativeDate>map("getEffectiveDate", CalculationPeriodDates::getEffectiveDate).<AdjustableDate>map("getAdjustableDate", AdjustableOrRelativeDate::getAdjustableDate).<BusinessDayAdjustments>map("getDateAdjustments", AdjustableDate::getDateAdjustments), false).or(lessThan(MapperS.of(contract).<TradeDate>map("getTradeDate", Contract::getTradeDate).<Date>map("getDate", TradeDate::getDate), MapperS.of(contract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<AdjustableOrRelativeDate>map("getEffectiveDate", EconomicTerms::getEffectiveDate).<AdjustableDate>map("getAdjustableDate", AdjustableOrRelativeDate::getAdjustableDate).map("getAdjustedDate", AdjustableDate::getAdjustedDate).<Date>map("getValue", FieldWithMeta::getValue)));
	}
}
