package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.records.Date;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.AdjustableDate;
import org.isda.cdm.AdjustableOrRelativeDate;
import org.isda.cdm.CalculationPeriodDates;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("FpML_ird_20")
public class FpMLIrd20DataRule implements Validator<CalculationPeriodDates> {
	
	private static final String NAME = "FpML_ird_20";
	private static final String DEFINITION = "when CalculationPeriodDates -> lastRegularPeriodEndDate exists\n" + 
		"then CalculationPeriodDates -> lastRegularPeriodEndDate > CalculationPeriodDates -> effectiveDate -> adjustableDate -> unadjustedDate";
	
	@Override
	public ValidationResult<CalculationPeriodDates> validate(RosettaPath path, CalculationPeriodDates calculationPeriodDates) {
		ComparisonResult result = executeDataRule(calculationPeriodDates);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "CalculationPeriodDates", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "CalculationPeriodDates", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<CalculationPeriodDates> validate(RosettaPath path, RosettaModelObjectBuilder calculationPeriodDates) {
		ComparisonResult result = executeDataRule((CalculationPeriodDates)calculationPeriodDates.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "CalculationPeriodDates", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "CalculationPeriodDates", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(CalculationPeriodDates calculationPeriodDates) {
		if (ruleIsApplicable(calculationPeriodDates).get()) {
			return evaluateThenExpression(calculationPeriodDates);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(CalculationPeriodDates calculationPeriodDates) {
		return exists(MapperS.of(calculationPeriodDates).<Date>map("getLastRegularPeriodEndDate", CalculationPeriodDates::getLastRegularPeriodEndDate), false);
	}
	
	private ComparisonResult evaluateThenExpression(CalculationPeriodDates calculationPeriodDates) {
		return greaterThan(MapperS.of(calculationPeriodDates).<Date>map("getLastRegularPeriodEndDate", CalculationPeriodDates::getLastRegularPeriodEndDate), MapperS.of(calculationPeriodDates).<AdjustableOrRelativeDate>map("getEffectiveDate", CalculationPeriodDates::getEffectiveDate).<AdjustableDate>map("getAdjustableDate", AdjustableOrRelativeDate::getAdjustableDate).<Date>map("getUnadjustedDate", AdjustableDate::getUnadjustedDate));
	}
}
