package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.records.Date;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.LastRegularPaymentDate;
import org.isda.cdm.PaymentDates;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("FpML_ird_35_cd_31")
public class FpMLIrd35Cd31DataRule implements Validator<PaymentDates> {
	
	private static final String NAME = "FpML_ird_35_cd_31";
	private static final String DEFINITION = "when PaymentDates -> firstPaymentDate exists\n" + 
		"		and PaymentDates -> lastPaymentDate -> lastRegularPaymentDate exists\n" + 
		"then PaymentDates -> firstPaymentDate < PaymentDates -> lastPaymentDate -> lastRegularPaymentDate";
	
	@Override
	public ValidationResult<PaymentDates> validate(RosettaPath path, PaymentDates paymentDates) {
		ComparisonResult result = executeDataRule(paymentDates);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "PaymentDates", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "PaymentDates", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<PaymentDates> validate(RosettaPath path, RosettaModelObjectBuilder paymentDates) {
		ComparisonResult result = executeDataRule((PaymentDates)paymentDates.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "PaymentDates", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "PaymentDates", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(PaymentDates paymentDates) {
		if (ruleIsApplicable(paymentDates).get()) {
			return evaluateThenExpression(paymentDates);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(PaymentDates paymentDates) {
		return exists(MapperS.of(paymentDates).<Date>map("getFirstPaymentDate", PaymentDates::getFirstPaymentDate), false).and(exists(MapperS.of(paymentDates).<LastRegularPaymentDate>map("getLastPaymentDate", PaymentDates::getLastPaymentDate).<Date>map("getLastRegularPaymentDate", LastRegularPaymentDate::getLastRegularPaymentDate), false));
	}
	
	private ComparisonResult evaluateThenExpression(PaymentDates paymentDates) {
		return lessThan(MapperS.of(paymentDates).<Date>map("getFirstPaymentDate", PaymentDates::getFirstPaymentDate), MapperS.of(paymentDates).<LastRegularPaymentDate>map("getLastPaymentDate", PaymentDates::getLastPaymentDate).<Date>map("getLastRegularPaymentDate", LastRegularPaymentDate::getLastRegularPaymentDate));
	}
}
