package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.records.Date;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.MandatoryEarlyTerminationAdjustedDates;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("FpML_ird_44")
public class FpMLIrd44DataRule implements Validator<MandatoryEarlyTerminationAdjustedDates> {
	
	private static final String NAME = "FpML_ird_44";
	private static final String DEFINITION = "when MandatoryEarlyTerminationAdjustedDates exists\n" + 
		"then MandatoryEarlyTerminationAdjustedDates -> adjustedEarlyTerminationDate <= MandatoryEarlyTerminationAdjustedDates -> adjustedCashSettlementValuationDate\n" + 
		"		and MandatoryEarlyTerminationAdjustedDates -> adjustedCashSettlementValuationDate <= MandatoryEarlyTerminationAdjustedDates -> adjustedCashSettlementPaymentDate";
	
	@Override
	public ValidationResult<MandatoryEarlyTerminationAdjustedDates> validate(RosettaPath path, MandatoryEarlyTerminationAdjustedDates mandatoryEarlyTerminationAdjustedDates) {
		ComparisonResult result = executeDataRule(mandatoryEarlyTerminationAdjustedDates);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "MandatoryEarlyTerminationAdjustedDates", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "MandatoryEarlyTerminationAdjustedDates", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<MandatoryEarlyTerminationAdjustedDates> validate(RosettaPath path, RosettaModelObjectBuilder mandatoryEarlyTerminationAdjustedDates) {
		ComparisonResult result = executeDataRule((MandatoryEarlyTerminationAdjustedDates)mandatoryEarlyTerminationAdjustedDates.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "MandatoryEarlyTerminationAdjustedDates", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "MandatoryEarlyTerminationAdjustedDates", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(MandatoryEarlyTerminationAdjustedDates mandatoryEarlyTerminationAdjustedDates) {
		if (ruleIsApplicable(mandatoryEarlyTerminationAdjustedDates).get()) {
			return evaluateThenExpression(mandatoryEarlyTerminationAdjustedDates);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(MandatoryEarlyTerminationAdjustedDates mandatoryEarlyTerminationAdjustedDates) {
		return exists(MapperS.of(mandatoryEarlyTerminationAdjustedDates), false);
	}
	
	private ComparisonResult evaluateThenExpression(MandatoryEarlyTerminationAdjustedDates mandatoryEarlyTerminationAdjustedDates) {
		return lessThanEquals(MapperS.of(mandatoryEarlyTerminationAdjustedDates).<Date>map("getAdjustedEarlyTerminationDate", MandatoryEarlyTerminationAdjustedDates::getAdjustedEarlyTerminationDate), MapperS.of(mandatoryEarlyTerminationAdjustedDates).<Date>map("getAdjustedCashSettlementValuationDate", MandatoryEarlyTerminationAdjustedDates::getAdjustedCashSettlementValuationDate)).and(lessThanEquals(MapperS.of(mandatoryEarlyTerminationAdjustedDates).<Date>map("getAdjustedCashSettlementValuationDate", MandatoryEarlyTerminationAdjustedDates::getAdjustedCashSettlementValuationDate), MapperS.of(mandatoryEarlyTerminationAdjustedDates).<Date>map("getAdjustedCashSettlementPaymentDate", MandatoryEarlyTerminationAdjustedDates::getAdjustedCashSettlementPaymentDate)));
	}
}
