package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.CalculationPeriodDates;
import org.isda.cdm.CalculationPeriodFrequency;
import org.isda.cdm.CompoundingMethodEnum;
import org.isda.cdm.Frequency;
import org.isda.cdm.InterestRatePayout;
import org.isda.cdm.PaymentDates;
import org.isda.cdm.PeriodExtendedEnum;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("FpML_ird_7_2")
public class FpMLIrd72DataRule implements Validator<InterestRatePayout> {
	
	private static final String NAME = "FpML_ird_7_2";
	private static final String DEFINITION = "when ( ( InterestRatePayout -> paymentDates -> paymentFrequency -> period and InterestRatePayout -> calculationPeriodDates -> calculationPeriodFrequency -> period ) exists\n" + 
		" 			and InterestRatePayout -> paymentDates -> paymentFrequency -> period <> InterestRatePayout -> calculationPeriodDates -> calculationPeriodFrequency -> period )\n" + 
		"		or ( ( InterestRatePayout -> paymentDates -> paymentFrequency -> periodMultiplier and InterestRatePayout -> calculationPeriodDates -> calculationPeriodFrequency -> periodMultiplier ) exists\n" + 
		" 			and InterestRatePayout -> paymentDates -> paymentFrequency -> periodMultiplier <> InterestRatePayout -> calculationPeriodDates -> calculationPeriodFrequency -> periodMultiplier )\n" + 
		"then InterestRatePayout -> compoundingMethod exists";
	
	@Override
	public ValidationResult<InterestRatePayout> validate(RosettaPath path, InterestRatePayout interestRatePayout) {
		ComparisonResult result = executeDataRule(interestRatePayout);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "InterestRatePayout", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "InterestRatePayout", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<InterestRatePayout> validate(RosettaPath path, RosettaModelObjectBuilder interestRatePayout) {
		ComparisonResult result = executeDataRule((InterestRatePayout)interestRatePayout.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "InterestRatePayout", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "InterestRatePayout", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(InterestRatePayout interestRatePayout) {
		if (ruleIsApplicable(interestRatePayout).get()) {
			return evaluateThenExpression(interestRatePayout);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(InterestRatePayout interestRatePayout) {
		return exists(MapperTree.and(MapperTree.of(MapperS.of(interestRatePayout).<PaymentDates>map("getPaymentDates", InterestRatePayout::getPaymentDates).<Frequency>map("getPaymentFrequency", PaymentDates::getPaymentFrequency).<PeriodExtendedEnum>map("getPeriod", Frequency::getPeriod)), MapperTree.of(MapperS.of(interestRatePayout).<CalculationPeriodDates>map("getCalculationPeriodDates", InterestRatePayout::getCalculationPeriodDates).<CalculationPeriodFrequency>map("getCalculationPeriodFrequency", CalculationPeriodDates::getCalculationPeriodFrequency).<PeriodExtendedEnum>map("getPeriod", Frequency::getPeriod))), false).and(notEqual(MapperS.of(interestRatePayout).<PaymentDates>map("getPaymentDates", InterestRatePayout::getPaymentDates).<Frequency>map("getPaymentFrequency", PaymentDates::getPaymentFrequency).<PeriodExtendedEnum>map("getPeriod", Frequency::getPeriod), MapperS.of(interestRatePayout).<CalculationPeriodDates>map("getCalculationPeriodDates", InterestRatePayout::getCalculationPeriodDates).<CalculationPeriodFrequency>map("getCalculationPeriodFrequency", CalculationPeriodDates::getCalculationPeriodFrequency).<PeriodExtendedEnum>map("getPeriod", Frequency::getPeriod))).or(exists(MapperTree.and(MapperTree.of(MapperS.of(interestRatePayout).<PaymentDates>map("getPaymentDates", InterestRatePayout::getPaymentDates).<Frequency>map("getPaymentFrequency", PaymentDates::getPaymentFrequency).<Integer>map("getPeriodMultiplier", Frequency::getPeriodMultiplier)), MapperTree.of(MapperS.of(interestRatePayout).<CalculationPeriodDates>map("getCalculationPeriodDates", InterestRatePayout::getCalculationPeriodDates).<CalculationPeriodFrequency>map("getCalculationPeriodFrequency", CalculationPeriodDates::getCalculationPeriodFrequency).<Integer>map("getPeriodMultiplier", Frequency::getPeriodMultiplier))), false).and(notEqual(MapperS.of(interestRatePayout).<PaymentDates>map("getPaymentDates", InterestRatePayout::getPaymentDates).<Frequency>map("getPaymentFrequency", PaymentDates::getPaymentFrequency).<Integer>map("getPeriodMultiplier", Frequency::getPeriodMultiplier), MapperS.of(interestRatePayout).<CalculationPeriodDates>map("getCalculationPeriodDates", InterestRatePayout::getCalculationPeriodDates).<CalculationPeriodFrequency>map("getCalculationPeriodFrequency", CalculationPeriodDates::getCalculationPeriodFrequency).<Integer>map("getPeriodMultiplier", Frequency::getPeriodMultiplier))));
	}
	
	private ComparisonResult evaluateThenExpression(InterestRatePayout interestRatePayout) {
		return exists(MapperS.of(interestRatePayout).<CompoundingMethodEnum>map("getCompoundingMethod", InterestRatePayout::getCompoundingMethod), false);
	}
}
