package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.Account;
import org.isda.cdm.InterestRatePayout;
import org.isda.cdm.Party;
import org.isda.cdm.PayerReceiver;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("FpML_ird_8")
public class FpMLIrd8DataRule implements Validator<InterestRatePayout> {
	
	private static final String NAME = "FpML_ird_8";
	private static final String DEFINITION = "when InterestRatePayout exists\n" + 
		"then InterestRatePayout -> payerReceiver -> payerPartyReference <> InterestRatePayout -> payerReceiver -> receiverPartyReference\n" + 
		"		or InterestRatePayout -> payerReceiver -> payerAccountReference <> InterestRatePayout -> payerReceiver -> receiverAccountReference";
	
	@Override
	public ValidationResult<InterestRatePayout> validate(RosettaPath path, InterestRatePayout interestRatePayout) {
		ComparisonResult result = executeDataRule(interestRatePayout);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "InterestRatePayout", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "InterestRatePayout", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<InterestRatePayout> validate(RosettaPath path, RosettaModelObjectBuilder interestRatePayout) {
		ComparisonResult result = executeDataRule((InterestRatePayout)interestRatePayout.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "InterestRatePayout", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "InterestRatePayout", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(InterestRatePayout interestRatePayout) {
		if (ruleIsApplicable(interestRatePayout).get()) {
			return evaluateThenExpression(interestRatePayout);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(InterestRatePayout interestRatePayout) {
		return exists(MapperS.of(interestRatePayout), false);
	}
	
	private ComparisonResult evaluateThenExpression(InterestRatePayout interestRatePayout) {
		return notEqual(MapperS.of(interestRatePayout).<PayerReceiver>map("getPayerReceiver", InterestRatePayout::getPayerReceiver).map("getPayerPartyReference", PayerReceiver::getPayerPartyReference).<Party>map("getValue", FieldWithMeta::getValue), MapperS.of(interestRatePayout).<PayerReceiver>map("getPayerReceiver", InterestRatePayout::getPayerReceiver).map("getReceiverPartyReference", PayerReceiver::getReceiverPartyReference).<Party>map("getValue", FieldWithMeta::getValue)).or(notEqual(MapperS.of(interestRatePayout).<PayerReceiver>map("getPayerReceiver", InterestRatePayout::getPayerReceiver).map("getPayerAccountReference", PayerReceiver::getPayerAccountReference).<Account>map("getValue", FieldWithMeta::getValue), MapperS.of(interestRatePayout).<PayerReceiver>map("getPayerReceiver", InterestRatePayout::getPayerReceiver).map("getReceiverAccountReference", PayerReceiver::getReceiverAccountReference).<Account>map("getValue", FieldWithMeta::getValue)));
	}
}
