package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.records.Date;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.AdjustableDate;
import org.isda.cdm.AdjustableOrRelativeDate;
import org.isda.cdm.CalculationPeriodDates;
import org.isda.cdm.ContractualQuantity;
import org.isda.cdm.FutureValueAmount;
import org.isda.cdm.InterestRatePayout;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("FutureValueNotional_terminationDate")
public class FutureValueNotionalTerminationDateDataRule implements Validator<InterestRatePayout> {
	
	private static final String NAME = "FutureValueNotional_terminationDate";
	private static final String DEFINITION = "when InterestRatePayout -> quantity -> futureValueNotional exists\n" + 
		"then InterestRatePayout -> quantity -> futureValueNotional -> valueDate = InterestRatePayout -> calculationPeriodDates -> terminationDate -> adjustableDate -> adjustedDate";
	
	@Override
	public ValidationResult<InterestRatePayout> validate(RosettaPath path, InterestRatePayout interestRatePayout) {
		ComparisonResult result = executeDataRule(interestRatePayout);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "InterestRatePayout", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "InterestRatePayout", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<InterestRatePayout> validate(RosettaPath path, RosettaModelObjectBuilder interestRatePayout) {
		ComparisonResult result = executeDataRule((InterestRatePayout)interestRatePayout.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "InterestRatePayout", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "InterestRatePayout", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(InterestRatePayout interestRatePayout) {
		if (ruleIsApplicable(interestRatePayout).get()) {
			return evaluateThenExpression(interestRatePayout);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(InterestRatePayout interestRatePayout) {
		return exists(MapperS.of(interestRatePayout).<ContractualQuantity>map("getQuantity", InterestRatePayout::getQuantity).<FutureValueAmount>map("getFutureValueNotional", ContractualQuantity::getFutureValueNotional), false);
	}
	
	private ComparisonResult evaluateThenExpression(InterestRatePayout interestRatePayout) {
		return areEqual(MapperS.of(interestRatePayout).<ContractualQuantity>map("getQuantity", InterestRatePayout::getQuantity).<FutureValueAmount>map("getFutureValueNotional", ContractualQuantity::getFutureValueNotional).<Date>map("getValueDate", FutureValueAmount::getValueDate), MapperS.of(interestRatePayout).<CalculationPeriodDates>map("getCalculationPeriodDates", InterestRatePayout::getCalculationPeriodDates).<AdjustableOrRelativeDate>map("getTerminationDate", CalculationPeriodDates::getTerminationDate).<AdjustableDate>map("getAdjustableDate", AdjustableOrRelativeDate::getAdjustableDate).map("getAdjustedDate", AdjustableDate::getAdjustedDate).<Date>map("getValue", FieldWithMeta::getValue));
	}
}
