package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.ForeignExchange;
import org.isda.cdm.ForwardPayout;
import org.isda.cdm.OptionCashSettlement;
import org.isda.cdm.OptionPhysicalSettlement;
import org.isda.cdm.OptionSettlement;
import org.isda.cdm.Product;
import org.isda.cdm.SingleUnderlier;
import org.isda.cdm.Underlier;

import static com.rosetta.model.lib.validation.ValidatorHelper.exists;
import static com.rosetta.model.lib.validation.ValidatorHelper.notExists;

/**
 * @version 2.5.4
 */
@RosettaDataRule("FxForward_fxSettlement")
public class FxForwardFxSettlementDataRule implements Validator<ForwardPayout> {
	
	private static final String NAME = "FxForward_fxSettlement";
	private static final String DEFINITION = "if forwardFX exists then ( ForwardPayout -> settlementTerms -> cashSettlementTerms is absent and ForwardPayout -> settlementTerms -> physicalSettlementTerms is absent )";
	
	@Override
	public ValidationResult<ForwardPayout> validate(RosettaPath path, ForwardPayout forwardPayout) {
		ComparisonResult result = executeDataRule(forwardPayout);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "ForwardPayout", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "ForwardPayout", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<ForwardPayout> validate(RosettaPath path, RosettaModelObjectBuilder forwardPayout) {
		ComparisonResult result = executeDataRule((ForwardPayout)forwardPayout.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationType.DATA_RULE, "ForwardPayout", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationType.DATA_RULE,  "ForwardPayout", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(ForwardPayout forwardPayout) {
		if (ruleIsApplicable(forwardPayout).get()) {
			return evaluateThenExpression(forwardPayout);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(ForwardPayout forwardPayout) {
		return exists(MapperS.of(forwardPayout).<Underlier>map("getUnderlier", ForwardPayout::getUnderlier).<SingleUnderlier>map("getSingleUnderlier", Underlier::getSingleUnderlier).<Product>map("getUnderlyingProduct", SingleUnderlier::getUnderlyingProduct).<ForeignExchange>map("getForeignExchange", Product::getForeignExchange), false);
	}
	
	private ComparisonResult evaluateThenExpression(ForwardPayout forwardPayout) {
		return notExists(MapperS.of(forwardPayout).<OptionSettlement>map("getSettlementTerms", ForwardPayout::getSettlementTerms).<OptionCashSettlement>map("getCashSettlementTerms", OptionSettlement::getCashSettlementTerms)).and(notExists(MapperS.of(forwardPayout).<OptionSettlement>map("getSettlementTerms", ForwardPayout::getSettlementTerms).<OptionPhysicalSettlement>map("getPhysicalSettlementTerms", OptionSettlement::getPhysicalSettlementTerms)));
	}
}
