package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.records.Date;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.AdjustableOrAdjustedOrRelativeDate;
import org.isda.cdm.Cashflow;
import org.isda.cdm.ForeignExchange;
import org.isda.cdm.ForwardPayout;
import org.isda.cdm.OptionSettlement;
import org.isda.cdm.Product;
import org.isda.cdm.SettlementTerms;
import org.isda.cdm.SingleUnderlier;
import org.isda.cdm.Underlier;

import static com.rosetta.model.lib.validation.ValidatorHelper.areEqual;
import static com.rosetta.model.lib.validation.ValidatorHelper.exists;
import static com.rosetta.model.lib.validation.ValidatorHelper.notExists;

/**
 * @version 2.5.4
 */
@RosettaDataRule("FxForward_settlementDate")
public class FxForwardSettlementDateDataRule implements Validator<ForwardPayout> {
	
	private static final String NAME = "FxForward_settlementDate";
	private static final String DEFINITION = "if forwardFX exists then ( ForwardPayout -> settlementTerms -> valueDate exists and forwardFX -> exchangedCurrency1 -> cashflowDate is absent and forwardFX -> exchangedCurrency2 -> cashflowDate is absent ) or ( ForwardPayout -> settlementTerms -> valueDate is absent and forwardFX -> exchangedCurrency1 -> cashflowDate exists and forwardFX -> exchangedCurrency2 -> cashflowDate exists and forwardFX -> exchangedCurrency1 -> cashflowDate = forwardFX -> exchangedCurrency2 -> cashflowDate )";
	
	@Override
	public ValidationResult<ForwardPayout> validate(RosettaPath path, ForwardPayout forwardPayout) {
		ComparisonResult result = executeDataRule(forwardPayout);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "ForwardPayout", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "ForwardPayout", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<ForwardPayout> validate(RosettaPath path, RosettaModelObjectBuilder forwardPayout) {
		ComparisonResult result = executeDataRule((ForwardPayout)forwardPayout.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationType.DATA_RULE, "ForwardPayout", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationType.DATA_RULE,  "ForwardPayout", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(ForwardPayout forwardPayout) {
		if (ruleIsApplicable(forwardPayout).get()) {
			return evaluateThenExpression(forwardPayout);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(ForwardPayout forwardPayout) {
		return exists(MapperS.of(forwardPayout).<Underlier>map("getUnderlier", ForwardPayout::getUnderlier).<SingleUnderlier>map("getSingleUnderlier", Underlier::getSingleUnderlier).<Product>map("getUnderlyingProduct", SingleUnderlier::getUnderlyingProduct).<ForeignExchange>map("getForeignExchange", Product::getForeignExchange), false);
	}
	
	private ComparisonResult evaluateThenExpression(ForwardPayout forwardPayout) {
		return exists(MapperS.of(forwardPayout).<OptionSettlement>map("getSettlementTerms", ForwardPayout::getSettlementTerms).<Date>map("getValueDate", SettlementTerms::getValueDate), false).and(notExists(MapperS.of(forwardPayout).<Underlier>map("getUnderlier", ForwardPayout::getUnderlier).<SingleUnderlier>map("getSingleUnderlier", Underlier::getSingleUnderlier).<Product>map("getUnderlyingProduct", SingleUnderlier::getUnderlyingProduct).<ForeignExchange>map("getForeignExchange", Product::getForeignExchange).<Cashflow>map("getExchangedCurrency1", ForeignExchange::getExchangedCurrency1).<AdjustableOrAdjustedOrRelativeDate>map("getCashflowDate", Cashflow::getCashflowDate))).and(notExists(MapperS.of(forwardPayout).<Underlier>map("getUnderlier", ForwardPayout::getUnderlier).<SingleUnderlier>map("getSingleUnderlier", Underlier::getSingleUnderlier).<Product>map("getUnderlyingProduct", SingleUnderlier::getUnderlyingProduct).<ForeignExchange>map("getForeignExchange", Product::getForeignExchange).<Cashflow>map("getExchangedCurrency2", ForeignExchange::getExchangedCurrency2).<AdjustableOrAdjustedOrRelativeDate>map("getCashflowDate", Cashflow::getCashflowDate))).or(notExists(MapperS.of(forwardPayout).<OptionSettlement>map("getSettlementTerms", ForwardPayout::getSettlementTerms).<Date>map("getValueDate", SettlementTerms::getValueDate)).and(exists(MapperS.of(forwardPayout).<Underlier>map("getUnderlier", ForwardPayout::getUnderlier).<SingleUnderlier>map("getSingleUnderlier", Underlier::getSingleUnderlier).<Product>map("getUnderlyingProduct", SingleUnderlier::getUnderlyingProduct).<ForeignExchange>map("getForeignExchange", Product::getForeignExchange).<Cashflow>map("getExchangedCurrency1", ForeignExchange::getExchangedCurrency1).<AdjustableOrAdjustedOrRelativeDate>map("getCashflowDate", Cashflow::getCashflowDate), false)).and(exists(MapperS.of(forwardPayout).<Underlier>map("getUnderlier", ForwardPayout::getUnderlier).<SingleUnderlier>map("getSingleUnderlier", Underlier::getSingleUnderlier).<Product>map("getUnderlyingProduct", SingleUnderlier::getUnderlyingProduct).<ForeignExchange>map("getForeignExchange", Product::getForeignExchange).<Cashflow>map("getExchangedCurrency2", ForeignExchange::getExchangedCurrency2).<AdjustableOrAdjustedOrRelativeDate>map("getCashflowDate", Cashflow::getCashflowDate), false)).and(areEqual(MapperS.of(forwardPayout).<Underlier>map("getUnderlier", ForwardPayout::getUnderlier).<SingleUnderlier>map("getSingleUnderlier", Underlier::getSingleUnderlier).<Product>map("getUnderlyingProduct", SingleUnderlier::getUnderlyingProduct).<ForeignExchange>map("getForeignExchange", Product::getForeignExchange).<Cashflow>map("getExchangedCurrency1", ForeignExchange::getExchangedCurrency1).<AdjustableOrAdjustedOrRelativeDate>map("getCashflowDate", Cashflow::getCashflowDate), MapperS.of(forwardPayout).<Underlier>map("getUnderlier", ForwardPayout::getUnderlier).<SingleUnderlier>map("getSingleUnderlier", Underlier::getSingleUnderlier).<Product>map("getUnderlyingProduct", SingleUnderlier::getUnderlyingProduct).<ForeignExchange>map("getForeignExchange", Product::getForeignExchange).<Cashflow>map("getExchangedCurrency2", ForeignExchange::getExchangedCurrency2).<AdjustableOrAdjustedOrRelativeDate>map("getCashflowDate", Cashflow::getCashflowDate))));
	}
}
