package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.ContractualQuantity;
import org.isda.cdm.CreditDefaultPayout;
import org.isda.cdm.InterestRatePayout;
import org.isda.cdm.OptionPayout;
import org.isda.cdm.Payout;
import org.isda.cdm.PayoutBase;
import org.isda.cdm.ResolvablePayoutQuantity;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("InterestRatePayout_quantity")
public class InterestRatePayoutQuantityDataRule implements Validator<Payout> {
	
	private static final String NAME = "InterestRatePayout_quantity";
	private static final String DEFINITION = "when Payout -> interestRatePayout exists\n" + 
		"		and Payout -> creditDefaultPayout is absent\n" + 
		"		and Payout -> optionPayout is absent\n" + 
		"then ( Payout -> interestRatePayout -> quantity exists or Payout -> interestRatePayout -> payoutQuantity exists )";
	
	@Override
	public ValidationResult<Payout> validate(RosettaPath path, Payout payout) {
		ComparisonResult result = executeDataRule(payout);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "Payout", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "Payout", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<Payout> validate(RosettaPath path, RosettaModelObjectBuilder payout) {
		ComparisonResult result = executeDataRule((Payout)payout.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "Payout", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "Payout", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(Payout payout) {
		if (ruleIsApplicable(payout).get()) {
			return evaluateThenExpression(payout);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(Payout payout) {
		return exists(MapperS.of(payout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout), false).and(notExists(MapperS.of(payout).<CreditDefaultPayout>map("getCreditDefaultPayout", Payout::getCreditDefaultPayout))).and(notExists(MapperS.of(payout).<OptionPayout>mapC("getOptionPayout", Payout::getOptionPayout)));
	}
	
	private ComparisonResult evaluateThenExpression(Payout payout) {
		return exists(MapperS.of(payout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<ContractualQuantity>map("getQuantity", InterestRatePayout::getQuantity), false).or(exists(MapperS.of(payout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<ResolvablePayoutQuantity>map("getPayoutQuantity", PayoutBase::getPayoutQuantity), false));
	}
}
