package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.Frequency;
import org.isda.cdm.PeriodExtendedEnum;
import org.isda.cdm.ResetDates;
import org.isda.cdm.ResetFrequency;
import org.isda.cdm.WeeklyRollConventionEnum;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("ResetFrequency_weekly")
public class ResetFrequencyWeeklyDataRule implements Validator<ResetDates> {
	
	private static final String NAME = "ResetFrequency_weekly";
	private static final String DEFINITION = "when ResetDates -> resetFrequency -> period = PeriodExtendedEnum.W\n" + 
		"then ResetDates -> resetFrequency -> weeklyRollConvention exists";
	
	@Override
	public ValidationResult<ResetDates> validate(RosettaPath path, ResetDates resetDates) {
		ComparisonResult result = executeDataRule(resetDates);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "ResetDates", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "ResetDates", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<ResetDates> validate(RosettaPath path, RosettaModelObjectBuilder resetDates) {
		ComparisonResult result = executeDataRule((ResetDates)resetDates.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "ResetDates", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "ResetDates", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(ResetDates resetDates) {
		if (ruleIsApplicable(resetDates).get()) {
			return evaluateThenExpression(resetDates);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(ResetDates resetDates) {
		return areEqual(MapperS.of(resetDates).<ResetFrequency>map("getResetFrequency", ResetDates::getResetFrequency).<PeriodExtendedEnum>map("getPeriod", Frequency::getPeriod), MapperS.of(PeriodExtendedEnum.W));
	}
	
	private ComparisonResult evaluateThenExpression(ResetDates resetDates) {
		return exists(MapperS.of(resetDates).<ResetFrequency>map("getResetFrequency", ResetDates::getResetFrequency).<WeeklyRollConventionEnum>map("getWeeklyRollConvention", ResetFrequency::getWeeklyRollConvention), false);
	}
}
