package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.Contract;
import org.isda.cdm.ContractualMatrix;
import org.isda.cdm.ContractualProduct;
import org.isda.cdm.CreditDefaultPayout;
import org.isda.cdm.CreditEvents;
import org.isda.cdm.Documentation;
import org.isda.cdm.DocumentationIdentification;
import org.isda.cdm.EconomicTerms;
import org.isda.cdm.MatrixTypeEnum;
import org.isda.cdm.Payout;
import org.isda.cdm.ProtectionTerms;
import org.isda.cdm.Restructuring;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("Restructuring_physicalSettlementMatrix")
public class RestructuringPhysicalSettlementMatrixDataRule implements Validator<Contract> {
	
	private static final String NAME = "Restructuring_physicalSettlementMatrix";
	private static final String DEFINITION = "when ( Contract -> documentation -> documentationIdentification -> contractualMatrix -> matrixType <> MatrixTypeEnum.CreditDerivativesPhysicalSettlementMatrix\n" + 
		"		or Contract -> documentation -> documentationIdentification -> contractualMatrix -> matrixType is absent )\n" + 
		"		and Contract -> contractualProduct -> economicTerms -> payout -> creditDefaultPayout -> protectionTerms -> creditEvents -> restructuring exists\n" + 
		"then Contract -> contractualProduct -> economicTerms -> payout -> creditDefaultPayout -> protectionTerms -> creditEvents -> restructuring -> multipleHolderObligation exists";
	
	@Override
	public ValidationResult<Contract> validate(RosettaPath path, Contract contract) {
		ComparisonResult result = executeDataRule(contract);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "Contract", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "Contract", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<Contract> validate(RosettaPath path, RosettaModelObjectBuilder contract) {
		ComparisonResult result = executeDataRule((Contract)contract.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "Contract", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "Contract", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(Contract contract) {
		if (ruleIsApplicable(contract).get()) {
			return evaluateThenExpression(contract);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(Contract contract) {
		return notEqual(MapperS.of(contract).<Documentation>map("getDocumentation", Contract::getDocumentation).<DocumentationIdentification>map("getDocumentationIdentification", Documentation::getDocumentationIdentification).<ContractualMatrix>mapC("getContractualMatrix", DocumentationIdentification::getContractualMatrix).map("getMatrixType", ContractualMatrix::getMatrixType).<MatrixTypeEnum>map("getValue", FieldWithMeta::getValue), MapperS.of(MatrixTypeEnum.CREDIT_DERIVATIVES_PHYSICAL_SETTLEMENT_MATRIX)).or(notExists(MapperS.of(contract).<Documentation>map("getDocumentation", Contract::getDocumentation).<DocumentationIdentification>map("getDocumentationIdentification", Documentation::getDocumentationIdentification).<ContractualMatrix>mapC("getContractualMatrix", DocumentationIdentification::getContractualMatrix).map("getMatrixType", ContractualMatrix::getMatrixType).<MatrixTypeEnum>map("getValue", FieldWithMeta::getValue))).and(exists(MapperS.of(contract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<CreditDefaultPayout>map("getCreditDefaultPayout", Payout::getCreditDefaultPayout).<ProtectionTerms>mapC("getProtectionTerms", CreditDefaultPayout::getProtectionTerms).<CreditEvents>map("getCreditEvents", ProtectionTerms::getCreditEvents).<Restructuring>map("getRestructuring", CreditEvents::getRestructuring), false));
	}
	
	private ComparisonResult evaluateThenExpression(Contract contract) {
		return exists(MapperS.of(contract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<CreditDefaultPayout>map("getCreditDefaultPayout", Payout::getCreditDefaultPayout).<ProtectionTerms>mapC("getProtectionTerms", CreditDefaultPayout::getProtectionTerms).<CreditEvents>map("getCreditEvents", ProtectionTerms::getCreditEvents).<Restructuring>map("getRestructuring", CreditEvents::getRestructuring).<Boolean>map("getMultipleHolderObligation", Restructuring::getMultipleHolderObligation), false);
	}
}
