package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.ContractualQuantity;
import org.isda.cdm.CrossCurrencyTerms;
import org.isda.cdm.FxLinkedNotionalSchedule;
import org.isda.cdm.InterestRatePayout;
import org.isda.cdm.Money;
import org.isda.cdm.NonDeliverableSettlement;
import org.isda.cdm.NonNegativeAmountSchedule;
import org.isda.cdm.NotionalSchedule;
import org.isda.cdm.SettlementProvision;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("SettlementProvision_settlementCurrency")
public class SettlementProvisionSettlementCurrencyDataRule implements Validator<InterestRatePayout> {
	
	private static final String NAME = "SettlementProvision_settlementCurrency";
	private static final String DEFINITION = "when InterestRatePayout -> crossCurrencyTerms -> settlementProvision -> nonDeliverableSettlement exists\n" + 
		"then InterestRatePayout -> crossCurrencyTerms -> settlementProvision -> settlementCurrency <> InterestRatePayout -> quantity -> notionalAmount -> currency\n" + 
		"		or InterestRatePayout -> crossCurrencyTerms -> settlementProvision -> settlementCurrency <> InterestRatePayout -> quantity -> notionalSchedule -> notionalStepSchedule -> currency\n" + 
		"		or InterestRatePayout -> crossCurrencyTerms -> settlementProvision -> settlementCurrency <> InterestRatePayout -> quantity -> fxLinkedNotional -> varyingNotionalCurrency";
	
	@Override
	public ValidationResult<InterestRatePayout> validate(RosettaPath path, InterestRatePayout interestRatePayout) {
		ComparisonResult result = executeDataRule(interestRatePayout);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "InterestRatePayout", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "InterestRatePayout", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<InterestRatePayout> validate(RosettaPath path, RosettaModelObjectBuilder interestRatePayout) {
		ComparisonResult result = executeDataRule((InterestRatePayout)interestRatePayout.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "InterestRatePayout", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "InterestRatePayout", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(InterestRatePayout interestRatePayout) {
		if (ruleIsApplicable(interestRatePayout).get()) {
			return evaluateThenExpression(interestRatePayout);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(InterestRatePayout interestRatePayout) {
		return exists(MapperS.of(interestRatePayout).<CrossCurrencyTerms>map("getCrossCurrencyTerms", InterestRatePayout::getCrossCurrencyTerms).<SettlementProvision>map("getSettlementProvision", CrossCurrencyTerms::getSettlementProvision).<NonDeliverableSettlement>map("getNonDeliverableSettlement", SettlementProvision::getNonDeliverableSettlement), false);
	}
	
	private ComparisonResult evaluateThenExpression(InterestRatePayout interestRatePayout) {
		return notEqual(MapperS.of(interestRatePayout).<CrossCurrencyTerms>map("getCrossCurrencyTerms", InterestRatePayout::getCrossCurrencyTerms).<SettlementProvision>map("getSettlementProvision", CrossCurrencyTerms::getSettlementProvision).map("getSettlementCurrency", SettlementProvision::getSettlementCurrency).<String>map("getValue", FieldWithMeta::getValue), MapperS.of(interestRatePayout).<ContractualQuantity>map("getQuantity", InterestRatePayout::getQuantity).map("getNotionalAmount", ContractualQuantity::getNotionalAmount).<Money>map("getValue", FieldWithMeta::getValue).map("getCurrency", Money::getCurrency).<String>map("getValue", FieldWithMeta::getValue)).or(notEqual(MapperS.of(interestRatePayout).<CrossCurrencyTerms>map("getCrossCurrencyTerms", InterestRatePayout::getCrossCurrencyTerms).<SettlementProvision>map("getSettlementProvision", CrossCurrencyTerms::getSettlementProvision).map("getSettlementCurrency", SettlementProvision::getSettlementCurrency).<String>map("getValue", FieldWithMeta::getValue), MapperS.of(interestRatePayout).<ContractualQuantity>map("getQuantity", InterestRatePayout::getQuantity).<NotionalSchedule>map("getNotionalSchedule", ContractualQuantity::getNotionalSchedule).<NonNegativeAmountSchedule>map("getNotionalStepSchedule", NotionalSchedule::getNotionalStepSchedule).map("getCurrency", NonNegativeAmountSchedule::getCurrency).<String>map("getValue", FieldWithMeta::getValue))).or(notEqual(MapperS.of(interestRatePayout).<CrossCurrencyTerms>map("getCrossCurrencyTerms", InterestRatePayout::getCrossCurrencyTerms).<SettlementProvision>map("getSettlementProvision", CrossCurrencyTerms::getSettlementProvision).map("getSettlementCurrency", SettlementProvision::getSettlementCurrency).<String>map("getValue", FieldWithMeta::getValue), MapperS.of(interestRatePayout).<ContractualQuantity>map("getQuantity", InterestRatePayout::getQuantity).<FxLinkedNotionalSchedule>map("getFxLinkedNotional", ContractualQuantity::getFxLinkedNotional).map("getVaryingNotionalCurrency", FxLinkedNotionalSchedule::getVaryingNotionalCurrency).<String>map("getValue", FieldWithMeta::getValue)));
	}
}
