package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.math.BigDecimal;
import org.isda.cdm.CreditDefaultPayout;
import org.isda.cdm.GeneralTerms;
import org.isda.cdm.IndexReferenceInformation;
import org.isda.cdm.Payout;
import org.isda.cdm.TransactedPrice;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("TransactedPrice_marketPrice")
public class TransactedPriceMarketPriceDataRule implements Validator<Payout> {
	
	private static final String NAME = "TransactedPrice_marketPrice";
	private static final String DEFINITION = "when Payout -> creditDefaultPayout -> generalTerms -> indexReferenceInformation is absent\n" + 
		"then Payout -> creditDefaultPayout -> transactedPrice -> marketFixedRate is absent\n" + 
		"		and Payout -> creditDefaultPayout -> transactedPrice -> marketPrice is absent";
	
	@Override
	public ValidationResult<Payout> validate(RosettaPath path, Payout payout) {
		ComparisonResult result = executeDataRule(payout);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "Payout", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "Payout", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<Payout> validate(RosettaPath path, RosettaModelObjectBuilder payout) {
		ComparisonResult result = executeDataRule((Payout)payout.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "Payout", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "Payout", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(Payout payout) {
		if (ruleIsApplicable(payout).get()) {
			return evaluateThenExpression(payout);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(Payout payout) {
		return notExists(MapperS.of(payout).<CreditDefaultPayout>map("getCreditDefaultPayout", Payout::getCreditDefaultPayout).<GeneralTerms>map("getGeneralTerms", CreditDefaultPayout::getGeneralTerms).<IndexReferenceInformation>map("getIndexReferenceInformation", GeneralTerms::getIndexReferenceInformation));
	}
	
	private ComparisonResult evaluateThenExpression(Payout payout) {
		return notExists(MapperS.of(payout).<CreditDefaultPayout>map("getCreditDefaultPayout", Payout::getCreditDefaultPayout).<TransactedPrice>map("getTransactedPrice", CreditDefaultPayout::getTransactedPrice).<BigDecimal>map("getMarketFixedRate", TransactedPrice::getMarketFixedRate)).and(notExists(MapperS.of(payout).<CreditDefaultPayout>map("getCreditDefaultPayout", Payout::getCreditDefaultPayout).<TransactedPrice>map("getTransactedPrice", CreditDefaultPayout::getTransactedPrice).<BigDecimal>map("getMarketPrice", TransactedPrice::getMarketPrice)));
	}
}
