package org.isda.cdm.qualify.event;

import com.rosetta.model.lib.annotations.RosettaQualifiable;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.qualify.QualifyResult;
import com.rosetta.model.lib.records.Date;
import com.rosetta.model.lib.validation.ComparisonResult;
import java.math.BigDecimal;
import java.util.function.Function;
import org.isda.cdm.AdjustableDate;
import org.isda.cdm.AdjustableOrRelativeDate;
import org.isda.cdm.AdjustedRelativeDateOffset;
import org.isda.cdm.CalculationPeriodDates;
import org.isda.cdm.Contract;
import org.isda.cdm.ContractState;
import org.isda.cdm.ContractualProduct;
import org.isda.cdm.ContractualQuantity;
import org.isda.cdm.CreditDefaultPayout;
import org.isda.cdm.EconomicTerms;
import org.isda.cdm.EquityPayout;
import org.isda.cdm.Event;
import org.isda.cdm.ExecutionQuantity;
import org.isda.cdm.FxLinkedNotionalSchedule;
import org.isda.cdm.Identifier;
import org.isda.cdm.Inception;
import org.isda.cdm.InterestRatePayout;
import org.isda.cdm.Money;
import org.isda.cdm.NonNegativeAmountSchedule;
import org.isda.cdm.NonNegativeQuantity;
import org.isda.cdm.NonNegativeSchedule;
import org.isda.cdm.NonNegativeStep;
import org.isda.cdm.NotionalSchedule;
import org.isda.cdm.NotionalStepRule;
import org.isda.cdm.OptionPayout;
import org.isda.cdm.Payout;
import org.isda.cdm.PostInceptionState;
import org.isda.cdm.PrimitiveEvent;
import org.isda.cdm.ProtectionTerms;
import org.isda.cdm.Quantity;
import org.isda.cdm.QuantityChangePrimitive;
import org.isda.cdm.QuantityNotation;
import org.isda.cdm.RelativeDateOffset;
import org.isda.cdm.Trade;
import org.isda.cdm.TradeDate;
import org.isda.cdm.validation.datarule.*;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaQualifiable("PartialNovation")
public class IsPartialNovation implements Function<Event, QualifyResult>{
	
	private static final String QUALIFY_DEFINITION = "isEvent PartialNovation <The qualification of a novation event from the fact that (i) the intent is Novation when specified, (ii) the primitives quantityChange and inception exist, (iii) the contract quantity/notional has decreased as part of the quantityChange primitive, while (iv) there is a remaining quantity/notional, (v) the stepped-in contract has a different contract identifier than the original contract, (vi) the stepped-in contract has the novation event date and the novation event effective date, and (vii) the contract counterparties have changed.>\n" + 
		"	Event -> primitive -> quantityChange exists\n" + 
		"	and Event -> primitive -> inception exists\n" + 
		"	and Event -> primitive -> inception -> after -> contract -> contractIdentifier <> Event -> primitive -> quantityChange -> before -> contract -> contractIdentifier\n" + 
		"	and Event -> eventDate = Event -> primitive -> inception -> after -> contract -> tradeDate -> date\n" + 
		"	and Event -> effectiveDate = novatedContractEffectiveDate\n" + 
		"	and quantityBeforeQuantityChange > quantityAfterQuantityChange\n" + 
		"	and quantityAfterQuantityChange > 0.0";
	
	private static final String EXPR_DEFINITION = "Event -> primitive -> quantityChange exists\n" + 
		"	and Event -> primitive -> inception exists\n" + 
		"	and Event -> primitive -> inception -> after -> contract -> contractIdentifier <> Event -> primitive -> quantityChange -> before -> contract -> contractIdentifier\n" + 
		"	and Event -> eventDate = Event -> primitive -> inception -> after -> contract -> tradeDate -> date\n" + 
		"	and Event -> effectiveDate = novatedContractEffectiveDate\n" + 
		"	and quantityBeforeQuantityChange > quantityAfterQuantityChange\n" + 
		"	and quantityAfterQuantityChange > 0.0";
	
	@Override
	public QualifyResult apply(Event event) {
		ComparisonResult exprResult = exists(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange), false).and(exists(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<Inception>mapC("getInception", PrimitiveEvent::getInception), false)).and(notEqual(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<Inception>mapC("getInception", PrimitiveEvent::getInception).<PostInceptionState>map("getAfter", Inception::getAfter).<Contract>map("getContract", ContractState::getContract).<Identifier>mapC("getContractIdentifier", Contract::getContractIdentifier), MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getBefore", QuantityChangePrimitive::getBefore).<Contract>map("getContract", Trade::getContract).<Identifier>mapC("getContractIdentifier", Contract::getContractIdentifier))).and(areEqual(MapperS.of(event).<Date>map("getEventDate", Event::getEventDate), MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<Inception>mapC("getInception", PrimitiveEvent::getInception).<PostInceptionState>map("getAfter", Inception::getAfter).<Contract>map("getContract", ContractState::getContract).<TradeDate>map("getTradeDate", Contract::getTradeDate).<Date>map("getDate", TradeDate::getDate))).and(areEqual(MapperTree.of(MapperS.of(event).<Date>map("getEffectiveDate", Event::getEffectiveDate)), MapperTree.or(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<Inception>mapC("getInception", PrimitiveEvent::getInception).<PostInceptionState>map("getAfter", Inception::getAfter).<Contract>map("getContract", ContractState::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<AdjustableOrRelativeDate>map("getEffectiveDate", EconomicTerms::getEffectiveDate).<AdjustableDate>map("getAdjustableDate", AdjustableOrRelativeDate::getAdjustableDate).map("getAdjustedDate", AdjustableDate::getAdjustedDate).<Date>map("getValue", FieldWithMeta::getValue)), MapperTree.or(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<Inception>mapC("getInception", PrimitiveEvent::getInception).<PostInceptionState>map("getAfter", Inception::getAfter).<Contract>map("getContract", ContractState::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<AdjustableOrRelativeDate>map("getEffectiveDate", EconomicTerms::getEffectiveDate).<AdjustableDate>map("getAdjustableDate", AdjustableOrRelativeDate::getAdjustableDate).<Date>map("getUnadjustedDate", AdjustableDate::getUnadjustedDate)), MapperTree.or(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<Inception>mapC("getInception", PrimitiveEvent::getInception).<PostInceptionState>map("getAfter", Inception::getAfter).<Contract>map("getContract", ContractState::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<AdjustableOrRelativeDate>map("getEffectiveDate", EconomicTerms::getEffectiveDate).<AdjustedRelativeDateOffset>map("getRelativeDate", AdjustableOrRelativeDate::getRelativeDate).<Date>map("getAdjustedDate", RelativeDateOffset::getAdjustedDate)), MapperTree.or(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<Inception>mapC("getInception", PrimitiveEvent::getInception).<PostInceptionState>map("getAfter", Inception::getAfter).<Contract>map("getContract", ContractState::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<CalculationPeriodDates>map("getCalculationPeriodDates", InterestRatePayout::getCalculationPeriodDates).<AdjustableOrRelativeDate>map("getEffectiveDate", CalculationPeriodDates::getEffectiveDate).<AdjustableDate>map("getAdjustableDate", AdjustableOrRelativeDate::getAdjustableDate).map("getAdjustedDate", AdjustableDate::getAdjustedDate).<Date>map("getValue", FieldWithMeta::getValue)), MapperTree.or(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<Inception>mapC("getInception", PrimitiveEvent::getInception).<PostInceptionState>map("getAfter", Inception::getAfter).<Contract>map("getContract", ContractState::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<CalculationPeriodDates>map("getCalculationPeriodDates", InterestRatePayout::getCalculationPeriodDates).<AdjustableOrRelativeDate>map("getEffectiveDate", CalculationPeriodDates::getEffectiveDate).<AdjustableDate>map("getAdjustableDate", AdjustableOrRelativeDate::getAdjustableDate).<Date>map("getUnadjustedDate", AdjustableDate::getUnadjustedDate)), MapperTree.or(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<Inception>mapC("getInception", PrimitiveEvent::getInception).<PostInceptionState>map("getAfter", Inception::getAfter).<Contract>map("getContract", ContractState::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<CalculationPeriodDates>map("getCalculationPeriodDates", InterestRatePayout::getCalculationPeriodDates).<AdjustableOrRelativeDate>map("getEffectiveDate", CalculationPeriodDates::getEffectiveDate).<AdjustedRelativeDateOffset>map("getRelativeDate", AdjustableOrRelativeDate::getRelativeDate).<Date>map("getAdjustedDate", RelativeDateOffset::getAdjustedDate)), MapperTree.or(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<Inception>mapC("getInception", PrimitiveEvent::getInception).<PostInceptionState>map("getAfter", Inception::getAfter).<Contract>map("getContract", ContractState::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<EquityPayout>mapC("getEquityPayout", Payout::getEquityPayout).<CalculationPeriodDates>map("getCalculationPeriodDates", EquityPayout::getCalculationPeriodDates).<AdjustableOrRelativeDate>map("getEffectiveDate", CalculationPeriodDates::getEffectiveDate).<AdjustableDate>map("getAdjustableDate", AdjustableOrRelativeDate::getAdjustableDate).map("getAdjustedDate", AdjustableDate::getAdjustedDate).<Date>map("getValue", FieldWithMeta::getValue)), MapperTree.or(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<Inception>mapC("getInception", PrimitiveEvent::getInception).<PostInceptionState>map("getAfter", Inception::getAfter).<Contract>map("getContract", ContractState::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<EquityPayout>mapC("getEquityPayout", Payout::getEquityPayout).<CalculationPeriodDates>map("getCalculationPeriodDates", EquityPayout::getCalculationPeriodDates).<AdjustableOrRelativeDate>map("getEffectiveDate", CalculationPeriodDates::getEffectiveDate).<AdjustableDate>map("getAdjustableDate", AdjustableOrRelativeDate::getAdjustableDate).<Date>map("getUnadjustedDate", AdjustableDate::getUnadjustedDate)), MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<Inception>mapC("getInception", PrimitiveEvent::getInception).<PostInceptionState>map("getAfter", Inception::getAfter).<Contract>map("getContract", ContractState::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<EquityPayout>mapC("getEquityPayout", Payout::getEquityPayout).<CalculationPeriodDates>map("getCalculationPeriodDates", EquityPayout::getCalculationPeriodDates).<AdjustableOrRelativeDate>map("getEffectiveDate", CalculationPeriodDates::getEffectiveDate).<AdjustedRelativeDateOffset>map("getRelativeDate", AdjustableOrRelativeDate::getRelativeDate).<Date>map("getAdjustedDate", RelativeDateOffset::getAdjustedDate)))))))))))).and(greaterThan(MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getBefore", QuantityChangePrimitive::getBefore).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<ContractualQuantity>map("getQuantity", InterestRatePayout::getQuantity).<Quantity>map("getQuantity", ContractualQuantity::getQuantity).<BigDecimal>map("getAmount", Quantity::getAmount)), MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getBefore", QuantityChangePrimitive::getBefore).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<ContractualQuantity>map("getQuantity", InterestRatePayout::getQuantity).map("getNotionalAmount", ContractualQuantity::getNotionalAmount).<Money>map("getValue", FieldWithMeta::getValue).<String>groupBy(g->new MapperS<>(g).map("getCurrency", Money::getCurrency).<String>map("getValue", FieldWithMeta::getValue)).<BigDecimal>map("getAmount", Money::getAmount)), MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getBefore", QuantityChangePrimitive::getBefore).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<ContractualQuantity>map("getQuantity", InterestRatePayout::getQuantity).<NotionalSchedule>map("getNotionalSchedule", ContractualQuantity::getNotionalSchedule).<NonNegativeAmountSchedule>map("getNotionalStepSchedule", NotionalSchedule::getNotionalStepSchedule).<String>groupBy(g->new MapperS<>(g).map("getCurrency", NonNegativeAmountSchedule::getCurrency).<String>map("getValue", FieldWithMeta::getValue)).<BigDecimal>map("getInitialValue", NonNegativeSchedule::getInitialValue)), MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getBefore", QuantityChangePrimitive::getBefore).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<ContractualQuantity>map("getQuantity", InterestRatePayout::getQuantity).<NotionalSchedule>map("getNotionalSchedule", ContractualQuantity::getNotionalSchedule).<NonNegativeAmountSchedule>map("getNotionalStepSchedule", NotionalSchedule::getNotionalStepSchedule).<String>groupBy(g->new MapperS<>(g).map("getCurrency", NonNegativeAmountSchedule::getCurrency).<String>map("getValue", FieldWithMeta::getValue)).<NonNegativeStep>mapC("getStep", NonNegativeSchedule::getStep).<BigDecimal>map("getStepValue", NonNegativeStep::getStepValue)), MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getBefore", QuantityChangePrimitive::getBefore).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<ContractualQuantity>map("getQuantity", InterestRatePayout::getQuantity).<NotionalSchedule>map("getNotionalSchedule", ContractualQuantity::getNotionalSchedule).<String>groupBy(g->new MapperS<>(g).<NonNegativeAmountSchedule>map("getNotionalStepSchedule", NotionalSchedule::getNotionalStepSchedule).map("getCurrency", NonNegativeAmountSchedule::getCurrency).<String>map("getValue", FieldWithMeta::getValue)).<NotionalStepRule>map("getNotionalStepParameters", NotionalSchedule::getNotionalStepParameters).<BigDecimal>map("getNotionalStepAmount", NotionalStepRule::getNotionalStepAmount)), MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getBefore", QuantityChangePrimitive::getBefore).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<ContractualQuantity>map("getQuantity", InterestRatePayout::getQuantity).<FxLinkedNotionalSchedule>map("getFxLinkedNotional", ContractualQuantity::getFxLinkedNotional).<String>groupBy(g->new MapperS<>(g).map("getVaryingNotionalCurrency", FxLinkedNotionalSchedule::getVaryingNotionalCurrency).<String>map("getValue", FieldWithMeta::getValue)).<BigDecimal>map("getInitialValue", FxLinkedNotionalSchedule::getInitialValue)), MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getBefore", QuantityChangePrimitive::getBefore).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<CreditDefaultPayout>map("getCreditDefaultPayout", Payout::getCreditDefaultPayout).<ProtectionTerms>mapC("getProtectionTerms", CreditDefaultPayout::getProtectionTerms).<Money>map("getNotionalAmount", ProtectionTerms::getNotionalAmount).<String>groupBy(g->new MapperS<>(g).map("getCurrency", Money::getCurrency).<String>map("getValue", FieldWithMeta::getValue)).<BigDecimal>map("getAmount", Money::getAmount)), MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getBefore", QuantityChangePrimitive::getBefore).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<OptionPayout>mapC("getOptionPayout", Payout::getOptionPayout).<ContractualQuantity>map("getQuantity", OptionPayout::getQuantity).map("getNotionalAmount", ContractualQuantity::getNotionalAmount).<Money>map("getValue", FieldWithMeta::getValue).<String>groupBy(g->new MapperS<>(g).map("getCurrency", Money::getCurrency).<String>map("getValue", FieldWithMeta::getValue)).<BigDecimal>map("getAmount", Money::getAmount)), MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getBefore", QuantityChangePrimitive::getBefore).<Contract>map("getContract", Trade::getContract).<ExecutionQuantity>map("getContractualQuantity", Contract::getContractualQuantity).<QuantityNotation>mapC("getQuantityNotation", ExecutionQuantity::getQuantityNotation).<NonNegativeQuantity>map("getQuantity", QuantityNotation::getQuantity).<String>groupBy(g->new MapperS<>(g).map("getCurrency", Quantity::getCurrency).<String>map("getValue", FieldWithMeta::getValue)).<BigDecimal>map("getAmount", Quantity::getAmount)))))))))), MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getAfter", QuantityChangePrimitive::getAfter).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<ContractualQuantity>map("getQuantity", InterestRatePayout::getQuantity).<Quantity>map("getQuantity", ContractualQuantity::getQuantity).<BigDecimal>map("getAmount", Quantity::getAmount)), MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getAfter", QuantityChangePrimitive::getAfter).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<ContractualQuantity>map("getQuantity", InterestRatePayout::getQuantity).map("getNotionalAmount", ContractualQuantity::getNotionalAmount).<Money>map("getValue", FieldWithMeta::getValue).<String>groupBy(g->new MapperS<>(g).map("getCurrency", Money::getCurrency).<String>map("getValue", FieldWithMeta::getValue)).<BigDecimal>map("getAmount", Money::getAmount)), MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getAfter", QuantityChangePrimitive::getAfter).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<ContractualQuantity>map("getQuantity", InterestRatePayout::getQuantity).<NotionalSchedule>map("getNotionalSchedule", ContractualQuantity::getNotionalSchedule).<NonNegativeAmountSchedule>map("getNotionalStepSchedule", NotionalSchedule::getNotionalStepSchedule).<String>groupBy(g->new MapperS<>(g).map("getCurrency", NonNegativeAmountSchedule::getCurrency).<String>map("getValue", FieldWithMeta::getValue)).<BigDecimal>map("getInitialValue", NonNegativeSchedule::getInitialValue)), MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getAfter", QuantityChangePrimitive::getAfter).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<ContractualQuantity>map("getQuantity", InterestRatePayout::getQuantity).<NotionalSchedule>map("getNotionalSchedule", ContractualQuantity::getNotionalSchedule).<NonNegativeAmountSchedule>map("getNotionalStepSchedule", NotionalSchedule::getNotionalStepSchedule).<String>groupBy(g->new MapperS<>(g).map("getCurrency", NonNegativeAmountSchedule::getCurrency).<String>map("getValue", FieldWithMeta::getValue)).<NonNegativeStep>mapC("getStep", NonNegativeSchedule::getStep).<BigDecimal>map("getStepValue", NonNegativeStep::getStepValue)), MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getAfter", QuantityChangePrimitive::getAfter).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<ContractualQuantity>map("getQuantity", InterestRatePayout::getQuantity).<NotionalSchedule>map("getNotionalSchedule", ContractualQuantity::getNotionalSchedule).<String>groupBy(g->new MapperS<>(g).<NonNegativeAmountSchedule>map("getNotionalStepSchedule", NotionalSchedule::getNotionalStepSchedule).map("getCurrency", NonNegativeAmountSchedule::getCurrency).<String>map("getValue", FieldWithMeta::getValue)).<NotionalStepRule>map("getNotionalStepParameters", NotionalSchedule::getNotionalStepParameters).<BigDecimal>map("getNotionalStepAmount", NotionalStepRule::getNotionalStepAmount)), MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getAfter", QuantityChangePrimitive::getAfter).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<ContractualQuantity>map("getQuantity", InterestRatePayout::getQuantity).<FxLinkedNotionalSchedule>map("getFxLinkedNotional", ContractualQuantity::getFxLinkedNotional).<String>groupBy(g->new MapperS<>(g).map("getVaryingNotionalCurrency", FxLinkedNotionalSchedule::getVaryingNotionalCurrency).<String>map("getValue", FieldWithMeta::getValue)).<BigDecimal>map("getInitialValue", FxLinkedNotionalSchedule::getInitialValue)), MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getAfter", QuantityChangePrimitive::getAfter).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<CreditDefaultPayout>map("getCreditDefaultPayout", Payout::getCreditDefaultPayout).<ProtectionTerms>mapC("getProtectionTerms", CreditDefaultPayout::getProtectionTerms).<Money>map("getNotionalAmount", ProtectionTerms::getNotionalAmount).<String>groupBy(g->new MapperS<>(g).map("getCurrency", Money::getCurrency).<String>map("getValue", FieldWithMeta::getValue)).<BigDecimal>map("getAmount", Money::getAmount)), MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getAfter", QuantityChangePrimitive::getAfter).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<OptionPayout>mapC("getOptionPayout", Payout::getOptionPayout).<ContractualQuantity>map("getQuantity", OptionPayout::getQuantity).map("getNotionalAmount", ContractualQuantity::getNotionalAmount).<Money>map("getValue", FieldWithMeta::getValue).<String>groupBy(g->new MapperS<>(g).map("getCurrency", Money::getCurrency).<String>map("getValue", FieldWithMeta::getValue)).<BigDecimal>map("getAmount", Money::getAmount)), MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getAfter", QuantityChangePrimitive::getAfter).<Contract>map("getContract", Trade::getContract).<ExecutionQuantity>map("getContractualQuantity", Contract::getContractualQuantity).<QuantityNotation>mapC("getQuantityNotation", ExecutionQuantity::getQuantityNotation).<NonNegativeQuantity>map("getQuantity", QuantityNotation::getQuantity).<String>groupBy(g->new MapperS<>(g).map("getCurrency", Quantity::getCurrency).<String>map("getValue", FieldWithMeta::getValue)).<BigDecimal>map("getAmount", Quantity::getAmount)))))))))))).and(greaterThan(MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getAfter", QuantityChangePrimitive::getAfter).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<ContractualQuantity>map("getQuantity", InterestRatePayout::getQuantity).<Quantity>map("getQuantity", ContractualQuantity::getQuantity).<BigDecimal>map("getAmount", Quantity::getAmount)), MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getAfter", QuantityChangePrimitive::getAfter).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<ContractualQuantity>map("getQuantity", InterestRatePayout::getQuantity).map("getNotionalAmount", ContractualQuantity::getNotionalAmount).<Money>map("getValue", FieldWithMeta::getValue).<String>groupBy(g->new MapperS<>(g).map("getCurrency", Money::getCurrency).<String>map("getValue", FieldWithMeta::getValue)).<BigDecimal>map("getAmount", Money::getAmount)), MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getAfter", QuantityChangePrimitive::getAfter).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<ContractualQuantity>map("getQuantity", InterestRatePayout::getQuantity).<NotionalSchedule>map("getNotionalSchedule", ContractualQuantity::getNotionalSchedule).<NonNegativeAmountSchedule>map("getNotionalStepSchedule", NotionalSchedule::getNotionalStepSchedule).<String>groupBy(g->new MapperS<>(g).map("getCurrency", NonNegativeAmountSchedule::getCurrency).<String>map("getValue", FieldWithMeta::getValue)).<BigDecimal>map("getInitialValue", NonNegativeSchedule::getInitialValue)), MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getAfter", QuantityChangePrimitive::getAfter).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<ContractualQuantity>map("getQuantity", InterestRatePayout::getQuantity).<NotionalSchedule>map("getNotionalSchedule", ContractualQuantity::getNotionalSchedule).<NonNegativeAmountSchedule>map("getNotionalStepSchedule", NotionalSchedule::getNotionalStepSchedule).<String>groupBy(g->new MapperS<>(g).map("getCurrency", NonNegativeAmountSchedule::getCurrency).<String>map("getValue", FieldWithMeta::getValue)).<NonNegativeStep>mapC("getStep", NonNegativeSchedule::getStep).<BigDecimal>map("getStepValue", NonNegativeStep::getStepValue)), MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getAfter", QuantityChangePrimitive::getAfter).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<ContractualQuantity>map("getQuantity", InterestRatePayout::getQuantity).<NotionalSchedule>map("getNotionalSchedule", ContractualQuantity::getNotionalSchedule).<String>groupBy(g->new MapperS<>(g).<NonNegativeAmountSchedule>map("getNotionalStepSchedule", NotionalSchedule::getNotionalStepSchedule).map("getCurrency", NonNegativeAmountSchedule::getCurrency).<String>map("getValue", FieldWithMeta::getValue)).<NotionalStepRule>map("getNotionalStepParameters", NotionalSchedule::getNotionalStepParameters).<BigDecimal>map("getNotionalStepAmount", NotionalStepRule::getNotionalStepAmount)), MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getAfter", QuantityChangePrimitive::getAfter).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<ContractualQuantity>map("getQuantity", InterestRatePayout::getQuantity).<FxLinkedNotionalSchedule>map("getFxLinkedNotional", ContractualQuantity::getFxLinkedNotional).<String>groupBy(g->new MapperS<>(g).map("getVaryingNotionalCurrency", FxLinkedNotionalSchedule::getVaryingNotionalCurrency).<String>map("getValue", FieldWithMeta::getValue)).<BigDecimal>map("getInitialValue", FxLinkedNotionalSchedule::getInitialValue)), MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getAfter", QuantityChangePrimitive::getAfter).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<CreditDefaultPayout>map("getCreditDefaultPayout", Payout::getCreditDefaultPayout).<ProtectionTerms>mapC("getProtectionTerms", CreditDefaultPayout::getProtectionTerms).<Money>map("getNotionalAmount", ProtectionTerms::getNotionalAmount).<String>groupBy(g->new MapperS<>(g).map("getCurrency", Money::getCurrency).<String>map("getValue", FieldWithMeta::getValue)).<BigDecimal>map("getAmount", Money::getAmount)), MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getAfter", QuantityChangePrimitive::getAfter).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<OptionPayout>mapC("getOptionPayout", Payout::getOptionPayout).<ContractualQuantity>map("getQuantity", OptionPayout::getQuantity).map("getNotionalAmount", ContractualQuantity::getNotionalAmount).<Money>map("getValue", FieldWithMeta::getValue).<String>groupBy(g->new MapperS<>(g).map("getCurrency", Money::getCurrency).<String>map("getValue", FieldWithMeta::getValue)).<BigDecimal>map("getAmount", Money::getAmount)), MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getAfter", QuantityChangePrimitive::getAfter).<Contract>map("getContract", Trade::getContract).<ExecutionQuantity>map("getContractualQuantity", Contract::getContractualQuantity).<QuantityNotation>mapC("getQuantityNotation", ExecutionQuantity::getQuantityNotation).<NonNegativeQuantity>map("getQuantity", QuantityNotation::getQuantity).<String>groupBy(g->new MapperS<>(g).map("getCurrency", Quantity::getCurrency).<String>map("getValue", FieldWithMeta::getValue)).<BigDecimal>map("getAmount", Quantity::getAmount)))))))))), MapperTree.of(MapperS.of(new BigDecimal(0.0)))));
		
		return QualifyResult.builder()
				.setName("PartialNovation")
				.setDefinition(QUALIFY_DEFINITION)
				.setExpressionResult(EXPR_DEFINITION, exprResult)
				.build();
	}
}
